package AlertStor2rrd;
use Date::Parse;
use RRDp;
use strict;
use POSIX qw(strftime);
use Data::Dumper;

my @config_alert_data = "";
my %inventory_data;
my @config_alert     = ( "logs/alert_history.log", "0",      "25",          "60",             "30",                "" );               # predefined defaults
my @config_alert_key = ( "ALERT_HISTORY",          "NAGIOS", "EMAIL_GRAPH", "REPEAT_DEFAULT", "PEAK_TIME_DEFAULT", "EXTERN_ALERT" );
my $ALERT_HISTORY    = 0;
my $NAGIOS           = 1;
my $EMAIL_GRAPH      = 2;
my $REPEAT_DEFAULT   = 3;
my $PEAK_TIME_DEFAULT = 4;
my $EXTERN_ALERT      = 5;
my $alert_test        = $ENV{ALERT_TEST};

if ( !defined($alert_test) || isdigit($alert_test) == 0 ) {
  $alert_test = 0;
}

sub alert_test_email {
  my $cfg      = shift;
  my $VM_IMAGE = shift;
  my $VI_IMAGE = shift;
  my %inventory_test;
  my $extern_alert        = "";
  my $alert_history_event = "";
  my $email_event         = "";
  my $nagios_event        = "";
  my $trap_event          = "";

  if ( !defined $VM_IMAGE ) {
    $VM_IMAGE = "";
  }
  if ( !defined $VI_IMAGE ) {
    $VI_IMAGE = "";
  }

  if ( !-f "$cfg" ) {
    warning("Alrt cfg does not exist!  : $cfg");
    return 0;    # alerting is not in place
  }
  print "Alrt cfg read  : $cfg\n";

  open( FH, "< $cfg" ) || error( "could not open $cfg : $!" . __FILE__ . ":" . __LINE__ ) && return (0);
  my @lines = <FH>;
  close(FH);

  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line =~ s/^\s+|\s+$//g;

    #if ($line =~ m/^EXTERN_ALERT_EVENT=/){
    #  (undef, $extern_alert) = split("=",$line);
    #}
    if ( $line =~ m/^ALERT_HISTORY_EVENT=/ ) {
      ( undef, $alert_history_event ) = split( "=", $line );
    }
    if ( $line =~ m/^EMAIL_EVENT=/ ) {
      ( undef, $email_event ) = split( "=", $line );
    }
    if ( $line =~ m/^TRAP_EVENT=/ ) {
      ( undef, $trap_event ) = split( "=", $line );
    }

    #if ($line =~ m/^NAGIOS_EVENT=/){
    #  (undef, $nagios_event) = split("=",$line);
    #}
  } ## end foreach my $line (@lines)

  #if (defined $extern_alert ){
  #  $extern_alert =~ s/\#.*//g;
  #  $extern_alert =~ s/^\s+|\s+$//g;
  #}
  if ( defined $alert_history_event ) {
    $alert_history_event =~ s/\#.*//g;
    $alert_history_event =~ s/^\s+|\s+$//g;
  }
  if ( defined $email_event ) {
    $email_event =~ s/\#.*//g;
    $email_event =~ s/^\s+|\s+$//g;
  }

  #if (defined $nagios_event ){
  #  $nagios_event =~ s/\#.*//g;
  #  $nagios_event =~ s/\#.*//g;
  #}

  if ( defined $email_event && $email_event ne "" ) {
    my $report = "Test email event alerting";
    sendmail_test( $email_event, $report );
  }
  else {
    warning("Email does not defined!");
  }

  if ( ( defined $alert_history_event && $alert_history_event ne "" ) && ( $VI_IMAGE eq "1" || $VM_IMAGE eq "1" ) ) {
    my $report = "Test event alerting";
    log_to_alert_log_test( $report, $alert_history_event, $email_event );
  }
  if ( ( defined $alert_history_event && $alert_history_event ne "" ) && ( $VI_IMAGE ne "1" || $VM_IMAGE ne "1" ) ) {
    my $report = "Alert has not been written into $alert_history_event";
    warning($report);
  }

  ### trap
  if ( defined $trap_event && $trap_event ne "" ) {
    snmp_trap_alarm_test( $trap_event, "test_storage_name" );
  }
  else {
    warning("SNMP trap host does not defined!");
  }

} ## end sub alert_test_email

sub log_to_alert_log_test {
  my $report   = shift;
  my $file     = shift;
  my $email    = shift;
  my $time_act = localtime;

  open( FHW, ">> $file" ) || error( "could not open $file : $!" . __FILE__ . ":" . __LINE__ ) && return 1;
  print "Test event alert: $report writtten to $file\n";
  if ( defined($email) && !$email eq '' ) {
    print FHW "$time_act: $report,  email: $email\n";
  }
  else {
    print FHW "$time_act: $report, no email\n";
  }
  close(FHW);
  return 1;
} ## end sub log_to_alert_log_test

sub sendmail_test {

  my ( $email, $report ) = @_;
  my $mailfrom   = "stor2rrd";
  my $mailprog   = "/usr/sbin/sendmail";
  my @email_list = split( " ", $email );

  foreach my $email (@email_list) {
    $email =~ s/ //g;
    print "Emailing       : $report sending email to $email\n";
    if ( open MAIL, "|$mailprog -t" ) {
      print MAIL "To: $email\n";
      print MAIL "From: $mailfrom\n";
      print MAIL "Reply-To: $mailfrom\n";
      print MAIL "X-Mailer: stor2rrd 1.0\n";
      print MAIL "Subject: $report\n\n";
      print MAIL "$report\n";
      print MAIL "\n\n";
      close(MAIL);
    } ## end if ( open MAIL, "|$mailprog -t")
  } ## end foreach my $email (@email_list)
  return 1;
} ## end sub sendmail_test

sub alert {
  my ( $storage, $st_type, $time_act, $basedir, $DEBUG, $VI_IMAGE, $VM_IMAGE ) = @_;

  my $debug = $ENV{DEBUG_ALERT};
  if ( defined($debug) && isdigit($debug) && $debug == 1 ) {
    $DEBUG = 22;    # 22 that is alerting verbose level
  }

  if ( !defined($storage) || $storage eq '' || !defined($st_type) || $st_type eq '' || !defined($time_act) || $time_act eq '' ) {
    error( "Alerting cannot run, is not set one of following storage/st_type/time_act ($storage/$st_type/$time_act) " . __FILE__ . ":" . __LINE__ );
    return (1);
  }

  # check if is passed time is in text or unix form, convert necessary
  my $time_act_u = $time_act;
  if ( isdigit($time_act) ) {
    $time_act = strftime( "%H:%M:%S %d/%m/%Y", localtime($time_act_u) );
  }
  else {
    $time_act_u = str2time($time_act);
    $time_act = strftime( "%H:%M:%S %d/%m/%Y", localtime($time_act_u) );
  }

  print "\n"                                                                                  if $DEBUG == 22;
  print "Alerting starts: storage=$storage, st_type=$st_type, time=$time_act, debug=$DEBUG\n" if $DEBUG == 22;

  # get cmd line params
  my $wrkdir     = "$basedir/data";
  my $alert_log  = "$basedir/alert.log";
  my $nagios_dir = "$basedir/nagios";
  my $cfg        = "$basedir/web_config/alerting.cfg";
  if ( !-f "$cfg" ) {
    $cfg = "$basedir/etc/alert.cfg";
  }
  if ( defined $ENV{ALERCFG} && -f $ENV{ALERCFG} ) {
    $cfg = $ENV{ALERCFG};
  }
  if ( -f "$basedir/etc/web_config/alerting.cfg" ) {
    $cfg = "$basedir/etc/web_config/alerting.cfg"    # one created by the GUI is prefered
  }

  # use alert here to be sure it does not hang due to any problem
  my $timeout = 600;                                 # it must be enough otherwise oit does not make sense to alert
  eval {
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "died in SIG ALRM: "; };
    alarm($timeout);

    my $ret = read_cfg( $cfg, $DEBUG, $storage, \@config_alert_key );
    if ( $ret == 0 ) {
      return 0;
    }

    if ( $DEBUG == 22 ) {
      my $line_no = 0;
      foreach my $l (@config_alert) {

        #print "Config global  : $config_alert_key[$line_no] : $l\n";
        $line_no++;
      }
      foreach my $line (@config_alert_data) {
        my $l = $line;
        if ( $l eq "" ) { next; }
        $l =~ s/===========doublecoma=========/:/g;
        print "Config alert   : $l\n";
      } ## end foreach my $line (@config_alert_data)
      foreach my $line ( @{ $inventory_data{"DATA_LINE"} } ) {
        my $l = $line;
        if ( $l =~ m/^EMAIL/ || $l eq "" ) { next; }
        $l =~ s/===========doublecoma=========/:/g;
        print "Config alert   : $l\n";
      } ## end foreach my $line ( @{ $inventory_data...})

    } ## end if ( $DEBUG == 22 )

    #print Dumper \%inventory_data;

    check_data( $basedir, $storage, $st_type, $time_act, $time_act_u, $wrkdir, $DEBUG, $nagios_dir, \@config_alert_data, $VI_IMAGE, $VM_IMAGE );
    check_data_host( $basedir, $storage, $st_type, $time_act, $time_act_u, $wrkdir, $DEBUG, $nagios_dir, $VI_IMAGE, $VM_IMAGE );

    alarm(0);

    # close RRD pipe
    my $unixt = str2time($act_time);
    $act_time = localtime();
    my $unixt_end = str2time($act_time);
    my $run_time  = $unixt_end - $unixt;
    print "Finished       : $act_time, run time: $run_time secs\n" if $DEBUG == 22;
  };

  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      error("$0 timed out after : $timeout seconds");
    }
    else {
      error("$0 failed: $@");
    }
    return (1);
  } ## end if ($@)
} ## end sub alert

sub read_cfg {
  my ( $cfg, $DEBUG, $storage, $config_alert_key_tmp ) = @_;
  my @config_alert_key = @{$config_alert_key_tmp};

  print "Alrt cfg read  : $cfg\n" if $DEBUG == 22;
  if ( !-f "$cfg" ) {
    return 0;    # alerting is not in place
  }

  open( FH, "< $cfg" ) || error( "could not open $cfg : $!" . __FILE__ . ":" . __LINE__ ) && return (0);
  my @lines = <FH>;
  close(FH);

  # allow usage of additional alert.cfg files like : etc/alert*.cfg
  ( my $cfg_pref, my $cfg_suff ) = split( /\./, $cfg );
  foreach my $cfg_cust (<$cfg_pref*\.$cfg_suff>) {
    chomp($cfg_cust);
    if ( $cfg =~ m/^$cfg_cust$/ ) {
      next;
    }
    open( FH, "< $cfg_cust" ) || error( "could not open $cfg_cust : $!" . __FILE__ . ":" . __LINE__ ) && return (0);
    my @lines_cust = <FH>;
    close(FH);
    my @merged = ( @lines, @lines_cust );
    @lines = @merged;

    #print "001 $cfg_cust\n";
  } ## end foreach my $cfg_cust (<$cfg_pref*\.$cfg_suff>)

  my @lines_sort = sort(@lines);

  my $data_line = 0;
  my $key_no    = 0;
  my $key       = "";
  foreach my $line (@lines) {
    if ( $line =~ m/^#/ ) {
      next;
    }
    chomp($line);
    $line =~ s/\\:/===========doublecoma=========/g;    # workround for lpars/pool/groups with double coma inside the name
    $line =~ s/#.*$//;
    if ( $line =~ m/^$/ ) {
      next;
    }

    $key_no = 0;
    foreach $key (@config_alert_key) {

      # search for default variables and replace defaults
      if ( $line =~ m/^$key=/ ) {
        ( my $trash, my $value ) = split( /=/, $line );
        if ( defined($value) && !$value eq '' ) {
          $value =~ s/ //g;
          $value =~ s/	//g;
          if ( defined($value) && !$value eq '' ) {
            $config_alert[$key_no] = $value;
          }
        } ## end if ( defined($value) &&...)
      } ## end if ( $line =~ m/^$key=/)
      $key_no++;
    } ## end foreach $key (@config_alert_key)
    if ( $key_no < $#config_alert_key ) {
      next;    # the key has been found
    }

    # continue with data lines
    if ( $line =~ m/^VOLUME:/ || $line =~ m/^POOL:/ ) {
      ( my $trash, my $storage_act ) = split( /:/, $line );
      if ( defined($storage_act) && !$storage_act eq '' && $storage =~ m/^$storage_act$/ ) {

        # save only data for actual storage
        $config_alert_data[$data_line] = $line;
        $data_line++;
        my $l = $line;
        $l =~ s/===========doublecoma=========/:/g;
        print "Data identified: $l\n" if $DEBUG == 22;
      } ## end if ( defined($storage_act...))
    } ## end if ( $line =~ m/^VOLUME:/...)
    if ( $line =~ m/^POOL-ALL:/ ) {
      ( my $trash, my $storage_act ) = split( /:/, $line );
      if ( defined($storage_act) && !$storage_act eq '' && $storage =~ m/^$storage_act$/ ) {

        # save only data for actual storage
        $config_alert_data[$data_line] = $line;
        $data_line++;
        print "Data identified: $line\n" if $DEBUG == 22;
      } ## end if ( defined($storage_act...))
    } ## end if ( $line =~ m/^POOL-ALL:/)
    if ( $line =~ m/^HOST:/ ) {
      ( my $trash, my $storage_act ) = split( /:/, $line );
      if ( defined($storage_act) && !$storage_act eq '' && $storage eq $storage_act ) {
        push @{ $inventory_data{"DATA_LINE"} }, "$line";
        print "Data identified: $line\n" if $DEBUG == 22;
      }
    } ## end if ( $line =~ m/^HOST:/)
    if ( $line =~ m/CUSTOM GROUP/ ) {
      push @{ $inventory_data{"DATA_LINE"} }, "$line";
      print "Data identified: $line\n" if $DEBUG == 22;
    }
    if ( $line =~ m/^EMAIL:/ ) {

      # save email groups
      $config_alert_data[$data_line] = $line;
      push @{ $inventory_data{"DATA_LINE"} }, "$line";
      $data_line++;

      #print "Data identified: $line\n" if $DEBUG == 22;
    } ## end if ( $line =~ m/^EMAIL:/)
    if ( $line =~ m/^TRAP=/ ) {
      $config_alert_data[$data_line] = $line;
      push @{ $inventory_data{"DATA_LINE"} }, "$line";
      $data_line++;

      #print "Data identified: $line\n" if $DEBUG == 22;
    } ## end if ( $line =~ m/^TRAP=/)
  } ## end foreach my $line (@lines)

  $key_no = 0;
  foreach $key (@config_alert_key) {
    if ( $key_no != $EXTERN_ALERT && $config_alert[$key_no] eq '' ) {
      error("$key is not set, exiting ...");
      return (0);
    }
    $key_no++;
  } ## end foreach $key (@config_alert_key)
  return (1);
} ## end sub read_cfg

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  print "ERROR          : $text \n";
  print STDERR "$act_time: $text\n";

  return 1;
} ## end sub error

sub warning {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  print "WARNING        : $text \n";

  return 1;
} ## end sub warning

sub basename {
  my $full = shift;
  my $out  = "";

  # basename without direct function
  my @base = split( /\//, $full );
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
} ## end sub basename

#VOLUME:storage:name:[io|io_read|io_write|data|data_read|data_write]:limit:peek time in min:alert repeat time in min:exclude time:email group
#========================================================================================================================
#VOLUME:storwize01:BBzOd02:ior:1:::test@stor2rrd.com
#POOL-ALL:storwize-home::io:5::::test
#POOL:storwize-home:pool01:resp_t_w:0.1::5::test

sub check_data_host {
  my ( $basedir, $storage, $st_type, $time_act, $time_act_u, $wrkdir, $DEBUG, $nagios_dir, $VI_IMAGE, $VM_IMAGE ) = @_;
  my $host_cfg_file     = "$wrkdir/$storage/HOST/hosts.cfg";
  my $custom_cfg_file   = "$basedir/etc/web_config/custom_groups.cfg";
  my $vol_cfg_file      = "$wrkdir/$storage/VOLUME/volumes.cfg";
  my $alert_repeat_file = "$basedir/tmp/alert_repeat-stor2rrd-$storage.tmp";
  my ( $sec, $min, $hour_act ) = localtime();
  my $email_graph = $config_alert[$EMAIL_GRAPH];
  my @host_cfg    = ();
  my @custom_cfg  = ();
  my @volumes_id  = ();
  my @volumes_rrd = ();
  my $type_act    = "";
  my @type        = ();

  my $ds5_v2 = 0;
  if ( -f "$wrkdir/$storage/DS5K-v2" ) {
    $ds5_v2 = 1;    # DS5k v1 does not have response time
  }

  # Volume translation
  if ( -f "$host_cfg_file" ) {
    open( FV, "< $host_cfg_file" ) || error( "Can't open $host_cfg_file $! " . __FILE__ . ":" . __LINE__ );    # continue here
    @host_cfg = <FV>;
    close(FV);
  }

  #warning ("Do not exist  $host_cfg_file ".__FILE__.":".__LINE__);

  #custom cfg
  if ( -f "$custom_cfg_file" ) {
    open( FV, "< $custom_cfg_file" ) || error( "Can't open $custom_cfg_file $! " . __FILE__ . ":" . __LINE__ );    # continue here
    @custom_cfg = <FV>;
    close(FV);
  }

  #  warning ("Do not exist  $custom_cfg_file ".__FILE__.":".__LINE__);

  my @vol_cfg = "";
  if ( -f "$vol_cfg_file" ) {
    open( FV, "< $vol_cfg_file" ) || error( "Can't open $vol_cfg_file $! " . __FILE__ . ":" . __LINE__ );          # continue here
    @vol_cfg = <FV>;
    close(FV);
  }
  my %inventory_volume = get_inventory_volume(\@vol_cfg,$st_type);
  #print Dumper \%inventory_volume;

  #warning ("Does not exist volume cfg file: $vol_cfg_file ".__FILE__.":".__LINE__);

  my @repeat_list = "";
  if ( -f $alert_repeat_file ) {
    open( FHR, "< $alert_repeat_file" );
    @repeat_list = sort(<FHR>);
    close(FHR);
  }
  my @config_email  = "";
  my $line_email_no = 0;
  my $snmp_trap     = "";

  ### create data structure
  foreach my $line ( @{ $inventory_data{"DATA_LINE"} } ) {

    my $host         = "";
    my $storage_act  = "";
    my $host_name    = "";
    my $item         = "";
    my $limit        = "";
    my $peak_time    = "";
    my $repeat_time  = "";
    my $exclude_time = "";
    my $email        = "";
    my $types = "";

    if ( $line =~ m/^TRAP=/ ) {
      ( undef, $snmp_trap ) = split( /=/, $line );
      $snmp_trap =~ s/ //g;
      $snmp_trap =~ s/ //g;
      $snmp_trap =~ s/#.*$//g;
      next;
    } ## end if ( $line =~ m/^TRAP=/)

    # separate EMAIL
    if ( $line =~ m/^EMAIL/ ) {
      $config_email[$line_email_no] = $line;
      $line_email_no++;
      next;
    }
    if ( $line =~ m/CUSTOM GROUP/ ) {
      ( $host, undef, $host_name, $item, $limit, $peak_time, $repeat_time, $exclude_time, $email ) = split( /:/, $line );
      $types = $host;
      $storage_act = $storage;
      $host        = "CUSTOM";
      if ( !grep { $host eq $_ } @type ) {
        push( @type, $host );
      }
    } ## end if ( $line =~ m/CUSTOM GROUP/)
    if ( $line =~ m/^HOST/ && $line !~ m/CUSTOM GROUP/ ) {
      ( $host, $storage_act, $host_name, $item, $limit, $peak_time, $repeat_time, $exclude_time, $email ) = split( /:/, $line );
      if ( !grep { $host eq $_ } @type ) {
        push( @type, $host );
      }
    } ## end if ( $line =~ m/^HOST/)

    if ( !defined($storage_act) || $storage_act eq '' || $storage !~ m/^$storage_act$/ ) {

      # email can be empty, then is used only for externa purposes like external scrip[t or nagios
      next;    # ignoring, it is not specified the storage
    }
    if ( ( $host =~ m/^HOST$/ || $line =~ m/CUSTOM GROUP/ ) && ( $storage_act eq '' || $host_name eq '' || $item eq '' || $limit eq '' ) ) {
      warning( "$host: $storage: alerting rule does not contain all required fields: $line " . __FILE__ . ":" . __LINE__ );
      next;
    }
    if ( $line =~ m/CUSTOM GROUP/ ) {
      $host = "CUSTOM";
    }
    if ( !defined($repeat_time) || $repeat_time eq '' || !isdigit($repeat_time) ) {

      #use the global one
      $repeat_time = $config_alert[$REPEAT_DEFAULT];
    }

    if ( !defined($peak_time) || $peak_time eq '' || !isdigit($peak_time) ) {

      #use the global one
      $peak_time = $config_alert[$PEAK_TIME_DEFAULT];
    }
    $storage_act =~ s/===========doublecoma=========/:/g;
    if ( defined($host_name) && !$host_name eq '' ) {
      $host_name =~ s/===========doublecoma=========/:/g;
    }
    if ( defined($email) && !$email eq '' ) {
      $email =~ s/===========doublecoma=========/:/g;
      $inventory_data{$storage}{$host}{$host_name}{EMAIL} = $email;
    }
    $line =~ s/===========doublecoma=========/:/g;

    $inventory_data{$storage}{$host}{$host_name}{$item}{LIMIT}       = $limit;
    $inventory_data{$storage}{$host}{$host_name}{$item}{PEAK_TIME}   = $peak_time;
    $inventory_data{$storage}{$host}{$host_name}{$item}{REPEAT_TIME} = $repeat_time;
    $inventory_data{$storage}{$host}{$host_name}{$item}{EMAIL}       = $email;
    $inventory_data{$storage}{$host}{$host_name}{$item}{LINE}        = $line;

    ### add host for custom groups
    if (defined $types && $types ne ""){
      $inventory_data{$storage}{$host}{$host_name}{TYPE}        = $types;
    }

    if ( defined($exclude_time) && !$exclude_time eq '' ) {
      $inventory_data{$storage}{$host}{$host_name}{$item}{EXCLUDE_TIME} = $exclude_time;
    }

  } ## end foreach my $line ( @{ $inventory_data...})
  delete $inventory_data{"DATA_LINE"};
  if ( !defined $inventory_data{$storage} ) { return 1; }
  my $size_hash = keys %inventory_data;
  if ( $size_hash == 0 ) { return 1; }

  #print Dumper \%inventory_data;
  $type_act = "HOST";
  foreach my $line (@host_cfg) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line =~ s/^\s+|\s+$//g;
    my ( $host_name, $volume ) = split( " : ", $line );
    $host_name =~ s/^\s+|\s+$//g;
    if ( defined $inventory_data{$storage} && $inventory_data{$storage}{$type_act} && defined $inventory_data{$storage}{$type_act}{$host_name} ) {
      if ( !defined $volume || $volume eq "" ) {
        warning( "$type_act: $storage:$host_name Host $host_name no connection volume " . __FILE__ . ":" . __LINE__ );
        next;
      }
      my @volumes = split( " ", $volume );
      foreach my $vol (@volumes) {
        chomp $vol;
        if ( $vol eq "" ) { next; }
        $vol =~ s/^\s+|\s+$//g;
        $vol =~ s/ //g;
        $vol =~ s/;$//;
        if ( $st_type =~ m/DS8K/ ) {
          $vol =~ s/^0x//;
        }
        push @{ $inventory_data{$storage}{$type_act}{$host_name}{VOLUME} }, "$vol";
      } ## end foreach my $vol (@volumes)
    } ## end if ( defined $inventory_data...)
    else { delete $inventory_data{$storage}{$type_act}{$host_name}; }
  } ## end foreach my $line (@host_cfg)
  $type_act = "CUSTOM";
  my %custom_test;
  my %custom_test_duplicit;
  #print Dumper \%inventory_data;
  foreach my $line (@custom_cfg) {
    chomp $line;
    if ( $line eq "" || $line !~ m/^VOLUME|^HOST/ ) { next; }
    $line =~ s/^\s+|\s+$//g;
    $line =~ s/\\:/===========doublecoma=========/g;
    ( undef, my $stor, my $vol, my $custom_name ) = split( ":", $line );

    ### test custom volumes from one storage
    $custom_test{$custom_name}{STORAGE}{$stor} = $stor;
    my $size_custom = keys %{ $custom_test{$custom_name}{STORAGE} };
    if ( $size_custom > 1 ) {

      #delete $inventory_data{$storage}{$type_act}{$custom_name};
      if ( defined $custom_test{$custom_name}{STORAGE}{$storage} ) {

        #print "Warning        : $type_act: $storage:$custom_name: Custom $custom_name has volumes of more storages -- ALERT_TEST ignoring it \n" if $DEBUG == 22;
        $inventory_data{$storage}{$type_act}{$custom_name}{"WARNING"} = "true";
      }
      else {
        delete $custom_test{$custom_name}{STORAGE}{$storage};
      }
      next;
    } ## end if ( $size_custom > 1 )
    ###

    if ( $stor ne $storage ) { next; }
    $vol =~ s/===========doublecoma=========/:/g;
    $custom_name =~ s/===========doublecoma=========/:/g;
    my $volume_id = "";
    if ( defined $inventory_data{$storage}{$type_act} && defined $inventory_data{$storage}{$type_act}{$custom_name} && defined $inventory_data{$storage}{$type_act}{$custom_name}{TYPE} && $inventory_data{$storage}{$type_act}{$custom_name}{TYPE} eq "VOLUME") {
      foreach my $vol_act (@vol_cfg) {
        chomp($vol_act);
        if ( $vol_act =~ m/^$vol : / ) {
          ( my $name, $volume_id ) = split( / : /, $vol_act );
          $volume_id =~ s/ //g;
          last;
        }
      } ## end foreach my $vol_act (@vol_cfg)
      $volume_id =~ s/ //g;
      $volume_id =~ s/;$//;
      if ( $st_type =~ m/DS8K/ ) {
        $volume_id =~ s/^0x//;
      }
      my $file_time_stamp = 0;
      if ( $volume_id =~ m/;/ ) {

        # Multilevel volumes (DS8k), 1 volume name might contain several physical volumes
        foreach my $volume_act ( split( /;/, $volume_id ) ) {
          $file_time_stamp = 0;
          $volume_act =~ s/^0x//;
          foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$volume_act*.rrd>) {

            # there can be volumes with pool identification in the name (DS5k and XIV at least)  vol_name-Pxy.rrd
            # select the most actual one as per the file timestamp
            my $file_time_stamp_act = ( stat("$vol_file_tmp") )[9];
            if ( $file_time_stamp_act > $file_time_stamp ) {
              push @{ $inventory_data{$storage}{$type_act}{$custom_name}{VOLUME} }, "$vol_file_tmp";
            }
          } ## end foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$volume_act*.rrd>)
        } ## end foreach my $volume_act ( split...)
      } ## end if ( $volume_id =~ m/;/)
      else {
        # Normal volumes just mapping ID to 1 file name
        foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$volume_id*.rrd>) {

          # there can be volumes with pool identification in the name (DS5k and XIV at least)  vol_name-Pxy.rrd
          # select the most actual one as per the file timestamp
          my $file_time_stamp_act = ( stat("$vol_file_tmp") )[9];
          if ( $file_time_stamp_act > $file_time_stamp ) {
            push @{ $inventory_data{$storage}{$type_act}{$custom_name}{VOLUME} }, "$vol_file_tmp";
          }
        } ## end foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$volume_id*.rrd>)
      } ## end else [ if ( $volume_id =~ m/;/)]
    } ## end if ( defined $inventory_data...)
    else {
      if ( defined $inventory_data{$storage}{$type_act} && defined $inventory_data{$storage}{$type_act}{$custom_name} && defined $inventory_data{$storage}{$type_act}{$custom_name}{TYPE} && $inventory_data{$storage}{$type_act}{$custom_name}{TYPE} eq "HOST") {
        foreach my $line (@host_cfg) {
          chomp $line;
          if ( $line eq "" ) { next; }
          $line =~ s/^\s+|\s+$//g;
          my ( $host_name, $volume ) = split( " : ", $line );
          $host_name =~ s/^\s+|\s+$//g;
          if ($vol ne $host_name){next;}
          if ( defined $inventory_data{$storage} && $inventory_data{$storage}{$type_act} && defined $inventory_data{$storage}{$type_act}{$custom_name} ) {
            if ( !defined $volume || $volume eq "" ) {
              warning( "$type_act: $storage:$host_name Host $host_name no connection volume " . __FILE__ . ":" . __LINE__ );
              next;
            }
            my @volumes = split( " ", $volume );
            foreach my $vol (@volumes) {
              chomp $vol;
              if ( $vol eq "" ) { next; }
              $vol =~ s/^\s+|\s+$//g;
              $vol =~ s/ //g;
              $vol =~ s/;$//;
              if ( $st_type =~ m/DS8K/ ) {
                $vol =~ s/^0x//;
              }
              my $file_time_stamp = 0;
              if ($st_type ne "DS8K"){
                if (defined $inventory_volume{$vol} && defined $inventory_volume{$vol}{"ID_ORIGINAL"} && $inventory_volume{$vol}{"ID_ORIGINAL"} ne ""){
                  my $vol_t = $inventory_volume{$vol}{"ID_ORIGINAL"};
	        }
                else{next;}
              }
              foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$vol*.rrd>) {
                my $file_time_stamp_act = ( stat("$vol_file_tmp") )[9];
                if ( $file_time_stamp_act > $file_time_stamp ) {
                  if (!defined $custom_test_duplicit{$storage}{$custom_name}{$vol}{TEST}){
                    push @{ $inventory_data{$storage}{$type_act}{$custom_name}{VOLUME} }, "$vol_file_tmp";
                    $custom_test_duplicit{$storage}{$custom_name}{$vol}{TEST} = $vol;
                  }
                }
              }
            }
            if ( $st_type =~ m/VSPG|3PAR|VMAX|OCEANSTOR|COMPELLENT|PURE/ ) {
              foreach my $host_file (<$wrkdir/$storage/HOST/*.rrd>) {
                my $file_time_stamp_act = ( stat("$host_file") )[9];
                my $act_time            = time();
                if ( $host_file =~ m/$host_name\.rrd$/ && $act_time - $file_time_stamp_act < 3600 ) {
                  if (!defined $custom_test_duplicit{$storage}{$custom_name}{$host_name}{TEST}){
                    push @{ $inventory_data{$storage}{$type_act}{$custom_name}{HOST} }, "$host_file";
                    $custom_test_duplicit{$storage}{$custom_name}{$host_name}{TEST} = $host_name;
                  }
                  last;
                }
              }
            } ## end foreach my $vol (@volumes)
          } ## end if ( defined $inventory_data...)
          else { delete $inventory_data{$storage}{$type_act}{$custom_name}; }
        } ## end foreach my $line (@host_cfg)
      }
      else{
        delete $inventory_data{$storage}{$type_act}{$custom_name};
      }
    }
  } ## end foreach my $line (@custom_cfg)
  #print Dumper \%inventory_data;
  foreach my $type (@type) {
    if ( !defined $inventory_data{$storage}{$type} ) { next; }
    foreach my $host ( keys %{ $inventory_data{$storage}{$type} } ) {
      my $type_data = "";
      my $text = "";
      if ( defined $inventory_data{$storage}{$type}{$host}{"TYPE"}){
          $type_data = $inventory_data{$storage}{$type}{$host}{"TYPE"};
        if ($type_data eq "VOLUME"){$text = "volumes";}
        if ($type_data eq "HOST"){$text = "hosts";}
      }
      foreach my $item ( keys %{ $inventory_data{$storage}{$type}{$host} } ) {
        if ( $item eq "EMAIL" || $item eq "STORAGE" ) { next; }
        if ( $item eq "VOLUME" || $item eq "WARNING" || $item eq "HOST" || $item eq "TYPE" ) {
          next;
        }
        if ( defined $inventory_data{$storage}{$type}{$host}{VOLUME} ) {
          @volumes_id = @{ $inventory_data{$storage}{$type}{$host}{VOLUME} };
        }
        else {
          if ( $type eq "CUSTOM") {
            if ( defined $inventory_data{$storage}{$type}{$host}{"WARNING"} ) {
              my $line = $inventory_data{$storage}{$type}{$host}{$item}{LINE};
              if ( !defined $line ) {
                $line = "";
              }
              print "Working for    : $type: $storage:$host:$item : $line\n"                                                              if $DEBUG == 22;
              print "Get utilization: $type: $storage:$host:$item Custom $host has $text of more storages -- ALERT_TEST ignoring it \n" if $DEBUG == 22;
              error("Custom $host has $text of more storages. Alerting is not supported!");
            } ## end if ( defined $inventory_data...)
          } ## end if ( $type eq "CUSTOM")
          next;
        } ## end else [ if ( defined $inventory_data...)]
        if ($type eq "CUSTOM" && defined $inventory_data{$storage}{$type}{$host}{HOST} && ref($inventory_data{$storage}{$type}{$host}{HOST}) eq "ARRAY"){
          @volumes_id = ();
          @volumes_id = @{ $inventory_data{$storage}{$type}{$host}{HOST} };
        }
        my $stor         = "";
        my $limit        = $inventory_data{$storage}{$type}{$host}{$item}{LIMIT};
        my $peak_time    = $inventory_data{$storage}{$type}{$host}{$item}{PEAK_TIME};
        my $repeat_time  = $inventory_data{$storage}{$type}{$host}{$item}{REPEAT_TIME};
        my $exclude_time = $inventory_data{$storage}{$type}{$host}{$item}{EXCLUDE_TIME};
        my $email        = $inventory_data{$storage}{$type}{$host}{$item}{EMAIL};
        my $line         = $inventory_data{$storage}{$type}{$host}{$item}{LINE};

        print "Working for    : $type: $storage:$host:$item : $line\n" if $DEBUG == 22;

        if ( $type eq "CUSTOM" && $type_data eq "VOLUME" ) {
          if ( defined $inventory_data{$storage}{$type}{$host}{"WARNING"} ) {
            print "Get utilization: $type: $storage:$host:$item Custom $host has volumes of more storages -- ALERT_TEST ignoring it \n" if $DEBUG == 22;
            error("Custom $host has volumes of more storages. Alerting is not supported!");
            next;
          }
        } ## end if ( $type eq "CUSTOM")
        if ( $type eq "CUSTOM" && $type_data eq "HOST" ) {
          if ( defined $inventory_data{$storage}{$type}{$host}{"WARNING"} ) {
            print "Get utilization: $type: $storage:$host:$item Custom $host has hosts of more storages -- ALERT_TEST ignoring it \n" if $DEBUG == 22;
            error("Custom $host has hosts of more storages. Alerting is not supported!");
            next;
          }
        } ## end if ( $type eq "CUSTOM")

        # check if the time is not excluded.
        if ( defined($exclude_time) && !$exclude_time eq '' ) {
          my ( $hour_start, $hour_end ) = split( /\-/, $exclude_time );
          $hour_start =~ s/ //g;
          if ( !defined($hour_end) ) {

            #error ("$volume_type: $storage:$volume:$item : wrong format of exclude time ($exclude_time), it should be like 1-5 to exclude time pro 1am to 5am, ignoring it ".__FILE__.":".__LINE__);
            $hour_end = $hour_start;
          }
          else {
            $hour_end =~ s/ //g;
            if ( isdigit($hour_start) && isdigit($hour_end) ) {
              my $cycle   = 0;
              my $exclude = 0;
              for ( my $hour_count = $hour_start; $cycle < 25; $cycle++ ) {

                #print "         $hour_count == $hour_act\n";
                if ( $hour_count == $hour_act ) {

                  #print " got it  $hour_count == $hour_act\n";
                  $exclude = 1;
                }
                $hour_count++;
                if ( $hour_count == 25 ) {
                  $hour_count = 0;
                }
                if ( $hour_count == $hour_end ) {
                  last;
                }
              } ## end for ( my $hour_count = ...)
              if ( $exclude == 1 ) {
                print "Excluded time  : $type: $storage:$host:$item : excluded due to time exclusion: $exclude_time (act time:$hour_act)\n" if $DEBUG == 22;
                next;
              }
              else {
                print "Excl time ok   : $type: $storage:$host:$item : $hour_start - $hour_end\n" if $DEBUG == 22;
              }
            } ## end if ( isdigit($hour_start...))
            else {
              error( "$type: $storage:$host:$item : wrong format of exclude time ($exclude_time), it should be like 1-5 to exclude time pro 1am to 5am, ignoring it " . __FILE__ . ":" . __LINE__ );
            }
          } ## end else [ if ( !defined($hour_end...))]
        } ## end if ( defined($exclude_time...))

        # check retention time
        my ( $last_alert_time_u, $last_alert_time ) = get_last_alert_time( "no update", "$storage:$host:$item", $time_act, $time_act_u, $DEBUG, \@repeat_list );
        if ( $time_act_u < $last_alert_time_u + $repeat_time * 60 ) {

          # retention time does not allow alarming
          if ( $alert_test == 0 ) {
            print "Retention      : $type: $storage:$host:$item : repeat time supress alarming now, last: $last_alert_time, retention: $repeat_time\n" if $DEBUG == 22;
            next;
          }
          else {
            print "Retention      : $type: $storage:$host:$item : repeat time supress alarming now, last: $last_alert_time, retention: $repeat_time -- ALERT_TEST ignoring it \n" if $DEBUG == 22;
          }
        } ## end if ( $time_act_u < $last_alert_time_u...)
        @volumes_rrd = ();
        my $vspg = 0;
        ### exception for VSP
        if ( $type eq "HOST" ) {
          if ( $st_type =~ m/VSPG|3PAR|VMAX|OCEANSTOR|COMPELLENT|PURE/ ) {
            foreach my $host_file (<$wrkdir/$storage/$type/*.rrd>) {
              my $file_time_stamp_act = ( stat("$host_file") )[9];
              my $act_time            = time();
              if ( $host_file =~ m/$host\.rrd$/ && $act_time - $file_time_stamp_act < 3600 ) {
                push( @volumes_rrd, "$host_file" );
                $vspg = 1;
                last;
              }
            } ## end foreach my $host_file (<$wrkdir/$storage/$type/*.rrd>)
            print "HOST           : $type: $storage:$host:$item : host files has not been found or it is too old (modification older than 1h), skipping to volume \n" if $DEBUG == 22;
          } ## end if ( $st_type =~ m/VSPG|3PAR|VMAX/)
          if ( $st_type !~ m/VSPG|3PAR|VMAX|OCEANSTOR|COMPELLENT|PURE/ || $vspg == 0 ) {

            # for all other storage which do not have direct host data or if host data file does not exist
            foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/*.rrd>) {
              foreach my $id (@volumes_id) {
                my $name = basename($vol_file_tmp);
                if ( $name =~ m/^$id.*\.rrd$/ && $vspg == 0 ) {

                  #$mapping_volume_id_rrd{$id} = $vol_file_tmp;
                  push( @volumes_rrd, "$vol_file_tmp" );
                  last;
                } ## end if ( $name =~ m/^$id.*\.rrd$/...)
              } ## end foreach my $id (@volumes_id)
            } ## end foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/*.rrd>)
          } ## end if ( $st_type !~ m/VSPG|3PAR|VMAX/...)
        } ## end if ( $type eq "HOST" )
        if ( $type eq "CUSTOM" ) {
          #### CUSTOM section in hash array rrd file volume
          @volumes_rrd = @volumes_id;
        }

        my $size = scalar @volumes_rrd;
        if ( $size == 0 ) {
          print "Get utilization: $type: $storage:$host:$item : no DB file has been identified: \n" if $DEBUG == 22;
          next;
        }

        # get utilization
        my $graph_path = "$basedir/tmp/alert-graph-$storage-$item.png";
        if ( $alert_test == 1 ) {
          my $username = $ENV{LOGNAME} || $ENV{USER} || getpwuid($<);
          $graph_path = "/var/tmp/stor2rrd-alert-$username-graph-$storage-$item.png";
        }

        # do not create graph unnecessary --> 0 instead of $email_graph
        #print Dumper \@volumes_rrd;
        my $limit_act = get_util( 0, "VOLUME", $graph_path, $storage, $st_type, $item, $time_act, $peak_time, $time_act_u, $host, $DEBUG, $basedir, $ds5_v2, \@volumes_rrd, $type );
        $inventory_data{$storage}{$type}{$host}{$item}{VALUE} = $limit_act;
        if ( $limit_act == -1 ) {
          print "Util           : $type: $storage:$host:$item : some problem encountered, continuing with next rule\n" if $DEBUG == 22;
          next;    # something was wrong
        }
        print "Util           : $type: $storage:$host:$item : $limit_act (limit is $limit)\n" if $DEBUG == 22;

        # check if it is over the limit
        if ( $limit_act < $limit ) {
          if ( $alert_test == 0 ) {
            next;
          }
        }

        # check retention time
        ( $last_alert_time_u, $last_alert_time, my @repeat_list_passed ) = get_last_alert_time( "update", "$storage:$host:$item", $time_act, $time_act_u, $DEBUG, \@repeat_list );
        if ( $time_act_u < $last_alert_time_u + $repeat_time * 60 ) {

          # retention time does not allow alarming
          if ( $alert_test == 0 ) {
            print "Retention      : $type: $storage:$host:$item : repeat time supress alarming now, last: $last_alert_time, retention: $repeat_time\n" if $DEBUG == 22;
            next;
          }
          else {
            print "Retention      : $type: $storage:$host:$item : repeat time supress alarming now, last: $last_alert_time, retention: $repeat_time -- ALERT_TEST ignoring it \n" if $DEBUG == 22;
          }
        } ## end if ( $time_act_u < $last_alert_time_u...)
        @repeat_list = @repeat_list_passed;    # must be here to do not overwrite the record when no alarm is issued

        # issue the alert
        if ( $alert_test == 0 ) {
          print "Alerting       : $type: $storage:$host:$item : utilization is over the limit ($limit_act > $limit) alerting\n" if $DEBUG == 22;
          log_to_alert_log( $time_act, $storage, $host, $item, $limit_act, $limit, $email, $type );

          #if ($DEBUG == 22){
          #  if ((defined $VI_IMAGE && $VI_IMAGE eq "1") || (defined $VM_IMAGE && $VM_IMAGE eq "1")){
          #    log_to_alert_log($time_act,$storage,$host,$item,$limit_act,$limit,$email,$type);
          #  }
          #  else{
          #    my $report = "Alert has not been written into $config_alert[$ALERT_HISTORY]";
          #    warning($report);
          #  }
          #}
          #else{
          #  log_to_alert_log($time_act,$storage,$host,$item,$limit_act,$limit,$email,$type);
          #}
        } ## end if ( $alert_test == 0 )
        else {
          if ( $limit_act > $limit ) {
            print "Alerting       : $type: $storage:$host:$item : utilization is over the limit ($limit_act > $limit) alerting anyway --ALERT_TEST\n" if $DEBUG == 22;
          }
          else {
            print "Alerting       : $type: $storage:$host:$item : utilization is NOT over the limit ($limit_act < $limit) alerting anyway --ALERT_TEST\n" if $DEBUG == 22;
          }
        } ## end else [ if ( $alert_test == 0 )]

        if ( $email_graph > 0 ) {

          # create graph now when there is sure that it will be send out
          get_util( $email_graph, $type, $graph_path, $storage, $st_type, $item, $time_act, $peak_time, $time_act_u, $host, $DEBUG, $basedir, $ds5_v2, \@volumes_rrd );
        }
        if ( defined($email) && !$email eq '' ) {
          sendmail( $type, $config_alert[$EMAIL_GRAPH], $graph_path, $email, $time_act, $peak_time, $storage, $host, $item, $limit_act, $limit, $line_email_no, $DEBUG, \@config_email );
        }

        if ( $config_alert[$NAGIOS] == 1 ) {

          # nagios alarm
          nagios_alarm( $type, $storage, $host, $item, $limit_act, $limit, $time_act, $nagios_dir, $DEBUG, $alert_test );
        }

        # SNMP TRAP
        if ( defined($snmp_trap) && !$snmp_trap eq '' && $snmp_trap !~ m/your_snmp_trap_server/ ) {
          snmp_trap_alarm( $type, $storage, $host, $item, $limit_act, $limit, $time_act, $snmp_trap, $DEBUG, $basedir );
        }

        # extern alert
        if ( defined( $config_alert[$EXTERN_ALERT] ) && !$config_alert[$EXTERN_ALERT] eq '' ) {
          extern_alarm( $type, $storage, $host, $item, $limit_act, $limit, $time_act, $config_alert[$EXTERN_ALERT], $DEBUG, $basedir );
        }
      } ## end foreach my $item ( keys %{ ...})
    } ## end foreach my $host ( keys %{ ...})
  } ## end foreach my $type (@type)

  update_repeat_table( $alert_test, $alert_repeat_file, \@repeat_list );
  return 1;
} ## end sub check_data_host

sub get_inventory_volume {
  my $data_tmp    = shift;
  my $st_type = shift;
  my @data = @{$data_tmp};
  my %inventory_volume;
  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line =~ s/^\s+|\s+$//g;
    ( my $name, my $id ) = split( " : ", $line );
    my $id_original = $id;
    if ( defined $name && defined $id ) {
      if ( $st_type eq "SWIZ" ) {
        $id =~ s/^0*//;
        if ( $id eq "" || $id eq ";" ) {
          $id = 0;
        }
      }
      if ( $st_type eq "DS8K" ) {
        $id =~ s/^0x//;
      }
      $id =~ s/;$//;
      $id_original =~ s/;$//;
      $inventory_volume{$id}{NAME} = $name;
      $inventory_volume{$id}{ID} = $id;
      $inventory_volume{$id}{"ID_ORIGINAL"} = $id_original;
    } ## end if ( defined $name && ...)
  } ## end foreach my $line (@data)
  return %inventory_volume;
}

sub check_data {
  my ( $basedir, $storage, $st_type, $time_act, $time_act_u, $wrkdir, $DEBUG, $nagios_dir, $config_alert_data_tmp, $VI_IMAGE, $VM_IMAGE ) = @_;
  my @config_alert_data = @{$config_alert_data_tmp};
  my $vol_cfg_file      = "$wrkdir/$storage/VOLUME/volumes.cfg";
  my $pool_cfg_file     = "$wrkdir/$storage/pool.cfg";
  my $alert_repeat_file = "$basedir/tmp/alert_repeat-stor2rrd-$storage.tmp";
  my ( $sec, $min, $hour_act ) = localtime();
  my $email_graph = $config_alert[$EMAIL_GRAPH];
  my $snmp_trap   = "";

  my $ds5_v2 = 0;
  if ( -f "$wrkdir/$storage/DS5K-v2" ) {
    $ds5_v2 = 1;    # DS5k v1 does not have response time
  }

  # Volume translation
  if ( !-f "$vol_cfg_file" && !-f "$pool_cfg_file" ) {

    #warning ("Do not exist  $vol_cfg_file and $pool_cfg_file ".__FILE__.":".__LINE__);
    return 1;       #make no sense to continue
  }

  my @vol_cfg = "";
  if ( -f "$vol_cfg_file" ) {
    open( FV, "< $vol_cfg_file" ) || error( "Can't open $vol_cfg_file $! " . __FILE__ . ":" . __LINE__ );    # continue here
    @vol_cfg = <FV>;
    close(FV);
  }

  #  warning ("Does not exist volume cfg file: $vol_cfg_file ".__FILE__.":".__LINE__);

  my @pool_cfg = "";
  if ( -f "$pool_cfg_file" ) {
    open( FV, "< $pool_cfg_file" ) || error( "Can't open $pool_cfg_file $! " . __FILE__ . ":" . __LINE__ );    # continue here
    @pool_cfg = <FV>;
    close(FV);
  }

  #  warning ("Does not exist pool cfg file: $pool_cfg_file ".__FILE__.":".__LINE__);

  my @repeat_list = "";
  if ( -f $alert_repeat_file ) {
    open( FHR, "< $alert_repeat_file" );
    @repeat_list = sort(<FHR>);
    close(FHR);
  }

  my $volume_prev = "NA";
  my $volume_id   = "NA";
  my @vol_file    = "";
  $vol_file[0] = "NA";

  my @config_email  = "";
  my $line_email_no = 0;

  # separate EMAIL
  foreach my $line (@config_alert_data) {
    chomp($line);
    if ( $line =~ m/^EMAIL/ ) {
      $config_email[$line_email_no] = $line;
      $line_email_no++;
    }

  } ## end foreach my $line (@config_alert_data)

  # go through line by line with alerts
  foreach my $line (@config_alert_data) {
    chomp($line);
    if ( $line =~ m/^EMAIL/ ) {
      next;
    }
    if ( $line =~ m/^TRAP=/ ) {
      ( undef, $snmp_trap ) = split( /=/, $line );
      $snmp_trap =~ s/ //g;
      next;
    }

    ( my $volume_type, my $storage_act, my $volume, my $item, my $limit, my $peak_time, my $repeat_time, my $exclude_time, my $email ) = split( /:/, $line );

    if ( !defined($storage_act) || $storage_act eq '' || $storage !~ m/^$storage_act$/ ) {

      # email can be empty, then is used only for externa purposes like external scrip[t or nagios
      next;    # ignoring, it is not specified the storage
    }
    if ( ( $volume_type =~ m/^VOLUME$/ || $volume_type =~ m/^POOL$/ ) && ( $storage_act eq '' || $volume eq '' || $item eq '' || $limit eq '' ) ) {
      warning( "$volume_type: $storage: alerting rule does not contain all required fields: $line " . __FILE__ . ":" . __LINE__ );
      next;
    }
    if ( $volume_type =~ m/^POOL-ALL$/ && ( $storage_act eq '' || $item eq '' || $limit eq '' ) ) {
      warning( "$volume_type: $storage: alerting rule does not contain all required fields: $line " . __FILE__ . ":" . __LINE__ );
      next;
    }
    if ( !defined($repeat_time) || $repeat_time eq '' || !isdigit($repeat_time) ) {

      #use the global one
      $repeat_time = $config_alert[$REPEAT_DEFAULT];
    }
    if ( !defined($peak_time) || $peak_time eq '' || !isdigit($peak_time) ) {

      #use the global one
      $peak_time = $config_alert[$PEAK_TIME_DEFAULT];
    }
    $storage_act =~ s/===========doublecoma=========/:/g;
    if ( defined($volume) && !$volume eq '' ) {
      $volume =~ s/===========doublecoma=========/:/g;
    }
    if ( defined($email) && !$email eq '' ) {
      $email =~ s/===========doublecoma=========/:/g;
    }
    $line =~ s/===========doublecoma=========/:/g;

    print "Working for    : $volume_type: $storage:$volume:$item : $line\n" if $DEBUG == 22;

    # check if the time is not excluded.
    if ( defined($exclude_time) && !$exclude_time eq '' ) {
      my ( $hour_start, $hour_end ) = split( /\-/, $exclude_time );
      $hour_start =~ s/ //g;
      if ( !defined($hour_end) ) {

        #error ("$volume_type: $storage:$volume:$item : wrong format of exclude time ($exclude_time), it should be like 1-5 to exclude time pro 1am to 5am, ignoring it ".__FILE__.":".__LINE__);
        $hour_end = $hour_start;
      }
      else {
        $hour_end =~ s/ //g;
        if ( isdigit($hour_start) && isdigit($hour_end) ) {
          my $cycle   = 0;
          my $exclude = 0;
          for ( my $hour_count = $hour_start; $cycle < 25; $cycle++ ) {

            #print "         $hour_count == $hour_act\n";
            if ( $hour_count == $hour_act ) {

              #print " got it  $hour_count == $hour_act\n";
              $exclude = 1;
            }
            $hour_count++;
            if ( $hour_count == 25 ) {
              $hour_count = 0;
            }
            if ( $hour_count == $hour_end ) {
              last;
            }
          } ## end for ( my $hour_count = ...)
          if ( $exclude == 1 ) {
            print "Excluded time  : $volume_type: $storage:$volume:$item : excluded due to time exclusion: $exclude_time (act time:$hour_act)\n" if $DEBUG == 22;
            next;
          }
          else {
            print "Excl time ok   : $volume_type: $storage:$volume:$item : $hour_start - $hour_end\n" if $DEBUG == 22;
          }
        } ## end if ( isdigit($hour_start...))
        else {
          error( "$volume_type: $storage:$volume:$item : wrong format of exclude time ($exclude_time), it should be like 1-5 to exclude time pro 1am to 5am, ignoring it " . __FILE__ . ":" . __LINE__ );
        }
      } ## end else [ if ( !defined($hour_end...))]
    } ## end if ( defined($exclude_time...))

    # check retention time
    my ( $last_alert_time_u, $last_alert_time ) = get_last_alert_time( "no update", "$storage:$volume:$item", $time_act, $time_act_u, $DEBUG, \@repeat_list );
    if ( $time_act_u < $last_alert_time_u + $repeat_time * 60 ) {

      # retention time does not allow alarming
      if ( $alert_test == 0 ) {
        print "Retention      : $volume_type: $storage:$volume:$item : repeat time supress alarming now, last: $last_alert_time, retention: $repeat_time\n" if $DEBUG == 22;
        next;
      }
      else {
        print "Retention      : $volume_type: $storage:$volume:$item : repeat time supress alarming now, last: $last_alert_time, retention: $repeat_time -- ALERT_TEST ignoring it \n" if $DEBUG == 22;
      }
    } ## end if ( $time_act_u < $last_alert_time_u...)

    if ( $volume_type =~ m/^VOLUME$/ ) {

      # Single volumes
      if ( $volume !~ m/^$volume_prev$/ ) {

        # skip that if the same volume next alert line
        $volume_id = "NA";
        foreach my $vol_act (@vol_cfg) {
          chomp($vol_act);
          if ( $vol_act =~ m/^$volume : / ) {
            ( my $name, $volume_id ) = split( / : /, $vol_act );
            $volume_id =~ s/ //g;
            last;
          }
        } ## end foreach my $vol_act (@vol_cfg)

        if ( $volume_id =~ m/^NA$/ ) {

          #error ("$storage:$volume:$item  Volume \"$volume\" has not been found in $vol_cfg_file ".__FILE__.":".__LINE__);
          # Just warning
          #warning ("$volume_type: $storage:$volume:$item  Volume \"$volume\" has not been found in $vol_cfg_file ".__FILE__.":".__LINE__);
          next;
        } ## end if ( $volume_id =~ m/^NA$/)
        $volume_id =~ s/ //g;
        $volume_id =~ s/;$//;
        if ( $st_type =~ m/DS8K/ ) {
          $volume_id =~ s/^0x//;
        }

        #print "03 $volume : $volume_id\n";

        @vol_file = "";
        $vol_file[0] = "NA";
        my $file_time_stamp = 0;

        if ( $volume_id =~ m/;/ ) {

          # Multilevel volumes (DS8k), 1 volume name might contain several physical volumes
          my $vol_file_indx = 0;
          foreach my $volume_act ( split( /;/, $volume_id ) ) {
            $file_time_stamp = 0;
            $volume_act =~ s/^0x//;
            foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$volume_act*.rrd>) {

              # there can be volumes with pool identification in the name (DS5k and XIV at least)  vol_name-Pxy.rrd
              # select the most actual one as per the file timestamp
              my $file_time_stamp_act = ( stat("$vol_file_tmp") )[9];
              if ( $file_time_stamp_act > $file_time_stamp ) {
                $vol_file[$vol_file_indx] = $vol_file_tmp;
                $vol_file_indx++;
              }
            } ## end foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$volume_act*.rrd>)
          } ## end foreach my $volume_act ( split...)

        } ## end if ( $volume_id =~ m/;/)
        else {
          # Normal volumes just mapping ID to 1 file name
          foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$volume_id*.rrd>) {

            # there can be volumes with pool identification in the name (DS5k and XIV at least)  vol_name-Pxy.rrd
            # select the most actual one as per the file timestamp
            my $file_time_stamp_act = ( stat("$vol_file_tmp") )[9];
            if ( $file_time_stamp_act > $file_time_stamp ) {
              $vol_file[0] = $vol_file_tmp;
            }
          } ## end foreach my $vol_file_tmp (<$wrkdir/$storage/VOLUME/$volume_id*.rrd>)
        } ## end else [ if ( $volume_id =~ m/;/)]

        if ( $vol_file[0] =~ m/^NA$/ ) {

          #error ("$volume_type: $storage:$volume:$item ($volume_id) volume file has not been found  ".__FILE__.":".__LINE__);
          next;
        }
      } ## end if ( $volume !~ m/^$volume_prev$/)
      if ( $DEBUG == 22 ) {
        foreach my $vol_file_act (@vol_file) {
          print "Volume file    : $volume_type: $storage:$volume:$item : $vol_file_act \n";
        }
      }
      $volume_prev = $volume;

      # Volume file has been identified!
    } ## end if ( $volume_type =~ m/^VOLUME$/)
    else {
      # clean up every run, do not do it for Volumes as there is act vrs previous and when they same then no new volume name discovery
      @vol_file = "";
      $vol_file[0] = "NA";
      if ( $volume_type =~ m/^POOL$/ ) {

        # POOL: Single pools

        # pool name --> id translation
        my $pool_id = -1;
        my $found   = 0;
        foreach my $linep (@pool_cfg) {
          chomp($linep);
          if ( $linep =~ m/^$/ || $linep !~ m/:/ ) {
            next;
          }
          ( my $id, my $name_pool ) = split( /:/, $linep );
          if ( defined($name_pool) && $volume =~ m/^$name_pool$/ ) {

            # do not use above construction "! $name_pool eq ''" as it does not work when a pool name == 0 (like for HUS for example)
            $pool_id = $id;
            $found   = 1;
            last;
          } ## end if ( defined($name_pool...))
        } ## end foreach my $linep (@pool_cfg)
        if ( $found == 0 ) {

          # must be used $found, this does not help as pool id can be UUIDs (like netapp) $pool_id == -1
          #warning ("$volume_type: $storage:$volume:$item  Pool \"$volume\" has not been found in $pool_cfg_file ".__FILE__.":".__LINE__);
          next;
        } ## end if ( $found == 0 )
        $vol_file[0] = "$wrkdir/$storage/POOL/$pool_id.rrd";
        if ( !-f $vol_file[0] ) {

          #warning ("$volume_type: $storage:$volume:$item  Pool \"$volume\" has not been found data file: $vol_file[0] ".__FILE__.":".__LINE__);
          next;
        }
        if ( $DEBUG == 22 ) {
          print "Pool file      : $volume_type: $storage:$volume:$item : $vol_file[0] \n";
        }
      } ## end if ( $volume_type =~ m/^POOL$/)
      else {
        if ( $volume_type =~ m/^POOL-ALL$/ ) {

          # POOL-ALL --> select all pools (it is furher checked for file timestamp, do not do it here)
          # If storage has pool data separated in POOL then use it, this is much faster
          $volume = "all pools";

          # all storages should have POOL data files since 1.30 at least for alerting purposes
          my $vol_file_indx = 0;
          foreach my $vol_file_tmp (<$wrkdir/$storage/POOL/*.rrd>) {
            if ( $vol_file_tmp =~ m/-cap.rrd$/ ) {
              next;    # skip capacity files
            }
            $vol_file[$vol_file_indx] = $vol_file_tmp;
            $vol_file_indx++;
          } ## end foreach my $vol_file_tmp (<$wrkdir/$storage/POOL/*.rrd>)
          if ( $vol_file_indx == 0 ) {
            warning( "$volume_type: $storage:$item: no data has been found : $wrkdir/$storage/POOL/*.rrd " . __FILE__ . ":" . __LINE__ );
            if ( $alert_test == 0 ) {
              next;    # when test alert then continue
            }
          } ## end if ( $vol_file_indx ==...)
        } ## end if ( $volume_type =~ m/^POOL-ALL$/)
        else {
          warning( "$volume_type: $storage:$volume:$item some trash " . __FILE__ . ":" . __LINE__ );
          next;
        }
      } ## end else [ if ( $volume_type =~ m/^POOL$/)]
    } ## end else [ if ( $volume_type =~ m/^VOLUME$/)]

    my $indx = 0;
    foreach my $vol_file_act (@vol_file) {
      if ( $vol_file_act =~ m/^NA$/ ) {
        next;
      }
      $indx++;
    } ## end foreach my $vol_file_act (@vol_file)
    if ( $indx == 0 ) {
      print "Get utilization: $volume_type: $storage:$volume:$item : no DB file has been identified: \n" if $DEBUG == 22;
      next;
    }

    # get utilization
    my $graph_path = "$basedir/tmp/alert-graph-$storage-$item.png";
    if ( $alert_test == 1 ) {
      my $username = $ENV{LOGNAME} || $ENV{USER} || getpwuid($<);
      $graph_path = "/var/tmp/stor2rrd-alert-$username-graph-$storage-$item.png";
    }

    # do not create graph unnecessary --> 0 instead of $email_graph
    my $limit_act = get_util( 0, $volume_type, $graph_path, $storage, $st_type, $item, $time_act, $peak_time, $time_act_u, $volume, $DEBUG, $basedir, $ds5_v2, \@vol_file );
    if ( $limit_act == -1 ) {
      if ( $alert_test == 0 ) {
        print "Util           : $volume_type: $storage:$volume:$item : some problem encountered, continuing with next rule\n" if $DEBUG == 22;
        next;    # something was wrong
      }
    } ## end if ( $limit_act == -1 )
    print "Util           : $volume_type: $storage:$volume:$item : $limit_act (limit is $limit)\n" if $DEBUG == 22;

    # check if it is over the limit
    if ( $limit_act < $limit ) {
      if ( $alert_test == 0 ) {
        next;
      }
    }

    # check retention time
    ( $last_alert_time_u, $last_alert_time, my @repeat_list_passed ) = get_last_alert_time( "update", "$storage:$volume:$item", $time_act, $time_act_u, $DEBUG, \@repeat_list );
    if ( $time_act_u < $last_alert_time_u + $repeat_time * 60 ) {

      # retention time does not allow alarming
      if ( $alert_test == 0 ) {
        print "Retention      : $volume_type: $storage:$volume:$item : repeat time supress alarming now, last: $last_alert_time, retention: $repeat_time\n" if $DEBUG == 22;
        next;
      }
      else {
        print "Retention      : $volume_type: $storage:$volume:$item : repeat time supress alarming now, last: $last_alert_time, retention: $repeat_time -- ALERT_TEST ignoring it \n" if $DEBUG == 22;
      }
    } ## end if ( $time_act_u < $last_alert_time_u...)
    @repeat_list = @repeat_list_passed;    # must be here to do not overwrite the record when no alarm is issued

    # issue the alert
    if ( $alert_test == 0 ) {
      print "Alerting       : $volume_type: $storage:$volume:$item : utilization is over the limit ($limit_act > $limit)\n" if $DEBUG == 22;
      log_to_alert_log( $time_act, $storage, $volume, $item, $limit_act, $limit, $email, $volume_type );

      #if ($DEBUG == 22){
      #  if ((defined $VI_IMAGE && $VI_IMAGE eq "1") || (defined $VM_IMAGE && $VM_IMAGE eq "1")){
      #    log_to_alert_log($time_act,$storage,$volume,$item,$limit_act,$limit,$email,$volume_type);
      #  }
      #  else{
      #    my $report = "Alert has not been written into $config_alert[$ALERT_HISTORY]";
      #    warning($report);
      #  }
      #}
      #else{
      #  log_to_alert_log($time_act,$storage,$volume,$item,$limit_act,$limit,$email,$volume_type);
      #}
    } ## end if ( $alert_test == 0 )
    else {
      if ( $limit_act > $limit ) {
        print "Alerting       : $volume_type: $storage:$volume:$item : utilization is over the limit ($limit_act > $limit) alerting anyway --ALERT_TEST\n" if $DEBUG == 22;
      }
      else {
        print "Alerting       : $volume_type: $storage:$volume:$item : utilization is NOT over the limit ($limit_act < $limit) alerting anyway --ALERT_TEST\n" if $DEBUG == 22;
      }
      if ( ( defined $VI_IMAGE && $VI_IMAGE eq "1" ) || ( defined $VM_IMAGE && $VM_IMAGE eq "1" ) ) {
        log_to_alert_log( $time_act, $storage, $volume, $item, $limit_act, $limit, $email, $volume_type );
      }
    } ## end else [ if ( $alert_test == 0 )]

    if ( $email_graph > 0 ) {

      # create graph now when there is sure that it will be send out
      get_util( $email_graph, $volume_type, $graph_path, $storage, $st_type, $item, $time_act, $peak_time, $time_act_u, $volume, $DEBUG, $basedir, $ds5_v2, \@vol_file );
    }

    if ( defined($email) && !$email eq '' ) {
      sendmail( $volume_type, $config_alert[$EMAIL_GRAPH], $graph_path, $email, $time_act, $peak_time, $storage, $volume, $item, $limit_act, $limit, $line_email_no, $DEBUG, \@config_email );
    }

    if ( $config_alert[$NAGIOS] == 1 ) {

      # nagios alarm
      nagios_alarm( $volume_type, $storage, $volume, $item, $limit_act, $limit, $time_act, $nagios_dir, $DEBUG, $alert_test );
    }

    # SNMP TRAP
    if ( defined($snmp_trap) && !$snmp_trap eq '' && $snmp_trap !~ m/your_snmp_trap_server/ ) {
      snmp_trap_alarm( $volume_type, $storage, $volume, $item, $limit_act, $limit, $time_act, $snmp_trap, $DEBUG, $basedir );
    }

    # extern alert
    if ( defined( $config_alert[$EXTERN_ALERT] ) && !$config_alert[$EXTERN_ALERT] eq '' ) {
      extern_alarm( $volume_type, $storage, $volume, $item, $limit_act, $limit, $time_act, $config_alert[$EXTERN_ALERT], $DEBUG, $basedir );
    }
  } ## end foreach my $line (@config_alert_data)

  update_repeat_table( $alert_test, $alert_repeat_file, \@repeat_list );
  return 1;
} ## end sub check_data

sub log_to_alert_log {
  my ( $time_act, $storage, $volume, $item, $limit_act, $limit, $email, $volume_type ) = @_;

  $volume_type =~ s/POOL-ALL/STORAGE/;
  $volume      =~ s/all pools/all_pools/;

  open( FHW, ">> $config_alert[$ALERT_HISTORY]" ) || error( "could not open $config_alert[$ALERT_HISTORY] : $!" . __FILE__ . ":" . __LINE__ ) && return 1;
  if ( defined($email) && !$email eq '' ) {
    print FHW "$time_act: $storage:$volume_type:$volume:$item over the limit: $limit_act (limit $limit), email: $email\n";
  }
  else {
    print FHW "$time_act: $storage:$volume_type:$volume:$item over the limit: $limit_act (limit $limit), no email\n";
  }
  close(FHW);
  return 1;
} ## end sub log_to_alert_log

# get time of the last alarm
sub get_last_alert_time {
  my ( $action, $identification, $time_act, $time_act_u, $DEBUG, $repeat_list_tmp ) = @_;
  my @repeat_list = @{$repeat_list_tmp};

  my $line_no = -1;
  foreach my $line (@repeat_list) {
    chomp($line);
    $line_no++;
    ( my $ident, my $time_alrt_u, my $time_alrt ) = split( /\|/, $line );

    if ( !defined($ident) || $ident eq '' || !defined($time_alrt_u) || $time_alrt_u eq '' ) {

      # something wrong
      next;
    }

    if ( $ident =~ m/^$identification$/ ) {

      # found a record
      $repeat_list[$line_no] = "$identification|$time_act_u|$time_act";
      return ( $time_alrt_u, $time_alrt, @repeat_list );
    } ## end if ( $ident =~ m/^$identification$/)
  } ## end foreach my $line (@repeat_list)
  $line_no++;
  $repeat_list[$line_no] = "$identification|$time_act_u|$time_act";

  if ( $action =~ m/no update/ ) {
    return ( -1, -1 );
  }
  return ( -1, -1, @repeat_list );
} ## end sub get_last_alert_time

sub update_repeat_table {
  my ( $alert_test, $alert_repeat_file, $repeat_list_tmp ) = @_;
  my @repeat_list = @{$repeat_list_tmp};

  if ( $alert_test == 1 ) {
    return 1;
  }

  # update retention alerts
  open( FHW, "> $alert_repeat_file" ) || error( "could not open $alert_repeat_file : $!" . __FILE__ . ":" . __LINE__ ) && return 1;
  foreach my $line (@repeat_list) {
    chomp($line);
    print FHW "$line\n";
  }
  close(FHW);

  return 1;
} ## end sub update_repeat_table

sub get_util {
  my ( $email_graph, $volume_type, $graph_path, $storage, $st_type, $item, $time_act, $peak_time, $time_act_u, $volume, $DEBUG, $basedir, $ds5_v2, $vol_file_tmp, $type ) = @_;
  my @vol_file      = @{$vol_file_tmp};
  my $peak_time_sec = $peak_time * 60;
  my $width         = 300;
  my $ret           = -1;

  # translate [io|read_io|write_io|data|read|write] into real names used by storages
  my $item_translated = $item;

  if ( $item =~ m/^io/ ) {
    $item_translated = "io_rate";
    $item            = "io_rate";    # old cfg file could use "io" still
  }
  if ( $item =~ m/^data/ ) {
    $item_translated = "data_rate";
    $item            = "data_rate";    # old cfg file could use "data" still
  }
  if ( $item =~ m/^resp time write/ ) {
    $item_translated =~ s/resp time write/resp_t_w/;
  }
  if ( $item =~ m/^resp time read/ ) {
    $item_translated =~ s/resp time read/resp_t_r/;
  }
  if ( $item =~ m/^resp time$/ ) {
    $item_translated =~ s/resp time/resp_t/;
  }

  if ( $st_type =~ m/^DS5K$/ ) {
    if ( $item =~ m/^data/ || $item =~ m/^io/ || ( $item =~ m/^resp_t$/ && $ds5_v2 == 1 ) ) {
      $ret = get_util_exec( $st_type, $email_graph, $volume_type, $graph_path, $time_act, $basedir, $time_act_u, $peak_time_sec, $width, $item_translated, $storage, $volume, $item, $DEBUG, \@vol_file, $type );
    }
    else {
      error( "$storage:$volume:$item : wrong $item, it is not supported for this ($st_type) kind of storage " . __FILE__ . ":" . __LINE__ ) && return -1;
    }
  } ## end if ( $st_type =~ m/^DS5K$/)
  else {
    if ( $st_type =~ m/^DS8K$/ ) {
      if ( $item =~ m/^read_io$/ || $item =~ m/^write_io$/ || $item =~ m/^resp_t_/ ) {
        if ( $volume_type =~ m/VOLUME$/ || $volume_type =~ m/HOST/ ) {
          my @vol_file_tmp = "";
          my $i            = 0;
          foreach my $vol_file_act (@vol_file) {
            $vol_file_act =~ s/\.rrd/\.rrc/;    # this is stored in .rrc file
            $vol_file_tmp[$i] = $vol_file_act;
            $i++;
          }
          @vol_file = @vol_file_tmp;
        } ## end if ( $volume_type =~ m/VOLUME$/...)
      } ## end if ( $item =~ m/^read_io$/...)
      $ret = get_util_exec( $st_type, $email_graph, $volume_type, $graph_path, $time_act, $basedir, $time_act_u, $peak_time_sec, $width, $item_translated, $storage, $volume, $item, $DEBUG, \@vol_file, $type );
    } ## end if ( $st_type =~ m/^DS8K$/)
    else {
      # NETAPP, HUS, VSPG, 3PAR so far ...
      # default, all new storage should look like it
      $ret = get_util_exec( $st_type, $email_graph, $volume_type, $graph_path, $time_act, $basedir, $time_act_u, $peak_time_sec, $width, $item_translated, $storage, $volume, $item, $DEBUG, \@vol_file, $type );
    }
  } ## end else [ if ( $st_type =~ m/^DS5K$/)]

  return $ret;
} ## end sub get_util

sub get_util_exec {
  my ( $st_type, $email_graph, $volume_type, $graph_path, $time_act, $basedir, $time_act_u, $peak_time_sec, $width, $item_translated, $storage, $volume, $item, $DEBUG, $vol_file_tmp, $type ) = @_;
  my @vol_file   = @{$vol_file_tmp};
  my $answer     = "";
  my $start_time = $time_act_u - $peak_time_sec;

  if ( $email_graph > 0 ) {
    print "Creating graph : $volume_type: $storage:$volume:$item : $graph_path\n" if $DEBUG == 22;
    $start_time = $time_act_u - $email_graph * 3600;
  }

  $graph_path =~ s/://g;
  my $cmd .= "graph \"$graph_path\"";
  $cmd    .= " --start \"$start_time\"";
  $cmd    .= " --end \"$time_act_u\"";
  if ( $email_graph > 0 ) {
    my $volume_double = $volume;
    $volume =~ s/:/\\:/g;
    $cmd .= " --title \\\"$storage:$volume:$item: last $email_graph" . "h\\\"";
    my $pic_col = $ENV{PICTURE_COLOR};
    $cmd .= " --imgformat PNG";
    $cmd .= " --slope-mode";
    $cmd .= " --no-minor";
    $cmd .= " --height=150";
    $cmd .= " --lower-limit=0.00";
    $cmd .= " --color=BACK#$pic_col";
    $cmd .= " --color=SHADEA#$pic_col";
    $cmd .= " --color=SHADEB#$pic_col";
    $cmd .= " --color=CANVAS#$pic_col";
    $cmd .= " --alt-autoscale-max";
    $cmd .= " --units-exponent=1.00";
    $cmd .= " --alt-y-grid";
    my $vertical_label = "--vertical-label=\"MB per seconds\"";

    if ( $item_translated =~ m/io/ ) {
      $vertical_label = "--vertical-label=\"IO per seconds\"";
    }
    else {
      if ( $item_translated =~ m/^resp/ ) {
        $vertical_label = "--vertical-label=\"ms\"";
      }
    }
    $cmd .= " $vertical_label";
  } ## end if ( $email_graph > 0 )
  $cmd .= " --width=$width";
  if ( $email_graph == 25 ) {
    $cmd .= " --x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
  }

  # Disable Tobi's promo
  my $rrd_ver = $RRDp::VERSION;
  if ( isdigit($rrd_ver) && $rrd_ver > 1.34 ) {

    # graphv is supported since rrdtool-1.3rc5
    $cmd .= " --disable-rrdtool-tag";
  }

  my $item_translated_resp = $item_translated;    # for resp_t
  if ( $item_translated =~ m/^resp/ && $item_translated =~ m/read/ ) {
    $item_translated_resp = "read_io";
  }
  if ( $item_translated =~ m/^resp/ && $item_translated =~ m/write/ ) {
    $item_translated_resp = "write_io";
  }

  my $indx     = 0;
  my $time_old = 0;
  my $file_old = "";
  foreach my $vol_file_act (@vol_file) {

    # more input files here can be --> ie DS8k multilevel volumes, pools etc
    if ( !-f $vol_file_act ) {
      warning("$storage:$volume:$item: RRD file does not exist: $vol_file_act ");
      next;
    }
    my $file_time = ( stat("$vol_file_act") )[9];
    if ( $file_time < $start_time ) {
      $time_old = $file_time;
      $file_old = $vol_file_act;
      next;
    }
    $vol_file_act =~ s/:/\\:/g;
    if ( $item_translated =~ m/^resp/ ) {

      # response time must be ballanced : sum (io * resp_t)/ io
      $cmd .= " DEF:item_io${indx}=\"$vol_file_act\":$item_translated_resp:AVERAGE";
      $cmd .= " CDEF:check_io${indx}=item_io${indx},UN,0,item_io${indx},IF";
      $cmd .= " DEF:item_resp${indx}=\"$vol_file_act\":$item_translated:AVERAGE";
      $cmd .= " CDEF:check_resp${indx}=item_resp${indx},UN,0,item_resp${indx},IF";
      $cmd .= " CDEF:item_part${indx}=check_io${indx},check_resp${indx},*";
    } ## end if ( $item_translated ...)
    else {
      if ( ( $st_type =~ m/^SWIZ$/ || ( $st_type =~ m/^XIV$/ && $volume_type =~ m/VOLUME$|CUSTOM/ ) || ( $st_type =~ m/^DS8K$/ && $volume_type =~ m/POOL/ ) ) && ( $item =~ m/^data/ || $item =~ m/^io/ ) ) {

        # get data and IO rate from read/write
        if ( $item =~ m/^data/ ) {
          $cmd .= " DEF:item_read${indx}=\"$vol_file_act\":read:AVERAGE";
          $cmd .= " DEF:item_write${indx}=\"$vol_file_act\":write:AVERAGE";
          $cmd .= " CDEF:item_part${indx}=item_read${indx},item_write${indx},+";
        }
        else {
          # io
          $cmd .= " DEF:item_read_io${indx}=\"$vol_file_act\":read_io:AVERAGE";
          $cmd .= " DEF:item_write_io${indx}=\"$vol_file_act\":write_io:AVERAGE";
          $cmd .= " CDEF:item_part${indx}=item_read_io${indx},item_write_io${indx},+";
        } ## end else [ if ( $item =~ m/^data/)]
      } ## end if ( ( $st_type =~ m/^SWIZ$/...))
      else {
        # normal processing io/data
        $cmd .= " DEF:item_part_t${indx}=\"$vol_file_act\":$item_translated:AVERAGE";
        $cmd .= " CDEF:item_part${indx}=item_part_t${indx},UN,0,item_part_t${indx},IF";
      }
    } ## end else [ if ( $item_translated ...)]
    $indx++;
  } ## end foreach my $vol_file_act (@vol_file)

  if ( $indx == 0 ) {
    if ( $time_old > 0 ) {
      my $time_act = strftime( "%H:%M:%S %d/%m/%Y", localtime($time_old) );
      if ( defined $type ) {
        print "Get utilization: $type: $storage:$volume:$item : DB file has not been recently modified, skipping it: $file_old (last modif: $time_act)\n" if $DEBUG == 22;
      }
      else {
        print "Get utilization: $volume_type: $storage:$volume:$item : DB file has not been recently modified, skipping it: $file_old (last modif: $time_act)\n" if $DEBUG == 22;
      }
    } ## end if ( $time_old > 0 )
    else {
      if ( defined $type ) {
        print "Get utilization: $type: $storage:$volume:$item : no DB file has been identified: \n" if $DEBUG == 22;
      }
      else {
        print "Get utilization: $volume_type: $storage:$volume:$item : no DB file has been identified: \n" if $DEBUG == 22;
      }
    } ## end else [ if ( $time_old > 0 ) ]
    return -1;
  } ## end if ( $indx == 0 )

  # summary IO if it is response time
  if ( $item =~ m/^resp/ ) {
    my $index_actual = 0;
    $cmd .= " CDEF:item_io_sum=check_io${index_actual}";
    $index_actual++;
    for ( ; $index_actual < $indx; $index_actual++ ) {
      $cmd .= ",check_io${index_actual},+";
    }
  } ## end if ( $item =~ m/^resp/)

  # get summary
  my $index_actual = 0;
  $cmd .= " CDEF:item=item_part${index_actual}";
  $index_actual++;
  for ( ; $index_actual < $indx; $index_actual++ ) {
    $cmd .= ",item_part${index_actual},+";
  }
  if ( $item =~ m/^resp/ ) {
    $cmd .= ",item_io_sum,/";
  }

  my $item_space = sprintf( "%-10s", $item );
  if ( $item_translated =~ m/io/ ) {
    if ( $email_graph > 0 ) {
      $cmd .= " COMMENT:\\\"                 average  maximum\\n\\\"";
      $cmd .= " LINE1:item#00FF00:\" $item_space \"";
      $cmd .= " GPRINT:item:AVERAGE:\\\"%6.0lf \\\"";
      $cmd .= " GPRINT:item:MAX:\\\"%6.0lf \\\"";
    } ## end if ( $email_graph > 0 )
    else {
      $cmd .= " PRINT:item:AVERAGE:\\\"%6.0lf \\\"";
    }
  } ## end if ( $item_translated ...)
  else {
    if ( $item_translated =~ m/^resp/ ) {
      if ( $email_graph > 0 ) {
        $cmd .= " COMMENT:\\\"                 average  maximum\\n\\\"";
        $cmd .= " LINE1:item#00FF00:\" $item_space \"";
        $cmd .= " GPRINT:item:AVERAGE:\\\"%6.2lf \\\"";
        $cmd .= " GPRINT:item:MAX:\\\"%6.2lf \\\"";
      } ## end if ( $email_graph > 0 )
      else {
        $cmd .= " PRINT:item:AVERAGE:\\\"%6.2lf \\\"";
      }
    } ## end if ( $item_translated ...)
    else {
      # data_rate, read, write
      $cmd .= " CDEF:itemmb=item,1024,/";
      if ( $email_graph > 0 ) {
        $cmd .= " COMMENT:\\\"                average   maximu \\n\\\"";
        $cmd .= " LINE1:itemmb#00FF00:\" $item_space \"";
        $cmd .= " GPRINT:itemmb:AVERAGE:\\\"%6.1lf \\\"";
        $cmd .= " GPRINT:itemmb:MAX:\\\"%6.1lf \\\"";
      } ## end if ( $email_graph > 0 )
      else {
        $cmd .= " PRINT:itemmb:AVERAGE:\\\"%6.3lf \\\"";
      }
    } ## end else [ if ( $item_translated ...)]
  } ## end else [ if ( $item_translated ...)]
  if ( $email_graph > 0 ) {
    $cmd .= " COMMENT:\\n";
    $cmd .= " HRULE:0#000000";
  }
  $cmd =~ s/\\"/"/g;

  #print "$cmd\n";

  eval {
    RRDp::cmd qq($cmd);
    $answer = RRDp::read;
  };

  if ($@) {
    chomp($@);
    error("$storage:$volume:$item: RRD file read problem: $vol_file[0] $@ ");    # file&line is apended there in that variable already
    return -1;
  }

  if ( $email_graph > 0 ) {
    return 1;                                                                    # only creating of graph, do not utilization value
  }

  chomp($$answer);
  if ( $$answer =~ m/NaN/ || $$answer =~ m/nan/ ) {
    warning( "$volume_type: $storage:$volume:$item : no data at: $time_act - $peak_time_sec seconds " . __FILE__ . ":" . __LINE__ );
    return -1;
  }
  chomp($$answer);
  my $page = -1;
  foreach my $line ( split( /\n/, $$answer ) ) {
    $page = $line;
    chomp($page);
    $page =~ s/ //g;
  }
  if ( $$answer =~ "ERROR" ) {
    error( "$volume_type: $storage:$volume:$item : $vol_file[0]: Graph rrdtool error : $$answer " . __FILE__ . ":" . __LINE__ );
    return -1;
  }

  if ( isdigit($page) == 1 && $page !~ -1 ) {
    return $page;
  }
  else {
    error( "$volume_type: $storage:$volume:$item : $vol_file[0]: Graph rrdtool error : $$answer ; page: $page " . __FILE__ . ":" . __LINE__ );
    return -1;
  }
} ## end sub get_util_exec

sub sendmail {
  my ( $volume_type, $email_graph, $graph_path, $email, $time_act, $peak_time, $storage, $volume, $item, $limit_act, $limit, $line_email_no, $DEBUG, $config_email_tmp ) = @_;
  my @config_email = @{$config_email_tmp};
  my $mailfrom     = "stor2rrd";
  my $mailprog     = "/usr/sbin/sendmail";
  my $value        = "";
  if ( $item =~ /io/ ) {
    $value = sprintf( "%.0f IO/sec", $limit_act );
  }
  else {
    if ( $item =~ /^resp/ ) {
      $value = sprintf( "%.2f ms", $limit_act );
    }
    else {
      $value = sprintf( "%.1f MB/sec", $limit_act );
    }
  } ## end else [ if ( $item =~ /io/ ) ]

  my $item_text = $item;
  if ( $item =~ m/resp_t/ ) {
    $item_text = "response time";
  }
  if ( $item =~ m/resp_t_r/ ) {
    $item_text = "response time read";
  }
  if ( $item =~ m/resp_t_w/ ) {
    $item_text = "response time write";
  }
  if ( $item =~ m/data_rate/ ) {
    $item_text = "data throughput";
  }
  if ( $item =~ m/read/ ) {
    $item_text = "data read throughput";
  }
  if ( $item =~ m/write/ ) {
    $item_text = "data write throughput";
  }
  if ( $item =~ m/io_rate/ ) {
    $item_text = "IO rate";
  }
  if ( $item =~ m/read_io/ ) {
    $item_text = "IO read";
  }
  if ( $item =~ m/write_io/ ) {
    $item_text = "IO write";
  }
  my $subject = "STOR2RRD alert: $storage:$volume: $volume_type: metric:$item_text, actual value: $value (Limit $limit)";

  if ( $line_email_no == 0 ) {

    #error("No any alerting targed has been specified, define email rules via the GUI ".__FILE__.":".__LINE__);
    return 1;
  }

  foreach my $email_act (@config_email) {
    chomp($email_act);
    ( my $trash, my $email_name, my $email_list_line ) = split( /:/, $email_act );
    if ( !defined($email_name) || $email_name eq '' || !defined($email_list_line) || $email_list_line eq '' ) {
      next;
    }
    if ( $email_name =~ m/^$email$/ ) {

      # email group has been found
      ( my @email_list ) = split( /,/, $email_list_line );
      foreach my $email (@email_list) {
        $email =~ s/ //g;
        print "Emailing       : $volume_type: $storage:$volume:$item : $value : sending email to $email\n" if $DEBUG == 22;
        if ( open MAIL, "|$mailprog -t" ) {
          print MAIL "To: $email\n";
          print MAIL "From: $mailfrom\n";
          print MAIL "Reply-To: $mailfrom\n";
          print MAIL "X-Mailer: stor2rrd 1.0\n";
          print MAIL "Subject: $subject\n\n";
          if ( $item =~ /^resp/ ) {
            print MAIL "STOR2RRD alert\n Time: $time_act\n Storage: $storage\n $volume_type: $volume\n Metric: $item_text\n Average response time during last $peak_time" . "mins: $value\n Limit: $limit\n";
          }
          else {
            print MAIL "STOR2RRD alert\n Time: $time_act\n Storage $storage:\n $volume_type: $volume\n Metric: $item_text\n Average throughput during last $peak_time" . "mins: $value\n Limit: $limit\n";
          }
          print MAIL "\n\n";
          if ( $email_graph > 0 && -f $graph_path ) {
            print "Attachment     : $volume_type: $storage:$volume:$item : attaching : $graph_path\n" if $DEBUG == 22;
            open( PNG, "uuencode \"$graph_path\" \"$storage:$volume:$item.png\" |" );
            binmode(PNG);
            while ( read( PNG, $b, 4096 ) ) {
              print MAIL "$b";
            }
            close(PNG);
          } ## end if ( $email_graph > 0 ...)
          close(MAIL);
        } ## end if ( open MAIL, "|$mailprog -t")
      } ## end foreach my $email (@email_list)
      if ( -f $graph_path ) {

        # remove here, to allow to be send to more addresses
        unlink($graph_path);
      }
    } ## end if ( $email_name =~ m/^$email$/)
  } ## end foreach my $email_act (@config_email)
  return 0;
} ## end sub sendmail

sub isdigit {
  my $digit = shift;

  if ( !defined($digit) ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  return 0;
} ## end sub isdigit

sub nagios_alarm {
  my ( $volume_type, $storage, $volume, $item, $limit_act, $limit, $time_act, $nagios_dir, $DEBUG, $alert_test ) = @_;
  my $value = "";
  if ( $item =~ /io/ ) {
    $value = sprintf( "%.0f IO/sec", $limit_act );
  }
  else {
    if ( $item =~ /^resp/ ) {
      $value = sprintf( "%.2f ms", $limit_act );
    }
    else {
      $value = sprintf( "%.1f MB/sec", $limit_act );
    }
  } ## end else [ if ( $item =~ /io/ ) ]

  #print "Alert nagios   : $volume_type: $storage:$volume:$item : utilization=$value\n" if $DEBUG == 22;

  if ( $alert_test == 1 ) {
    print "Alert nagios   : $volume_type: $storage:$volume:$item : not issuing due to permission\n" if $DEBUG == 22;
    return 0;
  }

  if ( !-d "$nagios_dir" ) {
    print "mkdir          : $nagios_dir\n" if $DEBUG;
    mkdir( "$nagios_dir", 0755 ) || error( "Cannot mkdir $nagios_dir: $!" . __FILE__ . ":" . __LINE__ ) && return 1;
    chmod 0777, "$nagios_dir" || error( "Can't chmod 666 $nagios_dir: $!" . __FILE__ . ":" . __LINE__ ) && return 1;
  }

  if ( !-d "$nagios_dir/$storage" ) {
    print "mkdir          : $nagios_dir/$storage\n" if $DEBUG;
    mkdir( "$nagios_dir/$storage", 0755 ) || error( "Cannot mkdir $nagios_dir/$storage: $!" . __FILE__ . ":" . __LINE__ ) && return 1;
    chmod 0777, "$nagios_dir/$storage" || error( "Can't chmod 666 $nagios_dir/$storage: $!" . __FILE__ . ":" . __LINE__ ) && return 1;
  }

  open( FH, "> $nagios_dir/$storage/$volume-$item" ) || error( "Can't create $nagios_dir/$storage/$volume-$item : $!" . __FILE__ . ":" . __LINE__ ) && return 1;

  print FH "Critical alert for: $storage:$volume:$item utilization=$value,MAX limit=$limit,time=$time_act\n";
  close(FH);

  chmod 0666, "$nagios_dir/$storage/$volume-$item" || error( "Can't chmod 666 $nagios_dir/$storage/$volume-$item : $!" . __FILE__ . ":" . __LINE__ ) && return 1;

  return 1;
} ## end sub nagios_alarm

sub extern_alarm {
  my ( $volume_type, $storage, $volume, $item, $limit_act, $limit, $time_act, $extern_alert, $DEBUG, $basedir ) = @_;

  if ( !-f "$basedir/$extern_alert" ) {
    if ( !-x "$basedir/$extern_alert" ) {
      error( "EXTERN_ALERT is set but the file is not executable : $basedir/$extern_alert " . __FILE__ . ":" . __LINE__ ) && return 1;
    }
  }
  else {
    if ( !-f "$extern_alert" ) {
      error( "EXTERN_ALERT is set but the file does not exist: $extern_alert : $basedir/$extern_alert " . __FILE__ . ":" . __LINE__ ) && return 1;
    }
    else {
      if ( !-x "$extern_alert" ) {
        error( "EXTERN_ALERT is set but the file is not executable : $extern_alert " . __FILE__ . ":" . __LINE__ ) && return 1;
      }
    }
  } ## end else [ if ( !-f "$basedir/$extern_alert")]

  my $value = "";
  if ( $item =~ /io/ ) {
    $value = sprintf( "%.0f IO/sec", $limit_act );
  }
  else {
    if ( $item =~ /^resp/ ) {
      $value = sprintf( "%.2f ms", $limit_act );
    }
    else {
      $value = sprintf( "%.1f MB/sec", $limit_act );
    }
  } ## end else [ if ( $item =~ /io/ ) ]

  #print "Alert external : $volume_type: $storage:$volume:$item : utilization=$value\n" if $DEBUG == 22;

  system( "$extern_alert", "$storage", "$volume", "$item", "$limit_act", "$limit", "$time_act", "$value" );

  return 1;
} ## end sub extern_alarm

sub snmp_trap_alarm {
  my ( $type, $storage, $volume, $item, $limit_act, $limit, $time_act, $trap_host, $DEBUG, $basedir ) = @_;

  my $SNMP_PEN = "40540";
  my $PRE      = "1.3.6.1.4.1.40540";

  print "Alert SNMP TRAP: $storage:$type:$item utilization=$limit_act\n";

  # this command sends canonical SNMP names
  # `snmptrap -v 1 -c public $trap_host XORUX-MIB::stor2rrdSendTrap '' 6 7 '' XORUX-MIB::stor2rrdStorageName s '$storage' XORUX-MIB::stor2rrdType s '$type' XORUX-MIB::stor2rrdItem s '$item' XORUX-MIB::stor2rrdValue s '$limit_act'`;
  # this one send numerical (it's OK for our needs)

  my $snmp_exe = "/opt/freeware/bin/snmptrap";    # AIX place
  if ( !-f "$snmp_exe" ) {
    $snmp_exe = "/usr/bin/snmptrap";              #linux one
    if ( !-f "$snmp_exe" ) {
      $snmp_exe = "snmptrap";                     # lets hope it is in the PATH
    }
  } ## end if ( !-f "$snmp_exe" )

  print "SNMP trap exec : $snmp_exe -v 1 -c public $trap_host $PRE.2.0.1.0.7 \'\' 6 7 \'\' $PRE.1.1 s $storage $PRE.1.2 s $type $PRE.1.3 s $item $PRE.1.4 s $limit_act\n";
  my $out = `$snmp_exe -v 1 -c public $trap_host $PRE.2.0.1.0.7 '' 6 7 '' $PRE.1.1 s '$storage' $PRE.1.2 s '$type' $PRE.1.3 s '$item' $PRE.1.4 s '$limit_act'  2>&1;`;
  if ( $out =~ m/not found/ ) {
    error("SNMP Trap: $snmp_exe binnary has not been found, install net-snmp as per http://www.stor2rrd.com/alert-snmp.htm ($out)");
  }
  if ( $out =~ m/Usage: snmptrap/ ) {
    error("SNMP Trap: looks like you use native AIX /usr/sbin/snmptrap, it is not supported, check here: http://www.stor2rrd.com/alert-snmp.htm");
  }

  return 1;
} ## end sub snmp_trap_alarm

sub snmp_trap_alarm_test {
  my $trap_host = shift;
  my $storage   = shift;
  my $type      = "event";
  my $item      = "test_only";
  my $limit_act = 0;

  my $SNMP_PEN = "40540";
  my $PRE      = "1.3.6.1.4.1.40540";

  print "Alert SNMP TRAP: TEST\n";

  # this command sends canonical SNMP names
  # `snmptrap -v 1 -c public $trap_host XORUX-MIB::stor2rrdSendTrap '' 6 7 '' XORUX-MIB::stor2rrdStorageName s '$storage' XORUX-MIB::stor2rrdType s '$type' XORUX-MIB::stor2rrdItem s '$item' XORUX-MIB::stor2rrdValue s '$limit_act'`;
  # this one send numerical (it's OK for our needs)

  my $snmp_exe = "/opt/freeware/bin/snmptrap";    # AIX place
  if ( !-f "$snmp_exe" ) {
    $snmp_exe = "/usr/bin/snmptrap";              #linux one
    if ( !-f "$snmp_exe" ) {
      $snmp_exe = "snmptrap";                     # lets hope it is in the PATH
    }
  } ## end if ( !-f "$snmp_exe" )

  print "SNMP trap exec : $snmp_exe -v 1 -c public $trap_host $PRE.2.0.1.0.7 \'\' 6 7 \'\' $PRE.1.1 s $storage $PRE.1.2 s $type $PRE.1.3 s $item $PRE.1.4 s $limit_act\n";
  my $out = `$snmp_exe -v 1 -c public $trap_host $PRE.2.0.1.0.7 '' 6 7 '' $PRE.1.1 s '$storage' $PRE.1.2 s '$type' $PRE.1.3 s '$item' $PRE.1.4 s '$limit_act'  2>&1;`;
  if ( $out =~ m/not found/ ) {
    error("SNMP Trap: $snmp_exe binnary has not been found, install net-snmp as per http://www.stor2rrd.com/alert-snmp.htm ($out)");
  }
  if ( $out =~ m/Usage: snmptrap/ ) {
    error("SNMP Trap: looks like you use native AIX /usr/sbin/snmptrap, it is not supported, check here: http://www.stor2rrd.com/alert-snmp.htm");
  }

  return 1;
} ## end sub snmp_trap_alarm_test

return 1;
