#!/usr/bin/perl

use strict;
use warnings;
use CGI::Carp qw(fatalsToBrowser);
use Date::Parse;

# use Data::Dumper;

use lib "../bin";    # maybe useless, but didn't work for me without that
use Alerting;        # use module

my $basedir = $ENV{INPUTDIR};
$basedir ||= "..";

# get URL parameters (could be GET or POST) and put them into hash %PAR
my ( $buffer, @pairs, $pair, $name, $value, %PAR );

if ( !defined $ENV{'REQUEST_METHOD'} ) {
  print "Content-type: text/plain\n\n";
  print "No command specified, exiting...";
  exit;
}

if ( lc $ENV{'REQUEST_METHOD'} eq "post" ) {
  read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
}
else {
  $buffer = $ENV{'QUERY_STRING'};
}

# Split information into name/value pairs
@pairs = split( /&/, $buffer );
foreach $pair (@pairs) {
  ( $name, $value ) = split( /=/, $pair );
  $value =~ tr/+/ /;
  $value =~ s/%(..)/pack("C", hex($1))/eg;
  if ( exists $PAR{$name} ) {
    if ( ref $PAR{$name} eq "ARRAY" ) {
      push @{ $PAR{$name} }, $value;
    }
    else {
      my $firstVal = $PAR{$name};
      $PAR{$name} = [];
      push @{ $PAR{$name} }, $firstVal;
      push @{ $PAR{$name} }, $value;
    } ## end else [ if ( ref $PAR{$name} eq...)]
  } ## end if ( exists $PAR{$name...})
  else {
    $PAR{$name} = $value;
  }
} ## end foreach $pair (@pairs)

# print STDERR Dumper \%PAR;

if ( !defined $PAR{cmd} ) {
  print "Content-type: text/plain\n\n";
  print "No command specified, exiting...";
  exit;
}

if ( $PAR{cmd} eq "form" ) {    ### Get list of credentials
  no warnings 'uninitialized';
  my %cfg = Alerting::getConfig();

  print "Content-type: text/html\n\n";

# TABs
  my $tab_number = 0;
  print <<_MARKER_;
<div id='tabs' style='text-align: center;'>
  <ul>
    <li><a href='#tabs-0'>Logs</a></li>
    <li><a href='#tabs-1'>Configuration</a></li>
    <li><a href='#tabs-2'>E-mail groups</a></li>
    <li><a href='#tabs-3'>Options perf</a></li>
    <li><a href='#tabs-4'>Options event</a></li>
  </ul>

<div id='tabs-0' style='display: inline-block'>
  <table id="logs">
    <tr>
      <th>Performance events</th>
      <th>Hardware events</th>
    </tr>
    <tr>
      <td>
        <div id="perf_logs">
        </div>
      </td>
      <td>
        <div id="hw_logs">
        </div>
      </td>
    </tr>
  </table>
</div>

<div id='tabs-1' style='display: inline-block'>
  <div style='float: left; margin-right: 10px; outline: none'>
    <fieldset class='estimator cggrpnames'>
      <table id="alrttree" class="cfgtree">
        <colgroup>
          <col width="2px">
          <col width="2px">
          <col width="250px">
          <col width="100px">
          <col width="45px">
          <col width="45px">
          <col width="60px">
          <col width="110px">
          <col width="120px">
          <col width="10px">
          <col width="20px">
        </colgroup>
        <thead>
        <tr>
          <th></th>
          <th></th>
          <th id="addcgrpth">Alert <button id="addnewalrt">Add New</button></th>
          <th>Subsys/Metric</th>
          <th style="text-align: center"><abbr title="limit value in MB/sec or IOPS">Limit</abbr></th>
          <th style="text-align: center"><abbr title="time in minutes for length peak above the limit [5-120]">Peak</abbr></th>
          <th style="text-align: center"><abbr title="minimum time in minutes between 2 alerts for the same rule [5-168]">Repeat</abbr></th>
          <th><abbr title="time range in hours when the alerting is off [0-24]-[0-24]. Ex. 22-05&nbsp;(excludes alerting from 10pm to 5am)">Exclude hours</abbr></th>
          <th>Mail group</th>
          <th></th>
          <!--th><button id="cgcfg-help-button" title="Help on usage">?</button></th-->
        </tr>
        </thead>
        <tbody>
        </tbody>
      </table>
    </fieldset>
    <div style="text-align: center">
      <input type='submit' style='font-weight: bold; margin-top: .7em' name='testbtn' id='testalrtcfg' value='Test alerting'>
      <input type='submit' style='font-weight: bold; margin-top: .7em' name='savegrp' class='savealrtcfg' value='Save configuration'>
    </div>
  </div>

  <br style="clear: both">
  <pre>
  <div id='aclfile' style='text-align: left; margin: auto; background: #fcfcfc; border: 1px solid #c0ccdf; border-radius: 10px; padding: 15px; display: none; overflow: auto'></div>
  </pre>
</div>

<div id='tabs-2' style='display: inline-block'>
  <div style='float: left; margin-right: 10px; outline: none'>
    <fieldset class='estimator cggrpnames'>
      <table id="alrtgrptree" class="cfgtree">
        <colgroup>
          <col width="2px">
          <col width="2px">
          <col width="320px">
          <col width="4px">
          <col width="20px">
        </colgroup>
        <thead>
        <tr>
        <th></th>
        <th></th>
        <th id="addcgrpth">E-mail group &nbsp;<button id="addalrtgrp">Add New</button></th>
        <th></th>
        <!--th><button id="cgcfg-help-button" title="Help on usage">?</button></th-->
        </tr>
        </thead>
        <tbody>
        </tbody>
      </table>
    </fieldset>
    <div style="text-align: center">
      <input type='submit' style='font-weight: bold; margin-top: .7em' name='savegrp' class='savealrtcfg' value='Save configuration'>
    </div>
  </div>
</div>
<div id='tabs-3' style='display: inline-block'>
  <div style='float: left; margin-right: 10px; outline: none'>
    <form id="optform" class="optform" method="post" action="" style="display: table;">
    <fieldset>
    <div>
      <label for="element_2">Nagios alerting &nbsp;</label>
      <input id="element_2" name="NAGIOS" class="alrtoption text medium" type="text" maxlength="255" title="Call this script from nrpe.cfg: bin/check_stor2rrd<br>More details on <a href='http://www.stor2rrd.com/nagios.html'>http://www.stor2rrd.com/nagios.html</a><br>[0/1] on/off" value="$cfg{NAGIOS}">
    </div>
    <div>
    <label for="element_3">External script for alerting &nbsp;</label>
      <input id="element_3" name="EXTERN_ALERT" class="alrtoption text medium" type="text" maxlength="255" title="It will be called once an alarm appears with these 5 parameters:<br><pre>script.sh  [storage] [volume] [metric] [actual value] [limit]</pre>- you can use <b>bin/external_alert_example.sh</b> as an example<br>- script must be placed in <b>{LPAR2RRD_HOME}/bin</b> and path start with <b>bin/</b>" value="$cfg{EXTERN_ALERT}">
    </div>
    <div>
    <label for="element_4">Include graphs &nbsp;</label>
      <input id="element_4" name="EMAIL_GRAPH" class="alrtoption text medium" type="text" maxlength="255" title="Include graphs into the email notification.<br>Any positive number gives number of hours which the graph contains. Examples: <br>0 - false<br>8 - last 8 hours in the graph<br>25 - last 25 hours in the graph<br>[0 - 256]" value="$cfg{EMAIL_GRAPH}">
    </div>
    <div>
    <label for="element_5">Default repeat time (min)&nbsp;</label>
      <input id="element_5" name="REPEAT_DEFAULT" class="alrtoption text medium" type="text" maxlength="255" title="Default time in minutes which says how often you should be alerted. You can specify per volume different value in <b>alert repeat time</b> column of each ALERT<br>[5 - 168]" value="$cfg{REPEAT_DEFAULT}">
    </div>
    <div>
    <label for="element_6">Default peak time (min)&nbsp;</label>
      <input id="element_6" name="PEAK_TIME_DEFAULT" class="alrtoption text medium" type="text" maxlength="255" title="The period of time in which avg traffic utilization has to be over the specified limit to generate an alert.<br>You can change it per volume level in <b>time in min</b> column of each ALERT note.<br> [15 - 120]" value="$cfg{PEAK_TIME_DEFAULT}">
    </div>
    <div>
    <label for="element_7">SNMP trap host&nbsp;</label>
      <input id="element_7" name="TRAP" class="alrtoption text medium" type="text" maxlength="80" title="Hostname or IP of SNMP trap receiver.<br> []" value="$cfg{TRAP}">
    </div>
    </fieldset>
    <div style="text-align: center">
      <input type='submit' style='font-weight: bold; margin-top: .7em' name='savegrp' class='savealrtcfg' value='Save configuration'>
    </div>
    </form>
  </div>
</div>
<div id='tabs-4' style='display: inline-block'>
  <div style='float: left; margin-right: 10px; outline: none'>
    <form id="eventform" class="optform" method="post" action="" style="display: table;">
    <fieldset>
    <div>
      <label for="element_2">Nagios alerting &nbsp;</label>
      <input id="element_2" name="NAGIOS_EVENT" class="alrtoption text medium" type="text" maxlength="255" title="Call this script from nrpe.cfg: bin/check_stor2rrd<br>More details on <a href='http://www.stor2rrd.com/nagios.html'>http://www.stor2rrd.com/nagios.html</a><br>[0/1] on/off" value="$cfg{NAGIOS_EVENT}">
    </div>
    <div>
    <label for="element_3">External script for alerting &nbsp;</label>
      <input id="element_3" name="EXTERN_ALERT_EVENT" class="alrtoption text medium" type="text" maxlength="255" title="It will be called once an alarm appears with these 5 parameters:<br><pre>script.sh  [storage] [volume] [metric] [actual value] [limit]</pre>- you can use <b>bin/external_alert_example.sh</b> as an example<br>- script must be placed in <b>{LPAR2RRD_HOME}/bin</b> and path start with <b>bin/</b>" value="$cfg{EXTERN_ALERT_EVENT}">
    </div>
    <div>
    <label for="element_4"> E-mail targets &nbsp;</label>
      <input id="element_4" name="EMAIL_EVENT" class="alrtoption text medium" type="text" maxlength="255" title="Define email targets separated by a space" value="$cfg{EMAIL_EVENT}">
    </div>
    <div>
    <label for="element_5">SNMP trap host&nbsp;</label>
      <input id="element_5" name="TRAP_EVENT" class="alrtoption text medium" type="text" maxlength="80" title="Hostname or IP of SNMP trap receiver.<br> []" value="$cfg{TRAP_EVENT}">
    </div>
    </fieldset>
    <div style="text-align: center">
      <button style='font-weight: bold; margin-top: .7em' name='testevbtn' id='testevents'>Test event alerting</button>
      <input type='submit' style='font-weight: bold; margin-top: .7em' name='savegrp' class='savealrtcfg' value='Save configuration'>
    </div>
    </form>
    <p>
    Alert is raised when any device status is changed to red color in the healthstatus page.
    </p>
  </div>
</div>
</div>
_MARKER_
}
elsif ( $PAR{cmd} eq "save" ) {    ### Save Alerting configuration

  print "Content-type: text/html\n\n";

  my $cfgdir = "$basedir/etc/web_config";

  my $tmpdir = "$basedir/tmp";
  if ( defined $ENV{TMPDIR_STOR} ) {
    $tmpdir = $ENV{TMPDIR_STOR};
  }

  if ( $PAR{acl} ) {
    my $buffer = $PAR{acl};
    my $retstr = "";
    if ( $ENV{'SERVER_NAME'} eq "demo.stor2rrd.com" ) {
      $buffer =~ s/\n/\\n/g;
      $buffer =~ s/\\:/\\\\:/g;
      $retstr = "\"msg\" : \"<div>This demo site does not allow saving any changes you do in the admin GUI panel.<br />";
      $retstr .= "You can only see the preview of alerting.cfg to be written.</div>\", \"cfg\" : \"";
      print "{ \"status\" : \"success\", $retstr" . "$buffer\" }";
    } ## end if ( $ENV{'SERVER_NAME'...})
    elsif ( open( CFG, ">$cfgdir/alerting.cfg" ) ) {
      print CFG "$buffer\n";
      close CFG;
      $buffer =~ s/\n/\\n/g;
      $buffer =~ s/\\:/\\\\:/g;
      $retstr = "\"msg\" : \"<div>Alerting configuration file has been succesfuly saved!<br /><br /></div>\", \"cfg\" : \"";
      print "{ \"status\" : \"success\", $retstr" . "$buffer\" }";
    } ## end elsif ( open( CFG, ">$cfgdir/alerting.cfg"...))
    else {
      print "{ \"status\" : \"fail\", \"msg\" : \"<div>File $cfgdir/alerting.cfg cannot be written by webserver, check apache user permissions: <span style='color: red'>$!</span></div>\" }";
    }
  } ## end if ($buffer)
  else {
    print "{ \"status\" : \"fail\", \"msg\" : \"<div>No data was written to alerting.cfg</div>\" }";
  }
}

elsif ( $PAR{cmd} eq "plog" ) {    ### get perfomance alerts log
  print "Content-type: text/html\n\n";
  my @loglines = tail_log("alert_history.log", 1, $PAR{count});
  if (@loglines) {
    print "<table class='tabconfig evlog'>";
    print "<thead>";
    print "<tr>";
    print "<th class='sortable'>Date/Time</th><th class='sortable'>Storage</th><th class='sortable'>Subsystem</th><th class='sortable'>Item</th><th class='sortable'>Reason</th>";
    print "</tr>";
    print "</thead>";
    print "<tbody>";
    foreach my $line (@loglines) {
      if (! $line) {
        next;
      }
      if ($line =~ /^[A-Z]/) {
        next;
      }
      print "<tr>";
      my @splitted = split(" ", $line, 4);
      if (scalar @splitted < 4) {
        next;
      }
      chop $splitted[1];
      my @ss = split(":", $splitted[2]);
      my $ahost = $ss[0];
      my $atype = $ss[1];
      my $aname = $ss[2];
      if ($ss[1] eq "CUSTOM") {
        $ahost  = "custom-group";
        $atype = "VOLUME";
      }
      elsif ($ss[2] eq "all_pools") {
        $atype = "POOL";
        my $aitem = (split("_", $ss[3]))[0];
        $aname = "$aitem";
      }
      #
      my ($d, $m, $y) = split("\/", $splitted[1]);
      # my $timestamp = str2time("${y}-${m}-${d}T$splitted[0]");
      my $timestamp = "${y}-${m}-${d} $splitted[0]";
      no warnings 'uninitialized';
      print "<td>$timestamp</td><td><a href='/stor2rrd-cgi/detail.sh?host=$ss[0]&type=POOL&name=io'>$ss[0]</a></td><td><a href='/stor2rrd-cgi/detail.sh?host=$ahost&type=$atype&name=$aname&item=$ss[3]'>$ss[1]: $ss[2]</a></td><td>$ss[3]</td><td>$splitted[3]</td>";
      print "</tr>";
    }
    print "</tbody>";
    print "</table>";
  }
}

elsif ( $PAR{cmd} eq "hlog" ) {    ### get haddware alerts log
  print "Content-type: text/html\n\n";
  my @loglines = tail_log("alert_event_history.log", 1, $PAR{count});
  if (@loglines) {
    print "<table class='tabconfig evlog'>";
      print "<thead>";
      print "<tr>";
      print "<th class='sortable'>Date/Time</th><th class='sortable'>Storage</th><th class='sortable'>Reason</th>";
      print "</tr>";
      print "</thead>";
      print "<tbody>";
    foreach my $line (@loglines) {
      if (! $line) {
        next;
      }
      if ($line =~ /^[A-Z]/) {
        next;
      }
      print "<tr>";
      my @splitted = split(" ", $line, 3);
      if (scalar @splitted < 3) {
        next;
      }
      chop $splitted[1];
      my @ss = split(":", $splitted[2], 2);
      #
      my ($d, $m, $y) = split("\/", $splitted[1]);
      # my $timestamp = str2time("${y}-${m}-${d}T$splitted[0]");
      my $timestamp = "${y}-${m}-${d} $splitted[0]";
      print "<td>$timestamp</td><td><a href='/stor2rrd-cgi/detail.sh?host=$ss[0]&type=POOL&name=io'>$ss[0]</a></td><td>$ss[1]</td>";
      print "</tr>";
    }
    print "</tbody>";
    print "</table>";
  }
}

sub tail_log {
  my $filename  = shift;
  my $reverse   = shift;
  my $max_rows  = shift;
  my $file      = $basedir . "/logs/" . $filename;

  if ( -f "$file" ) {
    my @lines;
    if ( $max_rows > 0 ) {
      @lines = `tail -$max_rows $file`;
    }
    else {
      @lines = `cat $file`;
    }
    if ( $reverse == 1 ) {
      return reverse @lines;
    }
    else {
      return @lines;
    }
    return 0;
  }
} ## end sub print_log

sub file_read {
  my $file = shift;
  open IO, $file or die "Cannot open $file for input: $!\n";
  my @data = <IO>;
  close IO;
  wantarray ? @data : join( '' => @data );
} ## end sub file_read
