#!/usr/bin/perl

use strict;
use warnings;
use Data::Dumper;
use LWP::UserAgent;
use HTTP::Request;
use Xorux_lib;
use JSON qw(encode_json decode_json);
use Time::Local;
use Storable;

my $host        = $ENV{'BNA_IP'}          ||= "";
my $host_sec    = $ENV{'BNA_IP_SEC'}      ||= "";
my $port        = $ENV{'BNA_PORT'}        ||= "";
my $user        = $ENV{'BNA_USER'}        ||= "";
my $password    = $ENV{'BNA_PASS'}        ||= "";
my $debug       = $ENV{'BNA_DEBUG'}       ||= 0;

$password = Xorux_lib::unobscure_password($password);

if ( ! defined $host || $host eq '' ) {
  error( "BNA IP must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( ! defined $port || $port eq '' ) {
  error( "BNA PORT must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( ! defined $user || $user eq '' ) {
  error( "BNA USER must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( ! defined $password || $password eq '' ) {
  error( "BNA PASSWORD must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}

my $basedir      = $ENV{INPUTDIR};
my $webdir       = $ENV{WEBDIR};
my $wrkdir       = "$basedir/data";
my $tmpdir       = "$basedir/tmp";

my $get_cpu_mem = 1;
if ( defined $ENV{'BNA_CPU_MEM_RESP'} && $ENV{'BNA_CPU_MEM_RESP'} eq "0" ) {
  $get_cpu_mem = 0;
}

my $url = "https://$host:$port";
my %bna_data;

# output perf file
my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
my $date2perf = sprintf( "%4d%02d%02d", $year + 1900, $month + 1, $day );
my $time2perf = sprintf( "%02d%02d", $hour, $min );


#
# Login to API
#
message("LOGIN") if $debug == 9;
my $wstoken = login_to_api("$url/rest/login");
if ( $wstoken eq "0" ) {
  if ( defined $host_sec && $host_sec ne '' ) { # try second bna ip
    error("Login failed : Can't login to API IP : $host! Try second IP : $host_sec");
    $url     = "https://$host_sec:$port";
    $wstoken = login_to_api("$url/rest/login");
    if ( $wstoken eq "0" ) {
      error("Login failed : Can't login to second IP : $host_sec") && exit;
    }
  }
  else {
    error("Login failed : Can't login to API : $host") && exit;
  }
}


#
# Search for switches
#
message("Search for switches : START") if $debug == 9;
get_all_switches_info();
message("Search for switches : END") if $debug == 9;


#
# Search for ports
#
my $fork_limit = $ENV{'BNA_FORK_LIMIT'} ||= 5;
my $fork_idx   = 0;
my @pids;

my %port_data;
my %last_port_data;
if ( defined $bna_data{'switches'} && $bna_data{'switches'} ne '' ) {
  foreach my $switch_name ( sort keys( %{ $bna_data{'switches'} } ) ) {
    if ( defined $bna_data{'switches'}{$switch_name}{'wwn'} && $bna_data{'switches'}{$switch_name}{'wwn'} ne '' ) {
      $fork_idx++;
      my $pid = fork();
      if (!defined $pid) {
        error( "Cannot fork! SWITCH \"$switch_name\" $!" . __FILE__ . ":" . __LINE__ ) && next;
      }
      elsif ($pid == 0) {
        message("SWITCH : $switch_name") if $debug == 9;

        # get ports perf stats
        #
        message("$switch_name : get ports perf stats : START") if $debug == 9;
        get_port_perf("$switch_name","$bna_data{'switches'}{$switch_name}{'wwn'}");
        message("$switch_name : get ports perf stats : END") if $debug == 9;

        # get ports configuration
        #
        message("$switch_name : get ports configuration : START") if $debug == 9;
        get_port_conf("$switch_name","$bna_data{'switches'}{$switch_name}{'wwn'}");
        message("$switch_name : get ports configuration : END") if $debug == 9;

        # Switch CPU/Memory/Response time
        #
        if ( $get_cpu_mem == 1 ) {
          message("Switch $switch_name cpu, mem, resp_t : START") if $debug == 9;
          get_switch_perf("$switch_name","$bna_data{'switches'}{$switch_name}{'wwn'}");
          message("Switch $switch_name cpu, mem, resp_t : END") if $debug == 9;
        }

        # create data dir, PORTS.cfg, SAN-BRCD, SAN-BNA, HEALTH_STATUS.txt, unused_ports.txt, global health status
        #
        create_switch_conf_files("$switch_name");

        # create perf file
        #
        my $stats_dump_filename = "$switch_name-laststats.dump";
        my $stats_dump_file     = "$wrkdir/$switch_name/$stats_dump_filename";
        my $perf_file           = "$wrkdir/$switch_name/$switch_name\_sanperf_$date2perf\_$time2perf.out.tmp";

        if ( -f "$stats_dump_file" ) {
          my $stats_dump_file_time_diff = file_time_diff("$stats_dump_file");
          if ( $stats_dump_file_time_diff > 3600 ) {
            unlink( "$stats_dump_file" );
            store( \%port_data, $stats_dump_file );
            next;
          }
          # load last stored data
          %last_port_data = %{retrieve($stats_dump_file)};
          message("$switch_name : print perf file : START") if $debug == 9;
          print_perf("$switch_name","$perf_file");
          message("$switch_name : print perf file : END") if $debug == 9;

          unlink( "$stats_dump_file" );
          store( \%port_data, $stats_dump_file );
        }
        else {
          store( \%port_data, $stats_dump_file );
        }
        %port_data      = ();
        %last_port_data = ();


        # save data from fork
        my $data_from_fork = "$wrkdir/$switch_name/$switch_name-forkdata.dump";
        if ( -f "$data_from_fork" ) { unlink( "$data_from_fork" ); }
        store( \%bna_data, $data_from_fork );

        exit 0;
      }
      else {
        push(@pids, $pid);

        if ( $fork_idx >= $fork_limit ) {
          wait4pids();

          @pids     = ();
          $fork_idx = 0;
        }
      }
    }
  }
}
wait4pids();
#
# load collected data from forks
#
if ( defined $bna_data{'switches'} && $bna_data{'switches'} ne '' ) {
  foreach my $switch_name ( sort keys( %{ $bna_data{'switches'} } ) ) {
    if ( defined $bna_data{'switches'}{$switch_name}{'wwn'} && $bna_data{'switches'}{$switch_name}{'wwn'} ne '' ) {
      my $data_from_fork = "$wrkdir/$switch_name/$switch_name-forkdata.dump";
      if ( -f "$data_from_fork" ) {
        my %bna_data_from_fork = %{retrieve($data_from_fork)};
        unlink( "$data_from_fork" );

        if ( exists $bna_data{'switches'}{$switch_name} && exists $bna_data_from_fork{'switches'}{$switch_name} ) {
          %{ $bna_data{'switches'}{$switch_name} } = ( %{ $bna_data_from_fork{'switches'}{$switch_name} }, %{ $bna_data{'switches'}{$switch_name} } );
        }
      }
    }
  }
}


#
# Fabrics
#
message("Search for Fabrics : START") if $debug == 9;
get_all_fabrics_info();
message("Search for Fabrics : END") if $debug == 9;


#
# ISL ports
#
message("Search for ISL ports : START") if $debug == 9;
get_all_isl_info();
message("Search for ISL ports : END") if $debug == 9;


#
# Zones, Configuration
#
my $zone_html = "$webdir/BNA/zones_cfg.html";
my $zone_html_file_time_diff = file_time_diff("$zone_html");
if ( !-f "$zone_html" || $zone_html_file_time_diff > 21599 ) { # once per 6 hours
  message("Search for Zones : START") if $debug == 9;
  get_all_zones_info();
  message("Search for Zones : END") if $debug == 9;
  message("create Zones configuration file : START") if $debug == 9;
  print_zones_conf("$zone_html");
  message("create Zones configuration file : END") if $debug == 9;

  create_configuration();
}


#
# Events
#
my $events_html = "$webdir/BNA/all_events.html";
my $events_html_file_time_diff = file_time_diff("$events_html");
if ( !-f "$events_html" || $events_html_file_time_diff > 3599 ) { # once per hour
  message("Search for Events : START") if $debug == 9;
  get_all_events();
  message("Search for Events : END") if $debug == 9;
}


#
# Totals : health status for switches
#
my $switches_hs_file = "$webdir/BNA/health_status_all.html";
message("create BNA/health_status_all.html : START") if $debug == 9;
create_totals_hs_file();
message("create BNA/health_status_all.html : END") if $debug == 9;


#
# Logout to API
#
logout_from_api("$url/rest/logout");
message("LOGOUT") if $debug == 9;

#print Dumper \%bna_data;
#print Dumper(\%{$bna_data{'fabrics'}});

exit;

sub create_totals_hs_file {
  open( THS, ">$switches_hs_file-tmp" ) || error( "Couldn't open file $switches_hs_file-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;
  print THS "<center><br><br><br>\n";
  print THS "<table class =\"tabcfgsum tablesorter tablesortercfgsum bnaallswitches\">\n";
  print THS "<thead><tr><th class= \"sortable\">Switch name</th><th class= \"sortable\">Operational Status</th><th class= \"sortable\">Status Reason</th>\n";
  print THS "</tr></thead><tbody>\n";

  if ( defined $bna_data{'switches'} && $bna_data{'switches'} ne '' ) {
    foreach my $switch_name ( sort keys( %{ $bna_data{'switches'} } ) ) {
      my $st_class = "st_green";
      my $status   = "";
      my $reason   = "";

      if ( defined $bna_data{'switches'}{$switch_name}{'operationalStatus'} && $bna_data{'switches'}{$switch_name}{'operationalStatus'} ne '' ) {
        $status = $bna_data{'switches'}{$switch_name}{'operationalStatus'};
      }
      if ( defined $bna_data{'switches'}{$switch_name}{'statusReason'} && $bna_data{'switches'}{$switch_name}{'statusReason'} ne '' ) {
        $reason = $bna_data{'switches'}{$switch_name}{'statusReason'};
      }
      if ( $status !~ m/^HEALTHY/ ) { $st_class = "st_red"; }

      print THS "<tr><td><a href=\"$switch_name/health_status.html\"><span class=\"$st_class\"></span>$switch_name</a></td><td>$status</td><td>$reason</td></tr>\n";
    }
  }

  print THS "</tbody></table>\n";
  my $last_upd = localtime();
  print THS "<br><br><br>Last update: $last_upd\n";

  close(THS);
  rename( "$switches_hs_file-tmp", "$switches_hs_file" );

  return 1;
}

sub wait4pids {
  my $timeout = 300;

  if (@pids) {
    foreach my $pid (@pids) {
      eval {
        local $SIG{ALRM} = sub { die "die in SIGALRM"; };
        alarm($timeout);
        my $res = waitpid( $pid, 0 );
        alarm(0);
      };
      if ( $@ =~ /die in SIGALRM/ ) {
        my $act_time = localtime();
        #error( "timed out after: $timeout seconds " . __FILE__ . ":" . __LINE__ );
        print "timed out after: $timeout seconds\n";
        kill -15, $pid;
        my $res = waitpid( $pid, 0 );
      }
    }
  }

  return 1;
}

sub get_all_events {
  # Severity:
  #   EMERGENCY
  #   ALERT
  #   CRITICAL
  #   ERROR
  #   WARNING
  #   NOTICE
  #   INFO
  #   DEBUG
  #   UNKNOWN

  my $event_idx = 0;
  my %events;

  my $result = new_get_request("$url/rest/resourcegroups/All/events?count=50&severity=EMERGENCY","");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'events'} && ref($result->{'events'}) eq "ARRAY"  ) {
      foreach ( @{ $result->{'events'} } ) {
        $event_idx++;

        my $event_timestamp = "";
        my $event_date      = "";
        my $severity        = "";
        my $module          = "";
        my $description     = "";
        my $sourceAddr      = "";
        my $sourceName      = "";
        my $eventCategory   = "";

        if ( defined $_->{'severity'} && $_->{'severity'} ne '' ) {
          $severity = $_->{'severity'};
        }
        if ( defined $_->{'module'} && $_->{'module'} ne '' ) {
          $module = $_->{'module'};
        }
        if ( defined $_->{'lastOccurrenceHostTime'} && isdigit($_->{'lastOccurrenceHostTime'}) ) {
          $event_timestamp = $_->{'lastOccurrenceHostTime'};
          $event_date      = localtime($event_timestamp);
        }
        if ( defined $_->{'description'} && $_->{'description'} ne '' ) {
          $description = $_->{'description'};
        }
        if ( defined $_->{'sourceAddr'} && $_->{'sourceAddr'} ne '' ) {
          $sourceAddr = $_->{'sourceAddr'};
        }
        if ( defined $_->{'sourceName'} && $_->{'sourceName'} ne '' ) {
          $sourceName = $_->{'sourceName'};
        }
        if ( defined $_->{'eventCategory'} && $_->{'eventCategory'} ne '' ) {
          $eventCategory = $_->{'eventCategory'};
        }

        $events{$event_timestamp}{$event_idx}{'date'} = $event_date;
        $events{$event_timestamp}{$event_idx}{'severity'} = $severity;
        $events{$event_timestamp}{$event_idx}{'module'} = $module;
        $events{$event_timestamp}{$event_idx}{'description'} = $description;
        $events{$event_timestamp}{$event_idx}{'sourceAddr'} = $sourceAddr;
        $events{$event_timestamp}{$event_idx}{'sourceName'} = $sourceName;
        $events{$event_timestamp}{$event_idx}{'eventCategory'} = $eventCategory;
      }
    }
  }
  $result = new_get_request("$url/rest/resourcegroups/All/events?count=50&severity=ALERT","");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'events'} && ref($result->{'events'}) eq "ARRAY"  ) {
      foreach ( @{ $result->{'events'} } ) {
        $event_idx++;

        my $event_timestamp = "";
        my $event_date      = "";
        my $severity        = "";
        my $module          = "";
        my $description     = "";
        my $sourceAddr      = "";
        my $sourceName      = "";
        my $eventCategory   = "";

        if ( defined $_->{'severity'} && $_->{'severity'} ne '' ) {
          $severity = $_->{'severity'};
        }
        if ( defined $_->{'module'} && $_->{'module'} ne '' ) {
          $module = $_->{'module'};
        }
        if ( defined $_->{'lastOccurrenceHostTime'} && isdigit($_->{'lastOccurrenceHostTime'}) ) {
          $event_timestamp = $_->{'lastOccurrenceHostTime'};
          $event_date      = localtime($event_timestamp);
        }
        if ( defined $_->{'description'} && $_->{'description'} ne '' ) {
          $description = $_->{'description'};
        }
        if ( defined $_->{'sourceAddr'} && $_->{'sourceAddr'} ne '' ) {
          $sourceAddr = $_->{'sourceAddr'};
        }
        if ( defined $_->{'sourceName'} && $_->{'sourceName'} ne '' ) {
          $sourceName = $_->{'sourceName'};
        }
        if ( defined $_->{'eventCategory'} && $_->{'eventCategory'} ne '' ) {
          $eventCategory = $_->{'eventCategory'};
        }

        $events{$event_timestamp}{$event_idx}{'date'} = $event_date;
        $events{$event_timestamp}{$event_idx}{'severity'} = $severity;
        $events{$event_timestamp}{$event_idx}{'module'} = $module;
        $events{$event_timestamp}{$event_idx}{'description'} = $description;
        $events{$event_timestamp}{$event_idx}{'sourceAddr'} = $sourceAddr;
        $events{$event_timestamp}{$event_idx}{'sourceName'} = $sourceName;
        $events{$event_timestamp}{$event_idx}{'eventCategory'} = $eventCategory;
      }
    }
  }
  $result = new_get_request("$url/rest/resourcegroups/All/events?count=50&severity=CRITICAL","");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'events'} && ref($result->{'events'}) eq "ARRAY"  ) {
      foreach ( @{ $result->{'events'} } ) {
        $event_idx++;

        my $event_timestamp = "";
        my $event_date      = "";
        my $severity        = "";
        my $module          = "";
        my $description     = "";
        my $sourceAddr      = "";
        my $sourceName      = "";
        my $eventCategory   = "";

        if ( defined $_->{'severity'} && $_->{'severity'} ne '' ) {
          $severity = $_->{'severity'};
        }
        if ( defined $_->{'module'} && $_->{'module'} ne '' ) {
          $module = $_->{'module'};
        }
        if ( defined $_->{'lastOccurrenceHostTime'} && isdigit($_->{'lastOccurrenceHostTime'}) ) {
          $event_timestamp = $_->{'lastOccurrenceHostTime'};
          $event_date      = localtime($event_timestamp);
        }
        if ( defined $_->{'description'} && $_->{'description'} ne '' ) {
          $description = $_->{'description'};
        }
        if ( defined $_->{'sourceAddr'} && $_->{'sourceAddr'} ne '' ) {
          $sourceAddr = $_->{'sourceAddr'};
        }
        if ( defined $_->{'sourceName'} && $_->{'sourceName'} ne '' ) {
          $sourceName = $_->{'sourceName'};
        }
        if ( defined $_->{'eventCategory'} && $_->{'eventCategory'} ne '' ) {
          $eventCategory = $_->{'eventCategory'};
        }

        $events{$event_timestamp}{$event_idx}{'date'} = $event_date;
        $events{$event_timestamp}{$event_idx}{'severity'} = $severity;
        $events{$event_timestamp}{$event_idx}{'module'} = $module;
        $events{$event_timestamp}{$event_idx}{'description'} = $description;
        $events{$event_timestamp}{$event_idx}{'sourceAddr'} = $sourceAddr;
        $events{$event_timestamp}{$event_idx}{'sourceName'} = $sourceName;
        $events{$event_timestamp}{$event_idx}{'eventCategory'} = $eventCategory;
      }
    }
  }
  $result = new_get_request("$url/rest/resourcegroups/All/events?count=50&severity=ERROR","");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'events'} && ref($result->{'events'}) eq "ARRAY"  ) {
      foreach ( @{ $result->{'events'} } ) {
        $event_idx++;

        my $event_timestamp = "";
        my $event_date      = "";
        my $severity        = "";
        my $module          = "";
        my $description     = "";
        my $sourceAddr      = "";
        my $sourceName      = "";
        my $eventCategory   = "";

        if ( defined $_->{'severity'} && $_->{'severity'} ne '' ) {
          $severity = $_->{'severity'};
        }
        if ( defined $_->{'module'} && $_->{'module'} ne '' ) {
          $module = $_->{'module'};
        }
        if ( defined $_->{'lastOccurrenceHostTime'} && isdigit($_->{'lastOccurrenceHostTime'}) ) {
          $event_timestamp = $_->{'lastOccurrenceHostTime'};
          $event_timestamp = $event_timestamp / 1000;
          $event_timestamp = sprintf "%.0f", $event_timestamp;
          $event_date      = localtime($event_timestamp);
        }
        if ( defined $_->{'description'} && $_->{'description'} ne '' ) {
          $description = $_->{'description'};
        }
        if ( defined $_->{'sourceAddr'} && $_->{'sourceAddr'} ne '' ) {
          $sourceAddr = $_->{'sourceAddr'};
        }
        if ( defined $_->{'sourceName'} && $_->{'sourceName'} ne '' ) {
          $sourceName = $_->{'sourceName'};
        }
        if ( defined $_->{'eventCategory'} && $_->{'eventCategory'} ne '' ) {
          $eventCategory = $_->{'eventCategory'};
        }

        $events{$event_timestamp}{$event_idx}{'date'} = $event_date;
        $events{$event_timestamp}{$event_idx}{'severity'} = $severity;
        $events{$event_timestamp}{$event_idx}{'module'} = $module;
        $events{$event_timestamp}{$event_idx}{'description'} = $description;
        $events{$event_timestamp}{$event_idx}{'sourceAddr'} = $sourceAddr;
        $events{$event_timestamp}{$event_idx}{'sourceName'} = $sourceName;
        $events{$event_timestamp}{$event_idx}{'eventCategory'} = $eventCategory;
      }
    }
  }


  # create BNA_events.html
  #
  open( EVE, ">$events_html-tmp" ) || error( "Couldn't open file $events_html-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;

  my $last_update = localtime();
  print EVE "<center>\n";
  print EVE "<br><br>Last update: $last_update<br><br>\n";

  print EVE "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
  print EVE "<thead><tr>\n";
  print EVE "<th class= \"sortable\">Time</th>\n";
  print EVE "<th class= \"sortable\">Severity</th>\n";
  print EVE "<th class= \"sortable\">Description</th>\n";
  print EVE "<th class= \"sortable\">sourceAddr</th>\n";
  print EVE "<th class= \"sortable\">sourceName</th>\n";
  print EVE "<th class= \"sortable\">Category</th>\n";
  print EVE "</tr></thead>\n";
  print EVE "<tbody>\n";

  if ( $event_idx > 0 ) {
    foreach my $timestamp ( sort { $b <=> $a } keys( %events ) ) {
      if ( defined $events{$timestamp} && $events{$timestamp} ne '' ) {
        foreach ( sort { $a <=> $b } keys( %{ $events{$timestamp} } ) ) {
          #print "$timestamp : $events{$timestamp}{$_}{'date'}\n";

          print EVE "<tr>\n";
          print EVE "<td>$events{$timestamp}{$_}{'date'}</td>\n";
          print EVE "<td>$events{$timestamp}{$_}{'severity'}</td>\n";
          print EVE "<td>$events{$timestamp}{$_}{'description'}</td>\n";
          print EVE "<td>$events{$timestamp}{$_}{'sourceAddr'}</td>\n";
          print EVE "<td>$events{$timestamp}{$_}{'sourceName'}</td>\n";
          print EVE "<td>$events{$timestamp}{$_}{'eventCategory'}</td>\n";
          print EVE "</tr>\n";
        }
      }
    }
  }

  print EVE "</tbody></table>\n";
  print EVE "</center>\n";
  close(EVE);
  rename( "$events_html-tmp", "$events_html" );


  return 1;
}

sub get_switch_perf {
  my $switch_name = shift;
  my $switch_wwn  = shift;

  my $act_time = time();
  $bna_data{'switches'}{$switch_name}{'perf'}{'timestamp'} = $act_time;

  #CPU
  my $result = new_get_request("$url/rest/resourcegroups/All/fcswitches/$switch_wwn/cpuutilpercentage","");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'summary'}{'summaryData'} && ref($result->{'summary'}{'summaryData'}) eq "ARRAY"  ) {
      foreach ( @{ $result->{'summary'}{'summaryData'} } ) {
        if ( defined $_->{'value'} && isdigit($_->{'value'}) ) {
          #print "$switch_name : CPU = $_->{'value'}\n";
          $bna_data{'switches'}{$switch_name}{'perf'}{'CPU'} = sprintf "%.2f", $_->{'value'};
        }
      }
    }
  }

  # Memory
  $result = new_get_request("$url/rest/resourcegroups/All/fcswitches/$switch_wwn/memoryutilpercentage","");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'summary'}{'summaryData'} && ref($result->{'summary'}{'summaryData'}) eq "ARRAY"  ) {
      foreach ( @{ $result->{'summary'}{'summaryData'} } ) {
        if ( defined $_->{'value'} && isdigit($_->{'value'}) ) {
          #print "$switch_name : MEM = $_->{'value'}\n";
          $bna_data{'switches'}{$switch_name}{'perf'}{'MEM'} = sprintf "%.2f", $_->{'value'};
        }
      }
    }
  }

  # Resp_t
  $result = new_get_request("$url/rest/resourcegroups/All/fcswitches/$switch_wwn/responsetime","");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'summary'}{'summaryData'} && ref($result->{'summary'}{'summaryData'}) eq "ARRAY"  ) {
      foreach ( @{ $result->{'summary'}{'summaryData'} } ) {
        if ( defined $_->{'value'} && isdigit($_->{'value'}) ) {
          #print "$switch_name : RESP_T = $_->{'value'}\n";
          $bna_data{'switches'}{$switch_name}{'perf'}{'RESP_T'} = sprintf "%.2f", $_->{'value'};
        }
      }
    }
  }

  return 1;
}

sub get_all_isl_info {
  if ( defined $bna_data{'fabrics'} && $bna_data{'fabrics'} ne '' ) {
    foreach my $fab_prin_wwn ( sort keys( %{ $bna_data{'fabrics'} } ) ) {
      my $result = new_get_request("$url/rest/resourcegroups/All/fcfabrics/$fab_prin_wwn/isls","");
      if ( defined $result && $result ne "0" ) {
        #print Dumper $result;
        if ( defined $result->{'isls'} && ref($result->{'isls'}) eq "ARRAY" ) {
          foreach ( @{ $result->{'isls'} } ) {
            if ( defined $_->{'sourceSwitchWwn'} && $_->{'sourceSwitchWwn'} ne '' && defined $_->{'sourceSwitchPortWwn'} && $_->{'sourceSwitchPortWwn'} ne '' && defined $_->{'destSwitchWwn'} && $_->{'destSwitchWwn'} ne '' && defined $_->{'destSwitchPortWwn'} && $_->{'destSwitchPortWwn'} ne '' ) {
              my $from_port_wwn   = $_->{'sourceSwitchPortWwn'};
              my $from_switch_wwn = $_->{'sourceSwitchWwn'};
              my $to_port_wwn     = $_->{'destSwitchPortWwn'};
              my $to_switch_wwn   = $_->{'destSwitchWwn'};

              my $from_port_name   = "";
              my $from_switch_name = "";
              my $to_port_name     = "";
              my $to_switch_name   = "";

              # add human names
              if ( defined $bna_data{'switches'} && $bna_data{'switches'} ne '' ) {
                foreach my $switch_name ( sort keys( %{ $bna_data{'switches'} } ) ) {
                  if ( defined $bna_data{'switches'}{$switch_name}{'wwn'} && $bna_data{'switches'}{$switch_name}{'wwn'} ne '' ) {
                    if ( $bna_data{'switches'}{$switch_name}{'wwn'} eq $from_switch_wwn ) {
                      $from_switch_name = $switch_name;
                      if ( exists $bna_data{'switches'}{$switch_name}{'ports'}{$from_port_wwn}{'PORT_ID'} ) {
                        $from_port_name = $bna_data{'switches'}{$switch_name}{'ports'}{$from_port_wwn}{'PORT_ID'};
                      }
                    }
                    if ( $bna_data{'switches'}{$switch_name}{'wwn'} eq $to_switch_wwn ) {
                      $to_switch_name = $switch_name;
                      if ( exists $bna_data{'switches'}{$switch_name}{'ports'}{$to_port_wwn}{'PORT_ID'} ) {
                        $to_port_name = $bna_data{'switches'}{$switch_name}{'ports'}{$to_port_wwn}{'PORT_ID'};
                      }
                    }
                  }
                }
              }
              if ( defined $from_port_name && $from_port_name ne '' && defined $from_switch_name && $from_switch_name ne '' && defined $to_port_name && $to_port_name ne '' && defined $to_switch_name && $to_switch_name ne '' ) {
                $bna_data{'ISL'}{$from_switch_name}{'ports'}{$from_port_name}{'to_switch'} = $to_switch_name;
                $bna_data{'ISL'}{$from_switch_name}{'ports'}{$from_port_name}{'to_port'}   = $to_port_name;
              }
            }
          }
        }
      }
    }
  }

  # create ISL.txt
  if ( defined $bna_data{'ISL'} && $bna_data{'ISL'} ne '' ) {
    foreach my $switch_name ( sort keys( %{ $bna_data{'ISL'} } ) ) {
      if ( -d "$wrkdir/$switch_name" ) {
        if ( defined $bna_data{'ISL'}{$switch_name}{'ports'} && $bna_data{'ISL'}{$switch_name}{'ports'} ne '' ) {

          my $isl_file = "$wrkdir/$switch_name/ISL.txt";
          open( ISL, ">$isl_file-tmp" ) || error( "Couldn't open file $isl_file-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;

          foreach my $isl_port ( sort keys( %{ $bna_data{'ISL'}{$switch_name}{'ports'} } ) ) {
            my $to_port   = $bna_data{'ISL'}{$switch_name}{'ports'}{$isl_port}{'to_port'};
            my $to_switch = $bna_data{'ISL'}{$switch_name}{'ports'}{$isl_port}{'to_switch'};

            # port16,/home/stor2rrd/stor2rrd/data/SAN-switch-A1/port16.rrd,SAN-switch-A1,to_port=port16,to_switch=SAN-switch-A2
            print ISL "port$isl_port,$wrkdir/$switch_name/port$isl_port.rrd,$switch_name,to_port=port$to_port,to_switch=$to_switch\n";
          }

          close(ISL);
          rename( "$isl_file-tmp", "$isl_file" );
        }
      }
      else {
        error( "ERROR : $switch_name : Can't create ISL.txt! Data directory $wrkdir/$switch_name not exists!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }

  return 1;
}

sub print_zones_conf {
  if ( defined $bna_data{'fabrics'} && $bna_data{'fabrics'} ne '' ) {

    open( MAIN, ">$zone_html-tmp" ) || error( "Couldn't open file $zone_html-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;
    print MAIN "<div id=\"tabs\">\n";
    print MAIN "<ul>\n";

    foreach my $fab_prin_wwn ( sort keys( %{ $bna_data{'fabrics'} } ) ) {
      if ( defined $bna_data{'fabrics'}{$fab_prin_wwn}{'name'} && $bna_data{'fabrics'}{$fab_prin_wwn}{'name'} ne '' ) {
        my $fabric_name      = $bna_data{'fabrics'}{$fab_prin_wwn}{'name'};
        my $fabric_file_name = "$fabric_name.html";
        my $fabric_file      = "$webdir/BNA/$fabric_file_name";

        print MAIN "<li><a href=\"BNA/$fabric_file_name\">$fabric_name</a></li>\n";

        open( FAB, ">$fabric_file-tmp" ) || error( "Couldn't open file $fabric_file-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;

        if ( defined $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'} && $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'} ne '' ) {
          foreach my $zoneset ( sort keys( %{ $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'} } ) ) {

            print FAB "<center><br><br><b>$zoneset</b><br><br>\n";

            if ( defined $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset}{'zones'} && $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset}{'zones'} ne '' ) {

              my %zone_data;
              foreach my $zone_name ( sort keys( %{ $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset}{'zones'} } ) ) {
                if ( defined $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset}{'zones'}{$zone_name}{'memberNames'} && $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset}{'zones'}{$zone_name}{'memberNames'} ne '' ) {
                  my $member_idx = 0;
                  foreach my $member_wwn ( sort keys( %{ $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset}{'zones'}{$zone_name}{'memberNames'} } ) ) {
                    $member_idx++;

                    $zone_data{'header_idx'}{$member_idx} = $member_idx;
                    $zone_data{'zone'}{$zone_name}{'member_idx'}{$member_idx}{'wwn'} = $member_wwn;
                    if ( exists $bna_data{'fabrics'}{$fab_prin_wwn}{'zone_aliases'}{$member_wwn} ) {
                      $zone_data{'zone'}{$zone_name}{'member_idx'}{$member_idx}{'alias'} = "$bna_data{'fabrics'}{$fab_prin_wwn}{'zone_aliases'}{$member_wwn}";
                    }
                  }
                }
              }

              # print html table
              # print header
              print FAB "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
              print FAB "<thead><tr><th class= \"sortable\">Zone name</th>\n";
              if ( defined $zone_data{'header_idx'} && $zone_data{'header_idx'} ne '' ) {
                foreach my $idx ( sort { $a <=> $b } keys( %{ $zone_data{'header_idx'} } ) ) {
                  #print FAB "<th class= \"sortable\">Member $idx - Alias</th><th class= \"sortable\">Member $idx - WWN</th>\n";
                  print FAB "<th class= \"sortable\">Alias $idx</th><th class= \"sortable\">WWN $idx</th>\n";
                }
              }
              print FAB "</tr></thead><tbody>\n";
              # print body
              if ( defined $zone_data{'zone'} && $zone_data{'zone'} ne '' ) {
                foreach my $zone_name ( sort keys( %{ $zone_data{'zone'} } ) ) {
                  print FAB "<tr><td>$zone_name</td>\n";
                  if ( defined $zone_data{'header_idx'} && $zone_data{'header_idx'} ne '' ) {
                    foreach my $idx ( sort { $a <=> $b } keys( %{ $zone_data{'header_idx'} } ) ) {
                      my $alias = "";
                      my $wwn   = "";
                      if ( exists $zone_data{'zone'}{$zone_name}{'member_idx'}{$idx}{'alias'} ) {
                        $alias = $zone_data{'zone'}{$zone_name}{'member_idx'}{$idx}{'alias'};
                      }
                      if ( exists $zone_data{'zone'}{$zone_name}{'member_idx'}{$idx}{'wwn'} ) {
                        $wwn = $zone_data{'zone'}{$zone_name}{'member_idx'}{$idx}{'wwn'};
                      }
                      print FAB "<td>$alias</td><td>$wwn</td>";
                    }
                  }
                  print FAB "</tr>\n";
                }
              }
              print FAB "</tbody></table>\n";

            }
            my $last_update = localtime();
            print FAB "<br><br>Last update: $last_update\n";
            print FAB "</center>\n";
          }
        }
        close(FAB);
        rename( "$fabric_file-tmp", "$fabric_file" );
      }
    }
    print MAIN "</ul>\n";
    print MAIN "</div>\n";
    close(MAIN);
    rename( "$zone_html-tmp", "$zone_html" );
  }

  return 1;
}

sub get_all_zones_info {
  if ( defined $bna_data{'fabrics'} && $bna_data{'fabrics'} ne '' ) {
    foreach my $fab_prin_wwn ( sort keys( %{ $bna_data{'fabrics'} } ) ) {
      # zonesets -> zone -> zone members
      my $result = new_get_request("$url/rest/resourcegroups/All/fcfabrics/$fab_prin_wwn/zonedbs?active=true","");
      if ( defined $result && $result ne "0" ) {
        #print Dumper $result;
        if ( defined $result->{'zonedbs'} && ref($result->{'zonedbs'}) eq "ARRAY" ) {
          foreach ( @{ $result->{'zonedbs'} } ) {
            my $zoneset_name = "";
            # zonesets
            if ( defined $_->{'zonesets'} && ref($_->{'zonesets'}) eq "ARRAY" ) {
              foreach ( @{ $_->{'zonesets'} } ) {
                if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
                  $zoneset_name = $_->{'name'};
                  if ( defined $_->{'zoneNames'} && ref($_->{'zoneNames'}) eq "ARRAY" ) {
                    foreach ( @{ $_->{'zoneNames'} } ) {
                      my $zone_name = $_;
                      $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset_name}{'zones'}{$zone_name}{'name'} = $zone_name;
                    }
                  }
                }
              }
            }
            # zones
            if ( defined $_->{'zones'} && ref($_->{'zones'}) eq "ARRAY" ) {
              foreach ( @{ $_->{'zones'} } ) {
                if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
                  my $zone_name = $_->{'name'};
                  if ( exists $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset_name}{'zones'}{$zone_name} ) {
                    if ( defined $_->{'memberNames'} && ref($_->{'memberNames'}) eq "ARRAY" ) {
                      foreach ( @{ $_->{'memberNames'} } ) {
                        my $member_name = $_;
                        $bna_data{'fabrics'}{$fab_prin_wwn}{'zonesets'}{$zoneset_name}{'zones'}{$zone_name}{'memberNames'}{$member_name} = $member_name;
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
      # zone -> zone alias
      $result = new_get_request("$url/rest/resourcegroups/All/fcfabrics/$fab_prin_wwn/zonealiases","");
      if ( defined $result && $result ne "0" ) {
        #print Dumper $result;
        if ( defined $result->{'zoneAliases'} && ref($result->{'zoneAliases'}) eq "ARRAY" ) {
          foreach ( @{ $result->{'zoneAliases'} } ) {
            if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
              my $zone_alias = $_->{'name'};
              if ( defined $_->{'memberNames'} && ref($_->{'memberNames'}) eq "ARRAY" ) {
                foreach my $member_name ( @{ $_->{'memberNames'} } ) {
                  $bna_data{'fabrics'}{$fab_prin_wwn}{'zone_aliases'}{$member_name} = $zone_alias;
                  $bna_data{'zones'}{$member_name} = $zone_alias;
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub get_all_fabrics_info {
  # Search for Fabrics
  #
  my $result = new_get_request("$url/rest/resourcegroups/All/fcfabrics","");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'fcFabrics'} && ref($result->{'fcFabrics'}) eq "ARRAY" ) {
      my $fabric_idx = 0;
      foreach ( @{ $result->{'fcFabrics'} } ) {
        if ( defined $_->{'principalSwitchWwn'} && $_->{'principalSwitchWwn'} ne '' ) {
          $fabric_idx++;
          my $prin_sw_wwn = $_->{'principalSwitchWwn'};
          my $fabric_name = "";

          $bna_data{'fabrics'}{$prin_sw_wwn}{'principalSwitchWwn'} = $prin_sw_wwn;

          if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
            $bna_data{'fabrics'}{$prin_sw_wwn}{'name'} = $_->{'name'};
            $fabric_name = $_->{'name'};
          }
          if ( defined $_->{'seedSwitchWwn'} && $_->{'seedSwitchWwn'} ne '' ) {
            $bna_data{'fabrics'}{$prin_sw_wwn}{'seedSwitchWwn'} = $_->{'seedSwitchWwn'};
          }
          if ( defined $_->{'seedSwitchIpAddress'} && $_->{'seedSwitchIpAddress'} ne '' ) {
            $bna_data{'fabrics'}{$prin_sw_wwn}{'seedSwitchIpAddress'} = $_->{'seedSwitchIpAddress'};
          }
          if ( defined $_->{'key'} && $_->{'key'} ne '' ) {
            $bna_data{'fabrics'}{$prin_sw_wwn}{'key'} = $_->{'key'};
          }
          if ( defined $_->{'fabricName'} && $_->{'fabricName'} ne '' ) {
            $bna_data{'fabrics'}{$prin_sw_wwn}{'fabricName'} = $_->{'fabricName'};
          }
          if ( defined $_->{'virtualFabricId'} && $_->{'virtualFabricId'} ne '' ) {
            $bna_data{'fabrics'}{$prin_sw_wwn}{'virtualFabricId'} = $_->{'virtualFabricId'};
          }

          # Search for switches in fabric
          #
          my $result = new_get_request("$url/rest/resourcegroups/All/fcfabrics/$prin_sw_wwn/fcswitches","");
          if ( defined $result && $result ne "0" ) {
            #print Dumper $result;
            if ( defined $result->{'fcSwitches'} && ref($result->{'fcSwitches'}) eq "ARRAY" ) {
              my $switch_idx = 0;
              foreach ( @{ $result->{'fcSwitches'} } ) {
                if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
                  $switch_idx++;
                  my $switch_name = "$_->{'name'}-BNA";
                  my $switch_wwn  = "";

                  $bna_data{'fabrics'}{$prin_sw_wwn}{'switches'}{$switch_name}{'name'} = $_->{'name'};
                  if ( defined $_->{'virtualFabricId'} && $_->{'virtualFabricId'} ne '' ) {
                    $bna_data{'fabrics'}{$prin_sw_wwn}{'switches'}{$switch_name}{'virtualFabricId'} = $_->{'virtualFabricId'};
                  }
                  if ( defined $_->{'fcsRole'} && $_->{'fcsRole'} ne '' ) {
                    $bna_data{'fabrics'}{$prin_sw_wwn}{'switches'}{$switch_name}{'fcsRole'} = $_->{'fcsRole'};
                  }
                  if ( defined $_->{'domainId'} && $_->{'domainId'} ne '' ) {
                    $bna_data{'fabrics'}{$prin_sw_wwn}{'switches'}{$switch_name}{'domainId'} = $_->{'domainId'};
                  }
                  if ( defined $_->{'type'} && $_->{'type'} ne '' ) {
                    $bna_data{'fabrics'}{$prin_sw_wwn}{'switches'}{$switch_name}{'type'} = $_->{'type'};
                  }
                  if ( defined $_->{'role'} && $_->{'role'} ne '' ) {
                    $bna_data{'fabrics'}{$prin_sw_wwn}{'switches'}{$switch_name}{'role'} = $_->{'role'};
                  }
                  if ( defined $_->{'wwn'} && $_->{'wwn'} ne '' ) {
                    $bna_data{'fabrics'}{$prin_sw_wwn}{'switches'}{$switch_name}{'wwn'} = $_->{'wwn'};
                    $switch_wwn = $_->{'wwn'};

                    # create fabric.txt
                    if ( -d "$wrkdir/$switch_name" ) {
                      my $principal_switch_wwn = $prin_sw_wwn;
                      $principal_switch_wwn    =~ s/://g;
                      $switch_wwn              =~ s/://g;

                      my $fab_file = "$wrkdir/$switch_name/fabric.txt";

                      open( FAB, ">$fab_file-tmp" ) || error( "Couldn't open file $fab_file-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;
                      print FAB "$principal_switch_wwn,$switch_wwn,$switch_name,$fabric_name\n";
                      close(FAB);
                      rename( "$fab_file-tmp", "$fab_file" );
                    }
                    else {
                      error( "$switch_name : Can't create fabric.txt file! Directory \"$wrkdir/$switch_name\" not exists! " . __FILE__ . ":" . __LINE__ );
                    }

                    #my $result = new_get_request("$url/rest/resourcegroups/All/fcfabrics/$prin_sw_wwn/fcswitches/$_->{'wwn'}/fcports","");
                    #if ( defined $result && $result ne "0" ) {
                    #  print Dumper $result;
                    #}
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub print_perf {
  my $switch_name = shift;
  my $perf_file   = shift;


  open( PERF, ">$perf_file-tmp" ) || error( "Couldn't open file $perf_file-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;
  print PERF "act_time,bytes_tra,bytes_rec,frames_tra,frames_rec,swFCPortNoTxCredits,swFCPortRxCrcs,port_speed,reserve1,reserve2,reserve3,reserve4,reserve5,reserve6,switch_name,db_name,rrd_file\n";

  if ( defined $port_data{'switch'}{$switch_name}{'ports'} && $port_data{'switch'}{$switch_name}{'ports'} ne '' ) {
    foreach my $port_wwn ( sort keys( %{ $port_data{'switch'}{$switch_name}{'ports'} } ) ) {
      if ( exists $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'timestamp'} && isdigit($port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'timestamp'}) && exists $last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'timestamp'} && isdigit($last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'timestamp'}) ) {
        my $interval  = $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'timestamp'} - $last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'timestamp'};
        my $stat_time = $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'timestamp'};

        my $port_id = "";
        if ( defined $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'} && $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'} ne '' ) {
          $port_id = $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'};
        }
        else {
          next; # it should not happen, just for sure...
        }

        my $data_out   = "U";
        my $data_in    = "U";
        my $frame_out  = "U";
        my $frame_in   = "U";
        my $bbcredits  = "U";
        my $crc        = "U";
        my $port_speed = "U";

        if ( defined $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'speed'} && isdigit($port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'speed'}) ) {
          $port_speed = $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'speed'} * 125000;
        }
        if ( exists $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_out'} && exists $last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_out'} ) {
          $data_out = make_gauge("$port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_out'}","$last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_out'}","$interval");
        }
        if ( exists $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_in'} && exists $last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_in'} ) {
          $data_in = make_gauge("$port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_in'}","$last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_in'}","$interval");
        }
        if ( exists $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_out'} && exists $last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_out'} ) {
          $frame_out = make_gauge("$port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_out'}","$last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_out'}","$interval");
        }
        if ( exists $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_in'} && exists $last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_in'} ) {
          $frame_in = make_gauge("$port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_in'}","$last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_in'}","$interval");
        }
        if ( exists $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'bbcredits'} && exists $last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'bbcredits'} ) {
          $bbcredits = make_gauge("$port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'bbcredits'}","$last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'bbcredits'}","$interval");
        }
        if ( exists $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'crc'} && exists $last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'crc'} ) {
          $crc = make_gauge("$port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'crc'}","$last_port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'crc'}","$interval");
        }

        print PERF "$stat_time,$data_out,$data_in,$frame_out,$frame_in,$bbcredits,$crc,$port_speed,,,,,,,$switch_name,port$port_id.rrd,$wrkdir/$switch_name/port$port_id.rrd\n";
      }
    }
  }

  # switch stats : cpu, mem, resp_t
  print PERF "\n";
  print PERF "act_time,cpu,mem,resp_t,reserve1,reserve2,switch_name,db_name,rrd_file\n";

  my $db_name  = "switch_stats.rrs";
  my $rrd_file = "$wrkdir/$switch_name/$db_name";
  my $act_time = time();
  my $cpu      = "";
  my $mem      = "";
  my $resp_t   = "";

  if ( defined $bna_data{'switches'}{$switch_name}{'perf'}{'timestamp'} && isdigit($bna_data{'switches'}{$switch_name}{'perf'}{'timestamp'}) ) {
    $act_time = $bna_data{'switches'}{$switch_name}{'perf'}{'timestamp'};
  }
  if ( defined $bna_data{'switches'}{$switch_name}{'perf'}{'CPU'} && isdigit($bna_data{'switches'}{$switch_name}{'perf'}{'CPU'}) ) {
    $cpu = $bna_data{'switches'}{$switch_name}{'perf'}{'CPU'};
  }
  if ( defined $bna_data{'switches'}{$switch_name}{'perf'}{'MEM'} && isdigit($bna_data{'switches'}{$switch_name}{'perf'}{'MEM'}) ) {
    $mem = $bna_data{'switches'}{$switch_name}{'perf'}{'MEM'};
  }
  if ( defined $bna_data{'switches'}{$switch_name}{'perf'}{'RESP_T'} && isdigit($bna_data{'switches'}{$switch_name}{'perf'}{'RESP_T'}) ) {
    $resp_t = $bna_data{'switches'}{$switch_name}{'perf'}{'RESP_T'};
  }

  print PERF "$act_time,$cpu,$mem,$resp_t,,,$switch_name,$db_name,$rrd_file\n";

  close(PERF);
  rename( "$perf_file-tmp", "$perf_file" );

  return 1;
}

sub create_configuration {
  if ( defined $bna_data{'switches'} && $bna_data{'switches'} ne '' ) {
    foreach my $switch_name ( sort keys( %{ $bna_data{'switches'} } ) ) {

      # Configuration
      my $switch_ip     = "";
      my $switch_wwn    = "";
      my $switch_model  = "";
      my $switch_os_v   = "";
      my $serial_number = "";
      my $fabric_name   = "";

      if ( defined $bna_data{'switches'}{$switch_name}{'phys_conf'}{'ipAddress'} && $bna_data{'switches'}{$switch_name}{'phys_conf'}{'ipAddress'} ne '' ) {
        $switch_ip = $bna_data{'switches'}{$switch_name}{'phys_conf'}{'ipAddress'};
      }
      if ( defined $bna_data{'switches'}{$switch_name}{'wwn'} && $bna_data{'switches'}{$switch_name}{'wwn'} ne '' ) {
        $switch_wwn = $bna_data{'switches'}{$switch_name}{'wwn'};
      }
      if ( defined $bna_data{'switches'}{$switch_name}{'phys_conf'}{'modelNumber'} && $bna_data{'switches'}{$switch_name}{'phys_conf'}{'modelNumber'} ne '' ) {
        $switch_model = $bna_data{'switches'}{$switch_name}{'phys_conf'}{'modelNumber'};
      }
      if ( defined $bna_data{'switches'}{$switch_name}{'phys_conf'}{'firmwareVersion'} && $bna_data{'switches'}{$switch_name}{'phys_conf'}{'firmwareVersion'} ne '' ) {
        $switch_os_v = $bna_data{'switches'}{$switch_name}{'phys_conf'}{'firmwareVersion'};
      }
      if ( defined $bna_data{'switches'}{$switch_name}{'phys_conf'}{'switchSerialNumber'} && $bna_data{'switches'}{$switch_name}{'phys_conf'}{'switchSerialNumber'} ne '' ) {
        $serial_number = $bna_data{'switches'}{$switch_name}{'phys_conf'}{'switchSerialNumber'};
      }

      if ( -f "$wrkdir/$switch_name/fabric.txt" ) {
        open( FAB, "<$wrkdir/$switch_name/fabric.txt" ) || error( "Couldn't open file $wrkdir/$switch_name/fabric.txt $!" . __FILE__ . ":" . __LINE__ ) && exit;
        my $fab_line = <FAB>;
        close(FAB);

        (undef, undef, undef, $fabric_name) = split(/,/, $fab_line);
      }

      my $ports_idx        = 0;
      my $unused_ports_idx = 0;
      if ( defined $bna_data{'switches'}{$switch_name}{'ports'} && $bna_data{'switches'}{$switch_name}{'ports'} ne '' ) {
        foreach my $port_wwn ( sort keys( %{ $bna_data{'switches'}{$switch_name}{'ports'} } ) ) {
          if ( defined $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'} && $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'} ne '' ) {
            $ports_idx++;

            # HEALTH_STATUS.txt
            my $port_phys_status = "";
            my $port_oper_status = "";
            if ( defined $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'status'} && $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'status'} ne '' ) {
              $port_phys_status = $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'status'};
            }
            if ( defined $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'state'} && $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'state'} ne '' ) {
              $port_oper_status = $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'state'};
            }

            # unused_ports.txt
            if ( defined $port_phys_status && $port_phys_status eq "NO_MODULE" && defined $port_oper_status && $port_oper_status eq "OFFLINE" ) {
              $unused_ports_idx++;
            }
          }
        }
      }


      # config.cfg
      my $config_cfg = "$wrkdir/$switch_name/config.cfg";

      open( CFG, ">$config_cfg-tmp" ) || error( "Couldn't open file $config_cfg-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;
      print CFG "switch_name=$switch_name\n";
      print CFG "switch_ip=$switch_ip\n";
      print CFG "switch_wwn=$switch_wwn\n";
      print CFG "switch_model=$switch_model\n";
      print CFG "switch_speed=\n";
      print CFG "switch_os_version=$switch_os_v\n";
      print CFG "switch_fabric_name=$fabric_name\n";
      print CFG "switch_days_up=\n";
      print CFG "switch_serial_number=$serial_number\n";
      print CFG "switch_total_ports=$ports_idx\n";
      print CFG "switch_unused_ports=$unused_ports_idx\n";
      print CFG "switch_total_isl_ports=\n";
      close(CFG);
      rename( "$config_cfg-tmp", "$config_cfg" );


      # config.html
      my $config_html = "$webdir/$switch_name/config.html";

      # read etc/alias.cfg
      my $afile = "$basedir/etc/alias.cfg";
      my @alines;
      if ( -f $afile ) {
        open( FHA, "< $afile" ) || error( "File cannot be opened for read, wrong user rights?? : $afile " . __FILE__ . ":" . __LINE__ ) && next;
        @alines = <FHA>;
        close(FHA);
      }

      # isl
      my $isl_cfg = "$wrkdir/$switch_name/ISL.txt";
      my @isl;
      if ( -f $isl_cfg ) {
        open( ISL, "<$isl_cfg" ) || error( "Couldn't open file $isl_cfg $!" . __FILE__ . ":" . __LINE__ ) && next;
        @isl = <ISL>;
        close(ISL);
      }

      my $total_isl_ports = "";
      if (@isl) {
        $total_isl_ports = @isl;
      }

      open( CFGH, ">$config_html-tmp" ) || error( "Couldn't open file $config_html-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;

      print CFGH "<br><br><br>\n";
      print CFGH "<center>\n";
      print CFGH "<table class =\"tabcfgsum tablesortercfgsum\">\n";
      print CFGH "<thead>\n";
      print CFGH "<tr>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Switch Name</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">IP Address</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">World Wide Name</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Model</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Speed</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">OS Version</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Days Up</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Serial Number</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Total Ports</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">ISL Ports</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Unused Ports</th>\n";
      print CFGH "</tr>\n";
      print CFGH "</thead>\n";
      print CFGH "<tbody>\n";

      print CFGH "<tr>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$switch_name</td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$switch_ip</td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$switch_wwn</td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$switch_model</td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\"></td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$switch_os_v</td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\"></td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$serial_number</td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$ports_idx</td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$total_isl_ports</td>\n";
      print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$unused_ports_idx</td>\n";
      print CFGH "</tr>\n";
      print CFGH "</tbody>\n";
      print CFGH "</table>\n";

      print CFGH "<br><br><br>\n";
      print CFGH "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
      print CFGH "<thead>\n";
      print CFGH "<tr>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Port</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Alias (from switch)</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Alias (from alias.cfg)</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Zone WWN</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Zone Alias</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Speed</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">ISL</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">State</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Encrypt</th>\n";
      print CFGH "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Compress</th>\n";
      print CFGH "</tr>\n";
      print CFGH "</thead>\n";
      print CFGH "<tbody>\n";

      # workaround for sorting
      if ( $ports_idx > 0 ) {
        foreach my $slot ( sort { $a <=> $b } keys ( %{ $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'} } ) ) {
          foreach ( sort { $a <=> $b } keys ( %{ $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'} } ) ) {
            my $port_name      = "";
            my $state          = "-";
            my $wwn            = "-";
            my $alias_from_sw  = "-";
            my $alias_from_cfg = "-";
            my $port_isl       = "-";
            my $zone_wwn       = "-";
            my $zone_alias     = "-";

            if ( $slot == 0 ) { $port_name = $_; }
            else              { $port_name = "$slot-$_"; }

            if ( defined $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'}{$_}{'unused_port_line'} && $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'}{$_}{'unused_port_line'} ne '' ) {
              $state = "unused";
            }
            if ( defined $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'}{$_}{'wwn'} && $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'}{$_}{'wwn'} ne '' ) {
              $wwn = $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'}{$_}{'wwn'};
              #$bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'portNumber'}
              if ( defined $bna_data{'switches'}{$switch_name}{'ports'}{$wwn}{'remotePortWwn'} && $bna_data{'switches'}{$switch_name}{'ports'}{$wwn}{'remotePortWwn'} ne '' ) {
                $zone_wwn = $bna_data{'switches'}{$switch_name}{'ports'}{$wwn}{'remotePortWwn'};

                #$bna_data{'zones'}{$member_name} = $zone_alias;
                if ( exists $bna_data{'zones'}{$zone_wwn} && $bna_data{'zones'}{$zone_wwn} ne '' ) {
                  $zone_alias = $bna_data{'zones'}{$zone_wwn};
                }
              }
            }
            if ( defined $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'}{$_}{'alias'} && $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'}{$_}{'alias'} ne '' ) {
              $alias_from_sw = $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$slot}{'port_id'}{$_}{'alias'};
            }
            # use alias from alias.cfg
            my $alias_from_admin_line  = "";
            my $alias_from_admin_line2 = "";
            ($alias_from_admin_line)  = grep {/^SANPORT:$switch_name:port$port_name:/} @alines;
            ($alias_from_admin_line2) = grep {/^SANPORT:$switch_name:$port_name:/} @alines;
            if ( defined $alias_from_admin_line && $alias_from_admin_line ne '' ) {
              my ( undef, undef, undef, $alias_from_admin ) = split( ":", $alias_from_admin_line );
              if ( defined $alias_from_admin && $alias_from_admin ne '' ) {
                $alias_from_cfg = "$alias_from_admin";
              }
            } ## end if ( defined $alias_from_admin_line...)
            if ( defined $alias_from_admin_line2 && $alias_from_admin_line2 ne '' ) {
              my ( undef, undef, undef, $alias_from_admin ) = split( ":", $alias_from_admin_line2 );
              if ( defined $alias_from_admin && $alias_from_admin ne '' ) {
                $alias_from_cfg = "$alias_from_admin";
              }
            } ## end if ( defined $alias_from_admin_line2...)

            # isl
            my $isl_line = "";
            ($isl_line) = grep {/^port$port_name,/} @isl;
            if ( defined $isl_line && $isl_line ne '' ) {
              chomp $isl_line;
              my ( undef, undef, undef, $to_port, $to_sw ) = split( ",", $isl_line );
              if ( defined $to_port && $to_port ne '' && defined $to_sw && $to_sw ne '' ) {
                $to_port =~ s/to_port=//;
                $to_sw =~ s/to_switch=//;
                $to_sw =~ s/-BNA$//;
                $port_isl = "to $to_sw, $to_port";
              } ## end if ( defined $to_port ...)
            } ## end if ( defined $isl_line...)

            print CFGH "<tr>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$port_name</td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$alias_from_sw</td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$alias_from_cfg</td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$zone_wwn</td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$zone_alias</td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\"></td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$port_isl</td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\">$state</td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\"></td>\n";
            print CFGH "<td style=\"text-align:center; color:black;\" nowrap=\"\"></td>\n";
            print CFGH "</tr>\n";
          }
        }
      }
      print CFGH "</tbody></table>\n";

      close(CFGH);
      rename( "$config_html-tmp", "$config_html" );
    }
  }

  return 1;
}

# create data dir, PORTS.cfg, SAN-BRCD, SAN-BNA, HEALTH_STATUS.txt, unused_ports.txt
sub create_switch_conf_files {
  my $switch_name = shift;

  if ( ! -d "$wrkdir/$switch_name" ) {
    mkdir( "$wrkdir/$switch_name", 0755 ) || error( "Cannot mkdir $wrkdir/$switch_name: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }
  if ( ! -d "$webdir/$switch_name" ) {
    mkdir( "$webdir/$switch_name", 0755 ) || error( "Cannot mkdir $webdir/$switch_name: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }
  if ( ! -d "$webdir/BNA" ) {
    mkdir( "$webdir/BNA", 0755 ) || error( "Cannot mkdir $webdir/BNA: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }
  if ( ! -f "$wrkdir/$switch_name/SAN-BRCD" ) {
    `touch "$wrkdir/$switch_name/SAN-BRCD"`;
  }
  if ( ! -f "$wrkdir/$switch_name/SAN-BNA" ) {
    `touch "$wrkdir/$switch_name/SAN-BNA"`;
  }

  # PORTS.cfg, HEALTH_STATUS.txt, unused_ports.txt
  my $ports_idx        = 0;
  my $unused_ports_idx = 0;
  my %ports2cfg;
  if ( defined $bna_data{'switches'}{$switch_name}{'ports'} && $bna_data{'switches'}{$switch_name}{'ports'} ne '' ) {
    foreach my $port_wwn ( sort keys( %{ $bna_data{'switches'}{$switch_name}{'ports'} } ) ) {
      my $wwn2cfg = $port_wwn;
      $wwn2cfg    =~ s/://g;

      if ( defined $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'} && $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'} ne '' ) {
        $ports_idx++;

        my $port_slot    = 0;
        my $port_id      = $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'};
        my $port_name    = "port$port_id";
        my $port_phys_id = $port_id;

        if ( $port_id =~ m/^[0-9]+-[0-9]+$/ ) {
          ($port_slot, $port_phys_id) = split(/-/, $port_id);
        }

        # PORTS.cfg
        my $rrd_file   = "$wrkdir/$switch_name/$port_name.rrd";
        my $port_alias = "";
        if ( defined $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'name'} && $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'name'} ne '' ) {
          $port_alias = $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'name'};
        }
        $ports2cfg{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'port_cfg_line'} = "$port_name : $rrd_file : $port_alias : $wwn2cfg";
        $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'port_cfg_line'} = "$port_name : $rrd_file : $port_alias : $wwn2cfg";

        # config.html
        $ports2cfg{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'wwn'}   = $port_wwn;
        $ports2cfg{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'alias'} = $port_alias;
        $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'wwn'}   = $port_wwn;
        $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'alias'} = $port_alias;

        # HEALTH_STATUS.txt
        my $port_phys_status = "";
        my $port_oper_status = "";
        if ( defined $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'status'} && $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'status'} ne '' ) {
          $port_phys_status = $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'status'};
        }
        if ( defined $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'state'} && $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'state'} ne '' ) {
          $port_oper_status = $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'state'};
        }
        $ports2cfg{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'port_status_line'} = "$port_id,$port_phys_status,$port_oper_status";
        $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'port_status_line'} = "$port_id,$port_phys_status,$port_oper_status";

        # unused_ports.txt
        if ( defined $port_phys_status && $port_phys_status eq "NO_MODULE" && defined $port_oper_status && $port_oper_status eq "OFFLINE" ) {
          $unused_ports_idx++;
          $ports2cfg{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'unused_port_line'} = "$port_name : $port_phys_status : $port_oper_status";
          $bna_data{'switches'}{$switch_name}{'ports2cfg_all'}{'port_slot'}{$port_slot}{'port_id'}{$port_phys_id}{'unused_port_line'} = "$port_name : $port_phys_status : $port_oper_status";
        }
      }
    }
  }
  if ( $ports_idx > 0 ) {
    my $ports_cfg     = "$wrkdir/$switch_name/PORTS.cfg";
    my $status_file   = "$wrkdir/$switch_name/HEALTH_STATUS.txt";
    my $unused_p_file = "$wrkdir/$switch_name/unused_ports.txt";

    open( CFG, ">$ports_cfg-tmp" ) || error( "Couldn't open file $ports_cfg-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;
    open( HS, ">$status_file-tmp" ) || error( "Couldn't open file $status_file-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;
    open( UNU, ">$unused_p_file-tmp" ) || error( "Couldn't open file $unused_p_file-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;

    print HS "port,Operational status,Physical status\n";
    print HS "---------------------------------------\n";

    # workaround for sorting
    foreach my $slot ( sort { $a <=> $b } keys ( %{ $ports2cfg{'port_slot'} } ) ) {
      foreach ( sort { $a <=> $b } keys ( %{ $ports2cfg{'port_slot'}{$slot}{'port_id'} } ) ) {
        if ( defined $ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'port_cfg_line'} && $ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'port_cfg_line'} ne '' ) {
          print CFG "$ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'port_cfg_line'}\n";
        }
        if ( defined $ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'port_status_line'} && $ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'port_status_line'} ne '' ) {
          print HS "$ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'port_status_line'}\n";
        }
        if ( defined $ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'unused_port_line'} && $ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'unused_port_line'} ne '' ) {
          print UNU "$ports2cfg{'port_slot'}{$slot}{'port_id'}{$_}{'unused_port_line'}\n";
        }
      }
    }
    close(CFG);
    close(HS);
    close(UNU);
    rename( "$ports_cfg-tmp", "$ports_cfg" );
    rename( "$status_file-tmp", "$status_file" );
    rename( "$unused_p_file-tmp", "$unused_p_file" );
  }
  else {
    error( "ERROR : SWITCH : $switch_name : Can't create PORTS.cfg and HEALTH_STATUS.txt! Ports not found!" . __FILE__ . ":" . __LINE__ );
  }

  #
  # Global health status
  #
  my $act_timestamp  = time();
  my $main_state     = "OK";
  my $state_suffix   = "ok";
  my $component_name = $switch_name;

  my $sw_oper_st = "";
  my $sw_st_reas = "";
  if ( defined $bna_data{'switches'}{$switch_name}{'operationalStatus'} && $bna_data{'switches'}{$switch_name}{'operationalStatus'} ne '' ) {
    $sw_oper_st = $bna_data{'switches'}{$switch_name}{'operationalStatus'};
  }
  if ( defined $bna_data{'switches'}{$switch_name}{'statusReason'} && $bna_data{'switches'}{$switch_name}{'statusReason'} ne '' ) {
    $sw_st_reas = $bna_data{'switches'}{$switch_name}{'statusReason'};
  }

  my $switch_hs_html = "$webdir/$switch_name/SWITCH_HEALTH_STATUS.html";
  open( SHS, ">$switch_hs_html-tmp" ) || error( "Couldn't open file $switch_hs_html-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit;
  print SHS "<br><br><br>\n";
  print SHS "<br>Operational Status : <b>$sw_oper_st</b>\n";
  print SHS "<br>Status Reason      : <b>$sw_st_reas</b>\n";
  my $last_upd = localtime();
  print SHS "<br><br><br>Last update: $last_upd\n";
  close(SHS);
  rename( "$switch_hs_html-tmp", "$switch_hs_html" );

  $component_name =~ s/\s+//g;
  my $alert_status = 0;

  if ( $sw_oper_st !~ m/^HEALTHY/ ) { $main_state = "NOT_OK"; $state_suffix = "nok"; }

  if ( !-d "$tmpdir/health_status_summary" ) {
    mkdir( "$tmpdir/health_status_summary", 0755 ) || error( "Cannot mkdir $tmpdir/health_status_summary: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }

  if ( !-f "$tmpdir/health_status_summary/$component_name.ok" && !-f "$tmpdir/health_status_summary/$component_name.nok" ) {
    $alert_status = 1;
  }

  if ( -f "$tmpdir/health_status_summary/$component_name.ok" ) { unlink("$tmpdir/health_status_summary/$component_name.ok"); $alert_status = 1; }
  if ( -f "$tmpdir/health_status_summary/$component_name.nok" ) { unlink("$tmpdir/health_status_summary/$component_name.nok"); }

  if ( $alert_status == 1 && $state_suffix eq "nok" ) {
    ### alert health status
    if ( exists &Xorux_lib::health_status_alerting ) {
      Xorux_lib::health_status_alerting($component_name);
    }
  } ## end if ( $alert_status == ...)

  open( MAST, ">$tmpdir/health_status_summary/$component_name.$state_suffix" ) || error( "Couldn't open file $tmpdir/health_status_summary/$component_name.$state_suffix $!" . __FILE__ . ":" . __LINE__ ) && exit;
  print MAST "SAN-SWITCH : $switch_name : $main_state : $act_timestamp\n";
  close(MAST);


  return 1;
}

sub get_port_conf {
  my $switch_name = shift;
  my $switch_wwn  = shift;

  my $result = new_get_request("$url/rest/resourcegroups/All/fcports/","switchWWN,$switch_wwn");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'fcPorts'} && ref($result->{'fcPorts'}) eq "ARRAY" ) {
      my $port_idx = 0;
      foreach ( @{ $result->{'fcPorts'} } ) {
        if ( defined $_->{'wwn'} && $_->{'wwn'} ne "" ) {
          $port_idx++;
          my $port_wwn  = $_->{'wwn'};
          $port_wwn     = uc $port_wwn;
          my $port_name = "";

          if ( exists $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn} && exists $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn} ) {
            if ( defined $_->{'state'} && $_->{'state'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'state'} = $_->{'state'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'state'} = $_->{'state'};
            }
            if ( defined $_->{'portNumber'} && $_->{'portNumber'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'portNumber'} = $_->{'portNumber'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'portNumber'} = $_->{'portNumber'};
            }
            if ( defined $_->{'name'} && $_->{'name'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'name'} = $_->{'name'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'name'} = $_->{'name'};
            }
            if ( defined $_->{'maxPortSpeed'} && $_->{'maxPortSpeed'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'maxPortSpeed'} = $_->{'maxPortSpeed'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'maxPortSpeed'} = $_->{'maxPortSpeed'};
            }
            if ( defined $_->{'slotNumber'} && $_->{'slotNumber'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'slotNumber'} = $_->{'slotNumber'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'slotNumber'} = $_->{'slotNumber'};
            }
            if ( defined $_->{'portId'} && $_->{'portId'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'portId'} = $_->{'portId'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'portId'} = $_->{'portId'};
            }
            if ( defined $_->{'status'} && $_->{'status'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'status'} = $_->{'status'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'status'} = $_->{'status'};
            }
            if ( defined $_->{'portIndex'} && $_->{'portIndex'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'portIndex'} = $_->{'portIndex'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'portIndex'} = $_->{'portIndex'};
            }
            if ( defined $_->{'userPortNumber'} && $_->{'userPortNumber'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'userPortNumber'} = $_->{'userPortNumber'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'userPortNumber'} = $_->{'userPortNumber'};
              $port_name = $_->{'userPortNumber'};
            }
            if ( defined $_->{'speed'} && $_->{'speed'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'speed'} = $_->{'speed'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'speed'} = $_->{'speed'};
            }
            if ( defined $_->{'remotePortWwn'} && $_->{'remotePortWwn'} ne "" ) {
              $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'remotePortWwn'} = $_->{'remotePortWwn'};
              $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'remotePortWwn'} = $_->{'remotePortWwn'};
            }

            # port name
            if ( defined $_->{'slotNumber'} && isdigit($_->{'slotNumber'}) && $_->{'slotNumber'} > 0 && defined $_->{'portNumber'} && isdigit($_->{'portNumber'}) ) {
              $port_name = "$_->{'slotNumber'}-$_->{'portNumber'}";
            }
            $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'}  = $port_name;
            $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'PORT_ID'} = $port_name;
          }
        }
      }
    }
  }

  return 1;
}

sub get_port_perf {
  my $switch_name = shift;
  my $switch_wwn  = shift;

  my $result = new_get_request("$url/rest/fcswitchstats","switchWWN,$switch_wwn");
  if ( defined $result && $result ne "0" ) {
    #print Dumper $result;
    if ( defined $result->{'responseList'} && ref($result->{'responseList'}) eq "ARRAY" ) {
      my $port_idx = 0;
      my $act_timestamp = time();
      my $act_date      = localtime();
      foreach ( @{ $result->{'responseList'} } ) {
        if ( defined $_->{'portWWN'} && $_->{'portWWN'} ne '' ) {
          $port_idx++;
          my $port_wwn = $_->{'portWWN'};
          $port_wwn    = uc $port_wwn;

          $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'wwn'} = $port_wwn;
          $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'wwn'} = $port_wwn;

          # stats time
          $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'timestamp'} = $act_timestamp;
          $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'date'}      = $act_date;

          if ( defined $_->{'bytesTransmitted'} && isdigit($_->{'bytesTransmitted'}) ) {
            my $data_out = $_->{'bytesTransmitted'};
            $data_out    = $data_out / 4;
            $data_out    = sprintf "%.0f", $data_out;
            $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_out'} = $data_out;
            $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_out'} = $data_out;
          }
          if ( defined $_->{'bytesReceived'} && isdigit($_->{'bytesReceived'}) ) {
            my $data_in = $_->{'bytesReceived'};
            $data_in    = $data_in / 4;
            $data_in    = sprintf "%.0f", $data_in;
            $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_in'} = $data_in;
            $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'data_in'} = $data_in;
          }
          if ( defined $_->{'pktsTransmitted'} && isdigit($_->{'pktsTransmitted'}) ) {
            $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_out'} = $_->{'pktsTransmitted'};
            $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_out'} = $_->{'pktsTransmitted'};
          }
          if ( defined $_->{'pktsReceived'} && isdigit($_->{'pktsReceived'}) ) {
            $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_in'} = $_->{'pktsReceived'};
            $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'frame_in'} = $_->{'pktsReceived'};
          }
          if ( defined $_->{'bbCreditZero'} && isdigit($_->{'bbCreditZero'}) ) {
            $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'bbcredits'} = $_->{'bbCreditZero'};
            $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'bbcredits'} = $_->{'bbCreditZero'};
          }
          if ( defined $_->{'rxCRCs'} && isdigit($_->{'rxCRCs'}) ) {
            $port_data{'switch'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'crc'} = $_->{'rxCRCs'};
            $bna_data{'switches'}{$switch_name}{'ports'}{$port_wwn}{'perf'}{'crc'} = $_->{'rxCRCs'};
          }
        }
      }
    }
  }

  return 1;
}

sub get_all_switches_info {
  my $result = new_get_request("$url/rest/resourcegroups/All/fcswitches/","");
  if ( defined $result && $result ne "0" ) {
    if ( defined $result->{'fcSwitches'} && ref($result->{'fcSwitches'}) eq "ARRAY" ) {
      #print Dumper $result;
      my $switch_idx = 0;
      foreach ( @{ $result->{'fcSwitches'} } ) {
        $switch_idx++;

        if ( defined $_->{'name'} && $_->{'name'} ne "" ) {
          my $switch_name = "$_->{'name'}-BNA";

          $bna_data{'switches'}{$switch_name}{'name'} = $switch_name;

          if ( defined $_->{'operationalStatus'} && $_->{'operationalStatus'} ne "" ) {
            $bna_data{'switches'}{$switch_name}{'operationalStatus'} = $_->{'operationalStatus'};
          }
          if ( defined $_->{'state'} && $_->{'state'} ne "" ) {
            $bna_data{'switches'}{$switch_name}{'state'} = $_->{'state'};
          }
          if ( defined $_->{'statusReason'} && $_->{'statusReason'} ne "" ) {
            $bna_data{'switches'}{$switch_name}{'statusReason'} = $_->{'statusReason'};
          }
          if ( defined $_->{'role'} && $_->{'role'} ne "" ) {
            $bna_data{'switches'}{$switch_name}{'role'} = $_->{'role'};
          }
          if ( defined $_->{'type'} && $_->{'type'} ne "" ) {
            $bna_data{'switches'}{$switch_name}{'type'} = $_->{'type'};
          }
          if ( defined $_->{'key'} && $_->{'key'} ne "" ) {
            $bna_data{'switches'}{$switch_name}{'key'} = $_->{'key'};
          }
          if ( defined $_->{'virtualFabricId'} && $_->{'virtualFabricId'} ne "" ) {
            $bna_data{'switches'}{$switch_name}{'virtualFabricId'} = $_->{'virtualFabricId'};
          }
          if ( defined $_->{'wwn'} && $_->{'wwn'} ne "" ) {
            $bna_data{'switches'}{$switch_name}{'wwn'} = $_->{'wwn'};

            my $result = new_get_request("$url/rest/resourcegroups/All/fcswitches/$_->{'wwn'}/physicalswitches","");
            if ( defined $result && $result ne "0" ) {
              #print Dumper $result;
              if ( defined $result->{'physicalSwitches'} && ref($result->{'physicalSwitches'}) eq "ARRAY" ) {
                foreach ( @{ $result->{'physicalSwitches'} } ) {
                  if ( defined $_->{'firmwareVersion'} && $_->{'firmwareVersion'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'firmwareVersion'} = $_->{'firmwareVersion'};
                  }
                  if ( defined $_->{'firmwareVersion'} && $_->{'firmwareVersion'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'firmwareVersion'} = $_->{'firmwareVersion'};
                  }
                  if ( defined $_->{'ipAddress'} && $_->{'ipAddress'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'ipAddress'} = $_->{'ipAddress'};
                  }
                  if ( defined $_->{'partNumber'} && $_->{'partNumber'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'partNumber'} = $_->{'partNumber'};
                  }
                  if ( defined $_->{'modelNumber'} && $_->{'modelNumber'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'modelNumber'} = $_->{'modelNumber'};
                  }
                  if ( defined $_->{'switchSerialNumber'} && $_->{'switchSerialNumber'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'switchSerialNumber'} = $_->{'switchSerialNumber'};
                  }
                  if ( defined $_->{'location'} && $_->{'location'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'location'} = $_->{'location'};
                  }
                  if ( defined $_->{'userId'} && $_->{'userId'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'userId'} = $_->{'userId'};
                  }
                  if ( defined $_->{'description'} && $_->{'description'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'description'} = $_->{'description'};
                  }
                  if ( defined $_->{'vendorPartNumber'} && $_->{'vendorPartNumber'} ne "" ) {
                    $bna_data{'switches'}{$switch_name}{'phys_conf'}{'vendorPartNumber'} = $_->{'vendorPartNumber'};
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub new_get_request {
  my $url         = shift;
  my $header_item = shift;

  my $agent   = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET => $url );

  $request->header(
    'Content-Type' => 'application/vnd.brocade.networkadvisor+json;version=v1',
    'Accept'       => 'application/vnd.brocade.networkadvisor+json;version=v1',
    'WStoken'      => $wstoken
  );

  if ( defined $header_item && $header_item ne '' ) {
    my ($key, $value) = split(/,/, $header_item);
    $request->header($key => $value);
  }

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    error("Request error: $st_line");
    error("GET url      : $url");

    my $content = $results->content;
    error("Content      : $content");

    return 0;
  } ## end if ( !$results->is_success)

  my $result = decode_json( $results->content );

  return $result;
} ## end sub new_get_request

sub logout_from_api {
  my $url = shift;

  my $agent   = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( POST => $url );

  $request->header(
    'Content-Type' => 'application/vnd.brocade.networkadvisor+json;version=v1',
    'Accept'       => 'application/vnd.brocade.networkadvisor+json;version=v1',
    'WStoken'      => $wstoken
  );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    error("Request error: $st_line");
    error("POST url     : $url");

    exit 0;
  }

  return 1;
}

sub login_to_api {
  my $url = shift;

  my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( POST => $url );

  $request->header(
    'Content-Type' => 'application/vnd.brocade.networkadvisor+json;version=v1',
    'Accept'       =>'application/vnd.brocade.networkadvisor+json;version=v1',
    'WSUsername'   => $user,
    'WSPassword'   => $password
  );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    error("Request error: $st_line");
    error("POST url     : $url");

    return 0;
  }
  else {
    my $result = $results->headers;
    if ( defined $result->{'wstoken'} && $result->{'wstoken'} ne '' ) {
      return $result->{'wstoken'};
    }
    else {
      error( "wstoken not found! Login failed! $!" . __FILE__ . ":" . __LINE__ ) && exit 0;
    }
  }

  return 0;
}

sub make_gauge {
  my $curr     = shift;
  my $last     = shift;
  my $interval = shift;
  my $value    = "U";

  if ( isdigit($curr) && isdigit($last) && isdigit($interval) && $interval != 0 ) {
    $value = ($curr - $last) / $interval;
    $value = sprintf "%.2f", $value;
  }

  return $value;
}

sub isdigit {
  my $digit = shift;

  if ( !defined($digit) || $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  return 0;
} ## end sub isdigit

sub file_time_diff {
  my $file = shift;

  my $act_time  = time();
  my $file_time = $act_time;
  my $time_diff = 0;

  if ( -f $file ) {
    $file_time = ( stat($file) )[9];
    $time_diff = $act_time - $file_time;
  }

  return ($time_diff);
} ## end sub file_time_diff

sub message {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  print "$act_time: $text\n";

  return 1;
}

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  #print "$act_time: $text : $!\n" if $DEBUG > 2;;

  return 1;
} ## end sub error
