#!/usr/bin/perl

use strict;
use warnings;
use Data::Dumper;
use LWP::UserAgent;
use HTTP::Request;
use JSON qw(encode_json decode_json);
use Date::Parse;
use Xorux_lib;
use Storable;

### global variables

my $ip_adress_a           = "";
my $user                  = "";
my $passwd                = "";
my $compellent_enviroment = 1;    ### real or testing
my $storage_name          = "";
my $basedir               = "";
my $bindir                = "";
my $webdir                = "";
my $ERRLOG_COMPELLENT     = "";
my $wrkdir                = "";
my $health_status_html    = "";
my $data_directory        = "";
my $summary_health        = "";
my %inventory_data;
my $cookie = "";
my $port = "";
my $proto  = "https";
my %header_section;
my $debug_data       = "";
my $instance_storage = "";

###

### main function
set_variables();
data_structure();

$instance_storage = set_instance_id_name();
if ( !defined $instance_storage || $instance_storage eq "" ) {
  error("ERROR: Instance Storage \"$storage_name\" is not defined!");
  error_die("ERROR: Storage cannot be accessed, check if user \"$user\" has allowed storage \"$storage_name\"");
}

set_data_port();
set_data_volume();
set_data_server();
set_data_disk();
set_data_node();
set_data_volume_folder();
set_data_resp_time_total();    ### total latency calculator
set_data_pool();
set_interval();
set_header();
### for debug
my $file = "$debug_data/perf.txt";
store \%inventory_data, $file or error("Store data to file $file failed.");
###
write_data();
set_health_status();

#print Dumper \%inventory_data;

###

sub set_data_pool{
  my $section_vol = "Volume Level Statistics";
  my $section = "Pool Level Statistics";

  my $read_io_sum = "nan";
  my $write_io_sum = "nan";
  my $io_rate_sum = "nan";
  my $read_sum = "nan";
  my $write_sum = "nan";
  my $data_rate_sum = "nan";
  my $resp_t_sum = "nan";
  my $resp_t_r_sum = "nan";
  my $resp_t_w_sum = "nan";
  my $xfer_latency_sum = "nan";
  my $io_pending_sum = "nan";
  my $qs_latency_sum = "nan";
  my $time_sum = "nan";
  my $cap_total_sum = "nan";
  my $cap_used_sum = "nan";
  my $cap_free_sum = "nan";
  my $cap_snapshot_sum = "nan";


  foreach my $id_vol (keys %{$inventory_data{$section_vol}}){
    my $read_io = $inventory_data{$section_vol}{$id_vol}{"readIops"};
    my $write_io = $inventory_data{$section_vol}{$id_vol}{"writeIops"};
    my $io_rate = $inventory_data{$section_vol}{$id_vol}{"totalIops"};
    my $write = $inventory_data{$section_vol}{$id_vol}{"writeKbPerSecond"};
    my $read = $inventory_data{$section_vol}{$id_vol}{"readKbPerSecond"};
    my $data_rate = $inventory_data{$section_vol}{$id_vol}{"totalKbPerSecond"};
    my $resp_t = $inventory_data{$section_vol}{$id_vol}{"totalLatency"};
    my $resp_t_r = $inventory_data{$section_vol}{$id_vol}{"readLatency"};
    my $resp_t_w = $inventory_data{$section_vol}{$id_vol}{"writeLatency"};
    my $xfer_latency = $inventory_data{$section_vol}{$id_vol}{"xferLatency"};
    my $io_pending = $inventory_data{$section_vol}{$id_vol}{"ioPending"};
    my $qs_latency = $inventory_data{$section_vol}{$id_vol}{"qosLatency"};
    my $time = $inventory_data{$section_vol}{$id_vol}{"time"};
    my $cap_total = $inventory_data{$section_vol}{$id_vol}{"Capacity(MB)"};
    my $cap_used = $inventory_data{$section_vol}{$id_vol}{"Used(MB)"};
    my $cap_free = $inventory_data{$section_vol}{$id_vol}{"Free(MB)"};
    my $cap_snapshot = $inventory_data{$section_vol}{$id_vol}{"Snapshot(MB)"};

    if (defined $time){$time_sum = $time;}

    if (isdigit($cap_total) && isdigit($cap_total_sum)){
      $cap_total_sum = $cap_total_sum + $cap_total;
    }
    if (isdigit($cap_total) && !isdigit($cap_total_sum)){
      $cap_total_sum = $cap_total;
    }

    if (isdigit($cap_used) && isdigit($cap_used_sum)){
      $cap_used_sum = $cap_used_sum + $cap_used;
    }
    if (isdigit($cap_used) && !isdigit($cap_used_sum)){
      $cap_used_sum = $cap_used;
    }

    if (isdigit($cap_free) && isdigit($cap_free_sum)){
      $cap_free_sum = $cap_free_sum + $cap_free;
    }
    if (isdigit($cap_free) && !isdigit($cap_free_sum)){
      $cap_free_sum = $cap_free;
    }

    if (isdigit($cap_snapshot) && isdigit($cap_snapshot_sum)){
      $cap_snapshot_sum = $cap_snapshot_sum + $cap_snapshot;
    }
    if (isdigit($cap_snapshot) && !isdigit($cap_snapshot_sum)){
      $cap_snapshot_sum = $cap_snapshot;
    }

    if (isdigit($read_io) && isdigit($read_io_sum)){
      $read_io_sum = $read_io_sum + $read_io;
    }
    if (isdigit($read_io) && !isdigit($read_io_sum)){
      $read_io_sum = $read_io;
    }

    if (isdigit($write_io) && isdigit($write_io_sum)){
      $write_io_sum = $write_io_sum + $write_io;
    }
    if (isdigit($write_io) && !isdigit($write_io_sum)){
      $write_io_sum = $write_io;
    }

    if (isdigit($io_rate) && isdigit($io_rate_sum)){
      $io_rate_sum = $io_rate_sum + $io_rate;
    }
    if (isdigit($io_rate) && !isdigit($io_rate_sum)){
      $io_rate_sum = $io_rate;
    }

    if (isdigit($read) && isdigit($read_sum)){
      $read_sum = $read_sum + $read;
    }
    if (isdigit($read) && !isdigit($read_sum)){
      $read_sum = $read;
    }

    if (isdigit($write) && isdigit($write_sum)){
      $write_sum = $write_sum + $write;
    }
    if (isdigit($write) && !isdigit($write_sum)){
      $write_sum = $write;
    }

    if (isdigit($data_rate) && isdigit($data_rate_sum)){
      $data_rate_sum = $data_rate_sum + $data_rate;
    }
    if (isdigit($data_rate) && !isdigit($data_rate_sum)){
      $data_rate_sum = $data_rate;
    }

    if (isdigit($io_pending) && isdigit($io_pending_sum)){
      $io_pending_sum = $io_pending_sum + $io_pending;
    }
    if (isdigit($io_pending) && !isdigit($io_pending_sum)){
      $io_pending_sum = $io_pending;
    }

    if (isdigit($resp_t) && isdigit($resp_t_sum) && isdigit($io_rate)){
      $resp_t_sum = $resp_t_sum + ($resp_t * $io_rate);
    }
    if (isdigit($resp_t) && !isdigit($resp_t_sum) && isdigit($io_rate)){
      $resp_t_sum = ($resp_t * $io_rate);
    }

    if (isdigit($resp_t_r) && isdigit($resp_t_r_sum) && isdigit($read_io)){
      $resp_t_r_sum = $resp_t_r_sum + ($resp_t_r * $read_io);
    }
    if (isdigit($resp_t_r) && !isdigit($resp_t_r_sum) && isdigit($read_io)){
      $resp_t_r_sum = ($resp_t_r * $read_io);
    }

    if (isdigit($resp_t_w) && isdigit($resp_t_w_sum) && isdigit($write_io)){
      $resp_t_w_sum = $resp_t_w_sum + ($resp_t_w * $write_io);
    }
    if (isdigit($resp_t_w) && !isdigit($resp_t_w_sum) && isdigit($write_io)){
      $resp_t_w_sum = ($resp_t_w * $write_io);
    }

   if (isdigit($qs_latency) && isdigit($qs_latency_sum) && isdigit($io_rate)){
      $qs_latency_sum = $qs_latency_sum + ($qs_latency * $io_rate);
    }
    if (isdigit($qs_latency) && !isdigit($qs_latency_sum) && isdigit($io_rate)){
      $qs_latency_sum = ($qs_latency * $io_rate);
    }

    if (isdigit($xfer_latency) && isdigit($xfer_latency_sum) && isdigit($io_rate)){
      $xfer_latency_sum = $xfer_latency_sum + ($xfer_latency * $io_rate);
    }
    if (isdigit($xfer_latency) && !isdigit($xfer_latency_sum) && isdigit($io_rate)){
      $xfer_latency_sum = ($xfer_latency * $io_rate);
    }


  }

  $inventory_data{$section}{0}{NAME} = "pool0";
  $inventory_data{$section}{0}{ID} = 0;
  if (defined $time_sum && $time_sum ne ""){
    $inventory_data{$section}{0}{time} = $time_sum;
  }
  $inventory_data{$section}{0}{"readIops"} = $read_io_sum;
  $inventory_data{$section}{0}{"writeIops"} = $write_io_sum;
  $inventory_data{$section}{0}{"totalIops"} = $io_rate_sum;
  $inventory_data{$section}{0}{"writeKbPerSecond"} = $write_sum;
  $inventory_data{$section}{0}{"readKbPerSecond"} = $read_sum;
  $inventory_data{$section}{0}{"totalKbPerSecond"} = $data_rate_sum;

  $inventory_data{$section}{0}{"Capacity(TB)"} = round($cap_total_sum / (1024 * 1024),3);
  $inventory_data{$section}{0}{"Used(TB)"} = round($cap_used_sum / (1024 * 1024),3);
  $inventory_data{$section}{0}{"Free(TB)"} = round($cap_free_sum / (1024 * 1024),3);
  $inventory_data{$section}{0}{"Snapshot(TB)"} = round($cap_snapshot_sum / (1024 * 1024),3);

  if (isdigit($io_rate_sum) && $io_rate_sum != 0){
    $inventory_data{$section}{0}{"totalLatency"} = round($resp_t_sum / $io_rate_sum,3);
    $inventory_data{$section}{0}{"xferLatency"} = round( $xfer_latency_sum / $io_rate_sum,3);
    $inventory_data{$section}{0}{"qosLatency"} = round($qs_latency_sum / $io_rate_sum,3);
  }
  if (isdigit($read_io_sum) && $read_io_sum != 0){
    $inventory_data{$section}{0}{"readLatency"} = round($resp_t_r_sum / $read_io_sum,3);
  }
  if (isdigit($write_io_sum) && $write_io_sum != 0){
    $inventory_data{$section}{0}{"writeLatency"} = round($resp_t_w_sum / $write_io_sum,3);
  }
  $inventory_data{$section}{0}{"ioPending"} = $io_pending_sum;


}


sub set_instance_id_name {

  my ( $cookie, $api_version ) = login_api();

  if ( !defined $cookie || $cookie eq "" ) {
    error("Cookie is not defined!");
    return "";
  }
  my $url = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/StorageCenter/GetList";
  my $data = get_data_api_instance( $cookie, $api_version, $url );
  if ( defined $data && $data ne "" ) {
    foreach my $element ( @{$data} ) {
      my $instance     = $element->{"instanceId"};
      my $name_storage = $element->{"scName"};
      if ($compellent_enviroment == 0){return $instance;}
      if ( defined $instance && defined $name_storage && $instance ne "" && $name_storage ne "" ) {
        if ( $storage_name eq $name_storage ) {
          ### find right storage
          return $instance;
        }
      } ## end if ( defined $instance...)
    } ## end foreach my $element ( @{$data...})
  } ## end if ( defined $data && ...)
  else {
    return "";
  }

  return "";

} ## end sub set_instance_id_name

sub get_data_api_instance {
  my $cookie      = shift;
  my $api_version = shift;
  my $url         = shift;

  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $req = HTTP::Request->new( POST => $url );
  $req->header( 'x-dell-api-version' => $api_version );
  $req->header( 'Content-Type'       => 'application/json; charset=utf-8' );
  $req->header( 'Accept'             => 'application/json' );
  $req->header( 'cookie'             => $cookie );
  my $res = $ua->request($req);

  if ( $res->is_success ) {
    ### for debug
    my $file = "$debug_data/instance.txt";
    store \$res->content, $file or error("Store data to file $file failed.");
    ###
    my $data = decode_json( $res->content );
    return $data;

  } ## end if ( $res->is_success )
  else {
    error("ERROR: Command $url failed!");
    error( $res->content );
    return "";
  }

} ## end sub get_data_api_instance

sub set_health_status {

  my ( $cookie, $api_version ) = login_api();

  if ( !defined $cookie || $cookie eq "" ) {
    error("Cookie is not defined!");
    return;
  }
  my $object = "ScAlert";
  my $url    = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object";
  my $data   = get_data_api_configuration( $cookie, $api_version, $url, $object );

  logout_api( $cookie, $api_version );

  if ( $data eq "" ) {
    error("ERROR: No valid data for object $object");
    return;
  }
  my $table = "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
  my $thead = "<thead><tr><th class = \"sortable\" title=\"Severity\" nowrap=\"\">Severity</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"Date\" nowrap=\"\">Date</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"Storage Center\" nowrap=\"\">Storage Center</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"Category\" nowrap=\"\">Category</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"Object\" nowrap=\"\">Object</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"Type\" nowrap=\"\">Type</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"Message\" nowrap=\"\">Message</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"Support url\" nowrap=\"\">Support url</th>\n";
  $thead = $thead . "</tr>\n</thead>\n";
  my $tbody = "<tbody>\n";
  my $tr    = "";

  my $error_health = 0;

  foreach my $element ( @{$data} ) {
    my $instance = $element->{"instanceId"};
    if ( !defined $instance || $instance eq "" ) { next; }
    my $id_storage = $element->{"scSerialNumber"};
    if ( !defined $id_storage || $id_storage ne $instance_storage ) { next; }
    my $active = $element->{"active"};
    if (isdigit($active) && $active == 0){next;}
    my $acknowledged = $element->{"acknowledged"};
    if (isdigit($acknowledged) && $acknowledged == 1){next;}
    my $status = $element->{"alertStatus"};
    if ( !defined $status || $status eq "" ) {
      $status = $element->{"status"};
      if ( !defined $status || $status eq "" ) { next; }
    }
    my $date = $element->{"createTime"};
    my $test_time = str2time($date);
    if ( !defined $date || $date eq "" || (isdigit($date) && $date == 0) || (isdigit($test_time) && $test_time == 0) ) {
      $date = $element->{"createdOn"};
      $test_time = str2time($date);
      if ( !defined $date || $date eq "" || (isdigit($date) && $date == 0) || (isdigit($test_time) && $test_time == 0)) {
        $test_time = str2time($date);
        $date = $element->{"modifiedOn"};
      }
    } ## end if ( !defined $date ||...)
    if ( !defined $date || $date eq  "" || (isdigit($date) && $date == 0) || (isdigit($test_time) && $test_time == 0)) {
      $date = "N/A";
    }

    #if ($date ne ""){
    #  my $time_loc_act = str2time($date);
    #  my $time_all = get_time($time_loc_act);
    #  $date = $time_all;
    #}
    if ( $status eq "Critical" ) {
      $error_health++;
    }
    my $storage_center = $element->{"scName"};
    my $category       = $element->{"category"};
    my $message        = $element->{"message"};
    my $type           = $element->{"type"};
    my $object         = $element->{"scObjectName"};
    my $support        = $element->{"supportUrl"};

    if ( !defined $storage_center ) { $storage_center = ""; }
    if ( !defined $category )       { $category       = ""; }
    if ( !defined $message )        { $message        = ""; }
    if ( !defined $type )           { $type           = ""; }
    if ( !defined $object )         { $object         = ""; }
    if ( !defined $support )        { $support        = ""; }
    if ( $support ne "" ) {
      my $url = "<a href=\"$support\" target=\"_blank\">Link</a>";
      $support = $url;
    }
    $tr = $tr . "<tr><td>$status</td><td>$date</td><td>$storage_center</td><td>$category</td><td>$object</td><td>$type</td><td>$message</td><td>$support</td></tr>\n";
  } ## end foreach my $element ( @{$data...})

  my $time = localtime();
  $time = $time . "<br>\n";
  my $report = "";
  $tbody = $tbody . $tr . "</tbody>\n";
  if ( $error_health == 0 ) {
    $report = "Storage $storage_name health status = Optimal\n";
  }
  else {
    $report = "Storage $storage_name health status = Error\n";
  }
  $table = $time . $report . $table . $thead . $tbody . "</table>\n";
  write_health_status_html( $health_status_html, $table );
  summary_health_status($error_health);

} ## end sub set_health_status

sub summary_health_status {
  my $error_status = shift;
  my $timestamp    = time();
  my $time         = localtime();
  my $alert_status = 0;

  if ( !-d "$summary_health" ) {
    mkdir( "$summary_health", 0755 ) || error( "$time: Cannot mkdir $summary_health: $!" . __FILE__ . ":" . __LINE__ );
  }

  if ( !-f "$summary_health/$storage_name.ok" && !-f "$summary_health/$storage_name.nok" ) {
    $alert_status = 1;
  }

  if ( -f "$summary_health/$storage_name.ok" ) {
    unlink("$summary_health/$storage_name.ok");
    $alert_status = 1;
  }
  if ( -f "$summary_health/$storage_name.nok" ) { unlink("$summary_health/$storage_name.nok"); }

  if ( $error_status == 0 ) {
    my $report = "STORAGE : $storage_name : OK : $timestamp";
    write_health_status_html( "$summary_health/$storage_name.ok", $report );
    return;
  }
  else {
    my $report = "STORAGE : $storage_name : NOT_OK : $timestamp";
    write_health_status_html( "$summary_health/$storage_name.nok", $report );
    if ( $alert_status == 1 ) {
      ### alert health status
      if ( exists &Xorux_lib::health_status_alerting ) {
        Xorux_lib::health_status_alerting($storage_name);
      }
    } ## end if ( $alert_status == ...)
    return;
  } ## end else [ if ( $error_status == ...)]

} ## end sub summary_health_status

sub write_health_status_html {
  my $file  = shift;
  my $table = shift;
  open( DATA, ">$file" ) or error("Cannot open file: $file : $!") && return 0;
  print DATA $table;
  close(DATA);
} ## end sub write_health_status_html

sub test_file_exist {
  my $file = shift;
  my $run  = shift;
  if ( !defined $file || $file eq "" ) {
    error("File does not defined !");
    return 0;
  }
  if ( !-e $file || -z $file ) {
    if ( defined $run && -z $file ) { return 1; }
    else {
      return 0;
    }
  } ## end if ( !-e $file || -z $file)
  else {
    return 1;
  }
} ## end sub test_file_exist

sub data_structure {
  my $directory = $data_directory;
  if ( !-d $directory ) {
    mkdir $directory || error_die( "Cannot read $directory: $!" . __FILE__ . ":" . __LINE__ );
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || error_die( "Cannot read $directory: $!" . __FILE__ . ":" . __LINE__ );
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || error_die( "Cannot read $directory: $!" . __FILE__ . ":" . __LINE__ );
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || error_die( "Cannot read $directory: $!" . __FILE__ . ":" . __LINE__ );
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
} ## end sub data_structure

sub set_data_resp_time_total {

  foreach my $section ( keys %inventory_data ) {
    foreach my $id ( keys %{ $inventory_data{$section} } ) {
      my $item_read_resp  = "readLatency";
      my $item_write_resp = "writeLatency";
      my $item_total_resp = "totalLatency";
      my $item_read_iops  = "readIops";
      my $item_write_iops = "writeIops";
      my $read_resp       = $inventory_data{$section}{$id}{$item_read_resp};
      my $write_resp      = $inventory_data{$section}{$id}{$item_write_resp};
      my $total_resp      = "";
      my $read_io         = $inventory_data{$section}{$id}{$item_write_iops};
      my $write_io        = $inventory_data{$section}{$id}{$item_read_iops};

      ### convert microsecond to milisecond
      if ( isdigit($read_resp) ) {
        my $pom = $read_resp / 1000;
        $inventory_data{$section}{$id}{$item_read_resp} = round( $pom, 3 );
        $read_resp = round( $pom, 3 );
      }
      if ( isdigit($write_resp) ) {
        my $pom = $write_resp / 1000;
        $inventory_data{$section}{$id}{$item_write_resp} = round( $pom, 3 );
        $write_resp = round( $pom, 3 );
      }

      if ( isdigit($read_resp) && isdigit($write_resp) && isdigit($write_io) && isdigit($read_io) ) {
        if ( $write_io == 0 && $read_io == 0 ) {
          $inventory_data{$section}{$id}{$item_total_resp} = 0;
          next;
        }
        if ( $read_resp == 0 && $write_resp == 0 ) {
          $inventory_data{$section}{$id}{$item_total_resp} = 0;
          next;
        }
        my $pom = ( ( $read_resp * $read_io ) + ( $write_io * $write_resp ) ) / ( $read_io + $write_io );
        $total_resp = round( $pom, 3 );
        $inventory_data{$section}{$id}{$item_total_resp} = $total_resp;
        next;

      } ## end if ( isdigit($read_resp...))
    } ## end foreach my $id ( keys %{ $inventory_data...})
  } ## end foreach my $section ( keys ...)

} ## end sub set_data_resp_time_total

sub get_data {
  my $section          = shift;
  my $section_contents = shift;
  my @values           = "";
  my @headers          = split( ",", $section_contents );
  foreach my $property ( keys %{ $inventory_data{$section} } ) {
    if ( !defined $inventory_data{$section}{$property}{ $headers[0] } || $inventory_data{$section}{$property}{ $headers[0] } eq "" ) { next; }
    if ( !defined $inventory_data{$section}{$property}{ $headers[2] } || $inventory_data{$section}{$property}{ $headers[2] } eq "" ) {
      if ( defined $inventory_data{$section}{TIME}{END} ) {
        $inventory_data{$section}{$property}{ $headers[2] } = $inventory_data{$section}{TIME}{END};
      }
      else {
        next;    ### no valid line time emty
      }
    } ## end if ( !defined $inventory_data...)
    if ( !@values eq "" ) {
      my $last = pop(@values);
      $last =~ s/,//g;
      push( @values, $last );
    }
    push( @values, "\n" );
    my $index = 1;
    foreach my $element (@headers) {
      chomp $element;
      $element =~ s/^\s+|\s+$//g;
      if ( defined $inventory_data{$section}{$property}{$element} && $inventory_data{$section}{$property}{$element} ne "" ) {
        if ( $index > 3 && isdigit( $inventory_data{$section}{$property}{$element} ) && $inventory_data{$section}{$property}{$element} !~ m/-/ ) {
          push( @values, "$inventory_data{$section}{$property}{$element}," );
          $index++;
          next;
        }
        if ( $section eq "Port Level Statistics" && $element eq "protocol" && defined $inventory_data{$section}{$property}{$element} ) {    ### exception for port protocol
          push( @values, "$inventory_data{$section}{$property}{$element}," );
          $index++;
          next;
        }
        if ( $index < 4 ) {
          push( @values, "$inventory_data{$section}{$property}{$element}," );
          $index++;
          next;
        }
        else {
          push( @values, "," );
          $index++;
          next;
        }
      } ## end if ( defined $inventory_data...)
      else {
        push( @values, "," );
        $index++;
      }
    } ## end foreach my $element (@headers)
  } ## end foreach my $property ( keys...)
  my $last = pop(@values);
  $last =~ s/,//g;
  push( @values, $last );
  return \@values;
} ## end sub get_data

sub get_name_file {
  my $type       = shift;
  my @time       = localtime();
  my $time_human = localtime();
  my $sec        = $time[0];
  my $minute     = $time[1];
  my $hour       = $time[2];
  my $day        = $time[3];
  my $month      = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;

  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $name_file     = "$wrkdir/$storage_name" . "_compellent$type" . "_" . "$year$month$day" . "_" . "$hour$minute" . ".out";
  my $name_file_tmp = "$wrkdir/$storage_name" . "_compellent$type" . "_" . "$year$month$day" . "_" . "$hour$minute" . ".out-tmp";
  return "$name_file,$name_file_tmp";

} ## end sub get_name_file

sub write_data {
  ( my $file, my $file_tmp ) = split( ",", get_name_file("perf") );

  foreach my $title ( keys %header_section ) {
    open( DATA, ">>$file_tmp" ) or error_die("Cannot open file: $file_tmp : $!");
    my ($values_tmp) = get_data( "$title", "$header_section{$title}" );
    my @values = @{$values_tmp};
    if ( !defined $inventory_data{$title}{TIME}{START} || !defined $inventory_data{$title}{TIME}{END} || !defined $inventory_data{$title}{TIME}{STEP} ) {
      close DATA;
      next;
    }
    print DATA "$title\n";
    my $interval_start  = " Interval Start:  " . $inventory_data{$title}{TIME}{START};
    my $interval_end    = " Interval End:    " . $inventory_data{$title}{TIME}{END};
    my $interval_length = " Interval Length: " . $inventory_data{$title}{TIME}{STEP};
    print DATA" $interval_start";
    print DATA"\n";
    print DATA" $interval_end";
    print DATA"\n";
    print DATA" $interval_length";
    print DATA"\n";
    print DATA "-----------------------------------\n";
    print DATA $header_section{$title};
    print DATA @values;
    print DATA "\n\n";
    close DATA;
  } ## end foreach my $title ( keys %header_section)
  rename( "$file_tmp", "$file" );

} ## end sub write_data

sub set_header {

  ### VOLUME
  my $section = "Volume Level Statistics";
  my $metric  = "ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,xferLatency,qosLatency,ioPending,Capacity(MB),Used(MB),Free(MB),Snapshot(MB)";
  $header_section{$section} = $metric;

  ### PORT
  $metric                   = "ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,protocol";
  $section                  = "Port Level Statistics";
  $header_section{$section} = $metric;

  ### HOST (SERVER)
  $section                  = "Host Level Statistics";
  $metric                   = "ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,xferLatency,qosLatency";
  $header_section{$section} = $metric;

  ### DISK
  $metric                   = "ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,ioPending,Capacity(MB),Used(MB),Free(MB)";
  $section                  = "Drive Level Statistics";
  $header_section{$section} = $metric;

  ### NODE
  $metric                   = "ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,cpuPercentUsage";
  $section                  = "Node Level Statistics";
  $header_section{$section} = $metric;

  ### Pool ()
  $section                  = "Pool Level Statistics";
  $metric                   = "ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,xferLatency,qosLatency,ioPending,Capacity(TB),Used(TB),Free(TB),Snapshot(TB)";
  $header_section{$section} = $metric;

  $section                  = "Volume-Folder Level Statistics";
  $metric                   = "ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,xferLatency,qosLatency,ioPending,Capacity(TB),Used(TB),Free(TB)";
  $header_section{$section} = $metric;

} ## end sub set_header

sub set_interval {

  foreach my $section ( keys %inventory_data ) {
    my $time        = "";
    #my $size_global = keys %{ $inventory_data{$section} };
    my $size_act    = 0;
    foreach my $id ( keys %{ $inventory_data{$section} } ) {
      if ( defined $inventory_data{$section}{$id}{"time"} && $inventory_data{$section}{$id}{"time"} ne "" ) {
        my $time_act = $inventory_data{$section}{$id}{"time"};
        #### convert to actual time zone
        my $time_loc_act = str2time($time_act);
        if (!defined $time_loc_act){
          error("ERROR: variable time_loc_act is not defined for $section $id $time_act");
          next;
        }
        my $time_all     = get_time($time_loc_act);

        #(my $year, my $moonth, my $other ) = split("-",$time_act);
        #(my $day, my $time_loc) = split("T",$other);
        #my $time_last = substr($time_loc,0,8);
        #if (defined $year && defined $moonth && defined $day && defined $time_last){
        #  my $time_all = "$year:$moonth:$day". "T" . "$time_last";
        #  $inventory_data{$section}{$id}{"TIME"} = $time_all;
        #  if ($time eq ""){
        #    $size_act++;
        #    $time = $time_all;
        #  }
        #  else{
        #    if ($time eq "$time_all"){
        #      $size_act++;
        #    }
        #  }
        #}
        if ( defined $time_all ) {
          $inventory_data{$section}{$id}{"TIME"} = $time_all;
          if ( $time eq "" ) {
            $size_act++;
            $time = $time_all;
          }
          else {
            if ( $time eq "$time_all" ) {
              $size_act++;
            }
          }
        } ## end if ( defined $time_all)
        if ( $time ne "" ) {
          my $time_start = str2time($time);
          if ( !isdigit($time_start) ) { next; }    ### no corect time
          $time_start                            = $time_start - 300;
          $inventory_data{$section}{TIME}{STEP}  = 300;
          $inventory_data{$section}{TIME}{START} = get_time($time_start);
         $inventory_data{$section}{TIME}{END}   = $time;
        } ## end if ( $size_act == $size_global)
      } ## end if ( defined $inventory_data...)
    } ## end foreach my $id ( keys %{ $inventory_data...})
  } ## end foreach my $section ( keys ...)
  foreach my $section ( keys %inventory_data ) {    ### if not defined time from perf data
    if ( !defined $inventory_data{$section}{TIME} ) {
      my $time_end          = time;
      my $time_start        = $time_end - 300;
      my $time_end_string   = get_time($time_end);
      my $time_start_string = get_time($time_start);
      $inventory_data{$section}{TIME}{STEP}  = 300;
      $inventory_data{$section}{TIME}{START} = $time_start_string;
      $inventory_data{$section}{TIME}{END}   = $time_start_string;
    } ## end if ( !defined $inventory_data...)
  } ## end foreach my $section ( keys ...)

} ## end sub set_interval

sub get_time {
  my $timestamp_end = shift;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $time_end = "$year:$month:$day" . "T" . "$hour:$minute:$sec";
  return $time_end;

} ## end sub get_time

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  if ( !defined($digit) ) {
    return 0;
  }
  if ( $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
} ## end sub isdigit

sub get_data_api_configuration {

  my $cookie      = shift;
  my $api_version = shift;
  my $url         = shift;
  my $object      = shift;

  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $req = HTTP::Request->new( GET => $url );
  $req->header( 'x-dell-api-version' => $api_version );
  $req->header( 'Content-Type'       => 'application/json; charset=utf-8' );
  $req->header( 'Accept'             => 'application/json' );
  $req->header( 'cookie'             => $cookie );
  my $res = $ua->request($req);

  if ( $res->is_success ) {
    ### for debug
    my $file = "$debug_data/$object.txt";
    store \$res->content, $file or error("Store data to file $file failed.");
    ###
    my $data = decode_json( $res->content );
    return $data;

  } ## end if ( $res->is_success )
  else {
    error("ERROR: Command $url failed!");
    error( $res->content );
    return "";
  }

} ## end sub get_data_api_configuration

sub set_data_server {

  my $section = "Host Level Statistics";
  my ( $cookie, $api_version ) = login_api();

  if ( !defined $cookie || $cookie eq "" ) {
    error("Cookie is not defined!");
    return;
  }
  ### first get id all volume
  my $object = "ScServer";
  my $url    = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object";
  my $data   = get_data_api_configuration( $cookie, $api_version, $url, $object );
  if ( $data eq "" ) {
    error("ERROR: No valid data for object $object");
    return;
  }

  $object = "ScServerIoUsage";
  my $method = "GetFilter";
  $url = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object/$method";

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $section_type     = $element->{"type"};
    my $storage_id       = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( !defined $section_type || $section_type ne "Physical" ) {
      next;    ### Only Physical is Host
    }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      $section_id =~ s/$instance_storage\.//;
      my $data = get_data_api_performance( $cookie, $api_version, $url, $section_instance );
      if ( defined $data && $data ne "" ) {
        $inventory_data{$section}{$section_id} = $data;
        $inventory_data{$section}{$section_id}{ID} = $section_id;
        if ( defined $section_name && $section_name ne "" ) {
          $section_name =~ s/://g;
          $section_name =~ s/\///g;
          $section_name =~ s/\\//g;
          $section_name =~ s/"//g;
          $section_name =~ s/'//g;
          $section_name = remove_unauthorized_characters($section_name);
          $inventory_data{$section}{$section_id}{NAME} = $section_name;
        } ## end if ( defined $section_name...)
      } ## end if ( defined $data && ...)
    } ## end if ( defined $section_instance...)
  } ## end foreach my $element ( @{$data...})
  logout_api( $cookie, $api_version );

} ## end sub set_data_server

sub set_data_volume_folder {

  my $section = "Volume-Folder Level Statistics";
  my ( $cookie, $api_version ) = login_api();

  if ( !defined $cookie || $cookie eq "" ) {
    error("Cookie is not defined!");
    return;
  }
  ### first get id all volume
  my $object = "ScVolumeFolder";
  my $url    = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object";
  my $data   = get_data_api_configuration( $cookie, $api_version, $url, $object );
  if ( $data eq "" ) {
    error("ERROR: No valid data for object $object");
    return;
  }

  $object = "ScVolumeFolderIoUsage";
  my $method = "GetFilter";
  $url = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object/$method";

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $storage_id       = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      $section_id =~ s/$instance_storage\.//;
      if ( defined $section_id && $section_id eq "0" ) { next; }    ### this is pool is per statistics for all pools
      my $data = get_data_api_performance( $cookie, $api_version, $url, $section_instance );
      if ( defined $data && $data ne "" ) {
        $inventory_data{$section}{$section_instance} = $data;
        $inventory_data{$section}{$section_instance}{ID} = $section_id;
        set_capacity( $section_instance, $cookie, $api_version, "ScVolumeFolder", $section );
        if ( defined $section_name && $section_name ne "" ) {
          $section_name =~ s/://g;
          $section_name =~ s/\///g;
          $section_name =~ s/\\//g;
          $section_name =~ s/"//g;
          $section_name =~ s/'//g;
          $section_name = remove_unauthorized_characters($section_name);
          $inventory_data{$section}{$section_instance}{NAME} = $section_name;
        } ## end if ( defined $section_name...)
      } ## end if ( defined $data && ...)
    } ## end if ( defined $section_instance...)
  } ## end foreach my $element ( @{$data...})
  logout_api( $cookie, $api_version );

} ## end sub set_data_volume_folder

sub set_data_node {

  my $section = "Node Level Statistics";
  my ( $cookie, $api_version ) = login_api();

  if ( !defined $cookie || $cookie eq "" ) {
    error("Cookie is not defined!");
    return;
  }
  ### first get id all volume
  my $object = "ScController";
  my $url    = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object";
  my $data   = get_data_api_configuration( $cookie, $api_version, $url, $object );
  if ( $data eq "" ) {
    error("ERROR: No valid data for object $object");
    return;
  }

  $object = "ScControllerIoUsage";
  my $method = "GetFilter";
  $url = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object/$method";

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $storage_id       = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      $section_id =~ s/$instance_storage\.//;
      my $data = get_data_api_performance( $cookie, $api_version, $url, $section_instance );
      if ( defined $data && $data ne "" ) {
        $inventory_data{$section}{$section_id} = $data;
        $inventory_data{$section}{$section_id}{ID} = $section_id;
        if ( defined $section_name && $section_name ne "" ) {
          $section_name =~ s/://g;
          $section_name =~ s/\///g;
          $section_name =~ s/\\//g;
          $section_name =~ s/"//g;
          $section_name =~ s/'//g;
          $section_name = remove_unauthorized_characters($section_name);
          $inventory_data{$section}{$section_id}{NAME} = $section_name;
        } ## end if ( defined $section_name...)
      } ## end if ( defined $data && ...)
    } ## end if ( defined $section_instance...)
  } ## end foreach my $element ( @{$data...})
  logout_api( $cookie, $api_version );

} ## end sub set_data_node

sub set_data_disk {

  my $section = "Drive Level Statistics";
  my ( $cookie, $api_version ) = login_api();

  if ( !defined $cookie || $cookie eq "" ) {
    error("Cookie is not defined!");
    return;
  }
  ### first get id all volume
  my $object = "ScDisk";
  my $url    = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object";
  my $data   = get_data_api_configuration( $cookie, $api_version, $url, $object );
  if ( $data eq "" ) {
    error("ERROR: No valid data for object $object");
    return;
  }

  $object = "ScDiskIoUsage";
  my $method = "GetFilter";
  $url = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object/$method";

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $storage_id       = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      $section_id =~ s/$instance_storage\.//;
      my $data = get_data_api_performance( $cookie, $api_version, $url, $section_instance );
      if ( defined $data && $data ne "" ) {
        $inventory_data{$section}{$section_instance} = $data;
        $inventory_data{$section}{$section_instance}{ID} = $section_id;
        set_capacity( $section_instance, $cookie, $api_version, "ScDisk", $section );
        if ( defined $section_name && $section_name ne "" ) {
          $section_name =~ s/://g;
          $section_name =~ s/\///g;
          $section_name =~ s/\\//g;
          $section_name =~ s/"//g;
          $section_name =~ s/'//g;
          $section_name = remove_unauthorized_characters($section_name);
          $inventory_data{$section}{$section_instance}{NAME} = $section_name;
        } ## end if ( defined $section_name...)
      } ## end if ( defined $data && ...)
    } ## end if ( defined $section_instance...)
  } ## end foreach my $element ( @{$data...})
  logout_api( $cookie, $api_version );

} ## end sub set_data_disk

sub remove_whitespace {
  my $value = shift;
  if ( !defined $value || $value eq "" ) {
    return "";
  }
  $value =~ s/^\s+|\s+$//g;
  return $value;

} ## end sub remove_whitespace

sub set_capacity {

  my $element_instance           = shift;
  my $cookie                     = shift;
  my $api_version                = shift;
  my $object                     = shift;
  my $section                    = shift;
  my $denominator                = 1024 * 1024;
  my $string_total_cap           = "configuredSpace";
  my $string_used_cap            = "activeSpace";
  my $string_free_cap            = "freeSpace";
  my $string_snapshot_cap        = "snapshotOverheadOnDisk";
  my $string_output_total_cap    = "Capacity(MB)";
  my $string_output_used_cap     = "Used(MB)";
  my $string_output_free_cap     = "Free(MB)";
  my $string_output_snapshot_cap = "Snapshot(MB)";

  if ( $object eq "ScDisk" ) {
    $string_total_cap = "totalSpace";
    $string_used_cap  = "allocatedSpace";
    $string_free_cap  = "freeSpace";
  }
  if ( $object eq "ScVolumeFolder" ) {
    $string_total_cap           = "configuredSpace";
    $string_used_cap            = "activeSpace";
    $string_free_cap            = "freeSpace";
    $string_snapshot_cap        = "replaySpace";
    $denominator                = 1024 * 1024 * 1024 * 1024;
    $string_output_total_cap    = "Capacity(TB)";
    $string_output_used_cap     = "Used(TB)";
    $string_output_free_cap     = "Free(TB)";
    $string_output_snapshot_cap = "Snapshot(TB)";
  } ## end if ( $object eq "ScVolumeFolder")

  my $url = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object/$element_instance/StorageUsage";
  my $data = get_data_api_configuration( $cookie, $api_version, $url, $object );
  if ( $data eq "" ) {
    error("ERROR: No valid data for object $object");
    return;
  }

  my $section_instance  = $element_instance;
  my $total_capacity    = $data->{$string_total_cap};
  my $free_capacity     = $data->{$string_free_cap};
  my $used_capacity     = $data->{$string_used_cap};
  my $snapshot_capacity = $data->{$string_snapshot_cap};
  if ( defined $total_capacity ) {
    $total_capacity =~ s/Bytes//g;
    $total_capacity = remove_whitespace($total_capacity);
  }
  if ( defined $used_capacity ) {
    $used_capacity =~ s/Bytes//g;
    $used_capacity = remove_whitespace($used_capacity);
  }
  if ( defined $free_capacity ) {
    $free_capacity =~ s/Bytes//g;
    $free_capacity = remove_whitespace($free_capacity);
  }
  if ( defined $snapshot_capacity ) {
    $snapshot_capacity =~ s/Bytes//g;
    $snapshot_capacity = remove_whitespace($snapshot_capacity);
  }
  if ( isdigit($total_capacity) && !isdigit($free_capacity) && isdigit($used_capacity) ) {
    $free_capacity = $total_capacity - $used_capacity;
  }

  if ( isdigit($total_capacity) && isdigit($free_capacity) && !isdigit($used_capacity) ) {
    $used_capacity = $total_capacity - $free_capacity;
  }

  if ( !isdigit($total_capacity) && isdigit($free_capacity) && isdigit($used_capacity) ) {
    $total_capacity = $free_capacity + $used_capacity;
  }

  if ( defined $section_instance && $section_instance ne "" ) {
    if ( isdigit($total_capacity) ) {
      my $pom = $total_capacity / $denominator;
      $total_capacity = round( $pom, 3 );
      if ( defined $inventory_data{$section}{$section_instance} ) {
        $inventory_data{$section}{$section_instance}{$string_output_total_cap} = $total_capacity;
      }

    } ## end if ( isdigit($total_capacity...))
    if ( isdigit($free_capacity) ) {
      my $pom = $free_capacity / $denominator;
      $free_capacity = round( $pom, 3 );
      if ( defined $inventory_data{$section}{$section_instance} ) {
        $inventory_data{$section}{$section_instance}{$string_output_free_cap} = $free_capacity;
      }

    } ## end if ( isdigit($free_capacity...))
    if ( isdigit($snapshot_capacity) ) {
      my $pom = $snapshot_capacity / $denominator;
      $snapshot_capacity = round( $pom, 3 );
      if ( defined $inventory_data{$section}{$section_instance} ) {
        $inventory_data{$section}{$section_instance}{$string_output_snapshot_cap} = $snapshot_capacity;
      }

    } ## end if ( isdigit($snapshot_capacity...))
    if ( isdigit($used_capacity) ) {
      my $pom = $used_capacity / $denominator;
      $used_capacity = round( $pom, 3 );
      if ( defined $inventory_data{$section}{$section_instance} ) {
        $inventory_data{$section}{$section_instance}{$string_output_used_cap} = $used_capacity;
      }

    } ## end if ( isdigit($used_capacity...))

  } ## end if ( defined $section_instance...)

} ## end sub set_capacity

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub set_data_volume {

  my $section = "Volume Level Statistics";
  my ( $cookie, $api_version ) = login_api();

  if ( !defined $cookie || $cookie eq "" ) {
    error("Cookie is not defined!");
    return;
  }
  ### first get id all volume
  my $object = "ScVolume";
  my $url    = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object";
  my $data   = get_data_api_configuration( $cookie, $api_version, $url, $object );
  if ( $data eq "" ) {
    error("ERROR: No valid data for object $object");
    return;
  }

  $object = "ScVolumeIoUsage";
  my $method = "GetFilter";
  $url = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object/$method";

  foreach my $element ( @{$data} ) {
    my $volume_instance = $element->{"instanceId"};
    my $volume_id       = $element->{"deviceId"};
    my $volume_name     = $element->{"name"};
    my $storage_id      = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( defined $volume_instance && $volume_instance ne "" && defined $volume_id && $volume_id ne "" ) {
      my $data = get_data_api_performance( $cookie, $api_version, $url, $volume_instance );
      if ( defined $data && $data ne "" ) {
        $inventory_data{$section}{$volume_instance} = $data;
        $inventory_data{$section}{$volume_instance}{ID} = $volume_id;
        set_capacity( $volume_instance, $cookie, $api_version, "ScVolume", $section );
        if ( defined $volume_name && $volume_name ne "" ) {
          $volume_name =~ s/://g;
          $volume_name =~ s/\///g;
          $volume_name =~ s/\\//g;
          $volume_name =~ s/"//g;
          $volume_name =~ s/'//g;
          $volume_name = remove_unauthorized_characters($volume_name);
          $inventory_data{$section}{$volume_instance}{NAME} = $volume_name;
        } ## end if ( defined $volume_name...)
      } ## end if ( defined $data && ...)
    } ## end if ( defined $volume_instance...)
  } ## end foreach my $element ( @{$data...})
  logout_api( $cookie, $api_version );

} ## end sub set_data_volume

sub set_data_port {

  my $section = "Port Level Statistics";
  my ( $cookie, $api_version ) = login_api();

  if ( !defined $cookie || $cookie eq "" ) {
    error("Cookie is not defined!");
    return;
  }
  ### first get id all ports
  my $object = "ScControllerPort";
  my $url    = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object";
  my $data   = get_data_api_configuration( $cookie, $api_version, $url, $object );
  if ( $data eq "" ) {
    error("ERROR: No valid data for object $object");
    return;
  }

  $object = "ScControllerPortIoUsage";
  my $method = "GetFilter";
  $url = $proto . '://' . $ip_adress_a . ':' . $port . "/api/rest/StorageCenter/$object/$method";

  foreach my $element ( @{$data} ) {
    my $port_id    = $element->{"instanceId"};
    my $port_ids   = $port_id;
    my $virtual    = $element->{"virtual"};
    my $storage_id = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }

    #my $type = $element->{"purpose"};
    #if (!defined $type || ($type ne "FrontEnd" && $type ne "BackEnd")){next;}
    my $protocol = $element->{"transportType"};
    if ( isdigit($virtual) && $virtual == 1 ) { next; }
    my $port_name = $element->{"name"};
    if ( defined $port_id && $port_id ne "" && defined $port_ids && $port_ids ne "" ) {
      $port_ids =~ s/$instance_storage\.//;
      my $data = get_data_api_performance( $cookie, $api_version, $url, $port_id );
      if ( defined $data && $data ne "" ) {
        $inventory_data{$section}{$port_ids} = $data;
        $inventory_data{$section}{$port_ids}{ID} = $port_ids;
        if ( defined $port_name && $port_name ne "" ) {
          $port_name =~ s/://g;
          $port_name =~ s/\///g;
          $port_name =~ s/\\//g;
          $port_name =~ s/"//g;
          $port_name =~ s/'//g;
          $port_name = remove_unauthorized_characters($port_name);
          $inventory_data{$section}{$port_ids}{NAME} = $port_name;
        } ## end if ( defined $port_name...)
      } ## end if ( defined $data && ...)
      if ( defined $protocol && $protocol ne "" ) {
        my $protocol_short = "Unknown";
        if ( $protocol eq "FibreChannel" ) {
          $protocol_short = "FC";
        }
        if ( $protocol eq "FibreChannelOverEthernet" ) {
          $protocol_short = "FCoE";
        }
        if ( $protocol eq "Iscsi" ) {
          $protocol_short = "iSCSI";
        }
        if ( $protocol eq "Sas" ) {
          $protocol_short = "SAS";
        }
        $inventory_data{$section}{$port_ids}{"protocol"} = $protocol_short;
      } ## end if ( defined $protocol...)
    } ## end if ( defined $port_id ...)
  } ## end foreach my $element ( @{$data...})

  logout_api( $cookie, $api_version );

} ## end sub set_data_port

sub get_data_api_performance {
  my $cookie      = shift;
  my $api_version = shift;
  my $url         = shift;
  my $id          = shift;
  my $ua          = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  ### body for request
  my %hash_filter;

  #$hash_filter{"HistoricalFilter"}{'StartTime'} = "2017-06-09T12:35:45Z";
  #$hash_filter{"HistoricalFilter"}{"EndTime"} = "2017-06-09T12:40:45Z";
  $hash_filter{"HistoricalFilter"}{"FilterTime"}    = "Other";
  $hash_filter{"HistoricalFilter"}{"UseEndOfDay"}   = "true";
  $hash_filter{"HistoricalFilter"}{"UseStartOfDay"} = "true";
  $hash_filter{"HistoricalFilter"}{"UseCurrent"}    = "true";

  #$hash_filter{"HistoricalFilter"}{"MaxCountReturn"} = "0";
  $hash_filter{"ObjectInstanceId"} = "$id";
  my $json_body = encode_json( \%hash_filter );
  ###

  my $req = HTTP::Request->new( POST => $url );
  $req->header( 'x-dell-api-version' => $api_version );
  $req->header( 'Content-Type'       => 'application/json; charset=utf-8' );
  $req->header( 'Accept'             => 'application/json' );
  $req->header( 'cookie'             => $cookie );
  $req->content($json_body);
  my $res = $ua->request($req);

  if ( $res->is_success ) {
    my $data         = decode_json( $res->content );
    my $first_record = @{$data}[-1];
    return $first_record;
  }
  else {
    error("ERROR: Command $url failed!");
    error( $res->content );
    return "";
  }

} ## end sub get_data_api_performance

sub logout_api {
  my $cookie      = shift;
  my $api_version = shift;

  ### Logout
  my $url = $proto . '://' . $ip_adress_a . ':' . $port . '/api/rest/ApiConnection/Logout';
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( POST => $url );
  $req->header( 'x-dell-api-version' => $api_version );
  $req->header( 'Content-Type'       => 'application/json; charset=utf-8' );
  $req->header( 'Accept'             => 'application/json' );
  $req->header( 'cookie'             => $cookie );
  my $res = $ua->request($req);

  if ( !$res->is_success ) {
    error("ERROR: Logout to API failed\n");
    error( $res->content );
  }

} ## end sub logout_api

sub login_api {

  my $cookie      = "";
  my $api_version = "3.1";
  my $ua          = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $url         = $proto . '://' . $ip_adress_a . ':' . $port . '/api/rest/ApiConnection/Login';
  my $req         = HTTP::Request->new( POST => $url );
  $req->header( 'Content-Type'       => 'application/json' );
  $req->header( 'Accept'             => 'application/json' );
  $req->header( 'x-dell-api-version' => '3.1' );                ### why should be for login?
  $req->authorization_basic( $user, $passwd );
  my $res = $ua->request($req);

  if ( !$res->is_success ) {
    error("ERROR: Login to API failed\n");
    error( $res->content );
    return $cookie;
  } ## end if ( !$res->is_success)
  $cookie = $res->headers->{"set-cookie"};
  my $result = decode_json( $res->content );
  if ( defined $result->{apiVersion} && $result->{apiVersion} ) {
    $api_version = $result->{apiVersion};
  }

  return ( $cookie, $api_version );

} ## end sub login_api

sub set_variables {

  if ($compellent_enviroment) {
    my $act_directory = localtime();
    my @entry_time = split( " ", $act_directory );
    ( undef, my $active_minute, undef ) = split( ":", $entry_time[3] );
    $storage_name      = $ENV{STORAGE_NAME};
    $basedir           = $ENV{INPUTDIR};
    $bindir            = $ENV{BINDIR};
    $webdir            = $ENV{WEBDIR};
    $ERRLOG_COMPELLENT = $ENV{ERRLOG_COMPELLENT};
    if ( defined $webdir && $webdir ne "" && defined $storage_name && $storage_name ne "" ) {
      $health_status_html = "$webdir/$storage_name/health_status.html";
      if (!-d "$webdir/$storage_name"){
        mkdir "$webdir/$storage_name" || error_die( "Cannot read $webdir/$storage_name: $!" . __FILE__ . ":" . __LINE__ );
      }
      $wrkdir             = "$basedir/data/$storage_name";
      $data_directory     = "$wrkdir/IOSTATS";
      if ( !-d "$data_directory/$active_minute" ) {
        mkdir "$data_directory/$active_minute" || error_die( "Cannot read $data_directory: $!" . __FILE__ . ":" . __LINE__ );
      }
      $debug_data         = "$data_directory/$active_minute";
      if ( defined $basedir && $basedir ne "" ) {
        $summary_health = "$basedir/tmp/health_status_summary";
      }
      else {
        error_die("BASEDIR is not defined!");
      }
    } ## end if ( defined $webdir &&...)
    else {
      error_die("WEBDIR or STORAGE NAME is not defined!");
    }
    $user        = $ENV{COMPELLENT_USER};
    $passwd      = $ENV{COMPELLENT_PW};
    $ip_adress_a = $ENV{COMPELLENT_IP_SPA};
    $port = $ENV{COMPELLENT_PORT};
    if (!defined $port || $port eq ""){$port = 3033;}
    if ( defined $passwd ) {
      my $encode = Xorux_lib::unobscure_password($passwd);
      if ( defined $encode ) {
        $passwd = $encode;
      }
    } ## end if ( defined $passwd )
    if ( !defined $user || $user eq "" || !defined $passwd || $passwd eq "" || !defined $ip_adress_a || $ip_adress_a eq "" ) {
      error_die("USER, IP ADDRESS or PASSWORD is not defined!");
    }
  } ## end if ($compellent_enviroment)
  else {
    my $act_directory = localtime();
    my @entry_time = split( " ", $act_directory );
    ( undef, my $active_minute, undef ) = split( ":", $entry_time[3] );
    $storage_name = "";
    $basedir      = "";
    $webdir             = "";
    $health_status_html = "";
    $summary_health     = "";
    $passwd             = "";
    $user               = "";
    $ip_adress_a        = "";
    $wrkdir             = "";
    $data_directory     = "";
    $port = "";
    if ( !-d "$data_directory/$active_minute" ) {
        mkdir "$data_directory/$active_minute" || error_die( "Cannot read $data_directory: $!" . __FILE__ . ":" . __LINE__ );
    }
    $debug_data         = "";
  } ## end else [ if ($compellent_enviroment)]

} ## end sub set_variables

sub remove_unauthorized_characters {
  my $name = shift;

  if ( !defined $ENV{UNAUTHORIZED_CHARACTERS} || $ENV{UNAUTHORIZED_CHARACTERS} eq "" ) {
    return $name;
  }
  my $string = $ENV{UNAUTHORIZED_CHARACTERS};
  my @chars = split( "", $string );

  foreach my $char (@chars) {
    $name =~ s/$char//g;
  }

  return $name;

} ## end sub remove_unauthorized_characters

sub error_die {
  my $message  = shift;
  my $act_time = localtime();
  print STDERR "$act_time: $message : $!\n";
  exit(1);
} ## end sub error_die

sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
} ## end sub error

