#!/bin/ksh

# Parameters:
# Param1 - Storage name

#set -x

PATH=$PATH:/usr/bin:/bin:/usr/local/bin:/opt/freeware/bin
export PATH

# it is necessary as files need to be readable also for WEB server user
umask 022

if [ ! -d "etc" ]; then
  if [ -d "../etc" ]; then
    cd ..
  else
    echo "Problem with actual directory, assure you are in LPAR2RRD home"
    echo "Then run : sh ./bin/config_check.sh"
    exit
  fi
fi

storage=$1

LHOME=`pwd|sed -e 's/bin//' -e 's/\/\//\//' -e 's/\/$//'` # it must be here
pwd=`pwd`

cleanup()
{
  echo "Exiting on alarm $1"
  exit 2
}

trap "cleanup 1" ALRM
trap "cleanup 2" TERM


# Load STOR2RRD configuration
CFG="$pwd/etc/stor2rrd.cfg"
. $CFG
DEBUG=0

if [ "$PERL" = "/opt/freeware/bin/perl" ]; then
  # Perl LWP module on AIX requires openssl from /usr/lib!!! It does not work with  /opt/freeware/lib one
  # It is a must for VMAX, PURE and others using https REST API
  export LIBPATH=/usr/lib:/opt/freeware/lib:$LIBPATH
fi

# Load "magic" setup
if [ -f "$pwd/etc/.magic" ]; then
  . $pwd/etc/.magic
fi

# correct BINDIR, it contains: ./bin/bin when it runs as ./bin/config_check.sh
BINDIR=`echo "$BINDIR"|sed 's/\/bin\/bin/\/bin/'`
INPUTDIR=`echo "$INPUTDIR"|sed -e 's/\/bin$//' -e 's/bin$/\./'`

PERL5LIB=$PERL5LIB:$BINDIR
export PERL5LIB

# it covers everything, just a wrapper to be able tail output in logs dir
main () {


if [ "$INPUTDIR" = "." ]; then
   INPUTDIR=`pwd`
   export INPUTDIR
fi

# Basic checks ...
if [ ! -d "$WEBDIR" ]; then
  echo "Set correct path to WEBDIR in stor2rrd.cfg, it does not exist here: $WEBDIR"
  exit 0
fi
if [ x"$STORAGE_USER" = "x" ]; then
  echo "STORAGE_USER does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi
if [ x"$DS8_CLIDIR" = "x" ]; then
  echo "DS8_CLIDIR does not seem to be set up, correct it in stor2rrd.cfg"
fi
if [ x"$STORAGE_CFG" = "x" ]; then
  echo "STORAGE_CFG does not seem to be set up, correct it in stor2rrd.cfg"
  exit 1
fi
if [ ! -f "$RRDTOOL" ]; then
  echo "Set correct path to RRDTOOL binary in etc/stor2rrd.cfg, it does not exist here: $RRDTOOL"
  exit 0
fi
if [ x"$PERL" = "x" ]; then
  echo "PERL does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi
if [ x"$SAMPLE_RATE" = "x" ]; then
  echo "SAMPLE_RATE does not seem to be set up, correct it in stor2rrd.cfg"
fi
if [ x"$SAN_CFG" = "x" ]; then
  echo "SAN_CFG does not seem to be set up, correct it in stor2rrd.cfg"
fi
SAN_CFG=`echo "$SAN_CFG"| sed 's/bin\///'`

if [ x"$DS5_CLIDIR" = "x" ]; then
  echo "DS5_CLIDIR does not seem to be set up, correct it in stor2rrd.cfg"
fi
if [ x"$VNX_BLOCK_CLIDIR" = "x" ]; then
  echo "VNX_BLOCK_CLIDIR does not seem to be set up, correct it in stor2rrd.cfg"
fi

$PERL bin/perl_modules_check.pl $PERL


STATUS_FILE=$INPUTDIR/logs/config_check_sum.out
rm -f $STATUS_FILE

# exclude "-q" as it suppress error messages
SSH_tmp=`echo $SSH| sed 's/ -q / /'`
SSH=$SSH_tmp

count=0
date=`date`
SAMPLE_RATE_DEF=$SAMPLE_RATE


for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":DS8K:"|egrep -v "^#"`
do
  # Only DS8K load ....
  # Name:DS8K:DEVID:HMC1:HMC2:
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  DS8_DEVID=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  DS8_HMC1=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  DS8_HMC2=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
  SAMPLE_RATE_min=`echo $line | awk 'BEGIN{FS=":"}{print $8}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi 

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE "
  echo "========================="

  SAMPLE_RATE=$SAMPLE_RATE_DEF
  perl bin/conntest.pl $DS8_HMC1 1750 WARNING
  perl bin/conntest.pl $DS8_HMC1 1751 WARNING

  if [ "$DS8_HMC2"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg, check $STORAGE_CFG line format: $line"
    echo "Right format is: Name:DS8K:DEVID:HMC1:HMC2:"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  if [ ! -f "$DS8_CLIDIR/dscli" ]; then
    echo "DSCLI binnary does not exist here : $DS8_CLIDIR/dscli"
    echo "If it is installed then configure proper path in etc/stor2rrd.cfg, param DS8_CLIDIR"
    continue
  fi
  if [ "$DS8_HMC2"x = "x" ]; then
    echo "  $DS8_CLIDIR/dscli -hmc1 $DS8_HMC1 -user $STORAGE_USER whoami"
    $DS8_CLIDIR/dscli -hmc1 $DS8_HMC1 -user $STORAGE_USER whoami
    echo "  $DS8_CLIDIR/dscli -hmc1 $DS8_HMC1 -user $STORAGE_USER ver -l"
    $DS8_CLIDIR/dscli -hmc1 $DS8_HMC1 -user $STORAGE_USER ver -l
    if [ $? -eq 0 ]; then
      echo "$STORAGE_NAME  ok" >> $STATUS_FILE
      echo "  connection ok!!"          
    else
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
      echo "  connection failed!!"          
    fi
  else
    echo "  $DS8_CLIDIR/dscli -hmc1 $DS8_HMC1 -hmc2 $DS8_HMC2 -user $STORAGE_USER whoami"
    $DS8_CLIDIR/dscli -hmc1 $DS8_HMC1 -hmc2 $DS8_HMC2 -user $STORAGE_USER whoami
    echo "  $DS8_CLIDIR/dscli -hmc1 $DS8_HMC1 -hmc2 $DS8_HMC2 -user $STORAGE_USER ver -l"
    $DS8_CLIDIR/dscli -hmc1 $DS8_HMC1 -hmc2 $DS8_HMC2 -user $STORAGE_USER ver -l
    if [ $? -eq 0 ]; then
      echo "$STORAGE_NAME  ok" >> $STATUS_FILE
      echo "  connection ok!!"
    else
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
      echo "  connection failed!!"
    fi
  fi
  echo ""
  (( count = count + 1 ))
done
if [ $count -gt 0 ]; then
  echo ""
  echo "$STORAGE_USER user on all IBM DS8K must belong to \"monitor\" role at least"
  echo ""
fi

count=0
TMP_FILE_SWIZ="$INPUTDIR/tmp/swiz_tmp.$$"
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":SWIZ:"|egrep -v "^#"`
do
        # Name:SWIZ:DEVID:HMC1:HMC2:
        STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
        STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
        SAMPLE_RATE_min=`echo $line | awk 'BEGIN{FS=":"}{print $7}'`

        if [ ! "$storage"x = "x" ]; then
          if [ ! "$storage" = "$STORAGE_NAME" ]; then
            continue
          fi
        fi 

        # SVC/Storwize Storage Family
        SVC_IP=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
        #SVC_USER=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
        SVC_USER=$STORAGE_USER
        if [ ! x"$STORAGE_USER_SWIZ" = "x" ]; then
          SVC_USER=$STORAGE_USER_SWIZ
        fi
        SVC_KEY_tmp=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
        if [ -f "$SVC_KEY_tmp" ]; then
          SVC_KEY="$SVC_KEY_tmp" # go ahead only if this file exists
        fi

        SVC_DIR="$INPUTDIR/data/$STORAGE_NAME"

        if [ "$SVC_IP"x = "x" ]; then
                echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
                echo "$STORAGE_NAME nok" >> $STATUS_FILE
                continue
        fi

        STOR2RRD_STEP=$SAMPLE_RATE            # in seconds
        if [ ! "$SAMPLE_RATE_min"x = "x" ]; then
          # set specific sample rate if it is defined in storage-list.cfg
          (( STOR2RRD_STEP = SAMPLE_RATE_min * 60 ))
        fi
        if [ "$STOR2RRD_STEP"x = "x" ]; then
          STOR2RRD_STEP=300
        fi
        if [ $STOR2RRD_STEP -eq 0 ]; then
          STOR2RRD_STEP=300
        fi

        echo "========================="
        echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE" : sample rate: $STOR2RRD_STEP seconds
        echo "========================="

        # connection test
        perl bin/conntest.pl $SVC_IP 22



        if [ "$SVC_KEY"x = "x" ]; then
          echo "  $SSH $STORAGE_USER@$SVC_IP  \"lscurrentuser\""
          $SSH $STORAGE_USER@$SVC_IP  "lscurrentuser" 2>&1 > $TMP_FILE_SWIZ
          egrep "^role" $TMP_FILE_SWIZ >/dev/null
          if [ $? -eq 0 ]; then
            egrep "Administrator" $TMP_FILE_SWIZ >/dev/null
	    if [ $? -eq 0 ]; then
              echo "  connection ok"          
              echo "$STORAGE_NAME  ok" >> $STATUS_FILE
            else 
              cat $TMP_FILE_SWIZ| sed 's/^/  /g'
              echo "  current account does not have Administration access!!"          
              echo "$STORAGE_USER user on all IBM SVC/Storwize must belong to \"admin\" or \"Administrator\" UserRole"
              $SSH $STORAGE_USER@$SVC_IP  "lscurrentuser"
            fi
          else 
            egrep "UserRole " $TMP_FILE_SWIZ >/dev/null
            if [ $? -eq 0 ]; then
              echo "  Looks like it is a Unified box and you use wrong IP, use System IP"
              echo "  Refer to http://www.stor2rrd.com/storwize-unified.htm"
            fi
            cat $TMP_FILE_SWIZ| sed 's/^/  /g'
            echo "  connection failed!!"          
            echo "$STORAGE_NAME nok" >> $STATUS_FILE
          fi
        else
          echo "  $SSH -i $SVC_KEY  $STORAGE_USER@$SVC_IP  \"lscurrentuser\""
          $SSH -o StrictHostKeyChecking=no -i $SVC_KEY  $STORAGE_USER@$SVC_IP  "lscurrentuser" 2>&1 > $TMP_FILE_SWIZ
          egrep "^role" $TMP_FILE_SWIZ >/dev/null
          if [ $? -eq 0 ]; then
            egrep "Administrator" $TMP_FILE_SWIZ >/dev/null
	    if [ $? -eq 0 ]; then
              echo "  connection ok"          
              echo "$STORAGE_NAME  ok" >> $STATUS_FILE
            else 
              cat $TMP_FILE_SWIZ| sed 's/^/  /g'
              echo "  current account does not have Administration access!!"          
              echo "$STORAGE_USER user on all IBM SVC/Storwize must belong to \"admin\" or \"Administrator\" UserRole"
              $SSH -o StrictHostKeyChecking=no -i $SVC_KEY  $STORAGE_USER@$SVC_IP  "lscurrentuser"
            fi
          else 
            egrep "UserRole " $TMP_FILE_SWIZ >/dev/null
            if [ $? -eq 0 ]; then
              echo "  Looks like it is a Unified box and you use wrong IP, use System IP"
              echo "  Refer to http://www.stor2rrd.com/storwize-unified.htm"
            fi
            cat $TMP_FILE_SWIZ| sed 's/^/  /g'
            echo "  connection failed!!"          
            echo "$STORAGE_NAME nok" >> $STATUS_FILE
          fi
        fi
        grep "CMMVC7016E Authorization has failed because the private key is not valid for the user name that you have specified" $TMP_FILE_SWIZ  >/dev/null
        if [ $? -eq 0 ]; then
          echo "  User \"$STORAGE_USER\" is wrong, correct name in etc/stpor2rrd.cfg STORAGE_USER item"
        fi
        rm -f  $TMP_FILE_SWIZ
        echo ""
        (( count = count + 1 ))
done


count=0
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":XIV:"|egrep -v "^#"`
do
        # Storage name alias:XIV:_xiv_ip_:_password_:
        STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
        STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`

        export STORAGE_NAME STORAGE_TYPE

        if [ ! "$storage"x = "x" ]; then
          if [ ! "$storage" = "$STORAGE_NAME" ]; then
            continue
          fi
        fi 

        echo "========================="
        echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
        echo "========================="

        # XiV Storage Family
        XIV_IP=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
        XIV_USER=$STORAGE_USER
        if [ ! x"$STORAGE_USER_XIV" = "x" ]; then
          XIV_USER=$STORAGE_USER_XIV
        fi
        XIV_PASSWD=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
        XIV_DIR="$INPUTDIR/data/$STORAGE_NAME"
        XIV_INTERVAL=$SAMPLE_RATE
        export XIV_IP XIV_USER XIV_PASSWD XIV_DIR XIV_INTERVAL

        if [ "$XIV_IP"x = "x" ]; then
                echo "Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
                echo "$STORAGE_NAME nok" >> $STATUS_FILE
                continue
        fi

        ERRLOG_XIV="$ERRLOG-${STORAGE_NAME}"
        OUTLOG_XIV="$INPUTDIR/logs/${STORAGE_NAME}.output_xiv.log"
        export ERRLOG_XIV OUTLOG_XIV

        WBEMCLI="/usr/bin/wbemcli"
        if [ ! -f $WBEMCLI ]; then
          WBEMCLI="/opt/freeware/bin/wbemcli"
          if [ ! -f $WBEMCLI ]; then
            WBEMCLI="/usr/local/bin/wbemcli"
            if [ ! -f $WBEMCLI ]; then
              echo "XIV Error: could not found wbemcli binnary, searched /usr/bin/wbemcli, /opt/freeware/bin/wbemcli, /usr/local/bin/wbemcli"
              echo "$STORAGE_NAME nok" >> $STATUS_FILE
              continue
            fi
          fi
        fi
       
        # connection test
        perl bin/conntest.pl $XIV_IP 5989

        TMP_FILE_XIV="$INPUTDIR/tmp/xiv_tmp.$$"
        echo "  $WBEMCLI -noverify -nl ei https://$XIV_USER:XXXXXXXXXXX@$XIV_IP:5989/root/ibm:IBMTSDS_StorageSystem"
        $WBEMCLI -noverify -nl ei https://$XIV_USER:$XIV_PASSWD@$XIV_IP:5989/root/ibm:IBMTSDS_StorageSystem > $TMP_FILE_XIV
        if [ $? -eq 0 -a `grep CodeLevel $TMP_FILE_XIV| wc -l| sed 's/ //g'` -gt 0 ]; then
          echo "$STORAGE_NAME  ok" >> $STATUS_FILE
          echo "  connection ok!!"
        else
          echo "$STORAGE_NAME nok" >> $STATUS_FILE
          echo "  connection failed!!"
        fi
        rm -f $TMP_FILE_XIV

        echo ""          
        (( count = count + 1 ))
done

for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":DS5K:"|egrep -v "^#"`
do
        # Storage name alias:XIV:_xiv_ip_:_password_:
        STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
        STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
        DS5K_USER=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
        DS5K_PW=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`

        export STORAGE_NAME STORAGE_TYPE

        if [ ! "$storage"x = "x" ]; then
          if [ ! "$storage" = "$STORAGE_NAME" ]; then
            continue
          fi
        fi 

        echo "========================="
        echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
        echo "========================="

        DS5_DIR="$INPUTDIR/data/$STORAGE_NAME"
        DS5_INTERVAL=$SAMPLE_RATE
        export DS5_IP DS5_USER DS5_PASSWD DS5_DIR DS5_INTERVAL

        ERRLOG_DS5="$ERRLOG-${STORAGE_NAME}"
        OUTLOG_DS5="$INPUTDIR/logs/${STORAGE_NAME}.output_ds5.log"
        export ERRLOG_DS5 OUTLOG_DS5

        if [ ! -f $DS5_CLIDIR/SMcli ]; then
          if [ ! -f /usr/bin/SMcli ]; then
            echo "DS5K Error: could not found SMcli binnary : $DS5_CLIDIR/SMcli, Install SMcli or update DS5_CLIDIR in etc/stor2rrd.cfg"
            continue
          fi
          DS5_CLIDIR=/usr/bin
        fi
       
        if [ ! "$DS5K_PW"x = "x" ] ; then
          if [ "$DS5K_PW" = "IT5gYGAK" -o "$DS5K_PW" = "IT5gYGAK/KT4mXVI9N0BUPjZdVQo=" ]; then
            echo "Error: you have not changed default password hash, use \"perl ./bin/spasswd.pl\" to get a new hash and place it into storage line in etc/storage-list.cfg"
            echo "  $line"
            echo "$STORAGE_NAME nok" >> $STATUS_FILE
            continue
          fi
        fi

        if [ "$DS5K_PW"x = "x" ]; then
          # no user/pw is set
          echo "  $DS5_CLIDIR/SMcli -n $STORAGE_NAME -e -c \"show hostTopology;\""
	  if [ `$DS5_CLIDIR/SMcli -n $STORAGE_NAME -e -c "show hostTopology;"| grep "SMcli completed successfully"|wc -l|sed 's/ //g'` -gt 0 ]; then
            echo "  connection ok"          
            echo "$STORAGE_NAME  ok" >> $STATUS_FILE
          else 
            echo "  connection failed!!"          
            echo "$STORAGE_NAME nok" >> $STATUS_FILE
          fi
        else
          # no user/pw is set
          echo "  $DS5_CLIDIR/SMcli -n $STORAGE_NAME -R $DS5K_USER -p XXXXXXX -e -c \"show hostTopology;\""
	  if [ `$DS5_CLIDIR/SMcli -n $STORAGE_NAME -R $DS5K_USER -p $DS5K_PW -e -c "show hostTopology;"| grep "SMcli completed successfully"|wc -l|sed 's/ //g'` -gt 0 ]; then
            echo "  connection ok"          
            echo "$STORAGE_NAME  ok" >> $STATUS_FILE
          else 
            echo "  connection failed!!"          
            echo "$STORAGE_NAME nok" >> $STATUS_FILE
          fi
        fi
        echo ""          
done
echo ""



# Hitachi libraries and variables
#AIX
LIBPATH=$LIBPATH:$HUS_CLIDIR/lib
#HP-UX
SHLIB_PATH=$SHLIB_PATH:$HUS_CLIDIR/lib
#Linux
LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HUS_CLIDIR/lib
STONAVM_HOME=$HUS_CLIDIR
STONAVM_ACT=on
STONAVM_RSP_PASS=on
export  LIBPATH SHLIB_PATH LD_LIBRARY_PATH STONAVM_HOME STONAVM_ACT STONAVM_RSP_PASS

for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":HUS:"|egrep -v "^#"`
do
        # Storage name alias:XIV:_xiv_ip_:_password_:
        STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
        STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`

        export STORAGE_NAME STORAGE_TYPE

        if [ ! "$storage"x = "x" ]; then
          if [ ! "$storage" = "$STORAGE_NAME" ]; then
            continue
          fi
        fi 

        echo "========================="
        echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
        echo "========================="

        ERRLOG_HUS="$ERRLOG-${STORAGE_NAME}"
        OUTLOG_HUS="$INPUTDIR/logs/${STORAGE_NAME}.output_hus.log"
        export ERRLOG_HUS OUTLOG_HUS

        if [ ! -f $HUS_CLIDIR/auperform ]; then
          echo "HUS Error: could not found HSNM2 binnary : $HUS_CLIDIR/auperform, Install HSNM2 CLI or update HUS_CLIDIR in etc/stor2rrd.cfg"
          echo "$STORAGE_NAME nok" >> $STATUS_FILE
          continue
        fi
       
        echo "  $HUS_CLIDIR/auunitinfo -unit $STORAGE_NAME"
	if [ `$HUS_CLIDIR/auunitinfo -unit "$STORAGE_NAME" 2>/dev/null| wc -l|sed 's/ //g'` -gt 10 ]; then
          echo "  connection ok"
          echo "$STORAGE_NAME  ok" >> $STATUS_FILE
        else 
          echo "  connection failed!!"
          echo "$STORAGE_NAME nok" >> $STATUS_FILE
        fi
        echo ""
done
echo ""

count=0
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":NETAPP:"|egrep -v "^#"`
do
        # Name:SWIZ:DEVID:HMC1:HMC2:
        STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
        STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
        STORAGE_MODE=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`

        if [ ! "$storage"x = "x" ]; then
          if [ ! "$storage" = "$STORAGE_NAME" ]; then
            continue
          fi
        fi 

        echo "========================="
        echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE/$STORAGE_MODE"
        echo "========================="

        # NetApp Storage Family
        NA_IP=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
        NA_PORT_SSH=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
        NA_PORT_API=`echo $line | awk 'BEGIN{FS=":"}{print $6}'`
        NA_PROTO_API=`echo $line | awk 'BEGIN{FS=":"}{print $7}'`
        NA_USER=`echo $line | awk 'BEGIN{FS=":"}{print $8}'`
        NA_PASSWD=`echo $line | awk 'BEGIN{FS=":"}{print $9}'`
        export STORAGE_NAME STORAGE_TYPE STORAGE_MODE NA_IP NA_PORT_SSH NA_PORT_API NA_PROTO_API NA_USER NA_PASSWD

        if [ "$NA_IP"x = "x" ]; then
          echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP/hostname must be set. Check $STORAGE_CFG line: $line"
          continue
        fi

        if [ "$NA_PROTO_API"x = "x" ]; then
          echo "$STORAGE_NAME: Some problem with storage cfg. API protocol must be set. Check $STORAGE_CFG line: $line"
          continue
        fi

        if [ "$NA_PORT_API"x = "x" ]; then
          echo "$STORAGE_NAME: Some problem with storage cfg. API port must be set. Check $STORAGE_CFG line: $line"
          continue
        fi

        if [ "$NA_USER"x = "x" ]; then
          echo "$STORAGE_NAME: Some problem with storage cfg. User must be set. Check $STORAGE_CFG line: $line"
          continue
        fi

        if [ "$NA_PASSWD"x = "x" ]; then
          echo "$STORAGE_NAME: Some problem with storage cfg. Password must be set. Check $STORAGE_CFG line: $line"
          continue
        fi

        if [ $NA_PORT_API -eq 80 -a $NA_PROTO_API -eq 1 ]; then
          echo "$STORAGE_NAME: Some problem with storage cfg. API port is 80 (http) and API protocol is https (1). Check $STORAGE_CFG line: $line"
          continue
        fi

        if [ $NA_PORT_API -eq 443 -a $NA_PROTO_API -eq 0 ]; then
          echo "$STORAGE_NAME: Some problem with storage cfg. API port is 443 (https) and API protocol is http (0). Check $STORAGE_CFG line: $line"
          continue
        fi

        # connection test
        perl bin/conntest.pl $NA_IP $NA_PORT_SSH
        perl bin/conntest.pl $NA_IP $NA_PORT_API

        if [ "$NA_PASSWD" = "IT5gYGAK" -o "$NA_PASSWD" = "IT5gYGAK/KT4mXVI9N0BUPjZdVQo=" ]; then
          echo "Error: you have not changed default password hash, use \"perl ./bin/spasswd.pl\" to get a new hash and place it into storage line in etc/storage-list.cfg"
          echo "  $line"
          echo "$STORAGE_NAME nok" >> $STATUS_FILE
          continue
        fi

        echo "  $SSH -p $NA_PORT_SSH $NA_USER@$NA_IP  \"?\""
        $SSH -p $NA_PORT_SSH $NA_USER@$NA_IP  "?" >/dev/null
        if [ $? -eq 0 ]; then
          echo "  SSH connection OK"
          echo "  Testing API connection..."
          $PERL $BINDIR/na_apitest.pl
          if [ $? -eq 0 ]; then
            echo "  API connection OK"
            echo "$STORAGE_NAME  ok" >> $STATUS_FILE
          else 
            echo "  API connection failed!!!"
            lwp=`$PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'| sed -e 's/LWP Version: //' -e 's/\..*$//'`
            if [  $lwp -lt 6 ]; then
              echo "  Perl LWP module is too old, it might be a problem"
              echo "  Upgrade it, refer here: http://www.stor2rrd.com/https.htm"
            fi
            echo "$STORAGE_NAME nok" >> $STATUS_FILE
          fi
        else 
          echo "  SSH connection failed!!!"
          echo "$STORAGE_NAME nok" >> $STATUS_FILE
        fi
        echo ""
        (( count = count + 1 ))
done
echo ""


# 3PAR

for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":3PAR:"|egrep -v "^#"`
do
        # Name:SWIZ:DEVID:HMC1:HMC2:
        STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
        STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`

        if [ ! "$storage"x = "x" ]; then
          if [ ! "$storage" = "$STORAGE_NAME" ]; then
            continue
          fi
        fi 

        echo "========================="
        echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
        echo "========================="

        # 3PAR/Storwize Storage Family
        PAR_IP=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
        PAR_USER=$STORAGE_USER
        if [ ! x"$STORAGE_USER_3PAR" = "x" ]; then
          PAR_USER=$STORAGE_USER_3PAR
        fi
        PAR_KEY_tmp=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
        if [ -f "$PAR_KEY_tmp" ]; then
          PAR_KEY="$PAR_KEY_tmp" # go ahead only if this file exists
        fi

        PAR_DIR="$INPUTDIR/data/$STORAGE_NAME"
        PAR_INTERVAL=`expr $SAMPLE_RATE / 60`

        if [ "$PAR_IP"x = "x" ]; then
                echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
                echo "$STORAGE_NAME nok" >> $STATUS_FILE
                continue
        fi

        perl bin/conntest.pl  $PAR_IP 22

        if [ -f "$HPE3PAR_CLIDIR/cli" ]; then
          $HPE3PAR_CLIDIR/cli -v
        fi

        tpm_3par_file="$INPUTDIR/tmp/$PAR_IP.tmp-$$"
        PWF="$INPUTDIR/etc/.3par-$PAR_IP.pwf"
        if [ -f "$HPE3PAR_CLIDIR/cli" -a ! -f "$PWF" ]; then
          echo ""
          echo "3PAR CLI is installed but credentials are not saved!"
          echo "To save credential starting: ./bin/3par-credential-save.sh $STORAGE_NAME"
          export STOR2RRD_NO_HEADER=1
          $INPUTDIR/bin/3par-credential-save.sh $STORAGE_NAME
        fi
        if [ -f "$HPE3PAR_CLIDIR/cli" -a -f "$PWF" ]; then
          perl bin/conntest.pl  $PAR_IP 5782
          TPDCERTDIR="$INPUTDIR/etc"
          export TPDCERTDIR
          echo "Using 3PAR CLI $HPE3PAR_CLIDIR/cli, PWF:$PWF"
          echo "export TPDPWFILE="$PWF"; $HPE3PAR_CLIDIR/cli -sys $PAR_IP -pwf $PWF showuser"
          export TPDPWFILE="$PWF"; $HPE3PAR_CLIDIR/cli -sys $PAR_IP showuser > $tpm_3par_file
          if [ ! $? -eq 0 -o ! -f $tpm_3par_file ]; then
            echo "  connection nok"
            echo "$STORAGE_NAME nok" >> $STATUS_FILE
            rm -f $tpm_3par_file
            continue
          fi
          egrep -v "^Username" $tpm_3par_file| grep browse >/dev/null
	  if [ $? -eq 0 ]; then
            echo "  connection ok"
            echo "$STORAGE_NAME  ok" >> $STATUS_FILE
          else
            echo "  current account does not have right access role !!"
            echo "$STORAGE_USER user on all 3PAR should have \"browse\" role and access to all domains"
            echo "  connection failed!!"
            echo "$STORAGE_NAME nok" >> $STATUS_FILE
          fi
        else
          echo "Using ssh, 3PAR CLI is not either installed or not configured properly in etc/stor2rrd.cfg (HPE3PAR_CLIDIR)"
          if [ "$PAR_KEY"x = "x" ]; then
            echo "  $SSH $STORAGE_USER@$PAR_IP  \"showuser\""
            $SSH $STORAGE_USER@$PAR_IP  "showuser" >/dev/null
	    if [ $? -eq 0 ]; then
              $SSH $STORAGE_USER@$PAR_IP  "showuser" >$tpm_3par_file
              if [ ! $? -eq 0 -o ! -f $tpm_3par_file ]; then
                echo "  connection nok"
                echo "$STORAGE_NAME nok" >> $STATUS_FILE
                rm -f $tpm_3par_file
                continue
              fi
              egrep -v "^Username" $tpm_3par_file| grep browse >/dev/null
	      if [ $? -eq 0 ]; then
                echo "  connection ok"          
                echo "$STORAGE_NAME  ok" >> $STATUS_FILE
              else 
                echo "  current account does not have right access role !!"          
                echo "$STORAGE_USER user on all 3PAR should have \"browse\" role and access to all domains"
                $SSH $STORAGE_USER@$PAR_IP  "showuser"
                echo "  connection failed!!"          
                echo "$STORAGE_NAME nok" >> $STATUS_FILE
              fi
            else 
              echo "  connection failed!!"          
              echo "$STORAGE_NAME nok" >> $STATUS_FILE
            fi
          else
            echo "  $SSH -i $PAR_KEY  $STORAGE_USER@$PAR_IP  \"showuser\""
            $SSH -i $PAR_KEY  $STORAGE_USER@$PAR_IP  "showuser" >/dev/null
	    if [ $? -eq 0 ]; then
              $SSH -i $PAR_KEY  $STORAGE_USER@$PAR_IP  "showuser" >$tpm_3par_file
              if [ ! $? -eq 0 -o ! -f $tpm_3par_file ]; then
                echo "  connection nok"
                echo "$STORAGE_NAME nok" >> $STATUS_FILE
                rm -f $tpm_3par_file
                continue
              fi
	      if [ $? -eq 0 ]; then
                echo "  connection ok"          
                echo "$STORAGE_NAME  ok" >> $STATUS_FILE
              else 
                echo "  current account does not have Administration access!!"          
                echo "$STORAGE_USER user on all 3PAR should have \"browse\" role and access to all domains"
                $SSH -i $PAR_KEY  $STORAGE_USER@$PAR_IP  "showuser"
                echo "  connection failed!!"          
                echo "$STORAGE_NAME nok" >> $STATUS_FILE
              fi
            else 
              echo "  connection failed!!"          
              echo "$STORAGE_NAME nok" >> $STATUS_FILE
            fi
          fi
        fi
        rm -f $tpm_3par_file
        echo ""
done

### VNXF

for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":VNXF:"|egrep -v "^#"`
do
  # Storage Alias:VNXF:IP address:USER:ssh_key_file:VOLUME_AGG_DATA_LIM:VOLUME_AGG_IO_LIM:SAMPLE_RATE_MINS
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  VNXF_IP=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  VNXF_USER=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  VNXF_KEY_tmp=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi 

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  if [ "$VNXF_IP"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$VNXF_USER"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage USER must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  if [ -f "$VNXF_KEY_tmp" ]; then
    VNXF_KEY="$VNXF_KEY" # go ahead only if this file exists
  fi

  # connection test
  perl bin/conntest.pl $VNXF_IP 22

  if [ "$VNXF_KEY"x = "x" ]; then
    echo "  $SSH $VNXF_USER@$VNXF_IP  \"server_name ALL\""
    $SSH $VNXF_USER@$VNXF_IP 'export NAS_DB=/nas; /nas/bin/server_name ALL' >/dev/null
    if [ $? -eq 0 ]; then
      echo "  connection ok"
      echo "$STORAGE_NAME  ok" >> $STATUS_FILE
    else
      echo "  connection failed!!"
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
    fi
  else
    echo "  $SSH -i $VNXF_KEY $VNXF_USER@$VNXF_IP  \"server_name ALL\""
    $SSH -i $VNXF_KEY  $VNXF_USER@$VNXF_IP 'export NAS_DB=/nas; /nas/bin/server_name ALL' >/dev/null
    if [ $? -eq 0 ]; then
      echo "  connection ok"
      echo "$STORAGE_NAME  ok" >> $STATUS_FILE
    else
      echo "  connection failed!!"
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
    fi
  fi
  echo ""
done


# PURE

for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":PURE:"|egrep -v "^#"`
do

  #Storage name alias:storage_type:compellent_ip_a:user:passwd:VOLUME_AGG_DATA_LIM:VOLUME_AGG_IO_LIM:SAMPLE_RATE_MINS
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  PURE_IP_SPA=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  PURE_USER=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  PURE_PW=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  if [ "$STORAGE_NAME"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage NAME must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$PURE_IP_SPA"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$PURE_USER"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage USER must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$PURE_PW"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage PASSWORD must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  if [ "$PURE_PW" = "IT5gYGAK" -o "$PURE_PW" = "IT5gYGAK/KT4mXVI9N0BUPjZdVQo=" ]; then
    echo "Error: you have not changed default password hash, use \"perl ./bin/spasswd.pl\" to get a new hash and place it into storage line in etc/storage-list.cfg"
    echo "  $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$PURE_PW'")'`
  if [ ! $? -eq 0  ]; then
    echo "  PURE Error: password cannot be decoded"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  PORT=443

  # connection test
  perl bin/conntest.pl $PURE_IP_SPA $PORT

  # modules test
  $PERL  $BINDIR/perl_https_check.pl
  if [ $? -ne 0  ]; then
    echo "  connection failed!!"
    echo ""
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  echo "$PERL $BINDIR/pure_apitest.pl $PURE_USER XXXXX $PURE_IP_SPA"
  $PERL $BINDIR/pure_apitest.pl $PURE_USER $encode_pw $PURE_IP_SPA $STORAGE_NAME
  if [ $? -eq 0  ]; then
    echo "  connection ok"
    echo ""
    echo ""
    echo "$STORAGE_NAME ok" >> $STATUS_FILE
  else
    echo "  connection failed!!"
    lwp=`$PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'| sed -e 's/LWP Version: //' -e 's/\..*$//'`
    if [  $lwp -lt 6 ]; then
      echo "  Perl LWP module is too old, it might be a problem"
      echo "  Upgrade it, refer here: http://www.stor2rrd.com/https.htm"
    fi
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
  fi

done


# COMPELLENT


for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":COMPELLENT:"|egrep -v "^#"|sed 's/ /===space===/g'`
do


  #Storage name alias:storage_type:compellent_ip_a:port:user:passwd:VOLUME_AGG_DATA_LIM:VOLUME_AGG_IO_LIM:SAMPLE_RATE_MINS
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'| sed 's/===space===/ /g'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}' | sed 's/===space===/ /g'`
  COMPELLENT_IP_SPA=`echo $line | awk 'BEGIN{FS=":"}{print $3}' | sed 's/===space===/ /g'`
  COMPELLENT_PORT=`echo $line | awk 'BEGIN{FS=":"}{print $4}' | sed 's/===space===/ /g'`
  COMPELLENT_USER=`echo $line | awk 'BEGIN{FS=":"}{print $5}' | sed 's/===space===/ /g'`
  COMPELLENT_PW=`echo $line | awk 'BEGIN{FS=":"}{print $6}' | sed 's/===space===/ /g'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  if [ "$STORAGE_NAME"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage NAME must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$COMPELLENT_IP_SPA"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$COMPELLENT_USER"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage USER must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$COMPELLENT_PW"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage PASSWORD must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$COMPELLENT_PORT"x = "x" ]; then
    COMPELLENT_PORT=3033
  fi

  if [ "$COMPELLENT_PW" = "IT5gYGAK" -o "$COMPELLENT_PW" = "IT5gYGAK/KT4mXVI9N0BUPjZdVQo=" ]; then
    echo "Error: you have not changed default password hash, use \"perl ./bin/spasswd.pl\" to get a new hash and place it into storage line in etc/storage-list.cfg"
    echo "  $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$COMPELLENT_PW'")'`
  if [ ! $? -eq 0  ]; then
    echo "  COMPELLENT Error: password cannot be decoded"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  # connection test
  perl bin/conntest.pl $COMPELLENT_IP_SPA $COMPELLENT_PORT

  # modules test
  $PERL  $BINDIR/perl_https_check.pl
  if [ $? -ne 0  ]; then
    echo "  connection failed!!"
    echo ""
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  echo "$PERL $BINDIR/compellent_apitest.pl \"$COMPELLENT_USER\" XXXXX \"$COMPELLENT_IP_SPA\" \"$COMPELLENT_PORT\""
  $PERL $BINDIR/compellent_apitest.pl "$COMPELLENT_USER" "$encode_pw" "$COMPELLENT_IP_SPA" "$STORAGE_NAME" "$COMPELLENT_PORT"
  if [ $? -eq 0  ]; then
    echo "  connection ok"
    echo ""
    echo ""
    echo "$STORAGE_NAME ok" >> $STATUS_FILE
  else
    echo "  connection failed!!"  
    lwp=`$PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'| sed -e 's/LWP Version: //' -e 's/\..*$//'`
    if [  $lwp -lt 6 ]; then
      echo "  Perl LWP module is too old, it might be a problem"
      echo "  Upgrade it, refer here: http://www.stor2rrd.com/https.htm"
    fi
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
  fi

done

# OCEANSTOR

for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":OCEANSTOR:"|egrep -v "^#"`
do

  #Storage name alias:storage_type:oceanstor_ip_a:oceanstor_ip_b:user:passwd:VOLUME_AGG_DATA_LIM:VOLUME_AGG_IO_LIM:SAMPLE_RATE_MINS
  #Huawei-Oceanstor01:OCEANSTOR:172.31.33.126:172.31.33.127:stor2rrd:KTI0KS0tJ0VPPTRgTgo=:::
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  OCEANSTOR_IP_SPA=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  OCEANSTOR_IP_SPB=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  OCEANSTOR_USER=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
  OCEANSTOR_PW=`echo $line | awk 'BEGIN{FS=":"}{print $6}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  if [ "$STORAGE_NAME"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage NAME must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$OCEANSTOR_IP_SPA"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$OCEANSTOR_IP_SPB"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$OCEANSTOR_USER"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage USER must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$OCEANSTOR_PW"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage PASSWORD must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$OCEANSTOR_PW" = "IT5gYGAK" -o "$OCEANSTOR_PW" = "IT5gYGAK/KT4mXVI9N0BUPjZdVQo=" ]; then
    echo "Error: you have not changed default password hash, use \"perl ./bin/spasswd.pl\" to get a new hash and place it into storage line in etc/storage-list.cfg"
    echo "  $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$OCEANSTOR_PW'")'`
  if [ ! $? -eq 0  ]; then
    echo "  OCEANSTOR Error: password cannot be decoded"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  PORT=8088

  # connection test
  perl bin/conntest.pl $OCEANSTOR_IP_SPA $PORT
  perl bin/conntest.pl $OCEANSTOR_IP_SPB $PORT

  # modules test
  $PERL  $BINDIR/perl_https_check.pl
  if [ $? -ne 0  ]; then
    echo "  connection failed!!"
    echo ""
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  echo "$PERL $BINDIR/oceanstor_apitest.pl $OCEANSTOR_USER XXXXX $OCEANSTOR_IP_SPA $OCEANSTOR_IP_SPB $STORAGE_NAME"
  echo ""
  $PERL  $BINDIR/oceanstor_apitest.pl $OCEANSTOR_USER $encode_pw $OCEANSTOR_IP_SPA $OCEANSTOR_IP_SPB $STORAGE_NAME
  if [ $? -eq 0  ]; then
    echo "  connection ok"
    echo ""
    echo ""
    echo "$STORAGE_NAME ok" >> $STATUS_FILE
  else
    echo "  connection failed!!"
    lwp=`$PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'| sed -e 's/LWP Version: //' -e 's/\..*$//'`
    if [  $lwp -lt 6 ]; then
      echo "  Perl LWP module is too old, it might be a problem"
      echo "  Upgrade it, refer here: http://www.stor2rrd.com/https.htm"
    fi
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
  fi

done


# DOTHILL

for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":DOTHILL:"|egrep -v "^#"`
do

  #Storage name alias:storage_type:dothill_ip_a:dothill_ip_b:HTTPS:user:passwd:VOLUME_AGG_DATA_LIM:VOLUME_AGG_IO_LIM:SAMPLE_RATE_MINS
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  DOTHILL_IP_SPA=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  DOTHILL_IP_SPB=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  DOTHILL_PROTOCOL=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
  DOTHILL_USER=`echo $line | awk 'BEGIN{FS=":"}{print $6}'`
  DOTHILL_PW=`echo $line | awk 'BEGIN{FS=":"}{print $7}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi 

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  if [ "$STORAGE_NAME"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage NAME must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$DOTHILL_PROTOCOL"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage protocol must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  if [ "$DOTHILL_IP_SPA"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP SPA must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$DOTHILL_IP_SPB"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP SPB must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$DOTHILL_USER"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage USER must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$DOTHILL_PW"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage PASSWORD must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  WBEMCLI="/usr/bin/wbemcli"
  if [ ! -f $WBEMCLI ]; then
    WBEMCLI="/opt/freeware/bin/wbemcli"
    if [ ! -f $WBEMCLI ]; then
      WBEMCLI="/usr/local/bin/wbemcli"
      if [ ! -f $WBEMCLI ]; then
        echo "DOTHILL Error: could not found wbemcli binnary, searched /usr/bin/wbemcli, /opt/freeware/bin/wbemcli, /usr/local/bin/wbemcli"
        echo "$STORAGE_NAME nok" >> $STATUS_FILE
        continue
      fi
    fi
  fi

  if [ "$DOTHILL_PW" = "IT5gYGAK" -o "$DOTHILL_PW" = "IT5gYGAK/KT4mXVI9N0BUPjZdVQo=" ]; then
    echo "Error: you have not changed default password hash, use \"perl ./bin/spasswd.pl\" to get a new hash and place it into storage line in etc/storage-list.cfg"
    echo "  $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi


  encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$DOTHILL_PW'")'`
  if [ ! $? -eq 0  ]; then
    echo "  DOTHILL Error: password cannot be decoded"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  PORT=5988
  DOTHILL_PROTOCOL_NO=80
  if [ "$DOTHILL_PROTOCOL" = "https" -o "$DOTHILL_PROTOCOL" = "HTTPS" ]; then
    PORT=5989
    DOTHILL_PROTOCOL_NO=443
  fi

  # connection test
  perl bin/conntest.pl $DOTHILL_IP_SPA $PORT
  perl bin/conntest.pl $DOTHILL_IP_SPA $DOTHILL_PROTOCOL_NO
  perl bin/conntest.pl $DOTHILL_IP_SPB $PORT
  perl bin/conntest.pl $DOTHILL_IP_SPB $DOTHILL_PROTOCOL_NO

  #types=( dhs hpq smis )
  types[0]="dhs"
  types[1]="hpq"
  types[2]="smis"
  ips[0]=$DOTHILL_IP_SPA
  ips[1]=$DOTHILL_IP_SPB
  #ips=( $DOTHILL_IP_SPA $DOTHILL_IP_SPB )
  check=0
  TMP_FILE_DOTHILL="$INPUTDIR/tmp/dothill_tmp.$$"
  for type_element in "${types[@]}"
  do
    for ip_element in "${ips[@]}"
    do
      if [ $check -eq 0 -o $check -eq 3 ]; then
        echo "$WBEMCLI -noverify -nl ei $DOTHILL_PROTOCOL://$DOTHILL_USER:XXXXX@$ip_element:$PORT/root/$type_element:cim_computersystem"
        OUT_CMD=`$WBEMCLI -noverify -nl ei $DOTHILL_PROTOCOL://$DOTHILL_USER:$encode_pw@$ip_element:$PORT/root/$type_element:cim_computersystem 2>>$TMP_FILE_DOTHILL`
      else 
        true >/dev/null # to get $? == 0
      fi
      if [ $? -eq 0  -a ! "$OUT_CMD"x = "x" ]; then
        echo "SMI-S API is ok, going to test Web-browser interface (WBE) through $DOTHILL_PROTOCOL"
        echo "$PERL $BINDIR/dothill_apitest.pl $DOTHILL_USER XXXXX $ip_element $DOTHILL_PROTOCOL"
        $PERL $BINDIR/dothill_apitest.pl $DOTHILL_USER $encode_pw $ip_element $DOTHILL_PROTOCOL
        if [ ! $? -eq 0  ]; then
          check=1
          break   
        else
          echo "WBE via $DOTHILL_PROTOCOL is ok"
          check=2
          break   
        fi
      else
       check=3
      fi
    done
    if [ $check -eq 2 ]; then
      echo "  connection ok"
      echo ""
      echo ""
      echo "$STORAGE_NAME  ok" >> $STATUS_FILE
      break    
    fi
  done
  if [ ! $check -eq 2 ]; then
    if [ -f "$TMP_FILE_DOTHILL" -a $check -eq 3 ]; then 
      cat $TMP_FILE_DOTHILL
      echo ""
    fi
    echo "  connection failed!!"
    echo ""
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
  fi
  rm -f TMP_FILE_DOTHILL
done
echo ""



# HITACHI VSPG

for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":VSPG:"|egrep -v "^#"`
do

  #Storage name alias:VSPG:serial_number:ip_adress:user:user_password:horcm.conf
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  VSPG_IP=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  VSPG_USER=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  VSPG_PW=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
  VSPG_CONF=`echo $line | awk 'BEGIN{FS=":"}{print $6}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi 

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  if [ "$VSPG_IP"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage IP must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$VSPG_USER"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage USER must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$VSPG_PW"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage PASSWORD must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$VSPG_CONF"x = "x" ]; then
    echo "$STORAGE_NAME: Some problem with storage cfg. Storage horcm.conf  must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  #export STORAGE_NAME STORAGE_TYPE VSPG_IP VSPG_DEVID VSPG_USER VSPG_PW

  #ERRLOG_VSPG="$ERRLOG-${STORAGE_NAME}"
  #OUTLOG_VSPG="$INPUTDIR/logs/${STORAGE_NAME}.output_vspg.log"
  #export ERRLOG_VSPG OUTLOG_VSPG

  if [ x"$VSP_CLIDIR" = "x" ]; then
    echo "VSPG_CLIDIR does not seem to be set up, correct it in stor2rrd.cfg"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  if [ "$VSPG_PW" = "IT5gYGAK" -o "$VSPG_PW" = "IT5gYGAK/KT4mXVI9N0BUPjZdVQo=" ]; then
    echo "Error: you have not changed default password hash, use \"perl ./bin/spasswd.pl\" to get a new hash and place it into storage line in etc/storage-list.cfg"
    echo "  $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$VSPG_PW'")'`
  if [ ! $? -eq 0  ]; then
    echo "VSPG Error: password cannot be decoded"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi


  #if [ `raidqry -h 2>/dev/null| wc -l|sed 's/ //g'` -gt 2 ]; then
  #CONF_EXIST=`ls -l "$VSPG_CONF" 2>/dev/null | wc -l`
  #if [ $CONF_EXIST -eq 0 ]; then
  #  echo "file $VSPG_CONF does not exist, correct it in storage-list.cfg"
  #  echo "  connection failed!!"
  #  continue
  #fi
  if [ ! -f "$VSPG_CONF" ]; then
    echo "VSPG Error: could not found file : "$VSPG_CONF". Install Hitachi CCI or correct it in etc/storage-list.cfg"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  ID_ARRAY=`echo "$VSPG_CONF" | sed 's/.*horcm//g' | sed 's/.conf$//g'`
  #PROC_HORM=`ps -ef | grep "horcmd_"`
  #CHECK_PROC=`echo $PROC_HORM | egrep "horcmd_$ID_ARRAY" | wc -l`
  CHECK_PROC=`ps -ef | grep "horcmd_" |  egrep "horcmd_$ID_ARRAY$" | wc -l`

  if [ $CHECK_PROC -eq 0 ]; then
    #CHECK_PROC=`echo $PROC_HORM | egrep "horcmd_0$ID_ARRAY" | wc -l`
    CHECK_PROC=`ps -ef | grep "horcmd_" |  egrep "horcmd_0$ID_ARRAY$" | wc -l`
    if [ $CHECK_PROC -eq 0 ]; then
     echo "VSPG Error: process "horcmd_$ID_ARRAY" or "horcmd_0$ID_ARRAY" is not running"
     echo "  connection failed!!"
     echo "$STORAGE_NAME nok" >> $STATUS_FILE
     continue
    fi
  fi
  AUTENTIZATION="-I$ID_ARRAY"
  raidcom -login $VSPG_USER $encode_pw $AUTENTIZATION 2>/dev/null
  if [ ! $? -eq 0 ]; then
    echo "VSPG Error: command failed: raidcom -login $VSPG_USER $encode_pw $AUTENTIZATION"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  FILE_FIRMWARE=`raidqry -l $AUTENTIZATION 2>/dev/null | egrep -v "Serial#"`
  if [ ! $? -eq 0 ]; then
    echo "VSPG Error: command failed: raidqry -l $AUTENTIZATION"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  FIRMWARE_SERIAL=`echo $FILE_FIRMWARE | awk '{print $7}' | sed 's/\/.*//g'`
  serial_number=`echo $FILE_FIRMWARE | awk '{print $6}'`
  if [ ! -d "$VSP_CLIDIR/$FIRMWARE_SERIAL" ]; then
    echo "VSPG Error: directory "$VSP_CLIDIR/$FIRMWARE_SERIAL" does not exist. Install Export Tool for this storage into directory "$VSP_CLIDIR/$FIRMWARE_SERIAL" "
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  $BINDIR/hds_check.sh $VSPG_USER $encode_pw $VSPG_IP $serial_number $AUTENTIZATION $FIRMWARE_SERIAL
  if [ ! $? -eq 0 ]; then
    #echo "VSPG Error: set up or check HDS_EXPORT_TOOL_JAVAPATH in stor2rrd.cfg"
    echo "  connection failed!!"
    raidcom -logout $AUTENTIZATION 2>/dev/null
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    if [ ! $? -eq 0 ]; then
      echo "VSPG Error: command failed: raidcom -logout $AUTENTIZATION"
      echo "  connection failed!!"
      continue
    fi
    continue
  fi
  raidcom -logout $AUTENTIZATION 2>/dev/null
  if [ ! $? -eq 0 ]; then
    echo "VSPG Error: command failed: raidcom -logout $AUTENTIZATION"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  echo "  connection ok"
  echo "$STORAGE_NAME  ok" >> $STATUS_FILE
  echo ""
done
echo ""

# EMC VNX block
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":VNX:"|egrep -v "^#"`
do
  # Storage name alias:EMCVNXB:SPA IP:SPB IP
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  IP_SPA=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  IP_SPB=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`

  export STORAGE_NAME STORAGE_TYPE

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi 

  VNXB_DIR="$INPUTDIR/data/$STORAGE_NAME"
  VNXB_INTERVAL=$SAMPLE_RATE
  export IP_SPA IP_SPB VNXB_DIR VNX_INTERVAL

  ERRLOG_VNXB="$ERRLOG-${STORAGE_NAME}"
  OUTLOG_VNXB="$INPUTDIR/logs/${STORAGE_NAME}.output_vnxb.log"
  export ERRLOG_VNXB OUTLOG_VNXB

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  # test naviseccli paths
  if [ -f "$VNX_BLOCK_CLIDIR/naviseccli" ]; then
    NAVISECCLI="$VNX_BLOCK_CLIDIR/naviseccli"
  fi

  if [ "$NAVISECCLI"x = "x" -a -f /opt/Navisphere/bin/naviseccli ]; then
    NAVISECCLI="/opt/Navisphere/bin/naviseccli" # Linux
  fi

  if [ "$NAVISECCLI"x = "x" -a -f /usr/bin/naviseccli ]; then
    NAVISECCLI="/usr/bin/naviseccli" # AIX
  fi


  if [ "$NAVISECCLI"x = "x" ]; then
    echo "VNXB Error: could not found naviseccli binnary! Install naviseccli or update VNX_BLOCK_CLIDIR in etc/stor2rrd.cfg"
    echo "Not exist /opt/Navisphere/bin/naviseccli"
    echo "Not exist /usr/bin/naviseccli"
    echo "Not exist $VNX_BLOCK_CLIDIR"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  SEC_FILE_PATH=""
  if [ -f "$INPUTDIR/etc/SecuredCLISecurityFile.xml" -a -f "$INPUTDIR/etc/SecuredCLIXMLEncrypted.key" ]; then
    SEC_FILE_PATH="-secfilepath $INPUTDIR/etc"
  fi

  # connection test
  perl bin/conntest.pl $IP_SPA 443 WARNING
  perl bin/conntest.pl $IP_SPA 6389 WARNING

  # SPA IP
  if [ "$IP_SPA"x = "x" ]; then
    echo "Not defined first IP address!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
    echo ""
  else
    echo "first IP address : $IP_SPA"
    echo "$NAVISECCLI -h $IP_SPA $SEC_FILE_PATH getcontrol"
    CMD_OUT=`$NAVISECCLI -h $IP_SPA $SEC_FILE_PATH getcontrol | egrep "System Fault LED:|Statistics Logging:|System Date:|Day of the week:|System Time:"`
    echo "$CMD_OUT"
    echo ""

    if [ `echo $CMD_OUT | grep "Statistics Logging:"|wc -l` -gt 0 ]; then
      echo "  connection ok"
      echo "$STORAGE_NAME SPA  ok" >> $STATUS_FILE
      echo ""
      if [ `echo $CMD_OUT | egrep "Statistics Logging:\s*OFF"|wc -l` -gt 0 ]; then
        echo "  Statistics Logging is OFF."
        echo "  Allow statistics logging on the storage."
        echo "  GUI -> Storage -> System properties -> Statistics loging -> yes"
        echo ""
        echo ""
      fi
    else
      echo "  connection failed!!"
      echo "$STORAGE_NAME SPA nok" >> $STATUS_FILE
      echo ""
    fi
  fi

  # SPB IP
  if [ "$IP_SPB"x = "x" ]; then
    echo "Not defined second IP address!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    echo ""
    continue
  else
    echo "second IP address : $IP_SPB"
    echo "$NAVISECCLI -h $IP_SPB $SEC_FILE_PATH getcontrol"
    CMD_OUT=`$NAVISECCLI -h $IP_SPB $SEC_FILE_PATH getcontrol | egrep "System Fault LED:|Statistics Logging:|System Date:|Day of the week:|System Time:"`
    echo "$CMD_OUT"
    echo ""

    if [ `echo $CMD_OUT | grep "Statistics Logging:"|wc -l` -gt 0 ]; then
      echo "  connection ok"
      echo "$STORAGE_NAME SPB  ok" >> $STATUS_FILE
      echo ""
      if [ `echo $CMD_OUT | egrep "Statistics Logging:\s*OFF"|wc -l` -gt 0 ]; then
        echo "  Statistics Logging is OFF."
        echo "  Allow statistics logging on the storage."
        echo "  GUI -> Storage -> System properties -> Statistics loging -> yes"
        echo ""
        echo ""
      fi
    else
      echo "  connection failed!!"
      echo ""
      echo "$STORAGE_NAME SPB nok" >> $STATUS_FILE
    fi
  fi

done
echo ""

# EMC Unity
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":UNITY:"|egrep -v "^#"`
do
  # Storage Alias:UNITY:IP address:user:password
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  IP=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  UNITY_USER=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  UNITY_PW=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi 

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$UNITY_PW'")'`
  if [ ! $? -eq 0  ]; then
    echo "Error: password cannot be decoded"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  # test uemcli paths
  UEMCLI=""
  if [ -f "$UNITY_CLIDIR/uemcli" ]; then
    UEMCLI="$UNITY_CLIDIR/uemcli"
  else
    UEMCLI="uemcli"
  fi

  if [ "$IP"x = "x" ]; then
    echo "Not defined IP address!"
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  else
    # connection test
    perl bin/conntest.pl $IP 443 WARNING

    echo "$UEMCLI -d $IP -u $UNITY_USER -p XXX -noHeader /sys/general show -detail"
    CMD_OUT=`$UEMCLI -d $IP -u $UNITY_USER -p $encode_pw -noHeader /sys/general show -detail`
    echo "$CMD_OUT"
    echo ""

    if [ `echo $CMD_OUT | grep "Model"|wc -l` -gt 0 ]; then
      echo "  connection ok"
      echo ""
      echo "$STORAGE_NAME  ok" >> $STATUS_FILE
    else
      echo "  connection failed!!"
      echo ""
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
    fi
  fi
done
echo ""


# EMC VNXe
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":VNXE:"|egrep -v "^#"`
do
  # Storage Alias:UNITY:IP address:user:password
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  IP=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  VNXE_USER=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  VNXE_PW=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi 

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$VNXE_PW'")'`
  if [ ! $? -eq 0  ]; then
    echo "Error: password cannot be decoded"
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  # test uemcli paths
  UEMCLI=""
  if [ -f "$VNXE_CLIDIR/uemcli" ]; then
    UEMCLI="$VNXE_CLIDIR/uemcli"
  else
    UEMCLI="uemcli"
  fi

  if [ "$IP"x = "x" ]; then
    echo "Not defined IP address!"
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  else
    # connection test
    perl bin/conntest.pl $IP 443 WARNING

    echo "$UEMCLI -d $IP -u $VNXE_USER -p XXX -noHeader /sys/general show -detail"
    CMD_OUT=`$UEMCLI -d $IP -u $VNXE_USER -p $encode_pw -noHeader /sys/general show -detail`
    echo "$CMD_OUT"
    echo ""

    if [ `echo $CMD_OUT | grep "Model"|wc -l` -gt 0 ]; then
      echo "  connection ok"
      echo ""
      echo "$STORAGE_NAME  ok" >> $STATUS_FILE
    else
      echo "  connection failed!!"
      echo ""
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
    fi
  fi
done
echo ""


# EMC VMAX
VMAX_ID_PREV=""
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":VMAX:"|egrep -v "^#"|sort -t : -k 3`
do
  # Storage Alias:VMAX:symmetrix_id:univmax_ip:univmax_port:user:pass
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  VMAX_ID=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  VMAX_IP=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  VMAX_PORT=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
  VMAX_USER=`echo $line | awk 'BEGIN{FS=":"}{print $6}'`
  VMAX_PW=`echo $line | awk 'BEGIN{FS=":"}{print $7}'`


  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi

  if [ ! "$VMAX_ID_PREV"x = "x" ]; then
    if [ "$VMAX_ID_PREV" = "$VMAX_ID" ]; then
      echo "Exluding this line from cfg as Symmetrix ID is duplicated"
      echo "   $line"
      continue
    fi
  fi

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  #encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$VMAX_PW'")'`
  #if [ ! $? -eq 0  ]; then
  #  echo "Error: password cannot be decoded"
  #  echo "  connection failed!!"
  #  continue
  #fi

  export STORAGE_NAME STORAGE_TYPE VMAX_ID VMAX_IP VMAX_PORT VMAX_USER VMAX_PW

  # connection test
  perl bin/conntest.pl $VMAX_IP 8443 

  # modules test
  $PERL  $BINDIR/perl_https_check.pl
  if [ $? -ne 0  ]; then
    echo "  connection failed!!"
    echo ""
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  VMAX_TMP_OUT=$INPUTDIR/tmp/vmax_tmp_out-$$
  rm -f $VMAX_TMP_OUT
  $PERL $BINDIR/vmax_restapi_test.pl  1>$VMAX_TMP_OUT 2>&1
  ret=$?
  cat $VMAX_TMP_OUT
  if [ ! $ret -eq 0 ]; then
    if [ `grep "Connection refused" $VMAX_TMP_OUT| wc -l` -gt 0 ]; then
      echo ""
      echo "Looks like a network issue, check if Unispehere \"$VMAX_IP\" is reachable via port \"$VMAX_PORT\""
      echo "telnet $VMAX_IP $VMAX_PORT"
      echo ""
    fi
    echo "  connection failed!!"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
  else 
    echo "  connection ok"
    echo "$STORAGE_NAME  ok" >> $STATUS_FILE
  fi
  rm -f $VMAX_TMP_OUT
  echo ""
  VMAX_ID_PREV=$VMAX_ID
done
echo ""


# EMC VPLEX
TMP_FILE_VPLEX="$INPUTDIR/tmp/vplex_tmp.$$"
TMP_FILE_VPLEX_ERR="$INPUTDIR/tmp/vplex_tmp_err.$$"
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":VPLEX:"|egrep -v "^#"|sort -t : -k 3`
do
  # Storage Alias:VPLEX:Cluster name:Mgmt server IP or hostname:Mgmt server ssh port:use API [1|0] ([yes|no]):Unisphere/API IP or hostname:API port:API user name:API encrypted password

  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
  # EMC VPLEX
  EMC_VPLEX_CLUSTER_ID=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  EMC_VPLEX_SSH_IP=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
  EMC_VPLEX_SSH_PORT=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
  EMC_VPLEX_SSH_USER=`echo $line | awk 'BEGIN{FS=":"}{print $6}'`
  EMC_VPLEX_API=`echo $line | awk 'BEGIN{FS=":"}{print $7}'`
  EMC_VPLEX_API_IP=`echo $line | awk 'BEGIN{FS=":"}{print $8}'`
  EMC_VPLEX_API_PORT=`echo $line | awk 'BEGIN{FS=":"}{print $9}'`
  EMC_VPLEX_USER=`echo $line | awk 'BEGIN{FS=":"}{print $10}'`
  EMC_VPLEX_PASSWD=`echo $line | awk 'BEGIN{FS=":"}{print $11}'`

  if [ ! "$storage"x = "x" ]; then
    if [ ! "$storage" = "$STORAGE_NAME" ]; then
      continue
    fi
  fi

  echo "========================="
  echo "STORAGE: $STORAGE_NAME: $STORAGE_TYPE"
  echo "========================="

  # storage-list.cfg variables
  if [ "$EMC_VPLEX_CLUSTER_ID"x = "x" ]; then
    echo "Cluster name must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$EMC_VPLEX_SSH_IP"x = "x" ]; then
    echo "Mgmt server IP or hostname must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$EMC_VPLEX_SSH_PORT"x = "x" ]; then
    echo "Mgmt server ssh port must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi
  if [ "$EMC_VPLEX_SSH_USER"x = "x" ]; then
    echo "Mgmt server user must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  export STORAGE_NAME STORAGE_TYPE EMC_VPLEX_SSH_IP EMC_VPLEX_SSH_PORT EMC_VPLEX_SSH_USER EMC_VPLEX_CLUSTER_ID

  # modules test
  $PERL  $BINDIR/perl_https_check.pl
  if [ $? -ne 0  ]; then
    echo "  connection failed!!"
    echo ""
    echo ""
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  # test API
  if [ "$EMC_VPLEX_API"x = "x" ]; then
    echo "Variable \"use API\" must be set. Check $STORAGE_CFG line: $line"
    echo "$STORAGE_NAME nok" >> $STATUS_FILE
    continue
  fi

  echo ""
  echo "Use API = $EMC_VPLEX_API"

  if [ "$EMC_VPLEX_API" -eq "1" ]; then
    if [ "$EMC_VPLEX_API_IP"x = "x" ]; then
      echo "Mgmt server IP or hostname must be set. Check $STORAGE_CFG line: $line"
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
      continue
    fi
    if [ "$EMC_VPLEX_API_PORT"x = "x" ]; then
      echo "API port must be set. Check $STORAGE_CFG line: $line"
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
      continue
    fi
    if [ "$EMC_VPLEX_USER"x = "x" ]; then
      echo "API encrypted password must be set. Check $STORAGE_CFG line: $line"
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
      continue
    fi
    if [ "$EMC_VPLEX_PASSWD"x = "x" ]; then
      echo "API user name must be set. Check $STORAGE_CFG line: $line"
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
      continue
    fi

    encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$EMC_VPLEX_PASSWD'")'`
    if [ ! $? -eq 0  ]; then
      echo "Error: password cannot be decoded"
      echo "  connection failed!!"
      continue
    fi

    export EMC_VPLEX_USER EMC_VPLEX_PASSWD EMC_VPLEX_API EMC_VPLEX_API_IP EMC_VPLEX_API_PORT

    echo "API connection:"
    perl bin/conntest.pl $EMC_VPLEX_API_IP $EMC_VPLEX_API_PORT WARNING # api port

    $PERL $BINDIR/vplex_apitest.pl
    if [ $? -eq 1 ]; then
      echo ""
      echo "  API connection OK"
    else
      echo ""
      echo "  API connection failed!!!"
      lwp=`$PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'| sed -e 's/LWP Version: //' -e 's/\..*$//'`
      if [  $lwp -lt 6 ]; then
        echo "  Perl LWP module is too old, it might be a problem"
        echo "  Upgrade it, refer here: http://www.stor2rrd.com/https.htm"
      fi
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
    fi
  fi

  # ssh connection
  echo ""
  echo "SSH connection:"

  # connection test
  perl bin/conntest.pl $EMC_VPLEX_SSH_IP $EMC_VPLEX_SSH_PORT WARNING # ssh port

  echo "$SSH $EMC_VPLEX_SSH_USER@$EMC_VPLEX_SSH_IP -p $EMC_VPLEX_SSH_PORT \"LC_ALL=en_US LANG=en_US LC_NUMERIC=en_US id\""
  $SSH $EMC_VPLEX_SSH_USER@$EMC_VPLEX_SSH_IP -p $EMC_VPLEX_SSH_PORT "LC_ALL=en_US LANG=en_US LC_NUMERIC=en_US id" >$TMP_FILE_VPLEX 2>$TMP_FILE_VPLEX_ERR
  user_id=`cat $TMP_FILE_VPLEX | egrep "$EMC_VPLEX_SSH_USER" | wc -l`
  vplex_cli=`cat $TMP_FILE_VPLEX | egrep "VPlexcli:" | wc -l`
  if [ $user_id -gt 0 ]; then
    echo ""
    echo "  SSH connection OK"
    echo ""

    # perpetual perf monitors
    echo "performance monitors:"
    echo "$SSH $EMC_VPLEX_SSH_USER@$EMC_VPLEX_SSH_IP -p $EMC_VPLEX_SSH_PORT \"LC_ALL=en_US LANG=en_US LC_NUMERIC=en_US ls -l /var/log/VPlex/cli/director-*_PERPETUAL_vplex_sys_perf_mon.log /var/log/VPlex/cli/director-*_VIRTUAL_VOLUMES_PERPETUAL_MONITOR.log\""
    $SSH $EMC_VPLEX_SSH_USER@$EMC_VPLEX_SSH_IP -p $EMC_VPLEX_SSH_PORT "LC_ALL=en_US LANG=en_US LC_NUMERIC=en_US ls -l /var/log/VPlex/cli/director-*_PERPETUAL_vplex_sys_perf_mon.log /var/log/VPlex/cli/director-*_VIRTUAL_VOLUMES_PERPETUAL_MONITOR.log" 2>&1 > $TMP_FILE_VPLEX
    monitors_count=`cat $TMP_FILE_VPLEX| egrep "_PERPETUAL_" | wc -l`
    if [ $monitors_count = 0 ]; then
      echo ""
      echo "  perpetual monitors were not found!"
    else
      echo ""
      cat $TMP_FILE_VPLEX
      for mon_line in `cat $TMP_FILE_VPLEX | sed 's/  / /g' | sed 's/ /XORUX/g' | egrep "_PERPETUAL_"`
      do
        MONITOR_SIZE=`echo $mon_line | awk 'BEGIN{FS="XORUX"}{print $5}'`
        MONITOR_NAME=`echo $mon_line | sed 's/.*XORUX//g'`
        if [ $MONITOR_SIZE -eq 0 ]; then
          echo ""
          echo "WARNING : Monitor $MONITOR_NAME is empty! File size = $MONITOR_SIZE"
          echo "WARNING : Problem description and fix: http://www.stor2rrd.com/VPLEX-monitor-stopped.htm"
          echo ""
        fi
      done
      echo "$STORAGE_NAME  ok" >> $STATUS_FILE
    fi
  else
    if [ $vplex_cli -gt 0 ]; then
      echo ""
      echo "  SSH connection failed!"
      echo "  Problem description and fix: http://www.stor2rrd.com/VPLEX_cli_access.htm"
    else
      echo ""
      echo "  SSH connection failed: $TMP_FILE_VPLEX_ERR"
      echo "$STORAGE_NAME nok" >> $STATUS_FILE
    fi
  fi
  rm -f $TMP_FILE_VPLEX $TMP_FILE_VPLEX_ERR
  echo ""

done
echo ""


### SAN ###
if [ -f "$SAN_CFG" ]; then
  SNMP_PORT=`cat $SAN_CFG|sed 's/#.*$//'|egrep "^SNMP_PORT"|egrep -v "^#"|sed -e 's/^SNMP_PORT=//'`
  export SNMP_PORT

  for line in `cat $SAN_CFG|sed 's/#.*$//'|egrep ".*:.*:.*:"|egrep -v "^#"`
  do
    SAN_IP=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
    export SAN_IP

    SAN_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
    export SAN_TYPE

    if [ ! "$storage"x = "x" ]; then
      if [ ! "$storage" = "$SAN_IP" ]; then
        continue
      fi
    fi 

    if [ "$SAN_TYPE" = "BNA" ]; then

      if [ "$PERL" = "/opt/freeware/bin/perl" ]; then
        # Perl LWP module on AIX requires openssl from /usr/lib!!! It does not work with  /opt/freeware/lib one
        # It is a must be here for BNA only, not for switches!
        export LIBPATH=/usr/lib:/opt/freeware/lib:$LIBPATH
      fi

      # BNA
      #
      BNA_IP=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
      BNA_PORT=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
      BNA_USER=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
      BNA_PASS=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
      BNA_IP_SEC=`echo $line | awk 'BEGIN{FS=":"}{print $6}'`

      echo "========================="
      echo "BNA: $BNA_IP"
      echo "========================="

      # storage-list.cfg variables
      if [ "$BNA_IP"x = "x" ]; then
        echo "BNA IP must be set. Check $SAN line: $line"
        echo "BNA: $BNA_IP nok" >> $STATUS_FILE
        continue
      fi
      if [ "$BNA_PORT"x = "x" ]; then
        echo "BNA PORT must be set. Check $SAN line: $line"
        echo "BNA: $BNA_IP nok" >> $STATUS_FILE
        continue
      fi
      if [ "$BNA_USER"x = "x" ]; then
        echo "BNA USER must be set. Check $SAN line: $line"
        echo "BNA: $BNA_IP nok" >> $STATUS_FILE
        continue
      fi

      # modules test
      $PERL $BINDIR/perl_https_check.pl
      if [ $? -ne 0  ]; then
        echo "  connection failed!!"
        echo ""
        echo ""
        echo "BNA: $BNA_IP nok" >> $STATUS_FILE
        continue
      fi

      encode_pw=`perl -I. -MXorux_lib -e 'print Xorux_lib::unobscure_password("'$EMC_VPLEX_PASSWD'")'`
      if [ ! $? -eq 0  ]; then
        echo "Error: password cannot be decoded"
        echo "  connection failed!!"
        continue
      fi

      perl bin/conntest.pl $BNA_IP $BNA_PORT WARNING # api port

      export BNA_IP BNA_PORT BNA_USER BNA_PASS

      echo "API connection:"
      $PERL $BINDIR/bna_apitest.pl $BNA_IP
      if [ $? -eq 1 ]; then
        echo ""
        echo "  API connection OK"
        echo "BNA: $BNA_IP ok" >> $STATUS_FILE
      else
        echo ""
        echo "  API connection failed!!!"
        lwp=`$PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'| sed -e 's/LWP Version: //' -e 's/\..*$//'`
        if [  $lwp -lt 6 ]; then
          echo "  Perl LWP module is too old, it might be a problem"
          echo "  Upgrade it, refer here: http://www.stor2rrd.com/https.htm"
        fi
        echo "BNA: $BNA_IP nok" >> $STATUS_FILE
      fi

      # try second BNA IP
      if [ ! "$BNA_IP_SEC"x = "x" ]; then
        echo ""
        echo "Try second IP: $BNA_IP_SEC"

        perl bin/conntest.pl $BNA_IP_SEC $BNA_PORT WARNING # api port

        echo "Second API connection:"
        $PERL $BINDIR/bna_apitest.pl $BNA_IP_SEC
        if [ $? -eq 1 ]; then
          echo ""
          echo "  API connection OK"
          echo "BNA: $BNA_IP_SEC ok" >> $STATUS_FILE
        else
          echo ""
          echo "  API connection failed!!!"
          echo "BNA: $BNA_IP_SEC nok" >> $STATUS_FILE
        fi
      fi
    else

      if [ "$PERL" = "/opt/freeware/bin/perl" ]; then
        # SNMP module on AIX requires openssl from /opt/freeware/lib!!! It does not work with the system one
        # Only for switches, not for BNA!
        # Do not use this setup for storages which uses https (REST API)!!!
        export LIBPATH=/opt/freeware/lib:$LIBPATH
      fi

      # BRCD|CISCO
      #
      SNMP_VERSION=`cat $SAN_CFG|sed 's/#.*$//'|egrep "^SNMP_VERSION"|egrep -v "^#"|sed -e 's/^SNMP_VERSION=//'`
      SNMP_VERSION_FOR_ONE_SWITCH=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
      if [ ! "$SNMP_VERSION_FOR_ONE_SWITCH"x = "x" ]; then
        SNMP_VERSION=$SNMP_VERSION_FOR_ONE_SWITCH
      fi
      export SNMP_VERSION

      echo "========================="
      echo "SWITCH: $SAN_IP"
      echo "========================="

      #test path to snmpwalk
      snmpwalk="snmpwalk"
      `$snmpwalk 1>/dev/null 2>/dev/null`
      if [ ! "$?" -eq "1" ]; then
        snmpwalk="/usr/bin/snmpwalk"
        `$snmpwalk 1>/dev/null 2>/dev/null`
        if [ ! "$?" -eq "1" ]; then
          snmpwalk="/opt/freeware/bin/snmpwalk"
          `$snmpwalk 1>/dev/null 2>/dev/null`
          if [ ! "$?" -eq "1" ]; then
            echo "`date` : Path to snmpwalk not found! $0"
            snmpwalk=""
            #exit 0
          fi
        fi
      fi

      if [ "$snmpwalk"x != "x" ]; then
        snmpwalk_ver="1"
        if [ "$SAN_TYPE" = "CISCO" ]; then
          snmpwalk_ver="2c"
        fi
        if [ "$SNMP_VERSION"x != "x" ]; then
          snmpwalk_ver="$SNMP_VERSION"
        fi
        community_string=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
        if [ "$community_string"x = "x" ]; then
          community_string="public"
          if [ "$snmpwalk_ver" = "3" ]; then
            community_string="snmpuser1"
          fi
        fi

        # test snmpwalk command
        echo "test snmpwalk:"

        if [ "$SNMP_PORT"x = "x" ]; then
          snmpwalk_cmd="$snmpwalk -v $snmpwalk_ver -c $community_string $SAN_IP 1.3.6.1.2.1.1.5"
          if [ "$snmpwalk_ver" = "3" ]; then
            snmpwalk_cmd="$snmpwalk -v $snmpwalk_ver -u $community_string $SAN_IP 1.3.6.1.2.1.1.5"
          fi
        else
          snmpwalk_cmd="$snmpwalk -v $snmpwalk_ver -c $community_string $SAN_IP:$SNMP_PORT 1.3.6.1.2.1.1.5"
          if [ "$snmpwalk_ver" = "3" ]; then
            snmpwalk_cmd="$snmpwalk -v $snmpwalk_ver -u $community_string $SAN_IP:$SNMP_PORT 1.3.6.1.2.1.1.5"
          fi
        fi

        echo "$snmpwalk_cmd"
        snmpwalk_out=`$snmpwalk_cmd`
        echo "$snmpwalk_out"

        if [ `echo $snmpwalk_out | grep "= STRING:"|wc -l` -gt 0 ]; then
          echo "snmpwalk cmd : ok"
        else
          echo "snmpwalk cmd : failed!"
        fi
        echo ""
      fi


      #
      echo "san_verify.pl:"
      $PERL $BINDIR/san_verify.pl $SAN_IP

      if [ ! $? -eq 0 ]; then
        echo "  connection failed!!"
        echo "  Check network connectivity and user access"
        echo "SAN: $SAN_IP nok" >> $STATUS_FILE
      else 
        echo "  connection ok"
        echo "SAN: $SAN_IP  ok" >> $STATUS_FILE
      fi

      echo ""
      echo ""
    fi
  done
else
  echo "No SAN configured : $SAN_CFG"
fi

if [ -f $STATUS_FILE -a "$1"x = "x" ]; then
  echo ""
  echo "========"
  echo "Summary:"
  echo "========"
  cat $STATUS_FILE
fi

}

STATUS_FILE_ALL=$INPUTDIR/logs/config_check.out
main $1 2>&1| tee $STATUS_FILE_ALL
