
use strict;
use warnings;

use Data::Dumper;

my $DEBUG   = $ENV{DEBUG};
my $errlog  = $ENV{ERRLOG};
my $webdir  = $ENV{WEBDIR};
my $basedir = $ENV{INPUTDIR};
my $tmp_dir = "$basedir/tmp";
my $wrkdir  = "$basedir/data";

# CGI-BIN HTML header
print "Content-type: text/html\n\n";
print "<div style=\"text-align: center\">\n";
print "<div style=\"display: inline-block\">\n";

open( OUT, ">> $errlog" ) if $DEBUG == 2;

# get QUERY_STRING
use Env qw(QUERY_STRING);
print OUT "-- $QUERY_STRING\n" if $DEBUG == 2;

#print STDERR "$QUERY_STRING\n";

( my $host, my $type, my $name, my $st_type, my $item, my $gui, my $referer ) = split( /&/, $QUERY_STRING );

# no URL decode here, as it goes immediately into URL again

$host =~ s/host=//;
my $host_url = $host;
$host =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
$host =~ s/\+/ /g;
$host =~ s/%23/\#/g;
$type =~ s/type=//;
$name =~ s/name=//;
$name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
$name =~ s/\+/ /g;
$name =~ s/%23/\#/g;
$item =~ s/item=//;
$st_type =~ s/storage=//;

#print STDERR "$host $name $type $st_type $item\n";
#print STDERR "$QUERY_STRING : $name\n";

my $gray_st   = "<img src=\"css/images/status_gray.png\">";
my $red_st    = "<img src=\"css/images/status_red.png\">";
my $orange_st = "<img src=\"css/images/status_orange.png\">";
my $green_st  = "<img src=\"css/images/status_green.png\">";

my %cisco_legend;


#
# health status for one SAN switch
#
if ( $type eq "HS" && defined $host && $host ne "" ) {
  my $switch_type = get_switch_type($host);

  print "<div id=\"tabs\"><ul>\n";
  print "<li><a href=\"#tabs-0\">Ports</a></li>\n";

  # add events if exists
  if ( -f "$webdir/$host/event.html" ) {
    print "<li><a href=\"$host/event.html\">Events</a></li>\n";
  }
  # add switch status if exists
  if ( -f "$webdir/$host/SWITCH_HEALTH_STATUS.html" ) {
    print "<li><a href=\"$host/SWITCH_HEALTH_STATUS.html\">Switch status</a></li>\n";
  }

  print "</ul>\n";
  print "<div id=\"tabs-0\" class=\"blacktext\">\n";

  # parse and print HS table for one switch
  print "<br><br>\n";
  if ( $switch_type eq "BRCD" ) {
    print_hs_table_brcd( $host);
    print "<br>\n";
    print_legend_brcd();
  }
  if ( $switch_type eq "BNA" ) {
    print_hs_table_bna( $host);
    print "<br>\n";
    print_legend_bna();
  }
  if ( $switch_type eq "CISCO" ) {
    print_hs_table_cisco($host);
    print "<br>\n";
    print_legend_cisco();
  }

  print "</div>\n";
} ## end if ( $type eq "HS" && ...)


#
# health status for all SAN switches except BNA
#
if ( $type eq "HS-total" && $host eq "total" ) {
  print "<div id=\"tabs\"><ul>\n";
  print "<li><a href=\"#tabs-0\">Ports</a></li>\n";

  # add events if exists
  if ( -f "$webdir/SAN_total_events.html" ) {
    print "<li><a href=\"SAN_total_events.html\">Events</a></li>\n";
  }

  print "</ul>\n";
  print "<div id=\"tabs-0\" class=\"blacktext\">\n";

  my @switches_brcd  = <$wrkdir/*\/SAN-BRCD>;
  my @switches_cisco = <$wrkdir/*\/SAN-CISCO>;
  my $cisco_count    = @switches_cisco;

  # BRCD
  my $brcd_idx = 0;
  foreach my $switch_name (@switches_brcd) {
    chomp $switch_name;
    if ( $switch_name =~ m/SAN-BRCD$/ ) {
      $switch_name =~ s/^$wrkdir\///g;
      $switch_name =~ s/\/SAN-BRCD$//g;

      my $switch_type = get_switch_type($switch_name);
      if ( $switch_type ne "BRCD" ) { next; }

      $brcd_idx++;

      if ( defined $cisco_count && $cisco_count ne "0" && $brcd_idx == 1 ) {
        print "<b>Brocade SAN switches:</b><br><br>\n";
      }

      print "<b>$switch_name</b><br><br>\n";
      print_hs_table_brcd($switch_name);
      print "<br>\n";
    }
  }
  if ( $brcd_idx > 0 ) {
    print_legend_brcd();
  }

  # CISCO
  my $cisco_idx = 0;
  if ( defined $cisco_count && $cisco_count ne "0" ) {
    foreach my $switch_name (@switches_cisco) {
      chomp $switch_name;
      $switch_name =~ s/^$wrkdir\///g;
      $switch_name =~ s/\/SAN-CISCO$//g;

      $cisco_idx++;

      if ( $brcd_idx > 0 && $cisco_idx == 1 ) {
        print "<br><br><b>Cisco SAN switches:</b><br><br>\n";
      }

      print "<b>$switch_name</b><br><br>\n";
      print_hs_table_cisco($switch_name);
      print "<br>\n";
    }
    print_legend_cisco();
  }
}


#
# Health status for all BNA
#
if ( $type eq "HS-total" && $host eq "total-BNA" ) {
  print "<div id=\"tabs\"><ul>\n";
  print "<li><a href=\"#tabs-0\">Ports</a></li>\n";

  # add events if exists
  if ( -f "$webdir/BNA/all_events.html" ) {
    print "<li><a href=\"BNA/all_events.html\">Events</a></li>\n";
  }
  # add HS table for all switches if exists
  if ( -f "$webdir/BNA/health_status_all.html" ) {
    print "<li><a href=\"BNA/health_status_all.html\">Switches</a></li>\n";
  }

  print "</ul>\n";
  print "<div id=\"tabs-0\" class=\"blacktext\">\n";

  my @switches_bna = <$wrkdir/*\/SAN-BNA>;
  foreach my $switch_name (@switches_bna) {
    chomp $switch_name;
    $switch_name =~ s/^$wrkdir\///g;
    $switch_name =~ s/\/SAN-BNA$//g;

    my $switch_type = get_switch_type($switch_name);

    print "<b>$switch_name</b><br><br>\n";

    print_hs_table_bna( $switch_name, $switch_type );
    print "<br>\n";
  }
  print "<br>\n";
  print_legend_bna();
}


print "</div>\n";
print "</div>\n";
exit;

sub print_hs_table_brcd {
  my $switch_name = shift;

  if ( -f "$wrkdir/$switch_name/HEALTH_STATUS.txt" ) {
    open( HS, "<$wrkdir/$switch_name/HEALTH_STATUS.txt" ) || error( "Couldn't open file $wrkdir/$switch_name/HEALTH_STATUS.txt $!" . __FILE__ . ":" . __LINE__ ) && exit;
    my @lines = <HS>;
    close(HS);

    my %data;
    my $port_idx = 0;

    foreach my $line (@lines) {
      chomp $line;

      # port,Operational status,Physical status
      # ---------------------------------------
      # 0,online,inSync

      if ( $line =~ m/^port,Operational/ || $line =~ m/^-----------------/ ) { next; }

      $port_idx++;

      my ( $port_id, $oper_st, $phys_st ) = split( ",", $line );

      $data{'table'}{'id'}{$port_idx}{'port'}    = $port_id;
      $data{'table'}{'id'}{$port_idx}{'oper_st'} = $oper_st;
      $data{'table'}{'id'}{$port_idx}{'phys_st'} = $phys_st;

      # class for operational status
      my $oper_st_class = "UN";
      if ( defined $oper_st && $oper_st ne "" ) {
        if ( $oper_st =~ "unknown" ) {
          $oper_st_class = "st_gray";
        }
        if ( $oper_st =~ "faulty" ) {
          $oper_st_class = "st_red";
        }
        if ( $oper_st =~ "offline" || $oper_st =~ "testing" ) {
          $oper_st_class = "st_orange";
        }
        if ( $oper_st =~ "online" ) {
          $oper_st_class = "st_green";
        }
      } ## end if ( defined $oper_st ...)

      # class for physical status
      my $phys_st_class = "UN";
      if ( defined $phys_st && $phys_st ne "" ) {
        if ( $phys_st =~ "noCard" || $phys_st =~ "noTransceiver" ) {
          $phys_st_class = "st_gray";
        }
        if ( $phys_st =~ "laserFault" || $phys_st =~ "portFault" || $phys_st =~ "diagFault" ) {
          $phys_st_class = "st_red";
        }
        if ( $phys_st =~ "noLight" || $phys_st =~ "noSync" || $phys_st =~ "lockRef" ) {
          $phys_st_class = "st_orange";
        }
        if ( $phys_st =~ "inSync" ) {
          $phys_st_class = "st_green";
        }
      } ## end if ( defined $phys_st ...)

      $data{'table'}{'id'}{$port_idx}{'oper_st_class'} = $oper_st_class;
      $data{'table'}{'id'}{$port_idx}{'phys_st_class'} = $phys_st_class;

    } ## end foreach my $line (@lines)

    # print table
    my $print_table      = 0;
    my $port_table_line  = "";
    my $phys_table_line  = "";
    my $oper_table_line  = "";

    foreach my $id ( sort { $a <=> $b } keys( %{ $data{table}{id} } )) {
      my $port_name     = "";
      my $oper_st       = "";
      my $phys_st       = "";
      my $oper_st_class = "";
      my $phys_st_class = "";
      if ( defined $data{table}{id}{$id}{port} && $data{table}{id}{$id}{port} ne '' ) { $port_name = $data{table}{id}{$id}{port}; }
      if ( defined $data{table}{id}{$id}{oper_st} && $data{table}{id}{$id}{oper_st} ne '' ) { $oper_st = $data{table}{id}{$id}{oper_st}; }
      if ( defined $data{table}{id}{$id}{phys_st} && $data{table}{id}{$id}{phys_st} ne '' ) { $phys_st = $data{table}{id}{$id}{phys_st}; }
      if ( defined $data{table}{id}{$id}{oper_st_class} && $data{table}{id}{$id}{oper_st_class} ne '' ) { $oper_st_class = $data{table}{id}{$id}{oper_st_class}; }
      if ( defined $data{table}{id}{$id}{phys_st_class} && $data{table}{id}{$id}{phys_st_class} ne '' ) { $phys_st_class = $data{table}{id}{$id}{phys_st_class}; }

      #print "$port_name : $oper_st : $oper_st_class : $phys_st : $phys_st_class<br>\n";

      $port_table_line .= "<td title=\"$port_name\">$port_name</td>";
      $phys_table_line .= "<td class=\"$phys_st_class\" title=\"$port_name : $phys_st\"><a href=\"/stor2rrd-cgi/detail.sh?host=$switch_name&type=SANPORT&name=port$port_name&storage=SAN-BRCD&item=san&gui=1&none=none\"></td>";
      $oper_table_line .= "<td class=\"$oper_st_class\" title=\"$port_name : $oper_st\"><a href=\"/stor2rrd-cgi/detail.sh?host=$switch_name&type=SANPORT&name=port$port_name&storage=SAN-BRCD&item=san&gui=1&none=none\"></td>";

      if ( $id % 48 == 0 ) {
        print "<table class=\"san_stat_legend\">\n";
        print "<tr><td><b>Port</b></td>$port_table_line</tr>\n";
        print "<tr><td><b>Physical status</b></td>$phys_table_line</tr>";
        print "<tr><td><b>Operational status<b></td>$oper_table_line</tr>";
        print "</table>";

        $port_table_line = "";
        $phys_table_line = "";
        $oper_table_line = "";
        $print_table     = 0;
      }
      else {
        $print_table = 1;
      }
    }
    if ( defined $print_table && $print_table == 1 ) {
      print "<table class=\"san_stat_legend\">\n";
      print "<tr><td><b>Port</b></td>$port_table_line</tr>\n";
      print "<tr><td><b>Physical status</b></td>$phys_table_line</tr>";
      print "<tr><td><b>Operational status<b></td>$oper_table_line</tr>";
      print "</table>";
    }

  } ## end if ( -f "$wrkdir/$switch_name/HEALTH_STATUS.txt")
  else {
    print "Health status data not found! SAN switch skipped!<br><br>\n";
    error( "$wrkdir/$switch_name/HEALTH_STATUS.txt not exists! " . __FILE__ . ":" . __LINE__ );
  }

  return 1;
} ## end sub print_hs_table_brcd

sub print_hs_table_bna {
  my $switch_name = shift;

  if ( -f "$wrkdir/$switch_name/HEALTH_STATUS.txt" ) {
    open( HS, "<$wrkdir/$switch_name/HEALTH_STATUS.txt" ) || error( "Couldn't open file $wrkdir/$switch_name/HEALTH_STATUS.txt $!" . __FILE__ . ":" . __LINE__ ) && exit;
    my @lines = <HS>;
    close(HS);

    my %data;
    my $port_idx = 0;

    foreach my $line (@lines) {
      chomp $line;

      # port,Operational status,Physical status
      # ---------------------------------------
      # 0,online,inSync

      if ( $line =~ m/^port,Operational/ || $line =~ m/^-----------------/ ) { next; }

      $port_idx++;

      #my ( $port_id, $oper_st, $phys_st ) = split( ",", $line );
      my ( $port_id, $phys_st, $oper_st ) = split( ",", $line );

      $data{'table'}{'id'}{$port_idx}{'port'}    = $port_id;
      $data{'table'}{'id'}{$port_idx}{'oper_st'} = $oper_st;
      $data{'table'}{'id'}{$port_idx}{'phys_st'} = $phys_st;

      # class for operational status
      my $oper_st_class = "UN";
      if ( defined $oper_st && $oper_st ne "" ) {
        if ( $oper_st =~ "UNKNOWN" ) {
          $oper_st_class = "st_gray";
        }
        #if ( $oper_st =~ "faulty" ) {
        #  $oper_st_class = "st_red";
        #}
        if ( $oper_st =~ "OFFLINE" ) {
          $oper_st_class = "st_orange";
        }
        if ( $oper_st =~ "ONLINE" ) {
          $oper_st_class = "st_green";
        }
      } ## end if ( defined $oper_st ...)

      # class for physical status
      my $phys_st_class = "UN";
      if ( defined $phys_st && $phys_st ne "" ) {
        if ( $phys_st =~ "UNKNOWN" || $phys_st =~ "NO_MODULE" ) {
          $phys_st_class = "st_gray";
        }
        #if ( $phys_st =~ "diagFault" ) {
        #  $phys_st_class = "st_red";
        #}
        if ( $phys_st =~ "NO_SYNC" || $phys_st =~ "NO_LIGHT" || $phys_st =~ "DISABLED" ) {
          $phys_st_class = "st_orange";
        }
        if ( $phys_st =~ "IN_SYNC" || $phys_st =~ "ONLINE" || $phys_st =~ "ENABLED" ) {
          $phys_st_class = "st_green";
        }
      } ## end if ( defined $phys_st ...)

      $data{'table'}{'id'}{$port_idx}{'oper_st_class'} = $oper_st_class;
      $data{'table'}{'id'}{$port_idx}{'phys_st_class'} = $phys_st_class;

    } ## end foreach my $line (@lines)

    # print table
    my $print_table      = 0;
    my $port_table_line  = "";
    my $phys_table_line  = "";
    my $oper_table_line  = "";

    foreach my $id ( sort { $a <=> $b } keys( %{ $data{table}{id} } )) {
      my $port_name     = "";
      my $oper_st       = "";
      my $phys_st       = "";
      my $oper_st_class = "";
      my $phys_st_class = "";
      if ( defined $data{table}{id}{$id}{port} && $data{table}{id}{$id}{port} ne '' ) { $port_name = $data{table}{id}{$id}{port}; }
      if ( defined $data{table}{id}{$id}{oper_st} && $data{table}{id}{$id}{oper_st} ne '' ) { $oper_st = $data{table}{id}{$id}{oper_st}; }
      if ( defined $data{table}{id}{$id}{phys_st} && $data{table}{id}{$id}{phys_st} ne '' ) { $phys_st = $data{table}{id}{$id}{phys_st}; }
      if ( defined $data{table}{id}{$id}{oper_st_class} && $data{table}{id}{$id}{oper_st_class} ne '' ) { $oper_st_class = $data{table}{id}{$id}{oper_st_class}; }
      if ( defined $data{table}{id}{$id}{phys_st_class} && $data{table}{id}{$id}{phys_st_class} ne '' ) { $phys_st_class = $data{table}{id}{$id}{phys_st_class}; }

      #print "$port_name : $oper_st : $oper_st_class : $phys_st : $phys_st_class<br>\n";

      $port_table_line .= "<td title=\"$port_name\">$port_name</td>";
      $phys_table_line .= "<td class=\"$phys_st_class\" title=\"$port_name : $phys_st\"><a href=\"/stor2rrd-cgi/detail.sh?host=$switch_name&type=SANPORT&name=port$port_name&storage=SAN-BRCD&item=san&gui=1&none=none\"></td>";
      $oper_table_line .= "<td class=\"$oper_st_class\" title=\"$port_name : $oper_st\"><a href=\"/stor2rrd-cgi/detail.sh?host=$switch_name&type=SANPORT&name=port$port_name&storage=SAN-BRCD&item=san&gui=1&none=none\"></td>";

      if ( $id % 48 == 0 ) {
        print "<table class=\"san_stat_legend\">\n";
        print "<tr><td><b>Port</b></td>$port_table_line</tr>\n";
        print "<tr><td><b>Physical status</b></td>$phys_table_line</tr>";
        print "<tr><td><b>Operational status<b></td>$oper_table_line</tr>";
        print "</table>";

        $port_table_line = "";
        $phys_table_line = "";
        $oper_table_line = "";
        $print_table     = 0;
      }
      else {
        $print_table = 1;
      }
    }
    if ( defined $print_table && $print_table == 1 ) {
      print "<table class=\"san_stat_legend\">\n";
      print "<tr><td><b>Port</b></td>$port_table_line</tr>\n";
      print "<tr><td><b>Physical status</b></td>$phys_table_line</tr>";
      print "<tr><td><b>Operational status<b></td>$oper_table_line</tr>";
      print "</table>";
    }

  } ## end if ( -f "$wrkdir/$switch_name/HEALTH_STATUS.txt")
  else {
    print "Health status data not found! SAN switch skipped!<br><br>\n";
    error( "$wrkdir/$switch_name/HEALTH_STATUS.txt not exists! " . __FILE__ . ":" . __LINE__ );
  }

  return 1;
} ## end sub print_hs_table_brcd

sub print_hs_table_cisco {
  my $switch_name = shift;

  if ( -f "$wrkdir/$switch_name/HEALTH_STATUS.txt" ) {
    open( HS, "<$wrkdir/$switch_name/HEALTH_STATUS.txt" ) || error( "Couldn't open file $wrkdir/$switch_name/HEALTH_STATUS.txt $!" . __FILE__ . ":" . __LINE__ ) && exit;
    my @lines = <HS>;
    close(HS);

    my @states;
    if ( -f "$basedir/bin/cisco-status-custom.txt" ) {
      open( CST, "<$basedir/bin/cisco-status-custom.txt" ) || error( "Couldn't open file $basedir/bin/cisco-status-custom.txt $!" . __FILE__ . ":" . __LINE__ ) && exit;
      @states = <CST>;
      close(CST);
    }
    else {
      open( CST, "<$basedir/etc/cisco-status.txt" ) || error( "Couldn't open file $basedir/etc/cisco-status.txt $!" . __FILE__ . ":" . __LINE__ ) && exit;
      @states = <CST>;
      close(CST);
    }

    my %data;
    my $port_idx = 0;

    foreach my $line (@lines) {
      chomp $line;

      # port,Operational status,Physical status,Operational status reason
      # ---------------------------------------
      # fc1-1,up,up,none

      if ( $line =~ m/^port,Operational/ || $line =~ m/^-----------------/ ) { next; }

      $port_idx++;

      my ( $port_id, $oper_st, $phys_st, $oper_st_reason ) = split( ",", $line );

      $data{'table'}{'id'}{$port_idx}{'port'}           = $port_id;
      $data{'table'}{'id'}{$port_idx}{'oper_st'}        = $oper_st;
      $data{'table'}{'id'}{$port_idx}{'phys_st'}        = $phys_st;
      $data{'table'}{'id'}{$port_idx}{'oper_st_reason'} = $oper_st_reason;

      # class for physical status (admin status)
      my $phys_st_class = "UN";
      if ( defined $phys_st && $phys_st ne "" ) {
        if ( $phys_st =~ "testing" || $phys_st =~ "down" ) {
          $phys_st_class = "st_orange";
        }
        if ( $phys_st =~ "up" ) {
          $phys_st_class = "st_green";
        }
      } ## end if ( defined $phys_st ...)

      # class for operational status
      my $oper_st_class = "UN";
      if ( defined $oper_st && $oper_st ne "" ) {
        if ( $oper_st =~ "unknown" || $oper_st =~ "notPresent" ) {
          $oper_st_class = "st_gray";
        }
        if ( $oper_st =~ "dormant" || $oper_st =~ "lowerLayerDown" ) {
          $oper_st_class = "st_red";
        }
        if ( $oper_st =~ "testing" || $oper_st =~ "down" ) {
          $oper_st_class = "st_orange";
        }
        if ( $oper_st =~ "up" ) {
          $oper_st_class = "st_green";
        }
      } ## end if ( defined $oper_st ...)

      my $oper_st_reason_class = "UN";
      if ( defined $oper_st_reason && $oper_st_reason ne '' ) {
        my ($act_state_line) = grep ( /: $oper_st_reason/, @states );
        chomp $act_state_line;
        my ( $def_color, undef, $act_state ) = split( " : ", $act_state_line );
        my $sort_ind = "";

        if ( $def_color eq "red" )    { $oper_st_reason_class = "st_red";    $sort_ind = 4; }
        if ( $def_color eq "green" )  { $oper_st_reason_class = "st_green";  $sort_ind = 1; }
        if ( $def_color eq "yellow" ) { $oper_st_reason_class = "st_orange"; $sort_ind = 3; }
        if ( $def_color eq "grey" )   { $oper_st_reason_class = "st_gray";   $sort_ind = 2; }

        $cisco_legend{'sort_ind'}{$sort_ind}{'class'} = "$oper_st_reason_class";
        $cisco_legend{'sort_ind'}{$sort_ind}{'status'}{$act_state} = "$act_state";
      }

      $data{'table'}{'id'}{$port_idx}{'oper_st_class'}        = $oper_st_class;
      $data{'table'}{'id'}{$port_idx}{'phys_st_class'}        = $phys_st_class;
      $data{'table'}{'id'}{$port_idx}{'oper_st_reason_class'} = $oper_st_reason_class;

    } ## end foreach my $line (@lines)

    # print table
    my $print_table        = 0;
    my $port_table_line    = "";
    my $phys_table_line    = "";
    my $oper_table_line    = "";
    my $oper_r_table_line  = "";

    foreach my $id ( sort { $a <=> $b } keys( %{ $data{table}{id} } )) {
      my $port_name             = "";
      my $oper_st               = "";
      my $phys_st               = "";
      my $oper_st_reason        = "";
      my $oper_st_class         = "";
      my $phys_st_class         = "";
      my $oper_st_reason_class  = "";
      if ( defined $data{table}{id}{$id}{port} && $data{table}{id}{$id}{port} ne '' ) { $port_name = $data{table}{id}{$id}{port}; }
      if ( defined $data{table}{id}{$id}{oper_st} && $data{table}{id}{$id}{oper_st} ne '' ) { $oper_st = $data{table}{id}{$id}{oper_st}; }
      if ( defined $data{table}{id}{$id}{phys_st} && $data{table}{id}{$id}{phys_st} ne '' ) { $phys_st = $data{table}{id}{$id}{phys_st}; }
      if ( defined $data{table}{id}{$id}{oper_st_reason} && $data{table}{id}{$id}{oper_st_reason} ne '' ) { $oper_st_reason = $data{table}{id}{$id}{oper_st_reason}; }
      if ( defined $data{table}{id}{$id}{oper_st_class} && $data{table}{id}{$id}{oper_st_class} ne '' ) { $oper_st_class = $data{table}{id}{$id}{oper_st_class}; }
      if ( defined $data{table}{id}{$id}{phys_st_class} && $data{table}{id}{$id}{phys_st_class} ne '' ) { $phys_st_class = $data{table}{id}{$id}{phys_st_class}; }
      if ( defined $data{table}{id}{$id}{oper_st_reason_class} && $data{table}{id}{$id}{oper_st_reason_class} ne '' ) { $oper_st_reason_class = $data{table}{id}{$id}{oper_st_reason_class}; }

      my $port_num = $port_name;
      $port_num    =~ s/[a-z]//g;
      $port_num    =~ s/[A-Z]//g;

      $port_table_line .= "<td title=\"$port_name\">$port_num</td>";
      $phys_table_line .= "<td class=\"$phys_st_class\" title=\"$port_name : $phys_st\"><a href=\"/stor2rrd-cgi/detail.sh?host=$switch_name&type=SANPORT&name=port$port_name&storage=SAN-BRCD&item=san&gui=1&none=none\"></td>";
      $oper_table_line .= "<td class=\"$oper_st_class\" title=\"$port_name : $oper_st\"><a href=\"/stor2rrd-cgi/detail.sh?host=$switch_name&type=SANPORT&name=port$port_name&storage=SAN-BRCD&item=san&gui=1&none=none\"></td>";
      $oper_r_table_line .= "<td class=\"$oper_st_reason_class\" title=\"$port_name : $oper_st_reason\"><a href=\"/stor2rrd-cgi/detail.sh?host=$switch_name&type=SANPORT&name=port$port_name&storage=SAN-BRCD&item=san&gui=1&none=none\"></td>";

      if ( $id % 48 == 0 ) {
        print "<table class=\"san_stat_legend\">\n";
        print "<tr><td><b>Port</b></td>$port_table_line</tr>\n";
        print "<tr><td><b>Admin status</b></td>$phys_table_line</tr>";
        print "<tr><td><b>Operational status<b></td>$oper_table_line</tr>";
        print "<tr><td><b>Operational status reason<b></td>$oper_r_table_line</tr>";
        print "</table>";

        $port_table_line = "";
        $phys_table_line = "";
        $oper_table_line = "";
        $print_table     = 0;
      }
      else {
        $print_table = 1;
      }
    }
    if ( defined $print_table && $print_table == 1 ) {
      print "<table class=\"san_stat_legend\">\n";
      print "<tr><td><b>Port</b></td>$port_table_line</tr>\n";
      print "<tr><td><b>Admin status</b></td>$phys_table_line</tr>";
      print "<tr><td><b>Operational status<b></td>$oper_table_line</tr>";
      print "<tr><td><b>Operational status reason<b></td>$oper_r_table_line</tr>";
      print "</table>";
    }

  } ## end if ( -f "$wrkdir/$switch_name/HEALTH_STATUS.txt")
  else {
    print "Health status data not found! SAN switch skipped!<br><br>\n";
    error( "$wrkdir/$switch_name/HEALTH_STATUS.txt not exists! " . __FILE__ . ":" . __LINE__ );
  }

  return 1;
} ## end sub print_hs_table_brcd

sub get_switch_type {
  my $switch_name = shift;
  my $switch_type = "UN";

  if ( -f "$wrkdir/$switch_name/SAN-BRCD" ) {
    $switch_type = "BRCD";

    if ( -f "$wrkdir/$switch_name/SAN-BNA" ) {
      $switch_type = "BNA";
    }
  } ## end if ( -f "$wrkdir/$switch_name/SAN-BRCD")
  if ( -f "$wrkdir/$switch_name/SAN-CISCO" ) {
    $switch_type = "CISCO";
  }

  if ( $switch_type eq "UN" ) {
    error( "Unknown switch type for \"$switch_name\"!" . __FILE__ . ":" . __LINE__ );
  }

  return $switch_type;
} ## end sub get_switch_type

sub print_legend_brcd {
  print <<_MARKER_;
    <table>
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Physical status</b></td>
          </tr>
          <tr>
            <td class="st_green"></td>
            <td><b>inSync:</b> The module is receiving light and is in sync.</td>
          </tr>
          <tr>
            <td class="st_gray"></td>
            <td><b>noCard:</b> No card is present in this switch slot.</td>
          </tr>
          <tr>
            <td></td>
            <td><b>noTransceiver:</b> No Transceiver module in this port.</td>
          </tr>
          <tr>
            <td></td>
            <td>(Transceiver is the generic name for GBIC, SFP, and so on)</td>
          </tr>
          <tr>
            <td class="st_orange"></td>
            <td><b>noLight:</b> The module is not receiving light.</td>
          </tr>
          <tr>
            <td></td>
            <td><b>noSync:</b> The module is receiving light but is out of sync.</td>
          </tr>
          <tr>
            <td></td>
            <td><b>lockRef:</b> Port is locking to the reference signal.</td>
          </tr>
            <td class="st_red"></td>
            <td><b>laserFault:</b> The module is signaling a laser fault.</td>
          </tr>
          <tr>
            <td></td>
            <td>(defective GBIC)</td>
          </tr>
          <tr>
            <td></td>
            <td><b>portFault:</b> The port is marked faulty.</td>
          </tr>
          <tr>
            <td></td>
            <td>(defective GBIC, cable, or device)</td>
          </tr>
          <tr>
            <td></td>
            <td><b>diagFault:</b> The port failed diagnostics.</td>
          </tr>
          <tr>
            <td></td>
            <td>(defective G_Port or FL_Port card or motherboard)</td>
          </tr>
        </table>
      </td>
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Operational status</b></td>
          </tr>
          <tr>
            <td class="st_green"></td>
            <td><b>online:</b> User frames can be passed.</td>
          </tr>
          <tr>
            <td class="st_gray"></td>
            <td><b>unknown:</b> The port module is physically absent.</td>
          </tr>
          <tr>
            <td class="st_orange"></td>
            <td><b>offline:</b> No user frames can be passed.</td>
          </tr>
          <tr>
            <td class="st_red"></td>
            <td><b>faulty:</b> The port module is physically faulty.</td>
          </tr>
        </table>
      </td>
    </table>
_MARKER_

  return 1;
} ## end sub print_legend_brcd

sub print_legend_bna {
  print <<_MARKER_;
    <table>
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Physical status</b></td>
          </tr>
          <tr>
            <td class="st_green"></td>
            <td><b>IN_SYNC</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>ONLINE</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>ENABLED</b></td>
          </tr>
          <tr>
            <td class="st_gray"></td>
            <td><b>UNKNOWN</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>NO_MODULE</b></td>
          </tr>
          <tr>
            <td class="st_orange"></td>
            <td><b>NO_SYNC</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>NO_LIGHT</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>DISABLED</b></td>
          </tr>
        </table>
      </td>
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Operational status</b></td>
          </tr>
          <tr>
            <td class="st_green"></td>
            <td><b>ONLINE</b></td>
          </tr>
          <tr>
            <td class="st_gray"></td>
            <td><b>UNKNOWN</b></td>
          </tr>
          <tr>
            <td class="st_orange"></td>
            <td><b>OFFLINE</b></td>
          </tr>
        </table>
      </td>
    </table>
_MARKER_

  return 1;
} ## end sub print_legend_bna

sub print_legend_cisco {
  print <<_MARKER_;
    <table>
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Admin status</b></td>
          </tr>
          <tr>
            <td colspan="2">The desired state of the interface.</td>
          </tr>
          <tr>
            <td class="st_green"></td>
            <td><b>up</b></td>
          </tr>
          <tr>
            <td class="st_orange"></td>
            <td><b>down</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>testing</b></td>
          </tr>
        </table>
      </td>
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Operational status</b></td>
          </tr>
          <tr>
            <td colspan="2">The current operational state of the interface.</td>
          </tr>
          <tr>
            <td class="st_green"></td>
            <td><b>up</b></td>
          </tr>
          <tr>
            <td class="st_gray"></td>
            <td><b>notPresent</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>unknown</b></td>
          </tr>
          <tr>
            <td class="st_orange"></td>
            <td><b>down</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>testing</b></td>
          </tr>
          <tr>
            <td class="st_red"></td>
            <td><b>lowerLayerDown</b></td>
          </tr>
        </table>
      </td>
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Operational status reason</b></td>
          </tr>
          <tr>
            <td colspan="2">The cause of current operational state of the port.</td>
          </tr>
_MARKER_

  # Operational status reason
  if ( defined $cisco_legend{'sort_ind'} && $cisco_legend{'sort_ind'} ne '' ) {
    foreach my $index ( sort { $a <=> $b } keys( %{ $cisco_legend{'sort_ind'} } ) ) {
      my $st_id    = 0;
      my $st_class = "";

      if ( defined $cisco_legend{'sort_ind'}{$index}{'class'} && $cisco_legend{'sort_ind'}{$index}{'class'} ne '' ) {
        $st_class = $cisco_legend{'sort_ind'}{$index}{'class'};
      }
      if ( defined $cisco_legend{'sort_ind'}{$index}{'status'} && $cisco_legend{'sort_ind'}{$index}{'status'} ne '' ) {
        foreach my $status ( sort keys( %{ $cisco_legend{'sort_ind'}{$index}{'status'} } ) ) {
          $st_id++;
          if ( $st_id == 1 ) {
            print "<tr>\n";
            print "  <td class=\"$st_class\"></td>\n";
            print "  <td><b>$status</b></td>\n";
            print "</tr>\n";
          }
          else {
            print "<tr>\n";
            print "  <td></td>\n";
            print "  <td><b>$status</b></td>\n";
            print "</tr>\n";
          }
        }
      }
    }
  }

  print <<_MARKER_;
        </table>
      </td>
    </table>
_MARKER_

  return 1;
} ## end sub print_legend_cisco

sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);
  print STDERR "$act_time: $text : $!\n";
  return 1;
} ## end sub error

