#!/usr/bin/perl

use strict;
use warnings;
use RRDp;
use Xorux_lib;
use Data::Dumper;
use File::Copy;

my $rrdtool        = $ENV{RRDTOOL};
my $basedir        = $ENV{INPUTDIR};
my $wrkdir         = "$basedir/data";
my $tmpdir         = "$basedir/tmp";
my $KEEP_OUT_FILES = $ENV{KEEP_OUT_FILES};
my $demo           = 0;

if ( defined $ENV{DEMO} ) { $demo = $ENV{DEMO}; }

if ( !defined($KEEP_OUT_FILES) || $KEEP_OUT_FILES eq '' ) {
  $KEEP_OUT_FILES = 0;    # delete data out files as default
}

my $last_update_file = "$tmpdir/san_last_upd";
my $act_time         = localtime();
my $act_timestamp    = time();
my $last_timestamp   = $act_timestamp;
my $last_update_time = localtime($act_timestamp);
my $timestamp_diff   = 0;

if ( -f $last_update_file ) {
  $last_timestamp = ( stat($last_update_file) )[9];
  $timestamp_diff = $act_timestamp - $last_timestamp;
}

my $last_update_file_tmp = "$tmpdir/san_last_upd-tmp";
my $last_timestamp_tmp   = $act_timestamp;
my $last_update_time_tmp = localtime($act_timestamp);
my $timestamp_diff_tmp   = 0;

if ( -f $last_update_file_tmp ) {
  $last_timestamp_tmp = ( stat($last_update_file_tmp) )[9];
  $timestamp_diff_tmp = $act_timestamp - $last_timestamp_tmp;
}

if ( -f $last_update_file_tmp && $timestamp_diff_tmp < 10800 ) { exit; }
if ( -f $last_update_file_tmp && $timestamp_diff_tmp > 10800 ) {
  unlink($last_update_file_tmp);
  error( "$last_update_file_tmp is still exist!" . __FILE__ . ":" . __LINE__ );
}

RRDp::start "$rrdtool";

# test if is time for update
if ( !-f $last_update_file || $timestamp_diff > 590 ) {
  open( LUF, ">$last_update_file_tmp" ) || error( "Couldn't open file $last_update_file $!" . __FILE__ . ":" . __LINE__ ) && exit;
  print LUF "$act_timestamp\n$act_time\n";
  close(LUF);

  load_output_data($wrkdir);

  # create|update rrd file for total summary per switch
  load_total_summary_per_switch($wrkdir);
} ## end if ( !-f $last_update_file...)

# close RRD pipe
RRDp::end;

rename "$last_update_file_tmp", "$last_update_file";

exit;

sub load_output_data {

  my $wrkdir     = shift;
  my @switches_a = <$wrkdir/*\/SAN-*>;
  my @switches_all;
  foreach my $sw_n (@switches_a) {
    chomp $sw_n;
    if ( $sw_n =~ "SAN-BRCD" || $sw_n =~ "SAN-CISCO" ) {
      push( @switches_all, "$sw_n\n" );
    }
  } ## end foreach my $sw_n (@switches_a)

  foreach my $switch_name (@switches_all) {
    chomp $switch_name;
    $switch_name =~ s/^$wrkdir\///g;
    $switch_name =~ s/\/SAN-BRCD$//g;
    $switch_name =~ s/\/SAN-CISCO$//g;

    my $perf_string = "sanperf";
    opendir( DIR, "$wrkdir/$switch_name" ) || main::error( "directory does not exists : $wrkdir/$switch_name " . __FILE__ . ":" . __LINE__ ) && return 0;

    my @files_unsorted = grep( /$switch_name\_$perf_string\_20.*/, readdir(DIR) );
    my @files = sort { lc $a cmp lc $b } @files_unsorted;
    closedir(DIR);

    foreach my $file (@files) {
      chomp $file;
      my $input_file = "$wrkdir/$switch_name/$file";

      #print "$input_file\n";
      open( IN, "<$input_file" ) || error( "Couldn't open file $input_file $!" . __FILE__ . ":" . __LINE__ ) && exit;
      my @data = <IN>;
      close(IN);

      my $type = "";

      foreach my $port_data (@data) {
        chomp $port_data;
        if ( $port_data =~ m/^$/ )                            { $type = "NA"  ;   next; }
        if ( $port_data =~ "^act_time,bytes_tra,bytes_rec," ) { $type = "PORT";   next; }
        if ( $port_data =~ "^act_time,cpu,mem,resp_t," )      { $type = "SWITCH"; next; }

        if ( $type eq "PORT" ) {
          my ( $act_time, $bytes_tra, $bytes_rec, $frames_tra, $frames_rec, $swFCPortNoTxCredits, $swFCPortRxCrcs, $port_speed, $reserve1, $reserve2, $reserve3, $reserve4, $reserve5, $reserve6, $switch_name, $db_name, undef ) = split( ",", $port_data );

          my $rrd_file = "$wrkdir/$switch_name/$db_name";

          if ( !-f $rrd_file ) {
            #print "$act_time : create rrd : $rrd_file\n";
            rrd_create( $act_time, $switch_name, $db_name, $rrd_file, $type );
          }
          if ( -f $rrd_file ) {
            #print "$act_time : update rrd : $rrd_file\n";
            rrd_update( $act_time, $bytes_tra, $bytes_rec, $frames_tra, $frames_rec, $swFCPortNoTxCredits, $swFCPortRxCrcs, $port_speed, $reserve1, $reserve2, $reserve3, $reserve4, $reserve5, $reserve6, $switch_name, $db_name, $rrd_file );
          }
        }


        if ( $type eq "SWITCH" ) {
          # act_time,cpu,mem,resp_t,switch_name,db_name,rrd_file
          my ( $act_time, $cpu, $mem, $resp_t, $reserve1, $reserve2, $switch_name, $db_name, undef ) = split( ",", $port_data );

          my $rrd_file = "$wrkdir/$switch_name/$db_name";

          if ( !-f $rrd_file ) {
            #print "$act_time : create rrd : $rrd_file\n";
            rrd_create( $act_time, $switch_name, $db_name, $rrd_file, $type );
          }
          if ( -f $rrd_file ) {
            #print "$act_time : update rrd : $rrd_file\n";
            rrd_update_switch( $act_time, $cpu, $mem, $resp_t, $reserve1, $reserve2, $switch_name, $db_name, $rrd_file );
          }
        }
      } ## end foreach my $port_data (@data)

      if ( -f "$tmpdir/$switch_name-perf-last.txt" ) {
        move( "$tmpdir/$switch_name-perf-last.txt", "$tmpdir/$switch_name-perf-last1.txt" );
      }

      copy( $input_file, "$tmpdir/$switch_name-perf-last.txt" );
      if ( "$KEEP_OUT_FILES" eq "1" || "$KEEP_OUT_FILES" eq "$switch_name" ) { next; }
      if ( "$KEEP_OUT_FILES" eq "SAN-BNA" && $switch_name =~ m/-BNA$/ ) { next; }
      if ( -f "$input_file" ) {
        unlink("$input_file");    # delete already processed file
      }

    } ## end foreach my $file (@files)
  } ## end foreach my $switch_name (@switches_all)
} ## end sub load_output_data

sub load_total_summary_per_switch {

  my $wrkdir     = shift;
  my $act_time   = time();
  my @switches_a = <$wrkdir/*\/SAN-*>;
  my @switches_all;
  foreach my $sw_n (@switches_a) {
    chomp $sw_n;
    if ( $sw_n =~ "SAN-BRCD" || $sw_n =~ "SAN-CISCO" ) {
      push( @switches_all, "$sw_n\n" );
    }
  } ## end foreach my $sw_n (@switches_a)

  foreach my $switch_name (@switches_all) {
    chomp $switch_name;
    $switch_name =~ s/^$wrkdir\///g;
    $switch_name =~ s/\/SAN-BRCD$//g;
    $switch_name =~ s/\/SAN-CISCO$//g;

    my $db_name  = "total_summary.rrs";
    my $rrd_file = "$wrkdir/$switch_name/$db_name";

    # test if exists rrd files for ports
    my @rrd_files       = <$wrkdir/$switch_name/*.rrd>;
    my $rrd_files_count = @rrd_files;
    if ( $rrd_files_count == 0 ) { return 0; }

    if ( -f $rrd_file ) {

      # find last update time
      RRDp::cmd qq(last $rrd_file);
      my $last_tt        = RRDp::read;
      my $last_timestamp = $$last_tt;
      chomp $last_timestamp;

      fetch_data_for_total_summary( $act_time, $switch_name, $db_name, $rrd_file, $last_timestamp );

    } ## end if ( -f $rrd_file )
    else {
      rrd_create( $act_time, $switch_name, $db_name, $rrd_file, "PORT" );

      my $start_time = $act_time - 5097600;
      my $end_time   = $act_time - 60;
      my $start_date = localtime($start_time);
      my $end_date   = localtime($end_time);

      error("san_rrdupdate.pl : $switch_name : xport and update \($start_date - $end_date\) : START");

      load_total_summary_data( $act_time, $switch_name, $db_name, $rrd_file, $start_time, $end_time );

      error("san_rrdupdate.pl : $switch_name : xport and update \($start_date - $end_date\) : END");

      #my $timestamp_before_sixty_days = $act_time - 5097600;
      ##my $timestamp_before_sixty_days = $act_time - 604800;

      #my $fetch_and_update_start_time = localtime($timestamp_before_sixty_days);
      #my $fetch_and_update_end_time   = localtime($act_time - 60);
      #my $fetch_and_update_act_time   = localtime();

      #error ("san_rrdupdate.pl : $switch_name : fetch and update \($fetch_and_update_start_time - $fetch_and_update_end_time\) : START time $fetch_and_update_act_time ");
      #fetch_data_for_total_summary($act_time, $switch_name, $db_name, $rrd_file, $timestamp_before_sixty_days);
      #$fetch_and_update_act_time   = localtime();
      #error ("san_rrdupdate.pl : $switch_name : fetch and update \($fetch_and_update_start_time - $fetch_and_update_end_time\) : END time $fetch_and_update_act_time ");

    } ## end else [ if ( -f $rrd_file ) ]
  } ## end foreach my $switch_name (@switches_all)

} ## end sub load_total_summary_per_switch

sub load_total_summary_data {

  my $act_time    = shift;
  my $switch_name = shift;
  my $db_name     = shift;
  my $rrd_total   = shift;
  my $start_time  = shift;
  my $end_time    = shift;

  $start_time = int( $start_time / 60 ) * 60;
  $end_time   = int( $end_time / 60 ) * 60;

  # read PORTS.cfg
  open( FHR, "< $wrkdir/$switch_name/PORTS.cfg" ) || error( "file does not exists : $wrkdir/$switch_name/PORTS.cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @ports = <FHR>;
  close(FHR);

  my @items = ( "bytes_tra", "bytes_rec", "frames_tra", "frames_rec", "swFCPortNoTxCredits", "swFCPortRxCrcs", "port_speed", "reserve1", "reserve2", "reserve3", "reserve4", "reserve5", "reserve6" );
  my %data_summary;

  foreach my $item (@items) {
    my $cmd      = "";
    my $STEP     = 60;
    my $max_rows = 85000;
    my $xport    = "xport";

    if ( -f "$wrkdir/$switch_name/SAN-BNA" ) { $STEP = 300; }

    $cmd .= "$xport";
    $cmd .= " --start \\\"$start_time\\\"";
    $cmd .= " --end \\\"$end_time\\\"";
    $cmd .= " --step \\\"$STEP\\\"";
    $cmd .= " --maxrows \\\"$max_rows\\\"";

    my $i       = 0;
    my $cmd_tot = " CDEF:tot_$item=";

    my $ports_found = 0;
    foreach my $line (@ports) {
      chomp($line);
      my ( $port_name, $rrd, undef ) = split( " : ", $line );

      $ports_found++;

      if ( $port_name eq '' || $rrd eq '' ) { next; }
      if ( !-f "$rrd" ) { error( "Could not find $port_name file: $rrd " . __FILE__ . ":" . __LINE__ ); next; }

      $cmd .= " DEF:$item${i}=\\\"$rrd\\\":$item:AVERAGE";

      if ( $ports_found == 1 ) {
        $cmd_tot .= "$item${i},0,+";
      }
      else {
        $cmd_tot .= ",$item${i},+";
      }
      $i++;
    } ## end foreach my $line (@ports)

    $cmd .= "$cmd_tot";
    $cmd .= " XPORT:tot_$item:$item";

    if ( $ports_found == 0 ) {
      error( "Could not find ports for $switch_name in $wrkdir/$switch_name/PORTS.cfg " . __FILE__ . ":" . __LINE__ );
      return 0;
    }
    $cmd =~ s/\\"/"/g;

    RRDp::cmd qq($cmd);
    my $ret = RRDp::read;
    if ( $$ret =~ "ERROR" ) {
      error( "sum detail: Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    my @rrd_result;
    if ( $ret =~ /0x/ ) {
      @rrd_result = split( '\n', $$ret );
    }
    else {
      @rrd_result = split( '\n', $ret );
    }

    foreach my $row (@rrd_result) {
      chomp $row;
      $row =~ s/^\s+//g;
      $row =~ s/\s+$//g;
      if ( $row =~ "^<row><t>" ) {
        my ( $timestamp, $value ) = split( "</t><v>", $row );
        $timestamp =~ s/<row><t>//g;
        $value =~ s/<\/v><\/row>//g;
        if ( $value =~ /nan/i ) {
          $value = 'U';
        }
        else {
          $value = sprintf '%.9g', $value;
        }

        #print "$item : $timestamp : $value\n";
        $data_summary{$timestamp}{$item} = $value;
      } ## end if ( $row =~ "^<row><t>")
    } ## end foreach my $row (@rrd_result)
  } ## end foreach my $item (@items)

  foreach my $time_to_update ( sort { $a <=> $b } keys(%data_summary) ) {
    my $bytes_tra           = 'U';
    my $bytes_rec           = 'U';
    my $frames_tra          = 'U';
    my $frames_rec          = 'U';
    my $swFCPortNoTxCredits = 'U';
    my $swFCPortRxCrcs      = 'U';
    my $port_speed          = 'U';
    my $reserve1            = 'U';
    my $reserve2            = 'U';
    my $reserve3            = 'U';
    my $reserve4            = 'U';
    my $reserve5            = 'U';
    my $reserve6            = 'U';

    if ( defined $data_summary{$time_to_update}{bytes_tra}           && isdigit( $data_summary{$time_to_update}{bytes_tra} ) )           { $bytes_tra           = $data_summary{$time_to_update}{bytes_tra}; }
    if ( defined $data_summary{$time_to_update}{bytes_rec}           && isdigit( $data_summary{$time_to_update}{bytes_rec} ) )           { $bytes_rec           = $data_summary{$time_to_update}{bytes_rec}; }
    if ( defined $data_summary{$time_to_update}{frames_tra}          && isdigit( $data_summary{$time_to_update}{frames_tra} ) )          { $frames_tra          = $data_summary{$time_to_update}{frames_tra}; }
    if ( defined $data_summary{$time_to_update}{frames_rec}          && isdigit( $data_summary{$time_to_update}{frames_rec} ) )          { $frames_rec          = $data_summary{$time_to_update}{frames_rec}; }
    if ( defined $data_summary{$time_to_update}{swFCPortNoTxCredits} && isdigit( $data_summary{$time_to_update}{swFCPortNoTxCredits} ) ) { $swFCPortNoTxCredits = $data_summary{$time_to_update}{swFCPortNoTxCredits}; }
    if ( defined $data_summary{$time_to_update}{swFCPortRxCrcs}      && isdigit( $data_summary{$time_to_update}{swFCPortRxCrcs} ) )      { $swFCPortRxCrcs      = $data_summary{$time_to_update}{swFCPortRxCrcs}; }
    if ( defined $data_summary{$time_to_update}{port_speed}          && isdigit( $data_summary{$time_to_update}{port_speed} ) )          { $port_speed          = $data_summary{$time_to_update}{port_speed}; }
    if ( defined $data_summary{$time_to_update}{reserve1}            && isdigit( $data_summary{$time_to_update}{reserve1} ) )            { $reserve1            = $data_summary{$time_to_update}{reserve1}; }
    if ( defined $data_summary{$time_to_update}{reserve2}            && isdigit( $data_summary{$time_to_update}{reserve2} ) )            { $reserve2            = $data_summary{$time_to_update}{reserve2}; }
    if ( defined $data_summary{$time_to_update}{reserve3}            && isdigit( $data_summary{$time_to_update}{reserve3} ) )            { $reserve3            = $data_summary{$time_to_update}{reserve3}; }
    if ( defined $data_summary{$time_to_update}{reserve4}            && isdigit( $data_summary{$time_to_update}{reserve4} ) )            { $reserve4            = $data_summary{$time_to_update}{reserve4}; }
    if ( defined $data_summary{$time_to_update}{reserve5}            && isdigit( $data_summary{$time_to_update}{reserve5} ) )            { $reserve5            = $data_summary{$time_to_update}{reserve5}; }
    if ( defined $data_summary{$time_to_update}{reserve6}            && isdigit( $data_summary{$time_to_update}{reserve6} ) )            { $reserve6            = $data_summary{$time_to_update}{reserve6}; }

    #print "$time_to_update, $bytes_tra, $bytes_rec, $frames_tra, $frames_rec, $swFCPortNoTxCredits, $swFCPortRxCrcs, $port_speed, $reserve1, $reserve2, $reserve3, $reserve4, $reserve5, $reserve6, $switch_name, $db_name, $rrd_total\n";

    rrd_update( $time_to_update, $bytes_tra, $bytes_rec, $frames_tra, $frames_rec, $swFCPortNoTxCredits, $swFCPortRxCrcs, $port_speed, $reserve1, $reserve2, $reserve3, $reserve4, $reserve5, $reserve6, $switch_name, $db_name, $rrd_total );
  } ## end foreach my $time_to_update ...
} ## end sub load_total_summary_data

sub fetch_data_for_total_summary {

  my $act_time    = shift;
  my $switch_name = shift;
  my $db_name     = shift;
  my $rrd_total   = shift;
  my $start_time  = shift;

  # read PORTS.cfg
  open( FHR, "< $wrkdir/$switch_name/PORTS.cfg" ) || error( "file does not exists : $wrkdir/$switch_name/PORTS.cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files = <FHR>;
  close(FHR);

  my %data_summary;
  my $ports_found = 0;

  foreach my $line (@files) {
    chomp($line);
    my ( $port_name, $rrd, undef ) = split( " : ", $line );

    $ports_found++;

    if ( $port_name eq '' || $rrd eq '' ) { next; }
    if ( !-f "$rrd" ) { error( "Could not find $port_name file: $rrd " . __FILE__ . ":" . __LINE__ ); next; }

    #error ("$port_name : $rrd -> fetch start\n");

    # find last update time
    RRDp::cmd qq(last $rrd);
    my $last_tt        = RRDp::read;
    my $end_time = $$last_tt;
    chomp $last_timestamp;

    if ( -f "$wrkdir/$switch_name/SAN-BNA" ) {
      $end_time = $end_time - 300;
    }
    else {
      $end_time = $end_time - 60;
    }

    if ( $start_time > $end_time ) { next; } # probably some unused port, port was not updated

    RRDp::cmd qq(fetch "$rrd" AVERAGE --start $start_time --end $end_time );
    my $ret = RRDp::read;

    #error ("$port_name : $rrd -> fetch end\n");

    my @rrd_result;
    if ( $ret =~ /0x/ ) {
      @rrd_result = split( '\n', $$ret );
    }
    else {
      @rrd_result = split( '\n', $ret );
    }

    #error ("$port_name : $rrd -> summary start \n");

    my $fetch_line_idx = 0;
    foreach my $fetch_line (@rrd_result) {
      chomp $fetch_line;

      if ( $fetch_line !~ "^[0-9]+: " ) { next; }
      $fetch_line_idx++;

      #error ("$port_name : $fetch_line_idx\n");

      my ( $fetch_timestamp, $bytes_tra, $bytes_rec, $frames_tra, $frames_rec, $swFCPortNoTxCredits, $swFCPortRxCrcs, $port_speed, $reserve1, $reserve2, $reserve3, $reserve4, $reserve5, $reserve6 ) = split( " ", $fetch_line );

      $fetch_timestamp =~ s/:$//;

      if ( $bytes_tra !~ /nan/i )           { $bytes_tra           = sprintf '%.9g', $bytes_tra; }
      if ( $bytes_rec !~ /nan/i )           { $bytes_rec           = sprintf '%.9g', $bytes_rec; }
      if ( $frames_tra !~ /nan/i )          { $frames_tra          = sprintf '%.9g', $frames_tra; }
      if ( $frames_rec !~ /nan/i )          { $frames_rec          = sprintf '%.9g', $frames_rec; }
      if ( $swFCPortNoTxCredits !~ /nan/i ) { $swFCPortNoTxCredits = sprintf '%.9g', $swFCPortNoTxCredits; }
      if ( $swFCPortRxCrcs !~ /nan/i )      { $swFCPortRxCrcs      = sprintf '%.9g', $swFCPortRxCrcs; }
      if ( $port_speed !~ /nan/i )          { $port_speed          = sprintf '%.9g', $port_speed; }
      if ( $reserve1 !~ /nan/i )            { $reserve1            = sprintf '%.9g', $reserve1; }
      if ( $reserve2 !~ /nan/i )            { $reserve2            = sprintf '%.9g', $reserve2; }
      if ( $reserve3 !~ /nan/i )            { $reserve3            = sprintf '%.9g', $reserve3; }
      if ( $reserve4 !~ /nan/i )            { $reserve4            = sprintf '%.9g', $reserve4; }
      if ( $reserve5 !~ /nan/i )            { $reserve5            = sprintf '%.9g', $reserve5; }
      if ( $reserve6 !~ /nan/i )            { $reserve6            = sprintf '%.9g', $reserve6; }

      # bytes_tra
      if ( !defined $data_summary{$fetch_timestamp}{bytes_tra} ) {
        $data_summary{$fetch_timestamp}{bytes_tra} = $bytes_tra;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{bytes_tra} =~ /nan/i  && isdigit($bytes_tra) ) { $data_summary{$fetch_timestamp}{bytes_tra} = $bytes_tra; }
        if ( isdigit( $data_summary{$fetch_timestamp}{bytes_tra} ) && isdigit($bytes_tra) ) { $data_summary{$fetch_timestamp}{bytes_tra} = $data_summary{$fetch_timestamp}{bytes_tra} + $bytes_tra; }
      }

      # bytes_rec
      if ( !defined $data_summary{$fetch_timestamp}{bytes_rec} ) {
        $data_summary{$fetch_timestamp}{bytes_rec} = $bytes_rec;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{bytes_rec} =~ /nan/i  && isdigit($bytes_rec) ) { $data_summary{$fetch_timestamp}{bytes_rec} = $bytes_rec; }
        if ( isdigit( $data_summary{$fetch_timestamp}{bytes_rec} ) && isdigit($bytes_rec) ) { $data_summary{$fetch_timestamp}{bytes_rec} = $data_summary{$fetch_timestamp}{bytes_rec} + $bytes_rec; }
      }

      # frames_tra
      if ( !defined $data_summary{$fetch_timestamp}{frames_tra} ) {
        $data_summary{$fetch_timestamp}{frames_tra} = $frames_tra;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{frames_tra} =~ /nan/i  && isdigit($frames_tra) ) { $data_summary{$fetch_timestamp}{frames_tra} = $frames_tra; }
        if ( isdigit( $data_summary{$fetch_timestamp}{frames_tra} ) && isdigit($frames_tra) ) { $data_summary{$fetch_timestamp}{frames_tra} = $data_summary{$fetch_timestamp}{frames_tra} + $frames_tra; }
      }

      # frames_rec
      if ( !defined $data_summary{$fetch_timestamp}{frames_rec} ) {
        $data_summary{$fetch_timestamp}{frames_rec} = $frames_rec;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{frames_rec} =~ /nan/i  && isdigit($frames_rec) ) { $data_summary{$fetch_timestamp}{frames_rec} = $frames_rec; }
        if ( isdigit( $data_summary{$fetch_timestamp}{frames_rec} ) && isdigit($frames_rec) ) { $data_summary{$fetch_timestamp}{frames_rec} = $data_summary{$fetch_timestamp}{frames_rec} + $frames_rec; }
      }

      # swFCPortNoTxCredits
      if ( !defined $data_summary{$fetch_timestamp}{swFCPortNoTxCredits} ) {
        $data_summary{$fetch_timestamp}{swFCPortNoTxCredits} = $swFCPortNoTxCredits;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{swFCPortNoTxCredits} =~ /nan/i  && isdigit($swFCPortNoTxCredits) ) { $data_summary{$fetch_timestamp}{swFCPortNoTxCredits} = $swFCPortNoTxCredits; }
        if ( isdigit( $data_summary{$fetch_timestamp}{swFCPortNoTxCredits} ) && isdigit($swFCPortNoTxCredits) ) { $data_summary{$fetch_timestamp}{swFCPortNoTxCredits} = $data_summary{$fetch_timestamp}{swFCPortNoTxCredits} + $swFCPortNoTxCredits; }
      }

      # swFCPortRxCrcs
      if ( !defined $data_summary{$fetch_timestamp}{swFCPortRxCrcs} ) {
        $data_summary{$fetch_timestamp}{swFCPortRxCrcs} = $swFCPortRxCrcs;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{swFCPortRxCrcs} =~ /nan/i  && isdigit($swFCPortRxCrcs) ) { $data_summary{$fetch_timestamp}{swFCPortRxCrcs} = $swFCPortRxCrcs; }
        if ( isdigit( $data_summary{$fetch_timestamp}{swFCPortRxCrcs} ) && isdigit($swFCPortRxCrcs) ) { $data_summary{$fetch_timestamp}{swFCPortRxCrcs} = $data_summary{$fetch_timestamp}{swFCPortRxCrcs} + $swFCPortRxCrcs; }
      }

      # port_speed
      if ( !defined $data_summary{$fetch_timestamp}{port_speed} ) {
        $data_summary{$fetch_timestamp}{port_speed} = $port_speed;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{port_speed} =~ /nan/i  && isdigit($port_speed) ) { $data_summary{$fetch_timestamp}{port_speed} = $port_speed; }
        if ( isdigit( $data_summary{$fetch_timestamp}{port_speed} ) && isdigit($port_speed) ) { $data_summary{$fetch_timestamp}{port_speed} = $data_summary{$fetch_timestamp}{port_speed} + $port_speed; }
      }

      # reserve1
      if ( !defined $data_summary{$fetch_timestamp}{reserve1} ) {
        $data_summary{$fetch_timestamp}{reserve1} = $reserve1;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{reserve1} =~ /nan/i  && isdigit($reserve1) ) { $data_summary{$fetch_timestamp}{reserve1} = $reserve1; }
        if ( isdigit( $data_summary{$fetch_timestamp}{reserve1} ) && isdigit($reserve1) ) { $data_summary{$fetch_timestamp}{reserve1} = $data_summary{$fetch_timestamp}{reserve1} + $reserve1; }
      }

      # reserve2
      if ( !defined $data_summary{$fetch_timestamp}{reserve2} ) {
        $data_summary{$fetch_timestamp}{reserve2} = $reserve2;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{reserve2} =~ /nan/i  && isdigit($reserve2) ) { $data_summary{$fetch_timestamp}{reserve2} = $reserve2; }
        if ( isdigit( $data_summary{$fetch_timestamp}{reserve2} ) && isdigit($reserve2) ) { $data_summary{$fetch_timestamp}{reserve2} = $data_summary{$fetch_timestamp}{reserve2} + $reserve2; }
      }

      # reserve3
      if ( !defined $data_summary{$fetch_timestamp}{reserve3} ) {
        $data_summary{$fetch_timestamp}{reserve3} = $reserve3;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{reserve3} =~ /nan/i  && isdigit($reserve3) ) { $data_summary{$fetch_timestamp}{reserve3} = $reserve3; }
        if ( isdigit( $data_summary{$fetch_timestamp}{reserve3} ) && isdigit($reserve3) ) { $data_summary{$fetch_timestamp}{reserve3} = $data_summary{$fetch_timestamp}{reserve3} + $reserve3; }
      }

      # reserve4
      if ( !defined $data_summary{$fetch_timestamp}{reserve4} ) {
        $data_summary{$fetch_timestamp}{reserve4} = $reserve4;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{reserve4} =~ /nan/i  && isdigit($reserve4) ) { $data_summary{$fetch_timestamp}{reserve4} = $reserve4; }
        if ( isdigit( $data_summary{$fetch_timestamp}{reserve4} ) && isdigit($reserve4) ) { $data_summary{$fetch_timestamp}{reserve4} = $data_summary{$fetch_timestamp}{reserve4} + $reserve4; }
      }

      # reserve5
      if ( !defined $data_summary{$fetch_timestamp}{reserve5} ) {
        $data_summary{$fetch_timestamp}{reserve5} = $reserve5;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{reserve5} =~ /nan/i  && isdigit($reserve5) ) { $data_summary{$fetch_timestamp}{reserve5} = $reserve5; }
        if ( isdigit( $data_summary{$fetch_timestamp}{reserve5} ) && isdigit($reserve5) ) { $data_summary{$fetch_timestamp}{reserve5} = $data_summary{$fetch_timestamp}{reserve5} + $reserve5; }
      }

      # reserve6
      if ( !defined $data_summary{$fetch_timestamp}{reserve6} ) {
        $data_summary{$fetch_timestamp}{reserve6} = $reserve6;
      }
      else {
        if ( $data_summary{$fetch_timestamp}{reserve6} =~ /nan/i  && isdigit($reserve6) ) { $data_summary{$fetch_timestamp}{reserve6} = $reserve6; }
        if ( isdigit( $data_summary{$fetch_timestamp}{reserve6} ) && isdigit($reserve6) ) { $data_summary{$fetch_timestamp}{reserve6} = $data_summary{$fetch_timestamp}{reserve6} + $reserve6; }
      }

    } ## end foreach my $fetch_line (@rrd_result)

    #error ("$port_name : $rrd -> summary end \n");

  } ## end foreach my $line (@files)

  #print "$switch_name\n";
  #print Dumper(\%data_summary);

  #error ("-> update start \n");

  foreach my $time_to_update ( sort { $a <=> $b } keys(%data_summary) ) {
    my $bytes_tra           = 'U';
    my $bytes_rec           = 'U';
    my $frames_tra          = 'U';
    my $frames_rec          = 'U';
    my $swFCPortNoTxCredits = 'U';
    my $swFCPortRxCrcs      = 'U';
    my $port_speed          = 'U';
    my $reserve1            = 'U';
    my $reserve2            = 'U';
    my $reserve3            = 'U';
    my $reserve4            = 'U';
    my $reserve5            = 'U';
    my $reserve6            = 'U';

    if ( defined $data_summary{$time_to_update}{bytes_tra}           && isdigit( $data_summary{$time_to_update}{bytes_tra} ) )           { $bytes_tra           = $data_summary{$time_to_update}{bytes_tra}; }
    if ( defined $data_summary{$time_to_update}{bytes_rec}           && isdigit( $data_summary{$time_to_update}{bytes_rec} ) )           { $bytes_rec           = $data_summary{$time_to_update}{bytes_rec}; }
    if ( defined $data_summary{$time_to_update}{frames_tra}          && isdigit( $data_summary{$time_to_update}{frames_tra} ) )          { $frames_tra          = $data_summary{$time_to_update}{frames_tra}; }
    if ( defined $data_summary{$time_to_update}{frames_rec}          && isdigit( $data_summary{$time_to_update}{frames_rec} ) )          { $frames_rec          = $data_summary{$time_to_update}{frames_rec}; }
    if ( defined $data_summary{$time_to_update}{swFCPortNoTxCredits} && isdigit( $data_summary{$time_to_update}{swFCPortNoTxCredits} ) ) { $swFCPortNoTxCredits = $data_summary{$time_to_update}{swFCPortNoTxCredits}; }
    if ( defined $data_summary{$time_to_update}{swFCPortRxCrcs}      && isdigit( $data_summary{$time_to_update}{swFCPortRxCrcs} ) )      { $swFCPortRxCrcs      = $data_summary{$time_to_update}{swFCPortRxCrcs}; }
    if ( defined $data_summary{$time_to_update}{port_speed}          && isdigit( $data_summary{$time_to_update}{port_speed} ) )          { $port_speed          = $data_summary{$time_to_update}{port_speed}; }
    if ( defined $data_summary{$time_to_update}{reserve1}            && isdigit( $data_summary{$time_to_update}{reserve1} ) )            { $reserve1            = $data_summary{$time_to_update}{reserve1}; }
    if ( defined $data_summary{$time_to_update}{reserve2}            && isdigit( $data_summary{$time_to_update}{reserve2} ) )            { $reserve2            = $data_summary{$time_to_update}{reserve2}; }
    if ( defined $data_summary{$time_to_update}{reserve3}            && isdigit( $data_summary{$time_to_update}{reserve3} ) )            { $reserve3            = $data_summary{$time_to_update}{reserve3}; }
    if ( defined $data_summary{$time_to_update}{reserve4}            && isdigit( $data_summary{$time_to_update}{reserve4} ) )            { $reserve4            = $data_summary{$time_to_update}{reserve4}; }
    if ( defined $data_summary{$time_to_update}{reserve5}            && isdigit( $data_summary{$time_to_update}{reserve5} ) )            { $reserve5            = $data_summary{$time_to_update}{reserve5}; }
    if ( defined $data_summary{$time_to_update}{reserve6}            && isdigit( $data_summary{$time_to_update}{reserve6} ) )            { $reserve6            = $data_summary{$time_to_update}{reserve6}; }

    #print "$time_to_update, $bytes_tra, $bytes_rec, $frames_tra, $frames_rec, $swFCPortNoTxCredits, $swFCPortRxCrcs, $port_speed, $reserve1, $reserve2, $reserve3, $reserve4, $reserve5, $reserve6, $switch_name, $db_name, $rrd_total\n";

    rrd_update( $time_to_update, $bytes_tra, $bytes_rec, $frames_tra, $frames_rec, $swFCPortNoTxCredits, $swFCPortRxCrcs, $port_speed, $reserve1, $reserve2, $reserve3, $reserve4, $reserve5, $reserve6, $switch_name, $db_name, $rrd_total );
  } ## end foreach my $time_to_update ...

  #error ("-> update end \n");

} ## end sub fetch_data_for_total_summary

sub rrd_create {

  my $act_time    = shift;
  my $switch_name = shift;
  my $db_name     = shift;
  my $rrd         = shift;
  my $type        = shift;
  my $STEP        = 60;
  my $data_type   = "COUNTER";

  touch( $switch_name, $rrd );

  # only for demo site!
  if ( $demo == 1 ) { $data_type = "GAUGE"; }

  my $time    = $act_time - $STEP;    # start time lower than actual one being updated
  my $no_time = $STEP * 15;           # says the time interval when RRDTOOL considers a gap in input data

  # rrdfiles for total summary per switch must use data type GAUGE
  if ( $db_name eq "total_summary.rrs" ) {
    $data_type = "GAUGE";
    $time      = $time - 5097600;
  }

  # standard data retentions
  my $one_minute_sample = 86400;
  my $five_mins_sample  = 25920;
  my $one_hour_sample   = 4320;
  my $five_hours_sample = 1734;
  my $one_day_sample    = 1080;

  if ( $type eq "PORT" ) {
    # BNA
    # rrd_update_switch( $act_time, $cpu, $mem, $resp_t, $reserve1, $reserve2, $switch_name, $db_name, $rrd_file );
    if ( -f "$wrkdir/$switch_name/SAN-BNA" ) {
      $data_type = "GAUGE";
      $STEP      = 300;
      $no_time   = $STEP * 4;

      my $first_rra  = 1;      # 5min
      my $second_rra = 12;     # 1h
      my $third_rra  = 72;     # 5 h
      my $forth_rra  = 288;    # 1day

      RRDp::cmd qq(create "$rrd"  --start "$time"  --step "$STEP"
      "DS:bytes_tra:$data_type:$no_time:0:102400000000"
      "DS:bytes_rec:$data_type:$no_time:0:102400000000"
      "DS:frames_tra:$data_type:$no_time:0:102400000000"
      "DS:frames_rec:$data_type:$no_time:0:102400000000"
      "DS:swFCPortNoTxCredits:$data_type:$no_time:0:102400000000"
      "DS:swFCPortRxCrcs:$data_type:$no_time:0:102400000000"
      "DS:port_speed:GAUGE:$no_time:0:102400000000"
      "DS:reserve1:$data_type:$no_time:0:102400000000"
      "DS:reserve2:$data_type:$no_time:0:102400000000"
      "DS:reserve3:$data_type:$no_time:0:102400000000"
      "DS:reserve4:$data_type:$no_time:0:102400000000"
      "DS:reserve5:$data_type:$no_time:0:102400000000"
      "DS:reserve6:$data_type:$no_time:0:102400000000"
      "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
      "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
      "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
      "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
      );
      if ( !Xorux_lib::create_check("file: $rrd, $five_mins_sample, $one_hour_sample, $five_hours_sample, $one_day_sample") ) {
        error( "unable to create $rrd : at " . __FILE__ . ": line " . __LINE__ );
        RRDp::end;
        RRDp::start "$rrdtool";
        return 0;
      } ## end if ( !Xorux_lib::create_check...)
    }
    else {
      RRDp::cmd qq(create "$rrd"  --start "$time"  --step "$STEP"
      "DS:bytes_tra:$data_type:$no_time:0:102400000000"
      "DS:bytes_rec:$data_type:$no_time:0:102400000000"
      "DS:frames_tra:$data_type:$no_time:0:102400000000"
      "DS:frames_rec:$data_type:$no_time:0:102400000000"
      "DS:swFCPortNoTxCredits:$data_type:$no_time:0:102400000000"
      "DS:swFCPortRxCrcs:$data_type:$no_time:0:102400000000"
      "DS:port_speed:GAUGE:$no_time:0:102400000000"
      "DS:reserve1:$data_type:$no_time:0:102400000000"
      "DS:reserve2:$data_type:$no_time:0:102400000000"
      "DS:reserve3:$data_type:$no_time:0:102400000000"
      "DS:reserve4:$data_type:$no_time:0:102400000000"
      "DS:reserve5:$data_type:$no_time:0:102400000000"
      "DS:reserve6:$data_type:$no_time:0:102400000000"
      "RRA:AVERAGE:0.5:1:$one_minute_sample"
      "RRA:AVERAGE:0.5:5:$five_mins_sample"
      "RRA:AVERAGE:0.5:60:$one_hour_sample"
      "RRA:AVERAGE:0.5:300:$five_hours_sample"
      "RRA:AVERAGE:0.5:1440:$one_day_sample"
      );
      if ( !Xorux_lib::create_check("file: $rrd, $one_minute_sample, $five_mins_sample, $one_hour_sample, $five_hours_sample, $one_day_sample") ) {
        error( "unable to create $rrd : at " . __FILE__ . ": line " . __LINE__ );
        RRDp::end;
        RRDp::start "$rrdtool";
        return 0;
      } ## end if ( !Xorux_lib::create_check...)
    }
  }

  if ( $type eq "SWITCH" ) {
    # BNA
    # rrd_update_switch( $act_time, $cpu, $mem, $resp_t, $reserve1, $reserve2, $switch_name, $db_name, $rrd_file );
    if ( -f "$wrkdir/$switch_name/SAN-BNA" ) {
      $data_type = "GAUGE";
      $STEP      = 300;
      $no_time   = $STEP * 4;

      my $first_rra  = 1;      # 5min
      my $second_rra = 12;     # 1h
      my $third_rra  = 72;     # 5 h
      my $forth_rra  = 288;    # 1day

      RRDp::cmd qq(create "$rrd"  --start "$time"  --step "$STEP"
      "DS:cpu:$data_type:$no_time:0:200"
      "DS:mem:$data_type:$no_time:0:200"
      "DS:resp_t:$data_type:$no_time:0:102400000000"
      "DS:reserve1:$data_type:$no_time:0:102400000000"
      "DS:reserve2:$data_type:$no_time:0:102400000000"
      "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
      "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
      "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
      "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
      );
      if ( !Xorux_lib::create_check("file: $rrd, $five_mins_sample, $one_hour_sample, $five_hours_sample, $one_day_sample") ) {
        error( "unable to create $rrd : at " . __FILE__ . ": line " . __LINE__ );
        RRDp::end;
        RRDp::start "$rrdtool";
        return 0;
      } ## end if ( !Xorux_lib::create_check...)
    }
  }

  return 1;
} ## end sub rrd_create

sub rrd_update_switch {

  my $act_time    = shift;
  my $cpu         = shift;
  my $mem         = shift;
  my $resp_t      = shift;
  my $reserve1    = shift;
  my $reserve2    = shift;
  my $switch_name = shift;
  my $db_name     = shift;
  my $rrd         = shift;
  my $last_rec    = "";

  if ( !defined($cpu)      || $cpu eq '' )      { $cpu      = 'U'; }
  if ( !defined($mem)      || $mem eq '' )      { $mem      = 'U'; }
  if ( !defined($resp_t)   || $resp_t eq '' )   { $resp_t   = 'U'; }
  if ( !defined($reserve1) || $reserve1 eq '' ) { $reserve1 = 'U'; }
  if ( !defined($reserve2) || $reserve2 eq '' ) { $reserve2 = 'U'; }

  eval {
    RRDp::cmd qq(last "$rrd" );
    $last_rec = RRDp::read;
  };
  if ($@) {
    error("failed during read last time $rrd");
    return 0;
  }
  chomp($$last_rec);
  if ( isdigit($$last_rec) && $$last_rec >= $act_time ) {

    #error ("bad update time! $$last_rec >= $act_time");
    return 0;
  }

  RRDp::cmd qq(update "$rrd" $act_time:$cpu:$mem:$resp_t:$reserve1:$reserve2);

  my $answer = RRDp::read;
  if ( !$$answer eq '' && $$answer =~ m/ERROR/ ) {
    error(" $switch_name: $rrd : $act_time:$cpu:$mem:$resp_t:$reserve1:$reserve2 ... : $$answer");
    if ( $$answer =~ m/is not an RRD file/ ) {
      ( my $err, my $file, my $txt ) = split( /'/, $$answer );
      error("Removing as it seems to be corrupted: $rrd");
      unlink("$rrd") || error("Cannot rm $rrd : $!");
    }
    return 0;
  } ## end if ( !$$answer eq '' &&...)
}

sub rrd_update {

  my $act_time            = shift;
  my $bytes_tra           = shift;
  my $bytes_rec           = shift;
  my $frames_tra          = shift;
  my $frames_rec          = shift;
  my $swFCPortNoTxCredits = shift;
  my $swFCPortRxCrcs      = shift;
  my $port_speed          = shift;
  my $reserve1            = shift;
  my $reserve2            = shift;
  my $reserve3            = shift;
  my $reserve4            = shift;
  my $reserve5            = shift;
  my $reserve6            = shift;
  my $switch_name         = shift;
  my $db_name             = shift;
  my $rrd                 = shift;
  my $last_rec            = "";

  if ( !defined($bytes_tra)           || $bytes_tra eq '' )           { $bytes_tra           = 'U'; }
  if ( !defined($bytes_rec)           || $bytes_rec eq '' )           { $bytes_rec           = 'U'; }
  if ( !defined($frames_tra)          || $frames_tra eq '' )          { $frames_tra          = 'U'; }
  if ( !defined($frames_rec)          || $frames_rec eq '' )          { $frames_rec          = 'U'; }
  if ( !defined($swFCPortNoTxCredits) || $swFCPortNoTxCredits eq '' ) { $swFCPortNoTxCredits = 'U'; }
  if ( !defined($swFCPortRxCrcs)      || $swFCPortRxCrcs eq '' )      { $swFCPortRxCrcs      = 'U'; }
  if ( !defined($port_speed)          || $port_speed eq '' )          { $port_speed          = 'U'; }
  if ( !defined($reserve1)            || $reserve1 eq '' )            { $reserve1            = 'U'; }
  if ( !defined($reserve2)            || $reserve2 eq '' )            { $reserve2            = 'U'; }
  if ( !defined($reserve3)            || $reserve3 eq '' )            { $reserve3            = 'U'; }
  if ( !defined($reserve4)            || $reserve4 eq '' )            { $reserve4            = 'U'; }
  if ( !defined($reserve5)            || $reserve5 eq '' )            { $reserve5            = 'U'; }
  if ( !defined($reserve6)            || $reserve6 eq '' )            { $reserve6            = 'U'; }

  #print "update $rrd $act_time:$bytes_tra:$bytes_rec:$frames_tra:$frames_rec:$swFCPortNoTxCredits:$swFCPortRxCrcs:$port_speed:$reserve1:$reserve2:$reserve3:$reserve4:$reserve5:$reserve6\n";
  #error ("update $rrd $act_time:$bytes_tra:$bytes_rec:$frames_tra:$frames_rec:$swFCPortNoTxCredits:$swFCPortRxCrcs:$port_speed:$reserve1:$reserve2:$reserve3:$reserve4:$reserve5:$reserve6\n");

  eval {
    RRDp::cmd qq(last "$rrd" );
    $last_rec = RRDp::read;
  };
  if ($@) {
    error("failed during read last time $rrd");
    return 0;
  }
  chomp($$last_rec);
  if ( isdigit($$last_rec) && $$last_rec >= $act_time ) {

    #error ("bad update time! $$last_rec >= $act_time");
    return 0;
  }

  RRDp::cmd qq(update "$rrd" $act_time:$bytes_tra:$bytes_rec:$frames_tra:$frames_rec:$swFCPortNoTxCredits:$swFCPortRxCrcs:$port_speed:$reserve1:$reserve2:$reserve3:$reserve4:$reserve5:$reserve6);

  my $answer = RRDp::read;
  if ( !$$answer eq '' && $$answer =~ m/ERROR/ ) {
    error(" $switch_name: $rrd : $act_time:$bytes_tra:$bytes_rec:$frames_tra:$frames_rec:$swFCPortNoTxCredits:$swFCPortRxCrcs:$port_speed:$reserve1:$reserve2:$reserve3:$reserve4:$reserve5:$reserve6 ... : $$answer");
    if ( $$answer =~ m/is not an RRD file/ ) {
      ( my $err, my $file, my $txt ) = split( /'/, $$answer );
      error("Removing as it seems to be corrupted: $rrd");
      unlink("$rrd") || error("Cannot rm $rrd : $!");
    }
    return 0;
  } ## end if ( !$$answer eq '' &&...)
} ## end sub rrd_update

sub touch {
  my $switch_name = shift;
  my $rrd_file    = shift;

  my $version    = $ENV{version};
  my $basedir    = $ENV{INPUTDIR};
  my $new_change = "$basedir/tmp/$version-run";
  my $DEBUG      = $ENV{DEBUG};
  my $host       = "$switch_name : $rrd_file";

  if ( $host eq '' ) {
    $host = "na";
  }

  if ( !-f $new_change ) {
    `touch $new_change`;    # say install_html.sh that there was any change
                            #print "touch          : $host $new_change\n" if $DEBUG ;
    error("touch          : $host $new_change");
  }

  return 0;
} ## end sub touch

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  #print "$act_time: $text : $!\n" if $DEBUG > 2;;

  return 1;
} ## end sub error

sub isdigit {
  my $digit = shift;

  if ( !defined($digit) || $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  return 0;
} ## end sub isdigit

