package LoadDataModule;    #
use strict;
use Date::Parse;
use RRDp;
use File::Copy;
use File::Compare;
use Xorux_lib;
use AlertStor2rrd;
use POSIX qw(strftime);
use Data::Dumper;

my $rrdtool = $ENV{RRDTOOL};

# standard data retentions
# note this is set up once more in load_retentions
my $five_mins_sample  = 25920;
my $one_hour_sample   = 4320;
my $five_hours_sample = 1734;    # in fact 6 hours
my $one_day_sample    = 1080;

# must be Global here to persist load through more files
my @rank_name         = "";
my @rank_time         = "";
my @port_name         = "";
my @port_time         = "";
my @pool_name         = "";
my @pool_time         = "";
my @mdisk_name        = "";
my @mdisk_time        = "";
my @node_name         = "";
my @node_time         = "";
my @vol_name          = "";
my @vol_time          = "";
my @ldev_name         = "";
my @ldev_time         = "";
my @lun_name          = "";
my @lun_time          = "";
my @host_name         = "";
my @host_time         = "";
my @cpuc_name         = "";
my @cpuc_time         = "";
my @cache_name        = "";
my @cache_time        = "";
my @cachen_name       = "";
my @cachen_time       = "";
my @cpun_name         = "";
my @cpun_time         = "";
my @drive_name        = "";
my @drive_time        = "";
my @pool_cap_name     = "";
my @pool_cap_time     = "";
my @volume_cache_name = "";
my @volume_cache_time = "";
my @cifs_name         = "";
my @cifs_time         = "";
my @nfs_name          = "";
my @nfs_time          = "";
my @fs_name           = "";
my @fs_time           = "";
my @tier_name         = "";
my @tier_time         = "";
my %hosts;
my @volume_name = "";
my @volume_time = "";
my @qtree_name  = "";
my @qtree_time  = "";
my @volfolder_time = "";
my @volfolder_name = "";
my @hgroup_time = "";
my @hgroup_name = "";
my @io_group_time = "";
my @io_group_name = "";

# how long keep data
my $KEEP_OUT_FILES_HOURS = $ENV{KEEP_OUT_FILES_HOURS};
if ( !defined($KEEP_OUT_FILES_HOURS) || $KEEP_OUT_FILES_HOURS eq '' || isdigit($KEEP_OUT_FILES_HOURS) == 0 ) {
  $KEEP_OUT_FILES_HOURS = 0;
}
else {
  #conversion to seconds
  $KEEP_OUT_FILES_HOURS = $KEEP_OUT_FILES_HOURS * 3600;
}
my $KEEP_OUT_FILES = $ENV{KEEP_OUT_FILES};

# For external data processing
my $DO_NOT_SAVE_DATA = $ENV{DO_NOT_SAVE_DATA};
if ( !defined($DO_NOT_SAVE_DATA) || $DO_NOT_SAVE_DATA eq '' ) {
  $DO_NOT_SAVE_DATA = 0;    # normall processing
}

###################################################
#  DS3000/4000/5000
###################################################

sub load_data_ds5_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;               # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # at first config files like xivconf_20130904_115351.out
  #

  my $perf_string = "ds5conf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_ds5_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # now data files like xiv_svcperf_20130904_115351.out
  #

  $perf_string = "ds5perf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_ds5_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_ds5_all

sub load_data_ds5_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $pool_file_write   = 0;                             # true if any write there
  my $config_file_write = 0;                             # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD> 
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > 
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name           = "";
  my @pool_name_id        = "";
  my @pool_capacity_tot   = "";
  my @pool_capacity_free  = "";
  my @pool_cfg_id         = "";
  my @rank_cfg_id         = "";
  my $pool_indx           = 0;
  my $cfg_print           = 1;
  my $pool_cfg_processing = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/DS5K type : / ) {

      # read storage version v1 vrs v2
      ( my $trash, my $st_version ) = split( /:/, $line );
      if ( defined($st_version) && !$st_version eq '' ) {
        $st_version =~ s/ //g;
        $st_version =~ s/v//g;
        if ( isdigit($st_version) == 1 && !-f "$wrkdir/$host/$st_type-v$st_version" ) {
          `touch "$wrkdir/$host/$st_type-v$st_version"`;
        }
      } ## end if ( defined($st_version...))
      next;
    } ## end if ( $line =~ m/DS5K type : /)
  }    # foreach

  print FCFG "</pre></body></html>\n";
  close(FCFG);

  # POOL table is created in ds5_tmp as only there is know pool ID for DS5K!!!
  #if ($counter_ins) {
  #  print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG ;
  #}

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  #open(FHR, "< $pool_file-tmp") || main::error ("Can't open $pool_file-tmp : $! ".__FILE__.":".__LINE__) && return 0;
  #@lines = <FHR>;
  #close(FHR);
  #my @lines_write = sort { (split ':', $a)[0] <=> (split ':', $b)[0] } @lines; #numeric sorting

  #foreach $line (@lines) {
  #  # mean there is at least one row, --> replace
  #  open(FHR, "> $pool_file") || main::error ("Can't open $pool_file : $! ".__FILE__.":".__LINE__) && return 0;
  #  foreach my $line_write (@lines_write) {
  #    print FHR "$line_write";
  #    #print "$line_write";
  #  }
  #  close (FHR);
  #  print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG ;
  #  unlink ("$pool_file-tmp");
  #  last;
  #}

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Volume Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 0 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);
  return 0;
} ## end sub load_data_ds5_conf

sub load_data_ds5_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $time             = 3600;
  my $time_last_ok     = 3600;
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                          # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html
  my $pool_file        = "$wrkdir/$host/pool.cfg";
  my @pool_name_list   = "";
  if ( -f $pool_file ) {
    open( FHP, "< $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @pool_name_list = <FHP>;
    close(FHP);
  }

  my $ds5_v2 = 0;
  if ( -f "$wrkdir/$host/DS5K-v2" ) {
    $ds5_v2 = 1;                                     # DS5k v1 does not have response time
  }

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                         # true if any write there
  my $config_file_write = 0;                         # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id        = "";
  my @pool_name          = "";
  my @pool_name_id       = "";
  my $pool_name_indx     = 0;
  my $port_type_once     = 0;
  my $volume_cache_once  = 0;
  my $error_once         = 0;
  my $pool_table_created = 0;

  # pool fron-end data from volume data
  my @pool_data_rate = "";
  my @pool_io_rate   = "";
  my @pool_resp_t    = "";
  my @pool_ids       = "";
  my $pool_time      = 0;
  my $pool_ltime     = 0;
  my @none           = "";

  foreach $line (@lines) {
    chomp($line);

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      $type = uc($type_temp);
      next;
    }
    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }

    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/	//g;
      $line =~ s/Interval End:     //g;
      $line =~ s/GMT.*//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( !defined($time) || isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ m/^$/ ) {
      $type = "NA";      # clear out type when empty line
      if ( $pool_time > 0 ) {

        # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
        if ( $pool_time > $pool_ltime ) {
          if ( $ds5_v2 == 0 ) {
            @pool_resp_t = "";    # DS5k v1 does not have response time info at all
          }
          my @none = "";
          $pool_ltime = pool_sum_insert(
            $type,  $wrkdir, $step,  $DEBUG, $host,  $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@none, \@none, \@none, \@none, \@none, \@none, \@pool_ids, \@pool_data_rate, \@pool_io_rate, \@pool_resp_t,
            \@none, \@none,  \@none, \@none, \@none, \@none,   \@none,    \@none,   \@none,     \@none,      \@none, \@none, \@none, \@none, \@none
          );

        } ## end if ( $pool_time > $pool_ltime)

        # reinitialize structures
        # reinitialize structures
        @pool_ids       = "";
        @pool_data_rate = "";
        @pool_io_rate   = "";
        @pool_resp_t    = "";
        $pool_time      = 0;
      } ## end if ( $pool_time > 0 )
      next;
    } ## end if ( $line =~ m/^$/ )

    if ( $line =~ /^name,id/ || $line =~ m/^  Interval / || $line =~ m/^	Interval/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }

    # Volume
    ########################

    if ( $type =~ m/^VOLUME$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( my $volume_id, my $t1,                 my $t2,         my $volume_name,   my $pool_name, my $controller, my $total_io, my $total_io_rate, my $total_data_rate, my $read_hits, my $write_hits,
        my $read_pct,  my $ssd_read_cache_hit, my $io_latency, my $cache_hits_v1, my $used_cap
      ) = split( /,/, $line );

      # Volume ID,Time,Interval (s),Volume Name,Pool Name,Controler,Total IOs,Total IO Rate (IO/s),Total Data Rate (KB/s),Read Hits,Write Hits,Cache read %,SSD Read Cache Hit %,IO Latency,Cache hits, Capacity (MB)
      # v1
      #60080e500018450a0000af3a4dccbc49,2015:04:07T10:55:10,300,ADS11_ASRV11LPAR10_DATA0,ADS11_SATA_ARRAY1,B,57.0,0.2,29.1,,,100.0,,,84.2,1024000
      # v2
      #60080e50003e33340000029d54dca3b3,2015:04:07T11:05:11,300,ADS12_CLUSTER_ACCEPT_disk1,Disk_Pool_1,B,322.0,1.0,0,0.0,100.0,,0.0,0.2,750000

      #print "000 $line : $total_io\n";

      if ( !defined($pool_name) || $pool_name eq '' ) {

        # it might happen for some mirror repository
        #if ( $error_once == 0 ) {
        #  main::error ("$host:$type - pool name is not defined: $pool_name , reported only once  : $line".__FILE__.":".__LINE__);
        #  $error_once++;
        #}
        next;
      } ## end if ( !defined($pool_name...))

      if ( !defined($controller) || $controller eq '' ) {
        if ( $error_once == 0 ) {
          main::error( "$host:$type - controller name is not defined: $controller , reported only once  : $line" . __FILE__ . ":" . __LINE__ );
          $error_once++;
        }
        next;
      } ## end if ( !defined($controller...))

      $pool_name =~ s/===coma===/,/g;
      if ( !defined($read_hits) || $read_hits eq '' ) {
        $read_hits = 'U';
      }
      if ( !defined($write_hits) || $write_hits eq '' ) {
        $write_hits = 'U';
      }
      if ( !defined($read_pct) || $read_pct eq '' ) {
        $read_pct = 'U';
      }
      if ( !defined($ssd_read_cache_hit) || $ssd_read_cache_hit eq '' ) {
        $ssd_read_cache_hit = 'U';
      }
      if ( !defined($io_latency) || $io_latency eq '' ) {
        $io_latency = 'U';
      }
      if ( !defined($total_io) || $total_io eq '' ) {
        $total_io = 'U';
      }
      if ( !$controller eq '' && $controller =~ m/^[a|A]$/ ) {
        $controller = 0;
      }
      if ( !$controller eq '' && $controller =~ m/^[b|B]$/ ) {
        $controller = 1;
      }
      if ( main::isdigit($controller) == 0 ) {
        if ( $error_once == 0 ) {
          main::error( "$host:$type - controler info is not found : $controller , reported only once  : $line" . __FILE__ . ":" . __LINE__ );
          $error_once++;
        }
        next;
      } ## end if ( main::isdigit($controller...))

      #Volume ID,Time,Interval (s),Volume Name,Pool Name,Controler,Total IOs,Total IO Rate (IO/s),Total Data Rate (KB/s),Read Hits,Write Hits

      if ( !defined($volume_id) || $volume_id eq '' || !main::ishexa($volume_id) ) {
        if ( $error_once == 0 ) {
          main::error( "$host:$type - volume ID is not a digit or hexa: $volume_id , reported only once  : $line" . __FILE__ . ":" . __LINE__ );
          $error_once++;
        }
        next;
      } ## end if ( !defined($volume_id...))

      #find pool_id
      my $pool_id       = -1;
      my $pool_indx_tmp = 0;

      #print "001 $volume_id \n";
      foreach my $line_p (@pool_name_list) {
        chomp($line_p);
        if ( !defined($line_p) || $line_p eq '' || $line_p =~ m/^ *$/ ) {
          next;
        }
        ( my $pool_id_tmp, my $pool_name_tmp, my $volumes_ids_tmp ) = split( /:/, $line_p );
        if ( !defined($volumes_ids_tmp) || $volumes_ids_tmp eq '' ) {
          next;
        }
        if ( $volumes_ids_tmp =~ m/$volume_id,/ ) {
          $pool_id = $pool_id_tmp;

          # last; let it go through all pools to find last record index
        }
        if ( $pool_name eq $pool_name_tmp ) {
          $pool_id = $pool_id_tmp;
          $pool_name_list[$pool_indx_tmp] = $line_p . $volume_id . ",";    # add a volume id to the pool name
          $pool_table_created++;
          last;
        } ## end if ( $pool_name eq $pool_name_tmp)
        $pool_indx_tmp++;
      } ## end foreach my $line_p (@pool_name_list)
      if ( $pool_id == -1 ) {

        # set pool id
        $pool_name_list[$pool_indx_tmp] = "$pool_indx_tmp:$pool_name:$volume_id,";
        $pool_id = $pool_indx_tmp;
        $pool_table_created++;
      } ## end if ( $pool_id == -1 )

      #print "001 $pool_id : $wrkdir/$host/$type/$volume_id-P$pool_id.rrd \n";
      #print "001 $pool_indx_tmp : $pool_name_list[$pool_indx_tmp] \n";

      if ( !defined($pool_id) || $pool_id eq '' || !main::isdigit($pool_id) || $pool_id == -1 ) {
        if ( $error_once == 0 ) {
          main::error( "$host:$type - pool id issue: $pool_id (it should be a digit > 1) , reported only once : $line " . __FILE__ . ":" . __LINE__ );
          $error_once++;
        }
        next;
      } ## end if ( !defined($pool_id...))

      $rrd = "$wrkdir/$host/$type/$volume_id-P$pool_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $rrd : $volume_id: $ltime \n";
      } ## end else [ if ( $found > -1 ) ]

      #print "006: $rrd : $volume_id: $ltime : $time \n";

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($total_io) || $total_io eq '' ) {
          $total_io = 'U';
        }
        else {
          if ( isdigit($total_io) && $total_io == 0 ) {

            # when 0.0 then sprintf returns null!!
            $total_io = 0;
          }
          else {
            if ( isdigit($total_io) ) {
              my $total_io_tmp = $total_io;
              $total_io = sprintf( "%.0d", $total_io_tmp );
            }
          } ## end else [ if ( isdigit($total_io...))]
        } ## end else [ if ( !defined($total_io...))]
        if ( !defined($total_io_rate)      || $total_io_rate eq '' )      { $total_io_rate      = 'U'; }
        if ( !defined($total_data_rate)    || $total_data_rate eq '' )    { $total_data_rate    = 'U'; }
        if ( !defined($read_hits)          || $read_hits eq '' )          { $read_hits          = 'U'; }
        if ( !defined($write_hits)         || $write_hits eq '' )         { $write_hits         = 'U'; }
        if ( !defined($read_pct)           || $read_pct eq '' )           { $read_pct           = 'U'; }
        if ( !defined($ssd_read_cache_hit) || $ssd_read_cache_hit eq '' ) { $ssd_read_cache_hit = 'U'; }
        if ( !defined($io_latency)         || $io_latency eq '' )         { $io_latency         = 'U'; }
        if ( !defined($cache_hits_v1)      || $cache_hits_v1 eq '' )      { $cache_hits_v1      = 'U'; }
        if ( !defined($used_cap)           || $used_cap eq '' )           { $used_cap           = 'U'; }

        #print "004: VOL $volume_id = $time:$controller:$total_io:$total_io_rate:$total_data_rate:$cache_hits_v1:$read_hits:$write_hits:$read_pct:$ssd_read_cache_hit:$io_latency:$used_cap\n";
        RRDp::cmd qq(update "$rrd" $time:$controller:$total_io:$total_io_rate:$total_data_rate:$cache_hits_v1:$read_hits:$write_hits:$read_pct:$ssd_read_cache_hit:$io_latency:$used_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        if ( isdigit($pool_id) ) {

          # keep volume fron-end data summ per pool and save it at the end
          if ( !defined $pool_data_rate[$pool_id] || $pool_data_rate[$pool_id] eq '' ) { $pool_data_rate[$pool_id] = 0; }
          if ( !defined $pool_io_rate[$pool_id]   || $pool_io_rate[$pool_id] eq '' )   { $pool_io_rate[$pool_id]   = 0; }
          if ( !defined $pool_resp_t[$pool_id]    || $pool_resp_t[$pool_id] eq '' )    { $pool_resp_t[$pool_id]    = 0; }

          if ( isdigit($total_io_rate) == 0 )   { $total_io_rate   = 0; }
          if ( isdigit($total_data_rate) == 0 ) { $total_data_rate = 0; }
          if ( isdigit($io_latency) == 0 )      { $io_latency      = 0; }

          $pool_data_rate[$pool_id] += $total_data_rate;
          $pool_io_rate[$pool_id]   += $total_io_rate;
          $pool_resp_t[$pool_id]    += $io_latency * $total_io_rate;
          $pool_time = $time;
          $pool_ids[$pool_id] = $pool_id;
        } ## end if ( isdigit($pool_id))

      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^VOLUME$/)

    # Pool capacity
    ########################

    if ( $type =~ m/^POOLcap$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port capacity statistics
      ( my $name,     my $id,        my $t9,  my $t1,  my $t2,       my $total,     my $t3,  my $free, my $virt,     my $used, my $real, my $t4, my $t5, my $t6, my $t7, my $t8, my $t14,
        my $tier0cap, my $tier0free, my $t10, my $t11, my $tier1cap, my $tier1free, my $t12, my $t13,  my $tier2cap, my $tier2free
      ) = split( /,/, $line );

      #name,id,status,mdisk count,volume count,capacity (TB),extent size,free capacity (TB),virtual capacity (TB),used capacity (TB),real capacity (TB),
      #overallocation,warning (%),easy tier,easytier status,TIER-0 type,TIER-0 mdisk count,TIER-0 capacity (TB),TIER-0 free capacity (TB),TIER-1 type,
      #TIER-1 mdisk count,TIER-1 capacity (TB),TIER-1 free capacity (TB),TIER-2 type,TIER-2 mdisk count,TIER-2 capacity (TB),TIER-2 free capacity (TB),
      #compression active,compression virtual capacity (TB),compression compressed capacity (TB),compression uncompressed capacity (TB)

      #XXXXXX_SAS_300,2,online,2,14,2.450,256,0.700,1.750,1.750,1.750,71,80,auto,active,generic_ssd,1,0.272,0.000,generic_hdd,1,2.177,0.700,,0,0.000,0.000,no,0.000,0.000,0.000,

      if ( !defined($real) || $real eq '' ) {
        if ( !defined($used) || $used eq '' ) {
          if ( !defined($free) || $free eq '' || isdigit($free) == 0 ) {
            $free = 0;
          }
          if ( isdigit($total) ) {
            $real = $total - $free;
            $used = $real;
          }
        } ## end if ( !defined($used) ||...)
        else {
          $real = $used;
        }
      } ## end if ( !defined($real) ||...)

      #find pool_id
      foreach my $line_p (@pool_name_list) {
        chomp($line_p);
        if ( !defined($line_p) || $line_p eq '' || $line_p =~ m/^ *$/ ) {
          next;
        }
        ( my $pool_id_tmp, my $pool_name_tmp, my $volumes_ids_tmp ) = split( /:/, $line_p );
        if ( !defined($volumes_ids_tmp) || $volumes_ids_tmp eq '' ) {
          next;
        }
        if ( $name eq $pool_name_tmp ) {
          $id = $pool_id_tmp;
        }
      } ## end foreach my $line_p (@pool_name_list)

      if ( !defined($id) || $id eq '' || !isdigit($id) || $id =~ m/\./ ) {
        next;    # to be save against cooruptions in the input file!!!
      }

      print "001 POOL cap : $name - $id - $total - $free - $virt - $used - $real \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/POOL/$id-cap\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_cap_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_cap_name)
      if ( $found > -1 ) {
        $ltime = $pool_cap_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_cap_name[$l_count] = $rrd;
        $pool_cap_time[$l_count] = $$last_rec;
        $ltime                   = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data

      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_cap_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($total) || $total eq '' ) {
          $total = -1;
        }
        if ( !defined($free) || $free eq '' ) {
          $free = -1;
        }
        if ( !defined($virt) || $virt eq '' ) {
          $virt = -1;
        }
        if ( !defined($used) || $used eq '' ) {
          $used = -1;
        }
        if ( !defined($real) || $real eq '' ) {
          $real = -1;
        }
        if ( !defined($tier0cap) || $tier0cap eq '' ) {
          $tier0cap = -1;
        }
        if ( !defined($tier0free) || $tier0free eq '' ) {
          $tier0free = -1;
        }
        if ( !defined($tier1cap) || $tier1cap eq '' ) {
          $tier1cap = -1;
        }
        if ( !defined($tier1free) || $tier1free eq '' ) {
          $tier1free = -1;
        }
        if ( !defined($tier2cap) || $tier2cap eq '' ) {
          $tier2cap = -1;
        }
        if ( !defined($tier2free) || $tier2free eq '' ) {
          $tier2free = -1;
        }

        #print "004 POOL cap : $name - $id - $total - $free - $virt - $used - $real TIERS: $tier0cap:$tier0free $tier1cap:$tier1free $tier2cap:$tier2free\n" if $DEBUG == 3;
        if ( !isdigit($total) || !isdigit($free) || !isdigit($used) ) {
          if ( $error_once == 0 ) {
            main::error( "data error in POOL cap: $time:$total:$free:$virt:$used:$real:$tier0cap:$tier0free:$tier1cap:$tier1free:$tier2cap:$tier2free - $line " . __FILE__ . ":" . __LINE__ );
            $error_once = 1;
            $counter_ins--;
          }
          next;
        } ## end if ( !isdigit($total) ...)
        RRDp::cmd qq(update "$rrd" $time:$total:$free:$virt:$used:$real:$tier0cap:$tier0free:$tier1cap:$tier1free:$tier2cap:$tier2free);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^POOLcap$/)
  }    # foreach

  # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
  if ( $pool_time > $pool_ltime ) {
    if ( $ds5_v2 == 0 ) {
      @pool_resp_t = "";    # DS5k v1 does not have response time info at all
    }
    my @none = "";
    pool_sum_insert(
      $type,  $wrkdir, $step,  $DEBUG, $host,  $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@none, \@none, \@none, \@none, \@none, \@none, \@pool_ids, \@pool_data_rate, \@pool_io_rate, \@pool_resp_t,
      \@none, \@none,  \@none, \@none, \@none, \@none,   \@none,    \@none,   \@none,     \@none,      \@none, \@none, \@none, \@none, \@none
    );
  } ## end if ( $pool_time > $pool_ltime)

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);

  # save pool translate table
  if ( $pool_table_created > 0 ) {

    # replace pool.cfg only if there is no any problem and the table has been created, it does not have to happen if any corruption in the data
    open( FHPW, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_p (@pool_name_list) {
      if ( $line_p eq '' ) {
        next;
      }
      print FHPW "$line_p\n";
    } ## end foreach my $line_p (@pool_name_list)
    close(FHPW);
  } ## end if ( $pool_table_created...)
  return ( "0", "$time_last_ok" );
} ## end sub load_data_ds5_perf

###################################################
#  XIV  section : IBM XIV
###################################################

sub load_data_xiv_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # at first config files like xivconf_20130904_115351.out
  #

  my $perf_string = "xivconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_xiv_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # now data files like xiv_svcperf_20130904_115351.out
  #

  $perf_string = "xivperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_xiv_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_xiv_all

sub load_data_xiv_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $capacity_file     = "$wrkdir/$host/capacity.txt";
  my $pool_file_write   = 0;                             # true if any write there
  my $config_file_write = 0;                             # true if any write there
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD> 
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > 
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name           = "";
  my @pool_name_id        = "";
  my @pool_capacity_tot   = "";
  my @pool_capacity_free  = "";
  my @pool_cfg_id         = "";
  my @rank_cfg_id         = "";
  my $pool_indx           = 0;
  my $cfg_print           = 1;
  my $pool_cfg_processing = 0;
  my $cap_total = "";
  my $cap_free = "";

  foreach $line (@lines) {

    #print "$line";
    chomp($line);
    if ($line =~ m/Total Hard Capacity/){
      my ( $sys_name, $sys_value ) = split( /:/, $line );
      $sys_name =~ s/^\s+|\s+$//g;
      $sys_value =~ s/TiB//g;
      $sys_value =~ s/^\s+|\s+$//g;
      $cap_total  = $sys_value;
      next;
    }
    if ($line =~ m/Total Hard Free Capacity/){
      my ( $sys_name, $sys_value ) = split( /:/, $line );
      $sys_name =~ s/^\s+|\s+$//g;
      $sys_value =~ s/TiB//g;
      $sys_value =~ s/^\s+|\s+$//g;
      $cap_free  = $sys_value;
      next;
    }

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Pool Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id,status/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      my ( $pool_name_cfg_item, $pool_id_cfg_item, $t1, $t2, $t3, $pool_cap_tot, $t4, $pool_cap_free ) = split( /,/, $line );
      $pool_id_cfg_item =~ s/^.*\///;
      $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
      $pool_capacity_free[$pool_indx] = $pool_cap_free;

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)

  }    # foreach

  close(FHW);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  ### capacity
  if ( isdigit($cap_total) && isdigit($cap_free)) {
    my $cap_used    = $cap_total - $cap_free;
    my $message = "total $cap_total\nfree $cap_free\nused $cap_used\n";
    $message =~ s/&nbsp;//g;
    $message =~ s/TB//g;
    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime( time() );
    open( FHRC, "> $capacity_file" ) || main::error( "Can't open $capacity_file: $! " . __FILE__ . ":" . __LINE__ ) && return 1;
    print FHRC "$message\ntime $start_human\n";
    close(FHRC);
  } ## end if ( isdigit($total) &&...)


  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 1 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);
  return 0;
} ## end sub load_data_xiv_conf

sub load_data_xiv_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there
  my $config_file_write = 0;                       # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  # pool fron-end data from volume data
  my @pool_read      = "";
  my @pool_write     = "";
  my @pool_data_rate = "";
  my @pool_io_read   = "";
  my @pool_io_write  = "";
  my @pool_io_rate   = "";
  my @pool_resp_r    = "";
  my @pool_resp_w    = "";
  my @pool_resp      = "";
  my @pool_ids       = "";
  my $pool_time      = 0;
  my $pool_ltime     = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Managed/ ) {
        $type = "RANK";    #  Managed --> RANK conversion (mdisk --> rank)
      }
      else {
        $type = uc($type_temp);
      }
      next;
    } ## end if ( $line =~ m/Level Statistics/)

    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/	//g;
      $line =~ s/Interval End:     //g;
      $line =~ s/GMT.*//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ m/^$/ ) {
      $type = "NA";      # clear out type when empty line
      if ( $pool_time > 0 ) {

        # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
        if ( $pool_time > $pool_ltime ) {
          my @none = "";
          $pool_ltime = pool_sum_insert( $type, $wrkdir, $step, $DEBUG, $host, $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@pool_data_rate, \@pool_io_rate, \@pool_resp, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none );
        }

        # reinitialize structures
        @pool_ids       = "";
        @pool_read      = "";
        @pool_write     = "";
        @pool_data_rate = "";
        @pool_io_read   = "";
        @pool_io_write  = "";
        @pool_io_rate   = "";
        @pool_resp_r    = "";
        @pool_resp_w    = "";
        @pool_resp      = "";
        $pool_time      = 0;
      } ## end if ( $pool_time > 0 )
      next;
    } ## end if ( $line =~ m/^$/ )

    if ( $line =~ /^name,id/ || $line =~ m/^	Interval/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }

    # Port
    ########################
    if ( $type =~ m/^PORT$/ ) {

      # Not yet included
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $t1, my $t2, my $write_io, my $read_io, my $io_rate, my $write, my $read, my $data_rate, my $t10, my $t11, my $t12, my $t13, my $t14, my $t15, my $t16, my $t17, my $t18, my $t19, my $t20, my $t21, my $t22, my $t23, my $t24, my $t25, my $t26, my $t27, my $t28, my $t29, my $t30, my $t31, my $t32, my $t33, my $t34, my $t35, my $t36, my $t37, my $t38, my $t39, my $t40, my $t41, my $t42, my $t43, my $t44, my $port_type_text ) = split( /,/, $line );

      # type : FC / iSCSI / SAS (PCIe)
      my $port_type = 3;    # default unknown port type
      if ( $port_type_text eq '' ) {
        next;
      }
      if ( $port_type_text =~ m/^FC$/ )    { $port_type = 0; }
      if ( $port_type_text =~ m/^PCIe$/ )  { $port_type = 1; }    # PCIe == SAS
      if ( $port_type_text =~ m/^SAS$/ )   { $port_type = 1; }
      if ( $port_type_text =~ m/^iSCSI$/ ) { $port_type = 2; }
      if ( $port_type_text =~ m/^IPREP$/ ) { $port_type = 4; }

      # IPREP addition ...  replikaci pro asynchronn remote copy.

      if ( $port_type == 3 && $port_type_once == 0 ) {

        # unknown port type detected, print error message just once
        main::error( "$host: unknown port type: $name : $port_type_text " . __FILE__ . ":" . __LINE__ );
        $port_type_once = 1;
      } ## end if ( $port_type == 3 &&...)

      #print "001 PORT $name - $write_io - $read_io - $io_rate - $write - $read - $data_rate - type:$port_type:$port_type_text\n";
      print "001 PORT $name - $write_io - $read_io - $io_rate - $write - $read - $data_rate - type:$port_type\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        my $resp_t = "0";    # not available yet, could be in the next release
        print "004: PORT $time:$write_io:$read_io:$io_rate:$data_rate:$resp_t:$read:$write\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$write_io:$read_io:$io_rate:$data_rate:$resp_t:$read:$write:$port_type);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^PORT$/)

    # Volume
    ########################

    if ( $type =~ m/^VOLUME$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $t1, my $t2, my $read_io, my $write_io, my $total_io,  my $read,       my $write, my $total, $t3,  $t4, $t5, my $resp_t_r, my $resp_t_w,
        $t6,      $t7,    $t8,    $t9,         $t10,         my $read_hits, my $write_hits, $t11,      $t12,      $t13, my $pool_id
      ) = split( /,/, $line );

      # Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
      # Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),
      # Peak Read Response Time (ms),Peak Write Response Time (ms),,,Read Hits,Write Hits,Data Read (KB),Data Written (KB),Volume Name,Pool ID,Pool Name,,,,,,,
      # ,,,,,,,,,,,,,,,,Source Volume Name

      # 00102832,2014:10:26T06:10:07.000000,602,0.028,2.020,2.048,0.113,9.349,9.462,4.000,4.628,4.620,0.424,0.749,0.000,,,,,,,,,vio_1p_01,IBMTSDS:IBM.2810-XXXXXXX-VP102778,VIO_1P,,,,

      $pool_id =~ s/^.*VP//;
      if ( !defined($name) || $name eq '' || !main::ishexa($name) || $name =~ m/\./ || !defined($pool_id) || $pool_id eq '' || !main::ishexa($pool_id) || $pool_id =~ m/\./ ) {
        if ( $error_once == 0 ) {
          main::error( "$host:$type - volume ID is not a digit or hexa: $name or pool id is not a digit: $pool_id , reported only once " . __FILE__ . ":" . __LINE__ );
          $error_once++;
        }
        next;
      } ## end if ( !defined($name) ||...)
      if ( $read_hits eq '' ) {
        $read_hits = 'U';
      }
      if ( $write_hits eq '' ) {
        $write_hits = 'U';
      }

      print "001 VOL $name - $read_io - $write_io - $resp_t_r - $resp_t_w - $read - $write\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name-P$pool_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        print "004: VOL $time:$read_io:$write_io:$resp_t_r:$resp_t_w:$read:$write:$read_hits:$write_hits\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$resp_t_r:$resp_t_w:$read:$write:$read_hits:$write_hits);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        if ( !defined($pool_id) || $pool_id eq '' ) {
          next;    # for volumes not build above aggregates (could happened when it uses extern disks)
        }

        #convert pool_uuid to numeric index
        my $pool_id_num = 0;
        foreach my $pool_id_act (@pool_ids) {
          if ( !defined($pool_id_act) || $pool_id_act eq '' ) {
            next;
          }
          if ( $pool_id =~ m/^$pool_id_act$/ ) {
            last;
          }
          $pool_id_num++;
        } ## end foreach my $pool_id_act (@pool_ids)

        # keep volume fron-end data summ per pool and save it at the end

        if ( !defined $pool_read[$pool_id_num]      || $pool_read[$pool_id_num] eq '' )      { $pool_read[$pool_id_num]      = 0; }
        if ( !defined $pool_write[$pool_id_num]     || $pool_write[$pool_id_num] eq '' )     { $pool_write[$pool_id_num]     = 0; }
        if ( !defined $pool_data_rate[$pool_id_num] || $pool_data_rate[$pool_id_num] eq '' ) { $pool_data_rate[$pool_id_num] = 0; }
        if ( !defined $pool_io_read[$pool_id_num]   || $pool_io_read[$pool_id_num] eq '' )   { $pool_io_read[$pool_id_num]   = 0; }
        if ( !defined $pool_io_write[$pool_id_num]  || $pool_io_write[$pool_id_num] eq '' )  { $pool_io_write[$pool_id_num]  = 0; }
        if ( !defined $pool_io_rate[$pool_id_num]   || $pool_io_rate[$pool_id_num] eq '' )   { $pool_io_rate[$pool_id_num]   = 0; }
        if ( !defined $pool_resp_r[$pool_id_num]    || $pool_resp_r[$pool_id_num] eq '' )    { $pool_resp_r[$pool_id_num]    = 0; }
        if ( !defined $pool_resp_w[$pool_id_num]    || $pool_resp_w[$pool_id_num] eq '' )    { $pool_resp_w[$pool_id_num]    = 0; }
        if ( !defined $pool_resp[$pool_id_num]      || $pool_resp[$pool_id_num] eq '' )      { $pool_resp[$pool_id_num]      = 0; }

        if ( isdigit($read_io) == 0 )  { $read_io  = 0; }
        if ( isdigit($write_io) == 0 ) { $write_io = 0; }
        if ( isdigit($resp_t_r) == 0 ) { $resp_t_r = 0; }
        if ( isdigit($resp_t_w) == 0 ) { $resp_t_w = 0; }
        if ( isdigit($read) == 0 )     { $read     = 0; }
        if ( isdigit($write) == 0 )    { $write    = 0; }

        $pool_read[$pool_id_num]      += $read;
        $pool_write[$pool_id_num]     += $write;
        $pool_data_rate[$pool_id_num] += $read;
        $pool_data_rate[$pool_id_num] += $write;
        $pool_io_read[$pool_id_num]   += $read_io;
        $pool_io_write[$pool_id_num]  += $write_io;
        $pool_io_rate[$pool_id_num]   += $read_io;
        $pool_io_rate[$pool_id_num]   += $write_io;
        $pool_resp_r[$pool_id_num]    += $resp_t_r * $read_io;
        $pool_resp_w[$pool_id_num]    += $resp_t_w * $write_io;
        $pool_resp[$pool_id_num]      += $resp_t_r * $read_io;
        $pool_resp[$pool_id_num]      += $resp_t_w * $write_io;
        $pool_time = $time;
        $pool_ids[$pool_id_num] = $pool_id;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^VOLUME$/)
  }    # foreach

  # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
  if ( $pool_time > $pool_ltime ) {
    my @none = "";
    $pool_ltime = pool_sum_insert( $type, $wrkdir, $step, $DEBUG, $host, $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@pool_data_rate, \@pool_io_rate, \@pool_resp, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none );
  }

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_xiv_perf

##################################################
#  EMC VNX block
##################################################

sub load_data_vnxb_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  my $in_exist    = 0;
  my $perf_string = "";
  my @files_unsorted;
  my @files;
  my $time_last_ok = "";

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # conf files
  #
  $perf_string = "vnxbconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_vnxb_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # perf files
  #
  $perf_string = "vnxbperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    #print "$in_file\n";

    my ( $ret, $time_last_ok ) = load_data_vnxb_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_vnxb_all

sub load_data_vnxb_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  my $pool_file   = "$wrkdir/$host/pool.cfg";
  my $config_file = "$wrkdir/$host/config.html";

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( POOL, "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp: $! " . __FILE__ . ":" . __LINE__ )   && return 0;

  # print *conf* file to config.html
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
  <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my $section = "";

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    # print to config.html
    print FCFG "$line\n";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                               { next; }
    if ( $line =~ m/^Start:/ )                               { next; }
    if ( $line =~ m/^Name,lun_id,Current_owner,/ )           { next; }
    if ( $line =~ m/^RG_ID,type,Capacity/ )                  { next; }
    if ( $line =~ m/^Pool name,Pool ID,Raid Type,/ )         { next; }
    if ( $line =~ m/^SP Name,SP Port ID,Physical Port ID,/ ) { next; }
    if ( $line =~ m/^Disk ID,State,Vendor ID,Drive Type,/ )  { next; }
    if ( $line =~ m/^SP Name,Statistics Logging,/ )          { next; }
    if ( $line =~ m/^Host name,Volume IDs,Volume Names/ )    { next; }

    if ( $line =~ m/^$/ )                            { $section = "NA";     next; }
    if ( $line eq "Volume Level Configuration" )     { $section = "VOLUME"; next; }
    if ( $line eq "Raid-Group Level Configuration" ) { $section = "RG";     next; }
    if ( $line eq "Pool Level Configuration" )       { $section = "POOL";   next; }
    if ( $line eq "Port Level Configuration" )       { $section = "PORT";   next; }
    if ( $line eq "Disk Level Configuration" )       { $section = "DISK";   next; }
    if ( $line eq "SP Level Configuration" )         { $section = "SP";     next; }
    if ( $line eq "Host Level Configuration" )       { $section = "HOST";   next; }

    if ( $section eq "RG" ) {
      my ( $rg_id, undef ) = split( ",", $line );
      my $rg_name_out = "RG_" . $rg_id;

      # print RG to pool.cfg
      print POOL "$rg_name_out:$rg_name_out\n";
    } ## end if ( $section eq "RG" )

    if ( $section eq "POOL" ) {
      my ( $pool_name, $pool_id, undef ) = split( ",", $line );

      # print pool to pool.cfg
      print POOL "$pool_id:$pool_name\n";
    } ## end if ( $section eq "POOL")

  } ## end foreach my $line (@lines)

  print FCFG "</pre></body></html>\n";
  close(FCFG);
  close(POOL);

  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }
  if ( -f "$pool_file-tmp" ) {
    rename( "$pool_file-tmp", "$pool_file" );        # rename it anyway even if some problem
  }

  return 0;
} ## end sub load_data_vnxb_conf

sub load_data_vnxb_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);
  my $pool_file  = "$wrkdir/$host/pool.cfg";
  my $time       = 3600;

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  open( CFG, "< $pool_file" ) || main::error( "Can't open $pool_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @pool_cfg = <CFG>;
  close(CFG);

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  my $section    = "";
  my $rrd        = "";
  my $type       = "NA";
  my $pool_index = 0;
  my %pool_data;
  my $time_last_ok = "";

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                               { next; }
    if ( $line =~ m/^Name,lun_id,Current_owner,/ )           { next; }
    if ( $line =~ m/^RG_ID,type,Capacity/ )                  { next; }
    if ( $line =~ m/^Pool name,Pool ID,Raid Type,/ )         { next; }
    if ( $line =~ m/^SP Name,SP Port ID,Physical Port ID,/ ) { next; }
    if ( $line =~ m/^Disk ID,State,Vendor ID,Drive Type,/ )  { next; }
    if ( $line =~ m/^SP Name,Statistics Logging,/ )          { next; }
    if ( $line =~ m/^Host name,Volume IDs,Volume Names/ )    { next; }

    if ( $line =~ m/^$/ )                         { $section = "NA";     next; }
    if ( $line eq "Volume Level Statistics" )     { $section = "VOLUME"; next; }
    if ( $line eq "Raid-Group Level Statistics" ) { $section = "RG";     next; }
    if ( $line eq "Pool Level Statistics" )       { $section = "POOL";   next; }
    if ( $line eq "Port Level Statistics" )       { $section = "PORT";   next; }
    if ( $line eq "Disk Level Statistics" )       { $section = "DISK";   next; }
    if ( $line eq "SP Level Statistics" )         { $section = "SP";     next; }
    if ( $line eq "Host Level Statistics" )       { $section = "HOST";   next; }

    # set start time
    if ( $line =~ m/^Start:/ ) {
      $line =~ s/^Start:\s+//;
      $time = str2time($line);

      if ( !defined($time) || isdigit($time) == 0 ) {
        $section = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/^Start:/)

    $type = $section;

    # Volume
    ########################

    if ( $section eq "VOLUME" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Name,lun_id,Current_owner,RG_ID,Pool name,LUN_Capacity (MB),LUN_Capacity (Blocks),Read_Requests,Write_Requests,Blocks_read,Blocks_written,Read cache hits,Write cache hits,Read Hit Ratio,Write Hit Ratio,LUN_Capacity (MB),read (kB),write (kB),io rate,data rate,busy_ticks,idle_ticks,Average Read Time,Average Write Time,Read Requests SP A,Read Requests SP B,Write Requests SP A,Write Requests SP B,io_rate_spa,io_rate_spb,Non-zero Request Count Arrivals,Sum of Outstanding Requests
      # VMware-raid1-01,0,SP B,0,,102400,209715200,4302916,784192,24103800,114351374,3788959,4163687,,,102400,12051900,57175687,5087108,69227587,1335491193,3899346013,36574909832,4275385998,1354208,2948708,140447,643745,1494655,3592453,1965194,28492093

      my ( $volume_name, $volume_id, $owner, $raid_group_id, $pool_name, $capacity_mb, $capacity_block, $read_io, $write_io, $read_bl, $write_bl, undef, undef, undef, undef, $cap, $read_kB, $write_kB, $io_rate, $data_rate, $busy_ticks, $idle_ticks, $resp_t_r, $resp_t_w, undef, undef, undef, undef, $io_rate_spa, $io_rate_spb, $arr_queue, $sum_queue ) = split( /,/, $line );

      if ( !defined($capacity_mb)    || $capacity_mb eq ''    || !isdigit($capacity_mb) )    { $capacity_mb    = 'U'; }
      if ( !defined($capacity_block) || $capacity_block eq '' || !isdigit($capacity_block) ) { $capacity_block = 'U'; }
      if ( !defined($read_io)        || $read_io eq ''        || !isdigit($read_io) )        { $read_io        = 'U'; }
      if ( !defined($write_io)       || $write_io eq ''       || !isdigit($write_io) )       { $write_io       = 'U'; }
      if ( !defined($read_kB)        || $read_kB eq ''        || !isdigit($read_kB) )        { $read_kB        = 'U'; }
      if ( !defined($write_kB)       || $write_kB eq ''       || !isdigit($write_kB) )       { $write_kB       = 'U'; }
      if ( !defined($cap)            || $cap eq ''            || !isdigit($cap) )            { $cap            = 'U'; }
      if ( !defined($io_rate)        || $io_rate eq ''        || !isdigit($io_rate) )        { $io_rate        = 'U'; }
      if ( !defined($data_rate)      || $data_rate eq ''      || !isdigit($data_rate) )      { $data_rate      = 'U'; }
      if ( !defined($busy_ticks)     || $busy_ticks eq ''     || !isdigit($busy_ticks) )     { $busy_ticks     = 'U'; }
      if ( !defined($idle_ticks)     || $idle_ticks eq ''     || !isdigit($idle_ticks) )     { $idle_ticks     = 'U'; }
      if ( !defined($resp_t_r)       || $resp_t_r eq ''       || !isdigit($resp_t_r) )       { $resp_t_r       = 'U'; }
      if ( !defined($resp_t_w)       || $resp_t_w eq ''       || !isdigit($resp_t_w) )       { $resp_t_w       = 'U'; }
      if ( !defined($io_rate_spa)    || $io_rate_spa eq ''    || !isdigit($io_rate_spa) )    { $io_rate_spa    = 'U'; }
      if ( !defined($io_rate_spb)    || $io_rate_spb eq ''    || !isdigit($io_rate_spb) )    { $io_rate_spb    = 'U'; }
      if ( !defined($arr_queue)      || $arr_queue eq ''      || !isdigit($arr_queue) )      { $arr_queue      = 'U'; }
      if ( !defined($sum_queue)      || $sum_queue eq ''      || !isdigit($sum_queue) )      { $sum_queue      = 'U'; }

      # RG to pool name
      if ( defined($raid_group_id) && isdigit($raid_group_id) && $pool_name eq '' ) {
        $pool_name = "RG_" . $raid_group_id;
      }

      # Controller|SP (Storage Processor)
      my $controller = "";
      if ( defined $owner && $owner ne '' ) {
        $controller = $owner;
        $controller =~ s/^SP\s+//;
      }

      if ( !$controller eq '' && $controller =~ m/^[a|A]$/ ) {
        $controller = 0;
      }
      if ( !$controller eq '' && $controller =~ m/^[b|B]$/ ) {
        $controller = 1;
      }
      if ( $controller eq "N/A" || $controller eq '' ) {
        $controller = 'U';
      }
      else {
        if ( main::isdigit($controller) == 0 ) {
          main::error( "$host:$type - controler (Storage Processor) info is not found : $controller , reported only once  : $line" . __FILE__ . ":" . __LINE__ );
          $controller = 'U';
        }
      } ## end else [ if ( $controller eq "N/A"...)]

      print "001 VOL $volume_name - $volume_id - $owner - $raid_group_id - $pool_name - $capacity_mb - $capacity_block - $read_io - $write_io - $read_bl - $write_bl - $cap - $read_kB - $write_kB - $io_rate - $data_rate\n" if $DEBUG == 3;

      # update .rrr file
      #
      $rrd = "$wrkdir/$host/$section/$volume_id.rrr";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "005: $rrd : $volume_id: $ltime \n";
      #print "006: $rrd : $volume_id: $ltime : $time \n";

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$busy_ticks:$idle_ticks:$io_rate_spa:$io_rate_spb:$arr_queue:$sum_queue);
        my $answer = RRDp::read;
      }

      # RESPONSE TIME
      #
      my $resp_t        = "";
      my $io_rate_gauge = "";
      my $start_time    = $time - 300;

      my $basedir = $ENV{INPUTDIR};
      my $tmpdir  = "$basedir/tmp";
      if ( defined $ENV{TMPDIR_STOR} ) {
        $tmpdir = $ENV{TMPDIR_STOR};
      }
      my $cmd = "";
      $cmd .= "graph \\\"$basedir/tmp/name.png\\\"";
      $cmd .= " --start \\\"$start_time\\\"";
      $cmd .= " --end \\\"now\\\"";
      $cmd .= " --width=400";
      $cmd .= " --height=150";
      $cmd .= " --step=300";
      $cmd .= " DEF:io_rate=\\\"$rrd\\\":io_rate:AVERAGE";
      $cmd .= " DEF:sum_queue=\\\"$rrd\\\":sum_queue:AVERAGE";
      $cmd .= " DEF:arr_queue=\\\"$rrd\\\":arr_queue:AVERAGE";
      $cmd .= " CDEF:arr_queue_2=arr_queue,2,/";
      $cmd .= " CDEF:sum_queue_2=sum_queue,arr_queue_2,-";
      $cmd .= " CDEF:avg_busy_queue=sum_queue_2,io_rate,/";
      $cmd .= " DEF:busy_ticks=\\\"$rrd\\\":busy_ticks:AVERAGE";
      $cmd .= " DEF:idle_ticks=\\\"$rrd\\\":idle_ticks:AVERAGE";
      $cmd .= " CDEF:all_ticks=busy_ticks,idle_ticks,+";
      $cmd .= " CDEF:util_l=busy_ticks,all_ticks,/";
      $cmd .= " CDEF:util=util_l,200,*";
      $cmd .= " CDEF:queue_length_2=avg_busy_queue,util,*";
      $cmd .= " CDEF:queue_length=queue_length_2,100,/";
      $cmd .= " CDEF:resp_t_u=queue_length,io_rate,/";
      $cmd .= " CDEF:resp_t=resp_t_u,1000,*";
      $cmd .= " PRINT:resp_t:AVERAGE:\\\"%8.1lf resp_t\\\"";
      $cmd .= " PRINT:io_rate:AVERAGE:\\\"%8.1lf io_rate\\\"";
      $cmd =~ s/\\"/"/g;
      RRDp::cmd qq($cmd);
      my $data = RRDp::read;

      foreach my $data_line ( split( /\n/, $$data ) ) {
        chomp($data_line);

        #print "############$data_line\n";
        if ( $data_line =~ m/^0x/ ) { next; }
        $data_line =~ s/^\s+//;
        $data_line =~ s/\s+$//;

        if ( $data_line =~ m/resp_t$/ ) {
          $resp_t = $data_line;
          $resp_t =~ s/\s+resp_t$//;
        }
        if ( $data_line =~ m/io_rate$/ ) {
          $io_rate_gauge = $data_line;
          $io_rate_gauge =~ s/\s+io_rate$//;
        }
      } ## end foreach my $data_line ( split...)

      #print "resp_t=$resp_t,io_rate=$io_rate_gauge\n";

      if ( !defined($resp_t)        || $resp_t eq ''        || !isdigit($resp_t) )        { $resp_t        = 'U'; }
      if ( !defined($io_rate_gauge) || $io_rate_gauge eq '' || !isdigit($io_rate_gauge) ) { $io_rate_gauge = 'U'; }

      if ( isdigit($resp_t) && $resp_t > 1000 ) {
        if ( $io_rate_gauge eq 'U' || $io_rate_gauge < 32 ) {
          $resp_t = 'U';
        }
      }

      # update .rrd file with normal items
      #
      $rrd = "$wrkdir/$host/$section/$volume_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      $ltime    = "";
      $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "005: $rrd : $volume_id: $ltime \n";
      #print "006: $rrd : $volume_id: $ltime : $time \n";

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap:$controller:$resp_t);
        my $answer = RRDp::read;
      }

      # find pool ID from pool.cfg
      my $pool_id = "";
      my ($pool_line) = grep {/:\Q$pool_name\E$/} @pool_cfg;
      if ( defined $pool_line && $pool_line ne '' ) {
        chomp $pool_line;
        $pool_id = $pool_line;
        $pool_id =~ s/:\Q$pool_name\E$//;
      }

      # add values to pool
      if ( defined $pool_id && $pool_id ne '' ) {
        $pool_index++;

        #print "$pool_id .. $volume_name\n";
        $pool_data{$pool_id}{'pool_name'} = $pool_name;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $pool_data{$pool_id}{'io_rate'} && isdigit( $pool_data{$pool_id}{'io_rate'} ) ) {
            $pool_data{$pool_id}{'io_rate'} = $pool_data{$pool_id}{'io_rate'} + $io_rate;
          }
          else {
            $pool_data{$pool_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)

        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $pool_data{$pool_id}{'read_io'} && isdigit( $pool_data{$pool_id}{'read_io'} ) ) {
            $pool_data{$pool_id}{'read_io'} = $pool_data{$pool_id}{'read_io'} + $read_io;
          }
          else {
            $pool_data{$pool_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)

        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $pool_data{$pool_id}{'write_io'} && isdigit( $pool_data{$pool_id}{'write_io'} ) ) {
            $pool_data{$pool_id}{'write_io'} = $pool_data{$pool_id}{'write_io'} + $write_io;
          }
          else {
            $pool_data{$pool_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)

        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $pool_data{$pool_id}{'data_rate'} && isdigit( $pool_data{$pool_id}{'data_rate'} ) ) {
            $pool_data{$pool_id}{'data_rate'} = $pool_data{$pool_id}{'data_rate'} + $data_rate;
          }
          else {
            $pool_data{$pool_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)

        if ( defined $read_kB && isdigit($read_kB) ) {
          if ( defined $pool_data{$pool_id}{'read'} && isdigit( $pool_data{$pool_id}{'read'} ) ) {
            $pool_data{$pool_id}{'read'} = $pool_data{$pool_id}{'read'} + $read_kB;
          }
          else {
            $pool_data{$pool_id}{'read'} = $read_kB;
          }
        } ## end if ( defined $read_kB ...)

        if ( defined $write_kB && isdigit($write_kB) ) {
          if ( defined $pool_data{$pool_id}{'write'} && isdigit( $pool_data{$pool_id}{'write'} ) ) {
            $pool_data{$pool_id}{'write'} = $pool_data{$pool_id}{'write'} + $write_kB;
          }
          else {
            $pool_data{$pool_id}{'write'} = $write_kB;
          }
        } ## end if ( defined $write_kB...)

        if ( defined $cap && isdigit($cap) ) {
          if ( defined $pool_data{$pool_id}{'cap'} && isdigit( $pool_data{$pool_id}{'cap'} ) ) {
            $pool_data{$pool_id}{'cap'} = $pool_data{$pool_id}{'cap'} + $cap;
          }
          else {
            $pool_data{$pool_id}{'cap'} = $cap;
          }
        } ## end if ( defined $cap && isdigit...)
        if ( defined $resp_t && isdigit($resp_t) && defined $io_rate_gauge && isdigit($io_rate_gauge) ) {
          $resp_t = $resp_t * $io_rate_gauge;
          if ( defined $pool_data{$pool_id}{'resp_t'} && isdigit( $pool_data{$pool_id}{'resp_t'} ) ) {
            $pool_data{$pool_id}{'resp_t'} = $pool_data{$pool_id}{'resp_t'} + $resp_t;
          }
          else {
            $pool_data{$pool_id}{'resp_t'} = $resp_t;
          }
        } ## end if ( defined $resp_t &&...)
        if ( defined $io_rate_gauge && isdigit($io_rate_gauge) ) {
          if ( defined $pool_data{$pool_id}{'io_rate_gauge'} && isdigit( $pool_data{$pool_id}{'io_rate_gauge'} ) ) {
            $pool_data{$pool_id}{'io_rate_gauge'} = $pool_data{$pool_id}{'io_rate_gauge'} + $io_rate_gauge;
          }
          else {
            $pool_data{$pool_id}{'io_rate_gauge'} = $io_rate_gauge;
          }
        } ## end if ( defined $io_rate_gauge...)

        #print "$time:$volume_name:$pool_name:$pool_id:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap\n";
      } ## end if ( defined $pool_id ...)
    } ## end if ( $section eq "VOLUME")

    # Port
    ########################

    if ( $section eq "PORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      #SP Name,SP Port ID,Physical Port ID,Link Status,Port Status,SFP State,Usage,Speed Value,SP UID,Reads (IOPS),Writes (IOPS),Blocks Read,Blocks Written,read (kB),write (kB),io_rate,data_rate
      #SP A,3,5,Up,Online,Online,General,4Gbps,50:06:01:60:BE:A0:14:9F:50:06:01:63:3E:A0:14:9F,2373407,1403674,20009327,242263398,10004664,121131699,3777081,131136363

      my ( $sp_name, $sp_port_id, $phy_port_id, undef, undef, undef, undef, undef, undef, $read_io, $write_io, undef, undef, $read_kB, $write_kB, $io_rate, $data_rate ) = split( /,/, $line );

      my $port_name = $sp_name;
      $port_name =~ s/^SP\s+//g;
      $port_name .= "$sp_port_id";

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }

      print "001 PORT $port_name - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate\n" if $DEBUG == 3;

      $rrd = "$wrkdir/$host/$section/$port_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "PORT")

    # Disk
    ########################

    if ( $section eq "DISK" ) {
      print "000 $line\n" if $DEBUG == 3;

      #Disk ID,State,Vendor ID,Drive Type,Raid Group ID,Current Speed,Capacity,User Capacity,Luns,Prct Idle,Prct Busy,Idle Ticks,Busy Ticks,Number of Reads,Number of Writes,Kbytes Read,Kbytes Written,Read Requests,Write Requests,Queue Length,io_rate,data_rate
      #0-0-9,Enabled,SEAGATE,SAS,N/A,6Gbps,549691,0,Unbound,0,0,15107557,53211,775463,526017,166183007,77044228,775463,526017,4353366,1301480,243227235

      my ( $disk_id, undef, undef, undef, undef, undef, undef, undef, undef, $prct_idle, $prct_busy, $idle_ticks, $busy_ticks, undef, undef, $read_kB, $write_kB, $read_io, $write_io, $queue, $io_rate, $data_rate ) = split( /,/, $line );

      if ( !defined($read_io)    || $read_io eq ''    || !isdigit($read_io) )    { $read_io    = 'U'; }
      if ( !defined($write_io)   || $write_io eq ''   || !isdigit($write_io) )   { $write_io   = 'U'; }
      if ( !defined($read_kB)    || $read_kB eq ''    || !isdigit($read_kB) )    { $read_kB    = 'U'; }
      if ( !defined($write_kB)   || $write_kB eq ''   || !isdigit($write_kB) )   { $write_kB   = 'U'; }
      if ( !defined($io_rate)    || $io_rate eq ''    || !isdigit($io_rate) )    { $io_rate    = 'U'; }
      if ( !defined($data_rate)  || $data_rate eq ''  || !isdigit($data_rate) )  { $data_rate  = 'U'; }
      if ( !defined($prct_idle)  || $prct_idle eq ''  || !isdigit($prct_idle) )  { $prct_idle  = 'U'; }
      if ( !defined($prct_busy)  || $prct_busy eq ''  || !isdigit($prct_busy) )  { $prct_busy  = 'U'; }
      if ( !defined($idle_ticks) || $idle_ticks eq '' || !isdigit($idle_ticks) ) { $idle_ticks = 'U'; }
      if ( !defined($busy_ticks) || $busy_ticks eq '' || !isdigit($busy_ticks) ) { $busy_ticks = 'U'; }
      if ( !defined($queue)      || $queue eq ''      || !isdigit($queue) )      { $queue      = 'U'; }

      print "001 DISK $disk_id - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate - $prct_idle - $prct_busy - $idle_ticks - $busy_ticks - $queue\n" if $DEBUG == 3;

      $type = "DRIVE";
      $rrd  = "$wrkdir/$host/$type/$disk_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$queue:$prct_idle:$prct_busy:$idle_ticks:$busy_ticks);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "DISK")

    # RAID-GROUP capacity
    ########################

    if ( $section eq "RG" ) {

      #RG_ID,type,Capacity (MB),Capacity (Blocks),Free Capacity (MB),Free Capacity (Blocks),Luns,capacity (TB),capacity free (TB),capacity used (TB)
      #0,r1_0,359860,736993280,257460,527278080,0,0.343189239501953,0.245532989501953,0.09765625

      my ( $rg_id, undef, undef, undef, undef, undef, undef, $cap, $cap_free, $cap_used ) = split( /,/, $line );

      if ( !defined($cap)      || $cap eq ''      || !isdigit($cap) )      { $cap      = 'U'; }
      if ( !defined($cap_free) || $cap_free eq '' || !isdigit($cap_free) ) { $cap_free = 'U'; }
      if ( !defined($cap_used) || $cap_used eq '' || !isdigit($cap_used) ) { $cap_used = 'U'; }

      my $rg_name = "RG_$rg_id";

      print "001 RG cap $rg_name - $cap - $cap_free - $cap_used\n" if $DEBUG == 3;

      $type = "POOLcap";
      $rrd  = "$wrkdir/$host/POOL/$rg_name-cap.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "RG" )

    # POOL capacity
    ########################

    if ( $section eq "POOL" ) {
      print "000 $line\n" if $DEBUG == 3;

      #Pool name,Pool ID,Raid Type,Percent Full,Available Capacity (GBs),Available Capacity (Blocks),Consumed Capacity (GBs),Consumed Capacity (Blocks),User Capacity (GBs),User Capacity (Blocks),Luns,capacity (TB),capacity free (TB),capacity used (TB)
      #Pool 0,0,r_5,19.813,1285.506,2695901184,317.619,666095616,1603.125,3361996800,5 6,1.5655517578125,1.2553768157959,0.310174942016602

      my ( $pool_name, $pool_id, undef, undef, undef, undef, undef, undef, undef, undef, undef, $cap, $cap_free, $cap_used ) = split( /,/, $line );

      if ( !defined($cap)      || $cap eq ''      || !isdigit($cap) )      { $cap      = 'U'; }
      if ( !defined($cap_free) || $cap_free eq '' || !isdigit($cap_free) ) { $cap_free = 'U'; }
      if ( !defined($cap_used) || $cap_used eq '' || !isdigit($cap_used) ) { $cap_used = 'U'; }

      print "001 POOL cap $pool_name - $pool_id - $cap - $cap_free - $cap_used\n" if $DEBUG == 3;

      $type = "POOLcap";
      $rrd  = "$wrkdir/$host/POOL/$pool_id-cap.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "POOL")

    # SP - Storage Processor (NODE)
    ########################

    if ( $section eq "SP" ) {
      print "000 $line\n" if $DEBUG == 3;

      # SP Name,Statistics Logging,Serial Number,SP IP,Idle (%),Busy (%),Controller idle ticks,Controller busy ticks,Total Reads,Total Writes,Read_requests,Write_requests,Blocks_read,Blocks_written,Arrivals_to_non_zero_queue,Sum_queue_lengths_by_arrivals,read (kB),write (kB),io_rate,data_rate,Write_cache_blocks_flushed,Prct Dirty Cache Pages,Prct Cache Pages Owned
      # SP A,ON,CF2J9111700098,192.168.1.72,97.6,2.40,67371182,1655528,4322719,9797029,4322719,9797029,35575261,6875012199,11756948,219090338,17787630,3437506100,14119748,3455293730,2352221708,85,49

      my ( $SP, undef, undef, undef, $idle_prct, $busy_prct, $idle_ticks, $busy_ticks, undef, undef, $read_io, $write_io, undef, undef, $arr_queue, $sum_queue, $read, $write, $io_rate, $data_rate, $write_cache_blocks_flushed, $prct_dirty_cache_pages, $prct_cache_pages_owned ) = split( /,/, $line );

      if ( !defined($idle_prct)                  || $idle_prct eq ''                  || !isdigit($idle_prct) )                  { $idle_prct                  = 'U'; }
      if ( !defined($busy_prct)                  || $busy_prct eq ''                  || !isdigit($busy_prct) )                  { $busy_prct                  = 'U'; }
      if ( !defined($idle_ticks)                 || $idle_ticks eq ''                 || !isdigit($idle_ticks) )                 { $idle_ticks                 = 'U'; }
      if ( !defined($busy_ticks)                 || $busy_ticks eq ''                 || !isdigit($busy_ticks) )                 { $busy_ticks                 = 'U'; }
      if ( !defined($read_io)                    || $read_io eq ''                    || !isdigit($read_io) )                    { $read_io                    = 'U'; }
      if ( !defined($write_io)                   || $write_io eq ''                   || !isdigit($write_io) )                   { $write_io                   = 'U'; }
      if ( !defined($arr_queue)                  || $arr_queue eq ''                  || !isdigit($arr_queue) )                  { $arr_queue                  = 'U'; }
      if ( !defined($sum_queue)                  || $sum_queue eq ''                  || !isdigit($sum_queue) )                  { $sum_queue                  = 'U'; }
      if ( !defined($read)                       || $read eq ''                       || !isdigit($read) )                       { $read                       = 'U'; }
      if ( !defined($write)                      || $write eq ''                      || !isdigit($write) )                      { $write                      = 'U'; }
      if ( !defined($io_rate)                    || $io_rate eq ''                    || !isdigit($io_rate) )                    { $io_rate                    = 'U'; }
      if ( !defined($data_rate)                  || $data_rate eq ''                  || !isdigit($data_rate) )                  { $data_rate                  = 'U'; }
      if ( !defined($write_cache_blocks_flushed) || $write_cache_blocks_flushed eq '' || !isdigit($write_cache_blocks_flushed) ) { $write_cache_blocks_flushed = 'U'; }
      if ( !defined($prct_dirty_cache_pages)     || $prct_dirty_cache_pages eq ''     || !isdigit($prct_dirty_cache_pages) )     { $prct_dirty_cache_pages     = 'U'; }
      if ( !defined($prct_cache_pages_owned)     || $prct_cache_pages_owned eq ''     || !isdigit($prct_cache_pages_owned) )     { $prct_cache_pages_owned     = 'U'; }

      my $sp_name = $SP;
      if ( $SP eq "SP A" ) { $sp_name = "spa"; }
      if ( $SP eq "SP B" ) { $sp_name = "spb"; }

      print "001 SP (NODE) $sp_name - $SP - $idle_prct - $busy_prct - $idle_ticks - $busy_ticks - $read_io - $write_io - $arr_queue - $sum_queue - $read - $write\n" if $DEBUG == 3;

      $type = "NODE";
      $rrd  = "$wrkdir/$host/$type/$sp_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$idle_prct:$busy_prct:$idle_ticks:$busy_ticks:$read_io:$write_io:$arr_queue:$sum_queue:$read:$write:$io_rate:$data_rate:$write_cache_blocks_flushed:$prct_dirty_cache_pages:$prct_cache_pages_owned:U:U:U:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "SP" )

  }    # end of foreach

  # pool summary
  if ( isdigit($pool_index) && $pool_index > 0 ) {
    $type = "POOL";
    data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%pool_data );
  }

  return ( "0", "$time_last_ok" );
} ## end sub load_data_vnxb_perf

##################################################
# EMC VMAX
##################################################

sub load_data_vmax_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  my $in_exist    = 0;
  my $perf_string = "";
  my @files_unsorted;
  my @files;
  my $time_last_ok = "";

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # conf files
  #
  $perf_string = "vmaxconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_vmax_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # perf files
  #
  $perf_string = "vmaxperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    #print "$in_file\n";

    my ( $ret, $time_last_ok ) = load_data_vmax_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_vmax_all

sub load_data_vmax_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  my $pool_file   = "$wrkdir/$host/pool.cfg";
  my $tier_file   = "$wrkdir/$host/tier.cfg";
  my $config_file = "$wrkdir/$host/config.html";
  my $port_map    = "$webdir/$host/port_map.html";
  my $sg_alloc    = "$webdir/$host/sg_alloc.html";

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( POOL, "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp: $! " . __FILE__ . ":" . __LINE__ )   && return 0;
  open( TIER, "> $tier_file-tmp" )   || main::error( "Can't open $tier_file-tmp: $! " . __FILE__ . ":" . __LINE__ )   && return 0;

  # print *conf* file to config.html
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
  <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my $section  = "";
  my $port_idx = 0;
  my @ports;
  my %sg_data;
  my $tier_by_sg_idx = 0;

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    # print to config.html
    print FCFG "$line\n";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                 { next; }
    if ( $line =~ m/^ID,time,timestamp/ )      { next; }
    if ( $line =~ m/^SG_ID,time,timestamp/ )   { next; }
    if ( $line =~ m/^HOST_ID,time,timestamp/ ) { next; }
    if ( $line =~ m/^VMAX model :/ )           { next; }
    if ( $line =~ m/^SG_ID,Tier_id,/ )         { next; }

    if ( $line =~ m/^univmax_version : V8/ ) {
      if ( !-f "$wrkdir/$host/VMAX-V8" ) {
        `touch "$wrkdir/$host/VMAX-V8"`;
      }
      next;
    } ## end if ( $line =~ m/^univmax_version : V8/)

    if ( $line =~ m/^$/ )                                    { $section = "NA";     next; }
    if ( $line eq "StorageGroup Level Configuration" )       { $section = "VOLUME"; next; }
    if ( $line eq "ThinPool Level Configuration" )           { $section = "POOL";   next; }
    if ( $line eq "ThinTier Level Configuration" )           { $section = "TIER";   next; }
    if ( $line eq "FePort Level Configuration" )             { $section = "PORT";   next; }
    if ( $line eq "Host Level Configuration" )               { $section = "HOST";   next; }
    if ( $line eq "TierByStorageGroup Level Configuration" ) { $section = "SGTIER"; next; }

    if ( $section eq "POOL" ) {
      my ( $pool_name, undef ) = split( ",", $line );

      # print pool to pool.cfg
      print POOL "$pool_name:$pool_name\n";
    } ## end if ( $section eq "POOL")

    if ( $section eq "TIER" ) {
      my ( $tier_name, undef ) = split( ",", $line );

      # print pool to pool.cfg
      print TIER "$tier_name:$tier_name\n";
      $sg_data{'TIER'}{$tier_name} = $tier_name;
    } ## end if ( $section eq "TIER")

    if ( $section eq "SGTIER" ) {
      my ( $sg_id, $tier_id, $total_cap, $tier_alloc_cap ) = split( ",", $line );
      if ( defined $sg_id && $sg_id ne '' && defined $tier_id && $tier_id ne '' ) {
        $tier_by_sg_idx++;
        $sg_data{'SG'}{$sg_id}{'TIER'}{$tier_id} = $tier_alloc_cap;
        $sg_data{'SG'}{$sg_id}{'TOTAL'} = $total_cap;
      }
    } ## end if ( $section eq "SGTIER")

    if ( $section eq "PORT" ) {
      $port_idx++;

      # FA-10E-0,2017:04:18T15:25:00,1492521900,FibreChannel,129
      my ( $port_name, undef, undef, undef, $mapped_vols ) = split( ",", $line );
      push( @ports, "$port_name,$mapped_vols" );
    } ## end if ( $section eq "PORT")

  } ## end foreach my $line (@lines)

  print FCFG "</pre></body></html>\n";
  close(FCFG);
  close(POOL);
  close(TIER);

  # FE ports -> mapped volumes table
  if ( $port_idx > 0 ) {
    open( VOLS, "> $port_map-tmp" ) || main::error( "Can't open $port_map-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;

    print VOLS "<center>\n";
    print VOLS "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
    print VOLS "<thead>\n";
    print VOLS "<tr>\n";
    print VOLS "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Port</th>\n";
    print VOLS "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Mapped volumes</th>\n";
    print VOLS "</tr>\n";
    print VOLS "</thead>\n";
    print VOLS "<tbody>\n";

    @ports = sort @ports;

    foreach my $line (@ports) {
      my ( $port, $vols ) = split( ",", $line );
      print VOLS "<tr>\n";
      print VOLS "<td style=\"text-align:center; color:black;\" nowrap=\"\">$port</td>\n";
      print VOLS "<td style=\"text-align:center; color:black;\" nowrap=\"\">$vols</td>\n";
      print VOLS "</tr>\n";
    } ## end foreach my $line (@ports)

    print VOLS "</tbody>\n";
    print VOLS "</table>\n";

    my $last_update = localtime();

    print VOLS "<br><br><br>Last update: $last_update\n";
    print VOLS "</center>\n";

    close(VOLS);

    if ( -f "$port_map-tmp" ) {
      rename( "$port_map-tmp", "$port_map" );    # rename it anyway even if some problem
    }
  }    # end of FE ports

  # Storage groups allocated capacity to tiers
  if ( $tier_by_sg_idx > 0 && defined $sg_data{'TIER'} && $sg_data{'TIER'} ne '' ) {
    open( SGA, "> $sg_alloc-tmp" ) || main::error( "Can't open $sg_alloc-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;

    # header
    print SGA "<center>\n";
    print SGA "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
    print SGA "<thead>\n";
    print SGA "<tr>\n";
    print SGA "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Storage Group</th>\n";
    print SGA "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Allocated Capacity (TB)</th>\n";
    foreach my $tier_id ( sort keys( %{ $sg_data{'TIER'} } ) ) {
      chomp $tier_id;

      print SGA "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">$tier_id</th>\n";
    }
    print SGA "</tr>\n";
    print SGA "</thead>\n";
    print SGA "<tbody>\n";

    if ( defined $sg_data{'SG'} && $sg_data{'SG'} ne '' ) {
      foreach my $sg_id ( sort keys( %{ $sg_data{'SG'} } ) ) {
        chomp $sg_id;

        my $total_cap = "";
        if ( defined $sg_data{'SG'}{$sg_id}{'TOTAL'} && isdigit( $sg_data{'SG'}{$sg_id}{'TOTAL'} ) ) {
          $total_cap = $sg_data{'SG'}{$sg_id}{'TOTAL'} / 1024;    # from GB to TB
          if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {
            $total_cap = $total_cap * $ENV{DEMO_UPDATE_RRD};
          }
          $total_cap = sprintf( '%.2f', $total_cap );
        } ## end if ( defined $sg_data{...})

        print SGA "<tr>\n";
        print SGA "<td style=\"text-align:center; color:black;\" nowrap=\"\">$sg_id</td>\n";
        print SGA "<td style=\"text-align:center; color:black;\" nowrap=\"\">$total_cap</td>\n";

        foreach my $tier_id ( sort keys( %{ $sg_data{'TIER'} } ) ) {
          chomp $tier_id;

          my $tier_alloc_cap = "";
          if ( defined $sg_data{'SG'}{$sg_id}{'TIER'}{$tier_id} && $sg_data{'SG'}{$sg_id}{'TIER'}{$tier_id} ne '' ) {
            if ( isdigit( $sg_data{'SG'}{$sg_id}{'TIER'}{$tier_id} ) ) {
              $tier_alloc_cap = $sg_data{'SG'}{$sg_id}{'TIER'}{$tier_id} / 1024;    # from GB to TB
              if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
                $tier_alloc_cap = $tier_alloc_cap * $ENV{DEMO_UPDATE_RRD};
              }
              $tier_alloc_cap = sprintf( '%.2f', $tier_alloc_cap );
            } ## end if ( isdigit( $sg_data...))
            else {
              $tier_alloc_cap = $sg_data{'SG'}{$sg_id}{'TIER'}{$tier_id};
            }
          } ## end if ( defined $sg_data{...})

          print SGA "<td style=\"text-align:center; color:black;\" nowrap=\"\">$tier_alloc_cap</td>\n";
        } ## end foreach my $tier_id ( sort ...)
        print SGA "</tr>\n";
      } ## end foreach my $sg_id ( sort keys...)
    } ## end if ( defined $sg_data{...})

    print SGA "</tbody>\n";
    print SGA "</table>\n";

    my $last_update = localtime();

    print SGA "<br><br><br>Last update: $last_update\n";
    print SGA "</center>\n";

    close(SGA);

    if ( -f "$sg_alloc-tmp" ) {
      rename( "$sg_alloc-tmp", "$sg_alloc" );    # rename it anyway even if some problem
    }
  } ## end if ( $tier_by_sg_idx >...)

  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }
  if ( -f "$pool_file-tmp" ) {
    rename( "$pool_file-tmp", "$pool_file" );        # rename it anyway even if some problem
  }
  if ( -f "$tier_file-tmp" ) {
    rename( "$tier_file-tmp", "$tier_file" );        # rename it anyway even if some problem
  }

  return 0;
} ## end sub load_data_vmax_conf

sub load_data_vmax_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);
  my $pool_file  = "$wrkdir/$host/pool.cfg";
  my $time       = 3600;

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  open( CFG, "< $pool_file" ) || main::error( "Can't open $pool_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @pool_cfg = <CFG>;
  close(CFG);

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  my $section    = "";
  my $rrd        = "";
  my $type       = "NA";
  my $pool_index = 0;
  my %pool_data;
  my $time_last_ok = "";

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                        { next; }
    if ( $line =~ m/^ID,time,timestamp,/ )            { next; }
    if ( $line =~ m/^SG_ID,time,timestamp,/ )         { next; }
    if ( $line =~ m/^SG_ID,Tier_id,time,timestamp,/ ) { next; }

    if ( $line =~ m/^$/ )                                 { $section = "NA";          next; }
    if ( $line eq "CachePartition Level Statistics" )     { $section = "CACHE";       next; }
    if ( $line eq "Array Level Statistics" )              { $section = "ARRAY";       next; }
    if ( $line eq "StorageGroup Level Statistics" )       { $section = "VOLUME";      next; }
    if ( $line eq "ThinPool Level Statistics" )           { $section = "POOL";        next; }
    if ( $line eq "ThinTier Level Statistics" )           { $section = "TIER";        next; }
    if ( $line eq "FEDirectorByPort Level Statistics" )   { $section = "PORT";        next; }
    if ( $line eq "FePort Level Statistics" )             { $section = "PORT";        next; }
    if ( $line eq "BePort Level Statistics" )             { $section = "PORT";        next; }
    if ( $line eq "RDFPort Level Statistics" )            { $section = "PORT";        next; }
    if ( $line eq "FEDirector Level Statistics" )         { $section = "FEDIRECTOR";  next; }
    if ( $line eq "BEDirector Level Statistics" )         { $section = "BEDIRECTOR";  next; }
    if ( $line eq "RDFDirector Level Statistics" )        { $section = "RDFDIRECTOR"; next; }
    if ( $line eq "DiskGroup Level Statistics" )          { $section = "DISK";        next; }
    if ( $line eq "StorageGroupByTier Level Statistics" ) { $section = "FAST-SG";     next; }
    if ( $line eq "Host Level Statistics" )               { $section = "HOST";        next; }
    if ( $line eq "SLO Level Statistics" )                { $section = "SLO";         next; }

    ## set start time
    #if ( $line =~ m/^Start:/ ) {
    #  $line =~ s/^Start:\s+//;
    #  $time = str2time($line);

    #  if ( ! defined ($time) || isdigit ($time) == 0 ) {
    #    $section = "NA"; # clear out type when empty line
    #    main::error("no corret time format: $line ".__FILE__.":".__LINE__);
    #  }
    #  print "003 $line - $time\n" if $DEBUG == 3;
    #  next;
    #}

    $type = $section;

    # Volume
    ########################

    if ( $section eq "VOLUME" ) {
      print "000 $line\n" if $DEBUG == 3;

      # SG_ID,time,timestamp,DEVICE_CAPACITY (GB),IO_RATE,MB_READ_PER_SEC (kB),MB_WRITE_PER_SEC (kB),PERCENT_HIT,PERCENT_READ,PERCENT_READ_HIT,PERCENT_WRITE,PERCENT_WRITE_HIT,READ_HIT_PER_SEC,READS,WRITE_HIT_PER_SEC,TOTAL_HIT_PER_SEC,WRITES,RESPONSE_TIME,data_rate
      # migr_DATA_AIX-SG1PROD_DCH,2017:03:16T09:15:00,1489652100,1392.19,0.426667,0,0.00166667,99.99983,0,0,100,99.99983,0,0,0.4266663,0.4266663,0.426667,2.0627255,0

      my ( $volume_name, $human_time, $timestamp, $capacity_gb, $io_rate, $read, $write, $prct_hits, $prct_read, $prct_read_hits, $prct_write, $prct_write_hits, $read_hits, $reads, $write_hits, $hits_rate, $writes, $resp_t, $data_rate ) = split( /,/, $line );

      if ( !defined($capacity_gb)     || $capacity_gb eq ''     || !isdigit($capacity_gb) )     { $capacity_gb     = 'U'; }
      if ( !defined($data_rate)       || $data_rate eq ''       || !isdigit($data_rate) )       { $data_rate       = 'U'; }
      if ( !defined($io_rate)         || $io_rate eq ''         || !isdigit($io_rate) )         { $io_rate         = 'U'; }
      if ( !defined($read)            || $read eq ''            || !isdigit($read) )            { $read            = 'U'; }
      if ( !defined($write)           || $write eq ''           || !isdigit($write) )           { $write           = 'U'; }
      if ( !defined($prct_hits)       || $prct_hits eq ''       || !isdigit($prct_hits) )       { $prct_hits       = 'U'; }
      if ( !defined($prct_read)       || $prct_read eq ''       || !isdigit($prct_read) )       { $prct_read       = 'U'; }
      if ( !defined($prct_read_hits)  || $prct_read_hits eq ''  || !isdigit($prct_read_hits) )  { $prct_read_hits  = 'U'; }
      if ( !defined($prct_write)      || $prct_write eq ''      || !isdigit($prct_write) )      { $prct_write      = 'U'; }
      if ( !defined($prct_write_hits) || $prct_write_hits eq '' || !isdigit($prct_write_hits) ) { $prct_write_hits = 'U'; }
      if ( !defined($read_hits)       || $read_hits eq ''       || !isdigit($read_hits) )       { $read_hits       = 'U'; }
      if ( !defined($reads)           || $reads eq ''           || !isdigit($reads) )           { $reads           = 'U'; }
      if ( !defined($write_hits)      || $write_hits eq ''      || !isdigit($write_hits) )      { $write_hits      = 'U'; }
      if ( !defined($hits_rate)       || $hits_rate eq ''       || !isdigit($hits_rate) )       { $hits_rate       = 'U'; }
      if ( !defined($writes)          || $writes eq ''          || !isdigit($writes) )          { $writes          = 'U'; }
      if ( !defined($resp_t)          || $resp_t eq ''          || !isdigit($resp_t) )          { $resp_t          = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($capacity_gb) ) { $capacity_gb = $capacity_gb * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($data_rate) )   { $data_rate   = $data_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($io_rate) )     { $io_rate     = $io_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($read) )        { $read        = $read * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write) )       { $write       = $write * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($read_hits) )   { $read_hits   = $read_hits * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($reads) )       { $reads       = $reads * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write_hits) )  { $write_hits  = $write_hits * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($hits_rate) )   { $hits_rate   = $hits_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($writes) )      { $writes      = $writes * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      $rrd = "$wrkdir/$host/$section/$volume_name.rrd";

      print "001 VOL $volume_name - $human_time - $timestamp - $capacity_gb - $io_rate - $read - $write - $prct_hits - $prct_read - $prct_read_hits - $prct_write - $prct_write_hits - $read_hits - $reads - $write_hits - $hits_rate - $writes - $resp_t - $data_rate\n" if $DEBUG == 3;

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime    = "";
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "005: $rrd : $volume_id: $ltime \n";
      #print "006: $rrd : $volume_id: $ltime : $time \n";

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$capacity_gb:$io_rate:$read:$write:$prct_hits:$prct_read:$prct_read_hits:$prct_write:$prct_write_hits:$read_hits:$reads:$write_hits:$hits_rate:$writes:$resp_t:$data_rate);
        my $answer = RRDp::read;
      }
    } ## end if ( $section eq "VOLUME")

    # Port
    ########################

    if ( $section eq "PORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,Reads,Writes,IOs,MBRead (kB),MBWritten (kB),MBs (kB),AvgIOSize,SpeedGBs,MaxSpeedGBs,PercentBusy,ResponseTime,ReadResponseTime,WriteResponseTime
      # FA-10E-0,2017:04:06T09:35:00,1491464100,1096.39,909.30664,2005.6967,17506.494464,27073.54624,44580.038656,22.226711,16,16,2.6571772,0,0,0

      my ( $port_id, $human_time, $timestamp, $read_io, $write_io, $io_rate, $read, $write, $data_rate, undef, undef, undef, $busy, $resp_t, $resp_t_r, $resp_t_w ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($read)      || $read eq ''      || !isdigit($read) )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq ''     || !isdigit($write) )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($busy)      || $busy eq ''      || !isdigit($busy) )      { $busy      = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq ''    || !isdigit($resp_t) )    { $resp_t    = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq ''  || !isdigit($resp_t_r) )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq ''  || !isdigit($resp_t_w) )  { $resp_t_w  = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($read_io) )   { $read_io   = $read_io * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write_io) )  { $write_io  = $write_io * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($io_rate) )   { $io_rate   = $io_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($read) )      { $read      = $read * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write) )     { $write     = $write * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($data_rate) ) { $data_rate = $data_rate * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      print "001 PORT $port_id - $human_time - $timestamp - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $busy - $resp_t - $resp_t_r - $resp_t_w\n" if $DEBUG == 3;

      $rrd = "$wrkdir/$host/$section/$port_id.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$busy:$resp_t:$resp_t_r:$resp_t_w);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "PORT")

    # Disk
    ########################

    if ( $section eq "DISK" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,IO_RATE,MB_RATE (kB),MB_READ_PER_SEC (kB),MB_WRITE_PER_SEC (kB),PERCENT_DISK_BUSY,PERCENT_FREE_CAPACITY,PERCENT_USED_CAPACITY,READS,RESPONSE_TIME_READ,RESPONSE_TIME,RESPONSE_TIME_WRITE,USED_DISK_CAPACITY_GB (TB),WRITES,TOTAL_DISK_CAPACITY_GB (TB)
      # 4,2017:03:16T10:15:00,1489655700,2170.56,126.2053,60.3954,65.8099,6.67451,0.0009786147,99.99902,1172.5,8.204094,8.254002,8.312632,89411.234,998.063,89412.11

      my ( $disk_id, $human_time, $timestamp, $io_rate, $data_rate, $read, $write, $prct_busy, $prct_cap_free, $prct_cap_used, $reads, $resp_t_r, $resp_t, $resp_t_w, $cap_used, $writes, $cap ) = split( /,/, $line );

      if ( !defined($io_rate)       || $io_rate eq ''       || !isdigit($io_rate) )       { $io_rate       = 'U'; }
      if ( !defined($data_rate)     || $data_rate eq ''     || !isdigit($data_rate) )     { $data_rate     = 'U'; }
      if ( !defined($read)          || $read eq ''          || !isdigit($read) )          { $read          = 'U'; }
      if ( !defined($write)         || $write eq ''         || !isdigit($write) )         { $write         = 'U'; }
      if ( !defined($prct_busy)     || $prct_busy eq ''     || !isdigit($prct_busy) )     { $prct_busy     = 'U'; }
      if ( !defined($prct_cap_free) || $prct_cap_free eq '' || !isdigit($prct_cap_free) ) { $prct_cap_free = 'U'; }
      if ( !defined($prct_cap_used) || $prct_cap_used eq '' || !isdigit($prct_cap_used) ) { $prct_cap_used = 'U'; }
      if ( !defined($reads)         || $reads eq ''         || !isdigit($reads) )         { $reads         = 'U'; }
      if ( !defined($resp_t_r)      || $resp_t_r eq ''      || !isdigit($resp_t_r) )      { $resp_t_r      = 'U'; }
      if ( !defined($resp_t)        || $resp_t eq ''        || !isdigit($resp_t) )        { $resp_t        = 'U'; }
      if ( !defined($resp_t_w)      || $resp_t_w eq ''      || !isdigit($resp_t_w) )      { $resp_t_w      = 'U'; }
      if ( !defined($cap_used)      || $cap_used eq ''      || !isdigit($cap_used) )      { $cap_used      = 'U'; }
      if ( !defined($writes)        || $writes eq ''        || !isdigit($writes) )        { $writes        = 'U'; }
      if ( !defined($cap)           || $cap eq ''           || !isdigit($cap) )           { $cap           = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($io_rate) )   { $io_rate   = $io_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($data_rate) ) { $data_rate = $data_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($read) )      { $read      = $read * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write) )     { $write     = $write * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($reads) )     { $reads     = $reads * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($cap_used) )  { $cap_used  = $cap_used * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($writes) )    { $writes    = $writes * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($cap) )       { $cap       = $cap * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      print "001 DISK $disk_id - $human_time - $timestamp - $io_rate - $data_rate - $read - $write - $prct_busy - $prct_cap_free - $prct_cap_used - $reads - $resp_t_r - $resp_t - $resp_t_w - $cap_used - $writes - $cap\n" if $DEBUG == 3;

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      $type = "DRIVE";
      $rrd  = "$wrkdir/$host/$type/$disk_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$data_rate:$read:$write:$prct_busy:$prct_cap_free:$prct_cap_used:$reads:$resp_t_r:$resp_t:$resp_t_w:$cap_used:$writes:$cap);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "DISK")

    # POOL capacity and performance
    ########################

    if ( $section eq "POOL" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,ALLOCATED_POOL_CAPACITY_GB (TB),ENABLED_POOL_CAPACITY_GB (TB),PERCENT_USED_CAPACITY,TOTAL_POOL_CAPACITY_GB (TB),USED_POOL_CAPACITY_GB (TB),BE_MB_READ_RATE (kB),BE_MB_TRANSFERED_PER_SEC (kB),BE_MB_WRITE_RATE (kB),BE_PERCENT_READ,BE_PERCENT_WRITE,BE_READS,BE_WRITES,BE_RESPONSE_TIME,BE_RESPONSE_TIME_READ,BE_RESPONSE_TIME_WRITE,io_rate_b
      # VP_SATA,2017:03:16T10:15:00,1489655700,200510.75,399532.22,50.18638,399532.22,200510.75,66.8262,78.5851,11.7589,84.62223,15.3777685,1074.22,195.21,12.556188,12.686601,11.838532,920.421

      my ( $pool_name, $human_time, $timestamp, $cap_alloc, $cap_enab, $prct_cap_used, $cap, $cap_used, $b_read, $b_data_rate, $b_write, $b_prct_read, $b_prct_write, $b_reads, $b_writes, $b_resp_t, $b_resp_t_r, $b_resp_t_w, $b_io_rate, $subscription ) = split( /,/, $line );

      if ( !defined($cap_alloc)     || $cap_alloc eq ''     || !isdigit($cap_alloc) )     { $cap_alloc     = 'U'; }
      if ( !defined($cap_enab)      || $cap_enab eq ''      || !isdigit($cap_enab) )      { $cap_enab      = 'U'; }
      if ( !defined($prct_cap_used) || $prct_cap_used eq '' || !isdigit($prct_cap_used) ) { $prct_cap_used = 'U'; }
      if ( !defined($cap)           || $cap eq ''           || !isdigit($cap) )           { $cap           = 'U'; }
      if ( !defined($cap_used)      || $cap_used eq ''      || !isdigit($cap_used) )      { $cap_used      = 'U'; }
      if ( !defined($b_read)        || $b_read eq ''        || !isdigit($b_read) )        { $b_read        = 'U'; }
      if ( !defined($b_data_rate)   || $b_data_rate eq ''   || !isdigit($b_data_rate) )   { $b_data_rate   = 'U'; }
      if ( !defined($b_io_rate)     || $b_io_rate eq ''     || !isdigit($b_io_rate) )     { $b_io_rate     = 'U'; }
      if ( !defined($b_write)       || $b_write eq ''       || !isdigit($b_write) )       { $b_write       = 'U'; }
      if ( !defined($b_prct_read)   || $b_prct_read eq ''   || !isdigit($b_prct_read) )   { $b_prct_read   = 'U'; }
      if ( !defined($b_prct_write)  || $b_prct_write eq ''  || !isdigit($b_prct_write) )  { $b_prct_write  = 'U'; }
      if ( !defined($b_reads)       || $b_reads eq ''       || !isdigit($b_reads) )       { $b_reads       = 'U'; }
      if ( !defined($b_writes)      || $b_writes eq ''      || !isdigit($b_writes) )      { $b_writes      = 'U'; }
      if ( !defined($b_resp_t)      || $b_resp_t eq ''      || !isdigit($b_resp_t) )      { $b_resp_t      = 'U'; }
      if ( !defined($b_resp_t_r)    || $b_resp_t_r eq ''    || !isdigit($b_resp_t_r) )    { $b_resp_t_r    = 'U'; }
      if ( !defined($b_resp_t_w)    || $b_resp_t_w eq ''    || !isdigit($b_resp_t_w) )    { $b_resp_t_w    = 'U'; }
      if ( !defined($subscription)  || $subscription eq ''  || !isdigit($subscription) )  { $subscription  = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($cap_alloc) )   { $cap_alloc   = $cap_alloc * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($cap_enab) )    { $cap_enab    = $cap_enab * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($cap) )         { $cap         = $cap * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($cap_used) )    { $cap_used    = $cap_used * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_read) )      { $b_read      = $b_read * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_data_rate) ) { $b_data_rate = $b_data_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_io_rate) )   { $b_io_rate   = $b_io_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_write) )     { $b_write     = $b_write * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_reads) )     { $b_reads     = $b_reads * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_writes) )    { $b_writes    = $b_writes * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #
      # Capacity
      #
      print "001 POOL cap $pool_name - $human_time - $timestamp - $cap_alloc - $cap_enab - $prct_cap_used - $cap - $cap_used - $subscription\n" if $DEBUG == 3;

      $type = "POOLcap";
      $rrd  = "$wrkdir/$host/POOL/$pool_name-cap.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cap_alloc:$cap_enab:$prct_cap_used:$cap:$cap_used:$subscription);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }

      #
      # Performance
      #
      print "001 POOL cap $pool_name - $human_time - $timestamp - $b_read - $b_data_rate - $b_write - $b_prct_read - $b_prct_write - $b_reads - $b_writes - $b_resp_t - $b_resp_t_r - $b_resp_t_w - $b_io_rate\n" if $DEBUG == 3;

      $type = "POOL";
      $rrd  = "$wrkdir/$host/POOL/$pool_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      $ltime    = "";
      $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$b_read:$b_data_rate:$b_write:$b_reads:$b_writes:$b_resp_t:$b_resp_t_r:$b_resp_t_w:$b_io_rate:U:U:U:U:U:U:U:U:U);

        #RRDp::cmd qq(update "$rrd" $time:$b_read:$b_data_rate:$b_write:$b_reads:$b_writes:$b_resp_t:$b_resp_t_r:$b_resp_t_w:$b_io_rate:read:data_rate:write:reads:writes:resp_t:resp_t_r:resp_t_w:io_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $section eq "POOL")

    # TIER capacity and performance
    ########################

    if ( $section eq "TIER" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,ALLOCATED_POOL_CAPACITY_GB (TB),AVG_BE_DISK_TIME,BE_MB_READ_RATE (kB),BE_MB_TRANSFERED_PER_SEC (kB),BE_MB_WRITE_RATE (kB),BE_READS,BE_WRITES,BE_RESPONSE_TIME,BE_RESPONSE_TIME_READ,BE_RESPONSE_TIME_WRITE,ENABLED_POOL_CAPACITY_GB (TB),PERCENT_USED_CAPACITY,TOTAL_POOL_CAPACITY_GB (TB),USED_POOL_CAPACITY_GB (TB),io_rate_b
      # FC_TIER,2017:03:17T13:00:00,1489752000,210.186005859375,0,69887.1808,287880.3968,217993.216,1103.13,2773.25,8.900476,8.586051,9.025547,346.103671875,60.729206,346.103671875,210.186005859375,4514.13

      my ( $tier_name, $human_time, $timestamp, $cap_alloc, $b_disk_time, $b_read, $b_data_rate, $b_write, $b_reads, $b_writes, $b_resp_t, $b_resp_t_r, $b_resp_t_w, $cap_enab, $prct_cap_used, $cap, $cap_used, $b_io_rate ) = split( /,/, $line );

      if ( !defined($b_disk_time)   || $b_disk_time eq ''   || !isdigit($b_disk_time) )   { $b_disk_time   = 'U'; }
      if ( !defined($cap_alloc)     || $cap_alloc eq ''     || !isdigit($cap_alloc) )     { $cap_alloc     = 'U'; }
      if ( !defined($cap_enab)      || $cap_enab eq ''      || !isdigit($cap_enab) )      { $cap_enab      = 'U'; }
      if ( !defined($prct_cap_used) || $prct_cap_used eq '' || !isdigit($prct_cap_used) ) { $prct_cap_used = 'U'; }
      if ( !defined($cap)           || $cap eq ''           || !isdigit($cap) )           { $cap           = 'U'; }
      if ( !defined($cap_used)      || $cap_used eq ''      || !isdigit($cap_used) )      { $cap_used      = 'U'; }
      if ( !defined($b_read)        || $b_read eq ''        || !isdigit($b_read) )        { $b_read        = 'U'; }
      if ( !defined($b_data_rate)   || $b_data_rate eq ''   || !isdigit($b_data_rate) )   { $b_data_rate   = 'U'; }
      if ( !defined($b_io_rate)     || $b_io_rate eq ''     || !isdigit($b_io_rate) )     { $b_io_rate     = 'U'; }
      if ( !defined($b_write)       || $b_write eq ''       || !isdigit($b_write) )       { $b_write       = 'U'; }
      if ( !defined($b_reads)       || $b_reads eq ''       || !isdigit($b_reads) )       { $b_reads       = 'U'; }
      if ( !defined($b_writes)      || $b_writes eq ''      || !isdigit($b_writes) )      { $b_writes      = 'U'; }
      if ( !defined($b_resp_t)      || $b_resp_t eq ''      || !isdigit($b_resp_t) )      { $b_resp_t      = 'U'; }
      if ( !defined($b_resp_t_r)    || $b_resp_t_r eq ''    || !isdigit($b_resp_t_r) )    { $b_resp_t_r    = 'U'; }
      if ( !defined($b_resp_t_w)    || $b_resp_t_w eq ''    || !isdigit($b_resp_t_w) )    { $b_resp_t_w    = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($cap_alloc) )   { $cap_alloc   = $cap_alloc * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($cap_enab) )    { $cap_enab    = $cap_enab * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($cap) )         { $cap         = $cap * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($cap_used) )    { $cap_used    = $cap_used * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_read) )      { $b_read      = $b_read * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_data_rate) ) { $b_data_rate = $b_data_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_io_rate) )   { $b_io_rate   = $b_io_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_write) )     { $b_write     = $b_write * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_reads) )     { $b_reads     = $b_reads * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($b_writes) )    { $b_writes    = $b_writes * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #
      # Capacity
      #
      print "001 TIER cap $tier_name - $human_time - $timestamp - $cap_alloc - $cap_enab - $prct_cap_used - $cap - $cap_used - $b_io_rate\n" if $DEBUG == 3;

      $type = "POOLcap";
      $rrd  = "$wrkdir/$host/TIER/$tier_name-cap.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cap_alloc:$cap_enab:$prct_cap_used:$cap:$cap_used:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }

      #
      # Performance
      #
      print "001 TIER cap $tier_name - $human_time - $timestamp - $b_read - $b_data_rate - $b_write - U - U - $b_reads - $b_writes - $b_resp_t - $b_resp_t_r - $b_resp_t_w - $b_disk_time\n" if $DEBUG == 3;

      $type = "POOL";
      $rrd  = "$wrkdir/$host/TIER/$tier_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      $ltime    = "";
      $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$b_read:$b_data_rate:$b_write:$b_reads:$b_writes:$b_resp_t:$b_resp_t_r:$b_resp_t_w:$b_io_rate:U:U:U:U:U:U:U:U:U);

        #RRDp::cmd qq(update "$rrd" $time:$b_read:$b_data_rate:$b_write:$b_reads:$b_writes:$b_resp_t:$b_resp_t_r:$b_resp_t_w:$b_io_rate:read:data_rate:write:reads:writes:resp_t:resp_t_r:resp_t_w:io_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $section eq "TIER")

    # Cache
    ########################

    if ( $section eq "CACHE" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,IO_RATE,MB_RATE (kB),PERCENT_BUSY,PERCENT_HIT,SIZE (%),WP,WP_LIMIT
      # DEFAULT_PARTITION,2017:03:20T10:45:00,1490003100,39356006.4,1519.13,5.4475,77.9133,100,144198,2657400

      my ( $cache_name, $human_time, $timestamp, $IO_RATE, $MB_RATE, $PERCENT_BUSY, $PERCENT_HIT, $SIZE, $WP, $WP_LIMIT ) = split( /,/, $line );

      # PERCENT_BUSY = WP (Cache WP %)!
      # Calculated value: (WP Count / (total slots * % Cache Used / 100) * 100)

      if ( !defined($IO_RATE)      || $IO_RATE eq ''      || !isdigit($IO_RATE) )      { $IO_RATE      = 'U'; }
      if ( !defined($MB_RATE)      || $MB_RATE eq ''      || !isdigit($MB_RATE) )      { $MB_RATE      = 'U'; }
      if ( !defined($PERCENT_BUSY) || $PERCENT_BUSY eq '' || !isdigit($PERCENT_BUSY) ) { $PERCENT_BUSY = 'U'; }
      if ( !defined($PERCENT_HIT)  || $PERCENT_HIT eq ''  || !isdigit($PERCENT_HIT) )  { $PERCENT_HIT  = 'U'; }
      if ( !defined($SIZE)         || $SIZE eq ''         || !isdigit($SIZE) )         { $SIZE         = 'U'; }
      if ( !defined($WP)           || $WP eq ''           || !isdigit($WP) )           { $WP           = 'U'; }
      if ( !defined($WP_LIMIT)     || $WP_LIMIT eq ''     || !isdigit($WP_LIMIT) )     { $WP_LIMIT     = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($IO_RATE) ) { $IO_RATE = $IO_RATE * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_RATE) ) { $MB_RATE = $MB_RATE * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($SIZE) )    { $SIZE    = $SIZE * $ENV{DEMO_UPDATE_RRD}; }
      }

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      print "001 Cache $cache_name - $human_time - $timestamp - $IO_RATE - $MB_RATE - $PERCENT_BUSY - $PERCENT_HIT - $SIZE - $WP - $WP_LIMIT\n" if $DEBUG == 3;

      $type = "NODE-CACHE";
      $rrd  = "$wrkdir/$host/$type/$cache_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime    = "";
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$IO_RATE:$MB_RATE:$PERCENT_BUSY:$PERCENT_HIT:$SIZE:$WP:$WP_LIMIT);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "CACHE")

    # ARRAY
    ########################

    if ( $section eq "ARRAY" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,PERCENT_WRITE,PERCENT_READ,HIT_PER_SEC,READ_HIT_PER_SEC,READS,HA_MB_PER_SEC,MB_WRITE_PER_SEC (kB),WRITE_HIT_PER_SEC,TOTAL_CACHE_UTILIZATION,PERCENT_HIT,BE_READS,MB_READ_PER_SEC (kB),RESPONSE_TIME_READ,WRITES,BE_IOS_PER_SEC,RESPONSE_TIME_WRITE,BE_WRITES,IO_RATE,data_rate
      # 000295701111,2017:03:21T08:00:00,1490079600,35.476715,64.523285,37559.2,22077.9,19073.7,1203.9,290.2539,15481.3,3.69304,83.65447,15722.3,913.6465,2.986461,12647.3,37537.6,1.074569,8781.18,29824.6

      my ( $id, $human_time, $timestamp, $PERCENT_WRITE, $PERCENT_READ, $HIT_PER_SEC, $READ_HIT_PER_SEC, $READS, $HA_MB_PER_SEC, $MB_WRITE_PER_SEC, $WRITE_HIT_PER_SEC, $TOTAL_CACHE_UTILIZATION, $PERCENT_HIT, $BE_READS, $MB_READ_PER_SEC, $RESPONSE_TIME_READ, $WRITES, $BE_IOS_PER_SEC, $RESPONSE_TIME_WRITE, $BE_WRITES, $IO_RATE, $data_rate ) = split( /,/, $line );

      if ( !defined($PERCENT_WRITE)           || $PERCENT_WRITE eq ''           || !isdigit($PERCENT_WRITE) )           { $PERCENT_WRITE           = 'U'; }
      if ( !defined($PERCENT_READ)            || $PERCENT_READ eq ''            || !isdigit($PERCENT_READ) )            { $PERCENT_READ            = 'U'; }
      if ( !defined($HIT_PER_SEC)             || $HIT_PER_SEC eq ''             || !isdigit($HIT_PER_SEC) )             { $HIT_PER_SEC             = 'U'; }
      if ( !defined($READ_HIT_PER_SEC)        || $READ_HIT_PER_SEC eq ''        || !isdigit($READ_HIT_PER_SEC) )        { $READ_HIT_PER_SEC        = 'U'; }
      if ( !defined($READS)                   || $READS eq ''                   || !isdigit($READS) )                   { $READS                   = 'U'; }
      if ( !defined($HA_MB_PER_SEC)           || $HA_MB_PER_SEC eq ''           || !isdigit($HA_MB_PER_SEC) )           { $HA_MB_PER_SEC           = 'U'; }
      if ( !defined($MB_WRITE_PER_SEC)        || $MB_WRITE_PER_SEC eq ''        || !isdigit($MB_WRITE_PER_SEC) )        { $MB_WRITE_PER_SEC        = 'U'; }
      if ( !defined($WRITE_HIT_PER_SEC)       || $WRITE_HIT_PER_SEC eq ''       || !isdigit($WRITE_HIT_PER_SEC) )       { $WRITE_HIT_PER_SEC       = 'U'; }
      if ( !defined($TOTAL_CACHE_UTILIZATION) || $TOTAL_CACHE_UTILIZATION eq '' || !isdigit($TOTAL_CACHE_UTILIZATION) ) { $TOTAL_CACHE_UTILIZATION = 'U'; }
      if ( !defined($PERCENT_HIT)             || $PERCENT_HIT eq ''             || !isdigit($PERCENT_HIT) )             { $PERCENT_HIT             = 'U'; }
      if ( !defined($BE_READS)                || $BE_READS eq ''                || !isdigit($BE_READS) )                { $BE_READS                = 'U'; }
      if ( !defined($MB_READ_PER_SEC)         || $MB_READ_PER_SEC eq ''         || !isdigit($MB_READ_PER_SEC) )         { $MB_READ_PER_SEC         = 'U'; }
      if ( !defined($RESPONSE_TIME_READ)      || $RESPONSE_TIME_READ eq ''      || !isdigit($RESPONSE_TIME_READ) )      { $RESPONSE_TIME_READ      = 'U'; }
      if ( !defined($WRITES)                  || $WRITES eq ''                  || !isdigit($WRITES) )                  { $WRITES                  = 'U'; }
      if ( !defined($BE_IOS_PER_SEC)          || $BE_IOS_PER_SEC eq ''          || !isdigit($BE_IOS_PER_SEC) )          { $BE_IOS_PER_SEC          = 'U'; }
      if ( !defined($RESPONSE_TIME_WRITE)     || $RESPONSE_TIME_WRITE eq ''     || !isdigit($RESPONSE_TIME_WRITE) )     { $RESPONSE_TIME_WRITE     = 'U'; }
      if ( !defined($BE_WRITES)               || $BE_WRITES eq ''               || !isdigit($BE_WRITES) )               { $BE_WRITES               = 'U'; }
      if ( !defined($IO_RATE)                 || $IO_RATE eq ''                 || !isdigit($IO_RATE) )                 { $IO_RATE                 = 'U'; }
      if ( !defined($data_rate)               || $data_rate eq ''               || !isdigit($data_rate) )               { $data_rate               = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($READS) )            { $READS            = $READS * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($HA_MB_PER_SEC) )    { $HA_MB_PER_SEC    = $HA_MB_PER_SEC * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_WRITE_PER_SEC) ) { $MB_WRITE_PER_SEC = $MB_WRITE_PER_SEC * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($BE_READS) )         { $BE_READS         = $BE_READS * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_READ_PER_SEC) )  { $MB_READ_PER_SEC  = $MB_READ_PER_SEC * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($WRITES) )           { $WRITES           = $WRITES * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($BE_IOS_PER_SEC) )   { $BE_IOS_PER_SEC   = $BE_IOS_PER_SEC * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($BE_WRITES) )        { $BE_WRITES        = $BE_WRITES * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($IO_RATE) )          { $IO_RATE          = $IO_RATE * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($data_rate) )        { $data_rate        = $data_rate * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      print "001 Array total (POOL) $id - $human_time - $timestamp - $PERCENT_WRITE - $PERCENT_READ - $HIT_PER_SEC - $READ_HIT_PER_SEC - $READS - $HA_MB_PER_SEC - $MB_WRITE_PER_SEC - $WRITE_HIT_PER_SEC - $TOTAL_CACHE_UTILIZATION - $PERCENT_HIT - $BE_READS - $MB_READ_PER_SEC - $RESPONSE_TIME_READ - $WRITES - $BE_IOS_PER_SEC - $RESPONSE_TIME_WRITE - $BE_WRITES - $IO_RATE - $data_rate\n" if $DEBUG == 3;

      $type = "POOL";
      $rrd  = "$wrkdir/$host/$type/array_total.rra";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime    = "";
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$READS:$MB_WRITE_PER_SEC:$PERCENT_HIT:$MB_READ_PER_SEC:$RESPONSE_TIME_READ:$WRITES:$RESPONSE_TIME_WRITE:$IO_RATE:$data_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "ARRAY")

    # FEDirector (NODE)
    ########################

    if ( $section eq "FEDIRECTOR" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,READ_HIT_PER_SEC,HIT_PER_SEC,RESPONSE_TIME_WRITE,RESPONSE_TIME_READ,WRITE_HIT_PER_SEC,PERCENT_READ_HIT,PERCENT_WRITE_HIT,PERCENT_HIT,PERCENT_BUSY,WRITES,READS,IO_RATE
      # FA-9F,2017:03:27T21:10:00,1490641800,958.41,1479.55,1.3826495,1.6953582,521.14,84.44588,99.57335,89.22035,15.3642,523.373,1134.94,957.997

      my ( $dir_id, $human_time, $timestamp, $READ_HIT_PER_SEC, $HIT_PER_SEC, $RESPONSE_TIME_WRITE, $RESPONSE_TIME_READ, $WRITE_HIT_PER_SEC, $PERCENT_READ_HIT, $PERCENT_WRITE_HIT, $PERCENT_HIT, $PERCENT_BUSY, $WRITES, $READS, $IO_RATE ) = split( /,/, $line );

      if ( !defined($READ_HIT_PER_SEC)    || $READ_HIT_PER_SEC eq ''    || !isdigit($READ_HIT_PER_SEC) )    { $READ_HIT_PER_SEC    = 'U'; }
      if ( !defined($HIT_PER_SEC)         || $HIT_PER_SEC eq ''         || !isdigit($HIT_PER_SEC) )         { $HIT_PER_SEC         = 'U'; }
      if ( !defined($RESPONSE_TIME_WRITE) || $RESPONSE_TIME_WRITE eq '' || !isdigit($RESPONSE_TIME_WRITE) ) { $RESPONSE_TIME_WRITE = 'U'; }
      if ( !defined($RESPONSE_TIME_READ)  || $RESPONSE_TIME_READ eq ''  || !isdigit($RESPONSE_TIME_READ) )  { $RESPONSE_TIME_READ  = 'U'; }
      if ( !defined($WRITE_HIT_PER_SEC)   || $WRITE_HIT_PER_SEC eq ''   || !isdigit($WRITE_HIT_PER_SEC) )   { $WRITE_HIT_PER_SEC   = 'U'; }
      if ( !defined($PERCENT_READ_HIT)    || $PERCENT_READ_HIT eq ''    || !isdigit($PERCENT_READ_HIT) )    { $PERCENT_READ_HIT    = 'U'; }
      if ( !defined($PERCENT_WRITE_HIT)   || $PERCENT_WRITE_HIT eq ''   || !isdigit($PERCENT_WRITE_HIT) )   { $PERCENT_WRITE_HIT   = 'U'; }
      if ( !defined($PERCENT_HIT)         || $PERCENT_HIT eq ''         || !isdigit($PERCENT_HIT) )         { $PERCENT_HIT         = 'U'; }
      if ( !defined($PERCENT_BUSY)        || $PERCENT_BUSY eq ''        || !isdigit($PERCENT_BUSY) )        { $PERCENT_BUSY        = 'U'; }
      if ( !defined($WRITES)              || $WRITES eq ''              || !isdigit($WRITES) )              { $WRITES              = 'U'; }
      if ( !defined($READS)               || $READS eq ''               || !isdigit($READS) )               { $READS               = 'U'; }
      if ( !defined($IO_RATE)             || $IO_RATE eq ''             || !isdigit($IO_RATE) )             { $IO_RATE             = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($WRITES) )  { $WRITES  = $WRITES * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($READS) )   { $READS   = $READS * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($IO_RATE) ) { $IO_RATE = $IO_RATE * $ENV{DEMO_UPDATE_RRD}; }
      }

      print "001 NODE (FEDirector) $dir_id - $human_time - $timestamp - $READ_HIT_PER_SEC - $HIT_PER_SEC - $RESPONSE_TIME_WRITE - $RESPONSE_TIME_READ - $WRITE_HIT_PER_SEC - $PERCENT_READ_HIT - $PERCENT_WRITE_HIT - $PERCENT_HIT - $PERCENT_BUSY - $WRITES - $READS - $IO_RATE\n" if $DEBUG == 3;

      $type = "NODE";
      $rrd  = "$wrkdir/$host/$type/$dir_id.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$PERCENT_BUSY:$READS:$WRITES:$IO_RATE:$RESPONSE_TIME_READ:$RESPONSE_TIME_WRITE:$PERCENT_READ_HIT:$PERCENT_WRITE_HIT:$PERCENT_HIT:U:U:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "FEDIRECTOR")

    # BEDirector (NODE)
    ########################

    if ( $section eq "BEDIRECTOR" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,IO_RATE,MB_RATE (kB),MB_READ_PER_SEC (kB),MB_WRITE_PER_SEC (kB),PERCENT_BUSY,READS,WRITES
      # DF-6B,2017:03:28T09:55:00,1490687700,1100.42,54.8225,34.414257,20.408203,19.8171,470.967,253.43

      # RRDp::cmd qq(update "$rrd" $time:$PERCENT_BUSY:$READS:$WRITES:$IO_RATE:$RESPONSE_TIME_READ:$RESPONSE_TIME_WRITE:$PERCENT_READ_HIT:$PERCENT_WRITE_HIT:$PERCENT_HITi:U:U:U);
      # RRDp::cmd qq(update "$rrd" $time:$PERCENT_BUSY:$READS:$WRITES:$IO_RATE:U:U:U:U:U:MB_RATE:MB_READ_PER_SEC:MB_WRITE_PER_SEC);

      my ( $dir_id, $human_time, $timestamp, $IO_RATE, $MB_RATE, $MB_READ_PER_SEC, $MB_WRITE_PER_SEC, $PERCENT_BUSY, $READS, $WRITES ) = split( /,/, $line );

      if ( !defined($IO_RATE)          || $IO_RATE eq ''          || !isdigit($IO_RATE) )          { $IO_RATE          = 'U'; }
      if ( !defined($MB_RATE)          || $MB_RATE eq ''          || !isdigit($MB_RATE) )          { $MB_RATE          = 'U'; }
      if ( !defined($MB_READ_PER_SEC)  || $MB_READ_PER_SEC eq ''  || !isdigit($MB_READ_PER_SEC) )  { $MB_READ_PER_SEC  = 'U'; }
      if ( !defined($MB_WRITE_PER_SEC) || $MB_WRITE_PER_SEC eq '' || !isdigit($MB_WRITE_PER_SEC) ) { $MB_WRITE_PER_SEC = 'U'; }
      if ( !defined($PERCENT_BUSY)     || $PERCENT_BUSY eq ''     || !isdigit($PERCENT_BUSY) )     { $PERCENT_BUSY     = 'U'; }
      if ( !defined($READS)            || $READS eq ''            || !isdigit($READS) )            { $READS            = 'U'; }
      if ( !defined($WRITES)           || $WRITES eq ''           || !isdigit($WRITES) )           { $WRITES           = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($IO_RATE) )          { $IO_RATE          = $IO_RATE * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_RATE) )          { $MB_RATE          = $MB_RATE * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_READ_PER_SEC) )  { $MB_READ_PER_SEC  = $MB_READ_PER_SEC * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_WRITE_PER_SEC) ) { $MB_WRITE_PER_SEC = $MB_WRITE_PER_SEC * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($READS) )            { $READS            = $READS * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($WRITES) )           { $WRITES           = $WRITES * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      print "001 NODE (BEDirector) $dir_id - $human_time - $timestamp - $IO_RATE - $MB_RATE - $MB_READ_PER_SEC - $MB_WRITE_PER_SEC - $PERCENT_BUSY - $READS - $WRITES\n" if $DEBUG == 3;

      $type = "NODE";
      $rrd  = "$wrkdir/$host/$type/$dir_id.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$PERCENT_BUSY:$READS:$WRITES:$IO_RATE:U:U:U:U:U:$MB_RATE:$MB_READ_PER_SEC:$MB_WRITE_PER_SEC);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "BEDIRECTOR")

    # RDFDirector (NODE)
    ########################

    if ( $section eq "RDFDIRECTOR" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,IO_RATE,MB_RATE (kB),MB_READ_PER_SEC (kB),MB_WRITE_PER_SEC (kB),PERCENT_BUSY,READS,WRITES
      # DF-6B,2017:03:28T09:55:00,1490687700,1100.42,54.8225,34.414257,20.408203,19.8171,470.967,253.43

      # RRDp::cmd qq(update "$rrd" $time:$PERCENT_BUSY:$READS:$WRITES:$IO_RATE:$RESPONSE_TIME_READ:$RESPONSE_TIME_WRITE:$PERCENT_READ_HIT:$PERCENT_WRITE_HIT:$PERCENT_HITi:U:U:U);
      # RRDp::cmd qq(update "$rrd" $time:$PERCENT_BUSY:$READS:$WRITES:$IO_RATE:U:U:U:U:U:MB_RATE:MB_READ_PER_SEC:MB_WRITE_PER_SEC);

      my ( $dir_id, $human_time, $timestamp, $IO_RATE, $MB_RATE, $MB_READ_PER_SEC, $MB_WRITE_PER_SEC, $PERCENT_BUSY, $READS, $WRITES ) = split( /,/, $line );

      if ( !defined($IO_RATE)          || $IO_RATE eq ''          || !isdigit($IO_RATE) )          { $IO_RATE          = 'U'; }
      if ( !defined($MB_RATE)          || $MB_RATE eq ''          || !isdigit($MB_RATE) )          { $MB_RATE          = 'U'; }
      if ( !defined($MB_READ_PER_SEC)  || $MB_READ_PER_SEC eq ''  || !isdigit($MB_READ_PER_SEC) )  { $MB_READ_PER_SEC  = 'U'; }
      if ( !defined($MB_WRITE_PER_SEC) || $MB_WRITE_PER_SEC eq '' || !isdigit($MB_WRITE_PER_SEC) ) { $MB_WRITE_PER_SEC = 'U'; }
      if ( !defined($PERCENT_BUSY)     || $PERCENT_BUSY eq ''     || !isdigit($PERCENT_BUSY) )     { $PERCENT_BUSY     = 'U'; }
      if ( !defined($READS)            || $READS eq ''            || !isdigit($READS) )            { $READS            = 'U'; }
      if ( !defined($WRITES)           || $WRITES eq ''           || !isdigit($WRITES) )           { $WRITES           = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($IO_RATE) )          { $IO_RATE          = $IO_RATE * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_RATE) )          { $MB_RATE          = $MB_RATE * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_READ_PER_SEC) )  { $MB_READ_PER_SEC  = $MB_READ_PER_SEC * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($MB_WRITE_PER_SEC) ) { $MB_WRITE_PER_SEC = $MB_WRITE_PER_SEC * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($READS) )            { $READS            = $READS * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($WRITES) )           { $WRITES           = $WRITES * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      print "001 NODE (RDFDirector) $dir_id - $human_time - $timestamp - $IO_RATE - $MB_RATE - $MB_READ_PER_SEC - $MB_WRITE_PER_SEC - $PERCENT_BUSY - $READS - $WRITES\n" if $DEBUG == 3;

      $type = "NODE";
      $rrd  = "$wrkdir/$host/$type/$dir_id.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$PERCENT_BUSY:$READS:$WRITES:$IO_RATE:U:U:U:U:U:$MB_RATE:$MB_READ_PER_SEC:$MB_WRITE_PER_SEC);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "RDFDIRECTOR")

    # StorageGroupByTier (FAST-SG)
    ########################

    if ( $section eq "FAST-SG" ) {
      print "000 $line\n" if $DEBUG == 3;

      # SG_ID,Tier_id,time,timestamp,SG total capacity,Tier capacity,Tier capacity (%),SG total allocated tracks, Tier allocated tracks, Tier allocated tracks (%)
      # class_c,FC_TIER,2017:03:28T15:35:00,1490708100,125372.57,28675.1,22.9,2054107360,469864000,22.9

      my ( $sg_id, $tier_id, $human_time, $timestamp, $sg_cap_total, $tier_cap, $tier_cap_prct, $sg_alloc_total, $tier_alloc_tracks, $tier_alloc_tracks_prct ) = split( /,/, $line );

      if ( !defined $sg_id || $sg_id eq "" || !defined $tier_id || $tier_id eq "" ) {
        main::error( "Undefined STORAGE GROUP or TIER! $line " . __FILE__ . ":" . __LINE__ ) && next;
      }

      if ( !defined($sg_cap_total)           || $sg_cap_total eq ''           || !isdigit($sg_cap_total) )           { $sg_cap_total           = 'U'; }
      if ( !defined($tier_cap)               || $tier_cap eq ''               || !isdigit($tier_cap) )               { $tier_cap               = 'U'; }
      if ( !defined($tier_cap_prct)          || $tier_cap_prct eq ''          || !isdigit($tier_cap_prct) )          { $tier_cap_prct          = 'U'; }
      if ( !defined($sg_alloc_total)         || $sg_alloc_total eq ''         || !isdigit($sg_alloc_total) )         { $sg_alloc_total         = 'U'; }
      if ( !defined($tier_alloc_tracks)      || $tier_alloc_tracks eq ''      || !isdigit($tier_alloc_tracks) )      { $tier_alloc_tracks      = 'U'; }
      if ( !defined($tier_alloc_tracks_prct) || $tier_alloc_tracks_prct eq '' || !isdigit($tier_alloc_tracks_prct) ) { $tier_alloc_tracks_prct = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($sg_cap_total) )      { $sg_cap_total      = $sg_cap_total * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($tier_cap) )          { $tier_cap          = $tier_cap * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($sg_alloc_total) )    { $sg_alloc_total    = $sg_alloc_total * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($tier_alloc_tracks) ) { $tier_alloc_tracks = $tier_alloc_tracks * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      print "001 FAST-SG $sg_id - $tier_id - $human_time - $timestamp - $sg_cap_total - $tier_cap - $tier_cap_prct - $sg_alloc_total - $tier_alloc_tracks - $tier_alloc_tracks_prct\n" if $DEBUG == 3;

      $type = "FAST-SG";
      $rrd  = "$wrkdir/$host/$type/$sg_id-$tier_id.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$sg_cap_total:$tier_cap:$tier_cap_prct:$sg_alloc_total:$tier_alloc_tracks:$tier_alloc_tracks_prct);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "FAST-SG")

    # Host
    ########################

    if ( $section eq "HOST" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,HostIOs,HostMBReads,HostMBWrites,Reads,ResponseTime,ReadResponseTime,WriteResponseTime,Writes,MBs
      # IG_VPLEX01,2017:04:10T12:30:00,1491820200,41134.38,0,0,22183.566,2.5473168,3.4856849,1.448876,18950.812,1323135.5904

      my ( $host_id, $human_time, $timestamp, $io_rate, $read, $write, $read_io, $resp_t, $resp_t_r, $resp_t_w, $write_io, $data_rate ) = split( /,/, $line );

      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($read)      || $read eq ''      || !isdigit($read) )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq ''     || !isdigit($write) )     { $write     = 'U'; }
      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq ''    || !isdigit($resp_t) )    { $resp_t    = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq ''  || !isdigit($resp_t_r) )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq ''  || !isdigit($resp_t_w) )  { $resp_t_w  = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($io_rate) )   { $io_rate   = $io_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($read) )      { $read      = $read * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write) )     { $write     = $write * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($read_io) )   { $read_io   = $read_io * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write_io) )  { $write_io  = $write_io * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($data_rate) ) { $data_rate = $data_rate * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      print "001 HOST $host_id - $human_time - $timestamp - $io_rate - $read - $write - $read_io - $resp_t - $resp_t_r - $resp_t_w - $write_io - $data_rate\n" if $DEBUG == 3;

      $type = "HOST";
      $rrd  = "$wrkdir/$host/$type/$host_id.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read:$write:$read_io:$resp_t:$resp_t_r:$resp_t_w:$write_io:$data_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "HOST")

    # SLO (Service level objectives)
    ########################

    if ( $section eq "SLO" ) {
      print "000 $line\n" if $DEBUG == 3;

      # ID,time,timestamp,cap,io_rate,read,write,read_io,write_io,cache_hit,r_cache_hit,w_cache_hit,resp_t,data_rate
      # Bronze,,,107985.414215401,4734.8132647,278.456439135467,118.8739232622,2946.713307062,1788.10005004,40.4304806272727,33.9044322545454,41.6489838045454,1.40300332292072,

      my ( $slo_id, $human_time, $timestamp, $cap, $io_rate, $read, $write, $read_io, $write_io, $cache_hit, $r_cache_hit, $w_cache_hit, $resp_t, $data_rate ) = split( /,/, $line );

      if ( !defined($cap)         || $cap eq ''         || !isdigit($cap) )         { $cap         = 'U'; }
      if ( !defined($io_rate)     || $io_rate eq ''     || !isdigit($io_rate) )     { $io_rate     = 'U'; }
      if ( !defined($read)        || $read eq ''        || !isdigit($read) )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq ''       || !isdigit($write) )       { $write       = 'U'; }
      if ( !defined($read_io)     || $read_io eq ''     || !isdigit($read_io) )     { $read_io     = 'U'; }
      if ( !defined($write_io)    || $write_io eq ''    || !isdigit($write_io) )    { $write_io    = 'U'; }
      if ( !defined($cache_hit)   || $cache_hit eq ''   || !isdigit($cache_hit) )   { $cache_hit   = 'U'; }
      if ( !defined($r_cache_hit) || $r_cache_hit eq '' || !isdigit($r_cache_hit) ) { $r_cache_hit = 'U'; }
      if ( !defined($w_cache_hit) || $w_cache_hit eq '' || !isdigit($w_cache_hit) ) { $w_cache_hit = 'U'; }
      if ( !defined($resp_t)      || $resp_t eq ''      || !isdigit($resp_t) )      { $resp_t      = 'U'; }
      if ( !defined($data_rate)   || $data_rate eq ''   || !isdigit($data_rate) )   { $data_rate   = 'U'; }

      if ( defined $ENV{DEMO_UPDATE_RRD} && isdigit( $ENV{DEMO_UPDATE_RRD} ) ) {    # data to DEMO
        if ( isdigit($cap) )       { $cap       = $cap * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($io_rate) )   { $io_rate   = $io_rate * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($read) )      { $read      = $read * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write) )     { $write     = $write * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($read_io) )   { $read_io   = $read_io * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($write_io) )  { $write_io  = $write_io * $ENV{DEMO_UPDATE_RRD}; }
        if ( isdigit($data_rate) ) { $data_rate = $data_rate * $ENV{DEMO_UPDATE_RRD}; }
      } ## end if ( defined $ENV{DEMO_UPDATE_RRD...})

      print "001 SLO $slo_id - $human_time - $timestamp - $cap - $io_rate - $read - $write - $read_io - $write_io - $cache_hit - $r_cache_hit - $w_cache_hit - $resp_t - $data_rate\n" if $DEBUG == 3;

      $type = "SLO";
      $rrd  = "$wrkdir/$host/$type/$slo_id.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cap:$io_rate:$read:$write:$read_io:$write_io:$cache_hit:$r_cache_hit:$w_cache_hit:$resp_t:$data_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "SLO")

  }    # end of foreach

  return ( "0", "$time_last_ok" );
} ## end sub load_data_vmax_perf

##################################################
# EMC VPLEX
##################################################

sub load_data_vplex_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  my $in_exist    = 0;
  my $perf_string = "";
  my @files_unsorted;
  my @files;
  my $time_last_ok = "";

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # conf files
  #
  $perf_string = "vplexconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_vplex_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # perf files
  #
  $perf_string = "vplexperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    #print "$in_file\n";

    my ( $ret, $time_last_ok ) = load_data_vplex_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_vplex_all

sub load_data_vplex_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  my $pool_file     = "$wrkdir/$host/pool.cfg";
  my $config_file   = "$wrkdir/$host/config.html";
  my $capacity_file = "$wrkdir/$host/capacity.txt";

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( POOL, "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp: $! " . __FILE__ . ":" . __LINE__ )   && return 0;

  # print *conf* file to config.html
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
  <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my $section = "";

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    # print to config.html
    print FCFG "$line\n";

    if ( $line =~ m/^------/ )              { next; }
    if ( $line =~ m/^Timestamp,cap total/ ) { next; }

    if ( $line =~ m/^$/ )                       { $section = "NA";    next; }
    if ( $line eq "Array Level Configuration" ) { $section = "ARRAY"; next; }

    # total array capacity
    ########################
    if ( $section eq "ARRAY" ) {
      my ($timestamp, $cap_total, $cap_used, $cap_free) = split(",", $line);

      if ( defined $cap_total && isdigit($cap_total) ) {
        $cap_total = $cap_total / 1024; # from GiB  to TiB
        $cap_total = sprintf( '%.2f', $cap_total );
      }
      if ( defined $cap_used && isdigit($cap_used) ) {
        $cap_used = $cap_used / 1024; # from GiB  to TiB
        $cap_used = sprintf( '%.2f', $cap_used );
      }
      if ( defined $cap_free && isdigit($cap_free) ) {
        $cap_free = $cap_free / 1024; # from GiB  to TiB
        $cap_free = sprintf( '%.2f', $cap_free );
      }

      my $start_human = "";
      if ( defined $timestamp && isdigit($timestamp) ) {
        $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($timestamp);
      }

      open( CAP, "> $capacity_file" ) || main::error( "Can't open $capacity_file: $! " . __FILE__ . ":" . __LINE__ ) && return 0;

      print CAP "total $cap_total\n";
      print CAP "free $cap_free\n";
      print CAP "used $cap_used\n";
      print CAP "\n";
      print CAP "time $start_human\n";

      close(CAP);
    }
  } ## end foreach my $line (@lines)

  print FCFG "</pre></body></html>\n";
  close(FCFG);

  #array summary
  print POOL "array_total:array_total\n";
  close(POOL);

  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }
  if ( -f "$pool_file-tmp" ) {
    rename( "$pool_file-tmp", "$pool_file" );        # rename it anyway even if some problem
  }

  return 0;
} ## end sub load_data_vplex_conf

sub load_data_vplex_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);

  #my $pool_file  = "$wrkdir/$host/pool.cfg";
  my $time = 3600;

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  #open(CFG, "< $pool_file") || main::error ("Can't open $pool_file : $! : (ignore after fresh install) ".__FILE__.":".__LINE__) && return 0;
  #my @pool_cfg = <CFG>;
  #close(CFG);

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  my $section      = "";
  my $rrd          = "";
  my $type         = "NA";
  my $time_last_ok = "";
  my %pool_data;

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                                  { next; }
    if ( $line =~ m/^Virtual Volume,timestamp,date,director,/ ) { next; }
    if ( $line =~ m/^Director,timestamp,date,/ )                { next; }
    if ( $line =~ m/^Director,Port,timestamp,date,/ )           { next; }

    if ( $line =~ m/^$/ )                       { $section = "NA";     next; }
    if ( $line eq "Volume Level Statistics" )   { $section = "VOLUME"; next; }
    if ( $line eq "Director Level Statistics" ) { $section = "NODE";   next; }
    if ( $line eq "BEPort Level Statistics" )   { $section = "PORT";   next; }
    if ( $line eq "FEPort Level Statistics" )   { $section = "PORT";   next; }

    $type = $section;

    # Volume
    ########################

    if ( $section eq "VOLUME" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Virtual Volume,timestamp,date,director,io_rate,read (KB/s),write (KB/s),resp_t_r (ms),resp_t_w (ms),data_rate (KB/s)
      # DD_S111_29B8_106GB_T5VXDG_JOKDB3_1_vol,1500452326,2017-07-19 10:18:46,director-1-1-A,261.7,2093,21.2,0.7988,2.0858,2114.2

      my ( $volume_name, $timestamp, $human_time, $director, $io_rate, $read, $write, $resp_t_r, $resp_t_w, $data_rate ) = split( /,/, $line );

      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($read)      || $read eq ''      || !isdigit($read) )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq ''     || !isdigit($write) )     { $write     = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq ''  || !isdigit($resp_t_r) )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq ''  || !isdigit($resp_t_w) )  { $resp_t_w  = 'U'; }

      $rrd = "$wrkdir/$host/$section/$volume_name.rrd";

      print "001 VOL $volume_name - $human_time - $timestamp - $io_rate - $read - $write - $resp_t_r - $resp_t_w - $data_rate\n" if $DEBUG == 3;

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime    = "";
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "005: $rrd : $volume_id: $ltime \n";
      #print "006: $rrd : $volume_id: $ltime : $time \n";

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$data_rate:$read:$write:$resp_t_r:$resp_t_w);
        my $answer = RRDp::read;
      }

      #$rrd = "$wrkdir/$host/$type/array_total.rra";
      my $pool_id   = "array_total";
      my $pool_name = "array_total";

      # add values to pool
      if ( defined $pool_id && $pool_id ne '' ) {

        #print "$pool_id .. $volume_name\n";
        $pool_data{$pool_id}{'pool_name'} = $pool_name;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $pool_data{$pool_id}{'io_rate'} && isdigit( $pool_data{$pool_id}{'io_rate'} ) ) {
            $pool_data{$pool_id}{'io_rate'} = $pool_data{$pool_id}{'io_rate'} + $io_rate;
          }
          else {
            $pool_data{$pool_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)
        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $pool_data{$pool_id}{'data_rate'} && isdigit( $pool_data{$pool_id}{'data_rate'} ) ) {
            $pool_data{$pool_id}{'data_rate'} = $pool_data{$pool_id}{'data_rate'} + $data_rate;
          }
          else {
            $pool_data{$pool_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)
        if ( defined $read && isdigit($read) ) {
          if ( defined $pool_data{$pool_id}{'read'} && isdigit( $pool_data{$pool_id}{'read'} ) ) {
            $pool_data{$pool_id}{'read'} = $pool_data{$pool_id}{'read'} + $read;
          }
          else {
            $pool_data{$pool_id}{'read'} = $read;
          }
        } ## end if ( defined $read && ...)
        if ( defined $write && isdigit($write) ) {
          if ( defined $pool_data{$pool_id}{'write'} && isdigit( $pool_data{$pool_id}{'write'} ) ) {
            $pool_data{$pool_id}{'write'} = $pool_data{$pool_id}{'write'} + $write;
          }
          else {
            $pool_data{$pool_id}{'write'} = $write;
          }
        } ## end if ( defined $write &&...)
      } ## end if ( defined $pool_id ...)
    } ## end if ( $section eq "VOLUME")

    # Port
    ########################

    if ( $section eq "PORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Director,Port,timestamp,date,read (KB/s),write (KB/s),data_rate (KB/s),io_rate,resp_t_r,resp_t_w
      # director-1-1-A,A0-FC00,1500454439,2017-07-19 10:53:59,1924975,161286,2086261,42482,1.74,2.33
      # director-1-1-A,A1-FC00,1500454439,2017-07-19 10:53:59,2337230,177319,2514549,,,

      my ( $director, $port_id, $timestamp, $human_time, $read, $write, $data_rate, $io_rate, $resp_t_r, $resp_t_w ) = split( /,/, $line );

      if ( !defined($read)      || $read eq ''      || !isdigit($read) )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq ''     || !isdigit($write) )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq ''  || !isdigit($resp_t_r) )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq ''  || !isdigit($resp_t_w) )  { $resp_t_w  = 'U'; }

      print "001 PORT $director - $port_id - $human_time - $timestamp - $read - $write - $data_rate - $io_rate - $resp_t_r - $resp_t_w\n" if $DEBUG == 3;

      $rrd = "$wrkdir/$host/$section/$director-$port_id.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$read:$write:$data_rate:$io_rate:$resp_t_r:$resp_t_w);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "PORT")

    # Director (NODE)
    ########################

    if ( $section eq "NODE" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Director,timestamp,date,busy (%),io_rate_b,read_io_b,write_io_b,read_b (KB/s),write_b (KB/s),io_rate,read_io,write_io,read (KB/s),write (KB/s),data_rate_b (KB/s),data_rate (KB/s)
      # director-1-1-A,1500455944,2017-07-19 11:19:04,71,111744,87034,24710,3914919,681185,128449,108439,20009,4960416,557922,4596104,5518338

      my ( $director, $timestamp, $human_time, $busy, $io_rate_b, $read_io_b, $write_io_b, $read_b, $write_b, $io_rate, $read_io, $write_io, $read, $write, $data_rate_b, $data_rate, $resp_t_r, $resp_t_w ) = split( /,/, $line );

      if ( !defined($busy)        || $busy eq ''        || !isdigit($busy) )        { $busy        = 'U'; }
      if ( !defined($io_rate_b)   || $io_rate_b eq ''   || !isdigit($io_rate_b) )   { $io_rate_b   = 'U'; }
      if ( !defined($read_io_b)   || $read_io_b eq ''   || !isdigit($read_io_b) )   { $read_io_b   = 'U'; }
      if ( !defined($write_io_b)  || $write_io_b eq ''  || !isdigit($write_io_b) )  { $write_io_b  = 'U'; }
      if ( !defined($read_b)      || $read_b eq ''      || !isdigit($read_b) )      { $read_b      = 'U'; }
      if ( !defined($write_b)     || $write_b eq ''     || !isdigit($write_b) )     { $write_b     = 'U'; }
      if ( !defined($io_rate)     || $io_rate eq ''     || !isdigit($io_rate) )     { $io_rate     = 'U'; }
      if ( !defined($read_io)     || $read_io eq ''     || !isdigit($read_io) )     { $read_io     = 'U'; }
      if ( !defined($write_io)    || $write_io eq ''    || !isdigit($write_io) )    { $write_io    = 'U'; }
      if ( !defined($read)        || $read eq ''        || !isdigit($read) )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq ''       || !isdigit($write) )       { $write       = 'U'; }
      if ( !defined($data_rate_b) || $data_rate_b eq '' || !isdigit($data_rate_b) ) { $data_rate_b = 'U'; }
      if ( !defined($data_rate)   || $data_rate eq ''   || !isdigit($data_rate) )   { $data_rate   = 'U'; }
      if ( !defined($resp_t_r)    || $resp_t_r eq ''    || !isdigit($resp_t_r) )    { $resp_t_r    = 'U'; }
      if ( !defined($resp_t_w)    || $resp_t_w eq ''    || !isdigit($resp_t_w) )    { $resp_t_w    = 'U'; }

      print "001 DIRECTOR $director - $human_time - $timestamp - $busy - $io_rate_b - $read_io_b - $write_io_b - $read_b - $write_b - $io_rate - $read_io - $write_io - $read - $write - $data_rate_b - $data_rate - $resp_t_r - $resp_t_w\n" if $DEBUG == 3;

      $rrd = "$wrkdir/$host/$section/$director.rrd";

      if ( !defined $timestamp || $timestamp eq '' || !isdigit($timestamp) ) {
        main::error( "Time for update is not correct! \"$timestamp\" Skip this data line! \"$line\" " . __FILE__ . ":" . __LINE__ ) && next;
      }
      $time = $timestamp;

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$busy:$io_rate_b:$read_io_b:$write_io_b:$read_b:$write_b:$io_rate:$read_io:$write_io:$read:$write:$data_rate_b:$data_rate:$resp_t_r:$resp_t_w);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }

      #$rrd = "$wrkdir/$host/$type/array_total.rra";
      my $pool_id   = "array_total";
      my $pool_name = "array_total";

      # add values to pool
      if ( defined $pool_id && $pool_id ne '' ) {

        #print "$pool_id .. $volume_name\n";
        $pool_data{$pool_id}{'pool_name'} = $pool_name;

        if ( defined $resp_t_r && isdigit($resp_t_r) && defined $read_io && isdigit($read_io) ) {
          $resp_t_r = $resp_t_r * $read_io;
          if ( defined $pool_data{$pool_id}{'resp_t_r'} && isdigit( $pool_data{$pool_id}{'resp_t_r'} ) ) {
            $pool_data{$pool_id}{'resp_t_r'} = $pool_data{$pool_id}{'resp_t_r'} + $resp_t_r;
          }
          else {
            $pool_data{$pool_id}{'resp_t_r'} = $resp_t_r;
          }
        } ## end if ( defined $resp_t_r...)
        if ( defined $resp_t_w && isdigit($resp_t_w) && defined $write_io && isdigit($write_io) ) {
          $resp_t_w = $resp_t_w * $write_io;
          if ( defined $pool_data{$pool_id}{'resp_t_w'} && isdigit( $pool_data{$pool_id}{'resp_t_w'} ) ) {
            $pool_data{$pool_id}{'resp_t_w'} = $pool_data{$pool_id}{'resp_t_w'} + $resp_t_w;
          }
          else {
            $pool_data{$pool_id}{'resp_t_w'} = $resp_t_w;
          }
        } ## end if ( defined $resp_t_w...)
        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $pool_data{$pool_id}{'read_io'} && isdigit( $pool_data{$pool_id}{'read_io'} ) ) {
            $pool_data{$pool_id}{'read_io'} = $pool_data{$pool_id}{'read_io'} + $read_io;
          }
          else {
            $pool_data{$pool_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)
        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $pool_data{$pool_id}{'write_io'} && isdigit( $pool_data{$pool_id}{'write_io'} ) ) {
            $pool_data{$pool_id}{'write_io'} = $pool_data{$pool_id}{'write_io'} + $write_io;
          }
          else {
            $pool_data{$pool_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)
      } ## end if ( defined $pool_id ...)
    } ## end if ( $section eq "NODE")

  }    # end of foreach

  $type = "POOL";
  data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%pool_data );

  return ( "0", "$time_last_ok" );
} ## end sub load_data_vplex_perf

sub data_insert {
  my $host     = shift;
  my $step     = shift;
  my $wrkdir   = shift;
  my $DEBUG    = shift;
  my $no_time  = shift;
  my $act_time = shift;
  my $st_type  = shift;
  my $time     = shift;
  my $type     = shift;
  my $hash     = shift;
  my %data     = %$hash;

  foreach my $id ( keys(%data) ) {
    my $io_rate   = 'U';
    my $read_io   = 'U';
    my $write_io  = 'U';
    my $data_rate = 'U';
    my $read      = 'U';
    my $write     = 'U';
    my $cap       = 'U';
    my $cap_free  = 'U';
    my $cap_used  = 'U';
    my $pool_name = 'U';
    my $resp_t    = 'U';
    my $resp_t_r  = 'U';
    my $resp_t_w  = 'U';

    if ( defined $data{$id}{'io_rate'} && isdigit( $data{$id}{'io_rate'} ) ) {
      $io_rate = sprintf( '%.0f', $data{$id}{'io_rate'} );
    }
    if ( defined $data{$id}{'read_io'} && isdigit( $data{$id}{'read_io'} ) ) {
      $read_io = sprintf( '%.0f', $data{$id}{'read_io'} );
    }
    if ( defined $data{$id}{'write_io'} && isdigit( $data{$id}{'write_io'} ) ) {
      $write_io = sprintf( '%.0f', $data{$id}{'write_io'} );
    }
    if ( defined $data{$id}{'data_rate'} && isdigit( $data{$id}{'data_rate'} ) ) {
      $data_rate = sprintf( '%.0f', $data{$id}{'data_rate'} );
    }
    if ( defined $data{$id}{'read'} && isdigit( $data{$id}{'read'} ) ) {
      $read = sprintf( '%.0f', $data{$id}{'read'} );
    }
    if ( defined $data{$id}{'write'} && isdigit( $data{$id}{'write'} ) ) {
      $write = sprintf( '%.0f', $data{$id}{'write'} );
    }
    if ( defined $data{$id}{'cap'} && isdigit( $data{$id}{'cap'} ) ) {
      $cap = sprintf( '%.0f', $data{$id}{'cap'} );
    }
    if ( defined $data{$id}{'cap_free'} && isdigit( $data{$id}{'cap_free'} ) ) {
      $cap_free = sprintf( '%.0f', $data{$id}{'cap_free'} );
    }
    if ( defined $data{$id}{'cap_used'} && isdigit( $data{$id}{'cap_used'} ) ) {
      $cap_used = sprintf( '%.0f', $data{$id}{'cap_used'} );
    }
    if ( $st_type eq "VNX" && $type eq "POOL" ) {
      if ( defined $data{$id}{'resp_t'} && isdigit( $data{$id}{'resp_t'} ) && defined $data{$id}{'io_rate_gauge'} && isdigit( $data{$id}{'io_rate_gauge'} ) && $data{$id}{'io_rate_gauge'} != 0 ) {
        $resp_t = $data{$id}{'resp_t'} / $data{$id}{'io_rate_gauge'};
        $resp_t = sprintf( '%.0f', $resp_t );
      }
    } ## end if ( $st_type eq "VNX"...)
    else {
      if ( defined $data{$id}{'resp_t'} && isdigit( $data{$id}{'resp_t'} ) && defined $data{$id}{'io_rate'} && isdigit( $data{$id}{'io_rate'} ) && $data{$id}{'io_rate'} != 0 ) {
        $resp_t = $data{$id}{'resp_t'} / $data{$id}{'io_rate'};
        $resp_t = sprintf( '%.0f', $resp_t );
      }
    } ## end else [ if ( $st_type eq "VNX"...)]
    if ( defined $data{$id}{'resp_t_r'} && isdigit( $data{$id}{'resp_t_r'} ) && defined $data{$id}{'read_io'} && isdigit( $data{$id}{'read_io'} ) && $data{$id}{'read_io'} != 0 ) {
      $resp_t_r = $data{$id}{'resp_t_r'} / $data{$id}{'read_io'};
      $resp_t_r = sprintf( '%.0f', $resp_t_r );
    }
    if ( defined $data{$id}{'resp_t_w'} && isdigit( $data{$id}{'resp_t_w'} ) && defined $data{$id}{'write_io'} && isdigit( $data{$id}{'write_io'} ) && $data{$id}{'write_io'} != 0 ) {
      $resp_t_w = $data{$id}{'resp_t_w'} / $data{$id}{'write_io'};
      $resp_t_w = sprintf( '%.0f', $resp_t_w );
    }

    my $rrd = "$wrkdir/$host/$type/$id\.rrd";

    #print "$rrd : $id:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$cap:$cap_free:$cap_used\n";

    #create rrd db if necessary
    $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
    if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
      return 2;        # leave uit if any error during RRDTool file creation to keep data
    }

    my $ltime    = 0;
    my $last_rec = $ltime;
    if ( -f $rrd ) {
      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;
    } ## end if ( -f $rrd )

    # Update only latest data
    if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
      if ( $type eq "POOL" && $st_type eq "VNX" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$cap:$resp_t);
        my $answer = RRDp::read;
      }
      if ( $type eq "POOL" && $st_type eq "UNITY" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$cap:$resp_t);
        my $answer = RRDp::read;
      }
      if ( $type eq "NAS" && $st_type eq "UNITY" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
      }
      if ( $type eq "NASPOOL" && $st_type eq "UNITY" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
      }
      if ( $type eq "POOL" && $st_type eq "VNXE" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$cap:$resp_t);
        my $answer = RRDp::read;
      }
      if ( $type eq "NAS" && $st_type eq "VNXE" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
      }
      if ( $type eq "NASPOOL" && $st_type eq "VNXE" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
      }
      if ( $type eq "NAS" && $st_type eq "NETAPP" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate:$cap:$cap_used:U);
        my $answer = RRDp::read;
      }
      if ( $type eq "POOL" && $st_type eq "VPLEX" ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$data_rate:$read:$write:$read_io:$write_io:$resp_t_r:$resp_t_w);
        my $answer = RRDp::read;
      }
    } ## end if ( isdigit($time) &&...)
  } ## end foreach my $id ( keys(%data...))

  return (0);
} ## end sub data_insert

##################################################
#  EMC UNITY
##################################################

sub load_data_unity_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  my $in_exist    = 0;
  my $perf_string = "";
  my @files_unsorted;
  my @files;
  my $time_last_ok = "";

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # conf files
  #
  $perf_string = "unityconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_unity_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # perf files
  #
  $perf_string = "unityperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    #print "$in_file\n";

    my ( $ret, $time_last_ok ) = load_data_unity_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_unity_all

sub load_data_unity_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  my $pool_file   = "$wrkdir/$host/pool.cfg";
  my $config_file = "$wrkdir/$host/config.html";
  my $fs_file     = "$wrkdir/$host/FS/fs.cfg";
  my $nas_file    = "$wrkdir/$host/NAS/nas.cfg";

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( POOL, "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp: $! " . __FILE__ . ":" . __LINE__ )   && return 0;
  open( FS,   "> $fs_file-tmp" )     || main::error( "Can't open $fs_file-tmp: $! " . __FILE__ . ":" . __LINE__ )     && return 0;
  open( NAS,  "> $nas_file-tmp" )    || main::error( "Can't open $nas_file-tmp: $! " . __FILE__ . ":" . __LINE__ )    && return 0;

  # print *conf* file to config.html
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
  <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my $section = "";

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    # print to config.html
    print FCFG "$line\n";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                { next; }
    if ( $line =~ m/^Start:/ )                { next; }
    if ( $line =~ m/^Name,lun_id,SP_owner,/ ) { next; }
    if ( $line =~ m/^Pool name,Pool ID,/ )    { next; }
    if ( $line =~ m/^Host name,Host ID,/ )    { next; }
    if ( $line =~ m/^Name,fs_id,/ )           { next; }
    if ( $line =~ m/^Name,nas_id,/ )          { next; }

    if ( $line =~ m/^$/ )                        { $section = "NA";     next; }
    if ( $line eq "Volume Level Configuration" ) { $section = "VOLUME"; next; }
    if ( $line eq "Pool Level Configuration" )   { $section = "POOL";   next; }
    if ( $line eq "Port Level Configuration" )   { $section = "PORT";   next; }
    if ( $line eq "Disk Level Configuration" )   { $section = "DISK";   next; }
    if ( $line eq "SP Level Configuration" )     { $section = "SP";     next; }
    if ( $line eq "Host Level Configuration" )   { $section = "HOST";   next; }
    if ( $line eq "FS Level Configuration" )     { $section = "FS";     next; }
    if ( $line eq "NAS Level Configuration" )    { $section = "NAS";    next; }

    if ( $section eq "POOL" ) {
      my ( $pool_name, $pool_id, undef ) = split( ",", $line );

      # print pool to pool.cfg
      print POOL "$pool_id:$pool_name\n";
    } ## end if ( $section eq "POOL")
    if ( $section eq "FS" ) {
      my ( $fs_name, $fs_id, undef ) = split( ",", $line );

      # print FS to fs.cfg
      print FS "$fs_id:$fs_name\n";
    } ## end if ( $section eq "FS" )
    if ( $section eq "NAS" ) {
      my ( $nas_name, $nas_id, undef ) = split( ",", $line );

      # print NAS to nas.cfg
      print NAS "$nas_id:$nas_name\n";
    } ## end if ( $section eq "NAS")

  } ## end foreach my $line (@lines)

  print FCFG "</pre></body></html>\n";
  close(FCFG);
  close(POOL);
  close(FS);
  close(NAS);

  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }
  if ( -f "$pool_file-tmp" ) {
    rename( "$pool_file-tmp", "$pool_file" );        # rename it anyway even if some problem
  }
  if ( -f "$fs_file-tmp" ) {
    rename( "$fs_file-tmp", "$fs_file" );            # rename it anyway even if some problem
  }
  if ( -f "$nas_file-tmp" ) {
    rename( "$nas_file-tmp", "$nas_file" );          # rename it anyway even if some problem
  }

  return 0;
} ## end sub load_data_unity_conf

sub load_data_unity_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);
  my $pool_file  = "$wrkdir/$host/pool.cfg";
  my $time       = 3600;

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  open( CFG, "< $pool_file" ) || main::error( "Can't open $pool_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @pool_cfg = <CFG>;
  close(CFG);

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  my $section    = "";
  my $rrd        = "";
  my $type       = "NA";
  my $pool_index = 0;
  my %pool_data;
  my $nas_index = 0;
  my %nas_data;
  my $naspool_index = 0;
  my %naspool_data;
  my $time_last_ok = "";

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                                { next; }
    if ( $line =~ m/^,,,,/ )                                  { next; }
    if ( $line =~ m/^Name,lun_id,SP_owner,Storage pool ID,/ ) { next; }
    if ( $line =~ m/^Pool name,Pool ID,/ )                    { next; }
    if ( $line =~ m/^Disk ID,Name,Slot,/ )                    { next; }
    if ( $line =~ m/^Name,fs_id,/ )                           { next; }
    if ( $line =~ m/^port_id,Read KB/ )                       { next; }
    if ( $line =~ m/^sp,summary CPU Util/ )                   { next; }

    #if ( $line =~ m/^RG_ID,type,Capacity/ )                   { next; }
    #if ( $line =~ m/^Host name,Volume IDs,Volume Names/ )     { next; }

    if ( $line =~ m/^$/ )                            { $section = "NA";        next; }
    if ( $line eq "Volume Level Statistics" )        { $section = "VOLUME";    next; }
    if ( $line eq "Pool Level Statistics" )          { $section = "POOL";      next; }
    if ( $line eq "Disk Level Statistics" )          { $section = "DISK";      next; }
    if ( $line eq "FS Level Statistics" )            { $section = "FS";        next; }
    if ( $line eq "FC Port Level Statistics" )       { $section = "FCPORT";    next; }
    if ( $line eq "ETHERNET Port Level Statistics" ) { $section = "ETHPORT";   next; }
    if ( $line eq "ISCSI Port Level Statistics" )    { $section = "ISCSIPORT"; next; }
    if ( $line eq "SP Level Statistics" )            { $section = "SP";        next; }

    #if ( $line eq "Raid-Group Level Statistics" ) { $section = "RG";     next; }
    #if ( $line eq "Host Level Statistics" )       { $section = "HOST";   next; }

    # set start time
    if ( $line =~ m/^Start:/ ) {
      $line =~ s/^Start:\s+//;
      $time = str2time($line);

      if ( !defined($time) || isdigit($time) == 0 ) {
        $section = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/^Start:/)

    $type = $section;

    # Volume
    ########################

    if ( $section eq "VOLUME" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Name,lun_id,SP_owner,Storage pool ID,Storage pool,Size (MB),Average Read KB,Average Write KB,Read KB/s,Written KB/s,Read Counts/s,Write Counts/s,Total Call Counts/s,Response Time uSecs,Average Queue Length Count,data_rate
      # BIG2,sv_76,SPB,pool_8,SAS2,4194304,2.072,8.334,0.386,1.666,0.374,0.4,0.774,2728,0,2053.139

      my ( $volume_name, $volume_id, $owner, $pool_id, $pool_name, $cap, $avg_read_kb, $avg_write_kb, $read_kb, $write_kb, $read_io, $write_io, $io_rate, $resp_t, $queue, $data_rate ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kb)   || $read_kb eq ''   || !isdigit($read_kb) )   { $read_kb   = 'U'; }
      if ( !defined($write_kb)  || $write_kb eq ''  || !isdigit($write_kb) )  { $write_kb  = 'U'; }
      if ( !defined($cap)       || $cap eq ''       || !isdigit($cap) )       { $cap       = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq ''    || !isdigit($resp_t) )    { $resp_t    = 'U'; }

      # Controller|SP (Storage Processor)
      my $controller = "";
      if ( defined $owner && $owner ne '' ) {
        $controller = $owner;
        $controller =~ s/^SP//;
      }

      if ( !$controller eq '' && $controller =~ m/^[a|A]$/ ) {
        $controller = 0;
      }
      if ( !$controller eq '' && $controller =~ m/^[b|B]$/ ) {
        $controller = 1;
      }
      if ( $controller eq "N/A" || $controller eq '' ) {
        $controller = 'U';
      }
      else {
        if ( main::isdigit($controller) == 0 ) {
          main::error( "$host:$type - controler (Storage Processor) info is not found : $controller , reported only once  : $line" . __FILE__ . ":" . __LINE__ );
          $controller = 'U';
        }
      } ## end else [ if ( $controller eq "N/A"...)]

      #print "001 VOL $volume_name - $volume_id - $owner - $raid_group_id - $pool_name - $capacity_mb - $capacity_block - $read_io - $write_io - $read_bl - $write_bl - $cap - $read_kB - $write_kB - $io_rate - $data_rate\n" if $DEBUG == 3;

      $rrd = "$wrkdir/$host/$section/$volume_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "005: $rrd : $volume_id: $ltime \n";
      #print "006: $rrd : $volume_id: $ltime : $time \n";

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$resp_t:$read_kb:$write_kb:$data_rate:$cap:$controller);
        my $answer = RRDp::read;
      }

      # add values to pool
      if ( defined $pool_id && $pool_id ne '' && defined $pool_name && $pool_name ne '' ) {
        $pool_index++;

        #print "$pool_id .. $volume_name\n";
        $pool_data{$pool_id}{'pool_name'} = $pool_name;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $pool_data{$pool_id}{'io_rate'} && isdigit( $pool_data{$pool_id}{'io_rate'} ) ) {
            $pool_data{$pool_id}{'io_rate'} = $pool_data{$pool_id}{'io_rate'} + $io_rate;
          }
          else {
            $pool_data{$pool_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)

        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $pool_data{$pool_id}{'read_io'} && isdigit( $pool_data{$pool_id}{'read_io'} ) ) {
            $pool_data{$pool_id}{'read_io'} = $pool_data{$pool_id}{'read_io'} + $read_io;
          }
          else {
            $pool_data{$pool_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)

        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $pool_data{$pool_id}{'write_io'} && isdigit( $pool_data{$pool_id}{'write_io'} ) ) {
            $pool_data{$pool_id}{'write_io'} = $pool_data{$pool_id}{'write_io'} + $write_io;
          }
          else {
            $pool_data{$pool_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)

        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $pool_data{$pool_id}{'data_rate'} && isdigit( $pool_data{$pool_id}{'data_rate'} ) ) {
            $pool_data{$pool_id}{'data_rate'} = $pool_data{$pool_id}{'data_rate'} + $data_rate;
          }
          else {
            $pool_data{$pool_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)

        if ( defined $read_kb && isdigit($read_kb) ) {
          if ( defined $pool_data{$pool_id}{'read'} && isdigit( $pool_data{$pool_id}{'read'} ) ) {
            $pool_data{$pool_id}{'read'} = $pool_data{$pool_id}{'read'} + $read_kb;
          }
          else {
            $pool_data{$pool_id}{'read'} = $read_kb;
          }
        } ## end if ( defined $read_kb ...)

        if ( defined $write_kb && isdigit($write_kb) ) {
          if ( defined $pool_data{$pool_id}{'write'} && isdigit( $pool_data{$pool_id}{'write'} ) ) {
            $pool_data{$pool_id}{'write'} = $pool_data{$pool_id}{'write'} + $write_kb;
          }
          else {
            $pool_data{$pool_id}{'write'} = $write_kb;
          }
        } ## end if ( defined $write_kb...)

        if ( defined $cap && isdigit($cap) ) {
          if ( defined $pool_data{$pool_id}{'cap'} && isdigit( $pool_data{$pool_id}{'cap'} ) ) {
            $pool_data{$pool_id}{'cap'} = $pool_data{$pool_id}{'cap'} + $cap;
          }
          else {
            $pool_data{$pool_id}{'cap'} = $cap;
          }
        } ## end if ( defined $cap && isdigit...)
        if ( defined $resp_t && isdigit($resp_t) && defined $io_rate && isdigit($io_rate) ) {
          $resp_t = $resp_t * $io_rate;
          if ( defined $pool_data{$pool_id}{'resp_t'} && isdigit( $pool_data{$pool_id}{'resp_t'} ) ) {
            $pool_data{$pool_id}{'resp_t'} = $pool_data{$pool_id}{'resp_t'} + $resp_t;
          }
          else {
            $pool_data{$pool_id}{'resp_t'} = $resp_t;
          }
        } ## end if ( defined $resp_t &&...)

        #print "$time:$volume_name:$pool_name:$pool_id:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap\n";
      } ## end if ( defined $pool_id ...)
    } ## end if ( $section eq "VOLUME")

    # POOL capacity
    ########################

    if ( $section eq "POOL" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Pool name,Pool ID,capacity (GB),capacity_free (GB),capacity_used (GB),Volume IDs,Volume Names
      # SAS2,pool_8,3076,38.75,3037.25,sv_81 sv_82 sv_76,BIG6 xorux=====space=====test BIG2
      my ( $pool_name, $pool_id, $cap, $cap_free, $cap_used, undef, undef ) = split( /,/, $line );

      if ( !defined($cap)      || $cap eq ''      || !isdigit($cap) )      { $cap      = 'U'; }
      if ( !defined($cap_free) || $cap_free eq '' || !isdigit($cap_free) ) { $cap_free = 'U'; }
      if ( !defined($cap_used) || $cap_used eq '' || !isdigit($cap_used) ) { $cap_used = 'U'; }

      print "001 POOL cap $pool_name - $pool_id - $cap - $cap_free - $cap_used\n" if $DEBUG == 3;

      $type = "POOLcap";
      $rrd  = "$wrkdir/$host/POOL/$pool_id-cap.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "POOL")

    # Disk
    ########################

    if ( $section eq "DISK" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Disk ID,Name,Slot,Pool ID,Pool,User capacity,Read KB/s,Written KB/s,Reads IO/s,Writes IO/s,Total Calls IO/s,Average Response Time us,Average Service Time us,Avg Queue Length IO,data_rate
      # dpe_disk_6,DPE Disk 6,6,Unconfigured,Unconfigured,375568.9375,2046,6.136,2,0.273,2.273,333330,260898,0.568,2052.136

      my ( $disk_id, undef, undef, undef, undef, undef, $read_kB, $write_kB, $read_io, $write_io, $io_rate, $resp_t, undef, undef, $data_rate ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq ''    || !isdigit($resp_t) )    { $resp_t    = 'U'; }

      print "001 DISK $disk_id - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate - $resp_t" if $DEBUG == 3;

      $type = "DRIVE";
      $rrd  = "$wrkdir/$host/$type/$disk_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$resp_t);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "DISK")

    # FS
    ########################

    if ( $section eq "FS" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Name,fs_id,res_id,Server,SP_owner,Storage pool ID,Storage pool,Protocol,Size (MB),Size used (MB),Size free (MB),Avg Read Size KB,Avg Write Size KB,Read KB/s,Write KB/s,Read Requests/s,Write Requests/s,data_rate,io_rate
      # share-small,fs_4,res_4,nas_3,spb,pool_7,SAS1,nfs,10240,7852.8671875,2387.1328125,5.778,49.05,1.213,50228,0.210,1024,50229.213,1024.21

      my ( $name, $fs_id, undef, $nas_id, undef, $naspool_id, undef, undef, $cap, $cap_used, $cap_free, undef, undef, $read_kB, $write_kB, $read_io, $write_io, $data_rate, $io_rate ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($cap)       || $cap eq ''       || !isdigit($cap) )       { $cap       = 'U'; }
      if ( !defined($cap_free)  || $cap_free eq ''  || !isdigit($cap_free) )  { $cap_free  = 'U'; }
      if ( !defined($cap_used)  || $cap_used eq ''  || !isdigit($cap_used) )  { $cap_used  = 'U'; }

      print "001 FS $fs_id - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate" if $DEBUG == 3;

      $type = "FS";
      $rrd  = "$wrkdir/$host/$type/$fs_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
      }

      # add values to nas
      if ( defined $nas_id && $nas_id ne '' ) {
        $nas_index++;

        $nas_data{$nas_id}{'nas_name'} = $nas_id;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $nas_data{$nas_id}{'io_rate'} && isdigit( $nas_data{$nas_id}{'io_rate'} ) ) {
            $nas_data{$nas_id}{'io_rate'} = $nas_data{$nas_id}{'io_rate'} + $io_rate;
          }
          else {
            $nas_data{$nas_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)

        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $nas_data{$nas_id}{'read_io'} && isdigit( $nas_data{$nas_id}{'read_io'} ) ) {
            $nas_data{$nas_id}{'read_io'} = $nas_data{$nas_id}{'read_io'} + $read_io;
          }
          else {
            $nas_data{$nas_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)

        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $nas_data{$nas_id}{'write_io'} && isdigit( $nas_data{$nas_id}{'write_io'} ) ) {
            $nas_data{$nas_id}{'write_io'} = $nas_data{$nas_id}{'write_io'} + $write_io;
          }
          else {
            $nas_data{$nas_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)

        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $nas_data{$nas_id}{'data_rate'} && isdigit( $nas_data{$nas_id}{'data_rate'} ) ) {
            $nas_data{$nas_id}{'data_rate'} = $nas_data{$nas_id}{'data_rate'} + $data_rate;
          }
          else {
            $nas_data{$nas_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)

        if ( defined $read_kB && isdigit($read_kB) ) {
          if ( defined $nas_data{$nas_id}{'read'} && isdigit( $nas_data{$nas_id}{'read'} ) ) {
            $nas_data{$nas_id}{'read'} = $nas_data{$nas_id}{'read'} + $read_kB;
          }
          else {
            $nas_data{$nas_id}{'read'} = $read_kB;
          }
        } ## end if ( defined $read_kB ...)

        if ( defined $write_kB && isdigit($write_kB) ) {
          if ( defined $nas_data{$nas_id}{'write'} && isdigit( $nas_data{$nas_id}{'write'} ) ) {
            $nas_data{$nas_id}{'write'} = $nas_data{$nas_id}{'write'} + $write_kB;
          }
          else {
            $nas_data{$nas_id}{'write'} = $write_kB;
          }
        } ## end if ( defined $write_kB...)

        if ( defined $cap && isdigit($cap) ) {
          if ( defined $nas_data{$nas_id}{'cap'} && isdigit( $nas_data{$nas_id}{'cap'} ) ) {
            $nas_data{$nas_id}{'cap'} = $nas_data{$nas_id}{'cap'} + $cap;
          }
          else {
            $nas_data{$nas_id}{'cap'} = $cap;
          }
        } ## end if ( defined $cap && isdigit...)
        if ( defined $cap_free && isdigit($cap_free) ) {
          if ( defined $nas_data{$nas_id}{'cap_free'} && isdigit( $nas_data{$nas_id}{'cap_free'} ) ) {
            $nas_data{$nas_id}{'cap_free'} = $nas_data{$nas_id}{'cap_free'} + $cap_free;
          }
          else {
            $nas_data{$nas_id}{'cap_free'} = $cap_free;
          }
        } ## end if ( defined $cap_free...)
        if ( defined $cap_used && isdigit($cap_used) ) {
          if ( defined $nas_data{$nas_id}{'cap_used'} && isdigit( $nas_data{$nas_id}{'cap_used'} ) ) {
            $nas_data{$nas_id}{'cap_used'} = $nas_data{$nas_id}{'cap_used'} + $cap_used;
          }
          else {
            $nas_data{$nas_id}{'cap_used'} = $cap_used;
          }
        } ## end if ( defined $cap_used...)

        #print "$time:$name:$nas_id:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap:$cap_free:$cap_used\n";
      } ## end if ( defined $nas_id &&...)

      # add values to naspool
      if ( defined $naspool_id && $naspool_id ne '' ) {
        $naspool_index++;

        $naspool_data{$naspool_id}{'naspool_name'} = $naspool_id;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $naspool_data{$naspool_id}{'io_rate'} && isdigit( $naspool_data{$naspool_id}{'io_rate'} ) ) {
            $naspool_data{$naspool_id}{'io_rate'} = $naspool_data{$naspool_id}{'io_rate'} + $io_rate;
          }
          else {
            $naspool_data{$naspool_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)

        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $naspool_data{$naspool_id}{'read_io'} && isdigit( $naspool_data{$naspool_id}{'read_io'} ) ) {
            $naspool_data{$naspool_id}{'read_io'} = $naspool_data{$naspool_id}{'read_io'} + $read_io;
          }
          else {
            $naspool_data{$naspool_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)

        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $naspool_data{$naspool_id}{'write_io'} && isdigit( $naspool_data{$naspool_id}{'write_io'} ) ) {
            $naspool_data{$naspool_id}{'write_io'} = $naspool_data{$naspool_id}{'write_io'} + $write_io;
          }
          else {
            $naspool_data{$naspool_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)

        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $naspool_data{$naspool_id}{'data_rate'} && isdigit( $naspool_data{$naspool_id}{'data_rate'} ) ) {
            $naspool_data{$naspool_id}{'data_rate'} = $naspool_data{$naspool_id}{'data_rate'} + $data_rate;
          }
          else {
            $naspool_data{$naspool_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)

        if ( defined $read_kB && isdigit($read_kB) ) {
          if ( defined $naspool_data{$naspool_id}{'read'} && isdigit( $naspool_data{$naspool_id}{'read'} ) ) {
            $naspool_data{$naspool_id}{'read'} = $naspool_data{$naspool_id}{'read'} + $read_kB;
          }
          else {
            $naspool_data{$naspool_id}{'read'} = $read_kB;
          }
        } ## end if ( defined $read_kB ...)

        if ( defined $write_kB && isdigit($write_kB) ) {
          if ( defined $naspool_data{$naspool_id}{'write'} && isdigit( $naspool_data{$naspool_id}{'write'} ) ) {
            $naspool_data{$naspool_id}{'write'} = $naspool_data{$naspool_id}{'write'} + $write_kB;
          }
          else {
            $naspool_data{$naspool_id}{'write'} = $write_kB;
          }
        } ## end if ( defined $write_kB...)

        if ( defined $cap && isdigit($cap) ) {
          if ( defined $naspool_data{$naspool_id}{'cap'} && isdigit( $naspool_data{$naspool_id}{'cap'} ) ) {
            $naspool_data{$naspool_id}{'cap'} = $naspool_data{$naspool_id}{'cap'} + $cap;
          }
          else {
            $naspool_data{$naspool_id}{'cap'} = $cap;
          }
        } ## end if ( defined $cap && isdigit...)
        if ( defined $cap_free && isdigit($cap_free) ) {
          if ( defined $naspool_data{$naspool_id}{'cap_free'} && isdigit( $naspool_data{$naspool_id}{'cap_free'} ) ) {
            $naspool_data{$naspool_id}{'cap_free'} = $naspool_data{$naspool_id}{'cap_free'} + $cap_free;
          }
          else {
            $naspool_data{$naspool_id}{'cap_free'} = $cap_free;
          }
        } ## end if ( defined $cap_free...)
        if ( defined $cap_used && isdigit($cap_used) ) {
          if ( defined $naspool_data{$naspool_id}{'cap_used'} && isdigit( $naspool_data{$naspool_id}{'cap_used'} ) ) {
            $naspool_data{$naspool_id}{'cap_used'} = $naspool_data{$naspool_id}{'cap_used'} + $cap_used;
          }
          else {
            $naspool_data{$naspool_id}{'cap_used'} = $cap_used;
          }
        } ## end if ( defined $cap_used...)

        #print "$time:$name:$nas_id:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap:$cap_free:$cap_used\n";
      } ## end if ( defined $naspool_id...)
    } ## end if ( $section eq "FS" )

    # FC Port
    ########################

    if ( $section eq "FCPORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      # port_id,Read KB/s,Write KB/s,Read Requests/s,Write Requests/s,data_rate,io_rate
      # spb_fc4,350.5,11.96,7.833,1.540,362.46,9.373

      my ( $port_name, $read_kB, $write_kB, $read_io, $write_io, $data_rate, $io_rate ) = split( /,/, $line );

      if ( $port_name !~ m/^sp._fc/ && $port_name !~ m/sp[a-b]_iom_[0-9]+_fc/ ) { next; }    # must be in this format

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }

      print "001 FC PORT $port_name - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate\n" if $DEBUG == 3;

      $type = "PORT";
      $rrd  = "$wrkdir/$host/$type/$port_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "FCPORT")

    # ISCSI Port
    ########################

    if ( $section eq "ISCSIPORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      # port_id,Read KB/s,Write KB/s,Read Requests/s,Write Requests/s,data_rate,io_rate
      # spb_fc4,350.5,11.96,7.833,1.540,362.46,9.373

      my ( $port_name, $read_kB, $write_kB, $read_io, $write_io, $data_rate, $io_rate ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }

      print "001 FC PORT $port_name - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate\n" if $DEBUG == 3;

      $type = "PORT";
      $rrd  = "$wrkdir/$host/$type/$port_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "ISCSIPORT")

    # ETHERNET Port
    ########################

    if ( $section eq "ETHPORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      # port_id,Read KB/s,Write KB/s,Read packets,Write packets,data_rate,pkts_rate
      # spb_eth3,0.465,0.009,3.923,0.083,0.474,4.006

      my ( $port_name, $read_kB, $write_kB, $read_pkts, $write_pkts, $data_rate, $pkts_rate ) = split( /,/, $line );

      if ( $port_name !~ m/^sp[a-b]_eth[0-9]+/ )    { next; }    # must be in this format
      if ( $port_name =~ m/^sp[a-b]_eth[0-9]+eth/ ) { next; }

      if ( !defined($read_pkts)  || $read_pkts eq ''  || !isdigit($read_pkts) )  { $read_pkts  = 'U'; }
      if ( !defined($write_pkts) || $write_pkts eq '' || !isdigit($write_pkts) ) { $write_pkts = 'U'; }
      if ( !defined($read_kB)    || $read_kB eq ''    || !isdigit($read_kB) )    { $read_kB    = 'U'; }
      if ( !defined($write_kB)   || $write_kB eq ''   || !isdigit($write_kB) )   { $write_kB   = 'U'; }
      if ( !defined($pkts_rate)  || $pkts_rate eq ''  || !isdigit($pkts_rate) )  { $pkts_rate  = 'U'; }
      if ( !defined($data_rate)  || $data_rate eq ''  || !isdigit($data_rate) )  { $data_rate  = 'U'; }

      print "001 FC PORT $port_name - $read_pkts - $write_pkts - $read_kB - $write_kB - $pkts_rate - $data_rate\n" if $DEBUG == 3;

      $type = "PORT";
      $rrd  = "$wrkdir/$host/$type/$port_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$pkts_rate:$read_pkts:$write_pkts:$data_rate:$read_kB:$write_kB);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "ETHPORT")

    # SP
    ########################

    if ( $section eq "SP" ) {
      print "000 $line\n" if $DEBUG == 3;

      # sp,summary CPU Util %,NFS Read KB/s,NFS Write KB/s,NFS Avg Read Size KB,NFS Avg Write Size KB,NFS Read Ops/s,NFS Write Ops/s,Total NFS Ops/s,CIFS Read KB/s,CIFS Write KB/s,CIFS Avg Read Size KB,CIFS Avg Write Size KB,CIFS Read Ops/s,CIFS Write Ops/s,CIFS Total Ops/s,Block Cache Dirty Data MB,Block Cache Read Hits Hits/s,Block Cache Write Hits Hits/s,Block Cache Read Misses Misses/s,Block Cache Write Misses Misses/s,Network In KB/s,Network Out KB/s
      # spb,8.169,0,0,0,0,0,0,0.083,0,0,0,0,0,0,0,45.00,3.097,113.1,0.227,4.277,0.481,0.009

      my ( $sp, $cpu, $nfs_read_kb, $nfs_write_kb, $nfs_avg_read_kb, $nfs_avg_write_kb, $nfs_read_ops, $nfs_write_ops, $nfs_total_ops, $cifs_read_kb, $cifs_write_kb, $cifs_avg_read_kb, $cifs_avg_write_kb, $cifs_read_ops, $cifs_write_ops, $cifs_total_ops, $cache_dirty_data_mb, $cache_read_hits, $cache_write_hits, $cache_read_misses, $cache_write_misses, $net_read_kb, $net_write_kb ) = split( /,/, $line );

      if ( !defined($cpu)                 || $cpu eq ''                 || !isdigit($cpu) )                 { $cpu                 = 'U'; }
      if ( !defined($nfs_read_kb)         || $nfs_read_kb eq ''         || !isdigit($nfs_read_kb) )         { $nfs_read_kb         = 'U'; }
      if ( !defined($nfs_write_kb)        || $nfs_write_kb eq ''        || !isdigit($nfs_write_kb) )        { $nfs_write_kb        = 'U'; }
      if ( !defined($nfs_avg_read_kb)     || $nfs_avg_read_kb eq ''     || !isdigit($nfs_avg_read_kb) )     { $nfs_avg_read_kb     = 'U'; }
      if ( !defined($nfs_avg_write_kb)    || $nfs_avg_write_kb eq ''    || !isdigit($nfs_avg_write_kb) )    { $nfs_avg_write_kb    = 'U'; }
      if ( !defined($nfs_read_ops)        || $nfs_read_ops eq ''        || !isdigit($nfs_read_ops) )        { $nfs_read_ops        = 'U'; }
      if ( !defined($nfs_write_ops)       || $nfs_write_ops eq ''       || !isdigit($nfs_write_ops) )       { $nfs_write_ops       = 'U'; }
      if ( !defined($nfs_total_ops)       || $nfs_total_ops eq ''       || !isdigit($nfs_total_ops) )       { $nfs_total_ops       = 'U'; }
      if ( !defined($cifs_read_kb)        || $cifs_read_kb eq ''        || !isdigit($cifs_read_kb) )        { $cifs_read_kb        = 'U'; }
      if ( !defined($cifs_write_kb)       || $cifs_write_kb eq ''       || !isdigit($cifs_write_kb) )       { $cifs_write_kb       = 'U'; }
      if ( !defined($cifs_avg_read_kb)    || $cifs_avg_read_kb eq ''    || !isdigit($cifs_avg_read_kb) )    { $cifs_avg_read_kb    = 'U'; }
      if ( !defined($cifs_avg_write_kb)   || $cifs_avg_write_kb eq ''   || !isdigit($cifs_avg_write_kb) )   { $cifs_avg_write_kb   = 'U'; }
      if ( !defined($cifs_read_ops)       || $cifs_read_ops eq ''       || !isdigit($cifs_read_ops) )       { $cifs_read_ops       = 'U'; }
      if ( !defined($cifs_write_ops)      || $cifs_write_ops eq ''      || !isdigit($cifs_write_ops) )      { $cifs_write_ops      = 'U'; }
      if ( !defined($cifs_total_ops)      || $cifs_total_ops eq ''      || !isdigit($cifs_total_ops) )      { $cifs_total_ops      = 'U'; }
      if ( !defined($cache_dirty_data_mb) || $cache_dirty_data_mb eq '' || !isdigit($cache_dirty_data_mb) ) { $cache_dirty_data_mb = 'U'; }
      if ( !defined($cache_read_hits)     || $cache_read_hits eq ''     || !isdigit($cache_read_hits) )     { $cache_read_hits     = 'U'; }
      if ( !defined($cache_write_hits)    || $cache_write_hits eq ''    || !isdigit($cache_write_hits) )    { $cache_write_hits    = 'U'; }
      if ( !defined($cache_read_misses)   || $cache_read_misses eq ''   || !isdigit($cache_read_misses) )   { $cache_read_misses   = 'U'; }
      if ( !defined($cache_write_misses)  || $cache_write_misses eq ''  || !isdigit($cache_write_misses) )  { $cache_write_misses  = 'U'; }
      if ( !defined($net_read_kb)         || $net_read_kb eq ''         || !isdigit($net_read_kb) )         { $net_read_kb         = 'U'; }
      if ( !defined($net_write_kb)        || $net_write_kb eq ''        || !isdigit($net_write_kb) )        { $net_write_kb        = 'U'; }

      print "001 SP $sp : $cpu,$nfs_read_kb,$nfs_write_kb,$nfs_avg_read_kb,$nfs_avg_write_kb,$nfs_read_ops,$nfs_write_ops,$nfs_total_ops,$cifs_read_kb,$cifs_write_kb,$cifs_avg_read_kb,$cifs_avg_write_kb,$cifs_read_ops,$cifs_write_ops,$cifs_total_ops,$cache_dirty_data_mb,$cache_read_hits,$cache_write_hits,$cache_read_misses,$cache_write_misses,$net_read_kb,$net_write_kb\n" if $DEBUG == 3;

      $type = "NODE";
      $rrd  = "$wrkdir/$host/$type/$sp.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cpu:$nfs_read_kb:$nfs_write_kb:$nfs_avg_read_kb:$nfs_avg_write_kb:$nfs_read_ops:$nfs_write_ops:$nfs_total_ops:$cifs_read_kb:$cifs_write_kb:$cifs_avg_read_kb:$cifs_avg_write_kb:$cifs_read_ops:$cifs_write_ops:$cifs_total_ops:$cache_dirty_data_mb:$cache_read_hits:$cache_write_hits:$cache_read_misses:$cache_write_misses:$net_read_kb:$net_write_kb);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "SP" )

  }    # end of foreach

  # pool summary
  if ( isdigit($pool_index) && $pool_index > 0 ) {
    $type = "POOL";
    data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%pool_data );
  }

  # NAS summary
  if ( isdigit($nas_index) && $nas_index > 0 ) {
    $type = "NAS";
    data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%nas_data );
  }

  # NASPOOL summary
  if ( isdigit($naspool_index) && $naspool_index > 0 ) {
    $type = "NASPOOL";
    data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%naspool_data );
  }

  return ( "0", "$time_last_ok" );
} ## end sub load_data_unity_perf

##################################################
#  EMC VNXe
##################################################

sub load_data_vnxe_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  my $in_exist    = 0;
  my $perf_string = "";
  my @files_unsorted;
  my @files;
  my $time_last_ok = "";

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # conf files
  #
  $perf_string = "vnxeconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_vnxe_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # perf files
  #
  $perf_string = "vnxeperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    #print "$in_file\n";

    my ( $ret, $time_last_ok ) = load_data_vnxe_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_vnxe_all

sub load_data_vnxe_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  my $pool_file   = "$wrkdir/$host/pool.cfg";
  my $config_file = "$wrkdir/$host/config.html";
  my $fs_file     = "$wrkdir/$host/FS/fs.cfg";
  my $nas_file    = "$wrkdir/$host/NAS/nas.cfg";

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( POOL, "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp: $! " . __FILE__ . ":" . __LINE__ )   && return 0;
  open( FS,   "> $fs_file-tmp" )     || main::error( "Can't open $fs_file-tmp: $! " . __FILE__ . ":" . __LINE__ )     && return 0;
  open( NAS,  "> $nas_file-tmp" )    || main::error( "Can't open $nas_file-tmp: $! " . __FILE__ . ":" . __LINE__ )    && return 0;

  # print *conf* file to config.html
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
  <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my $section = "";

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    # print to config.html
    print FCFG "$line\n";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                { next; }
    if ( $line =~ m/^Start:/ )                { next; }
    if ( $line =~ m/^Name,lun_id,SP_owner,/ ) { next; }
    if ( $line =~ m/^Pool name,Pool ID,/ )    { next; }
    if ( $line =~ m/^Host name,Host ID,/ )    { next; }
    if ( $line =~ m/^Name,fs_id,/ )           { next; }
    if ( $line =~ m/^Name,nas_id,/ )          { next; }

    if ( $line =~ m/^$/ )                        { $section = "NA";     next; }
    if ( $line eq "Volume Level Configuration" ) { $section = "VOLUME"; next; }
    if ( $line eq "Pool Level Configuration" )   { $section = "POOL";   next; }
    if ( $line eq "Port Level Configuration" )   { $section = "PORT";   next; }
    if ( $line eq "Disk Level Configuration" )   { $section = "DISK";   next; }
    if ( $line eq "SP Level Configuration" )     { $section = "SP";     next; }
    if ( $line eq "Host Level Configuration" )   { $section = "HOST";   next; }
    if ( $line eq "FS Level Configuration" )     { $section = "FS";     next; }
    if ( $line eq "NAS Level Configuration" )    { $section = "NAS";    next; }

    if ( $section eq "POOL" ) {
      my ( $pool_name, $pool_id, undef ) = split( ",", $line );

      # print pool to pool.cfg
      print POOL "$pool_id:$pool_name\n";
    } ## end if ( $section eq "POOL")
    if ( $section eq "FS" ) {
      my ( $fs_name, $fs_id, undef ) = split( ",", $line );

      # print FS to fs.cfg
      print FS "$fs_id:$fs_name\n";
    } ## end if ( $section eq "FS" )
    if ( $section eq "NAS" ) {
      my ( $nas_name, $nas_id, undef ) = split( ",", $line );

      # print NAS to nas.cfg
      print NAS "$nas_id:$nas_name\n";
    } ## end if ( $section eq "NAS")

  } ## end foreach my $line (@lines)

  print FCFG "</pre></body></html>\n";
  close(FCFG);
  close(POOL);
  close(FS);
  close(NAS);

  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }
  if ( -f "$pool_file-tmp" ) {
    rename( "$pool_file-tmp", "$pool_file" );        # rename it anyway even if some problem
  }
  if ( -f "$fs_file-tmp" ) {
    rename( "$fs_file-tmp", "$fs_file" );            # rename it anyway even if some problem
  }
  if ( -f "$nas_file-tmp" ) {
    rename( "$nas_file-tmp", "$nas_file" );          # rename it anyway even if some problem
  }

  return 0;
} ## end sub load_data_vnxe_conf

sub load_data_vnxe_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $act_time_u = time();
  my $ltime      = localtime($act_time_u);
  my $pool_file  = "$wrkdir/$host/pool.cfg";
  my $time       = 3600;

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  open( CFG, "< $pool_file" ) || main::error( "Can't open $pool_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @pool_cfg = <CFG>;
  close(CFG);

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  my $section    = "";
  my $rrd        = "";
  my $type       = "NA";
  my $pool_index = 0;
  my %pool_data;
  my $nas_index = 0;
  my %nas_data;
  my $naspool_index = 0;
  my %naspool_data;
  my $time_last_ok = "";

  foreach my $line (@lines) {

    #print "$line";
    chomp($line);

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^------/ )                                { next; }
    if ( $line =~ m/^Name,lun_id,SP_owner,Storage pool ID,/ ) { next; }
    if ( $line =~ m/^Pool name,Pool ID,/ )                    { next; }
    if ( $line =~ m/^Disk ID,Name,Slot,/ )                    { next; }
    if ( $line =~ m/^Name,fs_id,/ )                           { next; }
    if ( $line =~ m/^port_id,Read KB/ )                       { next; }
    if ( $line =~ m/^sp,summary CPU Util/ )                   { next; }

    #if ( $line =~ m/^RG_ID,type,Capacity/ )                   { next; }
    #if ( $line =~ m/^Host name,Volume IDs,Volume Names/ )     { next; }

    if ( $line =~ m/^$/ )                            { $section = "NA";        next; }
    if ( $line eq "Volume Level Statistics" )        { $section = "VOLUME";    next; }
    if ( $line eq "Pool Level Statistics" )          { $section = "POOL";      next; }
    if ( $line eq "Disk Level Statistics" )          { $section = "DISK";      next; }
    if ( $line eq "FS Level Statistics" )            { $section = "FS";        next; }
    if ( $line eq "FC Port Level Statistics" )       { $section = "FCPORT";    next; }
    if ( $line eq "ETHERNET Port Level Statistics" ) { $section = "ETHPORT";   next; }
    if ( $line eq "ISCSI Port Level Statistics" )    { $section = "ISCSIPORT"; next; }
    if ( $line eq "SP Level Statistics" )            { $section = "SP";        next; }

    #if ( $line eq "Raid-Group Level Statistics" ) { $section = "RG";     next; }
    #if ( $line eq "Host Level Statistics" )       { $section = "HOST";   next; }

    # set start time
    if ( $line =~ m/^Start:/ ) {
      $line =~ s/^Start:\s+//;
      $time = str2time($line);

      if ( !defined($time) || isdigit($time) == 0 ) {
        $section = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/^Start:/)

    $type = $section;

    # Volume
    ########################

    if ( $section eq "VOLUME" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Name,lun_id,SP_owner,Storage pool ID,Storage pool,Size (MB),Average Read KB,Average Write KB,Read KB/s,Written KB/s,Read Counts/s,Write Counts/s,Total Call Counts/s,Response Time uSecs,Average Queue Length Count,data_rate
      # BIG2,sv_76,SPB,pool_8,SAS2,4194304,2.072,8.334,0.386,1.666,0.374,0.4,0.774,2728,0,2053.139

      my ( $volume_name, $volume_id, $owner, $pool_id, $pool_name, $cap, $avg_read_kb, $avg_write_kb, $read_kb, $write_kb, $read_io, $write_io, $io_rate, $resp_t, $queue, $data_rate ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kb)   || $read_kb eq ''   || !isdigit($read_kb) )   { $read_kb   = 'U'; }
      if ( !defined($write_kb)  || $write_kb eq ''  || !isdigit($write_kb) )  { $write_kb  = 'U'; }
      if ( !defined($cap)       || $cap eq ''       || !isdigit($cap) )       { $cap       = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq ''    || !isdigit($resp_t) )    { $resp_t    = 'U'; }

      # Controller|SP (Storage Processor)
      my $controller = "";
      if ( defined $owner && $owner ne '' ) {
        $controller = $owner;
        $controller =~ s/^SP//;
      }

      if ( !$controller eq '' && $controller =~ m/^[a|A]$/ ) {
        $controller = 0;
      }
      if ( !$controller eq '' && $controller =~ m/^[b|B]$/ ) {
        $controller = 1;
      }
      if ( $controller eq "N/A" || $controller eq '' ) {
        $controller = 'U';
      }
      else {
        if ( main::isdigit($controller) == 0 ) {
          main::error( "$host:$type - controler (Storage Processor) info is not found : $controller , reported only once  : $line" . __FILE__ . ":" . __LINE__ );
          $controller = 'U';
        }
      } ## end else [ if ( $controller eq "N/A"...)]

      #print "001 VOL $volume_name - $volume_id - $owner - $raid_group_id - $pool_name - $capacity_mb - $capacity_block - $read_io - $write_io - $read_bl - $write_bl - $cap - $read_kB - $write_kB - $io_rate - $data_rate\n" if $DEBUG == 3;

      $rrd = "$wrkdir/$host/$section/$volume_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "005: $rrd : $volume_id: $ltime \n";
      #print "006: $rrd : $volume_id: $ltime : $time \n";

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$resp_t:$read_kb:$write_kb:$data_rate:$cap:$controller);
        my $answer = RRDp::read;
      }

      # add values to pool
      if ( defined $pool_id && $pool_id ne '' && defined $pool_name && $pool_name ne '' ) {
        $pool_index++;

        #print "$pool_id .. $volume_name\n";
        $pool_data{$pool_id}{'pool_name'} = $pool_name;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $pool_data{$pool_id}{'io_rate'} && isdigit( $pool_data{$pool_id}{'io_rate'} ) ) {
            $pool_data{$pool_id}{'io_rate'} = $pool_data{$pool_id}{'io_rate'} + $io_rate;
          }
          else {
            $pool_data{$pool_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)

        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $pool_data{$pool_id}{'read_io'} && isdigit( $pool_data{$pool_id}{'read_io'} ) ) {
            $pool_data{$pool_id}{'read_io'} = $pool_data{$pool_id}{'read_io'} + $read_io;
          }
          else {
            $pool_data{$pool_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)

        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $pool_data{$pool_id}{'write_io'} && isdigit( $pool_data{$pool_id}{'write_io'} ) ) {
            $pool_data{$pool_id}{'write_io'} = $pool_data{$pool_id}{'write_io'} + $write_io;
          }
          else {
            $pool_data{$pool_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)

        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $pool_data{$pool_id}{'data_rate'} && isdigit( $pool_data{$pool_id}{'data_rate'} ) ) {
            $pool_data{$pool_id}{'data_rate'} = $pool_data{$pool_id}{'data_rate'} + $data_rate;
          }
          else {
            $pool_data{$pool_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)

        if ( defined $read_kb && isdigit($read_kb) ) {
          if ( defined $pool_data{$pool_id}{'read'} && isdigit( $pool_data{$pool_id}{'read'} ) ) {
            $pool_data{$pool_id}{'read'} = $pool_data{$pool_id}{'read'} + $read_kb;
          }
          else {
            $pool_data{$pool_id}{'read'} = $read_kb;
          }
        } ## end if ( defined $read_kb ...)

        if ( defined $write_kb && isdigit($write_kb) ) {
          if ( defined $pool_data{$pool_id}{'write'} && isdigit( $pool_data{$pool_id}{'write'} ) ) {
            $pool_data{$pool_id}{'write'} = $pool_data{$pool_id}{'write'} + $write_kb;
          }
          else {
            $pool_data{$pool_id}{'write'} = $write_kb;
          }
        } ## end if ( defined $write_kb...)

        if ( defined $cap && isdigit($cap) ) {
          if ( defined $pool_data{$pool_id}{'cap'} && isdigit( $pool_data{$pool_id}{'cap'} ) ) {
            $pool_data{$pool_id}{'cap'} = $pool_data{$pool_id}{'cap'} + $cap;
          }
          else {
            $pool_data{$pool_id}{'cap'} = $cap;
          }
        } ## end if ( defined $cap && isdigit...)
        if ( defined $resp_t && isdigit($resp_t) && defined $io_rate && isdigit($io_rate) ) {
          $resp_t = $resp_t * $io_rate;
          if ( defined $pool_data{$pool_id}{'resp_t'} && isdigit( $pool_data{$pool_id}{'resp_t'} ) ) {
            $pool_data{$pool_id}{'resp_t'} = $pool_data{$pool_id}{'resp_t'} + $resp_t;
          }
          else {
            $pool_data{$pool_id}{'resp_t'} = $resp_t;
          }
        } ## end if ( defined $resp_t &&...)

        #print "$time:$volume_name:$pool_name:$pool_id:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap\n";
      } ## end if ( defined $pool_id ...)
    } ## end if ( $section eq "VOLUME")

    # POOL capacity
    ########################

    if ( $section eq "POOL" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Pool name,Pool ID,capacity (GB),capacity_free (GB),capacity_used (GB),Volume IDs,Volume Names
      # SAS2,pool_8,3076,38.75,3037.25,sv_81 sv_82 sv_76,BIG6 xorux=====space=====test BIG2
      my ( $pool_name, $pool_id, $cap, $cap_free, $cap_used, undef, undef ) = split( /,/, $line );

      if ( !defined($cap)      || $cap eq ''      || !isdigit($cap) )      { $cap      = 'U'; }
      if ( !defined($cap_free) || $cap_free eq '' || !isdigit($cap_free) ) { $cap_free = 'U'; }
      if ( !defined($cap_used) || $cap_used eq '' || !isdigit($cap_used) ) { $cap_used = 'U'; }

      print "001 POOL cap $pool_name - $pool_id - $cap - $cap_free - $cap_used\n" if $DEBUG == 3;

      $type = "POOLcap";
      $rrd  = "$wrkdir/$host/POOL/$pool_id-cap.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "POOL")

    # Disk
    ########################

    if ( $section eq "DISK" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Disk ID,Name,Slot,Pool ID,Pool,User capacity,Read KB/s,Written KB/s,Reads IO/s,Writes IO/s,Total Calls IO/s,Average Response Time us,Average Service Time us,Avg Queue Length IO,data_rate
      # dpe_disk_6,DPE Disk 6,6,Unconfigured,Unconfigured,375568.9375,2046,6.136,2,0.273,2.273,333330,260898,0.568,2052.136

      my ( $disk_id, undef, undef, undef, undef, undef, $read_kB, $write_kB, $read_io, $write_io, $io_rate, $resp_t, undef, undef, $data_rate ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq ''    || !isdigit($resp_t) )    { $resp_t    = 'U'; }

      print "001 DISK $disk_id - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate - $resp_t" if $DEBUG == 3;

      $type = "DRIVE";
      $rrd  = "$wrkdir/$host/$type/$disk_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$resp_t);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "DISK")

    # FS
    ########################

    if ( $section eq "FS" ) {
      print "000 $line\n" if $DEBUG == 3;

      # Name,fs_id,res_id,Server,SP_owner,Storage pool ID,Storage pool,Protocol,Size (MB),Size used (MB),Size free (MB),Avg Read Size KB,Avg Write Size KB,Read KB/s,Write KB/s,Read Requests/s,Write Requests/s,data_rate,io_rate
      # share-small,fs_4,res_4,nas_3,spb,pool_7,SAS1,nfs,10240,7852.8671875,2387.1328125,5.778,49.05,1.213,50228,0.210,1024,50229.213,1024.21

      my ( $name, $fs_id, undef, $nas_id, undef, $naspool_id, undef, undef, $cap, $cap_used, $cap_free, undef, undef, $read_kB, $write_kB, $read_io, $write_io, $data_rate, $io_rate ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }
      if ( !defined($cap)       || $cap eq ''       || !isdigit($cap) )       { $cap       = 'U'; }
      if ( !defined($cap_free)  || $cap_free eq ''  || !isdigit($cap_free) )  { $cap_free  = 'U'; }
      if ( !defined($cap_used)  || $cap_used eq ''  || !isdigit($cap_used) )  { $cap_used  = 'U'; }

      print "001 FS $fs_id - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate" if $DEBUG == 3;

      $type = "FS";
      $rrd  = "$wrkdir/$host/$type/$fs_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap:$cap_free:$cap_used);
        my $answer = RRDp::read;
      }

      # add values to nas
      if ( defined $nas_id && $nas_id ne '' ) {
        $nas_index++;

        $nas_data{$nas_id}{'nas_name'} = $nas_id;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $nas_data{$nas_id}{'io_rate'} && isdigit( $nas_data{$nas_id}{'io_rate'} ) ) {
            $nas_data{$nas_id}{'io_rate'} = $nas_data{$nas_id}{'io_rate'} + $io_rate;
          }
          else {
            $nas_data{$nas_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)

        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $nas_data{$nas_id}{'read_io'} && isdigit( $nas_data{$nas_id}{'read_io'} ) ) {
            $nas_data{$nas_id}{'read_io'} = $nas_data{$nas_id}{'read_io'} + $read_io;
          }
          else {
            $nas_data{$nas_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)

        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $nas_data{$nas_id}{'write_io'} && isdigit( $nas_data{$nas_id}{'write_io'} ) ) {
            $nas_data{$nas_id}{'write_io'} = $nas_data{$nas_id}{'write_io'} + $write_io;
          }
          else {
            $nas_data{$nas_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)

        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $nas_data{$nas_id}{'data_rate'} && isdigit( $nas_data{$nas_id}{'data_rate'} ) ) {
            $nas_data{$nas_id}{'data_rate'} = $nas_data{$nas_id}{'data_rate'} + $data_rate;
          }
          else {
            $nas_data{$nas_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)

        if ( defined $read_kB && isdigit($read_kB) ) {
          if ( defined $nas_data{$nas_id}{'read'} && isdigit( $nas_data{$nas_id}{'read'} ) ) {
            $nas_data{$nas_id}{'read'} = $nas_data{$nas_id}{'read'} + $read_kB;
          }
          else {
            $nas_data{$nas_id}{'read'} = $read_kB;
          }
        } ## end if ( defined $read_kB ...)

        if ( defined $write_kB && isdigit($write_kB) ) {
          if ( defined $nas_data{$nas_id}{'write'} && isdigit( $nas_data{$nas_id}{'write'} ) ) {
            $nas_data{$nas_id}{'write'} = $nas_data{$nas_id}{'write'} + $write_kB;
          }
          else {
            $nas_data{$nas_id}{'write'} = $write_kB;
          }
        } ## end if ( defined $write_kB...)

        if ( defined $cap && isdigit($cap) ) {
          if ( defined $nas_data{$nas_id}{'cap'} && isdigit( $nas_data{$nas_id}{'cap'} ) ) {
            $nas_data{$nas_id}{'cap'} = $nas_data{$nas_id}{'cap'} + $cap;
          }
          else {
            $nas_data{$nas_id}{'cap'} = $cap;
          }
        } ## end if ( defined $cap && isdigit...)
        if ( defined $cap_free && isdigit($cap_free) ) {
          if ( defined $nas_data{$nas_id}{'cap_free'} && isdigit( $nas_data{$nas_id}{'cap_free'} ) ) {
            $nas_data{$nas_id}{'cap_free'} = $nas_data{$nas_id}{'cap_free'} + $cap_free;
          }
          else {
            $nas_data{$nas_id}{'cap_free'} = $cap_free;
          }
        } ## end if ( defined $cap_free...)
        if ( defined $cap_used && isdigit($cap_used) ) {
          if ( defined $nas_data{$nas_id}{'cap_used'} && isdigit( $nas_data{$nas_id}{'cap_used'} ) ) {
            $nas_data{$nas_id}{'cap_used'} = $nas_data{$nas_id}{'cap_used'} + $cap_used;
          }
          else {
            $nas_data{$nas_id}{'cap_used'} = $cap_used;
          }
        } ## end if ( defined $cap_used...)

        #print "$time:$name:$nas_id:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap:$cap_free:$cap_used\n";
      } ## end if ( defined $nas_id &&...)

      # add values to naspool
      if ( defined $naspool_id && $naspool_id ne '' ) {
        $naspool_index++;

        $naspool_data{$naspool_id}{'naspool_name'} = $naspool_id;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $naspool_data{$naspool_id}{'io_rate'} && isdigit( $naspool_data{$naspool_id}{'io_rate'} ) ) {
            $naspool_data{$naspool_id}{'io_rate'} = $naspool_data{$naspool_id}{'io_rate'} + $io_rate;
          }
          else {
            $naspool_data{$naspool_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)

        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $naspool_data{$naspool_id}{'read_io'} && isdigit( $naspool_data{$naspool_id}{'read_io'} ) ) {
            $naspool_data{$naspool_id}{'read_io'} = $naspool_data{$naspool_id}{'read_io'} + $read_io;
          }
          else {
            $naspool_data{$naspool_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)

        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $naspool_data{$naspool_id}{'write_io'} && isdigit( $naspool_data{$naspool_id}{'write_io'} ) ) {
            $naspool_data{$naspool_id}{'write_io'} = $naspool_data{$naspool_id}{'write_io'} + $write_io;
          }
          else {
            $naspool_data{$naspool_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)

        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $naspool_data{$naspool_id}{'data_rate'} && isdigit( $naspool_data{$naspool_id}{'data_rate'} ) ) {
            $naspool_data{$naspool_id}{'data_rate'} = $naspool_data{$naspool_id}{'data_rate'} + $data_rate;
          }
          else {
            $naspool_data{$naspool_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)

        if ( defined $read_kB && isdigit($read_kB) ) {
          if ( defined $naspool_data{$naspool_id}{'read'} && isdigit( $naspool_data{$naspool_id}{'read'} ) ) {
            $naspool_data{$naspool_id}{'read'} = $naspool_data{$naspool_id}{'read'} + $read_kB;
          }
          else {
            $naspool_data{$naspool_id}{'read'} = $read_kB;
          }
        } ## end if ( defined $read_kB ...)

        if ( defined $write_kB && isdigit($write_kB) ) {
          if ( defined $naspool_data{$naspool_id}{'write'} && isdigit( $naspool_data{$naspool_id}{'write'} ) ) {
            $naspool_data{$naspool_id}{'write'} = $naspool_data{$naspool_id}{'write'} + $write_kB;
          }
          else {
            $naspool_data{$naspool_id}{'write'} = $write_kB;
          }
        } ## end if ( defined $write_kB...)

        if ( defined $cap && isdigit($cap) ) {
          if ( defined $naspool_data{$naspool_id}{'cap'} && isdigit( $naspool_data{$naspool_id}{'cap'} ) ) {
            $naspool_data{$naspool_id}{'cap'} = $naspool_data{$naspool_id}{'cap'} + $cap;
          }
          else {
            $naspool_data{$naspool_id}{'cap'} = $cap;
          }
        } ## end if ( defined $cap && isdigit...)
        if ( defined $cap_free && isdigit($cap_free) ) {
          if ( defined $naspool_data{$naspool_id}{'cap_free'} && isdigit( $naspool_data{$naspool_id}{'cap_free'} ) ) {
            $naspool_data{$naspool_id}{'cap_free'} = $naspool_data{$naspool_id}{'cap_free'} + $cap_free;
          }
          else {
            $naspool_data{$naspool_id}{'cap_free'} = $cap_free;
          }
        } ## end if ( defined $cap_free...)
        if ( defined $cap_used && isdigit($cap_used) ) {
          if ( defined $naspool_data{$naspool_id}{'cap_used'} && isdigit( $naspool_data{$naspool_id}{'cap_used'} ) ) {
            $naspool_data{$naspool_id}{'cap_used'} = $naspool_data{$naspool_id}{'cap_used'} + $cap_used;
          }
          else {
            $naspool_data{$naspool_id}{'cap_used'} = $cap_used;
          }
        } ## end if ( defined $cap_used...)

        #print "$time:$name:$nas_id:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB:$cap:$cap_free:$cap_used\n";
      } ## end if ( defined $naspool_id...)
    } ## end if ( $section eq "FS" )

    # FC Port
    ########################

    if ( $section eq "FCPORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      # port_id,Read KB/s,Write KB/s,Read Requests/s,Write Requests/s,data_rate,io_rate
      # spb_fc4,350.5,11.96,7.833,1.540,362.46,9.373

      my ( $port_name, $read_kB, $write_kB, $read_io, $write_io, $data_rate, $io_rate ) = split( /,/, $line );

      if ( $port_name !~ m/^sp._fc/ ) { next; }    # must be in this format

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }

      print "001 FC PORT $port_name - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate\n" if $DEBUG == 3;

      $type = "PORT";
      $rrd  = "$wrkdir/$host/$type/$port_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB);
        my $answer = RRDp::read;
      }
    } ## end if ( $section eq "FCPORT")

    # ISCSI Port
    ########################

    if ( $section eq "ISCSIPORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      # port_id,Read KB/s,Write KB/s,Read Requests/s,Write Requests/s,data_rate,io_rate
      # spb_fc4,350.5,11.96,7.833,1.540,362.46,9.373

      my ( $port_name, $read_kB, $write_kB, $read_io, $write_io, $data_rate, $io_rate ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq ''   || !isdigit($read_io) )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq ''  || !isdigit($write_io) )  { $write_io  = 'U'; }
      if ( !defined($read_kB)   || $read_kB eq ''   || !isdigit($read_kB) )   { $read_kB   = 'U'; }
      if ( !defined($write_kB)  || $write_kB eq ''  || !isdigit($write_kB) )  { $write_kB  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq ''   || !isdigit($io_rate) )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' || !isdigit($data_rate) ) { $data_rate = 'U'; }

      print "001 FC PORT $port_name - $read_io - $write_io - $read_kB - $write_kB - $io_rate - $data_rate\n" if $DEBUG == 3;

      $type = "PORT";
      $rrd  = "$wrkdir/$host/$type/$port_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read_kB:$write_kB);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "ISCSIPORT")

    # ETHERNET Port
    ########################

    if ( $section eq "ETHPORT" ) {
      print "000 $line\n" if $DEBUG == 3;

      # port_id,Read KB/s,Write KB/s,Read packets,Write packets,data_rate,pkts_rate
      # spb_eth3,0.465,0.009,3.923,0.083,0.474,4.006

      my ( $port_name, $read_kB, $write_kB, $read_pkts, $write_pkts, $data_rate, $pkts_rate ) = split( /,/, $line );

      if ( $port_name !~ m/^sp._eth/ ) { next; }    # must be in this format

      if ( !defined($read_pkts)  || $read_pkts eq ''  || !isdigit($read_pkts) )  { $read_pkts  = 'U'; }
      if ( !defined($write_pkts) || $write_pkts eq '' || !isdigit($write_pkts) ) { $write_pkts = 'U'; }
      if ( !defined($read_kB)    || $read_kB eq ''    || !isdigit($read_kB) )    { $read_kB    = 'U'; }
      if ( !defined($write_kB)   || $write_kB eq ''   || !isdigit($write_kB) )   { $write_kB   = 'U'; }
      if ( !defined($pkts_rate)  || $pkts_rate eq ''  || !isdigit($pkts_rate) )  { $pkts_rate  = 'U'; }
      if ( !defined($data_rate)  || $data_rate eq ''  || !isdigit($data_rate) )  { $data_rate  = 'U'; }

      print "001 FC PORT $port_name - $read_pkts - $write_pkts - $read_kB - $write_kB - $pkts_rate - $data_rate\n" if $DEBUG == 3;

      $type = "PORT";
      $rrd  = "$wrkdir/$host/$type/$port_name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$pkts_rate:$read_pkts:$write_pkts:$data_rate:$read_kB:$write_kB);
        my $answer = RRDp::read;
      }
    } ## end if ( $section eq "ETHPORT")

    # SP
    ########################

    if ( $section eq "SP" ) {
      print "000 $line\n" if $DEBUG == 3;

      # sp,summary CPU Util %,NFS Read KB/s,NFS Write KB/s,NFS Avg Read Size KB,NFS Avg Write Size KB,NFS Read Ops/s,NFS Write Ops/s,Total NFS Ops/s,CIFS Read KB/s,CIFS Write KB/s,CIFS Avg Read Size KB,CIFS Avg Write Size KB,CIFS Read Ops/s,CIFS Write Ops/s,CIFS Total Ops/s,Block Cache Dirty Data MB,Block Cache Read Hits Hits/s,Block Cache Write Hits Hits/s,Block Cache Read Misses Misses/s,Block Cache Write Misses Misses/s,Network In KB/s,Network Out KB/s
      # spb,8.169,0,0,0,0,0,0,0.083,0,0,0,0,0,0,0,45.00,3.097,113.1,0.227,4.277,0.481,0.009

      my ( $sp, $cpu, $nfs_read_kb, $nfs_write_kb, $nfs_avg_read_kb, $nfs_avg_write_kb, $nfs_read_ops, $nfs_write_ops, $nfs_total_ops, $cifs_read_kb, $cifs_write_kb, $cifs_avg_read_kb, $cifs_avg_write_kb, $cifs_read_ops, $cifs_write_ops, $cifs_total_ops, $cache_dirty_data_mb, $cache_read_hits, $cache_write_hits, $cache_read_misses, $cache_write_misses, $net_read_kb, $net_write_kb ) = split( /,/, $line );

      if ( !defined($cpu)                 || $cpu eq ''                 || !isdigit($cpu) )                 { $cpu                 = 'U'; }
      if ( !defined($nfs_read_kb)         || $nfs_read_kb eq ''         || !isdigit($nfs_read_kb) )         { $nfs_read_kb         = 'U'; }
      if ( !defined($nfs_write_kb)        || $nfs_write_kb eq ''        || !isdigit($nfs_write_kb) )        { $nfs_write_kb        = 'U'; }
      if ( !defined($nfs_avg_read_kb)     || $nfs_avg_read_kb eq ''     || !isdigit($nfs_avg_read_kb) )     { $nfs_avg_read_kb     = 'U'; }
      if ( !defined($nfs_avg_write_kb)    || $nfs_avg_write_kb eq ''    || !isdigit($nfs_avg_write_kb) )    { $nfs_avg_write_kb    = 'U'; }
      if ( !defined($nfs_read_ops)        || $nfs_read_ops eq ''        || !isdigit($nfs_read_ops) )        { $nfs_read_ops        = 'U'; }
      if ( !defined($nfs_write_ops)       || $nfs_write_ops eq ''       || !isdigit($nfs_write_ops) )       { $nfs_write_ops       = 'U'; }
      if ( !defined($nfs_total_ops)       || $nfs_total_ops eq ''       || !isdigit($nfs_total_ops) )       { $nfs_total_ops       = 'U'; }
      if ( !defined($cifs_read_kb)        || $cifs_read_kb eq ''        || !isdigit($cifs_read_kb) )        { $cifs_read_kb        = 'U'; }
      if ( !defined($cifs_write_kb)       || $cifs_write_kb eq ''       || !isdigit($cifs_write_kb) )       { $cifs_write_kb       = 'U'; }
      if ( !defined($cifs_avg_read_kb)    || $cifs_avg_read_kb eq ''    || !isdigit($cifs_avg_read_kb) )    { $cifs_avg_read_kb    = 'U'; }
      if ( !defined($cifs_avg_write_kb)   || $cifs_avg_write_kb eq ''   || !isdigit($cifs_avg_write_kb) )   { $cifs_avg_write_kb   = 'U'; }
      if ( !defined($cifs_read_ops)       || $cifs_read_ops eq ''       || !isdigit($cifs_read_ops) )       { $cifs_read_ops       = 'U'; }
      if ( !defined($cifs_write_ops)      || $cifs_write_ops eq ''      || !isdigit($cifs_write_ops) )      { $cifs_write_ops      = 'U'; }
      if ( !defined($cifs_total_ops)      || $cifs_total_ops eq ''      || !isdigit($cifs_total_ops) )      { $cifs_total_ops      = 'U'; }
      if ( !defined($cache_dirty_data_mb) || $cache_dirty_data_mb eq '' || !isdigit($cache_dirty_data_mb) ) { $cache_dirty_data_mb = 'U'; }
      if ( !defined($cache_read_hits)     || $cache_read_hits eq ''     || !isdigit($cache_read_hits) )     { $cache_read_hits     = 'U'; }
      if ( !defined($cache_write_hits)    || $cache_write_hits eq ''    || !isdigit($cache_write_hits) )    { $cache_write_hits    = 'U'; }
      if ( !defined($cache_read_misses)   || $cache_read_misses eq ''   || !isdigit($cache_read_misses) )   { $cache_read_misses   = 'U'; }
      if ( !defined($cache_write_misses)  || $cache_write_misses eq ''  || !isdigit($cache_write_misses) )  { $cache_write_misses  = 'U'; }
      if ( !defined($net_read_kb)         || $net_read_kb eq ''         || !isdigit($net_read_kb) )         { $net_read_kb         = 'U'; }
      if ( !defined($net_write_kb)        || $net_write_kb eq ''        || !isdigit($net_write_kb) )        { $net_write_kb        = 'U'; }

      print "001 SP $sp : $cpu,$nfs_read_kb,$nfs_write_kb,$nfs_avg_read_kb,$nfs_avg_write_kb,$nfs_read_ops,$nfs_write_ops,$nfs_total_ops,$cifs_read_kb,$cifs_write_kb,$cifs_avg_read_kb,$cifs_avg_write_kb,$cifs_read_ops,$cifs_write_ops,$cifs_total_ops,$cache_dirty_data_mb,$cache_read_hits,$cache_write_hits,$cache_read_misses,$cache_write_misses,$net_read_kb,$net_write_kb\n" if $DEBUG == 3;

      $type = "NODE";
      $rrd  = "$wrkdir/$host/$type/$sp.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $ltime;
      my $last_rec = "";

      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        RRDp::cmd qq(update "$rrd" $time:$cpu:$nfs_read_kb:$nfs_write_kb:$nfs_avg_read_kb:$nfs_avg_write_kb:$nfs_read_ops:$nfs_write_ops:$nfs_total_ops:$cifs_read_kb:$cifs_write_kb:$cifs_avg_read_kb:$cifs_avg_write_kb:$cifs_read_ops:$cifs_write_ops:$cifs_total_ops:$cache_dirty_data_mb:$cache_read_hits:$cache_write_hits:$cache_read_misses:$cache_write_misses:$net_read_kb:$net_write_kb);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      }
    } ## end if ( $section eq "SP" )

  }    # end of foreach

  # pool summary
  if ( isdigit($pool_index) && $pool_index > 0 ) {
    $type = "POOL";
    data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%pool_data );
  }

  # NAS summary
  if ( isdigit($nas_index) && $nas_index > 0 ) {
    $type = "NAS";
    data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%nas_data );
  }

  # NASPOOL summary
  if ( isdigit($naspool_index) && $naspool_index > 0 ) {
    $type = "NASPOOL";
    data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%naspool_data );
  }

  return ( "0", "$time_last_ok" );
} ## end sub load_data_vnxe_perf

##################################################
#   VNX Filesystem section
##################################################

sub load_data_vnxf_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  my $perf_string = "vnxfconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;

  #print "@files\n";
  closedir(DIR);

  my $in_exist = 0;
  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";
    my $ret = load_data_vnxf_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  $perf_string = "vnxfperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);
  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_vnxf_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)
  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }
  return 0;
} ## end sub load_data_vnxf_all

sub load_data_vnxf_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  #my $lun_file = "$wrkdir/$host/LUN/luns.cfg";
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $fs_file           = "$wrkdir/$host/FS/fs.cfg";
  my $pool_file_write   = 0;                             # true if any write there
  my $config_file_write = 0;                             # true if any write there
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHF,  "> $fs_file-tmp" )     || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  #open(FHL, "> $lun_file-tmp") || main::error ("Can't open $lun_file-tmp : $! ".__FILE__.":".__LINE__) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
  <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name           = "";
  my @pool_name_id        = "";
  my @pool_capacity_tot   = "";
  my @pool_capacity_free  = "";
  my @pool_cfg_id         = "";
  my @rank_cfg_id         = "";
  my $pool_indx           = 0;
  my $cfg_print           = 1;
  my $lun_cfg_print       = 0;
  my $pool_cfg_processing = 0;
  my $lun_cfg_processing  = 0;
  my $fs_cfg_processing   = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);
    if ( $line eq "" ) {
      $pool_cfg_processing = 0;
      $fs_cfg_processing   = 0;
    }

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/LUN Level Configuration/ || $line =~ m/Lun Level Configuration/ ) {
      $lun_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/FS Level Configuration/ ) {
      $fs_cfg_processing = 1;
      next;
    }

    if ($fs_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $fs_cfg_processing = 0;
        next;
      }
      if ( $line =~ m/^---/ || $line =~ m/^id,name,inuse/ || $line =~ m/^,id,status/ ) {
        next;    # some trash like "---"
      }
      my ( $item_id_fs, $item_name_fs ) = split( /,/, $line );
      if ( $item_name_fs ne "" && $item_id_fs ne "" ) {
        print FHF "$item_id_fs:$item_name_fs\n";
        next;
      }
    } ## end if ($fs_cfg_processing)

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id,status/ || $line =~ m/^,id,status/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      my ( $pool_name_cfg_item, $pool_id_cfg_item, $t1, $t2, $t3, $pool_cap_tot, $t4, $pool_cap_free ) = split( /,/, $line );
      if ( $pool_name_cfg_item eq '' ) {

        # HUS does not have POOL alias
        $pool_name_cfg_item = $pool_id_cfg_item;
      }
      $pool_id_cfg_item =~ s/^.*\///;
      $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
      $pool_capacity_free[$pool_indx] = $pool_cap_free;

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)
  }    # foreach

  close(FHW);
  close(FHL);
  close(FHF);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  open( FHR, "< $fs_file-tmp" ) || main::error( "Can't open $fs_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $fs_file" ) || main::error( "Can't open $fs_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg filesystem found : $fs_file-tmp --> $fs_file\n" if $DEBUG;
    unlink("$fs_file-tmp");
    last;
  } ## end foreach $line (@lines)

  #if ( $lun_cfg_print > 0 && -f "$lun_file-tmp" ) {
  #  print "cfg processed  : $host LUN rows: $lun_cfg_print\n" if $DEBUG ;
  #  rename ("$lun_file-tmp","$lun_file");
  #}

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( !-f "$pool_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$pool_file-tmp", "$pool_file" );
  }

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)
  if ( $cfg_section > 0 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    print "cfg not found  : no config in this data file\n" if $DEBUG;
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  # PORT MAPPING #
  #my $type = "MAPPING";
  #port_mapping($type,$wrkdir,$host,$act_time,$st_type,$DEBUG,$webdir);

  close(FH);
  return 0;
} ## end sub load_data_vnxf_conf

sub load_data_vnxf_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $rri              = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there
  my $config_file_write = 0;                       # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  #$DEBUG=3;
  foreach $line (@lines) {
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/NFS|CIFS/ ) {
        if ( $trash ne "NODE" ) {
          $type = "NA";
          next;
        }
      } ## end if ( $type_temp =~ m/NFS|CIFS/)
      if ( $type_temp =~ m/Raid/ ) {
        $type = "RANK";    #  RG --> RANK conversion (RG --> rank)
      }
      else {
        if ( $type_temp =~ m/Node/ && $trash =~ m/Cache/ ) {
          $type = "NODE-CACHE";
        }
        else {
          $type = uc($type_temp);
        }
      } ## end else [ if ( $type_temp =~ m/Raid/)]
      next;
    } ## end if ( $line =~ m/Level Statistics/)
    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/Interval End://g;
      $line =~ s/^\s+|\s+$//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)
    if ( $line =~ m/^$/ ) {
      $type = "NA";      # clear out type when empty line
      next;
    }
    if ( $line =~ /^ID,IO Rate(IOPS)/ || $line =~ /^Controller,Partition,Write Pending/ || $line =~ /^name,id/ || $line =~ /^Controller/ || $line =~ m/^>-Interval/ || $line =~ m/Interval Start/ || $line =~ m/Interval Length/ || $line =~ m/ID/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ || $line =~ m/server_id,server_name/ ) {

      # avoid other trash
      next;
    }

    # Port
    ########################

    if ( $type =~ m/^PORT$/ || $type =~ m/^Port$/ ) {

      # Port statistics
      ( my $name, my $server_id, my $server_name, my $read_packet, my $read_error, my $write, my $write_packet, my $write_error, my $read ) = split( /,/, $line );
      my $data_rate;
      my $packet_rate;
      if ( !defined($name)         || $name eq "" )         { next; }                  ## no name
      if ( !defined($server_id)    || $server_id eq '' )    { $server_id = 'U'; }
      if ( !defined($read_packet)  || $read_packet eq '' )  { $read_packet = 'U'; }
      if ( !defined($read_error)   || $read_error eq '' )   { $read_error = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read = 'U'; }
      if ( !defined($write_packet) || $write_packet eq '' ) { $write_packet = 'U'; }
      if ( !defined($write_error)  || $write_error eq '' )  { $write_error = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      if ( $read_packet eq "U"   && $write_packet eq "U" )   { $packet_rate = 'U'; }
      if ( $read_packet eq "U"   && isdigit($write_packet) ) { $packet_rate = $write_packet; }
      if ( isdigit($read_packet) && isdigit($write_packet) ) { $packet_rate = $write_packet + $read_packet; }
      if ( isdigit($read_packet) && $write_packet eq "U" )   { $packet_rate = $read_packet; }
      if ( !defined($packet_rate) || $packet_rate eq '' ) { $packet_rate = 'U'; }

      print "001 PORT $name - $read_packet - $read_error - $read - $write_packet - $write_error - $write - $data_rate - $packet_rate  - $server_id\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave it if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array

        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        print "004: PORT $name $time:$read_packet:$read_error:$read:$write_packet:$write_error:$write:$data_rate:$packet_rate:$server_id\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$read_packet:$read_error:$read:$write_packet:$write_error:$write:$data_rate:$packet_rate:$server_id);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^PORT$/...)

    ### NODE CACHE

    if ( $type =~ m/^NODE-CACHE$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( my $server_id, my $server_name, my $dlnc_hit, my $cache_hit, my $buffer_hit ) = split( /,/, $line );
      if ( !defined($server_name) || $server_name eq "" ) { next; }
      if ( !defined($dlnc_hit)    || $dlnc_hit eq '' )    { $dlnc_hit = 'U'; }
      if ( !defined($cache_hit)   || $cache_hit eq '' )   { $cache_hit = 'U'; }
      if ( !defined($buffer_hit)  || $buffer_hit eq '' )  { $buffer_hit = 'U'; }

      print "001 NODE-CACHE $server_name - $dlnc_hit - $cache_hit - $buffer_hit\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$server_name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cache_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cache_name)
      if ( $found > -1 ) {
        $ltime = $cache_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cache_name[$l_count] = $rrd;
        $cache_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cache_time[$l_count] = $time;
        $counter_ins++;
        print "004 NODE-CACHE $server_name - $dlnc_hit - $cache_hit - $buffer_hit\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$dlnc_hit:$cache_hit:$buffer_hit);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^NODE-CACHE$/)

    ### CPU-CORE ###

    if ( $type =~ m/^CPU-CORE$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( my $server_id, my $server_name, my $sys, undef, undef, undef, undef ) = split( /,/, $line );

      if ( !defined($server_name) || $server_name eq "" ) { next; }
      if ( !defined($sys) || $sys eq '' ) { $sys = 'U'; }

      print "001 CPU $server_name - $sys\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$server_name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cpuc_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cpuc_name)
      if ( $found > -1 ) {
        $ltime = $cpuc_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cpuc_name[$l_count] = $rrd;
        $cpuc_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cpuc_time[$l_count] = $time;
        $counter_ins++;
        print "004 CPU $server_name - $sys\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$sys:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CPU-CORE$/)

    # POOL
    ########################
    if ( $type =~ m/^POOL$/ ) {

      ( my $id, my $name, my $server_id, my $server_name, my $io_rate, my $qdepth, my $read_io, my $read, my $write_io, my $write, my $busy, my $resp_t, my $cap, my $used_cap ) = split( /,/, $line );

      my $data_rate;
      my $free;
      if ( !defined($io_rate)  || $io_rate eq '' )  { $io_rate  = 'U'; }
      if ( !defined($read_io)  || $read_io eq '' )  { $read_io  = 'U'; }
      if ( !defined($write_io) || $write_io eq '' ) { $write_io = 'U'; }
      if ( !defined($resp_t)   || $resp_t eq '' )   { $resp_t   = 'U'; }
      if ( !defined($cap)      || $cap eq '' )      { $cap      = 'U'; }
      if ( !defined($used_cap) || $used_cap eq '' ) { $used_cap = 'U'; }
      if ( !defined($read)     || $read eq '' )     { $read     = 'U'; }
      if ( !defined($write)    || $write eq '' )    { $write    = 'U'; }
      if ( !defined($qdepth)   || $qdepth eq '' )   { $qdepth   = 'U'; }
      if ( !defined($busy)     || $busy eq '' )     { $busy     = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( isdigit($used_cap) && isdigit($cap) ) { $free = $cap - $used_cap; }
      if ( !defined($free) || $free eq '' ) { $free      = 'U'; }
      if ( !defined($free) || $free eq '' ) { $server_id = 'U'; }

      print "001 POOL $id - $io_rate - $read_io - $write_io - $read - $write - $cap - $used_cap - $resp_t - $busy - $data_rate - $free - $server_id\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }
      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }

      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_time[$l_count] = $time;
        $counter_ins++;
        print "004: POOL $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$qdepth:$used_cap:$free:$resp_t:$busy:$server_id\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$qdepth:$used_cap:$free:$resp_t:$busy:$server_id);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^POOL$/)

    # Volume
    ########################

    if ( $type =~ m/^VOLUME$/ ) {

      # volume statistics
      ( my $id, my $name, my $server_id, my $server_name, my $io_rate, my $qdepth, my $read_io, my $read, my $write_io, my $write, my $busy, my $resp_t, my $cap, my $used_cap ) = split( /,/, $line );

      if ( !defined($id) || $id eq '' ) { next; }
      my $data_rate;
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($server_id) || $server_id eq '' ) { $server_id = 'U'; }
      if ( !defined($qdepth)    || $qdepth eq '' )    { $qdepth    = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }
      if ( !defined($cap)       || $cap eq '' )       { $cap       = 'U'; }
      if ( !defined($used_cap)  || $used_cap eq '' )  { $used_cap  = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($busy)      || $busy eq '' )      { $busy      = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      print "001 Volume $name - $id - $io_rate - $qdepth - $read_io - $read - $write_io - $write - $data_rate - $busy - $resp_t - $cap - $used_cap - $server_id\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }
      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $ldev_time[$l_count] = $time;
        $counter_ins++;
        print "004: Volume $time:$io_rate:$qdepth:$read_io:$read:$write_io:$write:$data_rate:$busy:$resp_t:$cap:$used_cap:$server_id\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$qdepth:$read_io:$read:$write_io:$write:$data_rate:$busy:$resp_t:$cap:$used_cap:$server_id);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^VOLUME$/)

    ### CIFS NODE
    ### server_id,server_name,CIFS Total Ops/s,CIFS Read Ops/s,CIFS Read KiB/s,CIFS Write Ops/s,CIFS Write KiB/s,CIFS Share Connections,CIFS Open Files
    ### 1,server_2,684,37,793,21,2,73,38,2028,2037
    # IOPS rename Calls

    if ( $type =~ m/^CIFS$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( my $server_id, my $server_name, my $io_rate, my $read_io, my $read, my $write_io, my $write ) = split( /,/, $line );

      my $data_rate;
      if ( !defined($server_name) || $server_name eq "" ) { next; }
      if ( !defined($io_rate)     || $io_rate eq '' )     { $io_rate = 'U'; }
      if ( !defined($read_io)     || $read_io eq '' )     { $read_io = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read = 'U'; }
      if ( !defined($write_io)    || $write_io eq '' )    { $write_io = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      print "001 CIFS NODE $server_name - $io_rate - $read_io - $read  - $write_io - $write - $data_rate\n" if $DEBUG == 3;
      my $type_dir = "NODE";
      my $name_rrd = "$server_name" . "_cifs\.rrd";
      $rrd = "$wrkdir/$host/$type_dir/$name_rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type_dir, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cifs_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cifs_name)
      if ( $found > -1 ) {
        $ltime = $cifs_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cifs_name[$l_count] = $rrd;
        $cifs_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cifs_time[$l_count] = $time;
        $counter_ins++;
        print "004 CIFS NODE $server_name - $io_rate - $read_io - $read  - $write_io - $write - $data_rate\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$read:$write_io:$write:$data_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CIFS$/)

    ### NFS NODE
    ### server_id,server_name,NFS Total Ops/s,NFS Read Ops/s,NFS Read KiB/s,NFS Write Ops/s,NFS Write KiB/s,NFS Active Threads
    ### 1,server_2,0,0,0,0,0,0
    # IOPS rename Calls

    if ( $type =~ m/^NFS$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( my $server_id, my $server_name, my $io_rate, my $read_io, my $read, my $write_io, my $write ) = split( /,/, $line );

      my $data_rate;
      if ( !defined($server_name) || $server_name eq "" ) { next; }
      if ( !defined($io_rate)     || $io_rate eq '' )     { $io_rate = 'U'; }
      if ( !defined($read_io)     || $read_io eq '' )     { $read_io = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read = 'U'; }
      if ( !defined($write_io)    || $write_io eq '' )    { $write_io = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      print "001 NFS NODE $server_name - $io_rate - $read_io - $read  - $write_io - $write - $data_rate\n" if $DEBUG == 3;
      my $type_dir = "NODE";
      my $name_rrd = "$server_name" . "_nfs\.rrd";
      $rrd = "$wrkdir/$host/$type_dir/$name_rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type_dir, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@nfs_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@nfs_name)
      if ( $found > -1 ) {
        $ltime = $nfs_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $nfs_name[$l_count] = $rrd;
        $nfs_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $nfs_time[$l_count] = $time;
        $counter_ins++;
        print "004 NFS NODE $server_name - $io_rate - $read_io - $read  - $write_io - $write - $data_rate\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$read:$write_io:$write:$data_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^NFS$/)

    ### FS
    ### id,filesystem,server_id,server_name,Read KiB/s,Read Ops/s,Write KiB/s,Write Ops/s

    if ( $type =~ m/^FS$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( my $id, my $name, my $server_id, my $server_name, my $read, my $read_io, my $write, my $write_io ) = split( /,/, $line );

      my $io_rate;
      my $data_rate;
      if ( !defined($id)       || $id eq "" )       { next; }
      if ( !defined($read_io)  || $read_io eq '' )  { $read_io = 'U'; }
      if ( !defined($read)     || $read eq '' )     { $read = 'U'; }
      if ( !defined($write_io) || $write_io eq '' ) { $write_io = 'U'; }
      if ( !defined($write)    || $write eq '' )    { $write = 'U'; }
      if ( isdigit($write_io)  && isdigit($read_io) )  { $io_rate = $write_io + $read_io; }
      if ( !isdigit($write_io) && isdigit($read_io) )  { $io_rate = $read_io; }
      if ( isdigit($write_io)  && !isdigit($read_io) ) { $io_rate = $write_io; }
      if ( !defined $io_rate || $io_rate eq '' ) { $io_rate = 'U'; }

      if ( isdigit($write)  && isdigit($read) )  { $data_rate = $write + $read; }
      if ( !isdigit($write) && isdigit($read) )  { $data_rate = $read; }
      if ( isdigit($write)  && !isdigit($read) ) { $data_rate = $write; }
      if ( !defined $data_rate || $data_rate eq '' ) { $data_rate = 'U'; }

      if ( !defined($server_id) || $server_id eq '' ) { $server_id = 'U'; }

      print "001 FILESYSTEM  $id - $name - $read - $write - $data_rate - $read_io - $write_io - $io_rate - $server_id\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@fs_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@fs_name)
      if ( $found > -1 ) {
        $ltime = $fs_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $fs_name[$l_count] = $rrd;
        $fs_time[$l_count] = $$last_rec;
        $ltime             = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $fs_time[$l_count] = $time;
        $counter_ins++;
        print "004 FILESYSTEM $id - $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate  - $server_id\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$io_rate:$data_rate:$read:$write:$server_id);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^FS$/ )

    ### CPU-CORE ###

    if ( $type =~ m/^CPU-CORE$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( my $server_id, my $server_name, my $sys, undef, undef, undef, undef ) = split( /,/, $line );

      if ( !defined($server_name) || $server_name eq "" ) { next; }
      if ( !defined($sys) || $sys eq '' ) { $sys = 'U'; }

      print "001 CPU $server_name - $sys\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$server_name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cpuc_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cpuc_name)
      if ( $found > -1 ) {
        $ltime = $cpuc_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cpuc_name[$l_count] = $rrd;
        $cpuc_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cpuc_time[$l_count] = $time;
        $counter_ins++;
        print "004 CPU $server_name - $sys\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$sys:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CPU-CORE$/)

  }    # foreach
  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_vnxf_perf

##################################################
#    HITACHI VSPG section
##################################################

sub load_data_vspg_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  my $perf_string = "vspgconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;

  #print "@files\n";
  closedir(DIR);

  my $in_exist = 0;
  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";
    my $ret = load_data_vspg_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  $perf_string = "vspgperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);
  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_vspg_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)
  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }
  return 0;
} ## end sub load_data_vspg_all

sub load_data_vspg_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  #my $lun_file = "$wrkdir/$host/LUN/luns.cfg";
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $pool_file_write   = 0;                             # true if any write there
  my $config_file_write = 0;                             # true if any write there
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  #open(FHL, "> $lun_file-tmp") || main::error ("Can't open $lun_file-tmp : $! ".__FILE__.":".__LINE__) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
  <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name           = "";
  my @pool_name_id        = "";
  my @pool_capacity_tot   = "";
  my @pool_capacity_free  = "";
  my @pool_cfg_id         = "";
  my @rank_cfg_id         = "";
  my $pool_indx           = 0;
  my $cfg_print           = 1;
  my $lun_cfg_print       = 0;
  my $pool_cfg_processing = 0;
  my $lun_cfg_processing  = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/LUN Level Configuration/ || $line =~ m/Lun Level Configuration/ ) {
      $lun_cfg_processing = 1;
      next;
    }

    #if ($lun_cfg_processing) {
    #  # it does not have name but lus.cfg must be created, for install_st.sh at least, collor table etc
    #  if ($line =~ m/^$/ ) {
    #    $lun_cfg_processing=0;
    #    next; # end of cfg
    #  }
    #  $line =~ s/^0x//;

    #  if ( $line =~ m/^---/ || $line =~ m/^lun_id,/ ) {
    #    next; # some trash like "---"
    #  }

    #  $cfg_load_data_ok = 1;

    #  # create an array with mapping table between ranks and pools
    #  # lun_id,,,,,,pool_id,,capacity (MB),,,,,,,,,,,,interface_type,raid_group_id
    #  # 0,,,,,,,,204800,,,,,,,,,,,,,
    #  # 1,,,,,,,,537600,,,,,,,,,,,,,

    #  my ($lun_id_cfg_item) = split(/,/,$line);
    #  # save actual lun cfg
    #  print FHL "$lun_id_cfg_item : $lun_id_cfg_item\n";
    #  $lun_cfg_print++;
    #  next;
    #}

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id,status/ || $line =~ m/^,id,status/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      my ( $pool_name_cfg_item, $pool_id_cfg_item, $t1, $t2, $t3, $pool_cap_tot, $t4, $pool_cap_free ) = split( /,/, $line );
      if ( $pool_name_cfg_item eq '' ) {

        # HUS does not have POOL alias
        $pool_name_cfg_item = $pool_id_cfg_item;
      }
      $pool_id_cfg_item =~ s/^.*\///;
      $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
      $pool_capacity_free[$pool_indx] = $pool_cap_free;

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)
  }    # foreach

  close(FHW);
  close(FHL);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  #if ( $lun_cfg_print > 0 && -f "$lun_file-tmp" ) {
  #  print "cfg processed  : $host LUN rows: $lun_cfg_print\n" if $DEBUG ;
  #  rename ("$lun_file-tmp","$lun_file");
  #}

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( !-f "$pool_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$pool_file-tmp", "$pool_file" );
  }

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)
  if ( $cfg_section > 0 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    print "cfg not found  : no config in this data file\n" if $DEBUG;
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  # PORT MAPPING #
  my $type = "MAPPING";
  port_mapping( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG, $webdir );

  close(FH);
  return 0;
} ## end sub load_data_vspg_conf

sub load_data_vspg_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $rri              = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there
  my $config_file_write = 0;                       # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  #$DEBUG=3;
  foreach $line (@lines) {
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Raid/ ) {
        $type = "RANK";    #  RG --> RANK conversion (RG --> rank)
      }
      else {
        if ( $type_temp =~ m/Node/ && $trash =~ m/Cache/ ) {
          $type = "NODE-CACHE";
        }
        else {
          $type = uc($type_temp);
        }
      } ## end else [ if ( $type_temp =~ m/Raid/)]
      next;
    } ## end if ( $line =~ m/Level Statistics/)
    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/Interval End://g;
      $line =~ s/^\s+|\s+$//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)
    if ( $line =~ m/^$/ ) {
      $type = "NA";      # clear out type when empty line
      next;
    }
    if ( $line =~ /^ID,IO Rate(IOPS)/ || $line =~ /^Controller,Partition,Write Pending/ || $line =~ /^name,id/ || $line =~ /^Controller/ || $line =~ m/^>-Interval/ || $line =~ m/Interval Start/ || $line =~ m/Interval Length/ || $line =~ m/ID/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }

    # Port
    ########################

    if ( $type =~ m/^PORT$/ || $type =~ m/^Port$/ ) {

      # Port statistics
      ( my $name, my $io_rate, my $data_rate, my $resp, my $io_rate_ini, my $data_rate_ini, my $resp_ini ) = split( /,/, $line );
      if ( !defined($io_rate)       || $io_rate eq '' )       { $io_rate       = 'U'; }
      if ( !defined($data_rate)     || $data_rate eq '' )     { $data_rate     = 'U'; }
      if ( !defined($resp)          || $resp eq '' )          { $resp          = 'U'; }
      if ( !defined($io_rate_ini)   || $io_rate_ini eq '' )   { $io_rate_ini   = 'U'; }
      if ( !defined($data_rate_ini) || $data_rate_ini eq '' ) { $data_rate_ini = 'U'; }
      if ( !defined($resp_ini)      || $resp_ini eq '' )      { $resp_ini      = 'U'; }

      print "001 PORT $name - $io_rate - $data_rate - $resp - $io_rate_ini - $data_rate_ini - $resp_ini\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";
      $rri = "$wrkdir/$host/$type/$name\.rri";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave it if any error during RRDTool file creation to keep data
      }
      $rri =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rri, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave it if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array

        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        print "004: PORT $time:$io_rate:$data_rate:$resp:$io_rate_ini:$data_rate_ini:$resp_ini\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$data_rate:$resp);
        my $answer = RRDp::read;
        $time_last_ok = $time;
        RRDp::cmd qq(update "$rri" $time:$io_rate_ini:$data_rate_ini:$resp_ini);
        $answer       = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^PORT$/...)

    ### HOST
    ###########

    if ( $type =~ m/^HOST/ ) {
      $type = "HOST";    # backward compatability
                         # Host statistics
      ( my $name, my $io_rate, my $data_rate, my $resp ) = split( /,/, $line );
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp)      || $resp eq '' )      { $resp      = 'U'; }

      print "001 HOST-adapter $name - $io_rate - $data_rate - $resp \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@host_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@host_name)
      if ( $found > -1 ) {
        $ltime = $host_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $host_name[$l_count] = $rrd;
        $host_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $host_name[$l_count] - $host_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $host_time[$l_count] = $time;
        $counter_ins++;
        print "004: HOST-adapter $time:$io_rate:$data_rate:$resp\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$data_rate:$resp);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$host_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^HOST/)

    ### CACHE
    ### cache write peding and cache usage rate

    if ( $type =~ m/^NODE-CACHE$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( undef, my $cache_w_pen, my $cache_u_rate, my $cache_allocate ) = split( /,/, $line );

      if ( !defined($cache_w_pen)    || $cache_w_pen eq '' )    { $cache_w_pen    = 'U'; }
      if ( !defined($cache_u_rate)   || $cache_u_rate eq '' )   { $cache_u_rate   = 'U'; }
      if ( !defined($cache_allocate) || $cache_allocate eq '' ) { $cache_allocate = 'U'; }

      #ID,Cache Write Peding(%),Cache Usage Rate(%),Cache Allocate(%)
      #ALL,1,99,

      ### convert ALL to cache.rrd
      my $name = "cache";

      print "001 NODE-CACHE $name - $cache_w_pen - $cache_u_rate - $cache_allocate\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cache_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cache_name)
      if ( $found > -1 ) {
        $ltime = $cache_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cache_name[$l_count] = $rrd;
        $cache_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cache_time[$l_count] = $time;
        $counter_ins++;
        print "004 CPU-UTIL $name - $cache_w_pen - $cache_u_rate - $cache_allocate\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$cache_w_pen:$cache_u_rate:$cache_allocate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^NODE-CACHE$/)

    ### CPU-CORE ###

    if ( $type =~ m/^CPU-CORE$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      ( my $name_core, my $sys ) = split( /,/, $line );

      # name,Usage(%)
      # m1,0
      # m2,0

      print "001 CPU $name_core - $sys\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name_core\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cpuc_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cpuc_name)
      if ( $found > -1 ) {
        $ltime = $cpuc_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cpuc_name[$l_count] = $rrd;
        $cpuc_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cpuc_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($sys) || $sys eq '' ) { $sys = 'U'; }
        print "004 CPU $name_core - $sys\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$sys:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CPU-CORE$/)

    # POOL
    ########################
    if ( $type =~ m/^POOL$/ ) {

      ( my $id, my $io_rate, my $read_io, my $write_io, my $read, my $write, my $data_rate_b, my $io_rate_b, my $resp_r, my $resp_w, my $cap, my $used_cap ) = split( /,/, $line );

      #ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Data rate back,IO rate back,Read response time,Write response time,Total size(TB),Used size(TB)

      my $data_rate;
      my $free;
      if ( !defined($io_rate)     || $io_rate eq '' )     { $io_rate     = 'U'; }
      if ( !defined($read_io)     || $read_io eq '' )     { $read_io     = 'U'; }
      if ( !defined($write_io)    || $write_io eq '' )    { $write_io    = 'U'; }
      if ( !defined($resp_r)      || $resp_r eq '' )      { $resp_r      = 'U'; }
      if ( !defined($resp_w)      || $resp_w eq '' )      { $resp_w      = 'U'; }
      if ( !defined($cap)         || $cap eq '' )         { $cap         = 'U'; }
      if ( !defined($used_cap)    || $used_cap eq '' )    { $used_cap    = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write       = 'U'; }
      if ( !defined($data_rate_b) || $data_rate_b eq '' ) { $data_rate_b = 'U'; }
      if ( !defined($io_rate_b)   || $io_rate_b eq '' )   { $io_rate_b   = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( isdigit($used_cap) && isdigit($cap) ) { $free = $cap - $used_cap; }
      if ( !defined($free) || $free eq '' ) { $free = 'U'; }

      print "001 POOL $id - $io_rate - $read_io - $write_io - $read - $write - $cap - $used_cap - $resp_r - $resp_w - $data_rate_b - $io_rate_b - $data_rate - $free\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }
      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_time[$l_count] = $time;
        $counter_ins++;
        print "004: POOL $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$used_cap:$free:$resp_r:$resp_w:$data_rate_b:$io_rate_b\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$used_cap:$free:$resp_r:$resp_w:$data_rate_b:$io_rate_b:0:0:0:0:0:0);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^POOL$/)

    # LUN
    ########################

    if ( $type =~ m/^LUN$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # LUN statistics
      ( my $name, my $io_rate, my $read_io, my $write_io, my $r_cache_hit, my $w_cache_hit, my $read, my $write, my $resp_r, my $resp_w, my $cap, my $used_cap, my $data_rate_b, my $io_rate_b, undef, undef, undef, undef ) = split( /,/, $line );

      # ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Hit(%),Write Hit(%),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Read response time,Write response time,Capacity(MB),Used(MB),Data rate back,IO rate back,seq_read_io,rnd_read_io,seq_write_io,rnd_write_io,seq_read_hit,rnd_read_hit,seq_write_hit,rnd_write_hit

      my $data_rate;
      if ( !defined($io_rate)     || $io_rate eq '' )     { $io_rate     = 'U'; }
      if ( !defined($read_io)     || $read_io eq '' )     { $read_io     = 'U'; }
      if ( !defined($write_io)    || $write_io eq '' )    { $write_io    = 'U'; }
      if ( !defined($resp_r)      || $resp_r eq '' )      { $resp_r      = 'U'; }
      if ( !defined($resp_w)      || $resp_w eq '' )      { $resp_w      = 'U'; }
      if ( !defined($cap)         || $cap eq '' )         { $cap         = 'U'; }
      if ( !defined($used_cap)    || $used_cap eq '' )    { $used_cap    = 'U'; }
      if ( !defined($r_cache_hit) || $r_cache_hit eq '' ) { $r_cache_hit = 'U'; }
      if ( !defined($w_cache_hit) || $w_cache_hit eq '' ) { $w_cache_hit = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write       = 'U'; }
      if ( !defined($data_rate_b) || $data_rate_b eq '' ) { $data_rate_b = 'U'; }
      if ( !defined($io_rate_b)   || $io_rate_b eq '' )   { $io_rate_b   = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      print "001 VOL $name - $io_rate - $read_io - $write_io - $r_cache_hit - $w_cache_hit - $read - $write - $resp_r - $resp_w - $cap - $used_cap - $data_rate_b - $io_rate_b\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@lun_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@lun_name)
      if ( $found > -1 ) {
        $ltime = $lun_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $lun_name[$l_count] = $rrd;
        $lun_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        print "004: VOL $time:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$r_cache_hit:$w_cache_hit:$resp_r:$resp_w:$cap:$used_cap:$data_rate_b:$io_rate_b\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$resp_r:$resp_w:$read:$write:$data_rate:$r_cache_hit:$w_cache_hit:$cap:$used_cap:$data_rate_b:$io_rate_b);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^LUN$/)

    # Volume
    ########################

    if ( $type =~ m/^VOLUME$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # volume statistics
      ( my $name, my $io_rate, my $read_io, my $write_io, my $r_cache_hit, my $w_cache_hit, my $read, my $write, my $resp_r, my $resp_w, my $cap, my $used_cap, my $data_rate_b, my $io_rate_b, undef, undef, undef, undef ) = split( /,/, $line );

      #ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Hit(%),Write Hit(%),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Read response time,Write response time,Capacity(MB),Used(MB),Data rate back,IO rate back,seq_read_io,rnd_read_io,seq_write_io,rnd_write_io,seq_read_hit,rnd_read_hit,seq_write_hit,rnd_write_hit

      my $data_rate;
      if ( !defined($io_rate)  || $io_rate eq '' )  { $io_rate  = 'U'; }
      if ( !defined($read_io)  || $read_io eq '' )  { $read_io  = 'U'; }
      if ( !defined($write_io) || $write_io eq '' ) { $write_io = 'U'; }
      if ( !defined($resp_r)   || $resp_r eq '' )   { $resp_r   = 'U'; }
      if ( !defined($resp_w)   || $resp_w eq '' )   { $resp_w   = 'U'; }
      if ( !defined($cap)      || $cap eq '' )      { $cap      = 'U'; }
      if ( !defined($used_cap) || $used_cap eq '' ) { $used_cap = 'U'; }

      #if ( ! defined($tier1_used) || $tier1_used eq '' )     { $tier1_used = 'U'; }
      #if ( ! defined($tier2_used) || $tier2_used eq '' )     { $tier2_used = 'U'; }
      #if ( ! defined($tier3_used) || $tier3_used eq '' )     { $tier3_used = 'U'; }
      if ( !defined($r_cache_hit) || $r_cache_hit eq '' ) { $r_cache_hit = 'U'; }
      if ( !defined($w_cache_hit) || $w_cache_hit eq '' ) { $w_cache_hit = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write       = 'U'; }
      if ( !defined($data_rate_b) || $data_rate_b eq '' ) { $data_rate_b = 'U'; }
      if ( !defined($io_rate_b)   || $io_rate_b eq '' )   { $io_rate_b   = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      print "001 Volume $name - $io_rate - $read_io - $write_io - $r_cache_hit - $w_cache_hit - $read - $write - $resp_r - $resp_w - $cap - $used_cap - $data_rate_b - $io_rate_b\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }
      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $ldev_time[$l_count] = $time;
        $counter_ins++;
        print "004: Volume $time:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$r_cache_hit:$w_cache_hit:$resp_r:$resp_w:$cap:$used_cap:$data_rate_b:$io_rate_b\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$resp_r:$resp_w:$read:$write:$data_rate:$r_cache_hit:$w_cache_hit:$cap:$used_cap:$data_rate_b:$io_rate_b:0:0:0:0:0:0);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^VOLUME$/)

    #RANK
    ########################
    if ( $type =~ m/^RANK$/ ) {
      print "100 $line\n" if $DEBUG == 2;
      ( my $id, my $io_rate, my $read_io, my $write_io, my $read, my $write, my $cap, my $used_cap, my $resp_r, my $resp_w, my $data_rate_b, my $io_rate_b, undef, undef, undef, undef ) = split( /,/, $line );
      if ( !defined($id) || $id eq '' ) {
        next;
      }

      # ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Total size(TB),Used size(TB),Read response time,Write response time,Data rate back,IO rate back,seq_read_io,rnd_read_io,seq_write_io,rnd_write_io

      my $data_rate;
      my $free;
      if ( !defined($io_rate)     || $io_rate eq '' )     { $io_rate     = 'U'; }
      if ( !defined($read_io)     || $read_io eq '' )     { $read_io     = 'U'; }
      if ( !defined($write_io)    || $write_io eq '' )    { $write_io    = 'U'; }
      if ( !defined($resp_r)      || $resp_r eq '' )      { $resp_r      = 'U'; }
      if ( !defined($resp_w)      || $resp_w eq '' )      { $resp_w      = 'U'; }
      if ( !defined($cap)         || $cap eq '' )         { $cap         = 'U'; }
      if ( !defined($used_cap)    || $used_cap eq '' )    { $used_cap    = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write       = 'U'; }
      if ( !defined($data_rate_b) || $data_rate_b eq '' ) { $data_rate_b = 'U'; }
      if ( !defined($io_rate_b)   || $io_rate_b eq '' )   { $io_rate_b   = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( isdigit($used_cap) && isdigit($cap) ) { $free = $cap - $used_cap; }
      if ( !defined($free) || $free eq '' ) { $free = 'U'; }

      print "001 RANK $id - $io_rate - $read_io - $write_io - $read - $write - $cap - $used_cap - $resp_r - $resp_w - $data_rate_b - $io_rate_b - $data_rate - $free\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }
      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@rank_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@rank_name)
      if ( $found > -1 ) {
        $ltime = $rank_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $rank_name[$l_count] = $rrd;
        $rank_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
        print "005: $ltime - $l_count\n" if $DEBUG == 2;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $rank_time[$l_count] = $time;
        $counter_ins++;
        print "104: RANK $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$used_cap:$free:$resp_r:$resp_w:$data_rate_b:$io_rate_b\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$used_cap:$free:$resp_r:$resp_w:$data_rate_b:$io_rate_b);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^RANK$/)

  }    # foreach
  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_vspg_perf





#################################################
#   PURE section
#
#################################################


sub load_data_pure_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  my $perf_string = "pureconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;

  #print "@files\n";
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";
    my $ret = load_data_pure_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #now data files like name_dothillperf_20130904_115351.out

  $perf_string = "pureperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_pure_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_pure_all

sub load_data_pure_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  #my $port_file         = "$wrkdir/$host/PORT/port.cfg";
  #my $node_file         = "$wrkdir/$host/NODE/node.cfg";
  #my $fs_file           = "$wrkdir/$host/FS/fs.cfg";
  my $pool_file_write   = 0;                                 # true if any write there-
  my $config_file_write = 0;                                 # true if any write there-
  #my $drive_file        = "$wrkdir/$host/DRIVE/drive.cfg";
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  #open( FHP,  "> $port_file-tmp" )   || main::error( "Can't open $port_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  #open( FHD,  "> $drive_file-tmp" )  || main::error( "Can't open $drive_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  #open( FHN,  "> $node_file-tmp" )   || main::error( "Can't open $node_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  #open( FHF,  "> $fs_file-tmp" )     || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
    <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
    <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name            = "";
  my @pool_name_id         = "";
  my @pool_capacity_tot    = "";
  my @pool_capacity_free   = "";
  my @pool_cfg_id          = "";
  my @rank_cfg_id          = "";
  my $pool_indx            = 0;
  my $cfg_print            = 1;
  my $drive_cfg_processing = 0;
  my $pool_cfg_processing  = 0;
  my $port_cfg_processing  = 0;
  my $drive_cfg_print      = 1;
  my $node_cfg_print       = 1;
  my $node_cfg_processing  = 0;
  my $fs_cfg_processing    = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    #if ( $line =~ m/Port Level Configuration/ ) {
    #  $port_cfg_processing = 1;
    #  next;
    #}
    #if ( $line =~ m/Node Level Configuration/ ) {
    #  $node_cfg_processing = 1;
    #  next;
    #}
    #if ( $line =~ m/FS Level Configuration/ ) {
    #  $fs_cfg_processing = 1;
    #  next;
    #}
    if ($node_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $node_cfg_processing = 0;
        next;    # end of cfg
      }

      if ( $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,health_status,running_status
      my ( $node_id, $node_name ) = split( /,/, $line );
      if ( defined $node_id && defined $node_name && $node_name ne "" && $node_id ne "" ) {
        print FHN "$node_id:$node_name\n";
      }
      next;
    } ## end if ($node_cfg_processing)
    if ($fs_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $fs_cfg_processing = 0;
        next;
      }
      if ( $line =~ m/^---/ || $line =~ m/^id,name/ || $line =~ m/^,id,status/ ) {
        next;    # some trash like "---"
      }
      my ( $item_id_fs, $item_name_fs ) = split( /,/, $line );
      if ( $item_name_fs ne "" && $item_id_fs ne "" ) {
        print FHF "$item_id_fs:$item_name_fs\n";
        next;
      }
    } ## end if ($fs_cfg_processing)


    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id,health_status/ || $line =~ m/^,id,health_status/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      my ( $pool_name_cfg_item, $pool_id_cfg_item, $t1, $t2, $t3, $pool_cap_tot, $t4, $pool_cap_free ) = split( /,/, $line );
      if ( $pool_name_cfg_item eq '' ) {

        # HUS does not have POOL alias
        $pool_name_cfg_item = $pool_id_cfg_item;
      }
      $pool_id_cfg_item =~ s/^.*\///;
      $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
      $pool_capacity_free[$pool_indx] = $pool_cap_free;

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)
  }    # foreach

  close(FHW);
  close(FHP);
  close(FHD);
  close(FHN);
  close(FHF);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( !-f "$pool_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$pool_file-tmp", "$pool_file" );
  }

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 1 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);

  ### Port mapping ###
  my $type = "MAPPING";
  port_mapping( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG, $webdir );

  return 0;
} ## end sub load_data_pure_conf

sub load_data_pure_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $rrd_cap          = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html
  my $pool_data_rate = 0;
  my $pool_io_rate = 0;
  my $pool_read = 0;
  my $pool_write = 0;
  my $pool_read_io = 0;
  my $pool_write_io = 0;
  my $pool_resp_t = 0;
  my $pool_resp_t_r = 0;
  my $pool_resp_t_w = 0;
  my $pool_active = 0;
  my $pool_cap = 0;
  my $time_pool = "";


  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there-
  my $config_file_write = 0;                       # true if any write there-
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  #$DEBUG=3;

  foreach $line (@lines) {
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Raid/ ) {
        $type = "RANK";    #  RG --> RANK conversion (RG --> rank)
      }
      else {
        if ( $type_temp =~ m/Node/ && $trash =~ m/Cache/ ) {
          $type = "NODE-CACHE";
        }
        else {
          $type = uc($type_temp);
          if ( $type eq "LU" || $type eq "LUN" ) {
            $type = "VOLUME";
          }
        } ## end else [ if ( $type_temp =~ m/Node/...)]
      } ## end else [ if ( $type_temp =~ m/Raid/)]
      next;
    } ## end if ( $line =~ m/Level Statistics/)
    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/Interval End://g;
      $line =~ s/^\s+|\s+$//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ /^ID,IO Rate(IOPS)/ || $line =~ /^Controller,Partition,Write Pending/ || $line =~ /^name,id/ || $line =~ /^Controller/ || $line =~ m/^>-Interval/ || $line =~ m/Interval Start/ || $line =~ m/Interval Length/ || $line =~ m/^id,TIME,/ || $line =~ m/^id,name,TIME,/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }

    #if ( $line =~ m/^$/ && $pool_active == 1) {
      ## Pool
      #my $name = "0";
      #my $type = "POOL";

      #my $resp = "U";
      #my $resp_r = "U";
      #my $resp_w = "U";

      #if (isdigit($pool_io_rate) && $pool_io_rate != 0){
      #  $resp = round($pool_resp_t / $pool_io_rate,2);
      #}
      #if (isdigit($pool_read_io) && $pool_read_io != 0){
      #  $resp_r = round($pool_resp_t_r / $pool_read_io,2);
      #}
      #if (isdigit($pool_write_io) && $pool_write_io != 0){
      #  $resp_w = round($pool_resp_t_w / $pool_write_io,2);
      #}


      #print "001 POOL $name - $pool_read_io - $pool_write_io - $pool_io_rate - $resp_r - $resp_w - $resp - $pool_read - $pool_write - $pool_data_rate - $pool_cap\n" if $DEBUG == 3;
      #$rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      #if ( create_rrd( $rrd, $time_pool, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
      #  return 2;    # leave uit if any error during RRDTool file creation to keep data
      #}
      #my $l;
      #my $l_count = 0;
      #my $found   = -1;
      #my $ltime;
      #foreach $l (@pool_name) {
      #  if ( $l =~ m/^$rrd$/ ) {
      #    $found = $l_count;
      #    last;
      #  }
      #  $l_count++;
      #} ## end foreach $l (@vol_name)
      #if ( $found > -1 ) {
      #  $ltime = $pool_time[$found];
      #}
      #else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        #eval {
        #  RRDp::cmd qq(last "$rrd" );
        #  $last_rec = RRDp::read;
        #};
        #if ($@) {
        #  rrd_error( $@, $rrd );
        #  next;
        #}
        #chomp($$last_rec);
        #chomp($$last_rec);
        #$pool_name[$l_count] = $rrd;
        #$pool_time[$l_count] = $$last_rec;
        #$ltime              = $$last_rec;
      #} ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      #if ( isdigit($time_pool) && isdigit($ltime) && $time_pool > $ltime ) {
      #  $pool_time[$l_count] = $time_pool;
      #  $counter_ins++;
      #  print "004: POOL $time_pool:$pool_io_rate:$pool_read_io:$pool_write_io:$pool_read:$pool_write:$pool_data_rate:$resp_r:$resp_w:$resp:$pool_cap\n" if $DEBUG == 3;
      #  RRDp::cmd qq(update "$rrd" $time_pool:$pool_io_rate:$pool_read_io:$pool_write_io:$pool_read:$pool_write:$pool_data_rate:$resp_r:$resp_w:$resp:$pool_cap);
      #  my $answer = RRDp::read;
      #  $time_last_ok = $time_pool;
     #}
     #$pool_active = 0;

    #}

    if ( $line =~ m/^$/ ) {
      $type = "NA";    # clear out type when empty line
      next;
    }

    # Volume
    ########################
    #id,TIME,read_io,write_io,io_rate,read,write,data_rate,san_usec_per_read_op,san_usec_per_write_op,san_usec_per_resp,usec_per_read_op,usec_per_write_op,usec_per_resp,Capacity(MB),Used(MB),Free(MB)
    #99F8EBFE3B0D471800019C23,2017:08:16T21:45:02,0,0,0,0,0,0,,,,0,0,,52224
    #99F8EBFE3B0D471800011011,2017:08:16T21:45:02,13,554,567,52,8971,9023,,,0,0.40,0.60,0.60,8388608
    if ( $type =~ m/^VOLUME$/ || $type =~ m/^LUN$/ || $type =~ m/^Volume$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Volume statistics
      ( my $name, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, undef, undef, undef, my $resp_r, my $resp_w, my $resp, my $cap, my $used_cap, my $free_cap) = split( /,/, $line );

      if ( !defined $name || $name eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)      || $io_rate eq '' )      { $io_rate      = 'U'; }
      if ( !defined($read_io)      || $read_io eq '' )      { $read_io      = 'U'; }
      if ( !defined($write_io)     || $write_io eq '' )     { $write_io     = 'U'; }
      if ( !defined($cap)          || $cap eq '' )          { $cap          = 'U'; }
      if ( !defined($resp_r)       || $resp_r eq '' )       { $resp_r       = 'U'; }
      if ( !defined($resp_w)       || $resp_w eq '' )       { $resp_w       = 'U'; }
      if ( !defined($resp)         || $resp eq '' )         { $resp         = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read         = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write        = 'U'; }
      if ( !defined($data_rate)    || $data_rate eq '' )    { $data_rate    = 'U'; }
      if ( !defined($used_cap)          || $used_cap eq '' )          { $used_cap          = 'U'; }
      if ( !defined($free_cap)          || $free_cap eq '' )          { $free_cap          = 'U'; }
      $pool_active = 1;

      if (isdigit($io_rate)){
        $pool_io_rate = $pool_io_rate + $io_rate;
      }

      if (isdigit($data_rate)){
        $pool_data_rate = $pool_data_rate + $data_rate;
      }

      if (isdigit($read_io)){
        $pool_read_io = $pool_read_io + $read_io;
      }

      if (isdigit($write_io)){
        $pool_write_io = $pool_write_io + $write_io;
      }

      if (isdigit($write)){
        $pool_write = $pool_write + $write;
      }

      if (isdigit($read)){
        $pool_read = $pool_read + $read;
      }

      if (isdigit($resp) && isdigit($io_rate)){
        $pool_resp_t = $pool_resp_t + ($resp * $io_rate);
      }

      if (isdigit($resp_r) && isdigit($read_io)){
        $pool_resp_t_r = $pool_resp_t_r + ($resp_r * $read_io);
      }

      if (isdigit($resp_w) && isdigit($write_io)){
        $pool_resp_t_w = $pool_resp_t_w + ($resp_w * $write_io);
      }

      if (isdigit($cap)){
        $pool_cap = $pool_cap + $cap;
      }


      my $time_volume = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_volume = $time_act;
        }
      } ## end if ( defined $time_text...)
      $time_pool = $time_volume;
      print "001 VOL $name - $read_io - $write_io - $io_rate - $resp_r - $resp_w - $resp - $read - $write - $data_rate - $cap - $used_cap - $free_cap\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_volume, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_volume) && isdigit($ltime) && $time_volume > $ltime ) {
        $vol_time[$l_count] = $time_volume;
        $counter_ins++;
        print "004: VOL $time_volume:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$cap:$used_cap:$free_cap\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_volume:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$cap:$used_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time_volume;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_volume...))
    } ## end if ( $type =~ m/^VOLUME$/...)

    # POOL
    ########################

    if ( $type =~ m/^POOL$/) {

      # RG statistics
      #id,TIME,read_io,write_io,io_rate,read,write,data_rate,san_usec_per_read_op,san_usec_per_write_op,san_usec_per_resp,usec_per_read_op,usec_per_write_op,usec_per_resp,Capacity(TB),queue_length,Used(TB),Free(TB),System(TB),Volumes(TB),Shared_space(TB),Snapshots(TB),data_reduction,total_reduction,thin_provisioning
      #99f8ebfe-3b0d-4718-b76f-719558084025,2017:08:16T21:50:03,164,1477,1641,16299,56072,72371,,,0,0.51,0.60,0.59,59,6
      ( my $id, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, undef, undef, undef, my $resp_r, my $resp_w, my $resp, my $cap,my $queue_length, my $used_cap, my $free_cap, my $system, my $volumes, my $shared_space, my $snapshots, my $data_reduction, my $total_reduction, my $thin_provisioning ) = split( /,/, $line );

      if ( !defined $id || $id eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($cap)       || $cap eq '' )       { $cap       = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp_r)    || $resp_r eq '' )    { $resp_r    = 'U'; }
      if ( !defined($resp_w)    || $resp_w eq '' )    { $resp_w    = 'U'; }
      if ( !defined($resp)      || $resp eq '' )      { $resp      = 'U'; }
      if ( !defined($queue_length)     || $queue_length eq '' )     { $queue_length     = 'U'; }
      if ( !defined($used_cap)       || $used_cap eq '' )       { $used_cap       = 'U'; }
      if ( !defined($free_cap)       || $free_cap eq '' )       { $free_cap       = 'U'; }
      if ( !defined($system)       || $system eq '' )       { $system       = 'U'; }
      if ( !defined($volumes)       || $volumes eq '' )       { $volumes       = 'U'; }
      if ( !defined($shared_space)       || $shared_space eq '' )       { $shared_space       = 'U'; }
      if ( !defined($snapshots)       || $snapshots eq '' )       { $snapshots       = 'U'; }
      if ( !defined($data_reduction)       || $data_reduction eq '' )       { $data_reduction       = 'U'; }
      if ( !defined($total_reduction)       || $total_reduction eq '' )       { $total_reduction       = 'U'; }
      if ( !defined($thin_provisioning)       || $thin_provisioning eq '' )       { $thin_provisioning       = 'U'; }

      my $time_pool = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_pool = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 POOL $id - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_r - $resp_w - $resp - $cap - $queue_length - $used_cap - $free_cap - $system - $volumes - $shared_space - $snapshots - $data_reduction - $total_reduction - $thin_provisioning\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_pool, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time_pool) && isdigit($ltime) && $time_pool > $ltime ) {
        $pool_time[$l_count] = $time_pool;
        $counter_ins++;
        print "004: POOL $time_pool:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$cap:$queue_length:$used_cap:$free_cap:$system:$volumes:$shared_space:$snapshots:$data_reduction:$total_reduction:$thin_provisioning\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_pool:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$cap:$queue_length:$used_cap:$free_cap:$system:$volumes:$shared_space:$snapshots:$data_reduction:$total_reduction:$thin_provisioning);
        my $answer = RRDp::read;
        $time_last_ok = $time_pool;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_rg...))
    } ## end if ( $type =~ m/^rg$/...)


    if ( $type =~ m/^HOST_GROUP$/ ) {
      $type = "HOST_GROUP";

      # Host_group statistics
      #id,name,TIME,read_io,write_io,io_rate,read,write,data_rate,san_usec_per_read_op,san_usec_per_write_op,san_usec_per_resp,usec_per_read_op,usec_per_write_op,usec_per_resp,Capacity(MB),Used(MB),Free(MB)
      #WSQ40A,WSQ40A,2017:08:16T21:50:03,0,3,3,0,16,16,,,0,0,0.44,0.44
      ( my $id, my $name, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, undef, undef, undef, my $resp_t_r, my $resp_t_w, my $resp, my $cap, my $used_cap, my $free_cap ) = split( /,/, $line );
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp)      || $resp eq '' )      { $resp      = 'U'; }
      if ( !defined $name || $name eq "" ) { next; }    #inconsistent data
      if ( !defined($read)     || $read eq '' )     { $read     = 'U'; }
      if ( !defined($write)    || $write eq '' )    { $write    = 'U'; }
      if ( !defined($resp_t_r) || $resp_t_r eq '' ) { $resp_t_r = 'U'; }
      if ( !defined($resp_t_w) || $resp_t_w eq '' ) { $resp_t_w = 'U'; }
      if ( !defined($read_io)  || $read_io eq '' )  { $read_io  = 'U'; }
      if ( !defined($write_io) || $write_io eq '' ) { $write_io = 'U'; }
      if ( !defined($cap) || $cap eq '' ) { $cap = 'U'; }
      if ( !defined($used_cap)  || $used_cap eq '' )  { $used_cap  = 'U'; }
      if ( !defined($free_cap) || $free_cap eq '' ) { $free_cap = 'U'; }

      my $time_hgroup = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_hgroup = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 hgroup $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp - $cap - $used_cap - $free_cap\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/HOST-GROUP/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_hgroup, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@hgroup_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@host_name)
      if ( $found > -1 ) {
        $ltime = $hgroup_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $hgroup_name[$l_count] = $rrd;
        $hgroup_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $host_name[$l_count] - $host_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_hgroup) && isdigit($ltime) && $time_hgroup > $ltime ) {
        $hgroup_time[$l_count] = $time_hgroup;
        $counter_ins++;
        print "004: hgroup $time_hgroup:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp:$cap:$used_cap:$free_cap\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_hgroup:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp:$cap:$used_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time_hgroup;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$host_time[$l_count] = $t;
      } ## end if ( isdigit($time_host...))
    } ## end if ( $type =~ m/^HOST/)



    ### HOST
    if ( $type =~ m/^HOST$/ ) {
      $type = "HOST";

      # Host statistics
      #id,name,TIME,read_io,write_io,io_rate,read,write,data_rate,san_usec_per_read_op,san_usec_per_write_op,san_usec_per_resp,usec_per_read_op,usec_per_write_op,usec_per_resp
      #WSQ40A,WSQ40A,2017:08:16T21:50:03,0,3,3,0,16,16,,,0,0,0.44,0.44
      ( my $id, my $name, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, undef, undef, undef, my $resp_t_r, my $resp_t_w, my $resp, my $cap, my $used_cap, my $free_cap ) = split( /,/, $line );
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp)      || $resp eq '' )      { $resp      = 'U'; }
      if ( !defined $name || $name eq "" ) { next; }    #inconsistent data
      if ( !defined($read)     || $read eq '' )     { $read     = 'U'; }
      if ( !defined($write)    || $write eq '' )    { $write    = 'U'; }
      if ( !defined($resp_t_r) || $resp_t_r eq '' ) { $resp_t_r = 'U'; }
      if ( !defined($resp_t_w) || $resp_t_w eq '' ) { $resp_t_w = 'U'; }
      if ( !defined($read_io)  || $read_io eq '' )  { $read_io  = 'U'; }
      if ( !defined($write_io) || $write_io eq '' ) { $write_io = 'U'; }
      if ( !defined($cap) || $cap eq '' ) { $cap = 'U'; }
      if ( !defined($used_cap)  || $used_cap eq '' )  { $used_cap  = 'U'; }
      if ( !defined($free_cap) || $free_cap eq '' ) { $free_cap = 'U'; }

      my $time_host = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_host = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 HOST $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp - $cap - $used_cap - $free_cap\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_host, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@host_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@host_name)
      if ( $found > -1 ) {
        $ltime = $host_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $host_name[$l_count] = $rrd;
        $host_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $host_name[$l_count] - $host_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_host) && isdigit($ltime) && $time_host > $ltime ) {
        $host_time[$l_count] = $time_host;
        $counter_ins++;
        print "004: HOST $time_host:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp:$cap:$used_cap:$free_cap\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_host:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp:$cap:$used_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time_host;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$host_time[$l_count] = $t;
      } ## end if ( isdigit($time_host...))
    } ## end if ( $type =~ m/^HOST/)

  }    # foreach

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_oceanstor_perf




#################################################
#   OCEANSTOR section
#
#################################################

sub load_data_oceanstor_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  my $perf_string = "oceanstorconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;

  #print "@files\n";
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";
    my $ret = load_data_oceanstor_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #now data files like name_dothillperf_20130904_115351.out

  $perf_string = "oceanstorperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_oceanstor_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_oceanstor_all

sub load_data_oceanstor_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $port_file         = "$wrkdir/$host/PORT/port.cfg";
  my $node_file         = "$wrkdir/$host/NODE/node.cfg";
  my $fs_file           = "$wrkdir/$host/FS/fs.cfg";
  my $pool_file_write   = 0;                                 # true if any write there-
  my $config_file_write = 0;                                 # true if any write there-
  my $drive_file        = "$wrkdir/$host/DRIVE/drive.cfg";
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHP,  "> $port_file-tmp" )   || main::error( "Can't open $port_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHD,  "> $drive_file-tmp" )  || main::error( "Can't open $drive_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FHN,  "> $node_file-tmp" )   || main::error( "Can't open $node_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FHF,  "> $fs_file-tmp" )     || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
    <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
    <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name            = "";
  my @pool_name_id         = "";
  my @pool_capacity_tot    = "";
  my @pool_capacity_free   = "";
  my @pool_cfg_id          = "";
  my @rank_cfg_id          = "";
  my $pool_indx            = 0;
  my $cfg_print            = 1;
  my $drive_cfg_processing = 0;
  my $pool_cfg_processing  = 0;
  my $port_cfg_processing  = 0;
  my $drive_cfg_print      = 1;
  my $node_cfg_print       = 1;
  my $node_cfg_processing  = 0;
  my $fs_cfg_processing    = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Port Level Configuration/ ) {
      $port_cfg_processing = 1;
      next;
    }
    if ( $line =~ m/Node Level Configuration/ ) {
      $node_cfg_processing = 1;
      next;
    }
    if ( $line =~ m/FS Level Configuration/ ) {
      $fs_cfg_processing = 1;
      next;
    }
    if ($node_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $node_cfg_processing = 0;
        next;    # end of cfg
      }

      if ( $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,health_status,running_status
      my ( $node_id, $node_name ) = split( /,/, $line );
      if ( defined $node_id && defined $node_name && $node_name ne "" && $node_id ne "" ) {
        print FHN "$node_id:$node_name\n";
      }
      next;
    } ## end if ($node_cfg_processing)
    if ($fs_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $fs_cfg_processing = 0;
        next;
      }
      if ( $line =~ m/^---/ || $line =~ m/^id,name/ || $line =~ m/^,id,status/ ) {
        next;    # some trash like "---"
      }
      my ( $item_id_fs, $item_name_fs ) = split( /,/, $line );
      if ( $item_name_fs ne "" && $item_id_fs ne "" ) {
        print FHF "$item_id_fs:$item_name_fs\n";
        next;
      }
    } ## end if ($fs_cfg_processing)

    if ($port_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $port_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )

      if ( $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,,,,port_speed(Gbit/s),,,,,health_status,running_status,Protocol
      #549773705216,P0,,,,4,,,,,normal,link up,FC
      my ( $port_id, $port_name, $wwn, undef, undef, $port_speed, undef, undef, undef, undef, $health_status, $running_status, $protocol ) = split( /,/, $line );
      if ( defined $port_id && defined $port_name && $port_name ne "" && $port_id ne "" && defined $protocol && $protocol ne "" ) {
        if ( defined $wwn && $wwn ne "" ) {
          print FHP "$port_id:$port_name ($protocol $wwn)\n";
        }
        else {
          print FHP "$port_id:$port_name ($protocol)\n";
        }
      } ## end if ( defined $port_id ...)
      next;
    } ## end if ($port_cfg_processing)
    if ( $line =~ m/Drive Level Configuration/ ) {
      $drive_cfg_processing = 1;
      next;
    }
    if ($drive_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $drive_cfg_processing = 0;
        next;    # end of cfg

      }
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,health_status,,,,Capacity(GB),Used(GB),Free(GB),,,,,running_status
      #11,11,normal,,,,555.1,260.9,294.2,,,,,online
      my ( $drive_id, $drive_name ) = split( /,/, $line );
      if ( defined $drive_id && $drive_id ne "" && defined $drive_name && $drive_name ne "" ) {
        print FHD "$drive_id,$drive_name\n";
        $drive_cfg_print++;
        next;
      }
    } ## end if ($drive_cfg_processing)

    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id,health_status/ || $line =~ m/^,id,health_status/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      my ( $pool_name_cfg_item, $pool_id_cfg_item, $t1, $t2, $t3, $pool_cap_tot, $t4, $pool_cap_free ) = split( /,/, $line );
      if ( $pool_name_cfg_item eq '' ) {

        # HUS does not have POOL alias
        $pool_name_cfg_item = $pool_id_cfg_item;
      }
      $pool_id_cfg_item =~ s/^.*\///;
      $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
      $pool_capacity_free[$pool_indx] = $pool_cap_free;

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)
  }    # foreach

  close(FHW);
  close(FHP);
  close(FHD);
  close(FHN);
  close(FHF);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  open( FHR, "< $fs_file-tmp" ) || main::error( "Can't open $fs_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa
  foreach $line (@lines) {
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $fs_file" ) || main::error( "Can't open $fs_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";
    }
    close(FHR);
    print "cfg filesystem found : $fs_file-tmp --> $fs_file\n" if $DEBUG;
    unlink("$fs_file-tmp");
    last;
  } ## end foreach $line (@lines)

  open( FHR, "< $port_file-tmp" ) || main::error( "Can't open $port_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $port_file" ) || main::error( "Can't open $port_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg filesystem found : $port_file-tmp --> $port_file\n" if $DEBUG;
    unlink("$port_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( $drive_cfg_print > 0 && -f "$drive_file-tmp" ) {
    print "cfg processed  : $host drive rows: $drive_cfg_print\n" if $DEBUG;
    rename( "$drive_file-tmp", "$drive_file" );
  }

  if ( $node_cfg_print > 0 && -f "$node_file-tmp" ) {
    print "cfg processed  : $host node rows: $node_cfg_print\n" if $DEBUG;
    rename( "$node_file-tmp", "$node_file" );
  }

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( !-f "$pool_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$pool_file-tmp", "$pool_file" );
  }
  if ( !-f "$port_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$port_file-tmp", "$port_file" );
  }

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 1 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);

  ### Port mapping ###
  my $type = "MAPPING";
  port_mapping( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG, $webdir );

  return 0;
} ## end sub load_data_oceanstor_conf

sub load_data_oceanstor_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $rrd_cap          = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there-
  my $config_file_write = 0;                       # true if any write there-
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  #$DEBUG=3;

  foreach $line (@lines) {
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Raid/ ) {
        $type = "RANK";    #  RG --> RANK conversion (RG --> rank)
      }
      else {
        if ( $type_temp =~ m/Node/ && $trash =~ m/Cache/ ) {
          $type = "NODE-CACHE";
        }
        else {
          $type = uc($type_temp);
          if ( $type eq "LU" || $type eq "LUN" ) {
            $type = "VOLUME";
          }
        } ## end else [ if ( $type_temp =~ m/Node/...)]
      } ## end else [ if ( $type_temp =~ m/Raid/)]
      next;
    } ## end if ( $line =~ m/Level Statistics/)
    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/Interval End://g;
      $line =~ s/^\s+|\s+$//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ /^ID,IO Rate(IOPS)/ || $line =~ /^Controller,Partition,Write Pending/ || $line =~ /^name,id/ || $line =~ /^Controller/ || $line =~ m/^>-Interval/ || $line =~ m/Interval Start/ || $line =~ m/Interval Length/ || $line =~ m/^id,TIME,/ || $line =~ m/^id,name,TIME,/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }
    if ( $line =~ m/^$/ ) {
      $type = "NA";    # clear out type when empty line
      next;
    }

    # Port
    ########################
    #id,TIME,read_io,write_io,io_rate,read,write,data_rate,resp_t_r,resp_t_w,resp_t,protocol
    #549773705216,2017:07:21T09:52:50,2,1,3,3.413,1.707,5.12,0.154,0.524,0.27,FC

    if ( $type =~ m/^PORT$/ || $type =~ m/^Port$/ ) {

      # Port statistics
      ( my $id, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp_t, my $port_type_text ) = split( /,/, $line );
      my $time_port = $time;

      if ( !defined $id || $id eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)        || $io_rate eq '' )        { $io_rate        = 'U'; }
      if ( !defined($read_io)        || $read_io eq '' )        { $read_io        = 'U'; }
      if ( !defined($write_io)       || $write_io eq '' )       { $write_io       = 'U'; }
      if ( !defined($read)           || $read eq '' )           { $read           = 'U'; }
      if ( !defined($write)          || $write eq '' )          { $write          = 'U'; }
      if ( !defined($data_rate)      || $data_rate eq '' )      { $data_rate      = 'U'; }
      if ( !defined($resp_t_r)       || $resp_t_r eq '' )       { $resp_t_r       = 'U'; }
      if ( !defined($resp_t)         || $resp_t eq '' )         { $resp_t         = 'U'; }
      if ( !defined($resp_t_w)       || $resp_t_w eq '' )       { $resp_t_w       = 'U'; }
      if ( !defined($port_type_text) || $port_type_text eq '' ) { $port_type_text = ''; }

      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_port = $time_act;
        }
      } ## end if ( defined $time_text...)

      # type : FC / iSCSI / SAS (PCIe)
      my $port_type = 3;    # default unknown port type
      if ( !defined($port_type_text) || $port_type_text eq '' || $port_type_text eq "Unknown" ) {
        if ( $port_type_once == 0 ) {
          main::error( "$host: unknown port type text: $id : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
          $port_type_once = 1;
          $port_type_text = "Unknown";
        }

        #next;
      } ## end if ( !defined($port_type_text...))
      if ( $port_type_text =~ m/^FC$/ )   { $port_type = 0; }
      if ( $port_type_text =~ m/^SAS/ )   { $port_type = 1; }
      if ( $port_type_text =~ m/^iSCSI/ ) { $port_type = 2; }
      if ( $port_type_text =~ m/^IPREP/ ) { $port_type = 4; }    # tady se nejspise nepouziva, to je z SVC ale nechavam pro jistotu
      if ( $port_type_text =~ m/^FCoE/ )  { $port_type = 5; }
      if ( $port_type_text =~ m/^RCIP/ )  { $port_type = 6; }    # It could be RCIP1 etc
      if ( $port_type_text =~ m/^ETH/ )   { $port_type = 7; }

      if ( $port_type == 3 && $port_type_once == 0 ) {

        # unknown port type detected, print error message just once
        main::error( "$host: unknown port type: $id : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
        $port_type_once = 1;
      } ## end if ( $port_type == 3 &&...)

      print "001 PORT $id - $write_io - $read_io - $io_rate - $write - $read - $data_rate - $resp_t_r - $resp_t - $resp_t_w - $port_type:$port_type_text\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_port, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_port) && isdigit($ltime) && $time_port > $ltime ) {
        $port_time[$l_count] = $time_port;
        $counter_ins++;
        print "004: PORT $time_port:$write_io:$read_io:$io_rate:$read:$write:$resp_t_r:$resp_t_w:$resp_t:$port_type\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_port:$write_io:$read_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t:$port_type);
        my $answer = RRDp::read;
        $time_last_ok = $time_port;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_port...))
    } ## end if ( $type =~ m/^PORT$/...)

    # Volume
    ########################
    #id,TIME,read_io,write_io,io_rate,read,write,data_rate,resp_t_r,resp_t_w,resp_t,r_cache_hit,w_cache_hit,Capacity(MB),Used(MB),Free(MB),queue_length
    #4,2017:07:21T09:52:48,0,0,0,0,0,0,0,0,0,0,0,102400,1216,101184

    if ( $type =~ m/^VOLUME$/ || $type =~ m/^LUN$/ || $type =~ m/^Volume$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Volume statistics
      ( my $name, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_r, my $resp_w, my $resp, my $r_cache_hit, my $w_cache_hit, my $cap, my $used_cap, my $free_cap, my $queue_length ) = split( /,/, $line );

      if ( !defined $name || $name eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)      || $io_rate eq '' )      { $io_rate      = 'U'; }
      if ( !defined($read_io)      || $read_io eq '' )      { $read_io      = 'U'; }
      if ( !defined($write_io)     || $write_io eq '' )     { $write_io     = 'U'; }
      if ( !defined($cap)          || $cap eq '' )          { $cap          = 'U'; }
      if ( !defined($resp_r)       || $resp_r eq '' )       { $resp_r       = 'U'; }
      if ( !defined($resp_w)       || $resp_w eq '' )       { $resp_w       = 'U'; }
      if ( !defined($resp)         || $resp eq '' )         { $resp         = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read         = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write        = 'U'; }
      if ( !defined($data_rate)    || $data_rate eq '' )    { $data_rate    = 'U'; }
      if ( !defined($used_cap)     || $used_cap eq '' )     { $used_cap     = 'U'; }
      if ( !defined($free_cap)     || $free_cap eq '' )     { $free_cap     = 'U'; }
      if ( !defined($r_cache_hit)  || $r_cache_hit eq '' )  { $r_cache_hit  = 'U'; }
      if ( !defined($w_cache_hit)  || $w_cache_hit eq '' )  { $w_cache_hit  = 'U'; }
      if ( !defined($queue_length) || $queue_length eq '' ) { $queue_length = 'U'; }

      my $time_volume = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_volume = $time_act;
        }
      } ## end if ( defined $time_text...)
      print "001 VOL $name - $read_io - $write_io - $io_rate - $resp_r - $resp_w - $resp_r - $read - $write - $data_rate - $r_cache_hit - $w_cache_hit - $cap - $used_cap - $free_cap - $queue_length\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_volume, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_volume) && isdigit($ltime) && $time_volume > $ltime ) {
        $vol_time[$l_count] = $time_volume;
        $counter_ins++;
        print "004: VOL $time_volume:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$cap:$used_cap:$queue_length\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_volume:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$r_cache_hit:$w_cache_hit:$cap:$used_cap:$queue_length);
        my $answer = RRDp::read;
        $time_last_ok = $time_volume;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_volume...))
    } ## end if ( $type =~ m/^VOLUME$/...)

    # POOL
    ########################

    if ( $type =~ m/^POOL$/ || $type =~ m/^Pool$/ ) {

      # POOL statistics
      ( my $id, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_r, my $resp_w, my $resp, my $cap, my $used_cap, my $free_cap, my $tier1, my $tier2, my $tier3 ) = split( /,/, $line );

      #id,TIME,read_io,write_io,io_rate,read,write,data_rate,resp_t_r,resp_t_w,resp_t,Capacity(TB),Used(TB),Free(TB),Tier0(TB),Tier1(TB),Tier2(TB)
      #1,2017:07:21T09:52:48,0,0,0,0,0,0,0,0,0,0.033,0,0.033,0.033,,

      if ( !defined $id || $id eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($cap)       || $cap eq '' )       { $cap       = 'U'; }
      if ( !defined($used_cap)  || $used_cap eq '' )  { $used_cap  = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($free_cap)  || $free_cap eq '' )  { $free_cap  = 'U'; }
      if ( !defined($resp_r)    || $resp_r eq '' )    { $resp_r    = 'U'; }
      if ( !defined($resp_w)    || $resp_w eq '' )    { $resp_w    = 'U'; }
      if ( !defined($resp)      || $resp eq '' )      { $resp      = 'U'; }
      if ( !defined($tier1)     || $tier1 eq '' )     { $tier1     = 'U'; }
      if ( !defined($tier2)     || $tier2 eq '' )     { $tier2     = 'U'; }
      if ( !defined($tier3)     || $tier3 eq '' )     { $tier3     = 'U'; }

      my $time_pool = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_pool = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 POOL $id - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_r - $resp_w - $resp - $tier1 - $tier2 - $tier3 - $cap - $used_cap - $free_cap\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_pool, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time_pool) && isdigit($ltime) && $time_pool > $ltime ) {
        $pool_time[$l_count] = $time_pool;
        $counter_ins++;
        ### pool cap tier free - Yet we do not have it
        print "004: POOL $time_pool:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$cap:$used_cap:$free_cap:$tier1:U:$tier2:U:$tier3:U\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_pool:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$cap:$used_cap:$free_cap:$tier1:U:$tier2:U:$tier3:U);
        my $answer = RRDp::read;
        $time_last_ok = $time_pool;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_pool...))
    } ## end if ( $type =~ m/^POOL$/...)

    # DRIVE
    ########################

    if ( $type =~ m/^DRIVE$/ || $type =~ m/^Drive$/ ) {
      print "000 DRIVE $line\n" if $DEBUG == 3;

      # Drive statistics
      #id,TIME,read_io,write_io,io_rate,read,write,data_rate,resp_t_r,resp_t_w,resp_t,Capacity(MB),Used(MB),Free(MB),$queue_length
      #11,2017:07:21T09:52:48,0,0,0,0,0,0,0,0,0,568376,267137,301239
      ( my $name, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp_t, my $cap, my $used_cap, my $free_cap, my $queue_length ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }
      if ( !defined($name)      || $name eq '' ) {
        next;    # drive name is NULL
      }
      if ( !defined($resp_t_r)     || $resp_t_r eq '' )     { $resp_t_r     = 'U'; }
      if ( !defined($resp_t_w)     || $resp_t_w eq '' )     { $resp_t_w     = 'U'; }
      if ( !defined($cap)          || $cap eq '' )          { $cap          = 'U'; }
      if ( !defined($used_cap)     || $used_cap eq '' )     { $used_cap     = 'U'; }
      if ( !defined($free_cap)     || $free_cap eq '' )     { $free_cap     = 'U'; }
      if ( !defined($queue_length) || $queue_length eq '' ) { $queue_length = 'U'; }

      my $time_drive = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_drive = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 DRIVE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t - $cap - $used_cap - $free_cap - $queue_length\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_drive, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@drive_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@drive_name)
      if ( $found > -1 ) {
        $ltime = $drive_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      print "002 DRIVE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t - $cap - $used_cap - $free_cap - $queue_length\n : $time_drive > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time_drive) && isdigit($ltime) && $time_drive > $ltime ) {
        $drive_time[$l_count] = $time_drive;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time_drive:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t:$cap:$used_cap:$queue_length);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time_drive...))
    } ## end if ( $type =~ m/^DRIVE$/...)

    # CTL
    ########################
    if ( $type =~ m/NODE/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # CTL statistics
      #id,TIME,read_io,write_io,io_rate,read,write,data_rate,resp_t_r,resp_t_w,resp_t,r_cache_hit,w_cache_hit,r_cache_usage,w_cache_usage,cache_usage,cpu_usage,$queue_length
      #0B,2017:07:21T09:52:50,2,1,3,3.413,1.707,5.12,0.154,0.524,0.27,0,0,36,0,36,1
      ( my $ctl, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp_t, my $r_cache_hit, my $w_cache_hit, my $r_cache_usage, my $w_cache_usage, my $cache_usage, my $cpu_usage, my $queue_length ) = split( /,/, $line );

      if ( !defined $ctl || $ctl eq "" ) {next}
      ;    #inconsistent data

      my $name = $ctl;

      if ( !defined($io_rate)       || $io_rate eq '' )       { $io_rate       = 'U'; }
      if ( !defined($read_io)       || $read_io eq '' )       { $read_io       = 'U'; }
      if ( !defined($write_io)      || $write_io eq '' )      { $write_io      = 'U'; }
      if ( !defined($read)          || $read eq '' )          { $read          = 'U'; }
      if ( !defined($write)         || $write eq '' )         { $write         = 'U'; }
      if ( !defined($data_rate)     || $data_rate eq '' )     { $data_rate     = 'U'; }
      if ( !defined($resp_t)        || $resp_t eq '' )        { $resp_t        = 'U'; }
      if ( !defined($resp_t_r)      || $resp_t_r eq '' )      { $resp_t_r      = 'U'; }
      if ( !defined($resp_t_w)      || $resp_t_w eq '' )      { $resp_t_w      = 'U'; }
      if ( !defined($cpu_usage)     || $cpu_usage eq '' )     { $cpu_usage     = 'U'; }
      if ( !defined($r_cache_hit)   || $r_cache_hit eq '' )   { $r_cache_hit   = 'U'; }
      if ( !defined($w_cache_hit)   || $w_cache_hit eq '' )   { $w_cache_hit   = 'U'; }
      if ( !defined($r_cache_usage) || $r_cache_usage eq '' ) { $r_cache_usage = 'U'; }
      if ( !defined($w_cache_usage) || $w_cache_usage eq '' ) { $w_cache_usage = 'U'; }
      if ( !defined($cache_usage)   || $cache_usage eq '' )   { $cache_usage   = 'U'; }
      if ( !defined($queue_length)  || $queue_length eq '' )  { $queue_length  = 'U'; }

      my $time_node = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_node = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 NODE $ctl - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t - $r_cache_hit - $w_cache_hit - $r_cache_usage - $w_cache_usage - $cache_usage - $cpu_usage - $queue_length\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_node, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cachen_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cachen_name)
      if ( $found > -1 ) {
        $ltime = $cachen_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      print "002 NODE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t - $r_cache_hit - $w_cache_hit - $r_cache_usage - $w_cache_usage  - $cache_usage - $cpu_usage - $queue_length\n : $time_node > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time_node) && isdigit($ltime) && $time_node > $ltime ) {
        $drive_time[$l_count] = $time_node;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time_node:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t:$r_cache_hit:$w_cache_hit:$r_cache_usage:$w_cache_usage:$cache_usage:$cpu_usage:$queue_length);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time_node...))
    } ## end if ( $type =~ m/NODE/ )
    ### FS
    if ( $type =~ m/^FS/ ) {

      #id,TIME,read_io,write_io,io_rate,read,write,data_rate,resp_t_r,resp_t_w,resp_t
      ( my $name, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp, my $cap, my $used_cap, my $free_cap ) = split( /,/, $line );
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp)      || $resp eq '' )      { $resp      = 'U'; }
      if ( !defined $name || $name eq "" ) { next; }    #inconsistent data
      if ( !defined($read)     || $read eq '' )     { $read     = 'U'; }
      if ( !defined($write)    || $write eq '' )    { $write    = 'U'; }
      if ( !defined($resp_t_r) || $resp_t_r eq '' ) { $resp_t_r = 'U'; }
      if ( !defined($resp_t_w) || $resp_t_w eq '' ) { $resp_t_w = 'U'; }
      if ( !defined($read_io)  || $read_io eq '' )  { $read_io  = 'U'; }
      if ( !defined($write_io) || $write_io eq '' ) { $write_io = 'U'; }
      if ( !defined($cap)      || $cap eq '' )      { $cap      = 'U'; }
      if ( !defined($used_cap) || $used_cap eq '' ) { $used_cap = 'U'; }
      if ( !defined($free_cap) || $free_cap eq '' ) { $free_cap = 'U'; }

      my $time_fs = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_fs = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 FS $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp - $cap - $used_cap - $free_cap\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_fs, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@fs_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@fs_name)
      if ( $found > -1 ) {
        $ltime = $fs_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $fs_name[$l_count] = $rrd;
        $fs_time[$l_count] = $$last_rec;
        $ltime             = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $host_name[$l_count] - $host_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_fs) && isdigit($ltime) && $time_fs > $ltime ) {
        $fs_time[$l_count] = $time_fs;
        $counter_ins++;
        print "004: FS $time_fs:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp:$cap:$used_cap:$free_cap\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_fs:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp:$cap:$used_cap:$free_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time_fs;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$host_time[$l_count] = $t;
      } ## end if ( isdigit($time_fs)...)
    } ## end if ( $type =~ m/^FS/ )
    ### HOST
    if ( $type =~ m/^HOST/ ) {
      $type = "HOST";

      # Host statistics
      #id,name,TIME,read_io,write_io,io_rate,read,write,data_rate,resp_t_r,resp_t_w,resp_t
      #1,Windows,2017:07:21T09:52:48,0,0,0,0,0,0,0,0,0
      ( my $id, my $name, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp ) = split( /,/, $line );
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp)      || $resp eq '' )      { $resp      = 'U'; }
      if ( !defined $name || $name eq "" ) { next; }    #inconsistent data
      if ( !defined($read)     || $read eq '' )     { $read     = 'U'; }
      if ( !defined($write)    || $write eq '' )    { $write    = 'U'; }
      if ( !defined($resp_t_r) || $resp_t_r eq '' ) { $resp_t_r = 'U'; }
      if ( !defined($resp_t_w) || $resp_t_w eq '' ) { $resp_t_w = 'U'; }
      if ( !defined($read_io)  || $read_io eq '' )  { $read_io  = 'U'; }
      if ( !defined($write_io) || $write_io eq '' ) { $write_io = 'U'; }

      my $time_host = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_host = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 HOST $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_host, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@host_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@host_name)
      if ( $found > -1 ) {
        $ltime = $host_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $host_name[$l_count] = $rrd;
        $host_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $host_name[$l_count] - $host_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_host) && isdigit($ltime) && $time_host > $ltime ) {
        $host_time[$l_count] = $time_host;
        $counter_ins++;
        print "004: HOST $time_host:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_host:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp);
        my $answer = RRDp::read;
        $time_last_ok = $time_host;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$host_time[$l_count] = $t;
      } ## end if ( isdigit($time_host...))
    } ## end if ( $type =~ m/^HOST/)

  }    # foreach

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_oceanstor_perf

#################################################
#   COMPELLENT section
#
#################################################

sub load_data_compellent_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # at first config files like dothillconf_20130904_115351.out
  #

  my $perf_string = "compellentconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;

  #print "@files\n";
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";
    my $ret = load_data_compellent_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #now data files like name_dothillperf_20130904_115351.out

  $perf_string = "compellentperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_compellent_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_compellent_all

sub load_data_compellent_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $port_file         = "$wrkdir/$host/PORT/port.cfg";
  my $node_file         = "$wrkdir/$host/NODE/node.cfg";
  my $volfolder_file     = "$wrkdir/$host/VOLUME-FOLDER/volume_folder.cfg";
  my $pool_file_write   = 0;                                 # true if any write there-
  my $config_file_write = 0;                                 # true if any write there-
  my $drive_file        = "$wrkdir/$host/DRIVE/drive.cfg";
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHP,  "> $port_file-tmp" )   || main::error( "Can't open $port_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHD,  "> $drive_file-tmp" )  || main::error( "Can't open $drive_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FHN,  "> $node_file-tmp" )   || main::error( "Can't open $node_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FHVF, "> $volfolder_file-tmp" ) || main::error( "Can't open $volfolder_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
    <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
    <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name            = "";
  my @pool_name_id         = "";
  my @pool_capacity_tot    = "";
  my @pool_capacity_free   = "";
  my @pool_cfg_id          = "";
  my @rank_cfg_id          = "";
  my $pool_indx            = 0;
  my $cfg_print            = 1;
  my $drive_cfg_processing = 0;
  my $pool_cfg_processing  = 0;
  my $port_cfg_processing  = 0;
  my $drive_cfg_print      = 1;
  my $node_cfg_print       = 1;
  my $node_cfg_processing  = 0;
  my $volume_folder_cfg_processing = 0;
  my $volume_folder_cfg_print      = 1;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Port Level Configuration/ ) {
      $port_cfg_processing = 1;
      next;
    }
    if ( $line =~ m/Node Level Configuration/ ) {
      $node_cfg_processing = 1;
      next;
    }
    if ( $line =~ m/Volume-Folder Level Configuration/ ) {
      $volume_folder_cfg_processing = 1;
      next;
    }
    if ($volume_folder_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $volume_folder_cfg_processing = 0;
        next;    # end of cfg
      }

      if ( $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ || $line =~ m/^id,name/ || $line =~ m/^name,id/ ) {
        next;    # some trash like "---"
      }

      #id,name,,,,port_speed,,,,,status
      my ( $volume_folder_name, $volume_folder_id ) = split( /,/, $line );
      if ( defined $volume_folder_id && defined $volume_folder_name && $volume_folder_name ne "" && $volume_folder_id ne "" ) {
        print FHVF "$volume_folder_id:$volume_folder_name\n";
      }
      next;
    } ## end if ($node_cfg_processing)
    if ($node_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $node_cfg_processing = 0;
        next;    # end of cfg
      }

      if ( $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,,,,port_speed,,,,,status
      #id,name,status,
      my ( $node_id, $node_name ) = split( /,/, $line );
      if ( defined $node_id && defined $node_name && $node_name ne "" && $node_id ne "" ) {
        print FHN "$node_id:$node_name\n";
      }
      next;
    } ## end if ($node_cfg_processing)

    if ($port_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $port_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )

      if ( $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,,,,port_speed,,,,,status,protocol
      #0-2,A2,,,,0,,,,,Unknown,
      my ( $port_id, $port_name, undef, undef, undef, undef, undef, undef, undef, undef, undef, $protocol ) = split( /,/, $line );
      if ( defined $port_id && defined $port_name && $port_name ne "" && $port_id ne "" && defined $protocol && $protocol ne "" ) {
        print FHP "$port_id:$port_name ($protocol)\n";
      }
      next;
    } ## end if ($port_cfg_processing)
    if ( $line =~ m/Drive Level Configuration/ ) {
      $drive_cfg_processing = 1;
      next;
    }
    if ($drive_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $drive_cfg_processing = 0;
        next;    # end of cfg

      }
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,status,,,,Capacity(GB),,,,,,,disk_folder_name,disk_class_name
      #0-4,disk_00.04,Ok,,,,1000.205,,,,,,,
      my ( $drive_id, $drive_name,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,$disk_folder_name, $disk_class_name ) = split( /,/, $line );
      if ( defined $drive_id && $drive_id ne "" && defined $drive_name && $drive_name ne "" ) {
        if (!defined $disk_folder_name){$disk_folder_name = "";}
        if (!defined $disk_class_name){$disk_class_name = ""; }
        print FHD "$drive_id,$drive_name,$disk_folder_name,$disk_class_name\n";
        $drive_cfg_print++;
        next;
      }
    } ## end if ($drive_cfg_processing)

    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id,status/ || $line =~ m/^,id,status/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      my ( $pool_name_cfg_item, $pool_id_cfg_item, $t1, $t2, $t3, $pool_cap_tot, $t4, $pool_cap_free ) = split( /,/, $line );
      if ( $pool_name_cfg_item eq '' ) {

        # HUS does not have POOL alias
        $pool_name_cfg_item = $pool_id_cfg_item;
      }
      $pool_id_cfg_item =~ s/^.*\///;
      $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
      $pool_capacity_free[$pool_indx] = $pool_cap_free;

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)
  }    # foreach

  close(FHW);
  close(FHP);
  close(FHD);
  close(FHN);
  close(FHVF);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  open( FHR, "< $port_file-tmp" ) || main::error( "Can't open $port_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $port_file" ) || main::error( "Can't open $port_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg filesystem found : $port_file-tmp --> $port_file\n" if $DEBUG;
    unlink("$port_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( $drive_cfg_print > 0 && -f "$drive_file-tmp" ) {
    print "cfg processed  : $host drive rows: $drive_cfg_print\n" if $DEBUG;
    rename( "$drive_file-tmp", "$drive_file" );
  }

  if ( $node_cfg_print > 0 && -f "$node_file-tmp" ) {
    print "cfg processed  : $host node rows: $node_cfg_print\n" if $DEBUG;
    rename( "$node_file-tmp", "$node_file" );
  }
  if ( $volume_folder_cfg_print > 0 && -f "$volfolder_file-tmp" ) {
    print "cfg processed  : $host volume folder rows: $volume_folder_cfg_print\n" if $DEBUG;
    rename( "$volfolder_file-tmp", "$volfolder_file" );
  }

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( !-f "$pool_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$pool_file-tmp", "$pool_file" );
  }
  if ( !-f "$port_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$port_file-tmp", "$port_file" );
  }

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 1 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);

  ### Port mapping ###
  my $type = "MAPPING";
  port_mapping( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG, $webdir );

  return 0;
} ## end sub load_data_compellent_conf

sub load_data_compellent_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $rrd_cap          = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there-
  my $config_file_write = 0;                       # true if any write there-
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  #$DEBUG=3;

  foreach $line (@lines) {
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Raid/ ) {
        $type = "RANK";    #  RG --> RANK conversion (RG --> rank)
      }
      else {
        if ( $type_temp =~ m/Node/ && $trash =~ m/Cache/ ) {
          $type = "NODE-CACHE";
        }
        else {
          $type = uc($type_temp);
          if ( $type eq "LU" || $type eq "LUN" ) {
            $type = "VOLUME";
          }
        } ## end else [ if ( $type_temp =~ m/Node/...)]
      } ## end else [ if ( $type_temp =~ m/Raid/)]
      next;
    } ## end if ( $line =~ m/Level Statistics/)
    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/Interval End://g;
      $line =~ s/^\s+|\s+$//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ /^ID,IO Rate(IOPS)/ || $line =~ /^Controller,Partition,Write Pending/ || $line =~ /^name,id/ || $line =~ /^Controller/ || $line =~ m/^>-Interval/ || $line =~ m/Interval Start/ || $line =~ m/Interval Length/ || $line =~ m/^ID/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }
    if ( $line =~ m/^$/ ) {
      $type = "NA";    # clear out type when empty line
      next;
    }

    # Port
    ########################
    #ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency
    #5764839588765355528.15,port1,2017:06:16T13:10:45,0,0,0,0,0,0,0,0,0

    if ( $type =~ m/^PORT$/ || $type =~ m/^Port$/ ) {

      # Port statistics
      ( my $id, undef, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp_t, my $port_type_text ) = split( /,/, $line );
      my $time_port = $time;

      if ( !defined $id || $id eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)        || $io_rate eq '' )        { $io_rate        = 'U'; }
      if ( !defined($read_io)        || $read_io eq '' )        { $read_io        = 'U'; }
      if ( !defined($write_io)       || $write_io eq '' )       { $write_io       = 'U'; }
      if ( !defined($read)           || $read eq '' )           { $read           = 'U'; }
      if ( !defined($write)          || $write eq '' )          { $write          = 'U'; }
      if ( !defined($data_rate)      || $data_rate eq '' )      { $data_rate      = 'U'; }
      if ( !defined($resp_t_r)       || $resp_t_r eq '' )       { $resp_t_r       = 'U'; }
      if ( !defined($resp_t)         || $resp_t eq '' )         { $resp_t         = 'U'; }
      if ( !defined($resp_t_w)       || $resp_t_w eq '' )       { $resp_t_w       = 'U'; }
      if ( !defined($port_type_text) || $port_type_text eq '' ) { $port_type_text = ''; }

      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_port = $time_act;
        }
      } ## end if ( defined $time_text...)

      # type : FC / iSCSI / SAS (PCIe)
      my $port_type = 3;    # default unknown port type
      if ( !defined($port_type_text) || $port_type_text eq '' || $port_type_text eq "Unknown" ) {
        if ( $port_type_once == 0 ) {
          main::error( "$host: unknown port type text: $id : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
          $port_type_once = 1;
          $port_type_text = "Unknown";
        }

        #next;
      } ## end if ( !defined($port_type_text...))
      if ( $port_type_text =~ m/^FC$/ )   { $port_type = 0; }
      if ( $port_type_text =~ m/^SAS/ )   { $port_type = 1; }
      if ( $port_type_text =~ m/^iSCSI/ ) { $port_type = 2; }
      if ( $port_type_text =~ m/^IPREP/ ) { $port_type = 4; }    # tady se nejspise nepouziva, to je z SVC ale nechavam pro jistotu
      if ( $port_type_text =~ m/^FCoE/ )  { $port_type = 5; }
      if ( $port_type_text =~ m/^RCIP/ )  { $port_type = 6; }    # It could be RCIP1 etc

      if ( $port_type == 3 && $port_type_once == 0 ) {

        # unknown port type detected, print error message just once
        main::error( "$host: unknown port type: $id : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
        $port_type_once = 1;
      } ## end if ( $port_type == 3 &&...)

      print "001 PORT $id - $write_io - $read_io - $io_rate - $write - $read - $data_rate - $resp_t_r - $resp_t - $resp_t_w - $port_type:$port_type_text\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_port, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_port) && isdigit($ltime) && $time_port > $ltime ) {
        $port_time[$l_count] = $time_port;
        $counter_ins++;
        print "004: PORT $time_port:$write_io:$read_io:$io_rate:$read:$write:$resp_t_r:$resp_t_w:$resp_t:$port_type\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_port:$write_io:$read_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t:$port_type);
        my $answer = RRDp::read;
        $time_last_ok = $time_port;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_port...))
    } ## end if ( $type =~ m/^PORT$/...)

    # Volume
    ########################
    #ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,xferLatency,qosLatency,ioPending,Capacity(MB),Used(MB),Free(MB),Snapshot(MB)
    #6000d3100377ce000000000000000007,volume1,2017:06:16T13:10:45,0,4,4,3,265,268,0,1.855,0,906,0,0,307200,61158,246042,990

    if ( $type =~ m/^VOLUME$/ || $type =~ m/^LUN$/ || $type =~ m/^Volume$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Volume statistics
      ( my $name, undef, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_r, my $resp_w, my $resp, my $resp_xfer, my $resp_qos, my $io_pending, my $cap, my $used_cap, my $free_cap, my $snapshot ) = split( /,/, $line );

      if ( !defined $name || $name eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)    || $io_rate eq '' )    { $io_rate    = 'U'; }
      if ( !defined($read_io)    || $read_io eq '' )    { $read_io    = 'U'; }
      if ( !defined($write_io)   || $write_io eq '' )   { $write_io   = 'U'; }
      if ( !defined($cap)        || $cap eq '' )        { $cap        = 'U'; }
      if ( !defined($resp_r)     || $resp_r eq '' )     { $resp_r     = 'U'; }
      if ( !defined($resp_w)     || $resp_w eq '' )     { $resp_w     = 'U'; }
      if ( !defined($resp)       || $resp eq '' )       { $resp       = 'U'; }
      if ( !defined($resp_xfer)  || $resp_xfer eq '' )  { $resp_xfer  = 'U'; }
      if ( !defined($resp_qos)   || $resp_qos eq '' )   { $resp_qos   = 'U'; }
      if ( !defined($io_pending) || $io_pending eq '' ) { $io_pending = 'U'; }
      if ( !defined($read)       || $read eq '' )       { $read       = 'U'; }
      if ( !defined($write)      || $write eq '' )      { $write      = 'U'; }
      if ( !defined($data_rate)  || $data_rate eq '' )  { $data_rate  = 'U'; }
      if ( !defined($used_cap)   || $used_cap eq '' )   { $used_cap   = 'U'; }
      if ( !defined($free_cap)   || $free_cap eq '' )   { $free_cap   = 'U'; }
      if ( !defined($snapshot)   || $snapshot eq '' )   { $snapshot   = 'U'; }

      my $time_volume = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_volume = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 VOL $name - $read_io - $write_io - $io_rate - $resp_r - $resp_w - $resp_r - $resp_xfer - $resp_qos - $io_pending -  $read - $write - $data_rate - $cap - $used_cap - $free_cap - $snapshot\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_volume, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_volume) && isdigit($ltime) && $time_volume > $ltime ) {
        $vol_time[$l_count] = $time_volume;
        $counter_ins++;
        print "004: VOL $time_volume:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$resp_xfer:$resp_qos:$io_pending:$cap:$used_cap:$snapshot\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_volume:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$resp_xfer:$resp_qos:$io_pending:$cap:$used_cap:$snapshot);
        my $answer = RRDp::read;
        $time_last_ok = $time_volume;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_volume...))
    } ## end if ( $type =~ m/^VOLUME$/...)

        # VOLUME FOLDER
    ########################

    if ( $type =~ m/^VOLUME-FOLDER$/ ) {

      # VOLUME FOLDER statistics
      ( my $id, undef, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_r, my $resp_w, my $resp, my $resp_xfer, my $resp_qos, my $io_pending, my $cap, my $used_cap, my $free_cap ) = split( /,/, $line );

      #ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,xferLatency,qosLatency,ioPending,Capacity(TB),Used(TB),Free(TB)
      #0,pool1,2017:06:16T13:10:45,49,84,133,9237,2416,11653,2.456,1.855,2.235,0,0,0,0,0,0

      if ( !defined $id || $id eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)    || $io_rate eq '' )    { $io_rate    = 'U'; }
      if ( !defined($read_io)    || $read_io eq '' )    { $read_io    = 'U'; }
      if ( !defined($write_io)   || $write_io eq '' )   { $write_io   = 'U'; }
      if ( !defined($cap)        || $cap eq '' )        { $cap        = 'U'; }
      if ( !defined($used_cap)   || $used_cap eq '' )   { $used_cap   = 'U'; }
      if ( !defined($read)       || $read eq '' )       { $read       = 'U'; }
      if ( !defined($write)      || $write eq '' )      { $write      = 'U'; }
      if ( !defined($data_rate)  || $data_rate eq '' )  { $data_rate  = 'U'; }
      if ( !defined($free_cap)   || $free_cap eq '' )   { $free_cap   = 'U'; }
      if ( !defined($resp_r)     || $resp_r eq '' )     { $resp_r     = 'U'; }
      if ( !defined($resp_w)     || $resp_w eq '' )     { $resp_w     = 'U'; }
      if ( !defined($resp)       || $resp eq '' )       { $resp       = 'U'; }
      if ( !defined($resp_xfer)  || $resp_xfer eq '' )  { $resp_xfer  = 'U'; }
      if ( !defined($resp_qos)   || $resp_qos eq '' )   { $resp_qos   = 'U'; }
      if ( !defined($io_pending) || $io_pending eq '' ) { $io_pending = 'U'; }

      my $time_volfolder = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_volfolder = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 VOLUME FOLDER $id - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_r - $resp_w - $resp - $resp_xfer - $resp_qos - $io_pending - $cap - $used_cap - $free_cap\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_volfolder, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@volfolder_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@volfolder_name)
      if ( $found > -1 ) {
        $ltime = $volfolder_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $volfolder_name[$l_count] = $rrd;
        $volfolder_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time_volfolder) && isdigit($ltime) && $time_volfolder > $ltime ) {
        $volfolder_time[$l_count] = $time_volfolder;
        $counter_ins++;
        print "004: VOLUME FOLDER $time_volfolder:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$resp_xfer:$resp_qos:$io_pending:$cap:$used_cap:$free_cap\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_volfolder:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$resp_xfer:$resp_qos:$io_pending:$cap:$used_cap:$free_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time_volfolder;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_pool...))
    } ## end if ( $type =~ m/^POOL$/...)

 

    # POOL
    ########################

    if ( $type =~ m/^POOL$/ || $type =~ m/^Pool$/ ) {

      # POOL statistics
      ( my $id, undef, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_r, my $resp_w, my $resp, my $resp_xfer, my $resp_qos, my $io_pending, my $cap, my $used_cap, my $free_cap, my $snapshot ) = split( /,/, $line );

      #ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,xferLatency,qosLatency,ioPending,Capacity(TB),Used(TB),Free(TB)
      #0,pool1,2017:06:16T13:10:45,49,84,133,9237,2416,11653,2.456,1.855,2.235,0,0,0,0,0,0

      if ( !defined $id || $id eq "" ) { next; }    ### inconsistent data
      if ( !defined($io_rate)    || $io_rate eq '' )    { $io_rate    = 'U'; }
      if ( !defined($read_io)    || $read_io eq '' )    { $read_io    = 'U'; }
      if ( !defined($write_io)   || $write_io eq '' )   { $write_io   = 'U'; }
      if ( !defined($cap)        || $cap eq '' )        { $cap        = 'U'; }
      if ( !defined($used_cap)   || $used_cap eq '' )   { $used_cap   = 'U'; }
      if ( !defined($read)       || $read eq '' )       { $read       = 'U'; }
      if ( !defined($write)      || $write eq '' )      { $write      = 'U'; }
      if ( !defined($data_rate)  || $data_rate eq '' )  { $data_rate  = 'U'; }
      if ( !defined($free_cap)   || $free_cap eq '' )   { $free_cap   = 'U'; }
      if ( !defined($resp_r)     || $resp_r eq '' )     { $resp_r     = 'U'; }
      if ( !defined($resp_w)     || $resp_w eq '' )     { $resp_w     = 'U'; }
      if ( !defined($resp)       || $resp eq '' )       { $resp       = 'U'; }
      if ( !defined($resp_xfer)  || $resp_xfer eq '' )  { $resp_xfer  = 'U'; }
      if ( !defined($resp_qos)   || $resp_qos eq '' )   { $resp_qos   = 'U'; }
      if ( !defined($io_pending) || $io_pending eq '' ) { $io_pending = 'U'; }
      if ( !defined($snapshot) || $snapshot eq '' ) { $snapshot = 'U'; }

      my $time_pool = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_pool = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 POOL $id - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_r - $resp_w - $resp - $resp_xfer - $resp_qos - $io_pending - $cap - $used_cap - $free_cap $snapshot\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_pool, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time_pool) && isdigit($ltime) && $time_pool > $ltime ) {
        $pool_time[$l_count] = $time_pool;
        $counter_ins++;
        print "004: POOL $time_pool:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$resp_xfer:$resp_qos:$io_pending:$cap:$used_cap:$free_cap:$snapshot\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_pool:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$resp_r:$resp_w:$resp:$resp_xfer:$resp_qos:$io_pending:$cap:$used_cap:$free_cap:$snapshot);
        my $answer = RRDp::read;
        $time_last_ok = $time_pool;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time_pool...))
    } ## end if ( $type =~ m/^POOL$/...)

    # DRIVE
    ########################

    if ( $type =~ m/^DRIVE$/ || $type =~ m/^Drive$/ ) {
      print "000 DRIVE $line\n" if $DEBUG == 3;

      # Drive statistics
      #ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,ioPending,Capacity(MB),Used(MB),Free(MB)
      #5,drive1,2017:06:16T13:10:45,5,6,11,1613,275,1888,2.345,6.465,4.218,0,572325.961,472429.516,99896.445
      ( my $name, undef, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp_t, my $io_pending, my $cap, my $used_cap, my $free_cap ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }
      if ( !defined($name)      || $name eq '' ) {
        next;    # drive name is NULL
      }
      if ( !defined($resp_t_r)   || $resp_t_r eq '' )   { $resp_t_r   = 'U'; }
      if ( !defined($resp_t_w)   || $resp_t_w eq '' )   { $resp_t_w   = 'U'; }
      if ( !defined($io_pending) || $io_pending eq '' ) { $io_pending = 'U'; }
      if ( !defined($cap)        || $cap eq '' )        { $cap        = 'U'; }
      if ( !defined($used_cap)   || $used_cap eq '' )   { $used_cap   = 'U'; }
      if ( !defined($free_cap)   || $free_cap eq '' )   { $free_cap   = 'U'; }

      my $time_drive = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_drive = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 DRIVE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t - $io_pending - $cap - $used_cap - $free_cap \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_drive, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@drive_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@drive_name)
      if ( $found > -1 ) {
        $ltime = $drive_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      print "002 DRIVE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t - $io_pending - $cap - $used_cap - $free_cap\n : $time_drive > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time_drive) && isdigit($ltime) && $time_drive > $ltime ) {
        $drive_time[$l_count] = $time_drive;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time_drive:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t:$io_pending:$cap:$used_cap);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time_drive...))
    } ## end if ( $type =~ m/^DRIVE$/...)

    # CTL
    ########################

    if ( $type =~ m/NODE/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # CTL statistics
      #ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,cpuPercentUsage
      #227278,node1,2017:06:16T13:10:45,68,156,224,18503,3981,22484,2.469,2.04,2.339,2
      ( my $ctl, undef, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp_t, my $cpu_usage ) = split( /,/, $line );

      if ( !defined $ctl || $ctl eq "" ) {next}
      ;    #inconsistent data

      my $name = $ctl;

      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq '' )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq '' )  { $resp_t_w  = 'U'; }
      if ( !defined($cpu_usage) || $cpu_usage eq '' ) { $cpu_usage = 'U'; }

      my $time_node = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_node = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 NODE $ctl - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t - $cpu_usage\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_node, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cachen_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cachen_name)
      if ( $found > -1 ) {
        $ltime = $cachen_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      print "002 NODE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t - $cpu_usage \n : $time_node > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time_node) && isdigit($ltime) && $time_node > $ltime ) {
        $drive_time[$l_count] = $time_node;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time_node:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t:$cpu_usage);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time_node...))
    } ## end if ( $type =~ m/NODE/ )
    ### HOST
    if ( $type =~ m/^HOST/ ) {
      $type = "HOST";

      # Host statistics
      #ID,NAME,TIME,readIops,writeIops,totalIops,readKbPerSecond,writeKbPerSecond,totalKbPerSecond,readLatency,writeLatency,totalLatency,xferLatency,qosLatency
      #8,host1,2017:06:16T13:10:45,0,0,0,0,0,0,0,0,0,0,0
      ( my $id, my $name, my $time_text, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp, my $resp_xfer, my $resp_qos ) = split( /,/, $line );
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp)      || $resp eq '' )      { $resp      = 'U'; }
      if ( !defined $name || $name eq "" ) { next; }    #inconsistent data
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq '' )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq '' )  { $resp_t_w  = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($resp_xfer) || $resp_xfer eq '' ) { $resp_xfer = 'U'; }
      if ( !defined($resp_qos)  || $resp_qos eq '' )  { $resp_qos  = 'U'; }

      my $time_host = $time;
      if ( defined $time_text || $time_text ne "" ) {
        my $time_act = str2time($time_text);
        if ( defined $time_act && isdigit($time_act) ) {
          $time_host = $time_act;
        }
      } ## end if ( defined $time_text...)

      print "001 HOST $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp - $resp_xfer - $resp_qos\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      if ( create_rrd( $rrd, $time_host, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;    # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@host_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@host_name)
      if ( $found > -1 ) {
        $ltime = $host_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $host_name[$l_count] = $rrd;
        $host_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $host_name[$l_count] - $host_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time_host) && isdigit($ltime) && $time_host > $ltime ) {
        $host_time[$l_count] = $time_host;
        $counter_ins++;
        print "004: HOST $time_host:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp:$resp_xfer:$resp_qos\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_host:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp:$resp_xfer:$resp_qos);
        my $answer = RRDp::read;
        $time_last_ok = $time_host;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$host_time[$l_count] = $t;
      } ## end if ( isdigit($time_host...))
    } ## end if ( $type =~ m/^HOST/)

  }    # foreach

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_compellent_perf

##################################################
#    DOTHILL section
##################################################

sub load_data_dothill_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # at first config files like dothillconf_20130904_115351.out
  #

  my $perf_string = "dothillconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;

  #print "@files\n";
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";
    my $ret = load_data_dothill_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #now data files like name_dothillperf_20130904_115351.out

  $perf_string = "dothillperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_dothill_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_dothill_all

sub load_data_dothill_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $port_file         = "$wrkdir/$host/PORT/port.cfg";
  my $pool_file_write   = 0;                                 # true if any write there-
  my $config_file_write = 0;                                 # true if any write there-
  my $drive_file        = "$wrkdir/$host/DRIVE/drive.cfg";
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHP,  "> $port_file-tmp" )   || main::error( "Can't open $port_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHD,  "> $drive_file-tmp" )  || main::error( "Can't open $drive_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>
    <BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >
    <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name            = "";
  my @pool_name_id         = "";
  my @pool_capacity_tot    = "";
  my @pool_capacity_free   = "";
  my @pool_cfg_id          = "";
  my @rank_cfg_id          = "";
  my $pool_indx            = 0;
  my $cfg_print            = 1;
  my $drive_cfg_processing = 0;
  my $pool_cfg_processing  = 0;
  my $port_cfg_processing  = 0;
  my $drive_cfg_print      = 1;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Port Level Configuration/ ) {
      $port_cfg_processing = 1;
      next;
    }

    if ($port_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $port_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )

      if ( $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,,,,port_speed,,,,,status,
      #0-2,A2,,,,0,,,,,Unknown,
      my ( $port_id, $port_name ) = split( /,/, $line );
      if ( defined $port_id && defined $port_name && $port_name ne "" && $port_id ne "" ) {
        print FHP "$port_id:$port_name\n";
      }
      next;
    } ## end if ($port_cfg_processing)
    if ( $line =~ m/Drive Level Configuration/ ) {
      $drive_cfg_processing = 1;
      next;
    }
    if ($drive_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $drive_cfg_processing = 0;
        next;    # end of cfg
      }
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^id,name/ ) {
        next;    # some trash like "---"
      }

      #id,name,status,,,,Capacity(GB),,,,,,,
      #0-4,disk_00.04,Ok,,,,1000.205,,,,,,,
      my ( $drive_id, $drive_name ) = split( /,/, $line );
      if ( defined $drive_id && $drive_id ne "" && defined $drive_name && $drive_name ne "" ) {
        print FHD "$drive_id,$drive_name\n";
        $drive_cfg_print++;
        next;
      }
    } ## end if ($drive_cfg_processing)

    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id,status/ || $line =~ m/^,id,status/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      my ( $pool_name_cfg_item, $pool_id_cfg_item, $t1, $t2, $t3, $pool_cap_tot, $t4, $pool_cap_free ) = split( /,/, $line );
      if ( $pool_name_cfg_item eq '' ) {

        # HUS does not have POOL alias
        $pool_name_cfg_item = $pool_id_cfg_item;
      }
      $pool_id_cfg_item =~ s/^.*\///;
      $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
      $pool_capacity_free[$pool_indx] = $pool_cap_free;

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)
  }    # foreach

  close(FHW);
  close(FHP);
  close(FHD);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  open( FHR, "< $port_file-tmp" ) || main::error( "Can't open $port_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $port_file" ) || main::error( "Can't open $port_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg filesystem found : $port_file-tmp --> $port_file\n" if $DEBUG;
    unlink("$port_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( $drive_cfg_print > 0 && -f "$drive_file-tmp" ) {
    print "cfg processed  : $host drive rows: $drive_cfg_print\n" if $DEBUG;
    rename( "$drive_file-tmp", "$drive_file" );
  }

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( !-f "$pool_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$pool_file-tmp", "$pool_file" );
  }
  if ( !-f "$port_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$port_file-tmp", "$port_file" );
  }

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 1 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);

  ### Port mapping ###
  my $type = "MAPPING";
  port_mapping( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG, $webdir );

  return 0;
} ## end sub load_data_dothill_conf

sub load_data_dothill_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $rrd_cap          = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there-
  my $config_file_write = 0;                       # true if any write there-
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  #$DEBUG=3;

  foreach $line (@lines) {
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Raid/ ) {
        $type = "RANK";    #  RG --> RANK conversion (RG --> rank)
      }
      else {
        if ( $type_temp =~ m/Node/ && $trash =~ m/Cache/ ) {
          $type = "NODE-CACHE";
        }
        else {
          $type = uc($type_temp);
          if ( $type eq "LU" || $type eq "LUN" ) {
            $type = "VOLUME";
          }
        } ## end else [ if ( $type_temp =~ m/Node/...)]
      } ## end else [ if ( $type_temp =~ m/Raid/)]
      next;
    } ## end if ( $line =~ m/Level Statistics/)
    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/Interval End://g;
      $line =~ s/^\s+|\s+$//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ /^ID,IO Rate(IOPS)/ || $line =~ /^Controller,Partition,Write Pending/ || $line =~ /^name,id/ || $line =~ /^Controller/ || $line =~ m/^>-Interval/ || $line =~ m/Interval Start/ || $line =~ m/Interval Length/ || $line =~ m/ID/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }
    if ( $line =~ m/^$/ ) {
      $type = "NA";    # clear out type when empty line
      next;
    }

    # Port
    ########################
    #id,ReadIOs,WriteIOs,TotalIOs,KBytesRead,KBytesWritten,KBytesTransferred

    if ( $type =~ m/^PORT$/ || $type =~ m/^Port$/ ) {

      # Port statistics
      ( my $id, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t_r, my $resp_t_w, my $resp_t ) = split( /,/, $line );

      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq '' )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq '' )  { $resp_t_w  = 'U'; }

      print "001 PORT $id - $write_io - $read_io - $io_rate - $write - $read - $data_rate - $resp_t_r - $resp_t - $resp_t_w \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        print "004: PORT $time:$write_io:$read_io:$io_rate:$read:$write:$resp_t_r:$resp_t_w:$resp_t\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$write_io:$read_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^PORT$/...)

    # Volume
    ########################
    #id,ReadIOs,WriteIOs,TotalIOs,KBytesRead,KBytesWritten,KBytesTransferred,ReadHitIOs,WriteHitIOs,capacity (MB),used_Capacity(MB),free_capacity(MB)

    if ( $type =~ m/^VOLUME$/ || $type =~ m/^LUN$/ || $type =~ m/^Volume$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Volume statistics
      ( my $name, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $read_hit_io, my $write_hit_io, my $cap, my $used_cap, my $free_cap ) = split( /,/, $line );

      my $pool_id = "";
      $pool_id =~ s/^.*VP//;

      if ( !defined($io_rate)      || $io_rate eq '' )      { $io_rate      = 'U'; }
      if ( !defined($read_io)      || $read_io eq '' )      { $read_io      = 'U'; }
      if ( !defined($write_io)     || $write_io eq '' )     { $write_io     = 'U'; }
      if ( !defined($cap)          || $cap eq '' )          { $cap          = 'U'; }
      if ( !defined($read_hit_io)  || $read_hit_io eq '' )  { $read_hit_io  = 'U'; }
      if ( !defined($write_hit_io) || $write_hit_io eq '' ) { $write_hit_io = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read         = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write        = 'U'; }
      if ( !defined($data_rate)    || $data_rate eq '' )    { $data_rate    = 'U'; }
      if ( !defined($used_cap)     || $used_cap eq '' )     { $used_cap     = 'U'; }
      if ( !defined($free_cap)     || $free_cap eq '' )     { $free_cap     = 'U'; }

      ### ReadHitIOs to percent
      #if ($read_hit_io ne "U" && $read_io ne "U"){
      #  my $percent = "U";
      #  if ($read_io  != 0){
      #    $percent = ($read_hit_io / $read_io) * 100;
      #  }
      #  else{
      #    $percent = 0;
      #  }
      #  $read_hit_io = $percent;
      #}
      #else{
      #  $read_hit_io = "U";
      #}

      ### WriteHitIOs to percent

      #if ($write_hit_io ne "U" && $write_io ne "U"){
      #  my $percent = "U";
      #  if ($write_io  != 0){
      #    $percent = ($write_hit_io / $write_io) * 100;
      #  }
      #  else{
      #    $percent = 0;
      #  }
      #  $write_hit_io = $percent;
      #}
      #else{
      #  $write_hit_io = "U";
      #}

      print "001 VOL $name - $read_io - $write_io - $io_rate - $read_hit_io - $write_hit_io - $read - $write - $data_rate - $cap - $used_cap - $free_cap\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        print "004: VOL $time:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$read_hit_io:$write_hit_io:$cap:$used_cap:$free_cap\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$read_hit_io:$write_hit_io:$read:$write:$data_rate:$cap:$used_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^VOLUME$/...)

    # POOL
    ########################

    if ( $type =~ m/^POOL$/ || $type =~ m/^Pool$/ ) {

      # POOL statistics
      ( my $id, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $read_hit_io, my $write_hit_io, my $cap, my $used_cap, my $free_cap ) = split( /,/, $line );

      #id,ReadIOs,WriteIOs,TotalIOs,KBytesRead,KBytesWritten,KBytesTransferred,ReadHitIOs,WriteHitIOs,capacity (TB),used_capacity (TB),free_capacity (TB)

      if ( !defined($io_rate)      || $io_rate eq '' )      { $io_rate      = 'U'; }
      if ( !defined($read_io)      || $read_io eq '' )      { $read_io      = 'U'; }
      if ( !defined($write_io)     || $write_io eq '' )     { $write_io     = 'U'; }
      if ( !defined($cap)          || $cap eq '' )          { $cap          = 'U'; }
      if ( !defined($used_cap)     || $used_cap eq '' )     { $used_cap     = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read         = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write        = 'U'; }
      if ( !defined($data_rate)    || $data_rate eq '' )    { $data_rate    = 'U'; }
      if ( !defined($free_cap)     || $free_cap eq '' )     { $free_cap     = 'U'; }
      if ( !defined($read_hit_io)  || $read_hit_io eq '' )  { $read_hit_io  = 'U'; }
      if ( !defined($write_hit_io) || $write_hit_io eq '' ) { $write_hit_io = 'U'; }

      ### ReadHitIOs to percent
      #if ($read_hit_io ne "U" && $read_io ne "U"){
      #  my $percent = "U";
      #  if ($read_io  != 0){
      #    $percent = ($read_hit_io / $read_io) * 100;
      #  }
      #  else{
      #    $percent = 0;
      #  }
      #  $read_hit_io = $percent;
      #}
      #else{
      #  $read_hit_io = "U";
      #}

      ### WriteHitIOs to percent

      if ( $write_hit_io ne "U" && $write_io ne "U" ) {
        my $percent = "U";
        if ( $write_io != 0 ) {
          $percent = ( $write_hit_io / $write_io ) * 100;
        }
        else {
          $percent = 0;
        }
        $write_hit_io = $percent;
      } ## end if ( $write_hit_io ne ...)
      else {
        $write_hit_io = "U";
      }

      print "001 POOL $id - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $read_hit_io - $write_hit_io - $cap - $used_cap - $free_cap\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_time[$l_count] = $time;
        $counter_ins++;
        print "004: POOL $time:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$cap:$used_cap:$free_cap:$read_hit_io:$write_hit_io\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$read_hit_io:$write_hit_io:$cap:$used_cap:$free_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^POOL$/...)

    #RANK
    ########################
    if ( $type =~ m/^RANK$/ ) {
      print "100 $line\n" if $DEBUG == 2;
      ( my $id, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $resp_t ) = split( /,/, $line );
      if ( !defined($id) || $id eq '' ) {
        next;
      }

      # raid group statistics
      #id,ReadIOs,WriteIOs,TotalIOs,KBytesRead,KBytesWritten,KBytesTransferred
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }

      print "001 RANK $id - $io_rate - $read_io - $write_io - $read - $write - $data_rate $resp_t\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }
      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@rank_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@rank_name)
      if ( $found > -1 ) {
        $ltime = $rank_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $rank_name[$l_count] = $rrd;
        $rank_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
        print "005: $ltime - $l_count\n" if $DEBUG == 2;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $rank_time[$l_count] = $time;
        $counter_ins++;
        print "104: RANK $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$resp_t\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$resp_t);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^RANK$/)

    # DRIVE
    ########################

    if ( $type =~ m/^DRIVE$/ || $type =~ m/^Drive$/ ) {
      print "000 DRIVE $line\n" if $DEBUG == 3;

      # Drive statistics
      #id,ReadIOs,WriteIOs,TotalIOs,KBytesRead,KBytesWritten,KBytesTransferred,Capacity(GB)
      ( my $name, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $cap, my $resp_t ) = split( /,/, $line );

      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }
      if ( !defined($cap)       || $cap eq '' )       { $cap       = 'U'; }
      if ( !defined($name)      || $name eq '' ) {
        next;    # drive name is NULL
      }
      print "001 DRIVE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $cap - $resp_t \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@drive_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@drive_name)
      if ( $found > -1 ) {
        $ltime = $drive_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      print "002 DRIVE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $cap - $resp_t\n : $time > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $drive_time[$l_count] = $time;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$cap:$resp_t);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^DRIVE$/...)

    # CTL
    ########################

    if ( $type =~ m/NODE/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # CTL statistics
      #id,ReadIOs,WriteIOs,TotalIOs,KBytesRead,KBytesWritten,KBytesTransferred,ReadHitIOs,WriteHitIOs
      ( my $ctl, my $read_io, my $write_io, my $io_rate, my $read, my $write, my $data_rate, my $read_hit_io, my $write_hit_io ) = split( /,/, $line );

      my $name = $ctl;

      if ( !defined($io_rate)      || $io_rate eq '' )      { $io_rate      = 'U'; }
      if ( !defined($read_io)      || $read_io eq '' )      { $read_io      = 'U'; }
      if ( !defined($write_io)     || $write_io eq '' )     { $write_io     = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read         = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write        = 'U'; }
      if ( !defined($data_rate)    || $data_rate eq '' )    { $data_rate    = 'U'; }
      if ( !defined($read_hit_io)  || $read_hit_io eq '' )  { $read_hit_io  = 'U'; }
      if ( !defined($write_hit_io) || $write_hit_io eq '' ) { $write_hit_io = 'U'; }

      ### ReadHitIOs to percent
      #if ($read_hit_io ne "U" && $read_io ne "U"){
      #  my $percent = "U";
      #  if ($read_io  != 0){
      #    $percent = ($read_hit_io / $read_io) * 100;
      #  }
      #  else{
      #    $percent = 0;
      #  }
      #  $read_hit_io = $percent;
      #}
      #else{
      #  $read_hit_io = "U";
      #}

      ### WriteHitIOs to percent

      #if ($write_hit_io ne "U" && $write_io ne "U"){
      #  my $percent = "U";
      #  if ($write_io  != 0){
      #    $percent = ($write_hit_io / $write_io) * 100;
      #  }
      #  else{
      #    $percent = 0;
      #  }
      #  $write_hit_io = $percent;
      #}
      #else{
      #  $write_hit_io = "U";
      #}

      print "001 Controller $ctl - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $read_hit_io - $write_hit_io\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cachen_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cachen_name)
      if ( $found > -1 ) {
        $ltime = $cachen_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      print "002 NODE $name - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $read_hit_io - $write_hit_io \n : $time > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $drive_time[$l_count] = $time;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$read_hit_io:$write_hit_io);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/NODE/ )

  }    # foreach

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_dothill_perf

##################################################
#    HITACHI HUS section
##################################################

sub load_data_hus_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # at first config files like husconf_20130904_115351.out
  #

  my $perf_string = "husconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;

  #print "@files\n";
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";
    my $ret = load_data_hus_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #now data files like namehus_husperf_20130904_115351.out

  $perf_string = "husperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  ### sort files wrong name files ###
  #$size = scalar @files;
  #print $size . ".......\n";
  #if ($size eq 0 || $size eq 1){}
  #else{
  #  my @timestamp_files = "";
  #  foreach my $file (@files_unsorted){
  #    #print "$file\n";
  #    my $time = (stat("$wrkdir/$host/$file"))[9];
  #    #print $time . "\n";
  #    push(@timestamp_files,$time);
  #  }
  #  my @sort_files = "";
  #  my @sort_timestamp = sort {  $a <=> $b } @timestamp_files;
  #  foreach my $times (@sort_timestamp){
  #    foreach my $file (@files_unsorted){
  #      my $time = (stat("$wrkdir/$host/$file"))[9];
  #      if ($time == $times){
  #        push(@sort_files,$file);
  #      }
  #      else{next;}
  #    }
  #  }
  #  @files = @sort_files;
  #}
  ###

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_hus_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_hus_all

sub load_data_hus_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $pool_file_write   = 0;                             # true if any write there
  my $config_file_write = 0;                             # true if any write there
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD> 
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > 
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name           = "";
  my @pool_name_id        = "";
  my @pool_capacity_tot   = "";
  my @pool_capacity_free  = "";
  my @pool_cfg_id         = "";
  my @rank_cfg_id         = "";
  my $pool_indx           = 0;
  my $cfg_print           = 1;
  my $pool_cfg_processing = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id,status/ || $line =~ m/^,id,status/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      my ( $pool_name_cfg_item, $pool_id_cfg_item, $t1, $t2, $t3, $pool_cap_tot, $t4, $pool_cap_free ) = split( /,/, $line );
      if ( $pool_name_cfg_item eq '' ) {

        # HUS does not have POOL alias
        $pool_name_cfg_item = $pool_id_cfg_item;
      }
      $pool_id_cfg_item =~ s/^.*\///;
      $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
      $pool_capacity_free[$pool_indx] = $pool_cap_free;

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)
  }    # foreach

  close(FHW);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  if ( !-f "$pool_file" ) {

    # looks like no pool has been identified, this could happen on HUS when only RG are in place, create the fake on
    copy( "$pool_file-tmp", "$pool_file" );
  }

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 1 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);

  ### Port mapping ###
  my $type = "MAPPING";
  port_mapping( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG, $webdir );

  return 0;
} ## end sub load_data_hus_conf

sub load_data_hus_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there
  my $config_file_write = 0;                       # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  #$DEBUG=3;

  foreach $line (@lines) {
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Raid/ ) {
        $type = "RANK";    #  RG --> RANK conversion (RG --> rank)
      }
      else {
        if ( $type_temp =~ m/Node/ && $trash =~ m/Cache/ ) {
          $type = "NODE-CACHE";
        }
        else {
          $type = uc($type_temp);
          if ( $type eq "LU" || $type eq "LUN" ) {
            $type = "VOLUME";
          }
        } ## end else [ if ( $type_temp =~ m/Node/...)]
      } ## end else [ if ( $type_temp =~ m/Raid/)]
      next;
    } ## end if ( $line =~ m/Level Statistics/)

    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/Interval End://g;
      $line =~ s/^\s+|\s+$//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ m/^$/ ) {
      $type = "NA";      # clear out type when empty line
      next;
    }

    if ( $line =~ /^ID,IO Rate(IOPS)/ || $line =~ /^Controller,Partition,Write Pending/ || $line =~ /^name,id/ || $line =~ /^Controller/ || $line =~ m/^	Interval/ || $line =~ m/Interval Start/ || $line =~ m/Interval Length/ || $line =~ m/ID/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }

    # RANK ( Raid Group)
    ########################

    if ( $type =~ m/RANK/ ) {
      print "100 $line\n" if $DEBUG == 2;

      # Rank statistics
      ( my $name, my $io_rate, my $read_io, my $write_io, my $read, my $write, my $cap, my $used, my $resp_t_r, my $resp_t_w ) = split( /,/, $line );

      # ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Total size(TB),Used size(TB),Read response time,Write response time
      #
      # 7,52,12,40,228.693,457.387,,,,
      # 2,42,12,28,395.946,385.707,,,,

      if ( !defined($name) || $name eq '' ) {
        next;
      }

      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@rank_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@rank_name)
      if ( $found > -1 ) {
        $ltime = $rank_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $rank_name[$l_count] = $rrd;
        $rank_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
        print "005: $ltime - $l_count\n" if $DEBUG == 2;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $rank_time[$l_count] = $time;
        $counter_ins++;
        my $data_rate = $read;
        if ( isdigit($write) && isdigit($data_rate) ) {
          $data_rate = $data_rate + $write;
        }
        if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
        if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
        if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
        if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
        if ( !defined($resp_t_r)  || $resp_t_r eq '' )  { $resp_t_r  = 'U'; }
        if ( !defined($resp_t_w)  || $resp_t_w eq '' )  { $resp_t_w  = 'U'; }
        if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
        if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
        if ( !defined($cap)       || $cap eq '' )       { $cap       = 'U'; }
        if ( !defined($used)      || $used eq '' )      { $used      = 'U'; }
        print "104: $time:$read_io:$write_io:$read:$write:$resp_t_r:$resp_t_w:$io_rate:$data_rate:$cap:$used\n" if $DEBUG == 2;
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$read:$write:$resp_t_r:$resp_t_w:$io_rate:$data_rate:$cap:$used);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/RANK/ )

    # Port
    ########################

    if ( $type =~ m/^PORT$/ || $type =~ m/^Port$/ ) {

      # Port statistics
      ( my $name, my $io_rate, my $read_io, my $write_io, my $r_cache_hit, my $w_cache_hit, my $read, my $write ) = split( /,/, $line );
      my $data_rate;

      if ( !defined($io_rate)     || $io_rate eq '' )     { $io_rate     = 'U'; }
      if ( !defined($read_io)     || $read_io eq '' )     { $read_io     = 'U'; }
      if ( !defined($write_io)    || $write_io eq '' )    { $write_io    = 'U'; }
      if ( !defined($r_cache_hit) || $r_cache_hit eq '' ) { $r_cache_hit = 'U'; }
      if ( !defined($w_cache_hit) || $w_cache_hit eq '' ) { $w_cache_hit = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write       = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      print "001 PORT $name - $write_io - $read_io - $io_rate - $write - $read -$data_rate \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        print "004: PORT $time:$write_io:$read_io:$io_rate:$r_cache_hit:$w_cache_hit:$read:$write\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$write_io:$read_io:$io_rate:$r_cache_hit:$w_cache_hit:$read:$write:$data_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^PORT$/...)

    # Volume
    ########################

    if ( $type =~ m/^VOLUME$/ || $type =~ m/^LU$/ || $type =~ m/^LUN$/ || $type =~ m/^Volume$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $ctl, my $io_rate, my $read_io, my $write_io, my $r_cache_hit, my $w_cache_hit, my $read, my $write, my $resp_r, my $resp_w, my $cap, my $used_cap ) = split( /,/, $line );

      # Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
      # Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),
      # Peak Read Response Time (ms),Peak Write Response Time (ms),,,Read Hits,Write Hits,Data Read (KB),Data Written (KB),Volume Name,Pool ID,Pool Name,,,,,,,
      # ,,,,,,,,,,,,,,,,Source Volume Name

      # 00102832,2014:10:26T06:10:07.000000,602,0.028,2.020,2.048,0.113,9.349,9.462,4.000,4.628,4.620,0.424,0.749,0.000,,,,,,,,,vio_1p_01,IBMTSDS:IBM.2810-XXXXXXX-VP102778,VIO_1P,,,,
      my $pool_id = "";
      $pool_id =~ s/^.*VP//;

      my $data_rate;
      if ( !defined($io_rate)     || $io_rate eq '' )     { $io_rate     = 'U'; }
      if ( !defined($read_io)     || $read_io eq '' )     { $read_io     = 'U'; }
      if ( !defined($write_io)    || $write_io eq '' )    { $write_io    = 'U'; }
      if ( !defined($resp_r)      || $resp_r eq '' )      { $resp_r      = 'U'; }
      if ( !defined($resp_w)      || $resp_w eq '' )      { $resp_w      = 'U'; }
      if ( !defined($cap)         || $cap eq '' )         { $cap         = 'U'; }
      if ( !defined($used_cap)    || $used_cap eq '' )    { $used_cap    = 'U'; }
      if ( !defined($r_cache_hit) || $r_cache_hit eq '' ) { $r_cache_hit = 'U'; }
      if ( !defined($w_cache_hit) || $w_cache_hit eq '' ) { $w_cache_hit = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write       = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      print "001 VOL $name - $read_io - $write_io - $r_cache_hit - $w_cache_hit - $read - $write - $data_rate - $resp_r - $resp_w\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        print "004: VOL $time:$ctl:$io_rate:$read_io:$write_io:$read:$write:$data_rate:$r_cache_hit:$w_cache_hit:$resp_r:$resp_w:$cap:$used_cap\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$ctl:$io_rate:$read_io:$write_io:$resp_r:$resp_w:$read:$write:$data_rate:$r_cache_hit:$w_cache_hit:$cap:$used_cap);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^VOLUME$/...)

    # POOL
    ########################

    if ( $type =~ m/^DP$/ || $type =~ m/^POOL$/ || $type =~ m/^Pool$/ ) {
      if ( $type eq "DP" ) { $type = "POOL"; }

      # Port statistics
      ( my $id, my $io_rate, my $read_io, my $write_io, my $read, my $write, my $cap, my $used_cap, my $resp_r, my $resp_w ) = split( /,/, $line );

      # Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
      # Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),
      # Peak Read Response Time (ms),Peak Write Response Time (ms),,,Read Hits,Write Hits,Data Read (KB),Data Written (KB),Volume Name,Pool ID,Pool Name,,,,,,,
      # ,,,,,,,,,,,,,,,,Source Volume Name

      my $data_rate;
      my $free;
      if ( !defined($io_rate)  || $io_rate eq '' )  { $io_rate  = 'U'; }
      if ( !defined($read_io)  || $read_io eq '' )  { $read_io  = 'U'; }
      if ( !defined($write_io) || $write_io eq '' ) { $write_io = 'U'; }
      if ( !defined($resp_r)   || $resp_r eq '' )   { $resp_r   = 'U'; }
      if ( !defined($resp_w)   || $resp_w eq '' )   { $resp_w   = 'U'; }
      if ( !defined($cap)      || $cap eq '' )      { $cap      = 'U'; }
      if ( !defined($used_cap) || $used_cap eq '' ) { $used_cap = 'U'; }
      if ( !defined($read)     || $read eq '' )     { $read     = 'U'; }
      if ( !defined($write)    || $write eq '' )    { $write    = 'U'; }
      if ( $read eq "U"   && $write eq "U" )   { $data_rate = 'U'; }
      if ( $read eq "U"   && isdigit($write) ) { $data_rate = $write; }
      if ( isdigit($read) && isdigit($write) ) { $data_rate = $write + $read; }
      if ( isdigit($read) && $write eq "U" )   { $data_rate = $read; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( isdigit($used_cap) && isdigit($cap) ) { $free = $cap - $used_cap; }
      if ( !defined($free) || $free eq '' ) { $free = 'U'; }

      print "001 POOL $id - $read_io - $write_io  - $read - $data_rate\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_time[$l_count] = $time;
        $counter_ins++;
        print "004: POOL $time:$io_rate:$read_io:$write_io:$read:$write:$used_cap:$free:$resp_r:$resp_w\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$used_cap:$free:$resp_r:$resp_w);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^DP$/ ...)

    # DRIVE
    ########################

    if ( $type =~ m/^DRIVE$/ || $type =~ m/^Drive$/ ) {
      print "000 DRIV $line\n" if $DEBUG == 3;

      # Drive statistics
      ( my $name, undef, undef, my $read_io, my $write_io, my $read, my $write, my $operating_rate ) = split( /,/, $line );

      if ( !defined($read_io)        || $read_io eq '' )        { $read_io        = 'U'; }
      if ( !defined($write_io)       || $write_io eq '' )       { $write_io       = 'U'; }
      if ( !defined($read)           || $read eq '' )           { $read           = 'U'; }
      if ( !defined($write)          || $write eq '' )          { $write          = 'U'; }
      if ( !defined($operating_rate) || $operating_rate eq '' ) { $operating_rate = 'U'; }
      if ( !defined($name)           || $name eq '' ) {
        next;    # drive name is NULL
      }
      print "001 DRIV $name - $read_io - $write_io -  $read - $write - $operating_rate \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@drive_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@drive_name)
      if ( $found > -1 ) {
        $ltime = $drive_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      print "002 DRIV $name - $read_io -  $write_io - $read - $write - w\n : $time > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $drive_time[$l_count] = $time;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$read:$write:$operating_rate);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^DRIVE$/...)

    # Node Cache
    ########################

    if ( $type =~ m/NODE-CACHE/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $ctl, my $partition, my $write_pend, my $clean_usage, my $middle_usage, my $phys_usage ) = split( /,/, $line );

      if ( isdigit($ctl) == 0 ) {
        next;
      }

      my $name = "node" . $ctl . "_part" . $partition;

      # Controller,Partition,Write Pending Rate(%),Clean Queue Usage Rate(%),Middle Queue Usage Rate(%),Physical Queue Usage Rate(%)
      # 1,1,7,93,3,4

      print "001 CPU $ctl - $partition - $write_pend - $clean_usage - $middle_usage - $phys_usage\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cachen_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cachen_name)
      if ( $found > -1 ) {
        $ltime = $cachen_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cachen_name[$l_count] = $rrd;
        $cachen_time[$l_count] = $$last_rec;
        $ltime                 = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cachen_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($write_pend)   || $write_pend eq '' )   { $write_pend   = 'U'; }
        if ( !defined($clean_usage)  || $clean_usage eq '' )  { $clean_usage  = 'U'; }
        if ( !defined($middle_usage) || $middle_usage eq '' ) { $middle_usage = 'U'; }
        if ( !defined($phys_usage)   || $phys_usage eq '' )   { $phys_usage   = 'U'; }
        print "004 CPU $name - $write_pend:$clean_usage:$middle_usage:$phys_usage\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$write_pend:$clean_usage:$middle_usage:$phys_usage);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/NODE-CACHE/)

    # CPU-Core
    ########################

    if ( $type =~ m/^CPU-CORE$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $ctl, my $name_core, my $sys ) = split( /,/, $line );

      if ( isdigit($ctl) == 0 ) {
        next;
      }

      my $name = "node" . $ctl . "_core" . $name_core;

      # Controller,Core,Usage(%)
      # 1,X,0
      # 0,X,0
      # if there is mor CPU cores than core is "Y" etc ...

      print "001 CPU $name - $sys\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cpuc_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cpuc_name)
      if ( $found > -1 ) {
        $ltime = $cpuc_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cpuc_name[$l_count] = $rrd;
        $cpuc_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cpuc_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($sys) || $sys eq '' ) { $sys = 'U'; }
        print "004 CPU $name - $sys\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$sys:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CPU-CORE$/)

  }    # foreach

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_hus_perf

###################################################
#  DS8K section
###################################################

sub load_data_ds8k_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;
  my $perf_string = "perf";

  my @files_unsorted = grep( /$host\.$perf_string\.20.*/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_ds8k( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_ds8k_all

sub load_data_ds8k {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my @rank_capacity_tot  = "";
  my @rank_capacity_used = "";
  my $counter_ins        = 0;
  my $last_rec           = "";
  my $rrd                = "";
  my $time               = 3600;
  my $time_last_ok       = "";
  my $t                  = "";
  my $type               = "NA";
  my $t1                 = "";
  my $t2                 = "";
  my $t3                 = "";
  my $t4                 = "";
  my $t5                 = "";
  my $t6                 = "";
  my $t7                 = "";
  my $t8                 = "";
  my $t9                 = "";
  my $t10                = "";
  my $t11                = "";
  my $t12                = "";
  my $act_time_u         = time();
  my $ltime              = localtime($act_time_u);
  my $cfg_load_data_ok   = 0;                        # it is updated when there is found any rank, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $pool_file_write   = 0;                             # true if any write there
  my $config_file_write = 0;                             # true if any write there
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD> 
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > 
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @rank_cfg_id         = "";
  my @pool_cfg_id         = "";
  my $rank_cfg_processing = 0;
  my $rank_cfg_indx       = 0;
  my @pool_name           = "";
  my @pool_name_id        = "";
  my $pool_name_indx      = 0;
  my $cfg_print           = 1;
  my $error_pprc_once     = 0;

  # pool fron-end data from volume data
  my @pool_read     = "";
  my @pool_write    = "";
  my @pool_io_read  = "";
  my @pool_io_write = "";
  my @pool_resp_r   = "";
  my @pool_resp_w   = "";
  my @pool_ids      = "";
  my $pool_time     = 0;
  my $pool_ltime    = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      $type = uc($type_temp);
      if ( $cfg_print == 1 ) {
        print FCFG "</pre></body></html>\n";
        close(FCFG);
      }
      $cfg_print = 0;
      next;
    } ## end if ( $line =~ m/Level Statistics/)

    if ( $cfg_print == 1 ) {

      # print config file
      print FCFG "$line\n";
    }

    if ( $line =~ m/Rank Level Configuration/ ) {
      $rank_cfg_processing = 1;
      next;
    }

    if ($rank_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $rank_cfg_processing = 0;

        # load capacity usage of ranks (once a day into *.rrc)
        load_rank_capacity( $st_type, $wrkdir, $host, "RANK", $act_time_u, $DEBUG, $act_time, \@rank_cfg_id, \@pool_cfg_id, \@rank_capacity_tot, \@rank_capacity_used );
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/---/ || $line =~ m/^Rank ID/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      ( my $rank_id_cfg_item, my $pool_id_cfg_item, $t3, my $rank_cap_tot, $t5, $t6, $t7, $t8, my $pool_name_cfg_item, my $rank_cap_used ) = split( /,/, $line );
      $pool_id_cfg_item =~ s/^.*\///;

      #$rank_id_cfg_item =~ s/^0x//;
      $rank_cfg_id[$rank_cfg_indx]        = $rank_id_cfg_item;
      $pool_cfg_id[$rank_cfg_indx]        = $pool_id_cfg_item;
      $rank_capacity_tot[$rank_cfg_indx]  = $rank_cap_tot;
      $rank_capacity_used[$rank_cfg_indx] = $rank_cap_used;

      #print "009 $rank_cfg_id[$rank_cfg_indx] $pool_cfg_id[$rank_cfg_indx]\n";
      #print "009 $rank_cfg_id[$rank_cfg_indx] $pool_cfg_id[$rank_cfg_indx]\n";

      # save actual pool cfg
      my $found = -1;
      foreach my $pool_item (@pool_name) {
        if ( $pool_item =~ m/^$pool_name_cfg_item$/ ) {
          $found = $pool_name_indx;
          last;
        }
      } ## end foreach my $pool_item (@pool_name)
      if ( $found == -1 ) {
        $pool_name[$pool_name_indx]    = $pool_name_cfg_item;
        $pool_name_id[$pool_name_indx] = $pool_id_cfg_item;
        $pool_name_indx++;
        my $pool_number = $pool_id_cfg_item;    # there is still P letter
        $pool_number =~ s/^P//;
        print FHW "$pool_number:$pool_name_cfg_item\n";
      } ## end if ( $found == -1 )

      $rank_cfg_indx++;
      next;
    } ## end if ($rank_cfg_processing)

    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/	//g;
      $line =~ s/Interval End:     //g;
      $line =~ s/GMT.*//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 2;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ m/^$/ ) {
      $type = "NA";      # clear out type when empty line
      if ( $pool_time > 0 ) {

        # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
        if ( $pool_time > $pool_ltime ) {
          my @none = "";
          $pool_ltime = pool_sum_insert(
            $type,  $wrkdir, $step,  $DEBUG, $host,  $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@none, \@none, \@none,
            \@none, \@none,  \@none, \@none, \@none, \@none,   \@none,    \@none,   \@none,     \@none,      \@none,      \@none,       \@none,         \@none,          \@none
          );
        } ## end if ( $pool_time > $pool_ltime)

        # reinitialize structures
        @pool_ids      = "";
        @pool_read     = "";
        @pool_write    = "";
        @pool_io_read  = "";
        @pool_io_write = "";
        @pool_resp_r   = "";
        @pool_resp_w   = "";
        $pool_time     = 0;
      } ## end if ( $pool_time > 0 )
      next;
    } ## end if ( $line =~ m/^$/ )

    if ( $line !~ m/^0x/ ) {

      # avoid other trash
      next;
    }
    $line =~ s/^0x//;

    # Port
    ########################

    if ( $type =~ m/PORT/ ) {
      print "000 $line\n" if $DEBUG == 2;

      # Port statistics
      ( my $name,     $t1,          my $io_rate,     my $data_rate,   $t2,           my $resp_t, $t3, $t4, my $read, my $write, $t5, $t6, $t7, $t8, $t9, $t10, $t11, $t12,
        my $pprc_rio, my $pprc_wio, my $pprc_data_r, my $pprc_data_w, my $pprc_rt_r, my $pprc_rt_w
      ) = split( /,/, $line );

      # 0x30,4.0,218.721154,6366.153846,29.106256,0.836330,0,0,0,0,0,0,0,0,0,0,0,0,0,45494,0,1324160,0,38048,208
      #Port ID,Speed (Gbps),I/O Rate,Data Rate,Avg Xfer Size,Avg Resp Time,FB Read I/Os,FB Write I/Os,FB KBs Read,FB KBs Written,FB Accum Read Time,FB Accum
      #Write Time,CKD Read I/Os,CKD Write I/Os,CKD KBs Read,CKD KBs Written,CKD Accum Read Time,CKD Accum Write Time,PPRC Receive I/Os,PPRC Send I/Os,PPRC
      #KBs Received,PPRC KBs Sent,PPRC Accum Recv Time,PPRC Accum Send Time,Interval Length

      print "001 $name - $io_rate - $data_rate - $resp_t - $read - $write\n" if $DEBUG == 2;

      # PPRC response times : PPRC Accum Recv Time,PPRC Accum Send Time
      if ( $pprc_rio != 0 ) {
        $pprc_rt_r = $pprc_rt_r / $pprc_rio * 1000;    # to have that in ms
      }
      else {
        $pprc_rt_r = 0;
      }
      if ( $pprc_wio != 0 ) {
        $pprc_rt_w = $pprc_rt_w / $pprc_wio * 1000;    # to have that in ms
      }
      else {
        $pprc_rt_w = 0;
      }

      $rrd = "$wrkdir/$host/$type/$name\.rrd";
      my $rrdp = "$wrkdir/$host/$type/$name\.rrp";     # PPRC DBs

      my $pprc_yes = 0;
      if ( isdigit($pprc_rio) && isdigit($pprc_wio) && isdigit($pprc_data_r) && isdigit($pprc_data_w) && isdigit($pprc_rt_r) && isdigit($pprc_rt_w) ) {
        $pprc_yes = $pprc_rio + $pprc_wio + $pprc_data_r + $pprc_data_w + $pprc_rt_r + $pprc_rt_w;
      }
      else {
        if ( $error_pprc_once == 0 ) {
          main::error( "$rrdp: PPRC data digit issue: $pprc_rio,$pprc_wio,$pprc_data_r,$pprc_data_w,$pprc_rt_r,$pprc_rt_w " . __FILE__ . ":" . __LINE__ );
          $error_pprc_once++;
        }
      } ## end else [ if ( isdigit($pprc_rio...))]

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      if ( $pprc_yes > 0 ) {

        # create PPRC data files only when any trafic is there
        $rrdp =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
        if ( create_rrd( $rrdp, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
          return 2;         # leave uit if any error during RRDTool file creation to keep data
        }
      } ## end if ( $pprc_yes > 0 )

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array

        # construction against crashing daemon Perl code when RRDTool error appears
        # this does not work well in old RRDTOool: $RRDp::error_mode = 'catch';
        # construction is not too costly as it runs once per each load
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 2;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        print "004: $time:$io_rate:$data_rate:$resp_t:$read:$write\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$data_rate:$resp_t:$read:$write);
        my $answer = RRDp::read;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
        if ( $pprc_yes > 0 ) {

          # go further only for ports where PPRC traffic has been identified and DB created, no matter is all is 0!
          print "004: $time:$pprc_rio:$pprc_wio:$pprc_data_r:$pprc_data_w:$pprc_rt_r:$pprc_rt_w\n" if $DEBUG == 3;
          RRDp::cmd qq(update "$rrdp" $time:$pprc_rio:$pprc_wio:$pprc_data_r:$pprc_data_w:$pprc_rt_r:$pprc_rt_w);
          my $answer = RRDp::read;
          $time_last_ok = $time;
        } ## end if ( $pprc_yes > 0 )
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/PORT/ )

    # RANK
    ########################

    if ( $type =~ m/RANK/ ) {
      print "100 $line\n" if $DEBUG == 2;

      # Rank statistics
      ( my $name, $t1, $t2, my $read_io, my $write_io, my $read, my $write, $t7, $t8, my $resp_t_r, my $resp_t_w ) = split( /,/, $line );

      #(my $name, $t1, $t2, my $t3, my $t4, my $t5, my $t6, $t7, $t8, my $resp_t_r, my $resp_t_w, my $read_io, my $write_io, my $read, my $write) = split(/,/,$line);
      #Rank ID,RAID Type,Num of Arrays,Read I/O Rate,Write I/O Rate,Read Data Rate,Write Data Rate,Avg Read Xfer Size,Avg Write Xfer Size,Avg Read Resp Time
      #,Avg Write Resp Time,Read I/Os,Write I/Os,KBs Read,KBs Written,Accum Read Time,Accum Write Time,Interval Length
      #0x0,RAID-0,1,25.454090,1.704508,1294.103506,980.834725,50.840690,575.435847,2.575195,18.664055,15247,1021,775168,587520,39264,19056,599

      if ( $pool_name_indx == 0 ) {

        # mean that cfg header has not been in the file
        # map pool id to ranks id only first time
        $pool_name_indx = 1;
        my @files    = <$wrkdir/$host/$type/*.rrd>;
        my $indx_tmp = 0;
        foreach my $filep (@files) {
          chomp($filep);
          $filep =~ s/.*\///;
          $filep =~ s/.rrd//;
          ( my $rank_id, my $pool_id ) = split( /-/, $filep );

          # print "001 $rank_id, $pool_id , $filep, $indx_tmp\n";
          $rank_cfg_id[$indx_tmp] = $rank_id;
          $pool_cfg_id[$indx_tmp] = $pool_id;
          $indx_tmp++;
        } ## end foreach my $filep (@files)
      } ## end if ( $pool_name_indx ==...)

      # find out pool id for the rank and place it into rrd file name
      $rank_cfg_indx = 0;
      foreach my $r_id (@rank_cfg_id) {
        if ( $r_id =~ m/^$name$/ ) {
          last;
        }
        $rank_cfg_indx++;
      } ## end foreach my $r_id (@rank_cfg_id)

      if ( !defined($name) || $name eq '' || !isdigit($name) || $name =~ m/\./ ) {
        next;
      }

      $rrd = "$wrkdir/$host/$type/$name-$pool_cfg_id[$rank_cfg_indx]\.rrd";

      if ( $pool_cfg_id[$rank_cfg_indx] !~ /P/ ) {
        next;    # some problem with pool name (does not contain P) --> leave it here
      }

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@rank_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@rank_name)
      if ( $found > -1 ) {
        $ltime = $rank_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $rank_name[$l_count] = $rrd;
        $rank_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
        print "005: $ltime - $l_count\n" if $DEBUG == 2;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $rank_time[$l_count] = $time;
        $counter_ins++;
        print "104: $time:$read_io:$write_io:$read:$write:$resp_t_r,$resp_t_w\n" if $DEBUG == 2;

        #my $io_rate = $read_io + $write_io;
        #my $data_rate = $read + $write;
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$read:$write:$resp_t_r:$resp_t_w);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/RANK/ )

    # Volume
    ########################

    if ( $type =~ m/VOLUME/ ) {
      print "000 $line\n" if $DEBUG == 2;

      #print "000 $line\n";
      # Port statistics
      my (
        $name, $io_rate, $data_rate, $t1, $resp_t, $t2, $t3, $r_cache_hit, $w_cache_hit, $readio_ns, $writeio_ns,
        $readio_se, $writeio_se, $t9,  $t10, $t11, $t12, $read, $write, $resp_t_r, $resp_t_w, $x1,  $x2,        $x3,         $x4,     $x5,      $x6,         $x7, $x8, $x9, $x10
        ,           $x11,        $x12, $x13, $x14, $x15, $x16,  $x17,   $x18,      $x19,      $x20, $read_io_b, $write_io_b, $read_b, $write_b, $resp_t_r_b, $resp_t_w_b,
        $x21,       $x22,        $x23, $x24, $pool_id
      ) = split( /,/, $line );

      # 0x0000,4.324503,1.907285,0.441041,0.128637,0.000000,100.000000,100.000000,100.000000,1432,1180,0,0,1432,1180,0,0,640.000000,
      #  512.000000,80,256,0,0,1,1180,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0.000000,0.000000,0,144,0,0,0,604,IBM.2107-75ANM81/P0
      # Volume ID,I/O Rate,Data Rate,Avg Xfer Size,Avg Resp Time,Delayed I/O Perc,Total Hit Perc,Read Hit Perc,Write Hit Perc,Non-seq Read I/Os,
      #  Non-seq Write I/Os,Seq Read I/Os,Seq Write I/Os,Non-seq Read Hits,Non-seq Write Hits,Seq Read Hits,Seq Write Hits,KBs Read,KBs Written,
      #  Accum Read Time,Accum Write Time,Non-seq Disk->Cache Ops,Seq Disk->Cache Ops,Cache->Disk Ops,NVS Allocs,Non-seq DFW I/Os,Seq DFW I/Os,
      #  NVS Delayed DFW I/Os,Cache Delayed I/Os,Rec Mode Read I/Os,Rec Mode Read Hits,CC/XRC Trks Read,CC/XRC Contam Writes,PPRC Trk Xfers,
      #  Quick Write Prom,CFW Read I/Os,CFW Write I/Os,CFW Read Hits,CFW Write Hits,Irreg Trk Acc,Irreg Trk Acc Hits,Backend Read Ops,
      #  Backend Write Ops,Backend KBs Read,Backend KBs Written,Backend Accum Read Time,Backend Accum Write Time,ICL Read I/Os,
      #  Cache Bypass Write I/Os,Backend Data Xfer Time,Interval Length,Pool ID

      my $read_io  = $readio_ns + $readio_se;
      my $write_io = $writeio_ns + $writeio_se;
      if ( $read != 0 ) {
        $read = $read / 1024;
      }
      if ( $write != 0 ) {
        $write = $write / 1024;
      }

      print "001 $name - $io_rate - $data_rate - $resp_t - $read - $write - $r_cache_hit - $w_cache_hit - $resp_t_r - $resp_t_w - $read_io - $write_io\n" if $DEBUG == 2;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      # read response time = Accum Read Time / ( Non-seq Read I/Os + Seq Read I/Os)
      # write response time = Accum Write Time / ( Non-seq Write I/Os + Seq Write I/Os )
      # fron-end
      my $pool_resp_t_r = $resp_t_r;    #save original value for pool usage
      if ( $read_io != 0 ) {
        $resp_t_r = $resp_t_r / $read_io;
      }
      else {
        $resp_t_r      = 0;
        $pool_resp_t_r = 0;
      }
      my $pool_resp_t_w = $resp_t_w;    #save original value for pool usage
      if ( $write_io != 0 ) {
        $resp_t_w = $resp_t_w / $write_io;
      }
      else {
        $pool_resp_t_r = 0;
        $resp_t_w      = 0;
      }

      # back end
      if ( $read_io_b != 0 ) {
        $resp_t_r_b = $resp_t_r_b / $read_io_b;
      }
      else {
        $resp_t_r_b = 0;
      }
      if ( $write_io_b != 0 ) {
        $resp_t_w_b = $resp_t_w_b / $write_io_b;
      }
      else {
        $resp_t_w_b = 0;
      }
      my $rrdc = "$wrkdir/$host/$type/$name\.rrc";    # response times and cache stats

      #create rrd db if necessary
      $rrd =~ s/"//g;                                 # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;                                     # leave uit if any error during RRDTool file creation to keep data
      }
      $rrdc =~ s/"//g;                                # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrdc, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;                                     # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 2;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        print "004: $time:$io_rate:$data_rate:$resp_t:$read:$write\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$data_rate:$resp_t:$read:$write);
        my $answer = RRDp::read;
        RRDp::cmd qq(update "$rrdc" $time:$r_cache_hit:$w_cache_hit:$resp_t_r:$resp_t_w:$read_io:$write_io:$read_b:$write_b:$read_io_b:$write_io_b:$resp_t_r_b:$resp_t_w_b);
        $answer       = RRDp::read;
        $time_last_ok = $time;

        if ( !$pool_id eq '' && $pool_id =~ m/\/P/ ) {

          # keep volume fron-end data summ per pool and save it at the end
          $pool_id =~ s/^.*\/P//;
          if ( isdigit($pool_id) == 1 ) {
            if ( !defined $pool_read[$pool_id]     || $pool_read[$pool_id] eq '' )     { $pool_read[$pool_id]     = 0; }
            if ( !defined $pool_write[$pool_id]    || $pool_write[$pool_id] eq '' )    { $pool_write[$pool_id]    = 0; }
            if ( !defined $pool_io_read[$pool_id]  || $pool_io_read[$pool_id] eq '' )  { $pool_io_read[$pool_id]  = 0; }
            if ( !defined $pool_io_write[$pool_id] || $pool_io_write[$pool_id] eq '' ) { $pool_io_write[$pool_id] = 0; }
            if ( !defined $pool_resp_r[$pool_id]   || $pool_resp_r[$pool_id] eq '' )   { $pool_resp_r[$pool_id]   = 0; }
            if ( !defined $pool_resp_w[$pool_id]   || $pool_resp_w[$pool_id] eq '' )   { $pool_resp_w[$pool_id]   = 0; }
            $pool_read[$pool_id]     += $read;
            $pool_write[$pool_id]    += $write;
            $pool_io_read[$pool_id]  += $read_io;
            $pool_io_write[$pool_id] += $write_io;
            $pool_resp_r[$pool_id]   += $pool_resp_t_r;    # do not multiple by $read_io, it is cumulative olready!
            $pool_resp_w[$pool_id]   += $pool_resp_t_w;    # do not multiple by $read_io, it is cumulative olready!
            $pool_time = $time;
            $pool_ids[$pool_id] = $pool_id;
          } ## end if ( isdigit($pool_id)...)
        } ## end if ( !$pool_id eq '' &&...)
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/VOLUME/)
  }    # foreach

  close(FHW);

  # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
  if ( $pool_time > $pool_ltime ) {
    my @none = "";
    pool_sum_insert(
      $type,  $wrkdir, $step,  $DEBUG, $host,  $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@none, \@none, \@none,
      \@none, \@none,  \@none, \@none, \@none, \@none,   \@none,    \@none,   \@none,     \@none,      \@none,      \@none,       \@none,         \@none,          \@none
    );
  } ## end if ( $pool_time > $pool_ltime)

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { lc $a cmp lc $b } @lines;

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write\n";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    last;
  } ## end foreach $line (@lines)

  # same as above for config.html
  if ( $cfg_load_data_ok == 1 ) {
    open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @lines = <FCFGR>;
    close(FCFGR);
    my $cfg_section = 0;
    foreach $line (@lines) {
      chomp($line);
      if ( $line =~ m/Port Level/ || $line =~ m/Rank Level/ || $line =~ m/Volume Level/ ) {

        # the file must contain all 3 sections
        $cfg_section++;
      }
    } ## end foreach $line (@lines)

    if ( $cfg_section == 3 ) {
      print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
    }
    else {
      if ( $cfg_section > 0 ) {
        main::error( "problem with config file " . __FILE__ . ":" . __LINE__ );
      }
      else {
        print "cfg not found  : no config in this data file\n" if $DEBUG;
      }
    } ## end else [ if ( $cfg_section == 3)]
    if ( -f "$config_file-tmp" ) {
      rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
    }
  } ## end if ( $cfg_load_data_ok...)

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_ds8k

sub create_rrd {
  my $rrd         = shift;
  my $start_time  = shift;
  my $step        = shift;
  my $DEBUG       = shift;
  my $host        = shift;
  my $no_time     = shift;
  my $act_time    = shift;
  my $type        = shift;
  my $st_type     = shift;
  my $step_new    = $step;
  my $no_time_new = $no_time;

  if ( $step_new == 60 && $st_type !~ m/^SWIZ$/ ) {

    # Only SVC/Storwize has tested 60 secs step (same limitation is even in load.sh
    $step_new = 300;
  }

  my $first_rra  = 1;      # 5min
  my $second_rra = 12;     # 1h
  my $third_rra  = 72;     # 5 h
  my $forth_rra  = 288;    # 1day
  if ( $step_new == 60 ) {
    $first_rra  = 1;       # 1min
    $second_rra = 60;      # 1h
    $third_rra  = 300;     # 5 h
    $forth_rra  = 1440;    # 1day
  } ## end if ( $step_new == 60 )

  $start_time = $start_time - 3600;    # there must be actual time due to further summing pools and ranks and dealing with NaN
  if ( not -f $rrd ) {
    load_retentions( $step, $act_time );    # load data retentions
    {
      $type =~ m/^PORT$/ && do {

        #print "0000 - $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd PORT: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^SWIZ$/ ) {
          if ( $rrd =~ m/rrp$/ ) {
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:b2b:GAUGE:$no_time_new:0:10000000000"
          "DS:crc:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/rrp$/ )
          else {
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:type:GAUGE:$no_time_new:0:10"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );

            # type : FC (0) / SAS (PCIe) (1) / iSCSI (2)
          } ## end else [ if ( $rrd =~ m/rrp$/ )]
        } ## end if ( $st_type =~ m/^SWIZ$/)

        if ( $st_type =~ m/^NETAPP$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read_packet:GAUGE:$no_time_new:0:10000000000"
          "DS:write_packet:GAUGE:$no_time_new:0:10000000000"
          "DS:packet_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_error:GAUGE:$no_time_new:0:10000000000"
          "DS:write_error:GAUGE:$no_time_new:0:10000000000"
          "DS:type:GAUGE:$no_time_new:0:20"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

          # type : FC (1)
        } ## end if ( $st_type =~ m/^NETAPP$/)

        # old probably back-end ports
        #if ( $st_type =~ m/^NETAPP$/ ) {
        #  RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        #    "DS:write_io:GAUGE:$no_time_new:0:100000000"
        #    "DS:read_io:GAUGE:$no_time_new:0:100000000"
        #    "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        #    "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
        #    "DS:read:GAUGE:$no_time_new:0:10000000000"
        #    "DS:write:GAUGE:$no_time_new:0:10000000000"
        #    "DS:type:GAUGE:$no_time_new:0:10"
        #    "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        #    "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        #    "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        #    "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        #  );
        #  # type : FC (1)
        #}

        if ( $st_type =~ m/^3PAR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:type:GAUGE:$no_time_new:0:10"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

          # type : FC (0) / SAS (1) / iSCSI (2) / FCoE (5) / RCIP (6)
        } ## end if ( $st_type =~ m/^3PAR$/)
        if ( $st_type =~ m/^DS8K$/ ) {
          if ( $rrd =~ m/rrp$/ ) {

            # PPRC stats --> *.rrp
            # must be ABSOLUTE data type here for all
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:pprc_rio:ABSOLUTE:$no_time_new:0:10000000000"
          "DS:pprc_wio:ABSOLUTE:$no_time_new:0:10000000000"
          "DS:pprc_data_r:ABSOLUTE:$no_time_new:0:10000000000"
          "DS:pprc_data_w:ABSOLUTE:$no_time_new:0:10000000000"
          "DS:pprc_rt_r:ABSOLUTE:$no_time_new:0:10000000"
          "DS:pprc_rt_w:ABSOLUTE:$no_time_new:0:10000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/rrp$/ )
          else {
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
         );
          } ## end else [ if ( $rrd =~ m/rrp$/ )]
        } ## end if ( $st_type =~ m/^DS8K$/)

        if ( $st_type =~ m/^HUS$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:write_io:GAUGE:$no_time_new:0:100000000"
        "DS:read_io:GAUGE:$no_time_new:0:100000000"
        "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
        "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
        "DS:read:GAUGE:$no_time_new:0:100000000"
        "DS:write:GAUGE:$no_time_new:0:100000000"
        "DS:data_rate:GAUGE:$no_time_new:0:100000000"
        "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^HUS$/)
        if ( $st_type =~ m/^DOTHILL$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:write_io:GAUGE:$no_time_new:0:100000000"
        "DS:read_io:GAUGE:$no_time_new:0:100000000"
        "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        "DS:read:GAUGE:$no_time_new:0:100000000"
        "DS:write:GAUGE:$no_time_new:0:100000000"
        "DS:data_rate:GAUGE:$no_time_new:0:100000000"
        "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
        "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
        "DS:resp_t:GAUGE:$no_time_new:0:100000"
        "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

        } ## end if ( $st_type =~ m/^DOTHILL$/)
        if ( $st_type =~ m/^COMPELLENT$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:write_io:GAUGE:$no_time_new:0:100000000"
        "DS:read_io:GAUGE:$no_time_new:0:100000000"
        "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        "DS:read:GAUGE:$no_time_new:0:100000000"
        "DS:write:GAUGE:$no_time_new:0:100000000"
        "DS:data_rate:GAUGE:$no_time_new:0:100000000"
        "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
        "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
        "DS:resp_t:GAUGE:$no_time_new:0:100000"
        "DS:type:GAUGE:$no_time_new:0:10"
        "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

        } ## end if ( $st_type =~ m/^COMPELLENT$/)
        if ( $st_type =~ m/^OCEANSTOR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:write_io:GAUGE:$no_time_new:0:100000000"
        "DS:read_io:GAUGE:$no_time_new:0:100000000"
        "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        "DS:read:GAUGE:$no_time_new:0:100000000"
        "DS:write:GAUGE:$no_time_new:0:100000000"
        "DS:data_rate:GAUGE:$no_time_new:0:100000000"
        "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
        "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
        "DS:resp_t:GAUGE:$no_time_new:0:100000"
        "DS:type:GAUGE:$no_time_new:0:10"
        "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

        } ## end if ( $st_type =~ m/^OCEANSTOR$/)
        if ( $st_type =~ m/^VNXF$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_packet:GAUGE:$no_time_new:0:100000000"
          "DS:read_error:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write_packet:GAUGE:$no_time_new:0:100000000"
          "DS:write_error:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:packet_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:server_id:GAUGE:$no_time_new:0:1000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXF$/)
        if ( $st_type =~ m/^VSPG$/ ) {
          if ( $rrd =~ m/rrd$/ ) {
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/rrd$/ )
          else {    ### initiator
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate_ini:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate_ini:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_ini:GAUGE:$no_time_new:0:100000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end else [ if ( $rrd =~ m/rrd$/ )]
        } ## end if ( $st_type =~ m/^VSPG$/)
        if ( $st_type =~ m/^VNX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:COUNTER:$no_time_new:0:2000000"
          "DS:read_io:COUNTER:$no_time_new:0:1000000"
          "DS:write_io:COUNTER:$no_time_new:0:1000000"
          "DS:data_rate:COUNTER:$no_time_new:0:2147483648"
          "DS:read:COUNTER:$no_time_new:0:1073741824"
          "DS:write:COUNTER:$no_time_new:0:1073741824"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

          # MAX limits to update:
          # IO (read, write)   -> 1000000
          # DATA (read, write) -> 1073741824 (1 TB = 1073741824 kB)
          # CAPACITY           -> 1048576000 (1000 TB = 1048576000 MB)
        } ## end if ( $st_type =~ m/^VNX$/)
        if ( $st_type =~ m/^VMAX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:busy:GAUGE:$no_time_new:0:100"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VMAX$/)
        if ( $st_type =~ m/^VPLEX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VPLEX$/)
        if ( $st_type =~ m/UNITY/ ) {
          if ( $rrd =~ m/_fc[0-9]+\.rrd$/ || $rrd =~ m/iscsinode_/ ) {    # FC ports (fibreChannel) spb_fc4.rrd and iSCSI ports iscsinode_spb_eth3
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:io_rate:GAUGE:$no_time_new:0:100000000"
            "DS:read_io:GAUGE:$no_time_new:0:100000000"
            "DS:write_io:GAUGE:$no_time_new:0:100000000"
            "DS:data_rate:GAUGE:$no_time_new:0:100000000"
            "DS:read:GAUGE:$no_time_new:0:100000000"
            "DS:write:GAUGE:$no_time_new:0:100000000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/_fc[0-9]+\.rrd$/...)
          if ( $rrd =~ m/_eth[0-9]+\.rrd$/ && $rrd !~ m/iscsinode_/ ) {    # ETHERNET ports spb_eth3.rrd
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:packet_rate:GAUGE:$no_time_new:0:100000000"
            "DS:read_packet:GAUGE:$no_time_new:0:100000000"
            "DS:write_packet:GAUGE:$no_time_new:0:100000000"
            "DS:data_rate:GAUGE:$no_time_new:0:100000000"
            "DS:read:GAUGE:$no_time_new:0:100000000"
            "DS:write:GAUGE:$no_time_new:0:100000000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/_eth[0-9]+\.rrd$/...)
        } ## end if ( $st_type =~ m/UNITY/)
        if ( $st_type =~ m/VNXE$/ ) {
          if ( $rrd =~ m/_fc[0-9]+\.rrd$/ || $rrd =~ m/iscsinode_/ ) {    # FC ports (fibreChannel) spb_fc4.rrd and iSCSI ports iscsinode_spb_eth3
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:io_rate:GAUGE:$no_time_new:0:100000000"
            "DS:read_io:GAUGE:$no_time_new:0:100000000"
            "DS:write_io:GAUGE:$no_time_new:0:100000000"
            "DS:data_rate:GAUGE:$no_time_new:0:100000000"
            "DS:read:GAUGE:$no_time_new:0:100000000"
            "DS:write:GAUGE:$no_time_new:0:100000000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/_fc[0-9]+\.rrd$/...)
          if ( $rrd =~ m/_eth[0-9]+\.rrd$/ && $rrd !~ m/iscsinode_/ ) {    # ETHERNET ports spb_eth3.rrd
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:packet_rate:GAUGE:$no_time_new:0:100000000"
            "DS:read_packet:GAUGE:$no_time_new:0:100000000"
            "DS:write_packet:GAUGE:$no_time_new:0:100000000"
            "DS:data_rate:GAUGE:$no_time_new:0:100000000"
            "DS:read:GAUGE:$no_time_new:0:100000000"
            "DS:write:GAUGE:$no_time_new:0:100000000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/_eth[0-9]+\.rrd$/...)
        } ## end if ( $st_type =~ m/VNXE$/)
        last;
      };

      $type =~ m/^QTREE$/ && do {

        #print "0000 - $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd QTREE : $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^NETAPP$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:cifs_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_ops:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^NETAPP$/)
        last;
      };

      $type =~ m/^LUN$/ && do {

        #print "0000 - $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd LUN : $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^NETAPP$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^NETAPP$/)
        if ( $st_type =~ m/^VSPG$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate_b:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate_b:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VSPG$/)
        last;
      };

      $type =~ m/^NODE$/ && do {

        #print "0000 - $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd PORT: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^NETAPP$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:total_cpu_busy:GAUGE:$no_time_new:0:10000"
          "DS:cpu_busy:GAUGE:$no_time_new:0:10000"
          "DS:cifs_resp_t:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_io_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_read_io:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_write_io:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_connects:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_est_ses:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_ofiles:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_path_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_pers_opn:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_read:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_write:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:3_io_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:3_resp_t_r:GAUGE:$no_time_new:0:10000000000"
          "DS:3_read_io:GAUGE:$no_time_new:0:10000000000"
          "DS:3_resp_t_w:GAUGE:$no_time_new:0:10000000000"
          "DS:3_write_io:GAUGE:$no_time_new:0:10000000000"
          "DS:3_resp_t:GAUGE:$no_time_new:0:10000000000"
          "DS:3_data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:3_read:GAUGE:$no_time_new:0:10000000000"
          "DS:3_write:GAUGE:$no_time_new:0:10000000000"
          "DS:3_link_resp_t:GAUGE:$no_time_new:0:10000000000"
          "DS:4_calls:GAUGE:$no_time_new:0:10000000000"
          "DS:4_io_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:4_resp_t:GAUGE:$no_time_new:0:10000000000"
          "DS:4_read_io:GAUGE:$no_time_new:0:10000000000"
          "DS:4_resp_t_r:GAUGE:$no_time_new:0:10000000000"
          "DS:4_write_io:GAUGE:$no_time_new:0:10000000000"
          "DS:4_resp_t_w:GAUGE:$no_time_new:0:10000000000"
          "DS:4_data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:4_read:GAUGE:$no_time_new:0:10000000000"
          "DS:4_write:GAUGE:$no_time_new:0:10000000000"
          "DS:4_link_resp_t:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_read_io:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_write_io:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_io_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_read:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_write:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_resp_t_r:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_resp_t_w:GAUGE:$no_time_new:0:10000000000"
          "DS:iscsi_resp_t:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_read_io:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_write_io:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_io_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_read:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_write:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_resp_t_r:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_resp_t_w:GAUGE:$no_time_new:0:10000000000"
          "DS:fcp_resp_t:GAUGE:$no_time_new:0:10000000000"
          "DS:wafl_exempt:GAUGE:$no_time_new:0:100000"
          "DS:kahuna:GAUGE:$no_time_new:0:100000"
          "DS:exempt:GAUGE:$no_time_new:0:100000"
          "DS:hostOS:GAUGE:$no_time_new:0:100000"
          "DS:nwk_legacy:GAUGE:$no_time_new:0:100000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^NETAPP$/)
        if ( $st_type =~ m/^DOTHILL$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DOTHILL$/)
        if ( $st_type =~ m/^COMPELLENT$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:sys:GAUGE:$no_time_new:0:10000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^COMPELLENT$/)
        if ( $st_type =~ m/^OCEANSTOR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:r_cache_usage:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_usage:GAUGE:$no_time_new:0:1000"
          "DS:cache_usage:GAUGE:$no_time_new:0:1000"
          "DS:sys:GAUGE:$no_time_new:0:10000000"
          "DS:queue:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^OCEANSTOR$/)
        if ( $st_type =~ m/^VNXF$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:calls_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_calls:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write_calls:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXF$/)
        if ( $st_type =~ m/^3PAR$/ && $rrd =~ m/rrd$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:r_io_acc:GAUGE:$no_time_new:0:100000000"
          "DS:w_io_acc:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit_io:GAUGE:$no_time_new:0:10000000000"
          "DS:w_cache_hit_io:GAUGE:$no_time_new:0:10000000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:10000000000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:10000000000"
          "DS:r_lock_block:GAUGE:$no_time_new:0:10000000000"
          "DS:w_lock_block:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

        } ## end if ( $st_type =~ m/^3PAR$/...)
        if ( $st_type =~ m/^3PAR$/ && $rrd =~ m/rrt$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:cfc_dirty:GAUGE:$no_time_new:0:100000000"
          "DS:cfc_max:GAUGE:$no_time_new:0:100000000"
          "DS:del_ack:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

        } ## end if ( $st_type =~ m/^3PAR$/...)
        if ( $st_type =~ m/UNITY/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:sys:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_read:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_write:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_avg_read:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_avg_write:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_read_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_write_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_ops_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_read:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_write:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_avg_read:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_avg_write:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_read_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_write_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_ops_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cache_d_data:GAUGE:$no_time_new:0:10000000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:10000000000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:10000000000"
          "DS:r_cache_misses:GAUGE:$no_time_new:0:10000000000"
          "DS:w_cache_misses:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/UNITY/)
        if ( $st_type =~ m/VNXE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:sys:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_read:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_write:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_avg_read:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_avg_write:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_read_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_write_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:nfs_ops_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_read:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_write:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_avg_read:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_avg_write:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_read_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_write_ops:GAUGE:$no_time_new:0:10000000000"
          "DS:cifs_ops_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cache_d_data:GAUGE:$no_time_new:0:10000000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:10000000000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:10000000000"
          "DS:r_cache_misses:GAUGE:$no_time_new:0:10000000000"
          "DS:w_cache_misses:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/VNXE$/)
        if ( $st_type =~ m/^VNX$/ ) {

          #RRDp::cmd qq(update "$rrd" $time:$idle_prct:$busy_prct:$idle_ticks:$busy_ticks:$read_io:$write_io:$arr_queue:$sum_queue:$read:$write:$io_rate:$data_rate:$write_cache_blocks_flushed:$prct_dirty_cache_pages:$prct_cache_pages_owned);
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:idle_prct:GAUGE:$no_time_new:0:200"
          "DS:busy_prct:GAUGE:$no_time_new:0:200"
          "DS:idle_ticks:COUNTER:$no_time_new:0:1048576000"
          "DS:busy_ticks:COUNTER:$no_time_new:0:1048576000"
          "DS:read_io:COUNTER:$no_time_new:0:1000000"
          "DS:write_io:COUNTER:$no_time_new:0:1000000"
          "DS:arr_queue:COUNTER:$no_time_new:0:1048576000"
          "DS:sum_queue:COUNTER:$no_time_new:0:1048576000"
          "DS:read:COUNTER:$no_time_new:0:1073741824"
          "DS:write:COUNTER:$no_time_new:0:1073741824"
          "DS:io_rate:COUNTER:$no_time_new:0:1000000"
          "DS:data_rate:COUNTER:$no_time_new:0:1073741824"
          "DS:w_cache_flushed:COUNTER:$no_time_new:0:1073741824"
          "DS:dirty_cache_pages:GAUGE:$no_time_new:0:200"
          "DS:cache_pages_owned:GAUGE:$no_time_new:0:200"
          "DS:reserve1:COUNTER:$no_time_new:0:1073741824"
          "DS:reserve2:COUNTER:$no_time_new:0:1073741824"
          "DS:reserve3:GAUGE:$no_time_new:0:1073741824"
          "DS:reserve4:GAUGE:$no_time_new:0:1073741824"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNX$/)
        if ( $st_type =~ m/^VMAX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:busy:GAUGE:$no_time_new:0:200"
          "DS:read_io:GAUGE:$no_time_new:0:1000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VMAX$/)
        if ( $st_type =~ m/^VPLEX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:busy:GAUGE:$no_time_new:0:200"
          "DS:io_rate_b:GAUGE:$no_time_new:0:1000000"
          "DS:read_io_b:GAUGE:$no_time_new:0:1000000"
          "DS:write_io_b:GAUGE:$no_time_new:0:1000000"
          "DS:read_b:GAUGE:$no_time_new:0:10000000000"
          "DS:write_b:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000"
          "DS:read_io:GAUGE:$no_time_new:0:1000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate_b:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VPLEX$/)
        last;
      };

      $type =~ m/^FS$/ && do {
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd FS: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^VNXF$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:server_id:GAUGE:$no_time_new:0:1000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

        } ## end if ( $st_type =~ m/^VNXF$/)
        if ( $st_type =~ m/^UNITY$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_free:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^UNITY$/)
        if ( $st_type =~ m/^OCEANSTOR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_ops:GAUGE:$no_time_new:0:100000000"
          "DS:write_ops:GAUGE:$no_time_new:0:100000000"
          "DS:ops_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_free:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^OCEANSTOR$/)
        if ( $st_type =~ m/^VNXE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_free:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXE$/)
        last;
      };

      $type =~ m/^NAS$/ && do {
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd NAS: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^UNITY$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_free:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^UNITY$/)
        if ( $st_type =~ m/^VNXE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_free:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXE$/)
        if ( $st_type =~ m/^NETAPP$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_real:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^NETAPP$/)
        last;
      };

      $type =~ m/^NASPOOL$/ && do {
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd NAS: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^UNITY$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_free:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^UNITY$/)
        if ( $st_type =~ m/^VNXE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_free:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXE$/)
        last;
      };
      $type =~ m/^HOST_GROUP$/ && do {
        #print "0000 - $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd HOST_GROUP: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        if ( $st_type =~ m/^PURE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^PURE$/)
        last;
      };

      $type =~ m/^HOST$/ && do {

        #print "0000 - $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd HOST: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^VMAX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        "DS:read:GAUGE:$no_time_new:0:100000000"
        "DS:write:GAUGE:$no_time_new:0:100000000"
        "DS:read_io:GAUGE:$no_time_new:0:100000000"
        "DS:resp_t:GAUGE:$no_time_new:0:100000"
        "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
        "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
        "DS:write_io:GAUGE:$no_time_new:0:100000000"
        "DS:data_rate:GAUGE:$no_time_new:0:100000000"
        "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VMAX$/)
        if ( $st_type =~ m/^3PAR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:read_io:GAUGE:$no_time_new:0:100000000"
        "DS:write_io:GAUGE:$no_time_new:0:100000000"
        "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        "DS:read:GAUGE:$no_time_new:0:100000000"
        "DS:write:GAUGE:$no_time_new:0:100000000"
        "DS:data_rate:GAUGE:$no_time_new:0:100000000"
        "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
        "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
        "DS:resp_t:GAUGE:$no_time_new:0:100000"
        "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^3PAR$/)
        if ( $st_type =~ m/^COMPELLENT$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_xfer:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_qos:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^COMPELLENT$/)
        if ( $st_type =~ m/^OCEANSTOR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^OCEANSTOR$/)
        if ( $st_type =~ m/^PURE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^PURE$/)
        if ( $st_type =~ m/^VSPG$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        "DS:data_rate:GAUGE:$no_time_new:0:100000000"
        "DS:resp_t:GAUGE:$no_time_new:0:100000"
        "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VSPG$/)
        last;
      };

      $type =~ m/^RANK$/ && do {

        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd RANK: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        if ( rename_only( $rrd, $host, $DEBUG ) == 1 ) {
          return 0;    # rank already exist under different pool ID, it has been renamed
        }
        touch();

        if ( $st_type =~ m/^HUS$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:used:GAUGE:$no_time_new:0:10000000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^HUS$/)

        if ( $st_type =~ m/^VSPG$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:data_rate_b:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate_b:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VSPG$/)
        #if ( $st_type =~ m/^PURE$/ ) {
        #  RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        #  "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        #  "DS:read_io:GAUGE:$no_time_new:0:100000000"
        #  "DS:write_io:GAUGE:$no_time_new:0:100000000"
        #  "DS:read:GAUGE:$no_time_new:0:100000000"
        #  "DS:write:GAUGE:$no_time_new:0:100000000"
        #  "DS:data_rate:GAUGE:$no_time_new:0:100000000"
        #  "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
        #  "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
        #  "DS:resp_t:GAUGE:$no_time_new:0:100000000"
        #  "DS:cap:GAUGE:$no_time_new:0:10000000000"
        #  "DS:queue:GAUGE:$no_time_new:0:100000000"
        #  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        #  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        #  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        #  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        #);
        #} ## end if ( $st_type =~ m/^PURE$/)

        if ( $st_type =~ m/^DOTHILL$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DOTHILL$/)

        if ( $st_type =~ m/^3PAR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^3PAR$/)
        if ( $st_type =~ m/^DS5K$/ || $st_type =~ m/^SWIZ$/ || $st_type =~ m/^DS8K$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DS5K$/...)
        last;
      };

      $type =~ m/^DRIVE$/ && do {
        print "002 $start_time - $step - $rrd \n" if $DEBUG == 2;
        print "create_rrd DRIV: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^3PAR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000"
          "DS:type:GAUGE:$no_time_new:0:10"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

          # type: 1:FC 2:SSD 3:NL 4:SAS 5:unknown
        } ## end if ( $st_type =~ m/^3PAR$/)

        if ( $st_type =~ m/^HUS$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:operating_rate:GAUGE:$no_time_new:0:1000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^HUS$/)
        if ( $st_type =~ m/^UNITY$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^UNITY$/)
        if ( $st_type =~ m/^VMAX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:prct_busy:GAUGE:$no_time_new:0:100"
          "DS:prct_cap_free:GAUGE:$no_time_new:0:100"
          "DS:prct_cap_used:GAUGE:$no_time_new:0:100"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
          "DS:cap_used:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VMAX$/)
        if ( $st_type =~ m/^VNXE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXE$/)
        if ( $st_type =~ m/^VNX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:COUNTER:$no_time_new:0:200000"
          "DS:read_io:COUNTER:$no_time_new:0:100000"
          "DS:write_io:COUNTER:$no_time_new:0:100000"
          "DS:data_rate:COUNTER:$no_time_new:0:20971520"
          "DS:read:COUNTER:$no_time_new:0:10485760"
          "DS:write:COUNTER:$no_time_new:0:10485760"
          "DS:queue:COUNTER:$no_time_new:0:1048576000"
          "DS:prct_idle:GAUGE:$no_time_new:0:200"
          "DS:prct_busy:GAUGE:$no_time_new:0:200"
          "DS:idle_ticks:COUNTER:$no_time_new:0:1048576000"
          "DS:busy_ticks:COUNTER:$no_time_new:0:1048576000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

          # MAX limits to update:
          # IO (read, write)   -> 100000
          # DATA (read, write) -> 1073741824 (1 TB = 1073741824 kB) --old
          # DATA (read, write) -> 10485760 (10 GB = 10485760 kB)
          # CAPACITY           -> 1048576000 (1000 TB = 1048576000 MB)
        } ## end if ( $st_type =~ m/^VNX$/)
        if ( $st_type =~ m/^DOTHILL$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:io_rate:GAUGE:$no_time_new:0:1000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DOTHILL$/)
        if ( $st_type =~ m/^COMPELLENT$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:io_pending:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^COMPELLENT$/)
        if ( $st_type =~ m/^OCEANSTOR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "DS:queue:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^OCEANSTOR$/)
        if ( $st_type ne "3PAR" && $st_type ne "HUS" && $st_type ne "VNX" && $st_type ne "UNITY" && $st_type ne "VNXE" && $st_type ne "DOTHILL" && $st_type ne "COMPELLENT" && $st_type ne "VMAX" && $st_type ne "OCEANSTOR" ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:1000000000"
          "DS:write_io:GAUGE:$no_time_new:0:1000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type ne "3PAR"...)
        last;
      };

      $type =~ m/^VOLUMEcapcust$/ && do {

        # Pool custom capacity stats
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd Vcap: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        if ( $st_type =~ m/^DS8K$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_phys:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_real:GAUGE:$no_time_new:0:10000000000"
          "DS:vol_ext:GAUGE:$no_time_new:0:10000000000"
          "DS:ext_size:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DS8K$/)
        last;
      };

      $type =~ m/^POOLcapcust$/ && do {

        # Pool custom capacity stats
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd Pcap: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        if ( $st_type =~ m/^DS8K$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:cap_phys:GAUGE:$no_time_new:0:10000000000"
          "DS:ext_phys:GAUGE:$no_time_new:0:10000000000"
          "DS:ext_size:GAUGE:$no_time_new:0:10000000000"
          "DS:ext_num:GAUGE:$no_time_new:0:10000000000"
          "DS:ext_used:GAUGE:$no_time_new:0:10000000000"
          "DS:free_2_30:GAUGE:$no_time_new:0:10000000000"
          "DS:free:GAUGE:$no_time_new:0:10000000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DS8K$/)
        last;
      };

      $type =~ m/^TIER$/ && do {

        # Pool front-end stats
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd TIER: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        if ( $st_type =~ m/^3PAR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:read_size:GAUGE:$no_time_new:0:10000000000"
          "DS:write_size:GAUGE:$no_time_new:0:10000000000"
          "DS:data_size:GAUGE:$no_time_new:0:10000000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^3PAR$/)
        last;
      };

      $type =~ m/^SLO$/ && do {

        # SLO (Service level objectives) front-end stats
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd SLO: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        #RRDp::cmd qq(update "$rrd" $time:$cap:$io_rate:$read:$write:$read_io:$write_io:$cache_hit:$r_cache_hit:$w_cache_hit:$resp_t:$data_rate);
        if ( $st_type =~ m/^VMAX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VMAX$/)
        last;
      };

      $type =~ m/^VOLUME-FOLDER$/ && do {

        # VOLUME FOLDER front-end stats
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd VOLUME FOLDER: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        if ( $st_type =~ m/^COMPELLENT$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_xfer:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_qos:GAUGE:$no_time_new:0:100000000"
          "DS:io_pending:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
       } ## end if ( $st_type =~ m/^COMPELLENT$/)
       last;
      };
      $type =~ m/^IO-GROUP$/ && do {

        # IO-GROUP front-end stats
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd IO-GROUP: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^SWIZ$/ ) {
          # Storwize only
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:read:GAUGE:$no_time_new:0:10000000000"
            "DS:write:GAUGE:$no_time_new:0:10000000000"
            "DS:read_io:GAUGE:$no_time_new:0:100000000"
            "DS:write_io:GAUGE:$no_time_new:0:100000000"
            "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
            "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
            "DS:r_transfer_size:GAUGE:$no_time_new:0:10000000000"
            "DS:w_transfer_size:GAUGE:$no_time_new:0:10000000000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
        } ## end if ( $st_type =~ m/^SWIZ$/)
        last;
      };

      $type =~ m/^POOL$/ && do {

        # Pool front-end stats
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd POOL: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        if ( $st_type =~ m/^NETAPP$/ || $st_type =~ m/^XIV/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^NETAPP$/...)

        if ( $st_type =~ m/^3PAR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
	      "DS:read:GAUGE:$no_time_new:0:10000000000"
	      "DS:write:GAUGE:$no_time_new:0:10000000000"
	      "DS:read_io:GAUGE:$no_time_new:0:100000000"
	      "DS:write_io:GAUGE:$no_time_new:0:100000000"
	      "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
	      "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
	      "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
	      "DS:io_rate:GAUGE:$no_time_new:0:100000000"
	      "DS:resp_t:GAUGE:$no_time_new:0:1000000"
	      "DS:real:GAUGE:$no_time_new:0:100000000"
	      "DS:free:GAUGE:$no_time_new:0:100000000"
	      "DS:cap:GAUGE:$no_time_new:0:100000000"
	      "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	      "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	      "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	      "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
	    );
        } ## end if ( $st_type =~ m/^3PAR$/)

        #"DS:read_size:GAUGE:$no_time_new:0:10000000000"
        #"DS:write_size:GAUGE:$no_time_new:0:10000000000"
        #"DS:data_size:GAUGE:$no_time_new:0:10000000000"

        # netapp aggregates back-end
        #if ( $st_type =~ m/^NETAPP$/ ) {
        #  RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        #    "DS:io_rate:GAUGE:$no_time_new:0:100000000"
        #    "DS:read:GAUGE:$no_time_new:0:10000000000"
        #    "DS:write:GAUGE:$no_time_new:0:10000000000"
        #    "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
        #    "DS:read_io:GAUGE:$no_time_new:0:100000000"
        #    "DS:write_io:GAUGE:$no_time_new:0:100000000"
        #    "DS:real:GAUGE:$no_time_new:0:100000000"
        #    "DS:free:GAUGE:$no_time_new:0:100000000"
        #    "DS:read_hdd:GAUGE:$no_time_new:0:10000000000"
        #    "DS:read_sdd:GAUGE:$no_time_new:0:10000000000"
        #    "DS:write_hdd:GAUGE:$no_time_new:0:10000000000"
        #    "DS:write_sdd:GAUGE:$no_time_new:0:10000000000"
        #    "DS:read_io_hdd:GAUGE:$no_time_new:0:100000000"
        #    "DS:read_io_sdd:GAUGE:$no_time_new:0:100000000"
        #    "DS:write_io_hdd:GAUGE:$no_time_new:0:100000000"
        #    "DS:write_io_sdd:GAUGE:$no_time_new:0:100000000"
        #    "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        #    "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        #    "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        #    "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        #  );
        #}

        if ( $st_type =~ m/^DOTHILL$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DOTHILL$/)
        if ( $st_type =~ m/^COMPELLENT$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_xfer:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_qos:GAUGE:$no_time_new:0:100000000"
          "DS:io_pending:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:snapshot:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^COMPELLENT$/)
        if ( $st_type =~ m/^OCEANSTOR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:tier0cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier0free:GAUGE:$no_time_new:0:10000000000"
          "DS:tier1cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier1free:GAUGE:$no_time_new:0:10000000000"
          "DS:tier2cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier2free:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^OCEANSTOR$/)
        if ( $st_type =~ m/^PURE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:queue:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:system:GAUGE:$no_time_new:0:100000000"
          "DS:volumes:GAUGE:$no_time_new:0:100000000"
          "DS:shared_space:GAUGE:$no_time_new:0:100000000"
          "DS:snapshots:GAUGE:$no_time_new:0:100000000"
          "DS:data_reduction:GAUGE:$no_time_new:0:10000"
          "DS:total_reduction:GAUGE:$no_time_new:0:10000"
          "DS:thin_provisioning:GAUGE:$no_time_new:0:10000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^PURE$/)
        if ( $st_type =~ m/^HUS$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^HUS$/)

        if ( $st_type =~ m/^VSPG$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:data_rate_b:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate_b:GAUGE:$no_time_new:0:100000000"
          "DS:tier0cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier0free:GAUGE:$no_time_new:0:10000000000"
          "DS:tier1cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier1free:GAUGE:$no_time_new:0:10000000000"
          "DS:tier2cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier2free:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VSPG$/)
        if ( $st_type =~ m/^VNXF$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:qdepth:GAUGE:$no_time_new:0:100000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:busy:GAUGE:$no_time_new:0:10000"
          "DS:server_id:GAUGE:$no_time_new:0:1000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXF$/)

        if ( $st_type =~ m/^DS8K$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read:ABSOLUTE:$no_time_new:0:10000000000"
          "DS:write:ABSOLUTE:$no_time_new:0:10000000000"
          "DS:read_io:ABSOLUTE:$no_time_new:0:100000000"
          "DS:write_io:ABSOLUTE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DS8K$/)

        if ( $st_type =~ m/^DS5K$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:real:GAUGE:$no_time_new:0:100000000"
          "DS:free:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DS5K$/)
        if ( $st_type =~ m/^VNX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:COUNTER:$no_time_new:0:20000000"
          "DS:read_io:COUNTER:$no_time_new:0:10000000"
          "DS:write_io:COUNTER:$no_time_new:0:10000000"
          "DS:data_rate:COUNTER:$no_time_new:0:2147483648"
          "DS:read:COUNTER:$no_time_new:0:1073741824"
          "DS:write:COUNTER:$no_time_new:0:1073741824"
          "DS:cap:GAUGE:$no_time_new:0:1048576000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );

          # MAX limits to update:
          # IO (read, write)   -> 10000000
          # DATA (read, write) -> 1073741824 (1 TB = 1073741824 kB)
          # RESP_T -> 1000000 (1000 sec = 1000000 ms)
        } ## end if ( $st_type =~ m/^VNX$/)
        if ( $st_type =~ m/^UNITY$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:1048576000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^UNITY$/)
        if ( $st_type =~ m/^VMAX$/ ) {
          if ( $rrd =~ m/array_total\.rra$/ ) {
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:read_io:GAUGE:$no_time_new:0:100000000"
            "DS:write:GAUGE:$no_time_new:0:10000000000"
            "DS:cache_hit:GAUGE:$no_time_new:0:1000"
            "DS:read:GAUGE:$no_time_new:0:10000000000"
            "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
            "DS:write_io:GAUGE:$no_time_new:0:100000000"
            "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
            "DS:io_rate:GAUGE:$no_time_new:0:100000000"
            "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/array_total\.rra$/)
          else {
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:read_b:GAUGE:$no_time_new:0:10000000000"
            "DS:data_rate_b:GAUGE:$no_time_new:0:10000000000"
            "DS:write_b:GAUGE:$no_time_new:0:10000000000"
            "DS:read_io_b:GAUGE:$no_time_new:0:100000000"
            "DS:write_io_b:GAUGE:$no_time_new:0:100000000"
            "DS:resp_t_b:GAUGE:$no_time_new:0:1000000"
            "DS:resp_t_r_b:GAUGE:$no_time_new:0:1000000"
            "DS:resp_t_w_b:GAUGE:$no_time_new:0:1000000"
            "DS:io_rate_b:GAUGE:$no_time_new:0:100000000"
            "DS:read:GAUGE:$no_time_new:0:10000000000"
            "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
            "DS:write:GAUGE:$no_time_new:0:10000000000"
            "DS:read_io:GAUGE:$no_time_new:0:100000000"
            "DS:write_io:GAUGE:$no_time_new:0:100000000"
            "DS:resp_t:GAUGE:$no_time_new:0:1000000"
            "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
            "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
            "DS:io_rate:GAUGE:$no_time_new:0:100000000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end else [ if ( $rrd =~ m/array_total\.rra$/)]
        } ## end if ( $st_type =~ m/^VMAX$/)
        if ( $st_type =~ m/^VPLEX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VPLEX$/)
        if ( $st_type =~ m/^VNXE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:1048576000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXE$/)
        if ( $st_type =~ m/^SWIZ$/ ) {

          # Storwize only
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^SWIZ$/)
        last;
      };
      $type =~ m/^VVOL$/ && do {    ### SECTION FOR 3PAR
                                    #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd VVOL : $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^3PAR$/ ) {

          # custom 3PAR sample rate for Volumes could be used when HP3PAR_VV_SAMPLE_RATE is set in environment, supported are 30 or 60 secs rates only (instead of default 300secs)
          my $par_vol                = $ENV{HP3PAR_VV_SAMPLE_RATE};
          my $five_mins_sample_3par  = $five_mins_sample;
          my $one_hour_sample_3par   = $one_hour_sample;
          my $five_hours_sample_3par = $five_hours_sample;
          my $one_day_sample_3par    = $one_day_sample;
          my $step_new_3par          = $step_new;
          my $hour_sample            = 12;
          my $hours_sample5          = 72;
          my $day_sample             = 288;
          if ( defined($par_vol) && !$par_vol eq '' && isdigit($par_vol) ) {

            if ( $par_vol > 300 || $par_vol < 30 ) {
              print "create_rrd VVOL : $host:$type $rrd ; HP3PAR_VV_SAMPLE_RATE is out of tested range ($par_vol), using original STEP=$step_new \n" if $DEBUG;
            }
            else {
              # all below is expecting 30secs, when is used 1 min then retentions will be twice longer
              print "create_rrd VVOL : $host:$type $rrd ; HP3PAR_VV_SAMPLE_RATE is used ($par_vol) instead of original STEP=$step_new \n" if $DEBUG;
              $step_new_3par          = $par_vol;
              $five_mins_sample_3par  = 86400;                # 30 days full for 30 secs
              $one_hour_sample_3par   = $five_mins_sample;    # here are 5 mins intervals
              $five_hours_sample_3par = $one_hour_sample;
              $one_day_sample_3par    = $one_day_sample;

              # (5hours is skipped)

              $hour_sample   = 10;                            # 5 mins
              $hours_sample5 = 120;                           # hour
              $day_sample    = 2880;                          # day
              if ( $par_vol == 60 ) {
                $hour_sample   = 5;                           # 5 mins
                $hours_sample5 = 60;                          # hour
                $day_sample    = 1440;                        # day
              }
            } ## end else [ if ( $par_vol > 300 ||...)]
          } ## end if ( defined($par_vol)...)
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new_3par"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_real:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample_3par"
          "RRA:AVERAGE:0.5:$hour_sample:$one_hour_sample_3par"
          "RRA:AVERAGE:0.5:$hours_sample5:$five_hours_sample_3par"
          "RRA:AVERAGE:0.5:$day_sample:$one_day_sample_3par"
        );
          if ( defined($par_vol) && !$par_vol eq '' && isdigit($par_vol) ) {

            # it has to be here as different params to create_check() are passed
            my $mode = 0644;
            chmod $mode, "$rrd";
            if ( !Xorux_lib::create_check("file: $rrd, $five_mins_sample_3par, $one_hour_sample_3par, $five_hours_sample_3par, $one_day_sample_3par") ) {
              main::error( "create_rrd err : unable to create $rrd (filesystem is full?) at " . __FILE__ . ": line " . __LINE__ );
              RRDp::end;
              RRDp::start "$rrdtool";
              return 1;
            } ## end if ( !Xorux_lib::create_check...)
            return 0;
          } ## end if ( defined($par_vol)...)
        } ## end if ( $st_type =~ m/^3PAR$/)
        last;
      };

      $type =~ m/^VOLUME$/ && do {

        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd VOL : $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^DS5K$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:controller:GAUGE:$no_time_new:0:10"
          "DS:io_total:ABSOLUTE:$no_time_new:0:10000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:read_pct:GAUGE:$no_time_new:0:1000"
          "DS:ssd_r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DS5K$/)
        if ( $st_type =~ m/^XIV$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:read_hits:GAUGE:$no_time_new:0:10000000000"
          "DS:write_hits:GAUGE:$no_time_new:0:10000000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^XIV$/)
        if ( $st_type =~ m/^SWIZ$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_real:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^SWIZ$/)

        if ( $st_type =~ m/^NETAPP$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_real:GAUGE:$no_time_new:0:10000000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^NETAPP$/)

        if ( $st_type =~ m/^3PAR$/ ) {

          # custom 3PAR sample rate for Volumes could be used when HP3PAR_VV_SAMPLE_RATE is set in environment, supported are 30 or 60 secs rates only (instead of default 300secs)
          my $par_vol                = $ENV{HP3PAR_VV_SAMPLE_RATE};
          my $five_mins_sample_3par  = $five_mins_sample;
          my $one_hour_sample_3par   = $one_hour_sample;
          my $five_hours_sample_3par = $five_hours_sample;
          my $one_day_sample_3par    = $one_day_sample;
          my $step_new_3par          = $step_new;
          my $hour_sample            = 12;
          my $hours_sample5          = 72;
          my $day_sample             = 288;
          if ( defined($par_vol) && !$par_vol eq '' && isdigit($par_vol) ) {

            if ( $par_vol > 300 || $par_vol < 30 ) {
              print "create_rrd VOL : $host:$type $rrd ; HP3PAR_VV_SAMPLE_RATE is out of tested range ($par_vol), using original STEP=$step_new \n" if $DEBUG;
            }
            else {
              # all below is expecting 30secs, when is used 1 min then retentions will be twice longer
              print "create_rrd VOL : $host:$type $rrd ; HP3PAR_VV_SAMPLE_RATE is used ($par_vol) instead of original STEP=$step_new \n" if $DEBUG;
              $step_new_3par          = $par_vol;
              $five_mins_sample_3par  = 86400;                # 30 days full for 30 secs
              $one_hour_sample_3par   = $five_mins_sample;    # here are 5 mins intervals
              $five_hours_sample_3par = $one_hour_sample;
              $one_day_sample_3par    = $one_day_sample;

              # (5hours is skipped)

              $hour_sample   = 10;                            # 5 mins
              $hours_sample5 = 120;                           # hour
              $day_sample    = 2880;                          # day
              if ( $par_vol == 60 ) {
                $hour_sample   = 5;                           # 5 mins
                $hours_sample5 = 60;                          # hour
                $day_sample    = 1440;                        # day
              }
            } ## end else [ if ( $par_vol > 300 ||...)]
          } ## end if ( defined($par_vol)...)
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new_3par"
		  "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
		  "DS:resp_t:GAUGE:$no_time_new:0:1000000"
		  "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
		  "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
		  "DS:read:GAUGE:$no_time_new:0:10000000000"
		  "DS:write:GAUGE:$no_time_new:0:10000000000"
		  "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_used:GAUGE:$no_time_new:0:10000000000"
          "DS:cap_real:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample_3par"
          "RRA:AVERAGE:0.5:$hour_sample:$one_hour_sample_3par"
          "RRA:AVERAGE:0.5:$hours_sample5:$five_hours_sample_3par"
          "RRA:AVERAGE:0.5:$day_sample:$one_day_sample_3par"
        );
          if ( defined($par_vol) && !$par_vol eq '' && isdigit($par_vol) ) {

            # it has to be here as different params to create_check() are passed
            my $mode = 0644;
            chmod $mode, "$rrd";
            if ( !Xorux_lib::create_check("file: $rrd, $five_mins_sample_3par, $one_hour_sample_3par, $five_hours_sample_3par, $one_day_sample_3par") ) {
              main::error( "create_rrd err : unable to create $rrd (filesystem is full?) at " . __FILE__ . ": line " . __LINE__ );
              RRDp::end;
              RRDp::start "$rrdtool";
              return 1;
            } ## end if ( !Xorux_lib::create_check...)
            return 0;
          } ## end if ( defined($par_vol)...)
        } ## end if ( $st_type =~ m/^3PAR$/)

        if ( $st_type =~ m/^HUS$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:controller:GAUGE:$no_time_new:0:100"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^HUS$/)
        if ( $st_type =~ m/^DOTHILL$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^DOTHILL$/)
        if ( $st_type =~ m/^COMPELLENT$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_xfer:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_qos:GAUGE:$no_time_new:0:100000000"
          "DS:io_pending:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "DS:snapshot:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^COMPELLENT$/)
        if ( $st_type =~ m/^OCEANSTOR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "DS:queue:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^OCEANSTOR$/)
        if ( $st_type =~ m/^PURE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:100000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^PURE$/)
        if ( $st_type =~ m/^VSPG$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate_b:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate_b:GAUGE:$no_time_new:0:100000000"
          "DS:tier0cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier0free:GAUGE:$no_time_new:0:10000000000"
          "DS:tier1cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier1free:GAUGE:$no_time_new:0:10000000000"
          "DS:tier2cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier2free:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VSPG$/)
        if ( $st_type =~ m/^VNXF$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:qdepth:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:100000000"
          "DS:busy:GAUGE:$no_time_new:0:10000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:cap:GAUGE:$no_time_new:0:100000000"
          "DS:used_cap:GAUGE:$no_time_new:0:100000000"
          "DS:server_id:GAUGE:$no_time_new:0:1000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXF$/)
        if ( $st_type =~ m/^VNX$/ ) {
          if ( $rrd =~ m/\.rrr$/ ) {
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:io_rate:COUNTER:$no_time_new:0:200000"
            "DS:read_io:COUNTER:$no_time_new:0:100000"
            "DS:write_io:COUNTER:$no_time_new:0:100000"
            "DS:busy_ticks:COUNTER:$no_time_new:0:1073741824"
            "DS:idle_ticks:COUNTER:$no_time_new:0:1073741824"
            "DS:io_rate_spa:COUNTER:$no_time_new:0:200000"
            "DS:io_rate_spb:COUNTER:$no_time_new:0:200000"
            "DS:arr_queue:COUNTER:$no_time_new:0:1048576000"
            "DS:sum_queue:COUNTER:$no_time_new:0:1048576000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );

            # MAX limits to update:
            # IO (read, write)   -> 100000
            # DATA (read, write) -> 10485760 (10 GB = 10485760 kB)
          } ## end if ( $rrd =~ m/\.rrr$/)
          else {
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:io_rate:COUNTER:$no_time_new:0:200000"
            "DS:read_io:COUNTER:$no_time_new:0:100000"
            "DS:write_io:COUNTER:$no_time_new:0:100000"
            "DS:data_rate:COUNTER:$no_time_new:0:20971520"
            "DS:read:COUNTER:$no_time_new:0:10485760"
            "DS:write:COUNTER:$no_time_new:0:10485760"
            "DS:cap:GAUGE:$no_time_new:0:1048576000"
            "DS:controller:GAUGE:$no_time_new:0:1"
            "DS:resp_t:GAUGE:$no_time_new:0:100000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end else [ if ( $rrd =~ m/\.rrr$/)]

          # MAX limits to update:
          # IO (read, write)   -> 100000
          # DATA (read, write) -> 10485760 (10 GB = 10485760 kB)
          # CAPACITY           -> 1048576000 (1000 TB = 1048576000 MB)
          # RESP_T             -> 100000 (100 sec = 100000 ms)
        } ## end if ( $st_type =~ m/^VNX$/)
        if ( $st_type =~ m/^UNITY$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:controller:GAUGE:$no_time_new:0:1"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^UNITY$/)
        if ( $st_type =~ m/^VMAX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:cache_hit:GAUGE:$no_time_new:0:10000"
          "DS:prct_read:GAUGE:$no_time_new:0:100"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:10000"
          "DS:prct_write:GAUGE:$no_time_new:0:100"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:10000"
          "DS:prct_r_cache_hit:GAUGE:$no_time_new:0:100"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:prct_w_cache_hit:GAUGE:$no_time_new:0:100"
          "DS:prct_cache_hit:GAUGE:$no_time_new:0:100"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:data_rate:GAUGE:$no_time_new:0:20000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VMAX$/)

        #RRDp::cmd qq(update "$rrd" $time:$io_rate:$data_rate:$read:$write:$resp_t_r:$resp_t_w);
        if ( $st_type =~ m/^VPLEX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:20000000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VPLEX$/)
        if ( $st_type =~ m/^VNXE$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:read_io:GAUGE:$no_time_new:0:100000000"
          "DS:write_io:GAUGE:$no_time_new:0:100000000"
          "DS:resp_t:GAUGE:$no_time_new:0:1000000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:cap:GAUGE:$no_time_new:0:10000000000"
          "DS:controller:GAUGE:$no_time_new:0:1"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXE$/)
        if ( $st_type =~ m/^DS8K$/ ) {
          if ( $rrd =~ m/rrc$/ ) {

            # *.rrc --> $r_cache_hit:$w_cache_hit:$resp_t_r:$resp_t_w
            # ABSOLUTE data type is here, data is zeroead after each interval
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:10000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:10000"
          "DS:resp_t_r:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w:GAUGE:$no_time_new:0:1000000"
          "DS:read_io:ABSOLUTE:$no_time_new:0:100000000"
          "DS:write_io:ABSOLUTE:$no_time_new:0:100000000"
          "DS:read_b:ABSOLUTE:$no_time_new:0:10000000000"
          "DS:write_b:ABSOLUTE:$no_time_new:0:10000000000"
          "DS:read_io_b:ABSOLUTE:$no_time_new:0:100000000"
          "DS:write_io_b:ABSOLUTE:$no_time_new:0:100000000"
          "DS:resp_t_r_b:GAUGE:$no_time_new:0:1000000"
          "DS:resp_t_w_b:GAUGE:$no_time_new:0:1000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end if ( $rrd =~ m/rrc$/ )
          else {
            # ABSOLUTE should be also read and write items, unfortunatelly too late discovered therefore devided by $step in CDEF during graphing
            RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:resp_t:GAUGE:$no_time_new:0:10000"
          "DS:read:GAUGE:$no_time_new:0:10000000000"
          "DS:write:GAUGE:$no_time_new:0:10000000000"
	  "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
	  "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
	  "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
	  "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
          );
          } ## end else [ if ( $rrd =~ m/rrc$/ )]
        } ## end if ( $st_type =~ m/^DS8K$/)
        last;
      };

      $type =~ m/^RANKcapacity$/ && do {

        # only for capacity of ranks/pools, once a day data
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd RC  : $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:tot:GAUGE:$no_time_new:0:10000000"
          "DS:used:GAUGE:$no_time_new:0:10000000"
	  "RRA:AVERAGE:0.5:$first_rra:$one_day_sample"  
      );
        my $mode = 0644;
        chmod $mode, "$rrd";

        # it must be here as it passes another parameter (retentions), not default one
        if ( !Xorux_lib::create_check("file: $rrd, $one_day_sample") ) {

          # $one_minute_sample is avoided here
          main::error( "create_rrd err : unable to create $rrd (filesystem is full?) at " . __FILE__ . ": line " . __LINE__ );
          RRDp::end;
          RRDp::start "$rrdtool";
          return 1;
        } ## end if ( !Xorux_lib::create_check...)
        return 0;
      };

      $type =~ m/^CPU-CORE$/ && do {

        # only for SWIZ
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd CPU : $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        if ( $st_type !~ m/^3PAR$/ && $st_type !~ m/^NETAPP$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:sys:GAUGE:$no_time_new:0:10000000"
          "DS:compress:GAUGE:$no_time_new:0:10000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type !~ m/^3PAR$/...)

        if ( $st_type =~ m/^NETAPP$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:busy:GAUGE:$no_time_new:0:10000"
          "DS:sk_switches:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^NETAPP$/)

        if ( $st_type =~ m/^3PAR$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:usr:GAUGE:$no_time_new:0:10000000"
          "DS:sys:GAUGE:$no_time_new:0:10000000"
          "DS:idle:GAUGE:$no_time_new:0:10000000"
          "DS:compress:GAUGE:$no_time_new:0:10000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^3PAR$/)
        last;
      };

      $type =~ m/^NODE-CACHE$/ && do {

        # only for HUS  and VSPG
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd NODE-CACHE : $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^HUS$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
            "DS:write_pend:GAUGE:$no_time_new:0:1000"
            "DS:clean_usage:GAUGE:$no_time_new:0:1000"
            "DS:middle_usage:GAUGE:$no_time_new:0:1000"
            "DS:phys_usage:GAUGE:$no_time_new:0:1000"
            "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
            "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
            "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
            "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^HUS$/)
        if ( $st_type =~ m/^VNXF$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:dnlc_hit:GAUGE:$no_time_new:0:10000"
          "DS:cache_hit:GAUGE:$no_time_new:0:10000"
          "DS:buffer_hit:GAUGE:$no_time_new:0:10000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNXF$/)
        if ( $st_type =~ m/^VMAX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:io_rate:GAUGE:$no_time_new:0:100000000"
          "DS:data_rate:GAUGE:$no_time_new:0:10000000000"
          "DS:WP:GAUGE:$no_time_new:0:100"
          "DS:cache_hit:GAUGE:$no_time_new:0:10000"
          "DS:size:GAUGE:$no_time_new:0:10000000000"
          "DS:wp:GAUGE:$no_time_new:0:10000000000"
          "DS:wp_limit:GAUGE:$no_time_new:0:100"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VMAX$/)
        if ( $st_type =~ m/^VSPG$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:write_pend:GAUGE:$no_time_new:0:10000000"
          "DS:usage_rate:GAUGE:$no_time_new:0:10000000"
          "DS:allocate:GAUGE:$no_time_new:0:10000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VSPG$/)
        last;
      };

      $type =~ m/^CPU-NODE$/ && do {

        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd CPU : $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();

        # only for SWIZ
        RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:bussy:GAUGE:$no_time_new:0:10000"
        "DS:lim:GAUGE:$no_time_new:0:10000"
        "DS:sys:GAUGE:$no_time_new:0:10000000"
        "DS:compress:GAUGE:$no_time_new:0:10000000"
        "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
        "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
        "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
        "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
      );
        last;
      };

      $type =~ m/^TIERcap$/ && do {

        # 3PAR so far only
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd TC  : Tier capacity: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^3PAR/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:total:GAUGE:$no_time_new:0:10000000000"
          "DS:free:GAUGE:$no_time_new:0:10000000000"
          "DS:used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^3PAR/)
        last;
      };

      $type =~ m/^POOLcap$/ && do {

        # only for capacity of pools for SVC
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd PC  : Pool capacity: $host:$type $rrd ; STEP=$step_new \n" if $DEBUG;
        touch();
        if ( $st_type =~ m/^VNX$/ || $st_type =~ m/UNITY/ || $st_type =~ m/VNXE$/ || $st_type =~ m/3PAR/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:total:GAUGE:$no_time_new:0:10000000000"
          "DS:free:GAUGE:$no_time_new:0:10000000000"
          "DS:used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end if ( $st_type =~ m/^VNX$/...)

        #RRDp::cmd qq(update "$rrd" $time:$cap_alloc:$cap_enab:$prct_cap_used:$cap:$cap_used);
        elsif ( $st_type =~ m/^VMAX$/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:alloc:GAUGE:$no_time_new:0:10000000000"
          "DS:enabled:GAUGE:$no_time_new:0:10000000000"
          "DS:prct_used:GAUGE:$no_time_new:0:100"
          "DS:total:GAUGE:$no_time_new:0:10000000000"
          "DS:used:GAUGE:$no_time_new:0:10000000000"
          "DS:subscription:GAUGE:$no_time_new:0:100000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end elsif ( $st_type =~ m/^VMAX$/)
        elsif ( $st_type =~ m/^SWIZ/ && $rrd =~ m/nl-cap\.rrd/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:total:GAUGE:$no_time_new:0:10000000000"
          "DS:free:GAUGE:$no_time_new:0:10000000000"
          "DS:used:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end elsif ( $st_type =~ m/^SWIZ/...)
        elsif ( $st_type =~ m/^SWIZ/ && $rrd =~ m/-comp\.rrd/ ) {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:compress_virt_cap:GAUGE:$no_time_new:0:10000000000"
          "DS:compress_cap:GAUGE:$no_time_new:0:10000000000"
          "DS:uncompress_cap:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end elsif ( $st_type =~ m/^SWIZ/...)
        else {
          RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:total:GAUGE:$no_time_new:0:10000000000"
          "DS:free:GAUGE:$no_time_new:0:10000000000"
          "DS:virt:GAUGE:$no_time_new:0:10000000000"
          "DS:used:GAUGE:$no_time_new:0:10000000000"
          "DS:real:GAUGE:$no_time_new:0:10000000000"
          "DS:tier0cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier0free:GAUGE:$no_time_new:0:10000000000"
          "DS:tier1cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier1free:GAUGE:$no_time_new:0:10000000000"
          "DS:tier2cap:GAUGE:$no_time_new:0:10000000000"
          "DS:tier2free:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
        );
        } ## end else [ if ( $st_type =~ m/^VNX$/...)]
        last;
      };

      $type =~ m/^VOLUMECache$/ && do {

        # only for capacity of pools for SVC
        #print "0000 $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd VC  : Volume cache: $host:$type $rrd ; STEP=$step_new time=$start_time\n" if $DEBUG;
        touch();
        RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:r_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:w_cache_hit:GAUGE:$no_time_new:0:1000"
          "DS:r_cache_usage:GAUGE:$no_time_new:0:10000000000"
          "DS:w_cache_usage:GAUGE:$no_time_new:0:10000000000"
          "DS:total_usage:GAUGE:$no_time_new:0:10000000000"
          "DS:res1:GAUGE:$no_time_new:0:10000000000"
          "DS:res2:GAUGE:$no_time_new:0:10000000000"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
      );
        last;
      };

      $type =~ m/^FAST-SG$/ && do {

        # VMAX FAST - Storage group by Tier
        #RRDp::cmd qq(update "$rrd" $time:$sg_cap_total:$tier_cap:$tier_cap_prct:$sg_alloc_total:$tier_alloc_tracks:$tier_alloc_tracks_prct);
        print "002 $start_time - $step - $rrd\n" if $DEBUG == 2;
        print "create_rrd FAST-SG : $host:$type $rrd ; STEP=$step_new time=$start_time\n" if $DEBUG;
        touch();
        RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
          "DS:sg_cap_tot:GAUGE:$no_time_new:0:100000000"
          "DS:tier_cap:GAUGE:$no_time_new:0:100000000"
          "DS:tier_cap_prct:GAUGE:$no_time_new:0:100"
          "DS:sg_alloc_tot:GAUGE:$no_time_new:0:1000000000000"
          "DS:tier_alloc:GAUGE:$no_time_new:0:1000000000000"
          "DS:tier_alloc_prct:GAUGE:$no_time_new:0:100"
          "RRA:AVERAGE:0.5:$first_rra:$five_mins_sample"
          "RRA:AVERAGE:0.5:$second_rra:$one_hour_sample"
          "RRA:AVERAGE:0.5:$third_rra:$five_hours_sample"
          "RRA:AVERAGE:0.5:$forth_rra:$one_day_sample"
      );
        last;
      };
    }
    my $mode = 0644;
    chmod $mode, "$rrd";

    if ( !Xorux_lib::create_check("file: $rrd, $five_mins_sample, $one_hour_sample, $five_hours_sample, $one_day_sample") ) {

      # $one_minute_sample is avoided here
      main::error( "create_rrd err : unable to create $rrd (filesystem is full?) at " . __FILE__ . ": line " . __LINE__ );
      RRDp::end;
      RRDp::start "$rrdtool";
      return 1;
    } ## end if ( !Xorux_lib::create_check...)
    return 0;
  } ## end if ( not -f $rrd )
  return 2;    # the rrdtool file already exist
} ## end sub create_rrd

# workaround for situation when during LPM move
# those counters appears all 0 on the targed systems what causes
# huge peak in the graph

sub data_check {
  my $ent   = shift;
  my $cap   = shift;
  my $uncap = shift;

  if ( !$ent eq '' ) {
    if ( !$cap eq '' ) {
      if ( !$uncap eq '' ) {
        if ( $ent == 0 && $cap == 0 && $uncap == 0 ) {
          return 1;
        }
      }
    } ## end if ( !$cap eq '' )
  } ## end if ( !$ent eq '' )
  return 0;
} ## end sub data_check

sub touch {
  my $version    = "$ENV{version}";
  my $basedir    = $ENV{INPUTDIR};
  my $new_change = "$basedir/tmp/$version-run";
  my $host       = $ENV{STORAGE_NAME};
  my $DEBUG      = $ENV{DEBUG};

  if ( $host eq '' ) {
    $host = "na";
  }

  if ( !-f $new_change ) {
    `touch $new_change`;    # say install_html.sh that there was any change
    print "touch          : $host $new_change\n" if $DEBUG;
  }

  return 0;
} ## end sub touch

sub load_retentions {
  my $step     = shift;
  my $act_time = shift;
  my $basedir  = $ENV{INPUTDIR};

  # standards
  # note this is set up once more  on the top of the file

  if ( $step == 60 ) {
    $five_mins_sample  = 44640;    #  1 min : 31 days
    $one_hour_sample   = 4320;     #  180 days
    $five_hours_sample = 1734;     # 361 days , in fact 6 hours
    $one_day_sample    = 1080;     # ~ 3 years
  } ## end if ( $step == 60 )
  else {                           # 5mins and more
    $five_mins_sample  = 25920;    # 90 days
    $one_hour_sample   = 4320;     # 180 days
    $five_hours_sample = 1734;     # 361 days , in fact 6 hours
    $one_day_sample    = 1080;     # ~ 3 years
  } ## end else [ if ( $step == 60 ) ]

  if ( !-f "$basedir/etc/retention.cfg" ) {

    # standard retentions in place
    return 0;
  }

  # extra retentions are specifiled in $basedir/etc/retention.cfg
  open( FH, "< $basedir/etc/retention.cfg" ) || main::error( "Can't read from: $basedir/etc/retention.cfg: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  foreach my $line (@lines) {
    chomp($line);
    if ( $line =~ m/^5min/ ) {
      ( my $trash, my $five_mins_sample_tmp ) = split( /:/, $line );
      if ( isdigit($five_mins_sample_tmp) ) {
        $five_mins_sample = $five_mins_sample_tmp;
      }
    } ## end if ( $line =~ m/^5min/)
    if ( $line =~ m/^60min/ ) {
      ( my $trash, my $one_hour_sample_tmp ) = split( /:/, $line );
      if ( isdigit($one_hour_sample_tmp) ) {
        $one_hour_sample = $one_hour_sample_tmp;
      }
    } ## end if ( $line =~ m/^60min/)
    if ( $line =~ m/^300min/ ) {
      ( my $trash, my $five_hours_sample_tmp ) = split( /:/, $line );
      if ( isdigit($five_hours_sample_tmp) ) {
        $five_hours_sample = $five_hours_sample_tmp;
      }
    } ## end if ( $line =~ m/^300min/)
    if ( $line =~ m/^1440min/ ) {
      ( my $trash, my $one_day_sample_tmp ) = split( /:/, $line );
      if ( isdigit($one_day_sample_tmp) ) {
        $one_day_sample = $one_day_sample_tmp;
      }
    } ## end if ( $line =~ m/^1440min/)
  } ## end foreach my $line (@lines)

  close(FH);

  # question is how this would work with 5mins step????
  # it would need definitelly some change
  #$step = $step / 60;
  #$five_mins_sample = $five_mins_sample / $step;
  #$one_hour_sample = $one_hour_sample / $step;
  #$five_hours_sample = $five_hours_sample / $step;
  #$one_day_sample = $one_day_sample / $step;

  return 1;
} ## end sub load_retentions

# load capacity usage of ranks (once a day into *.rrc)
sub load_rank_capacity {
  my ( $st_type, $wrkdir, $host, $type, $time, $DEBUG, $act_time, $rank_cfg_id_tmp, $pool_cfg_id_tmp, $rank_capacity_tot_tmp, $rank_capacity_used_tmp ) = @_;
  my @rank_cfg_id        = @{$rank_cfg_id_tmp};
  my @pool_cfg_id        = @{$pool_cfg_id_tmp};
  my @rank_capacity_tot  = @{$rank_capacity_tot_tmp};
  my @rank_capacity_used = @{$rank_capacity_used_tmp};
  my $step               = 86400;                        # daily
  my $no_time            = 2 * $step;

  my $indx       = 0;
  my $wrong      = 0;
  my $wrong_text = "";

  # Put on the last time possition "0000"!!! --> for smoth updates
  substr( $time, 6, 4, "0000" );

  foreach my $r_id (@rank_cfg_id) {

    #print "$rank_cfg_id[$indx]:$pool_cfg_id[$indx]:$rank_capacity_tot[$indx]:$rank_capacity_used[$indx]\n";
    if ( $rank_capacity_tot[$indx] eq '' || $rank_capacity_used[$indx] eq '' ) {
      $wrong_text = $rank_cfg_id[$indx] . $pool_cfg_id[$indx];
      next;    # something wrong
    }

    my $rrd = "$wrkdir/$host/$type/$rank_cfg_id[$indx]-$pool_cfg_id[$indx]\.rrc";

    #create rrd db if necessary
    $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
    if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type . "capacity", $st_type ) == 1 ) {
      return 2;        # leave uit if any error during RRDTool file creation to keep data
    }

    # find out last record in the db
    # as this makes it slowly to test it each time then it is done
    # once per a lpar for whole load and saved into the array
    my $last_rec = 0;
    eval {
      RRDp::cmd qq(last "$rrd" );
      $last_rec = RRDp::read;
    };
    if ($@) {
      rrd_error( $@, $rrd );
      next;
    }
    chomp($$last_rec);
    my $ltime = $$last_rec;
    $ltime = $ltime + 86400;
    substr( $ltime, 6, 4, "0000" );

    if ( isdigit($time) && isdigit($ltime) && $time > ( $ltime + 86000 ) ) {

      # when rrd db has been created then load data immedistally
      RRDp::cmd qq(update "$rrd" $time:$rank_capacity_tot[$indx]:$rank_capacity_used[$indx]);
      my $answer = RRDp::read;
    } ## end if ( isdigit($time) &&...)
    $indx++;
  } ## end foreach my $r_id (@rank_cfg_id)

  if ( $wrong == 1 ) {
    main::error( "Rank capacity load error: $wrong_text " . __FILE__ . ":" . __LINE__ );
  }

  return 0;
} ## end sub load_rank_capacity

sub find_id {
  my $volume = shift;
  my $host   = shift;
  my $wrkdir = shift;

  my $file  = "$wrkdir/$host/VOLUME/volumes.cfg";
  my @lines = ();

  if ( -f $file ) {
    open( FHR, "< $file" ) || main::error( "Can't open $file : $! " . __FILE__ . ":" . __LINE__ );
    @lines = <FHR>;
    close(FHR);
  }
  else {
    #main::error ("$file does not exist : $! ".__FILE__.":".__LINE__);
    return "";
  }

  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line =~ s/^\s+|\s+$//g;
    my ( $name, $id ) = split( " : ", $line );
    if ( $name eq $volume ) {
      if ( $id =~ m/;$/ ) {
        chop $id;
      }
      return $id;
    } ## end if ( $name eq $volume )
  } ## end foreach my $line (@lines)
  return "";

} ## end sub find_id

sub isdigit {
  my $digit = shift;

  if ( !defined($digit) || $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  return 0;
} ## end sub isdigit

###################################################
#  SWIZ section : SVC/Storwize
###################################################

sub load_data_svc_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # at first config files like svc01_svcconf_20130904_115351.out
  #

  my $perf_string = "svcconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_svc_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    # It must be called here otherwise HOST info wouild not work
    # once a day there is created svcconfig backup, every other 5minutes small config files without host info
    # it is here to do not overwrite small config info the complete one
    my $type_nick = "HOST";
    LoadDataModule::load_nicks( $type_nick, $wrkdir, $host, $act_time, $st_type, $DEBUG );
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # now data files like svc01_svcperf_20130904_115351.out
  #

  $perf_string = "svcperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host  " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_svc_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_svc_all

sub load_data_svc_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $port_file         = "$wrkdir/$host/PORT/port.cfg";
  my $mdisk_file        = "$wrkdir/$host/mdisk.cfg";
  my $io_group_file     = "$wrkdir/$host/IO-GROUP/io_group.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $pool_file_write   = 0;                               # true if any write there
  my $config_file_write = 0;                               # true if any write there
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FM,   "> $mdisk_file-tmp" )  || main::error( "Can't open $mdisk_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHWP, "> $port_file-tmp" )   || main::error( "Can't open $port_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FHWI, "> $io_group_file-tmp" )   || main::error( "Can't open $io_group_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD> 
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > 
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name            = "";
  my $pool_name_indx       = 0;
  my $cfg_print            = 1;
  my $mdisk_cfg_processing = 0;
  my $port_cfg_processing  = 0;
  my $io_group_cfg_processing = 0;
  my %io_group_inventory;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Managed Disk Level Configuration/ ) {
      $mdisk_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/Port Level Configuration/ ) {
      $port_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/Volume Level Configuration/ ) {
      $io_group_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/Configuration/ ) {
      $mdisk_cfg_processing = 0;
      $port_cfg_processing  = 0;
    }

    if ($io_group_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $io_group_cfg_processing = 0;
        next;
      }
      if ( $line =~ m/^volume_id,id/ || $line =~ m/^------/ ) {
        next;    # some trash like "---"
      }
      #volume_id,id,name,IO_group_id,IO_group_name,status,mdisk_grp_id,mdisk_grp_name,capacity,type,FC_id,FC_name,RC_id,RC_name,vdisk_UID,fc_map_count,copy_count,fast_write_state,se_copy_count,RC_change,compressed_copy_count
      #0000,0,vmware-vcenter,0,io_grp0,online,2,pool01,107374182400,striped,,,,,6005076300808AE51000000000000009,0,1,not_empty,0,no,0
      my (undef, undef, undef, $id_io_group, $name_io_group) = split (/,/, $line);
      if (defined $id_io_group && $id_io_group ne ""){
        if(!defined $name_io_group || $name_io_group eq ""){
          $name_io_group = $id_io_group;
        }
        if (!defined $io_group_inventory{$id_io_group}){
          $io_group_inventory{$id_io_group} = $name_io_group;
          print FHWI "$id_io_group:$name_io_group\n";
        }
      }
    }

    if ($port_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $port_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        # --PH not done yet
        #load_rank_capacity($st_type,$wrkdir,$host,"POOL",$act_time_u,$DEBUG,$act_time,\@rank_cfg_id,\@pool_cfg_id,\@pool_capacity_tot,\@pool_capacity_free);
        # --> not necessary here, HUS has already capacity in *.rrd
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )

      if ( $line =~ m/^port_name,id/ || $line =~ m/^------/ ) {
        next;    # some trash like "---"
      }

      # Only FC ports are here
      #port_name,id,fc_io_port_id,port_id,type,port_speed,node_id,node_name,WWPN,nportid,status,attachment
      #node1_p01,0,1,1,fc,4Gb,1,node1,500507680306B942,330200,active,switch
      my ( $port_id, undef, undef, undef, undef, $fc_speed, undef, undef, $wwn ) = split( /,/, $line );
      if ( $wwn eq '' ) {

        # no wwn   defined
        print FHWP "$port_id:(FC)\n";
      }
      else {
        print FHWP "$port_id:(FC $wwn)\n";
      }

      # Save speed for each port
      if ( defined($fc_speed) && !$fc_speed eq '' ) {
        my $port_speed_file = "$wrkdir/$host/PORT/$port_id-speed.txt";
        open( PORT_SPEED, "> $port_speed_file" ) || main::error( "Can't open $port_speed_file : $! " . __FILE__ . ":" . __LINE__ );
        print PORT_SPEED "$fc_speed\n";
        close(PORT_SPEED);
      } ## end if ( defined($fc_speed...))

      next;
    } ## end if ($port_cfg_processing)

    if ($mdisk_cfg_processing) {
      $line =~ s/^0x//;

      if ( $line =~ m/^$/ || $line =~ m/---/ || $line =~ m/^mdisk_id/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between mdisk and pools
      # mdisk_id,id,name,status,mode,mdisk_grp_id,mdisk_grp_name,capacity,ctrl_LUN_#,controller_name,UID,tier
      ( my $mdisk_id, my $id, my $mdisk_name, my $status, my $mode, my $pool_cfg_id_item, my $pool_name_cfg_item ) = split( /,/, $line );

      #print "009 $mdisk_cfg_id[$mdisk_cfg_indx] $pool_cfg_id[$mdisk_cfg_indx]\n";
      #print "009 $line\n";

      print FM "$mdisk_id:$mdisk_name\n";

      if ( !defined($pool_cfg_id_item) || $pool_cfg_id_item eq '' || !isdigit($pool_cfg_id_item) || $pool_cfg_id_item =~ m/\./ ) {
        next;
      }

      # save actual pool cfg
      my $found = -1;
      foreach my $pool_item (@pool_name) {
        if ( $pool_item =~ m/^$pool_name_cfg_item$/ && !$pool_name_cfg_item eq '' ) {
          $found = $pool_cfg_id_item;
          last;
        }
      } ## end foreach my $pool_item (@pool_name)
      if ( $found == -1 ) {
        $pool_name[$pool_name_indx] = $pool_name_cfg_item;
        $pool_name_indx++;
        print FHW "$pool_cfg_id_item:$pool_name_cfg_item\n";
      }
    } ## end if ($mdisk_cfg_processing)
  }    # foreach

  close(FHW);
  close(FHWP);
  close(FM);
  close(FHWI);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  # check if cfg section has been in the input file and ports have been found, if so then replace port.cfg by port.cfg-tmp
  if ( -f "$port_file-tmp" ) {
    open( FHR, "< $port_file-tmp" );    # do not check return status, might happen when at the same time storage data load and system data load processing it
    @lines = <FHR>;
    if ( defined( $lines[0] ) ) {
      close(FHR);

      #my @lines_write = sort { (split ':', $a)[0] <=> (split ':', $b)[0] } @lines; #numeric sorting

      foreach $line (@lines) {

        # mean there is at least one row, --> replace
        if ( !defined($line) || $line eq '' ) {
          next;
        }
        open( FHR, "> $port_file" ) || main::error( "Can't open $port_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
        foreach my $line_write (@lines) {
          print FHR "$line_write";
        }
        close(FHR);
        print "cfg port found : $port_file-tmp --> $port_file\n" if $DEBUG;
        unlink("$port_file-tmp");
        last;
      } ## end foreach $line (@lines)
    } ## end if ( defined( $lines[0...]))
  } ## end if ( -f "$port_file-tmp")

  if ( -f "$io_group_file-tmp" ) {
    open( FHR, "< $io_group_file-tmp" );    # do not check return status, might happen when at the same time storage data load and system data load processing it
    @lines = <FHR>;
    if ( defined( $lines[0] ) ) {
      close(FHR);

      foreach $line (@lines) {
        # mean there is at least one row, --> replace
        if ( !defined($line) || $line eq '' ) {
          next;
        }
        open( FHR, "> $io_group_file" ) || main::error( "Can't open $io_group_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
        foreach my $line_write (@lines) {
          print FHR "$line_write";
        }
        close(FHR);
        print "cfg io-group found : $io_group_file-tmp --> $io_group_file\n" if $DEBUG;
        unlink("$io_group_file-tmp");
        last;
      } ## end foreach $line (@lines)
    } ## end if ( defined( $lines[0...]))
  } ## end if ( -f "$port_file-tmp")

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  if ( -f "$pool_file-tmp" ) {
    open( FHR, "< $pool_file-tmp" );
    @lines = <FHR>;
    if ( defined( $lines[0] ) ) {
      close(FHR);
      my @lines_write = sort { ( split ':', $a )[0] <=> ( split ':', $b )[0] } @lines;    #numeric sorting

      foreach $line (@lines) {

        # mean there is at least one row, --> replace
        if ( !defined($line) || $line eq '' ) {
          next;
        }
        open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
        foreach my $line_write (@lines_write) {
          print FHR "$line_write";

          #print "$line_write";
        }
        close(FHR);
        print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
        unlink("$pool_file-tmp");
        last;
      } ## end foreach $line (@lines)
    } ## end if ( defined( $lines[0...]))
  } ## end if ( -f "$pool_file-tmp")

  # check if mdisk.cfg-tmp has any rows, then replace it and sort it out
  if ( -f "$mdisk_file-tmp" ) {
    open( FHR, "< $mdisk_file-tmp" );
    @lines = <FHR>;
    if ( defined( $lines[0] ) ) {
      close(FHR);
      my @lines_write = sort { ( split ':', $a )[0] <=> ( split ':', $b )[0] } @lines;    #numeric sorting

      foreach $line (@lines) {

        # mean there is at least one row, --> replace
        open( FHR, "> $mdisk_file" ) || main::error( "Can't open $mdisk_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
        foreach my $line_write (@lines_write) {
          print FHR "$line_write";

          #print "$line_write";
        }
        close(FHR);
        print "cfg mdisk found: $mdisk_file-tmp --> $mdisk_file\n" if $DEBUG;
        unlink("$mdisk_file-tmp");
        last;
      } ## end foreach $line (@lines)
    } ## end if ( defined( $lines[0...]))
  } ## end if ( -f "$mdisk_file-tmp")

  # same as above for config.html
  if ( $cfg_load_data_ok == 1 ) {
    if ( -f "$config_file-tmp" ) {
      open( FCFGR, "< $config_file-tmp" );
      @lines = <FCFGR>;
      if ( defined( $lines[0] ) ) {
        close(FCFGR);
        my $cfg_section = 0;
        foreach $line (@lines) {
          chomp($line);
          if ( $line =~ m/Pool Level/ || $line =~ m/Drive Level/ || $line =~ m/Node Level/ || $line =~ m/Port Level/ || $line =~ m/Managed Disk Level/ || $line =~ m/Volume Level/ ) {

            # the file must contain all 6 sections
            $cfg_section++;
          }
        } ## end foreach $line (@lines)

        if ( $cfg_section == 6 ) {
          print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
        }
        else {
          if ( $cfg_section == 0 ) {
            print "cfg not found  : no config in this data file\n" if $DEBUG;
          }
        }
        if ( -f "$config_file-tmp" ) {
          rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
        }
      } ## end if ( defined( $lines[0...]))
    } ## end if ( -f "$config_file-tmp")
  } ## end if ( $cfg_load_data_ok...)

  close(FH);
  return 0;
} ## end sub load_data_svc_conf

sub load_data_svc_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $time_last_ok        = "";
  my @mdisk_capacity_tot  = "";
  my @mdisk_capacity_used = "";
  my $counter_ins         = 0;
  my $last_rec            = "";
  my $rrd                 = "";
  my $time                = "";
  my $t                   = "";
  my $type                = "NA";
  my $t1                  = "";
  my $t2                  = "";
  my $t3                  = "";
  my $t4                  = "";
  my $t5                  = "";
  my $t6                  = "";
  my $t7                  = "";
  my $t8                  = "";
  my $t9                  = "";
  my $t10                 = "";
  my $t11                 = "";
  my $t12                 = "";
  my $act_time_u          = time();
  my $ltime               = localtime($act_time_u);
  my $cfg_load_data_ok    = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html
  my $data_error_first    = 0;

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                          # true if any write there
  my $config_file_write = 0;                          # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @mdisk_cfg_id         = "";
  my @pool_cfg_id          = "";
  my $mdisk_cfg_processing = 0;
  my $mdisk_cfg_indx       = 0;
  my @pool_name            = "";
  my @pool_name_id         = "";
  my $pool_name_indx       = 0;
  my $port_type_once       = 0;
  my $volume_cache_once    = 0;
  my $error_once           = 0;

  # pool fron-end data from volume data
  my @pool_read     = "";
  my @pool_write    = "";
  my @pool_io_read  = "";
  my @pool_io_write = "";
  my @pool_resp_r   = "";
  my @pool_resp_w   = "";
  my @pool_ids      = "";
  my $pool_time     = 0;
  my $pool_ltime    = 0;
  my %inventory_transfer_size;
  my %inventory_io_group;
  my $active_group = 0;

  #$DEBUG=2;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      $data_error_first = 0;
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Managed/ ) {
        $type = "RANK";    #  Managed --> RANK conversion (mdisk --> rank)
      }
      else {
        $type = uc($type_temp);
      }
      next;
    } ## end if ( $line =~ m/Level Statistics/)

    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/	//g;
      $line =~ s/Interval End:     //g;
      $line =~ s/GMT.*//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)
    if ( $line =~ m/^$/ ) {
      $type = "NA";      # clear out type when empty line
      if ($active_group == 1){ $active_group = 2;}
      if ( $pool_time > 0 ) {

        # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
        if ( $pool_time > $pool_ltime ) {
          my @none = "";
          $pool_ltime = pool_sum_insert(
            $type,  $wrkdir, $step,  $DEBUG, $host,  $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@none, \@none, \@none,
            \@none, \@none,  \@none, \@none, \@none, \@none,   \@none,    \@none,   \@none,     \@none,      \@none,      \@none,       \@none,         \@none,          \@none
          );
        } ## end if ( $pool_time > $pool_ltime)

        # reinitialize structures
        @pool_ids      = "";
        @pool_read     = "";
        @pool_write    = "";
        @pool_io_read  = "";
        @pool_io_write = "";
        @pool_resp_r   = "";
        @pool_resp_w   = "";
        $pool_time     = 0;
      } ## end if ( $pool_time > 0 )
      next;
    } ## end if ( $line =~ m/^$/ )

    if ( $line =~ /^name,id/ || $line =~ m/^	Interval/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ ) {

      # avoid other trash
      next;
    }

    # Port
    ########################

    if ( $type =~ m/^PORT$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $t1, my $t2, my $write_io, my $read_io, my $io_rate, my $write, my $read, my $data_rate, my $t10, my $t11, my $t12, my $t13, my $t14, my $t15, my $t16, my $t17, my $t18, my $t19, my $t20, my $t21, my $t22, my $t23, my $t24, my $t25, my $t26, my $t27, my $t28, my $t29, my $t30, my $t31, my $t32, my $t33, my $t34, my $t35, my $t36, my $t37, my $t38, my $t39, my $t40, my $t41, my $crc, my $b2b, my $t44, my $port_type_text ) = split( /,/, $line );

      #Port,Time,Interval (s),Port Send IO Rate (IO/s),Port Receive IO Rate (IO/s),Total Port IO Rate (IO/s),Port Send Data Rate (KB/s),Port Receive Data Rate (KB/s)
      #,TotalPort Data Rate (KB/s),Port Send Transfer Size (kB),Port Receive Transfer Size (kB),Overall Port Transfer Size (kB),Port to Host Send IO Rate (IO/s),
      #Port to Host Receive IO Rate (IO/s),Overall Port to Host IO Rate (IO/s),Port to Host Send Data Rate (KB/s),Port to Host Receive Data Rate (KB/s),
      #Overall Port to Host Data Rate (KB/s),Port to Controller Send IO Rate (IO/s),Port to Controller Receive IO Rate (IO/s),Overall Port to Controller IO Rate (IO/s),
      #Port to Controller Send Data Rate (KB/s),Port to Controller Receive Data Rate (KB/s),Overall Port to Controller Data Rate (KB/s),
      #Port to Local Node Send IO Rate (IO/s),Port to Local Node Receive IO Rate (IO/s),Overall Port to Local Node IO Rate (IO/s),Port to Local Node Send Data Rate (KB/s),
      #Port to Local Node Receive Data Rate (KB/s),Overall Port to Local Node Data Rate (KB/s),Port to Remote Node Send IO Rate (IO/s),
      #Port to Remote Node Receive IO Rate (IO/s),Overall Port to Remote Node IO Rate (IO/s),Port to Remote Node Send DataRate (KB/s),
      #Port to Remote Node Receive Data Rate (KB/s),Overall Port to Reomte Node Data Rate (KB/s),Link Failure,Loss of Synch,Loss of Signal,Primitive Sequence Protocol Error,
      #Invalid Transmission Word Count,Invalid CRC Count,Zero b2b (%),Node,Port type,WWPN,FC WWPN,FCoE WWPN,iSCSI IQN,

      #node1_p01,2013-09-04 11:49:19,300,1.25,800.21,801.46,20091.62,10011.64,30103.26,16073.30,12.51,8042.90,0.00,798.96,798.96,20091.62,10011.64,30103.26,0.00,0.00,0.00,
      #0.00,0.00,0.00,1.25,1.25,1.25,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0,0,0,0,0,0,7870,node1,FC,0x5005076802105e40,0x5005076802105e40,,,

      # type : FC / iSCSI / SAS (PCIe)
      my $port_type = 3;    # default unknown port type
      if ( !defined($port_type_text) || $port_type_text eq '' ) {
        if ( $port_type_once == 0 ) {
          main::error( "$host: unknown port type text: $name : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
          $port_type_once = 1;
        }
        next;
      } ## end if ( !defined($port_type_text...))
      if ( $port_type_text =~ m/^FC$/ )    { $port_type = 0; }
      if ( $port_type_text =~ m/^PCIe$/ )  { $port_type = 1; }
      if ( $port_type_text =~ m/^SAS$/ )   { $port_type = 1; }
      if ( $port_type_text =~ m/^iSCSI$/ ) { $port_type = 2; }
      if ( $port_type_text =~ m/^IPREP$/ ) { $port_type = 4; }

      # IPREP addition ...  replikaci pro asynchronn remote copy.

      if ( $port_type == 3 && $port_type_once == 0 ) {

        # unknown port type detected, print error message just once
        main::error( "$host: unknown port type: $name : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
        $port_type_once = 1;
      } ## end if ( $port_type == 3 &&...)

      #print "001 PORT $name - $write_io - $read_io - $io_rate - $write - $read - $data_rate - type:$port_type:$port_type_text\n";
      print "001 PORT $name - $write_io - $read_io - $io_rate - $write - $read - $data_rate - type:$port_type\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      # b2b & crc
      my $rrd_b2b = "$wrkdir/$host/$type/$name\.rrp";
      if ( $port_type == 0 ) {
        $rrd_b2b =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
        if ( create_rrd( $rrd_b2b, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
          return 2;            # leave uit if any error during RRDTool file creation to keep data
        }
      } ## end if ( $port_type == 0 )

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        my $resp_t = -1;    # not available yet, could be in the next release
        if ( !defined($read_io) || $read_io eq '' ) {
          $read_io = -1;
        }
        if ( !defined($read) || $read eq '' ) {
          $read = -1;
        }
        if ( !defined($write_io) || $write_io eq '' ) {
          $write_io = -1;
        }
        if ( !defined($write) || $write eq '' ) {
          $write = -1;
        }
        if ( !defined($resp_t) || $resp_t eq '' ) {
          $resp_t = -1;
        }
        if ( !defined($io_rate) || $io_rate eq '' ) {
          $io_rate = -1;
        }
        if ( !defined($data_rate) || $data_rate eq '' ) {
          $data_rate = -1;
        }
        if ( !defined($port_type) || $port_type eq '' ) {
          $port_type = -1;
        }
        if ( !defined($b2b) ) {
          $b2b = -1;
        }
        if ( !defined($crc) ) {
          $crc = -1;
        }
        print "004: PORT $time:$write_io:$read_io:$io_rate:$data_rate:$resp_t:$read:$write\n" if $DEBUG == 3;
        if ( !isdigit($write_io)
          || !isdigit($read_io)
          || !isdigit($io_rate)
          || !isdigit($data_rate)
          || !isdigit($resp_t)
          || !isdigit($read)
          || !isdigit($write)
          || !isdigit($port_type) )
        {
          if ( $data_error_first == 0 ) {
            main::error( "data error in PORT: $time:$write_io:$read_io:$io_rate:$data_rate:$resp_t:$read:$write:$port_type - $line " . __FILE__ . ":" . __LINE__ );
            $data_error_first = 1;
            $counter_ins--;
          }
          next;
        } ## end if ( !isdigit($write_io...))
        RRDp::cmd qq(update "$rrd" $time:$write_io:$read_io:$io_rate:$data_rate:$resp_t:$read:$write:$port_type);
        my $answer = RRDp::read;

        if ( $port_type == 0 ) {

          # added 1.35-4
          if ( !isdigit($b2b) || !isdigit($crc) ) {
            if ( $data_error_first == 0 ) {
              main::error( "data error in PORT: $time:$b2b:$crc - $line " . __FILE__ . ":" . __LINE__ );
              $data_error_first = 1;
              $counter_ins--;
            }
            next;
          } ## end if ( !isdigit($b2b) ||...)
          RRDp::cmd qq(update "$rrd_b2b" $time:$b2b:$crc);
          $answer = RRDp::read;
        } ## end if ( $port_type == 0 )

        $time_last_ok = $time;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^PORT$/)

    # RANK == mdisk in SWIZ terminology
    ########################

    if ( $type =~ m/^RANK$/ ) {
      print "100 $line\n" if $DEBUG == 3;

      # Rank statistics
      ( my $name, $t1, $t2, my $read_io, my $write_io, my $total_io, my $read, my $write, my $total, $t7, $t8, $t9, my $resp_t_r, my $resp_t_w, $t3, $t4, $t5, $t6, $t10, $t11, $t12, my $pool_id ) = split( /,/, $line );

      #Managed Disk ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
      #Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Backend Read Response Time (ms),Backend Write Response Time (ms),
      #Overall Backend Response Time (ms),Backend Read Queue Time (ms),Backend Write Queue Time (ms),Overall Backend Queue Time (ms),Peak Read Response Time (ms),
      #Peak Write Response Time(ms),Managed Disk Name,Managed DiskGroup ID,Managed Disk Group Name,Controller ID,Controller Name,Controller WWNN,Controller LUN (Decimal),
      #Controller LUN (Hex),Preferred WWPN,Quorum Index,Tier
      #0000,2013-09-04 11:49:19,300,233.91,20.48,254.40,21633.12,3689.96,25323.08,92.48,180.14,136.31,1.33,11.71,2.17,0.00,0.00,0.00,1.50,41.95,mdisk1,0,PRDSK1_SAS_600,
      # 0,,,0,,,0,generic_hdd,

      # SWIZ has a pool_id directly in each row!!!, DS8K must translate it from the table ...
      if ( !defined($pool_id) || $pool_id eq '' || !isdigit($pool_id) || $pool_id =~ m/\./ ) {
        if ( $error_once == 0 ) {
          if ( defined($pool_id) && $pool_id eq '' ) {
            next;    # it is a RANK without a pool assigment, skip it quietly ...
          }
          main::error( "Wrong pool_id for : $host:$type:$name : $pool_id : the error is reported only once per a run : $line " . __FILE__ . ":" . __LINE__ );
          $error_once++;
        } ## end if ( $error_once == 0 )
        next;        # something wrong
      } ## end if ( !defined($pool_id...))
      $rrd = "$wrkdir/$host/$type/$name-P$pool_id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@mdisk_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@mdisk_name)
      if ( $found > -1 ) {
        $ltime = $mdisk_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $mdisk_name[$l_count] = $rrd;
        $mdisk_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
        print "005: $ltime - $l_count\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $mdisk_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($read_io) || $read_io eq '' ) {
          $read_io = -1;
        }
        if ( !defined($read) || $read eq '' ) {
          $read = -1;
        }
        if ( !defined($write_io) || $write_io eq '' ) {
          $write_io = -1;
        }
        if ( !defined($write) || $write eq '' ) {
          $write = -1;
        }
        if ( !defined($resp_t_r) || $resp_t_r eq '' ) {
          $resp_t_r = -1;
        }
        if ( !defined($resp_t_w) || $resp_t_w eq '' ) {
          $resp_t_w = -1;
        }
        print "104: MDISK $time:$read_io:$write_io:$read:$write:$resp_t_r,$resp_t_w\n" if $DEBUG == 3;
        if ( !isdigit($read_io) || !isdigit($write_io) || !isdigit($read) || !isdigit($write) || !isdigit($resp_t_r) || !isdigit($resp_t_w) ) {
          if ( $data_error_first == 0 ) {
            main::error( "data error in RANK: $time:$read_io:$write_io:$read:$write:$resp_t_r:$resp_t_w - $line " . __FILE__ . ":" . __LINE__ );
            $data_error_first = 1;
            $counter_ins--;
          }
          next;
        } ## end if ( !isdigit($read_io...))
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$read:$write:$resp_t_r:$resp_t_w);
        my $answer = RRDp::read;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^RANK$/)

    # Volume
    ########################

    if ( $type =~ m/^VOLUME$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      if ($active_group == 0){ $active_group = 1;}


      # Volume statistics
      ( my $name, my $t1, my $t2, my $io_read, my $io_write, my $total_io, my $read, my $write, my $total, my $read_transfer_size, my $write_transfer_size, my $t5, my $resp_t_r, my $resp_t_w, my $t6, my $t7,
        my $t8, my $t9, my $t10, my $t11, my $t12, my $t13, my $t14, my $t15,      my $pool_id_vol, my $pool_name, my $group_id, my $group_name, undef, undef, undef, undef, undef, undef,
        undef,  undef,  undef,   undef,   undef,   undef,   my $cap, my $cap_real, my $cap_used
      ) = split( /,/, $line );

      #Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
      #Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),
      #Peak Read Response Time (ms),Peak Write Response Time (ms),Host Delay (assuming that all host delay is writes) (ms),Host Delay (assuming that host delay is
      # evenly spread between read and writes) (ms),Read Hits,Write Hits,Data Read (KB),Data Written (KB),Volume (Vdisk) Name,Managed Disk Group ID,
      #Managed Disk Group Name,IO Group ID,IO Group Name,Remote Copy relationship ID,Remote Copy relationship name,Remote Copy Change Volume relationship,
      #FlashCopy map ID,FlashCopy map name,FlashCopy map count,Copy Count,Space Efficient Copy Count,Cache state,Easy Tier On/Off,Easy Tier Status,Preferred Node ID,
      #Capacity (TB),Real Capacity (TB),Used Capacity (TB),Auto Expand,Grain Size,Throttle Unit,Throttle Rate,UDID (for HPUX)

      #0000,2013-09-04 11:49:19,300,0.10,0.74,0.84,0.05,3.07,3.12,0.50,4.13,2.31,0.83,0.26,0.55,0.00,0.00,0.00,0.00,30.00,223.00,15.00,920.00,A3201023rvg,0,
      # PRDSK1_SAS_600,0,io_grp0,0,,no,0,,0,1,0,not_empty,,,,53687091200.00,0,0,0,0,,,,

      if ( !defined($name) || $name eq '' || !isdigit($name) || $name =~ m/\./ ) {
        next;
      }
      if ( defined $cap && $cap ne '' && isdigit($cap) ) {
        $cap = $cap / 1024 / 1024;    # from B -> MB
      }
      if ( defined $cap_real && $cap_real ne '' && isdigit($cap_real) ) {
        $cap_real = $cap_real / 1024 / 1024;    # from B -> MB
      }
      if ( defined $cap_used && $cap_used ne '' && isdigit($cap_used) ) {
        $cap_used = $cap_used / 1024 / 1024;    # from B -> MB
      }

      #my $cap = -1;
      #my $cap_real = -1;
      #my $cap_used = -1;
      #my $cache_tot = -1;
      #my $cache_write_usage = -1;
      #my $cache_read_hits = -1;
      #my $cache_write_hits = -1;

      print "001 VOL $name - $io_read - $io_write - $resp_t_r - $resp_t_w - $read - $write\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime actuall: $time\n";
      # Update only latest data
      if ( defined($time) && isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($io_read) || $io_read eq '' ) {
          $io_read = -1;
        }
        if ( !defined($read) || $read eq '' ) {
          $read = -1;
        }
        if ( !defined($io_write) || $io_write eq '' ) {
          $io_write = -1;
        }
        if ( !defined($write) || $write eq '' ) {
          $write = -1;
        }
        if ( !defined($resp_t_r) || $resp_t_r eq '' ) {
          $resp_t_r = -1;
        }
        if ( !defined($resp_t_w) || $resp_t_w eq '' ) {
          $resp_t_w = -1;
        }
        if ( !defined($cap) || $cap eq '' ) {
          $cap = -1;
        }
        if ( !defined($cap_real) || $cap_real eq '' ) {
          $cap_real = -1;
        }
        if ( !defined($cap_used) || $cap_used eq '' ) {
          $cap_used = -1;
        }
        if (!defined($read_transfer_size) || $read_transfer_size eq '' ){
          $read_transfer_size = -1;
        }
        if (!defined($write_transfer_size) || $write_transfer_size eq '' ){
          $write_transfer_size = -1;
        }
        ### IO GROUPS from volume data
        if (defined $group_id && $group_id ne "" ){
          $inventory_io_group{$group_id}{$name}{"read_io"} = "$io_read";
          $inventory_io_group{$group_id}{$name}{"write_io"} = "$io_write";
          $inventory_io_group{$group_id}{$name}{"read"} = "$read";
          $inventory_io_group{$group_id}{$name}{"write"} = "$write";
          $inventory_io_group{$group_id}{$name}{"resp_t_r"} = "$resp_t_r";
          $inventory_io_group{$group_id}{$name}{"resp_t_w"} = "$resp_t_w";
          $inventory_io_group{$group_id}{$name}{"time"} = "$time";
          $inventory_io_group{$group_id}{$name}{"read_transfer_size"} = "$read_transfer_size";
          $inventory_io_group{$group_id}{$name}{"write_transfer_size"} = "$write_transfer_size";
        }

        ### metrics transfer size to hash save metrics in other section
        $inventory_transfer_size{$name}{"READ_TRANSFER_SIZE"} = $read_transfer_size;
        $inventory_transfer_size{$name}{"WRITE_TRANSFER_SIZE"} = $write_transfer_size;
        ###
        print "004: VOL $time:$io_read:$io_write:$resp_t_r:$resp_t_w:$read:$write:$cap:$cap_real:$cap_used:$read_transfer_size:$write_transfer_size\n" if $DEBUG == 3;

        #print "004: VOL $time:$io_read:$io_write:$resp_t_r:$resp_t_w:$read:$write:$cap:$cap_real:$cap_used\n";
        if ( !isdigit($io_read)
          || !isdigit($io_write)
          || !isdigit($resp_t_r)
          || !isdigit($resp_t_w)
          || !isdigit($read)
          || !isdigit($write)
          || !isdigit($cap)
          || !isdigit($cap_real)
          || !isdigit($cap_used))
        {
          if ( $data_error_first == 0 ) {
            main::error( "data error in VOLUME $time:$io_read:$io_write:$resp_t_r:$resp_t_w:$read:$write:$cap:$cap_real:$cap_used:$read_transfer_size:$write_transfer_size - $line " . __FILE__ . ":" . __LINE__ );
            $data_error_first = 1;
            $counter_ins--;
          }
          next;
        } ## end if ( !isdigit($io_read...))
        RRDp::cmd qq(update "$rrd" $time:$io_read:$io_write:$resp_t_r:$resp_t_w:$read:$write:$cap:$cap_real:$cap_used);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        # keep volume front-end data summ per pool and save it at the end
        #print "099 $pool_id_vol \n";
        if ( defined($pool_id_vol) ) {    # this does not work here if pool_id = 0, why?? " && ! $pool_id eq '' "
                                          # does not do isdigit as there might be 2 ids separated by a space (some mirror copies)
          my $pool_id_multi = 0;
          if ( $pool_id_vol =~ m/ / ) {
            $pool_id_multi = 1;
          }
          ### count whitespace
          #my $count_whitespace = () = $pool_id_vol =~ / /g;
          my $pom = $pool_id_vol;
          my $count_whitespace = $pom =~ tr/ //;
          my $denominator = $count_whitespace + 1;

          #print "900 $pool_id_vol : $pool_id_multi\n";
          foreach my $pool_id_act ( split( / +/, $pool_id_vol ) ) {

            #print "910 $pool_id_vol : $pool_id_multi : $pool_id_act \n";
            if ( isdigit($pool_id_act) == 0 ) {
              next;
            }
            if ( !defined $pool_read[$pool_id_act]     || $pool_read[$pool_id_act] eq '' )     { $pool_read[$pool_id_act]     = 0; }
            if ( !defined $pool_write[$pool_id_act]    || $pool_write[$pool_id_act] eq '' )    { $pool_write[$pool_id_act]    = 0; }
            if ( !defined $pool_io_read[$pool_id_act]  || $pool_io_read[$pool_id_act] eq '' )  { $pool_io_read[$pool_id_act]  = 0; }
            if ( !defined $pool_io_write[$pool_id_act] || $pool_io_write[$pool_id_act] eq '' ) { $pool_io_write[$pool_id_act] = 0; }
            if ( !defined $pool_resp_r[$pool_id_act]   || $pool_resp_r[$pool_id_act] eq '' )   { $pool_resp_r[$pool_id_act]   = 0; }
            if ( !defined $pool_resp_w[$pool_id_act]   || $pool_resp_w[$pool_id_act] eq '' )   { $pool_resp_w[$pool_id_act]   = 0; }
            if ( $pool_id_multi == 1 ) {

              # the volume belongs to 2 pools, some kind of mirror ??
              # read is half of each, write 100% each
              #print "98 $pool_id_act : $pool_id_multi : $read $io_read\n";
              $read    = $read / $denominator;
              $io_read = $io_read / $denominator;
              $write    = $write / $denominator;
              $io_write = $io_write / $denominator;
              $pool_id_multi++;

              # $resp_t_r should stay as it is
              #print "99 $pool_id_act : $pool_id_multi : $read $io_read\n";
            } ## end if ( $pool_id_multi ==...)
            $pool_read[$pool_id_act]     += $read;
            $pool_write[$pool_id_act]    += $write;
            $pool_io_read[$pool_id_act]  += $io_read;
            $pool_io_write[$pool_id_act] += $io_write;

            # convert resp time to acumulated time and after all summing devide it by number of IOs
            $pool_resp_r[$pool_id_act] += $resp_t_r * $io_read;
            $pool_resp_w[$pool_id_act] += $resp_t_w * $io_write;
            $pool_time = $time;
            $pool_ids[$pool_id_act] = $pool_id_act;

            #print "100 $pool_id_vol : $pool_id_act : $pool_read[$pool_id_act] $pool_read[$pool_id_act],$pool_write[$pool_id_act],$pool_io_read[$pool_id_act],$pool_io_write[$pool_id_act]\n";
          } ## end foreach my $pool_id_act ( split...)
        } ## end if ( defined($pool_id_vol...))
      } ## end if ( defined($time) &&...)
    } ## end if ( $type =~ m/^VOLUME$/)

    # Drives
    ########################

    if ( $type =~ m/^DRIVE$/ ) {
      print "000 DRIV $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $t1, my $t2, my $io_read, my $io_write, my $total_io, my $read, my $write, my $total, $t3, $t4, $t5, my $resp_t_r, my $resp_t_w ) = split( /,/, $line );

      #Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
      #Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms) ....

      #0000,2013-09-04 11:54:19,300,152.89,392.26,545.15,7166.40,12149.28,19315.68,46.87,30.97,38.92,1.95,3.93,3.38,2.92,13.00,10.17,8.35,8.18,11,12,0,,7,
      # mdisk7,,,,,,0,,,,0,0,299462819840.00,0.27,

      if ( !defined($name) || $name eq '' || !isdigit($name) || $name =~ m/\./ ) {
        next;
      }
      print "001 DRIV $name - $io_read - $io_write - $read - $write - $resp_t_r - $resp_t_w\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@drive_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@drive_name)
      if ( $found > -1 ) {
        $ltime = $drive_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "008 $rrd : $ltime : $time : $found : $l_count \n";

      print "002 DRIV $name - $io_read - $io_write - $read - $write - $resp_t_r - $resp_t_w\n : $time > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $drive_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($io_read) || $io_read eq '' ) {
          $io_read = -1;
        }
        if ( !defined($read) || $read eq '' ) {
          $read = -1;
        }
        if ( !defined($io_write) || $io_write eq '' ) {
          $io_write = -1;
        }
        if ( !defined($write) || $write eq '' ) {
          $write = -1;
        }
        if ( !defined($resp_t_r) || $resp_t_r eq '' ) {
          $resp_t_r = -1;
        }
        if ( !defined($resp_t_w) || $resp_t_w eq '' ) {
          $resp_t_w = -1;
        }
        print "004: DRIV $time:$io_read:$io_write:$read:$write:$resp_t_r:$resp_t_w\n" if $DEBUG == 3;
        if ( !isdigit($io_read) || !isdigit($io_write) || !isdigit($read) || !isdigit($write) || !isdigit($resp_t_r) || !isdigit($resp_t_w) ) {
          if ( $data_error_first == 0 ) {
            main::error( "data error in DRIVE: $time:$io_read:$io_write:$read:$write:$resp_t_r:$resp_t_w - $line " . __FILE__ . ":" . __LINE__ );
            $data_error_first = 1;
            $counter_ins--;
          }
          next;
        } ## end if ( !isdigit($io_read...))
        RRDp::cmd qq(update "$rrd" $time:$io_read:$io_write:$read:$write:$resp_t_r:$resp_t_w);
        my $answer = RRDp::read;

        # to avoid a bug on HMC when it sometimes reports 2 different values for the same time!!
        #$port_time[$l_count] = $t;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^DRIVE$/)

    # CPU-Node
    ########################

    if ( $type =~ m/^CPU-NODE$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $t1, my $t2, my $cpu_bussy, my $cpu_lim, my $cpu_sys, my $cpu_compress ) = split( /,/, $line );

      # Node,Time,Interval,CPU Busy,CPU Limited,CPU Utilization - System,CPU Utilization - Compression,
      # node1,2014-09-01 11:39:43,300,21.224,0.000,21.224,0.006,

      print "001 CPU $name - $cpu_bussy - $cpu_sys - $cpu_compress\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cpun_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cpun_name)
      if ( $found > -1 ) {
        $ltime = $cpun_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cpun_name[$l_count] = $rrd;
        $cpun_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cpun_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($cpu_bussy) || $cpu_bussy eq '' ) {
          $cpu_bussy = -1;
        }
        if ( !defined($cpu_lim) || $cpu_lim eq '' ) {
          $cpu_lim = -1;
        }
        if ( !defined($cpu_sys) || $cpu_sys eq '' ) {
          $cpu_sys = -1;
        }
        if ( !defined($cpu_compress) || $cpu_compress eq '' ) {
          $cpu_compress = -1;
        }
        print "004 CPU NODE $cpu_bussy $cpu_lim $cpu_sys $cpu_compress\n" if $DEBUG == 3;
        if ( !isdigit($cpu_bussy) || !isdigit($cpu_lim) || !isdigit($cpu_sys) || !isdigit($cpu_compress) ) {
          if ( $data_error_first == 0 ) {
            main::error( "data error in CPU NODE:$time:$cpu_bussy:$cpu_lim:$cpu_sys:$cpu_compress - $line " . __FILE__ . ":" . __LINE__ );
            $data_error_first = 1;
            $counter_ins--;
          }
          next;
        } ## end if ( !isdigit($cpu_bussy...))
        RRDp::cmd qq(update "$rrd" $time:$cpu_bussy:$cpu_lim:$cpu_sys:$cpu_compress);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CPU-NODE$/)

    # CPU-Core
    ########################

    if ( $type =~ m/^CPU-CORE$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $t1, my $t2, my $node, my $id, my $sys, my $compress ) = split( /,/, $line );

      #CPU Core,Time,Interval,Node,CPU Core ID,CPU Utilization - System,CPU Utilization - Compression,

      #node1_core00,2013-09-04 11:54:19,300,node1,0,6.77,0.00,
      #node1_core01,2013-09-04 11:54:19,300,node1,1,8.90,0.00,

      print "001 CPU $name - $sys - $compress\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cpuc_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cpuc_name)
      if ( $found > -1 ) {
        $ltime = $cpuc_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cpuc_name[$l_count] = $rrd;
        $cpuc_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cpuc_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($sys) || $sys eq '' ) {
          $sys = -1;
        }
        if ( !defined($compress) || $compress eq '' ) {
          $compress = -1;
        }
        print "004 CPU $name - $sys - $compress\n" if $DEBUG == 3;
        if ( !isdigit($sys) || !isdigit($compress) ) {
          if ( $data_error_first == 0 ) {
            main::error( "data error in CPU: $time:$sys:$compress - $line " . __FILE__ . ":" . __LINE__ );
            $data_error_first = 1;
            $counter_ins--;
          }
          next;
        } ## end if ( !isdigit($sys) ||...)
        RRDp::cmd qq(update "$rrd" $time:$sys:$compress);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CPU-CORE$/)

    # Pool capacity
    ########################

    if ( $type =~ m/^POOLcap$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port capacity statistics
      ( my $name,     my $id,        my $t9,       undef,           undef,        my $total,     undef,          my $free, my $virt,     my $used, my $real, undef, undef, undef, undef, undef, undef,
        my $tier0cap, my $tier0free, undef,        undef,           my $tier1cap, my $tier1free, undef,          undef,    my $tier2cap, my $tier2free,
        my $comp_act, my $comp_virt, my $comp_cap, my $comp_uncomp, undef,        undef,         my $tier0nlcap, my $tier0nlfree
      ) = split( /,/, $line );

      #name,id,status,mdisk count,volume count,capacity (TB),extent size,free capacity (TB),virtual capacity (TB),used capacity (TB),real capacity (TB),overallocation,warning (%),easy tier,easy tier status,
      #TIER-0 type,TIER-0 mdisk count,TIER-0 capacity (TB),TIER-0 free capacity (TB),TIER-1 type,TIER-1 mdisk count,TIER-1 capacity (TB),TIER-1 free capacity (TB),
      #TIER-2 type,TIER-2 mdisk count,TIER-2 capacity (TB),TIER-2 free capacity (TB),compression active,compression virtual capacity (TB),compression compressed capacity (TB),compression uncompressed capacity (TB),
      #TIER-0-NL type,TIER-0-NL mdisk count,TIER-0-NL capacity (TB),TIER-0-NL free capacity (TB)

      #pool01,2,online,1,8,1.631,1024,0.293,1.924,1.315,1.334,117,98,auto,balanced,ssd,0,0.000,0.000,enterprise,1,1.631,0.293,nearline,0,0.000,0.000,no,0.000,0.000,0.000,,0,0.000,0.000,

      if ( !defined($id) || $id eq '' || !isdigit($id) || $id =~ m/\./ ) {
        next;    # to be save against cooruptions in the input file!!!
      }

      print "001 POOL cap : $name - $id - $total - $free - $virt - $used - $real \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/POOL/$id-cap\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave it if any error during RRDTool file creation to keep data
      }

      my $rrd_comp = "$wrkdir/$host/POOL/$id-comp\.rrd";
      if ( $comp_act =~ m/yes/ ) {

        # only if compression is active
        #create rrd db if necessary
        $rrd_comp =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
        if ( create_rrd( $rrd_comp, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
          return 2;             # leave it if any error during RRDTool file creation to keep data
        }
      } ## end if ( $comp_act =~ m/yes/)

      my $rrd_nl = "$wrkdir/$host/POOL/$id-nl-cap\.rrd";

      #create rrd db if necessary
      $rrd_nl =~ s/"//g;        # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd_nl, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;               # leave it if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_cap_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_cap_name)
      if ( $found > -1 ) {
        $ltime = $pool_cap_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_cap_name[$l_count] = $rrd;
        $pool_cap_time[$l_count] = $$last_rec;
        $ltime                   = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_cap_time[$l_count] = $time;
        $counter_ins++;
        if ( !defined($total) || $total eq '' ) {
          $total = -1;
        }
        if ( !defined($free) || $free eq '' ) {
          $free = -1;
        }
        if ( !defined($virt) || $virt eq '' ) {
          $virt = -1;
        }
        if ( !defined($used) || $used eq '' ) {
          $used = -1;
        }
        if ( !defined($real) || $real eq '' ) {
          $real = -1;
        }
        if ( !defined($tier0cap) || $tier0cap eq '' ) {
          $tier0cap = -1;
        }
        if ( !defined($tier0free) || $tier0free eq '' ) {
          $tier0free = -1;
        }
        if ( !defined($tier1cap) || $tier1cap eq '' ) {
          $tier1cap = -1;
        }
        if ( !defined($tier1free) || $tier1free eq '' ) {
          $tier1free = -1;
        }
        if ( !defined($tier2cap) || $tier2cap eq '' ) {
          $tier2cap = -1;
        }
        if ( !defined($tier2free) || $tier2free eq '' ) {
          $tier2free = -1;
        }
        if ( !defined($comp_virt) || $comp_virt eq '' ) {
          $comp_virt = -1;
        }
        if ( !defined($comp_cap) || $comp_cap eq '' ) {
          $comp_cap = -1;
        }
        if ( !defined($comp_uncomp) || $comp_uncomp eq '' ) {
          $comp_uncomp = -1;
        }
        if ( !defined($tier0nlcap) || $tier0nlcap eq '' ) {
          $tier0nlcap = -1;
        }
        if ( !defined($tier0nlfree) || $tier0nlfree eq '' ) {
          $tier0nlfree = -1;
        }
        my $tier0nlused = -1;
        if ( isdigit($tier0nlcap) && isdigit($tier0nlfree) ) {
          $tier0nlused = $tier0nlcap - $tier0nlfree;
          if ( $tier0nlused < 0 ) {
            $tier0nlused = -1;
          }
        } ## end if ( isdigit($tier0nlcap...))

        print "004 POOL cap : $name - $id - $total - $free - $virt - $used - $real TIERS: $tier0cap:$tier0free $tier1cap:$tier1free $tier2cap:$tier2free\n" if $DEBUG == 3;
        if ( !isdigit($total)
          || !isdigit($free)
          || !isdigit($virt)
          || !isdigit($used)
          || !isdigit($real)
          || !isdigit($tier0cap)
          || !isdigit($tier0free)
          || !isdigit($tier1cap)
          || !isdigit($tier1free)
          || !isdigit($tier2cap)
          || !isdigit($tier2free)
          || !isdigit($comp_virt)
          || !isdigit($comp_cap)
          || !isdigit($comp_uncomp)
          || !isdigit($tier0nlcap)
          || !isdigit($tier0nlfree) )
        {
          if ( $data_error_first == 0 ) {
            main::error( "data error in POOL cap: $time:$total:$free:$virt:$used:$real:$tier0cap:$tier0free:$tier1cap:$tier1free:$tier2cap:$tier2free:$comp_virt:$comp_cap:$comp_uncomp:$tier0nlcap:$tier0nlfree - $line " . __FILE__ . ":" . __LINE__ );
            $data_error_first = 1;
            $counter_ins--;
          }
          next;
        } ## end if ( !isdigit($total) ...)
        RRDp::cmd qq(update "$rrd" $time:$total:$free:$virt:$used:$real:$tier0cap:$tier0free:$tier1cap:$tier1free:$tier2cap:$tier2free);
        my $answer = RRDp::read;
        RRDp::cmd qq(update "$rrd_nl" $time:$tier0nlcap:$tier0nlfree:$tier0nlused);
        $answer = RRDp::read;
        if ( $comp_act =~ m/yes/ ) {
          RRDp::cmd qq(update "$rrd_comp" $time:$comp_virt:$comp_cap:$comp_uncomp);
          $answer = RRDp::read;
        }
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^POOLcap$/)

    # Volume cache
    ########################

    if ( $type =~ m/^VOLUMECache$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port capacity statistics
      ( my $id, my $t1, my $t2, my $t3, my $rcache_hit, my $wcache_hit, my $rcache_usage, my $wcache_usage, my $totcache_usage ) = split( /,/, $line );

      # Volume ID,Time,Interval (s),Volume (Vdisk) Name,Cache read hits (%),Cache write hits (%),Read cache usage (kB),Write cache usage (kB),Total cache usage (kB),
      #  Cache data prestaged (kB/s),Cache data destaged (kB/s),Cache read hits (%),Cache read hits on prestaged data (%),Cache write hits on dirty data (%),
      #  Cache writes in Flush Through mode (kB/s),Cachewrites in Write Through mode (kB/s),Cache writes in Fast Write mode (kB/s),
      #  Cache writes in Fast Write mode that were written in Write Through mode due to the lack of memory (kB/s),

      #0000,2013-10-17 21:01:52,300,XXXXX00vg,30.742,36.165,2708,0,2708,5.760,27.253,30.742,25.442,36.165,0.000,0.000,42.693,0.000,

      if ( !defined($id) || $id eq '' || !isdigit($id) || $id =~ m/\./ ) {
        next;
      }

      print "001 VOLUMECache: $id - $rcache_hit - $wcache_hit - $rcache_usage - $wcache_usage - $totcache_usage \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/VOLUME/$id\.rrc";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@volume_cache_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@volume_cache_name)
      if ( $found > -1 ) {
        $ltime = $volume_cache_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $volume_cache_name[$l_count] = $rrd;
        $volume_cache_time[$l_count] = $$last_rec;
        $ltime                       = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "009 $found : $ltime : $rrd : $volume_cache_name[$l_count] : $volume_cache_time[$l_count]\n";

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $volume_cache_time[$l_count] = $time;
        $counter_ins++;
        print "004 VOLUMECache: $id - $rcache_hit - $wcache_hit - $rcache_usage - $wcache_usage - $totcache_usage - $time\n" if $DEBUG == 3;
        my $reserve1 = -1;    # to get NaN there
        my $reserve2 = -1;
        if ( !defined($rcache_hit) || $rcache_hit eq '' ) {
          $rcache_hit = -1;
        }
        else {
          if ( isdigit($rcache_hit) && $rcache_hit > 100 ) {
            $rcache_hit = 100;    # sometimes could see slightly above 100%, polishing here
          }
        }
        if ( !defined($wcache_hit) || $wcache_hit eq '' ) {
          $wcache_hit = -1;
        }
        else {
          if ( isdigit($wcache_hit) && $wcache_hit > 100 ) {
            $wcache_hit = 100;    # sometimes could see slightly above 100%, polishing here
          }
        }
        if ( !defined($rcache_usage) || $rcache_usage eq '' ) {
          $rcache_usage = -1;
        }
        if ( !defined($wcache_usage) || $wcache_usage eq '' ) {
          $wcache_usage = -1;
        }
        if ( !defined($totcache_usage) || $totcache_usage eq '' ) {
          $totcache_usage = -1;
        }
        ### transfer size metrics
        my $read_transfer_size = -1;
        my $write_transfer_size = -1;
        if (defined $inventory_transfer_size{$id}{"READ_TRANSFER_SIZE"} && isdigit($inventory_transfer_size{$id}{"READ_TRANSFER_SIZE"})){
          $read_transfer_size = $inventory_transfer_size{$id}{"READ_TRANSFER_SIZE"};
        }
        else{
          $read_transfer_size = -1;
        }
        if (defined $inventory_transfer_size{$id}{"WRITE_TRANSFER_SIZE"} && isdigit($inventory_transfer_size{$id}{"WRITE_TRANSFER_SIZE"})){
          $write_transfer_size = $inventory_transfer_size{$id}{"WRITE_TRANSFER_SIZE"};
        }
        else{
          $write_transfer_size = -1;
        }
        ###

        #print "005 VOLUMECache: $id - $rcache_hit - $wcache_hit - $rcache_usage - $wcache_usage - $totcache_usage - $time\n";
        if ( !isdigit($rcache_hit)
          || !isdigit($wcache_hit)
          || !isdigit($rcache_usage)
          || !isdigit($wcache_usage)
          || !isdigit($totcache_usage)
          || !isdigit($read_transfer_size)
          || !isdigit($write_transfer_size) )
        {
          if ( $data_error_first == 0 ) {

            #main::error("data error in VOLUMECache: $time:$rcache_hit:$wcache_hit:$rcache_usage:$wcache_usage:$totcache_usage:$reserve1:$reserve2 - $line ".__FILE__.":".__LINE__);
            # since 2.0 there is no read/write cache usage for new Storwize firmwares, justb total ones (before there was not anything)
            main::error( "data error in VOLUMECache: $time:$rcache_hit:$wcache_hit:U:U:$totcache_usage:$read_transfer_size:$write_transfer_size - $line " . __FILE__ . ":" . __LINE__ );
            $data_error_first = 1;
            $counter_ins--;
          } ## end if ( $data_error_first...)
          next;
        } ## end if ( !isdigit($rcache_hit...))
        RRDp::cmd qq(update "$rrd" $time:$rcache_hit:$wcache_hit:$rcache_usage:$wcache_usage:$totcache_usage:$read_transfer_size:$write_transfer_size);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^VOLUMECache$/)
    if ($type =~ m/NODE/ && $active_group == 2){
      ### IO GROUP implementation
      #print Dumper \%inventory_io_group;
      $active_group = 3;

      foreach my $id_group (keys %inventory_io_group){
        my $write_io = "U";
        my $read_io = "U";
        my $read = "U";
        my $write = "U";
        my $resp_t_r = "U";
        my $resp_t_w = "U";
        my $read_transfer_size = "U";
        my $write_transfer_size = "U";
        my $time = "";
        foreach my $id_volume (keys %{$inventory_io_group{$id_group}}){
          if (defined $inventory_io_group{$id_group}{$id_volume}{"time"}){
            $time = $inventory_io_group{$id_group}{$id_volume}{"time"};
          }
          if (defined $inventory_io_group{$id_group}{$id_volume}{"write_io"} && isdigit($inventory_io_group{$id_group}{$id_volume}{"write_io"})){
            if (isdigit($write_io) ){
              $write_io = $write_io + $inventory_io_group{$id_group}{$id_volume}{"write_io"};
            }
            else{
              $write_io = $inventory_io_group{$id_group}{$id_volume}{"write_io"};
            }
          }
          if (defined $inventory_io_group{$id_group}{$id_volume}{"read_io"} && isdigit($inventory_io_group{$id_group}{$id_volume}{"read_io"})){
            if (isdigit($read_io) ){
              $read_io = $read_io + $inventory_io_group{$id_group}{$id_volume}{"read_io"};
            }
            else{
              $read_io = $inventory_io_group{$id_group}{$id_volume}{"read_io"};
            }
          }
          if (defined $inventory_io_group{$id_group}{$id_volume}{"write"} && isdigit($inventory_io_group{$id_group}{$id_volume}{"write"})){
            if (isdigit($write)){
              $write = $write + $inventory_io_group{$id_group}{$id_volume}{"write"};
            }
            else{
              $write = $inventory_io_group{$id_group}{$id_volume}{"write"};
            }
          }
          if (defined $inventory_io_group{$id_group}{$id_volume}{"read"} && isdigit($inventory_io_group{$id_group}{$id_volume}{"read"})){
            if (isdigit($read)){
              $read = $read + $inventory_io_group{$id_group}{$id_volume}{"read"};
            }
            else{
              $read = $inventory_io_group{$id_group}{$id_volume}{"read"};
            }
          }
          if (defined $inventory_io_group{$id_group}{$id_volume}{"resp_t_r"} && isdigit($inventory_io_group{$id_group}{$id_volume}{"resp_t_r"})){
            my $read_io_local = $inventory_io_group{$id_group}{$id_volume}{"read_io"};
            my $resp_t_r_local = $inventory_io_group{$id_group}{$id_volume}{"resp_t_r"};
            if (isdigit($read_io_local) && isdigit($resp_t_r_local)){
              if (isdigit($resp_t_r)){
                $resp_t_r = $resp_t_r + ($read_io_local * $resp_t_r_local);
              }
              else{
                $resp_t_r = ($read_io_local * $resp_t_r_local);
              }
            }
          }
          if (defined $inventory_io_group{$id_group}{$id_volume}{"resp_t_w"} && isdigit($inventory_io_group{$id_group}{$id_volume}{"resp_t_w"})){
            my $write_io_local = $inventory_io_group{$id_group}{$id_volume}{"write_io"};
            my $resp_t_w_local = $inventory_io_group{$id_group}{$id_volume}{"resp_t_w"};
            if (isdigit($write_io_local) && isdigit($resp_t_w_local)){
              if (isdigit($resp_t_w)){
                $resp_t_w = $resp_t_w + ($write_io_local * $resp_t_w_local);
              }
              else{
                $resp_t_w = ($write_io_local * $resp_t_w_local);
              }
            }
          }
          if (defined $inventory_io_group{$id_group}{$id_volume}{"read_transfer_size"} && isdigit($inventory_io_group{$id_group}{$id_volume}{"read_transfer_size"})){
            my $read_io_local = $inventory_io_group{$id_group}{$id_volume}{"read_io"};
            my $read_transfer_size_local = $inventory_io_group{$id_group}{$id_volume}{"read_transfer_size"};
            if (isdigit($read_io_local) && isdigit($read_transfer_size_local)){
              if (isdigit($read_transfer_size)){
                $read_transfer_size = $read_transfer_size + ($read_io_local * $read_transfer_size_local);
              }
              else{
                $read_transfer_size = ($read_io_local * $read_transfer_size_local);
              }
            }
          }
          if (defined $inventory_io_group{$id_group}{$id_volume}{"write_transfer_size"} && isdigit($inventory_io_group{$id_group}{$id_volume}{"write_transfer_size"})){
            my $write_io_local = $inventory_io_group{$id_group}{$id_volume}{"write_io"};
            my $write_transfer_size_local = $inventory_io_group{$id_group}{$id_volume}{"write_transfer_size"};
            if (isdigit($write_io_local) && isdigit($write_transfer_size_local)){
              if (isdigit($write_transfer_size)){
                $write_transfer_size = $write_transfer_size + ($write_io_local * $write_transfer_size_local);
              }
              else{
                $write_transfer_size = ($write_io_local * $write_transfer_size_local);
              }
            }
          }
        }
        ### calculate resp time and transfer size
        if (isdigit($resp_t_r) && isdigit($read_io) && $read_io != 0){
          my $pom = round($resp_t_r / $read_io, 2);
          $resp_t_r = $pom;
        }
        if (isdigit($resp_t_r) && isdigit($read_io) && $read_io == 0){
          $resp_t_r = 0;
        }
        if (!isdigit($resp_t_r)){
         $resp_t_r = "U";
        }

        if (isdigit($resp_t_w) && isdigit($write_io) && $write_io != 0){
          my $pom = round($resp_t_w / $write_io, 2);
            $resp_t_w = $pom;
        }
        if (isdigit($resp_t_w) && isdigit($write_io) && $write_io == 0){
          $resp_t_w = 0;
        }
        if (!isdigit($resp_t_w)){
         $resp_t_w = "U";
        }

        if (isdigit($read_transfer_size) && isdigit($read_io) && $read_io != 0){
          my $pom = round($read_transfer_size / $read_io, 0);
          $read_transfer_size = $pom;
        }
        if (isdigit($read_transfer_size) && isdigit($read_io) && $read_io == 0){
          $read_transfer_size = 0;
        }
        if (!isdigit($read_transfer_size)){
          $read_transfer_size = "U";
        }

        if (isdigit($write_transfer_size) && isdigit($write_io) && $write_io != 0){
          my $pom = round($write_transfer_size / $write_io, 2);
          $write_transfer_size = $pom;
        }
        if (isdigit($write_transfer_size) && isdigit($write_io) && $write_io == 0){
          $write_transfer_size = 0;
        }
        if (!isdigit($write_transfer_size)){
          $write_transfer_size = "U";
        }

        print "001 : IO-GROUP $id_group - $read_io - $write_io - $read - $write - $resp_t_r $resp_t_w $read_transfer_size $write_transfer_size \n" if $DEBUG == 3;
        $rrd = "$wrkdir/$host/IO-GROUP/$id_group\.rrd";

        #create rrd db if necessary
        $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
        if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, "IO-GROUP", $st_type ) == 1 ) {
          return 2;        # leave uit if any error during RRDTool file creation to keep data
        }

        my $l;
        my $l_count = 0;
        my $found   = -1;
        my $ltime;
        foreach $l (@io_group_name) {
          if ( $l =~ m/^$rrd$/ ) {
            $found = $l_count;
            last;
          }
          $l_count++;
        } ## end foreach $l (@volume_cache_name)
        if ( $found > -1 ) {
          $ltime = $io_group_time[$found];
        }
        else {
          # find out last record in the db
          # as this makes it slowly to test it each time then it is done
          # once per a lpar for whole load and saved into the array
          eval {
            RRDp::cmd qq(last "$rrd" );
            $last_rec = RRDp::read;
          };
          if ($@) {
            rrd_error( $@, $rrd );
            next;
          }
          chomp($$last_rec);
          chomp($$last_rec);
          $io_group_name[$l_count] = $rrd;
          $io_group_time[$l_count] = $$last_rec;
          $ltime                       = $$last_rec;
        } ## end else [ if ( $found > -1 ) ]

        if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
          $io_group_time[$l_count] = $time;
          $counter_ins++;
          print "004 IO-GROUP: id_group - $read_io - $write_io - $read - $write - $resp_t_r - $resp_t_w - $read_transfer_size - $write_transfer_size - $time\n" if $DEBUG == 3;
          RRDp::cmd qq(update "$rrd" $time:$read:$write:$read_io:$write_io:$resp_t_r:$resp_t_w:$read_transfer_size:$write_transfer_size);
          my $answer = RRDp::read;
          $time_last_ok = $time;
        } ## end if ( isdigit($time) &&...)
      }
    }

    # Node Cache
    ########################

    if ( $type =~ m/NODE-CACHE/ ) {
      print "000 $type: $line \n" if $DEBUG == 3;
	  print "000 $type: $line \n";
      next;    # not implemented so far
    }
  }    # foreach

  # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
  if ( $pool_time > $pool_ltime ) {
    my @none = "";
    pool_sum_insert(
      $type,  $wrkdir, $step,  $DEBUG, $host,  $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@none, \@none, \@none,
      \@none, \@none,  \@none, \@none, \@none, \@none,   \@none,    \@none,   \@none,     \@none,      \@none,      \@none,       \@none,         \@none,          \@none
    );
  } ## end if ( $pool_time > $pool_ltime)

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_svc_perf

sub rrd_error {
  my $err_text = shift;
  my $rrd_file = shift;
  my $basedir  = $ENV{INPUTDIR};
  my $tmpdir   = "$basedir/tmp";
  if ( defined $ENV{TMPDIR_STOR} ) {
    $tmpdir = $ENV{TMPDIR_STOR};
  }

  chomp($err_text);

  if ( !-f "$rrd_file" ) {
    return 0;
  }

  if ( $err_text =~ m/ERROR:/ && $err_text !~ m/This RRD was created on another architecture/ ) {
    # copy of the corrupted file into "save" place and remove the original one
    copy( "$rrd_file", "$tmpdir/" ) || main::error( "Cannot: cp $rrd_file $tmpdir/: $! " . __FILE__ . ":" . __LINE__ );
    unlink("$rrd_file") || main::error( "Cannot rm $rrd_file : $! " . __FILE__ . ":" . __LINE__ );
    main::error( "$err_text, moving it into: $tmpdir/ " . __FILE__ . ":" . __LINE__ );
  } ## end if ( $err_text =~ m/ERROR:/)
  else {
    # exlude "This RRD was created on another architecture" as this might happen after Linux upgrade for example, this is not a corruption
    main::error( "$err_text " . __FILE__ . ":" . __LINE__ );
  }
  return 0;
} ## end sub rrd_error

# check if RANK already does not exist under different POOL ID, if so then rename it and do not create a new rrd file
sub rename_only {
  my $rrd   = shift;
  my $host  = shift;
  my $DEBUG = shift;

  my $rrd_find = $rrd;
  $rrd_find =~ s/-P[0-9][0-9]*\.rrd/-P\*\.rrd/;    # preparing search string

  my $rank_found_file = "";
  my $rank_found_time = 0;
  foreach my $file (<$rrd_find*>) {
    if ( $file !~ m/-P[0-9][0-9]*\.rrd/ ) {
      next;                                        # exlude 0001-P.rrd --> ranks without valid pool ID
    }
    my $file_time = ( stat("$file") )[9];
    if ( $file_time > $rank_found_time ) {

      # if more old rank files the select the most recent
      $rank_found_time = $file_time;
      $rank_found_file = $file;
    } ## end if ( $file_time > $rank_found_time)
  } ## end foreach my $file (<$rrd_find*>)

  if ( $rank_found_time > 0 ) {

    # rename it
    print "RANK rename    : $host $rank_found_file --> $rrd\n" if $DEBUG;
    if ( -f "$rank_found_file" ) {
      rename( "$rank_found_file", "$rrd" ) || main::error( " Cannot mv $rank_found_file $rrd: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    }
    return 1;
  } ## end if ( $rank_found_time ...)
  return 0;
} ## end sub rename_only

sub pool_sum_insert {
  my (
    $type,          $wrkdir,         $step, $DEBUG, $host, $no_time, $act_time, $st_type, $time, $ltime, $read_tmp, $write_tmp, $io_read_tmp, $io_write_tmp, $resp_r_tmp, $resp_w_tmp, $pool_ids_tmp, $data_rate_tmp, $io_rate_tmp, $resp_t_tmp,
    $read_size_tmp, $write_size_tmp, $data_size_tmp
  ) = @_;

  my @pool_ids   = @{$pool_ids_tmp};
  my @read       = @{$read_tmp};
  my @write      = @{$write_tmp};
  my @io_read    = @{$io_read_tmp};
  my @io_write   = @{$io_write_tmp};
  my @resp_t_r   = @{$resp_r_tmp};
  my @resp_t_w   = @{$resp_w_tmp};
  my @data_rate  = @{$data_rate_tmp};
  my @io_rate    = @{$io_rate_tmp};
  my @resp_t     = @{$resp_t_tmp};
  my @read_size  = @{$read_size_tmp};
  my @write_size = @{$write_size_tmp};
  my @data_size  = @{$data_size_tmp};
  my $last_rec   = $ltime;
  my $count      = 0;
  my $type_org   = $type;

  if ( $st_type =~ m/^3PAR/ && $type =~ m/^DRIVE/ ) {
    $type = "TIER";
    print "TIER summ      : $host\n" if $DEBUG;
  }
  else {
    # for all the rest it is always POOL
    $type = "POOL";
    print "POOL summ      : $host\n" if $DEBUG;
  }

  #print "003 $host $type_org $st_type\n" if $DEBUG ;
  # check if it is netapp (uuid conversion and enhanced data about data_rate, io_rate and resp_t
  if ( $st_type =~ m/NETAPP/ || $st_type =~ m/XIV/ ) {
    foreach my $uuid_act (@pool_ids) {
      if ( !defined($uuid_act) || $uuid_act eq '' ) {
        next;
      }
      $count++;
      last;
    } ## end foreach my $uuid_act (@pool_ids)
  } ## end if ( $st_type =~ m/NETAPP/...)
  my $uuid_trans = 0;
  if ( $count > 0 ) {
    $uuid_trans = 1;
  }
  $count = 0;

  if ( ( $st_type =~ m/NETAPP/ || $st_type =~ m/XIV/ ) && $uuid_trans == 0 ) {

    # uuid table is empty, do not continue for netapp further
    main::error( "$host: UUID table is empty, no pools (aggregates) were identified " . __FILE__ . ":" . __LINE__ );
    return $ltime;
  } ## end if ( ( $st_type =~ m/NETAPP/...))

  my $pool_id = -1;
  foreach my $id (@pool_ids) {
    $pool_id++;
    if ( !defined($id) || $id eq '' ) {
      next;    # there is a gap in pool id row, no problem at all ...
    }
    if ( $st_type !~ m/NETAPP/ && $st_type !~ m/XIV/ ) {

      # for netapp tehre is UUID, for XIV hexa, the rest has there ID
      $pool_id = $id;
    }

    # pool by pool
    if ( !defined( $read[$pool_id] ) || $read[$pool_id] eq '' || !isdigit( $read[$pool_id] ) ) {
      $read[$pool_id] = -1;
    }
    if ( !defined( $write[$pool_id] ) || $write[$pool_id] eq '' || !isdigit( $write[$pool_id] ) ) {
      $write[$pool_id] = -1;
    }
    if ( !defined( $io_read[$pool_id] ) || $io_read[$pool_id] eq '' || !isdigit( $io_read[$pool_id] ) ) {
      $io_read[$pool_id] = -1;
    }
    if ( !defined( $io_write[$pool_id] ) || $io_write[$pool_id] eq '' || !isdigit( $io_write[$pool_id] ) ) {
      $io_write[$pool_id] = -1;
    }
    if ( !defined( $resp_t_r[$pool_id] ) || $resp_t_r[$pool_id] eq '' || !isdigit( $resp_t_r[$pool_id] ) ) {
      $resp_t_r[$pool_id] = -1;
    }
    if ( !defined( $resp_t_w[$pool_id] ) || $resp_t_w[$pool_id] eq '' || !isdigit( $resp_t_w[$pool_id] ) ) {
      $resp_t_w[$pool_id] = -1;
    }
    if ( !defined( $resp_t[$pool_id] ) || $resp_t[$pool_id] eq '' || !isdigit( $resp_t[$pool_id] ) ) {
      $resp_t[$pool_id] = -1;
    }
    if ( !defined( $read_size[$pool_id] ) || $read_size[$pool_id] eq '' || !isdigit( $read_size[$pool_id] ) ) {
      $read_size[$pool_id] = -1;
    }
    if ( !defined( $write_size[$pool_id] ) || $write_size[$pool_id] eq '' || !isdigit( $write_size[$pool_id] ) ) {
      $write_size[$pool_id] = -1;
    }
    if ( !defined( $data_size[$pool_id] ) || $data_size[$pool_id] eq '' || !isdigit( $data_size[$pool_id] ) ) {
      $data_size[$pool_id] = -1;
    }

    # non netapp
    my $rrd = "$wrkdir/$host/$type/$pool_id\.rrd";
    if ( $st_type =~ m/3PAR/ && $type =~ m/TIER/ ) {
      my $tier = "unknown";
      if ( $pool_id == 1 ) {
        $tier = "FC";
      }
      if ( $pool_id == 2 ) {
        $tier = "SSD";
      }
      if ( $pool_id == 3 ) {
        $tier = "NL";
      }
      if ( $pool_id == 4 ) {
        $tier = "SAS";
      }
      if ( $tier =~ m/unknown/ ) {

        # do not put there any "unknown" tier
        print "TIER create    : unknown TIER : $pool_id, ignoring\n";
        next;
      } ## end if ( $tier =~ m/unknown/)
      $rrd = "$wrkdir/$host/$type/$tier\.rrd";
    } ## end if ( $st_type =~ m/3PAR/...)

    if ( $resp_t_r[$pool_id] > 0 && $io_read[$pool_id] > 0 ) {
      $resp_t_r[$pool_id] = sprintf( "%.3f", $resp_t_r[$pool_id] / $io_read[$pool_id] );
    }
    else {
      $resp_t_r[$pool_id] = 'U';
    }
    if ( $resp_t_w[$pool_id] > 0 && $io_write[$pool_id] > 0 ) {
      $resp_t_w[$pool_id] = sprintf( "%.3f", $resp_t_w[$pool_id] / $io_write[$pool_id] );
    }
    else {
      $resp_t_w[$pool_id] = 'U';
    }
    if ( $resp_t[$pool_id] > 0 && $io_rate[$pool_id] > 0 ) {
      $resp_t[$pool_id] = sprintf( "%.3f", $resp_t[$pool_id] / $io_rate[$pool_id] );
    }
    else {
      $resp_t[$pool_id] = 'U';
    }

    if ( $read_size[$pool_id] > 0 && $io_read[$pool_id] > 0 ) {
      $read_size[$pool_id] = sprintf( "%.3f", $read_size[$pool_id] / $io_read[$pool_id] );
    }
    else {
      $read_size[$pool_id] = 'U';
    }
    if ( $write_size[$pool_id] > 0 && $io_write[$pool_id] > 0 ) {
      $write_size[$pool_id] = sprintf( "%.3f", $write_size[$pool_id] / $io_write[$pool_id] );
    }
    else {
      $write_size[$pool_id] = 'U';
    }
    if ( $data_size[$pool_id] > 0 && $io_rate[$pool_id] > 0 ) {
      $data_size[$pool_id] = sprintf( "%.3f", $data_size[$pool_id] / $io_rate[$pool_id] );
    }
    else {
      $data_size[$pool_id] = 'U';
    }

    if ( isdigit( $io_read[$pool_id] ) && isdigit( $io_write[$pool_id] ) && $io_read[$pool_id] > -1 && $io_write[$pool_id] > -1 && !isdigit( $io_rate[$pool_id] ) ) {
      $io_rate[$pool_id] = $io_read[$pool_id] + $io_write[$pool_id];
    }

    if ( isdigit( $read[$pool_id] ) && isdigit( $write[$pool_id] ) && $read[$pool_id] > -1 && $write[$pool_id] > 0 && !isdigit( $data_rate[$pool_id] ) ) {
      $data_rate[$pool_id] = $read[$pool_id] + $write[$pool_id];
    }

    if ( $uuid_trans > 0 ) {

      # NetApp
      $rrd = "$wrkdir/$host/$type/$id\.rrd";
    }

    #create rrd db if necessary
    $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
    if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
      return 2;        # leave uit if any error during RRDTool file creation to keep data
    }

    if ( $ltime == 0 ) {

      # find out last record in the db
      # as this makes it slowly to test it each time then it is done
      # once per a lpar for whole load and saved into the array
      eval {
        RRDp::cmd qq(last "$rrd" );
        $last_rec = RRDp::read;
      };
      if ($@) {
        rrd_error( $@, $rrd );
        next;
      }
      chomp($$last_rec);
      $ltime = $$last_rec;
    } ## end if ( $ltime == 0 )

    # Update only latest data
    if ( isdigit($time) && isdigit($ltime) && $time > $$last_rec ) {

      #print "004: pool_id:$pool_id - $time:$io_read[$pool_id]:$io_write[$pool_id]:$read[$pool_id]:$write[$pool_id]:$resp_t_r[$pool_id]:$resp_t_w[$pool_id]\n";

      if ( !defined( $io_rate[$pool_id] )    || $io_rate[$pool_id] eq '' )    { $io_rate[$pool_id]    = 'U'; }
      if ( !defined( $io_read[$pool_id] )    || $io_read[$pool_id] eq '' )    { $io_read[$pool_id]    = 'U'; }
      if ( !defined( $io_write[$pool_id] )   || $io_write[$pool_id] eq '' )   { $io_write[$pool_id]   = 'U'; }
      if ( !defined( $resp_t[$pool_id] )     || $resp_t[$pool_id] eq '' )     { $resp_t[$pool_id]     = 'U'; }
      if ( !defined( $resp_t_r[$pool_id] )   || $resp_t_r[$pool_id] eq '' )   { $resp_t_r[$pool_id]   = 'U'; }
      if ( !defined( $resp_t_w[$pool_id] )   || $resp_t_w[$pool_id] eq '' )   { $resp_t_w[$pool_id]   = 'U'; }
      if ( !defined( $read[$pool_id] )       || $read[$pool_id] eq '' )       { $read[$pool_id]       = 'U'; }
      if ( !defined( $write[$pool_id] )      || $write[$pool_id] eq '' )      { $write[$pool_id]      = 'U'; }
      if ( !defined( $data_rate[$pool_id] )  || $data_rate[$pool_id] eq '' )  { $data_rate[$pool_id]  = 'U'; }
      if ( !defined( $read_size[$pool_id] )  || $read_size[$pool_id] eq '' )  { $read_size[$pool_id]  = 'U'; }
      if ( !defined( $write_size[$pool_id] ) || $write_size[$pool_id] eq '' ) { $write_size[$pool_id] = 'U'; }
      if ( !defined( $data_size[$pool_id] )  || $data_size[$pool_id] eq '' )  { $data_size[$pool_id]  = 'U'; }

      if ( $st_type =~ m/DS5K/ ) {
        RRDp::cmd qq(update "$rrd" $time:$data_rate[$pool_id]:$io_rate[$pool_id]:$resp_t[$pool_id]:U:U:U);
      }
      else {
        if ( $st_type =~ m/NETAPP/ || $st_type =~ m/XIV/ ) {

          # last 2 are : free:cap, not used yet
          RRDp::cmd qq(update "$rrd" $time:$read[$pool_id]:$write[$pool_id]:$io_read[$pool_id]:$io_write[$pool_id]:$resp_t_r[$pool_id]:$resp_t_w[$pool_id]:$data_rate[$pool_id]:$io_rate[$pool_id]:$resp_t[$pool_id]:U:U:U);
        }
        else {
          if ( $st_type =~ m/3PAR/ ) {

            #print "004: $type_org pool_id:$pool_id - $time:$read[$pool_id]:$write[$pool_id]:$io_read[$pool_id]:$io_write[$pool_id]:$resp_t_r[$pool_id]:$resp_t_w[$pool_id]:$data_rate[$pool_id]:$io_rate[$pool_id]:$resp_t[$pool_id]\n";
            # last 3 capacity are not used yet
            if ( $type_org =~ m/DRIVE/ ) {

              # TIER
              RRDp::cmd qq(update "$rrd" $time:$read[$pool_id]:$write[$pool_id]:$io_read[$pool_id]:$io_write[$pool_id]:$resp_t_r[$pool_id]:$resp_t_w[$pool_id]:$data_rate[$pool_id]:$io_rate[$pool_id]:$resp_t[$pool_id]:$read_size[$pool_id]:$data_size[$pool_id]:$data_size[$pool_id]:U:U:U]);
            }
            else {
              # POOL
              RRDp::cmd qq(update "$rrd" $time:$read[$pool_id]:$write[$pool_id]:$io_read[$pool_id]:$io_write[$pool_id]:$resp_t_r[$pool_id]:$resp_t_w[$pool_id]:$data_rate[$pool_id]:$io_rate[$pool_id]:$resp_t[$pool_id]:U:U:U]);
            }
          } ## end if ( $st_type =~ m/3PAR/)
          else {
            # SWIZ
            RRDp::cmd qq(update "$rrd" $time:$read[$pool_id]:$write[$pool_id]:$io_read[$pool_id]:$io_write[$pool_id]:$resp_t_r[$pool_id]:$resp_t_w[$pool_id]);
          }
        } ## end else [ if ( $st_type =~ m/NETAPP/...)]
      } ## end else [ if ( $st_type =~ m/DS5K/)]
      my $answer = RRDp::read;
      $ltime = $time if !$$answer;
      $count++;
    } ## end if ( isdigit($time) &&...)

    #print "003 --$pool_id-- $read[$pool_id] - $write[$pool_id] - $io_read[$pool_id] - $io_write[$pool_id] - $resp_t_r[$pool_id] - $resp_t_w[$pool_id]\n";
  } ## end foreach my $id (@pool_ids)
  print "$type summ      : $host inserted: $count\n" if $DEBUG;

  return $ltime;
} ## end sub pool_sum_insert

sub urlencode {
  my $s = shift;
  $s =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
  return $s;
}

sub port_mapping {
  my $subsystem = shift;    # port mapping
  my $wrkdir    = shift;
  my $host      = shift;
  my $act_time  = shift;
  my $st_type   = shift;
  my $DEBUG     = shift;
  my $webdir    = shift;
  my @lines;
  my $active        = 0;
  my $port_header   = "PORT";
  my $host_header   = "HOST";
  my $group_header  = "HOST GROUP";
  my $volume_header = "VOLUME";

  my $style = "<head><style> td.tab {vertical-align:top} td.host {background-color:#FF8080; white-space: nowrap; text-align:center} td.group {background-color:#80FF80; white-space: nowrap; text-align:center} td.port {background-color:#9beff4; white-space: nowrap; text-align:center} td.volume{background-color:#FFE4B5; white-space: nowrap; text-align:center} table {border-spacing: 1px;}</style></head>";

  #my $html_table = "<!DOCTYPE html><html>$style<body><table><tbody><tr>";

  #print "...............$subsystem : $wrkdir : $host : $act_time : $st_type : $DEBUG............................................\n";

  my $header_port_map = "Port-Mapping";
  my $end_find        = "</pre>";

  if ( defined $ENV{DEMO} && ( $ENV{DEMO} == 1 ) ) {
    print "\n";
    print "DEMO is set, do not update por create port-mapping.html\n";
    print "\n";
    return 1;    # skip it on the demo site
  } ## end if ( defined $ENV{DEMO...})

  ###legend ###

  my $legend     = "<table><tbody><tr><td class=" . '"' . "host" . '"' . ">$host_header</td><td class=" . '"' . "group" . '"' . ">$group_header</td><td class=" . '"' . "port" . '"' . ">$port_header</td><td class=" . '"' . "volume" . '"' . ">$volume_header</td></tr></tbody></table>\n";
  my $space      = "<br><br><br>\n";
  my $html_table = "<!DOCTYPE html><html>$style<body>$legend $space<table><tbody><tr>";

  print "load nicks     : $host : $subsystem\n" if $DEBUG;
  open( DATA, "< $wrkdir/$host/config.html" ) || main::error( "$host: Can't open $wrkdir/$host/config.html (fresh install or upgrade?, then it is ok): $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  @lines = <DATA>;
  close(DATA);
  my $host_encode = urlencode($host);

  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" ) {
      $active = 0;
      next;
    }
    $line =~ s/^\s+|\s+$//g;

    if ( index( $line, $end_find ) > -1 ) {
      last;
    }
    if ( index( $line, $header_port_map ) > -1 ) {
      $active++;
      next;
    }
    if ( index( $line, "Host," ) > -1 && $active == 1 ) {
      $active++;
      next;
    }
    if ( $active == 2 ) {
      ( my $host_name, my $group, my $port, my $volume ) = split( ",", $line );

      #print "HOST $host\n";
      #print "GROUPS $group\n";
      #print "PORTS $port\n";
      #print "VOLUMES $volume\n";
      my $host_encode_name = urlencode($host_name);
      $html_table = $html_table . "<td class = " . '"' . "tab" . '"' . "><table><tbody><tr><td class =" . '"' . "host" . '"' . "><a href =" . '"' . "/stor2rrd-cgi/detail.sh?host=$host_encode&type=HOST&name=$host_encode_name&storage=$st_type&none=none" . '"' . "> $host_name</a></td></tr>\n";

      if ( index( $group, " " ) > -1 ) {
        my @groups = split( " ", $group );
        foreach my $group_element (@groups) {
          chomp $group_element;
          $group_element =~ s/^\s+|\s+$//g;
          $html_table = $html_table . "<tr><td class = " . '"' . "group" . '"' . ">$group_element</td></tr>\n";
        }
      } ## end if ( index( $group, " "...))
      else {
        chomp $group;
        $group =~ s/^\s+|\s+$//g;
        if ( $group ne "" ) {
          $html_table = $html_table . "<tr><td class = " . '"' . "group" . '"' . ">$group</td></tr>\n";
        }
      } ## end else [ if ( index( $group, " "...))]

      if ( index( $port, " " ) > -1 ) {
        my @ports = split( " ", $port );
        foreach my $port_element (@ports) {
          chomp $port_element;
          $port_element =~ s/^\s+|\s+$//g;
          my $port_encode = urlencode($port_element);
          $html_table = $html_table . "<tr><td class = " . '"' . "port" . '"' . "> <a href =" . '"' . "/stor2rrd-cgi/detail.sh?host=$host_encode&type=PORT&name=$port_encode&storage=$st_type&none=none" . '"' . ">$port_element</a></td></tr>\n";
        } ## end foreach my $port_element (@ports)
      } ## end if ( index( $port, " "...))
      else {
        chomp $port;
        $port =~ s/^\s+|\s+$//g;
        if ( $port ne "" ) {
          my $port_encode = urlencode($port);
          $html_table = $html_table . "<tr><td class = " . '"' . "port" . '"' . "><a href =" . '"' . "/stor2rrd-cgi/detail.sh?host=$host_encode&type=PORT&name=$port_encode&storage=$st_type&none=none" . '"' . ">$port</a></td></tr>\n";
        }
      } ## end else [ if ( index( $port, " "...))]

      if ( index( $volume, " " ) > -1 ) {
        my @volumes = split( " ", $volume );
        foreach my $volume_element (@volumes) {
          chomp $volume_element;
          $volume_element =~ s/^\s+|\s+$//g;
          my $volume_encode = urlencode($volume_element);
          $html_table = $html_table . "<tr><td class = " . '"' . "volume" . '"' . "> <a href =" . '"' . "/stor2rrd-cgi/detail.sh?host=$host_encode&type=$volume_header&name=$volume_encode&storage=$st_type&none=none" . '"' . ">$volume_element</a></td></tr>\n";
        } ## end foreach my $volume_element ...
      } ## end if ( index( $volume, " "...))
      else {
        chomp $volume;
        $volume =~ s/^\s+|\s+$//g;
        if ( $volume ne "" ) {
          my $volume_encode = urlencode($volume);
          $html_table = $html_table . "<tr><td class = " . '"' . "volume" . '"' . "><a href =" . '"' . "/stor2rrd-cgi/detail.sh?host=$host_encode&type=VOLUME&name=$volume_encode&storage=$st_type&none=none" . '"' . ">$volume</a></td></tr>\n";
        }
      } ## end else [ if ( index( $volume, " "...))]
      $html_table = $html_table . "</tbody></table></td>";
    } ## end if ( $active == 2 )
  } ## end foreach my $line (@lines)

  my $info = "<br><br>It is updated every hour, last update : " . localtime() . "\n";
  $html_table = $html_table . "</tr></tbody></table>$info</html>";
  my $dir = "$webdir/$host";

  ### new add mapping cgi
  my $wraper = "<div id=\"tabs\">\n";
  $wraper = $wraper . "<ul>\n";
  $wraper = $wraper . "<li><a href=\"/stor2rrd-cgi/mapping.sh?host=$host" . "&" . "storage=$st_type\">host</a></li>\n";
  $wraper = $wraper . "<li><a href=\"$host/mapping-port.html\">port</a></li>\n";
  $wraper = $wraper . "</ul>\n";
  $wraper = $wraper . "</div>\n";

  open( FHW, "> $dir/mapping.html" ) || main::error( "$host: Can't open $dir/mapping.html: $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  print FHW "$wraper\n";
  close(FHW);
  open( FHW, "> $dir/mapping-port.html" ) || main::error( "$host: Can't open $dir/mapping-port.html: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHW "$html_table\n";
  close(FHW);

  #print "$html_table\n";

} ## end sub port_mapping

# It loads volumes nicks from config.html and sort them out and save them into VOLUME/volumes.cfg

sub load_nicks {
  my $subsystem = shift;    # volume, host, ldev
  my $wrkdir    = shift;
  my $host      = shift;
  my $act_time  = shift;
  my $st_type   = shift;
  my $DEBUG     = shift;

  my @vol_nicks = "";
  my @vol_id    = "";

  #my @ldev_nicks = "";
  #my @ldev_id = "";

  if ( defined $ENV{DEMO} && ( $ENV{DEMO} == 1 ) ) {
    print "\n";
    print "DEMO is set, do not update por create volumes.cfg and hosts.cfg\n";
    print "\n";
    return 1;    # skip it on the demo site
  } ## end if ( defined $ENV{DEMO...})

  print "load nicks     : $host : $subsystem\n" if $DEBUG;
  if ( !-f "$wrkdir/$host/config.html" ) {
    main::error( "$host: Can't open $wrkdir/$host/config.html is missing (fresh install or upgrade?, then wait about 20mins) " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  open( FHN, "< $wrkdir/$host/config.html" ) || main::error( "$host: Can't open $wrkdir/$host/config.html: $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines       = <FHN>;
  my @lines_valid = "";

  my $line  = "";
  my $found = 0;
  my $indx  = 0;

  # host search strings
  my $search_head   = "host_id,";
  my $search_head_1 = "host_id,";
  my $search_head_2 = "^,,name,,,,,Volume IDs,";    # HUS
  my $search_head_3 = "host_id,";                   # fake to have same number as for volumes
  my $search_head_4 = "Host name,";                 # VNX Host section
  my $search_head_5 = "HOST_ID,";                   # VMAX Host section
  my $search_head_6 = "id,host_id";                 # XIV host section
  if ( $subsystem =~ m/VOLUME/ ) {
    $search_head   = "Volume ID,";                  # DS8k
    $search_head_1 = "volume_id,";                  # storwize
    $search_head_2 = "lun_id,";                     # HUS
    $search_head_3 = "uuid,name,";                  # netApp
    $search_head_4 = "Name,lun_id,";                # VNX
    $search_head_5 = "SG_ID,time,timestamp";        # VMAX Storage groups (VOLUME)
  } ## end if ( $subsystem =~ m/VOLUME/)

  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/^$search_head/ || $line =~ m/^$search_head_1/ || $line =~ m/^$search_head_2/ || $line =~ m/^$search_head_3/ || $line =~ m/^$search_head_4/ || $line =~ m/^$search_head_5/ || $line =~ m/^$search_head_6/  ) {
      $found = 1;
      next;
    }
    if ( $found == 0 ) {
      next;
    }
    if ( $st_type eq 'DS5K' && $line =~ m/,no_perf_data$/ ) {
      next;
    }
    if ( $found == 1 ) {
      if ( $line eq '' || $line =~ m/<\/html>/ || $line =~ m/Level Configuration/ || $line =~ m/------------------/ ) {
        last;
      }
    }
    $lines_valid[$indx] = $line;
    $indx++;
  } ## end foreach $line (@lines)
  close(FHN);

  if ( $indx == 0 && $subsystem !~ m/HOST/ ) {

    # host info is loaded once a day!!!
    main::error( "$host: Problem with configuration load, no $subsystem has been found, is there really exist one? : $wrkdir/$host/config.html " . __FILE__ . ":" . __LINE__ );
    return 0;    # ignore the rest as there is not really volume cfg
  } ## end if ( $indx == 0 && $subsystem...)

  # sort per nicks
  {
    no warnings;    ### uninitialized value in string comparison (cmp)
    @lines_valid = sort { ( split ',', $a )[2] cmp( split ',', $b )[2] } @lines_valid;
  }
  $found = 0;
  $indx  = 0;
  my $vols = 0;

  my @host_nicks = "";
  my @host_id    = 0;

  foreach $line (@lines_valid) {
    chomp($line);

    $vols++;
    if ( $line eq '' ) {
      next;
    }

    my $id   = -1;
    my $nick = "na";
    my $ids  = "";
    if ( $st_type =~ m/3PAR/ && $subsystem =~ m/VOLUME/ ) {
      ( $id, $nick, my $pool_id ) = split( /,/, $line );
      $ids = '';
      if ( defined($pool_id) && $pool_id =~ m/^-$/ ) {
        next;    # internal volume of the storage, it has no pool id
      }
    } ## end if ( $st_type =~ m/3PAR/...)
    elsif ( $st_type eq "VMAX" && $subsystem =~ m/VOLUME/ ) {
      ( $id, undef ) = split( /,/, $line );
      $nick = $id;
    }
    elsif ( $st_type eq "VPLEX" && $subsystem =~ m/VOLUME/ ) {
      ( $id, undef ) = split( /,/, $line );
      $nick = $id;
    }
    elsif ( $st_type eq "VMAX" && $subsystem =~ m/HOST/ ) {
      ( $id, undef, undef, $ids ) = split( /,/, $line );
      $nick = $id;
    }
    elsif ( $st_type eq "VPLEX" && $subsystem =~ m/HOST/ ) {
      ( $id, undef, $ids ) = split( /,/, $line );
      $nick = $id;
    }
    else {
      if ( $st_type =~ m/NETAPP|VNXF|DOTHILL|COMPELLENT|OCEANSTOR|PURE/ && $subsystem =~ m/VOLUME/ ) {
        ( $id, $nick ) = split( /,/, $line );
      }
      else {
        if ( $st_type eq "VNX" || $st_type eq "UNITY" || $st_type eq "VNXE" ) {
          if ( $subsystem =~ m/VOLUME/ ) {
            ( $nick, $id ) = split( /,/, $line );
            if ( ! defined $nick || $nick eq "" ) {
              $nick = $id;
            }
          }
          if ( $subsystem =~ m/HOST/ && $st_type eq "VNX" ) {
            ( $nick, $ids ) = split( /,/, $line );
          }
          if ( $subsystem =~ m/HOST/ && $st_type eq "UNITY" ) {
            ( $nick, undef, undef, undef, $ids ) = split( /,/, $line );

            #Host name,Host ID,Address,OS type,Volume IDs,Volume Names
          }
          if ( $subsystem =~ m/HOST/ && $st_type eq "VNXE" ) {
            ( $nick, undef, undef, undef, $ids ) = split( /,/, $line );

            #Host name,Host ID,Address,OS type,Volume IDs,Volume Names
          }
        } ## end if ( $st_type eq "VNX"...)
        else{
          if ( $subsystem =~ m/HOST/ && $st_type eq "XIV" ) {
            ( $id, my $sn, $nick, my $t1, my $t2, $ids ) = split( /,/, $line );
          }
          else {
            ( $id, my $sn, $nick, my $t1, my $t2, my $t3, my $t4, $ids ) = split( /,/, $line );
          }
        }
      } ## end else [ if ( $st_type =~ m/NETAPP|VNXF|DOTHILL|COMPELLENT|OCEANSTOR/...)]
    } ## end else [ if ( $st_type =~ m/3PAR/...)]

    # NetApp
    # uuid,name,containing-aggregate-uuid,containing-aggregate-name,owning-vserver-uuid,owning-vserver-name,type,instance-uuid,state,size-total,size-available,filesystem-size,size-used,percentage-used,files-used,files-total,snapshot-percent-reserved
    # 8af39b88-9961-4038-8ede-135aa81bd5ac,BigOne,,aggr2,,,flex,64c9f0ff-1a20-4c4f-8731-108db097bc2c,online,1.000,0.372,1.000,0.628,63,98,31122,0
    # 3PAR
    # Volume ID,Volume name,CPG ID,CPG name,Status,Provisioning,Type,Copy Of,Admin size [MB],Snap size [MB],User size [MB],Real size [MB],Virtual size [MB],^M
    # 35,DL980-NAS1,1,FC_r5,normal,tpvv,base,---,640,16256,514048,530944,614400,^M
    # DS8K
    # Volume ID,Volume SN,Volume Nickname,LSS Number,Volume Number,Volume Size,Volume Type
    # 0x0000,IBM.2107-75ANM81/0000,INPH10,0x00,0x00,1.000000,FB 512
    # SVC
    # volume_id,id,name,IO_group_id,IO_group_name,status,mdisk_grp_id,mdisk_grp_name,capacity,type,FC_id,FC_name,RC_id,RC_name,
    #    vdisk_UID,fc_map_count,copy_count,fast_write_state,se_copy_count, RC_change,compressed_copy_count
    # 0000,0,A3105005rvg,0,io_grp0,online,1,PMDSK1_SAS_600,128849018880,striped,,,,,600507680280827CA800000000000000,0,1,empty,0,no,0
    # DS5K (no IDs)
    # ,,ASRV11LPAR7,,,,,60080e500018469e00000f2d4d885cf0 60080e500018450a0000aed84dccba79
    # HUS ( no names)
    # lun_id,,,,,,pool_id,,capacity (MB),,,,,,,,,,,,interface_type
    # 0,,,,,,1,,8388608,,,,,,,,,,,,SAS
    # 3PAR
    #
    # VNX volumes
    # Name,lun_id,Current_owner,RG_ID,Pool name,UID,LUN_Capacity (MB),LUN_Capacity (Blocks)
    # VMware-raid1-01,0,SP B,0,,60:06:01:60:69:10:2D:00:33:0C:7D:4E:44:61:E6:11,102400,209715200
    # VNX hosts
    # Host name,Volume IDs,Volume Names
    # p710,0 1 2 4 5 6,VMware-raid1-01 VMware-raid5-01 VMware-raid5-02 VMware-test-10G VMware-raid5-pool-01 VMware-raid5-pool-02
    #
    # VMAX volumes (storage groups)
    # ID,time,timestamp
    # migr_DATA_AIX-SG1PROD_DCH,2017:03:16T10:15:00,1489655700
    # VMAX hosts
    # HOST_ID,time,timestamp,storage groups
    # IG_IT10CZ0008,2017:04:19T13:50:00,1492602600,SG_UNIS_GTK SG_VMWARE

    #host_id,id,name,port_count,iogrp_count,status,WWPN,Volume IDs,Volume Names
    #0002,2,pmnim1,2,4,online,10000000C9FD5FF4 10000000C9FD3784,57 58 59 60,pmnim1bvg1 pmnim1bvg2 pmnim1bvg3 pmnim1bvg4

    if ( !defined($nick) ) {
      next;    # somehing strange ...
    }

    if ( $subsystem =~ m/HOST/ ) {
      if ( $nick eq '' ) {
        next;    # somehing strange ...
      }
      if ( !defined($ids) ) {
        $ids = "";
      }
      $ids =~ s/ 0x/ /g;    # filter hexa prefix
      $ids =~ s/^0x//;      # filter hexa prefix
      $host_nicks[$indx] = $nick;
      $host_id[$indx]    = $ids;
      $indx++;
    } ## end if ( $subsystem =~ m/HOST/)

    if ( $subsystem =~ m/VOLUME/ ) {
      if ( !defined($id) || $id eq '' ) {
        next;               # somehing strange ...
      }
      if ( $nick eq '' ) {
        $nick = $id;
      }

      if ( $vol_nicks[$indx] eq '' ) {

        # only for the first run!
        $vol_nicks[$indx] = $nick;
        $vol_id[$indx]    = $id . ";";
        next;
      } ## end if ( $vol_nicks[$indx]...)

      if ( $vol_nicks[$indx] =~ m/^$nick/ ) {
        $vol_id[$indx] .= $id . ";";
      }
      else {
        $indx++;
        $vol_nicks[$indx] = $nick;
        $vol_id[$indx]    = $id . ";";

        #print "$vol_nicks[$indx] $vol_id[$indx] \n";
      } ## end else [ if ( $vol_nicks[$indx]...)]
    } ## end if ( $subsystem =~ m/VOLUME/)

  } ## end foreach $line (@lines_valid)

  # save old structure, in install-html.sh make a diff and when anything diff then run install-html.sh
  # convert the string to lowercase
  my $subsystem_low = lc $subsystem;

  my $subsystem_file = $subsystem_low . "s.cfg";
  copy( "$wrkdir/$host/$subsystem/$subsystem_file", "$wrkdir/$host/$subsystem/$subsystem_file-old" );

  # save structured and grouped volumes per their nick
  open( FHW, "> $wrkdir/$host/$subsystem/$subsystem_file-act" ) || main::error( "$host: Can't open $wrkdir/$host/$subsystem/$subsystem_file-act: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  $indx = 0;
  if ( $subsystem =~ m/VOLUME/ ) {
    @host_nicks = @vol_nicks;
  }

  foreach $line (@host_nicks) {
    if ( $line eq '' ) {
      next;
    }
    if ( $subsystem =~ m/VOLUME/ ) {
      print FHW "$line : $vol_id[$indx] \n";
    }
    else {
      print FHW "$line : $host_id[$indx] \n";
    }
    $indx++;
  } ## end foreach $line (@host_nicks)
  close(FHW);

  if ( $indx == 0 ) {
    return 1;    # nothing has been found, it could be for HOST info which is downloaded just once a day
  }
  if ( $subsystem =~ m/VOLUME/ && $st_type eq "3PAR" ) {
    copy( "$wrkdir/$host/$subsystem/$subsystem_file-act", "$wrkdir/$host/VVOL/$subsystem_file-act" );
  }

  unlink("$wrkdir/$host/$subsystem/$subsystem_file");
  if ( $subsystem =~ m/VOLUME/ && $st_type eq "3PAR" ) {
    unlink("$wrkdir/$host/VVOL/$subsystem_file");
  }
  if ( -f "$wrkdir/$host/$subsystem/$subsystem_file-act" ) {
    rename( "$wrkdir/$host/$subsystem/$subsystem_file-act", "$wrkdir/$host/$subsystem/$subsystem_file" );
  }
  if ( $subsystem =~ m/VOLUME/ && $st_type eq "3PAR" && -f "$wrkdir/$host/VVOL/$subsystem_file-act" ) {
    rename( "$wrkdir/$host/VVOL/$subsystem_file-act", "$wrkdir/$host/VVOL/$subsystem_file" );
  }
  print "loaded nicks   : $host $indx ($subsystem_low: $vols) \n" if $DEBUG;

  my $diff = compare( "$wrkdir/$host/$subsystem/$subsystem_file", "$wrkdir/$host/$subsystem/$subsystem_file-old" );

  if ( $diff > 0 ) {
    print "$subsystem_low cfg changed: $diff \n" if $DEBUG;
    LoadDataModule::touch();
  }

  return 0;
} ## end sub load_nicks

return 1;

##################################################
#    NetAPP section
##################################################

sub load_data_netapp_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # at first config files like xivconf_20130904_115351.out
  #

  my $perf_string = "netappconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_20.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_netapp_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # now data files like xiv_svcperf_20130904_115351.out
  #

  $perf_string = "netappperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_20.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;
      it is not finised input data file;
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_netapp_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_netapp_all

sub load_data_netapp_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $lun_file          = "$wrkdir/$host/LUN/luns.cfg";
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $drive_file        = "$wrkdir/$host/drive.cfg";
  my $nas_file          = "$wrkdir/$host/NAS/nas.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $pool_file_write   = 0;                              # true if any write there
  my $config_file_write = 0;                              # true if any write there
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHD,  "> $drive_file-tmp" )  || main::error( "Can't open $drive_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FHN,  "> $nas_file-tmp" )    || main::error( "Can't open $nas_file-tmp : $! " . __FILE__ . ":" . __LINE__ )   && return 0;
  open( FHL,  "> $lun_file-tmp" )    || main::error( "Can't open $lun_file-tmp : $! " . __FILE__ . ":" . __LINE__ )   && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD> 
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > 
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @lun_name              = "";
  my @lun_name_id           = "";
  my @pool_name             = "";
  my @pool_name_id          = "";
  my @pool_capacity_tot     = "";
  my @pool_capacity_free    = "";
  my @pool_cfg_id           = "";
  my @rank_cfg_id           = "";
  my $pool_indx             = 0;
  my $lun_cfg_print         = 0;
  my $cfg_print             = 1;
  my $drive_cfg_print       = 1;
  my $nas_cfg_print         = 1;
  my $lun_cfg_processing    = 0;
  my $pool_cfg_processing   = 0;
  my $drive_cfg_processing  = 0;
  my $volume_cfg_processing = 0;
  my @nas_cfg               = "";

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/Drive Level Configuration/ ) {
      $drive_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/LUN Level Configuration/ ) {
      $lun_cfg_processing = 1;
      next;
    }

    if ( $line =~ m/Volume Level Configuration/ ) {
      $volume_cfg_processing = 1;
      next;
    }

    if ($volume_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $volume_cfg_processing = 0;
        next;    # end of cfg
      }
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^uuid,name,/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      #uuid,name,containing-aggregate-uuid,containing-aggregate-name,owning-vserver-uuid,owning-vserver-name,type,instance-uuid,state,size-total,size-available,filesystem-size,size-used,percentage-used,files-used,files-total,snapshot-percent-reserved,node_name
      #18d16b60-91d5-49e6-8c55-191a19624cd2,vol0 (node-01),7eb99482-f039-46c0-97f2-c8ebe5f9f78d,aggr0,9a72f152-fd78-11e5-8210-db85c0f85098,node-01,flex,f665296d-1664-4f05-bc25-16c23aaa2670,online,4.7500,0.7508,5.0000,3.9992,84,18466,155630,5,node-01

      my ( undef, undef, undef, undef, $nas_id, $nas_name, undef ) = split( /,/, $line );
      if ( defined $nas_id && $nas_id ne '' && defined $nas_name && $nas_name ne '' ) {
        push( @nas_cfg, "$nas_id,$nas_name\n" );
      }
      next;
    } ## end if ($volume_cfg_processing)

    if ($lun_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $lun_cfg_processing = 0;
        next;    # end of cfg
      }
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^uuid,name,/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      # uuid,name,node_name,serial-number,block-size,staging,is-space-reservation-enabled,alignment,size,size-used,suffix-size,multiprotocol-type,share-state,online,prefix-size,read-only,mapped
      # b360c63e-02a6-4632-a55a-98c63321ce8b,/vol/vol0/lun0,,BpEw0$Hdrf4n,512,false,true,indeterminate,0.000,0.000,0,linux,none,true,0,false,false

      my ( $lun_id_cfg_item, $lun_name_cfg_item ) = split( /,/, $line );
      if ( $lun_name_cfg_item eq '' ) {
        $lun_name_cfg_item = $lun_id_cfg_item;
      }

      # save actual lun cfg
      print FHL "$lun_name_cfg_item : $lun_id_cfg_item\n";
      $lun_cfg_print++;
      next;
    } ## end if ($lun_cfg_processing)

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;
        next;    # end of cfg
      }
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^name,id/ ) {
        next;    # some trash like "---"
      }

      $cfg_load_data_ok = 1;

      # create an array with mapping table between ranks and pools
      # name,id,node_name,node_uuid,status,capacity,used_capacity
      # aggr0,107d3696-f7ac-4db8-9adc-48208635e9a0,xorux-01,,,0.000,0.000

      my ( $pool_name_cfg_item, $pool_id_cfg_item ) = split( /,/, $line );
      if ( $pool_name_cfg_item eq '' ) {
        $pool_name_cfg_item = $pool_id_cfg_item;
      }

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)

    if ($drive_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $drive_cfg_processing = 0;
        next;    # end of cfg
      }
      $line =~ s/^0x//;

      if ( $line =~ m/^---/ || $line =~ m/^disk-uid/ ) {
        next;    # some trash like "---"
      }

      # create an array with mapping table between drive UUID and drive name
      # disk-uid,name,id,port-name,raid-group,used-space,disk_capacity,port,raid-type,vendor-id,disk-type,node,plex,used-blocks,pool,aggregate,serial-number,disk-model,effective-disk-type,host-adapter,physical-space
      # 4E455441:50502020:56442D34:3030304D:422D465A:2D353230:31313738:31383030:00000000:00000000,NET-1.16,,rg0,0.491,,,,NETAPP,FCAL,xorux-01,plex0,1029248,Pool0,aggr0,11781800,VD-4000MB-FZ-520,FCAL,,0.492

      my ( $drive_uuid, $drive_name ) = split( /,/, $line );
      if ( $drive_name eq '' ) {
        $drive_name = $drive_uuid;
      }
      print FHD "$drive_uuid,$drive_name\n";
      $drive_cfg_print++;
      next;
    } ## end if ($drive_cfg_processing)
  }    # foreach

  #add nas id and nas name to nas.cfg
  @nas_cfg = sort @nas_cfg;
  my $last_found_nas = "";

  foreach my $nas_line (@nas_cfg) {
    chomp $nas_line;
    my $found = $nas_line;
    if ( $found eq $last_found_nas ) { next; }
    $last_found_nas = $found;

    my ( $nas_id, $nas_name ) = split( ",", $nas_line );
    print FHN "$nas_id:$nas_name\n";
    $nas_cfg_print++;
  } ## end foreach my $nas_line (@nas_cfg)

  close(FHW);
  close(FHD);
  close(FHN);
  close(FHL);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  if ( $lun_cfg_print > 0 && -f "$lun_file-tmp" ) {
    print "cfg processed  : $host LUN rows: $lun_cfg_print\n" if $DEBUG;
    rename( "$lun_file-tmp", "$lun_file" );
  }
  if ( $drive_cfg_print > 0 && -f "$drive_file-tmp" ) {
    print "cfg processed  : $host drive rows: $drive_cfg_print\n" if $DEBUG;
    rename( "$drive_file-tmp", "$drive_file" );
  }
  if ( $nas_cfg_print > 0 && -f "$nas_file-tmp" ) {
    print "cfg processed  : $host nas rows: $nas_cfg_print\n" if $DEBUG;
    rename( "$nas_file-tmp", "$nas_file" );
  }

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 1 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);
  return 0;
} ## end sub load_data_netapp_conf

sub load_data_netapp_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there
  my $config_file_write = 0;                       # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  # pool fron-end data from volume data
  my @pool_read      = "";
  my @pool_write     = "";
  my @pool_data_rate = "";
  my @pool_io_read   = "";
  my @pool_io_write  = "";
  my @pool_io_rate   = "";
  my @pool_resp_r    = "";
  my @pool_resp_w    = "";
  my @pool_resp      = "";
  my @pool_ids       = "";
  my $pool_time      = 0;
  my $pool_ltime     = 0;

  my $nas_index = 0;
  my %nas_data;

  #$DEBUG=2;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      $type = uc($type_temp);
      next;
    }

    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/       //g;
      $line =~ s/Interval End:     //g;
      $line =~ s/GMT.*//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( !defined($time) || isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ m/^$/ ) {
      if ( $pool_time > 0 ) {

        # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
        if ( $pool_time > $pool_ltime ) {
          my @none = "";
          $pool_ltime = pool_sum_insert( $type, $wrkdir, $step, $DEBUG, $host, $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@pool_data_rate, \@pool_io_rate, \@pool_resp, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none, \@none );
        }

        # reinitialize structures
        @pool_ids       = "";
        @pool_read      = "";
        @pool_write     = "";
        @pool_data_rate = "";
        @pool_io_read   = "";
        @pool_io_write  = "";
        @pool_io_rate   = "";
        @pool_resp_r    = "";
        @pool_resp_w    = "";
        @pool_resp      = "";
        $pool_time      = 0;
      } ## end if ( $pool_time > 0 )
      $type = "NA";    # clear out type when empty line
      next;
    } ## end if ( $line =~ m/^$/ )

    if ( $line =~ m/^name,total_reads/ || $line =~ m/^uuid,name,node_name/ || $line =~ m/^node_name,read_ops,/ || $line =~ m/^cpu,node_name,/ || $line =~ m/^name,node_name/ || $line =~ m/^uuid,/ || $line =~ m/^disk-uid,/ || $line =~ m/instance_uuid,instance_name,/ || $line =~ m/^volume_id/ || $line =~ m/^name,id/ || $line =~ /^Controller/ || $line =~ m/^	Interval/ || $line =~ m/Interval Start/ || $line =~ m/Interval Lenght/ || $line =~ m/ID/ || $line =~ m/^-----------/ || $line =~ m/^Node,Time/ || $line =~ m/^Node,cfav/ || $line =~ m/^CPU Core,Time/ || $line =~ m/^Port,Time/ || $line =~ m/^Drive,Time/ || $line =~ m/^Managed Disk ID,Time/ || $line =~ m/^Volume ID,Time/ || $line =~ m/^qtree,volume,/ ) {

      # avoid other trash
      next;
    }

    # Qtree
    #########

    if ( $type =~ m/^QTREE$/ ) {    ### only for 7MODE
      print "000 QTREE: $line\n" if $DEBUG == 3;

      #qtree,volume,cifs_ops,nfs_ops
      #another_qtree,BigOne,0,0
      ( my $qtree, my $volume, my $cifs_ops, my $nfs_ops ) = split( /,/, $line );
      if ( !defined $qtree || !defined $volume || $qtree eq "" || $volume eq "" ) { next; }
      my $name = "$volume" . "@" . "$qtree";
      $rrd = "$wrkdir/$host/VOLUME/$name.rrq";

      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@qtree_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@qtree_name)
      if ( $found > -1 ) {
        $ltime = $qtree_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $qtree_name[$l_count] = $rrd;
        $qtree_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;

        if ( !defined($cifs_ops) || $cifs_ops eq '' ) { $cifs_ops = 'U'; }
        if ( !defined($nfs_ops)  || $nfs_ops eq '' )  { $nfs_ops  = 'U'; }
      } ## end else [ if ( $found > -1 ) ]

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        print "004: QTREE $name:$time:$cifs_ops:$nfs_ops\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$cifs_ops:$nfs_ops);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)

    } ## end if ( $type =~ m/^QTREE$/)

    # Port
    ########################

    if ( $type =~ m/^PORT$/ ) {
      print "000 PORT: $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $node_name, my $read, my $read_packet, my $read_error, my $write, my $write_packet, my $write_error ) = split( /,/, $line );

      # name,node_name,recv-data,recv-packets,recv-errors,sent-data,sent-packets,sent-errors
      # e0a,node-01,1.26,1155.54,0.00,0.54,966.24,0.00

      #(my $name, my $node_name, my $read_io, my $write_io, my $read, my $write)  = split(/,/,$line);
      # name,node_name,total_reads,total_writes,kbytes_read,kbytes_written
      # v0,node-01,1,1,51.103,58.571
      #--> original (perhaps back-end ports)

      my $port_type = 0;    # default, not used yet

      print "001 PORT $name - $write - $read - type:$port_type\n" if $DEBUG == 3;

      $name =~ s/:/-/g;     # rrdtool does not like ":"
      if ( !defined($name) || $name eq '' ) {
        next;               # name is null
      }
      if ( !defined($node_name) || $node_name eq '' ) {
        $rrd = "$wrkdir/$host/$type/$name.rrd";
      }
      else {
        $rrd = "$wrkdir/$host/$type/$node_name-$name.rrd";
      }

      #create rrd db if necessary
      $rrd =~ s/"//g;       # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;           # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      my $data_rate = "";
      if ( isdigit($read) && isdigit($write) ) {
        $data_rate = $read + $write;
      }
      my $packet_rate = "";
      if ( isdigit($read_packet) && isdigit($write_packet) ) {
        $packet_rate = $read_packet + $write_packet;
      }
      if ( !defined($read_packet)  || $read_packet eq '' )  { $read_packet  = 'U'; }
      if ( !defined($write_packet) || $write_packet eq '' ) { $write_packet = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read         = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write        = 'U'; }
      if ( !defined($read_error)   || $read_error eq '' )   { $read_error   = 'U'; }
      if ( !defined($write_error)  || $write_error eq '' )  { $write_error  = 'U'; }
      if ( !defined($data_rate)    || $data_rate eq '' )    { $data_rate    = 'U'; }
      if ( !defined($packet_rate)  || $packet_rate eq '' )  { $packet_rate  = 'U'; }

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        print "004: PORT $time:$packet_rate:$read_packet:$write_packet:$read:$write:$data_rate:$port_type\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$read:$write:$data_rate:$read_packet:$write_packet:$packet_rate:$read_error:$write_error:$port_type);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^PORT$/)

    # LUN
    ########################

    if ( $type =~ m/^LUN$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # LUN  statistics
      ( my $uuid, my $name, my $node_name, my $read_io, my $write_io, my $read, my $write, my $resp_t_r, my $resp_t_w, my $resp_t, my $total_ops, my $other_ops ) = split( /,/, $line );

      # uuid,name,node_name,read_ops,write_ops,read_data,write_data,avg_read_latency,avg_write_latency,avg_latency,total_ops,other_ops
      # 42b87d2d-7794-4fb4-8578-a17628a54838,/vol/LUN_Linux_vol/LUN_Linux,,20,6,81,27,,,0,27,0

      if ( !defined($uuid) || $uuid eq '' ) {
        next;    # name is null
      }
      $rrd = "$wrkdir/$host/$type/$uuid\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@lun_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@lun_name)
      if ( $found > -1 ) {
        $ltime = $lun_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $lun_name[$l_count] = $rrd;
        $lun_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      my $data_rate = $read;
      if ( isdigit($data_rate) && isdigit($write) ) {
        $data_rate = $data_rate + $write;
      }
      my $io_rate = $read_io;
      if ( isdigit($io_rate) && isdigit($write_io) ) {
        $io_rate = $io_rate + $write_io;
      }
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq '' )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq '' )  { $resp_t_w  = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        print "004: LUN $time:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate:U:U:U\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^LUN$/)

    # Volume
    ########################

    if ( $type =~ m/^VOLUME$/ ) {
      print "000 $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $name_text, my $io_rate, my $resp_t, my $read_io, my $read, my $resp_t_r, my $write_io, my $write, my $resp_t_w, my $cap, my $cap_used, my $pool_uuid, my $nas_id ) = split( /,/, $line );

      #volume_id,name,total_ops,avg_latency,read_ops,read_data,read_latency,write_ops,write_data,write_latency,size-total,size-used,containing-aggregate-uuid,owning-vserver-uuid
      #18d16b60-91d5-49e6-8c55-191a19624cd2,vol0 (node-01),287,0.030,62,908,0.040,23,35,0.095,4.75000,3.94825,7eb99482-f039-46c0-97f2-c8ebe5f9f78d,9a72f152-fd78-11e5-8210-db85c0f85098

      if ( !defined($name) || $name eq '' ) {
        next;    # name is null
      }
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      my $data_rate = $read;
      if ( isdigit($data_rate) && isdigit($write) ) {
        $data_rate = $data_rate + $write;
      }
      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($resp_t)    || $resp_t eq '' )    { $resp_t    = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq '' )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq '' )  { $resp_t_w  = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }
      if ( !defined($cap)       || $cap eq '' )       { $cap       = 'U'; }
      if ( !defined($cap_used)  || $cap_used eq '' )  { $cap_used  = 'U'; }

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        print "004: VOL $time:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate:$cap:$cap_used:U\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate:$cap:$cap_used:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;

        if ( !defined($pool_uuid) || $pool_uuid eq '' ) {

          #$pool_uuid = $name; # for volumes not build above aggregates (could happened when it uses extern disks)
          # no, no, when volume does not belong to a pool then cannot be saved as pool
          next;
        } ## end if ( !defined($pool_uuid...))

        #convert pool_uuid to numeric index
        my $pool_id = 0;
        foreach my $pool_uuid_act (@pool_ids) {
          if ( !defined($pool_uuid_act) || $pool_uuid_act eq '' ) {
            next;
          }
          if ( $pool_uuid =~ m/^$pool_uuid_act$/ ) {
            last;
          }
          $pool_id++;
        } ## end foreach my $pool_uuid_act (...)

        # keep volume fron-end data summ per pool and save it at the end

        if ( !defined $pool_read[$pool_id]      || $pool_read[$pool_id] eq '' )      { $pool_read[$pool_id]      = 0; }
        if ( !defined $pool_write[$pool_id]     || $pool_write[$pool_id] eq '' )     { $pool_write[$pool_id]     = 0; }
        if ( !defined $pool_data_rate[$pool_id] || $pool_data_rate[$pool_id] eq '' ) { $pool_data_rate[$pool_id] = 0; }
        if ( !defined $pool_io_read[$pool_id]   || $pool_io_read[$pool_id] eq '' )   { $pool_io_read[$pool_id]   = 0; }
        if ( !defined $pool_io_write[$pool_id]  || $pool_io_write[$pool_id] eq '' )  { $pool_io_write[$pool_id]  = 0; }
        if ( !defined $pool_io_rate[$pool_id]   || $pool_io_rate[$pool_id] eq '' )   { $pool_io_rate[$pool_id]   = 0; }
        if ( !defined $pool_resp_r[$pool_id]    || $pool_resp_r[$pool_id] eq '' )    { $pool_resp_r[$pool_id]    = 0; }
        if ( !defined $pool_resp_w[$pool_id]    || $pool_resp_w[$pool_id] eq '' )    { $pool_resp_w[$pool_id]    = 0; }
        if ( !defined $pool_resp[$pool_id]      || $pool_resp[$pool_id] eq '' )      { $pool_resp[$pool_id]      = 0; }

        if ( isdigit($io_rate) == 0 )   { $io_rate   = 0; }
        if ( isdigit($read_io) == 0 )   { $read_io   = 0; }
        if ( isdigit($write_io) == 0 )  { $write_io  = 0; }
        if ( isdigit($resp_t_r) == 0 )  { $resp_t_r  = 0; }
        if ( isdigit($resp_t_w) == 0 )  { $resp_t_w  = 0; }
        if ( isdigit($read) == 0 )      { $read      = 0; }
        if ( isdigit($write) == 0 )     { $write     = 0; }
        if ( isdigit($data_rate) == 0 ) { $data_rate = 0; }

        $pool_read[$pool_id]      += $read;
        $pool_write[$pool_id]     += $write;
        $pool_data_rate[$pool_id] += $data_rate;
        $pool_io_read[$pool_id]   += $read_io;
        $pool_io_write[$pool_id]  += $write_io;
        $pool_io_rate[$pool_id]   += $io_rate;
        $pool_resp_r[$pool_id]    += $resp_t_r * $read_io;
        $pool_resp_w[$pool_id]    += $resp_t_w * $write_io;
        $pool_resp[$pool_id]      += $resp_t_r * $read_io;
        $pool_resp[$pool_id]      += $resp_t_w * $write_io;
        $pool_time = $time;
        $pool_ids[$pool_id] = $pool_uuid;

        #print STDERR "555 $line - $pool_ids[$pool_id] : $pool_resp[$pool_id] = $pool_resp_r[$pool_id] + $pool_resp_w[$pool_id] : $read_io : $write_io : $io_rate : $pool_id :: $resp_t_r : $read_io : $resp_t_w : $write_io : $time\n";
      } ## end if ( isdigit($time) &&...)

      # add values to nas
      if ( defined $nas_id && $nas_id ne '' ) {
        $nas_index++;

        $nas_data{$nas_id}{'nas_name'} = $nas_id;

        if ( defined $io_rate && isdigit($io_rate) ) {
          if ( defined $nas_data{$nas_id}{'io_rate'} && isdigit( $nas_data{$nas_id}{'io_rate'} ) ) {
            $nas_data{$nas_id}{'io_rate'} = $nas_data{$nas_id}{'io_rate'} + $io_rate;
          }
          else {
            $nas_data{$nas_id}{'io_rate'} = $io_rate;
          }
        } ## end if ( defined $io_rate ...)

        if ( defined $read_io && isdigit($read_io) ) {
          if ( defined $nas_data{$nas_id}{'read_io'} && isdigit( $nas_data{$nas_id}{'read_io'} ) ) {
            $nas_data{$nas_id}{'read_io'} = $nas_data{$nas_id}{'read_io'} + $read_io;
          }
          else {
            $nas_data{$nas_id}{'read_io'} = $read_io;
          }
        } ## end if ( defined $read_io ...)

        if ( defined $write_io && isdigit($write_io) ) {
          if ( defined $nas_data{$nas_id}{'write_io'} && isdigit( $nas_data{$nas_id}{'write_io'} ) ) {
            $nas_data{$nas_id}{'write_io'} = $nas_data{$nas_id}{'write_io'} + $write_io;
          }
          else {
            $nas_data{$nas_id}{'write_io'} = $write_io;
          }
        } ## end if ( defined $write_io...)

        if ( defined $data_rate && isdigit($data_rate) ) {
          if ( defined $nas_data{$nas_id}{'data_rate'} && isdigit( $nas_data{$nas_id}{'data_rate'} ) ) {
            $nas_data{$nas_id}{'data_rate'} = $nas_data{$nas_id}{'data_rate'} + $data_rate;
          }
          else {
            $nas_data{$nas_id}{'data_rate'} = $data_rate;
          }
        } ## end if ( defined $data_rate...)

        if ( defined $read && isdigit($read) ) {
          if ( defined $nas_data{$nas_id}{'read'} && isdigit( $nas_data{$nas_id}{'read'} ) ) {
            $nas_data{$nas_id}{'read'} = $nas_data{$nas_id}{'read'} + $read;
          }
          else {
            $nas_data{$nas_id}{'read'} = $read;
          }
        } ## end if ( defined $read && ...)

        if ( defined $write && isdigit($write) ) {
          if ( defined $nas_data{$nas_id}{'write'} && isdigit( $nas_data{$nas_id}{'write'} ) ) {
            $nas_data{$nas_id}{'write'} = $nas_data{$nas_id}{'write'} + $write;
          }
          else {
            $nas_data{$nas_id}{'write'} = $write;
          }
        } ## end if ( defined $write &&...)

        if ( defined $cap && isdigit($cap) ) {
          if ( defined $nas_data{$nas_id}{'cap'} && isdigit( $nas_data{$nas_id}{'cap'} ) ) {
            $nas_data{$nas_id}{'cap'} = $nas_data{$nas_id}{'cap'} + $cap;
          }
          else {
            $nas_data{$nas_id}{'cap'} = $cap;
          }
        } ## end if ( defined $cap && isdigit...)
        if ( defined $cap_used && isdigit($cap_used) ) {
          if ( defined $nas_data{$nas_id}{'cap_used'} && isdigit( $nas_data{$nas_id}{'cap_used'} ) ) {
            $nas_data{$nas_id}{'cap_used'} = $nas_data{$nas_id}{'cap_used'} + $cap_used;
          }
          else {
            $nas_data{$nas_id}{'cap_used'} = $cap_used;
          }
        } ## end if ( defined $cap_used...)
        if ( defined $resp_t && isdigit($resp_t) && defined $io_rate && isdigit($io_rate) ) {
          $resp_t = $resp_t * $io_rate;
          if ( defined $nas_data{$nas_id}{'resp_t'} && isdigit( $nas_data{$nas_id}{'resp_t'} ) ) {
            $nas_data{$nas_id}{'resp_t'} = $nas_data{$nas_id}{'resp_t'} + $resp_t;
          }
          else {
            $nas_data{$nas_id}{'resp_t'} = $resp_t;
          }
        } ## end if ( defined $resp_t &&...)
        if ( defined $resp_t_r && isdigit($resp_t_r) && defined $read_io && isdigit($read_io) ) {
          $resp_t_r = $resp_t_r * $read_io;
          if ( defined $nas_data{$nas_id}{'resp_t_r'} && isdigit( $nas_data{$nas_id}{'resp_t_r'} ) ) {
            $nas_data{$nas_id}{'resp_t_r'} = $nas_data{$nas_id}{'resp_t_r'} + $resp_t_r;
          }
          else {
            $nas_data{$nas_id}{'resp_t_r'} = $resp_t_r;
          }
        } ## end if ( defined $resp_t_r...)
        if ( defined $resp_t_w && isdigit($resp_t_w) && defined $write_io && isdigit($write_io) ) {
          $resp_t_w = $resp_t_w * $write_io;
          if ( defined $nas_data{$nas_id}{'resp_t_w'} && isdigit( $nas_data{$nas_id}{'resp_t_w'} ) ) {
            $nas_data{$nas_id}{'resp_t_w'} = $nas_data{$nas_id}{'resp_t_w'} + $resp_t_w;
          }
          else {
            $nas_data{$nas_id}{'resp_t_w'} = $resp_t_w;
          }
        } ## end if ( defined $resp_t_w...)
      } ## end if ( defined $nas_id &&...)
    } ## end if ( $type =~ m/^VOLUME$/)

    # Pool  --> only capacity stats, data comes from Volume stats
    ########################

    if ( $type =~ m/^POOL$/ ) {
      print "000 $line\n" if $DEBUG == 3;
      my $type_real = "POOLcap";

      # Port capacity statistics
      ( my $id, my $name, my $t9, my $t1, my $t2, my $t4, my $t3, my $cap, my $used ) = split( /,/, $line );

      # instance_uuid,instance_name,total_transfers,user_reads,user_writes,user_read_blocks,user_write_blocks,cap,used_cap,,,,,user_reads_hdd,user_reads_ssd,user_writes_hdd,user_writes_ssd
      # 7eb99482-f039-46c0-97f2-c8ebe5f9f78d,aggr0,6,0,2,3,30,36.73828,3.17757,,,,,0,0,2,0

      if ( !defined($id) || $id eq '' ) {
        next;    # to be save against cooruptions in the input file!!!
      }

      #print "001 POOL cap : $name - $id - $total - $free - $virt - $used - $real \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/POOL/$id-cap\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type_real, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_cap_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_cap_name)
      if ( $found > -1 ) {
        $ltime = $pool_cap_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $pool_cap_name[$l_count] = $rrd;
        $pool_cap_time[$l_count] = $$last_rec;
        $ltime                   = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_cap_time[$l_count] = $time;
        $counter_ins++;
        if ( isdigit($cap) == 0 || isdigit($used) == 0 ) {
          next;
        }
        my $free = $cap - $used;

        print "004 POOL cap : $name - $id - $cap - $free - $used \n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$cap:$free:U:$used:$used:U:U:U:U:U:U);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^POOL$/)

    # POOL
    ########################

    # excluded actually
    if ( $type =~ m/^POOLxxxxx$/ ) {

      print "005 $line\n" if $DEBUG == 3;
      ( my $id,       my $pool_name, my $io_rate,   my $read_io,   my $write_io,    my $read,        my $write,        my $cap, my $used_cap, my $t1, my $t2, my $t3, my $t4,
        my $read_hdd, my $read_sdd,  my $write_hdd, my $write_sdd, my $read_io_hdd, my $read_io_sdd, my $write_io_hdd, my $write_io_sdd,
      ) = split( /,/, $line );

      #instance_uuid,instance_name,total_transfers,user_reads,user_writes,user_read_blocks,user_write_blocks,cap,used_cap,,,,,user_reads_hdd,user_reads_ssd,user_writes_hdd,user_writes_ssd
      #9b8558e9-c5ad-4968-a5cc-f2106eaca1c3,aggr0,7,2,2,2,16,2.63671875,1.32632446289062,,,,,2,0,2,0

      print "001 POOL $id - $read_io - $write_io  - $read - $write\n" if $DEBUG == 3;
      if ( !defined($id) || $id eq '' ) {
        next;    # name is null
      }
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      my $data_rate = 'U';
      if ( isdigit($read) && isdigit($write) ) {
        $data_rate = $read + $write;
      }
      my $free = "";
      if ( isdigit($used_cap) && isdigit($cap) ) {
        $free = $cap - $used_cap;
      }

      if ( !defined($io_rate)      || $io_rate eq '' )      { $io_rate      = 'U'; }
      if ( !defined($read_io)      || $read_io eq '' )      { $read_io      = 'U'; }
      if ( !defined($write_io)     || $write_io eq '' )     { $write_io     = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read         = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write        = 'U'; }
      if ( !defined($used_cap)     || $used_cap eq '' )     { $used_cap     = 'U'; }
      if ( !defined($free)         || $free eq '' )         { $free         = 'U'; }
      if ( !defined($write_hdd)    || $write_hdd eq '' )    { $write_hdd    = 'U'; }
      if ( !defined($read_io_hdd)  || $read_io_hdd eq '' )  { $read_io_hdd  = 'U'; }
      if ( !defined($write_io_hdd) || $write_io_hdd eq '' ) { $write_io_hdd = 'U'; }
      if ( !defined($read_hdd)     || $read_hdd eq '' )     { $read_hdd     = 'U'; }
      if ( !defined($write_sdd)    || $write_sdd eq '' )    { $write_sdd    = 'U'; }
      if ( !defined($read_io_sdd)  || $read_io_sdd eq '' )  { $read_io_sdd  = 'U'; }
      if ( !defined($write_io_sdd) || $write_io_sdd eq '' ) { $write_io_sdd = 'U'; }
      if ( !defined($read_sdd)     || $read_sdd eq '' )     { $read_sdd     = 'U'; }

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_time[$l_count] = $time;
        $counter_ins++;
        print "004: POOL $time:$io_rate:$read_io:$write_io:$read:$write:$used_cap:$free:$read_hdd:$read_sdd:$write_hdd:$write_sdd:$read_io_hdd:$read_io_sdd:$write_io_hdd:$write_io_sdd\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read:$write:$data_rate:$read_io:$write_io:$used_cap:$free:$read_hdd:$read_sdd:$write_hdd:$write_sdd:$read_io_hdd:$read_io_sdd:$write_io_hdd:$write_io_sdd);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^POOLxxxxx$/)

    # Node
    ########################

    if ( $type =~ m/^NODE$/ ) {

      print "005 $line\n" if $DEBUG == 3;

      ( my $id,           my $read_io,            my $write_io,       my $io_rate,             my $read,             my $write,                     my $data_rate,        my $resp_t_r,              my $resp_t_w,               my $resp_t,         my $total_processor_busy, my $cpu_busy,
        my $cifs_latency, my $cifs_ops,           my $cifs_read_ops,  my $cifs_write_ops,      my $cifs_connections, my $cifs_established_sessions, my $cifs_open_files,  my $cifs_path_based_ops,   my $cifs_persistent_opens,  my $cifs_read_data, my $cifs_write_data,
        my $nfsv3_ops,    my $nfsv3_read_latency, my $nfsv3_read_ops, my $nfsv3_write_latency, my $nfsv3_write_ops,  my $nfsv3_avg_op_latency,      my $nfsv3_throughput, my $nfsv3_read_throughput, my $nfsv3_write_throughput, my $nfsv3_link_avg_latency,
        my $nfsv4_calls,   my $nfsv4_ops,      my $nfsv4_ops_latency, my $nfsv4_read_ops, my $nfsv4_read_latency, my $nfsv4_write_ops, my $nfsv4_write_latency, my $nfsv4_throughput, my $nfsv4_read_throughput, my $nfsv4_write_throughput, my $nfsv4_link_avg_latency,
        my $iscsi_read_io, my $iscsi_write_io, my $iscsi_io_rate,     my $iscsi_read,     my $iscsi_write,        my $iscsi_data_rate, my $iscsi_resp_t_r,      my $iscsi_resp_t_w,   my $iscsi_resp_t,
        my $fcp_read_io,   my $fcp_write_io,   my $fcp_io_rate,       my $fcp_read,       my $fcp_write,          my $fcp_data_rate,   my $fcp_resp_t_r,        my $fcp_resp_t_w,     my $fcp_resp_t,
        my $wafl_exempt,   my $kahuna,         my $exempt,            my $hostOS,         my $nwk_legacy
      ) = split( /,/, $line );

      # node_name,read_ops,write_ops,total_ops,read_data,write_data,total_data,read_latency,write_latency,total_latency,total_processor_busy,cpu_busy
      # node-01,20,7,27,84078,29280,113359,200,217,204,5%,2%

      # C-mode
      # node_name,read_ops,write_ops,total_ops,read_data,write_data,total_data,read_latency,write_latency,total_latency,total_processor_busy,cpu_busy,
      #  cifs_latency,cifs_ops,cifs_read_ops,cifs_write_ops,cifs_connections,cifs_established_sessions,cifs_open_files,cifs_path_based_ops,cifs_persistent_opens,cifs_read_data,cifs_write_data,
      #  nfsv3_ops,nfsv3_read_latency,nfsv3_read_ops,nfsv3_write_latency,nfsv3_write_ops,nfsv3_avg_op_latency,nfsv3_throughput,nfsv3_read_throughput,nfsv3_write_throughput,nfsv3_link_avg_latency,
      #  nfsv4_calls,nfsv4_ops,nfsv4_ops_latency,nfsv4_read_ops,nfsv4_read_latency,nfsv4_write_ops,nfsv4_write_latency,nfsv4_throughput,nfsv4_read_throughput,nfsv4_write_throughput,nfsv4_link_avg_latency,
      #  iscsi_read_ops,iscsi_write_ops,iscsi_io_rate,iscsi_read_data,iscsi_write_data,iscsi_data_rate,iscsi_avg_read_latency,iscsi_avg_write_latency,iscsi_avg_latency
      #  fcp_read_ops,fcp_write_ops,fcp_io_rate,fcp_read_data,fcp_write_data,fcp_data_rate,fcp_avg_read_latency,fcp_avg_write_latency,fcp_avg_latency,wafl_exempt,kahuna,exempt,hostOS,nwk_legacy
      # node-01,21,7,28,292.83,98.18,391.02,397,486,419,7,3,,,,,,,,,,,,0,0,0,0,0,0,0.00,0.00,0.00,0,27,64,0.243,20,0.397,6,0.486,391.02,292.83,98.18,0,0,0,0,4,20
      # node-02,42,14,57,588.77,193.65,782.41,84,110,91,6,3,,,,,,,,,,,,0,0,0,0,0,0,0.00,0.00,0.00,0,0,0,0,0,0,0,0,0.00,0.00,0.00,0,11,0,1,4,17

      # 7-mode
      # node_name,read_ops,write_ops,total_ops,read_data,write_data,total_data,read_latency,write_latency,total_latency,total_processor_busy,cpu_busy,
      #  cifs_latency,cifs_ops,cifs_read_ops,cifs_write_ops,cifs_connections,cifs_established_sessions,cifs_open_files,cifs_path_based_ops,cifs_persistent_opens,cifs_read_data,cifs_write_data,
      #  nfsv3_ops,nfsv3_read_latency,nfsv3_read_ops,nfsv3_write_latency,nfsv3_write_ops,nfsv3_avg_op_latency,nfsv3_throughput,nfsv3_read_throughput,nfsv3_write_throughput,nfsv3_link_avg_latency,
      #  nfsv4_calls,nfsv4_ops,nfsv4_ops_latency,nfsv4_read_ops,nfsv4_read_latency,nfsv4_write_ops,nfsv4_write_latency,nfsv4_throughput,nfsv4_read_throughput,nfsv4_write_throughput,nfsv4_link_avg_latency
      #system,3,1,,0.00,0.00,0.00,,,,2,1,0,0,0,0,,,,,,,,4,19.30,3,0.13,1,14.102,0.00,0.00,0.00,,0,0,,,,,,,0.00,0.00,

      #print "001 POOL $id - $read_io - $write_io  - $read - $write\n" if $DEBUG == 3;
      if ( !defined($id) || $id eq '' ) {
        next;    # name is null
      }
      $rrd = "$wrkdir/$host/$type/$id\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@node_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@node_name)
      if ( $found > -1 ) {
        $ltime = $node_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $node_name[$l_count] = $rrd;
        $node_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $node_name[$l_count] - $node_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      my $cifs_data_rate = $cifs_read_data;
      if ( isdigit($cifs_data_rate) && isdigit($cifs_write_data) ) {
        $cifs_data_rate = $cifs_data_rate + $cifs_write_data;
      }

      if ( !defined($io_rate)                   || $io_rate eq '' )                   { $io_rate                   = 'U'; }
      if ( !defined($read_io)                   || $read_io eq '' )                   { $read_io                   = 'U'; }
      if ( !defined($write_io)                  || $write_io eq '' )                  { $write_io                  = 'U'; }
      if ( !defined($read)                      || $read eq '' )                      { $read                      = 'U'; }
      if ( !defined($write)                     || $write eq '' )                     { $write                     = 'U'; }
      if ( !defined($data_rate)                 || $data_rate eq '' )                 { $data_rate                 = 'U'; }
      if ( !defined($resp_t_r)                  || $resp_t_r eq '' )                  { $resp_t_r                  = 'U'; }
      if ( !defined($resp_t_w)                  || $resp_t_w eq '' )                  { $resp_t_w                  = 'U'; }
      if ( !defined($resp_t)                    || $resp_t eq '' )                    { $resp_t                    = 'U'; }
      if ( !defined($total_processor_busy)      || $total_processor_busy eq '' )      { $total_processor_busy      = 'U'; }
      if ( !defined($cpu_busy)                  || $cpu_busy eq '' )                  { $cpu_busy                  = 'U'; }
      if ( !defined($cifs_latency)              || $cifs_latency eq '' )              { $cifs_latency              = 'U'; }
      if ( !defined($cifs_ops)                  || $cifs_ops eq '' )                  { $cifs_ops                  = 'U'; }
      if ( !defined($cifs_read_ops)             || $cifs_read_ops eq '' )             { $cifs_read_ops             = 'U'; }
      if ( !defined($cifs_write_ops)            || $cifs_write_ops eq '' )            { $cifs_write_ops            = 'U'; }
      if ( !defined($cifs_connections)          || $cifs_connections eq '' )          { $cifs_connections          = 'U'; }
      if ( !defined($cifs_established_sessions) || $cifs_established_sessions eq '' ) { $cifs_established_sessions = 'U'; }
      if ( !defined($cifs_open_files)           || $cifs_open_files eq '' )           { $cifs_open_files           = 'U'; }
      if ( !defined($cifs_path_based_ops)       || $cifs_path_based_ops eq '' )       { $cifs_path_based_ops       = 'U'; }
      if ( !defined($cifs_persistent_opens)     || $cifs_persistent_opens eq '' )     { $cifs_persistent_opens     = 'U'; }
      if ( !defined($cifs_read_data)            || $cifs_read_data eq '' )            { $cifs_read_data            = 'U'; }
      if ( !defined($cifs_write_data)           || $cifs_write_data eq '' )           { $cifs_write_data           = 'U'; }
      if ( !defined($cifs_data_rate)            || $cifs_data_rate eq '' )            { $cifs_data_rate            = 'U'; }
      if ( !defined($nfsv3_ops)                 || $nfsv3_ops eq '' )                 { $nfsv3_ops                 = 'U'; }
      if ( !defined($nfsv3_read_latency)        || $nfsv3_read_latency eq '' )        { $nfsv3_read_latency        = 'U'; }
      if ( !defined($nfsv3_read_ops)            || $nfsv3_read_ops eq '' )            { $nfsv3_read_ops            = 'U'; }
      if ( !defined($nfsv3_write_latency)       || $nfsv3_write_latency eq '' )       { $nfsv3_write_latency       = 'U'; }
      if ( !defined($nfsv3_write_ops)           || $nfsv3_write_ops eq '' )           { $nfsv3_write_ops           = 'U'; }
      if ( !defined($nfsv3_avg_op_latency)      || $nfsv3_avg_op_latency eq '' )      { $nfsv3_avg_op_latency      = 'U'; }
      if ( !defined($nfsv3_throughput)          || $nfsv3_throughput eq '' )          { $nfsv3_throughput          = 'U'; }
      if ( !defined($nfsv3_read_throughput)     || $nfsv3_read_throughput eq '' )     { $nfsv3_read_throughput     = 'U'; }
      if ( !defined($nfsv3_write_throughput)    || $nfsv3_write_throughput eq '' )    { $nfsv3_write_throughput    = 'U'; }
      if ( !defined($nfsv3_link_avg_latency)    || $nfsv3_link_avg_latency eq '' )    { $nfsv3_link_avg_latency    = 'U'; }
      if ( !defined($nfsv4_calls)               || $nfsv4_calls eq '' )               { $nfsv4_calls               = 'U'; }
      if ( !defined($nfsv4_ops)                 || $nfsv4_ops eq '' )                 { $nfsv4_ops                 = 'U'; }
      if ( !defined($nfsv4_ops_latency)         || $nfsv4_ops_latency eq '' )         { $nfsv4_ops_latency         = 'U'; }
      if ( !defined($nfsv4_read_ops)            || $nfsv4_read_ops eq '' )            { $nfsv4_read_ops            = 'U'; }
      if ( !defined($nfsv4_read_latency)        || $nfsv4_read_latency eq '' )        { $nfsv4_read_latency        = 'U'; }
      if ( !defined($nfsv4_write_ops)           || $nfsv4_write_ops eq '' )           { $nfsv4_write_ops           = 'U'; }
      if ( !defined($nfsv4_write_latency)       || $nfsv4_write_latency eq '' )       { $nfsv4_write_latency       = 'U'; }
      if ( !defined($nfsv4_throughput)          || $nfsv4_throughput eq '' )          { $nfsv4_throughput          = 'U'; }
      if ( !defined($nfsv4_read_throughput)     || $nfsv4_read_throughput eq '' )     { $nfsv4_read_throughput     = 'U'; }
      if ( !defined($nfsv4_write_throughput)    || $nfsv4_write_throughput eq '' )    { $nfsv4_write_throughput    = 'U'; }
      if ( !defined($nfsv4_link_avg_latency)    || $nfsv4_link_avg_latency eq '' )    { $nfsv4_link_avg_latency    = 'U'; }
      if ( !defined($iscsi_read_io)             || $iscsi_read_io eq '' )             { $iscsi_read_io             = 'U'; }
      if ( !defined($iscsi_write_io)            || $iscsi_write_io eq '' )            { $iscsi_write_io            = 'U'; }
      if ( !defined($iscsi_io_rate)             || $iscsi_io_rate eq '' )             { $iscsi_io_rate             = 'U'; }
      if ( !defined($iscsi_read)                || $iscsi_read eq '' )                { $iscsi_read                = 'U'; }
      if ( !defined($iscsi_write)               || $iscsi_write eq '' )               { $iscsi_write               = 'U'; }
      if ( !defined($iscsi_data_rate)           || $iscsi_data_rate eq '' )           { $iscsi_data_rate           = 'U'; }
      if ( !defined($iscsi_resp_t_r)            || $iscsi_resp_t_r eq '' )            { $iscsi_resp_t_r            = 'U'; }
      if ( !defined($iscsi_resp_t_w)            || $iscsi_resp_t_w eq '' )            { $iscsi_resp_t_w            = 'U'; }
      if ( !defined($iscsi_resp_t)              || $iscsi_resp_t eq '' )              { $iscsi_resp_t              = 'U'; }
      if ( !defined($fcp_read_io)               || $fcp_read_io eq '' )               { $fcp_read_io               = 'U'; }
      if ( !defined($fcp_write_io)              || $fcp_write_io eq '' )              { $fcp_write_io              = 'U'; }
      if ( !defined($fcp_io_rate)               || $fcp_io_rate eq '' )               { $fcp_io_rate               = 'U'; }
      if ( !defined($fcp_read)                  || $fcp_read eq '' )                  { $fcp_read                  = 'U'; }
      if ( !defined($fcp_write)                 || $fcp_write eq '' )                 { $fcp_write                 = 'U'; }
      if ( !defined($fcp_data_rate)             || $fcp_data_rate eq '' )             { $fcp_data_rate             = 'U'; }
      if ( !defined($fcp_resp_t_r)              || $fcp_resp_t_r eq '' )              { $fcp_resp_t_r              = 'U'; }
      if ( !defined($fcp_resp_t_w)              || $fcp_resp_t_w eq '' )              { $fcp_resp_t_w              = 'U'; }
      if ( !defined($fcp_resp_t)                || $fcp_resp_t eq '' )                { $fcp_resp_t                = 'U'; }
      if ( !defined($wafl_exempt)               || $wafl_exempt eq '' )               { $wafl_exempt               = 'U'; }
      if ( !defined($kahuna)                    || $kahuna eq '' )                    { $kahuna                    = 'U'; }
      if ( !defined($exempt)                    || $exempt eq '' )                    { $exempt                    = 'U'; }
      if ( !defined($hostOS)                    || $hostOS eq '' )                    { $hostOS                    = 'U'; }
      if ( !defined($nwk_legacy)                || $nwk_legacy eq '' )                { $nwk_legacy                = 'U'; }

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_time[$l_count] = $time;
        $counter_ins++;
        print "004: NODE $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$resp_t_r:$resp_t_w:$resp_t:$total_processor_busy:$cpu_busy\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$resp_t_r:$resp_t_w:$resp_t:$total_processor_busy:$cpu_busy:$cifs_latency:$cifs_ops:$cifs_read_ops:$cifs_write_ops:$cifs_connections:$cifs_established_sessions:$cifs_open_files:$cifs_path_based_ops:$cifs_persistent_opens:$cifs_read_data:$cifs_write_data:$cifs_data_rate:$nfsv3_ops:$nfsv3_read_latency:$nfsv3_read_ops:$nfsv3_write_latency:$nfsv3_write_ops:$nfsv3_avg_op_latency:$nfsv3_throughput:$nfsv3_read_throughput:$nfsv3_write_throughput:$nfsv3_link_avg_latency:$nfsv4_calls:$nfsv4_ops:$nfsv4_ops_latency:$nfsv4_read_ops:$nfsv4_read_latency:$nfsv4_write_ops:$nfsv4_write_latency:$nfsv4_throughput:$nfsv4_read_throughput:$nfsv4_write_throughput:$nfsv4_link_avg_latency:$iscsi_read_io:$iscsi_write_io:$iscsi_io_rate:$iscsi_read:$iscsi_write:$iscsi_data_rate:$iscsi_resp_t_r:$iscsi_resp_t_w:$iscsi_resp_t:$fcp_read_io:$fcp_write_io:$fcp_io_rate:$fcp_read:$fcp_write:$fcp_data_rate:$fcp_resp_t_r:$fcp_resp_t_w:$fcp_resp_t:$wafl_exempt:$kahuna:$exempt:$hostOS:$nwk_legacy);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^NODE$/)

    #
    # Drives

    if ( $type =~ m/^DRIVE$/ ) {
      print "000 DRIV $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $disk_uid, my $name, my $io_read, my $io_write, my $read, my $write, my $user_read, my $user_write, my $resp_r, my $resp_w, my $user_read_b, my $user_write_b ) = split( /,/, $line );

      #disk-uid,name,read_ops,write_ops,read_data,write_data,user_reads,user_writes,user_read_latency,user_write_latency,user_read_blocks,user_write_blocks,total_transfers
      #4E455441:50502020:56442D31:3030304D:422D465A:2D353230:38383934:33303030:00000000:00000000,v5.16,,,,,0,0,12.042,4.081,0,3,1

      if ( !defined($disk_uid) || $name eq '' || $disk_uid eq '' ) {
        next;
      }
      print "001 DRIV $name - $io_read - $io_write -  $read - $write  \n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$disk_uid\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@drive_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@drive_name)
      if ( $found > -1 ) {
        $ltime = $drive_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $drive_name[$l_count] = $rrd;
        $drive_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      if ( !defined($io_read)      || $io_read eq '' )      { $io_read      = 'U'; }
      if ( !defined($io_write)     || $io_write eq '' )     { $io_write     = 'U'; }
      if ( !defined($read)         || $read eq '' )         { $read         = 'U'; }
      if ( !defined($write)        || $write eq '' )        { $write        = 'U'; }
      if ( !defined($user_read_b)  || $user_read_b eq '' )  { $user_read_b  = 'U'; }
      if ( !defined($user_write_b) || $user_write_b eq '' ) { $user_write_b = 'U'; }
      if ( !defined($user_read)    || $user_read eq '' )    { $user_read    = 'U'; }
      if ( !defined($user_write)   || $user_write eq '' )   { $user_write   = 'U'; }
      if ( !defined($resp_r)       || $resp_r eq '' )       { $resp_r       = 'U'; }
      if ( !defined($resp_w)       || $resp_w eq '' )       { $resp_w       = 'U'; }

      print "002 DRIV $name - $io_read -  $io_write - $read - $write - w\n : $time > $ltime" if $DEBUG == 3;

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $drive_time[$l_count] = $time;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time:$io_read:$io_write:$read:$write:$resp_r:$resp_w);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^DRIVE$/)

    # CPU-Core
    ########################

    if ( $type =~ m/^CPU-CORE$/ ) {
      print "000 CPU-Core: $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $cpu_name, my $node, my $busy, my $sk_switches ) = split( /,/, $line );

      # cpu,node_name,processor_busy,sk_switches
      # processor0,node-01,2%,2081

      if ( !defined($cpu_name) || $cpu_name eq '' ) {
        next;
      }
      my $name = "$node-$cpu_name";
      if ( !defined($node) || $node eq '' ) {
        $name = $cpu_name;
      }

      print "001 CPU $name - $cpu_name, $node, $busy, $sk_switches\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cpuc_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cpuc_name)
      if ( $found > -1 ) {
        $ltime = $cpuc_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cpuc_name[$l_count] = $rrd;
        $cpuc_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      if ( !defined($busy)        || $busy eq '' )        { $busy        = 'U'; }
      if ( !defined($sk_switches) || $sk_switches eq '' ) { $sk_switches = 'U'; }

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cpuc_time[$l_count] = $time;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time:$busy:$sk_switches);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CPU-CORE$/)

  }    # foreach

  # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
  if ( $pool_time > $pool_ltime ) {
    my @none = "";
    pool_sum_insert(
      $type, $wrkdir, $step,  $DEBUG, $host,  $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@pool_data_rate, \@pool_io_rate, \@pool_resp_r,
      @none, \@none,  \@none, \@none, \@none, \@none,   \@none,    \@none,   \@none,     \@none,      \@none,      \@none,       \@none,         \@none,          \@none
    );
  } ## end if ( $pool_time > $pool_ltime)

  # NAS summary
  if ( isdigit($nas_index) && $nas_index > 0 ) {
    $type = "NAS";
    data_insert( $host, $step, $wrkdir, $DEBUG, $no_time, $act_time, $st_type, $time, $type, \%nas_data );
  }

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_netapp_perf

##################################################
#    3PAR section
##################################################

sub load_data_3par_all {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  #
  # at first config files like xivconf_20130904_115351.out
  #

  my $perf_string = "hp3parconf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @files_unsorted = grep( /$host\_$perf_string\_.*out/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  my $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_3par_conf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {
    print "config file    : $host: no new config input file found\n" if $DEBUG;
  }

  #
  # now data files like xiv_svcperf_20130904_115351.out
  #

  $perf_string = "hp3parperf";
  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;

  @files_unsorted = grep( /$host\_$perf_string\_.*/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  $in_exist = 0;

  foreach my $file (@files) {
    chomp($file);
    if ( $file =~ m/-tmp$/ ) {
      next;    #it is not finised inpud data file
    }
    $in_exist = 1;
    my $in_file = "$wrkdir/$host/$file";

    my ( $ret, $time_last_ok ) = load_data_3par_perf( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
    if ( isdigit($time_last_ok) == 1 && $time_last_ok > 1000000000 ) {

      # run alerting
      AlertStor2rrd::alert( $host, $st_type, $time_last_ok, "$wrkdir/..", $DEBUG );
    }
  } ## end foreach my $file (@files)

  if ( $in_exist == 0 ) {

    #main::error ("$host: NOTE: no new input files, exiting data load");
    return 1;
  }

  return 0;
} ## end sub load_data_3par_all

sub load_data_3par_conf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter          = 0;
  my $counter_ins      = 0;
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file         = "$wrkdir/$host/pool.cfg";
  my $port_file         = "$wrkdir/$host/PORT/port.cfg";
  my $config_file       = "$wrkdir/$host/config.html";
  my $pool_file_write   = 0;                               # true if any write there
  my $config_file_write = 0;                               # true if any write there
  open( FH,   "< $in_file" )         || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ )        && return 0;
  open( FHW,  "> $pool_file-tmp" )   || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ )  && return 0;
  open( FHWP, "> $port_file" )       || main::error( "Can't open $port_file : $! " . __FILE__ . ":" . __LINE__ )      && return 0;
  open( FCFG, "> $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FCFG "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD> 
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > 
        <HR><CENTER><B>System overview:</B>(it is generated once a day, last run : $ltime)</CENTER><HR><PRE>\n";

  my @lines = <FH>;
  my $line  = "";

  my @pool_name            = "";
  my @pool_name_id         = "";
  my @pool_capacity_tot    = "";
  my @pool_capacity_free   = "";
  my @pool_cfg_id          = "";
  my $pool_indx            = 0;
  my $cfg_print            = 1;
  my $pool_cfg_processing  = 0;
  my $port_cfg_processing  = 0;
  my $drive_cfg_processing = 0;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);
    $line =~ s/
//g;
    $line =~ s/^ *//g;

    # print config file
    print FCFG "$line\n";
    $counter_ins++;

    if ( $line =~ m/Pool Level Configuration/ || $line =~ m/POOL Level Configuration/ ) {
      $pool_cfg_processing = 1;
      next;
    }
    if ( $line =~ m/Port Level Configuration/ ) {
      $port_cfg_processing = 1;
      next;
    }
    if ( $line =~ m/Physical Disk Configuration/ ) {
      $drive_cfg_processing = 1;
      next;
    }

    if ($drive_cfg_processing) {

      # Physical Disk ID,Cage Position,Type,RPM,State,Total Size [MB],Free Size [MB],Port A,Port B,Capacity [GB],
      # 0,0:0:0,FC,10,normal,417792,273408,1:0:1*,0:0:1,450,

      if ( $line =~ m/^$/ ) {
        $drive_cfg_processing = 0;
        load_pool_capacity( $no_time, $step, $st_type, $wrkdir, $host, "TIER", $act_time_u, $DEBUG, $act_time, \@pool_cfg_id, \@pool_capacity_tot, \@pool_capacity_free, \@pool_name );
        @pool_capacity_tot  = "";
        @pool_capacity_free = "";
        @pool_cfg_id        = "";
        @pool_name          = "";
        next;
      } ## end if ( $line =~ m/^$/ )
      if ( $line =~ m/^Physical Disk/ || $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ ) {
        next;    # some trash like "---"
      }
      my ( undef, undef, $disk_type, undef, undef, $total, $free ) = split( /,/, $line );
      my $pool_id = 5;    # unknown
      if ( $disk_type =~ m/^FC/ )  { $pool_id = 1; }
      if ( $disk_type =~ m/^SSD/ ) { $pool_id = 2; }
      if ( $disk_type =~ m/^NL/ )  { $pool_id = 3; }
      if ( $disk_type =~ m/^SAS/ ) { $pool_id = 4; }
      if ( defined($total) && isdigit($total) ) {
        $total = $total / 1048576;
        $pool_capacity_tot[$pool_id] += $total;
      }
      if ( defined($free) && isdigit($free) ) {
        $free = $free / 1048576;
        $pool_capacity_free[$pool_id] += $free;
      }
      $pool_cfg_id[$pool_id] = $pool_id;
      $pool_name[$pool_id]   = $disk_type;

      #print "001 $pool_id - $line\n";
    } ## end if ($drive_cfg_processing)

    if ($port_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $port_cfg_processing = 0;
        next;    # end of cfg
      }

      if ( $line =~ m/^N:S:P/ || $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ ) {
        next;    # some trash like "---"
      }

      # N:S:P,Mode,State,Node WWN,Port WWN,Type,Protocol,Label,Partner,Failover State,
      # 0:0:1,initiator,ready,50002ACFF700AC03,50002AC00100AC03,disk,SAS,DP-1,-,-,
      my ( $port_id, undef, undef, undef, undef, undef, $prot, $label ) = split( /,/, $line );
      $port_id =~ s/:/-/g;    # filter ":"
      if ( $label eq "-" ) {

        # no label defined
        print FHWP "$port_id:$port_id ($prot)\n";
      }
      else {
        print FHWP "$port_id:$label ($prot)\n";
      }
      next;
    } ## end if ($port_cfg_processing)

    if ($pool_cfg_processing) {
      if ( $line =~ m/^$/ ) {
        $pool_cfg_processing = 0;

        # load capacity usage of pools (once a day into *.rrc)
        load_pool_capacity( $no_time, $step, $st_type, $wrkdir, $host, "POOL", $act_time_u, $DEBUG, $act_time, \@pool_cfg_id, \@pool_capacity_tot, \@pool_capacity_free, \@pool_name );
        @pool_capacity_tot  = "";
        @pool_capacity_free = "";
        @pool_cfg_id        = "";
        @pool_name          = "";
        next;    # end of cfg
      } ## end if ( $line =~ m/^$/ )
      $line =~ s/^0x//;

      if ( $line =~ m/^CPG ID,CPG name/ || $line =~ m/Interval Start:/ || $line =~ m/Interval Length:/ || $line =~ m/^---/ || $line =~ m/^name,id,status/ || $line =~ m/^,id,status/ || $line =~ m/^name,id,node_name/ ) {
        next;    # some trash like "---"
      }

      # create an array with mapping table of pools
      $cfg_load_data_ok = 1;
      my $pool_cap_free = 0;
      my ( $pool_id_cfg_item, $pool_name_cfg_item, $pool_cap_tot, $pool_cap_used ) = split( /,/, $line );

      # CPG ID,CPG name,User Total Capacity [MB],User Used Capacity [MB],Snapshot Total Capacity [MB],Snapshot Used Capacity [MB],Admin Total Capacity [MB],Admin Used Capacity [MB],
      # 0,FC_r1,512,512,32256,0,8192,128,

      if ( $pool_name_cfg_item eq '' ) {
        next;
      }

      if ( isdigit($pool_cap_tot) == 1 && isdigit($pool_cap_used) == 1 ) {
        $pool_cap_tot                   = $pool_cap_tot / 1048576;
        $pool_cap_used                  = $pool_cap_used / 1048576;
        $pool_cap_free                  = $pool_cap_tot - $pool_cap_used;
        $pool_cfg_id[$pool_id_cfg_item] = $pool_id_cfg_item;
        $pool_capacity_tot[$pool_indx]  = $pool_cap_tot;
        $pool_capacity_free[$pool_indx] = $pool_cap_free;
      } ## end if ( isdigit($pool_cap_tot...))

      # save actual pool cfg
      $pool_name[$pool_indx]    = $pool_name_cfg_item;
      $pool_name_id[$pool_indx] = $pool_id_cfg_item;
      $pool_indx++;
      print FHW "$pool_id_cfg_item:$pool_name_cfg_item\n";
      next;
    } ## end if ($pool_cfg_processing)
  }    # foreach

  close(FHW);
  close(FHWP);
  print FCFG "</pre></body></html>\n";
  close(FCFG);

  if ($counter_ins) {
    print "cfg processed  : $host cfg rows: $counter_ins\n" if $DEBUG;
  }

  # check if cfg section has been in the input file and pools have been found, if so then replace pool.cfg by pool.cfg-tmp
  open( FHR, "< $pool_file-tmp" ) || main::error( "Can't open $pool_file-tmp : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FHR>;
  close(FHR);
  my @lines_write = sort { ( split ':', $a )[0] cmp( split ':', $b )[0] } @lines;    # not numeric sorting! alphabetical one as pool ID might be in hexa

  foreach $line (@lines) {

    # mean there is at least one row, --> replace
    if ( !defined($line) || $line eq '' ) {
      next;
    }
    open( FHR, "> $pool_file" ) || main::error( "Can't open $pool_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_write (@lines_write) {
      print FHR "$line_write";

      #print "$line_write";
    }
    close(FHR);
    print "cfg pool found : $pool_file-tmp --> $pool_file\n" if $DEBUG;
    unlink("$pool_file-tmp");
    last;
  } ## end foreach $line (@lines)

  # same as above for config.html
  open( FCFGR, "< $config_file-tmp" ) || main::error( "Can't open $config_file-tmp: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FCFGR>;
  close(FCFGR);
  my $cfg_section = 0;
  foreach $line (@lines) {
    chomp($line);
    if ( $line =~ m/Pool Level/ || $line =~ m/Volume Level/ || $line =~ m/POOL Level/ || $line =~ m/VOLUME Level/ ) {

      # the file must contain all 3 sections
      $cfg_section++;
    }
  } ## end foreach $line (@lines)

  if ( $cfg_section > 1 ) {
    print "cfg found      : $config_file-tmp --> $config_file\n" if $DEBUG;
  }
  else {
    if ( $cfg_section == 0 ) {
      print "cfg not found  : no config in this data file\n" if $DEBUG;
    }
  }
  if ( -f "$config_file-tmp" ) {
    rename( "$config_file-tmp", "$config_file" );    # rename it anyway even if some problem
  }

  close(FH);
  return 0;
} ## end sub load_data_3par_conf

sub line_to_hash_3par {
  my ( $line, $tmp_hash ) = @_;
  my %inventory_host = %{$tmp_hash};

  my @data = split( ",", $line );

  ### id host
  if ( defined $data[0] && $data[0] ne "" && defined $data[23] && $data[23] ne "" && defined $data[24] && $data[24] ne "" ) {
    $inventory_host{ $data[0] }{ID}     = $data[0];
    $inventory_host{ $data[0] }{NAME}   = $data[23];
    $inventory_host{ $data[0] }{VOLUME} = $data[24];
    $hosts{ $data[23] }                 = $data[23];
  } ## end if ( defined $data[0] ...)
  else {
    ## no valid line
    return %inventory_host;
  }
  if ( defined $data[1] ) {
    $inventory_host{ $data[0] }{ITEM}{"TIME"} = $data[1];
  }
  if ( defined $data[2] ) {
    $inventory_host{ $data[0] }{ITEM}{"RANGE"} = $data[2];
  }

  ### read_io
  if ( isdigit( $data[3] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"read_io"} = $data[3];
  }

  ### write_io
  if ( isdigit( $data[4] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"write_io"} = $data[4];
  }

  ### io_rate
  if ( isdigit( $data[5] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"io_rate"} = $data[5];
  }

  ### read
  if ( isdigit( $data[6] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"read"} = $data[6];
  }

  ### write
  if ( isdigit( $data[7] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"write"} = $data[7];
  }

  ### data_rate
  if ( isdigit( $data[8] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"data_rate"} = $data[8];
  }

  ### resp_t_r
  if ( isdigit( $data[12] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"resp_t_r"} = $data[12];
  }

  ### resp_t_w
  if ( isdigit( $data[13] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"resp_t_w"} = $data[13];
  }

  ### resp_t
  if ( isdigit( $data[14] ) ) {
    $inventory_host{ $data[0] }{ITEM}{"resp_t"} = $data[14];
  }

  return %inventory_host;

} ## end sub line_to_hash_3par

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub load_data_3par_perf {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;

  my $counter_ins      = 0;
  my $last_rec         = "";
  my $rrd              = "";
  my $time             = 3600;
  my $time_last_ok     = "";
  my $t                = "";
  my $type             = "NA";
  my $t1               = "";
  my $t2               = "";
  my $t3               = "";
  my $t4               = "";
  my $t5               = "";
  my $t6               = "";
  my $t7               = "";
  my $t8               = "";
  my $t9               = "";
  my $t10              = "";
  my $t11              = "";
  my $t12              = "";
  my $t13              = "";
  my $t14              = "";
  my $t15              = "";
  my $t16              = "";
  my $t17              = "";
  my $t18              = "";
  my $t19              = "";
  my $t20              = "";
  my $t21              = "";
  my $t22              = "";
  my $t23              = "";
  my $t24              = "";
  my $act_time_u       = time();
  my $ltime            = localtime($act_time_u);
  my $cfg_load_data_ok = 0;                        # it is updated when there is found any mdisk, if 0 then something wrong and do not replace config.html
  my %inventory_host;
  my $type_host = "HOST";
  my %inventory_pool;
  my %pool_mapping;
  my %in_pool;

  print "updating RRD   : $host:$in_file\n" if $DEBUG;
  my $pool_file_write   = 0;                       # true if any write there
  my $config_file_write = 0;                       # true if any write there
  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FH>;
  my $line  = "";

  my @pool_cfg_id       = "";
  my @pool_name         = "";
  my @pool_name_id      = "";
  my $pool_name_indx    = 0;
  my $port_type_once    = 0;
  my $volume_cache_once = 0;
  my $error_once        = 0;

  # pool fron-end data from volume data
  my @pool_data_rate  = "";
  my @pool_read       = "";
  my @pool_write      = "";
  my @pool_io_rate    = "";
  my @pool_io_read    = "";
  my @pool_io_write   = "";
  my @pool_resp       = "";
  my @pool_resp_r     = "";
  my @pool_resp_w     = "";
  my @pool_ids        = "";
  my $pool_time       = 0;
  my $pool_ltime      = 0;
  my @pool_read_size  = "";
  my @pool_write_size = "";
  my @pool_data_size  = "";
  my $host_data_all   = 0;
  my $pool_active     = 0;
  my $size_lines      = scalar @lines;
  my $count_lines     = 0;

  foreach $line (@lines) {
    chomp($line);
    $count_lines++;
    $line =~ s/
//g;
    $line =~ s/^ *//g;
    if ( $line eq "" && $type =~ m/^HOST$/ ) {
      $host_data_all = 1;
    }

    if ( $line =~ m/Pool Capacity Statistics/ ) {
      $type = "POOLcap";
      next;
    }
    if ( $line =~ m/Volume Cache Level Statistics/ ) {
      $type = "VOLUMECache";
      next;
    }
    if ( $line =~ m/Physical Disk Level Statistics/ ) {
      $type = "DRIVE";
      next;
    }

    if ( $line =~ m/Level Statistics/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      if ( $type_temp =~ m/Logical/ && $trash =~ m/Drive/ ) {
        $type = "RANK";    #  Logical Drive --> RANK conversion
      }
      else {
        $type = uc($type_temp);
      }
      next;
    } ## end if ( $line =~ m/Level Statistics/)

    if ( $line =~ m/Interval End:/ || $line =~ m/Interval Start:/ ) {
      $line =~ s/	//g;
      $line =~ s/Interval End:     //g;
      $line =~ s/GMT.*//g;
      my $line_conv = $line;    # temp fix when data is in wrong format
      if ( $line =~ m/_/ ) {
        $line =~ s/_/ /g;
        $line =~ s/Interval Start:   //g;
        $line =~ s/\(STOR2RRD Server time\)//g;
        $line_conv = "20" . substr( $line, 0, 2 ) . ":" . substr( $line, 2, 2 ) . ":" . substr( $line, 4, 2 ) . "T" . substr( $line, 7, 2 ) . ":" . substr( $line, 9, 2 ) . ":" . substr( $line, 11, 2 );

        #2015:01:27T03:30:03.000000 --> right one
        #160128_052000
      } ## end if ( $line =~ m/_/ )
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line_conv);
      if ( !defined($time) || isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $line_conv " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 3;
      next;
    } ## end if ( $line =~ m/Interval End:/...)

    if ( $line =~ m/^$/ ) {

      # no do not do it here as it combines drive data (back-end) and front-end data from volumes
      if ( $pool_time > 0 ) {

        # insert summed drive data for TIER, it must be even at the enf of that foreach as volumes normally ends the perf file
        if ( $pool_time > $pool_ltime ) {
          $pool_ltime = pool_sum_insert( $type, $wrkdir, $step, $DEBUG, $host, $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@pool_data_rate, \@pool_io_rate, \@pool_resp, \@pool_read_size, \@pool_write_size, \@pool_data_size );
        }

        # reinitialize structures
        @pool_ids        = "";
        @pool_read       = "";
        @pool_write      = "";
        @pool_data_rate  = "";
        @pool_io_read    = "";
        @pool_io_write   = "";
        @pool_io_rate    = "";
        @pool_resp_r     = "";
        @pool_resp_w     = "";
        @pool_resp       = "";
        $pool_time       = 0;
        $pool_ltime      = 0;    # do this only for 3PAR as there are POOL and TIERS independet
        @pool_read_size  = "";
        @pool_write_size = "";
        @pool_data_size  = "";
      } ## end if ( $pool_time > 0 )
      $type = "NA";              # clear out type when empty line, it must be after pool_sum_insert
      next;
    } ## end if ( $line =~ m/^$/ )

    if ( $line =~ m/^Physical Disk ID,Time/ || $line =~ m/Interval Length/ || $line =~ m/^Node,Time/ || $line =~ m/^Logical Drive ID,Time/ || $line =~ m/^Node-CPU Core,Time/ || $line =~ m/^N:S:P,Time,Interval/ || $line =~ m/^Volume ID,Time,Interval/ || $line =~ m/^  Interval / || $line =~ m/^        Interval/ || $line =~ m/^--------------------/ || $line =~ m/Interval Start:/ || $line =~ m/^host_id/ ) {

      # avoid other trash
      next;
    }

    if ($host_data_all) {
      $pool_active++;
      ### HOST SECTION
      my %inventory_volume;

      #print Dumper \%inventory_host;
      #print Dumper \%hosts;

      foreach my $host_act ( keys %hosts ) {
        my $read_io    = "";
        my $write_io   = "";
        my $io_rate    = "";
        my $data_rate  = "";
        my $read       = "";
        my $write      = "";
        my $resp_t     = "";
        my $resp_t_r   = "";
        my $resp_t_w   = "";
        my $time_text  = "";
        my $time_range = "";
        foreach my $id ( keys %inventory_host ) {

          if ( defined $inventory_host{$id}{NAME} && $inventory_host{$id}{NAME} eq "$host_act" ) {
            $time_text  = $inventory_host{$id}{ITEM}{"TIME"};
            $time_range = $inventory_host{$id}{ITEM}{"RANGE"};
            my $metric      = "read_io";
            my $read_io_loc = $inventory_host{$id}{ITEM}{$metric};
            $metric = "write_io";
            my $write_io_loc = $inventory_host{$id}{ITEM}{$metric};
            $metric = "io_rate";
            my $io_rate_loc = $inventory_host{$id}{ITEM}{$metric};

            $metric = "resp_t";
            my $resp_t_loc = $inventory_host{$id}{ITEM}{$metric};
            $metric = "resp_t_r";
            my $resp_t_r_loc = $inventory_host{$id}{ITEM}{$metric};
            $metric = "resp_t_w";
            my $resp_t_w_loc = $inventory_host{$id}{ITEM}{$metric};

            $metric = "data_rate";
            my $data_rate_loc = $inventory_host{$id}{ITEM}{$metric};
            $metric = "read";
            my $read_loc = $inventory_host{$id}{ITEM}{$metric};
            $metric = "write";
            my $write_loc = $inventory_host{$id}{ITEM}{$metric};

            $metric = "read_io";

            ### LUN RESP SECTION
            my $volume = $inventory_host{$id}{VOLUME};
            if ( defined $volume && $volume ne "" ) {
              if ( defined $inventory_volume{$volume} ) {
                my $metric  = "read_io";
                my $read_io = $inventory_volume{$volume}{ITEM}{$metric};
                $metric = "resp_t_r";
                my $resp_t_r = $inventory_volume{$volume}{ITEM}{$metric};
                $metric = "write_io";
                my $write_io = $inventory_volume{$volume}{ITEM}{$metric};
                $metric = "resp_t_w";
                my $resp_t_w = $inventory_volume{$volume}{ITEM}{$metric};
                $metric = "io_rate";
                my $io_rate = $inventory_volume{$volume}{ITEM}{$metric};
                $metric = "rest_t";
                my $resp_t = $inventory_volume{$volume}{ITEM}{$metric};

                $metric = "data_rate";
                my $data_rate = $inventory_volume{$volume}{ITEM}{$metric};
                $metric = "read";
                my $read = $inventory_volume{$volume}{ITEM}{$metric};
                $metric = "write";
                my $write = $inventory_volume{$volume}{ITEM}{$metric};

                if ( isdigit($read_io_loc) ) {
                  $metric = "read_io";
                  if ( isdigit($read_io) ) {
                    my $sum = $read_io + $read_io_loc;
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $read_io_loc;
                  }

                } ## end if ( isdigit($read_io_loc...))

                if ( isdigit($write_io_loc) ) {
                  $metric = "write_io";
                  if ( isdigit($write_io) ) {
                    my $sum = $write_io + $write_io_loc;
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $write_io_loc;
                  }

                } ## end if ( isdigit($write_io_loc...))

                if ( isdigit($io_rate_loc) ) {
                  $metric = "io_rate";
                  if ( isdigit($io_rate) ) {
                    my $sum = $io_rate + $io_rate_loc;
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $io_rate_loc;
                  }
                } ## end if ( isdigit($io_rate_loc...))

                if ( isdigit($data_rate_loc) ) {
                  $metric = "data_rate";
                  if ( isdigit($data_rate) ) {
                    my $sum = $data_rate + $data_rate_loc;
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $data_rate_loc;
                  }
                } ## end if ( isdigit($data_rate_loc...))

                if ( isdigit($read_loc) ) {
                  $metric = "read";
                  if ( isdigit($read) ) {
                    my $sum = $read + $read_loc;
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $read_loc;
                  }
                } ## end if ( isdigit($read_loc...))

                if ( isdigit($write_loc) ) {
                  $metric = "write";
                  if ( isdigit($write) ) {
                    my $sum = $write + $write_loc;
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $write_loc;
                  }
                } ## end if ( isdigit($write_loc...))

                if ( isdigit($resp_t_loc) && isdigit($io_rate_loc) ) {
                  $metric = "resp_t";
                  if ( isdigit($resp_t) ) {
                    my $sum = $resp_t + ( $resp_t_loc * $io_rate_loc );
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $resp_t_loc * $io_rate_loc;
                  }
                } ## end if ( isdigit($resp_t_loc...))

                if ( isdigit($resp_t_r_loc) && isdigit($read_io_loc) ) {
                  $metric = "resp_t_r";
                  if ( isdigit($resp_t_r) ) {
                    my $sum = $resp_t_r + ( $resp_t_r_loc * $read_io_loc );
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $resp_t_r_loc * $read_io_loc;
                  }
                } ## end if ( isdigit($resp_t_r_loc...))

                if ( isdigit($resp_t_w_loc) && isdigit($write_io_loc) ) {
                  $metric = "resp_t_w";
                  if ( isdigit($resp_t_w) ) {
                    my $sum = $resp_t_w + ( $resp_t_w_loc * $write_io_loc );
                    $inventory_volume{$volume}{ITEM}{$metric} = $sum;
                  }
                  else {
                    $inventory_volume{$volume}{ITEM}{$metric} = $resp_t_w_loc * $write_io_loc;
                  }
                } ## end if ( isdigit($resp_t_w_loc...))

              } ## end if ( defined $inventory_volume...)
              else {
                $inventory_volume{$volume}{ITEM}{"RANGE"} = $time_range;
                $inventory_volume{$volume}{ITEM}{"TIME"}  = $time_text;
                my $metric = "read_io";
                if ( isdigit($read_io_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $read_io_loc;
                }
                $metric = "resp_t_r";
                if ( isdigit($read_io_loc) && isdigit($resp_t_r_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $read_io_loc * $resp_t_r_loc;
                }
                $metric = "write_io";
                if ( isdigit($write_io_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $write_io_loc;
                }
                $metric = "resp_t_w";
                if ( isdigit($write_io_loc) && isdigit($resp_t_w_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $write_io_loc * $resp_t_w_loc;
                }
                $metric = "io_rate";
                if ( isdigit($io_rate_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $io_rate_loc;
                }
                $metric = "resp_t";
                if ( isdigit($io_rate_loc) && isdigit($resp_t_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $io_rate_loc * $resp_t_loc;
                }
                $metric = "data_rate";
                if ( isdigit($data_rate_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $data_rate_loc;
                }
                $metric = "read";
                if ( isdigit($read_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $read_loc;
                }
                $metric = "write";
                if ( isdigit($write_loc) ) {
                  $inventory_volume{$volume}{ITEM}{$metric} = $write_loc;
                }
              } ## end else [ if ( defined $inventory_volume...)]
            } ## end if ( defined $volume &&...)

            ###

            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($read_io) ) {
                $read_io = $read_io + $inventory_host{$id}{ITEM}{$metric};
              }
              else {
                $read_io = $inventory_host{$id}{ITEM}{$metric};
              }
            } ## end if ( isdigit( $inventory_host...))
            $metric = "write_io";
            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($write_io) ) {
                $write_io = $write_io + $inventory_host{$id}{ITEM}{$metric};
              }
              else {
                $write_io = $inventory_host{$id}{ITEM}{$metric};
              }
            } ## end if ( isdigit( $inventory_host...))
            $metric = "io_rate";
            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($io_rate) ) {
                $io_rate = $io_rate + $inventory_host{$id}{ITEM}{$metric};
              }
              else {
                $io_rate = $inventory_host{$id}{ITEM}{$metric};
              }
            } ## end if ( isdigit( $inventory_host...))
            $metric = "read";
            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($read) ) {
                $read = $read + $inventory_host{$id}{ITEM}{$metric};
              }
              else {
                $read = $inventory_host{$id}{ITEM}{$metric};
              }
            } ## end if ( isdigit( $inventory_host...))
            $metric = "write";
            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($write) ) {
                $write = $write + $inventory_host{$id}{ITEM}{$metric};
              }
              else {
                $write = $inventory_host{$id}{ITEM}{$metric};
              }
            } ## end if ( isdigit( $inventory_host...))
            $metric = "data_rate";
            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($data_rate) ) {
                $data_rate = $data_rate + $inventory_host{$id}{ITEM}{$metric};
              }
              else {
                $data_rate = $inventory_host{$id}{ITEM}{$metric};
              }
            } ## end if ( isdigit( $inventory_host...))
            $metric = "resp_t";
            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($resp_t) && isdigit($io_rate_loc) ) {
                $resp_t = $resp_t + ( $inventory_host{$id}{ITEM}{$metric} * $io_rate_loc );
              }
              else {
                $resp_t = $io_rate_loc * $inventory_host{$id}{ITEM}{$metric};
              }
            } ## end if ( isdigit( $inventory_host...))
            $metric = "resp_t_r";
            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($resp_t_r) && isdigit($read_io_loc) ) {
                $resp_t_r = $resp_t_r + ( $inventory_host{$id}{ITEM}{$metric} * $read_io_loc );
              }
              else {
                $resp_t_r = $inventory_host{$id}{ITEM}{$metric} * $read_io_loc;
              }
            } ## end if ( isdigit( $inventory_host...))
            $metric = "resp_t_w";
            if ( isdigit( $inventory_host{$id}{ITEM}{$metric} ) ) {
              if ( isdigit($resp_t_w) && isdigit($write_io_loc) ) {
                $resp_t_w = $resp_t_w + ( $inventory_host{$id}{ITEM}{$metric} * $write_io_loc );
              }
              else {
                $resp_t_w = $inventory_host{$id}{ITEM}{$metric} * $write_io_loc;
              }
            } ## end if ( isdigit( $inventory_host...))
          } ## end if ( defined $inventory_host...)

          #delete $inventory_host{$id};
        } ## end foreach my $id ( keys %inventory_host)
        ### calculate resp_t*
        if ( isdigit($read_io) ) {
          if ( $read_io != 0 ) {
            my $val = round( $resp_t_r / $read_io, 2 );
            $resp_t_r = $val;
          }
        } ## end if ( isdigit($read_io))
        else {
          $resp_t_r = "U";
        }
        if ( isdigit($write_io) ) {
          if ( $write_io != 0 ) {
            my $val = round( $resp_t_w / $write_io, 2 );
            $resp_t_w = $val;
          }
        } ## end if ( isdigit($write_io...))
        else {
          $resp_t_w = "U";
        }
        if ( isdigit($io_rate) ) {
          if ( $io_rate != 0 ) {
            my $val = round( $resp_t / $io_rate, 2 );
            $resp_t = $val;
          }
        } ## end if ( isdigit($io_rate))
        else {
          $resp_t = "U";
        }

        #my $metric = "read_io";
        #$inventory_host{$host_act}{ITEM}{$metric} = $read_io;
        #$metric = "write_io";
        #$inventory_host{$host_act}{ITEM}{$metric} = $write_io;
        #$metric = "io_rate";
        #$inventory_host{$host_act}{ITEM}{$metric} = $io_rate;
        #$metric = "read";
        #$inventory_host{$host_act}{ITEM}{$metric} = $read;
        #$metric = "write";
        #$inventory_host{$host_act}{ITEM}{$metric} = $write;
        #$metric = "data_rate";
        #$inventory_host{$host_act}{ITEM}{$metric} = $data_rate;
        #$metric = "resp_t";
        #$inventory_host{$host_act}{ITEM}{$metric} = $resp_t;
        #$metric = "resp_t_r";
        #$inventory_host{$host_act}{ITEM}{$metric} = $resp_t_r;
        #$metric = "resp_t_w";
        #$inventory_host{$host_act}{ITEM}{$metric} = $resp_t_w;

        if ( !isdigit($read_io) ) {
          $read_io = "U";
        }
        if ( !isdigit($write_io) ) {
          $write_io = "U";
        }
        if ( !isdigit($io_rate) ) {
          $io_rate = "U";
        }
        if ( !isdigit($read) ) {
          $read = "U";
        }
        if ( !isdigit($write) ) {
          $write = "U";
        }
        if ( !isdigit($data_rate) ) {
          $data_rate = "U";
        }
        if ( !isdigit($resp_t) ) {
          $resp_t = "U";
        }
        if ( !isdigit($resp_t_w) ) {
          $resp_t_w = "U";
        }
        if ( !isdigit($resp_t_r) ) {
          $resp_t_r = "U";
        }

        my $time_host = $time;
        if ( defined($time_range) && !$time_range eq '' && isdigit($time_range) && $time_range < 300 ) {

          # use actual $time_text instead of section one as there might be more intervals, HP3PAR_VV_SAMPLE_RATE is probably set (30 or 60 secs)
          my $time_act = str2time($time_text);
          if ( defined($time_act) && !$time_act eq '' && isdigit($time_act) ) {
            $time_host = $time_act;
          }
        } ## end if ( defined($time_range...))

        print "001 HOST $host_act - $read_io - $write_io - $io_rate - $read - $write - $data_rate - $resp_t_r - $resp_t_w - $resp_t \n" if $DEBUG == 3;

        #print "001 VOL $name - $read_io - $write_io - $resp_t_r - $resp_t_w - $read - $write\n";
        $rrd = "$wrkdir/$host/$type_host/$host_act.rrd";

        #create rrd db if necessary
        $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
        if ( create_rrd( $rrd, $time_host, $step, $DEBUG, $host, $no_time, $act_time, $type_host, $st_type ) == 1 ) {
          return 2;        # leave uit if any error during RRDTool file creation to keep data
        }

        my $l;
        my $l_count = 0;
        my $found   = -1;
        my $ltime;
        foreach $l (@host_name) {
          if ( $l =~ m/^$rrd$/ ) {
            $found = $l_count;
            last;
          }
          $l_count++;
        } ## end foreach $l (@host_name)
        if ( $found > -1 ) {
          $ltime = $host_time[$found];
        }
        else {
          # find out last record in the db
          # as this makes it slowly to test it each time then it is done
          # once per a lpar for whole load and saved into the array
          eval {
            RRDp::cmd qq(last "$rrd" );
            $last_rec = RRDp::read;
          };
          if ($@) {
            rrd_error( $@, $rrd );
            next;
          }
          chomp($$last_rec);
          chomp($$last_rec);
          $host_name[$l_count] = $rrd;
          $host_time[$l_count] = $$last_rec;
          $ltime               = $$last_rec;

          #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
        } ## end else [ if ( $found > -1 ) ]
        if ( isdigit($time_host) && isdigit($ltime) && $time_host > $ltime ) {
          $host_time[$l_count] = $time_host;
          $counter_ins++;
          print "004: HOST $time_host:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t\n" if $DEBUG == 3;
          RRDp::cmd qq(update "$rrd" $time_host:$read_io:$write_io:$io_rate:$read:$write:$data_rate:$resp_t_r:$resp_t_w:$resp_t);
          my $answer = RRDp::read;
          $time_last_ok = $time;
        } ## end if ( isdigit($time_host...))

      } ## end foreach my $host_act ( keys...)

      #### LUN RESP
      foreach my $volume ( keys %inventory_volume ) {
        my $metric  = "read_io";
        my $read_io = $inventory_volume{$volume}{ITEM}{$metric};
        $metric = "write_io";
        my $write_io = $inventory_volume{$volume}{ITEM}{$metric};
        $metric = "io_rate";
        my $io_rate = $inventory_volume{$volume}{ITEM}{$metric};
        $metric = "resp_t_r";
        my $resp_t_r = $inventory_volume{$volume}{ITEM}{$metric};
        $metric = "resp_t_w";
        my $resp_t_w = $inventory_volume{$volume}{ITEM}{$metric};
        $metric = "resp_t";
        my $resp_t = $inventory_volume{$volume}{ITEM}{$metric};
        $metric = "data_rate";
        my $data_rate = $inventory_volume{$volume}{ITEM}{$metric};
        $metric = "read";
        my $read = $inventory_volume{$volume}{ITEM}{$metric};
        $metric = "write";
        my $write = $inventory_volume{$volume}{ITEM}{$metric};

        my $time_range = $inventory_volume{$volume}{ITEM}{"RANGE"};
        my $time_text  = $inventory_volume{$volume}{ITEM}{"TIME"};

        my $time_volume = $time;

        if ( defined($time_range) && !$time_range eq '' && isdigit($time_range) && $time_range < 300 ) {

          # use actual $time_text instead of section one as there might be more intervals, HP3PAR_VV_SAMPLE_RATE is probably set (30 or 60 secs)
          my $time_act = str2time($time_text);
          if ( defined($time_act) && !$time_act eq '' && isdigit($time_act) ) {
            $time_volume = $time_act;
          }
        } ## end if ( defined($time_range...))

        ### calculate

        if ( !isdigit($read_io) ) {
          $resp_t_r = "U";
          $read_io  = "U";
        }
        if ( !isdigit($write_io) ) {
          $resp_t_w = "U";
          $write_io = "U";
        }
        if ( !isdigit($io_rate) ) {
          $io_rate = "U";
          $resp_t  = "U";
        }

        if ( !isdigit($read) ) {
          $read = "U";
        }
        if ( !isdigit($write) ) {
          $write = "U";
        }
        if ( !isdigit($data_rate) ) {
          $data_rate = "U";
        }

        if ( isdigit($resp_t_r) && isdigit($read_io) ) {
          if ( $read_io != 0 ) {
            my $val = round( $resp_t_r / $read_io, 2 );
            $resp_t_r = $val;
          }
          else { $resp_t_r = "U"; }
        } ## end if ( isdigit($resp_t_r...))

        if ( isdigit($resp_t_w) && isdigit($write_io) ) {
          if ( $write_io != 0 ) {
            my $val = round( $resp_t_w / $write_io, 2 );
            $resp_t_w = $val;
          }
          else { $resp_t_w = "U"; }
        } ## end if ( isdigit($resp_t_w...))

        if ( isdigit($resp_t) && isdigit($io_rate) ) {
          if ( $io_rate != 0 ) {
            my $val = round( $resp_t / $io_rate, 2 );
            $resp_t = $val;
          }
          else { $resp_t = "U"; }
        } ## end if ( isdigit($resp_t) ...)

        if ( !isdigit($resp_t) )   { $resp_t   = "U"; }
        if ( !isdigit($resp_t_r) ) { $resp_t_r = "U"; }
        if ( !isdigit($resp_t_w) ) { $resp_t_w = "U"; }
        my $cache_hit_r = 'U';
        my $cache_hit_w = 'U';
        my $used        = 'U';
        my $real        = 'U';
        my $cap         = 'U';

        print "001 VOLUME $volume - $io_rate - $read_io - $write_io - $data_rate - $read - $write $resp_t_r - $resp_t_w - $resp_t - $cache_hit_r - $cache_hit_w - $used - $real - $cap \n" if $DEBUG == 3;

        #print "001 VOL $name - $read_io - $write_io - $resp_t_r - $resp_t_w - $read - $write\n";

        my $id = find_id( $volume, $host, $wrkdir );
        if ( !defined $id || $id eq "" ) {
          next;
        }

        $rrd = "$wrkdir/$host/VOLUME/$id.rrd";

        #create rrd db if necessary
        $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
        if ( create_rrd( $rrd, $time_volume, $step, $DEBUG, $host, $no_time, $act_time, "VOLUME", $st_type ) == 1 ) {
          return 2;        # leave uit if any error during RRDTool file creation to keep data
        }

        my $l;
        my $l_count = 0;
        my $found   = -1;
        my $ltime;
        foreach $l (@volume_name) {
          if ( $l =~ m/^$rrd$/ ) {
            $found = $l_count;
            last;
          }
          $l_count++;
        } ## end foreach $l (@volume_name)
        if ( $found > -1 ) {
          $ltime = $volume_time[$found];
        }
        else {
          # find out last record in the db
          # as this makes it slowly to test it each time then it is done
          # once per a lpar for whole load and saved into the array
          eval {
            RRDp::cmd qq(last "$rrd" );
            $last_rec = RRDp::read;
          };
          if ($@) {
            rrd_error( $@, $rrd );
            next;
          }
          chomp($$last_rec);
          chomp($$last_rec);
          $volume_name[$l_count] = $rrd;
          $volume_time[$l_count] = $$last_rec;
          $ltime                 = $$last_rec;

          #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
        } ## end else [ if ( $found > -1 ) ]
        if ( isdigit($time_volume) && isdigit($ltime) && $time_volume > $ltime ) {
          $volume_time[$l_count] = $time_volume;
          $counter_ins++;
          print "004: VOLUME $volume:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate:$cache_hit_r:$cache_hit_w:$cap:$used:$real\n" if $DEBUG == 3;

          #### volume statistics to pool
          my $metric = "read_io";
          $inventory_pool{$id}{$metric}       = $read_io;
          $metric                             = "write_io";
          $inventory_pool{$id}{$metric}       = $write_io;
          $metric                             = "io_rate";
          $inventory_pool{$id}{$metric}       = $io_rate;
          $metric                             = "read";
          $inventory_pool{$id}{$metric}       = $read;
          $metric                             = "write";
          $inventory_pool{$id}{$metric}       = $write;
          $metric                             = "data_rate";
          $inventory_pool{$id}{$metric}       = $data_rate;
          $metric                             = "resp_t_r";
          $inventory_pool{$id}{$metric}       = $resp_t_r;
          $metric                             = "resp_t_w";
          $inventory_pool{$id}{$metric}       = $resp_t_w;
          $metric                             = "resp_t";
          $inventory_pool{$id}{$metric}       = $resp_t;
          $metric                             = "r_cache_hit";
          $inventory_pool{$id}{$metric}       = $cache_hit_r;
          $metric                             = "w_cache_hit";
          $inventory_pool{$id}{$metric}       = $cache_hit_w;
          $inventory_pool{$id}{ITEM}{"RANGE"} = $time_range;
          $inventory_pool{$id}{ITEM}{"TIME"}  = $time_text;

          ####

          RRDp::cmd qq(update "$rrd" $time_volume:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate:$cache_hit_r:$cache_hit_w:$cap:$used:$real);
          my $answer = RRDp::read;
          $time_last_ok = $time;
        } ## end if ( isdigit($time_volume...))

      } ## end foreach my $volume ( keys %inventory_volume)

      $host_data_all = 0;
    } ## end if ($host_data_all)

    # Port
    ########################

    if ( $type =~ m/^PORT$/ ) {
      print "000 PORT: $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name, my $time_text, my $time_range, my $read_io, my $write_io,       my $io_rate,        my $read, my $write, my $data_rate, my $read_size, my $write_size, my $data_size, my $resp_t_r, my $resp_t_w, my $resp_t, $t1, $t2, $t3, $t4,
        $t5,      $t6,           $t7,            $t8,         my $port_type_disk, my $port_type_text, my $label
      ) = split( /,/, $line );

      my $name_tmp = $name;
      $name_tmp =~ s/://g;
      if ( isdigit($name_tmp) == 0 ) {
        next;
      }

      # N:S:P,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),
      #  Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),,,,,,,,,Type,Protocol,Label,,,,,^M
      #  0:0:1,2016-01-24 04:25:55,300,6,7,13,93,141,234,16.20,20.30,18.50,4.15,8.75,6.67,,,,,,,,,disk,SAS,DP-1,,,,,,^M
      #  0:0:2,2016-01-24 04:25:55,300,3,3,6,47,46,92,14.80,16.40,15.50,0.47,16.53,8.00,,,,,,,,,disk,SAS,DP-2,,,,,,^M

      # type : FC / iSCSI / SAS (PCIe)
      my $port_type = 3;    # default unknown port type
      if ( !defined($port_type_text) || $port_type_text eq '' ) {
        if ( $port_type_once == 0 ) {
          main::error( "$host: unknown port type text: $name : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
          $port_type_once = 1;
        }
        next;
      } ## end if ( !defined($port_type_text...))
      if ( $port_type_text =~ m/^FC$/ )   { $port_type = 0; }
      if ( $port_type_text =~ m/^SAS/ )   { $port_type = 1; }
      if ( $port_type_text =~ m/^iSCSI/ ) { $port_type = 2; }
      if ( $port_type_text =~ m/^IPREP/ ) { $port_type = 4; }    # tady se nejspise nepouziva, to je z SVC ale nechavam pro jistotu
      if ( $port_type_text =~ m/^FCoE/ )  { $port_type = 5; }
      if ( $port_type_text =~ m/^RCIP/ )  { $port_type = 6; }    # It could be RCIP1 etc

      if ( $port_type == 3 && $port_type_once == 0 ) {

        # unknown port type detected, print error message just once
        main::error( "$host: unknown port type: $name : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
        $port_type_once = 1;
      } ## end if ( $port_type == 3 &&...)

      #print "001 PORT $name - $write_io - $read_io - $io_rate - $write - $read - $data_rate - type:$port_type:$port_type_text\n";
      print "001 PORT $name - $write_io - $read_io - $io_rate - $write - $read - $data_rate - type:$port_type\n" if $DEBUG == 3;

      ( my $node, my $sys, my $port ) = split( /:/, $name );
      if ( !defined($port) || $port eq '' ) {
        if ( $port_type_once == 0 ) {
          main::error( "$host: unknown port type text: $name : $port_type_text : $line " . __FILE__ . ":" . __LINE__ );
          $port_type_once = 1;
        }
        next;
      } ## end if ( !defined($port) ||...)

      #$rrd = "$wrkdir/$host/$type/node".$node."_sys".$sys."_p".$port."\.rrd";
      $name =~ s/:/-/g;    # rrdtool does not like ":" like it is here 0:1:2
      $rrd = "$wrkdir/$host/$type/$name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;      # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;          # leave uit if any error during RRDTool file creation to keep data
      }

      #print "001 $rrd,$time,$step,$DEBUG,$host,$no_time,$act_time,$type,$st_type\n" if $DEBUG == 3;

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@port_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@port_name)
      if ( $found > -1 ) {
        $ltime = $port_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $port_name[$l_count] = $rrd;
        $port_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      if ( !defined($io_rate)   || $io_rate eq '' )   { $io_rate   = 'U'; }
      if ( !defined($read_io)   || $read_io eq '' )   { $read_io   = 'U'; }
      if ( !defined($write_io)  || $write_io eq '' )  { $write_io  = 'U'; }
      if ( !defined($resp_t_r)  || $resp_t_r eq '' )  { $resp_t_r  = 'U'; }
      if ( !defined($resp_t_w)  || $resp_t_w eq '' )  { $resp_t_w  = 'U'; }
      if ( !defined($read)      || $read eq '' )      { $read      = 'U'; }
      if ( !defined($write)     || $write eq '' )     { $write     = 'U'; }
      if ( !defined($data_rate) || $data_rate eq '' ) { $data_rate = 'U'; }

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $port_time[$l_count] = $time;
        $counter_ins++;
        print "004: PORT $time:$io_rate:$read_io:$write_io:$resp_t_r:$resp_t_w:$read:$write:$data_rate:$port_type\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$read_io:$write_io:$read:$write:$resp_t_r:$resp_t_w:$io_rate:$data_rate:$port_type);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^PORT$/)

    if ( $type =~ m/^HOST$/ ) {
      print "000 HOST: $line\n" if $DEBUG == 3;

      # Host statistics
      %inventory_host = line_to_hash_3par( $line, \%inventory_host );
    } ## end if ( $type =~ m/^HOST$/)

    # NODE
    ########################

    if ( $type =~ m/^NODE$/ ) {
      print "000 Node: $line\n" if $DEBUG == 3;
      ( my $name, my $time_text, my $time_range, undef, undef, undef, undef, undef, undef, my $total_read_io_access, my $total_write_io_access, my $read_hit_io, my $write_hit_io, my $read_hit, my $write_hit, my $total_read_lock_blocked, my $total_write_lock_blocked, undef, undef, undef, undef, undef, undef, my @tier_data ) = split( /,/, $line );

      #Node,Time,Interval (s),Current Read IO Accesses,Current Write IO Accesses,Current Read IO Hits,Current Write IO Hits,Current Read IO Hits Percent,Current Write IO Hits Percent,Total Read IO Accesses,Total Write IO Accesses,Total Read IO Hits,Total Write IO Hits,Total Read IO Hits Percent,Total Write IO Hits Percent,Total Read Lock Blocked,Total Write Lock Blocked,,,,,,,Tier Name,CfcDirty,CfcMax,DelAck,Tier Name,CfcDirty,CfcMax,DelAck,Tier Name,CfcDirty,CfcMax,DelAck,Tier Name,CfcDirty,CfcMax,DelAck,
      #0,2017-04-27 10:43:34,300,842,9555,813,2891,97,30,842,9555,813,2891,97,30,0,0,,,,,,,FC_10KRPM,6,9600,13319,FC_15KRPM,0,0,0,NL,0,0,0,SSD,0,0,0,
      #1,2017-04-27 10:43:34,300,1181,9009,1167,3619,99,40,1181,9009,1167,3619,99,40,0,0,,,,,,,FC_10KRPM,5,9600,9938,FC_15KRPM,0,0,0,NL,0,0,0,SSD,0,0,0,

      if ( isdigit($name) == 0 ) {
        next;
      }

      my $time_node = $time;
      if ( defined($time_range) && !$time_range eq '' && isdigit($time_range) && $time_range < 300 ) {

        # use actual $time_text instead of section one as there might be more intervals, HP3PAR_VV_SAMPLE_RATE is probably set (30 or 60 secs)
        my $time_act = str2time($time_text);
        if ( defined($time_act) && !$time_act eq '' && isdigit($time_act) ) {
          $time_node = $time_act;
        }
      } ## end if ( defined($time_range...))

      ### NODE TIER
      my $i         = 1;
      my $tier_name = "";
      my $dirty_val = "";
      my $max_val   = "";
      my $del_val   = "";
      foreach my $element (@tier_data) {
        if ( $i == 1 ) {
          $tier_name = $element;
          $i++;
          next;
        }
        if ( $i == 2 ) {
          $dirty_val = $element;
          $i++;
          next;
        }
        if ( $i == 3 ) {
          $max_val = $element;
          $i++;
          next;
        }
        if ( $i == 4 ) {
          $del_val = $element;

          if ( $tier_name eq "" ) {
            $i = 1;
            next;
          }
          if ( !isdigit($dirty_val) ) {
            $dirty_val = "U";
          }
          if ( !isdigit($max_val) ) {
            $max_val = "U";
          }
          if ( !isdigit($del_val) ) {
            $del_val = "U";
          }
          print "001 TIER-NODE $tier_name - $dirty_val - $max_val - $del_val\n" if $DEBUG == 3;
          my $rrt = "$wrkdir/$host/$type/$tier_name-node$name.rrt";

          #create rrd db if necessary
          $rrt =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
          if ( create_rrd( $rrt, $time_node, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
            return 2;        # leave uit if any error during RRDTool file creation to keep data
          }

          my $l;
          my $l_count = 0;
          my $found   = -1;
          my $ltime;
          foreach $l (@tier_name) {
            if ( $l =~ m/^$rrt$/ ) {
              $found = $l_count;
              last;
            }
            $l_count++;
          } ## end foreach $l (@tier_name)
          if ( $found > -1 ) {
            $ltime = $tier_time[$found];
          }
          else {
            # find out last record in the db
            # as this makes it slowly to test it each time then it is done
            # once per a lpar for whole load and saved into the array
            eval {
              RRDp::cmd qq(last "$rrt" );
              $last_rec = RRDp::read;
            };
            if ($@) {
              rrd_error( $@, $rrt );
              next;
            }
            chomp($$last_rec);
            chomp($$last_rec);
            $tier_name[$l_count] = $rrt;
            $tier_time[$l_count] = $$last_rec;
            $ltime               = $$last_rec;

            #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
          } ## end else [ if ( $found > -1 ) ]
          if ( isdigit($time_node) && isdigit($ltime) && $time_node > $ltime ) {
            $tier_time[$l_count] = $time_node;
            $counter_ins++;
            print "001 TIER-NODE $tier_name $dirty_val:$max_val:$del_val\n" if $DEBUG == 3;
            RRDp::cmd qq(update "$rrt" $time_node:$dirty_val:$max_val:$del_val);
            my $answer = RRDp::read;
            $time_last_ok = $time;
          } ## end if ( isdigit($time_node...))

          $i = 1;
        } ## end if ( $i == 4 )
      } ## end foreach my $element (@tier_data)

      print "001 NODE $name - $total_read_io_access - $total_write_io_access - $read_hit_io - $write_hit_io - $read_hit - $write_hit - $total_read_lock_blocked - $total_write_lock_blocked\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/node$name.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time_node, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@node_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@node_name)
      if ( $found > -1 ) {
        $ltime = $node_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $node_name[$l_count] = $rrd;
        $node_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      if ( !isdigit($total_read_io_access) )     { $total_read_io_access     = "U"; }
      if ( !isdigit($total_write_io_access) )    { $total_read_io_access     = "U"; }
      if ( !isdigit($read_hit) )                 { $read_hit                 = "U"; }
      if ( !isdigit($write_hit) )                { $write_hit                = "U"; }
      if ( !isdigit($read_hit_io) )              { $read_hit_io              = "U"; }
      if ( !isdigit($write_hit_io) )             { $write_hit_io             = "U"; }
      if ( !isdigit($total_read_lock_blocked) )  { $total_read_lock_blocked  = "U"; }
      if ( !isdigit($total_write_lock_blocked) ) { $total_write_lock_blocked = "U"; }

      if ( isdigit($time_node) && isdigit($ltime) && $time_node > $ltime ) {
        $node_time[$l_count] = $time_node;
        $counter_ins++;
        print "001 NODE $name - $total_read_io_access - $total_write_io_access - $read_hit_io - $write_hit_io - $read_hit - $write_hit - $total_read_lock_blocked - $total_write_lock_blocked\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time_node:$total_read_io_access:$total_write_io_access:$read_hit_io:$write_hit_io:$read_hit:$write_hit:$total_read_lock_blocked:$total_write_lock_blocked);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time_node...))

    } ## end if ( $type =~ m/^NODE$/)

    # Volume
    ########################

    if ( $type =~ m/^VOLUME$/ ) {
      $pool_active++;
      print "000 VVOL: $line\n" if $DEBUG == 3;
      ( my $name, my $time_text, my $time_range, my $read_io, my $write_io,    my $io_rate, my $read, my $write, my $data_rate, my $read_size, my $write_size, my $data_size, my $resp_t_r, my $resp_t_w, my $resp_t, $t1, $t2, $t3, $t4,
        $t5,      $t6,           $t7,            $t8,         my $volume_name, my $pool_id, my $pol_name
      ) = split( /,/, $line );

      ### POOL section for last volume nconsistent
      if ( isdigit($name) == 0 || !defined($name) || $name eq '' || !main::ishexa($name) || $name =~ m/\./ || !defined($pool_id) || $pool_id eq '' || !main::ishexa($pool_id) || $pool_id =~ m/\./ || $pool_id =~ m/^-$/ ) {

        if ( $pool_active > 1 && $count_lines == $size_lines ) {
          foreach my $pool_id ( keys %in_pool ) {
            my $sum_io_rate  = "";
            my $sum_read_io  = "";
            my $sum_write_io = "";

            my $sum_data_rate = "";
            my $sum_read      = "";
            my $sum_write     = "";

            my $sum_resp_t   = "";
            my $sum_resp_t_r = "";
            my $sum_resp_t_w = "";
            my $time_range   = "";
            my $time_text    = "";
            my $time_pool    = $time;
            foreach my $volume_id ( keys %inventory_pool ) {
              if ( defined $pool_mapping{$volume_id} && $pool_mapping{$volume_id} eq $pool_id ) {
                $time_range = $inventory_pool{$volume_id}{ITEM}{"RANGE"};
                $time_text  = $inventory_pool{$volume_id}{ITEM}{"TIME"};
                my $metric  = "io_rate";
                my $io_rate = $inventory_pool{$volume_id}{$metric};
                $metric = "read_io";
                my $read_io = $inventory_pool{$volume_id}{$metric};
                $metric = "write_io";
                my $write_io = $inventory_pool{$volume_id}{$metric};
                $metric = "data_rate";
                my $data_rate = $inventory_pool{$volume_id}{$metric};
                $metric = "read";
                my $read = $inventory_pool{$volume_id}{$metric};
                $metric = "write";
                my $write = $inventory_pool{$volume_id}{$metric};
                $metric = "resp_t";
                my $resp_t = $inventory_pool{$volume_id}{$metric};
                $metric = "resp_t_r";
                my $resp_t_r = $inventory_pool{$volume_id}{$metric};
                $metric = "resp_t_w";
                my $resp_t_w = $inventory_pool{$volume_id}{$metric};

                if ( !isdigit($sum_io_rate) ) {
                  if ( isdigit($io_rate) ) {
                    $sum_io_rate = $io_rate;
                  }
                }
                else {
                  if ( isdigit($io_rate) ) {
                    $sum_io_rate = $sum_io_rate + $io_rate;
                  }
                }

                if ( !isdigit($sum_read_io) ) {
                  if ( isdigit($read_io) ) {
                    $sum_read_io = $read_io;
                  }
                }
                else {
                  if ( isdigit($read_io) ) {
                    $sum_read_io = $sum_read_io + $read_io;
                  }
                }
                if ( !isdigit($sum_write_io) ) {
                  if ( isdigit($write_io) ) {
                    $sum_write_io = $write_io;
                  }
                }
                else {
                  if ( isdigit($write_io) ) {
                    $sum_write_io = $sum_write_io + $write_io;
                  }
                }

                if ( !isdigit($sum_data_rate) ) {
                  if ( isdigit($data_rate) ) {
                    $sum_data_rate = $data_rate;
                  }
                }
                else {
                  if ( isdigit($data_rate) ) {
                    $sum_data_rate = $sum_data_rate + $data_rate;
                  }
                }

                if ( !isdigit($sum_read) ) {
                  if ( isdigit($read) ) {
                    $sum_read = $read;
                  }
                }
                else {
                  if ( isdigit($read) ) {
                    $sum_read = $sum_read + $read;
                  }
                }
                if ( !isdigit($sum_write) ) {
                  if ( isdigit($write) ) {
                    $sum_write = $write;
                  }
                }
                else {
                  if ( isdigit($write) ) {
                    $sum_write = $sum_write + $write;
                  }
                }

                if ( !isdigit($sum_resp_t) ) {
                  if ( isdigit($resp_t) && isdigit($io_rate) ) {
                    $sum_resp_t = $io_rate * $resp_t;
                  }
                }
                else {
                  if ( isdigit($io_rate) && isdigit($resp_t) ) {
                    $sum_resp_t = $sum_resp_t + ( $io_rate * $resp_t );
                  }
                }
                if ( !isdigit($sum_resp_t_r) ) {
                  if ( isdigit($resp_t_r) && isdigit($read_io) ) {
                    $sum_resp_t_r = $read_io * $resp_t_r;
                  }
                }
                else {
                  if ( isdigit($read_io) && isdigit($resp_t_r) ) {
                    $sum_resp_t_r = $sum_resp_t_r + ( $read_io * $resp_t_r );
                  }
                }
                if ( !isdigit($sum_resp_t_w) ) {
                  if ( isdigit($resp_t_w) && isdigit($write_io) ) {
                    $sum_resp_t_w = $write_io * $resp_t_w;
                  }
                }
                else {
                  if ( isdigit($write_io) && isdigit($resp_t_w) ) {
                    $sum_resp_t_w = $sum_resp_t_w + ( $write_io * $resp_t_w );
                  }
                }

              } ## end if ( defined $pool_mapping...)
            } ## end foreach my $volume_id ( keys...)
            if ( defined($time_range) && !$time_range eq '' && isdigit($time_range) && $time_range < 300 ) {

              # use actual $time_text instead of section one as there might be more intervals, HP3PAR_VV_SAMPLE_RATE is probably set (30 or 60 secs)
              my $time_act = str2time($time_text);
              if ( defined($time_act) && !$time_act eq '' && isdigit($time_act) ) {
                $time_pool = $time_act;
              }
            } ## end if ( defined($time_range...))
            if ( !isdigit($sum_io_rate) ) {
              $sum_io_rate = "U";
              $sum_resp_t  = "U";
            }
            if ( !isdigit($sum_read_io) ) {
              $sum_read_io  = "U";
              $sum_resp_t_r = "U";
            }
            if ( !isdigit($sum_write_io) ) {
              $sum_write_io = "U";
              $sum_resp_t_w = "U";
            }

            if ( !isdigit($sum_data_rate) ) {
              $sum_data_rate = "U";
            }
            if ( !isdigit($sum_read) ) {
              $sum_read = "U";
            }
            if ( !isdigit($sum_write) ) {
              $sum_write = "U";
            }

            if ( isdigit($sum_io_rate) && isdigit($sum_resp_t) ) {
              my $sum = round( $sum_resp_t / $sum_io_rate, 2 );
              $sum_resp_t = $sum;
            }
            if ( isdigit($sum_read_io) && isdigit($sum_resp_t_r) ) {
              my $sum = round( $sum_resp_t_r / $sum_read_io, 2 );
              $sum_resp_t_r = $sum;
            }
            if ( isdigit($sum_write_io) && isdigit($sum_resp_t_w) ) {
              my $sum = round( $sum_resp_t_w / $sum_write_io, 2 );
              $sum_resp_t_w = $sum;
            }
            print "001 POOL $pool_id - $sum_io_rate - $sum_read_io - $sum_write_io - $sum_data_rate - $sum_read - $sum_write - $sum_resp_t - $sum_resp_t_r - $sum_resp_t_w\n" if $DEBUG == 3;

            if ( !isdigit($sum_resp_t_r) ) {
              $sum_resp_t_r = "U";
            }
            if ( !isdigit($sum_resp_t_w) ) {
              $sum_resp_t_w = "U";
            }
            if ( !isdigit($sum_resp_t) ) {
              $sum_resp_t = "U";
            }

            my $rrd = "$wrkdir/$host/POOL/$pool_id.rrd";

            #create rrd db if necessary
            $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
            if ( create_rrd( $rrd, $time_pool, $step, $DEBUG, $host, $no_time, $act_time, "POOL", $st_type ) == 1 ) {
              return 2;        # leave uit if any error during RRDTool file creation to keep data
            }

            my $l;
            my $l_count = 0;
            my $found   = -1;
            my $ltime;
            foreach $l (@pool_name) {
              if ( $l =~ m/^$rrd$/ ) {
                $found = $l_count;
                last;
              }
              $l_count++;
            } ## end foreach $l (@pool_name)
            if ( $found > -1 ) {
              $ltime = $pool_time[$found];
            }
            else {
              # find out last record in the db
              # as this makes it slowly to test it each time then it is done
              # once per a lpar for whole load and saved into the array
              eval {
                RRDp::cmd qq(last "$rrd" );
                $last_rec = RRDp::read;
              };
              if ($@) {
                rrd_error( $@, $rrd );
                next;
              }
              chomp($$last_rec);
              chomp($$last_rec);
              $pool_name[$l_count] = $rrd;
              $pool_time[$l_count] = $$last_rec;
              $ltime               = $$last_rec;

              #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
            } ## end else [ if ( $found > -1 ) ]
            if ( isdigit($time_pool) && isdigit($ltime) && $time_pool > $ltime ) {
              $pool_time[$l_count] = $time_pool;
              $counter_ins++;
              print "004: POOL $pool_id:$sum_read:$sum_write:$sum_read_io:$sum_write_io:$sum_resp_t_r:$sum_resp_t_w:$sum_data_rate:$sum_io_rate:$sum_resp_t:U:U:U\n" if $DEBUG == 3;
              RRDp::cmd qq(update "$rrd" $time_pool:$sum_read:$sum_write:$sum_read_io:$sum_write_io:$sum_resp_t_r:$sum_resp_t_w:$sum_data_rate:$sum_io_rate:$sum_resp_t:U:U:U);
              my $answer = RRDp::read;
              $time_last_ok = $time;
            } ## end if ( isdigit($time_pool...))
            $pool_active = 0;
          } ## end foreach my $pool_id ( keys ...)
        } ## end if ( $pool_active > 1 ...)

      } ## end if ( isdigit($name) ==...)

      if ( isdigit($name) == 0 ) {
        next;
      }

      my $time_vol = $time;
      if ( defined($time_range) && !$time_range eq '' && isdigit($time_range) && $time_range < 300 ) {

        # use actual $time_text instead of section one as there might be more intervals, HP3PAR_VV_SAMPLE_RATE is probably set (30 or 60 secs)
        my $time_act = str2time($time_text);
        if ( defined($time_act) && !$time_act eq '' && isdigit($time_act) ) {
          $time_vol = $time_act;
        }
      } ## end if ( defined($time_range...))

      #Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),
      # Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),,,,,,,,,Volume (Vdisk) Name,CPG ID,CPG Name,,,,,,
      # 0,2016-01-24 04:25:52,300,0,1,1,0,9,9,0.00,6.40,6.40,0.00,0.08,0.08,,,,,,,,,admin,-,---,,,,,,
      # 1,2016-01-24 04:25:52,300,0,5,5,0,40,40,0.00,7.60,7.60,0.00,0.15,0.15,,,,,,,,,.srdata,-,---,,,,,,

      if ( !defined($name) || $name eq '' || !main::ishexa($name) || $name =~ m/\./ || !defined($pool_id) || $pool_id eq '' || !main::ishexa($pool_id) || $pool_id =~ m/\./ || $pool_id =~ m/^-$/ ) {
        if ( $pool_id =~ m/^-$/ ) {
          next;    #there are some storage internal volumes like 2 above used by the storage itself, as there is no pool ID then skipp them
        }
        if ( $error_once == 0 ) {
          main::error( "$host:$type - VVOL ID is not a digit or hexa: $name or pool id is not a digit: $pool_id , reported only once " . __FILE__ . ":" . __LINE__ );
          $error_once++;
        }
        next;
      } ## end if ( !defined($name) ||...)

      print "001 VVOL $name - $read_io - $write_io - $resp_t_r - $resp_t_w - $read - $write\n" if $DEBUG == 3;

      #print "001 VOL $name - $read_io - $write_io - $resp_t_r - $resp_t_w - $read - $write\n";
      $rrd = "$wrkdir/$host/VVOL/$name-P$pool_id.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time_vol, $step, $DEBUG, $host, $no_time, $act_time, "VVOL", $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      my $cache_hit_r = 'U';
      my $cache_hit_w = 'U';
      my $used        = 'U';
      my $real        = 'U';
      my $cap         = 'U';

      if ( !defined($io_rate)     || $io_rate eq '' )     { $io_rate     = 'U'; }
      if ( !defined($read_io)     || $read_io eq '' )     { $read_io     = 'U'; }
      if ( !defined($write_io)    || $write_io eq '' )    { $write_io    = 'U'; }
      if ( !defined($resp_t)      || $resp_t eq '' )      { $resp_t      = 'U'; }
      if ( !defined($resp_t_r)    || $resp_t_r eq '' )    { $resp_t_r    = 'U'; }
      if ( !defined($resp_t_w)    || $resp_t_w eq '' )    { $resp_t_w    = 'U'; }
      if ( !defined($read)        || $read eq '' )        { $read        = 'U'; }
      if ( !defined($write)       || $write eq '' )       { $write       = 'U'; }
      if ( !defined($data_rate)   || $data_rate eq '' )   { $data_rate   = 'U'; }
      if ( !defined($cache_hit_r) || $cache_hit_r eq '' ) { $cache_hit_r = 'U'; }
      if ( !defined($cache_hit_w) || $cache_hit_w eq '' ) { $cache_hit_w = 'U'; }
      if ( !defined($used)        || $used eq '' )        { $used        = 'U'; }
      if ( !defined($real)        || $real eq '' )        { $real        = 'U'; }
      if ( !defined($cap)         || $cap eq '' )         { $cap         = 'U'; }
      if ( !defined($read_size)   || $read_size eq '' )   { $read_size   = 'U'; }
      if ( !defined($write_size)  || $write_size eq '' )  { $write_size  = 'U'; }
      if ( !defined($data_size)   || $data_size eq '' )   { $data_size   = 'U'; }

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time_vol) && isdigit($ltime) && $time_vol > $ltime ) {
        $vol_time[$l_count] = $time_vol;
        $counter_ins++;
        print "004: VVOL $time_vol:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate:$cache_hit_r:$cache_hit_w:$cap:$used:$real\n" if $DEBUG == 3;
        ### pool mapping
        if ( isdigit($pool_id) ) {
          $pool_mapping{$name} = $pool_id;
          $in_pool{$pool_id}   = $pool_id;
        }

        RRDp::cmd qq(update "$rrd" $time_vol:$io_rate:$read_io:$write_io:$resp_t:$resp_t_r:$resp_t_w:$read:$write:$data_rate:$cache_hit_r:$cache_hit_w:$cap:$used:$real);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time_vol...))

      #if ( isdigit($pool_id) == 1) {
      # keep volume fron-end data summ per pool and save it at the end
      #  if ( ! defined $pool_read[$pool_id]      || $pool_read[$pool_id] eq '' )    { $pool_read[$pool_id] = 0; }
      #  if ( ! defined $pool_write[$pool_id]     || $pool_write[$pool_id] eq '' )   { $pool_write[$pool_id] = 0; }
      #  if ( ! defined $pool_data_rate[$pool_id] || $pool_data_rate[$pool_id] eq '' ){ $pool_data_rate[$pool_id] = 0; }
      #  if ( ! defined $pool_io_read[$pool_id]   || $pool_io_read[$pool_id] eq '' ) { $pool_io_read[$pool_id] = 0; }
      #  if ( ! defined $pool_io_write[$pool_id]  || $pool_io_write[$pool_id] eq '' ){ $pool_io_write[$pool_id] = 0; }
      #  if ( ! defined $pool_io_rate[$pool_id]   || $pool_io_rate[$pool_id] eq '' ) { $pool_io_rate[$pool_id] = 0; }
      #  if ( ! defined $pool_resp_r[$pool_id]    || $pool_resp_r[$pool_id] eq '' )  { $pool_resp_r[$pool_id] = 0; }
      #   if ( ! defined $pool_resp_w[$pool_id]    || $pool_resp_w[$pool_id] eq '' )  { $pool_resp_w[$pool_id] = 0; }
      #  if ( ! defined $pool_resp[$pool_id]      || $pool_resp[$pool_id] eq '' )    { $pool_resp[$pool_id] = 0; }
      #  if ( ! defined $pool_read_size[$pool_id] || $pool_read_size[$pool_id] eq '' )    { $pool_read_size[$pool_id] = 0; }
      #  if ( ! defined $pool_write_size[$pool_id]|| $pool_write_size[$pool_id] eq '' )   { $pool_write_size[$pool_id] = 0; }
      #  if ( ! defined $pool_data_size[$pool_id]|| $pool_data_size[$pool_id] eq '' )   { $pool_data_size[$pool_id] = 0; }

      #if ( isdigit($io_rate) == 0  )         { $io_rate = 0  ; }
      #if ( isdigit($read_io) == 0  )         { $read_io = 0  ; }
      #if ( isdigit($write_io) == 0 )         { $write_io = 0  ; }
      #if ( isdigit($resp_t_r) == 0 )         { $resp_t_r = 0  ; }
      #if ( isdigit($resp_t_w) == 0 )         { $resp_t_w = 0  ; }
      #if ( isdigit($read) == 0     )         { $read = 0  ; }
      #if ( isdigit($write) == 0    )         { $write = 0  ; }
      #if ( isdigit($data_rate) == 0 )        { $data_rate = 0  ; }
      #if ( isdigit($read_size) == 0     )    { $read_size = 0  ; }
      #if ( isdigit($write_size) == 0    )    { $write_size = 0  ; }
      #if ( isdigit($data_size) == 0    )    { $data_size = 0  ; }

      #$pool_read[$pool_id] += $read;
      #$pool_write[$pool_id] += $write;
      #$pool_data_rate[$pool_id] += $data_rate;
      #$pool_io_read[$pool_id] += $read_io;
      #$pool_io_write[$pool_id] += $write_io;
      #$pool_io_rate[$pool_id] += $io_rate;
      #$pool_resp_r[$pool_id] += $resp_t_r * $read_io;
      #$pool_resp_w[$pool_id] += $resp_t_w * $write_io;
      #$pool_resp[$pool_id] += $resp_t * $io_rate;
      #$pool_read_size[$pool_id] += $read_size * $read_io;
      #$pool_write_size[$pool_id] += $write_size * $write_io;
      #$pool_data_size[$pool_id] += $data_size * $io_rate;
      #$pool_time = $time;
      #$pool_ids[$pool_id] = $pool_id;
      #}
    } ## end if ( $type =~ m/^VOLUME$/)
    ### POOL SECTION
    if ( $pool_active > 1 && $count_lines == $size_lines ) {
      foreach my $pool_id ( keys %in_pool ) {
        my $sum_io_rate  = "";
        my $sum_read_io  = "";
        my $sum_write_io = "";

        my $sum_data_rate = "";
        my $sum_read      = "";
        my $sum_write     = "";

        my $sum_resp_t   = "";
        my $sum_resp_t_r = "";
        my $sum_resp_t_w = "";
        my $time_range   = "";
        my $time_text    = "";
        my $time_pool    = $time;
        foreach my $volume_id ( keys %inventory_pool ) {
          if ( defined $pool_mapping{$volume_id} && $pool_mapping{$volume_id} eq $pool_id ) {
            $time_range = $inventory_pool{$volume_id}{ITEM}{"RANGE"};
            $time_text  = $inventory_pool{$volume_id}{ITEM}{"TIME"};
            my $metric  = "io_rate";
            my $io_rate = $inventory_pool{$volume_id}{$metric};
            $metric = "read_io";
            my $read_io = $inventory_pool{$volume_id}{$metric};
            $metric = "write_io";
            my $write_io = $inventory_pool{$volume_id}{$metric};
            $metric = "data_rate";
            my $data_rate = $inventory_pool{$volume_id}{$metric};
            $metric = "read";
            my $read = $inventory_pool{$volume_id}{$metric};
            $metric = "write";
            my $write = $inventory_pool{$volume_id}{$metric};
            $metric = "resp_t";
            my $resp_t = $inventory_pool{$volume_id}{$metric};
            $metric = "resp_t_r";
            my $resp_t_r = $inventory_pool{$volume_id}{$metric};
            $metric = "resp_t_w";
            my $resp_t_w = $inventory_pool{$volume_id}{$metric};

            if ( !isdigit($sum_io_rate) ) {
              if ( isdigit($io_rate) ) {
                $sum_io_rate = $io_rate;
              }
            }
            else {
              if ( isdigit($io_rate) ) {
                $sum_io_rate = $sum_io_rate + $io_rate;
              }
            }

            if ( !isdigit($sum_read_io) ) {
              if ( isdigit($read_io) ) {
                $sum_read_io = $read_io;
              }
            }
            else {
              if ( isdigit($read_io) ) {
                $sum_read_io = $sum_read_io + $read_io;
              }
            }
            if ( !isdigit($sum_write_io) ) {
              if ( isdigit($write_io) ) {
                $sum_write_io = $write_io;
              }
            }
            else {
              if ( isdigit($write_io) ) {
                $sum_write_io = $sum_write_io + $write_io;
              }
            }

            if ( !isdigit($sum_data_rate) ) {
              if ( isdigit($data_rate) ) {
                $sum_data_rate = $data_rate;
              }
            }
            else {
              if ( isdigit($data_rate) ) {
                $sum_data_rate = $sum_data_rate + $data_rate;
              }
            }

            if ( !isdigit($sum_read) ) {
              if ( isdigit($read) ) {
                $sum_read = $read;
              }
            }
            else {
              if ( isdigit($read) ) {
                $sum_read = $sum_read + $read;
              }
            }
            if ( !isdigit($sum_write) ) {
              if ( isdigit($write) ) {
                $sum_write = $write;
              }
            }
            else {
              if ( isdigit($write) ) {
                $sum_write = $sum_write + $write;
              }
            }

            if ( !isdigit($sum_resp_t) ) {
              if ( isdigit($resp_t) && isdigit($io_rate) ) {
                $sum_resp_t = $io_rate * $resp_t;
              }
            }
            else {
              if ( isdigit($io_rate) && isdigit($resp_t) ) {
                $sum_resp_t = $sum_resp_t + ( $io_rate * $resp_t );
              }
            }
            if ( !isdigit($sum_resp_t_r) ) {
              if ( isdigit($resp_t_r) && isdigit($read_io) ) {
                $sum_resp_t_r = $read_io * $resp_t_r;
              }
            }
            else {
              if ( isdigit($read_io) && isdigit($resp_t_r) ) {
                $sum_resp_t_r = $sum_resp_t_r + ( $read_io * $resp_t_r );
              }
            }
            if ( !isdigit($sum_resp_t_w) ) {
              if ( isdigit($resp_t_w) && isdigit($write_io) ) {
                $sum_resp_t_w = $write_io * $resp_t_w;
              }
            }
            else {
              if ( isdigit($write_io) && isdigit($resp_t_w) ) {
                $sum_resp_t_w = $sum_resp_t_w + ( $write_io * $resp_t_w );
              }
            }

          } ## end if ( defined $pool_mapping...)
        } ## end foreach my $volume_id ( keys...)
        if ( defined($time_range) && !$time_range eq '' && isdigit($time_range) && $time_range < 300 ) {

          # use actual $time_text instead of section one as there might be more intervals, HP3PAR_VV_SAMPLE_RATE is probably set (30 or 60 secs)
          my $time_act = str2time($time_text);
          if ( defined($time_act) && !$time_act eq '' && isdigit($time_act) ) {
            $time_pool = $time_act;
          }
        } ## end if ( defined($time_range...))
        if ( !isdigit($sum_io_rate) ) {
          $sum_io_rate = "U";
          $sum_resp_t  = "U";
        }
        if ( !isdigit($sum_read_io) ) {
          $sum_read_io  = "U";
          $sum_resp_t_r = "U";
        }
        if ( !isdigit($sum_write_io) ) {
          $sum_write_io = "U";
          $sum_resp_t_w = "U";
        }

        if ( !isdigit($sum_data_rate) ) {
          $sum_data_rate = "U";
        }
        if ( !isdigit($sum_read) ) {
          $sum_read = "U";
        }
        if ( !isdigit($sum_write) ) {
          $sum_write = "U";
        }

        if ( isdigit($sum_io_rate) && isdigit($sum_resp_t) ) {
          my $sum = round( $sum_resp_t / $sum_io_rate, 2 );
          $sum_resp_t = $sum;
        }
        if ( isdigit($sum_read_io) && isdigit($sum_resp_t_r) ) {
          my $sum = round( $sum_resp_t_r / $sum_read_io, 2 );
          $sum_resp_t_r = $sum;
        }
        if ( isdigit($sum_write_io) && isdigit($sum_resp_t_w) ) {
          my $sum = round( $sum_resp_t_w / $sum_write_io, 2 );
          $sum_resp_t_w = $sum;
        }
        print "001 POOL $pool_id - $sum_io_rate - $sum_read_io - $sum_write_io - $sum_data_rate - $sum_read - $sum_write - $sum_resp_t - $sum_resp_t_r - $sum_resp_t_w\n" if $DEBUG == 3;

        if ( !isdigit($sum_resp_t_r) ) {
          $sum_resp_t_r = "U";
        }
        if ( !isdigit($sum_resp_t_w) ) {
          $sum_resp_t_w = "U";
        }
        if ( !isdigit($sum_resp_t) ) {
          $sum_resp_t = "U";
        }

        my $rrd = "$wrkdir/$host/POOL/$pool_id.rrd";

        #create rrd db if necessary
        $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
        if ( create_rrd( $rrd, $time_pool, $step, $DEBUG, $host, $no_time, $act_time, "POOL", $st_type ) == 1 ) {
          return 2;        # leave uit if any error during RRDTool file creation to keep data
        }

        my $l;
        my $l_count = 0;
        my $found   = -1;
        my $ltime;
        foreach $l (@pool_name) {
          if ( $l =~ m/^$rrd$/ ) {
            $found = $l_count;
            last;
          }
          $l_count++;
        } ## end foreach $l (@pool_name)
        if ( $found > -1 ) {
          $ltime = $pool_time[$found];
        }
        else {
          # find out last record in the db
          # as this makes it slowly to test it each time then it is done
          # once per a lpar for whole load and saved into the array
          eval {
            RRDp::cmd qq(last "$rrd" );
            $last_rec = RRDp::read;
          };
          if ($@) {
            rrd_error( $@, $rrd );
            next;
          }
          chomp($$last_rec);
          chomp($$last_rec);
          $pool_name[$l_count] = $rrd;
          $pool_time[$l_count] = $$last_rec;
          $ltime               = $$last_rec;

          #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 3;
        } ## end else [ if ( $found > -1 ) ]
        if ( isdigit($time_pool) && isdigit($ltime) && $time_pool > $ltime ) {
          $pool_time[$l_count] = $time_pool;
          $counter_ins++;
          print "004: POOL $pool_id:$sum_read:$sum_write:$sum_read_io:$sum_write_io:$sum_resp_t_r:$sum_resp_t_w:$sum_data_rate:$sum_io_rate:$sum_resp_t:U:U:U\n" if $DEBUG == 3;
          RRDp::cmd qq(update "$rrd" $time_pool:$sum_read:$sum_write:$sum_read_io:$sum_write_io:$sum_resp_t_r:$sum_resp_t_w:$sum_data_rate:$sum_io_rate:$sum_resp_t:U:U:U);
          my $answer = RRDp::read;
          $time_last_ok = $time;
        } ## end if ( isdigit($time_pool...))
        $pool_active = 0;
      } ## end foreach my $pool_id ( keys ...)
    } ## end if ( $pool_active > 1 ...)

    # DISK --> used for backend data per disk type
    ########################

    # drive data & sum per tier
    if ( $type =~ m/^DRIVE$/ ) {
      print "100 drive: $line\n" if $DEBUG == 3;

      # Rank statistics

      ( my $id, my $time_text, my $time_range, my $read_io,   my $write_io,  my $io_rate, my $read, my $write, my $data_rate, my $read_size, my $write_size, my $data_size, my $resp_t_r, my $resp_t_w, my $resp_t, undef, undef, undef, undef, undef,
        undef,  undef,         undef,          my $cage_name, my $disk_type, my $rpm
      ) = split( /,/, $line );

      if ( $id eq '' ) {
        next;
      }
      if ( $cage_name eq '' ) {
        $cage_name = $id;
      }
      $cage_name =~ s/:/-/g;

      # Physical Disk ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),Read Transfer Size (kB),Write Transfer Size (kB),
      #   Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),,,,,,,,,Cage Position,Type,RPM,,,,,,
      # 0,2016-04-16 11:20:19,300,0,1,1,0,8,8,0.00,7.30,7.30,0.00,2.76,2.76,,,,,,,,,0:0:0,FC,10,,,,,,
      # 1,2016-04-16 11:20:19,300,0,0,0,0,13,14,16.40,29.50,29.40,0.47,7.50,7.45,,,,,,,,,0:1:0,FC,10,,,,,,
      # 6,2016-04-16 11:20:19,300,0,0,0,0,0,0,0.00,0.00,0.00,0.00,0.00,0.00,,,,,,,,,0:6:0,SSD,150,,,,,,

      my $pool_id = 5;    # unknown
      if ( $disk_type =~ m/^FC/ )  { $pool_id = 1; }
      if ( $disk_type =~ m/^SSD/ ) { $pool_id = 2; }
      if ( $disk_type =~ m/^NL/ )  { $pool_id = 3; }
      if ( $disk_type =~ m/^SAS/ ) { $pool_id = 4; }

      # use pool rrd files
      $rrd = "$wrkdir/$host/$type/$cage_name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;     # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;         # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@mdisk_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@mdisk_name)
      if ( $found > -1 ) {
        $ltime = $mdisk_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $mdisk_name[$l_count] = $rrd;
        $mdisk_time[$l_count] = $$last_rec;
        $ltime                = $$last_rec;
        print "005: $ltime - $l_count\n" if $DEBUG == 3;
      } ## end else [ if ( $found > -1 ) ]

      if ( !defined($io_rate)    || $io_rate eq '' )    { $io_rate    = 'U'; }
      if ( !defined($read_io)    || $read_io eq '' )    { $read_io    = 'U'; }
      if ( !defined($write_io)   || $write_io eq '' )   { $write_io   = 'U'; }
      if ( !defined($resp_t)     || $resp_t eq '' )     { $resp_t     = 'U'; }
      if ( !defined($resp_t_r)   || $resp_t_r eq '' )   { $resp_t_r   = 'U'; }
      if ( !defined($resp_t_w)   || $resp_t_w eq '' )   { $resp_t_w   = 'U'; }
      if ( !defined($read)       || $read eq '' )       { $read       = 'U'; }
      if ( !defined($write)      || $write eq '' )      { $write      = 'U'; }
      if ( !defined($data_rate)  || $data_rate eq '' )  { $data_rate  = 'U'; }
      if ( !defined($read_size)  || $read_size eq '' )  { $read_size  = 'U'; }
      if ( !defined($write_size) || $write_size eq '' ) { $write_size = 'U'; }
      if ( !defined($data_size)  || $data_size eq '' )  { $data_size  = 'U'; }

      # Update only latest data --> NO NO, it must be summed here, all logical volumes, not only the first data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $mdisk_time[$l_count] = $time;
        $counter_ins++;
        print "104: DRIVE $time:$read_io:$write_io:$read:$write:$resp_t_r,$resp_t_w:$io_rate:$data_rate\n" if $DEBUG == 3;

        RRDp::cmd qq(update "$rrd" $time:$io_rate:$read_io:$write_io:$data_rate:$read:$write:$resp_t_r:$resp_t_w:$resp_t:$pool_id);
        my $answer = RRDp::read;
      } ## end if ( isdigit($time) &&...)

      if ( !defined $pool_read[$pool_id]       || $pool_read[$pool_id] eq '' )       { $pool_read[$pool_id]       = 0; }
      if ( !defined $pool_write[$pool_id]      || $pool_write[$pool_id] eq '' )      { $pool_write[$pool_id]      = 0; }
      if ( !defined $pool_data_rate[$pool_id]  || $pool_data_rate[$pool_id] eq '' )  { $pool_data_rate[$pool_id]  = 0; }
      if ( !defined $pool_io_read[$pool_id]    || $pool_io_read[$pool_id] eq '' )    { $pool_io_read[$pool_id]    = 0; }
      if ( !defined $pool_io_write[$pool_id]   || $pool_io_write[$pool_id] eq '' )   { $pool_io_write[$pool_id]   = 0; }
      if ( !defined $pool_io_rate[$pool_id]    || $pool_io_rate[$pool_id] eq '' )    { $pool_io_rate[$pool_id]    = 0; }
      if ( !defined $pool_resp_r[$pool_id]     || $pool_resp_r[$pool_id] eq '' )     { $pool_resp_r[$pool_id]     = 0; }
      if ( !defined $pool_resp_w[$pool_id]     || $pool_resp_w[$pool_id] eq '' )     { $pool_resp_w[$pool_id]     = 0; }
      if ( !defined $pool_resp[$pool_id]       || $pool_resp[$pool_id] eq '' )       { $pool_resp[$pool_id]       = 0; }
      if ( !defined $pool_read_size[$pool_id]  || $pool_read_size[$pool_id] eq '' )  { $pool_read_size[$pool_id]  = 0; }
      if ( !defined $pool_write_size[$pool_id] || $pool_write_size[$pool_id] eq '' ) { $pool_write_size[$pool_id] = 0; }
      if ( !defined $pool_data_size[$pool_id]  || $pool_data_size[$pool_id] eq '' )  { $pool_data_size[$pool_id]  = 0; }

      if ( isdigit($io_rate) == 0 )    { $io_rate    = 0; }
      if ( isdigit($read_io) == 0 )    { $read_io    = 0; }
      if ( isdigit($write_io) == 0 )   { $write_io   = 0; }
      if ( isdigit($resp_t_r) == 0 )   { $resp_t_r   = 0; }
      if ( isdigit($resp_t_w) == 0 )   { $resp_t_w   = 0; }
      if ( isdigit($read) == 0 )       { $read       = 0; }
      if ( isdigit($write) == 0 )      { $write      = 0; }
      if ( isdigit($data_rate) == 0 )  { $data_rate  = 0; }
      if ( isdigit($read_size) == 0 )  { $read_size  = 0; }
      if ( isdigit($write_size) == 0 ) { $write_size = 0; }
      if ( isdigit($data_size) == 0 )  { $data_size  = 0; }

      $pool_read[$pool_id]       += $read;
      $pool_write[$pool_id]      += $write;
      $pool_data_rate[$pool_id]  += $data_rate;
      $pool_io_read[$pool_id]    += $read_io;
      $pool_io_write[$pool_id]   += $write_io;
      $pool_io_rate[$pool_id]    += $io_rate;
      $pool_resp_r[$pool_id]     += $resp_t_r * $read_io;
      $pool_resp_w[$pool_id]     += $resp_t_w * $write_io;
      $pool_resp[$pool_id]       += $resp_t * $io_rate;
      $pool_read_size[$pool_id]  += $read_size * $read_io;
      $pool_write_size[$pool_id] += $write_size * $write_io;
      $pool_data_size[$pool_id]  += $data_size * $io_rate;
      $pool_time = $time;
      $pool_ids[$pool_id] = $pool_id;

      #print "000 $pool_id $pool_read[$pool_id] - $pool_write[$pool_id] - $pool_data_rate[$pool_id] - $pool_io_read[$pool_id] - $pool_resp_r[$pool_id] - $pool_resp_w[$pool_id] - $pool_resp[$pool_id] - $pool_read_size[$pool_id] - $pool_write_size[$pool_id] - $pool_data_size[$pool_id]\n";
    } ## end if ( $type =~ m/^DRIVE$/)

    # CPU-Core
    ########################

    if ( $type =~ m/^CPU-CORE$/ ) {
      print "000 CPU-Core: $line\n" if $DEBUG == 3;

      # Port statistics
      ( my $name_tmp, my $t1, my $t2, my $node, my $id, my $user, my $sys, my $idle ) = split( /,/, $line );

      if ( isdigit($name_tmp) == 0 ) {
        next;
      }

      my $name = "node$name_tmp";
      $name =~ s/-/_core/;

      my $compress = 'U';    # just for future usage

      # Node-CPU Core,Time,Interval (s),Node,CPU Core,CPU Utilization - User,CPU Utilization - System,CPU Utilization - Idle,^M
      # 0-0,2016-01-24 04:25:31,300,0,0,0,0,100,^M
      # 0-1,2016-01-24 04:25:31,300,0,1,0,2,98,^M

      print "001 CPU $name - $sys - $compress\n" if $DEBUG == 3;
      $rrd = "$wrkdir/$host/$type/$name\.rrd";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@cpuc_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@cpuc_name)
      if ( $found > -1 ) {
        $ltime = $cpuc_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        chomp($$last_rec);
        $cpuc_name[$l_count] = $rrd;
        $cpuc_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]

      if ( !defined($user) || $user eq '' ) { $user = 'U'; }
      if ( !defined($sys)  || $sys eq '' )  { $sys  = 'U'; }
      if ( !defined($idle) || $idle eq '' ) { $idle = 'U'; }

      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $cpuc_time[$l_count] = $time;
        $counter_ins++;
        RRDp::cmd qq(update "$rrd" $time:$user:$sys:$idle:$compress);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/^CPU-CORE$/)

  }    # foreach

  # insert summed pool data based on volume front-end data, it must be even at the enf of that foreach as volumes normally ends the perf file
  if ( $pool_time > $pool_ltime ) {
    my @none = "";
    pool_sum_insert( $type, $wrkdir, $step, $DEBUG, $host, $no_time, $act_time, $st_type, $pool_time, $pool_ltime, \@pool_read, \@pool_write, \@pool_io_read, \@pool_io_write, \@pool_resp_r, \@pool_resp_w, \@pool_ids, \@pool_data_rate, \@pool_io_rate, \@pool_resp, \@pool_read_size, \@pool_write_size, \@pool_data_size );
  }

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }

  # write down timestamp of last record , not for daily avg from HMC
  open( FHLT, "> $wrkdir/$host/last_rec" ) || main::error( "Can't open $wrkdir/$host/last_rec : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLT "$time";
  close(FHLT);

  close(FH);
  return ( "0", "$time_last_ok" );
} ## end sub load_data_3par_perf

sub delete_storage_data {
  my $ret     = shift;
  my $in_file = shift;
  my $host    = shift;
  my $DEBUG   = shift;
  my $wrkdir  = shift;

  if ( defined($KEEP_OUT_FILES) && !$KEEP_OUT_FILES eq '' ) {
    if ( isdigit($KEEP_OUT_FILES) && $KEEP_OUT_FILES == 1 ) {
      return 1;
    }
    else {
      if ( $KEEP_OUT_FILES =~ m/^$host$/ ) {
        return 1;
      }
      else {
        $KEEP_OUT_FILES       = 0;    # delete data out files as default
        $KEEP_OUT_FILES_HOURS = 0;
      }
    } ## end else [ if ( isdigit($KEEP_OUT_FILES...))]
  } ## end if ( defined($KEEP_OUT_FILES...))
  else {
    $KEEP_OUT_FILES       = 0;        # delete data out files as default
    $KEEP_OUT_FILES_HOURS = 0;
  }

  if ( $ret == 0 ) {
    if ( !defined($KEEP_OUT_FILES) || $KEEP_OUT_FILES == 0 ) {
      if ( $in_file =~ m/perf_/ || $in_file =~ m/perf-/ || $in_file =~ m/\.perf\./ ) {
        if ( -f "$wrkdir/../tmp/$host-perf-last.txt" ) {

          # keep one more copy to be able import data and see graphs (2 data points)
          move( "$wrkdir/../tmp/$host-perf-last.txt", "$wrkdir/../tmp/$host-perf-last1.txt" );
        }
        move( $in_file, "$wrkdir/../tmp/$host-perf-last.txt" );
      } ## end if ( $in_file =~ m/perf_/...)
      if ( $in_file =~ m/conf_/ || $in_file =~ m/conf-/ || $in_file =~ m/\.conf\./ ) {
        move( $in_file, "$wrkdir/../tmp/$host-conf-last.txt" );
      }
      if ( $in_file =~ m/\.ds8capreport\./ ) {
        move( $in_file, "$wrkdir/../tmp/$host-ds8capreport.txt" );
      }
      if ( -f $in_file ) {
        unlink($in_file);    # just to be sure in case "move" above fails
      }
      print "remove used    : $host:$in_file : $KEEP_OUT_FILES : $KEEP_OUT_FILES_HOURS\n" if $DEBUG;
      return 1;
    } ## end if ( !defined($KEEP_OUT_FILES...))
    if ( defined($KEEP_OUT_FILES_HOURS) && isdigit($KEEP_OUT_FILES) == 1 ) {

      # keep files for time in hours specified in environment variable KEEP_OUT_FILES_HOURS
      my $time      = ( stat("$in_file") )[9];
      my $time_save = time() - $KEEP_OUT_FILES_HOURS;
      if ( $time < $time_save ) {
        if ( $in_file =~ m/perf_/ || $in_file =~ m/perf-/ || $in_file =~ m/\.perf\./ ) {
          if ( -f "$wrkdir/../tmp/$host-perf-last.txt" ) {

            # keep one more copy to be able import data and see graphs (2 data points)
            move( "$wrkdir/../tmp/$host-perf-last.txt", "$wrkdir/../tmp/$host-perf-last1.txt" );
          }
          move( $in_file, "$wrkdir/../tmp/$host-perf-last.txt" );
        } ## end if ( $in_file =~ m/perf_/...)
        if ( $in_file =~ m/conf_/ || $in_file =~ m/conf-/ || $in_file =~ m/\.conf\./ ) {
          move( $in_file, "$wrkdir/../tmp/$host-conf-last.txt" );
        }
        if ( $in_file =~ m/\.ds8capreport\./ ) {
          move( $in_file, "$wrkdir/../tmp/$host-ds8capreport.txt" );
        }
        if ( -f $in_file ) {
          unlink($in_file);    # just to be sure in case "move" above fails
        }
        print "remove used    : $host:$in_file : $KEEP_OUT_FILES : $KEEP_OUT_FILES_HOURS\n" if $DEBUG;
      } ## end if ( $time < $time_save)
      else {
        if ( $in_file =~ m/perf_/ || $in_file =~ m/perf-/ || $in_file =~ m/\.perf\./ ) {
          if ( -f "$wrkdir/../tmp/$host-perf-last.txt" ) {

            # keep one more copy to be able import data and see graphs (2 data points)
            move( "$wrkdir/../tmp/$host-perf-last.txt", "$wrkdir/../tmp/$host-perf-last1.txt" );
          }
          copy( $in_file, "$wrkdir/../tmp/$host-perf-last.txt" );
        } ## end if ( $in_file =~ m/perf_/...)
        if ( $in_file =~ m/conf_/ || $in_file =~ m/conf-/ || $in_file =~ m/\.conf\./ ) {
          copy( $in_file, "$wrkdir/../tmp/$host-conf-last.txt" );
        }
        if ( $in_file =~ m/\.ds8capreport\./ ) {
          copy( $in_file, "$wrkdir/../tmp/$host-ds8capreport.txt" );
        }
        print "remove used    : $host:$in_file - not remove yet as $KEEP_OUT_FILES_HOURS (sec)\n" if $DEBUG;
      } ## end else [ if ( $time < $time_save)]
    } ## end if ( defined($KEEP_OUT_FILES_HOURS...))
  } ## end if ( $ret == 0 )
  return 1;
} ## end sub delete_storage_data

###################################################
#  Custom DS8K capacity reporting
###################################################

sub load_data_ds8k_all_cap {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $st_type ) = @_;

  if ( $DO_NOT_SAVE_DATA == 1 ) {
    print "skipping RRD up: $host:$st_type DO_NOT_SAVE_DATA==$DO_NOT_SAVE_DATA \n" if $DEBUG;
    return 1;    # just get storage data for off site processing
  }

  print "updating RRD   : $host:$st_type\n" if $DEBUG;

  opendir( DIR, "$wrkdir/$host" ) || main::error( "directory does not exists : $wrkdir/$host " . __FILE__ . ":" . __LINE__ ) && return 0;
  my $perf_string = "ds8capreport";

  my @files_unsorted = grep( /$host\.$perf_string\.20/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  foreach my $file (@files) {
    chomp($file);
    my $in_file = "$wrkdir/$host/$file";

    my $ret = load_data_ds8k_cap( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type );

    delete_storage_data( $ret, $in_file, $host, $DEBUG, $wrkdir );

    if ( $ret == 2 ) {
      last;    # RRD create issue, skip everything until it is resolved (usually caused by full filesystem)
    }
  } ## end foreach my $file (@files)
  return 0;
} ## end sub load_data_ds8k_all_cap

sub load_data_ds8k_cap {
  my ( $host, $wrkdir, $webdir, $act_time, $step, $DEBUG, $no_time, $in_file, $st_type ) = @_;
  my $counter_ins  = 0;
  my $last_rec     = "";
  my $rrd          = "";
  my $time         = 3600;
  my $time_last_ok = "";
  my $act_time_u   = time();
  my $ltime        = localtime($act_time_u);
  my $line         = "";
  my @vol_name     = "";
  my @vol_time     = "";
  my @pool_name    = "";
  my @pool_time    = "";
  my $type         = "NA";

  my $vol_tot_cap      = 0;
  my $vol_tot_cap_phys = 0;
  my $vol_num          = 0;
  my $vol_ext_typ      = 0;
  my $pool_ext_typ     = 0;
  my $vol_num_pool     = 0;
  my $cap_phys_tot     = 0;
  my $free_2_30_tot    = 0;
  my $free_tot         = 0;

  open( FH, "< $in_file" ) || main::error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  print "updating RRD   : $host:$in_file\n" if $DEBUG;

  foreach $line (@lines) {

    #print "$line";
    chomp($line);

    if ( $line =~ m/Level Configuration/ ) {
      ( my $type_temp, my $trash ) = split( / /, $line );
      $type = uc($type_temp);
      next;
    }

    if ( $line =~ m/Interval End:/ ) {
      $line =~ s/	//g;
      $line =~ s/Interval End:     //g;
      $line =~ s/GMT.*//g;
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $time = str2time($line);
      if ( isdigit($time) == 0 ) {
        $type = "NA";    # clear out type when empty line
        main::error( "no corret time format: $line : $time " . __FILE__ . ":" . __LINE__ );
      }
      print "003 $line - $time\n" if $DEBUG == 2;
      next;
    } ## end if ( $line =~ m/Interval End:/)

    if ( $line =~ m/^$/ ) {
      $type = "NA";      # clear out type when empty line
      next;
    }

    if ( $line =~ m/^Pool Name,Pool ID/ || $line =~ m/^Volume ID/ || $line =~ m/^----------------/ ) {

      # avoid other trash, allow only data lines here to go further
      next;
    }
    $line =~ s/^0x//;

    # POOL
    ########################

    if ( $type =~ m/POOL/ ) {
      my $type_cap = "POOLcapcust";
      print "000 $line\n" if $DEBUG == 2;
      $line =~ s/cyl,/,/g; # workaround for some error, "cyl" apeared in one item

      my ( $pool_name, $pool_id, $stgtype, $cap_phys, $ext_phys, $ext_size, $ext_num, $ext_used, $free_2_30, $free, $num_vols, $num_ranks, $num_tiers ) = split( /,/, $line );

      # Pool Name,Pool ID,stgtype,physical cap,ext physical cap,extsize,exts,usedexts,availstor_2_30B,available,numvols,numranks,numtiers,
      # extpool0,P0,fb,28800,1.51387720773759,1GiB,19024,14684,4340,4340,65,8,1,

      $ext_size =~ s/T.*//;
      $ext_size =~ s/G.*//;
      $ext_size =~ s/M.*//;
      $ext_size =~ s/B.*//;
      $pool_id =~ s/^P//;

      $rrd = "$wrkdir/$host/$type/$pool_id\.rrz";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type_cap, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@pool_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@pool_name)
      if ( $found > -1 ) {
        $ltime = $pool_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $pool_name[$l_count] = $rrd;
        $pool_time[$l_count] = $$last_rec;
        $ltime               = $$last_rec;
      } ## end else [ if ( $found > -1 ) ]
      if ( isdigit($cap_phys) == 1 ) {
        $cap_phys_tot = sprintf( "%.2f", $cap_phys_tot + $cap_phys );
      }
      if ( isdigit($free_2_30) == 1 ) {
        $free_2_30_tot = sprintf( "%.2f", $free_2_30_tot + $free_2_30 );
      }
      if ( isdigit($free) == 1 ) {
        $free_tot = sprintf( "%.2f", $free_tot + $free );
      }
      if ( isdigit($num_vols) == 1 ) {
        $vol_num_pool = sprintf( "%.2f", $vol_num_pool + $num_vols );
      }
      if ( isdigit($ext_used) == 1 ) {
        $pool_ext_typ = sprintf( "%.2f", $pool_ext_typ + $ext_used );
      }

      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $pool_time[$l_count] = $time;
        $counter_ins++;
        print "004: $time:$cap_phys:$ext_phys:$ext_size:$ext_num:$ext_used:$free_2_30:$free \n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$cap_phys:$ext_phys:$ext_size:$ext_num:$ext_used:$free_2_30:$free);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/POOL/ )

    # Volume
    ########################

    if ( $type =~ m/VOLUME/ ) {
      print "000 $line\n" if $DEBUG == 2;
      my $type_cap = "VOLUMEcapcust";

      my ( $id, $serial, $name, $lss_no, $volt_no, $cap, $vol_type, $host_map, $pool_id, $cap_phys, $vol_ext ) = split( /,/, $line );
      my $cap_real = 'U';    # for future usage
      my $ext_size = 'U';    # for future usage
      $vol_num++;

      # Volume ID,Volume SN,Volume Nickname,LSS Number,Volume Number,Volume Size,Volume Type,Host Maps,Pool ID,Volume Physical Cap,
      # 0x0000,IBM.2107-75ANM81/0000,IPA-INPhbeat-KP1,0x00,0x00,1.00,FB 512,IBM.2107-75ANM81/V4:IBM.2107-75ANM81/V5,IBM.2107-75ANM81/P0,1.51,

      if ( $id eq '' ) {
        next;
      }
      $rrd = "$wrkdir/$host/$type/$id\.rrz";

      #create rrd db if necessary
      $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
      if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type_cap, $st_type ) == 1 ) {
        return 2;        # leave uit if any error during RRDTool file creation to keep data
      }

      my $l;
      my $l_count = 0;
      my $found   = -1;
      my $ltime;
      foreach $l (@vol_name) {
        if ( $l =~ m/^$rrd$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      } ## end foreach $l (@vol_name)
      if ( $found > -1 ) {
        $ltime = $vol_time[$found];
      }
      else {
        # find out last record in the db
        # as this makes it slowly to test it each time then it is done
        # once per a lpar for whole load and saved into the array
        eval {
          RRDp::cmd qq(last "$rrd" );
          $last_rec = RRDp::read;
        };
        if ($@) {
          rrd_error( $@, $rrd );
          next;
        }
        chomp($$last_rec);
        $vol_name[$l_count] = $rrd;
        $vol_time[$l_count] = $$last_rec;
        $ltime              = $$last_rec;

        #print "005: $name: $ltime - $l_count : $rrd : $port_name[$l_count] - $port_time[$l_count]\n" if $DEBUG == 2;
      } ## end else [ if ( $found > -1 ) ]
      if ( $cap eq '' )      { $cap      = 'U'; }
      if ( $cap_phys eq '' ) { $cap_phys = 'U'; }
      if ( $vol_ext eq '' )  { $vol_ext  = 'U'; }
      if ( $ext_size eq '' ) { $ext_size = 'U'; }
      if ( $cap_real eq '' ) { $cap_real = 'U'; }

      if ( isdigit($vol_tot_cap) == 1 ) {
        $vol_tot_cap = sprintf( "%.2f", $vol_tot_cap + $cap );
      }
      if ( isdigit($vol_tot_cap_phys) == 1 ) {
        $vol_tot_cap_phys = sprintf( "%.2f", $vol_tot_cap_phys + $cap_phys );
      }
      if ( isdigit($vol_ext_typ) == 1 ) {
        $vol_ext_typ = sprintf( "%.2f", $vol_ext_typ + $vol_ext );
      }

      #print "001 $id : $cap - $cap_phys : $vol_tot_cap - $vol_tot_cap_phys\n";
      #print "last : $ltime $$last_rec  actuall: $t\n";
      # Update only latest data
      if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {
        $vol_time[$l_count] = $time;
        $counter_ins++;
        print "004: $time:$cap:$cap_phys:$cap_real,$vol_ext,$ext_size\n" if $DEBUG == 3;
        RRDp::cmd qq(update "$rrd" $time:$cap:$cap_phys:$cap_real:$vol_ext:$ext_size);
        my $answer = RRDp::read;
        $time_last_ok = $time;
      } ## end if ( isdigit($time) &&...)
    } ## end if ( $type =~ m/VOLUME/)
  }    # foreach

  if ($counter_ins) {
    print "inserted       : $host $counter_ins record(s)\n" if $DEBUG;
  }
  print "Volumes totals : num vols: $vol_num -4, alocated: $vol_tot_cap , physical: $vol_tot_cap_phys ext:$vol_ext_typ\n";
  print "Pool    totals : num vols: $vol_num_pool ,cap_phys:$cap_phys_tot , free_2_30:$free_2_30_tot free:$free_tot ,ext used: $pool_ext_typ  \n";
  return 0;
} ## end sub load_data_ds8k_cap

sub load_pool_capacity {
  my ( $no_time, $step, $st_type, $wrkdir, $host, $type, $time, $DEBUG, $act_time, $pool_id_all_tmp, $pool_capacity_tot_tmp, $pool_capacity_free_tmp, $pool_name_tmp ) = @_;
  my @pool_id_all        = @{$pool_id_all_tmp};
  my @pool_name          = @{$pool_name_tmp};
  my @pool_capacity_tot  = @{$pool_capacity_tot_tmp};
  my @pool_capacity_free = @{$pool_capacity_free_tmp};

  foreach my $pool_id (@pool_id_all) {
    if ( !defined($pool_id) || $pool_id eq '' ) {
      next;
    }

    my $pool_name_act = $pool_id;
    if ( $st_type =~ m/3PAR/ && $type =~ m/TIER/ && defined( $pool_name[$pool_id] ) && !$pool_name[$pool_id] eq '' ) {

      # 3PAR TIER does not have ID, just names
      $pool_name_act = $pool_name[$pool_id];
    }

    if ( !defined($pool_name_act) || $pool_name_act eq '' ) {
      next;
    }

    my $rrd = "$wrkdir/$host/$type/$pool_name_act-cap\.rrd";

    #create rrd db if necessary
    $rrd =~ s/"//g;    # " causing a problem during passing of cmds to rrdtool
    if ( create_rrd( $rrd, $time, $step, $DEBUG, $host, $no_time, $act_time, $type . "cap", $st_type ) == 1 ) {
      return 2;        # leave uit if any error during RRDTool file creation to keep data
    }

    # find out last record in the db
    # as this makes it slowly to test it each time then it is done
    # once per a lpar for whole load and saved into the array
    my $last_rec = 0;
    eval {
      RRDp::cmd qq(last "$rrd" );
      $last_rec = RRDp::read;
    };
    if ($@) {
      rrd_error( $@, $rrd );
      next;
    }
    chomp($$last_rec);
    my $ltime = $$last_rec;

    my $used = 'U';
    if ( !defined( $pool_capacity_tot[$pool_id] )  || !isdigit( $pool_capacity_tot[$pool_id] ) )  { $pool_capacity_tot[$pool_id]  = 'U'; }
    if ( !defined( $pool_capacity_free[$pool_id] ) || !isdigit( $pool_capacity_free[$pool_id] ) ) { $pool_capacity_free[$pool_id] = 'U'; }
    if ( isdigit( $pool_capacity_tot[$pool_id] ) && isdigit( $pool_capacity_free[$pool_id] ) ) {
      $used = $pool_capacity_tot[$pool_id] - $pool_capacity_free[$pool_id];
    }

    if ( isdigit($time) && isdigit($ltime) && $time > $ltime ) {

      # when rrd db has been created then load data immedistally
      RRDp::cmd qq(update "$rrd" $time:$pool_capacity_tot[$pool_id]:$pool_capacity_free[$pool_id]:$used);
      my $answer = RRDp::read;
    } ## end if ( isdigit($time) &&...)
  } ## end foreach my $pool_id (@pool_id_all)

  return 0;
} ## end sub load_pool_capacity
