
use strict;
use Date::Parse;
use RRDp;
use POSIX qw(strftime);
use File::Basename;
use CGI::Carp qw(fatalsToBrowser);

my $DEBUG  = $ENV{DEBUG};
my $errlog = $ENV{ERRLOG};

use lib "../bin";
use ACL;    # use module ACL.pm
my $useacl = ACL::useACL;

#$DEBUG=2;
open( OUT, ">> $errlog" ) if $DEBUG == 2;

# get QUERY_STRING
use Env qw(QUERY_STRING);
$QUERY_STRING .= ":.";

#print STDERR "-- $QUERY_STRING\n";
my $STEP    = $ENV{SAMPLE_RATE};
my $pic_col = $ENV{PICTURE_COLOR};
my $basedir = $ENV{INPUTDIR};
my $rrdtool = $ENV{RRDTOOL};
my $bindir  = $ENV{BINDIR};
my $tmp_dir = "$basedir/tmp";
if ( defined $ENV{TMPDIR_STOR} ) {
  $tmp_dir = $ENV{TMPDIR_STOR};
}
my $width                = 400;
my $height               = 150;
my $width_trend          = $width * 2;
my $detail_multipication = 3;                      # detail pop up will be 3 times bigger
my $wrkdir               = "$basedir/data";
my $act_time             = localtime();
my $font_def_normal      = "--font=DEFAULT:7:";
my $font_tit_normal      = "--font=TITLE:9:";
my $font_def_dashbo      = "--font=DEFAULT:6:";
my $font_tit_dashbo      = "--font=TITLE:7:";
my $font_def_popup       = "--font=DEFAULT:10:";
my $font_tit_popup       = "--font=TITLE:13:";
my $font_def_hist        = "--font=DEFAULT:8:";
my $font_tit_hist        = "--font=TITLE:10:";
my $font_def             = $font_def_normal;
my $font_tit             = $font_tit_normal;
my $png_end_heading      = "\n";                   # this or graphv dimensions

my $delimiter = "XORUX";                           # this is for rrdtool print lines for clickable legend
my @table_out = "";

my $rrd_legend = "table";                          #   "table" or "both" or "orig"
if ( -f "$tmp_dir/xorux_test_orig.txt" ) {
  $rrd_legend = "orig";
}
if ( -f "$tmp_dir/xorux_test_both.txt" ) {
  $rrd_legend = "both";
}
if ( -f "$tmp_dir/xorux_test_table.txt" ) {
  $rrd_legend = "table";
}
if ( !defined $ENV{HTTP_USER_AGENT} ) {
  $rrd_legend = "orig";
}
else {
  if ( defined $ENV{DEMO} && ( $ENV{DEMO} == 1 ) ) {
  }
  else {
    if ( $ENV{HTTP_USER_AGENT} =~ /MSIE 8.0/ ) {
      $rrd_legend = "orig";
    }
  }
} ## end else [ if ( !defined $ENV{HTTP_USER_AGENT...})]
my $ds5k_pool_org = 0;
if ( defined $ENV{DS5K_POOL_ORG} && isdigit( $ENV{DS5K_POOL_ORG} ) ) {
  $ds5k_pool_org = $ENV{DS5K_POOL_ORG};
}
my $xiv_pool_org = 0;
if ( defined $ENV{XIV_POOL_ORG} && isdigit( $ENV{XIV_POOL_ORG} ) ) {
  $xiv_pool_org = $ENV{XIV_POOL_ORG};
}

### graph or graphv
my $graph_cmd = "graph";
if ( -f "$tmp_dir/graphv" ) {
  $graph_cmd = "graphv";    # if exists - call this function
}

#print STDERR "query string ,$QUERY_STRING,\n";

( my $host,       my $type,     my $name,       my $item, my $time, my $detail,
  my $start_unix, my $end_unix, my $height_new, my $width_new
) = split( /&/, $QUERY_STRING );

if ( !defined($host) || !defined($type) || !defined($name) ) {
  error( "Not defined host/type/name, exiting: $QUERY_STRING " . __FILE__ . ":" . __LINE__ ) && exit(1);
}
$host =~ s/host=//;
$host =~ s/%([\dA-Fa-f][\dA-Fa-f])/ pack ("C",hex ($1))/eg;
$host =~ s/\+/ /g;
$host =~ s/%23/\#/g;
$type =~ s/type=//;
$item =~ s/item=//;
$name =~ s/name=//;
$name =~ tr/+/ /;
$name =~ s/%([\dA-Fa-f][\dA-Fa-f])/ pack ("C",hex ($1))/eg;
$name =~ s/\+/ /g;
$name =~ s/%23/\#/g;
$time =~ s/time=//;
$detail =~ s/detail=//;

if ( $host eq '' || $type eq '' || $name eq '' ) {
  error( "Not defined host/type/name, exiting: $QUERY_STRING " . __FILE__ . ":" . __LINE__ ) && exit(1);
}

#print STDERR " $QUERY_STRING : $name\n";

if ( $time !~ m/^d$/ && $width_new eq '' ) {
  select( undef, undef, undef, 0.15 );
}

# following is from historical reporting
$height_new =~ s/\+//;
$height_new =~ s/height=//;
$width_new =~ s/\+//;
$width_new =~ s/width=//;

# parameters called only from historical reports, normally they are NULL
# and from zoom
if ( !$start_unix eq '' ) {
  $start_unix =~ s/sunix=//;
}
if ( !$end_unix eq '' ) {
  $end_unix =~ s/eunix=//;
  $end_unix =~ s/:\.,//;    # some garbage could appear there
  $end_unix =~ s/:\.//;
  chomp($end_unix);
} ## end if ( !$end_unix eq '' )
if ( !$height_new eq '' && ($detail == 0 || $detail == 9) ) {
  $height_new =~ s/height=//;
  if ( isdigit($height_new) > 0 ) {
    $height = $height_new;    #replace HEIGHT by passed one
  }
} ## end if ( !$height_new eq ''...)
if ( !$width_new eq '' && ($detail == 0 || $detail == 9) ) {
  $width_new =~ s/width=//;
  if ( isdigit($width_new) > 0 ) {
    $width = $width_new;      #replace WIDTH by passed one
  }
} ## end if ( !$width_new eq ''...)

my $heading_png = "";

# only for CGI-BIN purposes
if ( $item =~ m/xport/ ) {

  # export to CSV, not implemented yet
  # It should be here to do not influence normal report when XML is not in Perl
  require "$bindir/xml.pl";

  # use XML::Simple; --> it has to be in separete file
  print "Content-type: application/octet-stream\n";
} ## end if ( $item =~ m/xport/)
else {
  if ( ( $type =~ m/VOLUME/ || $type =~ m/LUN/ ) && ( $name =~ m/^top$/ || $name =~ m/^topglobal$/ ) ) {

    # Volume Top tablesuff
    #print "Content-type: text/html\n";

    print "Content-type: image/png\n";
    print "Cache-Control: max-age=60, must-revalidate\n\n";    # workaround for caching on Chrome
  } ## end if ( ( $type =~ m/VOLUME/...))
  else {
    if ( $type =~ m/POOL/ && ( $name =~ m/^cap_total$/ || $name =~ m/^cap_pool$/ ) ) {

      # total capacity, just text values
      print "Content-type: text/html\n";
      print "\n";
    } ## end if ( $type =~ m/POOL/ ...)
    elsif ( $type eq "VOLUME" && $name eq "fast_sg" ) {

      # VMAX - FAST SG table (allocated to tier)
      print "Content-type: text/html\n";
      print "\n";
    } ## end elsif ( $type eq "VOLUME"...)
    else {
      # normal picture stuff
      $heading_png = "Content-type: image/png\nCache-Control: max-age=60, must-revalidate\n";

      # print "Content-type: image/png\nCache-Control: max-age=60, must-revalidate\n"; # workaround for caching on Chrome
    } ## end else [ if ( $type =~ m/POOL/ ...)]
  } ## end else [ if ( ( $type =~ m/VOLUME/...))]
} ## end else [ if ( $item =~ m/xport/)]

my $no_legend = "--interlaced";    # just nope string , it is deprecated anyway
if ( isdigit($detail) == 0 ) {
  $detail = 0;
}
if ( $detail == 2 ) {

  # dashboard size
  if ( defined $ENV{DASHB_RRDHEIGHT} ) {
    $height = $ENV{DASHB_RRDHEIGHT};
  }
  else {
    $height = 50;
  }
  if ( defined $ENV{DASHB_RRDWIDTH} ) {
    $width = $ENV{DASHB_RRDWIDTH};
  }
  else {
    $width = 120;
  }
  $font_def  = $font_def_dashbo;
  $font_tit  = $font_tit_dashbo;
  $no_legend = "--no-legend";
} ## end if ( $detail == 2 )
if ( $detail == 8 ) {
  # historical reports size
  $height = 150;
  $width  = 900;
}
my $super_detail = 0;
if ( $detail == 3 ) {
  $super_detail = 3;

  # customer size
  if ( defined $ENV{CUSTOMER_RRDHEIGHT} ) {
    $height = $ENV{CUSTOMER_RRDHEIGHT};
  }
  else {
    $height = 150;
  }
  if ( defined $ENV{CUSTOMER_RRDWIDTH} ) {
    $width = $ENV{CUSTOMER_RRDWIDTH};
  }
  else {
    $width = 550;
  }

  #$font_def =  $font_def_dashbo;
  #$font_tit =  $font_tit_dashbo;
  $no_legend = "--no-legend";
  $detail    = 2;               # same as dashboard
} ## end if ( $detail == 3 )
if ( $detail == 1 ) {
  $font_def = $font_def_popup;
  $font_tit = $font_tit_popup;
}

my $disable_rrdtool_tag = "--interlaced";    #just nope string, it is deprecated anyway
my $rrd_ver             = $RRDp::VERSION;
if ( isdigit($rrd_ver) && $rrd_ver > 1.35 ) {
  $disable_rrdtool_tag = "--disable-rrdtool-tag";
}

if ( !$height_new eq '' ) {
  $height = $height_new;
}
if ( !$width_new eq '' ) {
  $width = $width_new;
}

# keep here green - yellow - red - blue ...
my @color = (
  "#FF0000", "#0000FF", "#FFFF00", "#00FFFF", "#FFA500", "#00FF00", "#808080", "#FF00FF", "#800080",
  "#FDD017", "#0000A0", "#3BB9FF", "#008000", "#800000", "#ADD8E6", "#F778A1", "#800517", "#736F6E", "#F52887",
  "#C11B17", "#5CB3FF", "#A52A2A", "#FF8040", "#2B60DE", "#736AFF", "#1589FF", "#98AFC7", "#8D38C9", "#307D7E",
  "#F6358A", "#151B54", "#6D7B8D", "#FDEEF4", "#FF0080", "#F88017", "#2554C7", "#FFF8C6", "#D4A017", "#306EFF",
  "#151B8D", "#9E7BFF", "#EAC117", "#E0FFFF", "#15317E", "#6C2DC7", "#FBB917", "#FCDFFF", "#15317E", "#254117",
  "#FAAFBE", "#357EC7", "#4AA02C", "#38ACEC", "#C0C0C0"
);
my $color_max = 53;

if ( $detail == 1 ) {
  $width       = $width * $detail_multipication;
  $width_trend = $width_trend * $detail_multipication;
  $height      = $height * $detail_multipication;
}

my @name_list = "";
if ( $detail == 3 ) {    # real time Refresh
  $width       = $width * 2;
  $width_trend = $width_trend * 2;
  $height      = $height * 2;

  #@name_list = split(/;/,$line); # for aggregated sums ... --> it has to be finished
} ## end if ( $detail == 3 )

my $name_out = "/var/tmp/stor2rrd-$$.png";

if ( $useacl ) {    #NMON grapher if always free, the same with allerting
  my $cust_group = "";
  my $acl_obj    = "S";
  if ( $item =~ /^custom/ ) {
    $cust_group = $name;
    $acl_obj    = "C";
  }
  if ( $item =~ /^san_/ ) {
    $acl_obj    = "W";
  }
  if ( !ACL::isAdmin ) {
    if ( !ACL::canShow( $acl_obj, "$cust_group", "$host", "" ) ) {    #  ACL::canShow vrací 0 jestliže nevyhovuje ani jedna z ACL definicí
                                                                              #print STDERR "$cust_group, $server, $lpar, $item, test ACL \n";
      error( "ACL: not allowed for user $ENV{REMOTE_USER} $acl_obj: $cust_group, $host, $item " . __FILE__ . ":" . __LINE__ );
      exit(1);
    }
  } ## end if ( !ACL::isAdmin )
} ## end if ( $useacl && !$vmware...)

# start RRD via a pipe
RRDp::start "$rrdtool";

my $st_type = storage_type( $wrkdir, $host, $item, $name );

if ( $st_type eq "" && $host eq "ALL" ) {
  $st_type = "ALL";
}

if ( $st_type eq '' && $item !~ m/^custom/ && $name !~ m/^topglobal$/ ) {
  error( "Not defined storage type, exiting " . __FILE__ . ":" . __LINE__ ) && exit(1);
}

# mdisk name translation table, must be global one

my $return  = 1;
my $timeout = 1800;    # 30mins timeout for  graohs creation, then alarm --> exit

eval {
  # just to do not hang what migh happen when rrdtool crashes
  local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
  alarm($timeout);

  if ( $st_type eq "SAN-BRCD" || $st_type eq "SAN-CISCO" ) {
    $return = create_graph_san( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $wrkdir, $act_time );

    if ( $return == 1 ) {
      print_png($name_out);
    }

    # close RRD pipe
    RRDp::end;

    close(OUT) if $DEBUG == 2;
    exit(1);
  } ## end if ( $st_type eq "SAN-BRCD"...)

  if ( $item =~ m/^sum$/ || $item =~ m/^custom/ ) {

    # only for big summ graphs --> it uses already created rddtool cmd line in tmp dir
    # print STDERR  "1 $host,$type,$name,$item,$time,$name_out,$detail\n";

    if ( $rrd_legend eq "table" && ($detail < 1 || $detail == 9 || $detail == 8) ) {
      if ( ( $name =~ m/^sum_data$/ )
        || ( $name =~ m/^sum_data_b$/ )
        || ( $name =~ m/^sum_io$/ )
        || ( $name =~ m/^sum_resp/ )
        || ( $name =~ m/^sum_io_b$/ )
        || ( $name =~ m/^sum_io_total$/ )
        || ( $name =~ m/^sum_data_total$/ )
        || ( $name =~ m/^sum_io_trend$/ )
        || ( $name =~ m/^sum_io_b_trend$/ )
        || ( $name =~ m/^sum_io_total_trend$/ )
        || ( $name =~ m/^sum_data_trend$/ )
        || ( $name =~ m/^sum_data_b_trend$/ )
        || ( $name =~ m/^sum_data_total_trend$/ )
        || ( $name =~ m/^custom-group-.*-trend$/ )
        || ( $name =~ m/^sum_capacity$/ )
        || ( $name =~ m/^sum_capacity_aggregated$/ )
        || ( $name =~ m/compress/ )
        || ( $name =~ m/^tier0$/ )
        || ( $name =~ m/^tier0-nl$/ )
        || ( $name =~ m/^tier1$/ )
        || ( $name =~ m/^tier2$/ )
        || ( $name =~ m/^io_rate-subsys$/ )
        || ( $name =~ m/^data_rate-subsys$/ )
        || ( $name =~ m/^read_io-subsys$/ )
        || ( $name =~ m/^write_io-subsys$/ )
        || ( $name =~ m/^read-subsys$/ )
        || ( $name =~ m/^write-subsys$/ )
        || ( $name =~ m/^used$/ )
        || ( $name =~ m/^alloc$/ )
        || ( $name =~ m/^used_trend$/ )
        || ( $name =~ m/^real$/ )
        || ( $name =~ m/^sys$/ )
        || ( $name =~ m/^busy$/ )
        || ( $name =~ m/^sum_busy/ )
        || ( $name =~ m/^WP$/ )
        || ( $name =~ m/^clean_usage$/ )
        || ( $name =~ m/^middle_usage$/ )
        || ( $name =~ m/^phys_usage$/ )
        || ( $name =~ m/^write_pend$/ )
        || ( $name =~ m/^dnlc_hit$/ )
        || ( $name =~ m/^buffer_hit$/ )
        || ( $name =~ m/^cache_hit$/ )
        || ( $name =~ m/^transfer_size$/ ) )
      {
      }    # nothing
      else {
        $no_legend = "--no-legend";
      }

      if ( $st_type eq "VMAX" && $type eq "POOL" && $name !~ m/_b$/ ) {
        $no_legend = "";
      }
      if ( $st_type eq "VMAX" && ( $type eq "PORT" || $type eq "NODE" ) && $name eq "busy" ) {
        $no_legend = "--no-legend";
      }
      if ( $st_type eq "VPLEX" && $type eq "NODE" && $name eq "busy" ) {
        $no_legend = "--no-legend";
      }
      if ( $st_type eq "VMAX" && ( $type eq "VOLUME" || $type eq "SLO" ) && $name eq "cache_hit" ) {
        $no_legend = "--no-legend";
      }
      if ( $st_type eq "VPLEX" && $type eq "NODE" && $name eq "busy" ) {
        $no_legend = "--no-legend";
      }
    } ## end if ( $rrd_legend eq "table"...)
    if ( $type =~ m/ALL/ && $name =~ m/^cap_total$/ && $host =~ /ALL/ ) {
      print "Content-Disposition: attachment;filename=\"Capacity.csv\"\n\n";
      create_csv_total_capacity( $host, $type, $name, $item, $time, $name_out, $wrkdir, $act_time, $STEP );
      exit(0);
    }

    if ( $type =~ m/POOL/ && $name =~ m/^cap_total$/ ) {

      # total capacity, just text values
      create_text_total_capacity( $host, $type, $name, $item, $time, $name_out, $wrkdir, $act_time, $STEP, $st_type );
      exit(0);
    } ## end if ( $type =~ m/POOL/ ...)
    if ( $type =~ m/POOL/ && $name =~ m/^cap_pool$/ ) {

      # Pool capacity, just text values
      create_text_pool_capacity( $host, $type, $name, $item, $time, $name_out, $wrkdir, $act_time, $STEP, $st_type );
      exit(0);
    } ## end if ( $type =~ m/POOL/ ...)
    if ( $st_type eq "VMAX" && $name =~ m/^fast_sg$/ ) {

      # Pool capacity, just text values
      create_text_fast_sg( $host, $type, $name, $item, $time, $name_out, $wrkdir, $act_time, $STEP, $st_type );
      exit(0);
    } ## end if ( $st_type eq "VMAX"...)
    $return = create_graph_summ( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $st_type );
  } ## end if ( $item =~ m/^sum$/...)
  else {
    # normal detail processing, not summed parts
    $no_legend = "--interlaced";
    if ( $type =~ m/VOLUME$/ || $type =~ m/LUN/ || $type =~ m/VVOL/ ) {
      if ( $name =~ m/^top$/ ) {

        # print Top table, no graphs
        $return = volumes_top( $host, $type, $name, $item, $time, $st_type, $start_unix, $end_unix, "loc" );
      }
      else {
        if ( $name =~ m/^topglobal$/ ) {

          # print Top table, no graphs
          $return = volumes_top_global( $host, $type, $name, $item, $time, $st_type, $start_unix, $end_unix, $basedir );
        }
        elsif ( $st_type eq "VMAX" && $item eq "tier_alloc_prct" ) {
          $return = draw_graph_aggregated( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $wrkdir, $act_time, $STEP );
        }
        else {
          $return = create_graph_volume( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $st_type, $STEP );
        }
      } ## end else [ if ( $name =~ m/^top$/)]
    } ## end if ( $type =~ m/VOLUME/...)
    else {
      if ( $type =~ m/^POOL/ && $item !~ m/^sum_capacity$/ ) {
        $return = create_graph_pool( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $wrkdir, $act_time, $STEP, $st_type );
      }
      else {
        if ( $type =~ m/^HOST$/ ) {
          my $host_org = host_data_selection( $wrkdir, $host, $type, $st_type );
          if ( $host_org == 1 || $st_type eq "VMAX" || $st_type eq "3PAR" || $st_type eq "COMPELLENT" || $st_type eq "OCEANSTOR" || $st_type eq "PURE" ) {

            # VSPG probably, it has direct host data in rrd files, but it can be excluded in certain cases (etc/.magic)
            $return = create_graph( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $wrkdir, $act_time );
          }
          else {
            # volume aggregated data per host
            $return = draw_graph_host_volume( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $wrkdir, $act_time, $STEP );
          }
        } ## end if ( $type =~ m/^HOST$/)
        else {
          if ( ( $type =~ m/^TIER$/ && $st_type eq "VMAX" && $item !~ m/^sum_capacity$/ ) || ( $type =~ m/^DRIVE$/ && $st_type eq "VMAX" && $item =~ m/^used$/ ) ) {
            $return = create_graph_pool( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $wrkdir, $act_time, $STEP, $st_type );
          }
          else {
            $return = create_graph( $host, $type, $name, $item, $time, $name_out, $detail, $start_unix, $end_unix, $wrkdir, $act_time );
          }
        } ## end else [ if ( $type =~ m/^HOST$/)]
      } ## end else [ if ( $type =~ m/^POOL/...)]
    } ## end else [ if ( $type =~ m/VOLUME/...)]
  } ## end else [ if ( $item =~ m/^sum$/...)]
  alarm(0);

  if ( $return == 1 ) {
    print_png($name_out);
  }

  # close RRD pipe
  RRDp::end;

  close(OUT) if $DEBUG == 2;
};

if ($@) {
  if ( $@ =~ /died in SIG ALRM/ ) {
    error( "$host:$type:$name timed out after : $timeout " . __FILE__ . ":" . __LINE__ );
    exit(1);
  }
  else {
    error( "$host:$type:$name failed $@ " . __FILE__ . ":" . __LINE__ );
    exit(1);
  }
} ## end if ($@)

exit(0);

sub create_graph_san {
  my $host       = shift;
  my $type       = shift;
  my $name       = shift;
  my $item       = shift;
  my $time       = shift;
  my $name_out   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $wrkdir     = shift;
  my $act_time   = shift;
  my $rrd        = "$wrkdir/$host/$name\.rrd";

  $STEP = 60;

  #print STDERR "\$item ,$item,\n";

  if ( $rrd_legend eq "table" && ($detail < 1 || $detail == 9 || $detail == 8) ) {
    $no_legend = "--no-legend";
  }

  # only port section and switch CPU/Memory
  # SAN SWITCH : san name : port name
  if ( $item eq "san_data" || $item eq "san_io" || $item eq "san_io_size" || $item eq "san_errors" || $item eq "san_credits" || $item eq "san_cpu" || $item eq "san_mem" ) {
    my $name_text   = $name;
    my $type_text   = $type;
    my $value_short = "";
    my $value_long  = "";
    my $val         = 1;

    if ( $item =~ m/^san_data$/ )    { $value_short = "MB/sec";     $value_long = "MBytes per second"; }
    if ( $item =~ m/^san_io$/ )      { $value_short = "frames/sec"; $value_long = "Frames per second"; }
    if ( $item =~ m/^san_io_size$/ ) { $value_short = "bytes";      $value_long = "Bytes per frame"; }
    if ( $item =~ m/^san_credits$/ ) { $value_short = "credits";    $value_long = "missing credits in % of time"; }
    if ( $item =~ m/^san_errors$/ )  { $value_short = "errors";     $value_long = "errors"; }
    if ( $item =~ m/^san_cpu$/ )     { $value_short = "%";          $value_long = "CPU Utilization in %"; }
    if ( $item =~ m/^san_mem$/ )     { $value_short = "%";          $value_long = "Memory Utilization in %"; }

    if ( $item eq "san_cpu" || $item eq "san_mem" ) {
      $rrd = "$wrkdir/$host/switch_stats\.rrs";
    }

    if ( !-f $rrd ) {
      error( "Could not find $type ID for $host:$type:$name in neither in etc/alias.cfg " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    my $name_to_header = $name;
    if ( -f "$wrkdir/$host/SAN-BRCD" && -f "$wrkdir/$host/PHYS_PORTS.cfg" ) {
      open( PHY, "< $wrkdir/$host/PHYS_PORTS.cfg" ) || error( "Can't open : $wrkdir/$host/PHYS_PORTS.cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
      my @phys_cfg = <PHY>;
      close(PHY);

      my $phys_cfg_line = "";
      ($phys_cfg_line) = grep {/^$name : /} @phys_cfg;
      if ( defined $phys_cfg_line && $phys_cfg_line ne '' ) {
        my ( undef, $new_name ) = split( " : ", $phys_cfg_line );
        chomp $new_name;
        $name_to_header = $new_name;
      }

    } ## end if ( -f "$wrkdir/$host/SAN-BRCD"...)

    my $comment = sprintf( "%-25s", "$item [$value_short]" );
    my $xgrid_all = xgrid_set( $time, $detail );
    my $text      = text_set($time);
    my $last      = " last";
    my $header    = "$type : $name_to_header : $item$last $text";
    if ( $detail == 2 ) {
      $header = "$name_to_header : $item $text";
    }

    my $start_time = "now-1$time";
    my $end_time   = "now-1$time+1$time";
    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # Historical report detected
        $start_time = $start_unix;
        $end_time   = $end_unix;

        #$xgrid ="COMMENT: "; # just a trick to place there something otherwise rrdtool complains
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header    = "$type_text : $name_to_header : $item : $start_human - $end_human";
        $text      = "";
        $xgrid_all = "--alt-y-grid";
        if ( $detail ne "9" && $detail ne "8" ) {
          $font_def  = $font_def_hist;
          $font_tit  = $font_tit_hist;
        }
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)

    my $vertical_label = "--vertical-label=$value_long";
    $no_legend = "--interlaced";    # only for port legend
    if ( $detail == 2 ) {
      $vertical_label = "--interlaced";
      $no_legend      = "--no-legend";
    }

    # new zooming function without rrdtool title in graphs
    if ( $detail eq "9" || $detail eq "8" ) { $header = ""; }

    my $item1 .= "";
    my $item2 .= "";
    my $answ        = "";
    my $sprintf     = "%8.1lf";
    my $value_name1 = "IN               ";
    my $value_name2 = "OUT              ";

    #CRC errors
    if ( $item eq "san_errors" ) {
      $item1 .= "swFCPortRxCrcs";
      $val         = 1;
      $value_name1 = "CRC errors         ";
      $comment     = sprintf( "%-27s", "$item [$value_short]" );

      RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=0.2"
          "--units-exponent=1.00"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:val1=$rrd:$item1:AVERAGE"
          "COMMENT:$comment Avg        Max\\n"
          "LINE1:val1#00FF00:$value_name1"
          "GPRINT:val1:AVERAGE:$sprintf"
          "GPRINT:val1:MAX:$sprintf"
          "COMMENT:\\n"
          "HRULE:0#000000");
      $answ = RRDp::read;
      if ( $$answ =~ "ERROR" ) {
        error( "sum detail: Multi graph rrdtool error : $$answ  " . __FILE__ . ":" . __LINE__ );
        return 0;
      }
      $png_end_heading = ret_graph_param($answ);
    } ## end if ( $item eq "san_errors")

    #PORT data,io
    if ( $item eq "san_data" || $item eq "san_io" ) {
      my $units_exponent = "--units-exponent=1.00";
      if ( $item eq "san_data" ) {
        $item1 .= "bytes_rec";
        $item2 .= "bytes_tra";
        $val = ( 1024 * 1024 ) / 4;    # from bytes to megabytes
      }
      if ( $item eq "san_io" ) {
        $item1 .= "frames_rec";
        $item2 .= "frames_tra";
        $val            = 1;
        $sprintf        = "%8.0lf";
        $units_exponent = "--interlaced";
      } ## end if ( $item eq "san_io")
      RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=0.2"
          "$units_exponent"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:val1=$rrd:$item1:AVERAGE"
          "DEF:val2=$rrd:$item2:AVERAGE"
          "CDEF:valmb1=val1,$val,/"
          "CDEF:valmb2=val2,$val,/"
          "COMMENT:$comment Avg        Max\\n"
          "LINE1:valmb1#00FF00:$value_name1"
          "GPRINT:valmb1:AVERAGE:$sprintf"
          "GPRINT:valmb1:MAX:$sprintf"
          "COMMENT:\\n"
          "LINE1:valmb2#4444FF:$value_name2"
          "GPRINT:valmb2:AVERAGE:$sprintf"
          "GPRINT:valmb2:MAX:$sprintf"
          "COMMENT:\\n"
          "HRULE:0#000000");
      $answ = RRDp::read;
      if ( $$answ =~ "ERROR" ) {
        error( "sum detail: Multi graph rrdtool error : $$answ  " . __FILE__ . ":" . __LINE__ );
        return 0;
      }
      $png_end_heading = ret_graph_param($answ);
    } ## end if ( $item eq "san_data"...)

    # BBCredits, percentage
    if ( $item eq "san_credits" ) {
      $item1 .= "swFCPortNoTxCredits";
      $val         = 0.0000025;
      $value_name1 = "Credits            ";
      $comment     = sprintf( "%-27s", "$item [%]" );

      RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=0.2"
          "--units-exponent=1.00"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:val1=$rrd:$item1:AVERAGE"
          "CDEF:valmb1=val1,$val,*"
          "CDEF:valmb3=valmb1,60,/"
          "CDEF:valmb4=valmb3,100,*"
          "CDEF:valmb5=valmb4,1000,GT,UNKN,valmb4,IF"
          "COMMENT:$comment Avg        Max\\n"
          "LINE1:valmb5#00FF00:$value_name1"
          "GPRINT:valmb5:AVERAGE:$sprintf"
          "GPRINT:valmb5:MAX:$sprintf"
          "COMMENT:\\n"
          "HRULE:0#000000");
      $answ = RRDp::read;
      if ( $$answ =~ "ERROR" ) {
        error( "sum detail: Multi graph rrdtool error : $$answ  " . __FILE__ . ":" . __LINE__ );
        return 0;
      }
      $png_end_heading = ret_graph_param($answ);
    } ## end if ( $item eq "san_credits")

    # Frame size
    if ( $item eq "san_io_size" ) {
      $item1    .= "frames_rec";
      $item2    .= "frames_tra";
      my $item3 .= "bytes_rec";
      my $item4 .= "bytes_tra";
      $val     = 4;
      $sprintf = "%8.0lf";
      RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=0.2"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:val1=$rrd:$item1:AVERAGE"
          "DEF:val2=$rrd:$item2:AVERAGE"
          "DEF:val3=$rrd:$item3:AVERAGE"
          "DEF:val4=$rrd:$item4:AVERAGE"
          "CDEF:valmb3=val3,$val,*"
          "CDEF:valmb4=val4,$val,*"
          "CDEF:valmb1=valmb3,val1,/"
          "CDEF:valmb2=valmb4,val2,/"
          "COMMENT:$comment Avg        Max\\n"
          "LINE1:valmb1#00FF00:$value_name1"
          "GPRINT:valmb1:AVERAGE:$sprintf"
          "GPRINT:valmb1:MAX:$sprintf"
          "COMMENT:\\n"
          "LINE1:valmb2#4444FF:$value_name2"
          "GPRINT:valmb2:AVERAGE:$sprintf"
          "GPRINT:valmb2:MAX:$sprintf"
          "COMMENT:\\n"
          "HRULE:0#000000");
      $answ = RRDp::read;

      if ( $$answ =~ "ERROR" ) {
        error( "sum detail: Multi graph rrdtool error : $$answ  " . __FILE__ . ":" . __LINE__ );
        return 0;
      }
      $png_end_heading = ret_graph_param($answ);
    } ## end if ( $item eq "san_io_size")

    # CPU or Memory
    if ( $item eq "san_cpu" || $item eq "san_mem" ) {
      if ( $item eq "san_cpu" ) {
        $item1 .= "cpu";
      }
      if ( $item eq "san_mem" ) {
        $item1 .= "mem";
      }
      $value_name1 = sprintf( "%-17s", "$host" );
      $val         = 1;
      $sprintf     = "%8.0lf";
      RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=10"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:val1=$rrd:$item1:AVERAGE"
          "CDEF:valmb1=val1,$val,*"
          "COMMENT:$comment Avg        Max\\n"
          "LINE1:valmb1#00FF00:$value_name1"
          "GPRINT:valmb1:AVERAGE:$sprintf"
          "GPRINT:valmb1:MAX:$sprintf"
          "COMMENT:\\n"
          "HRULE:0#000000");
      $answ = RRDp::read;

      if ( $$answ =~ "ERROR" ) {
        error( "sum detail: Multi graph rrdtool error : $$answ  " . __FILE__ . ":" . __LINE__ );
        return 0;
      }
      $png_end_heading = ret_graph_param($answ);
    }

    return (1);
  } ## end if ( $item eq "san_data"...)

  # Total graphs for one switch and Totals CPU/Memory (for all switches)
  # SAN SWITCH : san name : DATA or IO
  if ( $item eq "san_data_sum_in" || $item eq "san_data_sum_out" || $item eq "san_io_sum_credits" || $item eq "san_io_sum_in" || $item eq "san_io_sum_out" || $item eq "san_io_sum_crc_errors" || $item eq "san_cpu_tot" || $item eq "san_mem_tot" ) {
    my $t              = "COMMENT: ";
    my $t2             = "COMMENT:\\n";
    my $last           = "COMMENT: ";
    my $act_time       = localtime();
    my $act_time_u     = time();
    my $req_time       = 0;
    my $xgrid          = "";
    my $value_short    = "";
    my $value_long     = "";
    my $val            = 1;
    my $units_exponent = "--units-exponent=1.00";
    my $upper_limit    = "0.1";

    if ( $item eq "san_data_sum_in" || $item eq "san_data_sum_out" ) { $value_short = "MB/sec"; $value_long = "MBytes per second"; $val = ( 1024 * 1024 ) / 4; }
    if ( $item eq "san_io_sum_in" || $item eq "san_io_sum_out" ) { $value_short = "frames/sec"; $value_long = "Frames per second"; $val = 1; $units_exponent = "--interlaced"; }
    if ( $item eq "san_io_sum_credits" ) { $value_short = "credits"; $value_long = "missing credits in % of time"; $val = 1; }
    if ( $item eq "san_io_sum_crc_errors" ) { $value_short = "errors"; $value_long = "errors"; $val = 1; }
    if ( $item eq "san_cpu_tot" ) { $value_short = "%"; $value_long = "CPU Utilization in %"; $val = 1; $upper_limit = "10"; }
    if ( $item eq "san_mem_tot" ) { $value_short = "%"; $value_long = "Memory Utilization in %"; $val = 1; $upper_limit = "10"; }

    #if ( $item eq "san_io_sum_encoding_errors" )                     { $value_short= "errors"; $value_long = "errors"; $val=1;}

    my $name_space = $name;
    $name_space = sprintf( "%-10s", "$name_space" );

    #my $comment = sprintf ("%-25s","$item [$value_short]");
    my $comment = sprintf( "%-46s", "[$value_short]" );
    my $comment_nospace = "[$value_short] ";

    my $xgrid_all = xgrid_set( $time, $detail );
    my $text      = text_set($time);
    my $header    = "$type : $name : Transferred data last $text";
    if ( $detail == 2 ) {
      $header = "$name : $item $text";
    }

    my $start_time = "now-1$time";
    my $end_time   = "now-1$time+1$time";
    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # Historical report detected
        $start_time = $start_unix;
        $end_time   = $end_unix;

        #$xgrid ="COMMENT: "; # just a trick to place there something otherwise rrdtool complains
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header    = "$type : $name : $item : $start_human - $end_human";
        $text      = "";
        $xgrid_all = "--alt-y-grid";
        if ( $detail ne "9" && $detail ne "8" ) {
          $font_def  = $font_def_hist;
          $font_tit  = $font_tit_hist;
        }
        $req_time  = $start_unix;
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)

    my $i   = 0;
    my $j   = 0;
    my $cmd = "";

    my $vertical_label = "--vertical-label=\\\"$value_long\\\"";
    if ( $detail == 2 ) {
      $vertical_label = "--interlaced";
      $no_legend      = "--no-legend";
    }

    # new zooming function without rrdtool title in graphs
    if ( $detail eq "9" || $detail eq "8" ) { $header = ""; }

    $cmd .= "$graph_cmd \\\"$name_out\\\"";
    $cmd .= " --title \\\"$header\\\"";
    $cmd .= " --start \\\"$start_time\\\"";
    $cmd .= " --end \\\"$end_time\\\"";
    $cmd .= " --imgformat PNG";
    $cmd .= " --slope-mode";
    $cmd .= " --width=$width";
    $cmd .= " --height=$height";
    $cmd .= " --step=$STEP";
    $cmd .= " --lower-limit=0.00";
    $cmd .= " --color=BACK#$pic_col";
    $cmd .= " --color=SHADEA#$pic_col";
    $cmd .= " --color=SHADEB#$pic_col";
    $cmd .= " --color=CANVAS#$pic_col";
    $cmd .= " --alt-autoscale-max";
    $cmd .= " --interlaced";
    $cmd .= " --upper-limit=$upper_limit";
    $cmd .= " $vertical_label";
    $cmd .= " $units_exponent";
    $cmd .= " $no_legend";
    $cmd .= " $disable_rrdtool_tag";
    $cmd .= " $xgrid_all";
    $cmd .= " $font_def";
    $cmd .= " $font_tit";
    $cmd .= " COMMENT:\\\"$comment   Avg         Max\\l\\\"";

    #my $gtype="LINE1";
    my $gtype = "AREA";

    my $col_indx               = 0;
    my $line_indx              = 0;
    my $legend_heading         = "Ports $delimiter $comment $delimiter Avg $delimiter Max ";
    my $legend_heading_nospace = "Ports $delimiter $comment_nospace $delimiter Avg $delimiter Max ";

    if ( $item eq "san_cpu_tot" || $item eq "san_mem_tot" ) {
      $legend_heading_nospace = "Switches $delimiter $comment_nospace $delimiter Avg $delimiter Max ";
      $legend_heading_nospace =~ s/%/%%/g;
    }

    my @files;
    my @files_org;
    my @alines;
    my @phys_lines;

    if ( $item eq "san_cpu_tot" || $item eq "san_mem_tot" ) {
      # switches CPU/Memory rrd files
      my @bna_switches = <$wrkdir/*/SAN-BNA>;
      foreach my $line (@bna_switches) {
        chomp $line;
        my $switch_name = $line;
        $switch_name    =~ s/^$wrkdir\///;
        $switch_name    =~ s/\/SAN-BNA$//;

        if ( -f "$wrkdir/$switch_name/switch_stats.rrs" ) {
          push(@files, "$switch_name : $wrkdir/$switch_name/switch_stats.rrs");
        }
        else {
          error( "Totals CPU/Memory : Could not find $wrkdir/$switch_name/switch_stats.rrs file! Switch $switch_name skipped! " . __FILE__ . ":" . __LINE__ ) && next;
        }
      }
    }
    else {
      # read PORTS.cfg
      open( FHR, "< $wrkdir/$host/PORTS.cfg" ) || error( "file does not exists : $wrkdir/$host/PORTS.cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
      @files     = <FHR>;
      @files_org = @files;
      close(FHR);

      # read etc/alias.cfg
      my $afile = "$basedir/etc/alias.cfg";
      if ( -f $afile ) {
        open( FHA, "< $afile" ) || error( "File cannot be opened for read, wrong user rights?? : $afile " . __FILE__ . ":" . __LINE__ ) && return "";
        @alines = <FHA>;
        close(FHA);
      }

      # read data/switch_name/PHYS_PORTS.cfg
      my $phys_ports_cfg = "$wrkdir/$host/PHYS_PORTS.cfg";
      if ( -f $phys_ports_cfg ) {
        open( PHY, "< $phys_ports_cfg" ) || error( "File cannot be opened for read, wrong user rights?? : $phys_ports_cfg " . __FILE__ . ":" . __LINE__ ) && return "";
        @phys_lines = <PHY>;
        close(PHY);
      }
    }

    my $ports_found = 0;
    foreach my $line (@files) {
      chomp($line);
      my ( $port_name, $rrd, $alias_from_switch ) = split( " : ", $line );

      # Show BBCredits only for fc ports
      if ( $item eq "san_io_sum_credits" && $st_type eq "SAN-CISCO" ) {
        my $go_bbc = 0;
        if ( $port_name =~ "^portfc" || $port_name =~ "portChannel" || $name =~ "bay" || $name =~ "ext" ) { $go_bbc = 1; }
        if ( $go_bbc == 0 ) { next; }
      }

      $ports_found++;

      $alias_from_switch =~ s/:/\\:/g;    # BRCD port alias with double colon

      if ( $port_name eq '' || $rrd eq '' ) { next; }
      if ( !-f "$rrd" ) { error( "Could not find $port_name file: $rrd " . __FILE__ . ":" . __LINE__ ); next; }
      if ( $ports_found > 1 ) { $gtype = "STACK"; }
      if ( $item eq "san_io_sum_credits" || $item eq "san_cpu_tot" || $item eq "san_mem_tot" ) { $gtype = "LINE1"; }

      #if ( $item eq "san_data_sum_crc_errors" )      { $gtype="LINE1"; }
      #if ( $item eq "san_data_sum_encoding_errors" ) { $gtype="LINE1"; }

      # avoid old ports which do not exist in the period
      my $rrd_upd_time = ( stat("$rrd") )[9];
      if ( $rrd_upd_time < $req_time ) { next; }

      my $port_space  = "";
      my $port_space2 = "";

      # BRCD physical port name
      my $phys_port_name = "";
      my ($phys_port_line) = grep {/^$port_name :/} @phys_lines;
      if ( defined $phys_port_line && $phys_port_line ne '' ) {
        chomp $phys_port_line;
        my ( undef, $new_name ) = split( " : ", $phys_port_line );
        $phys_port_name = $new_name;
      }

      # use alias from switch
      if ( defined $alias_from_switch && $alias_from_switch ne '' ) {
        my $port_name_without_port = $port_name;
        $port_name_without_port =~ s/^port//;
        if ( defined $phys_port_name && $phys_port_name ne '' ) {
          $port_space  = "$phys_port_name [$alias_from_switch]";
          $port_space2 = "$port_name-XOR-ALIAS-$phys_port_name [$alias_from_switch]";
        }
        else {
          $port_space  = "$port_name [$alias_from_switch]";
          $port_space2 = "$port_name-XOR-ALIAS-$port_name_without_port [$alias_from_switch]";
        }
      } ## end if ( defined $alias_from_switch...)
      else {
        if ( defined $phys_port_name && $phys_port_name ne '' ) {
          $port_space  = $phys_port_name;
          $port_space2 = "$port_name-XOR-ALIAS-$phys_port_name";
        }
        else {
          $port_space  = $port_name;
          $port_space2 = $port_name;
        }
      } ## end else [ if ( defined $alias_from_switch...)]
      $port_space =~ s/^port//;

      # use alias from admin
      my $alias_from_admin_line  = "";
      my $alias_from_admin_line2 = "";
      my $port_name_without_port = $port_name;
      $port_name_without_port =~ s/^port//g;

      if ( defined $phys_port_name && $phys_port_name ne '' ) {
        ($alias_from_admin_line)  = grep {/^SANPORT:$host:port$phys_port_name:/} @alines;
        ($alias_from_admin_line2) = grep {/^SANPORT:$host:$phys_port_name:/} @alines;
      }
      else {
        ($alias_from_admin_line)  = grep {/^SANPORT:$host:$port_name:/} @alines;
        ($alias_from_admin_line2) = grep {/^SANPORT:$host:$port_name_without_port:/} @alines;
      }

      if ( $alias_from_admin_line eq "" && $alias_from_admin_line2 ne "" ) { $alias_from_admin_line = $alias_from_admin_line2; }

      my ( undef, undef, undef, $alias_from_admin ) = split( ":", $alias_from_admin_line );
      if ( defined $alias_from_admin && $alias_from_admin ne '' ) {
        chomp($alias_from_admin);
        $port_space  = "$alias_from_admin";
        $port_space2 = "$port_name-XOR-ALIAS-$alias_from_admin";
      }
      chomp($port_space2);
      $port_space2 =~ s/. $/ /;

      my $item_rrd = "";
      if ( $item eq "san_data_sum_in" )       { $item_rrd = "bytes_rec"; }
      if ( $item eq "san_data_sum_out" )      { $item_rrd = "bytes_tra"; }
      if ( $item eq "san_io_sum_in" )         { $item_rrd = "frames_rec"; }
      if ( $item eq "san_io_sum_out" )        { $item_rrd = "frames_tra"; }
      if ( $item eq "san_io_sum_credits" )    { $item_rrd = "swFCPortNoTxCredits"; }
      if ( $item eq "san_io_sum_crc_errors" ) { $item_rrd = "swFCPortRxCrcs"; }
      if ( $item eq "san_cpu_tot" )           { $item_rrd = "cpu"; $port_space = $port_name; $port_space2 = $port_name; }
      if ( $item eq "san_mem_tot" )           { $item_rrd = "mem"; $port_space = $port_name; $port_space2 = $port_name; }

      # add spaces to volume name to have 15 chars total (for formating graph legend)
      $port_space =~ s/\&\&1/\//g;
      $port_space = sprintf( "%-40s", "$port_space" );

      #if ( $item eq "san_io_sum_encoding_errors" )   { $item_rrd = "swFCPortRxEncOutFrs"; }
      $rrd =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

      # build RRDTool cmd
      if ( $item eq "san_io_sum_credits" ) {
        $val = 0.0000025;
        $cmd .= " DEF:value${i}=\\\"$rrd\\\":$item_rrd:AVERAGE";
        $cmd .= " CDEF:valmb1_${i}=value${i},$val,*";
        $cmd .= " CDEF:valmb2_${i}=valmb1_${i},60,/";
        $cmd .= " CDEF:valmb3_${i}=valmb2_${i},100,*";
        $cmd .= " CDEF:valmb${i}=valmb3_${i},1000,GT,UNKN,valmb3_${i},IF";
        $cmd .= " $gtype:valmb${i}$color[$col_indx]:\\\"$port_space\\\"";
      } ## end if ( $item eq "san_io_sum_credits")
      else {
        $cmd .= " DEF:value${i}=\\\"$rrd\\\":$item_rrd:AVERAGE";
        $cmd .= " CDEF:valmb${i}=value${i},$val,/";
        $cmd .= " $gtype:valmb${i}$color[$col_indx]:\\\"$port_space\\\"";
      }

      if ( $item eq "san_io_sum_in" || $item eq "san_io_sum_out" ) {
        $cmd .= " PRINT:valmb${i}:AVERAGE:\\\"%8.0lf $delimiter $port_space2 $delimiter $legend_heading_nospace $delimiter $color[$col_indx]\\\"";
        $cmd .= " PRINT:valmb${i}:MAX:\\\" %8.0lf $delimiter $rrd $delimiter $st_type\\\"";
        $cmd .= " GPRINT:valmb${i}:AVERAGE:\\\"%8.0lf \\\"";
        $cmd .= " GPRINT:valmb${i}:MAX:\\\" %8.0lf \\\"";
      } ## end if ( $item eq "san_io_sum_in"...)
      if ( $item eq "san_data_sum_in" || $item eq "san_data_sum_out" ) {
        $cmd .= " PRINT:valmb${i}:AVERAGE:\\\"%8.1lf $delimiter $port_space2 $delimiter $legend_heading_nospace $delimiter $color[$col_indx]\\\"";
        $cmd .= " PRINT:valmb${i}:MAX:\\\" %8.1lf $delimiter $rrd $delimiter $st_type\\\"";
        $cmd .= " GPRINT:valmb${i}:AVERAGE:\\\"%8.1lf \\\"";
        $cmd .= " GPRINT:valmb${i}:MAX:\\\" %8.1lf \\\"";
      } ## end if ( $item eq "san_data_sum_in"...)
      if ( $item eq "san_io_sum_crc_errors" ) {
        $cmd .= " PRINT:valmb${i}:AVERAGE:\\\"%8.1lf $delimiter $port_space2 $delimiter $legend_heading_nospace $delimiter $color[$col_indx]\\\"";
        $cmd .= " PRINT:valmb${i}:MAX:\\\" %8.1lf $delimiter $rrd $delimiter $st_type\\\"";
        $cmd .= " GPRINT:valmb${i}:AVERAGE:\\\"%8.1lf \\\"";
        $cmd .= " GPRINT:valmb${i}:MAX:\\\" %8.1lf \\\"";
      } ## end if ( $item eq "san_io_sum_crc_errors")
      if ( $item eq "san_io_sum_credits" ) {
        $cmd .= " PRINT:valmb${i}:AVERAGE:\\\"%8.1lf $delimiter $port_space2 $delimiter $legend_heading_nospace $delimiter $color[$col_indx]\\\"";
        $cmd .= " PRINT:valmb${i}:MAX:\\\" %8.1lf $delimiter $rrd $delimiter $st_type\\\"";
        $cmd .= " GPRINT:valmb${i}:AVERAGE:\\\"%8.1lf \\\"";
        $cmd .= " GPRINT:valmb${i}:MAX:\\\" %8.1lf \\\"";
      } ## end if ( $item eq "san_io_sum_credits")
      if ( $item eq "san_cpu_tot" || $item eq "san_mem_tot" ) {
        $cmd .= " PRINT:valmb${i}:AVERAGE:\\\"%8.1lf $delimiter $port_space2 $delimiter $legend_heading_nospace $delimiter $color[$col_indx]\\\"";
        $cmd .= " PRINT:valmb${i}:MAX:\\\" %8.1lf $delimiter $rrd $delimiter $st_type\\\"";
        $cmd .= " GPRINT:valmb${i}:AVERAGE:\\\"%8.1lf \\\"";
        $cmd .= " GPRINT:valmb${i}:MAX:\\\" %8.1lf \\\"";
      }

      # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $col_indx++;
      if ( $col_indx > $color_max ) {
        $col_indx = 0;
      }
      $i++;
      $cmd .= " COMMENT:\\\"\\l\\\"";
    } ## end foreach my $line (@files)

    if ( $ports_found == 0 ) {
      error( "Could not find ports for $host in $wrkdir/$host/PORTS.cfg " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    $cmd .= " HRULE:0#000000";

    # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
    $cmd =~ s/\\"/"/g;

    RRDp::cmd qq($cmd);
    my $ret = RRDp::read;
    if ( $$ret =~ "ERROR" ) {
      error( "sum detail: Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    $png_end_heading = ret_graph_param($ret);
    return 1;
  } ## end if ( $item eq "san_data_sum_in"...)

  # Total graphs for all switches
  # SAN SWITCH : Totals : DATA or IO or Fabric or ISL
  if ( $item =~ "san_data_sum_tot_in" || $item =~ "san_data_sum_tot_out" || $item =~ "san_io_sum_tot_credits" || $item =~ "san_io_sum_tot_in" || $item =~ "san_io_sum_tot_out" || $item =~ "san_io_sum_tot_crc_errors" || $item =~ "san_fabric_data_in" || $item =~ "san_fabric_frames_in" || $item =~ "san_fabric_credits" || $item =~ "san_fabric_crc_errors" || $item =~ "san_isl_data_in" || $item =~ "san_isl_data_out" || $item =~ "san_isl_frames_in" || $item =~ "san_isl_frames_out" || $item =~ "san_isl_frame_size_in" || $item =~ "san_isl_frame_size_out" || $item =~ "san_isl_credits" || $item =~ "san_isl_crc_errors" || $item eq "san_data_sum_tot_in_trend" || $item eq "san_data_sum_tot_out_trend" || $item eq "san_data_sum_tot_in_bna_trend" || $item eq "san_data_sum_tot_out_bna_trend" ) {

    my $tmp_file = "$tmp_dir/SAN-totals/$item-$time.cmd";

    my $xgrid_all = xgrid_set( $time, $detail );
    my $text      = text_set($time);
    my $header    = "$type : $name : Transferred data last $text";
    if ( $detail == 2 ) {
      $header = "$name : $item $text";
    }

    my $start_time = "now-1$time";
    my $end_time   = "now-1$time+1$time";
    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # Historical report detected
        $start_time = $start_unix;
        $end_time   = $end_unix;

        #$xgrid ="COMMENT: "; # just a trick to place there something otherwise rrdtool complains
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header    = "$type : $name : $item : $start_human - $end_human";
        $text      = "";
        $xgrid_all = "--alt-y-grid";
        $font_def  = $font_def_hist;
        $font_tit  = $font_tit_hist;
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)

    # trends only
    if ( $item eq "san_data_sum_tot_in_trend" || $item eq "san_data_sum_tot_out_trend" || $item eq "san_data_sum_tot_in_bna_trend" || $item eq "san_data_sum_tot_out_bna_trend" ) {
      $end_time  = "now+1$time";
      $no_legend = "";
    }

    # replace graph size and total value
    open( FH, "< $tmp_file" ) || error( "SAN SWITCH totals: $host:$type:$item:$time: Can't open $tmp_file " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @lines = <FH>;
    close(FH);

    my $title;
    if ( $header != "" ) {
      ( undef, $title ) = split( "--", $lines[0] );
      $title =~ s/last .*/$header\"/;
      $lines[0] =~ s/--title.*--start/--$title --start/;
    }

    if ( $detail == 2 ) {
      $no_legend = "--no-legend";
    }

    my $lines_no = 0;
    my $cmd      = "";

    foreach my $line (@lines) {

      if ( $line !~ m/DEF:/ ) {
        next;    # some problem, probably no DEF input files
      }
      $lines_no = 1;

      $line =~ s/graph.* --title/$graph_cmd \"$name_out\" --title/;
      $line =~ s/--start now.*--imgformat/--start \"$start_time\" --end \"$end_time\" --imgformat/;
      $line =~ s/NO_LEGEND/$no_legend/;
      if ( $detail == 1 ) {
        $line =~ s/ --font=DEFAULT:[0-9]:/ /;
        $line =~ s/ --font=TITLE:[0-9]:/ /;
        $line =~ s/ --width=400/ --width=$width $font_def_popup /;
        $line =~ s/ --height=150/ --height=$height $font_tit_popup /;
        $line =~ s/ --x-grid=.* --alt-y-grid/ $xgrid_all --alt-y-grid/;
        $line =~ s/ --title .* --start/ --title \"$header\" --start /;
      } ## end if ( $detail == 1 )
      if ( $detail == 2 ) {
        $line =~ s/ --font=DEFAULT:[0-9]:/ /;
        $line =~ s/ --font=TITLE:[0-9]:/ /;
        $line =~ s/ --width=400/ --width=$width $font_def_dashbo  /;
        $line =~ s/ --height=150/ --height=$height $font_tit_dashbo $no_legend /;
        $line =~ s/ 4 weeks/ month/;
        $line =~ s/ --x-grid=.* --alt-y-grid/ $xgrid_all --alt-y-grid/;
        $line =~ s/ --vertical-label=.* --units-exponent/ --units-exponent /;
        $line =~ s/ --title .* --start/ --title \"$item: $text\" --start /;
      } ## end if ( $detail == 2 )
      # new zooming function without rrdtool title in graphs
      if ( $detail == 9 || $detail == 8 ) {
        $line =~ s/ --title .* --start/ --title \"\" --start /;
      }
      if ($detail == 8 ) {
        $line =~ s/ --width=400/ --width=$width/;
        $line =~ s/ --height=150/ --height=$height/;
      }

      #print STDERR "$line";
      $cmd = $line;
    } ## end foreach my $line (@lines)

    if ( $lines_no == 0 ) {
      no_data_yet( "cmd file does not contain data source files: $tmp_file " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    RRDp::cmd qq($cmd);
    my $ret = RRDp::read;
    if ( $$ret =~ "ERROR" ) {
      error( "sum detail: Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    $png_end_heading = ret_graph_param($ret);
    return 1;
  } ## end if ( $item eq "san_data_sum_tot_in"...)
} ## end sub create_graph_san

sub create_graph {
  my $host       = shift;
  my $type       = shift;
  my $name       = shift;
  my $item       = shift;
  my $time       = shift;
  my $name_out   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $wrkdir     = shift;
  my $act_time   = shift;
  my $name_full  = $name;
  my $tmp_file   = "/var/tmp/stor2rrd-$host-$type-$item-$time.cmd";

  if ( $type =~ m/RANK/ ) {
    $name =~ s/-P.*//;    # filter out pool ID from rank name
  }

  $item =~ s/-FC$//;      # SWIZ port graphs per FCS adapter necessary filtering

  my $name_text = $name;
  if ( $item =~ m/^sum_capacity$/ ) {

    # POOL name translation --> from ID
    open( FHR, "< $wrkdir/$host/pool.cfg" ) || error( "Can't open $wrkdir/$host/pool.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @lines_translate = <FHR>;
    close(FHR);
    my $pool_found = 0;
    foreach my $linep (@lines_translate) {
      chomp($linep);
      ( my $id, my $name_pool ) = split( /:/, $linep );
      if ( $name =~ m/^$id$/ ) {
        $pool_found = 1;
        $name_text  = $name_pool;
        last;
      }
    } ## end foreach my $linep (@lines_translate)
    if ( $pool_found == 0 ) {
      $name_text = $name;
    }
  } ## end if ( $item =~ m/^sum_capacity$/)

  if ( $type =~ m/RANK/ && $st_type =~ m/SWIZ/ ) {

    # translate id into names only for MDISKS
    if ( -f "$wrkdir/$host/mdisk.cfg" ) {
      open( FHR, "< $wrkdir/$host/mdisk.cfg" ) || error( "Can't open $wrkdir/$host/mdisk.cfg : $! " . __FILE__ . ":" . __LINE__ ) && exit(1);
      my @mdisk_trans = <FHR>;
      my $rank_found  = 0;
      foreach my $line (@mdisk_trans) {
        chomp($line);
        ( my $id, $name_text ) = split( /:/, $line );
        if ( defined($name_text) && !$name_text eq '' && $id =~ m/^$name$/ ) {
          $rank_found = 1;
          last;
        }
      } ## end foreach my $line (@mdisk_trans)
      if ( $rank_found == 0 ) {
        $name_text = $name;
      }
      close(FHR);
    } ## end if ( -f "$wrkdir/$host/mdisk.cfg")
    else {
      my $act_time = localtime();
      print STDERR "$act_time: File does not exist (no mdisk translation to names): $wrkdir/$host/mdisk.cfg\n";    # no error()
    }
  } ## end if ( $type =~ m/RANK/ ...)
  if ( $type =~ m/^FS$/ ) {
    if ( $st_type =~ m/VNXF/ || $st_type =~ m/UNITY/ || $st_type =~ m/VNXE$/ ) {

      # translate id into names only for filesystems
      if ( -f "$wrkdir/$host/$type/fs.cfg" ) {
        open( FHR, "< $wrkdir/$host/$type/fs.cfg" ) || error( "Can't open $wrkdir/$type/$host/fs.cfg : $! " . __FILE__ . ":" . __LINE__ ) && exit(1);
        my @fs_trans = <FHR>;
        my $fs_found = 0;
        foreach my $line (@fs_trans) {
          chomp($line);
          ( my $id, $name_text ) = split( /:/, $line );
          if ( defined($name_text) && $name_text ne '' && $id =~ m/^$name$/ ) {
            $fs_found = 1;
            last;
          }
        } ## end foreach my $line (@fs_trans)
        if ( $fs_found == 0 ) {
          $name_text = $name;
        }
        close(FHR);
      } ## end if ( -f "$wrkdir/$host/$type/fs.cfg")
      else {
        my $act_time = localtime();
        print STDERR "$act_time: File does not exist (no filesystem translation to names): $wrkdir/$host/$type/fs.cfg\n";    # no error()-
      }
    } ## end if ( $st_type =~ m/VNXF/...)
  } ## end if ( $type =~ m/^FS$/ )

  if ( $type =~ m/DRIVE/ && -f "$wrkdir/$host/drive.cfg" ) {

    # translate id into names only for DRIVE (NetAPP)
    my @drive_trans = "";
    if ( -f "$wrkdir/$host/drive.cfg" ) {
      open( FHR, "< $wrkdir/$host/drive.cfg" ) || error( "Can't open $wrkdir/$host/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      @drive_trans = <FHR>;
      close(FHR);
    }
    my $drive_found = 0;
    my $drive_id    = "";
    foreach my $line (@drive_trans) {
      chomp($line);
      ( my $id, $name_text ) = split( /,/, $line );
      if ( defined($name_text) && !$name_text eq '' && $id =~ m/^$name$/ ) {
        $drive_found = 1;
        last;
      }
    } ## end foreach my $line (@drive_trans)
    if ( $drive_found == 0 ) {
      $name_text = $name;
    }
  } ## end if ( $type =~ m/DRIVE/...)

  if ( $type =~ m/DRIVE/ && -f "$wrkdir/$host/$type/drive.cfg" ) {

    # translate id into names only for DRIVE (DOTHILL)
    my @drive_trans = "";
    if ( -f "$wrkdir/$host/$type/drive.cfg" ) {
      open( FHR, "< $wrkdir/$host/$type/drive.cfg" ) || error( "Can't open $wrkdir/$host/$type/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      @drive_trans = <FHR>;
      close(FHR);
    }
    my $drive_found = 0;
    my $drive_id    = "";
    foreach my $line (@drive_trans) {
      chomp($line);
      ( my $id, $name_text ) = split( /,/, $line );
      if ( defined($name_text) && !$name_text eq '' && $id =~ m/^$name$/ ) {
        $drive_found = 1;
        last;
      }
    } ## end foreach my $line (@drive_trans)
    if ( $drive_found == 0 ) {
      $name_text = $name;
    }
  } ## end if ( $type =~ m/DRIVE/...)


  if ( $type =~ m/IO-GROUP/ && -f "$wrkdir/$host/$type/io_group.cfg" ) {

    # translate id into names only for IO-GROUP (SWIZ)
    my @io_group_trans = "";
    if ( -f "$wrkdir/$host/$type/io_group.cfg" ) {
      open( FHR, "< $wrkdir/$host/$type/io_group.cfg" ) || error( "Can't open $wrkdir/$host/$type/io_group.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      @io_group_trans = <FHR>;
      close(FHR);
    }
    my $io_group_found = 0;
    my $io_group_id    = "";
    foreach my $line (@io_group_trans) {
      chomp($line);
      ( my $id, $name_text ) = split( /:/, $line );
      if ( defined($name_text) && !$name_text eq '' && $id =~ m/^$name$/ ) {
        $io_group_found = 1;
        last;
      }
    } ## end foreach my $line (@drive_trans)
    if ( $io_group_found == 0 ) {
      $name_text = $name;
    }
  } ## end if ( $type =~ m/DRIVE/...)


  if ( $type =~ m/NAS$/ && -f "$wrkdir/$host/$type/nas.cfg" ) {

    # translate id into names only for NAS
    my @nas_trans = "";
    if ( -f "$wrkdir/$host/$type/nas.cfg" ) {
      open( FHR, "< $wrkdir/$host/$type/nas.cfg" ) || error( "Can't open $wrkdir/$host/$type/nas.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      @nas_trans = <FHR>;
      close(FHR);
    }
    my $nas_found = 0;
    foreach my $line (@nas_trans) {
      chomp($line);
      ( my $id, $name_text ) = split( /:/, $line );
      if ( defined($name_text) && !$name_text eq '' && $id =~ m/^$name$/ ) {
        $nas_found = 1;
        last;
      }
    } ## end foreach my $line (@nas_trans)
    if ( $nas_found == 0 ) {
      $name_text = $name;
    }
  } ## end if ( $type =~ m/NAS$/ ...)

  if ( $st_type =~ m/^3PAR/ && $type =~ m/PORT/ && -f "$wrkdir/$host/$type/port.cfg" ) {

    # translate id into names only for PORT 3PAR
    # not use SWIZ here as teher are long WWN which do not need to be in the legend (it is already in the graph header)
    open( FHR, "< $wrkdir/$host/$type/port.cfg" ) || error( "Can't open $wrkdir/$host/$type/port.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @port_trans = <FHR>;
    close(FHR);
    my $port_found = 0;
    foreach my $line (@port_trans) {
      chomp($line);
      ( my $id, $name_text ) = split( /:/, $line );
      if ( defined($name_text) && !$name_text eq '' && $id eq $name ) {
        $name_text  = $name;
        $port_found = 1;
        last;
      }
    } ## end foreach my $line (@port_trans)
    if ( $port_found == 0 ) {
      $name_text = $name;
    }
  } ## end if ( $st_type =~ m/^3PAR/...)

  if ( $type =~ m/NODE/ && -f "$wrkdir/$host/$type/node.cfg" ) {

    # translate id into names only for PORT 3PAR
    # not use SWIZ here as teher are long WWN which do not need to be in the legend (it is already in the graph header)
    open( FHR, "< $wrkdir/$host/$type/node.cfg" ) || error( "Can't open $wrkdir/$host/$type/node.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @node_trans = <FHR>;
    close(FHR);
    my $node_found = 0;
    foreach my $line (@node_trans) {
      chomp($line);
      ( my $id, $name_text ) = split( /:/, $line );
      if ( defined($name_text) && !$name_text eq '' && $id eq $name ) {
        $node_found = 1;
        last;
      }
    } ## end foreach my $line (@node_trans)
    if ( $node_found == 0 ) {
      $name_text = $name;
    }
  } ## end if ( $type =~ m/NODE/ ...)

  if ( $type =~ m/VOLUME-FOLDER/ && -f "$wrkdir/$host/$type/volume_folder.cfg" ) {

    # translate id into names only for VOLUME-FOLDER COMPELLENT
    open( FHR, "< $wrkdir/$host/$type/volume_folder.cfg" ) || error( "Can't open $wrkdir/$host/$type/volume_folder.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @volume_folder_trans = <FHR>;
    close(FHR);
    my $volume_folder_found = 0;
    foreach my $line (@volume_folder_trans) {
      chomp($line);
      ( my $id, $name_text ) = split( /:/, $line );
      if ( defined($name_text) && !$name_text eq '' && $id eq $name ) {
        $volume_folder_found = 1;
        last;
      }
    } ## end foreach my $line (@node_trans)
    if ( $volume_folder_found == 0 ) {
      $name_text = $name;
    }
  }

  if ( $type =~ m/NASPOOL/ && -f "$wrkdir/$host/pool.cfg" ) {

    # translate id into names only for NAS
    my @pool_trans = "";
    if ( -f "$wrkdir/$host/pool.cfg" ) {
      open( FHR, "< $wrkdir/$host/pool.cfg" ) || error( "Can't open $wrkdir/$host/pool.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      @pool_trans = <FHR>;
      close(FHR);
    }
    my $naspool_found = 0;
    foreach my $line (@pool_trans) {
      chomp($line);
      ( my $id, $name_text ) = split( /:/, $line );
      if ( defined($name_text) && !$name_text eq '' && $id =~ m/^$name$/ ) {
        $naspool_found = 1;
        last;
      }
    } ## end foreach my $line (@pool_trans)
    if ( $naspool_found == 0 ) {
      $name_text = $name;
    }
  } ## end if ( $type =~ m/NASPOOL/...)

  my $type_text = $type;
  if ( $st_type =~ m/SWIZ/ && $type =~ m/RANK/ ) {
    $type_text = "MDISK";
  }
  if ( $st_type =~ m/HUS/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/NETAPP/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/VSPG/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/DOTHILL/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/NETAPP/ && $type =~ m/NAS/ ) {
    $type_text = "SVM";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/NODE$/ ) {
    $type_text = "DIRECTOR";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/NODE-CACHE/ ) {
    $type_text = "CACHE";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/DRIVE/ ) {
    $type_text = "DISK GROUP";
  }
  if ( $st_type =~ m/VPLEX/ && $type =~ m/NODE$/ ) {
    $type_text = "DIRECTOR";
  }

  #print OUT "$host:$type:$item:$name:$time:$name_out\n";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;
  my $suffix      = "rrd";    #standard suffix

  if ( $item =~ m/^read$/ )  { $value_short = "MB/sec"; $value_long = "Read in MBytes";  $val = 1024; }
  if ( $item =~ m/^write$/ ) { $value_short = "MB/sec"; $value_long = "Write in MBytes"; $val = 1024; }
  if ( $item =~ m/read_io/ ) { $value_short = "IOPS";   $value_long = "Read in IO per second"; }
  if ( $item =~ m/resp_t/ )  { $value_short = "msec";   $value_long = "Read response time in msec"; }
  if ( $item =~ m/write_io/ )      { $value_short = "IOPS"; $value_long  = "Write in IO per second"; }
  if ( $item =~ m/^r_cache_hit$/ ) { $value_short = "%";    $value_long  = "percent"; $val = 1; }
  if ( $item =~ m/^w_cache_hit$/ ) { $value_short = "%";    $value_long  = "percent"; $val = 1; }
  if ( $item =~ m/^cache_hit$/ )   { $value_short = "%";    $value_long  = "percent"; $val = 1; }
  if ( $item =~ m/^sys$/ )         { $value_short = "%";    $value_long  = "CPU utilization in %"; }
  if ( $item =~ m/^compress$/ )    { $value_short = "%";    $value_long  = "CPU utilization in %"; }
  if ( $item =~ m/^busy$/ )        { $value_short = "%";    $value_long  = "CPU utilization in %"; }
  if ( $item =~ m/^pprc_data_r$/ ) { $suffix      = "rrp";  $value_short = "MB/sec"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^pprc_data_w$/ ) { $suffix      = "rrp";  $value_short = "MB/sec"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^pprc_rio$/ )  { $suffix = "rrp"; $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^pprc_wio$/ )  { $suffix = "rrp"; $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^pprc_rt_r$/ ) { $suffix = "rrp"; $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^pprc_rt_w$/ ) { $suffix = "rrp"; $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^operating_rate$/ )                             { $value_short = "%";        $value_long = "Operating rate in %"; }
  if ( $item =~ m/^write_pend$/ )                                 { $value_short = "%";        $value_long = "Write Pending Rate in %"; }
  if ( $item =~ m/^usage_rate$/ )                                 { $value_short = "%";        $value_long = "Usage Rate in %"; }
  if ( $item =~ m/^clean_usage$/ )                                { $value_short = "%";        $value_long = "Clean Queue Usage Rate in %"; }
  if ( $item =~ m/^dnlc_hit$/ )                                   { $value_short = "%";        $value_long = "dnlc hit in %"; }
  if ( $item =~ m/^buffer_hit$/ )                                 { $value_short = "%";        $value_long = "buffer hit in %"; }
  if ( $item =~ m/^middle_usage$/ )                               { $value_short = "%";        $value_long = "Middle Queue Usage Rate in %"; }
  if ( $item =~ m/^phys_usage$/ )                                 { $value_short = "%";        $value_long = "Physical Queue Usage Rate in %"; }
  if ( $item =~ m/packet/ )                                       { $value_short = "pct/sec";  $value_long = "Packets/sec"; }
  if ( $item =~ m/_error/ )                                       { $value_short = "pct/sec";  $value_long = "Packets/sec"; }
  if ( $item =~ m/^cifs_read-3_read-4_read/ )                     { $value_short = "MB/sec";   $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^cifs_write-3_write-4_write/ )                  { $value_short = "MB/sec";   $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) { $value_short = "%";        $value_long = "%"; }
  if ( $item =~ m/data_rate/ )                                    { $value_short = "MB/sec";   $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^data_rate_ini$/ )                              { $value_short = "MB/sec";   $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^resp_t_ini$/ )                                 { $value_short = "msec";     $value_long = "Read response time in msec"; }
  if ( $item =~ m/io_rate/ )                                      { $value_short = "IOPS";     $value_long = "IO per second"; }
  if ( $item =~ m/^io_rate_ini$/ )                                { $value_short = "IOPS";     $value_long = "IO per second"; }
  if ( $item =~ m/read_calls/ )                                   { $value_short = "calls/s";  $value_long = "Read in Calls per second"; }
  if ( $item =~ m/write_calls/ )                                  { $value_short = "Calls/s";  $value_long = "Write in Calls per second"; }
  if ( $item =~ m/calls_rate/ )                                   { $value_short = "Calls/s";  $value_long = "Calls per second"; }
  if ( $item =~ m/b2b/ )                                          { $value_short = "%";        $value_long = "percent"; }
  if ( $item =~ m/crc/ )                                          { $value_short = "Errors/s"; $value_long = "Errors per sec"; }
  if ( $item =~ m/dirty_cache_pages/ )                            { $value_short = "%";        $value_long = "Dirty Cache Pages in %"; }
  if ( $item =~ m/^r_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; $val = 1; }
  if ( $item =~ m/^w_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; $val = 1; }
  if ( $item =~ m/^busy$/        && $st_type eq "VMAX" )    { $value_short = "%";    $value_long = "BUSY in %"; }
  if ( $item =~ m/^sum_busy/     && $st_type eq "VMAX" )    { $value_short = "%";    $value_long = "BUSY in %"; }
  if ( $item =~ m/^WP$/          && $st_type eq "VMAX" )    { $value_short = "%";    $value_long = "Cache WP in %"; }
  if ( $item =~ m/^busy$/        && $st_type eq "VPLEX" )   { $value_short = "%";    $value_long = "BUSY in %"; }
  if ( $item =~ m/^sum_busy/     && $st_type eq "VPLEX" )   { $value_short = "%";    $value_long = "BUSY in %"; }
  if ( $item =~ m/^io_pending$/ ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^cifs_ops$|^nfs_ops$/ && $st_type eq "NETAPP" && $type =~ m/^QTREE$/ ) { $value_short = "Ops"; $value_long = "Protocols in Ops"; $val = 1; }
  if ( $item =~ m/^queue$/ && $st_type eq "OCEANSTOR" ) { $value_short = "Unit/s"; $value_long = "Unit per second"; $val = 1; }
  if ( $item =~ m/read_ops/ )  { $value_short = "OPS"; $value_long = "OPS per second"; $val = 1; }
  if ( $item =~ m/write_ops/ ) { $value_short = "OPS"; $value_long = "OPS per second"; $val = 1; }
  if ( $item =~ m/ops_rate/ )  { $value_short = "OPS"; $value_long = "OPS per second"; $val = 1; }
  if ( $item =~ m/^r_transfer_size$/ )    { $value_short = "kB/sec";   $value_long = "kBytes/sec"; $val = 1; }
  if ( $item =~ m/^w_transfer_size$/ )    { $value_short = "kB/sec";   $value_long = "kBytes/sec"; $val = 1; }

  ## port initiator for port vspg
  if ( $st_type =~ m/VSPG/ && $type =~ m/PORT/ && $item =~ m/^data_rate_ini$|^io_rate_ini$|^resp_t_ini$/ ) {
    $suffix = "rri";
    if ($item eq "resp_t_ini"){
      $val = 1000;
    }
  }
  if ( $st_type =~ m/^SWIZ$/ && $type =~ m/^PORT$/ && ( $item =~ m/^b2b$/ || $item =~ m/^crc$/ ) ) {
    $suffix = "rrp";
  }

  # NetApp stuff, very sensitive for any change ... do not touch it
  #print STDERR "01 $item : $name : \n";
  if ( ( $item =~ m/^cifs/ || $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) && $name !~ m/\+/ && $name !~ m/ / ) {

    #print STDERR "02 $item : $name\n";
    # it is aggregated graphs, calls rrdtool directly, must be quoted
    $value_long =~ s/^/\\\"/;
    $value_long =~ s/$/\\\"/;
  } ## end if ( ( $item =~ m/^cifs/...))
  if ( $item =~ m/^cifs_ops$|^nfs_ops$/ && $st_type eq "NETAPP" && $type =~ m/^QTREE$/ ) {
    $value_long = "Protocols in Ops";
  }

  if ( ( $name =~ m/\+/ || $name =~ m/ / ) && ( $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ || $item =~ m/^cifs/ ) ) {

    #print STDERR "03 $item : $name\n";
    # name contains: node-02+wafl_exempt-kahuna-exempt-hostOS-nwk_legac
    # cifs_io_rate-3_io_rate-4_io_rate-iscsi_io_rate-fcp_io_rate
    if ( $item =~ m/^cifs/ ) {
      my $item_org = $item;
      $item_org =~ s/cifs_//;
      $item_org =~ s/-.*//;
      $item = lc($name);
      $item =~ s/nfs3/3_/;
      $item =~ s/nfs4/4_/;
      $item =~ s/iscsi/iscsi_/;
      $item =~ s/cifs/cifs_/;
      $item =~ s/fcp/fcp_/;
      $item .= $item_org;
    } ## end if ( $item =~ m/^cifs/)
    else {
      $item = $name;
    }
  } ## end if ( ( $name =~ m/\+/ ...))
  $item =~ s/^.*\+//;
  $item =~ s/^.* //;
  $name =~ s/ .*//;
  $name =~ s/\+.*//;

  # end of NetApp
  my $rrd = "$basedir/data/$host/$type/$name.$suffix";

  if ( $st_type =~ m/^NETAPP$/ && $type =~ m/^QTREE$/ ) {
    $suffix = "rrq";
    $rrd    = "$basedir/data/$host/VOLUME/$name.$suffix";
    $name_text =~ s/@/\//g;
  }
  if ( $type =~ m/RANK/ ) {

    # original name with POOL info "-PXY"
    $name =~ s/^mdisk//;
    $rrd = "$basedir/data/$host/$type/$name_full.$suffix";
    if ( !-f $rrd ) {

      # sometimes RANK might go here without pool info
      my $time_newest = 0;
      foreach my $file (<$basedir/data/$host/$type/$name-P*.$suffix>) {
        my $file_time = ( stat("$file") )[9];
        if ( $file_time > $time_newest ) {
          $rrd         = $file;
          $time_newest = $file_time;
          $name_text   = $name;
        }
      } ## end foreach my $file (<$basedir/data/$host/$type/$name-P*.$suffix>)
    } ## end if ( !-f $rrd )
  } ## end if ( $type =~ m/RANK/ )
  ### cache write pendign and usage rate for vspg
  if ( $type =~ m/NODE-CACHE/ && $st_type =~ /VSPG/ ) {
    $name = "cache";
    $rrd  = "$basedir/data/$host/$type/$name.$suffix";
  }

  if ( $type eq "DRIVE" && $st_type eq "NETAPP" ) {

    # translate id into names only for DRIVE (NetAPP)
    if ( -f "$basedir/data/$host/drive.cfg" ) {
      open( FHR, "< $basedir/data/$host/drive.cfg" ) || error( "Can't open $basedir/data/$host/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      my @drive_trans = <FHR>;
      close(FHR);
      my $drive_found = 0;
      foreach my $line (@drive_trans) {
        chomp($line);
        ( my $id, my $name_act ) = split( /,/, $line );
        if ( defined($name_act) && !$name_act eq '' && $name_act =~ m/^$name$/ ) {
          $rrd         = "$basedir/data/$host/$type/$id.$suffix";
          $drive_found = 1;
          last;
        }

        # might happen that there is already translated name (id), then allow go it trough
        if ( defined($id) && !$id eq '' && $id =~ m/^$name$/ ) {
          $rrd         = "$basedir/data/$host/$type/$id.$suffix";
          $drive_found = 1;
          last;
        }
      } ## end foreach my $line (@drive_trans)
      if ( $drive_found == 0 ) {
        $rrd = "$basedir/data/$host/$type/$name.$suffix";
        if ( !-f $rrd ) {
          error( "Cannot translate drive $name to DB file name " . __FILE__ . ":" . __LINE__ ) && return 0;
        }
      } ## end if ( $drive_found == 0)
    } ## end if ( -f "$basedir/data/$host/drive.cfg")
  } ## end if ( $type eq "DRIVE" ...)
  if ( $type eq "DRIVE" && $st_type eq "DOTHILL" ) {

    # translate id into names only for DRIVE (NetAPP)
    if ( -f "$basedir/data/$host/DRIVE/drive.cfg" ) {
      open( FHR, "< $basedir/data/$host/DRIVE/drive.cfg" ) || error( "Can't open $basedir/data/$host/DRIVE/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      my @drive_trans = <FHR>;
      close(FHR);
      my $drive_found = 0;
      foreach my $line (@drive_trans) {
        chomp($line);
        ( my $id, my $name_act ) = split( /,/, $line );
        if ( defined($name_act) && !$name_act eq '' && $name_act =~ m/^$name$/ ) {
          $rrd         = "$basedir/data/$host/$type/$id.$suffix";
          $drive_found = 1;
          last;
        }

        # might happen that there is already translated name (id), then allow go it trough
        if ( defined($id) && !$id eq '' && $id =~ m/^$name$/ ) {
          $rrd         = "$basedir/data/$host/$type/$id.$suffix";
          $drive_found = 1;
          last;
        }
      } ## end foreach my $line (@drive_trans)
      if ( $drive_found == 0 ) {
        $rrd = "$basedir/data/$host/$type/$name.$suffix";
        if ( !-f $rrd ) {
          error( "Cannot translate drive $name to DB file name " . __FILE__ . ":" . __LINE__ ) && return 0;
        }
      } ## end if ( $drive_found == 0)
    } ## end if ( -f "$basedir/data/$host/DRIVE/drive.cfg")
  } ## end if ( $type eq "DRIVE" ...)
  if ( !-f $rrd ) {

    # try alias
    my $alias = alias( $type, $host, $name );
    if ( $alias eq '' ) {
      error( "Could not find $type ID for $host:$type:$name in neither in etc/alias.cfg " . __FILE__ . ":" . __LINE__ );
      return 0;
    }
    $rrd = "$basedir/data/$host/$type/$alias.$suffix";
    if ( $type =~ m/RANK/ ) {

      # original name with POOL info "-PXY"
      # there can be more rank files in different pools (historical reasons) --> use always the latest one
      $rrd = "$basedir/data/$host/$type/$name_full.$suffix";
      my $time_newest = 0;
      foreach my $file (<$basedir/data/$host/$type/$alias-P*.$suffix>) {
        my $file_time = ( stat("$file") )[9];
        if ( $file_time > $time_newest ) {
          $rrd         = $file;
          $time_newest = $file_time;
          $name_text   = $name;
        }
      } ## end foreach my $file (<$basedir/data/$host/$type/$alias-P*.$suffix>)
    } ## end if ( $type =~ m/RANK/ )
  } ## end if ( !-f $rrd )
  if ( !-f $rrd ) {
    error( "Could not find $type ID for $host:$type:$name in neither in etc/alias.cfg " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( $st_type =~ m/3PAR/ && $type =~ /TIER/ && $item =~ m/used/ ) {
    $rrd =~ s/\.rrd/-cap\.rrd/;
  }

  if ( $item =~ m/^sum_capacity$/ ) { $value_short = "TB"; $value_long = "TBytes"; $rrd = "$basedir/data/$host/$type/$name-cap.rrd"; }

  if ( $detail == 1 ) {
    $font_def = $font_def_popup;
    $font_tit = $font_tit_popup;
  }
  my $name_space = sprintf( "%-10s", "$name_text" );
  my $comment    = sprintf( "%-25s", "$item [$value_short]" );
  my $xgrid_all = xgrid_set( $time, $detail );
  my $text      = text_set($time);
  my $last      = " last";
  my $item_text = $item;

  if ( $item =~ m/^cifs_io_rate-3_io_rate-4_io_rate/ )            { $item_text = "Protocols total"; }
  if ( $item =~ m/^cifs_read_io-3_read_io-4_read_io/ )            { $item_text = "Protocols read"; }
  if ( $item =~ m/^cifs_write_io-3_write_io-4_write_io/ )         { $item_text = "Protocols write"; }
  if ( $item =~ m/^cifs_data_rate-3_data_rate-4_data_rate/ )      { $item_text = "Protocols total"; }
  if ( $item =~ m/^cifs_read-3_read-4_read/ )                     { $item_text = "Protocols read"; }
  if ( $item =~ m/^cifs_write-3_write-4_write/ )                  { $item_text = "Protocols write"; }
  if ( $item =~ m/^cifs_resp_t-3_resp_t-4_resp_t/ )               { $item_text = "Protocols total"; }
  if ( $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) { $item_text = "CPU"; }

  my $header = "$type_text : $name_text : $item_text :$last $text";
  if ( $detail == 2 ) {
    $header = "$name_text : $item_text $text";
  }

  my $start_time = "now-1$time";
  my $end_time   = "now-1$time+1$time";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # Historical report detected
      $start_time = $start_unix;
      $end_time   = $end_unix;

      #$xgrid ="COMMENT: "; # just a trick to place there something otherwise rrdtool complains
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header    = "$type_text : $name_text : $item : $start_human - $end_human";
      $text      = "";
      $xgrid_all = "--alt-y-grid";
      if ( $detail ne "9" && $detail ne "8" ) {
        $font_def  = $font_def_hist;
        $font_tit  = $font_tit_hist;
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my $vertical_label = "--vertical-label=$value_long";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";
    $no_legend      = "--no-legend";
  }
  $rrd =~ s/:/\\:/g;           # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing
  $name_space =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

  my $answ = "";
  my $item1 .= "";
  my $item2 .= "";


  # new zooming function without rrdtool title in graphs
  if ( $detail eq "9" || $detail eq "8" ) { $header = ""; }

  if ( $item =~ m/^io$/ || $item =~ m/^data$/ ) {
    if ( $item =~ m/^io$/ ) {
      $item1 .= "read_" . $item;
      $item2 .= "write_" . $item;
    }
    if ( $item =~ m/^data$/ ) {
      $item1 .= "read";
      $item2 .= "write";
    }
    RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=0.2"
          "--units-exponent=1.00"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:value1="$rrd":$item1:AVERAGE"
          "DEF:value2="$rrd":$item2:AVERAGE"
          "CDEF:valmb1=value1,$val,/"
          "CDEF:valmb2=value2,$val,/"
          "CDEF:tot=valmb2,valmb1,+"
          "COMMENT:$comment Avg        Max\\n"
          "LINE1:valmb1#00FF00:read  $name_space "
          "GPRINT:valmb1:AVERAGE:%8.1lf"
          "GPRINT:valmb1:MAX:%8.1lf"
          "COMMENT:\\n"
          "LINE1:valmb2#4444FF:write $name_space "
          "GPRINT:valmb2:AVERAGE:%8.1lf"
          "GPRINT:valmb2:MAX:%8.1lf"
          "COMMENT:\\n"
          "COMMENT:Total              "
          "GPRINT:tot:AVERAGE:%8.1lf"
          "GPRINT:tot:MAX:%8.1lf"
          "COMMENT:\\n"
          "HRULE:0#000000");
    $answ = RRDp::read;
    if ( $$answ =~ "ERROR" ) {
      error("$host:$type Graph rrdtool error : $$answ");
      if ( $answ =~ "is not an RRD file" ) {
        ( my $err, my $file, my $txt ) = split( /'/, $answ );
        error("It need to be removed due to corruption: $file");
      }
      else {
        error("Graph rrdtool error : $answ");
      }
    } ## end if ( $$answ =~ "ERROR")
    $png_end_heading = ret_graph_param($answ);
  } ## end if ( $item =~ m/^io$/ ...)
  else {
    if ( $item =~ m/^resp$/ ) {
      $item1 .= "resp_t_r";
      $item2 .= "resp_t_w";
      RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=0.2"
          "--units-exponent=1.00"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:val1=$rrd:$item1:AVERAGE"
          "DEF:val2=$rrd:$item2:AVERAGE"
          "COMMENT:$comment Avg        Max\\n"
          "LINE1:val1#00FF00:read  $name_space "
          "GPRINT:val1:AVERAGE:%8.1lf"
          "GPRINT:val1:MAX:%8.1lf"
          "COMMENT:\\n"
          "LINE1:val2#4444FF:write $name_space "
          "GPRINT:val2:AVERAGE:%8.1lf"
          "GPRINT:val2:MAX:%8.1lf"
          "COMMENT:\\n"
          "HRULE:0#000000");
      $answ            = RRDp::read;
      $png_end_heading = ret_graph_param($answ);
    } ## end if ( $item =~ m/^resp$/)
    else {
      if ( $item =~ m/^sys$/ && $st_type =~ m/3PAR/ ) {
        $item1 = "sys";
        $item2 = "usr";
        my $name_space1 = sprintf( "%-10s", "CPU sys" );
        my $name_space2 = sprintf( "%-10s", "CPU user" );
        RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=0.2"
          "--units-exponent=1.00"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:val1=$rrd:$item1:AVERAGE"
          "DEF:val2=$rrd:$item2:AVERAGE"
          "COMMENT:$comment Avg        Max\\n"
          "AREA:val1#FF0000:$name_space1       "
          "GPRINT:val1:AVERAGE:%8.1lf"
          "GPRINT:val1:MAX:%8.1lf"
          "COMMENT:\\n"
          "STACK:val2#4444FF:$name_space2       "
          "GPRINT:val2:AVERAGE:%8.1lf"
          "GPRINT:val2:MAX:%8.1lf"
          "COMMENT:\\n"
          "HRULE:0#000000");
        $answ            = RRDp::read;
        $png_end_heading = ret_graph_param($answ);
      } ## end if ( $item =~ m/^sys$/...)
      else {
        if ( $item =~ m/^sum_capacity$/ || ( $st_type =~ m/3PAR/ && $type =~ /TIER/ && $item =~ m/used/ ) ) {
          RRDp::cmd qq($graph_cmd "$name_out"
          "--title" "$header"
          "--start" "$start_time"
          "--end" "$end_time"
          "--imgformat" "PNG"
          "--slope-mode"
          "--width=$width"
          "--height=$height"
          "--step=$STEP"
          "--lower-limit=0"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "$vertical_label"
          "--alt-autoscale-max"
          "--interlaced"
          "--upper-limit=0.2"
          "--units-exponent=1.00"
          "$no_legend"
          "$disable_rrdtool_tag"
          "$font_def"
          "$font_tit"
          "$xgrid_all"
          "DEF:total="$rrd":total:AVERAGE"
          "DEF:free="$rrd":free:AVERAGE"
          "CDEF:used=total,free,-"
          "COMMENT:$comment Avg        Max\\n"
          "AREA:used#FF3333:Used               "
          "GPRINT:used:AVERAGE:%7.2lf"
          "GPRINT:used:MAX:%7.2lf"
          "COMMENT:\\n"
          "STACK:free#33FF33:Free               "
          "GPRINT:free:AVERAGE:%7.2lf"
          "GPRINT:free:MAX:%7.2lf"
          "COMMENT:\\n"
          "HRULE:0#000000");
          $answ            = RRDp::read;
          $png_end_heading = ret_graph_param($answ);
        } ## end if ( $item =~ m/^sum_capacity$/...)
        else {
          if ( $item =~ m/^cifs_io_rate-3_io_rate-4_io_rate-iscsi_io_rate-fcp_io_rate/
            || $item =~ m/^cifs_read_io-3_read_io-4_read_io-iscsi_read_io-fcp_read_io/
            || $item =~ m/^cifs_write_io-3_write_io-4_write_io-iscsi_write_io-fcp_write_io/
            || $item =~ m/^cifs_data_rate-3_data_rate-4_data_rate-iscsi_data_rate-fcp_data_rate/
            || $item =~ m/^cifs_read-3_read-4_read-iscsi_read-fcp_read/
            || $item =~ m/cifs_write-3_write-4_write-iscsi_write-fcp_write/
            || $item =~ m/^cifs_resp_t-3_resp_t-4_resp_t-iscsi_resp_t-fcp_resp_t/
            || $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legac/ )
          {
            my $cmd .= "graph \\\"$name_out\\\"";
            $cmd    .= " --title \\\"$header\\\"";
            $cmd    .= " --start $start_time";
            $cmd    .= " --end $end_time";
            $cmd    .= " --imgformat PNG";
            $cmd    .= " --slope-mode";
            $cmd    .= " --width=$width";
            $cmd    .= " --height=$height";
            $cmd    .= " --step=$STEP";
            $cmd    .= " --lower-limit=0";
            $cmd    .= " --color=BACK#$pic_col";
            $cmd    .= " --color=SHADEA#$pic_col";
            $cmd    .= " --color=SHADEB#$pic_col";
            $cmd    .= " --color=CANVAS#$pic_col";
            $cmd    .= " $vertical_label";
            $cmd    .= " --alt-autoscale-max";
            $cmd    .= " --interlaced";
            $cmd    .= " --upper-limit=0.2";
            $cmd    .= " --units-exponent=1.00";
            $cmd    .= " $no_legend";
            $cmd    .= " $disable_rrdtool_tag";
            $cmd    .= " $font_def";
            $cmd    .= " $font_tit";
            $cmd    .= " $xgrid_all";
            $cmd    .= " HRULE:0#000000";

            my @items      = split( /-/, $item );
            my $gtype      = "AREA";                # here stack this in compare to storage.pl : wafl_exempt-kahuna-exempt-hostOS-nwk_legacy
            my $i          = 0;
            my $lpar_space = $name;
            my $color_indx = 0;
            my $cmd_print  = "";

            # add spaces to lpar name (for formating graph legend)
            my $legend = sprintf( "%-20s", "[$value_short]" );
            $cmd .= " COMMENT:\\\"$legend Avg      Max\\l\\\"";

            foreach my $item_act (@items) {
              my $item_act_text = $item_act;
              $item_act_text =~ s/^3.*/NFS 3/;
              $item_act_text =~ s/^4.*/NFS 4/;
              $item_act_text =~ s/^cifs.*/CIFS/;
              $item_act_text =~ s/^iscsi.*/iSCSI/;
              $item_act_text =~ s/^fcp.*/FCP/;

              #$item_act_text =~ s/_/ /;
              my $lpar_space_act = sprintf( "%-14s", "$item_act_text" );
              $cmd .= " DEF:$item_act${i}_u=\\\"$rrd\\\":$item_act:AVERAGE";
              $cmd .= " CDEF:$item_act-m${i}=$item_act${i}_u,$val,/";                         # convert into MB if necessary, normaly is there 1
              $cmd .= " $gtype:$item_act-m${i}$color[$color_indx]:\\\"$lpar_space_act\\\"";
              if ( $item =~ m/io/ ) {
                $cmd .= " GPRINT:$item_act-m${i}:AVERAGE:\\\"%6.0lf \\\"";
                $cmd .= " GPRINT:$item_act-m${i}:MAX:\\\"%6.0lf \\l\\\"";
              }
              else {
                $cmd .= " GPRINT:$item_act-m${i}:AVERAGE:\\\"%6.1lf \\\"";
                $cmd .= " GPRINT:$item_act-m${i}:MAX:\\\"%6.1lf \\l\\\"";
              }
              if ( $item !~ m/resp_t/ ) {

                # here stack this in compare to storage.pl : wafl_exempt-kahuna-exempt-hostOS-nwk_legacy
                $gtype = "STACK";
              }
              $i++;
              $color_indx++;
              if ( $color_indx > $color_max ) {
                $color_indx = 0;
              }
            } ## end foreach my $item_act (@items)
            $cmd =~ s/\\"/"/g;
            my $FH;
            open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
            print FH "$cmd\n";
            close(FH);
            my $answ = `$rrdtool - < $tmp_file 2>&1`;
            unlink("$tmp_file");

            if ( $answ =~ "ERROR" ) {
              if ( $answ =~ "ERROR: malloc fetch data area" ) {
                error( "$host:$type:$item:$time: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
              }
              else {
                error( "$host:$type:$item:$time: Multi graph rrdtool error : $answ " . __FILE__ . ":" . __LINE__ );
              }
            } ## end if ( $answ =~ "ERROR" )

            #$png_end_heading = ret_graph_param($answ);
          } ## end if ( $item =~ m/^cifs_io_rate-3_io_rate-4_io_rate-iscsi_io_rate-fcp_io_rate/...)
          else {
            if ( $st_type eq "VNX" && $item eq "sys" ) {
              RRDp::cmd qq($graph_cmd "$name_out"
              "--title" "$header"
              "--start" "$start_time"
              "--end" "$end_time"
              "--imgformat" "PNG"
              "--slope-mode"
              "--width=$width"
              "--height=$height"
              "--step=$STEP"
              "--lower-limit=0"
              "--color=BACK#$pic_col"
              "--color=SHADEA#$pic_col"
              "--color=SHADEB#$pic_col"
              "--color=CANVAS#$pic_col"
              "$vertical_label"
              "--alt-autoscale-max"
              "--interlaced"
              "--upper-limit=0.2"
              "--units-exponent=1.00"
              "$no_legend"
              "$disable_rrdtool_tag"
              "$font_def"
              "$font_tit"
              "$xgrid_all"
              "DEF:busy_ticks="$rrd":busy_ticks:AVERAGE"
              "DEF:idle_ticks="$rrd":idle_ticks:AVERAGE"
              "CDEF:all_ticks=busy_ticks,idle_ticks,+"
              "CDEF:util_l=busy_ticks,all_ticks,/"
              "CDEF:valmb=util_l,100,*"
              "COMMENT:$comment Avg        Max\\n"
              "LINE1:valmb#4444FF:$name_space       "
              "GPRINT:valmb:AVERAGE:%8.1lf"
              "GPRINT:valmb:MAX:%8.1lf"
              "COMMENT:\\n"
              "HRULE:0#000000");
              $answ            = RRDp::read;
              $png_end_heading = ret_graph_param($answ);
            } ## end if ( $st_type eq "VNX"...)
            else {
              RRDp::cmd qq($graph_cmd "$name_out"
                "--title" "$header"
                "--start" "$start_time"
                "--end" "$end_time"
                "--imgformat" "PNG"
                "--slope-mode"
                "--width=$width"
                "--height=$height"
                "--step=$STEP"
                "--lower-limit=0"
                "--color=BACK#$pic_col"
                "--color=SHADEA#$pic_col"
                "--color=SHADEB#$pic_col"
                "--color=CANVAS#$pic_col"
                "$vertical_label"
                "--alt-autoscale-max"
                "--interlaced"
                "--upper-limit=0.2"
                "--units-exponent=1.00"
                "$no_legend"
                "$disable_rrdtool_tag"
                "$font_def"
                "$font_tit"
                "$xgrid_all"
                "DEF:value=$rrd:$item:AVERAGE"
                "CDEF:valmb=value,$val,/"
                "COMMENT:$comment Avg        Max\\n"
                "LINE1:valmb#4444FF:$name_space       "
                "GPRINT:valmb:AVERAGE:%8.1lf"
                "GPRINT:valmb:MAX:%8.1lf"
                "COMMENT:\\n"
                "HRULE:0#000000");
              $answ            = RRDp::read;
              $png_end_heading = ret_graph_param($answ);
            } ## end else [ if ( $st_type eq "VNX"...)]
          } ## end else [ if ( $item =~ m/^cifs_io_rate-3_io_rate-4_io_rate-iscsi_io_rate-fcp_io_rate/...)]
        } ## end else [ if ( $item =~ m/^sum_capacity$/...)]
      } ## end else [ if ( $item =~ m/^sys$/...)]
    } ## end else [ if ( $item =~ m/^resp$/)]
  } ## end else [ if ( $item =~ m/^io$/ ...)]
  return (1);
} ## end sub create_graph

# Print the png out
sub print_png {
  my $name_out = shift;

  #print STDERR "99 $name_out\n";
  if ( !-f "$name_out" ) {
    error( "Graph rrdtool error : print_png : file does not exist : $name_out " . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  if ( defined $ENV{DEMO} && ( $ENV{DEMO} == 1 ) ) {
    my $name_out_8 = "$name_out" . ".png";
    `convert $name_out -colors 256 PNG8:$name_out_8;`;
    ` mv $name_out_8 $name_out`;

    # print STDERR "convert in use for $name_out\n";
  } ## end if ( defined $ENV{DEMO...})

  if ( @table_out > 1 && ($detail < 1 || $detail == 9 || $detail == 8) && ( $rrd_legend ne "orig" ) ) {
    $heading_png =~ s/image\/png/application\/json/;
    print "$heading_png";
    print "$png_end_heading";    # for graphv possibility
                                 #print STDERR "\$heading_png ,$heading_png,";
                                 #print STDERR "\$png_end_heading ,$png_end_heading,";
    my $picture = do {
      local $/ = undef;
      open my $fh, "<", $name_out || error( "Cannot open  $name_out: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      <$fh>;
    };
    my $encoded_pict = encode_base64( $picture, "" );
    $encoded_pict = "data:image/png;base64,$encoded_pict";

    my $table_one_string = "@table_out";
    my $encoded_table_out = encode_base64( $table_one_string, "" );

    my $lll = length $encoded_pict;
    my $llx = length $encoded_table_out;

    #print STDERR "length encoded pict is $lll\n";
    #print STDERR "length encoded table is $llx\n";

    print "{\"img\":\"$encoded_pict\", \"table\": \"$encoded_table_out\"}";

    #print STDERR "{\"img\":\"encoded_pict\", \"table\": \"$table_one_string\"}";

  } ## end if ( @table_out > 1 &&...)
  else {
    print "$heading_png";
    print "$png_end_heading";    # for graphv possibility
                                 #print STDERR "no table $heading_png ,$heading_png,";
                                 #print STDERR "no table $png_end_heading ,$png_end_heading,";

    open( PNG, "< $name_out" ) || error( "Cannot open  $name_out: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    binmode(PNG);
    while ( read( PNG, $b, 4096 ) ) {
      print "$b";
    }
    close(PNG);
  } ## end else [ if ( @table_out > 1 &&...)]
  unlink("$name_out");

  return 1;
} ## end sub print_png

sub create_graph_volume {
  my $host        = shift;
  my $type        = shift;
  my $lpar        = shift;
  my $item        = shift;
  my $time        = shift;
  my $name_out    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $st_type     = shift;
  my $step        = shift;           # for volumes must be step corect due to DS8K and read/write stored in GAUGE instead oif ABSOLUTE
                                     # print STDERR "create-graph-volume : $host,$type,$lpar,$item,$time,$name_out,$detail,$start_unix,$end_unix,$st_type,$step\n";
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $last        = "COMMENT: ";
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $req_time    = 0;
  my $xgrid       = "";
  my $suffix      = "rrd";
  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;
  my $type_lun    = 0;

  my $par_vol = $ENV{HP3PAR_VV_SAMPLE_RATE};
  if ( $st_type =~ m/3PAR/ && defined($par_vol) && !$par_vol eq '' && isdigit($par_vol) && $par_vol < 300 ) {

    # custom 3PAR sample rate for Volumes could be used when HP3PAR_VV_SAMPLE_RATE is set in environment, supported are 30 or 60 secs rates only (instead of default 300secs)
    $step = $par_vol;
  }

  if ( $item =~ m/^data_rate$/ ) { $value_short = "MB"; $value_long = "Data throughput in MBytes"; $val = 1024; }
  if ( $item =~ m/^read$/ )      { $value_short = "MB"; $value_long = "Data throughput in MBytes"; $val = 1024; }
  if ( $item =~ m/^write$/ )     { $value_short = "MB"; $value_long = "Data throughput in MBytes"; $val = 1024; }
  if ( $item =~ m/^io_rate$/ )     { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^resp_t$/ )      { $value_short = "msec"; $value_long = "Response time in msec"; $val = 1; }
  if ( $item =~ m/^resp_t_w$/ )    { $value_short = "msec"; $value_long = "Response time in msec"; $val = 1; }
  if ( $item =~ m/^resp_t_r$/ )    { $value_short = "msec"; $value_long = "Response time in msec"; $val = 1; }
  if ( $item =~ m/^read_io$/ )     { $value_short = "IOPS"; $value_long = "Read in IO per second"; }
  if ( $item =~ m/^write_io$/ )    { $value_short = "IOPS"; $value_long = "Write in IO per second"; }
  if ( $item =~ m/^r_cache_hit$/ ) { $value_short = "%";    $value_long = "percent"; $val = 1; }
  if ( $item =~ m/^w_cache_hit$/ ) { $value_short = "%";    $value_long = "percent"; $val = 1; }
  if ( $item =~ m/^r_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; $val = 1; }
  if ( $item =~ m/^w_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; $val = 1; }
  if ( $item =~ m/^r_cache_usage$/ ) { $value_short = "MB"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^w_cache_usage$/ ) { $value_short = "MB"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^total_usage$/ )   { $value_short = "MB"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^read_b$/ )        { $value_short = "MB"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^write_b$/ )       { $value_short = "MB"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^read_io_b$/ )  { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^write_io_b$/ ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^resp_t_r_b$/ ) { $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^resp_t_w_b$/ ) { $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^busy$/ )       { $value_short = "%";    $value_long = "CPU utilization in %"; }
  if ( $item =~ m/^qdepth$/ )     { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^io_pending$/ ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^queue$/ && $st_type eq "OCEANSTOR" ) { $value_short = "Unit/s"; $value_long = "Unit per second"; $val = 1; }
  if ( $item =~ m/^r_transfer_size$/ )    { $value_short = "kB/sec";   $value_long = "kBytes/sec"; $val = 1; }
  if ( $item =~ m/^w_transfer_size$/ )    { $value_short = "kB/sec";   $value_long = "kBytes/sec"; $val = 1; }

  if ( $item =~ m/resp_t_vlun/ && $st_type eq "3PAR" ) {
    $value_short = "msec";
    $value_long  = "Response time in msec";
    $val         = 1;
    $suffix      = "rrs";
    $item        = "resp_t";
    $type_lun    = 1;
  } ## end if ( $item =~ m/resp_t_vlun/...)
  if ( $item =~ m/resp_t_r_vlun/ && $st_type eq "3PAR" ) {
    $value_short = "msec";
    $value_long  = "Response time in msec";
    $val         = 1;
    $suffix      = "rrs";
    $item        = "resp_t_r";
    $type_lun    = 1;
  } ## end if ( $item =~ m/resp_t_r_vlun/...)
  if ( $item =~ m/resp_t_w_vlun/ && $st_type eq "3PAR" ) {
    $value_short = "msec";
    $value_long  = "Response time in msec";
    $val         = 1;
    $suffix      = "rrs";
    $item        = "resp_t_w";
    $type_lun    = 1;
  } ## end if ( $item =~ m/resp_t_w_vlun/...)

  if ( $st_type =~ m/^DS8K$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^resp_t_r$/
      || $item =~ m/^resp_t_w$/
      || $item =~ m/^r_cache_usage$/
      || $item =~ m/^r_cache_hit$/
      || $item =~ m/^w_cache_hit$/
      || $item =~ m/^w_cache_usage$/
      || $item =~ m/^read_io$/
      || $item =~ m/^write_io$/
      || $item =~ m/^read_io_b$/
      || $item =~ m/^write_io_b$/
      || $item =~ m/^read_b$/
      || $item =~ m/^write_b$/
      || $item =~ m/^resp_t_r_b$/
      || $item =~ m/^resp_t_w_b$/ )
    {
      # those are stored in .rrc files
      $suffix = "rrc";
    } ## end if ( $item =~ m/^resp_t_r$/...)
  } ## end if ( $st_type =~ m/^DS8K$/...)

  if ( $st_type =~ m/^SWIZ$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^r_cache_usage$/
      || $item =~ m/^r_cache_hit$/
      || $item =~ m/^w_cache_hit$/
      || $item =~ m/^w_cache_usage$/
      || $item =~ m/^total_usage$/
      || $item =~ m/^r_transfer_size$/
      || $item =~ m/^w_transfer_size$/ )
    {
      # those are stored in .rrc files
      $suffix = "rrc";
    } ## end if ( $item =~ m/^r_cache_usage$/...)
  } ## end if ( $st_type =~ m/^SWIZ$/...)

  if ( $st_type =~ m/^DS8K$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^read$/ || $item =~ m/^write$/ ) {

      # data is stored in wrong RRDTool type (GAUGE instead of ABSOLUTE)
      # this do data conversion
      $val = $step;    # do not use $val=1024
    } ## end if ( $item =~ m/^read$/...)
  } ## end if ( $st_type =~ m/^DS8K$/...)

  my $comment = "$item [$value_short]";
  $comment =~ s/%/percent/g;
  $comment =~ s/r_cache_//g;
  $comment =~ s/w_cache_//g;
  $comment =~ s/r_transfer_//g;
  $comment =~ s/w_transfer_//g;
  $comment = sprintf( "%-38s", "$comment" );

  my $type_text        = $type;
  my $type_text_legend = "Volume";
  if ( $st_type eq "VMAX" && $type eq "VOLUME" ) {
    $type_text        = "STORAGE GROUP";
    $type_text_legend = "Storage group";
  }

  my $xgrid_all = xgrid_set( $time, $detail );
  my $text      = text_set($time);
  my $header    = "$type_text : $name : $item last $text";
  if ( $detail == 2 ) {
    $header = "$name : $item $text";
  }

  my $start_time = "now-1$time";
  my $end_time   = "now-1$time+1$time";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # Historical report detected
      $start_time = $start_unix;
      $end_time   = $end_unix;

      #$xgrid ="COMMENT: "; # just a trick to place there something otherwise rrdtool complains
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header    = "$type : $name : $item : $start_human - $end_human";
      $text      = "";
      $xgrid_all = "--alt-y-grid";
      if ( $detail ne "9" && $detail ne "8" ) {
        $font_def  = $font_def_hist;
        $font_tit  = $font_tit_hist;
      }
      $req_time  = $start_unix;
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my $i   = 0;
  my $j   = 0;
  my $cmd = "";

  my $vertical_label = "--vertical-label=\\\"$value_long\\\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";
    $no_legend      = "--no-legend";
  }

  #if ($rrd_legend eq "table" && $detail < 1) {
  #  $no_legend = "--no-legend";
  #}

  # new zooming function without rrdtool title in graphs
  if ( $detail eq "9" || $detail eq "8" ) { $header = ""; }

  $cmd .= "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start \\\"$start_time\\\"";
  $cmd .= " --end \\\"$end_time\\\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $xgrid_all";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " COMMENT:\\\"$type_text_legend $comment  Avg         Max\\l\\\"";

  my $legend_heading = "$type_text_legend $delimiter $comment $delimiter Avg $delimiter Max ";

  my $gtype = "AREA";
  if ( $item =~ m/resp_t/ || $item =~ m/cache_hit/ || $item =~ m/r_transfer_size/ || $item =~ m/w_transfer_size/ ) {
    $gtype = "LINE1";
  }
  if ( $st_type !~ m/^DS8K$/ ) {

    # only DS8K must be stacked due to multivolumes
    $gtype = "LINE1";
  }

  my $col_indx  = 1;    # on purpose to start with the blue one
  my $line_indx = 0;    # place enter evry 3rd line

  # read volume cfg files with grouped volumes
  # translation volume names to ID(s)
  my $type_low = lc $type;
  if ( $type eq "VVOL" ) {
    $type_low = "volume";
  }
  my $file_cfg = "$basedir/data/$host/$type/" . $type_low . "s.cfg";
  open( FHR, "< $file_cfg" ) || error( "file does not exists : $file_cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files     = <FHR>;
  my @files_org = @files;
  close(FHR);

  my $vols      = "";
  my $vol_found = 0;
  foreach my $volg (@files) {
    chomp($volg);
    $vols = $volg;
    $volg =~ s/ : .*//;
    if ( $volg =~ m/^$lpar$/ || $volg eq $lpar ) {
      $vol_found++;
      last;
    }
  } ## end foreach my $volg (@files)
  if ( $vol_found == 0 ) {
    my $alias = alias( $type, $host, $lpar );
    if ( $alias eq '' ) {
      error( "Could not find volume ID for $host:$lpar in $file_cfg neither any alias in etc/alias.cfg " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    # search volume ID again after realiasing
    foreach my $volg (@files_org) {
      chomp($volg);
      $vols = $volg;
      $volg =~ s/ : .*//;
      if ( $volg =~ m/^$alias$/ ) {
        $vol_found++;
        last;
      }
    } ## end foreach my $volg (@files_org)
  } ## end if ( $vol_found == 0 )
  if ( $vol_found == 0 ) {
    error( "Could not find volume ID for $host:$lpar in $file_cfg neither any alias in etc/alias.cfg " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  $vols =~ s/^.* : //;    # it contains a list of volumes ID(s)

  my @vol_array = split( ';', $vols );

  #print OUT "009 vols:$vols\n";
  foreach my $lpar_file (@vol_array) {
    $lpar_file =~ s/^0x//;
    $lpar_file =~ s/ //;

    if ( $lpar_file eq '' ) {
      next;
    }

    #print OUT "019 $basedir/data/$host/$type/$lpar_file.rrd  $lpar\n";
    # go every each volume for particular group

    my $file = "$basedir/data/$host/$type/$lpar_file.$suffix";
    if ( $st_type =~ m/XIV/ || $st_type =~ m/DS5K/ || ( $st_type =~ m/3PAR/ && $type eq "VVOL" ) ) {

      # XIV volumes contain pool_id in their names: 00273763-P102932.rrd
      my $vol_update_last = 0;
      foreach my $file_xiv (<$wrkdir/$host/$type/$lpar_file-P*\.$suffix>) {
        my $vol_upd_time = ( stat("$file_xiv") )[9];
        if ( $vol_upd_time > $vol_update_last ) {
          $file            = $file_xiv;
          $vol_update_last = $vol_upd_time;
        }
      } ## end foreach my $file_xiv (<$wrkdir/$host/$type/$lpar_file-P*\.$suffix>)
      if ( $vol_update_last == 0 ) {

        #volume has not been found,m skipping it
        next;
      }
    } ## end if ( $st_type =~ m/XIV/...)

    if ( !-f "$file" ) {
      error( "Could not find volume file: $file " . __FILE__ . ":" . __LINE__ );
      next;
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$file") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    #print OUT "020 $basedir/data/$host/$type/$lpar_file.rrd  $lpar\n";
    my $lpar_space = $lpar;
    $lpar_space =~ s/:/\\:/g;

    # add spaces to volume name to have 15 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    if ( $st_type =~ m/^DS8K/ || $st_type =~ m/^SWIZ$/ ) {
      $lpar_space = sprintf( "%-38s", "$lpar_space [$lpar_file]" );
    }
    else {
      # DS5k has very long IDs and does not use multivolumes at all
      $lpar_space = sprintf( "%-38s", "$lpar_space" );
    }
    $file =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    #print OUT "020 $basedir/data/$host/$type/$lpar_file.rrd  $lpar\n";
    # bulid RRDTool cmd
    if ($st_type eq "SWIZ" && $type eq "VOLUME" && $item eq "r_transfer_size"){
      $cmd .= " DEF:value${i}=\\\"$file\\\":res1:AVERAGE";
    }
    else{
      if ($st_type eq "SWIZ" && $type eq "VOLUME" && $item eq "w_transfer_size"){
        $cmd .= " DEF:value${i}=\\\"$file\\\":res2:AVERAGE";
      }
      else{
        $cmd .= " DEF:value${i}=\\\"$file\\\":$item:AVERAGE";
      }
    }
    $cmd .= " CDEF:valmb${i}=value${i},$val,/";
    $cmd .= " $gtype:valmb${i}$color[$col_indx]:\\\"$lpar_space\\\"";
    $cmd .= " PRINT:valmb${i}:AVERAGE:\\\"%8.1lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$col_indx]\\\"";
    $cmd .= " PRINT:valmb${i}:MAX:\\\" %8.1lf $delimiter\\\"";
    $col_indx++;
    if ( $col_indx > $color_max ) {
      $col_indx = 0;
    }

    $cmd .= " GPRINT:valmb${i}:AVERAGE:\\\"%8.1lf \\\"";
    $cmd .= " GPRINT:valmb${i}:MAX:\\\" %8.1lf \\\"";

    # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
    if ( $item !~ m/resp_t/ && $item !~ m/cache_hit/ ) {
      if ( $st_type =~ m/^DS8K$/ ) {

        # only DS8K must be stacked due to multivolumes
        $gtype = "STACK";
      }
    } ## end if ( $item !~ m/resp_t/...)
    $i++;
    $cmd .= " COMMENT:\\\"\\l\\\"";
  } ## end foreach my $lpar_file (@vol_array)

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  RRDp::cmd qq($cmd);
  my $ret = RRDp::read;
  if ( $$ret =~ "ERROR" ) {
    error( "$host : Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  $png_end_heading = ret_graph_param($ret);
  return 1;
} ## end sub create_graph_volume

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  my $basedir  = $ENV{INPUTDIR};

  #print "ERROR          : $text : $!\n";
  #print STDERR "$act_time: $text : $!\n";
  print STDERR "$act_time: $host:$type:$name:$item:$detail: $text \n";

  print "\n";    # print missing enter to finish header HTTP

  # print error picture to check error-cgi.log or web error log
  if ( $text =~ /ACL: not allowed/ && -f "$basedir/html/acl_error.png" ) {
    open( PNG, "< $basedir/html/acl_error.png" ) || error("Cannot open  $basedir/html/acl_error.png: $!".__FILE__.":".__LINE__) && return 0;
  }
  else {
    open( PNG, "< $basedir/html/error.png" ) || error( "Cannot open  $basedir/html/error.png: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  binmode(PNG);
  while ( read( PNG, $b, 4096 ) ) {
    print "$b";
  }
  close(PNG);
} ## end sub error

sub create_graph_summ {
  my $host       = shift;
  my $type       = shift;
  my $name       = shift;
  my $item       = shift;
  my $time       = shift;
  my $name_out   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $st_type    = shift;
  my $text       = "";
  my $updated    = "";      #sending to clickable legend


  my $xgrid = xgrid_set( $time, $detail );

  if ( $time !~ m/d/ ) {

    # slow down all graphs except of the daily one to let create the daily one as the first
    select( undef, undef, undef, 0.2 );
  }

  my $tmp_file = "$tmp_dir/$host/$type-$name-$time.cmd";

  if ( $st_type eq "3PAR" && $type eq "LUN" ) {
    $type = "VOLUME";
  }

  my $rank_vol = "POOL";

  my $header     = "";
  my $start_time = "now-1$time";
  my $end_time   = "now-1$time+1$time";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header   = "$start_human - $end_human";
      $text     = "";
      $xgrid    = "--alt-y-grid";
      $font_def = "--font=DEFAULT:10:";
      $font_tit = "--font=TITLE:13:";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  if ( $item =~ m/^custom/ ) {

    # custom groups
    if ( $item =~ /^custom-group-.*-trend$/ ) {
      $no_legend = "";
      $end_time  = "now+1$time";
    }

    $item =~ s/^custom-group-//;
    $tmp_file = "$tmp_dir/$host/$name-$item-$time.cmd";
  } ## end if ( $item =~ m/^custom/)

  if ( $name eq "sum_io_trend" || $name eq "sum_io_total_trend" || $name eq "sum_data_trend" || $name eq "sum_data_total_trend" || $name eq "used_trend" ) {
    $end_time = "now+1$time";
  }

  # replace graph size and total value
  open( FH, "< $tmp_file" ) || error( "sum detail: $host:$type:$item:$time: Can't open $tmp_file " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);

  my $title;
  if ( $header != "" ) {
    ( undef, $title ) = split( "--", $lines[0] );
    $title =~ s/last .*/$header\"/;
    $lines[0] =~ s/--title.*--start/--$title --start/;
  }

  my $lines_no = 0;
  my $cmd      = "";

  foreach my $line (@lines) {

    if ( $line !~ m/DEF:/ ) {
      next;    # some problem, probably no DEF input files
    }
    $lines_no = 1;

    # get last update of the first RRDTool file
    my ( $trash, $source_rrd ) = split( / DEF:/, $line );
    $source_rrd =~ s/:.*//g;
    $source_rrd =~ s/^.*=//g;
    chomp($source_rrd);
    my $last_update = rrd_last_update( $source_rrd, $type, "$basedir/data/$host/$rank_vol/" );
    if ( $last_update eq '' ) {
      $line =~ s/ COMMENT:\"Updated.* HRULE:/ HRULE:/;
    }
    else {
      $updated = "Updated $last_update";
      $last_update =~ s/:/\\:/g;
      my $test_res = $line =~ s/ COMMENT:\"Updated.* HRULE:/ COMMENT:\"Updated $last_update\" HRULE:/;
      if ( !$test_res ) { $updated = "" }
      ;    #no subs -> no Updated
    } ## end else [ if ( $last_update eq '')]

    $line =~ s/graph .* --title/$graph_cmd \"$name_out\" --title/;
    $line =~ s/--start now.*--imgformat/--start $start_time --end $end_time $no_legend --imgformat /;
    $line =~ s/=====space=====/ /g;    # 1.24-3 fix
    if ( $detail == 1 ) {
      $line =~ s/ --step=3600/ --step=$STEP /;
      $line =~ s/ --step=86400/ --step=$STEP /;
      $line =~ s/ --font=DEFAULT:[0-9]:/ /;
      $line =~ s/ --font=TITLE:[0-9]:/ /;

      #$line =~ s/Total                                                .*\\l\"/Total                                                $total\\l\"/;
      $line =~ s/ --width=400/ --width=$width $font_def_popup /;
      $line =~ s/ --height=150/ --height=$height $font_tit_popup /;
      $line =~ s/ --x-grid=.* --alt-y-grid/ $xgrid --alt-y-grid/;
      $line =~ s/ $delimiter +$st_type//g;
      $line =~ s/$delimiter//g;
    } ## end if ( $detail == 1 )
    if ( $detail == 2 ) {
      $line =~ s/ --font=DEFAULT:[0-9]:/ /;
      $line =~ s/ --font=TITLE:[0-9]:/ /;
      $line =~ s/ --width=400/ --width=$width $font_def_dashbo  /;
      $line =~ s/ --height=150/ --height=$height $font_tit_dashbo $no_legend /;
      $line =~ s/ : last / : /;
      $line =~ s/ last / /;
      $line =~ s/ aggregated / /;

      #$line =~ s/ aggregated / agg /;
      $line =~ s/ 4 weeks/ month/;
      $line =~ s/ --x-grid=.* --alt-y-grid/ $xgrid --alt-y-grid/;
      $line =~ s/ --vertical-label=.* --units-exponent/ --units-exponent /;
      $line =~ s/ --vertical-label=.* --interlaced/ --interlaced /;
      $line =~ s/--title "POOL /--title "/;
      $line =~ s/--title "RANK /--title "/;
      $line =~ s/--title "PORT /--title "/;
      $line =~ s/--title "HOST /--title "/;
      $line =~ s/--title "MDISK /--title "/;
      $line =~ s/--title "DRIVE /--title "/;
      $line =~ s/--title "VOLUME /--title "/;
      $line =~ s/--title "SANPORT /--title "/;
      $line =~ s/--title "CPU-NODE /--title "/;
      $line =~ s/ $delimiter +$st_type//g;
      $line =~ s/$delimiter//g;
    } ## end if ( $detail == 2 )
    # new zooming function without rrdtool title in graphs
    if ( $detail == 9 || $detail == 8 ) {
      $line =~ s/ --title .* --start/ --title \"\" --start /;
      $line =~ s/ --x-grid=.* --alt-y-grid/ $xgrid --alt-y-grid/;
    }
    if ($detail == 8 ) {
      $line =~ s/ --width=400/ --width=$width/;
      $line =~ s/ --height=150/ --height=$height/;
    }


    #print STDERR "$line\n\n\n";
    $cmd = $line;
  } ## end foreach my $line (@lines)

  if ( $lines_no == 0 ) {
    no_data_yet( "cmd file does not contain data source files: $tmp_file " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  RRDp::cmd qq($cmd);
  my $ret = RRDp::read;
  if ( $$ret =~ "ERROR" ) {
    error( "sum detail: Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  $png_end_heading = ret_graph_param( $ret, $updated );

  #print STDERR "-- $png_end_heading\n";
  return 1;
} ## end sub create_graph_summ

sub create_graph_pool {
  my $host       = shift;
  my $type       = shift;
  my $lpar       = shift;
  my $item       = shift;
  my $time       = shift;
  my $name_out   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $wrkdir     = shift;
  my $act_time   = shift;
  my $step       = shift;
  my $st_type    = shift;
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $last       = "COMMENT: ";
  my $act_time   = localtime();
  my $act_time_u = time();
  my $req_time   = 0;
  my $itemm      = "";

  #print STDERR "$host,$type,$lpar,$item\n";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;

  if ( $item =~ m/^data_rate$/ )   { $value_short = "MB"; $value_long = "MBytes";                $val = 1024; }
  if ( $item =~ m/^data_rate_b$/ ) { $value_short = "MB"; $value_long = "data_rate_b in MBytes"; $val = 1024; }
  if ( $item =~ m/^read$/ )        { $value_short = "MB"; $value_long = "MBytes";                $val = 1024; }
  if ( $item =~ m/^read_b$/ )      { $value_short = "MB"; $value_long = "MBytes";                $val = 1024; }
  if ( $item =~ m/^write$/ )       { $value_short = "MB"; $value_long = "MBytes";                $val = 1024; }
  if ( $item =~ m/^write_b$/ )     { $value_short = "MB"; $value_long = "MBytes";                $val = 1024; }
  if ( $item =~ m/^io_rate$/ )    { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^io_rate_b$/ )  { $value_short = "IOPS"; $value_long = "io_rate_b in IO per second"; }
  if ( $item =~ m/^read_io$/ )    { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^read_io_b$/ )  { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^write_io$/ )   { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^write_io_b$/ ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^resp_t$/ )     { $value_short = "ms";   $value_long = "milliseconds"; $val = 1; }
  if ( $item =~ m/^resp_t_r$/ )   { $value_short = "ms";   $value_long = "milliseconds"; $val = 1; }
  if ( $item =~ m/^resp_t_r_b$/ ) { $value_short = "ms";   $value_long = "milliseconds"; $val = 1; }
  if ( $item =~ m/^resp_t_w$/ )   { $value_short = "ms";   $value_long = "milliseconds"; $val = 1; }
  if ( $item =~ m/^resp_t_w_b$/ ) { $value_short = "ms";   $value_long = "milliseconds"; $val = 1; }
  if ( $item =~ m/^tier0$/ )      { $value_short = "TB";   $value_long = "TBytes"; }
  if ( $item =~ m/^tier0-nl$/ )   { $value_short = "TB";   $value_long = "TBytes"; }
  if ( $item =~ m/^tier1$/ )      { $value_short = "TB";   $value_long = "TBytes"; }
  if ( $item =~ m/^tier2$/ )      { $value_short = "TB";   $value_long = "TBytes"; }
  if ( $item =~ m/compress/ )     { $value_short = "TB";   $value_long = "TBytes"; }
  if ( $item =~ m/^used$/ )       { $value_short = "TB";   $value_long = "TBytes"; }
  if ( $item =~ m/^alloc$/ )      { $value_short = "TB";   $value_long = "TBytes"; }
  if ( $item =~ m/^real$/ )       { $value_short = "TB";   $value_long = "TBytes"; }
  if ( $item =~ m/^qdepth$/ )     { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^busy$/ )       { $value_short = "%";    $value_long = "CPU utilization in %"; }
  if ( $item =~ m/^r_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; $val = 1; }
  if ( $item =~ m/^w_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; $val = 1; }
  if ( $item =~ m/^io_pending$/ ) { $value_short = "IOPS"; $value_long = "IO per second"; }

  my $name_text = $name;

  if ( $st_type =~ m/^DS5K$/ || $st_type =~ m/^DS8K$/ ) {
    if ( $item =~ m/^read$/ || $item =~ m/^write$/ ) {
      $val = 1;
    }
  }

  my $pool_file_name = "pool.cfg";
  if ( $st_type =~ m/^VMAX$/ && $type eq "TIER" ) {
    $pool_file_name = "tier.cfg";
  }

  if ( $st_type ne "VMAX" && $type ne "DRIVE" ) {

    # POOL name translation --> from ID
    open( FHR, "< $wrkdir/$host/$pool_file_name" ) || error( "Can't open $wrkdir/$host/$pool_file_name : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @lines_translate = <FHR>;
    close(FHR);
    my $pool_found = 0;
    foreach my $linep (@lines_translate) {
      chomp($linep);
      ( my $id, my $name_pool ) = split( /:/, $linep );

      #print STDERR "001 $id : $name : $name_pool : $lpar\n";
      if ( $name =~ m/^$id$/ ) {
        $name_text = $name_pool;
        $lpar      = $id;
        $pool_found++;
        last;
      } ## end if ( $name =~ m/^$id$/)
      if ( $name =~ m/^$name_pool$/ && $st_type ne "DS5K" ) {
        $name_text = $name_pool;
        $lpar      = $id;
        $pool_found++;
        last;
      } ## end if ( $name =~ m/^$name_pool$/...)
    } ## end foreach my $linep (@lines_translate)
    if ( $pool_found == 0 ) {
      my $alias = alias( $type, $host, $lpar );
      if ( $alias eq '' ) {
        error( "Could not find pool ID for $host:$lpar in $basedir/data/$host/pool.cfg neither any alias in etc/alias.cfg " . __FILE__ . ":" . __LINE__ );
        return 0;
      }
      foreach my $linep (@lines_translate) {
        chomp($linep);
        ( my $id, my $name_pool ) = split( /:/, $linep );
        if ( $alias =~ m/^$id$/ ) {
          $name_text = $lpar;
          $name      = $alias;
          $lpar      = $id;
          $pool_found++;
          last;
        } ## end if ( $alias =~ m/^$id$/)
      } ## end foreach my $linep (@lines_translate)
    } ## end if ( $pool_found == 0 )
    if ( $pool_found == 0 ) {
      error( "Could not find pool ID for $host:$lpar in $basedir/data/$host/pool.cfg neither any alias in etc/alias.cfg " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    #print STDERR "002 name:$name name_text:$name_text lpar:$lpar\n";
  } ## end if ( $st_type ne "VMAX"...)

  my $name_space = $name_text;
  $name_space = sprintf( "%-38s", "$name_space" );

  my $comment = "$item [$value_short]";
  if ( $item =~ m/^real$/ ) {
    $comment = "used [$value_short]";
  }
  $comment =~ s/%/percent/g;
  $comment = sprintf( "%-10s", "$comment" );

  if ( $time =~ m/d/ ) {
    $req_time = $act_time_u - 86400;
  }
  if ( $time =~ m/w/ ) {
    $req_time = $act_time_u - 604800;
  }
  if ( $time =~ m/m/ ) {
    $req_time = $act_time_u - 2764800;
  }
  if ( $time =~ m/y/ ) {
    $req_time = $act_time_u - 31536000;
  }

  my $type_text = $type;
  if ( $st_type eq "VMAX" && $type eq "DRIVE" ) {
    $type_text = "DISK GROUP";
  }

  my $xgrid_all = xgrid_set( $time, $detail );
  my $text      = text_set($time);
  my $header    = "$type_text : $name_text : $item last $text";
  if ( $item =~ m/^real$/ ) { $header = "$type_text : $name_text : used last $text"; }
  if ( $detail == 2 ) {
    $header = "$name_text : $item $text";
  }

  my $start_time = "now-1$time";
  my $end_time   = "now-1$time+1$time";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # Historical report detected
      $start_time = $start_unix;
      $end_time   = $end_unix;
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header    = "$type : $name_text : $item : $start_human - $end_human";
      $text      = "";
      $xgrid_all = "--alt-y-grid";
      if ( $detail ne "9" && $detail ne "8" ) {
        $font_def  = $font_def_hist;
        $font_tit  = $font_tit_hist;
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my $i   = 0;
  my $j   = 0;
  my $cmd = "";

  my $vertical_label = "--vertical-label=\\\"$value_long\\\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";
    $no_legend      = "--no-legend";

    # $vertical_label="COMMENT: "; --> no, no, some versions does not like comment wityhout anything
  } ## end if ( $detail == 2 )

  # new zooming function without rrdtool title in graphs
  if ( $detail == 9 || $detail == 8 ) { $header = ""; }

  $cmd .= "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start \\\"$start_time\\\"";
  $cmd .= " --end \\\"$end_time\\\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $xgrid_all";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " COMMENT:\\\"Pool   $comment                               Avg        Max\\l\\\"";

  my $gtype          = "LINE";
  my $col_indx       = 0;
  my $line_indx      = 0;        # place enter evry 3rd line
  my $nothing_to_sum = 0;

  my $rank_vol = "RANK";

  my $file_list = "$rank_vol/*-P$lpar\.rrd";

  if ( $st_type =~ m/3PAR/ || $st_type =~ m/XIV/ || $st_type =~ m/DS5K/ || $st_type =~ m/HUS/ || $st_type =~ m/NETAPP/ || $st_type =~ m/VSPG/ || $st_type =~ m/^VNX$/ || $st_type =~ m/^VNXF$/ || $st_type =~ m/UNITY/ || $st_type =~ m/^VNXE$/ || $st_type =~ m/^DOTHILL$/ || $st_type =~ m/^VMAX$/ || $st_type =~ m/^COMPELLENT$/ || $st_type =~ m/^VPLEX$/ || $st_type eq "OCEANSTOR" || $st_type eq "PURE" ) {
    $rank_vol  = "POOL";
    $file_list = "$rank_vol/$lpar\.rrd";
  }
  if ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ ) {
    $rank_vol  = "VOLUME";
    $file_list = "$rank_vol/*-P$lpar\.rrd";
  }
  if ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ ) {
    $rank_vol  = "VOLUME";
    $file_list = "$rank_vol/*-P$lpar\.rrd";
  }

  if ( $item =~ m/tier/ ) {

    # due to tier capacity files : POOL/0-cap.rrd
    $file_list = "POOL/$lpar-cap\.rrd";
    $gtype     = "AREA";
  } ## end if ( $item =~ m/tier/ )
  if ( $item =~ m/tier/ && $st_type eq "OCEANSTOR" ) {
    $file_list = "POOL/$lpar\.rrd";
  }
  if ( $st_type =~ m/^SWIZ$/ && $item =~ m/tier0-nl$/ ) {

    # due to tier capacity files : POOL/0-nl-cap.rrd
    $file_list = "POOL/$lpar-nl-cap\.rrd";
    $gtype     = "AREA";
  } ## end if ( $st_type =~ m/^SWIZ$/...)
  if ( $st_type =~ m/^SWIZ$/ && $item =~ m/compress/ ) {

    # due to tier capacity files : POOL/0-comp.rrd
    $file_list = "POOL/$lpar-comp\.rrd";
    $gtype     = "AREA";
  } ## end if ( $st_type =~ m/^SWIZ$/...)

  if ( $item =~ m/real/ ) {
    $gtype = "AREA";
  }

  if ( ( $st_type =~ m/^NETAPP$/ || $st_type =~ m/3PAR/ || $st_type =~ m/^DS5K$/ || $st_type =~ m/^VNX$/ || $st_type =~ m/UNITY/ || $st_type =~ m/VNXE$/ || $st_type =~ m/DOTHILL$/ ) && $item =~ m/used/ ) {

    # due to tier capacity files : POOL/0-cap.rrd
    $file_list = "POOL/$lpar-cap\.rrd";
    $gtype     = "AREA";
  } ## end if ( ( $st_type =~ m/^NETAPP$/...))
  if ( $st_type =~ m/^VMAX$/ && $type eq "POOL" && ( $item =~ m/used/ || $item =~ m/alloc/ ) ) {
    $file_list = "POOL/$lpar-cap\.rrd";
    $gtype     = "AREA";
  }
  if ( $st_type =~ m/^VMAX$/ && $type eq "TIER" ) {
    $rank_vol  = "TIER";
    $file_list = "$rank_vol/$lpar\.rrd";
  }
  if ( $st_type =~ m/^VMAX$/ && $type eq "TIER" && ( $item =~ m/used/ || $item =~ m/alloc/ ) ) {
    $file_list = "TIER/$lpar-cap\.rrd";
    $gtype     = "AREA";
  }
  if ( $st_type =~ m/^VMAX$/ && $item =~ m/used/ && $type eq "DRIVE" ) {
    $file_list = "DRIVE/$lpar\.rrd";
    $gtype     = "AREA";
  }

  if ( $st_type =~ m/^DS8K$/ || $st_type =~ m/^SWIZ/ ) {
    if ( $item =~ m/^io_rate$/ || $item =~ m/^data_rate$/ || $item =~ m/^read_io$/ || $item =~ m/^write_io$/ || $item =~ m/^read$/ || $item =~ m/^write$/ || $item =~ m/^resp_t_r$/ || $item =~ m/^resp_t_w$/ ) {

      # Front-end data are stored in POOL dir (based on Volume data) like POOL/0.rrd
      $file_list = "POOL/$lpar\.rrd";
    }
    else {
      # delete _b suffix, front-end data is in POOL dir and back-end in RANK, no any suffix is there
      $item =~ s/_b$//;
    }
  } ## end if ( $st_type =~ m/^DS8K$/...)

  my $time_first = find_real_data_start_pool( "$basedir/data/$host/$rank_vol", $lpar );

  # following loop is copied from storage.pl : draw_graph_sum ()
  my $host_space = check_space_name($host);
  foreach my $line (<$basedir/data/$host_space/$file_list>) {
    chomp($line);

    # avoid old lpars which do not exist in the period
    # No no here, here it is detail which should always work, it do not save anything here ...
    #my $rrd_upd_time = (stat("$line"))[9];
    #if ( $rrd_upd_time < $req_time ) {
    #  next;
    #}

    $nothing_to_sum = 0;

    my @link_l = split( /\//, $line );
    my $lpar = "";
    foreach my $m (@link_l) {
      $lpar = $m;
    }
    $lpar =~ s/\.rrd$//;
    $lpar =~ s/-P.*//;    # filter pool info
    $lpar =~ s/PORT-//;
    $lpar =~ s/RANK-//;
    $lpar =~ s/VOLUME-//;
    $lpar =~ s/0x//;
    $lpar =~ s/-cap//;
    $lpar =~ s/-comp//;

    my $lpar_space = $lpar;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar_space = sprintf( "%-8s", "$lpar_space" );

    # bulid RRDTool cmd
    $itemm = $item . "m";

    #$cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
    # do not use this UNKN as when 1 is UN then all is UNKN
    # same way it is done in storage.pl
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    if ( $item =~ m/sum_io/ || $item =~ m/^io_rate$/ ) {
      if ( $st_type =~ m/^DS5K$/ || $st_type =~ m/^VPLEX$/ ) {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":io_rate:AVERAGE";
        $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r=$item${i}_r,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      }
      else {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":read_io:AVERAGE";
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":write_io:AVERAGE";

        # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
        #$cmd .= " CDEF:$item${i}_r=$item${i}_r_u,UN,0,$item${i}_r_u,IF";
        #$cmd .= " CDEF:$item${i}_w=$item${i}_w_u,UN,0,$item${i}_w_u,IF";
        $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r=$item${i}_r,$val,/";                                                        # convert into MB if necessary, normaly is there 1
        $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      } ## end else [ if ( $st_type =~ m/^DS5K$/...)]
    } ## end if ( $item =~ m/sum_io/...)
    if ( $item =~ m/sum_data/ || $item =~ m/^data_rate$/ ) {
      if ( $st_type =~ m/^DS5K$/ ) {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":data_rate:AVERAGE";
        $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r=$item${i}_r,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      }
      else {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":read:AVERAGE";
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":write:AVERAGE";

        # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
        #$cmd .= " CDEF:$item${i}_r=$item${i}_r_u,UN,0,$item${i}_r_u,IF";
        #$cmd .= " CDEF:$item${i}_w=$item${i}_w_u,UN,0,$item${i}_w_u,IF";
        $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r=$item${i}_r,$val,/";                                                        # convert into MB if necessary, normaly is there 1
        $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      } ## end else [ if ( $st_type =~ m/^DS5K$/)]
    } ## end if ( $item =~ m/sum_data/...)
    if ( $item !~ m/^used$/ && $item !~ m/tier/ && $item !~ m/sum_cap/ && $item !~ m/sum_data/ && $item !~ m/sum_io/ && $item !~ m/^io_rate$/ && $item !~ m/^data_rate$/ && $item !~ m/^real$/ && $item !~ m/alloc/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":$item:AVERAGE";

      # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
      #$cmd .= " CDEF:$item${i}_r=$item${i}_r_u,UN,0,$item${i}_r_u,IF";
      $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_r,$val,/";                                                        # convert into MB if necessary, normaly is there 1
    } ## end if ( $item !~ m/^used$/...)
    if ( $item =~ m/tier0$/ && $st_type ne "OCEANSTOR" ) {
      $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":tier0cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier0free:AVERAGE";
      $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_c,$item${i}_w,-";
    } ## end if ( $item =~ m/tier0$/...)
    if ( $item =~ m/tier0$/ && $st_type eq "OCEANSTOR" ) {
      $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":tier0cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier0free:AVERAGE";
      $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_c";
    } ## end if ( $item =~ m/tier0$/...)
    if ( $item =~ m/tier0-nl/ ) {
      $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":total:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":free:AVERAGE";
      $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_c,$item${i}_w,-";
    } ## end if ( $item =~ m/tier0-nl/)

    if ( $item =~ m/tier1$/ && $st_type eq "OCEANSTOR" ) {
      $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":tier1cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier1free:AVERAGE";
      $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_c";
    } ## end if ( $item =~ m/tier1$/...)
    if ( $item =~ m/tier1/ && $st_type ne "OCEANSTOR" ) {
      $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":tier1cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier1free:AVERAGE";
      $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_c,$item${i}_w,-";
    } ## end if ( $item =~ m/tier1/...)
    if ( $item =~ m/tier2/ && $st_type ne "OCEANSTOR" ) {
      $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":tier2cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier2free:AVERAGE";
      $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_c,$item${i}_w,-";
    } ## end if ( $item =~ m/tier2/...)
    if ( $item =~ m/tier2$/ && $st_type eq "OCEANSTOR" ) {
      $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":tier2cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier2free:AVERAGE";
      $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_c";
    } ## end if ( $item =~ m/tier2$/...)
    if ( $item =~ m/^used$/ ) {
      if ( $st_type =~ m/^DS5K$/ ) {
        $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":$item:AVERAGE";
        $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      }
      if ( $st_type =~ m/^VNX$/ || $st_type =~ m/UNITY/ || $st_type =~ m/VNXE$/ || $st_type =~ m/DOTHILL$/ || $st_type =~ m/3PAR/ ) {
        $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":used:AVERAGE";
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":free:AVERAGE";
        $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
        $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
        $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      } ## end if ( $st_type =~ m/^VNX$/...)
      if ( $st_type =~ m/^VMAX$/ && $type ne "DRIVE" ) {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":used:AVERAGE";
        $cmd .= " DEF:$item${i}_t_u=\\\"$line\\\":total:AVERAGE";
        $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_t=TIME,$time_first,LT,$item${i}_t_u,$item${i}_t_u,UN,0,$item${i}_t_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_t,$itemm${i}_r,-";
      } ## end if ( $st_type =~ m/^VMAX$/...)
      if ( $st_type =~ m/^VMAX$/ && $type eq "DRIVE" ) {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":cap_used:AVERAGE";
        $cmd .= " DEF:$item${i}_t_u=\\\"$line\\\":cap:AVERAGE";
        $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_t=TIME,$time_first,LT,$item${i}_t_u,$item${i}_t_u,UN,0,$item${i}_t_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_t,$itemm${i}_r,-";
      } ## end if ( $st_type =~ m/^VMAX$/...)
      if ( $st_type ne "DS5K" && $st_type ne "VNX" && $st_type ne "UNITY" && $st_type ne "VNXE" && $st_type ne "DOTHILL" && $st_type ne "VMAX" && $st_type ne "3PAR" ) {
        $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":free:AVERAGE";
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":real:AVERAGE";
        $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
        $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
        $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
        $cmd .= " CDEF:$itemm${i}_r=$item${i}_c,$item${i}_w,-";
      } ## end if ( $st_type ne "DS5K"...)
    } ## end if ( $item =~ m/^used$/)
    if ( $item =~ m/^alloc$/ && $st_type eq "VMAX" ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":alloc:AVERAGE";
      $cmd .= " DEF:$item${i}_t_u=\\\"$line\\\":total:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_t=TIME,$time_first,LT,$item${i}_t_u,$item${i}_t_u,UN,0,$item${i}_t_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_t,$itemm${i}_r,-";
    } ## end if ( $item =~ m/^alloc$/...)
    if ( $item =~ m/^real$/ ) {
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":free:AVERAGE";
      $cmd .= " DEF:$item${i}_c_u=\\\"$line\\\":real:AVERAGE";
      $cmd .= " CDEF:$item${i}_c=TIME,$time_first,LT,$item${i}_c_u,$item${i}_c_u,UN,0,$item${i}_c_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_c=$item${i}_c,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";                                                        # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_c";
    } ## end if ( $item =~ m/^real$/)
    $i++;
  } ## end foreach my $line (<$basedir/data/$host/$file_list>)
  if ( $nothing_to_sum == 1 ) {

    # there is nothing to sum at the moment
    return 0;
  }

  my $zero = 0;
  if ( $item =~ m/^used$/ || $item =~ m/tier/ || $item =~ m/sum_data/ || $item =~ m/sum_io/ || $item =~ m/^data_rate$/ || $item =~ m/^io_rate$/ || $item =~ m/^real$/ || $item =~ m/^alloc$/ ) {
    #
    # read
    #

    # Load all items into the $cmd
    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:rsum${i}=$itemm${zero}_r,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",$itemm${index_actual}_r,+";
    }
    my $text = "read";
    if ( $st_type =~ m/^DS5K$/ || ( $st_type =~ m/^VPLEX$/ && $item =~ m/^io_rate$/ || ( $st_type =~ m/^OCEANSTOR$/ && $item =~ m/^tier/ ) ) ) {
      $text = "Total";
    }
    my $item_col = "#00FF00";
    my $value    = "%8.1lf";
    if ( ( $item =~ m/tier/ && $st_type !~ m/^OCEANSTOR$/ ) || $item =~ m/^used$/ || $item =~ m/^real$/ ) {
      $text     = "Used";
      $item_col = "#FF3333";
      $value    = "  %6.2lf";
    }
    if ( $item =~ m/alloc/ ) {
      $text     = "Allocated";
      $item_col = "#FF3333";
      $value    = "  %6.2lf";
    }
    my $name_space = sprintf( "%-38s", "$text" );
    $cmd .= " $gtype:rsum${i}$item_col:\\\"$name_space \\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"$value \\\"";
    $cmd .= " GPRINT:rsum${i}:MAX:\\\"$value \\l\\\"";

    #
    # write
    #

    if ( $st_type !~ m/^DS5K$/ && !( $st_type eq "VPLEX" && $item eq "io_rate" ) && !( $st_type eq "OCEANSTOR" && $item =~ m/^tier/ ) ) {
      $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
      $cmd .= " CDEF:wsum${i}=$itemm${zero}_w,0,+";
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",$itemm${index_actual}_w,+";
      }
      my $item_col = "#4444FF";
      my $text     = "write";
      my $value    = "%8.1lf";
      if ( $item =~ m/tier/ || $item =~ m/^used$/ || $item =~ m/^real$/ || $item =~ m/^alloc$/ ) {
        $text     = "Free ";
        $gtype    = "STACK";
        $item_col = "#33FF33";
        $value    = "  %6.2lf";
      } ## end if ( $item =~ m/tier/ ...)

      # print legend

      my $name_space = sprintf( "%-38s", "$text" );
      $cmd .= " $gtype:wsum${i}$item_col:\\\"$name_space \\\"";
      $cmd .= " GPRINT:wsum${i}:AVERAGE:\\\"$value \\\"";
      $cmd .= " GPRINT:wsum${i}:MAX:\\\"$value \\l\\\"";
    } ## end if ( $st_type !~ m/^DS5K$/...)
  } ## end if ( $item =~ m/^used$/...)
  else {
    # usual stuff , 1 item
    # even this must be summed!! --> there is no dedicated stuff for pools, all is taken from RANK/MDISKs
    # must be summed except response time

    # Load all items into the $cmd
    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x the first item
    $cmd .= " CDEF:rsum${i}=$itemm${zero}_r,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",$itemm${index_actual}_r,+";
    }

    if ( $item =~ m/^resp/ ) {
      if ( $index_actual == 0 ) {
        error( "No pool data found : $host,$type,$lpar,$item " . __FILE__ . ":" . __LINE__ );
        return 0;
      }
      $cmd .= ",$index_actual,/";
    } ## end if ( $item =~ m/^resp/)

    my $name_space = sprintf( "%-38s", "$item" );
    $cmd .= " $gtype:rsum${i}#4444FF:\\\"$name_space \\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%8.1lf \\\"";
    $cmd .= " GPRINT:rsum${i}:MAX:\\\"%8.1lf \\l\\\"";
  } ## end else [ if ( $item =~ m/^used$/...)]

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  RRDp::cmd qq($cmd);
  my $ret = RRDp::read;
  if ( $$ret =~ "ERROR" ) {
    error( "POOL: Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  $png_end_heading = ret_graph_param($ret);
  return 1;
} ## end sub create_graph_pool

sub rrd_last_update {
  my $rrd_file  = shift;
  my $type      = shift;
  my $full_path = shift;
  my $last_time = "";

  $rrd_file =~ s/^"//;
  $rrd_file =~ s/"$//;

  # must be full path, relative path somehow does not work (perhaps rrdtool is already started in some dira which cannot be changed)
  my $rrd_file_full = $full_path . $rrd_file;

  if ( !-f $rrd_file_full ) {
    if ( !-f $rrd_file ) {

      # some problem, ignore it and do not display last update
      # error ("$rrd_file does not exist : cannot obtain the last update time ".__FILE__.":".__LINE__);
      # --> do not call error()!!!
      return "";
    } ## end if ( !-f $rrd_file )
    $rrd_file_full = $rrd_file;
  } ## end if ( !-f $rrd_file_full)
  RRDp::cmd qq(last "$rrd_file_full");
  my $last_tt = RRDp::read;
  $last_time = localtime($$last_tt);

  #$last_time =~ s/:/\:/g;

  return $last_time;
} ## end sub rrd_last_update

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  if ( $digit eq '' ) {
    return 0;
  }
  if ( $digit eq 'U' ) {
    return 1;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
} ## end sub isdigit

sub check_space_name {
  my $name       = shift;
  my $name_space = $name;
  if ( $name =~ m/ / ) {
    $name_space = "\"" . $name . "\"";    # it must be here to support space with server names
    return $name_space;
  }
  else {
    return $name;
  }
} ## end sub check_space_name

sub xgrid_set {
  my $type   = shift;
  my $detail = shift;
  my $xgrid  = "--x-grid=";

  if ( $type =~ m/d/ ) {
    if ( $detail == 0 || $detail == 9 || $detail == 8 ) {
      $xgrid .= "MINUTE:60:HOUR:2:HOUR:4:0:%H";
    }
    if ( $detail == 1 ) {
      $xgrid .= "MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    if ( $detail == 2 ) {

      # dashboard
      #$xgrid .= "MINUTE:60:HOUR:1:HOUR:1:0:%H";
      if ( $super_detail == 3 ) {
        $xgrid .= "HOUR:1:DAY:1:HOUR:2:0:%H";
      }
      else {
        $xgrid .= "HOUR:6:DAY:1:HOUR:6:0:%H";
      }
    } ## end if ( $detail == 2 )
  } ## end if ( $type =~ m/d/ )
  if ( $type =~ m/w/ ) {
    if ( $detail == 0 || $detail == 9 || $detail == 8 ) {
      $xgrid .= "HOUR:8:DAY:1:DAY:1:0:%a";
    }
    if ( $detail == 1 ) {
      $xgrid .= "HOUR:12:HOUR:6:HOUR:12:0:%a-%H";
    }
    if ( $detail == 2 ) {

      # dashboard
      $xgrid .= "DAY:1:DAY:7:DAY:1:0:%d";
    }
  } ## end if ( $type =~ m/w/ )
  if ( $type =~ m/m/ ) {
    if ( $detail == 0 || $detail == 9 || $detail == 8 ) {
      $xgrid .= "DAY:1:DAY:2:DAY:2:0:%d";
    }
    if ( $detail == 1 ) {
      $xgrid .= "HOUR:12:DAY:1:DAY:1:0:%d";
    }
    if ( $detail == 2 ) {

      # dashboard
      $xgrid .= "DAY:7:MONTH:1:DAY:7:0:%d";
    }
  } ## end if ( $type =~ m/m/ )
  if ( $type =~ m/y/ ) {
    if ( $detail == 0 || $detail == 9 || $detail == 8 ) {
      $xgrid .= "MONTH:1:MONTH:1:MONTH:1:0:%b";
    }
    if ( $detail == 1 ) {
      $xgrid .= "MONTH:1:MONTH:1:MONTH:1:0:%b";
    }
    if ( $detail == 2 ) {

      # dashboard
      $xgrid .= "MONTH:4:MONTH:12:MONTH:4:0:%b";
    }
  } ## end if ( $type =~ m/y/ )

  return $xgrid;
} ## end sub xgrid_set

sub text_set {
  my $type = shift;
  my $text = "day";

  $text = "day";
  if ( $type =~ m/w/ ) {
    $text = "week";
  }
  if ( $type =~ m/m/ ) {
    $text = "month";
  }
  if ( $type =~ m/y/ ) {
    $text = "year";
  }
  return $text;
} ## end sub text_set

# search through aliases
sub alias {
  my $type = shift;
  my $host = shift;
  my $lpar = shift;

  #print STDERR "001 $host:$host: - :$lpar\n";
  my $afile = "$basedir/etc/alias.cfg";
  if ( -f $afile ) {
    open( FHA, "< $afile" ) || error( "File cannot be opened for read, wrong user rights?? : $afile " . __FILE__ . ":" . __LINE__ ) && return "";
    my @alines = <FHA>;
    close(FHA);

    # VOLUME:DS05:AAA11:OracleVolume --> example
    foreach my $aline (@alines) {
      chomp($aline);
      $aline =~ s/ *$//;    # zrusit mezery za koncem aliasy kdyby tam nejake omylem byly
      if ( $aline =~ m/^$type:$host:.*:$lpar$/ ) {

        # alias has been found
        ( my $t1, my $t2, my $alias ) = split( /:/, $aline );
        return "$alias";
      } ## end if ( $aline =~ m/^$type:$host:.*:$lpar$/)
    } ## end foreach my $aline (@alines)
  } ## end if ( -f $afile )

  return "";
} ## end sub alias

sub ret_graph_param {

  # for graphv returns
  #  "X-RRDGraph-Properties: $graph_left:$graph_top:$graph_width:$graph_height:$graph_start:$graph_end\n\n"
  #  for graph returns
  #  "\n"
  # if exist print lines containing $delimiter -> prepares @table_out for clickable legend
  # if defined second parameter > Updated time for LAN SAN1 SAN2 SEA & others for clickable legend

  my $answ    = shift;
  my $updated = shift;

  if ( !defined $updated ) {    # line Updated
    $updated = "";
  }

  my $graphv_line = "";         # end of HTML heading
  my $test_item   = "graph_";
  my @rrd_result;
  if ( $answ =~ /0x/ ) {
    @rrd_result = split( '\n', $$answ );
  }
  else {
    @rrd_result = split( '\n', $answ );
  }

  #print STDERR "\@rrd_result @rrd_result\n";

  for (@rrd_result) {s/\"//g};               # filtr
  for (@rrd_result) {s/print\[.*\] =//g};    #  remove 1st part of print line in newer rrd versions

  my @rrd_print = grep {/$delimiter/} @rrd_result;    #print lines for legend

  #  print STDERR "\nrrdprint @rrd_print";

  my @matches = grep {/^$test_item/} @rrd_result;
  my $param_num = @matches;

  if ( $param_num > 0 && $param_num != 6 ) {
    print STDERR "cmd graphv did not give 6 graph_ params, gave $param_num " . __FILE__ . ":" . __LINE__ . "\n";

    # return $graphv_line;   # let it go on, 5 params is maybe enough so lets try
  }
  if ( $param_num > 0 ) {
    $graphv_line = "X-RRDGraph-Properties: ";
    for ( my $i = 0; $i < $param_num; $i++ ) {
      ( undef, my $dimension ) = split( "=", $matches[$i] );
      $dimension *= 1;
      $graphv_line .= $dimension;
      if ( $i != 5 ) {
        $graphv_line .= ":";
      }
    } ## end for ( my $i = 0; $i < $param_num...)
    $graphv_line .= "\n\n";
  } ## end if ( $param_num > 0 )
  else {
    $graphv_line .= "\n";
  }

  # print STDERR "\$graphv_line $graphv_line\n";

  #now prepare HTML table for clickable legend for pool, shared pool, hmctotals lpar

  if ( @rrd_print < 1 ) {
    return "$graphv_line";
  }
  my $item_ret = "";

  # print STDERR join "\n", @rrd_print;
  # print STDERR "\n\$rrd_print[0] $rrd_print[0]\n";
  ( undef, my $item0_ret, my $item1_ret, my $item2_ret, my $item3_ret, undef ) = split( "$delimiter", $rrd_print[0] );

  #print STDERR "\$item1_ret $item1_ret \$item2_ret $item2_ret\n";
  # return "$graphv_line";
  #print STDERR "\$graphv_line $graphv_line : $item0_ret: $item1_ret : $item2_ret : $item3_ret\n";

  if ( defined $item1_ret && $item1_ret ne "" && $item0_ret !~ m/customg/ ) {

    # necessary to filter some none html legend pict
    if ( $item1_ret =~ /Volume/ && $item2_ret !~ /sum/ ) { return "$graphv_line"; }
    if ( $item1_ret =~ /HOST/ )        { return "$graphv_line"; }
    if ( $item1_ret =~ / Ports / )     { $no_legend = "--no-legend"; }    # Ports input
    if ( $item1_ret =~ / Switches / )  { $no_legend = "--no-legend"; }    # Ports input
    if ( $no_legend ne "--no-legend" ) { return "$graphv_line"; }

    my $line1 = "<table class=\"tablesorter tablegend\" data-sortby='4'>
    <thead>
      <tr>
        <th>&nbsp;</th>
        <th class=\"sortable header toleft\">$item1_ret $item2_ret</th>
        <th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2 = "    <tr>
        <td class=\"legsq\">xorux_lpar_color</td>
        <td class=\"clickabletd\">xorux_lpar_name</td>
        <td>xorux_lpar_avg</td>
        <td>xorux_lpar_max</td>
      </tr>";

    my $line4 = "</tbody><tfoot>";
    my @comments = grep {/XOR_COM/} @rrd_result;
    while ( my $print_line = shift(@comments) ) {
      ( undef, $print_line ) = split( "XOR_COM", $print_line );
      $line4 .= "<tr><td colspan=\"7\" class=\"toleft\">$print_line</td></tr>";
    }
    if ( $updated =~ "Updated" ) {
      $line4 .= "<tr><td colspan=\"7\" class=\"tdupdated toleft\">$updated</td></tr>";
    }
    $line4 .= "</table>\n";

    if ( $item =~ "^san_isl" ) {    #SAN ISL only
      $line1 = "<table class=\"tablesorter tablegend\" data-sortby='5'>
      <thead>
        <tr>
          <th>&nbsp;</th>
          <th class=\"sortable header toleft\">$item1_ret $item2_ret</th>
          <th class=\"sortable header toleft\">To port:</th>
          <th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
          <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
        </tr>
      </thead>
      <tbody>";
      $line2 = "    <tr>
          <td class=\"legsq\">xorux_lpar_color</td>
          <td class=\"clickabletd\">xorux_lpar_name</td>
          <td class=\"clickabletd\">xorux_port_name</td>
          <td>xorux_lpar_avg</td>
          <td>xorux_lpar_max</td>
        </tr>";
    } ## end if ( $item =~ "^san_isl")

    push @table_out, $line1;
    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2;
      ( my $average, my $lpar_space, undef, undef, undef, undef, my $color, my $rrd_file_all, my $st_type ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);
      ( my $max, undef ) = split( "$delimiter", $print_line );
      if ( !defined $rrd_file_all ) {    #if not in 1st line > try second
        ( undef, $rrd_file_all, $st_type ) = split( "$delimiter", $print_line );
      }
      $st_type =~ s/ //g;                # space might appear in RAN/MDISK somehow
      my @pathx = split( "/", $rrd_file_all );

      my $lpar_name_url = @pathx[ @pathx - 1 ];
      my $hmc_t         = @pathx[ @pathx - 2 ];
      my $server_t      = @pathx[ @pathx - 3 ];

      my $lpar_space_show = $lpar_space;
      $lpar_space_show =~ s/&&1/\//g;

      # find the right path of lpar
      my $lpar_url = $lpar_space_show;
      if ( $hmc_t eq "RANK" ) {
        $lpar_url = @pathx[ @pathx - 1 ];
      }
      if ( $rrd_file_all =~ /\/POOL\// && $st_type =~ "SWIZ" ) {

        # something like 'data/pmdsk1/POOL/1.rrd'
        $lpar_url = $lpar_name_url;
        $lpar_url =~ s/\.rrd//;
      } ## end if ( $rrd_file_all =~ ...)

      # --PH: commenetd out in 1.25-2, no idea whay it is here, something with DS8k
      #if ($rrd_file_all =~ /\/VOLUME\//) {
      #  # something like 'data/DS05/VOLUME/0001.rrc'
      #  # for this case name is like 'CMOD11_HB :0001' or 'sfutOa01       '
      #  # just take the first part
      #  ($lpar_url, undef) = split (':',$lpar_space);
      #  #print STDERR "\$lpar_url $lpar_url\n";
      #}
      $lpar_url = $lpar_space;
      $lpar_url =~ s/\.rr.$//;
      $lpar_url =~ s/^\s+|\s+$//g;
      $lpar_url =~ s/ /\+/g;
      $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;    # PH: keep it is it is exactly!!!
      $lpar_url =~ s/\#/%23/g;
      my $item_url = "$lpar_url";

      my $query_lpar  = "<a href=\"/stor2rrd-cgi/detail.sh?host=$server_t&type=$hmc_t&name=$lpar_url&storage=$st_type&item=$item_url&gui=1&none=none\">$lpar_space_show</a>";
      my $query_lpar2 = "";

      if ( $st_type eq "VMAX" && ( $hmc_t eq "PORT" || $hmc_t eq "NODE" ) ) {
        my $square_item = $item1_ret;
        $square_item =~ s/^\s+//g;
        $square_item =~ s/\s+$//g;
        if ( $square_item =~ m/_frontend$/ || $square_item =~ m/_backend$/ || $square_item =~ m/_rdf$/ ) {
          $square_item =~ s/_frontend$//;
          $square_item =~ s/_backend$//;
          $square_item =~ s/_rdf$//;

          $query_lpar = "<a href=\"/stor2rrd-cgi/detail.sh?host=$server_t&type=$hmc_t&name=$lpar_url&storage=$st_type&item=$item_url&gui=1&none=none&square_item=$square_item\">$lpar_space_show</a>";
        } ## end if ( $square_item =~ m/_frontend$/...)
      } ## end if ( $st_type eq "VMAX"...)
      if ( $st_type eq "VPLEX" && $hmc_t eq "PORT" ) {
        my $square_item = $item1_ret;
        $square_item =~ s/^\s+//g;
        $square_item =~ s/\s+$//g;
        if ( $square_item =~ m/_frontend$/ || $square_item =~ m/_backend$/ ) {
          $square_item =~ s/_frontend$//;
          $square_item =~ s/_backend$//;

          $query_lpar = "<a href=\"/stor2rrd-cgi/detail.sh?host=$server_t&type=$hmc_t&name=$lpar_url&storage=$st_type&item=$item_url&gui=1&none=none&square_item=$square_item\">$lpar_space_show</a>";
        } ## end if ( $square_item =~ m/_frontend$/...)
      } ## end if ( $st_type eq "VPLEX"...)

      # SAN-BRCD QUERY START
      if ( $st_type eq "SAN-BRCD" || $st_type eq "SAN-CISCO" ) {
        my $item_menu  = "";
        my $item_legsq = "";
        my $type_menu  = $type;

        if ( $item eq "san_data_sum_in" )       { $item_menu = "san"; $item_legsq = "san_data"; }
        if ( $item eq "san_data_sum_out" )      { $item_menu = "san"; $item_legsq = "san_data"; }
        if ( $item eq "san_io_sum_credits" )    { $item_menu = "san"; $item_legsq = "san_credits"; }
        if ( $item eq "san_io_sum_crc_errors" ) { $item_menu = "san"; $item_legsq = "san_errors"; }
        if ( $item eq "san_cpu_tot" )           { $item_menu = "san"; $item_legsq = "san_cpu_mem"; }
        if ( $item eq "san_mem_tot" )           { $item_menu = "san"; $item_legsq = "san_cpu_mem"; }

        #if ( $item eq "san_io_sum_encoding_errors" )     { $item_menu = "san";          $item_legsq = "san_errors"; }
        if ( $item eq "san_io_sum_in" )             { $item_menu = "san";          $item_legsq = "san_io"; }
        if ( $item eq "san_io_sum_out" )            { $item_menu = "san";          $item_legsq = "san_io"; }
        if ( $item eq "san_data_sum_tot_in" )       { $item_menu = "san_data_sum"; $item_legsq = "san_data_sum_in"; }
        if ( $item eq "san_data_sum_tot_out" )      { $item_menu = "san_data_sum"; $item_legsq = "san_data_sum_out"; }
        if ( $item eq "san_io_sum_tot_in" )         { $item_menu = "san_io_sum";   $item_legsq = "san_io_sum_in"; }
        if ( $item eq "san_io_sum_tot_out" )        { $item_menu = "san_io_sum";   $item_legsq = "san_io_sum_out"; }
        if ( $item eq "san_io_sum_tot_credits" )    { $item_menu = "san_io_sum";   $item_legsq = "san_io_sum_credits"; }
        if ( $item eq "san_io_sum_tot_crc_errors" ) { $item_menu = "san_io_sum";   $item_legsq = "san_io_sum_crc_errors"; }

        #if ( $item eq "san_io_sum_tot_encoding_errors" ) { $item_menu = "san_io_sum";   $item_legsq = "san_io_sum_encoding_errors"; }

        if ( $type_menu =~ "IO" )   { $type_menu = "Frame"; $hmc_t = $lpar_url; }
        if ( $type_menu =~ "DATA" ) { $type_menu = "Data";  $hmc_t = $lpar_url; }
        if ( $lpar_url =~ "^port" ) { $type_menu = "SANPORT"; }

        $lpar_space_show =~ s/^port//;

        # only Alias
        if ( $lpar_url =~ "-XOR-ALIAS-" ) {
          ( $lpar_url, $lpar_space_show ) = split( "-XOR-ALIAS-", $lpar_url );
          $lpar_space_show =~ s/\+/ /g;
        }

        # ISL
        if ( $item =~ m/^san_isl_data_in$/ )        { $item_menu = "san"; $item_legsq = "san_data"; }
        if ( $item =~ m/^san_isl_data_out$/ )       { $item_menu = "san"; $item_legsq = "san_data"; }
        if ( $item =~ m/^san_isl_frames_in$/ )      { $item_menu = "san"; $item_legsq = "san_io"; }
        if ( $item =~ m/^san_isl_frames_out$/ )     { $item_menu = "san"; $item_legsq = "san_io"; }
        if ( $item =~ m/^san_isl_frame_size_in$/ )  { $item_menu = "san"; $item_legsq = "san_io_size"; }
        if ( $item =~ m/^san_isl_frame_size_out$/ ) { $item_menu = "san"; $item_legsq = "san_io_size"; }
        if ( $item =~ m/^san_isl_credits$/ )        { $item_menu = "san"; $item_legsq = "san_credits"; }
        if ( $item =~ m/^san_isl_crc_errors$/ )     { $item_menu = "san"; $item_legsq = "san_errors"; }
        if ( $item =~ "^san_isl" ) {
          my ( $sw_name, $port_name, $port_alias, $to_port, $to_port_alias, $to_switch ) = split( ",", $lpar_url );
          my $lpar_space_show2 = "$to_switch-port$to_port_alias";
          $hmc_t           = $sw_name;
          $lpar_url        = $port_name;
          $lpar_space_show = "$sw_name-port$port_alias";
          $type_menu       = "SANPORT";
          $query_lpar2     = "<a href=\"/stor2rrd-cgi/detail.sh?host=$to_switch&type=$type_menu&name=$to_port&storage=$st_type&item=$item_menu&gui=1&none=none&square_item=$item_legsq\">$lpar_space_show2</a>";
        } ## end if ( $item =~ "^san_isl")

        $query_lpar = "<a href=\"/stor2rrd-cgi/detail.sh?host=$hmc_t&type=$type_menu&name=$lpar_url&storage=$st_type&item=$item_menu&gui=1&none=none&square_item=$item_legsq\">$lpar_space_show</a>";
      } ## end if ( $st_type eq "SAN-BRCD"...)

      # SAN-BRCD QUERY END

      #lowest level is not clickable
      # when $rrd_file_all == " " then no clickable legend is expected (DS5K and Controller graphs)
      #   --> sometimes there are more spaces
      if ( $hmc_t eq '' || $rrd_file_all eq '' || $rrd_file_all =~ m/^ *$/ ) {
        $line2_act =~ s/clickabletd/toleft/;
        $query_lpar = "$lpar_space_show";
      }
      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      $line2_act =~ s/xorux_lpar_color/$color/;
      if ( $item =~ "^san_isl" ) {    #SAN - ISL only
        $line2_act =~ s/xorux_port_name/$query_lpar2/;
      }
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)

    # print STDERR "print updated $line4\n";
    push @table_out, $line4;
    return $graphv_line;
  } ## end if ( defined $item1_ret...)

  #
  # Custom groups SAN
  #
  if ( defined $item1_ret && $item0_ret =~ m/customg-san/ ) {
    if ( $no_legend ne "--no-legend" ) { return "$graphv_line"; }
    my $line1 = "<table class=\"tablesorter tablegend\" data-sortby='5'>
    <thead>
      <tr>
        <td colspan=\"5\" align=\"left\"><p class='custom0'>$item2_ret $item3_ret</p></td>
      </tr>
      <tr>
        <th>&nbsp;</th>
        <th class=\"sortable header toleft\">Switch</th>
        <th class=\"sortable header toleft\">Port</th>
        <th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2 = "    <tr>
        <td class=\"legsq\">xorux_lpar_color</td>
        <td class=\"clickabletd\">xorux_server_name</td>
        <td class=\"clickabletd\">xorux_lpar_name</td>
        <td>xorux_lpar_avg</td>
        <td>xorux_lpar_max</td>
      </tr>";

    my $line4 = "</tbody><tfoot>";
    my @comments = grep {/XOR_COM/} @rrd_result;
    while ( my $print_line = shift(@comments) ) {
      ( undef, $print_line ) = split( "XOR_COM", $print_line );
      $line4 .= "<tr><td colspan=\"7\" class=\"toleft\">$print_line</td></tr>";
    }
    if ( $updated =~ "Updated" ) {
      $line4 .= "<tr><td colspan=\"7\" class=\"tdupdated toleft\">$updated</td></tr>";
    }
    $line4 .= "</table>\n";

    push @table_out, $line1;
    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2;
      ( my $average, my $type, my $lpar_space, my $item_act, undef, undef, undef, my $color, my $rrd_file_all, my $st_type ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);
      ( my $max, undef ) = split( "$delimiter", $print_line );
      if ( !defined $rrd_file_all ) {    #if not in 1st line > try second
        ( undef, $rrd_file_all, $st_type ) = split( "$delimiter", $print_line );
      }
      $st_type =~ s/ //g;                # space might appear in RAN/MDISK somehow
      my @pathx = split( "/", $rrd_file_all );

      my $lpar_name_url = @pathx[ @pathx - 1 ];
      my $server_t      = @pathx[ @pathx - 2 ];

      my $lpar_space_show = $lpar_space;

      # find the right path of lpar
      my $lpar_url = $lpar_space_show;
      $lpar_url =~ s/\.rr.$//;
      $lpar_url =~ s/^\s+|\s+$//g;
      $lpar_url =~ s/ /\+/g;
      $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;    # PH: keep it is it is exactly!!!
      $lpar_url =~ s/\#/%23/g;

      my $server_url = $server_t;
      $server_url =~ s/\.rr.$//;
      $server_url =~ s/^\s+|\s+$//g;
      $server_url =~ s/ /\+/g;
      $server_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;    # PH: keep it is it is exactly!!!
      $server_url =~ s/\#/%23/g;

      # /stor2rrd-cgi/detail.sh?host=ASAN11&type=SANPORT&name=port17&storage=SAN-BRCD&item=san&gui=1&none=none::
      my $st_type    = "SAN-BRCD";
      my $type_menu  = "SANPORT";
      my $item_menu  = "san";
      my $item_legsq = "san_data";

      if ( $item_act =~ m/^data_in$/ )    { $item_menu = "san"; $item_legsq = "san_data"; }
      if ( $item_act =~ m/^data_out$/ )   { $item_menu = "san"; $item_legsq = "san_data"; }
      if ( $item_act =~ m/^frames_in$/ )  { $item_menu = "san"; $item_legsq = "san_io"; }
      if ( $item_act =~ m/^frames_out$/ ) { $item_menu = "san"; $item_legsq = "san_io"; }
      if ( $item_act =~ m/^credits$/ )    { $item_menu = "san"; $item_legsq = "san_credits"; }
      if ( $item_act =~ m/^crc_errors$/ ) { $item_menu = "san"; $item_legsq = "san_errors"; }

      if ( $lpar_space =~ "XOR-href-name-XOR" ) {
        my ( $name_out, $name_to_url ) = split( "XOR-href-name-XOR", $lpar_space );
        chomp $name_out;
        chomp $name_to_url;
        $lpar_url        = $name_to_url;
        $lpar_space_show = $name_out;
      } ## end if ( $lpar_space =~ "XOR-href-name-XOR")

      my $query_lpar = "<a href=\"/stor2rrd-cgi/detail.sh?host=$server_url&type=$type_menu&name=port$lpar_url&storage=$st_type&item=$item_menu&gui=1&none=none&square_item=$item_legsq\">$lpar_space_show</a>";

      # /stor2rrd-cgi/detail.sh?host=ASAN11&type=Data&name=ASAN11&storage=SAN-BRCD&item=san_data_sum&gui=1&none=none::::
      my $type_t = "Data";
      if ( $item2_ret =~ m/frame/ ) { $type_t = "Frame"; }

      my $query_server = "<a href=\"/stor2rrd-cgi/detail.sh?host=$server_url&type=$type_t&name=$server_url&storage=$st_type&item=san_data_su&gui=1&none=none\">$server_t</a>";

      #lowest level is not clickable
      # when $rrd_file_all == " " then no clickable legend is expected (DS5K and Controller graphs)
      #   --> sometimes there are more spaces
      if ( $rrd_file_all eq '' || $rrd_file_all =~ m/^ *$/ ) {
        $line2_act =~ s/clickabletd/toleft/;
        $query_lpar = "$lpar_space_show";
      }
      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      $line2_act =~ s/xorux_lpar_color/$color/;
      $line2_act =~ s/xorux_server_name/$query_server/;
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)

    # print STDERR "print updated $line4\n";
    push @table_out, $line4;
    return $graphv_line;
  } ## end if ( defined $item1_ret...)

  #
  # Custom groups Storage
  #
  if ( defined $item1_ret && $item0_ret =~ m/customg-storage/ ) {
    if ( $no_legend ne "--no-legend" ) { return "$graphv_line"; }
    my $line1 = "<table class=\"tablesorter tablegend\" data-sortby='5'>
    <thead>
      <tr>
        <td colspan=\"5\" align=\"left\"><p class='custom0'>$item2_ret $item3_ret</p></td>
      </tr>
      <tr>
        <th>&nbsp;</th>
        <th class=\"sortable header toleft\">Storage</th>
        <th class=\"sortable header toleft\">Volume</th>
        <th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2 = "    <tr>
        <td class=\"legsq\">xorux_lpar_color</td>
        <td class=\"clickabletd\">xorux_server_name</td>
        <td class=\"clickabletd\">xorux_lpar_name</td>
        <td>xorux_lpar_avg</td>
        <td>xorux_lpar_max</td>
      </tr>";

    my $line4 = "</tbody><tfoot>";
    my @comments = grep {/XOR_COM/} @rrd_result;
    while ( my $print_line = shift(@comments) ) {
      ( undef, $print_line ) = split( "XOR_COM", $print_line );
      $line4 .= "<tr><td colspan=\"7\" class=\"toleft\">$print_line</td></tr>";
    }
    if ( $updated =~ "Updated" ) {
      $line4 .= "<tr><td colspan=\"7\" class=\"tdupdated toleft\">$updated</td></tr>";
    }
    $line4 .= "</table>\n";

    push @table_out, $line1;
    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2;
      ( my $average, my $type, my $lpar_space, my $item_act, undef, undef, undef, my $color, my $rrd_file_all, my $st_type ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);
      ( my $max, undef ) = split( "$delimiter", $print_line );
      if ( !defined $rrd_file_all ) {    #if not in 1st line > try second
        ( undef, $rrd_file_all, $st_type ) = split( "$delimiter", $print_line );
      }
      $st_type =~ s/ //g;                # space might appear in RAN/MDISK somehow
      my @pathx = split( "/", $rrd_file_all );

      my $lpar_name_url = @pathx[ @pathx - 1 ];
      my $server_t      = @pathx[ @pathx - 3 ];

      my $lpar_space_show = $lpar_space;

      # find the right path of lpar
      my $lpar_url = $lpar_space_show;
      $lpar_url =~ s/\.rr.$//;
      $lpar_url =~ s/^\s+|\s+$//g;
      $lpar_url =~ s/ /\+/g;
      $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;    # PH: keep it is it is exactly!!!
      $lpar_url =~ s/\#/%23/g;

      my $server_url = $server_t;
      $server_url =~ s/\.rr.$//;
      $server_url =~ s/^\s+|\s+$//g;
      $server_url =~ s/ /\+/g;
      $server_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;    # PH: keep it is it is exactly!!!
      $server_url =~ s/\#/%23/g;

      # /stor2rrd-cgi/detail.sh?host=DS3500&type=VOLUME&name=HB&storage=DS5K&item=lpar&gui=1&none=none:::

      my $type_menu  = "VOLUME";
      my $item_menu  = "lpar";
      my $item_legsq = "data_rate";

      if ( $item_act =~ m/data/ ) { $item_menu = "lpar"; $item_legsq = "data_rate"; }
      if ( $item_act =~ m/io/ )   { $item_menu = "lpar"; $item_legsq = "io_rate"; }
      my $query_lpar = "<a href=\"/stor2rrd-cgi/detail.sh?host=$server_url&type=$type_menu&name=$lpar_url&storage=$st_type&item=$item_menu&gui=1&none=none&square_item=$item_legsq\">$lpar_space_show</a>";

      # /stor2rrd-cgi/detail.sh?host=DS3500&type=VOLUME&name=data&storage=DS5K&item=sum&gui=1&none=none::
      # /stor2rrd-cgi/detail.sh?host=DS8800&type=VOLUME&name=data&storage=DS8K&item=sum&gui=1&none=none::
      my $name_t = "data";

      #if ( $item2_ret =~ m/frame/ )         { $name_t = "Frame"; }

      my $query_server = "<a href=\"/stor2rrd-cgi/detail.sh?host=$server_url&type=$type_menu&name=$name_t&storage=$st_type&item=sum&gui=1&none=none\">$server_t</a>";

      #lowest level is not clickable
      # when $rrd_file_all == " " then no clickable legend is expected (DS5K and Controller graphs)
      #   --> sometimes there are more spaces
      if ( $rrd_file_all eq '' || $rrd_file_all =~ m/^ *$/ ) {
        $line2_act =~ s/clickabletd/toleft/;
        $query_lpar = "$lpar_space_show";
      }
      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      $line2_act =~ s/xorux_lpar_color/$color/;
      $line2_act =~ s/xorux_server_name/$query_server/;
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)

    # print STDERR "print updated $line4\n";
    push @table_out, $line4;
    return $graphv_line;
  } ## end if ( defined $item1_ret...)

  error( "sub ret_graph_param - not recognised item : $item1_ret " . __FILE__ . ":" . __LINE__ );

  #print STDERR "??? @table_out\n";

} ## end sub ret_graph_param

# find list of vols for HOST
sub find_vols {
  my $host = shift;
  my $type = shift;
  my $lpar = shift;

  if ( !-f "$wrkdir/$host/$type/hosts.cfg" ) {
    return "";
  }
  open( FHH, "< $wrkdir/$host/$type/hosts.cfg" ) || error( "Can't open $wrkdir/$host/$type/hosts.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return "";
  my @hosts = <FHH>;
  close(FHH);

  foreach my $line (@hosts) {
    chomp($line);
    ( my $host, my $volumes ) = split( / : /, $line );

    # must be used this no regex construction otherwise m// does not work with names with ()
    if ( $host eq $lpar ) {
      return $volumes;
    }
  } ## end foreach my $line (@hosts)
  return "";
} ## end sub find_vols

sub draw_graph_aggregated {
  my $host       = shift;
  my $type       = shift;
  my $lpar       = shift;
  my $item       = shift;
  my $time       = shift;
  my $name_out   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $wrkdir     = shift;
  my $act_time   = shift;
  my $step       = shift;
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $last       = "COMMENT: ";
  my $act_time   = localtime();
  my $act_time_u = time();
  my $req_time   = 0;
  my $itemm      = "";

  # open a file with stored colours
  my @color_save = "";

  my $req_time = "";
  my $i        = 0;
  my $cmd      = "";
  my $j        = 0;

  my $color_indx  = 0;       # reset colour index
  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;
  my $suffix      = "rrd";

  my $type_text = $type;

  if ( $item =~ m/^tier_alloc_prct$/ ) { $value_short = "%%"; $value_long = "percent"; $type_text = "STORAGE GROUP"; }

  my $xgrid_all = xgrid_set( $time, $detail );
  my $text      = text_set($time);
  my $header    = "$type_text : $name : $item last $text";
  if ( $detail == 2 ) {
    $header = "$name : $item $text";
  }

  my $start_time = "now-1$time";
  my $end_time   = "now-1$time+1$time";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # Historical report detected
      $start_time = $start_unix;
      $end_time   = $end_unix;

      #$xgrid ="COMMENT: "; # just a trick to place there something otherwise rrdtool complains
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header    = "$type : $name : $item : $start_human - $end_human";
      $text      = "";
      $xgrid_all = "--alt-y-grid";
      if ( $detail ne "9" && $detail ne "8" ) {
        $font_def  = $font_def_hist;
        $font_tit  = $font_tit_hist;
      }
      $req_time  = $start_unix;
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  if ( $rrd_legend eq "table" && ($detail < 1 || $detail == 9 || $detail == 8) ) {
    $no_legend = "--no-legend";
  }
  my $vertical_label = "--vertical-label=\\\"$value_long\\\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";
    $no_legend      = "--no-legend";
  }

  # new zooming function without rrdtool title in graphs
  if ( $detail == 9 || $detail == 8 ) { $header = ""; }

  $cmd .= "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start \\\"$start_time\\\"";
  $cmd .= " --end \\\"$end_time\\\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $xgrid_all";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " --alt-y-grid";

  # print STDERR "001 $st_type,$type,$item,$suffix,$no_legend,\n";

  # add spaces to lpar name (for formating graph legend)
  my $item_short = $item;
  my $legend = sprintf( "%-38s", "$item_short [$value_short]" );
  $cmd .= " COMMENT:\\\"$legend        Avg       Max\\l\\\"";

  my $legend_heading = "$item_short [$value_short] $delimiter  $delimiter Avg $delimiter Max ";

  my $gtype      = "AREA";
  my $itemm      = "";
  my $lpar_space = "";
  my $once       = 0;
  my $last_time  = "na";
  my $indx       = 0;

  my $searchstr = "VOLUME/";

  if ( $st_type eq "VMAX" && $item =~ m/^tier_alloc_prct$/ ) { $searchstr = "FAST-SG/$lpar-"; }

  foreach my $vol_file (<$wrkdir/$host/$searchstr*\.$suffix>) {
    chomp($vol_file);

    my $vol_name = basename($vol_file);
    $vol_name =~ s/^$lpar-//;
    $vol_name =~ s/\.$suffix$//;
    my $itemm = $item . "m";
    $itemm =~ s/:/R/g;

    if ( $once == 0 ) {

      # find out time stamp of last data update
      # take just one volume to keep it simple
      $once++;
      RRDp::cmd qq(last "$vol_file");
      my $last_tt = RRDp::read;
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
    } ## end if ( $once == 0 )

    # bulid RRDTool cmd

    # add spaces to lpar name to have 18 chars total (for formating graph legend)
    $lpar_space = sprintf( "%-38s", "$vol_name" );
    $lpar_space =~ s/:/\\:/g;
    $vol_file =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    $cmd .= " DEF:$item${indx}=\\\"$vol_file\\\":$item:AVERAGE";
    $cmd .= " CDEF:$itemm${indx}=$item${indx},$val,/";                        # convert into MB if necessary, normaly is there 1
    $cmd .= " $gtype:$itemm${indx}$color[$color_indx]:\\\"$lpar_space\\\"";

    $cmd .= " PRINT:$itemm${indx}:AVERAGE:\\\"%6.1lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
    $cmd .= " PRINT:$itemm${indx}:MAX:\\\" %6.1lf $delimiter $vol_file\\\"";

    $gtype = "STACK";

    # put carriage return after each second lpar in the legend
    $cmd .= " GPRINT:$itemm${indx}:AVERAGE:\\\"%6.1lf \\\"";
    $cmd .= " GPRINT:$itemm${indx}:MAX:\\\"%6.1lf \\l\\\"";

    $color_indx++;
    if ( $color_indx > $color_max ) {
      $color_indx = 0;
    }
    $indx++;
  } ## end foreach my $vol_file (<$wrkdir/$host/$searchstr*\.$suffix>)
  if ( $indx == 0 ) {

    # no volume has been found
    error( "No volume has been found " . __FILE__ . ":" . __LINE__ );
    return 0;
  } ## end if ( $indx == 0 )

  if ( $j == 1 ) {
    $cmd .= " COMMENT:\\\"\\l\\\"";
  }

  if ( $time =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  RRDp::cmd qq($cmd);
  my $ret = RRDp::read;
  if ( $$ret =~ "ERROR" ) {
    error( "$host : Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  $png_end_heading = ret_graph_param($ret);
  return 1;
} ## end sub draw_graph_aggregated

sub draw_graph_host_volume {
  my $host       = shift;
  my $type       = shift;
  my $lpar       = shift;
  my $item       = shift;
  my $time       = shift;
  my $name_out   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $wrkdir     = shift;
  my $act_time   = shift;
  my $step       = shift;           # for volumes must be step correct due to DS8K and read/write stored in GAUGE instead of ABSOLUTE
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $last       = "COMMENT: ";
  my $act_time   = localtime();
  my $act_time_u = time();
  my $req_time   = 0;
  my $itemm      = "";
  my $volume     = "VOLUME";
  my $vol_col    = "volumes.cfg";

  my $vols_list_line = find_vols( $host, $type, $lpar );
  if ( $vols_list_line eq '' ) {

    # something wrong as it should not come here through detail-cgi.pl when there is no volumes assigned
    error( "$host:$type: no volumes detected " . __FILE__ . ":" . __LINE__ );
    return 0;
  } ## end if ( $vols_list_line eq...)
  my @vols_list = split( / +/, $vols_list_line );

  # open a file with stored colours
  my @color_save = "";

  #if ( $st_type =~ m/^VSPG$/ ){
  #  $volume = "LDEV";
  #  $vol_col = "ldevs.cfg";
  #}

  if ( -f "$wrkdir/$host/VOLUME/volumes.col" ) {
    open( FHC, "< $wrkdir/$host/VOLUME/volumes.col" ) || error( "file does not exists : $wrkdir/$host/VOLUME/volumes.col " . __FILE__ . ":" . __LINE__ ) && return 0;
    @color_save = <FHC>;
    close(FHC);
  }

  my $req_time = "";
  my $i        = 0;
  my $lpar     = "";
  my $cmd      = "";
  my $j        = 0;

  my $color_indx  = 0;       # reset colour index
  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;
  my $suffix      = "rrd";

  # do not use switch statement
  if ( $item =~ m/^data_rate$/ ) { $value_short = "MB"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^read$/ )      { $value_short = "MB"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^write$/ )     { $value_short = "MB"; $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^io_rate$/ )       { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^read_io$/ )       { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^write_io$/ )      { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^resp_t$/ )        { $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^resp_t_r$/ )      { $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^resp_t_w$/ )      { $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^r_cache_hit$/ )   { $value_short = "%%";   $value_long = "percent"; }
  if ( $item =~ m/^w_cache_hit$/ )   { $value_short = "%%";   $value_long = "percent"; }
  if ( $item =~ m/^r_cache_usage$/ ) { $value_short = "MB";   $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^w_cache_usage$/ ) { $value_short = "MB";   $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^total_usage$/ )   { $value_short = "MB";   $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^read_b$/ )        { $value_short = "MB";   $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^write_b$/ )       { $value_short = "MB";   $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^read_io_b$/ )     { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^write_io_b$/ )    { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^resp_t_r_b$/ )    { $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^resp_t_w_b$/ )    { $value_short = "ms";   $value_long = "milliseconds"; }
  if ( $item =~ m/^read_pct$/ )      { $value_short = "%%";   $value_long = "percent"; }

  if ( $st_type =~ m/^DS8K$/ && ( $type =~ /VOLUME/ || $type =~ /HOST/ ) ) {
    if ( $item =~ m/^resp_t_r$/
      || $item =~ m/^resp_t_w$/
      || $item =~ m/^r_cache_usage$/
      || $item =~ m/^r_cache_hit$/
      || $item =~ m/^w_cache_hit$/
      || $item =~ m/^w_cache_usage$/
      || $item =~ m/^read_io$/
      || $item =~ m/^write_io$/
      || $item =~ m/^read_io_b$/
      || $item =~ m/^write_io_b$/
      || $item =~ m/^read_b$/
      || $item =~ m/^write_b$/
      || $item =~ m/^resp_t_r_b$/
      || $item =~ m/^resp_t_w_b$/ )
    {
      # those are stored in .rrc files
      $suffix = "rrc";
    } ## end if ( $item =~ m/^resp_t_r$/...)
  } ## end if ( $st_type =~ m/^DS8K$/...)
  if ( $st_type =~ m/^SWIZ$/ && ( $type =~ /VOLUME/ || $type =~ /HOST/ ) ) {
    if ( $item =~ m/^r_cache_usage$/
      || $item =~ m/^r_cache_hit$/
      || $item =~ m/^w_cache_hit$/
      || $item =~ m/^w_cache_usage$/
      || $item =~ m/^total_usage$/
      || $item =~ m/^res1$/
      || $item =~ m/^res2$/ )
    {
      # those are stored in .rrc files
      $suffix = "rrc";
    } ## end if ( $item =~ m/^r_cache_usage$/...)
  } ## end if ( $st_type =~ m/^SWIZ$/...)

  if ( $st_type =~ m/^DS8K$/ && ( $type =~ /VOLUME/ || $type =~ /HOST/ ) ) {    #
    if ( $item =~ m/^read$/ || $item =~ m/^write$/ ) {

      # data is stored in wrong RRDTool type (GAUGE instead of ABSOLUTE)
      # this do data conversion
      $val = $step;    # do not use $val=1024
    } ## end if ( $item =~ m/^read$/...)
  } ## end if ( $st_type =~ m/^DS8K$/...)

  my $xgrid_all = xgrid_set( $time, $detail );
  my $text      = text_set($time);
  my $header    = "$type : $name : $item last $text";
  if ( $detail == 2 ) {
    $header = "$name : $item $text";
  }

  my $start_time = "now-1$time";
  my $end_time   = "now-1$time+1$time";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # Historical report detected
      $start_time = $start_unix;
      $end_time   = $end_unix;

      #$xgrid ="COMMENT: "; # just a trick to place there something otherwise rrdtool complains
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header    = "$type : $name : $item : $start_human - $end_human";
      $text      = "";
      $xgrid_all = "--alt-y-grid";
      if ( $detail ne "9" && $detail ne "8" ) {
        $font_def  = $font_def_hist;
        $font_tit  = $font_tit_hist;
      }
      $req_time  = $start_unix;
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  if ( $rrd_legend eq "table" && ($detail < 1 || $detail == 9 || $detail == 8) ) {
    $no_legend = "--no-legend";
  }
  my $vertical_label = "--vertical-label=\\\"$value_long\\\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";
    $no_legend      = "--no-legend";
  }

  # new zooming function without rrdtool title in graphs
  if ( $detail == 9 || $detail == 8 ) { $header = ""; }

  $cmd .= "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start \\\"$start_time\\\"";
  $cmd .= " --end \\\"$end_time\\\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $xgrid_all";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " --alt-y-grid";

  # print STDERR "001 $st_type,$type,$item,$suffix,$no_legend,\n";

  # add spaces to lpar name (for formating graph legend)
  my $item_short = $item;
  $item_short =~ s/r_cache_//;
  $item_short =~ s/w_cache_//;
  my $legend = sprintf( "%-38s", "$item_short [$value_short]" );
  $cmd .= " COMMENT:\\\"$legend        Avg       Max                                                Avg       Max\\l\\\"";

  my $legend_heading = "$item_short [$value_short] $delimiter  $delimiter Avg $delimiter Max ";

  my $gtype = "AREA";
  if ( $item =~ m/resp_t/ || $item =~ m/cache_hit/ || $item =~ m/^read_pct$/ ) {
    $gtype = "LINE1";
  }

  my $lpar_list     = "";
  my $lpar_list_tmp = "";

  open( FHR, "< $wrkdir/$host/VOLUME/volumes.cfg" ) || error( "file does not exists : $wrkdir/$host/VOLUME/volumes.cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @volumes = <FHR>;
  close(FHR);

  my $itemm      = "";
  my $lpar_space = "";
  my $once       = 0;
  my $last_time  = "na";
  my $indx       = 0;

  #print STDERR "$wrkdir/$host/VOLUME/*\.$suffix \n";
  foreach my $vol_file (<$wrkdir/$host/VOLUME/*\.$suffix>) {
    chomp($vol_file);

    my $vol_id = basename($vol_file);
    $vol_id =~ s/\.rrd$//;
    $vol_id =~ s/\.rrc$//;
    $vol_id =~ s/\-P.*$//;    # vyhodit POOL ID

    # check if the volume is on the host list
    my $found_vol  = 0;
    my $vol_id_cmp = $vol_id;    #filter our perfixed zeros for SVC, DS8k use hexa
    $vol_id_cmp =~ s/^0*//;
    foreach my $vol_host (@vols_list) {
      chomp($vol_host);
      $vol_host =~ s/^0x//;      # cut off hexa prefix for DS8k
      $vol_host =~ s/ //g;
      $vol_host =~ s/^0*//;
      if ( $vol_host =~ m/^$vol_id_cmp$/ ) {
        $found_vol = 1;
        last;
      }
    } ## end foreach my $vol_host (@vols_list)
    if ( $found_vol == 0 ) {
      next;                      # the volume is not allocated for this host
    }

    my $vol_name = find_vol_name( $vol_id, $st_type, \@volumes );
    $vol_name =~ s/\\:/:/g;      # DS8K return "name \:volume" --> ths mus be filtered here, it is globally backslashed in lpar_space below
    my $itemm = $item . "m" . $vol_id;
    $itemm =~ s/:/R/g;

    if ( $once == 0 ) {

      # find out time stamp of last data update
      # take just one volume to keep it simple
      $once++;
      RRDp::cmd qq(last "$vol_file");
      my $last_tt = RRDp::read;
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
    } ## end if ( $once == 0 )

    # bulid RRDTool cmd

    # add spaces to lpar name to have 18 chars total (for formating graph legend)
    $lpar_space = sprintf( "%-38s", "$vol_name" );
    $lpar_space =~ s/:/\\:/g;
    $vol_file =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    $cmd .= " DEF:$item${indx}=\\\"$vol_file\\\":$item:AVERAGE";
    $cmd .= " CDEF:$itemm${indx}=$item${indx},$val,/";                        # convert into MB if necessary, normaly is there 1
    $cmd .= " $gtype:$itemm${indx}$color[$color_indx]:\\\"$lpar_space\\\"";

    $cmd .= " PRINT:$itemm${indx}:AVERAGE:\\\"%8.1lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
    $cmd .= " PRINT:$itemm${indx}:MAX:\\\" %8.1lf $delimiter $vol_file\\\"";

    if ( $item !~ m/resp_t/ && $item !~ m/cache_hit/ && $item !~ m/^read_pct$/ ) {
      $gtype = "STACK";
    }

    # put carriage return after each second lpar in the legend
    if ( $j == 1 ) {
      if ( $item =~ m/io_rate/ || $item =~ m/read_io/ || $item =~ m/write_io/ ) {
        $cmd .= " GPRINT:$itemm${indx}:AVERAGE:\\\"%6.0lf \\\"";
        $cmd .= " GPRINT:$itemm${indx}:MAX:\\\"%6.0lf \\l\\\"";
      }
      else {
        $cmd .= " GPRINT:$itemm${indx}:AVERAGE:\\\"%6.1lf \\\"";
        $cmd .= " GPRINT:$itemm${indx}:MAX:\\\"%6.1lf \\l\\\"";
      }
      $j = 0;
    } ## end if ( $j == 1 )
    else {
      if ( $item =~ m/io_rate/ || $item =~ m/read_io/ || $item =~ m/write_io/ ) {
        $cmd .= " GPRINT:$itemm${indx}:AVERAGE:\\\"%6.0lf \\\"";
        $cmd .= " GPRINT:$itemm${indx}:MAX:\\\"%6.0lf \\\"";
      }
      else {
        $cmd .= " GPRINT:$itemm${indx}:AVERAGE:\\\"%6.1lf \\\"";
        $cmd .= " GPRINT:$itemm${indx}:MAX:\\\"%6.1lf \\\"";
      }
      $j++;
    } ## end else [ if ( $j == 1 ) ]
    $color_indx++;
    if ( $color_indx > $color_max ) {
      $color_indx = 0;
    }
    $indx++;
  } ## end foreach my $vol_file (<$wrkdir/$host/VOLUME/*\.$suffix>)
  if ( $indx == 0 ) {

    # no volume has been found
    error( "No volume has been found " . __FILE__ . ":" . __LINE__ );
    return 0;
  } ## end if ( $indx == 0 )

  if ( $j == 1 ) {
    $cmd .= " COMMENT:\\\"\\l\\\"";
  }

  if ( $time =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  #my $c = $cmd;
  #$c =~ s/         /\n/g;
  #print STDERR "$c\n";

  RRDp::cmd qq($cmd);
  my $ret = RRDp::read;
  if ( $$ret =~ "ERROR" ) {
    error( "$host : Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  $png_end_heading = ret_graph_param($ret);
  return 1;
} ## end sub draw_graph_host_volume

sub basename {
  my $full = shift;
  my $out  = "";

  # basename without direct function
  my @base = split( /\//, $full );
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
} ## end sub basename

sub find_vol_name {
  my ( $vol_id, $st_type, $volumes_tmp ) = @_;
  my @volumes = @{$volumes_tmp};

  foreach my $volg (@volumes) {
    chomp($volg);
    ( my $name, my $id ) = split( / : /, $volg );
    $name =~ s/ *$//;
    if ( $st_type =~ m/DS8K/ ) {

      #CMAL11 : 0x8008;0x8009;0x800A;0x800B;0x8014;0x8015;0x8108;0x8109;0x810A;0x810B;0x8114;0x8115;
      if ( $id =~ m/0x$vol_id;/ ) {
        return "$name \\:$vol_id";
      }
    } ## end if ( $st_type =~ m/DS8K/)
    else {
      #CMAL11 : 0x8008;
      #storwize: BBpr : 0012;0023;
      $id =~ s/ //g;
      if ( $id eq $vol_id || $id =~ m/^$vol_id;/ || $id =~ m/;$vol_id;/ || $id =~ m/;$vol_id$/ ) {

        # volume ID are HEXa, not digits!
        return $name;
      }
    } ## end else [ if ( $st_type =~ m/DS8K/)]
  } ## end foreach my $volg (@volumes)

  return $vol_id;
} ## end sub find_vol_name

# it finds out or create a file with first data stapm in all rank files of given pool
# it is necessary to know it as since that time is used 0 instead on NaN in rrdtool cmd
sub find_real_data_start_pool {
  my $rank_dir = shift;
  my $pool_id  = shift;
  if ( isdigit($pool_id) == 1 ) {

    # only for decimals, XIV can use hexa
    $pool_id = $pool_id + 1 - 1;    # trick to convert it into a number (it is "0001" here for example
  }
  my $rrd_pool_first = "$rank_dir/$pool_id.first";
  if ( !-f $rrd_pool_first ) {
    $rrd_pool_first = "$rank_dir/P$pool_id.first_pool";
  }
  my $time_first_def = 2000000000;        # just place here something hig engouh, higherb than actual unix time in seconds
  my $time_first     = $time_first_def;

  if ( -f $rrd_pool_first ) {

    # read real last time of the record in rrdtool from the file (refresh every day)
    open( FHF, "< $rrd_pool_first" ) || error( "Can't open $rrd_pool_first : $! " . __FILE__ . ":" . __LINE__ );
    foreach my $line_frst (<FHF>) {
      chomp($line_frst);
      if ( isdigit($line_frst) ) {
        $time_first = $line_frst;
        last;
      }
    } ## end foreach my $line_frst (<FHF>)
    close(FHF);
  } ## end if ( -f $rrd_pool_first)
  if ( $time_first < $time_first_def ) {
    return $time_first;
  }
  else {
    #error ("Pool first time has not been found in : $rrd_pool_first ".__FILE__.":".__LINE__);
    return 1;    # something is wrong, "1" causes ignoring followed rrdtool construction
  }
} ## end sub find_real_data_start_pool

# prints out the Top tables
sub volumes_top {
  my $host         = shift;
  my $type         = shift;
  my $name         = shift;
  my $item         = shift;
  my $time         = shift;
  my $st_type      = shift;
  my $start_unix   = shift;
  my $end_unix     = shift;
  my $glob_txt     = shift;
  my $in_file      = "$tmp_dir/$host/$type-$name-$time.cmd";
  my $file_cmd_max = "/var/tmp/stor2rrd-$type-$name-max-$time.cmd";

  #"read","write","read_io","write_io","resp_t_r","resp_t_w","r_cache_hit","w_cache_hit";
  #VNX:"read","write","read_io","write_io","resp_t"
  #DS5K-v1:"data_rate","io_rate","cache_hit";
  #DS5K-v2:"data_rate","io_rate","resp_t","r_cache_hit","w_cache_hit","ssd_r_cache_hit";

  my $width    = 288;    # to get 5 minute per a pixel
  my $cust_rep = 0;
  if ( defined($end_unix) && isdigit($end_unix) == 1 && isdigit($start_unix) == 1 && $end_unix > 1200000000 && $start_unix > 1200000000 ) {
    $cust_rep = 1;                                    # it is called from custom report (manually selected time)
    $width    = ( $end_unix - $start_unix ) / 300;    # to get 5 minute per a pixel
  }

  #print STDERR "$start_unix $end_unix $cust_rep\n";

  if ( !-f $in_file ) {
    if ( $glob_txt =~ m/loc/ ) {
      error( "$host:$type:top:$item - no cmd file:$in_file " . __FILE__ . ":" . __LINE__ ) if ($DEBUG);
    }
    return 0;
  } ## end if ( !-f $in_file )

  if ( $cust_rep == 0 ) {
    if ( $item =~ m/^d/ ) {
      print "last 24 hours<br>\n";
    }
    if ( $time =~ m/^w/ ) {
      print "last week without last day<br>\n";
      $width = 1728;    # to get 5 minute per a pixel, 6d
    }
    if ( $time =~ m/^m/ ) {
      print "last month without last week<br>\n";
      $width = 6912;    # to get 5 minute per a pixel, 24d
    }
    if ( $time =~ m/^y/ ) {
      print "last year without last month<br>\n";
      $width = 97056;    # to get 5 minute per a pixel, 337d
    }
  } ## end if ( $cust_rep == 0 )

  if ( $glob_txt =~ m/loc/ ) {

    # local volume top
    if ( $st_type =~ m/SWIZ/ || $st_type =~ m/DS8K/ || $st_type =~ m/HUS/ || $st_type =~ m/VSPG/ ) {
      print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"7 8\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;r&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;w&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;r&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;w&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
    } ## end if ( $st_type =~ m/SWIZ/...)
    if ( $st_type =~ m/3PAR/ || $st_type =~ m/NETAPP/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/PURE/ ) {
      print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"7 8\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;r&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;w&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
    } ## end if ( $st_type =~ m/3PAR/...)
    if ( $st_type =~ m/OCEANSTOR/ ) {
      print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"7 8\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;r&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;w&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;r&nbsp;cache&nbsp;hit&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;w&nbsp;cache&nbsp;hit&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
    } ## end if ( $st_type =~ m/OCEANSTOR/)
    if ( $st_type =~ m/VMAX/ ) {
      print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"7 8\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;r&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;w&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
    } ## end if ( $st_type =~ m/VMAX/)
    if ( $st_type =~ m/VNXF|UNITY|VNXE|VNX$/ ) {
      print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"7 8\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
          <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;r&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
    } ## end if ( $st_type =~ m/VNXF|UNITY|VNXE|VNX$/)
    if ( $st_type =~ m/DOTHILL/ ) {
      print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"7 8\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
    } ## end if ( $st_type =~ m/DOTHILL/)
    if ( $st_type =~ m/XIV/ ) {
      print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"7 8\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;r&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;w&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
    } ## end if ( $st_type =~ m/XIV/)
    if ( $st_type =~ m/DS5K/ ) {
      if ( -f "$wrkdir/$host/DS5K-v2" ) {
        print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"6\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;data&nbsp;rate&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;rate&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;time&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;r&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;w&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;SSD&nbsp;read&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
      } ## end if ( -f "$wrkdir/$host/DS5K-v2")
      else {
        #$wrkdir/$host/DS5K-v1
        print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"6\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;data&nbsp;rate&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;io&nbsp;rate&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
      } ## end else [ if ( -f "$wrkdir/$host/DS5K-v2")]
    } ## end if ( $st_type =~ m/DS5K/)
    if ( $st_type =~ m/VPLEX/ ) {
      print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"7\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;rate&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;r&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;w&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
    } ## end if ( $st_type =~ m/VPLEX/)
  } ## end if ( $glob_txt =~ m/loc/)

  if ( $item =~ m/max/ ) {

    # For printing peaks (MAX)
    open( FHR, "< $in_file" ) || error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @lines = <FHR>;
    close(FHR);
    open( FHW, "> $file_cmd_max" ) || error( "Can't open $file_cmd_max : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

    #binmode(FHW);

    foreach my $line (@lines) {
      if ( $cust_rep == 1 ) {
        $line =~ s/ --start now-1d --end now / --start $start_unix  --end $end_unix /g;
      }
      $line =~ s/:AVERAGE:/:MAX:/g;
      $line =~ s/ --width=.* --height=/ --width=$width --height=/;
      print FHW "$line";
    } ## end foreach my $line (@lines)
    close(FHW);
    $in_file = $file_cmd_max;
  } ## end if ( $item =~ m/max/ )

  open( FH, "< $in_file" ) || error( "$type: Can't open $in_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);
  my $cmd = "";
  foreach my $line (@lines) {
    if ( $cust_rep == 1 ) {
      $line =~ s/ --start now-1d --end now / --start $start_unix --end $end_unix /g;
      $line =~ s/ --width=.* --height=/ --width=$width --height=/;
    }
    $cmd = $line;
    last;
  } ## end foreach my $line (@lines)
  if ( $cmd eq '' ) {
    error( "$host : Multi graph rrdtool error $cmd " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  RRDp::cmd qq($cmd);
  my $ret = RRDp::read;
  if ( $$ret =~ "ERROR" ) {
    error( "$host : Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( $item =~ m/max/ ) {
    unlink($in_file);    # clean only temp files in /var/tmp
  }
  if ( $$ret =~ "ERROR" ) {
    if ( $$ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$name:$item: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
      print "</tbody></table>";
    }
    else {
      error( "$host:$type:$name:$item: Multi graph rrdtool error : $$ret " . __FILE__ . ":" . __LINE__ );
      print "</tbody></table>";
    }
    return 0;            # must be 0 to do not issue print_png
  } ## end if ( $$ret =~ "ERROR" )

  if ( $glob_txt =~ m/loc/ ) {

    # goes through all 4 files and create one outpud table with cache_hit
    my $volume_prev = "";
    (@lines) = split( /====start /, $$ret );
    foreach my $line (@lines) {
      chomp($line);
      if ( $line !~ m/^==== / ) {
        next;
      }

      $line =~ s/^==== / /g;
      $line =~ s/^ *//g;
      ( my $volume, my $item, my $value ) = split( / +/, $line );
      $value =~ s/OK.*//g;    #last one
      $value =~ s/ //g;
      chomp($value);          # last one record has en enter at the end
      $volume =~ s/=====space=====/ /g;

      #print "001 --$line: $volume:$item:$value <br>";
      if ( !defined($value) || $value eq '' || $volume =~ m/^$/ ) {
        next;
      }

      if ( $volume_prev eq '' || $volume_prev ne $volume ) {
        if ( !$volume_prev eq '' ) {
          print "</tr>\n";
        }
        print "<tr><td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=$type&name=$volume&storage=$st_type&none=none\">$volume</a></td>\n";
        my ( $pool_name, $cap ) = get_name_pool( $volume, $st_type, $host );
        if ( !defined $cap || !isdigit($cap) ) {
          $cap = "N/A";
        }

        if ( defined $pool_name && $pool_name ne "" ) {
          print "<td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=POOL&name=$pool_name&storage=$st_type&none=none\">$pool_name</a></td>\n";
        }
        else {
          print "<td>N/A</td>\n";
        }
        $volume_prev = $volume;
        ### capacity volume
        print "<td align=\"right\">$cap&nbsp;&nbsp;&nbsp;</td>\n";

        ### hosts volume
        my $hosts = get_hosts( $volume, $st_type, $host );
        if ( !defined $hosts || $hosts eq "" ) {
          $hosts = "N/A";
        }
        print "<td>$hosts&nbsp;&nbsp;&nbsp;</td>\n";
      } ## end if ( $volume_prev eq ''...)
      print "<td align=\"right\">$value&nbsp;&nbsp;&nbsp;</td>";
    } ## end foreach my $line (@lines)

    print "</tr></tbody></table>\n";
    print "</center>\n";
  } ## end if ( $glob_txt =~ m/loc/)
  else {
    # global volume top
    # code is the same as in storage.pl!! just printing on stdout

    my $storage     = $host;
    my $null        = "-";
    my $data_rate   = 0;
    my $io_rate     = 0;
    my $volume_prev = "";
    my $read_io;
    my $write_io;
    my $resp_numerator;
    my $read        = $null;
    my $write       = $null;
    my $resp_t_r    = $null;
    my $resp_t_w    = $null;
    my $r_cache_hit = $null;
    my $w_cache_hit = $null;
    my $resp_t      = $null;
    (@lines) = split( /====start /, $$ret );
    my $item_no = 0;
    my $count   = 0;

    foreach my $line (@lines) {
      chomp($line);
      if ( $line !~ m/^==== / ) {
        next;
      }

      $line =~ s/^==== / /g;
      $line =~ s/^ *//g;
      ( my $volume, my $item, my $value ) = split( / +/, $line );
      $value =~ s/OK.*//g;    #last one
      $value =~ s/ //g;
      chomp($value);          # last one record has en enter at the end
      $volume =~ s/=====space=====/ /g;

      #print "001 --$line: $volume:$item:$value \n";
      if ( !defined($value) || $value eq '' || $volume =~ m/^$/ ) {
        next;
      }

      if ( $volume_prev eq "" || $volume_prev eq $volume ) {

        if ( $item eq "read" ) {
          $read = $value;
        }
        if ( $item eq "write" ) {
          $write = $value;
        }
        if ( $item eq "resp_t_r" ) {
          $resp_t_r = $value;
        }
        if ( $item eq "resp_t_w" ) {
          $resp_t_w = $value;
        }
        if ( $item eq "r_cache_hit" ) {
          $r_cache_hit = $value;
        }
        if ( $item eq "w_cache_hit" ) {
          $w_cache_hit = $value;
        }
        if ( $item eq "resp_t" ) {
          $resp_t = $value;
        }
        if ( $item eq "data_rate" ) {
          $data_rate = $value;
        }
        if ( $item eq "io_rate" ) {
          $io_rate = $value;
        }
      } ## end if ( $volume_prev eq ""...)

      $count++;

      if ( $volume_prev eq '' || $volume_prev ne $volume ) {
        if ( !$volume_prev eq '' ) {

          ### pool name add volume top
          my ( $pool_name, $cap ) = get_name_pool( $volume_prev, $st_type, $storage );
          if ( !defined $cap || !isdigit($cap) ) {
            $cap = "N/A";
          }
          if ( !defined $pool_name || $pool_name eq "" ) {
            $pool_name = "N/A";
          }
          ### hosts volume
          my $hosts = get_hosts( $volume_prev, $st_type, $host );
          if ( !defined $hosts || $hosts eq "" ) {
            $hosts = "N/A";
          }

          my $line = "<tr><td style=\"white-space: nowrap\"><a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=VOLUME&name=io&storage=$st_type&item=sum&gui=1&none=none\">$storage</a></td><td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=VOLUME&name=$volume_prev&storage=$st_type&none=none\">$volume_prev</a></td>\n<td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=POOL&name=$pool_name&storage=$st_type&none=none\">$pool_name</a></td>\n<td align=\"right\">$cap&nbsp;&nbsp;&nbsp;</td>\n<td>$hosts&nbsp;&nbsp;&nbsp;</td>\n<td align=\"right\" title=\"io_rate\">$io_rate&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"read_io\">$read_io&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"write_io\">$write_io&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"data_rate\">$data_rate&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"read\">$read&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"write\">$write&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t\">$resp_t&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t_r\">$resp_t_r&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t_w\">$resp_t_w&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"r_cache_hit\">$r_cache_hit&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"w_cache_hit\">$w_cache_hit&nbsp;&nbsp;&nbsp;</td></tr>";

          print "$line\n";

          $volume_prev    = $volume;
          $item_no        = 0;
          $data_rate      = 0;
          $io_rate        = 0;
          $read_io        = $null;
          $write_io       = $null;
          $resp_numerator = 0;
          $read           = $null;
          $write          = $null;
          $resp_t_r       = $null;
          $resp_t_w       = $null;
          $r_cache_hit    = $null;
          $w_cache_hit    = $null;
          $resp_t         = $null;

          if ( $item eq "read" ) {
            $read = $value;
          }
          if ( $item eq "write" ) {
            $write = $value;
          }
          if ( $item eq "resp_t_r" ) {
            $resp_t_r = $value;
          }
          if ( $item eq "resp_t_w" ) {
            $resp_t_w = $value;
          }
          if ( $item eq "r_cache_hit" ) {
            $r_cache_hit = $value;
          }
          if ( $item eq "w_cache_hit" ) {
            $w_cache_hit = $value;
          }
          if ( $item eq "resp_t" ) {
            $resp_t = $value;
          }
          if ( $item eq "data_rate" ) {
            $data_rate = $value;
          }
          if ( $item eq "io_rate" ) {
            $io_rate = $value;
          }

        } ## end if ( !$volume_prev eq ...)
        else {
          $volume_prev = $volume;
        }
      } ## end if ( $volume_prev eq ''...)

      $item_no++;
      if ( $st_type =~ m/DS5K/ ) {
        next;
      }
      if ( $st_type =~ m/VPLEX/ && $item_no > 2 ) {
        next;
      }

      # get data/io rate from read/write values for != DS5k
      if ( $item_no == 1 ) {
        $data_rate = $value;
      }
      if ( $item_no == 2 ) {
        if ( isdigit($data_rate) && isdigit($value) ) {
          my $data_rate_tmp = $data_rate + $value;
          $data_rate = sprintf( "%.1f", $data_rate_tmp );
        }
      } ## end if ( $item_no == 2 )
      if ( $item_no == 3 ) {
        $io_rate = $value;
        $read_io = $value;
      }
      if ( $item_no == 4 ) {
        if ( isdigit($io_rate) && isdigit($value) ) {
          my $io_rate_tmp = $io_rate + $value;
          $write_io = $value;
          $io_rate = sprintf( "%.1f", $io_rate_tmp );
        }
      } ## end if ( $item_no == 4 )
      if ( $st_type !~ m/VNXF|UNITY|VNXE|VNX$|3PAR|NETAPP|VMAX/ && $item_no == 5 ) {
        if ( isdigit($value) && isdigit($read_io) ) {
          $resp_numerator = $value * $read_io;
        }
        else { $resp_numerator = 0; }
      } ## end if ( $st_type !~ m/VNXF|UNITY|VNXE|VNX$|3PAR|NETAPP|VMAX/...)
      if ( $st_type !~ m/VNXF|UNITY|VNXE|VNX$|3PAR|NETAPP|VMAX/ && $item_no == 6 ) {
        if ( isdigit($value) && isdigit($write_io) ) {
          $resp_numerator = $resp_numerator + ( $value * $write_io );
        }
        my $io_rate;
        if ( isdigit($read_io) ) {
          $io_rate = $read_io;
        }
        if ( isdigit($write_io) ) {
          if ( isdigit($io_rate) ) {
            $io_rate = $io_rate + $write_io;
          }
          else {
            $io_rate = $write_io;
          }
        } ## end if ( isdigit($write_io...))
        if ( defined $io_rate && ( !isdigit($io_rate) || $io_rate eq $null ) ) {
          $io_rate = 0;
        }
        if ( !defined $io_rate || $io_rate == 0 ) {
          $io_rate = $null;
        }
        else {
          $resp_t = round( $resp_numerator / $io_rate, 1 );
        }
      } ## end if ( $st_type !~ m/VNXF|UNITY|VNXE|VNX$|3PAR|NETAPP|VMAX/...)
    } ## end foreach my $line (@lines)

    if ( $count > 0 ) {
      ### pool name add volume top
      my ( $pool_name, $cap ) = get_name_pool( $volume_prev, $st_type, $storage );
      if ( !defined $cap || !isdigit($cap) ) {
        $cap = "N/A";
      }
      if ( !defined $pool_name || $pool_name eq "" ) {
        $pool_name = "N/A";
      }
      ### hosts volume
      my $hosts = get_hosts( $volume_prev, $st_type, $host );
      if ( !defined $hosts || $hosts eq "" ) {
        $hosts = "N/A";
      }

      my $line = "<tr><td style=\"white-space: nowrap\"><a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=VOLUME&name=io&storage=$st_type&item=sum&gui=1&none=none\">$storage</a></td><td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=VOLUME&name=$volume_prev&storage=$st_type&none=none\">$volume_prev</a></td>\n<td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=POOL&name=$pool_name&storage=$st_type&none=none\">$pool_name</a></td>\n<td align=\"right\">$cap&nbsp;&nbsp;&nbsp;</td>\n<td>$hosts&nbsp;&nbsp;&nbsp;</td>\n<td align=\"right\" title=\"io_rate\">$io_rate&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"read_io\">$read_io&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"write_io\">$write_io&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"data_rate\">$data_rate&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"read\">$read&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"write\">$write&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t\">$resp_t&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t_r\">$resp_t_r&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t_w\">$resp_t_w&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"r_cache_hit\">$r_cache_hit&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"w_cache_hit\">$w_cache_hit&nbsp;&nbsp;&nbsp;</td></tr>";
      print "$line\n";
    } ## end if ( $count > 0 )

  } ## end else [ if ( $glob_txt =~ m/loc/)]

  return 0;
} ## end sub volumes_top

sub get_hosts {
  my $volume  = shift;
  my $st_type = shift;
  my $storage = shift;
  my @ids     = ();
  my $hosts   = "";

  #first step find id volume

  my $volume_cfg = "$basedir/data/$storage/VOLUME/volumes.cfg";

  if ( !-f $volume_cfg ) {
    return "";
  }
  open( FHR, "< $volume_cfg" ) || error( "Can't open $volume_cfg : $! " . __FILE__ . ":" . __LINE__ ) && return "";
  my @lines = <FHR>;
  close(FHR);

  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" ) { next; }
    my ( $volume_name, $id ) = split( " : ", $line );
    if ( $volume_name ne "$volume" ) { next; }
    @ids = split( ";", $id );
    last;
  } ## end foreach my $line (@lines)
  my $size = scalar @ids;
  if ( $size == 0 ) {
    return "";
  }
  my $host_cfg = "$basedir/data/$storage/HOST/hosts.cfg";
  if ( !-f $host_cfg ) {
    return "";
  }
  open( FHR, "< $host_cfg" ) || error( "Can't open $host_cfg : $! " . __FILE__ . ":" . __LINE__ ) && return "";
  @lines = <FHR>;
  close(FHR);

  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" ) { next; }
    my ( $host_name, $volumes ) = split( " : ", $line );
    my @volumes_ids = split( " ", $volumes );
    foreach my $id (@ids) {
      if ( $st_type eq "SWIZ" ) {
        $id =~ s/^0*//;
        if ( $id eq "" ) {
          $id = 0;
        }
      } ## end if ( $st_type eq "SWIZ")
      if ( $st_type eq "DS8K" ) {
        $id =~ s/^0x//;
      }
      if ( grep { $id eq $_ } @volumes_ids ) {
        $hosts = $hosts . "<a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=HOST&name=$host_name&storage=$st_type&none=none\">$host_name</a>" . ", ";
        last;
      }
    } ## end foreach my $id (@ids)
  } ## end foreach my $line (@lines)
  if ( $hosts ne "" ) {
    $hosts =~ s/^\s+|\s+$//g;
    chop $hosts;
  }
  return $hosts;
} ## end sub get_hosts

sub get_name_pool {
  my $volume_name = shift;
  my $st_type     = shift;
  my $host        = shift;

  my $config = "$basedir/data/$host/config.html";
  if ( !-f $config ) {
    return ( "", "" );
  }
  open( FHR, "< $config" ) || error( "Can't open $config : $! " . __FILE__ . ":" . __LINE__ ) && return ( "", "" );
  my @lines = <FHR>;
  close(FHR);

  my $active_vol  = 0;
  my $active_pool = 0;
  my %pools;
  my $index_id        = "";
  my $index_name_pool = "";
  my $index_id_pool   = "";
  my $index_rg2       = "";
  my $index_rg1       = "";
  my $pool_id         = "";
  my $index_capacity  = "";
  my $index_name      = "";
  my $cap             = "";
  my $volume_pre      = "";
  my $line_pre        = "";
  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" || $line =~ m/^</ ) {
      if ( $st_type eq "DS8K" && $line_pre ne "" ) {
        if ( isdigit($index_id_pool) ) {
          my @data = split( ",", $line_pre );
          if ( defined $data[$index_id_pool] && defined $pools{ $data[$index_id_pool] } ) {
            return ( $pools{ $data[$index_id_pool] }, $cap );
          }
          else {
            return ( "", $cap );
          }
        } ## end if ( isdigit($index_id_pool...))
        return ( "", $cap );
      } ## end if ( $st_type eq "DS8K"...)
      $active_vol  = 0;
      $active_pool = 0;
      next;
    } ## end if ( $line eq "" || $line...)
    if ( $line =~ m/^---------/ && $active_vol == 1 ) {
      $active_vol++;
      next;
    }
    if ( $line =~ m/^---------/ && $active_pool == 1 ) {
      $active_pool++;
      next;
    }
    if ( $line =~ m/Volume Level Configuration/ ) {
      $active_vol = 1;
      next;
    }
    if ( $line =~ m/StorageGroup Level Configuration/ ) {
      $active_vol = 1;
      next;
    }
    if ( $line =~ m/Pool Level Configuration/ ) {
      $active_pool = 1;
      next;
    }
    if ( $line =~ m/Rank Level Configuration/ && $st_type eq "DS8K" ) {
      $active_pool = 1;
      next;
    }
    if ( $line =~ m/Raid Group Level Configuration/ && $st_type eq "HUS" ) {
      $active_pool = 1;
      next;
    }
    if ( $active_vol == 2 ) {
      my $index = -1;
      my @header = split( ",", $line );
      foreach my $element (@header) {
        $index++;
        if ( $element eq "name" || $element eq "volume_name" || $element eq "Volume name" || $element eq "Volume Nickname" || $element eq "Name" || $element eq "SG_ID" ) {
          $index_name = $index;
          next;
        }
        if ( $element eq "volume_id" || $element eq "lun_id" || $element eq "Volume ID" || $element eq "SG_ID" ) {
          $index_id = $index;
          next;
        }
        if ( $element eq "pool_id" || $element eq "CPG ID" || $element eq "Pool ID" || $element eq "Storage pool ID" ) {
          $index_id_pool = $index;
          next;
        }
        if ( $element eq "pool_name" || $element eq "mdisk_grp_name" || $element eq "containing-aggregate-name" || $element eq "CPG name" || $element eq "Pool name" || $element eq "Storage pool" ) {
          $index_name_pool = $index;
          next;
        }
        if ( $element eq "capacity (MB)" || $element eq "capacity" || $element eq "User size [MB]" || $element eq "size-total" || $element eq "Volume Size" || $element eq "LUN_Capacity (MB)" || $element eq "Size (MB)" || $element eq "AllocatedCapacity" || $element eq "DEVICE_CAPACITY" ) {
          $index_capacity = $index;
          next;
        }
        ### raid groups
        if ( $element eq "RG_ID" || $element eq "raid_group_id" ) {
          $index_rg1 = $index;
        }
        if ( $element eq "RG_name" ) {
          $index_rg2 = $index;
        }
      } ## end foreach my $element (@header)
      if ( !isdigit($index_name) ) {
        $index_name = $index_id;
      }
      $active_vol++;
      next;
    } ## end if ( $active_vol == 2 )
    if ( $active_vol == 3 && isdigit($index_name) ) {
      my @data = split( ",", $line );

      if ( !defined $data[$index_name] || $data[$index_name] eq "" ) {
        $index_name = $index_id;
      }
      if ( $st_type eq "DS8K" && $line_pre ne "" && $volume_name ne $data[$index_name] ) {
        next;
      }
      if ( defined $data[$index_name] && $data[$index_name] eq $volume_name ) {
        if ( isdigit($index_capacity) && defined $data[$index_capacity] && isdigit( $data[$index_capacity] ) ) {
          if ( $st_type eq "VSPG" || $st_type eq "HUS" || $st_type eq "3PAR" || $st_type eq "DS5K" || $st_type eq "VNXF" || $st_type eq "VNX" || $st_type eq "UNITY" || $st_type eq "DOTHILL" || $st_type eq "VNXE" || $st_type eq "COMPELLENT" || $st_type eq "OCEANSTOR" || $st_type eq "PURE" ) {
            $cap = convert_capacity( "MB", "GB", $data[$index_capacity] );
          }
          if ( $st_type eq "NETAPP" || $st_type eq "XIV" || $st_type eq "DS8K" || $st_type eq "VMAX" ) {
            if ( $st_type eq "DS8K" ) {
              $volume_pre = $volume_name;
              $line_pre   = $line;
              if ( isdigit($cap) ) {
                $cap = $cap + round( $data[$index_capacity], 1 );    ## sum capacity per virtual volumes for ds8k
                next;
              }
              else {
                $cap = round( $data[$index_capacity], 1 );
                next;
              }
            } ## end if ( $st_type eq "DS8K")
            else {
              $cap = round( $data[$index_capacity], 1 );
            }
          } ## end if ( $st_type eq "NETAPP"...)
          if ( $st_type eq "SWIZ" ) {
            $cap = convert_capacity( "B", "GB", $data[$index_capacity] );
          }
        } ## end if ( isdigit($index_capacity...))
        if ( isdigit($index_name_pool) && defined $data[$index_name_pool] && $data[$index_name_pool] ne "" && $st_type ne "DS8K" ) {

          #if ($st_type eq "VNX" && isdigit($index_id_pool) && defined $data[$index_id_pool] && $data[$index_id_pool] ne ""){
          #  $data[$index_name_pool] = "RG_" . $data[$index_id_pool];
          #}
          return ( $data[$index_name_pool], $cap );
        } ## end if ( isdigit($index_name_pool...))
        if ( isdigit($index_id_pool) && defined $data[$index_id_pool] && $data[$index_id_pool] ne "" ) {
          if ( defined $pools{ $data[$index_id_pool] } && $pools{ $data[$index_id_pool] } ne "" ) {
            return ( $pools{ $data[$index_id_pool] }, $cap );
          }
          else {
            $pool_id = $data[$index_id_pool];
            next;
          }
        } ## end if ( isdigit($index_id_pool...))
        if ( isdigit($index_rg2) && defined $data[$index_rg2] && $data[$index_rg2] ne "" ) {
          return ( $data[$index_rg2], $cap );
        }

        if ( isdigit($index_rg1) && defined $data[$index_rg1] && $data[$index_rg1] ne "" && $st_type eq "VNX" ) {
          my $pool_n = "RG_" . $data[$index_rg1];
          return ( $pool_n, $cap );
        }
        if ( isdigit($index_rg1) && defined $data[$index_rg1] && $data[$index_rg1] ne "" && $st_type eq "HUS" ) {
          $pool_id = $data[$index_rg1];
          next;
        }
      } ## end if ( defined $data[$index_name...])
    } ## end if ( $active_vol == 3 ...)
    if ( $active_pool == 2 ) {
      my $index = -1;
      my @header = split( ",", $line );
      foreach my $element (@header) {
        $index++;
        if ( $element eq "pool_id" || $element eq "CPG ID" || $element eq "Pool ID" || $element eq "Storage pool ID" || $element eq "id" ) {
          $index_id_pool = $index;
          next;
        }
        if ( $element eq "pool_name" || $element eq "mdisk_grp_name" || $element eq "containing-aggregate-name" || $element eq "CPG name" || $element eq "Pool name" || $element eq "Storage pool" || $element eq "name" || $element eq "Pool Name" ) {
          $index_name_pool = $index;
          next;
        }
      } ## end foreach my $element (@header)
      $active_pool++;
      next;
    } ## end if ( $active_pool == 2)
    if ( $active_pool == 3 && ( isdigit($index_id_pool) || isdigit($index_name_pool) ) ) {
      my @data = split( ",", $line );
      if ( isdigit($index_id_pool) && defined $data[$index_id_pool] && $data[$index_id_pool] ne "" ) {
        if ( isdigit($index_name_pool) && defined $data[$index_name_pool] && $data[$index_name_pool] ne "" ) {
          $pools{ $data[$index_id_pool] } = $data[$index_name_pool];
          next;
        }
        else {
          $pools{ $data[$index_id_pool] } = $data[$index_id_pool];
        }
      } ## end if ( isdigit($index_id_pool...))
    } ## end if ( $active_pool == 3...)
  } ## end foreach my $line (@lines)
  if ( defined $pools{$pool_id} && $pools{$pool_id} ne "" ) {
    return ( $pools{$pool_id}, $cap );
  }
  else {
    return ( "", $cap );
  }
} ## end sub get_name_pool

sub encode_base64 {
  if ( $] >= 5.006 ) {
    require bytes;
    if ( bytes::length( $_[0] ) > length( $_[0] )
      || ( $] >= 5.008 && $_[0] =~ /[^\0-\xFF]/ ) )
    {
      require Carp;
      Carp::croak("The Base64 encoding is only defined for bytes");
    } ## end if ( bytes::length( $_...))
  } ## end if ( $] >= 5.006 ) ([)

  use integer;

  my $eol = $_[1];
  $eol = "\n" unless defined $eol;

  my $res = pack( "u", $_[0] );

  # Remove first character of each line, remove newlines
  $res =~ s/^.//mg;
  $res =~ s/\n//g;

  $res =~ tr|` -_|AA-Za-z0-9+/|;    # `# help emacs
                                    # fix padding at the end
  my $padding = ( 3 - length( $_[0] ) % 3 ) % 3;
  $res =~ s/.{$padding}$/'=' x $padding/e if $padding;

  # break encoded string into lines of no more than 76 characters each
  if ( length $eol ) {
    $res =~ s/(.{1,76})/$1$eol/g;
  }
  return $res;
} ## end sub encode_base64

sub convert_capacity {
  my $unit_input  = shift;
  my $unit_output = shift;
  my $value       = shift;
  $unit_input = uc($unit_input);

  if ( $unit_input eq $unit_output ) {
    return $value;
  }
  if ( $unit_input eq "B" && $unit_output eq "GB" ) {
    return round( $value / ( 1024 * 1024 * 1024 ), 1 );
  }
  if ( $unit_input eq "MB" && $unit_output eq "GB" ) {
    return round( $value / 1024, 1 );
  }
  if ( $unit_input eq "MB" && $unit_output eq "TB" ) {
    return round( $value / ( 1024 * 1024 ) );
  }
  if ( $unit_input eq "MB" && $unit_output eq "PB" ) {
    return round( $value / ( 1024 * 1024 * 1024 ) );
  }
  if ( $unit_input eq "GB" && $unit_output eq "TB" ) {
    return round( $value / (1024) );
  }
  if ( $unit_input eq "GB" && $unit_output eq "PB" ) {
    return round( $value / ( 1024 * 1024 ) );
  }
  if ( $unit_input eq "TB" && $unit_output eq "PB" ) {
    return round( $value / (1024) );
  }
  if ( $unit_input eq "GB" && $unit_output eq "MB" ) {
    return round( $value * 1024, 0 );
  }
  if ( $unit_input eq "TB" && $unit_output eq "MB" ) {
    return round( $value * ( 1024 * 1024 ), 0 );
  }
  if ( $unit_input eq "TB" && $unit_output eq "GB" ) {
    return round( $value * (1024) );
  }
  if ( $unit_input eq "PB" && $unit_output eq "MB" ) {
    return round( $value * ( 1024 * 1024 * 1024 ), 0 );
  }
  return "";
} ## end sub convert_capacity

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

# if digit or hexa then true
sub ishexa {
  my $digit = shift;

  if ( !defined($digit) || $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  # hexa filter
  $digit_work =~ s/[a|b|c|d|e|f|A|B|C|D|E|F]//g;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  #main::error ("there was expected a digit but a string is there, field: , value: $digit");
  return 0;
} ## end sub ishexa

sub create_csv_total_capacity {
  my $host     = shift;
  my $type     = shift;
  my $lpar     = shift;
  my $item     = shift;
  my $time     = shift;
  my $name_out = shift;
  my $wrkdir   = shift;
  my $act_time = shift;
  my $step     = shift;
  my %inventory_capacity;

  #find actual storage
  my $actual_time = time();
  my @data = <$basedir/data/*>;
  foreach my $storage_full (@data) {
    my $storage_name = basename($storage_full);
    my $storage_space = check_space_name($storage_name);
    my $storage_full_file = "$basedir/data/$storage_name/VOLUME/volumes.cfg";
    if (! -f "$storage_full_file" ){next;}
    my $png_time = ( stat("$storage_full_file") )[9];
    if ( ( 172800 + $png_time ) < $actual_time ) {
      if ( !defined $ENV{DEMO} ) {    # demo.stor2rrd.com has static volumes.cfg files
        next;                         # skip not being updated storages
      }
    }
    #my $dirname  = dirname($storage_full);
    #my $dirname1 = dirname($dirname);
    my $storage  = $storage_name;

    my $st_type = storage_type( "$basedir/data", $storage, $item, $name );
    my $ret = create_text_total_capacity( $storage, $type, $lpar, $item, $time, $name_out, $wrkdir, $act_time, $step, $st_type, "CSV" );
    if ( $ret eq "0" || $ret eq "1" ) { next; }
    my @data_in = split( "\n", $ret );
    my $i = 0;
    foreach my $line (@data_in) {
      chomp $line;
      if ( $line eq "" ) { next; }
      $line =~ s/^\s+|\s+$//g;
      $line =~ s/\s+/ /g;
      ( my $item, my $value ) = split( " ", $line );
      if ( defined $item && $item ne "" && defined $value && $value ne "" ) {

        #$item =~ s/^\s+|\s+$//g;
        #$value = s/^\s+|\s+$//g;
        $inventory_capacity{$storage}{$i}{$item} = $value;
        $i++;

      } ## end if ( defined $item && ...)
    } ## end foreach my $line (@data_in)

    #if (defined $name && defined $value && $name ne "" && $value ne ""){
    #  $name =~ s/^\s+|\s+$//g;
    #  $value =~ s/^\s+|\s+$//g;
    #  print  STDERR " $storage $name federer $value\n";
    #  $inventory_capacity{$storage}{$name} = $value;
    #}
  } ## end foreach my $storage_full (<$basedir/data/*/VOLUME/volumes.cfg>)
  my $header = "Storage;Type;Total;Used;Free";
  print "$header\n";
  my @data_out = "";
  push( @data_out, "$header\n" );
  foreach my $storage ( keys %inventory_capacity ) {
    my $used  = "";
    my $free  = "";
    my $total = "";
    my $i     = 0;
    my $type  = "";
    foreach my $counter ( sort { $a <=> $b } keys %{ $inventory_capacity{$storage} } ) {
      foreach my $name ( keys %{ $inventory_capacity{$storage}{$counter} } ) {
        ### used free total
        if ( $name eq "used" ) {
          $used = $inventory_capacity{$storage}{$counter}{$name};
          $type = "Total";
          $i++;
        }
        if ( $name eq "free" ) {
          $free = $inventory_capacity{$storage}{$counter}{$name};
          $type = "Total";
          $i++;
        }
        if ( $name eq "total" ) {
          $total = $inventory_capacity{$storage}{$counter}{$name};
          $type  = "Total";
          $i++;
        }
        if ( $name =~ /^used/ && $name ne "used" ) {
          my $pom_name = $name;
          $pom_name =~ s/used//g;
          $type = $pom_name;
          $used = $inventory_capacity{$storage}{$counter}{$name};
          $i++;
        } ## end if ( $name =~ /^used/ ...)
        if ( $name =~ /^free/ && $name ne "free" ) {
          my $pom_name = $name;
          $pom_name =~ s/free//g;
          $type = $pom_name;
          $free = $inventory_capacity{$storage}{$counter}{$name};
          $i++;
        } ## end if ( $name =~ /^free/ ...)
        if ( $name =~ /^total/ && $name ne "total" ) {
          my $pom_name = $name;
          $pom_name =~ s/total//g;
          $type  = $pom_name;
          $total = $inventory_capacity{$storage}{$counter}{$name};
          $i++;
        } ## end if ( $name =~ /^total/...)
        my $mod = $i % 3;
        if ( $mod == 0 && $i != 0 ) {
          push( @data_out, "$storage;$type;$total;$used;$free\n" );
          print "$storage;$type;$total;$used;$free\n";
          $type  = "";
          $total = "";
          $used  = "";
          $free  = "";
          $i     = 0;
        } ## end if ( $mod == 0 && $i !=...)

      } ## end foreach my $name ( keys %{ ...})
    } ## end foreach my $counter ( sort ...)

  } ## end foreach my $storage ( keys ...)

} ## end sub create_csv_total_capacity

# it prints out LAST capacity values (used and free) in text
# http://nim/stor2rrd-cgi/detail-graph.sh?host=storwize01&type=POOL&name=cap_total&item=sum&time=d&detail=0&none=1436028554
sub create_text_total_capacity {
  my $host     = shift;
  my $type     = shift;
  my $lpar     = shift;
  my $item     = shift;
  my $time     = shift;
  my $name_out = shift;
  my $wrkdir   = shift;
  my $act_time = shift;
  my $step     = shift;
  my $st_type  = shift;
  my $csv      = shift;

  #print STDERR "$host,$type,$lpar,$item,$st_type\n";
  my $host_space = check_space_name($host);
  if ( $type eq "ALL" ) {
    if ( $st_type eq "VMAX" || $st_type eq "3PAR" ) {
      $type = "TIER";
    }
    else {
      $type = "POOL";
    }
  } ## end if ( $type eq "ALL" )

  my $i              = 0;
  my $cmd            = "";
  my $nothing_to_sum = 0;

  if ( $st_type eq "VMAX" && $type eq "POOL" ) {    #VMAX cap_total from TIERs
                                                    # Only if exist TIER data, it do not have to exist everywhere!
    foreach my $line (<$basedir/data/$host_space/TIER/*-cap\.rrd>) {
      $type = "TIER";
      last;
    }
  } ## end if ( $st_type eq "VMAX"...)

  if ( $st_type =~ m/3PAR/ ) {
    $type = "TIER";
  }

  my $file_list = "$type/*-cap\.rrd";

  # capacity files : POOL/0-cap.rrd

  if ( $st_type =~ m/HUS/ || $st_type =~ m/NETAPP/ || $st_type =~ m/VSPG/ || $st_type =~ m/VNXF/ || $st_type =~ m/DOTHILL/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/OCEANSTOR/ || $st_type =~ m/PURE/ ) {
    $file_list = "$type/*\.rrd";
  }

  # just to get last time
  my $last_record = 0;
  foreach my $line (<$basedir/data/$host_space/$file_list>) {
    chomp($line);
    $nothing_to_sum = 1;
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing
                           # find out time stamp of last data update
                           # take just one volume to keep it simple
    RRDp::cmd qq(last "$line");
    my $last_time = RRDp::read;
    chomp($$last_time);

    if ( isdigit($$last_time) && $$last_time > $last_record ) {
      $last_record = $$last_time;
    }
  } ## end foreach my $line (<$basedir/data/$host/$file_list>)

  if ( $last_record == 0) {

    # there is nothing to sum at the moment, it is something wring as storage must have some pool
    if ( !defined $csv ) {
      get_capacity_static( $basedir, $host, "total 0\nfree 0\nused 0\n" );
      return 0;
    }
    if (defined $csv && $st_type ne "XIV"){return 0;}
  } ## end if ( $last_record == 0)

  # use step 60 here, it is only what works for every step size which rrdtool have
  # note step=300 will ot work for rrdtool step size 600 and higher!
  $step = 60;

  # keep 1h step here as you might know real rrd step, when is 60 and here is 300 then it will never proint anything, 3600 step is always there
  my $start_record = $last_record - 7200;
  $last_record = $last_record - 3600;
  my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($last_record);

  my $cmd = "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --start \\\"$start_record\\\"";
  $cmd .= " --end \\\"$last_record\\\"";

  $cmd .= " --step=$step"; 

  my @file_list_base = "";

  # following loop is copied from storage.pl : draw_graph_sum ()
  foreach my $line (<$basedir/data/$host_space/$file_list>) {
    chomp($line);
    $nothing_to_sum = 1;
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    $file_list_base[$i] = basename($line);    # Keep name only for 3PAR tiers
    $file_list_base[$i] =~ s/-cap.rrd//;
    if ( !defined( $file_list_base[$i] ) || $file_list_base[$i] eq '' ) {
      next;
      something wrong;
    }

    if ( $st_type =~ m/HUS/ || $st_type =~ m/NETAPP/ || $st_type =~ m/VSPG/ || $st_type =~ m/VNXF/ || $st_type =~ m/VNX$/ || $st_type =~ m/UNITY/ || $st_type =~ m/VNXE/ || $st_type =~ m/DOTHILL/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/OCEANSTOR/ || $st_type =~ m/PURE/) {

      # bulid RRDTool cmd
      $cmd .= " DEF:free${i}_u=\\\"$line\\\":free:AVERAGE";
      if ( $st_type =~ m/VNX$/ || $st_type =~ m/UNITY/ || $st_type =~ m/VNXE/ ) {
        $cmd .= " DEF:used${i}_u=\\\"$line\\\":used:AVERAGE";
      }
      else {
        $cmd .= " DEF:used${i}_u=\\\"$line\\\":real:AVERAGE";
      }

      # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
      $cmd .= " CDEF:free${i}=free${i}_u,UN,0,free${i}_u,IF";
      $cmd .= " CDEF:used${i}=used${i}_u,UN,0,used${i}_u,IF";
      $cmd .= " CDEF:tot${i}=free${i},used${i},+";
    } ## end if ( $st_type =~ m/HUS/...)
    else {
      if ( $st_type eq "VMAX" || $st_type eq "3PAR" ) {

        # bulid RRDTool cmd
        $cmd .= " DEF:tot${i}_u=\\\"$line\\\":total:AVERAGE";
        $cmd .= " DEF:used${i}_u=\\\"$line\\\":used:AVERAGE";

        # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
        $cmd .= " CDEF:tot${i}=tot${i}_u,UN,0,tot${i}_u,IF";
        $cmd .= " CDEF:used${i}=used${i}_u,UN,0,used${i}_u,IF";
        $cmd .= " CDEF:free${i}=tot${i},used${i},-";
      } ## end if ( $st_type eq "VMAX"...)
      else {
        if ( $st_type =~ m/SWIZ/ ) {

          # bulid RRDTool cmd
          if ( $line =~ m/-nl-cap\.rrd/ ) {

            # special tier tier0-nl addedd in 2.00 in separate file
            $cmd .= " DEF:tot${i}_u=\\\"$line\\\":total:AVERAGE";
            $cmd .= " DEF:free${i}_u=\\\"$line\\\":free:AVERAGE";
            $cmd .= " DEF:used${i}_u=\\\"$line\\\":used:AVERAGE";

            $cmd .= " CDEF:tottier0nl${i}=tot${i}_u,UN,0,tot${i}_u,IF";
            $cmd .= " CDEF:freetier0nl${i}=free${i}_u,UN,0,free${i}_u,IF";
            $cmd .= " CDEF:usedtier0nl${i}=used${i}_u,UN,0,used${i}_u,IF";

            # just zero it
            $cmd .= " CDEF:tot${i}=tottier0nl${i},0,*";
            $cmd .= " CDEF:free${i}=tottier0nl${i},0,*";
            $cmd .= " CDEF:used${i}=tottier0nl${i},0,*";

            $cmd .= " CDEF:tottier0${i}=tottier0nl${i},0,*";
            $cmd .= " CDEF:freetier0${i}=tottier0nl${i},0,*";
            $cmd .= " CDEF:usedtier0${i}=tottier0nl${i},0,*";

            $cmd .= " CDEF:tottier1${i}=tottier0nl${i},0,*";
            $cmd .= " CDEF:freetier1${i}=tottier0nl${i},0,*";
            $cmd .= " CDEF:usedtier1${i}=tottier0nl${i},0,*";

            $cmd .= " CDEF:tottier2${i}=tottier0nl${i},0,*";
            $cmd .= " CDEF:freetier2${i}=tottier0nl${i},0,*";
            $cmd .= " CDEF:usedtier2${i}=tottier0nl${i},0,*";
          } ## end if ( $line =~ m/-nl-cap\.rrd/)
          else {
            $cmd .= " DEF:tot${i}_u=\\\"$line\\\":total:AVERAGE";
            $cmd .= " DEF:free${i}_u=\\\"$line\\\":free:AVERAGE";
            $cmd .= " DEF:used${i}_u=\\\"$line\\\":real:AVERAGE";

            # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
            $cmd .= " CDEF:tot${i}=tot${i}_u,UN,0,tot${i}_u,IF";
            $cmd .= " CDEF:free${i}=free${i}_u,UN,0,free${i}_u,IF";
            $cmd .= " CDEF:used${i}=used${i}_u,UN,0,used${i}_u,IF";

            $cmd .= " DEF:tottier0${i}_u=\\\"$line\\\":tier0cap:AVERAGE";
            $cmd .= " DEF:freetier0${i}_u=\\\"$line\\\":tier0free:AVERAGE";
            $cmd .= " CDEF:tottier0${i}=tottier0${i}_u,UN,0,tottier0${i}_u,IF";
            $cmd .= " CDEF:freetier0${i}=freetier0${i}_u,UN,0,freetier0${i}_u,IF";
            $cmd .= " CDEF:usedtier0${i}=tottier0${i},freetier0${i},-";

            # just 0 into tier0-nl here, it is not in this file
            $cmd .= " CDEF:tottier0nl${i}=tottier0${i},0,*";
            $cmd .= " CDEF:freetier0nl${i}=freetier0${i},0,*";
            $cmd .= " CDEF:usedtier0nl${i}=usedtier0${i},0,*";

            $cmd .= " DEF:tottier1${i}_u=\\\"$line\\\":tier1cap:AVERAGE";
            $cmd .= " DEF:freetier1${i}_u=\\\"$line\\\":tier1free:AVERAGE";
            $cmd .= " CDEF:tottier1${i}=tottier1${i}_u,UN,0,tottier1${i}_u,IF";
            $cmd .= " CDEF:freetier1${i}=freetier1${i}_u,UN,0,freetier1${i}_u,IF";
            $cmd .= " CDEF:usedtier1${i}=tottier1${i},freetier1${i},-";

            $cmd .= " DEF:tottier2${i}_u=\\\"$line\\\":tier2cap:AVERAGE";
            $cmd .= " DEF:freetier2${i}_u=\\\"$line\\\":tier2free:AVERAGE";
            $cmd .= " CDEF:tottier2${i}=tottier2${i}_u,UN,0,tottier2${i}_u,IF";
            $cmd .= " CDEF:freetier2${i}=freetier2${i}_u,UN,0,freetier2${i}_u,IF";
            $cmd .= " CDEF:usedtier2${i}=tottier2${i},freetier2${i},-";
          } ## end else [ if ( $line =~ m/-nl-cap\.rrd/)]
        } ## end if ( $st_type =~ m/SWIZ/)
        else {
          $cmd .= " DEF:tot${i}_u=\\\"$line\\\":total:AVERAGE";
          $cmd .= " DEF:free${i}_u=\\\"$line\\\":free:AVERAGE";
          $cmd .= " DEF:used${i}_u=\\\"$line\\\":used:AVERAGE";

          # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
          $cmd .= " CDEF:tot${i}=tot${i}_u,UN,0,tot${i}_u,IF";
          $cmd .= " CDEF:free${i}=free${i}_u,UN,0,free${i}_u,IF";
          $cmd .= " CDEF:used${i}=used${i}_u,UN,0,used${i}_u,IF";

          $cmd .= " DEF:tottier0${i}_u=\\\"$line\\\":tier0cap:AVERAGE";
          $cmd .= " DEF:freetier0${i}_u=\\\"$line\\\":tier0free:AVERAGE";
          $cmd .= " CDEF:tottier0${i}=tottier0${i}_u,UN,0,tottier0${i}_u,IF";
          $cmd .= " CDEF:freetier0${i}=freetier0${i}_u,UN,0,freetier0${i}_u,IF";
          $cmd .= " CDEF:usedtier0${i}=tottier0${i},freetier0${i},-";

          $cmd .= " DEF:tottier1${i}_u=\\\"$line\\\":tier1cap:AVERAGE";
          $cmd .= " DEF:freetier1${i}_u=\\\"$line\\\":tier1free:AVERAGE";
          $cmd .= " CDEF:tottier1${i}=tottier1${i}_u,UN,0,tottier1${i}_u,IF";
          $cmd .= " CDEF:freetier1${i}=freetier1${i}_u,UN,0,freetier1${i}_u,IF";
          $cmd .= " CDEF:usedtier1${i}=tottier1${i},freetier1${i},-";

          $cmd .= " DEF:tottier2${i}_u=\\\"$line\\\":tier2cap:AVERAGE";
          $cmd .= " DEF:freetier2${i}_u=\\\"$line\\\":tier2free:AVERAGE";
          $cmd .= " CDEF:tottier2${i}=tottier2${i}_u,UN,0,tottier2${i}_u,IF";
          $cmd .= " CDEF:freetier2${i}=freetier2${i}_u,UN,0,freetier2${i}_u,IF";
          $cmd .= " CDEF:usedtier2${i}=tottier2${i},freetier2${i},-";
        } ## end else [ if ( $st_type =~ m/SWIZ/)]
      } ## end else [ if ( $st_type eq "VMAX"...)]
    } ## end else [ if ( $st_type =~ m/HUS/...)]
    $i++;
  } ## end foreach my $line (<$basedir/data/$host/$file_list>)
  if ( $nothing_to_sum == 0) {

    # there is nothing to sum at the moment, it is something wring as storage must have some pool
    if ( !defined $csv ) {
      get_capacity_static( $basedir, $host, "total 0\nfree 0\nused 0\n" );
      return 0;
    }
    if (defined $csv && $st_type ne "XIV"){
      return 0;
    }
  } ## end if ( $nothing_to_sum ==...)
  #
  # Total summ
  #
  # Load all items into the $cmd
  my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
  $cmd .= " CDEF:tot_sum=tot0,0,+";
  for ( ; $index_actual < $i; $index_actual++ ) {
    $cmd .= ",tot${index_actual},+";
  }

  # AVERAGE might not be there, if there is not loaded all data untill "now" then averaging is bad
  # LAST is used but LAST = now - $start_time
  # it means that after initiall installation data appears here after $start_time  == 1/2 hour
  # corerction since 1.32, used 1h step, reason ois above, it means that capacity will work 1 hour  after initial install
  $cmd .= " PRINT:tot_sum:LAST:\\\"total %8.3lf\\\"";

  my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
  $cmd .= " CDEF:free_sum=free0,0,+";
  for ( ; $index_actual < $i; $index_actual++ ) {
    $cmd .= ",free${index_actual},+";
  }
  $cmd .= " PRINT:free_sum:LAST:\\\"free %8.3lf\\\"";

  my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
  $cmd .= " CDEF:used_sum=used0,0,+";
  for ( ; $index_actual < $i; $index_actual++ ) {
    $cmd .= ",used${index_actual},+";
  }
  $cmd .= " PRINT:used_sum:LAST:\\\"used %8.3lf\\\"";

  if ( $st_type !~ m/HUS/ && $st_type !~ m/NETAPP/ && $st_type !~ m/3PAR/ && $st_type !~ m/VSPG/ && $st_type !~ m/VNXF/ && $st_type !~ m/VNX$/ && $st_type !~ m/UNITY/ && $st_type !~ m/VNXE/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/VMAX/ && $st_type !~ m/COMPELLENT/ && $st_type !~ m/OCEANSTOR/ && $st_type !~ m/PURE/ ) {

    # HUS does not have tiers

    #
    # tier0 summ
    #
    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:tot_sum_tier0=tottier00,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",tottier0${index_actual},+";
    }
    $cmd .= " PRINT:tot_sum_tier0:LAST:\\\"totalTIER0 %8.3lf\\\"";

    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:free_sum_tier0=freetier00,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",freetier0${index_actual},+";
    }
    $cmd .= " PRINT:free_sum_tier0:LAST:\\\"freeTIER0 %8.3lf\\\"";

    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:used_sum_tier0=usedtier00,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",usedtier0${index_actual},+";
    }
    $cmd .= " PRINT:used_sum_tier0:LAST:\\\"usedTIER0 %8.3lf\\\"";

    if ( $st_type =~ m/SWIZ/ ) {

      # special tier tier0-nl addedd in 2.00 in separate file
      #
      # tier0-NL summ
      #
      my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
      $cmd .= " CDEF:tot_sum_tier0nl=tottier0nl0,0,+";
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",tottier0nl${index_actual},+";
      }
      $cmd .= " PRINT:tot_sum_tier0nl:LAST:\\\"totalTIER0-NL %8.3lf\\\"";

      my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
      $cmd .= " CDEF:free_sum_tier0nl=freetier0nl0,0,+";
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",freetier0nl${index_actual},+";
      }
      $cmd .= " PRINT:free_sum_tier0nl:LAST:\\\"freeTIER0-NL %8.3lf\\\"";

      my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
      $cmd .= " CDEF:used_sum_tier0nl=usedtier0nl0,0,+";
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",usedtier0nl${index_actual},+";
      }
      $cmd .= " PRINT:used_sum_tier0nl:LAST:\\\"usedTIER0-NL %8.3lf\\\"";
    } ## end if ( $st_type =~ m/SWIZ/)

    #
    # tier1 summ
    #
    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:tot_sum_tier1=tottier10,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",tottier1${index_actual},+";
    }
    $cmd .= " PRINT:tot_sum_tier1:LAST:\\\"totalTIER1 %8.3lf\\\"";

    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:free_sum_tier1=freetier10,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",freetier1${index_actual},+";
    }
    $cmd .= " PRINT:free_sum_tier1:LAST:\\\"freeTIER1 %8.3lf\\\"";

    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:used_sum_tier1=usedtier10,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",usedtier1${index_actual},+";
    }
    $cmd .= " PRINT:used_sum_tier1:LAST:\\\"usedTIER1 %8.3lf\\\"";

    #
    # tier2 summ
    #
    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:tot_sum_tier2=tottier20,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",tottier2${index_actual},+";
    }
    $cmd .= " PRINT:tot_sum_tier2:LAST:\\\"totalTIER2 %8.3lf\\\"";

    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:free_sum_tier2=freetier20,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",freetier2${index_actual},+";
    }
    $cmd .= " PRINT:free_sum_tier2:LAST:\\\"freeTIER2 %8.3lf\\\"";

    my $index_actual = 1;    # must be 1 here, othervise reuslt is 2x more
    $cmd .= " CDEF:used_sum_tier2=usedtier20,0,+";
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",usedtier2${index_actual},+";
    }
    $cmd .= " PRINT:used_sum_tier2:LAST:\\\"usedTIER2 %8.3lf\\\"";

  } ## end if ( $st_type !~ m/HUS/...)

  if ( $st_type =~ m/3PAR/ ) {
    my $index_actual = 0;
    for ( ; $index_actual < $i; $index_actual++ ) {
      my $tier_name = $file_list_base[$index_actual];
      $cmd .= " PRINT:tot${index_actual}:LAST:\\\"total$tier_name %8.3lf\\\"";
      $cmd .= " PRINT:free${index_actual}:LAST:\\\"free$tier_name %8.3lf\\\"";
      $cmd .= " PRINT:used${index_actual}:LAST:\\\"used$tier_name %8.3lf\\\"";
    } ## end for ( ; $index_actual <...)
  } ## end if ( $st_type =~ m/3PAR/)
  if ( $st_type =~ m/XIV/ && defined $csv){
    my $file = "$basedir/data/$host/capacity.txt";
    if (-f $file){
      open( FH, "< $file" ) || error( "Cannot read $file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      my @file_all = <FH>;
      close(FH);
      my $string = "";
      foreach my $line (@file_all){
        chomp $line;
        $line =~ s/^\s+|\s+$//g;
        if ($line =~ m/total|free|used/){
          $string = $string . $line . "\n";
        }
      }
      return $string;
    }
  }

  $cmd =~ s/\\"/"/g;

  #print STDERR "$cmd\n";

  RRDp::cmd qq($cmd);
  my $ret = RRDp::read;
  if ( $$ret =~ "ERROR" ) {
    error( "POOL: sum_cap rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  $$ret =~ s/^0x0//;
  $$ret =~ s/-nan/-/g;
  $$ret =~ s/NaNQ/-/g;
  $$ret =~ s/NAN/-/g;
  $$ret =~ s/NaN/-/g;
  $$ret =~ s/print\[.*\] =//g;
  $$ret =~ s/ "//g;
  $$ret =~ s/"//g;
  chomp($$ret);

  #print STDERR "\n$$ret\n";
  my $find_data = $$ret;
  $find_data =~ s/TIER1//g;
  $find_data =~ s/TIER2//g;

  #check if any real data returned, if not then use statical stuff (not RRD file)  which is updeated once a day
  if ( $find_data =~ m/[0-9]/ ) {

    # some data is in, print it and exit
    if ( defined $csv && $csv eq "CSV" ) {
      return $$ret;
    }
    else {
      print "$$ret\ntime $start_human\n";
      return 1;
    }
  } ## end if ( $find_data =~ m/[0-9]/)
  if ( !defined $csv ) {
    get_capacity_static( $basedir, $host, $$ret, $start_human );
  }

  return 1;
} ## end sub create_text_total_capacity

sub get_capacity_static {
  my ( $basedir, $host, $rrd_out, $start_human ) = @_;

  my $file_static = "$basedir/data/$host/capacity.txt";
  if ( !-f "$file_static" ) {

    # static file is missing, print zeros
    print "$rrd_out\ntime $start_human\n";
    return 1;
  } ## end if ( !-f "$file_static")

  open( FHR, "< $file_static" ) || error( "Can't open $file_static: $! " . __FILE__ . ":" . __LINE__ ) && print "$rrd_out\ntime $start_human\n" && return 1;
  my @capacity = <FHR>;
  close(FHR);

  my $found = 0;
  foreach my $line (@capacity) {
    chomp($line);
    if ( $line =~ m/[0-9]/ ) {

      # found data, print it
      $found = 1;
      print "$line\n";
    } ## end if ( $line =~ m/[0-9]/)
  } ## end foreach my $line (@capacity)
  if ( $found == 1 ) {
    return 1;
  }

  # no find any valid data, print zeros
  print "$rrd_out\ntime $start_human\n";

  return 0;
} ## end sub get_capacity_static

# error handling
sub no_data_yet {
  my $text     = shift;
  my $act_time = localtime();
  my $basedir  = $ENV{INPUTDIR};

  print STDERR "$act_time: $host:$type:$name:$item:$detail: no data yet: $text\n";
  print "\n";    # print missing enter to finish header HTTP

  # print "no data yet" picture
  open( PNG, "< $basedir/html/no-data.png" ) || error( "Cannot open  $basedir/html/error.png: $! " . __FILE__ . ":" . __LINE__ ) && return (0);
  binmode(PNG);
  while ( read( PNG, $b, 4096 ) ) {
    print "$b";
  }
  close(PNG);

  exit(1);       # exit here
} ## end sub no_data_yet

# it prints out LAST capacity values (used and free) in text
# http://nim/stor2rrd-cgi/detail-graph.sh?host=storwize01&type=POOL&name=cap_pool&item=sum&time=d&detail=0&none=1436028554
sub create_text_pool_capacity {
  my $host     = shift;
  my $type     = shift;
  my $lpar     = shift;
  my $item     = shift;
  my $time     = shift;
  my $name_out = shift;
  my $wrkdir   = shift;
  my $act_time = shift;
  my $step     = shift;
  my $st_type  = shift;
  my $i        = 0;

  #print STDERR "$host,$type,$lpar,$item\n";

  if ( $st_type =~ m/3PAR/ ) {

    #return "not available";
    # use TIER capacity instead
    $type = "TIER";
  } ## end if ( $st_type =~ m/3PAR/)

  my $file_list = "$type/*-cap\.rrd";

  # capacity files : POOL/0-cap.rrd

  if ( $st_type =~ m/HUS/ || $st_type =~ m/NETAPP/ || $st_type =~ m/VSPG/ || $st_type =~ m/VNXF/ || $st_type =~ m/DOTHILL/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/OCEANSTOR/ || $st_type =~ m/PURE/ ) {
    $file_list = "$type/*\.rrd";
  }
  my $req_time = time() - 86400;

  # get last time when data was stored
  my $last_record = 0;
  my $host_space = check_space_name($host);
  foreach my $line (<$basedir/data/$host_space/$file_list>) {
    chomp($line);
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;    # skip old pool files
    }

    # find out time stamp of last data update
    # take just one volume to keep it simple
    RRDp::cmd qq(last "$line");
    my $last_time = RRDp::read;
    chomp($$last_time);
    if ( isdigit($$last_time) && $$last_time > $last_record ) {
      $last_record = $$last_time;
    }
  } ## end foreach my $line (<$basedir/data/$host/$file_list>)
  if ( $last_record == 0 ) {
    print "No pool found\n";
    return 0;
  }

  # use step 60 here, it is only what works for every step size which rrdtool have
  # note step=300 will ot work for rrdtool step size 600 and higher!
  $step = 60;

  my $start_record = $last_record - 7200;
  $last_record = $last_record - 3600;
  my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($last_record);

  my $cmd_header = "$graph_cmd \\\"$name_out\\\"";
  $cmd_header .= " --start \\\"$start_record\\\"";
  $cmd_header .= " --end \\\"$last_record\\\"";
  $cmd_header .= " --step=$step";

  # following loop is copied from storage.pl : draw_graph_sum ()
  foreach my $line (<$basedir/data/$host_space/$file_list>) {
    chomp($line);
    if ( $st_type =~ m/SWIZ/ && $line =~ m/nl-cap.rrd$/ ) {
      next;    # this is tier capacity only, not pool
    }

    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;    # skip old pool files
    }

    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing
    my $cmd = $cmd_header;

    if ( $st_type =~ m/HUS/ || $st_type =~ m/NETAPP/ || $st_type =~ m/VSPG/ || $st_type =~ m/VNXF/ || $st_type =~ m/DOTHILL/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/OCEANSTOR/ || $st_type =~ m/PURE/ ) {

      # bulid RRDTool cmd
      $cmd .= " DEF:free_u=\\\"$line\\\":free:AVERAGE";
      $cmd .= " DEF:used_u=\\\"$line\\\":real:AVERAGE";

      # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
      $cmd .= " CDEF:free=free_u,UN,0,free_u,IF";
      $cmd .= " CDEF:used=used_u,UN,0,used_u,IF";
      $cmd .= " CDEF:tot=free,used,+";
      $cmd .= " PRINT:tot:LAST:\\\"%8.3lf,\\\"";
      $cmd .= " PRINT:free:LAST:\\\"%8.3lf,\\\"";
      $cmd .= " PRINT:used:LAST:\\\"%8.3lf\\\"";
    } ## end if ( $st_type =~ m/HUS/...)
    else {
      if ( $st_type =~ m/SWIZ/ ) {

        # Storwize sends virtual and real capacity in the GUI
        # bulid RRDTool cmd
        $cmd .= " DEF:tot_u=\\\"$line\\\":total:AVERAGE";
        $cmd .= " DEF:free_u=\\\"$line\\\":free:AVERAGE";
        $cmd .= " DEF:used_u=\\\"$line\\\":used:AVERAGE";
        $cmd .= " DEF:virt_u=\\\"$line\\\":virt:AVERAGE";
        $cmd .= " DEF:real_u=\\\"$line\\\":real:AVERAGE";

        # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
        $cmd .= " CDEF:tot=tot_u,UN,0,tot_u,IF";
        $cmd .= " CDEF:free=free_u,UN,0,free_u,IF";
        $cmd .= " CDEF:used=used_u,UN,0,used_u,IF";
        $cmd .= " CDEF:virt=virt_u,UN,0,virt_u,IF";
        $cmd .= " CDEF:real=real_u,UN,0,real_u,IF";
        $cmd .= " PRINT:tot:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:free:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:used:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:virt:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:real:LAST:\\\"%8.3lf\\\"";
      } ## end if ( $st_type =~ m/SWIZ/)
      elsif ( $st_type eq "VMAX" && -f "$basedir/data/$host/VMAX-V8" ) {
        $cmd .= " DEF:tot_u=\\\"$line\\\":total:AVERAGE";
        $cmd .= " DEF:used_u=\\\"$line\\\":used:AVERAGE";
        $cmd .= " DEF:subscription=\\\"$line\\\":subscription:AVERAGE";
        $cmd .= " CDEF:tot=tot_u,UN,0,tot_u,IF";
        $cmd .= " CDEF:used=used_u,UN,0,used_u,IF";
        $cmd .= " CDEF:free=tot,used,-";
        $cmd .= " PRINT:tot:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:free:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:used:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:subscription:LAST:\\\"%8.3lf\\\"";
      } ## end elsif ( $st_type eq "VMAX"...)
      elsif ( $st_type eq "VMAX" && !-f "$basedir/data/$host/VMAX-V8" ) {
        $cmd .= " DEF:tot_u=\\\"$line\\\":total:AVERAGE";
        $cmd .= " DEF:used_u=\\\"$line\\\":used:AVERAGE";
        $cmd .= " CDEF:tot=tot_u,UN,0,tot_u,IF";
        $cmd .= " CDEF:used=used_u,UN,0,used_u,IF";
        $cmd .= " CDEF:free=tot,used,-";
        $cmd .= " PRINT:tot:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:free:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:used:LAST:\\\"%8.3lf\\\"";
      } ## end elsif ( $st_type eq "VMAX"...)
      else {
        # bulid RRDTool cmd
        $cmd .= " DEF:tot_u=\\\"$line\\\":total:AVERAGE";
        $cmd .= " DEF:free_u=\\\"$line\\\":free:AVERAGE";
        $cmd .= " DEF:used_u=\\\"$line\\\":used:AVERAGE";

        # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
        $cmd .= " CDEF:tot=tot_u,UN,0,tot_u,IF";
        $cmd .= " CDEF:free=free_u,UN,0,free_u,IF";
        $cmd .= " CDEF:used=used_u,UN,0,used_u,IF";
        $cmd .= " PRINT:tot:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:free:LAST:\\\"%8.3lf,\\\"";
        $cmd .= " PRINT:used:LAST:\\\"%8.3lf\\\"";
      } ## end else [ if ( $st_type =~ m/SWIZ/)]
    } ## end else [ if ( $st_type =~ m/HUS/...)]
    $cmd =~ s/\\"/"/g;
    $i++;

    RRDp::cmd qq($cmd);
    my $ret = RRDp::read;
    if ( $$ret =~ "ERROR" ) {
      error( "POOL: sum_cap rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
      next;
    }
    $$ret =~ s/^0x0//;
    $$ret =~ s/-nan/-/g;
    $$ret =~ s/NaNQ/-/g;
    $$ret =~ s/NAN/-/g;
    $$ret =~ s/NaN/-/g;
    $$ret =~ s/print\[.*\] =//g;
    $$ret =~ s/ "//g;
    $$ret =~ s/"//g;
    $$ret =~ s/ //g;
    $$ret =~ s/\n//g;
    chomp($$ret);

    #check if any real data returned, if not then use statical stuff (not RRD file)  which is updeated once a day
    if ( $$ret =~ m/[1-9]/ ) {
      if ( $st_type eq "VMAX" && -f "$basedir/data/$host/VMAX-V8" ) {

        # some data is in, print it and exit
        my $pool_id = basename($line);
        $pool_id =~ s/-cap\.rrd//;
        $pool_id =~ s/\.rrd//;
        my $pool_name = pool_translate( $wrkdir, $host, $pool_id );

        my ( $tot, $free, $used, $subsc ) = split( /,/, $$ret );
        print "$pool_name,$tot,$free,$used,$start_human,$subsc\n";
      } ## end if ( $st_type eq "VMAX"...)
      else {
        # some data is in, print it and exit
        my $pool_id = basename($line);
        $pool_id =~ s/-cap\.rrd//;
        $pool_id =~ s/\.rrd//;
        my $pool_name = pool_translate( $wrkdir, $host, $pool_id );
        print "$pool_name,$$ret,$start_human\n";
      } ## end else [ if ( $st_type eq "VMAX"...)]
    } ## end if ( $$ret =~ m/[1-9]/)
  } ## end foreach my $line (<$basedir/data/$host/$file_list>)
  if ( $i == 0 ) {

    # there is nothing to sum at the moment, it is something wrong as storage must have some pool
    print "No pool found\n";
    return 0;
  } ## end if ( $i == 0 )
  return 1;
} ## end sub create_text_pool_capacity

# POOL name translation --> from ID
sub pool_translate {
  my $wrkdir  = shift;
  my $host    = shift;
  my $pool_id = shift;

  if ( -f "$wrkdir/$host/pool.cfg" ) {
    open( FHR, "< $wrkdir/$host/pool.cfg" ) || return $pool_id;
    my @lines_translate = <FHR>;
    close(FHR);
    foreach my $linep (@lines_translate) {
      chomp($linep);
      ( my $id, my $name_pool ) = split( /:/, $linep );
      if ( $pool_id =~ m/^$id$/ ) {
        return $name_pool;
      }
    } ## end foreach my $linep (@lines_translate)
  } ## end if ( -f "$wrkdir/$host/pool.cfg")
  return $pool_id;
} ## end sub pool_translate

sub host_data_selection {
  my $wrkdir  = shift;
  my $host    = shift;
  my $type    = shift;
  my $st_type = shift;

  my $wrkdir_space = check_space_name($wrkdir);
  my $host_space   = check_space_name($host);
  my @files        = <$wrkdir_space/$host_space/$type/*>;
  my $rrd          = grep {/\.rrd$/} @files;

  if ( $st_type !~ m/VSPG/ ) {
    return 0;
  }

  my $VSPG_NATIVE_HOSTS = 1;
  if ( $st_type =~ m/VSPG/ && defined $ENV{VSPG_NATIVE_HOSTS} ) {

    # VSPG_NATIVE_HOSTS=0 forces to use original volume based host graphs (etc/.magic)
    $VSPG_NATIVE_HOSTS = $ENV{VSPG_NATIVE_HOSTS};
    if ( defined $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST} && $VSPG_NATIVE_HOSTS == 0 ) {
      $VSPG_NATIVE_HOSTS = 1;    # host are specified, switch default to host data back
      foreach my $storage_act ( split( / /, $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST} ) ) {
        if ( $host =~ m/^$storage_act$/ ) {
          $VSPG_NATIVE_HOSTS = 0;
        }
      }
    } ## end if ( defined $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST...})
  } ## end if ( $st_type =~ m/VSPG/...)

  if ( $rrd > 0 && $VSPG_NATIVE_HOSTS == 1 ) {
    return 1;                    # use host original
  }
  else {
    return 0;                    # use volume data instead
  }
} ## end sub host_data_selection

sub volumes_top_global {
  my $host       = shift;
  my $type       = shift;
  my $name       = shift;
  my $item       = shift;
  my $time       = shift;
  my $st_type    = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $basedir    = shift;

  # global volume top
  print "<br><center><table class=\"lparsearch tablesorter\" data-sortby=\"6\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Storage&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp; <span>GB</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;rate&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;data&nbsp;rate&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;read&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;write&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;resp&nbsp;t&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;r&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;resp&nbsp;t&nbsp;w&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;r&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;&nbsp;w&nbsp;cache&nbsp;hit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";

  # loop through all storages
  $name = "top";    # change it from topglobal
  my $time_act = time();
  foreach my $storage_full (<$basedir/data/*/VOLUME/volumes.cfg>) {
    my $png_time = ( stat("$storage_full") )[9];
    if ( ( 172800 + $png_time ) < $time_act ) {
      if ( !defined $ENV{DEMO} ) {    # demo.stor2rrd.com has static volumes.cfg files
        next;                         # skip not being updated storages
      }
    }
    my $dirname  = dirname($storage_full);
    my $dirname1 = dirname($dirname);
    my $storage  = basename($dirname1);

    $st_type = storage_type( "$basedir/data", $storage, $item, $name );
    volumes_top( $storage, $type, $name, $item, $time, $st_type, $start_unix, $end_unix, "glob" );
  } ## end foreach my $storage_full (<$basedir/data/*/VOLUME/volumes.cfg>)

  print "</tr></tbody></table>\n";
  print "</center>\n";

  return 0;    # must be 0 to do not issue print_png
} ## end sub volumes_top_global

sub storage_type {
  my $wrkdir  = shift;
  my $host    = shift;
  my $item    = shift;
  my $name    = shift;
  my $st_type = "";

  if ( -f "$wrkdir/$host/SWIZ" ) {
    $st_type = "SWIZ";
  }
  if ( -f "$wrkdir/$host/XIV" ) {
    $st_type = "XIV";
  }
  if ( -f "$wrkdir/$host/DS8K" ) {
    $st_type = "DS8K";
  }
  if ( -f "$wrkdir/$host/DS5K" ) {
    $st_type = "DS5K";
  }
  if ( -f "$wrkdir/$host/SAN-BRCD" ) {
    $st_type = "SAN-BRCD";
  }
  if ( -f "$wrkdir/$host/SAN-CISCO" ) {
    $st_type = "SAN-CISCO";
  }
  if ( -f "$wrkdir/$host/HUS" ) {
    $st_type = "HUS";
  }
  if ( -f "$wrkdir/$host/3PAR" ) {
    $st_type = "3PAR";
  }
  if ( -f "$wrkdir/$host/NETAPP" ) {
    $st_type = "NETAPP";
  }
  if ( -f "$wrkdir/$host/VSPG" ) {
    $st_type = "VSPG";
  }
  if ( -f "$wrkdir/$host/VNXF" ) {
    $st_type = "VNXF";
  }
  if ( -f "$wrkdir/$host/VNX" ) {
    $st_type = "VNX";
  }
  if ( -f "$wrkdir/$host/UNITY" ) {
    $st_type = "UNITY";
  }
  if ( -f "$wrkdir/$host/VMAX" ) {
    $st_type = "VMAX";
  }
  if ( -f "$wrkdir/$host/VPLEX" ) {
    $st_type = "VPLEX";
  }
  if ( -f "$wrkdir/$host/VNXE" ) {
    $st_type = "VNXE";
  }
  if ( -f "$wrkdir/$host/DOTHILL" ) {
    $st_type = "DOTHILL";
  }
  if ( -f "$wrkdir/$host/COMPELLENT" ) {
    $st_type = "COMPELLENT";
  }
  if ( -f "$wrkdir/$host/OCEANSTOR" ) {
    $st_type = "OCEANSTOR";
  }
  if ( -f "$wrkdir/$host/PURE" ) {
    $st_type = "PURE";
  }

  if ( $host =~ "totals" && $name =~ "totals" && $item =~ "^san" ) {
    $st_type = "SAN-BRCD";
  }

  return $st_type;
} ## end sub storage_type

sub create_text_fast_sg {
  my $host     = shift;
  my $type     = shift;
  my $lpar     = shift;
  my $item     = shift;
  my $time     = shift;
  my $name_out = shift;
  my $wrkdir   = shift;
  my $act_time = shift;
  my $step     = shift;
  my $st_type  = shift;

  my @rrd_files = <$wrkdir/$host/FAST-SG/*.rrd>;
  my $tier_cfg  = "$wrkdir/$host/tier.cfg";
  my @tiers;
  my %data;

  my $act_time   = time();
  my $start_time = $act_time - 3600;    #last hour

  if ( -f "$tier_cfg" ) {
    open( CFG, "< $tier_cfg" ) || error( "file does not exists : $tier_cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
    @tiers = <CFG>;
    close(CFG);
  }
  else {
    error( "file does not exists : $tier_cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  foreach my $rrd (@rrd_files) {
    chomp $rrd;

    if ( !-f "$rrd" ) {
      error( "Something wrong! rrd file does not exists : $rrd " . __FILE__ . ":" . __LINE__ ) && next;
    }

    my $rrd_line = $rrd;
    $rrd_line =~ s/^$wrkdir\/$host\/FAST-SG\///;
    $rrd_line =~ s/\.rrd$//;

    my $tier_name = "";
    my $sg_name   = "";

    foreach my $tier_line (@tiers) {
      chomp $tier_line;
      my ( $tier_n, undef ) = split( ":", $tier_line );

      if ( $rrd_line =~ m/-$tier_n$/ ) {
        $tier_name = $tier_n;
        $sg_name   = $rrd_line;
        $sg_name =~ s/-$tier_n$//;
        last;
      } ## end if ( $rrd_line =~ m/-$tier_n$/)
    } ## end foreach my $tier_line (@tiers)

    my $cmd = "";
    $cmd .= "graph \\\"$basedir/tmp/name.png\\\"";
    $cmd .= " --start \\\"$start_time\\\"";
    $cmd .= " --end \\\"now\\\"";
    $cmd .= " --width=400";
    $cmd .= " --height=150";
    $cmd .= " --step=300";
    $cmd .= " DEF:tier_cap=\\\"$rrd\\\":tier_cap:AVERAGE";
    $cmd .= " PRINT:tier_cap:AVERAGE:\\\"%10.2lf tier_cap\\\"";
    $cmd =~ s/\\"/"/g;
    RRDp::cmd qq($cmd);
    my $data = RRDp::read;

    my $tier_cap = "";
    foreach my $data_line ( split( /\n/, $$data ) ) {
      chomp($data_line);

      if ( $data_line =~ m/^0x/ ) { next; }
      $data_line =~ s/^\s+//;
      $data_line =~ s/\s+$//;

      if ( $data_line =~ m/tier_cap$/ ) {
        $tier_cap = $data_line;
        $tier_cap =~ s/\s+tier_cap$//;
      }
    } ## end foreach my $data_line ( split...)

    if ( defined $sg_name && $sg_name ne "" && defined $tier_name && $tier_name ne "" ) {
      if ( defined $tier_cap && isdigit($tier_cap) ) {
        $data{$sg_name}{$tier_name}{'tier_cap'} = sprintf( '%.2f', $tier_cap );
      }
    }
  } ## end foreach my $rrd (@rrd_files)

  my $header = "Storage group";

  foreach my $tier_line (@tiers) {
    chomp $tier_line;
    my ( $tier_n, undef ) = split( ":", $tier_line );
    $header .= ",$tier_n";
  }
  print "$header\n";

  foreach my $sg ( sort keys %data ) {
    my $data_line = "$sg";
    foreach my $tier_line (@tiers) {
      chomp $tier_line;
      my ( $tier_n, undef ) = split( ":", $tier_line );
      if ( defined $data{$sg}{$tier_n}{'tier_cap'} && isdigit( $data{$sg}{$tier_n}{'tier_cap'} ) ) {
        $data_line .= ",$data{$sg}{$tier_n}{'tier_cap'}";
      }
      else {
        $data_line .= ",";
      }
    } ## end foreach my $tier_line (@tiers)
    print "$data_line\n";
  } ## end foreach my $sg ( sort keys ...)

  return 1;
} ## end sub create_text_fast_sg

