# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;
use Date::Parse;
use Math::BigInt;
use File::Copy;
use RRDp;

# our modules
use Storcfg2html;
use LoadDataModule;
use Xorux_lib;

#use lib qw (/opt/freeware/lib/perl/5.8.0);
# no longer need to use "use lib qw" as the library PATH is already in PERL5LIB env var (lpar2rrd.cfg)

# set unbuffered stdout
$| = 1;

# get cmd line params
my $version       = "$ENV{version}";
my $version_patch = "$ENV{version_patch}";
my $storage       = $ENV{STORAGE};
my $host          = $ENV{STORAGE};
my $st_type       = $ENV{STORAGE_TYPE};
my $webdir        = $ENV{WEBDIR};
my $bindir        = $ENV{BINDIR};
my $basedir       = $ENV{INPUTDIR};
my $rrdtool       = $ENV{RRDTOOL};
my $DEBUG         = $ENV{DEBUG};
my $pic_col       = $ENV{PICTURE_COLOR};
my $STEP          = $ENV{SAMPLE_RATE};
if ( defined $ENV{STOR2RRD_STEP} ) {
  $STEP = $ENV{STOR2RRD_STEP};    # it is in $STEP variable, viz load.sh, so far implemented only for SVC/Storwize switch to 60 secs
}
my $stor2txt   = $ENV{STOR2TXT};
my $upgrade    = $ENV{UPGRADE};
my $h          = $ENV{HOSTNAME};
my $tmp_dir    = "$basedir/tmp";
my $new_change = "$tmp_dir/$version-run";

# switch to old POOL usage : summ of volumes instead of using direct POOL/pool.rrd
my $ds5k_pool_org = 0;
if ( defined $ENV{DS5K_POOL_ORG} && isdigit( $ENV{DS5K_POOL_ORG} ) ) {
  $ds5k_pool_org = $ENV{DS5K_POOL_ORG};
}
my $xiv_pool_org = 0;
if ( defined $ENV{XIV_POOL_ORG} && isdigit( $ENV{XIV_POOL_ORG} ) ) {
  $xiv_pool_org = $ENV{XIV_POOL_ORG};
}

#print "++ $host $storage_user $basedir $webdir $STEP\n";
my $wrkdir = "$basedir/data";

if ( -f "$bindir/premium.pl" ) {
  require "$bindir/premium.pl";
}
else {
  require "$bindir/standard.pl";
}

# Global definitions
my $not_execute_rrdtool = 1;                     # new GUI, create only RRDTool command files
my $no_time             = 1380;                  # says the time interval when RRDTOOL consideres a gap in input data, usually 3 * 5 + 2 = 17mins
                                                 # to avoid small gaps every midnight
                                                 # --> use 23 minutes as default
                                                 # XIV is further adjusted to 33 minutes (1980) as it uses 15 mins step
my $YEAR_REFRESH        = 72000;                 # 20 hour, minimum time in sec when yearly graphs are updated (refreshed)
my $MONTH_REFRESH       = 20000;                 # 5 hour, minimum time in sec when monthly graphs are updated (refreshed)
my $WEEK_REFRESH        = 7000;                  # minimum time in sec when weekly  graphs are updated (refreshed)
my $font_def_normal     = "--font=DEFAULT:7:";
my $font_tit_normal     = "--font=TITLE:9:";
my $width               = 400;
my $width_trend         = $width * 2 + 128;      # 93 pixels for 1 X axiss legend and Tobi promo + 50 table space

if ( $upgrade == 1 ) {
  $YEAR_REFRESH = 0;                             # to force creation of trend graphs
}

my $IO_MAX       = $ENV{VOLUME_IO_MAX};          # which volumes place into volume aggregated graph
my $DATA_MAX     = $ENV{VOLUME_DATA_MAX};        # which volumes place into volume aggregated graph
my $RESPONSE_MAX = $ENV{VOLUME_RESPONSE_MAX};    # which volumes place into volume aggregated graph
my $CACHE_MAX    = $ENV{VOLUME_CACHE_MAX};       # which volumes place into volume aggregated graph

if ( !defined($IO_MAX) || $IO_MAX eq '' || isdigit($IO_MAX) == 0 ) {
  $IO_MAX = 50;                                  # default IO max
}
if ( !defined($DATA_MAX) || $DATA_MAX eq '' || isdigit($DATA_MAX) == 0 ) {
  $DATA_MAX = 500;                               # default data max  (kBytes)
}
if ( !defined($RESPONSE_MAX) || $RESPONSE_MAX eq '' || isdigit($RESPONSE_MAX) == 0 ) {
  $RESPONSE_MAX = 1;                             # default response time max  (ms)
}
if ( !defined($CACHE_MAX) || $CACHE_MAX eq '' || isdigit($CACHE_MAX) == 0 ) {
  $CACHE_MAX = 1;                                # default cache max (kBytes)
}
my $disable_rrdtool_tag = "--interlaced";        # just nope string, it is deprecated anyway
my $delimiter           = "XORUX";               # this is for rrdtool print lines for clickable legend
my $delim_com           = "XOR_COM";             #delimiter when comments from rrd are needed

rrdtool_graphv();

my @color = (
  "#FF0000", "#0000FF", "#FFFF00", "#00FFFF", "#FFA500", "#00FF00", "#808080", "#FF00FF", "#800080",
  "#FDD017", "#0000A0", "#3BB9FF", "#008000", "#800000", "#ADD8E6", "#F778A1", "#800517", "#736F6E", "#F52887",
  "#C11B17", "#5CB3FF", "#A52A2A", "#FF8040", "#2B60DE", "#736AFF", "#1589FF", "#98AFC7", "#8D38C9", "#307D7E",
  "#F6358A", "#151B54", "#6D7B8D", "#FDEEF4", "#FF0080", "#F88017", "#2554C7", "#FFF8C6", "#D4A017", "#306EFF",
  "#151B8D", "#9E7BFF", "#EAC117", "#E0FFFF", "#15317E", "#6C2DC7", "#FBB917", "#FCDFFF", "#15317E", "#254117",
  "#FAAFBE", "#357EC7", "#4AA02C", "#38ACEC", "#C0C0C0"
);

my $color_max          = 53;
my @color_lpar         = "";
my $color_rest_of_vols = "#000000";
my @mdisk_trans        = "";          # mdisk name translation, must be global one
my @drive_trans        = "";          # drive name translation, must be global one
my @fs_trans           = "";          # fileystem name translation, must be global one
my @nas_trans          = "";          # fileystem name translation, must be global one
my @port_trans         = "";          # fileystem name translation, must be global one
my @node_trans         = "";          # fileystem name translation, must be global one
my @volume_folder_trans = "";
my @io_group_trans = "";

# global for pool translation
my @pool_tr_table_id   = "";
my @pool_tr_table_name = "";

my $prem = premium();
print "STOR2RRD $prem version $version ($version_patch)\n" if $DEBUG;
print "Host           : $h\n"                              if $DEBUG;

# run touch tmp/$version-run once a day (first run after the midnight) to force recreation of the GUI
my $once = once_a_day("$basedir/tmp/$version");

my $act_time = localtime();

if ( !-d "$webdir" ) {
  die "Pls set correct path to Web server pages, it does not exist here: $webdir\n";
}

cfg_config_change();

# use 1h alert here to be sure (it hang reading from FIFO in RB)
my $timeout = 3600;
if ( $upgrade == 1 ) {
  $timeout = $timeout * 10;
}
eval {
  my $act_time = localtime();
  local $SIG{ALRM} = sub { die "$act_time: $storage : died in SIG ALRM : load_host"; };
  print "Alarm          : $timeout\n" if $DEBUG;
  alarm($timeout);

  # start RRD via a pipe
  RRDp::start "$rrdtool";

  print "RRDTool version: $RRDp::VERSION \n";

  load_host( $act_time, $once );

  # close RRD pipe
  RRDp::end;
  alarm(0);
};

if ($@) {
  kill -15, $$;    # clean up
  sleep(3);
  RRDp::end;       # it must be behind kill as it might hang itself
  if ( $@ =~ /died in SIG ALRM/ ) {
    my $act_time = localtime();
    error( "$storage : load_host timed out after : $timeout seconds " . __FILE__ . ":" . __LINE__ );
    exit(1);
  }
  else {
    error( "$storage : load_host failed: $@ " . __FILE__ . ":" . __LINE__ );
    exit(1);
  }
} ## end if ($@)

exit(0);

sub load_host {
  my $act_time = shift;
  my $once     = shift;

  my $rrd_ver = $RRDp::VERSION;
  if ( isdigit($rrd_ver) && $rrd_ver > 1.35 ) {
    $disable_rrdtool_tag = "--disable-rrdtool-tag";
  }

  if ( $host eq '' ) {
    error( "host not found : could not parse storage name : $host " . __FILE__ . ":" . __LINE__ );
    exit(1);
  }
  if ( $st_type eq '' ) {
    error( "host type not found : could not parse storage name : $st_type " . __FILE__ . ":" . __LINE__ );
    exit(1);
  }
  print "Storage        : $host\n"                                                                                              if $DEBUG;
  print "Storage type   : $st_type\n"                                                                                           if $DEBUG;
  print "IO/DATA limits : volume limits : IO_MAX:$IO_MAX; DATA_MAX:$DATA_MAX RESPONSE_MAX:$RESPONSE_MAX CACHE_MAX:$CACHE_MAX\n" if $DEBUG;

  create_dir_structure( $webdir, $wrkdir, $host, $st_type, $tmp_dir );

  if ( $st_type =~ m/^SWIZ$/ ) {
    if ( !-f "$wrkdir/$host/SWIZ" ) {
      `touch "$wrkdir/$host/SWIZ"`;
    }
    LoadDataModule::load_data_svc_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^SWIZ$/)
  if ( $st_type =~ m/^DS8K$/ ) {
    if ( !-f "$wrkdir/$host/DS8K" ) {
      `touch "$wrkdir/$host/DS8K"`;
    }
    LoadDataModule::load_data_ds8k_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^DS8K$/)
  if ( $st_type =~ m/^XIV$/ ) {
    if ( !-f "$wrkdir/$host/XIV" ) {
      `touch "$wrkdir/$host/XIV"`;
    }
    $no_time = 1980;    # XIV heart beat time increase due to bigger step (15 minutes)
    LoadDataModule::load_data_xiv_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^XIV$/)
  if ( $st_type =~ m/^DS5K$/ ) {
    if ( !-f "$wrkdir/$host/DS5K" ) {
      `touch "$wrkdir/$host/DS5K"`;
    }
    LoadDataModule::load_data_ds5_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^DS5K$/)
  if ( $st_type =~ m/^HUS$/ ) {
    if ( !-f "$wrkdir/$host/HUS" ) {
      `touch "$wrkdir/$host/HUS"`;
    }
    LoadDataModule::load_data_hus_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^HUS$/)
  if ( $st_type =~ m/^NETAPP$/ ) {
    if ( !-f "$wrkdir/$host/NETAPP" ) {
      `touch "$wrkdir/$host/NETAPP"`;
    }
    LoadDataModule::load_data_netapp_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^NETAPP$/)
  if ( $st_type =~ m/^3PAR/ ) {
    if ( !-f "$wrkdir/$host/3PAR" ) {
      `touch "$wrkdir/$host/3PAR"`;
    }
    LoadDataModule::load_data_3par_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^3PAR/)

  if ( $st_type =~ m/^VSPG$/ ) {
    if ( !-f "$wrkdir/$host/VSPG" ) {
      `touch "$wrkdir/$host/VSPG"`;
    }
    LoadDataModule::load_data_vspg_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^VSPG$/)
  if ( $st_type =~ m/^VNXF$/ ) {
    if ( !-f "$wrkdir/$host/VNXF" ) {
      `touch "$wrkdir/$host/VNXF"`;
    }
    LoadDataModule::load_data_vnxf_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^VNXF$/)
  if ( $st_type =~ m/^VNX$/ ) {
    if ( !-f "$wrkdir/$host/VNX" ) {
      `touch "$wrkdir/$host/VNX"`;
    }
    LoadDataModule::load_data_vnxb_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^VNX$/)
  if ( $st_type =~ m/^UNITY$/ ) {
    if ( !-f "$wrkdir/$host/UNITY" ) {
      `touch "$wrkdir/$host/UNITY"`;
    }
    LoadDataModule::load_data_unity_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^UNITY$/)
  if ( $st_type =~ m/^VMAX$/ ) {
    if ( !-f "$wrkdir/$host/VMAX" ) {
      `touch "$wrkdir/$host/VMAX"`;
    }
    LoadDataModule::load_data_vmax_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^VMAX$/)
  if ( $st_type =~ m/^VPLEX$/ ) {
    if ( !-f "$wrkdir/$host/VPLEX" ) {
      `touch "$wrkdir/$host/VPLEX"`;
    }
    LoadDataModule::load_data_vplex_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^VPLEX$/)
  if ( $st_type =~ m/^VNXE$/ ) {
    if ( !-f "$wrkdir/$host/VNXE" ) {
      `touch "$wrkdir/$host/VNXE"`;
    }
    LoadDataModule::load_data_vnxe_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^VNXE$/)
  if ( $st_type =~ m/^DOTHILL$/ ) {
    if ( !-f "$wrkdir/$host/DOTHILL" ) {
      `touch "$wrkdir/$host/DOTHILL"`;
    }
    LoadDataModule::load_data_dothill_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^DOTHILL$/)
  if ( $st_type =~ m/^COMPELLENT$/ ) {
    if ( !-f "$wrkdir/$host/COMPELLENT" ) {
      `touch "$wrkdir/$host/COMPELLENT"`;
    }
    LoadDataModule::load_data_compellent_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^COMPELLENT$/)
  if ( $st_type =~ m/^OCEANSTOR$/ ) {
    if ( !-f "$wrkdir/$host/OCEANSTOR" ) {
      `touch "$wrkdir/$host/OCEANSTOR"`;
    }
    LoadDataModule::load_data_oceanstor_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^OCEANSTOR$/)
  if ( $st_type =~ m/^PURE$/ ) {
    if ( !-f "$wrkdir/$host/PURE" ) {
      `touch "$wrkdir/$host/PURE"`;
    }
    LoadDataModule::load_data_pure_all( $host, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  } ## end if ( $st_type =~ m/^PURE$/)

  # must be here before Storcfg2html::cfg_ds8k
  my $type = "VOLUME";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );
  my $ret = set_colors_initial( $host, $type, "volumes" );
  if ( $ret == 2 ) {
    if ( $st_type eq "UNITY" ) {
      error( "$host : Volumes not found! EMC UNITY is probably configured only for NAS (file). Skip graphs for volumes! " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error("Exiting $host as some core cfg file is missing: config.html || pool.cfg || volumes.cfg\n                 === Check http://www.stor2rrd.com/install-storage.htm for more ===  ");
      return 1;    # when does not exist config files config.html, pool.cfg and volumes.cfg
    }
  }

  # storage independent so far ....
  $ret = Storcfg2html::cfg_ds8k( $host, $wrkdir, $webdir, $act_time, $DEBUG, $st_type );
  if ( $ret == 2 ) {
    if ( $st_type eq "UNITY" ) {
      error( "$host : Volumes not found! EMC UNITY is probably configured only for NAS (file). Skip graphs for volumes! " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error("Exiting $host as some core cfg file is missing: config.html || pool.cfg || volumes.cfg\n                 === Check http://www.stor2rrd.com/install-storage.htm for more ===  ");
      return 1;    # when does not exist config files config.html, pool.cfg and volumes.cfg
    }
  }

  clean_old_data_files( $host, $wrkdir, $DEBUG );

  # translate pool ids into names into global variable
  pool_translate( $act_time, $wrkdir, $host );

  if ( $st_type =~ m/^DS8K$/ ) {
    draw_graphs_ds8k( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^SWIZ$/ ) {
    draw_graphs_svc( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^XIV$/ ) {
    draw_graphs_xiv( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^DS5K$/ ) {
    draw_graphs_ds5( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^HUS$/ ) {
    draw_graphs_hus( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^NETAPP$/ ) {
    draw_graphs_netapp( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^3PAR$/ ) {
    draw_graphs_3par( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^VSPG$/ ) {
    draw_graphs_vspg( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^VNXF$/ ) {
    draw_graphs_vnxf( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^VNX$/ ) {
    draw_graphs_vnxb( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^UNITY$/ ) {
    draw_graphs_unity( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^VMAX$/ ) {
    draw_graphs_vmax( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^VPLEX$/ ) {
    draw_graphs_vplex( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^VNXE$/ ) {
    draw_graphs_vnxe( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^DOTHILL$/ ) {
    draw_graphs_dothill( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^COMPELLENT$/ ) {
    draw_graphs_compellent( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^OCEANSTOR$/ ) {
    draw_graphs_oceanstor( $host, $act_time, $st_type );
  }
  if ( $st_type =~ m/^PURE$/ ) {
    draw_graphs_pure( $host, $act_time, $st_type );
  }
  print_storage_sys( $wrkdir, $host );

  #if ( $once == 1  || $upgrade == 1 ) {
  #global_volumes_top($storage,$st_type,$webdir,$act_time);
  #global_hosts_top($storage,$st_type,$webdir,$act_time);
  #}

  return 1;
} ## end sub load_host

sub draw_graphs_netapp {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  if ( -f "$wrkdir/$host/drive.cfg" ) {
    open( FHR, "< $wrkdir/$host/drive.cfg" ) || error( "Can't open $wrkdir/$host/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @drive_trans = <FHR>;
    close(FHR);
  }

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  # POOL
  #
  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all_pool( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  # NODE
  #
  $type = "NODE";
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );

  # response time per node is very strange, totally different then others, skipping it for so far
  #$item = "resp_t_r";
  #draw_all ($host,$type,$item,$st_type);
  #$item = "resp_t_w";
  #draw_all ($host,$type,$item,$st_type);
  #$item = "resp_t";
  #draw_all ($host,$type,$item,$st_type);
  $item = "cifs_io_rate-3_io_rate-4_io_rate-iscsi_io_rate-fcp_io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "cifs_read_io-3_read_io-4_read_io-iscsi_read_io-fcp_read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "cifs_write_io-3_write_io-4_write_io-iscsi_write_io-fcp_write_io";
  draw_all( $host, $type, $item, $st_type );
  if ( !-f "$wrkdir/$host/7MODE" ) {

    # 7mode does not provide data throughput
    $item = "cifs_data_rate-3_data_rate-4_data_rate-iscsi_data_rate-fcp_data_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "cifs_read-3_read-4_read-iscsi_read-fcp_read";
    draw_all( $host, $type, $item, $st_type );
    $item = "cifs_write-3_write-4_write-iscsi_write-fcp_write";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( !-f "$wrkdir/$host/7MODE")
  $item = "wafl_exempt-kahuna-exempt-hostOS-nwk_legacy";    # kahuna etc is not supported on 7mode
  draw_all( $host, $type, $item, $st_type );

  # response time per node is very strange, totally different then others, skipping it for so far
  #$item = "cifs_resp_t-3_resp_t-4_resp_t-iscsi_resp_t-fcp_resp_t";
  #draw_all ($host,$type,$item,$st_type);
  #$item = "cpu_busy";

  # PORT
  #
  $type = "PORT";
  $item = "read_packet";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_error";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_packet";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_error";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "packet_rate";
  draw_all( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does  exist, without that makes no sense to continue, most probably it is initial run

    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # NAS
  $type = "NAS";
  my $dir       = "$wrkdir/$host/$type";
  my $suffix    = ".rrd";
  my $nas_files = 0;
  foreach my $line (<$dir/*$suffix>) {
    chomp($line);
    $nas_files++;
  }

  if ( $nas_files > 0 ) {
    if ( -f "$wrkdir/$host/$type/nas.cfg" ) {
      open( FHR, "< $wrkdir/$host/$type/nas.cfg" ) || error( "Can't open $wrkdir/$host/$type/nas.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      @nas_trans = <FHR>;
      close(FHR);
    }
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all( $host, $type, $item, $st_type );
    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $nas_files > 0 )

  # DRIVE
  $type = "DRIVE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  # LUN
  $type = "LUN";
  $item = "top";
  $ret  = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run
    set_colors_initial( $host, $type, "luns" );    # reinitialize colours, must be after Volume processing
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # CPU-CORE
  $type       = "CPU-CORE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "busy";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  return 1;
} ## end sub draw_graphs_netapp

sub draw_graphs_ds5 {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  # Draw graphs

  # POOL
  # pool processing always after RANK due to *.first files, pool one are created based on already created for ranks
  #
  # here must be minimal time of the first data point from all files
  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/POOL");    # switch to POOL in the future as pool data is used
  $item       = "sum_io_total";                                    # !!! used only here as DS5K does not have read/write metrics!!!
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data_total";                                        # !!! used only here as DS5K does not have read/write metrics!!!
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_cntl";
  draw_all_pool_controler( $host, $type, $item, $st_type );
  $item = "io_cntl";
  draw_all_pool_controler( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  if ( -f "$wrkdir/$host/DS5K-v2" ) {

    # only for new DS5K storages
    # No cache hit, it does not make sense here
    #$item = "r_cache_hit";
    #draw_all_pool ($host,$type,$item,$st_type);
    #$item = "w_cache_hit";
    #draw_all_pool ($host,$type,$item,$st_type);
    $item = "resp_t";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "sum_resp_t";    # !!! used only here as DS5K does not have read/write metrics!!!
    draw_all_sum( $host, $type, $item, $st_type, $time_first );

    # --PH: resp time per controller is a bit complicated, not supported for now
    #$item = "resp_t_cntl";
    #draw_all_pool_controler ($host,$type,$item,$st_type);
  } ## end if ( -f "$wrkdir/$host/DS5K-v2")

  #
  # HOST
  #
  # only touch files with used metrics
  $type = "HOST";
  $item = "io_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "data_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read_pct";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  if ( -f "$wrkdir/$host/DS5K-v2" ) {
    $item = "r_cache_hit";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
    $item = "w_cache_hit";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
    $item = "resp_t";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  } ## end if ( -f "$wrkdir/$host/DS5K-v2")
  else {
    $item = "cache_hit";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  }

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does  exist, without that makes no sense to continue, most probably it is initial run

    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read_pct";
    draw_all_volume( $host, $type, $item, $st_type );
    if ( -f "$wrkdir/$host/DS5K-v2" ) {

      # only for new DS5K storages
      $item = "r_cache_hit";
      draw_all_volume( $host, $type, $item, $st_type );
      $item = "w_cache_hit";
      draw_all_volume( $host, $type, $item, $st_type );
      $item = "resp_t";
      draw_all_volume( $host, $type, $item, $st_type );
      $item = "ssd_r_cache_hit";
      draw_all_volume( $host, $type, $item, $st_type );
    } ## end if ( -f "$wrkdir/$host/DS5K-v2")
    else {
      $item = "cache_hit";
      draw_all_volume( $host, $type, $item, $st_type );
    }
    draw_volume_cache_hit_all( $host, $type, $st_type );
  } ## end if ( $ret != 2 )

  return 1;
} ## end sub draw_graphs_ds5

sub draw_graphs_xiv {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST";
  LoadDataModule::load_nicks ($type,$wrkdir,$host,$act_time,$st_type,$DEBUG);

  #
  # HOST
  #
  # only touch files with used metrics
  $type = "HOST";
  $item = "read_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_r";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_w";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  # Draw graphs

  # POOL
  # pool processing always after RANK due to *.first files, pool one are created based on already created for ranks
  # XIV pool data is based on Volume data
  #
  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");    # POOL since 1.35 when was switch to pool data
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all_pool( $host, $type, $item, $st_type );

  #
  # HOST
  #
  # only touch files with used metrics
  #$type = "HOST";
  #$item = "read_io";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  return 1;
} ## end sub draw_graphs_xiv

sub draw_graphs_3par {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  #
  # HOST
  #
  # only touch files with used metrics
  #$type = "HOST";
  #$item = "read_io";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  #$item = "write_io";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  #$item = "read";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  #$item = "write";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  #$item = "resp_t_r";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  #$item = "resp_t_w";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  my @hosts_rrd  = <$wrkdir/$host/HOST/*.rrd>;
  my $host_count = scalar @hosts_rrd;
  if ( $host_count > 0 ) {

    $time_first = find_real_data_start_all("$wrkdir/$host/$type");

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $host_count > 0 )

  # DRIVE = Logical Drive
  #
  $type = "DRIVE";
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # TIER  = Logical Drive
  #
  $type = "TIER";
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  # POOL
  #
  $type = "POOL";

  # front end --> volumes
  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # DO NOT USE THIS POOL FRONT END STUFF!!
  # it is misleading, it summs volumes but they can belong to more pools, they can live migrate and pool stats are not real then, just total is ok
  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all_pool( $host, $type, $item, $st_type );

  # capacity
  #$item = "sum_capacity";
  #draw_all_sum ($host,$type,$item,$st_type,$time_first);
  # total capacity per pool does not make sense, it is virtual, total alway equal free
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  # PORT
  #
  $type = "PORT";
  if ( -f "$wrkdir/$host/$type/port.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/port.cfg" ) || error( "Can't open $wrkdir/$host/$type/port.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @port_trans = <FHR>;
    close(FHR);
  }

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "read_io-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "read-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "write_io-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "write-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "data_rate-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "io_rate-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  #draw_all_port_subsystem ($host,$type,$item);
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  #draw_all_port_subsystem ($host,$type,$item);

  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  ### VVOL
  $type = "VVOL";
  $item = "io_rate";
  draw_all_volume( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_volume( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all_volume( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all_volume( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_volume( $host, $type, $item, $st_type );

  #$item = "r_cache_hit";
  #draw_all_volume ($host,$type,$item,$st_type);
  $item = "resp_t_r";
  draw_all_volume( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_volume( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_volume( $host, $type, $item, $st_type );

  #$item = "w_cache_hit";
  #draw_all_volume ($host,$type,$item,$st_type);
  $item = "resp_t_w";
  draw_all_volume( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run

    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );

    #$item = "r_cache_hit";
    #draw_all_volume ($host,$type,$item,$st_type);
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );

    #$item = "w_cache_hit";
    #draw_all_volume ($host,$type,$item,$st_type);
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );

    #### Virtual lun resp time
    #$type = "LUN";
    #$item = "resp_t";
    #draw_all_volume ($host,$type,$item,$st_type);
    #$item = "resp_t_r";
    #draw_all_volume ($host,$type,$item,$st_type);
    #$item = "resp_t_w";
    #draw_all_volume ($host,$type,$item,$st_type);
  } ## end if ( $ret != 2 )

  # NODE
  $type = "NODE";
  $item = "r_cache_hit";
  draw_all( $host, $type, $item, $st_type );
  $item = "w_cache_hit";
  draw_all( $host, $type, $item, $st_type );

  # TIER NODE
  $item = "cfc_dirty";
  draw_all( $host, $type, $item, $st_type );

  # not sure what it is and how "dynamic" it is, excluding ...
  #$item = "cfc_max";
  #draw_all ($host,$type,$item,$st_type);
  $item = "del_ack";
  draw_all( $host, $type, $item, $st_type );

  # CPU-CORE
  $type       = "CPU-CORE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sys";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "top";
  $type = "HOST";
  $ret  = hosts_in_top_all( $host, $type, $item, $st_type );

  return 1;
} ## end sub draw_graphs_3par

sub draw_graphs_pure {

  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST-GROUP";
  # HOST-GROUP with rrd files
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );


  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  # HOST with rrd files
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  $item = "top";
  $type = "HOST";
  my $ret = hosts_in_top_all( $host, $type, $item, $st_type );

  if ( -f "$wrkdir/$host/DRIVE/drive.cfg" ) {
    open( FHR, "< $wrkdir/$host/DRIVE/drive.cfg" ) || error( "Can't open $wrkdir/$host/DRIVE/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @drive_trans = <FHR>;
    close(FHR);
  }

  my $size = ( stat "$wrkdir/$host/pool.cfg" )[7];
  if ( $size > 0 ) {

    # use POOL only if pools are in place, HUS do not need POOL, it can live with RG only
    # POOL
    #
    $type = "POOL";

    # here must be minimal time of the first data point from all files
    $time_first = find_real_data_start_all("$wrkdir/$host/$type");
    $item       = "sum_io";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_data";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "io_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_pool( $host, $type, $item, $st_type );

    # capacity
    $item = "sum_capacity";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_capacity_aggregated";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "real";
    draw_pool_cap( $host, $type, $item, $st_type );
  } ## end if ( $size > 0 )

  # RANK (RG)
  #
  #$type = "RANK";
  #$item = "read_io";
  #draw_all( $host, $type, $item, $st_type );
  #$item = "read";
  #draw_all( $host, $type, $item, $st_type );
  #$item = "resp_t_r";
  #draw_all( $host, $type, $item, $st_type );

  #$item = "write_io";
  #draw_all( $host, $type, $item, $st_type );
  #$item = "write";
  #draw_all( $host, $type, $item, $st_type );
  #$item = "resp_t_w";
  #draw_all( $host, $type, $item, $st_type );
  #$item = "io_rate";
  #draw_all( $host, $type, $item, $st_type );
  #$item = "data_rate";
  #draw_all( $host, $type, $item, $st_type );
  #$item = "resp_t";
  #draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  #$time_first = find_real_data_start_all("$wrkdir/$host/$type");
  #$item       = "sum_io";
  #draw_all_sum( $host, $type, $item, $st_type, $time_first );
  #$item = "sum_data";
  #draw_all_sum( $host, $type, $item, $st_type, $time_first );
  #$item = "sum_resp_t";
  #draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run
    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )


} ## end sub draw_graphs_oceanstor



sub draw_graphs_oceanstor {

  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  # HOST with rrd files
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  $item = "top";
  $type = "HOST";
  my $ret = hosts_in_top_all( $host, $type, $item, $st_type );

  if ( -f "$wrkdir/$host/DRIVE/drive.cfg" ) {
    open( FHR, "< $wrkdir/$host/DRIVE/drive.cfg" ) || error( "Can't open $wrkdir/$host/DRIVE/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @drive_trans = <FHR>;
    close(FHR);
  }

  my $size = ( stat "$wrkdir/$host/pool.cfg" )[7];
  if ( $size > 0 ) {

    # use POOL only if pools are in place, HUS do not need POOL, it can live with RG only
    # POOL
    #
    $type = "POOL";

    # here must be minimal time of the first data point from all files
    $time_first = find_real_data_start_all("$wrkdir/$host/$type");
    $item       = "sum_io";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_data";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "io_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_pool( $host, $type, $item, $st_type );

    # capacity
    $item = "sum_capacity";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "real";
    draw_pool_cap( $host, $type, $item, $st_type );
    $item = "tier0";
    draw_pool_cap( $host, $type, $item, $st_type );
    $item = "tier1";
    draw_pool_cap( $host, $type, $item, $st_type );
    $item = "tier2";
    draw_pool_cap( $host, $type, $item, $st_type );
  } ## end if ( $size > 0 )

  # FS
  $type = "FS";
  if ( -f "$wrkdir/$host/$type/fs.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/fs.cfg" ) || error( "Can't open $wrkdir/$host/$type/fs.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @fs_trans = <FHR>;
    close(FHR);
  }
  $item = "ops_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_ops";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_ops";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  # PORT
  #
  $type = "PORT";
  if ( -f "$wrkdir/$host/$type/port.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/port.cfg" ) || error( "Can't open $wrkdir/$host/$type/port.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @port_trans = <FHR>;
    close(FHR);
  }

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "read_io-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "read-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "write_io-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "write-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "data_rate-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "io_rate-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  #draw_all_port_subsystem ($host,$type,$item);
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  #draw_all_port_subsystem ($host,$type,$item);

  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # NODE
  $type = "NODE";

  if ( -f "$wrkdir/$host/$type/node.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/node.cfg" ) || error( "Can't open $wrkdir/$host/$type/node.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @node_trans = <FHR>;
    close(FHR);
  }

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "r_cache_hit";
  draw_all( $host, $type, $item, $st_type );
  $item = "w_cache_hit";
  draw_all( $host, $type, $item, $st_type );
  $item = "r_cache_usage";
  draw_all( $host, $type, $item, $st_type );
  $item = "w_cache_usage";
  draw_all( $host, $type, $item, $st_type );
  $item = "queue";
  draw_all( $host, $type, $item, $st_type );

  $item = "sys";
  draw_all( $host, $type, $item, $st_type, $time_first );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run
    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all( $host, $type, $item, $st_type );
    $item = "queue";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # DRIVE
  $type = "DRIVE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );
  $item = "queue";
  draw_all( $host, $type, $item, $st_type );

  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

} ## end sub draw_graphs_oceanstor

sub draw_graphs_compellent {

  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;


  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  # HOST with rrd files
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  $item = "top";
  $type = "HOST";
  my $ret = hosts_in_top_all( $host, $type, $item, $st_type );

  if ( -f "$wrkdir/$host/DRIVE/drive.cfg" ) {
    open( FHR, "< $wrkdir/$host/DRIVE/drive.cfg" ) || error( "Can't open $wrkdir/$host/DRIVE/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @drive_trans = <FHR>;
    close(FHR);
  }

  my $size = ( stat "$wrkdir/$host/pool.cfg" )[7];
  if ( $size > 0 ) {

    # use POOL only if pools are in place, HUS do not need POOL, it can live with RG only
    # POOL
    #
    $type = "POOL";

    # here must be minimal time of the first data point from all files
    my $host_space = check_space_name($host);
    $time_first = find_real_data_start_all("$wrkdir/$host_space/$type");
    $item       = "sum_io";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_data";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "io_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "io_pending";
    draw_all_pool( $host, $type, $item, $st_type );

    # capacity
    $item = "sum_capacity";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "real";
    draw_pool_cap( $host, $type, $item, $st_type );
  } ## end if ( $size > 0 )

  # PORT
  #
  $type = "PORT";
  if ( -f "$wrkdir/$host/$type/port.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/port.cfg" ) || error( "Can't open $wrkdir/$host/$type/port.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @port_trans = <FHR>;
    close(FHR);
  }
  $size = ( stat "$wrkdir/$host/$type/port.cfg" )[7];
  if ( isdigit($size) && $size > 0 ) {

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    draw_all_port_subsystem( $host, $type, $item );
    $item = "read_io-FC";
    draw_all_port_subsystem( $host, $type, $item );

    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    draw_all_port_subsystem( $host, $type, $item );
    $item = "read-FC";
    draw_all_port_subsystem( $host, $type, $item );

    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    draw_all_port_subsystem( $host, $type, $item );
    $item = "write_io-FC";
    draw_all_port_subsystem( $host, $type, $item );

    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    draw_all_port_subsystem( $host, $type, $item );
    $item = "write-FC";
    draw_all_port_subsystem( $host, $type, $item );

    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
    draw_all_port_subsystem( $host, $type, $item );
    $item = "data_rate-FC";
    draw_all_port_subsystem( $host, $type, $item );

    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    draw_all_port_subsystem( $host, $type, $item );
    $item = "io_rate-FC";
    draw_all_port_subsystem( $host, $type, $item );

    $item = "resp_t_r";
    draw_all( $host, $type, $item, $st_type );

    #draw_all_port_subsystem ($host,$type,$item);
    $item = "resp_t_w";
    draw_all( $host, $type, $item, $st_type );

    #draw_all_port_subsystem ($host,$type,$item);
    my $host_space = check_space_name($host);
    $time_first = find_real_data_start_all("$wrkdir/$host_space/$type");
    $item       = "sum_resp_t";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
  }

  # NODE
  $type = "NODE";

  if ( -f "$wrkdir/$host/$type/node.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/node.cfg" ) || error( "Can't open $wrkdir/$host/$type/node.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @node_trans = <FHR>;
    close(FHR);
  }

  $size = ( stat "$wrkdir/$host/$type/node.cfg" )[7];
  if ( isdigit($size) && $size > 0 ) {

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all( $host, $type, $item, $st_type );

    $item = "sys";
    draw_all( $host, $type, $item, $st_type, $time_first );
  }

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run
    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "io_pending";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # DRIVE
  $type = "DRIVE";

  $size = ( stat "$wrkdir/$host/$type/drive.cfg" )[7];
  if ( isdigit($size) && $size > 0 ) {

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_pending";
    draw_all( $host, $type, $item, $st_type );
    my $host_space = check_space_name($host);
    $time_first = find_real_data_start_all("$wrkdir/$host_space/$type");
    $item       = "sum_resp_t";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
  }

  # VOLUME-FOLDER
  $type = "VOLUME-FOLDER";

  if ( -f "$wrkdir/$host/$type/volume_folder.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/volume_folder.cfg" ) || error( "Can't open $wrkdir/$host/$type/volume_folder.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @volume_folder_trans = <FHR>;
    close(FHR);
  }
  $size = ( stat "$wrkdir/$host/$type/volume_folder.cfg" )[7];
  if ( isdigit($size) && $size > 0 ) {

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_pending";
    draw_all( $host, $type, $item, $st_type );
  }

  #$time_first = find_real_data_start_all("$wrkdir/$host/$type");
  #$item       = "sum_resp_t";
  #draw_all_sum( $host, $type, $item, $st_type, $time_first );

} ## end sub draw_graphs_compellent

sub draw_graphs_dothill {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  $item = "read_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  #$item = "resp_t_r";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  #$item = "resp_t_w";
  #print "creating cmd   : $host:$type:$item:d\n" if $DEBUG ;
  #`touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  if ( -f "$wrkdir/$host/DRIVE/drive.cfg" ) {
    open( FHR, "< $wrkdir/$host/DRIVE/drive.cfg" ) || error( "Can't open $wrkdir/$host/DRIVE/drive.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @drive_trans = <FHR>;
    close(FHR);
  }

  my $size = ( stat "$wrkdir/$host/pool.cfg" )[7];
  if ( $size > 0 ) {

    # use POOL only if pools are in place, HUS do not need POOL, it can live with RG only
    # POOL
    #
    $type = "POOL";

    # here must be minimal time of the first data point from all files
    $time_first = find_real_data_start_all("$wrkdir/$host/$type");
    $item       = "sum_io";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_data";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "io_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all_pool( $host, $type, $item, $st_type );

    # capacity
    $item = "sum_capacity";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "real";
    draw_pool_cap( $host, $type, $item, $st_type );
  } ## end if ( $size > 0 )

  # PORT
  #
  $type = "PORT";
  if ( -f "$wrkdir/$host/$type/port.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/port.cfg" ) || error( "Can't open $wrkdir/$host/$type/port.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @port_trans = <FHR>;
    close(FHR);
  }

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # NODE
  $type = "NODE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "r_cache_hit";
  draw_all( $host, $type, $item, $st_type );
  $item = "w_cache_hit";
  draw_all( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # DRIVE
  $type = "DRIVE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );

  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # RANK (RG)
  #
  $type = "RANK";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  return 1;
} ## end sub draw_graphs_dothill

sub draw_graphs_hus {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  #
  # HOST
  #
  # only touch files with used metrics
  $type = "HOST";
  $item = "read_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_r";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_w";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  my $size = ( stat "$wrkdir/$host/pool.cfg" )[7];
  if ( $size > 0 ) {

    # use POOL only if pools are in place, HUS do not need POOL, it can live with RG only
    # POOL
    #
    $type = "POOL";

    # here must be minimal time of the first data point from all files
    $time_first = find_real_data_start_all("$wrkdir/$host/$type");
    $item       = "sum_io";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_data";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_resp_t";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );

    $item = "io_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "data_cntl";
    draw_all_pool_controler( $host, $type, $item, $st_type );
    $item = "io_cntl";
    draw_all_pool_controler( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_pool( $host, $type, $item, $st_type );

    # capacity
    $item = "sum_capacity";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "real";
    draw_pool_cap( $host, $type, $item, $st_type );
  } ## end if ( $size > 0 )

  # RANK (RG)
  #
  $type = "RANK";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # capacity
  #$item = "sum_capacity";
  #draw_all_sum ($host,$type,$item,$st_type,$time_first);
  # --> not yet, we are not able to get size so far

  # PORT
  #
  $type = "PORT";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "r_cache_hit";
  draw_all( $host, $type, $item, $st_type );
  $item = "w_cache_hit";
  draw_all( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );
    draw_volume_cache_hit_all( $host, $type, $st_type );
  } ## end if ( $ret != 2 )

  # DRIVE
  $type = "DRIVE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "operating_rate";
  draw_all( $host, $type, $item, $st_type );

  # CPU-CORE
  $type       = "CPU-CORE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sys";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # NODE-CACHE
  $type       = "NODE-CACHE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "write_pend";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "clean_usage";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "middle_usage";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "phys_usage";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # PORT MAPPING #
  #LoadDataModule::port_mapping($type,$wrkdir,$host,$act_time,$st_type,$DEBUG,$webdir);

  return 1;
} ## end sub draw_graphs_hus

sub draw_graphs_vspg {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  ### VSPG ###

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  # HOST
  #
  $type = "HOST";
  my $host_org = host_data_selection( $wrkdir, $host, $type, $st_type );

  if ( $host_org == 1 ) {

    # HOST with rrd files
    # only if any data is available, it does not work for all VSP-G
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all( $host, $type, $item, $st_type );
    $item = "top";
    $type = "HOST";
    my $ret = hosts_in_top_all( $host, $type, $item, $st_type );
  } ## end if ( $host_org == 1 )
  else {
    # usuall way, getting host info by aggregating volume data
    # only touch files with used metrics - in case there is no host data
    $item = "read_io";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
    $item = "write_io";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
    $item = "read";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
    $item = "write";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
    $item = "resp_t_r";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
    $item = "resp_t_w";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  } ## end else [ if ( $host_org == 1 ) ]

  # PORT
  #
  $type = "PORT";
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );

  ### initiator port
  ### testing for exist initiator port
  my @files_ini = <"$wrkdir/$host/$type/*">;
  my $count     = 0;
  foreach my $file (@files_ini) {
    chomp $file;
    if ( $file eq "" ) { next; }
    if ( $file =~ m/\.rri$/ ) {
      $count++;
    }
  } ## end foreach my $file (@files_ini)
  if ( $count != 0 ) {

    $item = "io_rate_ini";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate_ini";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_ini";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $count != 0 )

  # NODE-CACHE
  $type       = "NODE-CACHE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "write_pend";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "usage_rate";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # CPU-CORE
  $type       = "CPU-CORE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sys";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "io_rate_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate_b";
  draw_all_pool( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "real";
  draw_pool_cap( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run

    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "io_rate_b";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate_b";
    draw_all_volume( $host, $type, $item, $st_type );
    draw_volume_cache_hit_all( $host, $type, $st_type );
  } ## end if ( $ret != 2 )

  # RANK (RG)
  #
  $type = "RANK";

  my $wrkdir_space = check_space_name($wrkdir);
  my $host_space   = check_space_name($host);
  my @files        = <$wrkdir_space/$host_space/$type/*>;
  my $rrd          = grep {/\.rrd$/} @files;
  if ( $rrd > 0 ) {

    # only if any data is available, it does not work for all VSP-G

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all( $host, $type, $item, $st_type );

    # here must be minimal time of the first data point from all files
    $time_first = find_real_data_start_all("$wrkdir/$host/$type");
    $item       = "sum_io";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_data";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "sum_resp_t";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );

    # metreics are there but not sure if draw_all is able to work with it, leaving so far ...
    #$item = "io_rate_b";
    #draw_all ($host,$type,$item,$st_type);
    #$item = "data_rate_b";
    #draw_all ($host,$type,$item,$st_type);

  } ## end if ( $rrd > 0 )

  # PORT MAPPING #
  #LoadDataModule::port_mapping($type,$wrkdir,$host,$act_time,$st_type,$DEBUG,$webdir);

  return 1;
} ## end sub draw_graphs_vspg

sub draw_graphs_vnxf {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  ### VNXF ###

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  # PORT
  #
  $type = "PORT";
  $item = "read_packet";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_error";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_packet";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_error";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "packet_rate";
  draw_all( $host, $type, $item, $st_type );

  # NODE
  $type = "NODE";
  $item = "calls_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_calls";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_calls";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );

  # CPU-CORE
  $type       = "CPU-CORE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sys";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # NODE-CACHE
  $type       = "NODE-CACHE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "dnlc_hit";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "cache_hit";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "buffer_hit";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # FS
  $type = "FS";
  if ( -f "$wrkdir/$host/$type/fs.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/fs.cfg" ) || error( "Can't open $wrkdir/$host/$type/fs.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @fs_trans = <FHR>;
    close(FHR);
  }
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );

  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "real";
  draw_pool_cap( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "busy";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "qdepth";
  draw_all_pool( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run

    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "busy";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "qdepth";
    draw_all_volume( $host, $type, $item, $st_type );

  } ## end if ( $ret != 2 )

  return 1;
} ## end sub draw_graphs_vnxf

sub draw_graphs_vnxb {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  ### VNX block ###

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  # NODE
  #
  $type       = "NODE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sys";
  draw_all( $host, $type, $item, $st_type, $time_first );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type, $time_first );
  $item = "dirty_cache_pages";
  draw_all( $host, $type, $item, $st_type, $time_first );

  # POOL
  #
  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "data_cntl";
  draw_all_pool_controler( $host, $type, $item, $st_type );
  $item = "io_cntl";
  draw_all_pool_controler( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  # front-end
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # PORT
  #
  $type = "PORT";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # HOST
  #
  # only touch files with used metrics
  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  $item = "read_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "io_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "data_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  # Drive
  #
  $type = "DRIVE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );

  return 1;
} ## end sub draw_graphs_vnxb

sub draw_graphs_unity {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  ### EMC Unity ###

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  # NODE
  #
  $type       = "NODE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sys";
  draw_all( $host, $type, $item, $st_type, $time_first );

  # POOL
  #
  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "data_cntl";
  draw_all_pool_controler( $host, $type, $item, $st_type );
  $item = "io_cntl";
  draw_all_pool_controler( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  # front-end
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # Drive
  #
  $type = "DRIVE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );

  # PORT
  #
  $type = "PORT";

  $item = "read_packet";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_packet";
  draw_all( $host, $type, $item, $st_type );
  $item = "packet_rate";
  draw_all( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );

  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );

  my $dir         = "$wrkdir/$host/$type";
  my $suffix      = ".rrd";
  my $iscsi_ports = 0;
  my $eth_ports   = 0;
  foreach my $line (<$dir/*$suffix>) {
    chomp($line);

    my $rrd = $line;
    $rrd =~ s/^$dir\///;
    $rrd =~ s/\.rrd$//;

    if ( $rrd =~ m/iscsinode_/ ) {
      $iscsi_ports++;
    }
    if ( $rrd =~ m/_eth/ && $rrd !~ m/iscsinode_/ ) {
      $eth_ports++;
    }
  } ## end foreach my $line (<$dir/*$suffix>)

  if ( $iscsi_ports > 0 ) {
    $type = "ISCSIPORT";

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );

    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $iscsi_ports > 0 )
  if ( $eth_ports > 0 ) {
    $type = "ETHPORT";

    $item = "read_packet";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_packet";
    draw_all( $host, $type, $item, $st_type );
    $item = "packet_rate";
    draw_all( $host, $type, $item, $st_type );

    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $eth_ports > 0 )

  # FS
  $type = "FS";
  if ( -f "$wrkdir/$host/$type/fs.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/fs.cfg" ) || error( "Can't open $wrkdir/$host/$type/fs.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @fs_trans = <FHR>;
    close(FHR);
  }
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );

  # NAS
  $type = "NAS";
  if ( -f "$wrkdir/$host/$type/nas.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/nas.cfg" ) || error( "Can't open $wrkdir/$host/$type/nas.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @nas_trans = <FHR>;
    close(FHR);
  }
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );

  # NASPOOL
  #
  $type = "NASPOOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );

  # HOST
  #
  # only touch files with used metrics
  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  $item = "read_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "io_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "data_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  return 1;
} ## end sub draw_graphs_unity

sub draw_graphs_vmax {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  ### EMC VMAX ###

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  # POOL
  #
  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");

  #back-end
  $item = "read_io_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_r_b";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "write_io_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_w_b";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "io_rate_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_b";
  draw_all_pool( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );
  $item = "alloc";
  draw_pool_cap( $host, $type, $item, $st_type );

  # TIER
  #
  # only if exists rrd files
  my @tiers_rrd  = <$wrkdir/$host/TIER/*.rrd>;
  my $tier_count = @tiers_rrd;
  if ( $tier_count > 0 ) {
    $type = "TIER";

    # here must be minimal time of the first data point from all files
    $time_first = find_real_data_start_all("$wrkdir/$host/$type");

    $item = "read_io_b";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "read_b";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_r_b";
    draw_all_pool( $host, $type, $item, $st_type );

    $item = "write_io_b";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "write_b";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_w_b";
    draw_all_pool( $host, $type, $item, $st_type );

    $item = "io_rate_b";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "data_rate_b";
    draw_all_pool( $host, $type, $item, $st_type );
    $item = "resp_t_b";
    draw_all_pool( $host, $type, $item, $st_type );

    # capacity
    $item = "sum_capacity";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "used";
    draw_pool_cap( $host, $type, $item, $st_type );
  } ## end if ( $tier_count > 0 )

  # SLO (Service level objectives)
  #
  # only if exists rrd files
  my @slos_rrd  = <$wrkdir/$host/SLO/*.rrd>;
  my $slo_count = @slos_rrd;
  if ( $slo_count > 0 ) {
    $type = "SLO";

    # here must be minimal time of the first data point from all files
    $time_first = find_real_data_start_all("$wrkdir/$host/$type");

    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all( $host, $type, $item, $st_type );
    $item = "cache_hit";
    draw_all( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $slo_count > 0 )

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  # front-end
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # FAST-SG
  #
  # only touch files with used metrics
  # only for Univmax version 1.6
  if ( !-f "$wrkdir/$host/VMAX-V8" ) {
    $type = "VOLUME";
    $item = "tier_alloc_prct";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  } ## end if ( !-f "$wrkdir/$host/VMAX-V8")

  # Volumes allocated to tiers
  # only if table exists
  if ( -f "$webdir/$host/sg_alloc.html" ) {

    # only touch files with used metrics
    $type = "VOLUME";
    $item = "alloc_vols";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  } ## end if ( -f "$webdir/$host/sg_alloc.html")

  # Drive
  #
  $type       = "DRIVE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  # PORT
  #
  $type       = "PORT";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");

  $item = "io_rate_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "sum_busy_frontend";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "busy_frontend";
  draw_all( $host, $type, $item, $st_type );

  if ( -f "$wrkdir/$host/VMAX-V8" ) {

    # Unisphere 1.6 does not have these items for FE ports
    $item = "read_io_frontend";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io_frontend";
    draw_all( $host, $type, $item, $st_type );
    $item = "read_frontend";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_frontend";
    draw_all( $host, $type, $item, $st_type );

    # Unisphere 1.6 does not have BE and RDF ports
    $item = "io_rate_backend";
    draw_all( $host, $type, $item, $st_type );
    $item = "read_io_backend";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io_backend";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate_backend";
    draw_all( $host, $type, $item, $st_type );
    $item = "read_backend";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_backend";
    draw_all( $host, $type, $item, $st_type );
    $item = "sum_busy_backend";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "busy_backend";
    draw_all( $host, $type, $item, $st_type );

    $item = "io_rate_rdf";
    draw_all( $host, $type, $item, $st_type );
    $item = "read_io_rdf";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io_rdf";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate_rdf";
    draw_all( $host, $type, $item, $st_type );
    $item = "read_rdf";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_rdf";
    draw_all( $host, $type, $item, $st_type );
    $item = "sum_busy_rdf";
    draw_all_sum( $host, $type, $item, $st_type, $time_first );
    $item = "busy_rdf";
    draw_all( $host, $type, $item, $st_type );

    # mapped vols table
    # only touch files with used metrics
    $item = "mapped_vols";
    print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
    `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  } ## end if ( -f "$wrkdir/$host/VMAX-V8")

  # NODE (directors)
  #
  $type       = "NODE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");

  $item = "io_rate_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate_backend";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate_rdf";
  draw_all( $host, $type, $item, $st_type );
  $item = "sum_busy_frontend";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_busy_backend";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_busy_rdf";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "busy_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "busy_backend";
  draw_all( $host, $type, $item, $st_type );
  $item = "busy_rdf";
  draw_all( $host, $type, $item, $st_type );

  # NODE-CACHE
  #
  $type       = "NODE-CACHE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "cache_hit";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "WP";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );

  # Host
  #
  my @hosts_rrd  = <$wrkdir/$host/HOST/*.rrd>;
  my $host_count = @hosts_rrd;
  if ( -f "$wrkdir/$host/VMAX-V8" && $host_count > 0 ) {
    $type = "HOST";

    # hosts.cfg
    LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

    $time_first = find_real_data_start_all("$wrkdir/$host/$type");

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( -f "$wrkdir/$host/VMAX-V8"...)

  $item = "top";
  $type = "HOST";
  $ret  = hosts_in_top_all( $host, $type, $item, $st_type );

  return 1;
} ## end sub draw_graphs_vmax

sub draw_graphs_vplex {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  ### EMC VPLEX ###

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  # front-end
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all ($host,$type,$item,$st_type);
  if ( $ret != 2 ) {
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );
  }

  # PORT
  #
  $type       = "PORT";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");

  # FE ports
  $item = "io_rate_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r_frontend";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w_frontend";
  draw_all( $host, $type, $item, $st_type );

  # BE ports
  $item = "data_rate_backend";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_backend";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_backend";
  draw_all( $host, $type, $item, $st_type );

  # NODE (directors)
  #
  $type       = "NODE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");

  $item = "sum_busy";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "busy";
  draw_all( $host, $type, $item, $st_type, $time_first );
  $item = "io_rate_b";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io_b";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io_b";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate_b";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_b";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_b";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );

  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # POOL
  #
  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");

  #$item = "io_rate";
  #draw_all_pool ($host,$type,$item,$st_type);
  #$item = "data_rate";
  #draw_all_pool ($host,$type,$item,$st_type);
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all_pool( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # HOST
  #
  $type = "HOST";

  # hosts.cfg
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "io_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "data_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_r";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_w";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  return 1;
} ## end sub draw_graphs_vplex

sub draw_graphs_vnxe {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  ### VNXe ###

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  # NODE
  #
  $type       = "NODE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sys";
  draw_all( $host, $type, $item, $st_type, $time_first );

  # POOL
  #
  $type = "POOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "data_cntl";
  draw_all_pool_controler( $host, $type, $item, $st_type );
  $item = "io_cntl";
  draw_all_pool_controler( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  # front-end
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {
    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  # Drive
  #
  $type = "DRIVE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";
  draw_all( $host, $type, $item, $st_type );

  # PORT
  #
  $type = "PORT";

  $item = "read_packet";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_packet";
  draw_all( $host, $type, $item, $st_type );
  $item = "packet_rate";
  draw_all( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );

  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );

  my $dir         = "$wrkdir/$host/$type";
  my $suffix      = ".rrd";
  my $iscsi_ports = 0;
  my $eth_ports   = 0;
  foreach my $line (<$dir/*$suffix>) {
    chomp($line);

    my $rrd = $line;
    $rrd =~ s/^$dir\///;
    $rrd =~ s/\.rrd$//;

    if ( $rrd =~ m/iscsinode_/ ) {
      $iscsi_ports++;
    }
    if ( $rrd =~ m/_eth/ && $rrd !~ m/iscsinode_/ ) {
      $eth_ports++;
    }
  } ## end foreach my $line (<$dir/*$suffix>)

  if ( $iscsi_ports > 0 ) {
    $type = "ISCSIPORT";

    $item = "read_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all( $host, $type, $item, $st_type );
    $item = "io_rate";
    draw_all( $host, $type, $item, $st_type );

    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $iscsi_ports > 0 )
  if ( $eth_ports > 0 ) {
    $type = "ETHPORT";

    $item = "read_packet";
    draw_all( $host, $type, $item, $st_type );
    $item = "write_packet";
    draw_all( $host, $type, $item, $st_type );
    $item = "packet_rate";
    draw_all( $host, $type, $item, $st_type );

    $item = "read";
    draw_all( $host, $type, $item, $st_type );
    $item = "write";
    draw_all( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all( $host, $type, $item, $st_type );
  } ## end if ( $eth_ports > 0 )

  # FS
  $type = "FS";
  if ( -f "$wrkdir/$host/$type/fs.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/fs.cfg" ) || error( "Can't open $wrkdir/$host/$type/fs.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @fs_trans = <FHR>;
    close(FHR);
  }
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );

  # NAS
  $type = "NAS";
  if ( -f "$wrkdir/$host/$type/nas.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/nas.cfg" ) || error( "Can't open $wrkdir/$host/$type/nas.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @nas_trans = <FHR>;
    close(FHR);
  }
  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );

  # NASPOOL
  #
  $type = "NASPOOL";

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "io_rate";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "read_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write";
  draw_all_pool( $host, $type, $item, $st_type );

  # HOST
  #
  # only touch files with used metrics
  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  $item = "read_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "io_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "data_rate";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  return 1;
} ## end sub draw_graphs_vnxe

sub draw_graphs_svc {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $type       = "";
  my $item       = "";
  my $time_first = 1;

  # mdisk name translation, must be global one
  if ( -f "$wrkdir/$host/mdisk.cfg" ) {
    open( FHR, "< $wrkdir/$host/mdisk.cfg" ) || error( "Can't open $wrkdir/$host/mdisk.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @mdisk_trans = <FHR>;
    close(FHR);
  }

  $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  # Draw graphs

  # PORT
  #
  $type = "PORT";
  if ( -f "$wrkdir/$host/$type/port.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/port.cfg" ) || error( "Can't open $wrkdir/$host/$type/port.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @port_trans = <FHR>;
    close(FHR);
  }

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "read_io-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "read-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "write_io-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "write-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "data_rate-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  draw_all_port_subsystem( $host, $type, $item );
  $item = "io_rate-FC";
  draw_all_port_subsystem( $host, $type, $item );

  $item = "b2b";
  draw_all( $host, $type, $item, $st_type );
  $item = "crc";
  draw_all( $host, $type, $item, $st_type );

  # Drive
  #
  $type = "DRIVE";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # CPU-CORE
  #
  # skip CPU-CORE, CPU-NODE is the prefered one

  # CPU-NODE
  #
  $type       = "CPU-NODE";
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sys";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # RANK == mdisk
  #
  $type = "RANK";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # POOL
  # pool processing always after RANK due to *.first files, pool one are created based on already created for ranks
  #
  $type = "POOL";

  # front-end graphs created from summed volume data

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # standard back-end (from mdisk data)
  $item = "read_io_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_r_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_io_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_w_b";
  draw_all_pool( $host, $type, $item, $st_type );

  # capacity
  $item = "sum_capacity";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "used";
  draw_pool_cap( $host, $type, $item, $st_type );

  #$item = "real";
  #draw_pool_cap ($host,$type,$item,$st_type);
  $item = "tier0";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "tier1";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "tier2";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "tier0-nl";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # compression
  $item = "compress_cap";
  draw_pool_cap( $host, $type, $item, $st_type );
  $item = "uncompress_cap";
  draw_pool_cap( $host, $type, $item, $st_type );
  $item = "compress_virt_cap";
  draw_pool_cap( $host, $type, $item, $st_type );

  cfg_copy( $wrkdir, $webdir, $host, $act_time );    # must be before volumes

  #
  # HOST
  #
  # only touch files with used metrics
  $type = "HOST";
  $item = "read_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_r";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_w";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  ### IO-GROUP
  $type = "IO-GROUP";

  if ( -f "$wrkdir/$host/$type/io_group.cfg" ) {
    open( FHR, "< $wrkdir/$host/$type/io_group.cfg" ) || error( "Can't open $wrkdir/$host/$type/io_group.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @io_group_trans = <FHR>;
    close(FHR);
  }

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );
  $item = "r_transfer_size";
  draw_all( $host, $type, $item, $st_type );
  $item = "w_transfer_size";
  draw_all( $host, $type, $item, $st_type );


  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";
    draw_all_volume( $host, $type, $item, $st_type );

    #$item = "w_cache_usage";
    #draw_all_volume ($host,$type,$item,$st_type);
    #$item = "r_cache_usage";
    #draw_all_volume ($host,$type,$item,$st_type);
    $item = "total_usage";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    draw_volume_cache_hit_all( $host, $type, $st_type );

    ### new items read/write transfer size
    $item = "r_transfer_size";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "w_transfer_size";
    draw_all_volume( $host, $type, $item, $st_type );

  } ## end if ( $ret != 2 )

  return 1;
} ## end sub draw_graphs_svc

sub draw_graphs_ds8k {
  my $host       = shift;
  my $act_time   = shift;
  my $st_type    = shift;
  my $time_first = 1;

  # load hosts info
  my $type = "HOST";
  LoadDataModule::load_nicks( $type, $wrkdir, $host, $act_time, $st_type, $DEBUG );

  # Draw graphs

  # PORT
  #
  $type = "PORT";
  my $item = "io_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "data_rate";
  draw_all( $host, $type, $item, $st_type );
  $item = "pprc_rio";
  draw_all( $host, $type, $item, $st_type );
  $item = "pprc_wio";
  draw_all( $host, $type, $item, $st_type );
  $item = "pprc_data_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "pprc_data_w";
  draw_all( $host, $type, $item, $st_type );
  $item = "pprc_rt_r";
  draw_all( $host, $type, $item, $st_type );
  $item = "pprc_rt_w";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t";    #--> need to check it out what it is, valuas are tehre under 0
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io_total";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  #$item = "sum_data"; # -PH something strage, do not care it is not necessary
  #draw_all_sum ($host,$type,$item,$st_type,$time_first);
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # RANK
  #
  $type = "RANK";
  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  # POOL
  #
  $type = "POOL";

  # standard back-end (from RANK data)
  $item = "read_io_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "read_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_r_b";
  draw_all_pool( $host, $type, $item, $st_type );

  $item = "write_io_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "write_b";
  draw_all_pool( $host, $type, $item, $st_type );
  $item = "resp_t_w_b";
  draw_all_pool( $host, $type, $item, $st_type );

  cfg_copy( $wrkdir, $webdir, $host, $act_time );

  # front-end graphs created from summed volume data

  # here must be minimal time of the first data point from all files
  $time_first = find_real_data_start_all("$wrkdir/$host/$type");
  $item       = "sum_io";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_data";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );
  $item = "sum_resp_t";
  draw_all_sum( $host, $type, $item, $st_type, $time_first );

  $item = "read_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "read";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_r";
  draw_all( $host, $type, $item, $st_type );

  $item = "write_io";
  draw_all( $host, $type, $item, $st_type );
  $item = "write";
  draw_all( $host, $type, $item, $st_type );
  $item = "resp_t_w";
  draw_all( $host, $type, $item, $st_type );

  #
  # HOST
  #
  # only touch files with used metrics
  $type = "HOST";
  $item = "read_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write_io";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "read";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "write";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_r";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;
  $item = "resp_t_w";
  print "creating cmd   : $host:$type:$item:d\n" if $DEBUG;
  `touch "$tmp_dir/$host/$type-$item-d.cmd"`;

  # VOLUME
  #
  # keep Volume graphs at the end as they might cause memory issues
  # front-end
  $type = "VOLUME";

  $item = "top";
  my $ret = volumes_in_top_all( $host, $type, $item, $st_type );
  if ( $ret != 2 ) {

    # volumes.cfg does exist, without that makes no sense to continue, most probably it is initial run

    $item = "io_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "data_rate";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t";
    draw_all_volume( $host, $type, $item, $st_type );

    $item = "read_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w";

    draw_all_volume( $host, $type, $item, $st_type );
    $item = "r_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "w_cache_hit";
    draw_all_volume( $host, $type, $item, $st_type );

    draw_volume_cache_hit_all( $host, $type, $st_type );

    # back-end
    $item = "read_io_b";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_io_b";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "read_b";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "write_b";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_r_b";
    draw_all_volume( $host, $type, $item, $st_type );
    $item = "resp_t_w_b";
    draw_all_volume( $host, $type, $item, $st_type );
  } ## end if ( $ret != 2 )

  return 1;
} ## end sub draw_graphs_ds8k

sub cfg_config_change {

  # If any change in lpar2rrd.cfg  then must be run install-html.sh
  my $cfg_update = "$tmp_dir/cfg_update_storage";

  if ( -f "$cfg_update" ) {
    my $last_line = "";
    open( FHLT, "< $cfg_update" ) || error( "Can't open $cfg_update: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line (<FHLT>) {
      $last_line = $line;
    }
    close(FHLT);
    my $png_time = ( stat("$basedir/etc/stor2rrd.cfg") )[9];
    if ( $last_line < $png_time ) {
      LoadDataModule::touch();
      open( FHLT, "> $cfg_update" ) || error( "Can't open $cfg_update: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
      my $act_time = time();
      print FHLT "$act_time";
      close(FHLT);
    } ## end if ( $last_line < $png_time)
  } ## end if ( -f "$cfg_update" )
  else {
    LoadDataModule::touch();
    my $png_time = ( stat("$basedir/etc/stor2rrd.cfg") )[9];
    open( FHLT, "> $cfg_update" ) || error( "Can't open $cfg_update: $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my $act_time = time();
    print FHLT "$act_time";
    close(FHLT);

    #print "$cfg_update $basedir/etc/stor2rrd.cfg $png_time\n";
  } ## end else [ if ( -f "$cfg_update" )]

  return 1;
} ## end sub cfg_config_change

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();

  print "ERROR          : $text \n";
  print STDERR "$act_time: $text \n";

  return 1;
} ## end sub error

sub draw_all {
  my $host    = shift;
  my $type    = shift;
  my $item    = shift;
  my $st_type = shift;

  draw_graph( "day",     "d", "MINUTE:60:HOUR:2:HOUR:4:0:%H", $host, $type, $item, $st_type );
  draw_graph( "week",    "w", "HOUR:8:DAY:1:DAY:1:86400:%a",  $host, $type, $item, $st_type );
  draw_graph( "4 weeks", "m", "DAY:1:DAY:2:DAY:2:0:%d",       $host, $type, $item, $st_type );
  draw_graph( "year",    "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, $item, $st_type );
  return 0;
} ## end sub draw_all

sub draw_graph {
  my $text        = shift;
  my $type_gr     = shift;
  my $xgrid       = shift;
  my $host        = shift;
  my $type        = shift;
  my $item        = shift;
  my $st_type     = shift;
  my $name        = "$tmp_dir/$host/$type-$item-agg-$type_gr";
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $last        = "COMMENT: ";
  my $rrd_time    = "";
  my $step_new    = $STEP;
  my $once        = 0;
  my $last_time   = "na";
  my $last_time_u = time() - 3600;

  # PORT types for Storwize
  # this is not being used --> viz draw_all_port_subsystem
  my $port_type = 3;    # default unknown port type
  if ( $item =~ m/_fc$/ )    { $port_type = 0; }
  if ( $item =~ m/_sas$/ )   { $port_type = 1; }
  if ( $item =~ m/_iscsi$/ ) { $port_type = 2; }
  if ( $item =~ m/_prep$/ )  { $port_type = 4; }

  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";
  my $act_utime = time();

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  print "creating cmd   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $type_text = $type;
  if ( $st_type =~ m/SWIZ/ && $type =~ m/RANK/ ) {
    $type_text = "MDISK";
  }
  if ( $st_type =~ m/HUS/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/NETAPP/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/VSPG/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/NETAPP/ && $type =~ m/NAS/ ) {
    $type_text = "SVM";
  }
  if ( $st_type =~ m/DOTHILL/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/NODE$/ ) {
    $type_text = "DIRECTOR";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/NODE-CACHE/ ) {
    $type_text = "CACHE";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/DRIVE/ ) {
    $type_text = "DISK GROUP";
  }
  if ( $st_type =~ m/PURE/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }

  my $item_header = $item;
  if ( $item =~ m/-/ ) {

    # netapp multi item
    $item_header = "Protocols";
  }
  if ( $item =~ m/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) {

    # netapp multi item
    $item_header = "CPU";
  }

  my $req_time = "";
  my $header   = "$type_text aggregated $item_header: last $text";
  if ( $item =~ m/resp_t/ ) {
    $header = "$type_text $item_header: last $text";
  }
  if ( $st_type eq "VMAX" ) {
    if ( $type eq "POOL" && $item !~ m/_b$/ ) {    # POOL front-end data from one file (array_total.rra)
      $header = "$type_text $item_header: last $text";
    }
    if ( $type eq "NODE" ) {
      $header = "$type_text $item_header: last $text";
    }
    if ( $type eq "PORT" && $item eq "busy" ) {
      $header = "$type_text $item_header: last $text";
    }
  } ## end if ( $st_type eq "VMAX")

  my $i    = 0;
  my $lpar = "";
  my $cmd  = "";
  my $j    = 0;

  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now-1$type_gr+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;    # base, the number will be devided by that constant

  my $suffix = "rrd";     #standard suffix

  # do not use switch statement
  if ( $item =~ m/data_rate/ )                                    { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^data_rate_ini$/ )                              { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^read$/ )                                       { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^read_b$/ )                                     { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^cifs_read-3_read-4_read/ )                     { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^write$/ )                                      { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^write_b$/ )                                    { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^cifs_write-3_write-4_write/ )                  { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) { $value_short = "%";      $value_long = "%";          $val = 1; }
  if ( $item =~ m/io_rate/ )                                      { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^io_rate_ini$/ )                                { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/read_io/ )                                      { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/write_io/ )                                     { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/read_ops/ )                                     { $value_short = "OPS";    $value_long = "OPS per second"; }
  if ( $item =~ m/write_ops/ )                                    { $value_short = "OPS";    $value_long = "OPS per second"; }
  if ( $item =~ m/ops_rate/ )                                     { $value_short = "OPS";    $value_long = "OPS per second"; }
  if ( $item =~ m/^resp$/ )                                       { $value_short = "ms";     $value_long = "milliseconds"; }
  if ( $item =~ m/resp_t/ )                                       { $value_short = "ms";     $value_long = "milliseconds"; }
  if ( $item =~ m/^resp_t_ini$/ )                                 { $value_short = "ms";     $value_long = "milliseconds"; }

  #if ( $item =~ m/^resp_t_r$/ )                                  { $value_short= "ms"; $value_long = "milliseconds";}
  #if ( $item =~ m/^resp_t_w$/ )                                  { $value_short= "ms"; $value_long = "milliseconds";}
  if ( $item =~ m/^sys$/ )            { $value_short = "CPU util"; $value_long  = "CPU utilization in %"; }
  if ( $item =~ m/^compress$/ )       { $value_short = "CPU util"; $value_long  = "CPU utilization in %"; }
  if ( $item =~ m/^operating_rate$/ ) { $value_short = "%";        $value_long  = "%"; }                                                 # HUS only  for drives
  if ( $item =~ m/^pprc_rio$/ )       { $suffix      = "rrp";      $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^pprc_wio$/ )       { $suffix      = "rrp";      $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^pprc_rt_r$/ )      { $suffix      = "rrp";      $value_short = "ms"; $value_long = "milliseconds"; }
  if ( $item =~ m/^pprc_rt_w$/ )      { $suffix      = "rrp";      $value_short = "ms"; $value_long = "milliseconds"; }
  if ( $item =~ m/^pprc_data_r$/ )    { $suffix      = "rrp";      $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^pprc_data_w$/ )    { $suffix      = "rrp";      $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^cache_hit$/ )     { $value_short = "%"; $value_long = "%"; $val = 1; }
  if ( $item =~ m/^w_cache_hit$/ )   { $value_short = "%"; $value_long = "%"; $val = 1; }
  if ( $item =~ m/^r_cache_hit$/ )   { $value_short = "%"; $value_long = "%"; $val = 1; }
  if ( $item =~ m/^w_cache_usage$/ ) { $value_short = "%"; $value_long = "%"; $val = 1; }
  if ( $item =~ m/^r_cache_usage$/ ) { $value_short = "%"; $value_long = "%"; $val = 1; }
  if ( $item =~ m/packet/ )            { $value_short = "pct/sec";    $value_long = "Packets per second"; }
  if ( $item =~ m/_error/ )            { $value_short = "pct/sec";    $value_long = "Error packets per second"; }
  if ( $item =~ m/read_calls/ )        { $value_short = "calls/s";    $value_long = "Calls per second"; $val = 1; }
  if ( $item =~ m/write_calls/ )       { $value_short = "Calls/s";    $value_long = "Calls per second"; $val = 1; }
  if ( $item =~ m/calls_rate/ )        { $value_short = "Calls/s";    $value_long = "Calls per second"; $val = 1; }
  if ( $item =~ m/dirty_cache_pages/ ) { $value_short = "%";          $value_long = "%"; $val = 1; }
  if ( $item =~ m/b2b/ )               { $value_short = "%";          $value_long = "%"; $val = 1; }
  if ( $item =~ m/crc/ )               { $value_short = "errors/sec"; $value_long = "Errors per second"; }
  if ( $item =~ m/^r_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^w_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^busy/ )            { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^read_frontend$/ )  { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^read_backend$/ )   { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^read_rdf$/ )       { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^write_frontend$/ ) { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^write_backend$/ )  { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^write_rdf$/ )      { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^cfc_dirty$/ )      { $value_short = "Pages";  $value_long = "Pages"; $val = 1; }
  if ( $item =~ m/^cfc_max$/ )        { $value_short = "Pages";  $value_long = "Pages"; $val = 1; }
  if ( $item =~ m/^del_ack$/ )        { $value_short = "Pages";  $value_long = "Pages"; $val = 1; }
  if ( $item =~ m/^io_pending$/ )     { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^queue$/ && $st_type eq "OCEANSTOR" ) { $value_short = "Unit/s"; $value_long = "Unit per second"; $val = 1; }
  if ( $item =~ m/^r_transfer_size$/ )    { $value_short = "kB/sec";   $value_long = "kBytes/sec"; $val = 1; }
  if ( $item =~ m/^w_transfer_size$/ )    { $value_short = "kB/sec";   $value_long = "kBytes/sec"; $val = 1; }

  if ( $st_type !~ m/^DS8K$/ ) {

    # rrp suffix only on DS8K so far
    $suffix = "rrd";
  }
  if ( $st_type =~ m/^VSPG$/ && $type =~ m/^PORT$/ && $item =~ m/^data_rate_ini$|^io_rate_ini$|^resp_t_ini$/ ) {
    $suffix = "rri";
    if ($item eq "resp_t_ini"){
      $val = 1000;
    }
  }
  if ( $st_type =~ m/^3PAR$/ && $type =~ m/^NODE$/ && $item =~ m/^cfc_dirty$|^cfc_max$|^del_ack$/ ) {
    $suffix = "rrt";
  }

  if ( $st_type =~ m/^SWIZ$/ && $type =~ m/^PORT$/ && ( $item =~ m/^b2b$/ || $item =~ m/^crc$/ ) ) {

    # rrp : b2b & crc
    $suffix = "rrp";
  }
  if ( $st_type =~ m/^DS8K$/ && $type =~ m/^POOL$/ ) {

    # rrd for DS8K front end
    $suffix = "rrd";
  }
  if ( $st_type =~ m/^VMAX$/ && $type =~ m/^POOL$/ ) {

    # rrd for VMAX front end
    $suffix = "rra";
  }

  # Why??? There should be 1024
  # excluded RANK in 1.35-6
  if ( $st_type =~ m/^DS8K$/ && $type !~ /RANK/ ) {
    if ( $item =~ m/^read$/ || $item =~ m/^write/ ) {
      $val = 1;
    }
  }

  if ( $item =~ m/^sys$/ || $item =~ m/^compress$/ ) {
    $cmd .= " --vertical-label=\\\"$value_long\\\"";
  }
  else {
    if ( $item =~ m/-/ ) {

      # netapp multi item
      $cmd .= " --vertical-label=\\\"$value_long\\\"";
    }
    else {
      $cmd .= " --vertical-label=\\\"$item in $value_long\\\"";
    }
  } ## end else [ if ( $item =~ m/^sys$/...)]

  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $item_print = $item;
  if ( $item =~ m/^cifs/ || $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) {

    # get real metric instead of that long string
    $item_print =~ s/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy//;
    $item_print =~ s/^cifs_//;
    $item_print =~ s/-.*$//;
  } ## end if ( $item =~ m/^cifs/...)
  my $legend = sprintf( "%-38s", "$item_print [$value_short]" );
  my $legend_heading = "$item_print $delimiter [$value_short] $delimiter Avrg $delimiter Max ";
  $legend_heading =~ s/%/%%/g;

  if ( ( $st_type eq "VNX" || $st_type eq "VNXE" || $st_type eq "UNITY" ) && $item eq "sys" ) {
    $cmd .= " COMMENT:\\\"$legend    Avrg   Max\\l\\\"";
  }
  else {
    $cmd .= " COMMENT:\\\"$legend       Avrg     Max\\l\\\"";
  }

  my $gtype = "AREA";
  if ( $item =~ m/operating_rate/ || $item =~ m/resp_t/ || $item =~ m/pprc_rt_/ || $item =~ m/^cache_hit$/ || $item =~ m/^r_cache_usage$/ || $item =~ m/^w_cache_usage$/ || $item =~ m/^w_cache_hit$/ || $item =~ m/^r_cache_hit$/ || $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ||  $item =~ m/r_transfer_size/ || $item =~ m/w_transfer_size/ ) {
    $gtype = "LINE1";
  }
  if ( ( $st_type =~ m/^3PAR$/ || $st_type =~ m/^SWIZ$/ || $st_type =~ m/^COMPELLENT$/ || $st_type =~ m/^OCEANSTOR$/ ) && $type =~ m/^PORT$/ ) {

    # do not stack front-end ports (FC) together with back-end ports (SAS)
    $gtype = "LINE1";
  }
  if ( $st_type =~ m/^SWIZ$/ && $type =~ m/^PORT$/ && ( $item =~ m/^b2b$/ || $item =~ m/^crc$/ ) ) {
    $gtype = "LINE1";
  }
  if ( $st_type =~ m/^VMAX$/ && $type =~ m/^POOL$/ ) {
    $gtype = "LINE1";
  }
  if ( $st_type =~ m/^VMAX$/ && ( $type =~ m/^PORT$/ || $type =~ m/^NODE$/ ) && $item =~ m/busy/ ) {
    $gtype = "LINE1";
  }
  if ( $st_type =~ m/^VPLEX$/ && $type =~ m/^NODE$/ && $item =~ m/busy/ ) {
    $gtype = "LINE1";
  }
  if ( $st_type =~ m/^COMPELLENT|OCEANSTOR$/ && $type =~ m/^NODE$/ && $item =~ m/sys/ ) {
    $gtype = "LINE1";
  }

  my $host_space = check_space_name($host);

  my $dir = "$wrkdir/$host_space/$type";

  if ( $st_type eq "UNITY" || $st_type eq "VNXE" ) {
    if ( $type eq "ISCSIPORT" || $type eq "ETHPORT" ) {
      $dir = "$wrkdir/$host_space/PORT";
    }
    if ( $type eq "NODE" && $item eq "sys" ) {
      $gtype = "LINE1";
    }
  } ## end if ( $st_type eq "UNITY"...)
  if ( $st_type eq "VNX" && $type eq "NODE" ) {
    if ( $item eq "dirty_cache_pages" || $item eq "sys" ) {
      $gtype = "LINE1";
    }
  }

  my $lpar_list     = "";
  my $pool_found    = 0;
  my $pool_id_tmp   = "";
  my $lpar_list_tmp = "";
  my $found_data    = 0;
  my $cmd_print     = "";    # print commands for clickable legend

  # open a file with stored colours
  my @color_save = "";
  my $type_low   = lc $type;
  my $file_col   = "$wrkdir/$host/$type/" . $type_low . "s.col";

  # change file name for UNITY ports
  if ( $st_type eq "UNITY" || $st_type eq "VNXE" ) {
    if ( $type eq "ISCSIPORT" || $type eq "ETHPORT" ) {
      $file_col = "$wrkdir/$host/PORT/ports.col";
    }
  }
  if ( -f $file_col ) {
    open( FHC, "< $file_col" ) || error( "file does not exists : $file_col " . __FILE__ . ":" . __LINE__ ) && return 0;
    @color_save = <FHC>;
    close(FHC);
  }
  my $color_indx        = 0;    # reset colour index
  my $color_file_change = 0;


  foreach my $line (<$dir/*$suffix>) {
    chomp($line);

    if ( $st_type !~ m/NETAPP/ && $st_type !~ m/HUS/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/PURE/ && $type =~ m/RANK/ && $line !~ m/\/.*-P[0-9][0-9]*\.$suffix$/ ) {

      #workaround for cleaning up rans without pool id (could hapen in the past by the error)
      next;
    }

    if ( $line =~ m/-cap.rrd$/ && ( $type =~ m/POOL/ || $type =~ m/TIER/ ) ) {
      next;    # exclude capacity data files from here (for POOL front-end stats
    }

    # UNITY FC ports only
    if ( $st_type eq "UNITY" && $type eq "PORT" && $line !~ m/_fc[0-9]+\.$suffix$/ ) {
      next;
    }

    # UNITY iSCSI ports only
    if ( $st_type eq "UNITY" && $type eq "ISCSIPORT" && $line !~ m/iscsinode_/ ) {
      next;
    }

    # UNITY ETHERNET ports only
    if ( $st_type eq "UNITY" && $type eq "ETHPORT" ) {
      if ( $line !~ m/_eth[0-9]+\.$suffix$/ ) {
        next;
      }
      if ( $line =~ m/iscsinode_/ ) {
        next;
      }
    } ## end if ( $st_type eq "UNITY"...)

    # VNXE FC ports only
    if ( $st_type eq "VNXE" && $type eq "PORT" && $line !~ m/_fc[0-9]+\.$suffix$/ ) {
      next;
    }

    # VNXE iSCSI ports only
    if ( $st_type eq "VNXE" && $type eq "ISCSIPORT" && $line !~ m/iscsinode_/ ) {
      next;
    }

    # VNXE ETHERNET ports only
    if ( $st_type eq "VNXE" && $type eq "ETHPORT" ) {
      if ( $line !~ m/_eth[0-9]+\.$suffix$/ ) {
        next;
      }
      if ( $line =~ m/iscsinode_/ ) {
        next;
      }
    } ## end if ( $st_type eq "VNXE"...)
    if ( $st_type =~ m/SWIZ/ && $type =~ m/POOL/ && $line =~ m/-comp\.rrd$/ ) {
      next;    # exclude data with compression -comp.rrd
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    my @link_l = split( /\//, $line );
    my $lpar = "";
    foreach my $m (@link_l) {
      $lpar = $m;
    }

    $lpar =~ s/\.rrd$//;
    $lpar =~ s/\.rrp$//;
    $lpar =~ s/\.rrc$//;
    $lpar =~ s/PORT-//;
    $lpar =~ s/RANK-//;
    $lpar =~ s/VOLUME-//;
    $lpar =~ s/0x//;

    if ( $type =~ m/RANK/ || $type =~ m/VOLUME/ ) {
      $lpar =~ s/-P.*//;    # filter pool info
    }

    # VMAX directors (front-end, backend)
    if ( $st_type eq "VMAX" && ( $type eq "NODE" || $type eq "PORT" ) && $item =~ m/_frontend$/ && $lpar !~ m/^F[A-Z]-/ ) { next; }
    if ( $st_type eq "VMAX" && ( $type eq "NODE" || $type eq "PORT" ) && $item =~ m/_backend$/  && $lpar !~ m/^D[A-Z]-/ ) { next; }
    if ( $st_type eq "VMAX" && ( $type eq "NODE" || $type eq "PORT" ) && $item =~ m/_rdf$/      && $lpar !~ m/^R[A-Z]-/ ) { next; }

    # VPLEX ports (front-end, backend)
    if ( $st_type eq "VPLEX" && $type eq "PORT" && $item =~ m/_frontend$/ && $lpar !~ m/-[A-B]0-/ ) { next; }
    if ( $st_type eq "VPLEX" && $type eq "PORT" && $item =~ m/_backend$/  && $lpar !~ m/-[A-B]1-/ ) { next; }

    my $lpar_space = $lpar;    # it is for fron-end POOL, like POOL/23.rrd

    if ( $type =~ m/POOL/ && ishexa($lpar_space) == 1 ) {

      # pool id tralslation to name
      $lpar_space = get_pool_name($lpar_space);
    }

    if ( $type =~ m/RANK/ && -f "$wrkdir/$host/mdisk.cfg" ) {

      # translate id into names only for MDISKS
      my $rank_found = 0;
      my $rank_id    = "";
      foreach my $line (@mdisk_trans) {
        chomp($line);
        ( my $id, my $name ) = split( /:/, $line );
        if ( defined($name) && !$name eq '' && $id =~ m/^$lpar$/ ) {
          $lpar_space = $name;
          $rank_found = 1;
          $rank_id    = $id;
          last;
        } ## end if ( defined($name) &&...)
      } ## end foreach my $line (@mdisk_trans)
      if ( $rank_found == 0 ) {
        my $lpar_id = $lpar;
        if ( isdigit($lpar_id) == 1 ) {
          $lpar_id = $lpar_id + 1 - 1;    # to clean prefixed zeros
          $rank_id = $lpar_id;
        }
        $lpar_space = "del-$lpar_id";
      } ## end if ( $rank_found == 0 )

      # No, no, there ca be ranks with the same ID (under different pool Ids)
      #$rank_id =~ s/ //g;
      #if (isdigit($rank_id) == 1 ) {
      #  # use RANK ID to keep color same
      #  $color_indx = $rank_id;
      #  while ($color_indx > $color_max ) {
      #    $color_indx = $color_indx - $color_max;
      #  }
      #}
    } ## end if ( $type =~ m/RANK/ ...)

    if ( $type =~ m/DRIVE/ && ( -f "$wrkdir/$host/drive.cfg" || -f "$wrkdir/$host/DRIVE/drive.cfg" ) ) {

      # translate id into names only for DRIVE (NetAPP and DOTHILL)
      my $drive_found = 0;
      my $drive_id    = "";
      foreach my $line (@drive_trans) {
        chomp($line);
        ( my $id, my $name ) = split( /,/, $line );
        if ( defined($name) && !$name eq '' && $id =~ m/^$lpar$/ ) {
          $lpar_space  = $name;
          $drive_found = 1;
          last;
        }
      } ## end foreach my $line (@drive_trans)
      if ( $drive_found == 0 ) {
        $lpar_space = $lpar;
      }
    } ## end if ( $type =~ m/DRIVE/...)

    if ( $type =~ m/NODE/ && -f "$wrkdir/$host/NODE/node.cfg" ) {

      # translate id into names only for NODE (COMPELLENT)
      my $node_found = 0;
      my $node_id    = "";
      foreach my $line (@node_trans) {
        chomp($line);
        ( my $id, my $name ) = split( /:/, $line );
        if ( defined($name) && !$name eq '' && "$id" eq "$lpar" ) {
          $lpar_space = $name;
          $node_found = 1;
          last;
        }
      } ## end foreach my $line (@node_trans)
      if ( $node_found == 0 ) {
        $lpar_space = $lpar;
      }
    } ## end if ( $type =~ m/NODE/ ...)

    if ( $type =~ m/VOLUME-FOLDER/ && -f "$wrkdir/$host/$type/volume_folder.cfg" ) {

      # translate id into names only for VOLUME-FOLDER (COMPELLENT)
      my $volume_folder_found = 0;
      my $volume_folder_id    = "";
      foreach my $line (@volume_folder_trans) {
        chomp($line);
        ( my $id, my $name ) = split( /:/, $line );
        if ( defined($name) && !$name eq '' && "$id" eq "$lpar" ) {
          $lpar_space = $name;
          $volume_folder_found = 1;
          last;
        }
      } ## end foreach my $line (@node_trans)
      if ( $volume_folder_found == 0 ) {
        $lpar_space = $lpar;
      }
    } ## end if ( $type =~ m/NODE/ ...)

    if ( $type =~ m/^IO-GROUP$/ && -f "$wrkdir/$host/$type/io_group.cfg" ) {

      # translate id into names only for IO-GROUP
      my $group_found = 0;
      my $group_id    = "";
      foreach my $line (@io_group_trans) {
        chomp($line);
        ( my $id, my $name ) = split( /:/, $line );
        if ( defined($name) && !$name eq '' && $id =~ m/^$lpar$/ ) {
          $lpar_space = $name;
          $group_found   = 1;
          last;
        }
      } ## end foreach my $line (@fs_trans)
      if ( $group_found == 0 ) {
        $lpar_space = $lpar;
      }
    } ## end if ( $type =~ m/^FS$/ ...)

    if ( $type =~ m/^FS$/ && -f "$wrkdir/$host/$type/fs.cfg" ) {

      # translate id into names only for Filesystem (VNXF and UNITY)
      my $fs_found = 0;
      my $fs_id    = "";
      foreach my $line (@fs_trans) {
        chomp($line);
        ( my $id, my $name ) = split( /:/, $line );
        if ( defined($name) && !$name eq '' && $id =~ m/^$lpar$/ ) {
          $lpar_space = $name;
          $fs_found   = 1;
          last;
        }
      } ## end foreach my $line (@fs_trans)
      if ( $fs_found == 0 ) {
        $lpar_space = $lpar;
      }
    } ## end if ( $type =~ m/^FS$/ ...)

    if ( $type =~ m/^NAS$/ && -f "$wrkdir/$host/$type/nas.cfg" ) {

      # translate id into names only for UNITY)
      my $nas_found = 0;
      my $nas_id    = "";
      foreach my $line (@nas_trans) {
        chomp($line);
        ( my $id, my $name ) = split( /:/, $line );
        if ( defined($name) && !$name eq '' && $id =~ m/^$lpar$/ ) {
          $lpar_space = $name;
          $nas_found  = 1;
          last;
        }
      } ## end foreach my $line (@nas_trans)
      if ( $nas_found == 0 ) {
        $lpar_space = $lpar;
      }
    } ## end if ( $type =~ m/^NAS$/...)

    if ( $type =~ m/^PORT$/ && -f "$wrkdir/$host/$type/port.cfg" ) {

      # translate id into names only for 3PAR)
      my $port_found   = 0;
      my $port_id      = "";
      my $lpar_filtred = $lpar;
      foreach my $line (@port_trans) {
        chomp($line);
        ( my $id, my $name ) = split( /:/, $line );
        if ( defined($name) && !$name eq '' && $id =~ m/^$lpar_filtred$/ ) {
          if ( $st_type =~ m/SWIZ/ || $st_type =~ m/3PAR/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/^OCEANSTOR$/ ) {
            $lpar_space = $lpar . " " . $name;
          }
          else {
            $lpar_space = $name;
          }
          $port_found = 1;
          last;
        } ## end if ( defined($name) &&...)
      } ## end foreach my $line (@port_trans)
      if ( $port_found == 0 ) {
        $lpar_space = $lpar;
      }
    } ## end if ( $type =~ m/^PORT$/...)

    # add spaces to lpar name to have 8 chars total (for formating graph legend)
    if ( $st_type =~ m/^DS8K$/ && $type =~ m/^POOL$/ ) {
      $lpar_space =~ s/extpool-P//;
      $lpar_space =~ s/extpool_P//;
      $lpar_space =~ s/extpool_//;
      $lpar_space =~ s/extpool-//;
      $lpar_space =~ s/extpool//;
    } ## end if ( $st_type =~ m/^DS8K$/...)
    $lpar_space =~ s/\.rr[a-z]//;
    $lpar_space = sprintf( "%-38s", "$lpar_space" );

    my $rrd_file       = "$line";
    my $rrd_file_first = $rrd_file;
    $rrd_file_first =~ s/rr.$/first/g;
    if ( $type_gr =~ m/d/ && $once == 0 ) {

      # find out time stamp of last data update
      # take just one volume to keep it simple
      $once++;
      RRDp::cmd qq(last "$line");
      my $last_tt = RRDp::read;
      chomp($$last_tt);
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
      $last_time_u = $$last_tt;
    } ## end if ( $type_gr =~ m/d/ ...)

    $found_data++;    # to avoid PPRC stats when there is no data files and empty PORT sas/iscsip/iprep ... stats

    # bulid RRDTool cmd
    my $itemm = $item . "m";
    $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/\#/Y/g;    # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)

    my $rrd_file_conv = $line;
    $rrd_file_conv =~ s/\.rr[a-z]//;
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing
    $lpar_space =~ s/:/\\:/g;
    $rrd_file_conv =~ s/:/\\:/g;

    # when UNKNOWN then place 0 --> there can be some old RANK datafiles without actual data then --> NaN in the GUI
    # if it is older than rrdtool first time then leave NaN to have proper average time
    # not summed then not care about NaN

    # put carriage return after each second lpar in the legend
    if ( $item =~ m/-/ ) {

      # multi item --> Netapp nodes only
      my @items = split( /-/, $item );
      foreach my $item_act (@items) {
        my $item_act_text = $item_act;
        $item_act_text =~ s/^3.*/NFS 3/;
        $item_act_text =~ s/^4.*/NFS 4/;
        $item_act_text =~ s/^cifs.*/CIFS/;
        $item_act_text =~ s/^iscsi.*/iSCSI/;
        $item_act_text =~ s/^fcp.*/FCP/;

        #$item_act_text =~ s/_/ /;
        $item_act_text =~ s/ //g;
        $lpar_space =~ s/ //g;
        my $lpar_space_act = sprintf( "%-38s", "$lpar_space $item_act_text" );

        # get color index
        my $ret_col = get_color( $lpar_space_act, $color_max, \@color_save );
        ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
        if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

          # new name has been identified, save it into an array and later on save into *col file
          $color_file_change = 1;
          $color_save[$color_indx] = $new_col_rec;
          ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
        } ## end if ( defined($new_col_rec...))

        $cmd .= " DEF:$item_act${i}_u=\\\"$line\\\":$item_act:AVERAGE";
        $cmd .= " CDEF:$item_act-m${i}=$item_act${i}_u,$val,/";                         # convert into MB if necessary, normaly is there 1
        $cmd .= " $gtype:$item_act-m${i}$color[$color_indx]:\\\"$lpar_space_act\\\"";
        if ( $item =~ m/io/ || $item !~ m/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) {
          $cmd       .= " GPRINT:$item_act-m${i}:AVERAGE:\\\"%6.0lf \\\"";
          $cmd       .= " GPRINT:$item_act-m${i}:MAX:\\\"%6.0lf \\l\\\"";
          $cmd_print .= " PRINT:$item_act-m${i}:AVERAGE:\\\"%6.0lf $delimiter $lpar_space_act $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
          $cmd_print .= " PRINT:$item_act-m${i}:MAX:\\\" %8.0lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";
        } ## end if ( $item =~ m/io/ ||...)
        else {
          $cmd       .= " GPRINT:$item_act-m${i}:AVERAGE:\\\"%6.1lf \\\"";
          $cmd       .= " GPRINT:$item_act-m${i}:MAX:\\\"%6.1lf \\l\\\"";
          $cmd_print .= " PRINT:$item_act-m${i}:AVERAGE:\\\"%6.1lf $delimiter $lpar_space_act $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
          $cmd_print .= " PRINT:$item_act-m${i}:MAX:\\\" %8.1lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";
        } ## end else [ if ( $item =~ m/io/ ||...)]
        if ( $item !~ m/resp_t/ && $item !~ m/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ && $item !~ m/^b2b$/ && $item !~ m/^crc$/ ) {
          $gtype = "STACK";
        }
        $i++;
      } ## end foreach my $item_act (@items)
    } ## end if ( $item =~ m/-/ )
    else {
      # get color index
      my $ret_col = get_color( $lpar_space, $color_max, \@color_save );
      ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
      if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

        # new name has been identified, save it into an array and later on save into *col file
        $color_file_change = 1;
        $color_save[$color_indx] = $new_col_rec;
        ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
      } ## end if ( defined($new_col_rec...))

      if ( $st_type eq "VNX" && $item eq "sys" ) {
        $cmd .= " DEF:busy_ticks${i}=\\\"$line\\\":busy_ticks:AVERAGE";
        $cmd .= " DEF:idle_ticks${i}=\\\"$line\\\":idle_ticks:AVERAGE";
        $cmd .= " CDEF:all_ticks${i}=busy_ticks${i},idle_ticks${i},+";
        $cmd .= " CDEF:util_l${i}=busy_ticks${i},all_ticks${i},/";
        $cmd .= " CDEF:$itemm${i}=util_l${i},100,*";
      } ## end if ( $st_type eq "VNX"...)
      elsif ( $st_type eq "VMAX" && ( $type eq "NODE" || $type eq "PORT" ) && ( $item =~ m/_backend$/ || $item =~ m/_frontend$/ || $item =~ m/_rdf$/ ) ) {
        my $new_item = $item;
        $new_item =~ s/_frontend$//;
        $new_item =~ s/_backend$//;
        $new_item =~ s/_rdf$//;
        $cmd .= " DEF:$item${i}_u=\\\"$line\\\":$new_item:AVERAGE";
        $cmd .= " CDEF:$itemm${i}=$item${i}_u,$val,/";                # convert into MB if necessary, normaly is there 1
      } ## end elsif ( $st_type eq "VMAX"...)
      elsif ( $st_type eq "VPLEX" && $type eq "PORT" && ( $item =~ m/_backend$/ || $item =~ m/_frontend$/ ) ) {
        my $new_item = $item;
        $new_item =~ s/_frontend$//;
        $new_item =~ s/_backend$//;
        $cmd .= " DEF:$item${i}_u=\\\"$line\\\":$new_item:AVERAGE";
        $cmd .= " CDEF:$itemm${i}=$item${i}_u,$val,/";                # convert into MB if necessary, normaly is there 1
      } ## end elsif ( $st_type eq "VPLEX"...)
      else {
        $cmd .= " DEF:$item${i}_u=\\\"$line\\\":$item:AVERAGE";
        $cmd .= " CDEF:$itemm${i}=$item${i}_u,$val,/";                # convert into MB if necessary, normaly is there 1
      }

      $cmd .= " $gtype:$itemm${i}$color[$color_indx]:\\\"$lpar_space\\\"";

      if ( $item =~ m/io/ ) {
        $cmd       .= " GPRINT:$itemm${i}:AVERAGE:\\\"%6.0lf \\\"";
        $cmd       .= " GPRINT:$itemm${i}:MAX:\\\"%6.0lf \\l\\\"";
        $cmd_print .= " PRINT:$itemm${i}:AVERAGE:\\\"%6.0lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
        $cmd_print .= " PRINT:$itemm${i}:MAX:\\\" %8.0lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";
      } ## end if ( $item =~ m/io/ )
      else {
        if ( $item =~ m/^sys$/ || $item =~ m/^compress$/ || $item =~ m/^operating_rate$/ ) {
          $cmd       .= " GPRINT:$itemm${i}:AVERAGE:\\\"%3.1lf \\\"";
          $cmd       .= " GPRINT:$itemm${i}:MAX:\\\"%3.1lf \\l\\\"";
          $cmd_print .= " PRINT:$itemm${i}:AVERAGE:\\\"%3.1lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
          $cmd_print .= " PRINT:$itemm${i}:MAX:\\\" %3.1lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";
        } ## end if ( $item =~ m/^sys$/...)
        else {
          $cmd       .= " GPRINT:$itemm${i}:AVERAGE:\\\"%6.1lf \\\"";
          $cmd       .= " GPRINT:$itemm${i}:MAX:\\\"%6.1lf \\l\\\"";
          $cmd_print .= " PRINT:$itemm${i}:AVERAGE:\\\"%6.1lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
          $cmd_print .= " PRINT:$itemm${i}:MAX:\\\" %6.1lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";
        } ## end else [ if ( $item =~ m/^sys$/...)]
      } ## end else [ if ( $item =~ m/io/ ) ]
    } ## end else [ if ( $item =~ m/-/ ) ]
    if ( $item !~ m/operating_rate/ && $item !~ m/resp_t/ && $item !~ m/pprc_rt_/ && $item !~ m/^cache_hit$/ && $item !~ m/^w_cache_hit$/ && $item !~ m/^r_cache_hit$/ && $item !~ m/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ && $item !~ m/^b2b$/ && $item !~ m/^crc$/ && $item !~ m/^r_cache_usage$/ && $item !~ m/^w_cache_usage$/ &&  $item !~ m/r_transfer_size/ && $item !~ m/w_transfer_size/  ) {
      $gtype = "STACK";
    }
    if ( $st_type eq "VNX" && $type eq "NODE" ) {
      if ( $item eq "dirty_cache_pages" || $item eq "sys" ) {
        $gtype = "LINE1";
      }
    }
    if ( $st_type =~ m/^VMAX$/ && ( $type =~ m/^PORT$/ || $type =~ m/^NODE$/ ) && $item =~ m/busy/ ) {
      $gtype = "LINE1";
    }
    if ( $st_type =~ m/^VPLEX$/ && $type =~ m/^NODE$/ && $item =~ m/busy/ ) {
      $gtype = "LINE1";
    }
    if ( $st_type =~ m/^COMPELLENT|OCEANSTOR$/ && $type =~ m/^NODE$/ && $item =~ m/sys/ ) {
      $gtype = "LINE1";
    }
    if ( ( $st_type =~ m/^3PAR$/ || $st_type =~ m/^SWIZ$/ || $st_type =~ m/^COMPELLENT$|^OCEANSTOR$/ ) && $type =~ m/^PORT$/ ) {

      # do not stack front-end ports (FC) together with back-end ports (SAS)
      $gtype = "LINE1";
    }
    $i++;
  } ## end foreach my $line (<$dir/*$suffix>)

  if ( $found_data == 0 && $item =~ m/pprc/ ) {
    print "no data files  : $host:$type:$item - skipped\n" if $DEBUG;
    return 1;    # mainly to avoid PPRC stats when there is no data files
  }

  if ( $type_gr =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  if ( $i > 0 && ( $item =~ m/^sys$/ || $item =~ m/^compress$/ || $item =~ m/^operating_rate$/ ) ) {
    my $total = $i * 100;

    #$cmd .= " LINE2:$total#000000:\\\"Total    $total % \\\"";
    if ( $st_type ne "UNITY" ) {
      $cmd .= " COMMENT:\\\"Total available \\\: $total %  \\\: $i CPU cores \\\"";
    }
  } ## end if ( $i > 0 && ( $item...))

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS

  $cmd .= $cmd_print;    # for clickable legend

  $cmd =~ s/\\"/"/g;

  # It must be here!!!, touch() only in case there is any rrdfile
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  # write colors into a file
  write_colors( $color_file_change, $file_col, \@color_save );

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  if ( $not_execute_rrdtool == 1 ) { return 1; }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < $tmp_file 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  # Create dirs here when all is fine above

  return 0;
} ## end sub draw_graph

sub draw_all_pool {
  my $host    = shift;
  my $type    = shift;
  my $item    = shift;
  my $st_type = shift;

  draw_graph_pool( "day",     "d", "MINUTE:60:HOUR:2:HOUR:4:0:%H", $host, $type, $item, $st_type );
  draw_graph_pool( "week",    "w", "HOUR:8:DAY:1:DAY:1:86400:%a",  $host, $type, $item, $st_type );
  draw_graph_pool( "4 weeks", "m", "DAY:1:DAY:2:DAY:2:0:%d",       $host, $type, $item, $st_type );
  draw_graph_pool( "year",    "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, $item, $st_type );
  return 0;
} ## end sub draw_all_pool

sub draw_graph_pool {
  my $text      = shift;
  my $type_gr   = shift;
  my $xgrid     = shift;
  my $host      = shift;
  my $type      = shift;
  my $item      = shift;
  my $st_type   = shift;
  my $name      = "$tmp_dir/$host-$type-$item-agg-$type_gr";
  my $t         = "COMMENT: ";
  my $t2        = "COMMENT:\\n";
  my $last      = "COMMENT: ";
  my $rrd_time  = "";
  my $step_new  = $STEP;
  my $once      = 0;
  my $last_time = "na";
  my $item_org  = $item;

  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";
  my $act_utime = time();

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  print "creating cmd   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $header   = "$type aggregated $item: last $text";
  if ( $item =~ m/resp_t/ ) {
    $header = "$type $item: last $text";
  }
  if ($item =~ m/io_pending/ && $st_type eq "COMPELLENT" && $type eq "POOL"){
    $header = "$type $item: last $text";
  }
  my $i    = 0;
  my $lpar = "";
  my $cmd  = "";
  my $j    = 0;

  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now-1$type_gr+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;

  # 3PAR & VSPG have native _b data in pool rrd files!!
  if ( $st_type !~ m/^VSPG/ && $st_type !~ m/^3PAR/ && $st_type !~ m/^VMAX/ ) {

    # cut back-end info for DS8k, and might be other
    # VSPG back up matrics finishes with _b (in rrdtool)
    $item =~ s/_b$//;
  } ## end if ( $st_type !~ m/^VSPG/...)

  # do not use switch statement
  if ( $item =~ m/^data_rate/ ) { $value_short = "MB/sec"; $value_long = "MBytes/sec";   $val = 1024; }
  if ( $item =~ m/^read$/ )     { $value_short = "MB/sec"; $value_long = "MBytes/sec";   $val = 1024; }
  if ( $item =~ m/^write$/ )    { $value_short = "MB/sec"; $value_long = "MBytes/sec";   $val = 1024; }
  if ( $item =~ m/^io_rate/ )   { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^read_io/ )   { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^write_io/ )  { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^read_b$/ )   { $value_short = "MB/sec"; $value_long = "MBytes/sec";   $val = 1024; }
  if ( $item =~ m/^write_b$/ )  { $value_short = "MB/sec"; $value_long = "MBytes/sec";   $val = 1024; }
  if ( $item =~ m/^resp_t/ )    { $value_short = "ms";     $value_long = "milliseconds"; $val = 1; }
  if ( $item =~ m/^busy$/ ) { $value_short = "percent"; $value_long = "percent"; }
  if ( $item =~ m/^r_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^w_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^io_pending$/ ) { $value_short = "IOPS"; $value_long = "IO per second"; }

  $cmd .= " --vertical-label=\\\"$item in $value_long\\\"";

  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $legend = sprintf( "%-38s", "$item [$value_short]" );

  my $legend_heading = "$item $delimiter [$value_short] $delimiter Avrg $delimiter Max ";
  $cmd .= " COMMENT:\\\"$legend        Avrg       Max\\l\\\"";

  my $gtype = "AREA";
  if ( $item =~ m/resp_t|busy/ ) {
    $gtype = "LINE1";
  }
  if ( $st_type eq "VPLEX" && $item eq "io_rate" ) {
    $gtype = "LINE1";
  }
  if ( $st_type eq "COMPELLENT" && $item eq "io_pending" && $type eq "POOL" ) {
    $gtype = "LINE1";
  }

  my $lpar_list     = "";
  my $pool_found    = 0;
  my $pool_id_tmp   = "";
  my $lpar_list_tmp = "";
  my $searchstr     = "-P[0-9]*";

  my $rank_vol = "RANK";
  if ( $st_type =~ m/3PAR/ || $st_type =~ m/DS5K/ || $st_type =~ m/XIV/ || $st_type =~ m/HUS/ || $st_type =~ m/NETAPP/ || $st_type =~ m/VSPG/ || $st_type =~ m/^VNX$/ || $st_type =~ m/^VNXF$/ || $st_type =~ m/UNITY/ || $st_type =~ m/^VNXE$/ || $st_type =~ m/DOTHILL/ || $st_type =~ m/VMAX/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/VPLEX/ || $st_type =~ m/^OCEANSTOR$/ || $st_type =~ m/^PURE$/) {
    $searchstr = "";
    $rank_vol  = "POOL";
  }
  if ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ ) {
    $rank_vol = "VOLUME";    # XIV uses volumes
  }
  if ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ ) {
    $rank_vol = "VOLUME";    # XIV uses volumes
  }
  if ( $st_type =~ m/UNITY/ && $type =~ m/NASPOOL/ ) {
    $searchstr = "";
    $rank_vol  = "NASPOOL";
  }
  if ( $st_type =~ m/^VNXF$/ && $type =~ m/NASPOOL/ ) {
    $searchstr = "";
    $rank_vol  = "NASPOOL";
  }
  if ( $st_type =~ m/^VMAX$/ && $type =~ m/TIER/ ) {
    $searchstr = "";
    $rank_vol  = "TIER";
  }

  # DS8K && 3PAR front-end stats are created in draw_all

  my $dir = "$wrkdir/$host/$rank_vol";
  chdir $dir;

  my @rank_list      = "";
  my $rank_list_indx = 0;

  # check all ranks and then sort them per pool id
  foreach my $line (<*$searchstr\.rrd>) {
    chomp($line);
    if ( $line !~ m/\.rrd$/ || $line =~ m/-cap\.rrd/ ) {
      next;    # it does not end bu .rrd suffix, "$ cannt be use in foreach <> and exclud capacity rrd files
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    $rank_list[$rank_list_indx] = $line;
    $rank_list_indx++;
  } ## end foreach my $line (<*$searchstr\.rrd>)

  if ( $rank_list_indx == 0 ) {

    # no new data found, skipping ...
    print "creating cmd   : $host:$type:$item_org:$type_gr no new data found, skipping ...\n" if $DEBUG;
    return 0;
  } ## end if ( $rank_list_indx ==...)

  my @rank_list_unsorted = @rank_list;
  if ( $rank_list[0] =~ m/-.*/ && $st_type !~ m/VSPG/ && $st_type !~ m/NETAPP/ && $st_type !~ m/^VNX/ && $st_type !~ m/UNITY/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/COMPELLENT/ && $st_type !~ m/^OCEANSTOR$/ ) {

    # netapp 3857b20f-2fc7-4bbf-a023-5b040cf17099.rrd
    @rank_list = sort { ( split '-', $a )[1] cmp( split '-', $b )[1] } @rank_list_unsorted;

    # it is important for 3PAR sorting, woithout that it shows more pools with ID 0 in graphs (especially yearly)
  } ## end if ( $rank_list[0] =~ ...)
  else {
    @rank_list = sort @rank_list_unsorted;
  }

  my $pool_id_old = -1;

  my $print_now         = 0;
  my $index_last        = 0;
  my $rrd_file_last     = "";
  my $index_actual      = 0;
  my $pool_name_space   = "NA";
  my $pool_id           = 0;
  my $pool_name_actual  = "NA";
  my $itemm             = "";
  my $no_of_ranks       = 0;
  my $print_now_pool_id = 0;

  # open a file with stored colours
  my @color_save = "";
  my $type_low   = lc $type;
  my $file_col   = "$wrkdir/$host/$type/" . $type_low . "s.col";
  if ( -f $file_col ) {
    open( FHC, "< $file_col" ) || error( "file does not exists : $file_col " . __FILE__ . ":" . __LINE__ ) && return 0;
    @color_save = <FHC>;
    close(FHC);
  }
  my $color_indx        = 0;    # reset colour index
  my $color_file_change = 0;

  # go through all ranks sorted per pool id, once a pool id is changed then is printed the old one
  foreach my $line (@rank_list) {
    chomp($line);
    if ( $st_type =~ m/NETAPP/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    if ( $st_type =~ m/^VNX$/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    if ( $st_type =~ m/^UNITY$/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    if ( $st_type =~ m/^VMAX$/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    if ( $st_type =~ m/^VNXE$/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    if ( $st_type =~ m/^DOTHILL$/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    if ( $st_type =~ m/^COMPELLENT$/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    if ( $st_type =~ m/^OCEANSTOR$/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    if ( $st_type =~ m/^PURE$/ && $line =~ m/-cap\.rrd$/ ) {
      next;                     # skip capacity rrd files for netapp
    }
    $pool_id = $line;
    $pool_id =~ s/^.*-P//;
    $pool_id =~ s/\.rrd$//;

    my $ret = ishexa($pool_id);
    if ( $ret == 0 && $st_type !~ m/^VNX$/ && $st_type !~ m/^UNITY$/ && $st_type !~ m/^VNXE$/ && $st_type !~ m/^VMAX$/ && $st_type !~ m/^VPLEX$/ ) {
      next;
    }

    if ( ishexa($pool_id_old) && $pool_id_old =~ m/^-1$/ ) {

      # could be already haxa here
      $pool_id_old = $pool_id;
    }
    else {
      if ( $pool_id_old !~ m/^$pool_id$/ ) {
        $print_now         = 1;
        $print_now_pool_id = $pool_id_old;
        $pool_name_actual  = $pool_name_space;
        $pool_id_old       = $pool_id;
      } ## end if ( $pool_id_old !~ m/^$pool_id$/)
    } ## end else [ if ( ishexa($pool_id_old...))]

    # get pool name, be aware this is a new pool name
    #print "002 $line\n";
    my $pool_name = "NA";
    $pool_name = get_pool_name($pool_id);
    if ( $st_type eq "VMAX" && $type eq "TIER" ) {
      $pool_name = $pool_id;
    }
    $pool_name_space = $pool_name;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $pool_name_space =~ s/extpool_P//;
    $pool_name_space =~ s/extpool-P//;
    $pool_name_space =~ s/extpool-//;
    $pool_name_space =~ s/extpool_//;
    $pool_name_space =~ s/extpool//;
    $pool_name_space = sprintf( "%-38s", "$pool_name_space" );

    my $rrd_file = "$dir/$line";
    $rrd_file_last = "$dir/$line";
    if ( "$type_gr" =~ "d" ) {

      # find out time stamp of last data update
      # take just one volume to keep it simple
      RRDp::cmd qq(last "$rrd_file");
      my $last_tt = RRDp::read;
      chomp($$last_tt);
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
    } ## end if ( "$type_gr" =~ "d")

    my $time_first = find_real_data_start_pool( "$dir", $pool_id, $st_type );

    $itemm = $item . "m";
    $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/\#/Y/g;    # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    $cmd .= " DEF:$item${i}_u=\\\"$dir/$line\\\":$item:AVERAGE";

    #$cmd .= " VDEF:$item${i}_avrg_u=$item${i}_u,AVERAGE"; # gets 1 everage value per all time range
    # TIME,937521357,GT,value,value,UN,0,value,IF,IF
    # if it is older than rrdtool first time
    # e then leave NaN to have proper average time
    # Do not use UNKN as one NaN in sum causes NaN in total
    $cmd .= " CDEF:$item${i}=TIME,$time_first,LT,$item${i}_u,$item${i}_u,UN,0,$item${i}_u,IF,IF";
    $cmd .= " CDEF:$itemm${i}=$item${i},$val,/";

    #print "001 $line - $print_now - $i - $index_actual : $index_last\n";

    if ($print_now) {

      $print_now    = 0;
      $index_actual = $index_last;
      $cmd .= " CDEF:pool${i}=$itemm${index_actual},0,+";    # workaround for pools with 1 rank
      $index_actual++;
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",$itemm${index_actual},+";
      }

      if ( $item =~ m/resp_t/ ) {

        # response time is not summary, must be average of all time (devide number of ranks)
        $cmd .= ",$no_of_ranks,/";
      }
      $no_of_ranks = 0;

      $pool_name_actual =~ s/extpool_P//;
      $pool_name_actual =~ s/extpool-P//;
      $pool_name_actual =~ s/extpool-//;
      $pool_name_actual =~ s/extpool_//;
      $pool_name_actual =~ s/extpool//;
      my $rrd_file_conv = $rrd_file;
      if ( $st_type =~ m/^3PAR$/ || $st_type =~ m/^DS8K$/ || $st_type =~ m/^SWIZ$/ ) {

        # DS8K stats from RANK mus be converted here to point to POOL to make working legend links
        $rrd_file_conv =~ s/\/RANK\/.*/\/POOL\/$print_now_pool_id\.rrd/;
      }
      if ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ ) {
        $rrd_file_conv =~ s/\/VOLUME\/.*/\/POOL\/$print_now_pool_id\.rrd/;
      }
      if ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ ) {
        $rrd_file_conv =~ s/\/VOLUME\/.*/\/POOL\/$print_now_pool_id\.rrd/;
      }

      #print "00 $rrd_file - $rrd_file_conv - $print_now_pool_id\n";

      # get color index
      my $ret_col = get_color( $pool_name_actual, $color_max, \@color_save );
      ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
      if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

        # new name has been identified, save it into an array and later on save into *col file
        $color_file_change = 1;
        $color_save[$color_indx] = $new_col_rec;
        ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
      } ## end if ( defined($new_col_rec...))

      $cmd .= " $gtype:pool${i}$color[$color_indx]:\\\"$pool_name_actual \\\"";
      if ( $item =~ m/resp_t/ ) {
        $cmd .= " GPRINT:pool${i}:AVERAGE:\\\"%7.1lf \\\"";
        $cmd .= " GPRINT:pool${i}:MAX:\\\"%7.1lf \\l\\\"";
        $cmd .= " PRINT:pool${i}:AVERAGE:\\\"%7.1lf $delimiter $pool_name_actual $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
        $cmd .= " PRINT:pool${i}:MAX:\\\" %7.1lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";
      } ## end if ( $item =~ m/resp_t/)
      else {
        $cmd .= " GPRINT:pool${i}:AVERAGE:\\\"%7.0lf \\\"";
        $cmd .= " GPRINT:pool${i}:MAX:\\\"%7.0lf \\l\\\"";
        $cmd .= " PRINT:pool${i}:AVERAGE:\\\"%7.0lf $delimiter $pool_name_actual $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
        $cmd .= " PRINT:pool${i}:MAX:\\\" %7.0lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";
      } ## end else [ if ( $item =~ m/resp_t/)]

      $index_last = $i;
      if ( $item !~ m/resp_t|busy/ ) {
        $gtype = "STACK";
      }
      if ( $st_type eq "VPLEX" && $item eq "io_rate" ) {
        $gtype = "LINE1";
      }
      if ( $st_type eq "COMPELLENT" && $item eq "io_pending" && $type eq "POOL" ) {
        $gtype = "LINE1";
      }
    } ## end if ($print_now)
    $no_of_ranks++;
    $i++;
  } ## end foreach my $line (@rank_list)

  $cmd .= " CDEF:pool${i}=$itemm${index_actual},0,+";    # workaround for pools with 1 rank
  $index_actual++;
  for ( ; $index_actual < $i; $index_actual++ ) {
    $cmd .= ",$itemm${index_actual},+";
  }

  if ( $item =~ m/resp_t/ ) {

    # response time is not summary, must be average of all time (devide number of ranks)
    $cmd .= ",$no_of_ranks,/";
  }

  $pool_name_space =~ s/extpool_P//;
  $pool_name_space =~ s/extpool-P//;
  $pool_name_space =~ s/extpool-//;
  $pool_name_space =~ s/extpool_//;
  $pool_name_space =~ s/extpool//;
  my $rrd_file_conv = $rrd_file_last;
  if ( $st_type =~ m/3PAR/ || $st_type =~ m/^DS8K$/ || $st_type =~ m/^SWIZ$/ ) {

    # DS8K back end stats from RANK mus be converted here to point to POOL
    $rrd_file_conv =~ s/\/RANK\/.*/\/POOL\/$pool_id\.rrd/;    # file does not have to exist, ID is parsed in the GUI
                                                              #$rrd_file_conv =~ s/\/RANK\/.*-P/\/POOL\//;
  } ## end if ( $st_type =~ m/3PAR/...)
  if ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ ) {
    $rrd_file_conv =~ s/\/VOLUME\/.*/\/POOL\/$print_now_pool_id\.rrd/;
  }
  if ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ ) {
    $rrd_file_conv =~ s/\/VOLUME\/.*/\/POOL\/$print_now_pool_id\.rrd/;
  }

  # get color index
  my $ret_col = get_color( $pool_name_space, $color_max, \@color_save );
  ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
  if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

    # new name has been identified, save it into an array and later on save into *col file
    $color_file_change = 1;
    $color_save[$color_indx] = $new_col_rec;
    ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
  } ## end if ( defined($new_col_rec...))

  $cmd .= " $gtype:pool${i}$color[$color_indx]:\\\"$pool_name_space \\\"";
  if ( $item =~ m/resp_t/ ) {
    $cmd .= " PRINT:pool${i}:AVERAGE:\\\"%7.1lf $delimiter $pool_name_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
    $cmd .= " PRINT:pool${i}:MAX:\\\" %7.1lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";

    $cmd .= " GPRINT:pool${i}:AVERAGE:\\\"%7.1lf \\\"";
    $cmd .= " GPRINT:pool${i}:MAX:\\\"%7.1lf \\l\\\"";
  } ## end if ( $item =~ m/resp_t/)
  else {
    $cmd .= " PRINT:pool${i}:AVERAGE:\\\"%7.0lf $delimiter $pool_name_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
    $cmd .= " PRINT:pool${i}:MAX:\\\" %7.0lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";

    $cmd .= " GPRINT:pool${i}:AVERAGE:\\\"%7.0lf \\\"";
    $cmd .= " GPRINT:pool${i}:MAX:\\\"%7.0lf \\l\\\"";
  } ## end else [ if ( $item =~ m/resp_t/)]

  if ( $type_gr =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # It must be here!!!, touch() only in case there is any rrdfile
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  # write colors into a file
  write_colors( $color_file_change, $file_col, \@color_save );

  # add last pool
  $index_actual = $index_last;

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  #$cmd =~ s/ /\n/g;
  #print  "00 $cmd\n";

  if ( $not_execute_rrdtool == 1 ) { return 1; }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < $tmp_file 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  return 0;
} ## end sub draw_graph_pool

sub cfg_copy {
  my $wrkdir   = shift;
  my $webdir   = shift;
  my $host     = shift;
  my $act_time = shift;

  if ( -f "$wrkdir/$host/config.html" ) {
    copy( "$wrkdir/$host/config.html", "$webdir/$host/config.html" ) || error( "Can't copy cfg file : $wrkdir/$host/config.html to $webdir/$host/config.html : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  return 1;
} ## end sub cfg_copy

sub draw_all_sum {
  my $host       = shift;
  my $type       = shift;
  my $item       = shift;
  my $st_type    = shift;
  my $time_first = shift;

  draw_graph_sum( "day",     "d", "MINUTE:60:HOUR:2:HOUR:4:0:%H", $host, $type, $item, $st_type, $time_first );
  draw_graph_sum( "week",    "w", "HOUR:8:DAY:1:DAY:1:86400:%a",  $host, $type, $item, $st_type, $time_first );
  draw_graph_sum( "4 weeks", "m", "DAY:1:DAY:2:DAY:2:0:%d",       $host, $type, $item, $st_type, $time_first );
  draw_graph_sum( "year",    "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, $item, $st_type, $time_first );

  if ( $item eq "sum_io" || $item eq "sum_io_total" || $item eq "sum_data" || $item eq "sum_data_total" ) {    # create trend graphs for these items
    draw_graph_sum_trend( "year", "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, "$item\_trend", $st_type, $time_first );
  }
  if ( $item eq "sum_io_b" || $item eq "sum_data_b" ) {                                                        # create trend graphs for these items
    draw_graph_sum_trend( "year", "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, "$item\_trend", $st_type, $time_first );
  }

  return 0;
} ## end sub draw_all_sum

sub draw_graph_sum_trend {
  my $text       = shift;
  my $type_gr    = shift;
  my $xgrid      = shift;
  my $host       = shift;
  my $type       = shift;
  my $item       = shift;
  my $st_type    = shift;
  my $time_first = shift;
  my $name       = "$tmp_dir/$host-$type-$item-agg-$type_gr";
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $last       = "COMMENT: ";
  my $rrd_time   = "";
  my $step_new   = $STEP;
  my $once       = 0;
  my $last_time  = "na";

  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";
  my $act_utime = time();

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  my $type_text = $type;
  if ( $st_type =~ m/SWIZ/ && $type =~ m/RANK/ ) {
    $type_text = "MDISK";
  }
  if ( $st_type =~ m/HUS/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/NETAPP/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/VSPG/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/PURE/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }

  print "creating cmd   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $header   = "$type_text aggregated $item: last $text";
  if ( $item =~ m/resp_t/ ) {
    $header = "$type_text $item: last $text";
  }
  my $i    = 0;
  my $y    = -1;
  my $lpar = "";
  my $cmd  = "";
  my $j    = 0;

  my $color_indx = 0;    # reset colour index

  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width_trend";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;

  # do not use switch statement
  if ( $item =~ m/^sum_data_trend$/ )   { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^sum_io_trend$/ )     { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^sum_data_b_trend$/ ) { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^sum_io_b_trend$/ )   { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^sum_io_total_trend$/ ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^sum_data_total_trend$/ ) { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }

  # Why??? There should be 1024
  if ( $st_type =~ m/^DS8K$/ && $type !~ /RANK/ ) {
    if ( $item =~ m/^sum_data_trend$/ ) {
      $val = 1;
    }
  }

  my $item_label = uc($item);
  $item_label =~ s/SUM_//;
  $cmd .= " --vertical-label=\\\"$item_label in $value_long\\\"";

  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $legend = sprintf( "%-38s", "$item [$value_short]" );
  $cmd .= " COMMENT:\\\"$legend          Avrg         Max \\l\\\"";

  my $gtype = "LINE";

  my $lpar_list      = "";
  my $pool_found     = 0;
  my $pool_id_tmp    = "";
  my $lpar_list_tmp  = "";
  my $itemm          = "";
  my $nothing_to_sum = 1;

  my $type_real = $type;    # real directory for file search --> 3PAR use VOLUME for POOL sum_io and sum_data
  if ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ && $type =~ m/POOL/ ) {
    if ( $item =~ m/sum_io/ || $item =~ m/sum_data/ ) {
      $type_real = "VOLUME";
    }
  }
  if ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ && $type =~ m/POOL/ ) {
    if ( $item =~ m/sum_io/ || $item =~ m/sum_data/ ) {
      $type_real = "VOLUME";
    }
  }

  my $suffix = "rrd";
  if ( $st_type eq "VMAX" && $type eq "POOL" && $item !~ m/^sum_capacity/ && $item !~ m/_b$/ ) {
    $suffix = "rra";
  }
  my $host_space = check_space_name($host);
  foreach my $line (<$wrkdir/$host_space/$type_real/*$suffix>) {
    chomp($line);
    if ( $line !~ m/\.rrd$/ && $st_type ne "VMAX" ) {
      next;    # it does not end by .rrd suffix, "$ cannt be use in foreach <>
    }
    if ( $type =~ m/TIER/ && $line !~ m/-cap\.rrd$/ && $item =~ m/^sum_capacity$/ ) {
      next;
    }
    if ( $type =~ m/TIER/ && $line =~ m/-cap\.rrd$/ && $item !~ m/^sum_capacity$/ ) {
      next;
    }

    if ( $st_type !~ m/NETAPP/ && $st_type !~ m/HUS/ && $type =~ m/RANK/ && $line !~ m/\/.*-P[0-9][0-9]*\.rrd$/ && $st_type !~ m/VSPG/ && $st_type !~ m/VNXF/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/PURE/ ) {

      #workaround for cleaning up ranks without pool id (could hapened in the past by the error)
      next;
    }
    if ( $type =~ m/POOL/ ) {
      if ( $st_type !~ m/HUS/ && $st_type !~ m/VSPG/ && $line !~ m/-cap\.rrd$/ && $st_type !~ m/VNXF/ && $st_type !~ m/DOTHILL/ ) {

        #HUS have capacity in pool.rrd
        # SVC & NetApp at least
        if ( $item =~ m/^sum_capacity$/ || $item =~ m/^tier[0-9]/ ) {
          next;    # use only *-cap.rrd file for POOL capacity stuff (there can be even POOL front-end stuff like $pool_id.rrd
        }
      } ## end if ( $st_type !~ m/HUS/...)
    } ## end if ( $type =~ m/POOL/ )
    if ( $type =~ m/POOL/ && $line =~ m/-cap\.rrd$/ ) {
      if ( $item =~ m/^sum_io/ || $item =~ m/^sum_data/ || $item =~ m/^sum_resp/ ) {
        next;      # skip *-cap.rrd files for sum_io and sum_data
      }
    }

    if ( $st_type =~ m/SWIZ/ && $type =~ m/POOL/ && $line =~ m/-cap.rrd$/ && $item =~ m/^tier[0-9]/ ) {
      if ( $item =~ m/tier0-nl/ && $line !~ m/nl-cap\.rrd$/ ) {

        # new TIER0-NL implementation in separate file 0-nl-cap.rrd
        next;
      }
      if ( $item =~ m/tier[0-9]$/ && $line =~ m/nl-cap\.rrd$/ ) {
        next;
      }
    } ## end if ( $st_type =~ m/SWIZ/...)

    if ( $st_type =~ m/SWIZ/ && $type =~ m/POOL/ && $line =~ m/-comp\.rrd$/ ) {
      next;    # exclude data with compression -comp.rrd
    }

    if ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ && $type =~ m/POOL/ ) {
      if ( $item =~ m/sum_io/ || $item =~ m/sum_data/ ) {
        my $line_base = basename($line);
        if ( $line_base !~ m/.*-P.*\.rrd$/ ) {
          next;    # it is not POOL data file
        }
      } ## end if ( $item =~ m/sum_io/...)
    } ## end if ( $ds5k_pool_org ==...)
    if ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ && $type =~ m/POOL/ ) {
      if ( $item =~ m/sum_io/ || $item =~ m/sum_data/ ) {
        my $line_base = basename($line);
        if ( $line_base !~ m/.*-P.*\.rrd$/ ) {
          next;    # it is not POOL data file
        }
      } ## end if ( $item =~ m/sum_io/...)
    } ## end if ( $xiv_pool_org == ...)

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }
    $nothing_to_sum = 0;

    my $rrd_file = "$line";
    if ( $type_gr =~ m/d/ && $once == 0 ) {
      $once++;

      # find out time stamp of last data update
      # take just one volume to keep it simple
      RRDp::cmd qq(last "$rrd_file");
      my $last_tt = RRDp::read;
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
    } ## end if ( $type_gr =~ m/d/ ...)

    my @link_l = split( /\//, $line );
    my $lpar = "";
    foreach my $m (@link_l) {
      $lpar = $m;
    }
    $lpar =~ s/\.rrd$//;
    $lpar =~ s/-P.*//;    # filter pool info
    $lpar =~ s/PORT-//;
    $lpar =~ s/RANK-//;
    $lpar =~ s/MDISK-//;
    $lpar =~ s/VOLUME-//;
    $lpar =~ s/0x//;
    $lpar =~ s/-cap//;

    # bulid RRDTool cmd
    $itemm = $item . "m";
    $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/\#/Y/g;    # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    # do not use below UNKN as this is total sum and one NaN --. whole sum is NaN
    if ( $item =~ m/sum_io_trend$/ ) {
      $cmd .= " DEF:$item${i}_r=\\\"$line\\\":read_io:AVERAGE";
      $cmd .= " DEF:$item${i}_w=\\\"$line\\\":write_io:AVERAGE";

      # 1 month
      $cmd .= " DEF:$item${i}_r_1m=\\\"$line\\\":read_io:AVERAGE:start=-1m";
      $cmd .= " DEF:$item${i}_w_1m=\\\"$line\\\":write_io:AVERAGE:start=-1m";

      # 3 month
      $cmd .= " DEF:$item${i}_r_3m=\\\"$line\\\":read_io:AVERAGE:start=-3m";
      $cmd .= " DEF:$item${i}_w_3m=\\\"$line\\\":write_io:AVERAGE:start=-3m";

      # 1 year
      $cmd .= " DEF:$item${i}_r_1y=\\\"$line\\\":read_io:AVERAGE:start=-1y";
      $cmd .= " DEF:$item${i}_w_1y=\\\"$line\\\":write_io:AVERAGE:start=-1y";
    } ## end if ( $item =~ m/sum_io_trend$/)
    if ( $item =~ m/sum_io_b_trend$/ ) {
      $cmd .= " DEF:$item${i}_r=\\\"$line\\\":read_io_b:AVERAGE";
      $cmd .= " DEF:$item${i}_w=\\\"$line\\\":write_io_b:AVERAGE";

      # 1 month
      $cmd .= " DEF:$item${i}_r_1m=\\\"$line\\\":read_io_b:AVERAGE:start=-1m";
      $cmd .= " DEF:$item${i}_w_1m=\\\"$line\\\":write_io_b:AVERAGE:start=-1m";

      # 3 month
      $cmd .= " DEF:$item${i}_r_3m=\\\"$line\\\":read_io_b:AVERAGE:start=-3m";
      $cmd .= " DEF:$item${i}_w_3m=\\\"$line\\\":write_io_b:AVERAGE:start=-3m";

      # 1 year
      $cmd .= " DEF:$item${i}_r_1y=\\\"$line\\\":read_io_b:AVERAGE:start=-1y";
      $cmd .= " DEF:$item${i}_w_1y=\\\"$line\\\":write_io_b:AVERAGE:start=-1y";
    } ## end if ( $item =~ m/sum_io_b_trend$/)
    if ( $item =~ m/sum_data_trend$/ ) {
      $cmd .= " DEF:$item${i}_r=\\\"$line\\\":read:AVERAGE";
      $cmd .= " DEF:$item${i}_w=\\\"$line\\\":write:AVERAGE";

      # 1 month
      $cmd .= " DEF:$item${i}_r_1m=\\\"$line\\\":read:AVERAGE:start=-1m";
      $cmd .= " DEF:$item${i}_w_1m=\\\"$line\\\":write:AVERAGE:start=-1m";

      # 3 month
      $cmd .= " DEF:$item${i}_r_3m=\\\"$line\\\":read:AVERAGE:start=-3m";
      $cmd .= " DEF:$item${i}_w_3m=\\\"$line\\\":write:AVERAGE:start=-3m";

      # 1 year
      $cmd .= " DEF:$item${i}_r_1y=\\\"$line\\\":read:AVERAGE:start=-1y";
      $cmd .= " DEF:$item${i}_w_1y=\\\"$line\\\":write:AVERAGE:start=-1y";
    } ## end if ( $item =~ m/sum_data_trend$/)
    if ( $item =~ m/sum_data_b_trend$/ ) {
      $cmd .= " DEF:$item${i}_r=\\\"$line\\\":read_b:AVERAGE";
      $cmd .= " DEF:$item${i}_w=\\\"$line\\\":write_b:AVERAGE";

      # 1 month
      $cmd .= " DEF:$item${i}_r_1m=\\\"$line\\\":read_b:AVERAGE:start=-1m";
      $cmd .= " DEF:$item${i}_w_1m=\\\"$line\\\":write_b:AVERAGE:start=-1m";

      # 3 month
      $cmd .= " DEF:$item${i}_r_3m=\\\"$line\\\":read_b:AVERAGE:start=-3m";
      $cmd .= " DEF:$item${i}_w_3m=\\\"$line\\\":write_b:AVERAGE:start=-3m";

      # 1 year
      $cmd .= " DEF:$item${i}_r_1y=\\\"$line\\\":read_b:AVERAGE:start=-1y";
      $cmd .= " DEF:$item${i}_w_1y=\\\"$line\\\":write_b:AVERAGE:start=-1y";
    } ## end if ( $item =~ m/sum_data_b_trend$/)
    if ( $item =~ m/sum_io_total/ ) {

      # DS5K does not have read/write details
      # To make it easier use _r and _w here
      $cmd .= " DEF:$item${i}_r=\\\"$line\\\":io_rate:AVERAGE";
      $cmd .= " CDEF:$item${i}_w=$item${i}_r,0,*";

      # 1 month
      $cmd .= " DEF:$item${i}_r_1m=\\\"$line\\\":io_rate:AVERAGE:start=-1m";
      $cmd .= " CDEF:$item${i}_w_1m=$item${i}_r_1m,0,*";

      # 3 month
      $cmd .= " DEF:$item${i}_r_3m=\\\"$line\\\":io_rate:AVERAGE:start=-3m";
      $cmd .= " CDEF:$item${i}_w_3m=$item${i}_r_3m,0,*";

      # 1 year
      $cmd .= " DEF:$item${i}_r_1y=\\\"$line\\\":io_rate:AVERAGE:start=-1y";
      $cmd .= " CDEF:$item${i}_w_1y=$item${i}_r_1y,0,*";
    } ## end if ( $item =~ m/sum_io_total/)
    if ( $item =~ m/sum_data_total/ ) {

      # DS5K does not have read/write details
      # To make it easier use _r and _w here
      $cmd .= " DEF:$item${i}_r=\\\"$line\\\":data_rate:AVERAGE";
      $cmd .= " CDEF:$item${i}_w=$item${i}_r,0,*";

      # 1 month
      $cmd .= " DEF:$item${i}_r_1m=\\\"$line\\\":data_rate:AVERAGE:start=-1m";
      $cmd .= " CDEF:$item${i}_w_1m=$item${i}_r_1m,0,*";

      # 3 month
      $cmd .= " DEF:$item${i}_r_3m=\\\"$line\\\":data_rate:AVERAGE:start=-3m";
      $cmd .= " CDEF:$item${i}_w_3m=$item${i}_r_3m,0,*";

      # 1 year
      $cmd .= " DEF:$item${i}_r_1y=\\\"$line\\\":data_rate:AVERAGE:start=-1y";
      $cmd .= " CDEF:$item${i}_w_1y=$item${i}_r_1y,0,*";
    } ## end if ( $item =~ m/sum_data_total/)
    if ( $item =~ m/sum_data/ || $item =~ m/sum_io/ ) {
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_r,$val,/";    # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";    # convert into MB if necessary, normaly is there 1

      # 1 month
      $cmd .= " CDEF:$itemm${i}_r_1m=$item${i}_r_1m,$val,/";    # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w_1m=$item${i}_w_1m,$val,/";    # convert into MB if necessary, normaly is there 1

      # 3 month
      $cmd .= " CDEF:$itemm${i}_r_3m=$item${i}_r_3m,$val,/";    # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w_3m=$item${i}_w_3m,$val,/";    # convert into MB if necessary, normaly is there 1

      # 1 year
      $cmd .= " CDEF:$itemm${i}_r_1y=$item${i}_r_1y,$val,/";    # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w_1y=$item${i}_w_1y,$val,/";    # convert into MB if necessary, normaly is there 1
    } ## end if ( $item =~ m/sum_data/...)

    if ( $i == 0 ) {
      $cmd .= " CDEF:main_res_r${i}=$itemm${i}_r";
      $cmd .= " CDEF:main_res_w${i}=$itemm${i}_w";

      $cmd .= " CDEF:main_res_r_1m${i}=$itemm${i}_r_1m";
      $cmd .= " CDEF:main_res_w_1m${i}=$itemm${i}_w_1m";

      $cmd .= " CDEF:main_res_r_3m${i}=$itemm${i}_r_3m";
      $cmd .= " CDEF:main_res_w_3m${i}=$itemm${i}_w_3m";

      $cmd .= " CDEF:main_res_r_1y${i}=$itemm${i}_r_1y";
      $cmd .= " CDEF:main_res_w_1y${i}=$itemm${i}_w_1y";
    } ## end if ( $i == 0 )
    else {
      $cmd .= " CDEF:pom_r${i}=main_res_r${y},UN,0,main_res_r${y},IF,$itemm${i}_r,UN,0,$itemm${i}_r,IF,+";
      $cmd .= " CDEF:main_res_r${i}=main_res_r${y},UN,$itemm${i}_r,UN,UNKN,pom_r${i},IF,pom_r${i},IF";
      $cmd .= " CDEF:pom_w${i}=main_res_w${y},UN,0,main_res_w${y},IF,$itemm${i}_w,UN,0,$itemm${i}_w,IF,+";
      $cmd .= " CDEF:main_res_w${i}=main_res_w${y},UN,$itemm${i}_w,UN,UNKN,pom_w${i},IF,pom_w${i},IF";

      $cmd .= " CDEF:pom_r_1m${i}=main_res_r_1m${y},UN,0,main_res_r_1m${y},IF,$itemm${i}_r_1m,UN,0,$itemm${i}_r_1m,IF,+";
      $cmd .= " CDEF:main_res_r_1m${i}=main_res_r_1m${y},UN,$itemm${i}_r_1m,UN,UNKN,pom_r_1m${i},IF,pom_r_1m${i},IF";
      $cmd .= " CDEF:pom_w_1m${i}=main_res_w_1m${y},UN,0,main_res_w_1m${y},IF,$itemm${i}_w_1m,UN,0,$itemm${i}_w_1m,IF,+";
      $cmd .= " CDEF:main_res_w_1m${i}=main_res_w_1m${y},UN,$itemm${i}_w_1m,UN,UNKN,pom_w_1m${i},IF,pom_w_1m${i},IF";

      $cmd .= " CDEF:pom_r_3m${i}=main_res_r_3m${y},UN,0,main_res_r_3m${y},IF,$itemm${i}_r_3m,UN,0,$itemm${i}_r_3m,IF,+";
      $cmd .= " CDEF:main_res_r_3m${i}=main_res_r_3m${y},UN,$itemm${i}_r_3m,UN,UNKN,pom_r_3m${i},IF,pom_r_3m${i},IF";
      $cmd .= " CDEF:pom_w_3m${i}=main_res_w_3m${y},UN,0,main_res_w_3m${y},IF,$itemm${i}_w_3m,UN,0,$itemm${i}_w_3m,IF,+";
      $cmd .= " CDEF:main_res_w_3m${i}=main_res_w_3m${y},UN,$itemm${i}_w_3m,UN,UNKN,pom_w_3m${i},IF,pom_w_3m${i},IF";

      $cmd .= " CDEF:pom_r_1y${i}=main_res_r_1y${y},UN,0,main_res_r_1y${y},IF,$itemm${i}_r_1y,UN,0,$itemm${i}_r_1y,IF,+";
      $cmd .= " CDEF:main_res_r_1y${i}=main_res_r_1y${y},UN,$itemm${i}_r_1y,UN,UNKN,pom_r_1y${i},IF,pom_r_1y${i},IF";
      $cmd .= " CDEF:pom_w_1y${i}=main_res_w_1y${y},UN,0,main_res_w_1y${y},IF,$itemm${i}_w_1y,UN,0,$itemm${i}_w_1y,IF,+";
      $cmd .= " CDEF:main_res_w_1y${i}=main_res_w_1y${y},UN,$itemm${i}_w_1y,UN,UNKN,pom_w_1y${i},IF,pom_w_1y${i},IF";
    } ## end else [ if ( $i == 0 ) ]

    $i++;
    $y++;
  } ## end foreach my $line (<$wrkdir/$host/$type_real/*$suffix>)

  if ( $nothing_to_sum == 1 ) {

    # there is nothing to sum at the moment
    print "creating cmd   : $host:$type:$lpar:$type_gr:$type nothing to summ\n" if $DEBUG;
    return 1;
  } ## end if ( $nothing_to_sum ==...)

  ###
  $cmd .= " CDEF:total=main_res_r${y},main_res_w${y},+";
  $cmd .= " CDEF:total_1m=main_res_r_1m${y},main_res_w_1m${y},+";
  $cmd .= " CDEF:total_3m=main_res_r_3m${y},main_res_w_3m${y},+";
  $cmd .= " CDEF:total_1y=main_res_r_1y${y},main_res_w_1y${y},+";

  my $lpar_space = sprintf( "%-38s", "Total" );
  $cmd .= " $gtype:total#00FF00:\\\"$lpar_space\\\"";
  $cmd .= " GPRINT:total:AVERAGE:\\\"%9.0lf \\\"";
  $cmd .= " GPRINT:total:MAX:\\\"%9.0lf \\l\\\"";

  # 1 year trend
  $cmd .= " VDEF:Dd_1y=total_1y,LSLSLOPE";
  $cmd .= " VDEF:Hd_1y=total_1y,LSLINT";
  $cmd .= " CDEF:trend_1y=total_1y,POP,Dd_1y,COUNT,*,Hd_1y,+";
  $cmd .= " LINE2:trend_1y#FF8080:\\\"last 1 year trend\\\"";
  $cmd .= " COMMENT:\\\"\\l\\\"";

  # 3 month trend
  $cmd .= " VDEF:Dd_3m=total_3m,LSLSLOPE";
  $cmd .= " VDEF:Hd_3m=total_3m,LSLINT";
  $cmd .= " CDEF:trend_3m=total_3m,POP,Dd_3m,COUNT,*,Hd_3m,+";
  $cmd .= " LINE2:trend_3m#80FFFF:\\\"last 3 months trend\\\"";
  $cmd .= " COMMENT:\\\"\\l\\\"";

  # 1 month trend
  $cmd .= " VDEF:Dd_1m=total_1m,LSLSLOPE";
  $cmd .= " VDEF:Hd_1m=total_1m,LSLINT";
  $cmd .= " CDEF:trend_1m=total_1m,POP,Dd_1m,COUNT,*,Hd_1m,+";
  $cmd .= " LINE2:trend_1m#0088FF:\\\"last 1 month trend\\\"";
  $cmd .= " COMMENT:\\\"\\l\\\"";
  ###

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # It must be here!!!, touch() only in case there is any rrdfile (caused touch() always for summing not existing DRIVES in SVC)
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  if ( $not_execute_rrdtool == 1 ) { return 1; }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < $tmp_file 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  return 0;
} ## end sub draw_graph_sum_trend

sub draw_graph_sum {
  my $text       = shift;
  my $type_gr    = shift;
  my $xgrid      = shift;
  my $host       = shift;
  my $type       = shift;
  my $item       = shift;
  my $st_type    = shift;
  my $time_first = shift;
  my $name       = "$tmp_dir/$host-$type-$item-agg-$type_gr";
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $last       = "COMMENT: ";
  my $rrd_time   = "";
  my $step_new   = $STEP;
  my $once       = 0;
  my $last_time  = "na";

  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";
  my $act_utime = time();

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  my $type_text = $type;
  if ( $st_type =~ m/SWIZ/ && $type =~ m/RANK/ ) {
    $type_text = "MDISK";
  }
  if ( $st_type =~ m/HUS/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/NETAPP/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/VSPG/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/DOTHILL/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/DRIVE/ ) {
    $type_text = "DISK GROUP";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/NODE-CACHE/ ) {
    $type_text = "CACHE";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/NODE$/ ) {
    $type_text = "DIRECTOR";
  }
  if ( $st_type =~ m/PURE/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }

  print "creating cmd   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $header   = "$type_text aggregated $item: last $text";
  if ( $item =~ m/^sum/ || $item =~ m/resp_t/ ) {
    $header = "$type_text $item: last $text";
  }
  if ($item eq "sum_capacity_aggregated" && $st_type eq "PURE"){
    $header = "$type_text aggregated: last $text";
  }
  my $i    = 0;
  my $lpar = "";
  my $cmd  = "";
  my $j    = 0;

  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now-1$type_gr+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;

  # do not use switch statement
  if ( $item =~ m/^sum_data$/ )   { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^sum_io$/ )     { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^sum_data_b$/ ) { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^sum_io_b$/ )   { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^sum_io_total$/ )   { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^sum_data_total$/ ) { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^sum_capacity$/ )   { $value_short = "TB";     $value_long = "TBytes"; }
  if ( $item =~ m/^sum_capacity_aggregated$/ )   { $value_short = "TB";     $value_long = "TBytes"; }
  if ( $item =~ m/^sum_resp/ )        { $value_short = "ms";     $value_long = "miliseconds"; }
  if ( $item =~ m/^sum_busy/ )        { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^tier0$/ )          { $value_short = "TB";     $value_long = "TBytes"; }
  if ( $item =~ m/^tier0-nl$/ )       { $value_short = "TB";     $value_long = "TBytes"; }
  if ( $item =~ m/^tier1$/ )          { $value_short = "TB";     $value_long = "TBytes"; }
  if ( $item =~ m/^tier2$/ )          { $value_short = "TB";     $value_long = "TBytes"; }
  if ( $item =~ m/^sys$/ )            { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^busy$/ )           { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^compress$/ )       { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^write_pend$/ )     { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^clean_usage$/ )    { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^middle_usage$/ )   { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^phys_usage$/ )     { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^usage_rate$/ )     { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^dnlc_hit$/ )       { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^buffer_hit$/ )     { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^cache_hit$/ )      { $value_short = "%";      $value_long = "%"; }
  if ( $item =~ m/^WP$/ )             { $value_short = "%";      $value_long = "%"; }

  # Why??? There should be 1024
  if ( $st_type =~ m/^DS8K$/ && $type !~ /RANK/ && $type !~ m/PORT/ ) {
    if ( $item =~ m/^sum_data$/ || $item =~ m/^read$/ || $item =~ m/^write/ ) {
      $val = 1;
    }
  }

  my $item_label = uc($item);
  $item_label =~ s/SUM_//;

  if ( $st_type eq "VMAX" && $item eq "WP" ) {
    $cmd .= " --vertical-label=\\\"Cache $item_label in $value_long\\\"";
  }
  else {
    $cmd .= " --vertical-label=\\\"$item_label in $value_long\\\"";
  }

  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $legend = sprintf( "%-38s", "$item [$value_short]" );
  $cmd .= " COMMENT:\\\"$legend          Avrg         Max \\l\\\"";

  my $gtype = "LINE";

  #if ( $item =~ m/resp_t/ ) {
  #  $gtype="LINE1";
  #}

  my $lpar_list      = "";
  my $pool_found     = 0;
  my $pool_id_tmp    = "";
  my $lpar_list_tmp  = "";
  my $itemm          = "";
  my $nothing_to_sum = 1;

  my $type_real = $type;    # real directory for file search --> 3PAR use VOLUME for POOL sum_io and sum_data
  if ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ && $type =~ m/POOL/ ) {
    if ( $item =~ m/sum_io/ || $item =~ m/sum_data/ ) {
      $type_real = "VOLUME";
    }
  }
  if ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ && $type =~ m/POOL/ ) {
    if ( $item =~ m/sum_io/ || $item =~ m/sum_data/ ) {
      $type_real = "VOLUME";
    }
  }

  my $suffix = "rrd";
  if ( $st_type eq "VMAX" && $type eq "POOL" && $item !~ m/^sum_capacity/ && $item !~ m/_b$/ ) {
    $suffix = "rra";
  }
  my $host_space = check_space_name($host);
  foreach my $line (<$wrkdir/$host_space/$type_real/*$suffix>) {
    chomp($line);
    if ( $line !~ m/\.rrd$/ && $st_type ne "VMAX" ) {
      next;    # it does not end by .rrd suffix, "$ cannt be use in foreach <>
    }

    if ( $st_type !~ m/NETAPP/ && $st_type !~ m/HUS/ && $st_type !~ m/VNXF/ && $type =~ m/RANK/ && $line !~ m/\/.*-P[0-9][0-9]*\.rrd$/ && $st_type !~ m/VSPG/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/COMPELLENT/ && $st_type !~ m/OCEANSTOR/ && $st_type !~ m/PURE/) {

      #workaround for cleaning up ranks without pool id (could hapened in the past by the error)
      next;
    }
    if ( $type =~ m/POOL/ ) {
      if ( $st_type !~ m/HUS/ && $st_type !~ m/VSPG/ && $st_type !~ m/VNXF/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/COMPELLENT/ && $st_type !~ m/OCEANSTOR/ && $line !~ m/-cap\.rrd$/ && $st_type !~ m/PURE/ ) {

        #HUS have capacity in pool.rrd
        # SVC & NetApp at least
        if ( $item =~ m/^sum_capacity$/ || $item =~ m/^tier[0-9]/ ) {
          next;    # use only *-cap.rrd file for POOL capacity stuff (there can be even POOL front-end stuff like $pool_id.rrd
        }
      } ## end if ( $st_type !~ m/HUS/...)
    } ## end if ( $type =~ m/POOL/ )
    if ( $type =~ m/TIER/ && $line !~ m/-cap\.rrd$/ && $item =~ m/^sum_capacity$/ ) {
      next;
    }
    if ( $type =~ m/TIER/ && $line =~ m/-cap\.rrd$/ && $item !~ m/^sum_capacity$/ ) {
      next;
    }
    if ( $type =~ m/POOL/ && $line =~ m/-cap\.rrd$/ ) {
      if ( $item =~ m/^sum_io/ || $item =~ m/^sum_data/ || $item =~ m/^sum_resp/ ) {
        next;      # skip *-cap.rrd files for sum_io and sum_data
      }
    }

    if ( $st_type =~ m/SWIZ/ && $type =~ m/POOL/ && $line =~ m/-cap.rrd$/ && $item =~ m/^tier[0-9]/ ) {
      if ( $item =~ m/tier0-nl/ && $line !~ m/nl-cap\.rrd$/ ) {

        # new TIER0-NL implementation in separate file 0-nl-cap.rrd
        next;
      }
      if ( $item =~ m/tier[0-9]$/ && $line =~ m/nl-cap\.rrd$/ ) {
        next;
      }
    } ## end if ( $st_type =~ m/SWIZ/...)
    if ( $st_type =~ m/SWIZ/ && $type =~ m/POOL/ && $line =~ m/-comp\.rrd$/ ) {
      next;    # exclude data with compression -comp.rrd
    }
    if ( $st_type =~ m/SWIZ/ && $type =~ m/POOL/ && $item =~ m/^sum_capacity$/ && $line =~ m/-nl-cap\.rrd$/ ) {
      next;     # exclude NL capacity file
    }

    if ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ && $type =~ m/POOL/ ) {
      if ( $item =~ m/sum_io/ || $item =~ m/sum_data/ ) {
        my $line_base = basename($line);
        if ( $line_base !~ m/.*-P.*\.rrd$/ ) {
          next;    # it is not POOL data file
        }
      } ## end if ( $item =~ m/sum_io/...)
    } ## end if ( $ds5k_pool_org ==...)
    if ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ && $type =~ m/POOL/ ) {
      if ( $item =~ m/sum_io/ || $item =~ m/sum_data/ ) {
        my $line_base = basename($line);
        if ( $line_base !~ m/.*-P.*\.rrd$/ ) {
          next;    # it is not POOL data file
        }
      } ## end if ( $item =~ m/sum_io/...)
    } ## end if ( $xiv_pool_org == ...)

    # VMAX directors (front-end, backend)
    if ( $st_type eq "VMAX" && ( $type eq "NODE" || $type eq "PORT" ) && $item =~ m/^sum_busy/ ) {
      my $line_base = basename($line);
      if ( $item =~ m/_frontend$/ && $line_base !~ m/^F[A-Z]-/ ) { next; }
      if ( $item =~ m/_backend$/  && $line_base !~ m/^D[A-Z]-/ ) { next; }
      if ( $item =~ m/_rdf$/      && $line_base !~ m/^R[A-Z]-/ ) { next; }
    } ## end if ( $st_type eq "VMAX"...)

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }
    $nothing_to_sum = 0;

    my $rrd_file = "$line";
    if ( $type_gr =~ m/d/ && $once == 0 ) {
      $once++;

      # find out time stamp of last data update
      # take just one volume to keep it simple
      RRDp::cmd qq(last "$rrd_file");
      my $last_tt = RRDp::read;
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
    } ## end if ( $type_gr =~ m/d/ ...)

    my @link_l = split( /\//, $line );
    my $lpar = "";
    foreach my $m (@link_l) {
      $lpar = $m;
    }
    $lpar =~ s/\.rrd$//;
    $lpar =~ s/-P.*//;    # filter pool info
    $lpar =~ s/PORT-//;
    $lpar =~ s/RANK-//;
    $lpar =~ s/MDISK-//;
    $lpar =~ s/VOLUME-//;
    $lpar =~ s/0x//;
    $lpar =~ s/-cap//;

    # bulid RRDTool cmd
    $itemm = $item . "m";
    $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/\#/Y/g;    # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    # do not use below UNKN as this is total sum and one NaN --. whole sum is NaN
    if ( $item =~ m/clean_usage/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":clean_usage:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_r_u,0,*";
    }
    if ( $item =~ m/middle_usage/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":middle_usage:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_r,0,*";
    }
    if ( $item =~ m/phys_usage/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":phys_usage:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_r,0,*";
    }
    if ( $item =~ m/write_pend/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":write_pend:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_r,0,*";
    }
    if ( $item =~ m/operating_rate/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":operating_rate:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_r,0,*";
    }
    if ( $item =~ m/dnlc_hit/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":dnlc_hit:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_r,0,*";
    }
    if ( $item =~ m/buffer_hit/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":buffer_hit:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_r,0,*";
    }
    if ( $item =~ m/cache_hit/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":cache_hit:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_r,0,*";
    }
    if ( $item =~ m/sys/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":sys:AVERAGE";
      if ( $st_type =~ m/3PAR/ ) {
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":usr:AVERAGE";
      }
      else {
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":compress:AVERAGE";
      }
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,UNKN,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,UNKN,$item${i}_w_u,IF,IF";
    } ## end if ( $item =~ m/sys/ )
    if ( $item =~ m/busy/ || $item =~ m/sum_busy/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":busy:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " DEF:$itemm${i}_w=\\\"$line\\\":busy:AVERAGE";

      #$cmd .= " CDEF:$item${i}_w=$item${i}_r,0,*";
    } ## end if ( $item =~ m/busy/ ...)
    if ( $item =~ m/^WP$/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":WP:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " DEF:$itemm${i}_w=\\\"$line\\\":WP:AVERAGE";
    }
    if ( $item =~ m/sum_io$/ ) {
      if ( $st_type =~ m/DS8K/ && $type =~ m/PORT/ ) {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":io_rate:AVERAGE";
        $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$item${i}_w=$item${i}_r,0,*";
      }
      else {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":read_io:AVERAGE";
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":write_io:AVERAGE";
        $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      } ## end else [ if ( $st_type =~ m/DS8K/...)]
    } ## end if ( $item =~ m/sum_io$/)
    if ( $item =~ m/sum_io_b$/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":read_io_b:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":write_io_b:AVERAGE";
      $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
    } ## end if ( $item =~ m/sum_io_b$/)
    if ( $item =~ m/sum_io_total/ ) {

      # DS5K does not have read/write details
      # To make it easier use _r and _w here
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":io_rate:AVERAGE";
      $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=$item${i}_r,0,*";
    } ## end if ( $item =~ m/sum_io_total/)
    if ( $item =~ m/sum_resp_t$/ ) {
      if ( $st_type =~ m/DS5K$/ || $st_type =~ m/VNX/ || $st_type =~ m/DOTHILL/ || $st_type =~ m/UNITY/ || ( $st_type =~ m/DS8K/ && $type =~ m/PORT/ ) ) {

        # they have only resp_t nothing else (no read/write)
        # DOTHILL only for RG
        $cmd .= " DEF:$item${i}_r_resp_u=\\\"$line\\\":resp_t:AVERAGE";
        $cmd .= " DEF:$item${i}_r_io_u=\\\"$line\\\":io_rate:AVERAGE";
        $cmd .= " CDEF:$item${i}_r_resp=TIME,$time_first,LT,$item${i}_r_resp_u,$item${i}_r_resp_u,UN,0,$item${i}_r_resp_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r_io=TIME,$time_first,LT,$item${i}_r_io_u,$item${i}_r_io_u,UN,0,$item${i}_r_io_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r_resp=$item${i}_r_resp,$itemm${i}_r_io,*";
      } ## end if ( $st_type =~ m/DS5K$/...)
      else {
        $cmd .= " DEF:$item${i}_r_resp_u=\\\"$line\\\":resp_t_r:AVERAGE";
        $cmd .= " DEF:$item${i}_w_resp_u=\\\"$line\\\":resp_t_w:AVERAGE";
        $cmd .= " DEF:$item${i}_r_io_u=\\\"$line\\\":read_io:AVERAGE";
        $cmd .= " DEF:$item${i}_w_io_u=\\\"$line\\\":write_io:AVERAGE";
        $cmd .= " CDEF:$item${i}_r_resp=TIME,$time_first,LT,$item${i}_r_resp_u,$item${i}_r_resp_u,UN,0,$item${i}_r_resp_u,IF,IF";
        $cmd .= " CDEF:$item${i}_w_resp=TIME,$time_first,LT,$item${i}_w_resp_u,$item${i}_w_resp_u,UN,0,$item${i}_w_resp_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r_io=TIME,$time_first,LT,$item${i}_r_io_u,$item${i}_r_io_u,UN,0,$item${i}_r_io_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_w_io=TIME,$time_first,LT,$item${i}_w_io_u,$item${i}_w_io_u,UN,0,$item${i}_w_io_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_r_resp=$item${i}_r_resp,$itemm${i}_r_io,*";
        $cmd .= " CDEF:$itemm${i}_w_resp=$item${i}_w_resp,$itemm${i}_w_io,*";
      } ## end else [ if ( $st_type =~ m/DS5K$/...)]
    } ## end if ( $item =~ m/sum_resp_t$/)
    if ( $item =~ m/sum_resp_t_b$/ ) {
      $cmd .= " DEF:$item${i}_r_resp_u=\\\"$line\\\":resp_t_r_b:AVERAGE";
      $cmd .= " DEF:$item${i}_w_resp_u=\\\"$line\\\":resp_t_w_b:AVERAGE";
      $cmd .= " DEF:$item${i}_r_io_u=\\\"$line\\\":read_io_b:AVERAGE";
      $cmd .= " DEF:$item${i}_w_io_u=\\\"$line\\\":write_io_b:AVERAGE";
      $cmd .= " CDEF:$item${i}_r_resp=TIME,$time_first,LT,$item${i}_r_resp_u,$item${i}_r_resp_u,UN,0,$item${i}_r_resp_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w_resp=TIME,$time_first,LT,$item${i}_w_resp_u,$item${i}_w_resp_u,UN,0,$item${i}_w_resp_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_r_io=TIME,$time_first,LT,$item${i}_r_io_u,$item${i}_r_io_u,UN,0,$item${i}_r_io_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w_io=TIME,$time_first,LT,$item${i}_w_io_u,$item${i}_w_io_u,UN,0,$item${i}_w_io_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_r_resp=$item${i}_r_resp,$itemm${i}_r_io,*";
      $cmd .= " CDEF:$itemm${i}_w_resp=$item${i}_w_resp,$itemm${i}_w_io,*";
    } ## end if ( $item =~ m/sum_resp_t_b$/)
    if ( $item =~ m/sum_data$/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":read:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":write:AVERAGE";
      $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
    } ## end if ( $item =~ m/sum_data$/)
    if ( $item =~ m/sum_data_b$/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":read_b:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":write_b:AVERAGE";
      $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
    } ## end if ( $item =~ m/sum_data_b$/)
    if ( $item =~ m/sum_data_total/ ) {

      # DS5K does not have read/write details
      # To make it easier use _r and _w here
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":data_rate:AVERAGE";
      $cmd .= " CDEF:$item${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$item${i}_w=$item${i}_r,0,*";
    } ## end if ( $item =~ m/sum_data_total/)
    if ( $item =~ m/sum_data/ || $item =~ m/sum_io/ ) {
      $cmd .= " CDEF:$itemm${i}_r=$item${i}_r,$val,/";    # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:$itemm${i}_w=$item${i}_w,$val,/";    # convert into MB if necessary, normaly is there 1
    }
    if ( $item =~ m/^sum_capacity$/ ) {
      if ( $st_type eq "3PAR" || $st_type eq "VNX" || $st_type eq "UNITY" || $st_type eq "VNXE" ) {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":used:AVERAGE";
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":free:AVERAGE";
        $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      } ## end if ( $st_type eq "3PAR"...)
      elsif ( $st_type eq "VMAX" && $type ne "DRIVE" ) {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":used:AVERAGE";
        $cmd .= " DEF:$item${i}_t_u=\\\"$line\\\":total:AVERAGE";
        $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_t=TIME,$time_first,LT,$item${i}_t_u,$item${i}_t_u,UN,0,$item${i}_t_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_t,$itemm${i}_r,-";
      } ## end elsif ( $st_type eq "VMAX"...)
      elsif ( $st_type eq "VMAX" && $type eq "DRIVE" ) {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":cap_used:AVERAGE";
        $cmd .= " DEF:$item${i}_t_u=\\\"$line\\\":cap:AVERAGE";
        $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_t=TIME,$time_first,LT,$item${i}_t_u,$item${i}_t_u,UN,0,$item${i}_t_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_w=$itemm${i}_t,$itemm${i}_r,-";
      } ## end elsif ( $st_type eq "VMAX"...)
      elsif ( $st_type eq "SWIZ" && $type eq "POOL" && $line =~ m/-nl-cap\.rrd/ ) {

        # Storwize tier0 NL new tier since 2.00
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":total:AVERAGE";
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":free:AVERAGE";
        $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      } ## end elsif ( $st_type eq "SWIZ"...)
      else {
        $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":real:AVERAGE";
        $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":free:AVERAGE";
        $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
        $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      } ## end else [ if ( $st_type eq "3PAR"...)]
    } ## end if ( $item =~ m/sum_capacity/)
    if ( $item =~ m/tier0$/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":tier0cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier0free:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r_cap=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_r=$itemm${i}_r_cap,$itemm${i}_w,-";
    } ## end if ( $item =~ m/tier0$/)
    if ( $item =~ m/tier0-nl/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":total:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":free:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r_cap=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_r=$itemm${i}_r_cap,$itemm${i}_w,-";
    } ## end if ( $item =~ m/tier0-nl/)
    if ( $item =~ m/tier1/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":tier1cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier1free:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r_cap=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_r=$itemm${i}_r_cap,$itemm${i}_w,-";
    } ## end if ( $item =~ m/tier1/)
    if ( $item =~ m/tier2/ ) {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":tier2cap:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":tier2free:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r_cap=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_r=$itemm${i}_r_cap,$itemm${i}_w,-";
    } ## end if ( $item =~ m/tier2/)
    if ( $item =~ m/sum_capacity_aggregated/ && $st_type eq "PURE") {
      $cmd .= " DEF:$item${i}_r_u=\\\"$line\\\":shared_space:AVERAGE";
      $cmd .= " DEF:$item${i}_w_u=\\\"$line\\\":system:AVERAGE";
      $cmd .= " DEF:$item${i}_s_u=\\\"$line\\\":volumes:AVERAGE";
      $cmd .= " DEF:$item${i}_u_u=\\\"$line\\\":snapshots:AVERAGE";
      $cmd .= " CDEF:$itemm${i}_r=TIME,$time_first,LT,$item${i}_r_u,$item${i}_r_u,UN,0,$item${i}_r_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_w=TIME,$time_first,LT,$item${i}_w_u,$item${i}_w_u,UN,0,$item${i}_w_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_s=TIME,$time_first,LT,$item${i}_s_u,$item${i}_s_u,UN,0,$item${i}_s_u,IF,IF";
      $cmd .= " CDEF:$itemm${i}_t=TIME,$time_first,LT,$item${i}_u_u,$item${i}_u_u,UN,0,$item${i}_u_u,IF,IF";
    }
    $i++;
  } ## end foreach my $line (<$wrkdir/$host/$type_real/*$suffix>)

  if ( $nothing_to_sum == 1 ) {

    # there is nothing to sum at the moment
    print "creating cmd   : $host:$type:$lpar:$type_gr:$type nothing to summ\n" if $DEBUG;
    return 1;
  } ## end if ( $nothing_to_sum ==...)

  # Load all items into the $cmd
  # Read
  my $index_actual = 0;
  if ( $item =~ m/^sum_resp_t/ ) {
    $cmd .= " CDEF:rsum${i}_io=$itemm${index_actual}_r_io";
    $index_actual++;
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",$itemm${index_actual}_r_io,+";
    }
    $index_actual = 0;
    $cmd .= " CDEF:rsum${i}_resp=$itemm${index_actual}_r_resp";
    $index_actual++;
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",$itemm${index_actual}_r_resp,+";
    }
    $cmd .= " CDEF:rsum${i}=rsum${i}_resp,rsum${i}_io,/";
  } ## end if ( $item =~ m/^sum_resp_t/)
  else {
    $index_actual = 0;
    $cmd .= " CDEF:rsum${i}=$itemm${index_actual}_r";
    $index_actual++;
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",$itemm${index_actual}_r,+";
    }
  } ## end else [ if ( $item =~ m/^sum_resp_t/)]

  ### pure snapshots and shared_space
  if ($item eq "sum_capacity_aggregated" && $st_type eq "PURE"){
    ### snapshots
    my $index_actual = 0;
    $cmd .= " CDEF:ssum${i}=$itemm${index_actual}_s";
    $index_actual++;
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",$itemm${index_actual}_s,+";
    }
    my $lpar_space = sprintf( "%-38s", "Volumes" );
    $cmd .= " AREA:ssum${i}#33FF33:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:ssum${i}:AVERAGE:\\\"%7.2lf \\\"";
    $cmd .= " GPRINT:ssum${i}:MAX:\\\"%7.2lf \\l\\\"";
    ### shared_space
    $index_actual = 0;
    $cmd .= " CDEF:tsum${i}=$itemm${index_actual}_t";
    $index_actual++;
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",$itemm${index_actual}_t,+";
    }
    $lpar_space = sprintf( "%-38s", "Snapshots" );
    $cmd .= " STACK:tsum${i}#0000fb:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:tsum${i}:AVERAGE:\\\"%7.2lf \\\"";
    $cmd .= " GPRINT:tsum${i}:MAX:\\\"%7.2lf \\l\\\"";
  }

  if ( $item =~ m/^busy$/ || $item =~ m/^sys$/ || $item =~ m/^operating_rate$/ || $item =~ m/^write_pend$/ || $item =~ m/^clean_usage$/ || $item =~ m/^middle_usage$/ || $item =~ m/^phys_usage$/ || $item =~ m/^WP$/ || $item =~ m/^sum_busy/ ) {
    if ( $index_actual > 0 ) {

      # create an average (not total sum)
      $cmd .= ",$index_actual,/";
    }
  } ## end if ( $item =~ m/^busy$/...)

  # Write
  if ( $item =~ m/^sum_resp_t/ ) {
    if ( $st_type !~ m/VNX/ && $st_type !~ m/DS5K$/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/UNITY/ && ( $st_type !~ m/DS8K/ && $type !~ m/PORT/ ) ) {
      $index_actual = 0;
      $cmd .= " CDEF:wsum${i}_io=$itemm${index_actual}_w_io";
      $index_actual++;
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",$itemm${index_actual}_w_io,+";
      }
      $index_actual = 0;
      $cmd .= " CDEF:wsum${i}_resp=$itemm${index_actual}_w_resp";
      $index_actual++;
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",$itemm${index_actual}_w_resp,+";
      }
      $cmd .= " CDEF:wsum${i}=wsum${i}_resp,wsum${i}_io,/";
    } ## end if ( $st_type !~ m/VNX/...)
  } ## end if ( $item =~ m/^sum_resp_t/)
  else {
    $index_actual = 0;
    $cmd .= " CDEF:wsum${i}=$itemm${index_actual}_w";
    $index_actual++;
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",$itemm${index_actual}_w,+";
    }
  } ## end else [ if ( $item =~ m/^sum_resp_t/)]

  if ( $item =~ m/^sys$/ || $item =~ m/^operating_rate$/ || $item =~ m/^write_pend$/ || $item =~ m/^clean_usage$/ || $item =~ m/^middle_usage$/ || $item =~ m/^phys_usage$/ || $item =~ m/^buffer_hit$/ || $item =~ m/^dnlc_hit$/ || $item =~ m/^cache_hit$/ ) {
    if ( $index_actual > 0 ) {

      # create an average (not total sum)
      $cmd .= ",$index_actual,/";
    }
  } ## end if ( $item =~ m/^sys$/...)

  #
  # read || total
  #

  # print LINE read and write
  if ( $item =~ m/^sum_resp_t/ ) {
    if ( $st_type !~ m/VNX/ && $st_type !~ m/DS5K$/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/UNITY/ && ( $st_type !~ m/DS8K/ && $type !~ m/PORT/ ) ) {
      my $lpar_space = sprintf( "%-38s", "read" );
      $cmd .= " $gtype:rsum${i}#00FF00:\\\"$lpar_space\\\"";
      $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%9.2lf \\\"";
    }
    else {
      my $lpar_space = sprintf( "%-38s", "total" );
      $cmd .= " $gtype:rsum${i}#00FF00:\\\"$lpar_space\\\"";
      $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%9.2lf \\\"";
    }
  } ## end if ( $item =~ m/^sum_resp_t/)
  if ( $item =~ m/sum_io$/ || $item =~ m/sum_data$/ || $item =~ m/sum_io_b$/ || $item =~ m/sum_data_b$/ ) {
    my $lpar_space = sprintf( "%-38s", "read" );
    $cmd .= " $gtype:rsum${i}#00FF00:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%9.0lf \\\"";
  }
  if ( $item =~ m/sum_io_total/ || $item =~ m/sum_data_total/ ) {

    # DS5k only
    my $lpar_space = sprintf( "%-38s", "Total" );
    $cmd .= " $gtype:rsum${i}#00FF00:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%9.0lf \\\"";
  } ## end if ( $item =~ m/sum_io_total/...)
  if ( $item =~ m/sum_capacity$/ ) {

    #my $lpar_space = sprintf ("%-38s","read");
    #$cmd .= " $gtype:rsum${i}#00FF00:\\\"$lpar_space\\\"";
    my $lpar_space = sprintf( "%-38s", "Used" );
    $cmd .= " AREA:rsum${i}#FF3333:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  } ## end if ( $item =~ m/sum_capacity/)
  if ( $item =~ m/tier/ ) {
    my $lpar_space = sprintf( "%-38s", "Used" );
    $cmd .= " AREA:rsum${i}#FF3333:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  }
  if ( $item =~ m/sum_capacity_aggregated/ && $st_type eq "PURE" ) {
    my $lpar_space = sprintf( "%-38s", "Shared_space" );
    $cmd .= " STACK:rsum${i}#e1fd00:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  }
  if ( $item =~ m/sys/ ) {
    my $lpar_space = sprintf( "%-38s", "CPU sys" );
    $cmd .= " AREA:rsum${i}#FF3333:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  }
  if ( $item =~ m/operating_rate/ || $item =~ m/write_pend/ || $item =~ m/clean_usage/ || $item =~ m/middle_usage/ || $item =~ m/phys_usage/ || $item =~ m/dnlc_hit/ || $item =~ m/buffer_hit/ || $item =~ m/cache_hit/ ) {
    my $lpar_space = sprintf( "%-40s", "usage" );
    $cmd .= " LINE1:rsum${i}#FF3333:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  }
  if ( $item =~ m/^busy$/ || $item =~ m/^sum_busy/ ) {
    my $lpar_space = sprintf( "%-40s", "busy" );
    $cmd .= " LINE1:rsum${i}#FF3333:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  }
  if ( $item =~ m/^WP$/ ) {
    my $lpar_space = sprintf( "%-40s", "WP" );
    $cmd .= " LINE1:rsum${i}#FF3333:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:rsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  }

  if ( $item =~ m/sum_data/ || $item =~ m/sum_io/ ) {
    $cmd .= " GPRINT:rsum${i}:MAX:\\\"%9.0lf \\l\\\"";
  }
  if ( $item =~ m/resp_t/ ) {
    $cmd .= " GPRINT:rsum${i}:MAX:\\\"%9.2lf \\l\\\"";
  }
  if ( $item =~ m/sum_capacity$/ || $item =~ m/tier/ ) {
    $cmd .= " GPRINT:rsum${i}:MAX:\\\"%7.2lf \\l\\\"";
  }
  if ( $item =~ m/sum_capacity_aggregated/ && $st_type eq "PURE" ) {
    $cmd .= " GPRINT:rsum${i}:MAX:\\\"%7.2lf \\l\\\"";
  }
  if ( $item =~ m/sys/ ) {
    $cmd .= " GPRINT:rsum${i}:MAX:\\\"%7.2lf \\l\\\"";
  }
  if ( $item =~ m/^busy$/ || $item =~ m/^sum_busy/ || $item =~ m/operating_rate/ || $item =~ m/write_pend/ || $item =~ m/clean_usage/ || $item =~ m/middle_usage/ || $item =~ m/phys_usage/ || $item =~ m/dnlc_hit/ || $item =~ m/buffer_hit/ || $item =~ m/cache_hit/ || $item =~ m/^WP$/ ) {
    $cmd .= " GPRINT:rsum${i}:MAX:\\\"%7.2lf \\l\\\"";
  }

  #
  # write || free
  #

  # print LINE read and write
  # DS5k sum_io_total && sum_data_total is skipped here

  if ( $item =~ m/^sum_resp_t/ ) {
    if ( $st_type !~ m/VNX/ && $st_type !~ m/DS5K$/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/UNITY/ && ( $st_type !~ m/DS8K/ && $type !~ m/PORT/ ) ) {
      my $lpar_space = sprintf( "%-38s", "write" );
      $cmd .= " $gtype:wsum${i}#4444FF:\\\"$lpar_space\\\"";
      $cmd .= " GPRINT:wsum${i}:AVERAGE:\\\"%9.2lf \\\"";
    }
  } ## end if ( $item =~ m/^sum_resp_t/)
  if ( $item =~ m/sum_io$/ || $item =~ m/sum_data$/ || $item =~ m/sum_io_b$/ || $item =~ m/sum_data_b$/ ) {
    if ( $st_type =~ m/DS8K/ && $type =~ m/PORT/ && $item =~ m/resp_t/ ) {

      # DS8K does not have read_io and write_io, just io_rate
      ;
    }
    else {
      my $lpar_space = sprintf( "%-38s", "write" );
      $cmd .= " $gtype:wsum${i}#4444FF:\\\"$lpar_space\\\"";
      $cmd .= " GPRINT:wsum${i}:AVERAGE:\\\"%9.0lf \\\"";
    }
  } ## end if ( $item =~ m/sum_io$/...)
  if ( $item =~ m/sum_capacity$/ || $item =~ m/tier/ ) {
    my $lpar_space = sprintf( "%-38s", "Free" );
    $cmd .= " STACK:wsum${i}#33FF33:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:wsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  }
  if ($item eq "sum_capacity_aggregated" && $st_type eq "PURE"){
    my $lpar_space = sprintf( "%-38s", "System" );
    $cmd .= " STACK:wsum${i}#FF3333:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:wsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  }
  if ( $item =~ m/sys/ && $st_type !~ m/^HUS$/ && $st_type !~ m/^VSPG$/ && $st_type !~ m/^NETAPP$/ && $st_type !~ m/^VNXF$/ ) {
    my $lpar_space = sprintf( "%-38s", "Compress" );
    if ( $st_type =~ m/3PAR/ ) {
      $lpar_space = sprintf( "%-38s", "CPU user" );
    }
    $cmd .= " STACK:wsum${i}#3333FF:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:wsum${i}:AVERAGE:\\\"%7.2lf \\\"";
  } ## end if ( $item =~ m/sys/ &&...)

  # print legend
  if ( $item =~ m/sum_data/ || $item =~ m/sum_io/ ) {
    $cmd .= " GPRINT:wsum${i}:MAX:\\\"%9.0lf \\l\\\"";
  }
  if ( $item =~ m/resp_t/ ) {
    if ( $st_type !~ m/VNX/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/DS5K$/ && $st_type !~ m/UNITY/ && ( $st_type !~ m/DS8K/ && $type !~ m/PORT/ ) ) {
      $cmd .= " GPRINT:wsum${i}:MAX:\\\"%9.2lf \\l\\\"";
    }
  }
  if ( $item =~ m/sum_capacity$/ || $item =~ m/tier/ ) {
    $cmd .= " GPRINT:wsum${i}:MAX:\\\"%7.2lf \\l\\\"";
  }
  if ($item eq "sum_capacity_aggregated" && $st_type eq "PURE"){
    $cmd .= " GPRINT:wsum${i}:MAX:\\\"%7.2lf \\l\\\"";
  }
  if ( $item =~ m/sys/ && $st_type !~ m/^HUS$/ && $st_type !~ m/^VSPG$/ && $st_type !~ m/^NETAPP$/ && $st_type !~ m/^VNXF$/ ) {
    $cmd .= " GPRINT:wsum${i}:MAX:\\\"%7.2lf \\l\\\"";
  }

  #
  # capacity total
  #
  if ( $item =~ m/sum_capacity$/ ) {
    my $lpar_space = sprintf( "%-40s", "Total" );
    $cmd .= " COMMENT:\\\"$lpar_space\\\"";
    $cmd .= " CDEF:total=rsum${i},wsum${i},+";            # convert into MB if necessary, normaly is there 1
    $cmd .= " GPRINT:total:AVERAGE:\\\"%7.2lf \\l\\\"";
  } ## end if ( $item =~ m/sum_capacity/)

  if ( $type_gr =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # It must be here!!!, touch() only in case there is any rrdfile (caused touch() always for summing not existing DRIVES in SVC)
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  if ( $not_execute_rrdtool == 1 ) { return 1; }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < $tmp_file 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  return 0;
} ## end sub draw_graph_sum

sub draw_all_volume {
  my $host    = shift;
  my $type    = shift;
  my $item    = shift;
  my $st_type = shift;

  draw_graph_volume( "day",     "d", "MINUTE:60:HOUR:2:HOUR:4:0:%H", $host, $type, $item, $st_type );
  draw_graph_volume( "week",    "w", "HOUR:8:DAY:1:DAY:1:86400:%a",  $host, $type, $item, $st_type );
  draw_graph_volume( "4 weeks", "m", "DAY:1:DAY:2:DAY:2:0:%d",       $host, $type, $item, $st_type );
  draw_graph_volume( "year",    "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, $item, $st_type );
  return 0;
} ## end sub draw_all_volume

sub draw_graph_volume {
  my $text        = shift;
  my $type_gr     = shift;
  my $xgrid       = shift;
  my $host        = shift;
  my $type        = shift;
  my $item        = shift;
  my $st_type     = shift;
  my $name        = "$tmp_dir/$host/$type-$item-agg-$type_gr";
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $last        = "COMMENT: ";
  my $rrd_time    = "";
  my @low_volumes = "";
  my $indx_low    = 0;
  my $prev        = -1;
  my $step_new    = $STEP;
  my $once        = 0;
  my $last_time   = "na";

  my $par_vol = $ENV{HP3PAR_VV_SAMPLE_RATE};
  if ( $st_type =~ m/3PAR/ && defined($par_vol) && !$par_vol eq '' && isdigit($par_vol) && $par_vol < 300 ) {

    # custom 3PAR sample rate for Volumes could be used when HP3PAR_VV_SAMPLE_RATE is set in environment, supported are 30 or 60 secs rates only (instead of default 300secs)
    $step_new = $par_vol;
  }

  my $act_utime = time();
  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  print "creating cmd   : $host:$type:$item:$type_gr\n" if $DEBUG;

  # open a file with stored colours
  my @color_save = "";
  my $type_low   = lc $type;
  my $file_col   = "$wrkdir/$host/$type/" . $type_low . "s.col";
  if ( $st_type eq "3PAR" && $type eq "LUN" ) {
    $type_low = "volume";
    $file_col = "$wrkdir/$host/VOLUME/" . $type_low . "s.col";
  }
  if ( -f $file_col ) {
    open( FHC, "< $file_col" ) || error( "file does not exists : $file_col " . __FILE__ . ":" . __LINE__ ) && return 0;
    @color_save = <FHC>;
    close(FHC);
  }
  my $color_indx        = 0;    # reset colour index
  my $color_file_change = 0;

  my $req_time = "";
  my $header   = "$type aggregated $item: last $text";
  if ( $item =~ m/resp_t/ ) {
    $header = "$type $item: last $text";
  }

  if ( $st_type eq "VMAX" && $type eq "VOLUME" ) {
    $header = "STORAGE GROUP $item: last $text";
  }

  my $i    = 0;
  my $lpar = "";
  my $cmd  = "";
  my $j    = 0;

  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now-1$type_gr+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;
  my $suffix      = "rrd";

  # do not use switch statement
  if ( $item =~ m/^data_rate$/ )   { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^data_rate_b$/ ) { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^read$/ )        { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^write$/ )       { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^io_rate$/ )     { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^io_rate_b$/ )   { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^read_io$/ )     { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^write_io$/ )    { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^resp_t/ )       { $value_short = "ms";     $value_long = "milliseconds"; }
  if ( $item =~ m/^cache_hit$/ )   { $value_short = "%";      $value_long = "percent"; }
  if ( $item =~ m/^ssd_r_cache_hit$/ ) { $value_short = "%"; $value_long = "percent"; }
  if ( $item =~ m/^r_cache_hit$/ )     { $value_short = "%"; $value_long = "percent"; }
  if ( $item =~ m/^w_cache_hit$/ )     { $value_short = "%"; $value_long = "percent"; }
  if ( $item =~ m/^r_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^w_cache_hit$/ && $st_type eq "DOTHILL" ) { $value_short = "IOPS"; $value_long = "IO per second"; }
  if ( $item =~ m/^read_pct$/ )      { $value_short = "%";      $value_long = "percent"; }
  if ( $item =~ m/^r_cache_usage$/ ) { $value_short = "MB";     $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^w_cache_usage$/ ) { $value_short = "MB";     $value_long = "MBytes"; $val = 1024; }
  if ( $item =~ m/^total_usage$/ )   { $value_short = "GB";     $value_long = "GBytes"; $val = 1024 * 1024; }
  if ( $item =~ m/^read_b$/ )        { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^write_b$/ )       { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^read_io_b$/ )     { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^write_io_b$/ )    { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^io_pending$/ )    { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^r_transfer_size$/ )    { $value_short = "kB/sec";   $value_long = "kBytes/sec"; $val = 1; }
  if ( $item =~ m/^w_transfer_size$/ )    { $value_short = "kB/sec";   $value_long = "kBytes/sec"; $val = 1; }


  if ( $st_type =~ m/^DS8K$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^resp_t_r$/
      || $item =~ m/^resp_t_w$/
      || $item =~ m/^r_cache_usage$/
      || $item =~ m/^r_cache_hit$/
      || $item =~ m/^w_cache_hit$/
      || $item =~ m/^w_cache_usage$/
      || $item =~ m/^read_io$/
      || $item =~ m/^write_io$/
      || $item =~ m/^read_io_b$/
      || $item =~ m/^write_io_b$/
      || $item =~ m/^read_b$/
      || $item =~ m/^write_b$/
      || $item =~ m/^resp_t_r_b$/
      || $item =~ m/^resp_t_w_b$/ )
    {
      # those are stored in .rrc files
      $suffix = "rrc";
    } ## end if ( $item =~ m/^resp_t_r$/...)
  } ## end if ( $st_type =~ m/^DS8K$/...)
  if ( $st_type =~ m/^SWIZ$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^r_cache_usage$/
      || $item =~ m/^r_cache_hit$/
      || $item =~ m/^w_cache_hit$/
      || $item =~ m/^w_cache_usage$/
      || $item =~ m/^total_usage$/
      || $item =~ m/^r_transfer_size$/
      || $item =~ m/^w_transfer_size$/ )
    {
      # those are stored in .rrc files
      $suffix = "rrc";
    } ## end if ( $item =~ m/^r_cache_usage$/...)
  } ## end if ( $st_type =~ m/^SWIZ$/...)

  if ( $st_type =~ m/^DS8K$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^read$/ || $item =~ m/^write$/ ) {

      # data is stored in wrong RRDTool type (GAUGE instead of ABSOLUTE)
      # this do data conversion
      $val = $step_new;    # do not use $val=1024
    } ## end if ( $item =~ m/^read$/...)
  } ## end if ( $st_type =~ m/^DS8K$/...)

  #print "001 $st_type $type $item $suffix\n";

  $cmd .= " --vertical-label=\\\"$item in $value_long\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $item_short = $item;
  $item_short =~ s/r_cache_//;
  $item_short =~ s/w_cache_//;
  $item_short =~ s/r_transfer_//;
  $item_short =~ s/w_transfer_//;
  my $legend = sprintf( "%-38s", "$item_short [$value_short]" );
  my $legend_heading = "$item_short $delimiter [$value_short] $delimiter Avrg $delimiter Max ";
  $legend_heading =~ s/%/%%/g;
  $cmd .= " COMMENT:\\\"$legend       Avrg      Max                                                Avrg      Max\\l\\\"";

  my $gtype = "AREA";
  if ( $item =~ m/resp_t/ || $item =~ m/cache_hit/ || $item =~ m/read_pct/ || $item =~ m/r_transfer_size/ || $item =~ m/w_transfer_size/ || ( $st_type eq "VMAX" && $type eq "VOLUME" ) ) {
    $gtype = "LINE1";
  }

  my $lpar_list     = "";
  my $lpar_list_tmp = "";

  # read volume cfg files with grouped volumes
  my $file_cfg = "$wrkdir/$host/$type/" . $type_low . "s.cfg";
  if ( $st_type =~ m/^3PAR$/ && $type =~ /VVOL/ ) {
    $type_low = "volume";
    $file_cfg = "$wrkdir/$host/$type/" . $type_low . "s.cfg";
  }
  if ( !-f $file_cfg ) {
    error( "$host: Volumes cfg file has been found (ignore after fresh install): $file_cfg " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  open( FHR, "< $file_cfg" ) || error( "$host: file does not exists : $file_cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files = <FHR>;
  close(FHR);

  my $vols                        = "";
  my $itemm_sum                   = "";
  my $lpar_space                  = "";
  my $last_vol                    = -1;
  my $item_name_for_print_comment = "";
  my $cmd_print                   = "";    # print commands for clickable legend
  my $once_report                 = 0;

  foreach my $volg (@files) {
    chomp($volg);
    $vols = $volg;
    $volg =~ s/ : .*//;
    $vols =~ s/^.* : //;

    my @vol_array = split( ';', $vols );

    $prev = -1;
    my $itemm = $item . "m" . $volg;
    $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/\#/Y/g;    # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)
    $itemm =~ s/\(/T/g;
    $itemm =~ s/\)/P/g;
    $itemm =~ s/\//-/g;
    $itemm_sum = $itemm . "sum";

    my $one_volume_nick = 2;    # volume nick has 1 physical volume == 1, or more < 1;
    foreach my $lpar (@vol_array) {
      $lpar =~ s/ //;           # it must be there!!!
      if ( $lpar eq '' ) {
        next;
      }
      $one_volume_nick--;
    } ## end foreach my $lpar (@vol_array)

    # avoid volumes not being loaded during the given interval
    # multivolumes (more volumes under one nick
    # print "002 $volg : $one_volume_nick : $type,$type_gr,$item,$val,$type_gr\n";
    if ( $one_volume_nick < 1 ) {
      my $low_util = find_max_load_multi( $st_type, $step_new, $req_time, $volg, $basedir, $wrkdir, $host, $type, $type_gr, $item, $val, $type_gr, \@vol_array );

      # print "003 $volg : $low_util \n";
      if ( $low_util == 1 ) {
        foreach my $lpar (@vol_array) {
          $lpar =~ s/ //;    # it must be there!!!
          $lpar =~ s/^0x//;
          if ( $lpar eq '' ) {
            next;
          }
          my $file = "$wrkdir/$host/$type/$lpar.$suffix";
          if ( $st_type =~ m/^3PAR$/ && $type =~ /LUN/ ) {
            $file = "$wrkdir/$host/VOLUME/$lpar.$suffix";
          }
          $low_volumes[$indx_low] = $file;    # list of all not graphed volumes
          $indx_low++;                        # summ total of all low volumes must be displayed as the last line
        } ## end foreach my $lpar (@vol_array)
        next;                                 # avoid low loaded volume # print it i total at the end
      } ## end if ( $low_util == 1 )
    } ## end if ( $one_volume_nick ...)

    #
    # follow each volume (there migh be more volumes under 1 volume nick!!!
    # first run just select files ... must be for mutilevel resp_t a cache_hit to count how many items are summed
    my @vol_array_work      = "";
    my $vol_array_work_indx = 0;
    foreach my $lpar (@vol_array) {

      $lpar =~ s/^0x//;
      $lpar =~ s/ //;    # it must be there!!!
      if ( $lpar eq '' ) {
        next;
      }
      my $file = "$wrkdir/$host/$type/$lpar.$suffix";
      if ( $st_type =~ m/^3PAR$/ && $type =~ /LUN/ ) {
        $file = "$wrkdir/$host/VOLUME/$lpar.$suffix";
      }

      if ( $st_type =~ m/XIV/ || $st_type =~ m/DS5K/ || ( $st_type =~ m/3PAR/ && $type eq "VVOL" ) ) {

        # XIV & DS5K volumes contain pool_id in their names: 00273763-P102932.rrd
        # there might be more files if volumes are moved between pool, select the most fresh one
        my $vol_update_last = 0;
        foreach my $file_xiv (<$wrkdir/$host/$type/$lpar-P*\.$suffix>) {
          my $vol_upd_time = ( stat("$file_xiv") )[9];
          if ( $vol_upd_time > $vol_update_last ) {
            $file            = $file_xiv;
            $vol_update_last = $vol_upd_time;
          }
        } ## end foreach my $file_xiv (<$wrkdir/$host/$type/$lpar-P*\.$suffix>)
        if ( $vol_update_last == 0 ) {

          #volume has not been found, skipping it
          if ( $once_report == 0 ) {

            # It might appear after the upgrade to 1.00 as *rrc files are not in place yet
            error( "volumes stats: $file does not exist, continuing " . __FILE__ . ":" . __LINE__ );
            $once_report++;
          } ## end if ( $once_report == 0)
          next;
        } ## end if ( $vol_update_last ...)
      } ## end if ( $st_type =~ m/XIV/...)

      if ( $lpar eq '' ) {
        next;
      }

      if ( !-f $file ) {

        # do not report it at all
        #if ( $once_report == 0 ) {
        #  # It might appear after the upgrade to 1.00 as *rrc files are not in place yet
        #  error("volumes stats: $file does not exist, continuing ".__FILE__.":".__LINE__);
        #  $once_report++;
        #}
        next;
      } ## end if ( !-f $file )

      # go every each volume for particular group

      # avoid old lpars which do not exist in the period
      my $rrd_upd_time = ( stat("$file") )[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }

      # avoid volumes not being loaded enough during the given interval
      if ( $one_volume_nick == 1 ) {

        # avoid that for caching
        if ( $item !~ m/cache_hit/ && $item !~ m/read_pct/ && $item !~ m/r_transfer_size/ && $item !~ m/w_transfer_size/ ) {

          # only for 1 phys volume nicks
          my $low_util = find_max_load( $st_type, $type, $step_new, $file, $item, $val, $type_gr );

          #print "find max load je $low_util $type $file $item $val\n";
          if ( $low_util == 1 ) {
            $low_volumes[$indx_low] = $file;
            $indx_low++;    # summ total of all low volumes must be displayed as the last line
            next;
          }
        } ## end if ( $item !~ m/cache_hit/...)
      } ## end if ( $one_volume_nick ...)
      $vol_array_work[$vol_array_work_indx] = $file;

      #print "001  $vol_array_work[$vol_array_work_indx] : $file : $vol_array_work_indx\n";
      $vol_array_work_indx++;
    } ## end foreach my $lpar (@vol_array)

    # lets make $cmd for multilevels ones
    my $file_for_print = "";
    foreach my $file (@vol_array_work) {
      if ( $file eq '' ) {
        next;
      }

      if ( $type_gr =~ m/d/ && $once == 0 ) {

        # find out time stamp of last data update
        # take just one volume to keep it simple
        $once++;
        RRDp::cmd qq(last "$file");
        my $last_tt = RRDp::read;
        $last_time = localtime($$last_tt);
        $last_time =~ s/:/\\:/g;
      } ## end if ( $type_gr =~ m/d/ ...)

      $file =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing
      $file_for_print = $file;

      # bulid RRDTool cmd
      $itemm_sum =~ s/:/R/g;    # in case there is ":" in the name

      if ($st_type eq "SWIZ" && $type eq "VOLUME" && $item eq "r_transfer_size"){
        $cmd .= " DEF:$item${i}=\\\"$file\\\":res1:AVERAGE";
      }
      else{
        if ($st_type eq "SWIZ" && $type eq "VOLUME" && $item eq "w_transfer_size"){
          $cmd .= " DEF:$item${i}=\\\"$file\\\":res2:AVERAGE";
        }
        else{
          $cmd .= " DEF:$item${i}=\\\"$file\\\":$item:AVERAGE";
        }
      }
      if ( $vol_array_work_indx > 1 ) {
        my $val_tot = $val;
        if ( $item =~ m/resp_t/ || $item =~ m/cache_hit/ || $item =~ m/read_pct/ || $item =~ m/r_transfer_size/ || $item =~ m/w_transfer_size/) {

          # resp_t and cache_hit must be averaged not summed for multivolumes ...
          $val_tot = $val * $vol_array_work_indx;
        }
        $cmd .= " CDEF:$itemm${i}=$item${i},$val_tot,/";    # convert into MB if necessary, normaly is there 1
      } ## end if ( $vol_array_work_indx...)
      else {
        $cmd .= " CDEF:$itemm${i}=$item${i},$val,/";        # convert into MB if necessary, normaly is there 1
      }

      if ( $prev == -1 ) {
        $cmd .= " CDEF:$itemm_sum${i}=$itemm${i}";
      }
      else {
        $cmd .= " CDEF:$itemm_sum${i}=$itemm_sum${last_vol},$itemm${i},+";
      }
      $i++;
      $prev++;
      $last_vol++;
    } ## end foreach my $file (@vol_array_work)

    if ( $prev == -1 ) {
      next;    # have not found any volume
    }

    # add spaces to lpar name to have 18 chars total (for formating graph legend)
    $lpar_space = $volg;
    $lpar_space = sprintf( "%-38s", "$lpar_space" );

    # get color index
    my $ret_col = get_color( $lpar_space, $color_max, \@color_save );
    ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
    if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

      # new name has been identified, save it into an array and later on save into *col file
      $color_file_change = 1;
      $color_save[$color_indx] = $new_col_rec;
      ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
    } ## end if ( defined($new_col_rec...))

    $lpar_space =~ s/:/\\:/g;
    $cmd .= " $gtype:$itemm_sum${last_vol}$color[$color_indx]:\\\"$lpar_space\\\"";
    $item_name_for_print_comment = "$itemm_sum${last_vol}";

    if ( $item !~ m/resp_t/ && $item !~ m/cache_hit/ && $item !~ m/read_pct/ && $item !~ m/r_transfer_size/ && $item !~ m/w_transfer_size/ ) {
      $gtype = "STACK";
    }
    if ( $st_type =~ m/VMAX/ && $type =~ m/VOLUME/ ) {
      $gtype = "LINE1";
    }


    # put carriage return after each second lpar in the legend
    if ( $j == 1 ) {
      if ( $item =~ m/io_rate/ || $item =~ m/read_io/ || $item =~ m/write_io/ ) {
        $cmd .= " GPRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.0lf \\\"";
        $cmd .= " GPRINT:$itemm_sum${last_vol}:MAX:\\\"%6.0lf \\l\\\"";

        $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.0lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
        $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %6.0lf $delimiter $file_for_print $delimiter $st_type\\\"";
      } ## end if ( $item =~ m/io_rate/...)
      else {
        if ( $item !~ m/cache_hit/ && $item !~ m/read_pct/ ) {
          $cmd .= " GPRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf \\\"";
          $cmd .= " GPRINT:$itemm_sum${last_vol}:MAX:\\\"%6.1lf \\l\\\"";

          $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
          $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %6.1lf $delimiter $file_for_print $delimiter $st_type\\\"";
        } ## end if ( $item !~ m/cache_hit/...)
        else {
          # cache_hit
          my $lpar_print = $volg;
          $lpar_print =~ s/:/\\:/g;
          $lpar_print =~ s/ /=====space=====/g;
          $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"$lpar_print %6.1lf \\\"";
          $cmd .= " GPRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf \\\"";
          $cmd .= " GPRINT:$itemm_sum${last_vol}:MAX:\\\"%6.1lf \\l\\\"";

          $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf $delimiter $lpar_print $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
          $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %6.1lf $delimiter $file_for_print $delimiter $st_type\\\"";
        } ## end else [ if ( $item !~ m/cache_hit/...)]
      } ## end else [ if ( $item =~ m/io_rate/...)]
      $j = 0;
    } ## end if ( $j == 1 )
    else {
      if ( $item =~ m/io_rate/ || $item =~ m/read_io/ || $item =~ m/write_io/ ) {
        $cmd .= " GPRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.0lf \\\"";
        $cmd .= " GPRINT:$itemm_sum${last_vol}:MAX:\\\"%6.0lf \\\"";

        $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.0lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
        $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %6.0lf $delimiter $file_for_print $delimiter $st_type\\\"";
      } ## end if ( $item =~ m/io_rate/...)
      else {
        if ( $item !~ m/cache_hit/ && $item !~ m/read_pct/ ) {
          $cmd .= " GPRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf \\\"";
          $cmd .= " GPRINT:$itemm_sum${last_vol}:MAX:\\\"%6.1lf \\\"";

          $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
          $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %6.1lf $delimiter $file_for_print $delimiter $st_type\\\"";
        } ## end if ( $item !~ m/cache_hit/...)
        else {
          # cache_hit
          my $lpar_print = $volg;
          $lpar_print =~ s/:/\\:/g;
          $lpar_print =~ s/ /=====space=====/g;
          $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"$lpar_print %6.1lf \\\"";
          $cmd .= " GPRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf \\\"";
          $cmd .= " GPRINT:$itemm_sum${last_vol}:MAX:\\\"%6.1lf \\l\\\"";

          $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf $delimiter $lpar_print $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
          $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %6.1lf $delimiter $file_for_print $delimiter $st_type\\\"";
        } ## end else [ if ( $item !~ m/cache_hit/...)]
      } ## end else [ if ( $item =~ m/io_rate/...)]
      $j++;
    } ## end else [ if ( $j == 1 ) ]
  } ## end foreach my $volg (@files)
  if ( $j == 1 ) {
    $cmd .= " COMMENT:\\\"\\l\\\"";
  }


  # get color index
  my $ret_col = get_color( "rest_of_vols", $color_max, \@color_save );
  ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
  if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

    # new name has been identified, save it into an array and later on save into *col file
    $color_file_change = 1;
    $color_save[$color_indx] = $new_col_rec;
    ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
  } ## end if ( defined($new_col_rec...))

  if ( $item !~ m/cache_hit/ && $item !~ m/resp_t/ && $item !~ m/read_pct/) {

    # cache_hit do not need it
    # response time also does not make sense show low volumes
    # place there total for rest of not displayed volumes (low loaded)
    $cmd .= sum_low_volume( $val, $gtype, $i, $j, $last_vol, $color_indx, $item, $type, \@low_volumes );
  } ## end if ( $item !~ m/cache_hit/...)
  if ( $item =~ m/resp_t/ ) {
    $cmd .= " COMMENT:\\\"Volumes under $RESPONSE_MAX msec in a peak are not graphed\\l\\\"";
    $cmd .= " COMMENT:\\\"etc/stor2rrd.cfg\\:VOLUME_RESPONSE_MAX\\\"";

    my $item_text = "Volumes";
    if ( $type =~ m/LUN/ ) {
      $item_text = "Luns";
    }

    my $cmd_txp1 = "$item_text under $RESPONSE_MAX msec in a peak are not graphed";
    my $cmd_txp2 = "etc/stor2rrd.cfg\\:VOLUME_RESPONSE_MAX";
    $cmd .= " PRINT:$item_name_for_print_comment:MAX:\\\" %8.1lf $delim_com $cmd_txp1\\\"";
    $cmd .= " PRINT:$item_name_for_print_comment:MAX:\\\" %8.1lf $delim_com $cmd_txp2\\\"";

  } ## end if ( $item =~ m/resp_t/)

  if ( $type_gr =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # write colors into a file
  write_colors( $color_file_change, $file_col, \@color_save );

  # It must be here!!!, touch() only in case there is any rrdfile
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  # this has to be executed as working with the ouptut $ret
  if ( $not_execute_rrdtool == 1 ) {
    if ( $item !~ m/cache_hit/ && $item !~ m/read_pct/ ) {
      return 0;
    }
  }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way

  # Must be in eval for volumes, high amount of volumes can cause memory alloc issue on the OS and this does not exit program
  my $ret = "";
  eval {
    $ret = `$rrdtool - < $tmp_file 2>&1`;

    #$ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  };
  if ($@) {
    error( "$rrdtool - < $tmp_file: $@ " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  if ( $item =~ m/cache_hit/ || $item =~ m/read_pct/ ) {

    # store output for cache_hit text table
    $tmp_file = "$tmp_dir/$host/$type-$item-$type_gr.out";
    open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    print FH "$ret\n";
    close(FH);
  } ## end if ( $item =~ m/cache_hit/...)

  return 0;
} ## end sub draw_graph_volume

# it return 1 if there is avg load higher than $limit othervise 0
# it is for filtering data especially unused volumes
# this one is for multi volume nicks
sub find_max_load_multi {
  my ( $st_type, $STEP, $req_time, $volg, $basedir, $wrkdir, $host, $type, $type_gr, $item, $val, $time_range, $vol_array_tmp ) = @_;
  my @vol_array = @{$vol_array_tmp};
  my $rrd       = "";
  my $limit     = 0;
  my $cmd       = "";
  my $itemm     = $item . "m" . $volg;
  $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
  $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
  $itemm =~ s/\#/Y/g;    # hash in rrdtool variable names causing problems for rrdtool parser
  $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
  $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)
  my $itemm_sum = $itemm . "sum";
  my $last_vol  = -1;

  if ( $item =~ m/io_rate/ || $item =~ m/read_io/ || $item =~ m/write_io/ ) {
    $limit = $IO_MAX;
  }

  if ( $item =~ m/data_rate/ || $item =~ m/^read$/ || $item =~ m/^write$/ || $item =~ m/^read_b$/ || $item =~ m/^write_b$/ || $item =~ m/^r_transfer_size$/ || $item =~ m/^w_transfer_size$/ ) {
    $limit = $DATA_MAX;
  }

  if ( $item =~ m/resp/ ) {
    $limit = $RESPONSE_MAX;
  }

  if ( $item =~ m/cache_usage/ ) {
    $limit = $CACHE_MAX;
  }

  if ( $item =~ m/cache_hit/ ) {
    return 0;    # do not use it for cache_hit
  }

  $cmd .= "graph \\\"$basedir/tmp/name.png\\\"";
  $cmd .= " --start now-1$time_range";
  $cmd .= " --end now";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$STEP";

  my $prev = -1;
  my $i    = 0;

  $val = 1;
  my $valmb  = 1;
  my $suffix = ".rrd";
  if ( $st_type =~ m/^DS8K$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^read$/ || $item =~ m/^write$/ ) {

      # data is stored in wrong RRDTool type (GAUGE instead of ABSOLUTE)
      # this do data conversion
      $val   = $STEP;    # it must be multiplied by STEP as it is cumulated since every 5 minutes
      $valmb = 1024;
    } ## end if ( $item =~ m/^read$/...)
    if ( $item =~ m/^resp_t_r$/
      || $item =~ m/^resp_t_w$/
      || $item =~ m/^read_io$/
      || $item =~ m/^write_io$/
      || $item =~ m/^read_b$/
      || $item =~ m/^write_b$/
      || $item =~ m/^read_io_b$/
      || $item =~ m/^write_io_b$/
      || $item =~ m/^resp_t_r_b$/
      || $item =~ m/^resp_t_w_b$/
      || $item =~ m/^w_cache_hit$/
      || $item =~ m/^r_cache_hit$/ )
    {
      $suffix = ".rrc";
    } ## end if ( $item =~ m/^resp_t_r$/...)
  } ## end if ( $st_type =~ m/^DS8K$/...)
  if ( $st_type =~ m/^SWIZ$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^r_cache_hit$/ || $item =~ m/^w_cache_hit$/ || $item =~ m/^r_cache_usage$/ || $item =~ m/^w_cache_usage$/ | $item =~ m/^total_usage$/ || $item =~ m/^r_transfer_size$/ || $item =~ m/^w_transfer_size$/ ) {
      $suffix = ".rrc";
    }
  }

  # follow each volume (there migh be more volumes under 1 volume nick!!!
  foreach my $lpar (@vol_array) {
    $lpar =~ s/^0x//;
    $lpar =~ s/ //;
    if ( $lpar eq '' ) {
      next;
    }
    my $file = "$wrkdir/$host/$type/$lpar$suffix";

    # avoid old lpars which do not exist in the period
    if ( !-f $file ) {
      next;
    }
    my $rrd_upd_time = ( stat("$file") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    ### convert r_transfer_size to res1
    if ($st_type eq "SWIZ" && $type eq "VOLUME" && $item eq "r_transfer_size"){
      $item  = "res1";
    }
    if ($st_type eq "SWIZ" && $type eq "VOLUME" && $item eq "w_transfer_size"){
      $item  = "res2";
    }

    # bulid RRDTool cmd
    $i++;
    $last_vol++;
    $file =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    $cmd .= " DEF:$item${i}_tmp_nan=\\\"$file\\\":$item:AVERAGE";
    $cmd .= " CDEF:$item${i}_tmp=$item${i}_tmp_nan,UN,0,$item${i}_tmp_nan,IF";    # it must be here to make cauculation regardless of already deleted volumes! since 1.10
    $cmd .= " CDEF:$item${i}kb=$item${i}_tmp,$val,*";
    $cmd .= " CDEF:$item${i}=$item${i}kb,$valmb,/";
    if ( $prev == -1 ) {
      $cmd .= " CDEF:$itemm_sum${i}=$item${i}";
    }
    else {
      $cmd .= " CDEF:$itemm_sum${i}=$itemm_sum${last_vol},$item${i},+";
    }
    $prev++;
  } ## end foreach my $lpar (@vol_array)
  if ( $prev == -1 ) {
    return 0;
  }

  $cmd .= " CDEF:result=$itemm_sum${i},$limit,LT,UNKN,$itemm_sum${i},IF";
  $cmd .= " PRINT:result:MAX:%10.1lf";
  $cmd =~ s/\\"/"/g;

  my $answer = "";

  eval {
    RRDp::cmd qq($cmd);
    $answer = RRDp::read;
  };
  if ($@) {
    error( "RRDp::read $@ : $cmd " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( $$answer =~ m/NaN/ || $$answer =~ m/nan/ ) {

    #print "001 $host-$type-$item-$volg: $val : $limit : $$answer : $cmd";
    return 1;
  }

  #print "010 $host-$type-$item-$volg: $val : $limit : $$answer";
  return 0;
} ## end sub find_max_load_multi

# same as above only for single volume nicks
sub find_max_load {
  my $st_type    = shift;
  my $type       = shift;
  my $STEP       = shift;
  my $rrd        = shift;
  my $item       = shift;
  my $val        = shift;
  my $time_range = shift;
  my $limit      = 0;

  if ( $item =~ m/io_rate/ || $item =~ m/read_io/ || $item =~ m/write_io/ ) {
    $limit = $IO_MAX;
  }

  if ( $item =~ m/data_rate/ || $item =~ m/^read$/ || $item =~ m/^write$/ || $item =~ m/^read_b$/ || $item =~ m/^write_b$/ || $item =~ m/^r_transfer_size$/ || $item =~ m/^w_transfer_size$/) {
    $limit = $DATA_MAX;
  }

  if ( $item =~ m/resp/ ) {
    $limit = $RESPONSE_MAX;
  }

  if ( $item =~ m/cache_usage/ ) {
    $limit = $CACHE_MAX;
  }

  if ( $item =~ m/cache_hit/ ) {
    return 0;    # do not use it for cache_hit
  }

  ### convert r_transfer_size to res1
  if ($st_type eq "SWIZ" && $type eq "VOLUME" && $item eq "r_transfer_size"){
    $item  = "res1";
  }
  if ($st_type eq "SWIZ" && $type eq "VOLUME" && $item eq "w_transfer_size"){
    $item  = "res2";
  }

  $val = 1;
  my $valmb = 1;
  if ( $st_type =~ m/^DS8K$/ && $type =~ /VOLUME/ ) {
    if ( $item =~ m/^read$/ || $item =~ m/^write$/ ) {

      # data is stored in wrong RRDTool type (GAUGE instead of ABSOLUTE)
      # this do data conversion
      $val   = $STEP;    # it must be multiplied by STEP as it is cumulated since every 5 minutes
      $valmb = 1024;
    } ## end if ( $item =~ m/^read$/...)
  } ## end if ( $st_type =~ m/^DS8K$/...)

  $rrd =~ s/:/\\:/g;     # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

  RRDp::cmd qq(graph "$basedir/tmp/name.png"
      "--start" "now-1$time_range"
      "--end" "now"
      "--width=400"
      "--height=150"
      "--step=$STEP"
      "DEF:item1=$rrd:$item:AVERAGE"
      "CDEF:itemkb=item1,$val,*"
      "CDEF:item=itemkb,$valmb,/"
      "CDEF:result=item,$limit,LT,UNKN,item,IF"
      "PRINT:result:MAX: %10.2lf"
  );

  my $answer = RRDp::read;
  if ( $$answer =~ m/NaN/ || $$answer =~ m/nan/ ) {

    #print "011 $rrd: $type-$item: $val : $limit : $$answer : $rrd";
    return 1;
  }

  #print "111 $rrd: $type-$item: $val : $limit : $$answer";
  return 0;
} ## end sub find_max_load

sub sum_low_volume {
  my ( $val, $gtype, $i, $j, $last_vol, $color_indx, $item, $type, $low ) = @_;
  my @low_volumes = @{$low};
  my $volg        = "rest";
  my $cmd         = "";
  my $cmd_text    = "";
  my $cmd_txp1    = "";
  my $cmd_txp2    = "";
  my $itemm       = $item . "m" . $volg;
  $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
  $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
  $itemm =~ s/\#/Y/g;    # hash in rrdtool variable names causing problems for rrdtool parser
  $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
  $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)
  my $itemm_sum     = $itemm . "sum";
  my $lpar_space    = sprintf( "%-38s", "rest of vols total" );
  my $found         = 0;
  my $prev          = -1;                                         # must be reinitialized
  my $item_text     = "Volumes";
  my $item_text_cap = "VOLUME";

  if ( $type =~ m/LUN/ ) {
    $item_text = "Luns";
  }

  if ( $item =~ m/io_rate/ || $item =~ m/read_io/ || $item =~ m/write_io/ ) {
    $cmd_text .= " COMMENT:\\\"$item_text under $IO_MAX iops in a peak are not graphed\\l\\\"";
    $cmd_text .= " COMMENT:\\\"etc/stor2rrd.cfg\\:VOLUME_IO_MAX etc/storage-list.cfg\\:VOLUME_AGG_IO_LIM\\\"";
    $cmd_txp1 = "$item_text under $IO_MAX iops in a peak are not graphed";
    $cmd_txp2 = "etc/stor2rrd.cfg\\:VOLUME_IO_MAX etc/storage-list.cfg\\:VOLUME_AGG_IO_LIM";
  } ## end if ( $item =~ m/io_rate/...)

  if ( $item =~ m/data_rate/ || $item =~ m/^read$/ || $item =~ m/^write$/ ) {
    $cmd_text .= " COMMENT:\\\"$item_text under $DATA_MAX kBytes in a peak are not graphed\\l\\\"";
    $cmd_text .= " COMMENT:\\\"etc/stor2rrd.cfg\\:VOLUME_DATA_MAX etc/storage-list.cfg\\:VOLUME_AGG_DATA_LIM\\\"";
    $cmd_txp1 = "$item_text under $DATA_MAX kBytes in a peak are not graphed";
    $cmd_txp2 = "etc/stor2rrd.cfg\\:VOLUME_DATA_MAX etc/storage-list.cfg\\:VOLUME_AGG_DATA_LIM";
  } ## end if ( $item =~ m/data_rate/...)

  if ( $item =~ m/resp/ ) {
    $cmd_text .= " COMMENT:\\\"$item_text under $RESPONSE_MAX msec in a peak are not graphed\\l\\\"";
    $cmd_text .= " COMMENT:\\\"etc/stor2rrd.cfg\\:VOLUME_RESPONSE_MAX\\\"";
    $cmd_txp1 = "$item_text under $RESPONSE_MAX msec in a peak are not graphed";
    $cmd_txp2 = "etc/stor2rrd.cfg\\:VOLUME_RESPONSE_MAX";
  } ## end if ( $item =~ m/resp/ )

  if ( $item =~ m/cache_usage/ ) {
    $cmd_text .= " COMMENT:\\\"$item_text under $CACHE_MAX kBytes in a peak are not graphed\\l\\\"";
    $cmd_text .= " COMMENT:\\\"etc/stor2rrd.cfg\\:VOLUME_CACHE_MAX\\\"";
    $cmd_txp1 = "$item_text under $CACHE_MAX kBytes in a peak are not graphed";
    $cmd_txp2 = "etc/stor2rrd.cfg\\:VOLUME_CACHE_MAX";
  } ## end if ( $item =~ m/cache_usage/)

  foreach my $file (@low_volumes) {
    if ( $file eq '' ) {
      next;
    }
    $file =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing
                           # bulid RRDTool cmd

    $cmd .= " DEF:$item${i}_nan=\\\"$file\\\":$item:AVERAGE";
    $cmd .= " CDEF:$item${i}=$item${i}_nan,UN,0,$item${i}_nan,IF";    # it must be here to make cauculation regardless of already deleted volumes! since 1.10
    $cmd .= " CDEF:$itemm${i}=$item${i},$val,/";                      # convert into MB if necessary, normaly is there 1
    if ( $prev == -1 ) {
      $cmd .= " CDEF:$itemm_sum${i}=$itemm${i}";
    }
    else {
      $cmd .= " CDEF:$itemm_sum${i}=$itemm_sum${last_vol},$itemm${i},+";
    }
    $i++;
    $prev++;
    $found++;
    $last_vol++;
  } ## end foreach my $file (@low_volumes)

  if ( $found == 0 ) {
    return "$cmd_text";    # have not found any low volume
  }
  $cmd .= " COMMENT:\\\"\\l\\\"";
  $cmd .= " $gtype:$itemm_sum${last_vol}$color_rest_of_vols:\\\"$lpar_space\\\"";
  $cmd .= " GPRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf \\\"";
  $cmd .= " GPRINT:$itemm_sum${last_vol}:MAX:\\\"%6.1lf \\l\\\"";

  my $legend_heading = "item_short $delimiter [value_short] $delimiter Avrg $delimiter Max ";
  $legend_heading =~ s/%/%%/g;
  my $file_for_print = "";
  $cmd .= " PRINT:$itemm_sum${last_vol}:AVERAGE:\\\"%6.1lf $delimiter $lpar_space $delimiter $legend_heading $delimiter $color_rest_of_vols\\\"";
  $cmd .= " COMMENT:\\\" $delimiter $file_for_print $delimiter $st_type\\\"";
  $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %8.1lf $delimiter\\\"";
  $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %8.1lf $delim_com $cmd_txp1\\\"";
  $cmd .= " PRINT:$itemm_sum${last_vol}:MAX:\\\" %8.1lf $delim_com $cmd_txp2\\\"";

  $cmd .= $cmd_text;

  return $cmd;
} ## end sub sum_low_volume

sub check_space_name {
  my $name       = shift;
  my $name_space = $name;
  if ( $name =~ m/ / ) {
    $name_space = "\"" . $name . "\"";    # it must be here to support space with server names
    return $name_space;
  }
  else {
    return $name;
  }
} ## end sub check_space_name

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub convert_capacity {
  my $unit_input  = shift;
  my $unit_output = shift;
  my $value       = shift;
  $unit_input = uc($unit_input);

  if ( $unit_input eq $unit_output ) {
    return $value;
  }
  if ( $unit_input eq "B" && $unit_output eq "GB" ) {
    return round( $value / ( 1024 * 1024 * 1024 ), 1 );
  }
  if ( $unit_input eq "MB" && $unit_output eq "GB" ) {
    return round( $value / 1024, 1 );
  }
  if ( $unit_input eq "MB" && $unit_output eq "TB" ) {
    return round( $value / ( 1024 * 1024 ) );
  }
  if ( $unit_input eq "MB" && $unit_output eq "PB" ) {
    return round( $value / ( 1024 * 1024 * 1024 ) );
  }
  if ( $unit_input eq "GB" && $unit_output eq "TB" ) {
    return round( $value / (1024) );
  }
  if ( $unit_input eq "GB" && $unit_output eq "PB" ) {
    return round( $value / ( 1024 * 1024 ) );
  }
  if ( $unit_input eq "TB" && $unit_output eq "PB" ) {
    return round( $value / (1024) );
  }
  if ( $unit_input eq "GB" && $unit_output eq "MB" ) {
    return round( $value * 1024, 0 );
  }
  if ( $unit_input eq "TB" && $unit_output eq "MB" ) {
    return round( $value * ( 1024 * 1024 ), 0 );
  }
  if ( $unit_input eq "TB" && $unit_output eq "GB" ) {
    return round( $value * (1024) );
  }
  if ( $unit_input eq "PB" && $unit_output eq "MB" ) {
    return round( $value * ( 1024 * 1024 * 1024 ), 0 );
  }
  return "";
} ## end sub convert_capacity

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  if ( !defined($digit) ) {
    return 0;
  }
  if ( $digit eq '' ) {
    return 0;
  }
  if ( $digit eq 'U' ) {
    return 1;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
} ## end sub isdigit

sub clean_old_data_files {
  my ( $host, $wrkdir, $DEBUG ) = @_;

  # runs that once a day
  my $file = "$tmp_dir/clean-$host.touch";
  if ( -f "$file" ) {
    my $ret = Storcfg2html::once_a_day($file);
    if ( $ret == 0 ) {
      print "clean old tmp  : not this time - same day\n" if $DEBUG;
      return 1;
    }
  } ## end if ( -f "$file" )
  `touch "$file"`;
  print "clean old tmp  : $host: now \n" if $DEBUG;

  # go through all tmp files like DS8300-1.perf.20130429-0020-tmp
  # and delete older than 5 days

  my $week   = 604800;
  my $time_u = time();
  my $host_space = check_space_name($host);
  my @files  = <$wrkdir/$host_space/$host_space.perf*-tmp>;
  foreach $file (@files) {
    chomp($file);
    my $time = ( stat("$file") )[9];
    if ( $time + $week < $time_u ) {

      #delete old file
      my $filesize = -s "$file";
      my $loc_time = localtime($time);
      error( "delete old tmp : $host:$file : $loc_time : size:$filesize " . __FILE__ . ":" . __LINE__ );
      unlink($file);
    } ## end if ( $time + $week < $time_u)
  } ## end foreach $file (@files)

  return 1;
} ## end sub clean_old_data_files

sub draw_pool_cap {
  my $host    = shift;
  my $type    = shift;
  my $item    = shift;
  my $st_type = shift;

  draw_graph_pool_cap( "day",     "d", "MINUTE:60:HOUR:2:HOUR:4:0:%H", $host, $type, $item, $st_type );
  draw_graph_pool_cap( "week",    "w", "HOUR:8:DAY:1:DAY:1:86400:%a",  $host, $type, $item, $st_type );
  draw_graph_pool_cap( "4 weeks", "m", "DAY:1:DAY:2:DAY:2:0:%d",       $host, $type, $item, $st_type );
  draw_graph_pool_cap( "year",    "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, $item, $st_type );

  # POOL used capacity trends
  if ( $item eq "used" ) {
    draw_graph_pool_cap_trend( "year", "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, "$item\_trend", $st_type );
  }

  return 0;
} ## end sub draw_pool_cap

sub draw_graph_pool_cap_trend {
  my $text      = shift;
  my $type_gr   = shift;
  my $xgrid     = shift;
  my $host      = shift;
  my $type      = shift;
  my $item      = shift;
  my $st_type   = shift;
  my $name      = "$tmp_dir/$host/$type-$item-agg-$type_gr";
  my $t         = "COMMENT: ";
  my $t2        = "COMMENT:\\n";
  my $last      = "COMMENT: ";
  my $rrd_time  = "";
  my $step_new  = $STEP;
  my $once      = 0;
  my $last_time = "na";

  my $act_utime = time();
  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  my $type_text = $type;
  if ( $st_type =~ m/SWIZ/ && $type =~ m/RANK/ ) {
    $type_text = "MDISK";
  }
  if ( $st_type =~ m/HUS/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }

  if ( $st_type =~ m/VSPG/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }

  print "creating cmd   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $header   = "$type_text capacity $item: last $text";
  my $i        = 0;
  my $y        = -1;
  my $lpar     = "";
  my $cmd      = "";
  my $j        = 0;

  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width_trend";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;

  # do not use switch statement
  if ( $item =~ m/^used/ )               { $value_short = "TB"; $value_long = "TBytes"; $val = 1024; }
  if ( $item =~ m/^real/ )               { $value_short = "TB"; $value_long = "TBytes"; $val = 1024; }
  if ( $item =~ m/^compress_cap$/ )      { $value_short = "TB"; $value_long = "TBytes"; $val = 1; }
  if ( $item =~ m/^uncompress_cap$/ )    { $value_short = "TB"; $value_long = "TBytes"; $val = 1; }
  if ( $item =~ m/^compress_virt_cap$/ ) { $value_short = "TB"; $value_long = "TBytes"; $val = 1; }

  my $item_label = uc($item);
  $cmd .= " --vertical-label=\\\"Capacity in $value_long\\\"";

  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $legend = sprintf( "%-30s", "$item [$value_short]" );

  $cmd .= " COMMENT:\\\"$legend         Avrg        Max \\l\\\"";

  my $lpar_list      = "";
  my $pool_found     = 0;
  my $pool_id_tmp    = "";
  my $lpar_list_tmp  = "";
  my $nothing_to_sum = 1;
  my $searchstr      = "-cap.rrd";
  if ( $st_type =~ m/VSPG/ || $st_type =~ m/HUS/ || $st_type =~ m/DOTHILL/ ) {
    $searchstr = ".rrd";
  }

  foreach my $line (<$wrkdir/$host/$type/*$searchstr>) {
    chomp($line);
    if ( $st_type !~ m/VSPG/ && $st_type !~ m/HUS/ && $st_type !~ m/VNXF/ && $st_type !~ m/DOTHILL/ && !( $st_type eq "VMAX" && $type eq "DRIVE" && $item eq "used" ) ) {
      if ( $line !~ m/-cap\.rrd$/ && $item !~ m/compress/ ) {
        next;    # it does not end bu .rrd suffix, "$ cannt be use in foreach <>
      }
    }
    if ( $st_type =~ m/SWIZ/ && $line =~ m/-nl-cap\.rrd$/ ) {
      next;      # exclude separate *-nl-cap.rrd file, there is only tier0, but here it is about whole pool
    }
    if ( $st_type =~ m/SWIZ/ && $line =~ m/-comp\.rrd$/ && $item !~ m/compress/ ) {
      next;      # exclude data with compression -comp.rrd
    }
    if ( $st_type =~ m/SWIZ/ && $line !~ m/-comp\.rrd$/ && $item =~ m/compress/ ) {
      next;      # include data with compression -comp.rrd
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    my @link_l = split( /\//, $line );
    my $lpar = "";
    foreach my $m (@link_l) {
      $lpar = $m;
    }
    $lpar =~ s/-cap\.rrd$//;     # filter pool info
    $lpar =~ s/-comp\.rrd$//;    # filter pool info
    $lpar =~ s/\.rrd$//;         # filter pool info for HUS & VSPG

    my $pool_tr_table_indx = 0;
    my $pool_name          = "NA";
    $pool_name = get_pool_name($lpar);

    # bulid RRDTool cmd
    $line =~ s/:/\\:/g;          # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    my $item_to_def = $item;
    $item_to_def =~ s/_trend$//;

    $cmd .= " DEF:$item${i}_used=\\\"$line\\\":$item_to_def:AVERAGE";
    $cmd .= " DEF:$item${i}_used_1m=\\\"$line\\\":$item_to_def:AVERAGE:start=-1m";
    $cmd .= " DEF:$item${i}_used_3m=\\\"$line\\\":$item_to_def:AVERAGE:start=-3m";
    $cmd .= " DEF:$item${i}_used_1y=\\\"$line\\\":$item_to_def:AVERAGE:start=-1y";

    if ( $i == 0 ) {
      $cmd .= " CDEF:main_res${i}=$item${i}_used";
      $cmd .= " CDEF:main_res_1m${i}=$item${i}_used_1m";
      $cmd .= " CDEF:main_res_3m${i}=$item${i}_used_3m";
      $cmd .= " CDEF:main_res_1y${i}=$item${i}_used_1y";
    } ## end if ( $i == 0 )
    else {
      $cmd .= " CDEF:pom${i}=main_res${y},UN,0,main_res${y},IF,$item${i}_used,UN,0,$item${i}_used,IF,+";
      $cmd .= " CDEF:main_res${i}=main_res${y},UN,$item${i}_used,UN,UNKN,pom${i},IF,pom${i},IF";

      $cmd .= " CDEF:pom_1m${i}=main_res_1m${y},UN,0,main_res_1m${y},IF,$item${i}_used_1m,UN,0,$item${i}_used_1m,IF,+";
      $cmd .= " CDEF:main_res_1m${i}=main_res_1m${y},UN,$item${i}_used_1m,UN,UNKN,pom_1m${i},IF,pom_1m${i},IF";

      $cmd .= " CDEF:pom_3m${i}=main_res_3m${y},UN,0,main_res_3m${y},IF,$item${i}_used_3m,UN,0,$item${i}_used_3m,IF,+";
      $cmd .= " CDEF:main_res_3m${i}=main_res_3m${y},UN,$item${i}_used_3m,UN,UNKN,pom_3m${i},IF,pom_3m${i},IF";

      $cmd .= " CDEF:pom_1y${i}=main_res_1y${y},UN,0,main_res_1y${y},IF,$item${i}_used_1y,UN,0,$item${i}_used_1y,IF,+";
      $cmd .= " CDEF:main_res_1y${i}=main_res_1y${y},UN,$item${i}_used_1y,UN,UNKN,pom_1y${i},IF,pom_1y${i},IF";
    } ## end else [ if ( $i == 0 ) ]

    $y++;
    $i++;
  } ## end foreach my $line (<$wrkdir/$host/$type/*$searchstr>)

  $cmd .= " CDEF:total=main_res${y}";
  $cmd .= " CDEF:total_1m=main_res_1m${y}";
  $cmd .= " CDEF:total_3m=main_res_3m${y}";
  $cmd .= " CDEF:total_1y=main_res_1y${y}";

  my $lpar_space = sprintf( "%-28s", "Total" );
  $cmd .= " LINE:total#00FF00:\\\"$lpar_space\\\"";
  $cmd .= " GPRINT:total:AVERAGE:\\\"%9.0lf \\\"";
  $cmd .= " GPRINT:total:MAX:\\\"%9.0lf \\l\\\"";

  # 1 year trend
  $cmd .= " VDEF:Dd_1y=total_1y,LSLSLOPE";
  $cmd .= " VDEF:Hd_1y=total_1y,LSLINT";
  $cmd .= " CDEF:trend_1y=total_1y,POP,Dd_1y,COUNT,*,Hd_1y,+";
  $cmd .= " LINE2:trend_1y#FF8080:\\\"last 1 year trend\\\"";
  $cmd .= " COMMENT:\\\"\\l\\\"";

  # 3 month trend
  $cmd .= " VDEF:Dd_3m=total_3m,LSLSLOPE";
  $cmd .= " VDEF:Hd_3m=total_3m,LSLINT";
  $cmd .= " CDEF:trend_3m=total_3m,POP,Dd_3m,COUNT,*,Hd_3m,+";
  $cmd .= " LINE2:trend_3m#80FFFF:\\\"last 3 months trend\\\"";
  $cmd .= " COMMENT:\\\"\\l\\\"";

  # 1 month trend
  $cmd .= " VDEF:Dd_1m=total_1m,LSLSLOPE";
  $cmd .= " VDEF:Hd_1m=total_1m,LSLINT";
  $cmd .= " CDEF:trend_1m=total_1m,POP,Dd_1m,COUNT,*,Hd_1m,+";
  $cmd .= " LINE2:trend_1m#0088FF:\\\"last 1 month trend\\\"";
  $cmd .= " COMMENT:\\\"\\l\\\"";

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # It must be here!!!, touch() only in case there is any rrdfile
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  if ( $not_execute_rrdtool == 1 ) { return 1; }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < $tmp_file 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  return 0;
} ## end sub draw_graph_pool_cap_trend

sub draw_graph_pool_cap {
  my $text      = shift;
  my $type_gr   = shift;
  my $xgrid     = shift;
  my $host      = shift;
  my $type      = shift;
  my $item      = shift;
  my $st_type   = shift;
  my $name      = "$tmp_dir/$host/$type-$item-agg-$type_gr";
  my $t         = "COMMENT: ";
  my $t2        = "COMMENT:\\n";
  my $last      = "COMMENT: ";
  my $rrd_time  = "";
  my $step_new  = $STEP;
  my $once      = 0;
  my $last_time = "na";

  my $act_utime = time();
  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  my $type_text = $type;
  if ( $st_type =~ m/SWIZ/ && $type =~ m/RANK/ ) {
    $type_text = "MDISK";
  }
  if ( $st_type =~ m/HUS/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( $st_type =~ m/VSPG/ && $type =~ m/RANK/ ) {
    $type_text = "Raid Group";
  }
  if ( ( $st_type =~ m/VMAX/ || $st_type =~ m/3PAR/ ) && $type =~ m/TIER/ ) {
    $type_text = "Tier";
  }
  if ( $st_type =~ m/VMAX/ && $type =~ m/DRIVE/ ) {
    $type_text = "DISK GROUP";
  }

  print "creating cmd   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $header   = "$type_text capacity $item: last $text";
  if ( $item =~ m/^real$/ ) { $header = "$type_text capacity used: last $text"; }
  my $i    = 0;
  my $lpar = "";
  my $cmd  = "";
  my $j    = 0;

  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now-1$type_gr+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;

  # do not use switch statement
  if ( $item =~ m/^used$/ )              { $value_short = "TB"; $value_long = "TBytes"; $val = 1024; }
  if ( $item =~ m/^real$/ )              { $value_short = "TB"; $value_long = "TBytes"; $val = 1024; }
  if ( $item =~ m/^alloc$/ )             { $value_short = "TB"; $value_long = "TBytes"; $val = 1024; }
  if ( $item =~ m/^compress_cap$/ )      { $value_short = "TB"; $value_long = "TBytes"; $val = 1; }
  if ( $item =~ m/^uncompress_cap$/ )    { $value_short = "TB"; $value_long = "TBytes"; $val = 1; }
  if ( $item =~ m/^compress_virt_cap$/ ) { $value_short = "TB"; $value_long = "TBytes"; $val = 1; }
  if ( $item =~ m/^tier0$/ && $st_type eq "OCEANSTOR" ) { $item = "tier0cap"; $value_short = "TB"; $value_long = "TBytes"; $val = 1024; }
  if ( $item =~ m/^tier1$/ && $st_type eq "OCEANSTOR" ) { $item = "tier1cap"; $value_short = "TB"; $value_long = "TBytes"; $val = 1024; }
  if ( $item =~ m/^tier2$/ && $st_type eq "OCEANSTOR" ) { $item = "tier2cap"; $value_short = "TB"; $value_long = "TBytes"; $val = 1024; }

  my $item_label = uc($item);
  $cmd .= " --vertical-label=\\\"Capacity in $value_long\\\"";

  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $item_legend = $item;
  $item_legend =~ s/uncompress_//;
  $item_legend =~ s/compress_//;    # make shorter compress legend (SWIZZ --> POOL --> Compression)
                                    #$item_legend =~ s/compress//;
  my $legend = sprintf( "%-38s", "$item_legend [$value_short]" );
  if ( $item =~ m/^real$/ ) {
    $legend = sprintf( "%-38s", "used [$value_short]" );
  }
  if ( $item =~ m/^tier0cap$/ ) {
    $legend = sprintf( "%-38s", "tier0 [$value_short]" );
  }
  if ( $item =~ m/^tier1cap$/ ) {
    $legend = sprintf( "%-38s", "tier1 [$value_short]" );
  }
  if ( $item =~ m/^tier2cap$/ ) {
    $legend = sprintf( "%-38s", "tier2 [$value_short]" );
  }

  $cmd .= " COMMENT:\\\"$legend         Avrg        Max \\l\\\"";

  my $gtype = "AREA";

  my $lpar_list      = "";
  my $pool_found     = 0;
  my $pool_id_tmp    = "";
  my $lpar_list_tmp  = "";
  my $nothing_to_sum = 1;
  my $searchstr      = "-cap.rrd";
  if ( $st_type =~ m/VSPG/ || $st_type =~ m/HUS/ || $st_type =~ m/VNXF/ || $st_type =~ m/DOTHILL/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/OCEANSTOR/ || $st_type =~ m/PURE/ ) {
    $searchstr = ".rrd";
  }
  if ( $st_type eq "VMAX" && $type eq "DRIVE" && $item eq "used" ) {
    $searchstr = ".rrd";
  }
  if ( $st_type eq "SWIZ" && $type eq "POOL" && $item =~ m/compress/ ) {
    $searchstr = "-comp.rrd";
  }

  # open a file with stored colours
  my @color_save = "";
  my $type_low   = lc $type;
  my $file_col   = "$wrkdir/$host/$type/" . $type_low . "s.col";
  if ( -f $file_col ) {
    open( FHC, "< $file_col" ) || error( "file does not exists : $file_col " . __FILE__ . ":" . __LINE__ ) && return 0;
    @color_save = <FHC>;
    close(FHC);
  }
  my $color_indx        = 0;    # reset colour index
  my $color_file_change = 0;

  my $host_space = check_space_name($host);

  foreach my $line (<$wrkdir/$host_space/$type/*$searchstr>) {
    chomp($line);
    if ( $st_type !~ m/VSPG/ && $st_type !~ m/HUS/ && $st_type !~ m/COMPELLENT/ && $st_type !~ m/PURE/ && $st_type !~ m/OCEANSTOR/ && $st_type !~ m/VNXF/ && $st_type !~ m/DOTHILL/ && !( $st_type eq "VMAX" && $type eq "DRIVE" && $item eq "used" ) ) {
      if ( $line !~ m/-cap\.rrd$/ && $item !~ m/compress/ ) {
        next;                   # it does not end bu .rrd suffix, "$ cannt be use in foreach <>
      }
    }
    if ( $st_type =~ m/SWIZ/ && $line =~ m/-nl-cap\.rrd$/ ) {
      next;                     # exclude separate *-nl-cap.rrd file, there is only tier0, but here it is about whole pool
    }
    if ( $st_type =~ m/SWIZ/ && $line =~ m/-comp\.rrd$/ && $item !~ m/compress/ ) {
      next;                     # exclude data with compression -comp.rrd
    }
    if ( $st_type =~ m/SWIZ/ && $line !~ m/-comp\.rrd$/ && $item =~ m/compress/ ) {
      next;                     # include data with compression -comp.rrd
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    if ( $type_gr =~ m/d/ && $once == 0 ) {

      # find out time stamp of last data update
      # take just one volume to keep it simple
      $once++;
      RRDp::cmd qq(last "$line");
      my $last_tt = RRDp::read;
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
    } ## end if ( $type_gr =~ m/d/ ...)

    my @link_l = split( /\//, $line );
    my $lpar = "";
    foreach my $m (@link_l) {
      $lpar = $m;
    }
    $lpar =~ s/-cap\.rrd$//;     # filter pool info
    $lpar =~ s/-comp\.rrd$//;    # filter pool info
    $lpar =~ s/\.rrd$//;         # filter pool info for HUS & VSPG

    my $rrd_item = $item;

    my $pool_tr_table_indx = 0;
    my $pool_name          = "NA";
    $pool_name = get_pool_name($lpar);
    if ( $st_type eq "VMAX" && $type eq "TIER" ) {
      $pool_name = $lpar;
    }
    if ( $st_type eq "3PAR" && $type eq "TIER" && $item eq "used" ) {
      $pool_name = $lpar;
    }
    if ( $st_type eq "VMAX" && $type eq "DRIVE" && $item eq "used" ) {
      $pool_name = $lpar;
      $rrd_item  = "cap_used";
    }

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    my $lpar_space = $pool_name;
    $lpar_space = sprintf( "%-38s", "$lpar_space" );

    # get color index
    my $ret_col = get_color( $lpar_space, $color_max, \@color_save );
    ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
    if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

      # new name has been identified, save it into an array and later on save into *col file
      $color_file_change = 1;
      $color_save[$color_indx] = $new_col_rec;
      ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
    } ## end if ( defined($new_col_rec...))

    # bulid RRDTool cmd
    $line =~ s/:/\\:/g;                                                 # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    $cmd .= " DEF:$item${i}_used=\\\"$line\\\":$rrd_item:AVERAGE";
    $cmd .= " CDEF:$item${i}_used_check=$item${i}_used,UN,0,$item${i}_used,IF";
    $cmd .= " $gtype:$item${i}_used_check$color[$color_indx]:\\\"$lpar_space\\\"";
    $cmd .= " GPRINT:$item${i}_used_check:AVERAGE:\\\"%6.2lf \\\"";
    $cmd .= " GPRINT:$item${i}_used_check:MAX:\\\"%6.2lf \\l\\\"";
    $i++;
    $gtype = "STACK";
  } ## end foreach my $line (<$wrkdir/$host/$type/*$searchstr>)

  if ( $i == 0 && $st_type eq "SWIZ" && $type eq "POOL" && $item =~ m/compress/ ) {
    print "creating cmd   : $host:$type:$item : no compression data\n" if $DEBUG;
    return 0;    #  no compression data
  }

  if ( $type_gr =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # It must be here!!!, touch() only in case there is any rrdfile
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  if ( $not_execute_rrdtool == 1 ) { return 1; }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < $tmp_file 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  return 0;
} ## end sub draw_graph_pool_cap

sub pool_translate {
  my $act_time = shift;
  my $wrkdir   = shift;
  my $host     = shift;

  # read  pool table to translate id to pool names

  open( FHP, "< $wrkdir/$host/pool.cfg" ) || error( "Can't open $wrkdir/$host/pool.cfg : $! : (ignore after fresh install) " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FHP>;
  close(FHP);
  @pool_tr_table_id   = "";
  @pool_tr_table_name = "";
  my $pool_tr_table_indx = 0;
  foreach my $line_p (@lines) {
    chomp($line_p);
    ( my $id, my $name ) = split( /:/, $line_p );
    if ( !defined($id) || $id eq '' || !defined($name) || $name eq '' ) {
      next;
    }
    $id =~ s/ //g;
    $name =~ s/^ //;
    $pool_tr_table_id[$pool_tr_table_indx]   = $id;
    $pool_tr_table_name[$pool_tr_table_indx] = $name;
    $pool_tr_table_indx++;
  } ## end foreach my $line_p (@lines)

  return 0;
} ## end sub pool_translate

sub get_pool_name {
  my $pool_id = shift;
  my $counter = 0;

  foreach my $id (@pool_tr_table_id) {
    if ( $id eq $pool_id ) {
      if ( defined( $pool_tr_table_name[$counter] ) && $pool_tr_table_name[$counter] ne '' ) {
        return $pool_tr_table_name[$counter];
      }
    }
    $counter++;
  } ## end foreach my $id (@pool_tr_table_id)
  return "NA";
} ## end sub get_pool_name

sub draw_all_port_subsystem {
  my $host = shift;
  my $type = shift;
  my $item = shift;

  draw_graph_port_subsystem( "day",     "d", "MINUTE:60:HOUR:2:HOUR:4:0:%H", $host, $type, $item );
  draw_graph_port_subsystem( "week",    "w", "HOUR:8:DAY:1:DAY:1:86400:%a",  $host, $type, $item );
  draw_graph_port_subsystem( "4 weeks", "m", "DAY:1:DAY:2:DAY:2:0:%d",       $host, $type, $item );
  draw_graph_port_subsystem( "year",    "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, $item );
  return 0;
} ## end sub draw_all_port_subsystem

sub draw_graph_port_subsystem {
  my $text       = shift;
  my $type_gr    = shift;
  my $xgrid      = shift;
  my $host       = shift;
  my $type       = shift;
  my $item       = shift;
  my $graph_item = "subsys";
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $last       = "COMMENT: ";
  my $rrd_time   = "";
  my $step_new   = $STEP;
  my $once       = 0;
  my $last_time  = "na";
  my $port_type  = -1;

  if ( $item =~ m/-FC$/ ) {
    $graph_item = "FC";
    $item =~ s/-FC$//;
    $port_type = 0;
  }
  else {
    if ( $item =~ m/-SAS$/ ) {
      $graph_item = "SAS";
      $item =~ s/-SAS$//;
      $port_type = 1;
    }
  } ## end else [ if ( $item =~ m/-FC$/ )]
  my $name = "$tmp_dir/$host/$type-$item-agg-$graph_item-$type_gr";

  my $act_utime = time();
  my $tmp_file  = "$tmp_dir/$host/$type-$item-$graph_item-$type_gr.cmd";

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  print "creating cmd   : $host:$type:$item:$graph_item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $header   = "$type per subsystem $item: last $text";
  if ( $port_type == 0 ) {
    $header = "$type per FC $item: last $text";
  }
  if ( $port_type == 1 ) {
    $header = "$type per SAS/PCi $item: last $text";
  }
  my $i         = 0;
  my $lpar      = "";
  my $cmd       = "";
  my $cmd_print = "";    # print commands for clickable legend
  my $j         = 0;

  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now-1$type_gr+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;    # base, the number will be devided by that constant

  # do not use switch statement
  if ( $item =~ m/^data_rate$/ ) { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^read$/ )      { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^write$/ )     { $value_short = "MB/sec"; $value_long = "MBytes/sec"; $val = 1024; }
  if ( $item =~ m/^io_rate$/ )   { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^read_io$/ )   { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^write_io$/ )  { $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^resp_t/ )     { $value_short = "ms";     $value_long = "ms"; }

  if ( $item =~ m/^sys$/ || $item =~ m/^compress$/ ) {
    $cmd .= " --vertical-label=\\\"$value_long\\\"";
  }
  else {
    $cmd .= " --vertical-label=\\\"$item in $value_long\\\"";
  }

  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $legend = sprintf( "%-38s", "$item [$value_short]" );
  $cmd .= " COMMENT:\\\"$legend    Avrg     Max \\l\\\"";

  my $legend_heading = "$item $delimiter [$value_short] $delimiter Avrg $delimiter Max ";
  $legend_heading =~ s/%/%%/g;

  my $gtype = "LINE1";
  if ( $port_type != -1 ) {
    $gtype = "AREA";
  }

  my $lpar_list     = "";
  my $pool_found    = 0;
  my $pool_id_tmp   = "";
  my $lpar_list_tmp = "";

  my @color_save = "";
  my $type_low   = lc $type;
  my $file_col   = "$wrkdir/$host/$type/" . $type_low . "s.col";
  if ( -f $file_col ) {
    open( FHC, "< $file_col" ) || error( "file does not exists : $file_col " . __FILE__ . ":" . __LINE__ ) && return 0;
    @color_save = <FHC>;
    close(FHC);
  }
  my $color_indx        = 0;    # reset colour index
  my $color_file_change = 0;

  foreach my $line (<$wrkdir/$host/$type/*rrd>) {
    chomp($line);
    if ( $line !~ m/\.rrd$/ ) {
      next;                     # it does not end bu .rrd suffix, "$ cannt be use in foreach <>
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    my @link_l = split( /\//, $line );
    my $lpar = "";
    foreach my $m (@link_l) {
      $lpar = $m;
    }
    $lpar =~ s/\.rrd$//;
    $lpar =~ s/PORT-//;
    $lpar =~ s/0x//;

    if ( $type_gr =~ m/d/ && $once == 0 ) {

      # find out time stamp of last data update
      # take just one volume to keep it simple
      $once++;
      RRDp::cmd qq(last "$line");
      my $last_tt = RRDp::read;
      chomp($$last_tt);
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
    } ## end if ( $type_gr =~ m/d/ ...)

    # bulid RRDTool cmd
    my $itemm = $item . "m";
    $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/\#/Y/g;    # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    # summ up traffic per FC/SAS and iSCSI sources
    if ( $port_type == -1 ) {
      $cmd .= " DEF:$item${i}=\\\"$line\\\":$item:AVERAGE";
      $cmd .= " DEF:type${i}=\\\"$line\\\":type:AVERAGE";
      $cmd .= " CDEF:$itemm${i}=$item${i},$val,/";                # convert into MB if necessary, normaly is there 1
      $cmd .= " CDEF:fc${i}=type${i},0,EQ,$itemm${i},0,IF";       # FC disky
      $cmd .= " CDEF:sas${i}=type${i},1,EQ,$itemm${i},0,IF";      # SAS disky
      $cmd .= " CDEF:iscsi${i}=type${i},2,EQ,$itemm${i},0,IF";    # iSCSI disky
      $cmd .= " CDEF:iprep${i}=type${i},4,EQ,$itemm${i},0,IF";    # IPREP disky
      $cmd .= " CDEF:fcoe${i}=type${i},5,EQ,$itemm${i},0,IF";     # IPREP disky
      $cmd .= " CDEF:rcip${i}=type${i},6,EQ,$itemm${i},0,IF";     # RCIP  disky
      $cmd .= " CDEF:eth${i}=type${i},7,EQ,$itemm${i},0,IF";      # Ethernet  disky
    } ## end if ( $port_type == -1 )
    else {
      my $rrd_file_conv = $line;
      $rrd_file_conv =~ s/\.rr[a-z]//;

      my $port_name = basename($line);                            # node1_p01.rrd
      $port_name =~ s/\.rrd$//;

      if ( $type =~ m/^PORT$/ && -f "$wrkdir/$host/$type/port.cfg" ) {

        # translate id into names only for 3PAR & SVC
        my $port_found   = 0;
        my $port_id      = "";
        my $lpar_filtred = $port_name;
        foreach my $line (@port_trans) {
          chomp($line);
          ( my $id, my $name ) = split( /:/, $line );
          if ( defined($name) && !$name eq '' && $id =~ m/^$lpar_filtred$/ ) {
            if ( $st_type =~ m/SWIZ/ || $st_type =~ m/3PAR/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/OCEANSTOR/ ) {
              $port_name .= " " . $name;
            }
            else {
              $port_name = $name;
            }
            $port_found = 1;
            last;
          } ## end if ( defined($name) &&...)
        } ## end foreach my $line (@port_trans)
        if ( $port_found == 0 ) {
          $port_name = $lpar;
        }
      } ## end if ( $type =~ m/^PORT$/...)

      # select on FC ports which has speed info file: PORT/node2_p03-speed.txt
      if ( $port_name !~ m/(FC.*)/ ) {
        next;

        #3PAR: 0-1-1:0-1-1 AIX1 (FC)
        #SWIZ: node1_p01:(FC 500507680306B942)
      } ## end if ( $port_name !~ m/(FC.*)/)

      $cmd .= " DEF:$item${i}=\\\"$line\\\":$item:AVERAGE";
      $cmd .= " DEF:type${i}=\\\"$line\\\":type:AVERAGE";
      $cmd .= " CDEF:$itemm${i}=$item${i},$val,/";            # convert into MB if necessary, normaly is there 1

      # get color index
      my $ret_col = get_color( $port_name, $color_max, \@color_save );
      ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
      if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

        # new name has been identified, save it into an array and later on save into *col file
        $color_file_change = 1;
        $color_save[$color_indx] = $new_col_rec;
        ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
      } ## end if ( defined($new_col_rec...))

      $cmd       .= " CDEF:port${i}=type${i},$port_type,EQ,$itemm${i},UNKN,IF";                                                                 # FC disky, a bit trick, place on non FCS "-1" and filter these lines out in detail-graph
      $cmd       .= " $gtype:port${i}$color[$color_indx]:\\\"$port_name\\\"";
      $cmd       .= " GPRINT:port${i}:AVERAGE:\\\"%6.1lf \\\"";
      $cmd       .= " GPRINT:port${i}:MAX:\\\"%6.1lf \\l\\\"";
      $cmd_print .= " PRINT:port${i}:AVERAGE:\\\"%6.1lf $delimiter $port_name $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
      $cmd_print .= " PRINT:port${i}:MAX:\\\" %8.1lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";
    } ## end else [ if ( $port_type == -1 )]
    $i++;
    if ( $port_type != -1 ) {
      $gtype = "STACK";
    }
  } ## end foreach my $line (<$wrkdir/$host/$type/*rrd>)

  # Load all items into the $cmd
  if ( $port_type == -1 ) {

    # FC
    my $index_actual = 0;
    $cmd .= " CDEF:fc=fc${index_actual}";
    $index_actual++;
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",fc${index_actual},+";
    }
    $cmd .= " $gtype:fc#00FF00:\\\"FC          \\\"";
    if ( $item =~ m/io/ ) {
      $cmd .= " GPRINT:fc:AVERAGE:\\\"%6.0lf \\\"";
      $cmd .= " GPRINT:fc:MAX:\\\"%6.0lf \\l\\\"";
    }
    else {
      $cmd .= " GPRINT:fc:AVERAGE:\\\"%6.1lf \\\"";
      $cmd .= " GPRINT:fc:MAX:\\\"%6.1lf \\l\\\"";
    }

    # SAS
    $index_actual = 0;
    $cmd .= " CDEF:sas=sas${index_actual}";
    $index_actual++;
    for ( ; $index_actual < $i; $index_actual++ ) {
      $cmd .= ",sas${index_actual},+";
    }
    $cmd .= " $gtype:sas#4444FF:\\\"SAS         \\\"";
    if ( $item =~ m/io/ ) {
      $cmd .= " GPRINT:sas:AVERAGE:\\\"%6.0lf \\\"";
      $cmd .= " GPRINT:sas:MAX:\\\"%6.0lf \\l\\\"";
    }
    else {
      $cmd .= " GPRINT:sas:AVERAGE:\\\"%6.1lf \\\"";
      $cmd .= " GPRINT:sas:MAX:\\\"%6.1lf \\l\\\"";
    }

    if ( $st_type ne "OCEANSTOR" ) {

      # iSCSI
      $index_actual = 0;
      $cmd .= " CDEF:iscsi=iscsi${index_actual}";
      $index_actual++;
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",iscsi${index_actual},+";
      }
      $cmd .= " $gtype:iscsi#FFFF00:\\\"iSCSI       \\\"";
      if ( $item =~ m/io/ ) {
        $cmd .= " GPRINT:iscsi:AVERAGE:\\\"%6.0lf \\\"";
        $cmd .= " GPRINT:iscsi:MAX:\\\"%6.0lf \\l\\\"";
      }
      else {
        $cmd .= " GPRINT:iscsi:AVERAGE:\\\"%6.1lf \\\"";
        $cmd .= " GPRINT:iscsi:MAX:\\\"%6.1lf \\l\\\"";
      }

    } ## end if ( $st_type ne "OCEANSTOR")

    if ( $st_type eq "OCEANSTOR" ) {

      # ethernet
      $index_actual = 0;
      $cmd .= " CDEF:eth=eth${index_actual}";
      $index_actual++;
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",eth${index_actual},+";
      }
      $cmd .= " $gtype:eth#FFFF00:\\\"ETH         \\\"";
      if ( $item =~ m/io/ ) {
        $cmd .= " GPRINT:eth:AVERAGE:\\\"%6.0lf \\\"";
        $cmd .= " GPRINT:eth:MAX:\\\"%6.0lf \\l\\\"";
      }
      else {
        $cmd .= " GPRINT:eth:AVERAGE:\\\"%6.1lf \\\"";
        $cmd .= " GPRINT:eth:MAX:\\\"%6.1lf \\l\\\"";
      }
    } ## end if ( $st_type eq "OCEANSTOR")

    if ( $st_type =~ m/SWIZ/ ) {

      # IPREP
      $index_actual = 0;
      $cmd .= " CDEF:iprep=iprep${index_actual}";
      $index_actual++;
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",iprep${index_actual},+";
      }
      $cmd .= " $gtype:iprep#FF4444:\\\"IP replicat \\\"";
      if ( $item =~ m/io/ ) {
        $cmd .= " GPRINT:iprep:AVERAGE:\\\"%6.0lf \\\"";
        $cmd .= " GPRINT:iprep:MAX:\\\"%6.0lf \\l\\\"";
      }
      else {
        $cmd .= " GPRINT:iprep:AVERAGE:\\\"%6.1lf \\\"";
        $cmd .= " GPRINT:iprep:MAX:\\\"%6.1lf \\l\\\"";
      }
    } ## end if ( $st_type =~ m/SWIZ/)

    if ( $st_type =~ m/3PAR/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/OCEANSTOR/ ) {

      # FC0E
      $index_actual = 0;
      $cmd .= " CDEF:fcoe=fcoe${index_actual}";
      $index_actual++;
      for ( ; $index_actual < $i; $index_actual++ ) {
        $cmd .= ",fcoe${index_actual},+";
      }
      $cmd .= " $gtype:fcoe#FF4444:\\\"FCoE        \\\"";
      if ( $item =~ m/io/ ) {
        $cmd .= " GPRINT:fcoe:AVERAGE:\\\"%6.0lf \\\"";
        $cmd .= " GPRINT:fcoe:MAX:\\\"%6.0lf \\l\\\"";
      }
      else {
        $cmd .= " GPRINT:fcoe:AVERAGE:\\\"%6.1lf \\\"";
        $cmd .= " GPRINT:fcoe:MAX:\\\"%6.1lf \\l\\\"";
      }

      if ( $st_type eq "3PAR" ) {

        # RCIP
        $index_actual = 0;
        $cmd .= " CDEF:rcip=rcip${index_actual}";
        $index_actual++;
        for ( ; $index_actual < $i; $index_actual++ ) {
          $cmd .= ",rcip${index_actual},+";
        }
        $cmd .= " $gtype:rcip#FF4444:\\\"RCIP        \\\"";
        if ( $item =~ m/io/ ) {
          $cmd .= " GPRINT:rcip:AVERAGE:\\\"%6.0lf \\\"";
          $cmd .= " GPRINT:rcip:MAX:\\\"%6.0lf \\l\\\"";
        }
        else {
          $cmd .= " GPRINT:rcip:AVERAGE:\\\"%6.1lf \\\"";
          $cmd .= " GPRINT:rcip:MAX:\\\"%6.1lf \\l\\\"";
        }
      } ## end if ( $st_type eq "3PAR")
    } ## end if ( $st_type =~ m/3PAR/...)
  } ## end if ( $port_type == -1 )
  else {
    # write colors into a file ponly for -FC
    write_colors( $color_file_change, $file_col, \@color_save );
  }

  if ( $type_gr =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  if ( $port_type != -1 ) {

    # -FC only
    $cmd .= $cmd_print;    # for clickable legend
  }
  $cmd =~ s/\\"/"/g;

  # It must be here!!!, touch() only in case there is any rrdfile
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  if ( $not_execute_rrdtool == 1 ) { return 1; }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < $tmp_file 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  return 0;
} ## end sub draw_graph_port_subsystem

sub draw_volume_cache_hit_all {
  my $host    = shift;
  my $type    = shift;
  my $st_type = shift;

  if ( $st_type =~ m/^DS5K$/ ) {
    if ( -f "$wrkdir/$host/DS5K-v2" ) {
      draw_volume_cache_hit( $host, $type, "w_cache_hit",     $st_type );
      draw_volume_cache_hit( $host, $type, "r_cache_hit",     $st_type );
      draw_volume_cache_hit( $host, $type, "read_pct",        $st_type );
      draw_volume_cache_hit( $host, $type, "ssd_r_cache_hit", $st_type );
    } ## end if ( -f "$wrkdir/$host/DS5K-v2")
    else {
      # Old DS5K has only cache_hit
      draw_volume_cache_hit( $host, $type, "cache_hit", $st_type );
      draw_volume_cache_hit( $host, $type, "read_pct",  $st_type );
    }
  } ## end if ( $st_type =~ m/^DS5K$/)
  else {
    draw_volume_cache_hit( $host, $type, "w_cache_hit", $st_type );
    draw_volume_cache_hit( $host, $type, "r_cache_hit", $st_type );
  }

} ## end sub draw_volume_cache_hit_all

sub draw_volume_cache_hit {
  my $host    = shift;
  my $type    = shift;
  my $item    = shift;
  my $st_type = shift;

  my $dfile    = "$tmp_dir/$host/$type-$item-d.out";
  my $wfile    = "$tmp_dir/$host/$type-$item-w.out";
  my $mfile    = "$tmp_dir/$host/$type-$item-m.out";
  my $yfile    = "$tmp_dir/$host/$type-$item-y.out";
  my $out_file = "$webdir/$host/cache-$item.html";

  my @dout = "";
  my @wout = "";
  my @mout = "";
  my @yout = "";

  if ( -f $dfile ) {
    open( FHD, "< $dfile" ) || error( "Can't open $dfile : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @dout = <FHD>;
    close(FHD);
  }
  else {
    error( "$host:$type:$item - no daily file " . __FILE__ . ":" . __LINE__ ) if ($DEBUG);
    return 0;
  }

  if ( -f $wfile ) {
    open( FHD, "< $wfile" ) || error( "Can't open $wfile : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @wout = <FHD>;
    close(FHD);
  }
  if ( -f $mfile ) {
    open( FHD, "< $mfile" ) || error( "Can't open $mfile : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @mout = <FHD>;
    close(FHD);
  }
  if ( -f $yfile ) {
    open( FHD, "< $yfile" ) || error( "Can't open $yfile : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    @yout = <FHD>;
    close(FHD);
  }

  #@dout = sort { (split ' ', $a)[2] cmp (split ' ', $b)[2] } @dout;
  #@dout = sort { (split ' ', $a)[3] <=> (split ' ', $b)[3] } @dout;

  my @name = "";
  my @util = "";
  my $indx = 0;

  # goes through all 4 files and create one outpud table with cache_hit
  foreach my $dline (@dout) {
    chomp($dline);
    if ( $dline =~ m/^OK u:/ || $dline =~ m/^$/ || $dline !~ m/ / ) {
      next;
    }
    $dline =~ s/=====space=====/ /g;
    ( my $dlpar, my $dutil ) = split( / +/, $dline );
    if ( $dlpar =~ m/^$/ ) {
      next;
    }
    $dutil =~ s/NaNQ/NA/g;
    $dutil =~ s/NaN/NA/g;
    $dutil =~ s/-nan/NA/g;
    $dutil =~ s/nan/NA/g;
    $name[$indx] = $dlpar;
    $util[$indx] .= "<td align=\"right\">$dutil</td>";
    $indx++;

  } ## end foreach my $dline (@dout)

  foreach my $wline (@wout) {
    if ( $wline =~ m/^OK u:/ || $wline =~ m/^$/ || $wline !~ m/ / ) {
      next;
    }
    $wline =~ s/=====space=====/ /g;
    ( my $wlpar, my $wutil ) = split( / +/, $wline );
    if ( $wlpar =~ m/^$/ ) {
      next;
    }
    $wutil =~ s/NaNQ/NA/g;
    $wutil =~ s/NaN/NA/g;
    $wutil =~ s/-nan/NA/g;
    $wutil =~ s/nan/NA/g;
    my $found = 0;
    $indx = 0;
    foreach my $line (@name) {

      if ( $line =~ m/^$wlpar$/ ) {
        $util[$indx] .= "<td align=\"right\">$wutil</td>";
        $found = 1;
        last;
      }
      $indx++;
    } ## end foreach my $line (@name)
    if ( $found == 0 ) {
      $name[$indx] = $wlpar;
      $util[$indx] .= "<td align=\"right\">NA</td><td align=\"right\">$wutil</td>";

      #print "003 $name[$indx] : $util[$indx]\n";
    } ## end if ( $found == 0 )
  } ## end foreach my $wline (@wout)

  foreach my $mline (@mout) {
    if ( $mline =~ m/^OK u:/ || $mline =~ m/^$/ || $mline !~ m/ / ) {
      next;
    }
    $mline =~ s/=====space=====/ /g;
    ( my $mlpar, my $mutil ) = split( / +/, $mline );
    if ( $mlpar =~ m/^$/ ) {
      next;
    }
    $mutil =~ s/NaNQ/NA/g;
    $mutil =~ s/NaN/NA/g;
    $mutil =~ s/-nan/NA/g;
    $mutil =~ s/nan/NA/g;
    my $found = 0;
    $indx = 0;
    foreach my $line (@name) {

      if ( $line =~ m/^$mlpar$/ ) {
        $util[$indx] .= "<td align=\"right\">$mutil</td>";
        $found = 1;
        last;
      }
      $indx++;
    } ## end foreach my $line (@name)
    if ( $found == 0 ) {
      $name[$indx] = $mlpar;
      $util[$indx] .= "<td align=\"right\">NA</td><td align=\"right\">NA</td><td align=\"right\">$mutil</td>";

      #print "003 $name[$indx] : $util[$indx]\n";
    } ## end if ( $found == 0 )
  } ## end foreach my $mline (@mout)

  foreach my $yline (@yout) {
    if ( $yline =~ m/^OK u:/ || $yline =~ m/^$/ || $yline !~ m/ / ) {
      next;
    }
    $yline =~ s/=====space=====/ /g;
    ( my $ylpar, my $yutil ) = split( / +/, $yline );
    if ( $ylpar =~ m/^$/ ) {
      next;
    }
    $yutil =~ s/NaNQ/NA/g;
    $yutil =~ s/NaN/NA/g;
    $yutil =~ s/-nan/NA/g;
    $yutil =~ s/nan/NA/g;
    my $found = 0;
    $indx = 0;
    foreach my $line (@name) {

      if ( $line =~ m/^$ylpar$/ ) {
        $util[$indx] .= "<td align=\"right\">$yutil</td>";
        $found = 1;
        last;
      }
      $indx++;
    } ## end foreach my $line (@name)
    if ( $found == 0 ) {
      $name[$indx] = $ylpar;
      $util[$indx] .= "<td align=\"right\">NA</td><td align=\"right\">NA</td><td align=\"right\">NA</td><td align=\"right\">$yutil</td>";
    }
  } ## end foreach my $yline (@yout)

  open( FHD, "> $out_file" ) || error( "Can't open $out_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHD "<center><table class=\"lparsearch tablesorter\"><thead><tr><th class=\"sortable\">Volume</th><th class=\"sortable\">&nbsp;day&nbsp;&nbsp;&nbsp;&nbsp;</th><th class=\"sortable\">&nbsp;week&nbsp;&nbsp;&nbsp;&nbsp;</th><th class=\"sortable\">&nbsp;month&nbsp;&nbsp;&nbsp;&nbsp;</th><th class=\"sortable\">&nbsp;year&nbsp;&nbsp;&nbsp;&nbsp;</th></tr></thead><tbody>\n";
  $indx = 0;
  foreach my $line (@name) {
    print FHD "<tr><td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=$type&name=$line&storage=$st_type&none=none\">$line</a></td>$util[$indx]</tr>\n";

    #print  "$line : $util[$indx]\n";
    $indx++;
  } ## end foreach my $line (@name)

  print FHD "</tbody></table></center>";
  close(FHD);

  return 0;
} ## end sub draw_volume_cache_hit

# it check if rrdtool supports graphv --> then zoom is supported
sub rrdtool_graphv {
  my $graph_cmd   = "graph";
  my $graphv_file = "$tmp_dir/graphv";

  my $ansx = `$rrdtool`;

  if ( index( $ansx, 'graphv' ) != -1 ) {

    # graphv exists, create a file to pass it to cgi-bin commands
    if ( !-f $graphv_file ) {
      `touch $graphv_file`;
    }
  } ## end if ( index( $ansx, 'graphv'...))
  else {
    if ( -f $graphv_file ) {
      unlink($graphv_file);
    }
  }

  return 0;
} ## end sub rrdtool_graphv

# it is iportant to have real data start, as before that are take NaN to keep good average and since then when NaN exist then == 0
# to make summary graphs working
# it is about total graphs, all POOL graphs, CPU, Drives total ...

sub find_real_data_start {
  my $rrd_file     = shift;
  my $time_first   = 1;
  my $refresh_time = 86400;    # 1 day
  my $data_line    = "";

  my $rrd_file_first = $rrd_file;
  $rrd_file_first =~ s/rr.$/first/g;

  # find out time stamp of first data update
  # note first data point with data, not first with NaN which returns rrdtool first
  RRDp::cmd qq(first "$rrd_file");
  my $first_tt = RRDp::read;
  chomp($$first_tt);    # start of predefined data points in rrdtool  , tehre could be NaN

  # find real start of data
  $time_first = $$first_tt;
  my $unix_act = time();
  if ( -f $rrd_file_first && ( ( stat("$rrd_file_first") )[9] + $refresh_time ) > $unix_act ) {

    # read real last time of the record in rrdtool from the file (refresh every day)
    open( FHF, "< $rrd_file_first" ) || error( "Can't open $rrd_file_first : $! " . __FILE__ . ":" . __LINE__ );
    foreach my $line_frst (<FHF>) {
      chomp($line_frst);
      if ( isdigit($line_frst) ) {
        $time_first = $line_frst;
        last;
      }
    } ## end foreach my $line_frst (<FHF>)
    close(FHF);
  } ## end if ( -f $rrd_file_first...)
  else {
    my $time_year_before = time() - 31622400;    # unix time - 1 year
                                                 #RRDp::cmd qq(fetch "$rrd_file" AVERAGE --end $time_year_before );
                                                 # rrdtool fetch and found the first record is a bit tricky, must be used --end stuff!!!
    RRDp::cmd qq(fetch "$rrd_file" AVERAGE --start $time_year_before );

    # strange, --start must be used!! at least on 1.4.8 RRDTool
    my $data           = RRDp::read;
    my $time_first_act = 2000000000;             # just place here something hig engouh, higher than actual unix time in seconds
    foreach $data_line ( split( /\n/, $$data ) ) {
      chomp($data_line);
      ( my $time_first_act_tmp, my $item1, my $item2 ) = split( / /, $data_line );
      if ( isdigit($item1) || isdigit($item2) ) {
        $time_first_act = $time_first_act_tmp;
        $time_first_act =~ s/://g;
        $time_first_act =~ s/ //g;
        if ( isdigit($time_first_act) ) {
          last;
        }
      } ## end if ( isdigit($item1) ||...)
    } ## end foreach $data_line ( split(...))
    if ( isdigit($time_first_act) && $time_first_act > $time_first ) {

      # when is rrdtool DB file older than retention of 300s data then rrdtool first has the right value of the first record
      $time_first = $time_first_act;
    }
    if ( $time_first_act == 2000000000 ) {

      # looks like the RRDfile is empty, no records --> place there 1 year old date
      $time_first = $time_year_before;
    }
    open( FHF, "> $rrd_file_first" ) || error( "Can't open $rrd_file_first : $! " . __FILE__ . ":" . __LINE__ );
    print FHF "$time_first";
    close(FHF);
  } ## end else [ if ( -f $rrd_file_first...)]

  $time_first =~ s/://g;
  if ( isdigit($time_first) == 0 ) {
    return 1;    # something is wrong, "1" causes ignoring followed rrdtool construction
  }
  return $time_first;
} ## end sub find_real_data_start

# it finds out or create a file with first data stapm in all rank files of given pool
# it is necessary to know it as since that time is used 0 instead on NaN in rrdtool cmd
sub find_real_data_start_pool {
  my $rank_dir = shift;
  my $pool_id  = shift;
  my $st_type  = shift;
  if ( isdigit($pool_id) == 1 && $st_type !~ m/VSPG/ ) {
    $pool_id = $pool_id + 1 - 1;    # integer conversion
  }
  my $rrd_pool_first = "$rank_dir/P$pool_id.first_pool";    # only 3PAR uses that actually (v1.35-1)
  my $refresh_time   = 86400;                               # 1 day
  my $time_first_def = 2000000000;                          # just place here something hig engouh, higher than actual unix time in seconds
  my $time_first     = $time_first_def;

  my $unix_act = time();

  # daily refresh pool first time
  if ( ( $ds5k_pool_org == 1 && $st_type =~ m/DS5K/ ) || ( $xiv_pool_org == 1 && $st_type =~ m/XIV/ ) ) {
    if ( -f $rrd_pool_first && ( ( stat("$rrd_pool_first") )[9] + $refresh_time ) > $unix_act ) {

      # read real last time of the record in rrdtool from the file (refresh every day)
      open( FHF, "< $rrd_pool_first" ) || error( "Can't open $rrd_pool_first : $! " . __FILE__ . ":" . __LINE__ );
      foreach my $line_frst (<FHF>) {
        chomp($line_frst);
        if ( isdigit($line_frst) ) {
          $time_first = $line_frst;
          last;
        }
      } ## end foreach my $line_frst (<FHF>)
      close(FHF);
    } ## end if ( -f $rrd_pool_first...)
    else {
      foreach my $file_rank (<$rank_dir/*-P$pool_id\.first>) {

        # use *-P[0-9][0-9]*\.first to avoid some bogus 000P.first without the pool ID
        open( FHF, "< $file_rank" ) || error( "Can't open $file_rank : $! " . __FILE__ . ":" . __LINE__ );
        foreach my $line_frst (<FHF>) {
          chomp($line_frst);
          if ( isdigit($line_frst) && $line_frst < $time_first ) {
            $time_first = $line_frst;
            last;
          }
        } ## end foreach my $line_frst (<FHF>)
        close(FHF);
      } ## end foreach my $file_rank (<$rank_dir/*-P$pool_id\.first>)
      if ( $time_first < $time_first_def ) {

        # time has been found, save it to do not have to search it every run as it does not get changed often
        open( FHF, "> $rrd_pool_first" ) || error( "Can't open $rrd_pool_first : $! " . __FILE__ . ":" . __LINE__ );
        print FHF "$time_first";
        close(FHF);
      } ## end if ( $time_first < $time_first_def)
    } ## end else [ if ( -f $rrd_pool_first...)]
  } ## end if ( ( $ds5k_pool_org ...))
  else {
    # common way for everything
    $rrd_pool_first = "$rank_dir/$pool_id.first";
    if ( -f $rrd_pool_first && ( ( stat("$rrd_pool_first") )[9] + $refresh_time ) > $unix_act ) {

      # read real last time of the record in rrdtool from the file (refresh every day)
      open( FHF, "< $rrd_pool_first" ) || error( "Can't open $rrd_pool_first : $! " . __FILE__ . ":" . __LINE__ );
      foreach my $line_frst (<FHF>) {
        chomp($line_frst);
        if ( isdigit($line_frst) ) {
          $time_first = $line_frst;
          last;
        }
      } ## end foreach my $line_frst (<FHF>)
      close(FHF);
    } ## end if ( -f $rrd_pool_first...)
  } ## end else [ if ( ( $ds5k_pool_org ...))]
  if ( $time_first < $time_first_def ) {
    return $time_first;
  }
  else {
    return 1;    # something is wrong, "1" causes ignoring followed rrdtool construction
  }
} ## end sub find_real_data_start_pool

# it finds the first valid data point in all rank files first files
# it is necessary to know it as since that time is used 0 instead on NaN in rrdtool cmd
# work for RANK and DRIVE and POOLs (capacity)
## first files are refreshed afer 24 hours
## they would start working properly after 6hours since initial data start!!!! --> but next day refresh does the job
### till next day refresh there is 1 year ahead first time date!!!

sub find_real_data_start_all {
  my $rank_dir       = shift;
  my $time_first_def = 2000000000;        # just place here something hig engouh, higherb than actual unix time in seconds
  my $time_first     = $time_first_def;

  #create first files and refresh every day
  foreach my $rrd_rank (<$rank_dir/*\.rrd>) {
    find_real_data_start($rrd_rank);
  }

  foreach my $file_first (<$rank_dir/*\.first>) {

    # read real last time of the record in rrdtool from the file (refresh every day)
    open( FHF, "< $file_first" ) || error( "Can't open $file_first : $! " . __FILE__ . ":" . __LINE__ );
    foreach my $line_frst (<FHF>) {
      chomp($line_frst);
      if ( isdigit($line_frst) ) {
        if ( $line_frst < $time_first ) {
          $time_first = $line_frst;
        }
        last;
      } ## end if ( isdigit($line_frst...))
    } ## end foreach my $line_frst (<FHF>)
    close(FHF);
  } ## end foreach my $file_first (<$rank_dir/*\.first>)

  if ( $time_first < $time_first_def ) {
    return $time_first;
  }
  else {
    return 1;    # something is wrong, "1" causes ignoring followed rrdtool construction
  }
} ## end sub find_real_data_start_all

sub volumes_in_top_all {
  my $host    = shift;
  my $type    = shift;
  my $item    = shift;
  my $st_type = shift;

  my $ret = volumes_in_top( "d", $host, $type, $st_type, $item );
  if ( $ret > 0 ) {
    return $ret;
  }
  $ret = volumes_in_top( "w", $host, $type, $st_type, $item );
  if ( $ret > 0 ) {
    return $ret;
  }
  $ret = volumes_in_top( "m", $host, $type, $st_type, $item );
  if ( $ret > 0 ) {
    return $ret;
  }
  $ret = volumes_in_top( "y", $host, $type, $st_type, $item );
  if ( $ret > 0 ) {
    return $ret;
  }
  return 0;
} ## end sub volumes_in_top_all

sub hosts_in_top_all {

  my $host    = shift;
  my $type    = shift;
  my $item    = shift;
  my $st_type = shift;

  my $ret = hosts_in_top( "d", $host, $type, $st_type, $item );
  if ( $ret > 0 ) {
    return $ret;
  }
  $ret = hosts_in_top( "w", $host, $type, $st_type, $item );
  if ( $ret > 0 ) {
    return $ret;
  }
  $ret = hosts_in_top( "m", $host, $type, $st_type, $item );
  if ( $ret > 0 ) {
    return $ret;
  }
  $ret = hosts_in_top( "y", $host, $type, $st_type, $item );
  if ( $ret > 0 ) {
    return $ret;
  }
  return 0;

} ## end sub hosts_in_top_all

sub hosts_in_top {
  my $type_gr     = shift;
  my $host        = shift;
  my $type        = shift;
  my $st_type     = shift;
  my $item        = shift;
  my $name        = "$tmp_dir/$host/$type-$item-$type_gr";
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $last        = "COMMENT: ";
  my $rrd_time    = "";
  my @low_volumes = "";
  my $indx_low    = 0;
  my $prev        = -1;
  my $step_new    = $STEP;
  my $once        = 0;
  my @item_list   = ( "data_rate", "io_rate", "resp_t" );

  if ( $st_type eq "VMAX" ) {
    @item_list = ( "resp_t", "resp_t_r", "resp_t_w", "io_rate", "data_rate", "read", "write", "read_io", "write_io" );
  }

  if ( $st_type eq "3PAR" ) {
    @item_list = ( "resp_t", "resp_t_r", "resp_t_w", "io_rate", "read", "write", "read_io", "write_io" );
  }
  if ( $st_type eq "COMPELLENT" || $st_type eq "PURE" ) {
    @item_list = ( "resp_t", "resp_t_r", "resp_t_w", "io_rate", "read", "write", "read_io", "write_io", "data_rate" );
  }
  if ( $st_type eq "OCEANSTOR" ) {
    @item_list = ( "resp_t", "resp_t_r", "resp_t_w", "io_rate", "read", "write", "read_io", "write_io", "data_rate" );
  }

  my $act_utime = time();
  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  print "creating top   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $i        = 0;
  my $lpar     = "";
  my $cmd      = "";

  my $color_indx = 0;    # reset colour index

  my $end_time = "now";
  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
    $end_time = "now-1d";
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
    $end_time = "now-1w";
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
    $end_time = "now-1m";
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --units-exponent=1.00";
  my $gtype = "LINE1";

  my $lpar_list     = "";
  my $lpar_list_tmp = "";

  my $type_low = lc $type;
  my $file_cfg = "$wrkdir/$host/$type/" . $type_low . "s.cfg";

  my $vols        = "";
  my $lpar_space  = "";
  my $once_report = 0;
  my $check       = 0;

  my @hosts_file = <$wrkdir/$host/$type/*>;
  foreach my $file_act (@hosts_file) {
    if ( $file_act !~ m/rrd$/ ) { next; }
    $check++;
    my $host_name = basename($file_act);
    $host_name =~ s/\.rrd//g;
    $host_name =~ s/:/\\:/g;
    $host_name =~ s/ /=====space=====/g;
    foreach my $item_act (@item_list) {

      # bulid RRDTool cmd
      $file_act =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

      $cmd .= " DEF:$item_act${i}=\\\"$file_act\\\":$item_act:AVERAGE";
      my $val = 1;
      if ( $item_act =~ m/^read$/ || $item_act =~ m/^write$/ || $item_act =~ m/^data_rate/ ) {
        $val = 1024;
      }
      $cmd .= " CDEF:$item_act${i}_sum=$item_act${i},$val,/";
      $cmd .= " PRINT:$item_act${i}_sum:AVERAGE:\\\"====start ==== $host_name $item_act %6.1lf\\\"";
      $i++;
    } ## end foreach my $item_act (@item_list)

  } ## end foreach my $file_act (@hosts_file)
  if ( $check == 0 ) {    ### no host rrd files
    return 0;
  }
  $cmd =~ s/\\"/"/g;

  #$cmd =~ s/ /\n/g; # for debug purposes

  # It must be here!!!, touch() only in case there is any rrdfile-
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 1;
  print FH "$cmd\n";
  close(FH);
  return 0;

} ## end sub hosts_in_top

sub volumes_in_top {
  my $type_gr     = shift;
  my $host        = shift;
  my $type        = shift;
  my $st_type     = shift;
  my $item        = shift;
  my $name        = "$tmp_dir/$host/$type-$item-$type_gr";
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $last        = "COMMENT: ";
  my $rrd_time    = "";
  my @low_volumes = "";
  my $indx_low    = 0;
  my $prev        = -1;
  my $step_new    = $STEP;
  my $once        = 0;
  my @item_list   = ( "read", "write", "read_io", "write_io", "resp_t_r", "resp_t_w", "r_cache_hit", "w_cache_hit" );
  my $no_of_items = 8;                                                                                                  # number items in @item_list
  if ( $st_type =~ m/DS5K/ && -f "$wrkdir/$host/DS5K-v1" ) {
    @item_list   = ( "data_rate", "io_rate", "cache_hit" );
    $no_of_items = 3;                                                                                                   # number items in @item_list
  }
  if ( $st_type =~ m/DS5K/ && -f "$wrkdir/$host/DS5K-v2" ) {
    @item_list   = ( "data_rate", "io_rate", "resp_t", "r_cache_hit", "w_cache_hit", "ssd_r_cache_hit" );
    $no_of_items = 6;                                                                                                   # number items in @item_list
  }
  if ( $st_type =~ m/XIV/ ) {
    @item_list   = ( "read", "write", "read_io", "write_io", "resp_t_r", "resp_t_w" );
    $no_of_items = 6;                                                                                                   # number items in @item_list
  }
  if ( $st_type =~ m/3PAR/ || $st_type =~ m/NETAPP/ || $st_type =~ m/COMPELLENT/ || $st_type =~ m/PURE/ ) {
    @item_list   = ( "read", "write", "read_io", "write_io", "resp_t_r", "resp_t_w", "resp_t" );
    $no_of_items = 7;                                                                                                   # number items in @item_list
  }
  if ( $st_type =~ m/OCEANSTOR/ ) {
    @item_list = ( "read", "write", "read_io", "write_io", "resp_t_r", "resp_t_w", "resp_t", "r_cache_hit", "w_cache_hit" );
    $no_of_items = 9;    # number items in @item_list
  }
  if ( $st_type =~ m/VNXF|UNITY|VNXE|VNX$/ ) {
    @item_list = ( "read", "write", "read_io", "write_io", "resp_t" );
    $no_of_items = 5;    # number items in @item_list
  }
  if ( $st_type =~ m/DOTHILL/ ) {
    @item_list = ( "read", "write", "read_io", "write_io" );
    $no_of_items = 4;    # number items in @item_list
  }
  if ( $st_type =~ m/VMAX/ ) {
    @item_list = ( "read", "write", "read_io", "write_io", "resp_t", "r_cache_hit", "w_cache_hit" );
    $no_of_items = 7;    # number items in @item_list
  }
  if ( $st_type =~ m/VPLEX/ ) {
    @item_list = ( "read", "write", "io_rate", "resp_t_r", "resp_t_w" );
    $no_of_items = 5;    # number items in @item_list
  }

  my $act_utime = time();
  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  print "creating top   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $i        = 0;
  my $lpar     = "";
  my $cmd      = "";

  my $color_indx = 0;    # reset colour index

  my $end_time = "now";
  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
    $end_time = "now-1d";
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
    $end_time = "now-1w";
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
    $end_time = "now-1m";
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --units-exponent=1.00";
  my $gtype = "LINE1";

  my $lpar_list     = "";
  my $lpar_list_tmp = "";

  my $type_low = lc $type;
  my $file_cfg = "$wrkdir/$host/$type/" . $type_low . "s.cfg";

  # read volume cfg files with grouped volumes
  if ( !-f $file_cfg ) {
    if ( $type =~ m/^VOLUME/ ) {
      error( "$host: $type cfg file has not been found : $file_cfg " . __FILE__ . ":" . __LINE__ );
    }
    else {
      # do not print error for LUNs (NetApp), they do not have to exist
      print "$host: $type cfg file has not been found : $file_cfg " . __FILE__ . ":" . __LINE__ . "\n";
    }
    return 2;
  } ## end if ( !-f $file_cfg )
  open( FHR, "< $file_cfg" ) || error( "$host: file does not exists : $file_cfg " . __FILE__ . ":" . __LINE__ ) && return 2;
  my @files = <FHR>;
  close(FHR);

  my $vols        = "";
  my $lpar_space  = "";
  my $once_report = 0;

  foreach my $volg (@files) {
    chomp($volg);
    $vols = $volg;
    $volg =~ s/ : .*//;
    $vols =~ s/^.* : //;

    my @vol_array = split( ';', $vols );

    $prev = -1;

    my $one_volume_nick = 2;    # volume nick has 1 physical volume == 1, or more < 1;
    foreach my $lpar (@vol_array) {
      $lpar =~ s/ //;           # it must be there!!!
      if ( $lpar eq '' ) {
        next;
      }
      $one_volume_nick--;
    } ## end foreach my $lpar (@vol_array)

    #
    # follow each volume (there migh be more volumes under 1 volume nick!!!
    # first run just select files ... must be for mutilevel resp_t a cache_hit to count how many items are summed
    my @vol_array_work      = "";
    my $vol_array_work_indx = 0;
    my $suffix              = "rrd";
    foreach my $lpar (@vol_array) {

      $lpar =~ s/^0x//;
      $lpar =~ s/ //;    # it must be there!!!
      my $file = "$wrkdir/$host/$type/$lpar.$suffix";    # always test .rrd

      if ( $lpar eq '' ) {
        next;
      }

      if ( $st_type =~ m/XIV/ || $st_type =~ m/DS5K/ ) {

        # XIV & DS5K volumes contain pool_id in their names: 00273763-P102932.rrd
        # there might be more files if volumes are moved between pool, select the most fresh one
        my $vol_update_last = 0;
        foreach my $file_xiv (<$wrkdir/$host/$type/$lpar-P*\.$suffix>) {
          my $vol_upd_time = ( stat("$file_xiv") )[9];
          if ( $vol_upd_time > $vol_update_last ) {
            $file            = $file_xiv;
            $vol_update_last = $vol_upd_time;
          }
        } ## end foreach my $file_xiv (<$wrkdir/$host/$type/$lpar-P*\.$suffix>)
        if ( $vol_update_last == 0 ) {

          # do not report it at all
          #volume has not been found,m skipping it
          #if ( $once_report == 0 ) {
          #  # It might appear after the upgrade to 1.00 as *rrc files are not in place yet
          #  error("volumes stats: $file does not exist, continuing ".__FILE__.":".__LINE__);
          #  $once_report++;
          #}
          next;
        } ## end if ( $vol_update_last ...)
      } ## end if ( $st_type =~ m/XIV/...)

      if ( !-f $file ) {

        # do not report it at all
        #if ( $once_report == 0 ) {
        #  # It might appear after the upgrade to 1.00 as *rrc files are not in place yet
        #  error("volumes stats: $file does not exist, continuing ".__FILE__.":".__LINE__);
        #  $once_report++;
        #}
        next;
      } ## end if ( !-f $file )

      # go every each volume for particular group

      # avoid old lpars which do not exist in the period
      my $rrd_upd_time = ( stat("$file") )[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }
      $vol_array_work[$vol_array_work_indx] = $file;
      $vol_array_work_indx++;
    } ## end foreach my $lpar (@vol_array)

    # lets make $cmd for multilevels ones
    foreach my $file (@vol_array_work) {
      if ( $file eq '' ) {
        next;
      }

      foreach my $item_act (@item_list) {

        my $file_act = $file;
        if ( $st_type =~ m/^DS8K$/ && ( $item_act =~ m/^read_io$/ || $item_act =~ m/^write_io$/ || $item_act =~ m/^resp_t_r$/ || $item_act =~ m/^resp_t_w$/ ) ) {
          $file_act =~ s/\.rrd$/\.rrc/;
        }
        if ( $item_act =~ m/^w_cache_hit$/ || $item_act =~ m/^r_cache_hit$/ ) {
          if ( $st_type =~ m/3PAR/ || $st_type =~ m/XIV/ || $st_type =~ m/NETAPP/ || $st_type =~ m/VNX/ ) {
            next;    # XIV do not support cache stats
          }
          if ( $st_type !~ m/DS5K/ && $st_type !~ m/HUS/ && $st_type !~ m/^NETAPP$/ && $st_type !~ m/^3PAR/ && $st_type !~ m/^VSPG/ && $st_type !~ m/VNX/ && $st_type !~ m/DOTHILL/ && $st_type !~ m/VMAX/ && $st_type !~ m/OCEANSTOR/ && $st_type !~ m/VPLEX/ ) {
            $file_act =~ s/\.rrd$/\.rrc/;
          }
        } ## end if ( $item_act =~ m/^w_cache_hit$/...)

        # bulid RRDTool cmd
        $file_act =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

        $cmd .= " DEF:$item_act${i}=\\\"$file_act\\\":$item_act:AVERAGE";

        if ( $prev == -1 ) {
          $cmd .= " CDEF:$item_act${i}_sum=$item_act${i}";
        }
        else {
          my $last_vol_item = $i - $no_of_items;
          $cmd .= " CDEF:$item_act${i}_sum=$item_act${last_vol_item}_sum,$item_act${i},+";
        }
        $i++;
      } ## end foreach my $item_act (@item_list)
      $prev++;
    } ## end foreach my $file (@vol_array_work)

    if ( $prev == -1 ) {
      next;    # nothing found for that volume
    }
    my $index_last = $i - $no_of_items;    # 8 items are printed
    my $volg_space = $volg;
    $volg_space =~ s/ /=====space=====/g;
    $volg_space =~ s/:/\\:/g;

    foreach my $item_act (@item_list) {
      my $val = 1;
      if ( $item_act =~ m/^read$/ || $item_act =~ m/^write$/ || $item_act =~ m/^data_rate/ ) {
        $val = 1024;
      }
      if ( $item_act =~ m/resp_t/ || $item_act =~ m/cache_hit/ ) {

        # multivolumes
        # resp_t and cache_hit must be averaged not summed for multivolumes ...
        $val = $vol_array_work_indx;
      } ## end if ( $item_act =~ m/resp_t/...)
      if ( $st_type =~ m/^DS8K$/ && ( $item_act =~ m/^read$/ || $item_act =~ m/^write$/ ) ) {

        # data is stored in wrong RRDTool type (GAUGE instead of ABSOLUTE)
        # this do data conversion
        $val = $step_new;    # do not use $val=1024 for them!!
      } ## end if ( $st_type =~ m/^DS8K$/...)
      $cmd .= " CDEF:$item_act${index_last}_sum_val=$item_act${index_last}_sum,$val,/";    # convert into MB if necessary, normaly is there 1

      if ( $item_act =~ m/resp/ || $item_act =~ m/^read$/ || $item_act =~ m/^write$/ ) {
        $cmd .= " PRINT:$item_act${index_last}_sum_val:AVERAGE:\\\"====start ==== $volg_space $item_act %6.1lf\\\"";
      }
      else {
        $cmd .= " PRINT:$item_act${index_last}_sum_val:AVERAGE:\\\"====start ==== $volg_space $item_act %6.1lf\\\"";
      }
      $index_last++;                                                                       # go back to show all items per a volume
    } ## end foreach my $item_act (@item_list)
  } ## end foreach my $volg (@files)
  $cmd =~ s/\\"/"/g;

  #$cmd =~ s/ /\n/g; # for debug purposes

  # It must be here!!!, touch() only in case there is any rrdfile
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 1;
  print FH "$cmd\n";
  close(FH);

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  #my $ret  = `$rrdtool - < $tmp_file 2>&1`;
  #if ( $ret =~ "ERROR" ) {
  #  if ( $ret =~ "ERROR: malloc fetch data area" ) {
  #    error ("$host:$type:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area ".__FILE__.":".__LINE__);
  #  }
  #  else {
  #    error ("$host:$type:$type_gr: Multi graph rrdtool error : $ret ".__FILE__.":".__LINE__);
  #  }
  #}
  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");
  #print "$ret\n";

  return 0;
} ## end sub volumes_in_top

# set colors for volumes in $wrkdir/$host/$type/volumes.col
sub set_colors_initial {
  my $host      = shift;
  my $type      = shift;
  my $type_file = shift;

  my $color_file_change = 0;
  my $color_indx        = 0;

  # open a file with stored colours
  my @color_save = "";
  my $file_col   = "$wrkdir/$host/$type/$type_file.col";
  if ( -f $file_col ) {
    open( FHC, "< $file_col" ) || error( "file does not exists : $file_col " . __FILE__ . ":" . __LINE__ ) && return 1;
    @color_save = <FHC>;
    close(FHC);
  }

  # read volume cfg files with grouped volumes
  if ( !-f "$wrkdir/$host/$type/$type_file.cfg" ) {
    error( "$host: Volumes cfg file has been found : $wrkdir/$host/$type/$type_file.cfg " . __FILE__ . ":" . __LINE__ );
    return 2;
  }
  open( FHR, "< $wrkdir/$host/$type/$type_file.cfg" ) || error( "$host: file does not exists : $wrkdir/$host/$type/$type_file.cfg " . __FILE__ . ":" . __LINE__ ) && return 2;
  my @volumes = <FHR>;
  close(FHR);

  foreach my $volume (@volumes) {
    chomp($volume);
    if ( !defined($volume) || $volume eq '' || $volume !~ m/ : / ) {
      next;
    }
    $volume =~ s/ : .*//;
    $volume =~ s/:/===========doublecoma=========/g;

    # get color index
    my $ret_col = get_color( $volume, $color_max, \@color_save );
    ( $color_indx, my $new_col_rec ) = split( /\|/, $ret_col );
    if ( defined($new_col_rec) && !$new_col_rec eq '' ) {

      # new name has been identified, save it into an array and later on save into *col file
      $color_file_change = 1;
      $color_save[$color_indx] = $new_col_rec;
      ( $color_indx, my $name_save ) = split( / : /, $new_col_rec );    # $color_indx contains here only number 0 - $color_max
    } ## end if ( defined($new_col_rec...))
  } ## end foreach my $volume (@volumes)

  # write colors into a file
  write_colors( $color_file_change, $file_col, \@color_save );

  return 1;
} ## end sub set_colors_initial

sub basename {
  my $full = shift;
  my $out  = "";

  # basename without direct function
  my @base = split( /\//, $full );
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
} ## end sub basename

# if digit or hexa then true
sub ishexa {
  my $digit = shift;

  if ( !defined($digit) || $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//g;    # to support UUID's 107d3696-f7ac-4db8-9adc-48208635e9a0

  # hexa filter
  $digit_work =~ s/[a|b|c|d|e|f|A|B|C|D|E|F]//g;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  #main::error ("there was expected a digit but a string is there, field: , value: $digit");
  return 0;
} ## end sub ishexa

sub create_dir_structure {
  my $webdir  = shift;
  my $wrkdir  = shift;
  my $host    = shift;
  my $st_type = shift;
  my $tmp_dir = shift;

  # be carefull, all those directories are also created in bin/data_load.pl
  if ( !-d "$webdir/$host" ) {
    print "mkdir          : $webdir/$host\n" if $DEBUG;
    mkdir( "$webdir/$host", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host: $!";
    LoadDataModule::touch();
  }
  if ( !-d "$webdir/$host/VOLUME" ) {
    print "mkdir          : $webdir/$host/VOLUME\n" if $DEBUG;
    mkdir( "$webdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/VOLUME: $!";
    LoadDataModule::touch();
  }
  if ( !-d "$wrkdir/$host" ) {
    print "mkdir          : $wrkdir/$host\n" if $DEBUG;
    mkdir( "$wrkdir/$host", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host: $!";
    LoadDataModule::touch();
  }
  if ( !-d "$wrkdir/$host/VOLUME" ) {
    print "mkdir          : $wrkdir/$host/VOLUME\n" if $DEBUG;
    mkdir( "$wrkdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/VOLUME: $!";
    LoadDataModule::touch();
  }
  if ( !-d "$wrkdir/$host/POOL" ) {
    print "mkdir          : $wrkdir/$host/POOL\n" if $DEBUG;
    mkdir( "$wrkdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/POOL: $!";
    LoadDataModule::touch();
  }
  if ( !-d "$webdir/$host/POOL" ) {
    print "mkdir          : $webdir/$host/POOL\n" if $DEBUG;
    mkdir( "$webdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/POOL: $!";
    LoadDataModule::touch();
  }

  if ( $st_type =~ m/^XIV$/ ) {
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
  }
  if ( $st_type =~ m/^SWIZ$/ || $st_type =~ m/^DS8K$/ ) {
    if ( !-d "$webdir/$host/RANK" ) {
      print "mkdir          : $webdir/$host/RANK\n" if $DEBUG;
      mkdir( "$webdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/RANK: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/RANK" ) {
      print "mkdir          : $wrkdir/$host/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/RANK: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/IO-GROUP" ) {
      print "mkdir          : $webdir/$host/IO-GROUP\n" if $DEBUG;
      mkdir( "$webdir/$host/IO-GROUP", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/IO-GROUP: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/IO-GROUP" ) {
      print "mkdir          : $wrkdir/$host/IO-GROUP\n" if $DEBUG;
      mkdir( "$wrkdir/$host/IO-GROUP", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/IO-GROUP: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^SWIZ$/...)
  if ( $st_type =~ m/^3PAR$/ ) {
    if ( !-d "$wrkdir/$host/TIER" ) {
      print "mkdir          : $wrkdir/$host/TIER\n" if $DEBUG;
      mkdir( "$wrkdir/$host/TIER", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/TIER: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/DRIVE" ) {
      print "mkdir          : $wrkdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/CPU-CORE" ) {
      print "mkdir          : $webdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$webdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NODE" ) {
      print "mkdir          : $wrkdir/$host/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/VVOL" ) {
      print "mkdir          : $wrkdir/$host/VVOL\n" if $DEBUG;
      mkdir( "$wrkdir/$host/VVOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/VVOL: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^3PAR$/)
  if ( $st_type =~ m/^HUS$/ ) {
    if ( !-d "$webdir/$host/RANK" ) {
      print "mkdir          : $webdir/$host/RANK\n" if $DEBUG;
      mkdir( "$webdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/RANK: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/RANK" ) {
      print "mkdir          : $wrkdir/$host/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/RANK: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$host/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE-CACHE" ) {
      print "mkdir          : $webdir/$host/NODE-CACHE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-l "$wrkdir/$host/RG" ) {
      print "ln -s          : $wrkdir/$host/RANK $wrkdir/$host/RG\n" if $DEBUG;
      symlink( "$wrkdir/$host/RANK", "$wrkdir/$host/RG" ) || die "$act_time: Cannot ln -s $wrkdir/$host/RG: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/DRIVE" ) {
      print "mkdir          : $wrkdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/DRIVE" ) {
      print "mkdir          : $webdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$webdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/CPU-CORE" ) {
      print "mkdir          : $webdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$webdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^HUS$/)
  if ( $st_type =~ m/^VNXF$/ ) {
    if ( !-d "$wrkdir/$host/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$host/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE-CACHE" ) {
      print "mkdir          : $webdir/$host/NODE-CACHE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NODE" ) {
      print "mkdir          : $wrkdir/$host/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE" ) {
      print "mkdir          : $webdir/$host/NODE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/FS" ) {
      print "mkdir          : $wrkdir/$host/FS\n" if $DEBUG;
      mkdir( "$wrkdir/$host/FS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/FS: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/FS" ) {
      print "mkdir          : $webdir/$host/FS\n" if $DEBUG;
      mkdir( "$webdir/$host/FS", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/FS: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/CPU-CORE" ) {
      print "mkdir          : $webdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$webdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^VNXF$/)
  if ( $st_type =~ m/^VSPG$/ ) {
    if ( !-d "$webdir/$host/RANK" ) {
      print "mkdir          : $webdir/$host/RANK\n" if $DEBUG;
      mkdir( "$webdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/RANK: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/RANK" ) {
      print "mkdir          : $wrkdir/$host/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/RANK: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST-PORT" ) {
      print "mkdir          : $webdir/$host/HOST-PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST-PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST-PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST-PORT" ) {
      print "mkdir          : $wrkdir/$host/HOST-PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST-PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST-PORT: $!";
      LoadDataModule::touch();
    }

    if ( !-d "$wrkdir/$host/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$host/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE-CACHE" ) {
      print "mkdir          : $webdir/$host/NODE-CACHE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-l "$wrkdir/$host/RG" ) {
      print "ln -s          : $wrkdir/$host/RANK $wrkdir/$host/RG\n" if $DEBUG;
      symlink( "$wrkdir/$host/RANK", "$wrkdir/$host/RG" ) || die "$act_time: Cannot ln -s $wrkdir/$host/RG: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/DRIVE" ) {
      print "mkdir          : $wrkdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/DRIVE" ) {
      print "mkdir          : $webdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$webdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/CPU-CORE" ) {
      print "mkdir          : $webdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$webdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^VSPG$/)
  if ( $st_type =~ m/^NETAPP$/ ) {
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/DRIVE" ) {
      print "mkdir          : $wrkdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/DRIVE" ) {
      print "mkdir          : $webdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$webdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/POOL" ) {
      print "mkdir          : $webdir/$host/POOL\n" if $DEBUG;
      mkdir( "$webdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/POOL" ) {
      print "mkdir          : $wrkdir/$host/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/LUN" ) {
      print "mkdir          : $webdir/$host/LUN\n" if $DEBUG;
      mkdir( "$webdir/$host/LUN", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/LUN: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/LUN" ) {
      print "mkdir          : $wrkdir/$host/LUN\n" if $DEBUG;
      mkdir( "$wrkdir/$host/LUN", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/LUN: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/CPU-CORE" ) {
      print "mkdir          : $webdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$webdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE" ) {
      print "mkdir          : $webdir/$host/NODE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NODE" ) {
      print "mkdir          : $wrkdir/$host/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NAS" ) {
      print "mkdir          : $wrkdir/$host/NAS\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NAS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NAS: $!";
    }
  } ## end if ( $st_type =~ m/^NETAPP$/)

  if ( $st_type =~ m/^DS5K$/ ) {
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^DS5K$/)
  if ( $st_type =~ m/^VNX$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^VNX$/)
  if ( $st_type =~ m/^VNXE$/ || $st_type =~ m/^UNITY$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/FS" ) {
      print "mkdir          : $wrkdir/$storage/FS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/FS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/FS: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NAS" ) {
      print "mkdir          : $wrkdir/$storage/NAS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NAS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NAS: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NASPOOL" ) {
      print "mkdir          : $wrkdir/$storage/NASPOOL\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NASPOOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NASPOOL: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^VNXE$/...)
  if ( $st_type =~ m/^VMAX$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/TIER" ) {
      print "mkdir          : $wrkdir/$storage/TIER\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/TIER", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/TIER: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$storage/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/FAST-SG" ) {
      print "mkdir          : $wrkdir/$storage/FAST-SG\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/FAST-SG", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/FAST-SG: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/SLO" ) {
      print "mkdir          : $wrkdir/$storage/SLO\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/SLO", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/SLO: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^VMAX$/)
  if ( $st_type =~ m/^VPLEX$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^VPLEX$/)
  if ( $st_type =~ m/^SWIZ$/ ) {
    if ( !-d "$wrkdir/$host/DRIVE" ) {
      print "mkdir          : $wrkdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/CPU-CORE" ) {
      print "mkdir          : $webdir/$host/CPU-CORE\n" if $DEBUG;
      mkdir( "$webdir/$host/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/CPU-NODE" ) {
      print "mkdir          : $wrkdir/$host/CPU-NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/CPU-NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/CPU-NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$host/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-l "$wrkdir/$host/MDISK" ) {
      print "ln -s          : $wrkdir/$host/RANK $wrkdir/$host/MDISK\n" if $DEBUG;
      symlink( "$wrkdir/$host/RANK", "$wrkdir/$host/MDISK" ) || die "$act_time: Cannot ln -s $wrkdir/$host/MDISK: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/DRIVE" ) {
      print "mkdir          : $webdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$webdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/CPU-NODE" ) {
      print "mkdir          : $webdir/$host/CPU-NODE\n" if $DEBUG;
      mkdir( "$webdir/$host/CPU-NODE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/CPU-NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE-CACHE" ) {
      print "mkdir          : $webdir/$host/NODE-CACHE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE-CACHE: $!";
      LoadDataModule::touch();
    }
    if ( !-l "$webdir/$host/MDISK" ) {
      print "ln -s          : $webdir/$host/RANK $webdir/$host/MDISK\n" if $DEBUG;
      symlink( "$webdir/$host/RANK", "$webdir/$host/MDISK" ) || die "$act_time: Cannot ln -s $webdir/$host/MDISK: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^SWIZ$/)
  if ( $st_type =~ m/^DOTHILL$/ ) {
    if ( !-d "$webdir/$host/RANK" ) {
      print "mkdir          : $webdir/$host/RANK\n" if $DEBUG;
      mkdir( "$webdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/RANK: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/RANK" ) {
      print "mkdir          : $wrkdir/$host/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/RANK: $!";
      LoadDataModule::touch();
    }
    if ( !-l "$wrkdir/$host/RG" ) {
      print "ln -s          : $wrkdir/$host/RANK $wrkdir/$host/RG\n" if $DEBUG;
      symlink( "$wrkdir/$host/RANK", "$wrkdir/$host/RG" ) || die "$act_time: Cannot ln -s $wrkdir/$host/RG: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/VOLUME" ) {
      print "mkdir          : $wrkdir/$host/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/VOLUME" ) {
      print "mkdir          : $webdir/$host/VOLUME\n" if $DEBUG;
      mkdir( "$webdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/POOL" ) {
      print "mkdir          : $wrkdir/$host/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/POOL" ) {
      print "mkdir          : $webdir/$host/POOL\n" if $DEBUG;
      mkdir( "$webdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/DRIVE" ) {
      print "mkdir          : $wrkdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/DRIVE" ) {
      print "mkdir          : $webdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$webdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NODE" ) {
      print "mkdir          : $wrkdir/$host/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE" ) {
      print "mkdir          : $webdir/$host/NODE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^DOTHILL$/)
  if ( $st_type =~ m/^OCEANSTOR$/ ) {
    if ( !-d "$wrkdir/$host/VOLUME" ) {
      print "mkdir          : $wrkdir/$host/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/VOLUME" ) {
      print "mkdir          : $webdir/$host/VOLUME\n" if $DEBUG;
      mkdir( "$webdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/POOL" ) {
      print "mkdir          : $wrkdir/$host/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/POOL" ) {
      print "mkdir          : $webdir/$host/POOL\n" if $DEBUG;
      mkdir( "$webdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/DRIVE" ) {
      print "mkdir          : $wrkdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/DRIVE" ) {
      print "mkdir          : $webdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$webdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NODE" ) {
      print "mkdir          : $wrkdir/$host/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE" ) {
      print "mkdir          : $webdir/$host/NODE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/FS" ) {
      print "mkdir          : $wrkdir/$host/FS\n" if $DEBUG;
      mkdir( "$wrkdir/$host/FS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/FS: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/FS" ) {
      print "mkdir          : $webdir/$host/FS\n" if $DEBUG;
      mkdir( "$webdir/$host/FS", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/FS: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^OCEANSTOR$/)
  if ( $st_type =~ m/^COMPELLENT$/ ) {
    if ( !-d "$wrkdir/$host/VOLUME" ) {
      print "mkdir          : $wrkdir/$host/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/VOLUME" ) {
      print "mkdir          : $webdir/$host/VOLUME\n" if $DEBUG;
      mkdir( "$webdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/POOL" ) {
      print "mkdir          : $wrkdir/$host/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/VOLUME-FOLDER" ) {
      print "mkdir          : $wrkdir/$host/VOLUME-FOLDER\n" if $DEBUG;
      mkdir( "$wrkdir/$host/VOLUME-FOLDER", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/VOLUME-FOLDER: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/VOLUME-FOLDER" ) {
      print "mkdir          : $webdir/$host/VOLUME-FOLDER\n" if $DEBUG;
      mkdir( "$webdir/$host/VOLUME-FOLDER", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/VOLUME-FOLDER: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/POOL" ) {
      print "mkdir          : $webdir/$host/POOL\n" if $DEBUG;
      mkdir( "$webdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/PORT" ) {
      print "mkdir          : $wrkdir/$host/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/DRIVE" ) {
      print "mkdir          : $wrkdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/DRIVE" ) {
      print "mkdir          : $webdir/$host/DRIVE\n" if $DEBUG;
      mkdir( "$webdir/$host/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/NODE" ) {
      print "mkdir          : $wrkdir/$host/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/NODE" ) {
      print "mkdir          : $webdir/$host/NODE\n" if $DEBUG;
      mkdir( "$webdir/$host/NODE", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/PORT" ) {
      print "mkdir          : $webdir/$host/PORT\n" if $DEBUG;
      mkdir( "$webdir/$host/PORT", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/PORT: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^COMPELLENT$/)
  if ( $st_type =~ m/^PURE$/ ) {
    if ( !-d "$wrkdir/$host/VOLUME" ) {
      print "mkdir          : $wrkdir/$host/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/VOLUME" ) {
      print "mkdir          : $webdir/$host/VOLUME\n" if $DEBUG;
      mkdir( "$webdir/$host/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/POOL" ) {
      print "mkdir          : $wrkdir/$host/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/POOL" ) {
      print "mkdir          : $webdir/$host/POOL\n" if $DEBUG;
      mkdir( "$webdir/$host/POOL", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST" ) {
      print "mkdir          : $wrkdir/$host/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST" ) {
      print "mkdir          : $webdir/$host/HOST\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$host/HOST-GROUP" ) {
      print "mkdir          : $wrkdir/$host/HOST-GROUP\n" if $DEBUG;
      mkdir( "$wrkdir/$host/HOST-GROUP", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/HOST-GROUP: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$host/HOST-GROUP" ) {
      print "mkdir          : $webdir/$host/HOST-GROUP\n" if $DEBUG;
      mkdir( "$webdir/$host/HOST-GROUP", 0755 ) || die "$act_time: Cannot mkdir $webdir/$host/HOST-GROUP: $!";
      LoadDataModule::touch();
    }
    #if ( !-d "$wrkdir/$host/RANK" ) {
    #  print "mkdir          : $wrkdir/$host/RANK\n" if $DEBUG;
    #  mkdir( "$wrkdir/$host/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$host/RANK: $!";
    #}
    #if ( !-l "$wrkdir/$host/RG" ) {
    #  print "ln -s          : $wrkdir/$host/RANK $wrkdir/$host/RG\n" if $DEBUG;
    #  symlink( "$wrkdir/$host/RANK", "$wrkdir/$host/RG" ) || die "$act_time: Cannot ln -s $wrkdir/$host/RG: $!";
    #  LoadDataModule::touch();
    #}
  } ## end if ( $st_type =~ m/^PURE$/)
  
  if ( !-d "$tmp_dir/$host" ) {
    print "mkdir          : $tmp_dir/$host\n" if $DEBUG;
    mkdir( "$tmp_dir/$host", 0755 ) || die "$act_time: Cannot mkdir $tmp_dir/$host: $!";
    LoadDataModule::touch();
  }

  # be carefull, all those directories are also created in bin/data_load.pl

  return 0;
} ## end sub create_dir_structure

sub draw_all_pool_controler {
  my $host    = shift;
  my $type    = shift;
  my $item    = shift;
  my $st_type = shift;

  draw_graph_pool_controler( "day",     "d", "MINUTE:60:HOUR:2:HOUR:4:0:%H", $host, $type, $item, $st_type );
  draw_graph_pool_controler( "week",    "w", "HOUR:8:DAY:1:DAY:1:86400:%a",  $host, $type, $item, $st_type );
  draw_graph_pool_controler( "4 weeks", "m", "DAY:1:DAY:2:DAY:2:0:%d",       $host, $type, $item, $st_type );
  draw_graph_pool_controler( "year",    "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $host, $type, $item, $st_type );
  return 0;
} ## end sub draw_all_pool_controler

sub draw_graph_pool_controler {
  my $text       = shift;
  my $type_gr    = shift;
  my $xgrid      = shift;
  my $host       = shift;
  my $type       = shift;
  my $item       = shift;
  my $st_type    = shift;
  my $name       = "$tmp_dir/$host-$type-$item-agg-$type_gr";
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $last       = "COMMENT: ";
  my $rrd_time   = "";
  my $step_new   = $STEP;
  my $once       = 0;
  my $last_time  = "na";
  my $color_indx = 0;                                           # reset colour index
  my $item_org   = $item;

  my $cntl = 0;                                                 # controler A
  if ( $item =~ m/^B/ ) {
    $cntl = 1;                                                  # controler B
  }

  my $tmp_file  = "$tmp_dir/$host/$type-$item-$type_gr.cmd";
  my $act_utime = time();

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "y" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $YEAR_REFRESH ) {

      #print "creating cmd   : $host:$:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "y" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "m" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $MONTH_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "m" &&...)

  # do not update charts if there is not new data in RRD DB
  if ( $type_gr =~ "w" && -f "$tmp_file" ) {
    my $png_time = ( stat("$tmp_file") )[9];
    if ( ( $act_utime - $png_time ) < $WEEK_REFRESH ) {

      #print "creating cmd   : $host:$type:$lpar:$type_gr:$type no update\n" if $DEBUG ;
      if ( $upgrade == 0 ) { return 0; }
    }
  } ## end if ( $type_gr =~ "w" &&...)

  print "creating cmd   : $host:$type:$item:$type_gr\n" if $DEBUG;

  my $req_time = "";
  my $header   = "$type aggregated $item: last $text";
  if ( $item =~ m/resp_t/ ) {
    $header = "$type $item: last $text";
  }
  my $i    = 0;
  my $lpar = "";
  my $cmd  = "";
  my $j    = 0;

  if ( "$type_gr" =~ "d" ) {
    $req_time = $act_utime - 86400;
  }
  if ( "$type_gr" =~ "w" ) {
    $req_time = $act_utime - 604800;
  }
  if ( "$type_gr" =~ "m" ) {
    $req_time = $act_utime - 2764800;
  }
  if ( "$type_gr" =~ "y" ) {
    $req_time = $act_utime - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type_gr";
  $cmd .= " --end now-1$type_gr+1$type_gr";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --interlaced";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $font_def_normal";
  $cmd .= " $font_tit_normal";

  my $value_short = "";
  my $value_long  = "";
  my $val         = 1;
  $item =~ s/_b$//;    # cut back-end info for DS8k, just cmd file will contain the backend info
  my $item_real = $item;

  # do not use switch statement
  if ( $item =~ m/^data_cntl$/ )   { $item_real = "data_rate"; $value_short = "MB/sec"; $value_long = "MBytes/sec";   $val = 1024; }
  if ( $item =~ m/^io_cntl$/ )     { $item_real = "io_rate";   $value_short = "IOPS";   $value_long = "IO per second"; }
  if ( $item =~ m/^resp_t_cntl$/ ) { $item_real = "resp_t";    $value_short = "ms";     $value_long = "milliseconds"; $val = 1; }

  $cmd .= " --vertical-label=\\\"$item in $value_long\\\"";

  $cmd .= " --units-exponent=1.00";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";

  # add spaces to lpar name (for formating graph legend)
  my $legend = sprintf( "%-20s", "$item [$value_short]" );    # -20s is OK here, here is fixed length of items!!

  my $legend_heading = "$item $delimiter [$value_short] $delimiter Avrg $delimiter Max ";
  $cmd .= " COMMENT:\\\"$legend         Avrg      Max\\l\\\"";

  my $gtype = "AREA";
  if ( $item =~ m/resp_t/ ) {
    $gtype = "LINE1";
  }

  my $lpar_list     = "";
  my $pool_found    = 0;
  my $pool_id_tmp   = "";
  my $lpar_list_tmp = "";

  my @rank_list      = "";
  my $rank_list_indx = 0;

  my $searchstr = "-P[0-9]*";
  my $rank_vol  = "RANK";
  if ( $st_type =~ m/3PAR/ ) {
    $rank_vol = "VOLUME";
  }
  if ( $st_type =~ m/XIV/ ) {
    $rank_vol  = "VOLUME";                  # XIV has hexa pool names
    $searchstr = "-P[0-9,a,b,c,d,e,f]*";    # like b5d13d00014-Pb5713800002.rrd
  }
  if ( $st_type =~ m/XIV/ || $st_type =~ m/DS5K/ || $st_type =~ m/HUS/ || $st_type =~ m/NETAPP/ || $st_type =~ m/^VNX$/ || $st_type =~ m/UNITY/ || $st_type =~ m/^VNXE$/ ) {
    $searchstr = "";
    $rank_vol  = "VOLUME";                  # HUS uses volumes
                                            # no no, controller info is only in volume.rrd
  }

  # DS8K front-end stats are created in draw_all
  my $dir = "$wrkdir/$host/$rank_vol";
  chdir $dir;

  # check all ranks and then sort them per pool id
  foreach my $line (<*$searchstr\.rrd>) {
    chomp($line);
    if ( $line !~ m/\.rrd$/ ) {
      next;                                 # it does not end bu .rrd suffix, "$ cannt be use in foreach <>
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$line") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    $rank_list[$rank_list_indx] = $line;
    $rank_list_indx++;
  } ## end foreach my $line (<*$searchstr\.rrd>)

  if ( $rank_list_indx == 0 ) {

    # no new data found, skipping ...
    print "creating cmd   : $host:$type:$item_org:$type_gr no new data found, skipping ...\n" if $DEBUG;
    return 0;
  } ## end if ( $rank_list_indx ==...)

  my @rank_list_unsorted = @rank_list;
  if ( $rank_list[0] =~ m/-/ && $st_type !~ m/NETAPP/ ) {

    # netapp 3857b20f-2fc7-4bbf-a023-5b040cf17099.rrd
    @rank_list = sort { ( split '-', $a )[1] cmp( split '-', $b )[1] } @rank_list_unsorted;
  }
  else {
    @rank_list = sort @rank_list_unsorted;
  }

  my $pool_id_old = -1;

  my $print_now         = 0;
  my $index_last        = 0;
  my $rrd_file_last     = "";
  my $index_actual      = 0;
  my $pool_name_space   = "NA";
  my $pool_id           = 0;
  my $pool_name_actual  = "NA";
  my $itemm             = "";
  my $no_of_ranks       = 0;
  my $print_now_pool_id = 0;

  # go through all ranks sorted per pool id, once a pool id is changed then is printed the old one
  foreach my $line (@rank_list) {
    chomp($line);
    $pool_id = $line;
    $pool_id =~ s/^.*-P//;
    $pool_id =~ s/\.rrd$//;

    my $ret = ishexa($pool_id);
    if ( $ret == 0 && $st_type ne 'UNITY' && $st_type ne 'VNXE' ) {
      next;
    }

    if ( ishexa($pool_id_old) && $pool_id_old =~ m/^-1$/ ) {

      # could be already haxa here
      $pool_id_old = $pool_id;
    }
    else {
      if ( $pool_id_old !~ m/^$pool_id$/ ) {
        $print_now         = 1;
        $print_now_pool_id = $pool_id_old;
        $pool_name_actual  = $pool_name_space;
        $pool_id_old       = $pool_id;
      } ## end if ( $pool_id_old !~ m/^$pool_id$/)
    } ## end else [ if ( ishexa($pool_id_old...))]

    # get pool name, be aware this is a new pool name
    #print "002 $line\n";
    my $pool_name = "NA";
    $pool_name = get_pool_name($pool_id);

    my $rrd_file = "$dir/$line";
    $rrd_file_last = "$dir/$line";
    if ( "$type_gr" =~ "d" ) {

      # find out time stamp of last data update
      # take just one volume to keep it simple
      RRDp::cmd qq(last "$rrd_file");
      my $last_tt = RRDp::read;
      chomp($$last_tt);
      $last_time = localtime($$last_tt);
      $last_time =~ s/:/\\:/g;
    } ## end if ( "$type_gr" =~ "d")

    my $time_first = find_real_data_start_pool( "$dir", $pool_id, $st_type );

    $itemm = $item . "m";
    $itemm =~ s/\./Z/g;    # dots in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/ /X/g;     # space in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/\#/Y/g;    # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/:/R/g;     # hash in rrdtool variable names causing problems for rrdtool parser
    $itemm =~ s/-/S/g;     # dash in rrdtool variable names causing problems for rrdtool parser (some old versions)
    $line =~ s/:/\\:/g;    # in case there is ":" in the rrd file name, RRDTool then has a problem with parsing

    $cmd .= " DEF:$item${i}_u_nan=\\\"$dir/$line\\\":$item_real:AVERAGE";
    $cmd .= " DEF:$item${i}_cntl_nan=\\\"$dir/$line\\\":controller:AVERAGE";
    $cmd .= " CDEF:$item${i}_u=$item${i}_u_nan,UN,0,$item${i}_u_nan,IF";
    $cmd .= " CDEF:$item${i}_cntl=$item${i}_cntl_nan,UN,2,$item${i}_cntl_nan,IF";
    $cmd .= " CDEF:$item${i}_val=$item${i}_u,$val,/";
    $cmd .= " CDEF:$item${i}_a=$item${i}_cntl,0,EQ,$item${i}_val,0,IF";
    $cmd .= " CDEF:$item${i}_b=$item${i}_cntl,1,EQ,$item${i}_val,0,IF";
    $cmd .= " CDEF:$itemm${i}_a=TIME,$time_first,LT,UNKN,$item${i}_a,IF";
    $cmd .= " CDEF:$itemm${i}_b=TIME,$time_first,LT,UNKN,$item${i}_b,IF";

    # UNKN must be there otherwise: ERROR: RPN stack underflow

    if ( $i > 0 ) {
      my $j = $i;
      $j--;
      $cmd .= " CDEF:cntlA${i}=cntlA${j},$itemm${i}_a,+";
      $cmd .= " CDEF:cntlB${i}=cntlB${j},$itemm${i}_b,+";
    } ## end if ( $i > 0 )
    else {
      # first run
      $cmd .= " CDEF:cntlA${i}=$itemm${i}_a,0,+";
      $cmd .= " CDEF:cntlB${i}=$itemm${i}_b,0,+";
    }

    $no_of_ranks++;
    $i++;
  } ## end foreach my $line (@rank_list)
  $i--;

  if ( $item =~ m/resp_t/ ) {

    # response time is not summary, must be average of all time (devide number of ranks)
    # --PH: resp time is a bit complicated, not supported for now
    $cmd .= ",$no_of_ranks,/";
  } ## end if ( $item =~ m/resp_t/)

  # print Controller A
  my $rrd_file_conv = " ";    # switch off clickable links in the POOL Controller legend, there is no reason for them

  if ( $st_type =~ m/^VNX$/ || $st_type =~ m/^UNITY$/ || $st_type =~ m/^VNXE$/ ) {
    $pool_name_space = sprintf( "%-20s", "SP A" );    # -20s is OK here, here is fixed length of items!!
  }
  else {
    $pool_name_space = sprintf( "%-20s", "Controller A" );    # -20s is OK here, here is fixed length of items!!
  }

  $cmd .= " $gtype:cntlA${i}$color[$color_indx]:\\\"$pool_name_space\\\"";
  if ( $item =~ m/resp_t/ ) {
    $cmd .= " PRINT:cntlA${i}:AVERAGE:\\\"%7.1lf $delimiter $pool_name_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
    $cmd .= " PRINT:cntlA${i}:MAX:\\\" %7.1lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";

    $cmd .= " GPRINT:cntlA${i}:AVERAGE:\\\"%7.1lf \\\"";
    $cmd .= " GPRINT:cntlA${i}:MAX:\\\"%7.1lf \\l\\\"";
  } ## end if ( $item =~ m/resp_t/)
  else {
    $cmd .= " PRINT:cntlA${i}:AVERAGE:\\\"%7.0lf $delimiter $pool_name_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
    $cmd .= " PRINT:cntlA${i}:MAX:\\\" %7.0lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";

    $cmd .= " GPRINT:cntlA${i}:AVERAGE:\\\"%7.0lf \\\"";
    $cmd .= " GPRINT:cntlA${i}:MAX:\\\"%7.0lf \\l\\\"";
  } ## end else [ if ( $item =~ m/resp_t/)]

  # print Controller B
  if ( $st_type =~ m/^VNX$/ || $st_type =~ m/^UNITY$/ || $st_type =~ m/^VNXE$/ ) {
    $pool_name_space = sprintf( "%-20s", "SP B" );    # -20s is OK here, here is fixed length of items!!
  }
  else {
    $pool_name_space = sprintf( "%-20s", "Controller B" );    # -20s is OK here, here is fixed length of items!!
  }

  $color_indx++;
  $gtype = "STACK";
  $cmd .= " $gtype:cntlB${i}$color[$color_indx]:\\\"$pool_name_space\\\"";
  if ( $item =~ m/resp_t/ ) {
    $cmd .= " PRINT:cntlB${i}:AVERAGE:\\\"%7.1lf $delimiter $pool_name_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
    $cmd .= " PRINT:cntlB${i}:MAX:\\\" %7.1lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";

    $cmd .= " GPRINT:cntlB${i}:AVERAGE:\\\"%7.1lf \\\"";
    $cmd .= " GPRINT:cntlB${i}:MAX:\\\"%7.1lf \\l\\\"";
  } ## end if ( $item =~ m/resp_t/)
  else {
    $cmd .= " PRINT:cntlB${i}:AVERAGE:\\\"%7.0lf $delimiter $pool_name_space $delimiter $legend_heading $delimiter $color[$color_indx]\\\"";
    $cmd .= " PRINT:cntlB${i}:MAX:\\\" %7.0lf $delimiter $rrd_file_conv $delimiter $st_type\\\"";

    $cmd .= " GPRINT:cntlB${i}:AVERAGE:\\\"%7.0lf \\\"";
    $cmd .= " GPRINT:cntlB${i}:MAX:\\\"%7.0lf \\l\\\"";
  } ## end else [ if ( $item =~ m/resp_t/)]

  if ( $type_gr =~ m/d/ ) {

    # last update timestamp
    $cmd .= " COMMENT:\\\"Updated\\\: $last_time\\\"";
  }

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # It must be here!!!, touch() only in case there is any rrdfile
  if ( !-f "$tmp_file" ) {
    LoadDataModule::touch();
  }

  my $FH;
  open( FH, "> $tmp_file" ) || error( "Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  #$cmd =~ s/ /\n/g;
  #print  "00 $cmd\n";

  if ( $not_execute_rrdtool == 1 ) { return 1; }

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < $tmp_file 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    if ( $ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:$type:$item:$type_gr: Multi graph rrdtool error : $ret " . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( $ret =~ "ERROR" )

  # Do not remove it!! It is used for detail pop-ups!!
  #unlink ("$tmp_file");

  return 0;
} ## end sub draw_graph_pool_controler

sub print_storage_sys {
  my $wrkdir = shift;
  my $host   = shift;

  # print storage sys info, firmwares etc
  my $file = "$wrkdir/$host/config_sys_storage.txt";

  if ( !-f $file ) {
    return 0;
  }
  open( FHCFG, "< $file" ) || error( "Can't open $file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FHCFG>;
  close(FHCFG);

  foreach my $line (@lines) {
    print "Storage sys inf: $host:$line";
  }
  return 0;
} ## end sub print_storage_sys

sub hosts_top {

  my $storage      = shift;
  my $type         = shift;
  my $name         = shift;
  my $item         = shift;
  my $time         = shift;
  my $st_type      = shift;
  my $webdir       = shift;
  my $act_time     = shift;
  my $in_file      = "$tmp_dir/$host/VOLUME-$name-$time.cmd";
  my $file_cmd_max = "/var/tmp/stor2rrd-VOLUME-$name-max-$time.cmd";
  my $out_file     = "$webdir/glob-$type-$name-$item-$time.html-tmp";
  my $host_cfg     = "$basedir/data/$storage/HOST/hosts.cfg";
  my $vol_cfg      = "$basedir/data/$storage/VOLUME/volumes.cfg";
  my @lines        = ();
  my $source_data  = 0;

  ### check if exists HOST TOP file (VSPG and 3PAR)
  if ( -f "$tmp_dir/$host/$type-$name-$time.cmd" ) {
    $in_file      = "$tmp_dir/$host/$type-$name-$time.cmd";
    $file_cmd_max = "/var/tmp/stor2rrd-$type-$name-max-$time.cmd";
    if ( $st_type eq "VSPG" && host_data_selection( $wrkdir, $storage, $type, $st_type ) == 0 ) {
      $source_data  = 0;
      $in_file      = "$tmp_dir/$host/VOLUME-$name-$time.cmd";
      $file_cmd_max = "/var/tmp/stor2rrd-VOLUME-$name-max-$time.cmd";
    }
    else {
      $source_data = 1;
    }
  } ## end if ( -f "$tmp_dir/$host/$type-$name-$time.cmd")

  my %inventory_host;
  my %inventory_volume;

  #"read","write","read_io","write_io","resp_t_r","resp_t_w","r_cache_hit","w_cache_hit";
  #DS5K-v1:"io_rate","data_rate","cache_hit";
  #DS5K-v2:"io_rate","data_rate","resp_t","r_cache_hit","w_cache_hit","ssd_r_cache_hit";
  #VNX:"read","write","read_io","write_io"

  #set up global inventory volume

  if ( !-f $in_file ) {
    error( "$host:VOLUME:top:$item - no cmd file:$in_file " . __FILE__ . ":" . __LINE__ ) if ($DEBUG);
    return 0;
  }

  my $width = 288;    # to get 5 minute per a pixel
  if ( $time =~ m/^w/ ) {
    $width = 1728;    # to get 5 minute per a pixel, 6d
  }
  if ( $time =~ m/^m/ ) {
    $width = 6912;    # to get 5 minute per a pixel, 24d
  }
  if ( $time =~ m/^y/ ) {
    $width = 97056;    # to get 5 minute per a pixel, 337d
  }

  if ( $item =~ m/max/ ) {

    # For printing peaks (MAX)
    open( FHR, "< $in_file" ) || error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @lines = <FHR>;
    close(FHR);
    open( FHW, "> $file_cmd_max" ) || error( "Can't open $file_cmd_max : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

    #binmode(FHW);

    foreach my $line (@lines) {
      $line =~ s/:AVERAGE:/:MAX:/g;
      $line =~ s/ --width=.* --height=/ --width=$width --height=/;
      $line =~ s/ --step=.* --units-exponent=/ --step=300 --units-exponent=/;    # just to be sure ...
      print FHW "$line";
    } ## end foreach my $line (@lines)
    close(FHW);
    $in_file = $file_cmd_max;
  } ## end if ( $item =~ m/max/ )
  open( FH, "< $in_file" ) || error( "VOLUME: Can't open $in_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  @lines = <FH>;
  close(FH);
  my $cmd = "";
  foreach my $line (@lines) {
    $cmd = $line;
    last;
  }
  if ( $cmd eq '' ) {
    error( "$host : Multi graph rrdtool error $cmd " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $ret = "";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "RRDp::read $@ " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( !defined($$ret) ) {
    return 0;    # old unused volumes??, it must be here as it caused a problem
  }
  if ( $$ret =~ "ERROR" ) {
    error( "$host : Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( $item =~ m/max/ ) {
    unlink($in_file);    # clean only temp files in /var/tmp
  }
  if ( $$ret =~ "ERROR" ) {
    if ( $$ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:VOLUME:$name:$item: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:VOLUME:$name:$item: Multi graph rrdtool error : $$ret " . __FILE__ . ":" . __LINE__ );
    }
    return 0;
  } ## end if ( $$ret =~ "ERROR" )

  my $null        = "-";
  my $data_rate   = 0;
  my $io_rate     = 0;
  my $volume_prev = "";
  (@lines) = split( /====start /, $$ret );
  my $item_no = 0;
  my $count   = 0;
  foreach my $line (@lines) {
    chomp($line);
    if ( $line !~ m/^==== / ) {
      next;
    }

    $line =~ s/^==== / /g;
    $line =~ s/^ *//g;
    ( my $volume, my $item, my $value ) = split( / +/, $line );
    $value =~ s/OK.*//g;    #last one
    $value =~ s/ //g;
    chomp($value);          # last one record has en enter at the end
    $volume =~ s/=====space=====/ /g;

    #print "001 --$line: $volume:$item:$value \n";
    if ( !defined($value) || $value eq '' || $volume =~ m/^$/ ) {
      next;
    }
    ### HOST TOP
    if ( $source_data == 0 ) {
      $inventory_volume{STORAGE}{$storage}{$volume}{$item} = $value;
    }
    if ( $source_data == 1 ) {
      $inventory_host{STORAGE}{$storage}{$volume}{ITEM}{$item} = $value;
    }
  } ## end foreach my $line (@lines)
  ### get inventory host
  if ( !( -f $host_cfg ) && $source_data == 0 ) {
    error( "$storage FILE $host_cfg does not exist : $! " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  if ( -f $host_cfg ) {
    open( FHR, "< $host_cfg" ) || error( "Can't open $host_cfg : $! " . __FILE__ . ":" . __LINE__ ) && return "";
    @lines = <FHR>;
    close(FHR);
  }

  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" ) { next; }
    my ( $host_name, $volumes ) = split( " : ", $line );
    if ( !defined $inventory_host{STORAGE}{$storage}{$host_name} && $source_data == 1 ) {
      delete $inventory_host{STORAGE}{$storage}{$host_name};    ### host data and host conf file si not consistent
      next;
    }
    my @volumes_ids = split( " ", $volumes );
    foreach my $id (@volumes_ids) {
      $inventory_host{STORAGE}{$storage}{$host_name}{VOLUMES}{$id}{ID} = $id;
    }
  } ## end foreach my $line (@lines)

  ### get volume name, id
  if ( !-f $vol_cfg ) {
    error( "$storage FILE $vol_cfg does not exist : $! " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  open( FHR, "< $vol_cfg" ) || error( "Can't open $vol_cfg : $! " . __FILE__ . ":" . __LINE__ ) && return "";
  @lines = <FHR>;
  close(FHR);

  foreach my $line (@lines) {
    chomp $line;
    my @check_name_pool = ();
    my $active          = 0;
    if ( $line eq "" ) { next; }
    my ( $volume_name, $ids ) = split( " : ", $line );
    my @volumes_ids = split( ";", $ids );
    foreach my $id (@volumes_ids) {
      foreach my $vol_c (@check_name_pool) {
        if ( $vol_c eq $volume_name ) {
          $active = 1;
          last;
        }
      } ## end foreach my $vol_c (@check_name_pool)
      if ( $st_type eq "SWIZ" ) {
        $id =~ s/^0*//;
        if ( $id eq "" ) {
          $id = 0;
        }
      } ## end if ( $st_type eq "SWIZ")
      if ( $st_type eq "DS8K" ) {
        $id =~ s/^0x//;
      }
      foreach my $host_name ( keys %{ $inventory_host{STORAGE}{$storage} } ) {
        if ( defined $inventory_host{STORAGE}{$storage}{$host_name}{VOLUMES}{$id} ) {
          if ( $active == 1 ) { last; }
          push( @check_name_pool, $volume_name );
          $inventory_host{STORAGE}{$storage}{$host_name}{VOLUMES}{$id}{NAME} = $volume_name;
        }
      } ## end foreach my $host_name ( keys...)
    } ## end foreach my $id (@volumes_ids)
  } ## end foreach my $line (@lines)
  my @items = ( "r_cache_hit", "w_cache_hit", "resp_t", "resp_t_r", "resp_t_w", "io_rate", "read_io", "write_io", "data_rate", "read", "write" );

  if ( $source_data == 0 ) {

    foreach my $host ( keys %{ $inventory_host{STORAGE}{$storage} } ) {
      foreach my $volume ( keys %{ $inventory_host{STORAGE}{$storage}{$host}{VOLUMES} } ) {
        if ( !defined $inventory_host{STORAGE}{$storage}{$host}{VOLUMES}{$volume}{NAME} ) {
          next;
        }
        my $name_vol = $inventory_host{STORAGE}{$storage}{$host}{VOLUMES}{$volume}{NAME};
        if ( !defined $inventory_volume{STORAGE}{$storage}{$name_vol} ) { delete $inventory_volume{STORAGE}{$storage}{$name_vol}; next; }
        foreach my $sort_item (@items) {
          foreach my $item ( keys %{ $inventory_volume{STORAGE}{$storage}{$name_vol} } ) {
            if ( $item ne $sort_item ) { next; }
            if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} && isdigit( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) ) {
              ### calculate resp_t and cache_hit
              if ( $item eq "resp_t_w" ) {
                my $io_rate_sum = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"write_io"};
                my $resp_t_sum  = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
                my $io_rate     = $inventory_volume{STORAGE}{$storage}{$name_vol}{"write_io"};
                my $resp_t      = $inventory_volume{STORAGE}{$storage}{$name_vol}{$item};
                if ( !isdigit($io_rate_sum) || !isdigit($io_rate) || !isdigit($resp_t) ) {next}
                my $check_denominator = $io_rate_sum + $io_rate;
                if ( $check_denominator == 0 ) { next; }
                my $value = ( ( $io_rate_sum * $resp_t_sum ) + ( $resp_t * $io_rate ) ) / ( $io_rate_sum + $io_rate );
                $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} = round( $value, 1 );
                next;
              } ## end if ( $item eq "resp_t_w")
              if ( $item eq "resp_t" ) {
                my $io_rate_sum = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"io_rate"};
                if ( !defined $io_rate_sum ) {
                  if ( isdigit( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"write_io"} ) ) {
                    $io_rate_sum = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"write_io"};
                  }
                  if ( isdigit( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"read_io"} ) ) {
                    if ( isdigit($io_rate_sum) ) {
                      $io_rate_sum = $io_rate_sum + $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"read_io"};
                    }
                  }
                } ## end if ( !defined $io_rate_sum)
                my $resp_t_sum = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
                my $io_rate    = $inventory_volume{STORAGE}{$storage}{$name_vol}{"io_rate"};
                if ( !defined $io_rate ) {
                  if ( isdigit( $inventory_volume{STORAGE}{$storage}{$name_vol}{"write_io"} ) ) {
                    $io_rate = $inventory_volume{STORAGE}{$storage}{$name_vol}{"write_io"};
                  }
                  if ( isdigit( $inventory_volume{STORAGE}{$storage}{$name_vol}{"read_io"} ) ) {
                    if ( isdigit($io_rate) ) {
                      $io_rate = $io_rate + $inventory_volume{STORAGE}{$storage}{$name_vol}{"read_io"};
                    }
                  }
                } ## end if ( !defined $io_rate)
                my $resp_t = $inventory_volume{STORAGE}{$storage}{$name_vol}{$item};
                if ( !isdigit($io_rate_sum) || !isdigit($io_rate) || !isdigit($resp_t) ) {next}
                my $check_denominator = $io_rate_sum + $io_rate;
                if ( $check_denominator == 0 ) { next; }
                my $value = ( ( $io_rate_sum * $resp_t_sum ) + ( $resp_t * $io_rate ) ) / ( $io_rate_sum + $io_rate );
                $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} = round( $value, 1 );
                next;
              } ## end if ( $item eq "resp_t")

              if ( $item eq "resp_t_r" ) {
                my $io_rate_sum = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"read_io"};
                my $resp_t_sum  = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
                my $io_rate     = $inventory_volume{STORAGE}{$storage}{$name_vol}{"read_io"};
                my $resp_t      = $inventory_volume{STORAGE}{$storage}{$name_vol}{$item};
                if ( !isdigit($io_rate_sum) || !isdigit($io_rate) || !isdigit($resp_t) ) {next}
                my $check_denominator = $io_rate_sum + $io_rate;
                if ( $check_denominator == 0 ) {
                  next;
                }
                my $value = ( ( $io_rate_sum * $resp_t_sum ) + ( $resp_t * $io_rate ) ) / ( $io_rate_sum + $io_rate );
                $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} = round( $value, 1 );
                next;
              } ## end if ( $item eq "resp_t_r")

              if ( $item eq "w_cache_hit" ) {
                my $io_rate_sum = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"write_io"};
                my $resp_t_sum  = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
                my $io_rate     = $inventory_volume{STORAGE}{$storage}{$name_vol}{"write_io"};
                my $resp_t      = $inventory_volume{STORAGE}{$storage}{$name_vol}{$item};
                if ( !isdigit($io_rate_sum) || !isdigit($io_rate) || !isdigit($resp_t) ) {next}
                my $check_denominator = $io_rate_sum + $io_rate;
                if ( $check_denominator == 0 ) { next; }
                my $value = ( ( $io_rate_sum * $resp_t_sum ) + ( $resp_t * $io_rate ) ) / ( $io_rate_sum + $io_rate );
                $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} = round( $value, 1 );
                next;
              } ## end if ( $item eq "w_cache_hit")

              if ( $item eq "r_cache_hit" ) {
                my $io_rate_sum = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{"read_io"};
                my $resp_t_sum  = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
                my $io_rate     = $inventory_volume{STORAGE}{$storage}{$name_vol}{"read_io"};
                my $resp_t      = $inventory_volume{STORAGE}{$storage}{$name_vol}{$item};
                if ( !isdigit($io_rate_sum) || !isdigit($io_rate) || !isdigit($resp_t) ) {next}
                my $check_denominator = $io_rate_sum + $io_rate;
                if ( $check_denominator == 0 ) { next; }
                my $value = ( ( $io_rate_sum * $resp_t_sum ) + ( $resp_t * $io_rate ) ) / ( $io_rate_sum + $io_rate );
                $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} = round( $value, 1 );
                next;
              } ## end if ( $item eq "r_cache_hit")

              my $value = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
              $value = $value + $inventory_volume{STORAGE}{$storage}{$name_vol}{$item};
              $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} = $value;
            } ## end if ( defined $inventory_host...)
            else {
              $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} = $inventory_volume{STORAGE}{$storage}{$name_vol}{$item};
            }
          } ## end foreach my $item ( keys %{ ...})
        } ## end foreach my $sort_item (@items)
      } ## end foreach my $volume ( keys %...)
    } ## end foreach my $host ( keys %{ ...})
  } ## end if ( $source_data == 0)

  ### save to file

  ### header

  if ( !-f $out_file ) {
    open( FHT, "> $out_file" ) || error( "Can't open $out_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    print FHT "<center>\n";
    if ( $time =~ m/^d/ ) {
      print FHT "last day (created on: $act_time)<br>\n";
    }
    if ( $time =~ m/^w/ ) {
      print FHT "last week without last day (created on: $act_time)<br>\n";
    }
    if ( $time =~ m/^m/ ) {
      print FHT "last month without last week (created on: $act_time)<br>\n";
    }
    print FHT "<table class=\"lparsearch tablesorter\" data-sortby=\"4\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Storage&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;IO&nbsp;rate&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;data&nbsp;rate&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;read&nbsp;&nbsp;<br>&nbsp;<span>MB/sec</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;write&nbsp;&nbsp;<br>&nbsp;<span>MB/sec</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;resp&nbsp;t&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;resp&nbsp;read&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;resp&nbsp;write&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;r&nbsp;cachehit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;w&nbsp;cachehit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
  } ## end if ( !-f $out_file )
  else {
    open( FHT, ">> $out_file" ) || error( "Can't open $out_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  foreach my $host ( keys %{ $inventory_host{STORAGE}{$storage} } ) {
    my $line      = "";
    my $read      = "-";
    my $write     = "-";
    my $data_rate = "-";
    my $io_read   = "-";
    my $io_write  = "-";
    my $io_rate   = "-";
    my $resp_r    = "-";
    my $resp_w    = "-";
    my $resp_t    = "-";
    my $cache_r   = "-";
    my $cache_w   = "-";
    my $volumes   = "";

    ### volumes
    foreach my $volume ( keys %{ $inventory_host{STORAGE}{$storage}{$host}{VOLUMES} } ) {
      if ( defined $inventory_host{STORAGE}{$storage}{$host}{VOLUMES}{$volume}{NAME} ) {
        my $name = $inventory_host{STORAGE}{$storage}{$host}{VOLUMES}{$volume}{NAME};

        if ( $volumes eq "" ) {
          $volumes = "<a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=VOLUME&name=$name&storage=$st_type&gui=1&none=none\">$name<\/a>";
        }
        else {
          my @vol = split( ", ", $volumes );
          my $active = 0;
          foreach my $lun (@vol) {
            $lun =~ s/\<a.*\"\>//;
            $lun =~ s/\<\/a\>//;
            $lun =~ s/^\s+|\s+$//g;
            if ( "$lun" eq "$name" ) {
              $active = 1;
              last;
            }
          } ## end foreach my $lun (@vol)
          if ( $active == 0 ) {
            $volumes = $volumes . ", " . "<a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=VOLUME&name=$name&storage=$st_type&gui=1&none=none\">$name<\/a>";
          }
        } ## end else [ if ( $volumes eq "" ) ]
      } ## end if ( defined $inventory_host...)
    } ## end foreach my $volume ( keys %...)
    my $item = "read";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $read = round( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item}, 1 );
    }

    $item = "write";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $write = round( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item}, 1 );
    }

    $item = "data_rate";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $data_rate = round( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item}, 1 );
    }
    else {
      if ( isdigit($read) ) {
        $data_rate = round( $read, 1 );
      }
      if ( isdigit($write) && $write ne "-" ) {
        if ( isdigit($data_rate) && $data_rate ne "-" ) {
          $data_rate = round( $data_rate + $write, 1 );
        }
        else {
          $data_rate = round( $write, 1 );
        }
      } ## end if ( isdigit($write) &&...)
    } ## end else [ if ( defined $inventory_host...)]

    $item = "read_io";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $io_read = round( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item}, 0 );
    }
    $item = "write_io";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $io_write = round( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item}, 0 );
    }

    $item = "io_rate";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $io_rate = round( $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item}, 0 );
    }
    else {
      if ( isdigit($io_read) ) {
        $io_rate = round( $io_read, 0 );
      }
      if ( isdigit($io_write) ) {
        if ( isdigit($io_rate) && $io_rate ne "-" ) {
          $io_rate = round( $io_rate + $io_write, 0 );
        }
        else {
          $io_rate = round( $io_write, 0 );
        }
      } ## end if ( isdigit($io_write...))
    } ## end else [ if ( defined $inventory_host...)]

    $item = "resp_t_r";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $resp_r = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
    }

    $item = "resp_t_w";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $resp_w = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
    }

    $item = "resp_t";
    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $resp_t = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
    }
    else {
      my $numerator;
      if ( isdigit($resp_w) && $resp_w ne "-" && isdigit($io_write) && $io_write ne "-" ) {
        $numerator = $resp_w * $io_write;
      }
      if ( isdigit($numerator) ) {
        if ( isdigit($resp_r) && $resp_r ne "-" && isdigit($io_read) && $io_read ne "-" ) {
          $numerator = $numerator + ( $resp_r * $io_read );
        }
      }
      else {
        if ( isdigit($resp_r) && $resp_r ne "-" && isdigit($io_read) && $io_read ne "-" ) {
          $numerator = $resp_r * $io_read;
        }
      }
      if ( isdigit($io_rate) && $io_rate ne "-" && $io_rate != 0 && isdigit($numerator) ) {
        $resp_t = round( $numerator / $io_rate, 1 );
      }
    } ## end else [ if ( defined $inventory_host...)]

    $item = "w_cache_hit";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $cache_w = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
    }

    $item = "r_cache_hit";

    if ( defined $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item} ) {
      $cache_r = $inventory_host{STORAGE}{$storage}{$host}{ITEM}{$item};
    }
    if ( $volumes eq "" ) {
      $volumes = "N/A";
    }

    $line = "<tr><td style=\"white-space: nowrap\"><a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=VOLUME&name=io&storage=$st_type&item=sum&gui=1&none=none\">$storage<\/a><\/td><td style=\"white-space: nowrap\"><a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=HOST&name=$host&storage=$st_type&gui=1&none=none\">$host<\/a><\/td><td>$volumes<\/td><td align=\"right\">$io_rate<\/td><td align=\"right\">$io_read<\/td><td align=\"right\">$io_write<\/td><td align=\"right\">$data_rate<\/td><td align=\"right\">$read<\/td><td align=\"right\">$write<\/td><td align=\"right\">$resp_t<\/td><td align=\"right\">$resp_r<\/td><td align=\"right\">$resp_w<\/td><td align=\"right\">$cache_r<\/td><td align=\"right\">$cache_w<\/td><\/tr>\n";

    print FHT "$line";
  } ## end foreach my $host ( keys %{ ...})

  #print Dumper \%inventory_host;
  #print Dumper \%inventory_volume;

  close FHT;

} ## end sub hosts_top

sub once_a_day {
  my $version_file = shift;

  # at first check whether it is a first run after the midnight
  if ( -f "$version_file-run" ) {
    return 1;    # "-run" file is already created in this run but it must report every time (for every storage)
  }

  if ( !-f $version_file ) {

    #error("version file er: $version_file does not exist, it should not happen, creating it");
    # --> no error, it is ok after upgrade
    `touch $version_file`;
    LoadDataModule::touch("$version_file does not exist");
    return 1;
  } ## end if ( !-f $version_file)
  else {
    my $run_time = ( stat("$version_file") )[9];
    ( my $sec, my $min, my $h, my $aday, my $m, my $y, my $wday, my $yday, my $isdst ) = localtime( time() );
    ( $sec, $min, $h, my $png_day, $m, $y, $wday, $yday, $isdst ) = localtime($run_time);
    if ( $aday != $png_day ) {
      LoadDataModule::touch("first run after the midnight: $aday != $png_day");
      `touch $version_file`;
      return 1;
    }
  } ## end else [ if ( !-f $version_file)]
  return 0;
} ## end sub once_a_day

# prints out the Global Volume Top tables --> it runs once a daty (1st run after the midnight or after upgrade)

sub global_volumes_top {
  my $storage  = shift;
  my $st_type  = shift;
  my $webdir   = shift;
  my $act_time = shift;
  my $type     = "VOLUME";
  my $name     = "top";

  print "global vol top : $storage:$type:$name:avrg:d\n";
  volumes_top( $storage, $type, $name, "avrg", "d", $st_type, $webdir, $act_time );
  print "global vol top : $storage:$type:$name:avrg:w\n";
  volumes_top( $storage, $type, $name, "avrg", "w", $st_type, $webdir, $act_time );
  print "global vol top : $storage:$type:$name:avrg:m\n";
  volumes_top( $storage, $type, $name, "avrg", "m", $st_type, $webdir, $act_time );
  print "global vol top : $storage:$type:$name:max:d\n";
  volumes_top( $storage, $type, $name, "max", "d", $st_type, $webdir, $act_time );
  print "global vol top : $storage:$type:$name:max:w\n";
  volumes_top( $storage, $type, $name, "max", "w", $st_type, $webdir, $act_time );

  if ( defined $ENV{TOP_MAX_MONTHLY} && isdigit( $ENV{TOP_MAX_MONTHLY} ) && $ENV{TOP_MAX_MONTHLY} == 1 ) {
    print "global vol top : $storage:$type:$name:max:m\n";
    volumes_top( $storage, $type, $name, "max", "m", $st_type, $webdir, $act_time );
  }

  return 1;
} ## end sub global_volumes_top

sub global_hosts_top {
  my $storage  = shift;
  my $st_type  = shift;
  my $webdir   = shift;
  my $act_time = shift;
  my $type     = "HOST";
  my $name     = "top";

  print "global host top: $storage:$type:$name:avrg:d\n";
  hosts_top( $storage, $type, $name, "avrg", "d", $st_type, $webdir, $act_time );

  print "global host top: $storage:$type:$name:avrg:w\n";
  hosts_top( $storage, $type, $name, "avrg", "w", $st_type, $webdir, $act_time );

  print "global host top: $storage:$type:$name:avrg:m\n";
  hosts_top( $storage, $type, $name, "avrg", "m", $st_type, $webdir, $act_time );

  print "global host top: $storage:$type:$name:max:d\n";
  hosts_top( $storage, $type, $name, "max", "d", $st_type, $webdir, $act_time );

  print "global host top: $storage:$type:$name:max:w\n";
  hosts_top( $storage, $type, $name, "max", "w", $st_type, $webdir, $act_time );

  if ( defined $ENV{TOP_MAX_MONTHLY} && isdigit( $ENV{TOP_MAX_MONTHLY} ) && $ENV{TOP_MAX_MONTHLY} == 1 ) {
    print "global host top: $storage:$type:$name:max:m\n";
    hosts_top( $storage, $type, $name, "max", "m", $st_type, $webdir, $act_time );
  }

  return 1;

} ## end sub global_hosts_top

# below code is copied and slightly modified from detail-graph-cgi.pl
sub volumes_top {
  my $host         = shift;
  my $type         = shift;
  my $name         = shift;
  my $item         = shift;
  my $time         = shift;
  my $st_type      = shift;
  my $webdir       = shift;
  my $act_time     = shift;
  my $in_file      = "$tmp_dir/$host/$type-$name-$time.cmd";
  my $file_cmd_max = "/var/tmp/stor2rrd-$type-$name-max-$time.cmd";
  my $out_file     = "$webdir/glob-$type-$name-$item-$time.html-tmp";

  #"read","write","read_io","write_io","resp_t_r","resp_t_w","r_cache_hit","w_cache_hit";
  #DS5K-v1:"io_rate","data_rate","cache_hit";
  #DS5K-v2:"io_rate","data_rate","resp_t","r_cache_hit","w_cache_hit","ssd_r_cache_hit";
  #VNX:"read","write","read_io","write_io"

  if ( !-f $in_file ) {
    error( "$host:VOLUME:top:$item - no cmd file:$in_file " . __FILE__ . ":" . __LINE__ ) if ($DEBUG);
    return 0;
  }

  my $width = 288;    # to get 5 minute per a pixel
  if ( $time =~ m/^w/ ) {
    $width = 1728;    # to get 5 minute per a pixel, 6d
  }
  if ( $time =~ m/^m/ ) {
    $width = 6912;    # to get 5 minute per a pixel, 24d
  }
  if ( $time =~ m/^y/ ) {
    $width = 97056;    # to get 5 minute per a pixel, 337d
  }

  if ( !-f $out_file ) {
    open( FHT, "> $out_file" ) || error( "Can't open $out_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    print FHT "<center>\n";
    if ( $time =~ m/^d/ ) {
      print FHT "last day (created on: $act_time)<br>\n";
    }
    if ( $time =~ m/^w/ ) {
      print FHT "last week without last day (created on: $act_time)<br>\n";
    }
    if ( $time =~ m/^m/ ) {
      print FHT "last month without last week (created on: $act_time)<br>\n";
    }
    print FHT "<table class=\"lparsearch tablesorter\" data-sortby=\"6\"><thead><tr> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Storage&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Volume&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;Capacity<br>&nbsp;&nbsp;&nbsp;<span>GB</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\" valign=\"top\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Host&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;IO&nbsp;rate&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;IO&nbsp;read&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;IO&nbsp;write&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>IOps</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;data&nbsp;rate&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>MB/sec</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;read&nbsp;&nbsp;<br>&nbsp;<span>MB/sec</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;write&nbsp;&nbsp;<br>&nbsp;<span>MB/sec</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;resp&nbsp;t&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;resp&nbsp;read&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;resp&nbsp;write&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>ms</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;r&nbsp;cachehit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;</th> \n
         <th class=\"sortable\" align=\"center\">&nbsp;&nbsp;w&nbsp;cachehit&nbsp;&nbsp;&nbsp;<br>&nbsp;&nbsp;<span>%</span>&nbsp;&nbsp;</th> \n
         </tr></thead><tbody>\n";
  } ## end if ( !-f $out_file )
  else {
    open( FHT, ">> $out_file" ) || error( "Can't open $out_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  if ( $item =~ m/max/ ) {

    # For printing peaks (MAX)
    open( FHR, "< $in_file" ) || error( "Can't open $in_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @lines = <FHR>;
    close(FHR);
    open( FHW, "> $file_cmd_max" ) || error( "Can't open $file_cmd_max : $! " . __FILE__ . ":" . __LINE__ ) && return 0;

    #binmode(FHW);

    foreach my $line (@lines) {
      $line =~ s/:AVERAGE:/:MAX:/g;
      $line =~ s/ --width=.* --height=/ --width=$width --height=/;
      $line =~ s/ --step=.* --units-exponent=/ --step=300 --units-exponent=/;    # just to be sure ...
      print FHW "$line";
    } ## end foreach my $line (@lines)
    close(FHW);
    $in_file = $file_cmd_max;
  } ## end if ( $item =~ m/max/ )

  open( FH, "< $in_file" ) || error( "VOLUME: Can't open $in_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);
  my $cmd = "";
  foreach my $line (@lines) {
    $cmd = $line;
    last;
  }
  if ( $cmd eq '' ) {
    error( "$host : Multi graph rrdtool error $cmd " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  # Must be in eval for volumes, high amount of volumes can cause memory alloc issue on the OS and this does not exit program
  my $ret = "";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "RRDp::read $@ " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( !defined($$ret) ) {
    return 0;    # old unused volumes??, it must be here as it caused a problem
  }
  if ( $$ret =~ "ERROR" ) {
    error( "$host : Multi graph rrdtool error : $$ret  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( $item =~ m/max/ ) {
    unlink($in_file);    # clean only temp files in /var/tmp
  }
  if ( $$ret =~ "ERROR" ) {
    if ( $$ret =~ "ERROR: malloc fetch data area" ) {
      error( "$host:VOLUME:$name:$item: Multi graph rrdtool error : ERROR: malloc fetch data area " . __FILE__ . ":" . __LINE__ );
    }
    else {
      error( "$host:VOLUME:$name:$item: Multi graph rrdtool error : $$ret " . __FILE__ . ":" . __LINE__ );
    }
    return 0;
  } ## end if ( $$ret =~ "ERROR" )

  #
  # below code is also in custom global volume top code in detail-graph-cgi.pl : volumes_top
  # just printing on stdout in detail-graph..
  #

  my $null        = "-";
  my $data_rate   = 0;
  my $io_rate     = 0;
  my $volume_prev = "";
  my $read_io     = $null;
  my $write_io    = $null;
  my $resp_numerator;

  my $read        = $null;
  my $write       = $null;
  my $resp_t_r    = $null;
  my $resp_t_w    = $null;
  my $r_cache_hit = $null;
  my $w_cache_hit = $null;
  my $resp_t      = $null;

  (@lines) = split( /====start /, $$ret );
  my $item_no = 0;
  my $count   = 0;
  foreach my $line (@lines) {
    chomp($line);
    if ( $line !~ m/^==== / ) {
      next;
    }

    $line =~ s/^==== / /g;
    $line =~ s/^ *//g;
    ( my $volume, my $item, my $value ) = split( / +/, $line );
    $value =~ s/OK.*//g;    #last one
    $value =~ s/ //g;
    chomp($value);          # last one record has en enter at the end
    $volume =~ s/=====space=====/ /g;

    #print "001 --$line: $volume:$item:$value \n";
    if ( !defined($value) || $value eq '' || $volume =~ m/^$/ ) {
      next;
    }

    if ( $volume_prev eq "" || $volume_prev eq $volume ) {

      if ( $item eq "read" ) {
        $read = $value;
      }
      if ( $item eq "write" ) {
        $write = $value;
      }
      if ( $item eq "resp_t_r" ) {
        $resp_t_r = $value;
      }
      if ( $item eq "resp_t_w" ) {
        $resp_t_w = $value;
      }
      if ( $item eq "r_cache_hit" ) {
        $r_cache_hit = $value;
      }
      if ( $item eq "w_cache_hit" ) {
        $w_cache_hit = $value;
      }
      if ( $item eq "resp_t" ) {
        $resp_t = $value;
      }
      if ( $item eq "data_rate" ) {
        $data_rate = $value;
      }
      if ( $item eq "io_rate" ) {
        $io_rate = $value;
      }
    } ## end if ( $volume_prev eq ""...)

    $count++;

    if ( $volume_prev eq '' || $volume_prev ne $volume ) {
      if ( !$volume_prev eq '' ) {

        ### pool name add volume top
        my ( $pool_name, $cap ) = get_name_pool( $volume_prev, $st_type, $storage );
        if ( !defined $cap || !isdigit($cap) ) {
          $cap = "N/A";
        }
        if ( !defined $pool_name || $pool_name eq "" ) {
          $pool_name = "N/A";
        }
        ### hosts volume
        my $hosts = get_hosts( $volume_prev, $st_type, $host );
        if ( !defined $hosts || $hosts eq "" ) {
          $hosts = "N/A";
        }

        my $line = "<tr><td style=\"white-space: nowrap\"><a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=VOLUME&name=io&storage=$st_type&item=sum&gui=1&none=none\">$storage</a></td><td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=VOLUME&name=$volume_prev&storage=$st_type&none=none\">$volume_prev</a></td>\n<td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=POOL&name=$pool_name&storage=$st_type&none=none\">$pool_name</a></td>\n<td align=\"right\">$cap&nbsp;&nbsp;&nbsp;</td>\n<td>$hosts&nbsp;&nbsp;&nbsp;</td>\n<td align=\"right\" title=\"io_rate\">$io_rate&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"read_io\">$read_io&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"write_io\">$write_io&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"data_rate\">$data_rate&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"read\">$read&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"write\">$write&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t\">$resp_t&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t_r\">$resp_t_r&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t_w\">$resp_t_w&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"r_cache_hit\">$r_cache_hit&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"w_cache_hit\">$w_cache_hit&nbsp;&nbsp;&nbsp;</td></tr>";

        print FHT "$line\n";

        $volume_prev    = $volume;
        $item_no        = 0;
        $data_rate      = 0;
        $io_rate        = 0;
        $read_io        = $null;
        $write_io       = $null;
        $resp_numerator = 0;
        $read           = $null;
        $write          = $null;
        $resp_t_r       = $null;
        $resp_t_w       = $null;
        $r_cache_hit    = $null;
        $w_cache_hit    = $null;
        $resp_t         = $null;

        if ( $item eq "read" ) {
          $read = $value;
        }
        if ( $item eq "write" ) {
          $write = $value;
        }
        if ( $item eq "resp_t_r" ) {
          $resp_t_r = $value;
        }
        if ( $item eq "resp_t_w" ) {
          $resp_t_w = $value;
        }
        if ( $item eq "r_cache_hit" ) {
          $r_cache_hit = $value;
        }
        if ( $item eq "w_cache_hit" ) {
          $w_cache_hit = $value;
        }
        if ( $item eq "resp_t" ) {
          $resp_t = $value;
        }
        if ( $item eq "data_rate" ) {
          $data_rate = $value;
        }
        if ( $item eq "io_rate" ) {
          $io_rate = $value;
        }

      } ## end if ( !$volume_prev eq ...)
      else {
        $volume_prev = $volume;
      }
    } ## end if ( $volume_prev eq ''...)

    $item_no++;
    if ( $st_type =~ m/DS5K/ ) {
      next;
    }

    # get data/io rate from read/write values for != DS5k
    if ( $item_no == 1 ) {
      $data_rate = $value;
    }
    if ( $item_no == 2 ) {
      if ( isdigit($data_rate) && isdigit($value) ) {
        my $data_rate_tmp = $data_rate + $value;
        $data_rate = sprintf( "%.1f", $data_rate_tmp );
      }
    } ## end if ( $item_no == 2 )
    if ( $item_no == 3 ) {
      $io_rate = $value;
      $read_io = $value;
    }
    if ( $item_no == 4 ) {
      if ( isdigit($io_rate) && isdigit($value) ) {
        my $io_rate_tmp = $io_rate + $value;
        $write_io = $value;
        $io_rate = sprintf( "%.1f", $io_rate_tmp );
      }
    } ## end if ( $item_no == 4 )
    if ( $st_type !~ m/VNXF|UNITY|VNXE|VNX$|3PAR|NETAPP|VMAX/ && $item_no == 5 ) {
      if ( isdigit($value) && isdigit($read_io) ) {
        $resp_numerator = $value * $read_io;
      }
      else { $resp_numerator = 0; }
    } ## end if ( $st_type !~ m/VNXF|UNITY|VNXE|VNX$|3PAR|NETAPP|VMAX/...)
    if ( $st_type !~ m/VNXF|UNITY|VNXE|VNX$|3PAR|NETAPP|VMAX/ && $item_no == 6 ) {
      if ( isdigit($value) && isdigit($write_io) ) {
        $resp_numerator = $resp_numerator + ( $value * $write_io );
      }
      my $io_rate;
      if ( isdigit($read_io) ) {
        $io_rate = $read_io;
      }
      if ( isdigit($write_io) ) {
        if ( isdigit($io_rate) ) {
          $io_rate = $io_rate + $write_io;
        }
        else {
          $io_rate = $write_io;
        }
      } ## end if ( isdigit($write_io...))
      if ( defined $io_rate && ( !isdigit($io_rate) || $io_rate eq $null ) ) {
        $io_rate = 0;
      }
      if ( !defined $io_rate || $io_rate == 0 ) {
        $io_rate = $null;
      }
      else {
        $resp_t = round( $resp_numerator / $io_rate, 1 );
      }
    } ## end if ( $st_type !~ m/VNXF|UNITY|VNXE|VNX$|3PAR|NETAPP|VMAX/...)
  } ## end foreach my $line (@lines)

  if ( $count > 0 ) {
    ### pool name add volume top
    my ( $pool_name, $cap ) = get_name_pool( $volume_prev, $st_type, $storage );
    if ( !defined $cap || !isdigit($cap) ) {
      $cap = "N/A";
    }
    if ( !defined $pool_name || $pool_name eq "" ) {
      $pool_name = "N/A";
    }
    ### hosts volume
    my $hosts = get_hosts( $volume_prev, $st_type, $host );
    if ( !defined $hosts || $hosts eq "" ) {
      $hosts = "N/A";
    }

    my $line = "<tr><td style=\"white-space: nowrap\"><a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=VOLUME&name=io&storage=$st_type&item=sum&gui=1&none=none\">$storage</a></td><td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=VOLUME&name=$volume_prev&storage=$st_type&none=none\">$volume_prev</a></td>\n<td><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=POOL&name=$pool_name&storage=$st_type&none=none\">$pool_name</a></td>\n<td align=\"right\">$cap&nbsp;&nbsp;&nbsp;</td>\n<td>$hosts&nbsp;&nbsp;&nbsp;</td>\n<td align=\"right\" title=\"io_rate\">$io_rate&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"read_io\">$read_io&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"write_io\">$write_io&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"data_rate\">$data_rate&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"read\">$read&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"write\">$write&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t\">$resp_t&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t_r\">$resp_t_r&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"resp_t_w\">$resp_t_w&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"r_cache_hit\">$r_cache_hit&nbsp;&nbsp;&nbsp;</td><td align=\"right\" title=\"w_cache_hit\">$w_cache_hit&nbsp;&nbsp;&nbsp;</td></tr>";
    print FHT "$line\n";
  } ## end if ( $count > 0 )
  close(FHT);
  return 0;
} ## end sub volumes_top

sub get_hosts {
  my $volume  = shift;
  my $st_type = shift;
  my $storage = shift;
  my @ids     = ();
  my $hosts   = "";

  #first step find id volume

  my $volume_cfg = "$basedir/data/$storage/VOLUME/volumes.cfg";

  if ( !-f $volume_cfg ) {
    return "";
  }
  open( FHR, "< $volume_cfg" ) || error( "Can't open $volume_cfg : $! " . __FILE__ . ":" . __LINE__ ) && return "";
  my @lines = <FHR>;
  close(FHR);

  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" ) { next; }
    my ( $volume_name, $id ) = split( " : ", $line );
    if ( $volume_name ne "$volume" ) { next; }
    @ids = split( ";", $id );
    last;
  } ## end foreach my $line (@lines)
  my $size = scalar @ids;
  if ( $size == 0 ) {
    return "";
  }
  my $host_cfg = "$basedir/data/$storage/HOST/hosts.cfg";
  if ( !-f $host_cfg ) {
    return "";
  }
  open( FHR, "< $host_cfg" ) || error( "Can't open $host_cfg : $! " . __FILE__ . ":" . __LINE__ ) && return "";
  @lines = <FHR>;
  close(FHR);

  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" ) { next; }
    my ( $host_name, $volumes ) = split( " : ", $line );
    my @volumes_ids = split( " ", $volumes );
    foreach my $id (@ids) {
      if ( $st_type eq "SWIZ" ) {
        $id =~ s/^0*//;
        if ( $id eq "" ) {
          $id = 0;
        }
      } ## end if ( $st_type eq "SWIZ")
      if ( $st_type eq "DS8K" ) {
        $id =~ s/^0x//;
      }
      if ( grep { $id eq $_ } @volumes_ids ) {
        $hosts = $hosts . "<a href=\"/stor2rrd-cgi/detail.sh?host=$storage&type=HOST&name=$host_name&storage=$st_type&none=none\">$host_name</a>" . ", ";
        last;
      }
    } ## end foreach my $id (@ids)
  } ## end foreach my $line (@lines)
  if ( $hosts ne "" ) {
    $hosts =~ s/^\s+|\s+$//g;
    chop $hosts;
  }
  return $hosts;
} ## end sub get_hosts

sub get_name_pool {
  my $volume_name = shift;
  my $st_type     = shift;
  my $host        = shift;

  my $config = "$basedir/data/$host/config.html";
  if ( !-f $config ) {
    return ( "", "" );
  }
  open( FHR, "< $config" ) || error( "Can't open $config : $! " . __FILE__ . ":" . __LINE__ ) && return ( "", "" );
  my @lines = <FHR>;
  close(FHR);

  my $active_vol  = 0;
  my $active_pool = 0;
  my %pools;
  my $index_id        = "";
  my $index_name_pool = "";
  my $index_id_pool   = "";
  my $index_rg2       = "";
  my $index_rg1       = "";
  my $pool_id         = "";
  my $index_capacity  = "";
  my $index_name      = "";
  my $cap             = "";
  foreach my $line (@lines) {
    chomp $line;
    if ( $line eq "" || $line =~ m/^</ ) {
      $active_vol  = 0;
      $active_pool = 0;
      next;
    }
    if ( $line =~ m/^---------/ && $active_vol == 1 ) {
      $active_vol++;
      next;
    }
    if ( $line =~ m/^---------/ && $active_pool == 1 ) {
      $active_pool++;
      next;
    }
    if ( $line =~ m/Volume Level Configuration/ ) {
      $active_vol = 1;
      next;
    }
    if ( $line =~ m/StorageGroup Level Configuration/ ) {
      $active_vol = 1;
      next;
    }
    if ( $line =~ m/Pool Level Configuration/ ) {
      $active_pool = 1;
      next;
    }
    if ( $line =~ m/Rank Level Configuration/ && $st_type eq "DS8K" ) {
      $active_pool = 1;
      next;
    }
    if ( $line =~ m/Raid Group Level Configuration/ && $st_type eq "HUS" ) {
      $active_pool = 1;
      next;
    }
    if ( $active_vol == 2 ) {
      my $index = -1;
      my @header = split( ",", $line );
      foreach my $element (@header) {
        $index++;
        if ( $element eq "name" || $element eq "volume_name" || $element eq "Volume name" || $element eq "Volume Nickname" || $element eq "Name" || $element eq "SG_ID" ) {
          $index_name = $index;
          next;
        }
        if ( $element eq "volume_id" || $element eq "lun_id" || $element eq "Volume ID" || $element eq "SG_ID" ) {
          $index_id = $index;
          next;
        }
        if ( $element eq "pool_id" || $element eq "CPG ID" || $element eq "Pool ID" || $element eq "Storage pool ID" ) {
          $index_id_pool = $index;
          next;
        }
        if ( $element eq "pool_name" || $element eq "mdisk_grp_name" || $element eq "containing-aggregate-name" || $element eq "CPG name" || $element eq "Pool name" || $element eq "Storage pool" || $element eq "Pool Name" ) {
          $index_name_pool = $index;
          next;
        }
        if ( $element eq "capacity (MB)" || $element eq "capacity" || $element eq "User size [MB]" || $element eq "size-total" || $element eq "Volume Size" || $element eq "LUN_Capacity (MB)" || $element eq "Size (MB)" || $element eq "AllocatedCapacity" || $element eq "DEVICE_CAPACITY" ) {
          $index_capacity = $index;
          next;
        }
        ### raid groups
        if ( $element eq "RG_ID" || $element eq "raid_group_id" ) {
          $index_rg1 = $index;
        }
        if ( $element eq "RG_name" ) {
          $index_rg2 = $index;
        }
      } ## end foreach my $element (@header)
      if ( !isdigit($index_name) ) {
        $index_name = $index_id;
      }
      $active_vol++;
      next;
    } ## end if ( $active_vol == 2 )
    if ( $active_vol == 3 && isdigit($index_name) ) {
      my @data = split( ",", $line );

      if ( !defined $data[$index_name] || $data[$index_name] eq "" ) {
        $index_name = $index_id;
      }
      if ( defined $data[$index_name] && $data[$index_name] eq $volume_name ) {
        if ( isdigit($index_capacity) && defined $data[$index_capacity] && isdigit( $data[$index_capacity] ) ) {
          if ( $st_type eq "VSPG" || $st_type eq "HUS" || $st_type eq "3PAR" || $st_type eq "DS5K" || $st_type eq "VNXF" || $st_type eq "VNX" || $st_type eq "UNITY" || $st_type eq "DOTHILL" || $st_type eq "VNXE" || $st_type eq "COMPELLENT" || $st_type eq "OCEANSTOR" ) {
            $cap = convert_capacity( "MB", "GB", $data[$index_capacity] );
          }
          if ( $st_type eq "NETAPP" || $st_type eq "XIV" || $st_type eq "DS8K" || $st_type eq "VMAX" ) {
            $cap = round( $data[$index_capacity], 1 );
          }
          if ( $st_type eq "SWIZ" ) {
            $cap = convert_capacity( "B", "GB", $data[$index_capacity] );
          }
        } ## end if ( isdigit($index_capacity...))
        if ( isdigit($index_name_pool) && defined $data[$index_name_pool] && $data[$index_name_pool] ne "" && $st_type ne "DS8K" ) {

          #if ($st_type eq "VNX" && isdigit($index_id_pool) && defined $data[$index_id_pool] && $data[$index_id_pool] ne ""){
          #  $data[$index_name_pool] = "RG_" . $data[$index_id_pool];
          #}
          return ( $data[$index_name_pool], $cap );
        } ## end if ( isdigit($index_name_pool...))
        if ( isdigit($index_id_pool) && defined $data[$index_id_pool] && $data[$index_id_pool] ne "" ) {
          if ( defined $pools{ $data[$index_id_pool] } && $pools{ $data[$index_id_pool] } ne "" ) {
            return ( $pools{ $data[$index_id_pool] }, $cap );
          }
          else {
            $pool_id = $data[$index_id_pool];
            next;
          }
        } ## end if ( isdigit($index_id_pool...))
        if ( isdigit($index_rg2) && defined $data[$index_rg2] && $data[$index_rg2] ne "" ) {
          return ( $data[$index_rg2], $cap );
        }

        if ( isdigit($index_rg1) && defined $data[$index_rg1] && $data[$index_rg1] ne "" && $st_type eq "VNX" ) {
          my $pool_n = "RG_" . $data[$index_rg1];
          return ( $pool_n, $cap );
        }
        if ( isdigit($index_rg1) && defined $data[$index_rg1] && $data[$index_rg1] ne "" && $st_type eq "HUS" ) {
          $pool_id = $data[$index_rg1];
          next;
        }
      } ## end if ( defined $data[$index_name...])
    } ## end if ( $active_vol == 3 ...)
    if ( $active_pool == 2 ) {
      my $index = -1;
      my @header = split( ",", $line );
      foreach my $element (@header) {
        $index++;
        if ( $element eq "pool_id" || $element eq "CPG ID" || $element eq "Pool ID" || $element eq "Storage pool ID" || $element eq "id" ) {
          $index_id_pool = $index;
          next;
        }
        if ( $element eq "pool_name" || $element eq "mdisk_grp_name" || $element eq "containing-aggregate-name" || $element eq "CPG name" || $element eq "Pool name" || $element eq "Storage pool" || $element eq "name" || $element eq "Pool Name" ) {
          $index_name_pool = $index;
          next;
        }
      } ## end foreach my $element (@header)
      $active_pool++;
      next;
    } ## end if ( $active_pool == 2)
    if ( $active_pool == 3 && ( isdigit($index_id_pool) || isdigit($index_name_pool) ) ) {
      my @data = split( ",", $line );
      if ( isdigit($index_id_pool) && defined $data[$index_id_pool] && $data[$index_id_pool] ne "" ) {
        if ( isdigit($index_name_pool) && defined $data[$index_name_pool] && $data[$index_name_pool] ne "" ) {
          $pools{ $data[$index_id_pool] } = $data[$index_name_pool];
          next;
        }
        else {
          $pools{ $data[$index_id_pool] } = $data[$index_id_pool];
        }
      } ## end if ( isdigit($index_id_pool...))
    } ## end if ( $active_pool == 3...)
  } ## end foreach my $line (@lines)
  if ( defined $pools{$pool_id} && $pools{$pool_id} ne "" ) {
    return ( $pools{$pool_id}, $cap );
  }
  else {
    return ( "", $cap );
  }
} ## end sub get_name_pool

sub host_data_selection {
  my $wrkdir  = shift;
  my $host    = shift;
  my $type    = shift;
  my $st_type = shift;

  my $wrkdir_space = check_space_name($wrkdir);
  my $host_space   = check_space_name($host);
  my @files        = <$wrkdir_space/$host_space/$type/*>;
  my $rrd          = grep {/\.rrd$/} @files;

  my $VSPG_NATIVE_HOSTS = 1;
  if ( $st_type =~ m/VSPG/ && defined $ENV{VSPG_NATIVE_HOSTS} ) {

    # VSPG_NATIVE_HOSTS=0 forces to use original volume based host graphs (etc/.magic)
    $VSPG_NATIVE_HOSTS = $ENV{VSPG_NATIVE_HOSTS};
    if ( defined $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST} && $VSPG_NATIVE_HOSTS == 0 ) {
      $VSPG_NATIVE_HOSTS = 1;    # host are specified, switch default to host data back
      foreach my $storage_act ( split( / /, $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST} ) ) {
        if ( $host =~ m/^$storage_act$/ ) {
          $VSPG_NATIVE_HOSTS = 0;
        }
      }
    } ## end if ( defined $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST...})
  } ## end if ( $st_type =~ m/VSPG/...)

  if ( $rrd > 0 && $VSPG_NATIVE_HOSTS == 1 ) {
    return 1;                    # use host original
  }
  else {
    return 0;                    # use volume data instead
  }
} ## end sub host_data_selection

sub get_color {
  my ( $name, $color_max, $color_save_tmp ) = @_;
  my @color_save = @{$color_save_tmp};

  # Found out stored color index to keep same color for the volume across all graphs
  my $color_indx     = -1;
  my $color_indx_all = 0;
  my $name_cmp       = $name;
  $name_cmp =~ s/\\/===========back=========/g;
  $name_cmp =~ s/:/===========doublecoma=========/g;
  $name_cmp =~ s/\(/==rbracket==/g;
  $name_cmp =~ s/\)/==lbracket==/g;
  $name_cmp =~ s/\[/==rbrackets==/g;
  $name_cmp =~ s/\]/==lbrackets==/g;
  $name_cmp =~ s/ *$//;

  foreach my $line_col (@color_save) {
    chomp($line_col);
    if ( $line_col eq '' || $line_col !~ m/ : / ) {
      next;
    }
    ( $color_indx, my $name_save ) = split( / : /, $line_col );

    # do not use here $name_save eq '' as this does not work when volume id is zero!
    $name_save =~ s/\\/===========back=========/g;
    $name_save =~ s/:/===========doublecoma=========/g;
    $name_save =~ s/\(/==rbracket==/g;
    $name_save =~ s/\)/==lbracket==/g;
    $name_save =~ s/\[/==rbrackets==/g;
    $name_save =~ s/\]/==lbrackets==/g;
    $name_save =~ s/ *$//;

    if ( $name_cmp =~ m/^$name_save$/ ) {
      return $color_indx;    # color found, return it
    }
    $color_indx_all++;
  } ## end foreach my $line_col (@color_save)
  $color_indx++;
  while ( $color_indx == $color_max ) {
    $color_indx = $color_indx - $color_max;
  }

  # $color_indx_all is pointer to new record for  @color_save
  # $color_indx is real color index
  return $color_indx_all . "\|" . $color_indx . " : " . $name;
} ## end sub get_color

# write colors into a file
sub write_colors {
  my ( $color_file_change, $file_col, $color_save_tmp ) = @_;
  my @color_save = @{$color_save_tmp};

  if ( $color_file_change == 1 ) {
    open( FHC, "> $file_col" ) || error( "file does not exists : $file_col " . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line_cs (@color_save) {
      chomp($line_cs);    # it must be there, somehow appear there \n ...
      if ( $line_cs eq '' ) {
        next;
      }
      if ( $line_cs =~ m/ : / ) {
        print FHC "$line_cs\n";
      }
    } ## end foreach my $line_cs (@color_save)
    close(FHC);
  } ## end if ( $color_file_change...)
} ## end sub write_colors

sub fast_sg_table {
  my $host      = shift;
  my $files     = shift;
  my @rrd_files = @{$files};
  my $tier_cfg  = "$wrkdir/$host/tier.cfg";
  my $fast_html = "$webdir/$host/fast-sg.html";
  my @tiers;
  my %data;

  my $act_time   = time();
  my $start_time = $act_time - 3600;    #last hour

  if ( -f "$tier_cfg" ) {
    open( CFG, "< $tier_cfg" ) || error( "file does not exists : $tier_cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
    @tiers = <CFG>;
    close(CFG);
  }
  else {
    error( "file does not exists : $tier_cfg " . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  foreach my $rrd (@rrd_files) {
    chomp $rrd;

    if ( !-f "$rrd" ) {
      error( "Something wrong! rrd file does not exists : $rrd " . __FILE__ . ":" . __LINE__ ) && next;
    }

    my $rrd_line = $rrd;
    $rrd_line =~ s/^$wrkdir\/$host\/FAST-SG\///;
    $rrd_line =~ s/\.rrd$//;

    my $tier_name = "";
    my $sg_name   = "";

    foreach my $tier_line (@tiers) {
      chomp $tier_line;
      my ( $tier_n, undef ) = split( ":", $tier_line );

      if ( $rrd_line =~ m/-$tier_n$/ ) {
        $tier_name = $tier_n;
        $sg_name   = $rrd_line;
        $sg_name =~ s/-$tier_n$//;
        last;
      } ## end if ( $rrd_line =~ m/-$tier_n$/)
    } ## end foreach my $tier_line (@tiers)

    my $cmd = "";
    $cmd .= "graph \\\"$basedir/tmp/name.png\\\"";
    $cmd .= " --start \\\"$start_time\\\"";
    $cmd .= " --end \\\"now\\\"";
    $cmd .= " --width=400";
    $cmd .= " --height=150";
    $cmd .= " --step=300";
    $cmd .= " DEF:tier_alloc_prct=\\\"$rrd\\\":tier_alloc_prct:AVERAGE";
    $cmd .= " DEF:sg_cap_tot=\\\"$rrd\\\":sg_cap_tot:AVERAGE";
    $cmd .= " DEF:tier_cap=\\\"$rrd\\\":tier_cap:AVERAGE";
    $cmd .= " PRINT:tier_alloc_prct:AVERAGE:\\\"%6.1lf tier_alloc_prct\\\"";
    $cmd .= " PRINT:sg_cap_tot:AVERAGE:\\\"%6.1lf sg_cap_tot\\\"";
    $cmd .= " PRINT:tier_cap:AVERAGE:\\\"%6.1lf tier_cap\\\"";
    $cmd =~ s/\\"/"/g;
    RRDp::cmd qq($cmd);
    my $data = RRDp::read;

    my $value        = "";
    my $sg_cap_total = "";
    my $tier_cap     = "";
    foreach my $data_line ( split( /\n/, $$data ) ) {
      chomp($data_line);

      if ( $data_line =~ m/^0x/ ) { next; }
      $data_line =~ s/^\s+//;
      $data_line =~ s/\s+$//;

      if ( $data_line =~ m/tier_alloc_prct$/ ) {
        $value = $data_line;
        $value =~ s/\s+tier_alloc_prct$//;
      }
      if ( $data_line =~ m/sg_cap_tot$/ ) {
        $sg_cap_total = $data_line;
        $sg_cap_total =~ s/\s+sg_cap_tot$//;
      }
      if ( $data_line =~ m/tier_cap$/ ) {
        $tier_cap = $data_line;
        $tier_cap =~ s/\s+tier_cap$//;
      }
    } ## end foreach my $data_line ( split...)

    if ( defined $sg_name && $sg_name ne "" && defined $tier_name && $tier_name ne "" ) {
      if ( defined $value && isdigit($value) ) {
        $data{$sg_name}{$tier_name}{'tier_alloc_prct'} = $value;
      }
      if ( defined $tier_cap && isdigit($tier_cap) ) {
        $data{$sg_name}{$tier_name}{'tier_cap'} = sprintf( '%.1f', $tier_cap / 1024 );
      }
      if ( defined $sg_cap_total && isdigit($sg_cap_total) ) {
        $data{$sg_name}{'sg_cap_total'} = sprintf( '%.1f', $sg_cap_total / 1024 );
      }
    } ## end if ( defined $sg_name ...)
  } ## end foreach my $rrd (@rrd_files)

  open( FAST, "> $fast_html" ) || error( "Couldn't open file $fast_html $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FAST "<br><br><br>\n";
  print FAST "<center>\n";
  print FAST "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
  print FAST "<thead>\n";
  print FAST "<tr>\n";

  my $header = "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">Storage group</th>";

  foreach my $tier_line (@tiers) {
    chomp $tier_line;
    my ( $tier_n, undef ) = split( ":", $tier_line );
    $header .= "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">$tier_n %</th>";
  }
  print FAST "$header\n";
  print FAST "</tr>\n";
  print FAST "</thead>\n";
  print FAST "<tbody>\n";

  foreach my $sg ( keys %data ) {
    my $data_line = "<td style=\"text-align:center; color:black;\" nowrap=\"\"><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=STORAGE GROUP&name=$sg&storage=VMAX&item=$sg&gui=1&none=none\">$sg</a></td>";
    if ( defined $data{$sg}{'sg_cap_total'} && isdigit( $data{$sg}{'sg_cap_total'} ) ) {
      $data_line = "<td style=\"text-align:center; color:black;\" nowrap=\"\"><a href=\"/stor2rrd-cgi/detail.sh?host=$host&type=STORAGE GROUP&name=$sg&storage=VMAX&item=$sg&gui=1&none=none\">$sg</a> ($data{$sg}{'sg_cap_total'} TB)</td>";
    }

    #http://karel.xorux.com/stor2rrd-cgi/detail.sh?host=EMC-VMAX-111&type=VOLUME&name=class_b&storage=VMAX&item=class_b&gui=1&none=none
    #<a href=\"/stor2rrd-cgi/detail.sh?host=$switch_name&type=SANPORT&name=port$port_id&storage=SAN-BRCD&item=san&gui=1&none=none\">$gray_st</a>
    foreach my $tier_line (@tiers) {
      chomp $tier_line;
      my ( $tier_n, undef ) = split( ":", $tier_line );
      if ( defined $data{$sg}{$tier_n}{'tier_alloc_prct'} && isdigit( $data{$sg}{$tier_n}{'tier_alloc_prct'} ) ) {
        if ( defined $data{$sg}{$tier_n}{'tier_cap'} && isdigit( $data{$sg}{$tier_n}{'tier_cap'} ) ) {
          $data_line .= "<td style=\"text-align:center; color:black;\" nowrap=\"\">$data{$sg}{$tier_n}{'tier_alloc_prct'} ($data{$sg}{$tier_n}{'tier_cap'} TB)</td>";
        }
        else {
          $data_line .= "<td style=\"text-align:center; color:black;\" nowrap=\"\">$data{$sg}{$tier_n}{'tier_alloc_prct'}</td>";
        }
      } ## end if ( defined $data{$sg...})
      else {
        $data_line .= "<td style=\"text-align:center; color:black;\" nowrap=\"\"></td>";
      }
    } ## end foreach my $tier_line (@tiers)
    print FAST "<tr>\n";
    print FAST "$data_line\n";
    print FAST "</tr>\n";
  } ## end foreach my $sg ( keys %data)
  print FAST "</tbody>\n";
  print FAST "</table>\n";
  close(FAST);

  return 1;
} ## end sub fast_sg_table
